@@ -117,9 +117,6 @@ void version(void)
 #ifdef  HAVE_LIBJASPER
   fprintf(stderr, " JASPER");
 #endif
-#ifdef  HAVE_LIBPROJ
-  fprintf(stderr, " PROJ.4");
-#endif
 #ifdef  HAVE_LIBDRMAA
   fprintf(stderr, " DRMAA");
 #endif
@@ -168,6 +165,7 @@ void usage(void)
   fprintf(stderr, "  Options:\n");
   fprintf(stderr, "    -d             Print debugging information\n");
   fprintf(stderr, "    -f <format>    Format of the output file. (grb, grb2, nc, nc2, nc4, nc4c, nc5, srv, ext or ieg)\n");
+  fprintf(stderr, "    -i <num>       Number of worker to decode/decompress GRIB records\n");
   fprintf(stderr, "    -s             give short information if ofile is missing\n");
   fprintf(stderr, "    -t <table>     Parameter table name/file\n");
   fprintf(stderr, "                   Predefined tables: ");
@@ -371,14 +369,14 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  // institute info
 	  const char *instptr = institutInqNamePtr(vlistInqVarInstitut(vlistID, varID));
 	  strcpy(tmpname, "unknown");
-	  if ( instptr ) strncpy(tmpname, instptr, CDI_MAX_NAME);
+	  if ( instptr ) strncpy(tmpname, instptr, CDI_MAX_NAME-1);
 	  limit_string_length(tmpname, CDI_MAX_NAME);
 	  fprintf(stdout, "%-8s ", tmpname);
 
 	  // source info
 	  const char *modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
 	  strcpy(tmpname, "unknown");
-	  if ( modelptr ) strncpy(tmpname, modelptr, CDI_MAX_NAME);
+	  if ( modelptr ) strncpy(tmpname, modelptr, CDI_MAX_NAME-1);
 	  limit_string_length(tmpname, CDI_MAX_NAME);
 	  fprintf(stdout, "%-8s ", tmpname);
 
