@@ -108,8 +108,10 @@ print_xvals(int gridID, int dig)
   if (xsize > 0 && gridInqXvals(gridID, NULL))
     {
       char xname[CDI_MAX_NAME], xunits[CDI_MAX_NAME];
-      gridInqXname(gridID, xname);
-      gridInqXunits(gridID, xunits);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, xname, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, xunits, &length);
 
       const double xfirst = gridInqXval(gridID, 0);
       const double xlast = gridInqXval(gridID, xsize - 1);
@@ -133,8 +135,10 @@ print_yvals(int gridID, int dig)
   if (ysize > 0 && gridInqYvals(gridID, NULL))
     {
       char yname[CDI_MAX_NAME], yunits[CDI_MAX_NAME];
-      gridInqYname(gridID, yname);
-      gridInqYunits(gridID, yunits);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, yname, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, yunits, &length);
 
       const double yfirst = gridInqYval(gridID, 0);
       const double ylast = gridInqYval(gridID, ysize - 1);
@@ -158,10 +162,14 @@ print_xyvals2D(int gridID, int dig)
   if (gridInqXvals(gridID, NULL) && gridInqYvals(gridID, NULL))
     {
       char xname[CDI_MAX_NAME], yname[CDI_MAX_NAME], xunits[CDI_MAX_NAME], yunits[CDI_MAX_NAME];
-      gridInqXname(gridID, xname);
-      gridInqYname(gridID, yname);
-      gridInqXunits(gridID, xunits);
-      gridInqYunits(gridID, yunits);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, xname, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, yname, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, xunits, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, yunits, &length);
 
       const size_t gridsize = gridInqSize(gridID);
       double *xvals2D = (double *) malloc(gridsize * sizeof(double));
@@ -257,15 +265,18 @@ print_xyvals2D(int gridID, int dig)
 static void
 printGridNp(int gridtype, int gridID, size_t gridsize, size_t xsize, size_t ysize)
 {
-
-  size_t xysize = xsize * ysize;
   fprintf(stdout, "points=%zu", gridsize);
   if (gridtype == GRID_GAUSSIAN_REDUCED)
     fprintf(stdout, "  nlat=%zu", ysize);
-  else if (xysize)
+  else if (xsize && ysize)
     fprintf(stdout, " (%zux%zu)", xsize, ysize);
 
-  if (gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED) fprintf(stdout, "  np=%d", gridInqNP(gridID));
+  const int numLPE = gridInqNP(gridID);
+  if (numLPE > 0)
+    {
+      if (gridtype == GRID_GAUSSIAN) fprintf(stdout, "  F%d", numLPE);
+      if (gridtype == GRID_GAUSSIAN_REDUCED) fprintf(stdout, "  N%d", numLPE);
+    }
 
   fprintf(stdout, "\n");
 }
@@ -303,8 +314,8 @@ printGridInfoKernel(int gridID, int index, bool lproj)
         }
 
       char name[CDI_MAX_NAME];
-      name[0] = 0;
-      cdiGridInqKeyStr(gridID, CDI_KEY_MAPNAME, CDI_MAX_NAME, name);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, name, &length);
       if (gridtype == GRID_PROJECTION || name[0])
         {
           if (name[0] == 0) strcpy(name, "undefined");
@@ -327,13 +338,13 @@ printGridInfoKernel(int gridID, int index, bool lproj)
     }
   else if (gridtype == GRID_SPECTRAL)
     {
-      fprintf(stdout, "points=%zu  nsp=%zu  truncation=%d", gridsize, gridsize / 2, trunc);
+      fprintf(stdout, "points=%zu  nsp=%zu  T%d", gridsize, gridsize / 2, trunc);
       if (gridInqComplexPacking(gridID)) fprintf(stdout, "  complexPacking");
       fprintf(stdout, "\n");
     }
   else if (gridtype == GRID_FOURIER)
     {
-      fprintf(stdout, "points=%zu  nfc=%zu  truncation=%d\n", gridsize, gridsize / 2, trunc);
+      fprintf(stdout, "points=%zu  nfc=%zu  T%d\n", gridsize, gridsize / 2, trunc);
     }
   else if (gridtype == GRID_GME)
     {
@@ -471,8 +482,8 @@ static void
 printZaxisHybridInfo(const int zaxisID)
 {
   char psname[CDI_MAX_NAME];
-  psname[0] = 0;
-  cdiZaxisInqKeyStr(zaxisID, CDI_KEY_PSNAME, CDI_MAX_NAME, psname);
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_PSNAME, psname, &length);
   const int vctsize = zaxisInqVctSize(zaxisID);
   if (vctsize || psname[0])
     {
@@ -487,27 +498,25 @@ static void
 printZaxisGenericInfo(const int ltype, const int zaxistype, const char *zaxisname)
 {
   if (zaxistype == ZAXIS_GENERIC && ltype != 0)
-    {
-      fprintf(stdout, "%-12s (ltype=%3d)", zaxisname, ltype);
-    }
+    fprintf(stdout, "%-12s (ltype=%3d)", zaxisname, ltype);
   else
-    {
-      fprintf(stdout, "%-24s", zaxisname);
-    }
+    fprintf(stdout, "%-24s", zaxisname);
 }
 
 static void
 printZaxisReferenceInfo(const int zaxisID)
 {
-  const int number = zaxisInqNumber(zaxisID);
-  if (number > 0)
+  int referenceNumber = 0;
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NUMBEROFVGRIDUSED, &referenceNumber);
+  if (referenceNumber > 0)
     {
       fprintf(stdout, "%33s : ", "zaxis");
-      fprintf(stdout, "number=%d\n", number);
+      fprintf(stdout, "number=%d\n", referenceNumber);
     }
 
   unsigned char uuidOfVGrid[CDI_UUID_SIZE];
-  zaxisInqUUID(zaxisID, uuidOfVGrid);
+  int length = CDI_UUID_SIZE;
+  cdiInqKeyBytes(zaxisID, CDI_GLOBAL, CDI_KEY_UUID, uuidOfVGrid, &length);
   if (!cdiUUIDIsNull(uuidOfVGrid))
     {
       char uuidOfVGridStr[37];
@@ -533,14 +542,17 @@ printZaxisInfo(int vlistID)
       double zinc = 0;
       const int zaxisID = vlistZaxis(vlistID, index);
       const int zaxistype = zaxisInqType(zaxisID);
-      const int ltype = zaxisInqLtype(zaxisID);
+      int ltype = 0;
+      cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
       const int levelsize = zaxisInqSize(zaxisID);
       // int prec      = zaxisInqDatatype(zaxisID);
       // int dig = (prec == CDI_DATATYPE_FLT64) ? 15 : 7;
 
       zaxisName(zaxistype, zaxisname);
-      zaxisInqName(zaxisID, zname);
-      zaxisInqUnits(zaxisID, zunits);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, zname, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, zunits, &length);
       zunits[12] = 0;
 
       fprintf(stdout, "  %4d : ", vlistZaxisIndex(vlistID, zaxisID) + 1);
