@@ -36,7 +36,7 @@ static int defineGrid()
   double area_vec[nlon*nlat];
   const double * ap = &area_vec[0];
   int i;
-  
+
   gridID = gridCreate(GRID_LONLAT, nlon*nlat);
   gridDefXsize(gridID, nlon);
   gridDefYsize(gridID, nlat);
@@ -45,35 +45,39 @@ static int defineGrid()
   gridDefNvertex ( gridID, 1 );
   gridDefXbounds ( gridID, lons );
   gridDefYbounds ( gridID, lats );
-  for ( i = 0; i < nlon*nlat; i++ )    
+  for ( i = 0; i < nlon*nlat; i++ )
     mask_vec[i] = i % 2 ;
   gridDefMaskGME ( gridID, mp );
-  for ( i = 0; i < nlon*nlat; i++ )    
+  for ( i = 0; i < nlon*nlat; i++ )
     mask_vec[i] = 1;
   gridDefMask ( gridID, mp );
-  gridDefXname ( gridID, "myXname" );
-  gridDefXlongname ( gridID, "myXlongname" );
-  gridDefXunits ( gridID, "myXunits" );
-  gridDefYname ( gridID, "myYname" );
-  gridDefYlongname ( gridID, "myYlongname" );
-  gridDefYunits ( gridID, "myYunits" );
+
+  cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, "myXname");
+  cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, "myYname");
+  cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_LONGNAME, "myXlongname");
+  cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_LONGNAME, "myYlongname");
+  cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, "myXunits");
+  cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, "myYunits");
+
   gridDefDatatype ( gridID, DOUBLE_PRECISION );
   gridDefTrunc ( gridID, 1 );
-  gridDefNumber ( gridID, 6 );
-  gridDefPosition ( gridID, 7 );
-  gridDefReference ( gridID, "myReference" );
+
+  cdiDefKeyInt(gridID, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDUSED, 6);
+  cdiDefKeyInt(gridID, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDINREFERENCE, 7);
+  cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, "myReference");
+
 /* gridDefLCC ( gridID, double originLon, double originLat,  */
 /* 	   double lonParY, double lat1, double lat2, double xinc, double yinc, int projflag, int scanflag); */
 /* gridDefLcc2 ( gridID, double earth_radius, double lon_0,  */
 /* 	    double lat_0, double lat_1,double lat_2);*/
 /* gridDefLaea ( gridID, double earth_radius, double lon_0, double lat_0); */
-  for ( i = 0; i < nlon*nlat; i++ )    
+  for ( i = 0; i < nlon*nlat; i++ )
     area_vec[i] = 0.1 * i;
   gridDefArea ( gridID, ap );
-  for ( i = 0; i < nlon*nlat; i++ )    
+  for ( i = 0; i < nlon*nlat; i++ )
     mask_vec[i] = i;
-  gridDefRowlon ( gridID, nlon*nlat, mp );
-  gridDefComplexPacking ( gridID, 1 );	       
+  gridDefReducedPoints ( gridID, nlon*nlat, mp );
+  gridDefComplexPacking ( gridID, 1 );
 
   return gridID;
 }
@@ -86,11 +90,13 @@ static int defineZaxis ()
   zaxisID = zaxisCreate(ZAXIS_PRESSURE, nlev);
   zaxisDefLevels(zaxisID, levs);
   zaxisDefLevel ( zaxisID, 2, 8507.3 );
-  zaxisDefName ( zaxisID, "myName" );
-  zaxisDefLongname ( zaxisID, "myLongname" );
-  zaxisDefUnits ( zaxisID, "myUnits" );
+
+  cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, "myName");
+  cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, "myLongname");
+  cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, "myUnits");
+
   zaxisDefDatatype ( zaxisID, DOUBLE_PRECISION );
-  zaxisDefLtype ( zaxisID, 1 );
+  cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, 1);
   zaxisDefVct ( zaxisID, 3, vct );
   zaxisDefLbounds ( zaxisID, &levs[0] );
   zaxisDefUbounds ( zaxisID, &levs[0] );
@@ -101,10 +107,8 @@ static int defineZaxis ()
 
 static int defineTaxis ()
 {
-  int taxisID = CDI_UNDEFID;
+  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
 
-  taxisID = taxisCreate(TAXIS_ABSOLUTE);
-  
   taxisDefType  ( taxisID, 0 );
   taxisDefVdate ( taxisID, 1 );
   taxisDefVtime ( taxisID, 2 );
@@ -198,8 +202,7 @@ static void modelRun ( MPI_Comm comm )
   reshPackBufferCreate(&sendBuffer, &bufferSize, &comm);
   xmpi ( MPI_Send ( sendBuffer, bufferSize, MPI_PACKED, 0, 0, comm ));
   recvBuffer = Malloc((size_t)bufferSize);
-  xmpi ( MPI_Recv ( recvBuffer, bufferSize, MPI_PACKED, 0,
-		    0, comm, &status ));
+  xmpi ( MPI_Recv ( recvBuffer, bufferSize, MPI_PACKED, 0, 0, comm, &status ));
 
   namespaceSetActive ( 1 );
   reshUnpackResources(recvBuffer, bufferSize, &comm);
