@@ -7,7 +7,6 @@
 #if defined(__cplusplus)
 extern "C" {
 #endif
-#include "cdi.h"
 #if defined (__cplusplus)
 }
 #endif
@@ -19,11 +18,12 @@ extern "C" {
  */
 CdiGrid::CdiGrid() { gridID = -1; }
 CdiGrid::CdiGrid(int gridid) {
-  char _name[CHARSIZE];
-  char _xname[CHARSIZE], _xlongname[CHARSIZE], _xstdname[CHARSIZE], _xunits[CHARSIZE];
-  char _yname[CHARSIZE], _ylongname[CHARSIZE], _ystdname[CHARSIZE], _yunits[CHARSIZE];
+  char _name[CDI_MAX_NAME];
+  char _xname[CDI_MAX_NAME], _xlongname[CDI_MAX_NAME], _xunits[CDI_MAX_NAME];
+  char _yname[CDI_MAX_NAME], _ylongname[CDI_MAX_NAME], _yunits[CDI_MAX_NAME];
 
   gridID = gridid;
+  //std::cout << "DEBUG:input gridid =" << gridid << std::endl;
   type   = gridInqType(gridID);
   /*  strcpy(_name,gridNamePtr(type)); */
   name = std::string(gridNamePtr(type));
@@ -33,7 +33,7 @@ CdiGrid::CdiGrid(int gridid) {
   ysize = gridInqYsize(gridID);
   xvalues.resize(xsize);
   yvalues.resize(ysize);
-//  std::cout << "(gridID:" << gridID<< ") xsize: " << xsize << " ysize:" << ysize << std::endl;
+  // std::cout << "DEBUG:(gridID:" << gridID<< ") xsize: " << xsize << " ysize:" << ysize << std::endl;
 
   getValues();
   getBounds();
@@ -42,19 +42,15 @@ CdiGrid::CdiGrid(int gridid) {
 
   gridInqXname(gridID    , _xname);
   gridInqXlongname(gridID, _xlongname);
-  gridInqXstdname(gridID , _xstdname);
   gridInqXunits(gridID   , _xunits);
   gridInqYname(gridID    , _yname);
   gridInqYlongname(gridID, _ylongname);
-  gridInqYstdname(gridID , _ystdname);
   gridInqYunits(gridID   , _yunits);
   xname     = _xname;
   xlongname = _xlongname;
-  xstdname  = _xstdname;
   xunits    = _xunits;
   yname     = _yname;
   ylongname = _ylongname;
-  ystdname  = _ystdname;
   yunits    = _yunits;
 
 }
@@ -208,13 +204,14 @@ CdiTaxis::~CdiTaxis() { if (taxisID >= 0) taxisID = -1; }
  */
 CdiZaxis::CdiZaxis() { zaxisID = -1; }
 CdiZaxis::CdiZaxis(int zaxisid) {
-  char name[CHARSIZE], longname[CHARSIZE], units[CHARSIZE];
+  char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
 
   zaxisID = zaxisid;
   size    = zaxisInqSize(zaxisID);
   prec    = zaxisInqDatatype(zaxisID);
   type    = zaxisInqType(zaxisID);
-  ltype   = zaxisInqLtype(zaxisID);
+  ltype   = 0;
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
 
   plevels  = (double *) malloc(size*sizeof(double));
   plbounds = (double *) malloc(size*sizeof(double));
@@ -243,7 +240,7 @@ CdiZaxis::~CdiZaxis() { if (zaxisID >= 0) zaxisID = -1; }
  */
 CdiVariable::CdiVariable() { size = -1; }
 CdiVariable::CdiVariable(int streamid,int vlistid, int varid) {
-  char _name[CHARSIZE],_longname[CHARSIZE], _units[CHARSIZE], _stdname[CHARSIZE];
+  char _name[CDI_MAX_NAME],_longname[CDI_MAX_NAME], _units[CDI_MAX_NAME], _stdname[CDI_MAX_NAME];
   streamID = streamid;
   vlistID  = vlistid;
   varID    = varid;
@@ -265,7 +262,12 @@ CdiVariable::CdiVariable(int streamid,int vlistid, int varid) {
 CdiVariable::~CdiVariable(){ size = -1; }
 
 void
-CdiVariable::sinfo() { std::cout << "code:"  << std::endl; }
+CdiVariable::sinfo() {
+  std::cout << "name:"  << name << std::endl;
+  std::cout << "longname:"  << longname << std::endl;
+  std::cout << "units:"  << units << std::endl;
+  std::cout << "code:"  << code << std::endl;
+}
 
 double *
 CdiVariable::getValuesAsPointer()
@@ -353,6 +355,11 @@ CdiVariable::getFValuesWithLevel(int tsID)
  */
 Cdi::Cdi(const char *path)  {
   streamID = streamOpenRead(path);
+
+  if (0 > streamID) {
+    std::cout << "Failed to read input '" << path <<"'";
+    std::cout << "Got streamID = " << streamID;
+  }
   vlistID  = streamInqVlist(streamID);
   nvars    = vlistNvars(vlistID);
 
@@ -362,7 +369,9 @@ Cdi::Cdi(const char *path)  {
   getVars();
 }
 
-Cdi::~Cdi() {}
+Cdi::~Cdi() {
+  streamClose(streamID);
+}
 
 void
 Cdi::getTaxes() {
@@ -388,17 +397,18 @@ void
 Cdi::getGrids() {
   int gridID;
   ngrids = vlistNgrids(vlistID);
+    std::cout << "ngrids=" << ngrids << std::endl;
   for (int i = 0; i < ngrids; i++)
   {
     gridID        = vlistGrid(vlistID, i);
-//    std::cout << "getGrids : gridID=" << gridID << std::endl;
+    std::cout << "getGrids : gridID=" << gridID << std::endl;
     grids[gridID] = CdiGrid(gridID);
   }
 }
 
 void
 Cdi::getVars() {
-  char name[CHARSIZE];
+  char name[CDI_MAX_NAME];
   int varID, code;
 //  std::cout << vlistID << std::endl;
   for (varID = 0; varID < nvars; varID++)
