@@ -27,25 +27,12 @@ int days_per_month(int calendar, int year, int month)
 {
   int daysperyear = calendar_dpy(calendar);
 
-  const int *dpm;
-  if      ( daysperyear == 360 ) dpm = month_360;
-  else if ( daysperyear == 365 ) dpm = month_365;
-  else                           dpm = month_366;
+  const int *dpm = (daysperyear == 360) ? month_360 : ((daysperyear == 365) ? month_365 : month_366);
 
-  int dayspermonth = 0;
-  if ( month >= 1 && month <= 12 )
-    dayspermonth = dpm[month-1];
-  /*
-  else
-    fprintf(stderr, "days_per_month: month %d out of range\n", month);
-  */
-  if ( daysperyear == 0 && month == 2 )
-    {
-      if ( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
-	dayspermonth = 29;
-      else
-	dayspermonth = 28;
-    }
+  int dayspermonth = ( month >= 1 && month <= 12 ) ? dpm[month-1] : 0;
+
+  if (daysperyear == 0 && month == 2)
+    dayspermonth = ((year%4 == 0 && year%100 != 0) || year%400 == 0) ? 29 : 28;
 
   return dayspermonth;
 }
@@ -55,11 +42,11 @@ int days_per_year(int calendar, int year)
 {
   int daysperyear = calendar_dpy(calendar);
 
-  if ( daysperyear == 0 )
+  if (daysperyear == 0)
     {
-      if ( year == 1582 && (calendar == CALENDAR_STANDARD || calendar == CALENDAR_GREGORIAN) )
+      if (year == 1582 && (calendar == CALENDAR_STANDARD || calendar == CALENDAR_GREGORIAN))
         daysperyear = 355;
-      else if ( (year%4 == 0 && year%100 != 0) || year%400 == 0 )
+      else if ((year%4 == 0 && year%100 != 0) || year%400 == 0)
         daysperyear = 366;
       else
         daysperyear = 365;
@@ -71,8 +58,6 @@ int days_per_year(int calendar, int year)
 
 static void decode_day(int dpy, int days, int *year, int *month, int *day)
 {
-  int i = 0;
-
   *year = (days-1) / dpy;
   days -= (*year*dpy);
 
@@ -81,6 +66,7 @@ static void decode_day(int dpy, int days, int *year, int *month, int *day)
   else if ( dpy == 365 ) dpm = month_365;
   else if ( dpy == 366 ) dpm = month_366;
 
+  int i = 0;
   if ( dpm )
     for ( i = 0; i < 12; i++ )
       {
@@ -93,9 +79,9 @@ static void decode_day(int dpy, int days, int *year, int *month, int *day)
 }
 
 
-static int encode_day(int dpy, int year, int month, int day)
+static int64_t encode_day(int dpy, int year, int month, int day)
 {
-  long rval = (long)dpy * year + day;
+  int64_t rval = (int64_t)dpy * year + day;
 
   const int *dpm = NULL;
   if      ( dpy == 360 ) dpm = month_360;
@@ -103,17 +89,16 @@ static int encode_day(int dpy, int year, int month, int day)
   else if ( dpy == 366 ) dpm = month_366;
 
   if ( dpm ) for ( int i = 0; i < month-1; i++ ) rval += dpm[i];
-  if ( rval > INT_MAX || rval < INT_MIN )
-    Error("Unhandled date: %ld", rval);
+  if ( rval > LONG_MAX || rval < LONG_MIN ) Error("Unhandled date: %lld", rval);
 
-  return (int)rval;
+  return rval;
 }
 
 
 void encode_caldaysec(int calendar, int year, int month, int day, int hour, int minute, int second,
-		      int *julday, int *secofday)
+		      int64_t *julday, int *secofday)
 {
-  int dpy = calendar_dpy(calendar);
+  const int dpy = calendar_dpy(calendar);
 
   if ( dpy == 360 || dpy == 365 || dpy == 366 )
     *julday = encode_day(dpy, year, month, day);
@@ -124,10 +109,10 @@ void encode_caldaysec(int calendar, int year, int month, int day, int hour, int
 }
 
 
-void decode_caldaysec(int calendar, int julday, int secofday,
+void decode_caldaysec(int calendar, int64_t julday, int secofday,
 		      int *year, int *month, int *day, int *hour, int *minute, int *second)
 {
-  int dpy = calendar_dpy(calendar);
+  const int dpy = calendar_dpy(calendar);
 
   if ( dpy == 360 || dpy == 365 || dpy == 366 )
     decode_day(dpy, julday, year, month, day);
@@ -143,7 +128,7 @@ void decode_caldaysec(int calendar, int julday, int secofday,
 #ifdef TEST
 static int date_to_calday(int calendar, int date)
 {
-  int dpy = calendar_dpy(calendar);
+  const int dpy = calendar_dpy(calendar);
 
   int year, month, day;
   cdiDecodeDate(date, &year, &month, &day);
@@ -161,14 +146,14 @@ static int date_to_calday(int calendar, int date)
 static int calday_to_date(int calendar, int calday)
 {
   int year, month, day;
-  int dpy = calendar_dpy(calendar);
+  const int dpy = calendar_dpy(calendar);
 
   if ( dpy == 360 || dpy == 365 || dpy == 366 )
     decode_day(dpy, calday, &year, &month, &day);
   else
     decode_julday(calendar, calday, &year, &month, &day);
 
-  int date = cdiEncodeDate(year, month, day);
+  const int date = cdiEncodeDate(year, month, day);
 
   return date;
 }
@@ -178,8 +163,8 @@ int main(void)
 {
   int calendar = CALENDAR_STANDARD;
   int nmin;
-  int vdate0, vtime0;
-  int vdate, vtime;
+  int64_t vdate0, vdate;
+  int vtime0, vtime;
   int ijulinc;
   int i, j = 0;
   int year, mon, day, hour, minute, second;
@@ -191,7 +176,7 @@ int main(void)
   vdate0 = -80001201;
   vtime0 = 120500;
 
-  printf("start time: %8d %4d\n", vdate0, vtime0);
+  printf("start time: %8ld %4d\n", vdate0, vtime0);
 
   for ( i = 0; i < nmin; i++ )
     {
@@ -205,7 +190,7 @@ int main(void)
       vtime = sec_to_time(secofday);
 
       if ( vdate0 != vdate || vtime0 != vtime )
-	printf("%4d %8d %4d %8d %4d %9d %9d\n",
+	printf("%4d %8ld %4d %8ld %4d %9d %9d\n",
 	       ++j, vdate0, vtime0, vdate, vtime, calday, secofday);
 
       year++;
@@ -213,7 +198,7 @@ int main(void)
       vtime0 = cdiEncodeTime(hour, minute, second);
     }
 
-  printf("stop time: %8d %4d\n", vdate0, vtime0);
+  printf("stop time: %8ld %4d\n", vdate0, vtime0);
 
   /* 2 - Check time increment of one minute */
 
@@ -222,7 +207,7 @@ int main(void)
   vdate0 = 20001201;
   vtime0 = 0;
 
-  printf("start time: %8d %4d\n", vdate0, vtime0);
+  printf("start time: %8ld %4d\n", vdate0, vtime0);
 
   calday = date_to_calday(calendar, vdate0);
   secofday = time_to_sec(vtime0);
@@ -257,13 +242,13 @@ int main(void)
       vdate = calday_to_date(calendar, calday);
       vtime = sec_to_time(secofday);
       if ( vdate0 != vdate || vtime0 != vtime )
-	printf("%4d %8d %4d %8d %4d %9d %9d\n",
+	printf("%4d %8ld %4d %8ld %4d %9d %9d\n",
 	       ++j, vdate0, vtime0, vdate, vtime, calday, secofday);
     }
 
-  printf("stop time: %8d %4d\n", vdate0, vtime0);
+  printf("stop time: %8ld %4d\n", vdate0, vtime0);
 
-  return (0);
+  return 0;
 }
 #endif
 
@@ -298,7 +283,7 @@ int main(void)
       julday_add_seconds(value*factor, &calday, &secofday);
     }
 
-  return (0);
+  return 0;
 }
 #endif
 /*
