@@ -80,7 +80,7 @@ void cdfComment(int ncid)
 	strcat(comment, "??");
       else
 	strcat(comment, libvers);
-      strcat(comment, " (http://mpimet.mpg.de/cdi)");
+      strcat(comment, " (https://mpimet.mpg.de/cdi)");
     }
 
   cdf_put_att_text(ncid, NC_GLOBAL, "CDI", strlen(comment), comment);
@@ -103,7 +103,7 @@ static int cdfOpenFile(const char *filename, const char *mode, int *filetype)
 	{
 	case 'r':
           {
-            int status = cdf_open(filename, readmode, &ncid);
+            const int status = cdf_open(filename, readmode, &ncid);
             if ( status > 0 && ncid < 0 ) ncid = CDI_ESYSTEM;
 #ifdef  HAVE_NETCDF4
             else
