@@ -1,17 +1,18 @@
-#ifndef _CDF_H
-#define _CDF_H
+#ifndef  CDF_H
+#define  CDF_H
 
 void cdfDebug(int debug);
 
+extern int CDF_Debug;
+
 const char *cdfLibraryVersion(void);
 const char *hdfLibraryVersion(void);
 
-int  cdfOpen(const char *filename, const char *mode);
-int  cdfOpen64(const char *filename, const char *mode);
+int  cdfOpen(const char *filename, const char *mode, int filetype);
 int  cdf4Open(const char *filename, const char *mode, int *filetype);
 void cdfClose(int fileID);
 
-#endif  /* _CDF_H */
+#endif  /* CDF_H */
 /*
  * Local Variables:
  * c-file-style: "Java"
