@@ -10,6 +10,7 @@
 #include "dmemory.h"
 #include "cdi.h"
 #include "cdi_int.h"
+#include "stream_grb.h"
 #include "stream_cdf.h"
 #include "cdf_int.h"
 #include "vlist.h"
@@ -83,13 +84,13 @@ void cdfGetSlapDescription(stream_t *streamptr, int varID, size_t (*start)[4], s
       Message("dim = %d  start = %d  count = %d", idim, start[idim], count[idim]);
 }
 
-//Scans the data array for missVals, optionally applying first a scale factor and then an offset.
-//Returns the number of missing + out-of-range values encountered.
+// Scans the data array for missVals, optionally applying first a scale factor and then an offset.
+// Returns the number of missing + out-of-range values encountered.
 static
 size_t cdfDoInputDataTransformationDP(int vlistID, int varID, size_t valueCount, double *data)
 {
   double missVal = vlistInqVarMissval(vlistID, varID);
-  const bool haveMissVal = vlistInqVarMissvalUsed(vlistID, varID);
+  const int haveMissVal = vlistInqVarMissvalUsed(vlistID, varID);
   double validRange[2];
   if (!(haveMissVal && vlistInqVarValidrange(vlistID, varID, validRange)))
     validRange[0] = DBL_MIN, validRange[1] = DBL_MAX;
@@ -98,20 +99,20 @@ size_t cdfDoInputDataTransformationDP(int vlistID, int varID, size_t valueCount,
   double offset = vlistInqVarAddoffset(vlistID, varID);
   double scaleFactor = vlistInqVarScalefactor(vlistID, varID);
 
-  const bool haveOffset = IS_NOT_EQUAL(offset, 0);
-  const bool haveScaleFactor = IS_NOT_EQUAL(scaleFactor, 1);
+  const bool missValIsNaN = DBL_IS_NAN(missVal);
+  const int haveOffset = IS_NOT_EQUAL(offset, 0.0);
+  const int haveScaleFactor = IS_NOT_EQUAL(scaleFactor, 1.0);
   size_t missValCount = 0;
 
   if ( IS_EQUAL(validMin, VALIDMISS) ) validMin = DBL_MIN;
   if ( IS_EQUAL(validMax, VALIDMISS) ) validMax = DBL_MAX;
 
-  bool haveRangeCheck = (IS_NOT_EQUAL(validMax, DBL_MAX)) | (IS_NOT_EQUAL(validMin,DBL_MIN));
+  const int haveRangeCheck = (IS_NOT_EQUAL(validMax, DBL_MAX)) | (IS_NOT_EQUAL(validMin, DBL_MIN));
   assert(!haveRangeCheck || haveMissVal);
 
-  switch ((int)haveMissVal | ((int)haveScaleFactor << 1)
-          | ((int)haveOffset << 2) | ((int)haveRangeCheck << 3))
+  switch (haveMissVal | (haveScaleFactor << 1) | (haveOffset << 2) | (haveRangeCheck << 3))
     {
-    case 15: /* haveRangeCheck & haveMissVal & haveScaleFactor & haveOffset */
+    case 15: // haveRangeCheck & haveMissVal & haveScaleFactor & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -121,7 +122,7 @@ size_t cdfDoInputDataTransformationDP(int vlistID, int varID, size_t valueCount,
             : isMissVal ? data[i] : data[i] * scaleFactor + offset;
         }
       break;
-    case 13: /* haveRangeCheck & haveMissVal & haveOffset */
+    case 13: // haveRangeCheck & haveMissVal & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -131,7 +132,7 @@ size_t cdfDoInputDataTransformationDP(int vlistID, int varID, size_t valueCount,
             : isMissVal ? data[i] : data[i] + offset;
         }
       break;
-    case 11: /* haveRangeCheck & haveMissVal & haveScaleFactor */
+    case 11: // haveRangeCheck & haveMissVal & haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -141,7 +142,7 @@ size_t cdfDoInputDataTransformationDP(int vlistID, int varID, size_t valueCount,
             : isMissVal ? data[i] : data[i] * scaleFactor;
         }
       break;
-    case 9: /* haveRangeCheck & haveMissVal */
+    case 9: // haveRangeCheck & haveMissVal
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -150,42 +151,50 @@ size_t cdfDoInputDataTransformationDP(int vlistID, int varID, size_t valueCount,
           data[i] = outOfRange ? missVal : data[i];
         }
       break;
-    case 7: /* haveMissVal & haveScaleFactor & haveOffset */
+    case 7: // haveMissVal & haveScaleFactor & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         if ( DBL_IS_EQUAL(data[i], missVal) )
           missValCount++;
         else
           data[i] = data[i] * scaleFactor + offset;
       break;
-    case 6: /* haveOffset & haveScaleFactor */
+    case 6: // haveOffset & haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         data[i] = data[i] * scaleFactor + offset;
       break;
-    case 5: /* haveMissVal & haveOffset */
+    case 5: // haveMissVal & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         if ( DBL_IS_EQUAL(data[i], missVal) )
           missValCount++;
         else
           data[i] += offset;
       break;
-    case 4: /* haveOffset */
+    case 4: // haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         data[i] += offset;
       break;
-    case 3: /* haveMissVal & haveScaleFactor */
+    case 3: // haveMissVal & haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         if ( DBL_IS_EQUAL(data[i], missVal) )
           missValCount++;
         else
           data[i] *= scaleFactor;
       break;
-    case 2: /* haveScaleFactor */
+    case 2: // haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         data[i] *= scaleFactor;
       break;
-    case 1: /* haveMissVal */
-      for ( size_t i = 0; i < valueCount; i++ )
-        missValCount += (unsigned)DBL_IS_EQUAL(data[i], missVal);
+    case 1: // haveMissVal
+      if (missValIsNaN)
+        {
+          for ( size_t i = 0; i < valueCount; i++ )
+            missValCount += (size_t)DBL_IS_NAN(data[i]);
+        }
+      else
+        {
+          for ( size_t i = 0; i < valueCount; i++ )
+            missValCount += (size_t)IS_EQUAL(data[i], missVal);
+        }
       break;
     }
 
@@ -196,7 +205,7 @@ static
 size_t cdfDoInputDataTransformationSP(int vlistID, int varID, size_t valueCount, float *data)
  {
   double missVal = vlistInqVarMissval(vlistID, varID);
-  const bool haveMissVal = vlistInqVarMissvalUsed(vlistID, varID);
+  const int haveMissVal = vlistInqVarMissvalUsed(vlistID, varID);
   double validRange[2];
   if (!(haveMissVal && vlistInqVarValidrange(vlistID, varID, validRange)))
     validRange[0] = DBL_MIN, validRange[1] = DBL_MAX;
@@ -205,20 +214,20 @@ size_t cdfDoInputDataTransformationSP(int vlistID, int varID, size_t valueCount,
   double offset = vlistInqVarAddoffset(vlistID, varID);
   double scaleFactor = vlistInqVarScalefactor(vlistID, varID);
 
-  const bool haveOffset = IS_NOT_EQUAL(offset, 0);
-  const bool haveScaleFactor = IS_NOT_EQUAL(scaleFactor, 1);
+  const bool missValIsNaN = DBL_IS_NAN(missVal);
+  const int haveOffset = IS_NOT_EQUAL(offset, 0.0);
+  const int haveScaleFactor = IS_NOT_EQUAL(scaleFactor, 1.0);
   size_t missValCount = 0;
 
   if ( IS_EQUAL(validMin, VALIDMISS) ) validMin = DBL_MIN;
   if ( IS_EQUAL(validMax, VALIDMISS) ) validMax = DBL_MAX;
 
-  bool haveRangeCheck = (IS_NOT_EQUAL(validMax, DBL_MAX)) | (IS_NOT_EQUAL(validMin,DBL_MIN));
+  const int haveRangeCheck = (IS_NOT_EQUAL(validMax, DBL_MAX)) | (IS_NOT_EQUAL(validMin, DBL_MIN));
   assert(!haveRangeCheck || haveMissVal);
 
-  switch ((int)haveMissVal | ((int)haveScaleFactor << 1)
-          | ((int)haveOffset << 2) | ((int)haveRangeCheck << 3))
+  switch (haveMissVal | (haveScaleFactor << 1) | (haveOffset << 2) | (haveRangeCheck << 3))
     {
-    case 15: /* haveRangeCheck & haveMissVal & haveScaleFactor & haveOffset */
+    case 15: // haveRangeCheck & haveMissVal & haveScaleFactor & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -228,7 +237,7 @@ size_t cdfDoInputDataTransformationSP(int vlistID, int varID, size_t valueCount,
             : isMissVal ? data[i] : (float)(data[i] * scaleFactor + offset);
         }
       break;
-    case 13: /* haveRangeCheck & haveMissVal & haveOffset */
+    case 13: // haveRangeCheck & haveMissVal & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -238,7 +247,7 @@ size_t cdfDoInputDataTransformationSP(int vlistID, int varID, size_t valueCount,
             : isMissVal ? data[i] : (float)(data[i] + offset);
         }
       break;
-    case 11: /* haveRangeCheck & haveMissVal & haveScaleFactor */
+    case 11: // haveRangeCheck & haveMissVal & haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -248,7 +257,7 @@ size_t cdfDoInputDataTransformationSP(int vlistID, int varID, size_t valueCount,
             : isMissVal ? data[i] : (float)(data[i] * scaleFactor);
         }
       break;
-    case 9: /* haveRangeCheck & haveMissVal */
+    case 9: // haveRangeCheck & haveMissVal
       for ( size_t i = 0; i < valueCount; i++ )
         {
           int outOfRange = data[i] < validMin || data[i] > validMax;
@@ -257,42 +266,50 @@ size_t cdfDoInputDataTransformationSP(int vlistID, int varID, size_t valueCount,
           data[i] = outOfRange ? (float)missVal : data[i];
         }
       break;
-    case 7: /* haveMissVal & haveScaleFactor & haveOffset */
+    case 7: // haveMissVal & haveScaleFactor & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         if ( DBL_IS_EQUAL(data[i], missVal) )
           missValCount++;
         else
           data[i] = (float)(data[i] * scaleFactor + offset);
       break;
-    case 6: /* haveOffset & haveScaleFactor */
+    case 6: // haveOffset & haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         data[i] = (float)(data[i] * scaleFactor + offset);
       break;
-    case 5: /* haveMissVal & haveOffset */
+    case 5: // haveMissVal & haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         if ( DBL_IS_EQUAL(data[i], missVal) )
           missValCount++;
         else
           data[i] = (float)(data[i] + offset);
       break;
-    case 4: /* haveOffset */
+    case 4: // haveOffset
       for ( size_t i = 0; i < valueCount; i++ )
         data[i] = (float)(data[i] + offset);
       break;
-    case 3: /* haveMissVal & haveScaleFactor */
+    case 3: // haveMissVal & haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         if ( DBL_IS_EQUAL(data[i], missVal) )
           missValCount++;
         else
           data[i] = (float)(data[i] * scaleFactor);
       break;
-    case 2: /* haveScaleFactor */
+    case 2: // haveScaleFactor
       for ( size_t i = 0; i < valueCount; i++ )
         data[i] = (float)(data[i] * scaleFactor);
       break;
-    case 1: /* haveMissVal */
-      for ( size_t i = 0; i < valueCount; i++ )
-        missValCount += (unsigned)DBL_IS_EQUAL(data[i], missVal);
+    case 1: // haveMissVal
+      if (missValIsNaN)
+        {
+          for ( size_t i = 0; i < valueCount; i++ )
+            missValCount += (size_t)DBL_IS_NAN(data[i]);
+        }
+      else
+        {
+          for ( size_t i = 0; i < valueCount; i++ )
+            missValCount += (size_t)IS_EQUAL(data[i], missVal);
+        }
       break;
     }
 
@@ -390,6 +407,10 @@ void cdfInqDimIds(stream_t *streamptr, int varId, int (*outDimIds)[3])
         (*outDimIds)[0] = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
         break;
 
+      case GRID_GAUSSIAN_REDUCED:
+        (*outDimIds)[0] = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
+        break;
+
       default:
         (*outDimIds)[0] = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
         (*outDimIds)[1] = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
@@ -430,28 +451,28 @@ int cdfGetSkipDim(int fileId, int ncvarid, int (*dimIds)[3])
 }
 
 static
-void cdfGetSliceSlapDescription(stream_t *streamptr, int varId, int levelId, bool *outSwapXY, size_t (*start)[4], size_t (*count)[4])
+void cdfGetSliceSlapDescription(stream_t *streamptr, int varID, int levelID, bool *outSwapXY, size_t (*start)[4], size_t (*count)[4])
 {
   const int tsID = streamptr->curTsID;
   if ( CDI_Debug ) Message("tsID = %d", tsID);
 
   const int fileId = streamptr->fileID;
-  const int vlistId = streamptr->vlistID;
-  const int ncvarid = streamptr->vars[varId].ncvarid;
+  const int vlistID = streamptr->vlistID;
+  const int ncvarid = streamptr->vars[varID].ncvarid;
 
-  const int gridId = vlistInqVarGrid(vlistId, varId);
-  const int timetype = vlistInqVarTimetype(vlistId, varId);
+  const int gridId = vlistInqVarGrid(vlistID, varID);
+  const int timetype = vlistInqVarTimetype(vlistID, varID);
   const size_t gridsize = gridInqSize(gridId);
 
   streamptr->numvals += gridsize;
 
   int dimIds[3]; // this array joins the old variables xid, yid, and zid
-  cdfInqDimIds(streamptr, varId, &dimIds);
+  cdfInqDimIds(streamptr, varID, &dimIds);
 
   const int skipdim = cdfGetSkipDim(fileId, ncvarid, &dimIds);
 
   int dimorder[3];
-  vlistInqVarDimorder(vlistId, varId, &dimorder);
+  vlistInqVarDimorder(vlistID, varID, &dimorder);
 
   *outSwapXY = (dimorder[2] == 2 || dimorder[0] == 1) && dimIds[0] != CDI_UNDEFID && dimIds[1] != CDI_UNDEFID ;
 
@@ -479,7 +500,7 @@ void cdfGetSliceSlapDescription(stream_t *streamptr, int varId, int levelId, boo
             addDimension(0, size);
             break;
           case 3:
-            addDimension((size_t)levelId, 1);
+            addDimension((size_t)levelID, 1);
             break;
           default:
             Error("Internal errror: Malformed dimension order encountered. Please report this bug.\n");
@@ -501,7 +522,11 @@ void cdfGetSliceSlapDescription(stream_t *streamptr, int varId, int levelId, boo
   cdf_inq_varndims(fileId, ncvarid, &nvdims);
 
   if ( nvdims != ndims )
-    Error("Internal error, variable %s has an unsupported array structure!", vlistInqVarNamePtr(vlistId, varId));
+    {
+      char name[CDI_MAX_NAME];
+      vlistInqVarName(vlistID, varID, name);
+      Error("Internal error, variable %s has an unsupported array structure!", name);
+    }
 }
 
 static
@@ -567,16 +592,32 @@ void cdfReadDataDP(stream_t *streamptr, int varID, size_t length, size_t start[4
   const int vlistID = streamptr->vlistID;
   const int fileID = streamptr->fileID;
   const int ncvarid = streamptr->vars[varID].ncvarid;
+  const int datatype = vlistInqVarDatatype(vlistID, varID);
 
-  if ( vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_FLT32 )
+  if (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64)
     {
-      cdfReadDataSliceSP2DP(fileID, ncvarid, length, start, count, data);
+      cdf_get_vara(fileID, ncvarid, start, count, data);
+      if (datatype == CDI_DATATYPE_CPX32)
+        {
+          for (long i = (long)length-1; i >= 0; i--)
+            {
+              data[2*i] = (double)(((float*)data)[2*i]);
+              data[2*i+1] = (double)(((float*)data)[2*i+1]);
+            }
+        }
     }
   else
     {
-      cdf_get_vara_double(fileID, ncvarid, start, count, data);
+      if ( datatype == CDI_DATATYPE_FLT32 )
+        {
+          cdfReadDataSliceSP2DP(fileID, ncvarid, length, start, count, data);
+        }
+      else
+        {
+          cdf_get_vara_double(fileID, ncvarid, start, count, data);
       
-      cdfCheckDataDP_UINT8(fileID, ncvarid, vlistID, varID, length, data);
+          cdfCheckDataDP_UINT8(fileID, ncvarid, vlistID, varID, length, data);
+        }
     }
 }
 
@@ -586,16 +627,38 @@ void cdfReadDataSP(stream_t *streamptr, int varID, size_t length, size_t start[4
   const int vlistID = streamptr->vlistID;
   const int fileID = streamptr->fileID;
   const int ncvarid = streamptr->vars[varID].ncvarid;
+  const int datatype = vlistInqVarDatatype(vlistID, varID);
 
-  if ( vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_FLT64 )
+  if (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64)
     {
-      cdfReadDataSliceDP2SP(fileID, ncvarid, length, start, count, data);
+      if (datatype == CDI_DATATYPE_CPX64)
+        {
+          double *cdata = (double *) Malloc(2*length*sizeof(double));
+          cdf_get_vara(fileID, ncvarid, start, count, cdata);
+          for (size_t i = 0; i < length; i++)
+            {
+              data[2*i] = (float)(cdata[2*i]);
+              data[2*i+1] = (float)(cdata[2*i+1]);
+            }
+          Free(cdata);
+        }
+      else
+        {
+          cdf_get_vara(fileID, ncvarid, start, count, data);
+        }
     }
   else
     {
-      cdf_get_vara_float(fileID, ncvarid, start, count, data);
+      if ( datatype == CDI_DATATYPE_FLT64 )
+        {
+          cdfReadDataSliceDP2SP(fileID, ncvarid, length, start, count, data);
+        }
+      else
+        {
+          cdf_get_vara_float(fileID, ncvarid, start, count, data);
       
-      cdfCheckDataSP_UINT8(fileID, ncvarid, vlistID, varID, length, data);
+          cdfCheckDataSP_UINT8(fileID, ncvarid, vlistID, varID, length, data);
+        }
     }
 }
 
@@ -727,9 +790,8 @@ void cdfReadVarSliceDPPart(stream_t *streamptr, int varID, int levelID, int varT
   const size_t gridsize = gridInqSize(gridId);
 
   unsigned int position = 0; 
-  for (int i=0 ; i<4 ; i++)
-    if (count[i] == gridsize)
-      position = i;
+  for (int i = 0; i < 4; i++)
+    if (count[i] == gridsize) position = i;
 
   start[position] = start[position]+startpoint;
   count[position] = length;
@@ -758,9 +820,8 @@ void cdfReadVarSliceSPPart(stream_t *streamptr, int varID, int levelID, int varT
   size_t gridsize = gridInqSize(gridId);
 
   unsigned int position = 0; 
-  for (int i=0 ; i<4 ; i++)
-    if (count[i] == gridsize)
-      position = i;
+  for (int i = 0; i < 4; i++)
+    if (count[i] == gridsize) position = i;
 
   start[position] = start[position]+startpoint;
   count[position] = length;
@@ -790,7 +851,15 @@ int cdiStreamReadVarSlicePart(int streamID, int varID, int levelID, int varType,
   // currently we only care for netcdf data
   switch (filetype)
     {
-#if defined (HAVE_LIBNETCDF)
+#ifdef HAVE_LIBGRIB
+    case CDI_FILETYPE_GRB:
+    case CDI_FILETYPE_GRB2:
+      {
+        grb_read_var_slice(streamptr, varID, levelID, memtype, data, nmiss);
+	break;
+      }
+#endif
+#ifdef HAVE_LIBNETCDF
     case CDI_FILETYPE_NC:
     case CDI_FILETYPE_NC2:
     case CDI_FILETYPE_NC4:
@@ -873,7 +942,15 @@ void cdiStreamReadVarPart(int streamID, int varID, int varType, int start, size_
   // currently we only care for netcdf data
   switch (filetype)
     {
-#if defined (HAVE_LIBNETCDF)
+#ifdef HAVE_LIBGRIB
+    case CDI_FILETYPE_GRB:
+    case CDI_FILETYPE_GRB2:
+      {
+        grb_read_var(streamptr, varID, memtype, data, nmiss);
+	break;
+      }
+#endif
+#ifdef HAVE_LIBNETCDF
     case CDI_FILETYPE_NC:
     case CDI_FILETYPE_NC2:
     case CDI_FILETYPE_NC4:
@@ -909,5 +986,4 @@ void streamReadVarPart(int streamID, int varID, int varType, int start, size_t s
   cdiStreamReadVarPart(streamID, varID, varType, start, size, memtype, data, nmiss);
 }
 
-#endif
-
+#endif /* HAVE_LIBNETCDF */
