@@ -13,6 +13,7 @@ bool is_pressure_units(const char *units);
 bool is_DBL_axis(/*const char *units,*/ const char *longname);
 bool is_depth_axis(const char *stdname, const char *longname);
 bool is_height_axis(const char *stdname, const char *longname);
+bool is_altitude_axis(const char *stdname, const char *longname);
 
 bool is_lon_axis(const char *units, const char *stdname);
 bool is_lat_axis(const char *units, const char *stdname);
@@ -20,8 +21,8 @@ bool is_lat_axis(const char *units, const char *stdname);
 bool is_x_axis(const char *units, const char *stdname);
 bool is_y_axis(const char *units, const char *stdname);
 
-void set_gridtype(const char *attstring, int *gridtype);
-void set_zaxistype(const char *attstring, int *zaxistype);
-void set_calendar(const char *attstring, int *calendar);
+void cdf_set_gridtype(const char *attstring, int *gridtype);
+void cdf_set_zaxistype(const char *attstring, int *zaxistype);
+void cdf_set_calendar(const char *attstring, int *calendar);
 
 #endif
