@@ -14,23 +14,22 @@
 #include "vlist_var.h"
 
 
-void cdfDefVarDeflate(int ncid, int ncvarid, int deflate_level)
+void cdfDefVarDeflate(int ncid, int ncvarID, int deflate_level)
 {
-#if  defined(HAVE_NETCDF4)
-  int retval;
-  /* Set chunking, shuffle, and deflate. */
-  const int shuffle = 1;
-  const int deflate = 1;
+#ifdef HAVE_NETCDF4
+  // Set chunking, shuffle, and deflate.
+  const int shuffle = 1, deflate = 1;
 
-  if ( deflate_level < 1 || deflate_level > 9 ) deflate_level = 1;
+  if (deflate_level < 1 || deflate_level > 9) deflate_level = 1;
 
-  if ( (retval = nc_def_var_deflate(ncid, ncvarid, shuffle, deflate, deflate_level)) )
+  int status;
+  if ((status = nc_def_var_deflate(ncid, ncvarID, shuffle, deflate, deflate_level)))
     {
-      Error("nc_def_var_deflate failed, status = %d", retval);
+      Error("nc_def_var_deflate failed; %s", nc_strerror(status));
     }
 #else
   static bool lwarn = true;
-  if ( lwarn )
+  if (lwarn)
     {
       lwarn = false;
       Warning("Deflate compression failed, NetCDF4 not available!");
@@ -38,6 +37,75 @@ void cdfDefVarDeflate(int ncid, int ncvarid, int deflate_level)
 #endif
 }
 
+void cdfDefVarSzip(int ncid, int ncvarID, int pixels_per_block)
+{
+#ifdef HAVE_NC_DEF_VAR_SZIP
+  // Set options_mask.
+  /*
+    H5_SZIP_ALLOW_K13_OPTION_MASK   1
+    H5_SZIP_CHIP_OPTION_MASK        2
+    H5_SZIP_EC_OPTION_MASK          4
+    H5_SZIP_NN_OPTION_MASK          32
+    H5_SZIP_ALL_MASKS (H5_SZIP_CHIP_OPTION_MASK|H5_SZIP_EC_OPTION_MASK|H5_SZIP_NN_OPTION_MASK)
+  */
+  int options_mask = 38;
+  int status;
+  if ((status = nc_def_var_szip(ncid, ncvarID, options_mask, pixels_per_block)))
+    Error("nc_def_var_szip failed; %s", nc_strerror(status));
+#else
+  static bool lwarn = true;
+  if (lwarn)
+    {
+      lwarn = false;
+      Warning("Szip compression failed, NetCDF4/szlib not available!");
+    }
+#endif
+}
+
+#ifdef HAVE_NETCDF4
+static
+nc_type cdfTypeComplexFloat(stream_t *streamptr)
+{
+  if (streamptr->nc_complex_float_id == CDI_UNDEFID)
+    {
+      typedef struct complex_float { float r, i; } complex_float;
+      const int fileID = streamptr->fileID;
+      int nc_complex_id;
+      int status;
+      status = nc_def_compound(fileID, sizeof(complex_float), "complex_float", &nc_complex_id);
+      if (status != NC_NOERR) Error("%s", nc_strerror(status));
+      status = nc_insert_compound(fileID, nc_complex_id, "r", NC_COMPOUND_OFFSET(complex_float, r), NC_FLOAT);
+      if (status != NC_NOERR) Error("%s", nc_strerror(status));
+      status = nc_insert_compound(fileID, nc_complex_id, "i", NC_COMPOUND_OFFSET(complex_float, i), NC_FLOAT);
+      if (status != NC_NOERR) Error("%s", nc_strerror(status));
+      streamptr->nc_complex_float_id = nc_complex_id;
+    }
+
+  return (nc_type) streamptr->nc_complex_float_id;
+}
+
+static
+nc_type cdfTypeComplexDouble(stream_t *streamptr)
+{
+  if (streamptr->nc_complex_double_id == CDI_UNDEFID)
+    {
+      typedef struct complex_double { double r, i; } complex_double;
+      const int fileID  = streamptr->fileID;
+      int nc_complex_id;
+      int status;
+      status = nc_def_compound(fileID, sizeof(complex_double), "complex_double", &nc_complex_id);
+      if (status != NC_NOERR) Error("%s", nc_strerror(status));
+      status = nc_insert_compound(fileID, nc_complex_id, "r", NC_COMPOUND_OFFSET(complex_double, r), NC_DOUBLE);
+      if (status != NC_NOERR) Error("%s", nc_strerror(status));
+      status = nc_insert_compound(fileID, nc_complex_id, "i", NC_COMPOUND_OFFSET(complex_double, i), NC_DOUBLE);
+      if (status != NC_NOERR) Error("%s", nc_strerror(status));
+      streamptr->nc_complex_double_id = nc_complex_id;
+    }
+
+  return (nc_type) streamptr->nc_complex_double_id;
+}
+#endif
+
 nc_type cdfDefDatatype(int datatype, stream_t *streamptr)
 {
   nc_type xtype = NC_FLOAT;
@@ -51,8 +119,8 @@ nc_type cdfDefDatatype(int datatype, stream_t *streamptr)
       else if ( datatype == CDI_DATATYPE_UINT8  ) xtype = NC_UBYTE;
       else if ( datatype == CDI_DATATYPE_UINT16 ) xtype = NC_USHORT;
       else if ( datatype == CDI_DATATYPE_UINT32 ) xtype = NC_UINT;
-      else if ( datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64 )
-        Error("CDI library does not support complex numbers with NetCDF4/HDF5!");
+      else if ( datatype == CDI_DATATYPE_CPX32  ) xtype = cdfTypeComplexFloat(streamptr);
+      else if ( datatype == CDI_DATATYPE_CPX64  ) xtype = cdfTypeComplexDouble(streamptr);
 #else
       else if ( datatype == CDI_DATATYPE_UINT8  ) xtype = NC_SHORT;
       else if ( datatype == CDI_DATATYPE_UINT16 ) xtype = NC_INT;
@@ -87,7 +155,7 @@ void cdfDefVarMissval(stream_t *streamptr, int varID, int dtype, int lcheck)
     {
       const int vlistID = streamptr->vlistID;
       const int fileID  = streamptr->fileID;
-      const int ncvarid = streamptr->vars[varID].ncvarid;
+      const int ncvarID = streamptr->vars[varID].ncvarid;
       const double missval = vlistInqVarMissval(vlistID, varID);
 
       if ( lcheck && streamptr->ncmode == 2 ) cdf_redef(fileID);
@@ -100,9 +168,9 @@ void cdfDefVarMissval(stream_t *streamptr, int varID, int dtype, int lcheck)
            streamptr->filetype == CDI_FILETYPE_NC ||
            streamptr->filetype == CDI_FILETYPE_NC2||
            streamptr->filetype == CDI_FILETYPE_NC5 )
-        cdf_put_att_double(fileID, ncvarid, "_FillValue", xtype, 1, &missval);
+        cdf_put_att_double(fileID, ncvarID, "_FillValue", xtype, 1, &missval);
 
-      cdf_put_att_double(fileID, ncvarid, "missing_value", xtype, 1, &missval);
+      cdf_put_att_double(fileID, ncvarID, "missing_value", xtype, 1, &missval);
 
       if ( lcheck && streamptr->ncmode == 2 ) cdf_enddef(fileID);
 
@@ -111,7 +179,7 @@ void cdfDefVarMissval(stream_t *streamptr, int varID, int dtype, int lcheck)
 }
 
 static
-void cdfDefInstitut(const stream_t *streamptr)
+void cdfDefInstituteGlobal(const stream_t *streamptr)
 {
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
@@ -134,7 +202,7 @@ void cdfDefInstitut(const stream_t *streamptr)
 }
 
 static
-void cdfDefSource(const stream_t *streamptr)
+void cdfDefSourceGlobal(const stream_t *streamptr)
 {
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
@@ -167,6 +235,33 @@ void *resizeBuf(void **buf, size_t *bufSize, size_t reqSize)
   return *buf;
 }
 
+static
+void cdfDefineCellMethods(stream_t *streamptr, int cdiID, int varID, int fileID, int ncvarID)
+{
+  taxis_t *taxis = &streamptr->tsteps[0].taxis;
+  if (!taxis->has_bounds) return;
+
+  const int time_varid = streamptr->basetime.ncvarid;
+  char timeVarName[CDI_MAX_NAME];
+  cdf_inq_varname(fileID, time_varid, timeVarName);
+
+  const int stepType = vlistInqVarTsteptype(cdiID, varID);
+
+  const char *cellMethod = NULL;
+  if      (stepType == TSTEP_AVG)   cellMethod = "mean";
+  else if (stepType == TSTEP_SUM)   cellMethod = "sum";
+  else if (stepType == TSTEP_RANGE) cellMethod = "range";
+  else if (stepType == TSTEP_MIN)   cellMethod = "minimum";
+  else if (stepType == TSTEP_MAX)   cellMethod = "maximum";
+
+  if (cellMethod)
+    {
+      const char *attname = "cell_methods";
+      char atttxt[CDI_MAX_NAME+10];
+      sprintf(atttxt, "%s: %s", timeVarName, cellMethod);
+      cdf_put_att_text(fileID, ncvarID, attname, strlen(atttxt), atttxt);
+    }
+}
 
 void cdfDefineAttributes(int cdiID, int varID, int fileID, int ncvarID)
 {
@@ -187,7 +282,7 @@ void cdfDefineAttributes(int cdiID, int varID, int fileID, int ncvarID)
 
       if ( atttype == CDI_DATATYPE_TXT )
         {
-          size_t attSize = (size_t)attlen*sizeof(char);
+          const size_t attSize = (size_t)attlen*sizeof(char);
           char *atttxt = (char *)resizeBuf(&attBuf, &attBufSize, attSize);
           cdiInqAttTxt(cdiID, varID, attname, attlen, atttxt);
           len = (size_t)attlen;
@@ -197,13 +292,13 @@ void cdfDefineAttributes(int cdiID, int varID, int fileID, int ncvarID)
                 atttype == CDI_DATATYPE_INT16 || atttype == CDI_DATATYPE_UINT16 ||
                 atttype == CDI_DATATYPE_INT32 || atttype == CDI_DATATYPE_UINT32 )
         {
-          size_t attSize = (size_t)attlen*sizeof(int);
+          const size_t attSize = (size_t)attlen*sizeof(int);
           int *attint = (int *)resizeBuf(&attBuf, &attBufSize, attSize);
           cdiInqAttInt(cdiID, varID, attname, attlen, &attint[0]);
           len = (size_t)attlen;
-          nc_type xtype = (atttype == CDI_DATATYPE_INT8)  ? NC_BYTE :
-                          (atttype == CDI_DATATYPE_INT16) ? NC_SHORT :
-#if  defined  (HAVE_NETCDF4)
+          nc_type xtype = (atttype == CDI_DATATYPE_INT8)   ? NC_BYTE :
+                          (atttype == CDI_DATATYPE_INT16)  ? NC_SHORT :
+#ifdef  HAVE_NETCDF4
                           (atttype == CDI_DATATYPE_UINT8)  ? NC_UBYTE :
                           (atttype == CDI_DATATYPE_UINT16) ? NC_USHORT :
                           (atttype == CDI_DATATYPE_UINT32) ? NC_UINT :
@@ -213,7 +308,7 @@ void cdfDefineAttributes(int cdiID, int varID, int fileID, int ncvarID)
         }
       else if ( atttype == CDI_DATATYPE_FLT32 || atttype == CDI_DATATYPE_FLT64 )
         {
-          size_t attSize = (size_t)attlen * sizeof(double);
+          const size_t attSize = (size_t)attlen * sizeof(double);
           double *attflt = (double *)resizeBuf(&attBuf, &attBufSize, attSize);
           cdiInqAttFlt(cdiID, varID, attname, attlen, attflt);
           len = (size_t)attlen;
@@ -233,6 +328,17 @@ void cdfDefineAttributes(int cdiID, int varID, int fileID, int ncvarID)
   Free(attBuf);
 }
 
+static
+void cdfDefineInstituteName(int vlistID, int varID, int fileID, int ncvarID)
+{
+  const int instID = vlistInqVarInstitut(vlistID, varID);
+  if (instID != CDI_UNDEFID)
+    {
+      const char *name = institutInqNamePtr(instID);
+      if (name) cdf_put_att_text(fileID, ncvarID, "institution", strlen(name), name);
+    }
+}
+
 static
 void cdfDefGlobalAtts(stream_t *streamptr)
 {
@@ -241,8 +347,8 @@ void cdfDefGlobalAtts(stream_t *streamptr)
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
 
-  cdfDefSource(streamptr);
-  cdfDefInstitut(streamptr);
+  cdfDefSourceGlobal(streamptr);
+  cdfDefInstituteGlobal(streamptr);
 
   int natts;
   cdiInqNatts(vlistID, CDI_GLOBAL, &natts);
@@ -256,64 +362,38 @@ void cdfDefGlobalAtts(stream_t *streamptr)
   streamptr->globalatts = 1;
 }
 
-static
-void cdfDefLocalAtts(stream_t *streamptr)
-{
-  const int vlistID = streamptr->vlistID;
-  const int fileID  = streamptr->fileID;
-
-  if ( streamptr->localatts ) return;
-  if ( vlistInqInstitut(vlistID) != CDI_UNDEFID ) return;
-
-  streamptr->localatts = 1;
-
-  if ( streamptr->ncmode == 2 ) cdf_redef(fileID);
-
-  for ( int varID = 0; varID < streamptr->nvars; varID++ )
-    {
-      const int instID = vlistInqVarInstitut(vlistID, varID);
-      if ( instID != CDI_UNDEFID )
-	{
-          const int ncvarid = streamptr->vars[varID].ncvarid;
-  	  const char *name = institutInqNamePtr(instID);
-	  if ( name )
-	    {
-              const size_t len = strlen(name);
-	      cdf_put_att_text(fileID, ncvarid, "institution", len, name);
-	    }
-	}
-      }
-
-  if ( streamptr->ncmode == 2 ) cdf_enddef(fileID);
-}
-
 static
 void cdf_get_gmapvarname(int gridID, char *gmapvarname)
 {
   int pgridID = gridID;
-  char mapping[CDI_MAX_NAME]; mapping[0] = 0;
-  cdiGridInqKeyStr(pgridID, CDI_KEY_MAPNAME, CDI_MAX_NAME, mapping);
+  char gmapname[CDI_MAX_NAME];
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(pgridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, gmapname, &length);
 
-  if ( !mapping[0] )
+  if ( !gmapname[0] )
     {
-      int projID = gridInqProj(gridID);
+      const int projID = gridInqProj(gridID);
       if ( projID != CDI_UNDEFID )
         {
           pgridID = projID;
-          cdiGridInqKeyStr(pgridID, CDI_KEY_MAPNAME, CDI_MAX_NAME, mapping);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(pgridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, gmapname, &length);
         }
     }
 
-  if ( mapping[0] )
-    cdiGridInqKeyStr(pgridID, CDI_KEY_MAPPING, CDI_MAX_NAME, gmapvarname);
+  if ( gmapname[0] )
+    {
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(pgridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, gmapvarname, &length);
+    }
 }
 
 static
 int nc_grid_index(stream_t *streamptr, int gridID)
 {
   int index = 0;
-  int vlistID = streamptr->vlistID;
-  int ngrids = vlistNgrids(vlistID);
+  const int vlistID = streamptr->vlistID;
+  const int ngrids = vlistNgrids(vlistID);
   for ( index = 0; index < ngrids; ++index )
     if ( streamptr->ncgrid[index].gridID == gridID ) break;
 
@@ -323,13 +403,28 @@ int nc_grid_index(stream_t *streamptr, int gridID)
 }
 
 static
-void cdfDefVarCompression(const stream_t *streamptr, int ncvarid, bool lchunk)
+int xtype2ppb(nc_type xtype)
+{
+  int ppb = 32;
+
+  if      (xtype == NC_BYTE)   ppb = 8;
+  else if (xtype == NC_SHORT)  ppb = 16;
+#ifdef  HAVE_NETCDF4
+  else if (xtype == NC_UBYTE)  ppb = 8;
+  else if (xtype == NC_USHORT) ppb = 16;
+#endif
+
+  return ppb;
+}
+
+static
+void cdfDefVarCompression(const stream_t *streamptr, int ncvarID, bool lchunk, nc_type xtype)
 {
   if ( streamptr->comptype == CDI_COMPRESS_ZIP )
     {
       if ( lchunk && (streamptr->filetype == CDI_FILETYPE_NC4 || streamptr->filetype == CDI_FILETYPE_NC4C) )
         {
-          cdfDefVarDeflate(streamptr->fileID, ncvarid, streamptr->complevel);
+          cdfDefVarDeflate(streamptr->fileID, ncvarID, streamptr->complevel);
         }
       else
         {
@@ -344,10 +439,29 @@ void cdfDefVarCompression(const stream_t *streamptr, int ncvarid, bool lchunk)
             }
         }
     }
+  else if ( streamptr->comptype == CDI_COMPRESS_SZIP )
+    {
+      if ( lchunk && (streamptr->filetype == CDI_FILETYPE_NC4 || streamptr->filetype == CDI_FILETYPE_NC4C) )
+        {
+          cdfDefVarSzip(streamptr->fileID, ncvarID, xtype2ppb(xtype));
+        }
+      else
+        {
+          if ( lchunk )
+            {
+              static bool lwarn = true;
+              if ( lwarn )
+                {
+                  lwarn = false;
+                  Warning("SZIP compression is only available for NetCDF4!");
+                }
+            }
+        }
+    }
 }
 
 static
-void cdfDefVarPacking(const stream_t *streamptr, int ncvarid, nc_type xtype, int vlistID, int varID)
+void cdfDefVarPacking(const stream_t *streamptr, int ncvarID, nc_type xtype, int vlistID, int varID)
 {
   //  if ( xtype == NC_BYTE || xtype == NC_SHORT || xtype == NC_INT )
     {
@@ -366,46 +480,77 @@ void cdfDefVarPacking(const stream_t *streamptr, int ncvarid, nc_type xtype, int
               astype = NC_FLOAT;
             }
 
-          cdf_put_att_double(streamptr->fileID, ncvarid, "add_offset",   astype, 1, &addoffset);
-          cdf_put_att_double(streamptr->fileID, ncvarid, "scale_factor", astype, 1, &scalefactor);
+          cdf_put_att_double(streamptr->fileID, ncvarID, "add_offset",   astype, 1, &addoffset);
+          cdf_put_att_double(streamptr->fileID, ncvarID, "scale_factor", astype, 1, &scalefactor);
         }
     }
 }
 
 static
-void cdfAppendCoordinates(int fileID, int ncvarid, char coordinates[CDI_MAX_NAME])
+void cdfAppendCoordinates(int fileID, int ncvarID, char coordinates[CDI_MAX_NAME])
 {
-  if (ncvarid != CDI_UNDEFID)
+  if (ncvarID != CDI_UNDEFID)
     {
       size_t len = strlen(coordinates);
       if (len) coordinates[len++] = ' ';
-      cdf_inq_varname(fileID, ncvarid, coordinates+len);
+      cdf_inq_varname(fileID, ncvarID, coordinates+len);
     }
 }
 
 static
-void cdfDefineCoordinates(const stream_t *streamptr, int ncvarid, int nczvarID, int gridtype, int gridID, int gridindex, int xid, int yid, size_t gridsize, char axis[5], size_t iax)
+void cdfDefineCoordinates(const stream_t *streamptr, int ncvarID, int nczvarID, int gridtype, int gridID, int gridindex,
+                          int xid, int yid, size_t gridsize, char axis[5], size_t iax)
 {
   const int fileID  = streamptr->fileID;
 
+  if ( gridtype != GRID_GENERIC && gridtype != GRID_LONLAT &&
+       gridtype != GRID_PROJECTION && gridtype != GRID_CURVILINEAR && gridtype != GRID_CHARXY )
+    {
+      const size_t len = strlen(gridNamePtr(gridtype));
+      if ( len > 0 ) cdf_put_att_text(fileID, ncvarID, "CDI_grid_type", len, gridNamePtr(gridtype));
+    }
+
+  char gmapvarname[CDI_MAX_NAME]; gmapvarname[0] = 0;
+  cdf_get_gmapvarname(gridID, gmapvarname);
+  if ( gmapvarname[0] ) cdf_put_att_text(fileID, ncvarID, "grid_mapping", strlen(gmapvarname), gmapvarname);
+
+  if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED )
+    {
+      const int numLPE = gridInqNP(gridID);
+      if (numLPE > 0)
+        cdf_put_att_int(fileID, ncvarID, "CDI_grid_num_LPE", NC_INT, 1, &numLPE);
+    }
+
+  if ( gridtype == GRID_GAUSSIAN_REDUCED )
+    {
+      const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
+      if (ncyvarID != CDI_UNDEFID)
+        {
+          char name[CDI_MAX_NAME]; name[0] = 0;
+          cdf_inq_varname(fileID, ncyvarID, name);
+          const size_t len = strlen(name);
+          cdf_put_att_text(fileID, ncvarID, "CDI_grid_latitudes", len, name);
+        }
+
+      const int ncrpvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_RP];
+      if (ncrpvarID != CDI_UNDEFID)
+        {
+          char name[CDI_MAX_NAME]; name[0] = 0;
+          cdf_inq_varname(fileID, ncrpvarID, name);
+          const size_t len = strlen(name);
+          cdf_put_att_text(fileID, ncvarID, "CDI_grid_reduced_points", len, name);
+        }
+    }
+
+  // define coordinates attribute
+
   char coordinates[CDI_MAX_NAME]; coordinates[0] = 0;
 
   if (nczvarID != CDI_UNDEFID) cdfAppendCoordinates(fileID, nczvarID, coordinates);
 
-  if ( gridtype != GRID_GENERIC && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN &&
-       gridtype != GRID_PROJECTION && gridtype != GRID_CURVILINEAR && gridtype != GRID_CHARXY )
-    {
-      const size_t len = strlen(gridNamePtr(gridtype));
-      if ( len > 0 ) cdf_put_att_text(fileID, ncvarid, "CDI_grid_type", len, gridNamePtr(gridtype));
-    }
-
-  char gmapvarname[CDI_MAX_NAME]; gmapvarname[0] = 0;
-  cdf_get_gmapvarname(gridID, gmapvarname);
-  if ( gmapvarname[0] ) cdf_put_att_text(fileID, ncvarid, "grid_mapping", strlen(gmapvarname), gmapvarname);
-
   if ( gridtype == GRID_TRAJECTORY )
     {
-      cdf_put_att_text(fileID, ncvarid, "coordinates", 9, "tlon tlat" );
+      cdf_put_att_text(fileID, ncvarID, "coordinates", 9, "tlon tlat");
     }
   else if ( gridtype == GRID_LONLAT && xid == CDI_UNDEFID && yid == CDI_UNDEFID && gridsize == 1 )
     {
@@ -416,14 +561,15 @@ void cdfDefineCoordinates(const stream_t *streamptr, int ncvarid, int nczvarID,
     }
   else if ( gridtype == GRID_GAUSSIAN_REDUCED )
     {
+      /*
       const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
       const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
       cdfAppendCoordinates(fileID, ncyvarID, coordinates);
       cdfAppendCoordinates(fileID, ncxvarID, coordinates);
+      */
     }
   else if ( gridtype == GRID_UNSTRUCTURED || gridtype == GRID_CURVILINEAR )
     {
-      char cellarea[CDI_MAX_NAME] = "area: ";
       const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
       const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
       const int ncavarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_A];
@@ -441,26 +587,27 @@ void cdfDefineCoordinates(const stream_t *streamptr, int ncvarid, int nczvarID,
 
       if ( ncavarID != CDI_UNDEFID )
         {
+          char cellarea[CDI_MAX_NAME] = "area: ";
           size_t len = strlen(cellarea);
           cdf_inq_varname(fileID, ncavarID, cellarea+len);
           len = strlen(cellarea);
-          cdf_put_att_text(fileID, ncvarid, "cell_measures", len, cellarea);
+          cdf_put_att_text(fileID, ncvarID, "cell_measures", len, cellarea);
         }
 
       if ( gridtype == GRID_UNSTRUCTURED )
         {
           int position = gridInqPosition(gridID);
           if ( position > 0 )
-            cdf_put_att_int(fileID, ncvarid, "number_of_grid_in_reference", NC_INT, 1, &position);
+            cdf_put_att_int(fileID, ncvarID, "number_of_grid_in_reference", NC_INT, 1, &position);
         }
     }
   else if ( gridtype == GRID_SPECTRAL || gridtype == GRID_FOURIER )
     {
       axis[iax++] = '-';
       axis[iax++] = '-';
-      cdf_put_att_text(fileID, ncvarid, "axis", iax, axis);
+      cdf_put_att_text(fileID, ncvarID, "axis", iax, axis);
       int gridTruncation = gridInqTrunc(gridID);
-      cdf_put_att_int(fileID, ncvarid, "truncation", NC_INT, 1, &gridTruncation);
+      cdf_put_att_int(fileID, ncvarID, "truncation", NC_INT, 1, &gridTruncation);
     }
   else if ( gridtype == GRID_CHARXY )
     {
@@ -477,7 +624,7 @@ void cdfDefineCoordinates(const stream_t *streamptr, int ncvarid, int nczvarID,
     }
 
   const size_t len = strlen(coordinates);
-  if ( len ) cdf_put_att_text(fileID, ncvarid, "coordinates", len, coordinates);
+  if ( len ) cdf_put_att_text(fileID, ncvarID, "coordinates", len, coordinates);
 }
 
 static
@@ -507,10 +654,11 @@ int cdfDefineDimsAndChunks(const stream_t *streamptr, int varID, int xid, int yi
     }
 
   int chunktype = vlistInqVarChunkType(vlistID, varID);
-  size_t chunk_size_max = 65536;
-  if ( chunktype != CDI_CHUNK_LINES && gridsize > INT32_MAX )
+  const size_t chunk_size_max = 65536;
+  const size_t chunk_size_lim = 1073741823;
+  if ( chunktype != CDI_CHUNK_LINES && gridsize > chunk_size_lim )
     {
-      if ( CDI_Debug ) fprintf(stderr, "gridsize > %d, changed chunktype to CDI_CHUNK_LINES!\n", INT32_MAX);
+      if ( CDI_Debug ) fprintf(stderr, "gridsize > %zu, changed chunktype to CDI_CHUNK_LINES!\n", chunk_size_lim);
       chunktype = CDI_CHUNK_LINES;
     }
 
@@ -537,7 +685,7 @@ int cdfDefineDimsAndChunks(const stream_t *streamptr, int varID, int xid, int yi
           if ( chunktype == CDI_CHUNK_AUTO && yid == CDI_UNDEFID )
             chunks[ndims] = (chunk_size_max > xsize) ? xsize : chunk_size_max;
           else
-            chunks[ndims] = (xsize > INT32_MAX) ? INT32_MAX : xsize;
+            chunks[ndims] = (xsize > chunk_size_lim) ? chunk_size_lim : xsize;
           dims[ndims] = xid;
           ndims++;
         }
@@ -546,16 +694,17 @@ int cdfDefineDimsAndChunks(const stream_t *streamptr, int varID, int xid, int yi
   if ( CDI_Debug )
     fprintf(stderr, "lchunk %d chunktype %d  chunks %zu %zu %zu %zu\n", lchunk, chunktype, chunks[0], chunks[1], chunks[2], chunks[3]);
 
-  iax = *piax;
+  *piax = iax;
   return ndims;
 }
 
 static
-void cdfDefineAttrLeveltype(int fileID, int ncvarid, int zaxisID, int zaxistype)
+void cdfDefineAttrLeveltype(int fileID, int ncvarID, int zaxisID, int zaxistype)
 {
   if ( zaxistype == ZAXIS_CLOUD_BASE          ||
        zaxistype == ZAXIS_CLOUD_TOP           ||
        zaxistype == ZAXIS_ISOTHERM_ZERO       ||
+       zaxistype == ZAXIS_TROPOPAUSE          ||
        zaxistype == ZAXIS_TOA                 ||
        zaxistype == ZAXIS_SEA_BOTTOM          ||
        zaxistype == ZAXIS_LAKE_BOTTOM         ||
@@ -566,21 +715,22 @@ void cdfDefineAttrLeveltype(int fileID, int ncvarid, int zaxisID, int zaxistype)
        zaxistype == ZAXIS_ATMOSPHERE )
     {
       char varname[CDI_MAX_NAME];
-      zaxisInqName(zaxisID, varname);
-      cdf_put_att_text(fileID, ncvarid, "level_type", strlen(varname), varname);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, varname, &length);
+      cdf_put_att_text(fileID, ncvarID, "level_type", strlen(varname), varname);
     }
 }
 
 static
-void cdfDefineAttrEnsemble(int fileID, int ncvarid, int vlistID, int varID)
+void cdfDefineAttrEnsemble(int fileID, int ncvarID, int vlistID, int varID)
 {
   int perturbationNumber, numberOfForecastsInEnsemble, typeOfEnsembleForecast;
   if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_PERTURBATIONNUMBER, &perturbationNumber) == 0 )
-    cdf_put_att_int(fileID, ncvarid, "realization", NC_INT, 1, &perturbationNumber);
+    cdf_put_att_int(fileID, ncvarID, "realization", NC_INT, 1, &perturbationNumber);
   if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_NUMBEROFFORECASTSINENSEMBLE, &numberOfForecastsInEnsemble) == 0 )
-    cdf_put_att_int(fileID, ncvarid, "ensemble_members", NC_INT, 1, &numberOfForecastsInEnsemble);
+    cdf_put_att_int(fileID, ncvarID, "ensemble_members", NC_INT, 1, &numberOfForecastsInEnsemble);
   if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_TYPEOFENSEMBLEFORECAST, &typeOfEnsembleForecast) == 0 )
-    cdf_put_att_int(fileID, ncvarid, "forecast_init_type", NC_INT, 1, &typeOfEnsembleForecast);
+    cdf_put_att_int(fileID, ncvarID, "forecast_init_type", NC_INT, 1, &typeOfEnsembleForecast);
 }
 
 static
@@ -588,7 +738,7 @@ void cdfCheckVarname(int fileID, char name[CDI_MAX_NAME])
 {
   if ( name[0] )
     {
-      int ncvarid;
+      int ncvarID;
       char varname[CDI_MAX_NAME];
       sprintf(varname, "%s", name);
       char *varname2 = varname+strlen(varname);
@@ -598,7 +748,7 @@ void cdfCheckVarname(int fileID, char name[CDI_MAX_NAME])
         {
           if ( iz ) sprintf(varname2, "_%u", iz+1);
 
-          if ( nc_inq_varid(fileID, varname, &ncvarid) != NC_NOERR ) break;
+          if ( nc_inq_varid(fileID, varname, &ncvarID) != NC_NOERR ) break;
 
           ++iz;
         }
@@ -628,14 +778,14 @@ void cdfGenVarname(int fileID, char name[CDI_MAX_NAME], int pnum, int pcat, int
     sprintf(varname, "param%d.%d.%d", pnum, pcat, *pdis);
 
   char *varname2 = varname+strlen(varname);
-  int ncvarid;
+  int ncvarID;
   unsigned iz = 0;
 
   do
     {
       if ( iz ) sprintf(varname2, "_%u", iz+1);
 
-      if ( nc_inq_varid(fileID, varname, &ncvarid) != NC_NOERR ) break;
+      if ( nc_inq_varid(fileID, varname, &ncvarID) != NC_NOERR ) break;
 
       ++iz;
     }
@@ -668,7 +818,8 @@ int cdfDefVar(stream_t *streamptr, int varID)
   const int gridtype = gridInqType(gridID);
   const int gridindex = nc_grid_index(streamptr, gridID);
   const int xid = (gridtype != GRID_TRAJECTORY) ? streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X] : CDI_UNDEFID;
-  const int yid = (gridtype != GRID_TRAJECTORY) ? streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y] : CDI_UNDEFID;
+  const int yid = (gridtype != GRID_TRAJECTORY && gridtype != GRID_GAUSSIAN_REDUCED) ?
+    streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y] : CDI_UNDEFID;
 
   const int zaxisID = vlistInqVarZaxis(vlistID, varID);
   const int zaxistype = zaxisInqType(zaxisID);
@@ -677,7 +828,7 @@ int cdfDefVar(stream_t *streamptr, int varID)
 
   int dimorder[3]; // ZYX and ZXY
   vlistInqVarDimorder(vlistID, varID, &dimorder);
-  const bool lchunk = (dimorder[0] == 3) ? (gridsize >= 16) : false;
+  const bool lchunk = (dimorder[0] == 3) ? (gridsize >= 32) : false;
 
   if ( ((dimorder[0]>0)+(dimorder[1]>0)+(dimorder[2]>0)) < ((xid!=CDI_UNDEFID)+(yid!=CDI_UNDEFID)+(zid!=CDI_UNDEFID)) )
     {
@@ -691,16 +842,20 @@ int cdfDefVar(stream_t *streamptr, int varID)
   size_t chunks[4];
   int ndims = cdfDefineDimsAndChunks(streamptr, varID, xid, yid, zid, gridsize, dimorder, dims, lchunk, chunks, axis, &iax);
 
-  char name[CDI_MAX_NAME]; name[0] = 0;
-  if ( vlistInqVarNamePtr(vlistID, varID) ) vlistInqVarName(vlistID, varID, name);
+  char name[CDI_MAX_NAME];
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(vlistID, varID, CDI_KEY_NAME, name, &length);
 
-  char longname[CDI_MAX_NAME]; longname[0] = 0;
-  char stdname[CDI_MAX_NAME]; stdname[0] = 0;
-  char units[CDI_MAX_NAME]; units[0] = 0;
+  char longname[CDI_MAX_NAME];
   vlistInqVarLongname(vlistID, varID, longname);
-  vlistInqVarStdname(vlistID, varID, stdname);
+
+  char units[CDI_MAX_NAME];
   vlistInqVarUnits(vlistID, varID, units);
 
+  char stdname[CDI_MAX_NAME];
+  length = CDI_MAX_NAME;
+  cdiInqKeyString(vlistID, varID, CDI_KEY_STDNAME, stdname, &length);
+
   const int tableID  = vlistInqVarTable(vlistID, varID);
   if (!name[0]) tableInqEntry(tableID, code, -1, name, longname, units);
   if (name[0]) cdfCheckVarname(fileID, name);
@@ -709,70 +864,75 @@ int cdfDefVar(stream_t *streamptr, int varID)
   const int dtype = vlistInqVarDatatype(vlistID, varID);
   const nc_type xtype = cdfDefDatatype(dtype, streamptr);
 
-  int ncvarid = -1;
-  cdf_def_var(fileID, name, xtype, ndims, dims, &ncvarid);
+  int ncvarID = -1;
+  cdf_def_var(fileID, name, xtype, ndims, dims, &ncvarID);
 
-#if  defined  (HAVE_NETCDF4)
-  if ( lchunk && (streamptr->filetype == CDI_FILETYPE_NC4 || streamptr->filetype == CDI_FILETYPE_NC4C) )
-    cdf_def_var_chunking(fileID, ncvarid, NC_CHUNKED, chunks);
+#ifdef  HAVE_NETCDF4
+  if (lchunk && (streamptr->filetype == CDI_FILETYPE_NC4 || streamptr->filetype == CDI_FILETYPE_NC4C))
+    cdf_def_var_chunking(fileID, ncvarID, NC_CHUNKED, chunks);
 #endif
 
-  cdfDefVarCompression(streamptr, ncvarid, lchunk);
+  cdfDefVarCompression(streamptr, ncvarID, lchunk, xtype);
 
-  if ( *stdname )  cdf_put_att_text(fileID, ncvarid, "standard_name", strlen(stdname), stdname);
-  if ( *longname ) cdf_put_att_text(fileID, ncvarid, "long_name", strlen(longname), longname);
-  if ( *units )    cdf_put_att_text(fileID, ncvarid, "units", strlen(units), units);
+  if ( *stdname )  cdf_put_att_text(fileID, ncvarID, "standard_name", strlen(stdname), stdname);
+  if ( *longname ) cdf_put_att_text(fileID, ncvarID, "long_name", strlen(longname), longname);
+  if ( *units )    cdf_put_att_text(fileID, ncvarID, "units", strlen(units), units);
 
   if ( code > 0 && pdis == 255 )
-    cdf_put_att_int(fileID, ncvarid, "code", NC_INT, 1, &code);
+    cdf_put_att_int(fileID, ncvarID, "code", NC_INT, 1, &code);
 
   if ( pdis != 255 )
     {
       char paramstr[32];
       cdiParamToString(param, paramstr, sizeof(paramstr));
-      cdf_put_att_text(fileID, ncvarid, "param", strlen(paramstr), paramstr);
+      cdf_put_att_text(fileID, ncvarID, "param", strlen(paramstr), paramstr);
     }
 
   if ( tableID != CDI_UNDEFID )
     {
       int tablenum = tableInqNum(tableID);
-      if (tablenum > 0) cdf_put_att_int(fileID, ncvarid, "table", NC_INT, 1, &tablenum);
+      if (tablenum > 0) cdf_put_att_int(fileID, ncvarID, "table", NC_INT, 1, &tablenum);
     }
 
   const bool zaxisIsScalar = (zid == CDI_UNDEFID) ? (zaxisInqScalar(zaxisID) > 0) : false;
   const int nczvarID = (zaxisIsScalar || zaxistype == ZAXIS_CHAR) ? streamptr->nczvarID[zaxisindex] : CDI_UNDEFID;
 
-  cdfDefineCoordinates(streamptr, ncvarid, nczvarID, gridtype, gridID, gridindex, xid, yid, gridsize, axis, iax);
+  cdfDefineCoordinates(streamptr, ncvarID, nczvarID, gridtype, gridID, gridindex, xid, yid, gridsize, axis, iax);
 
-  cdfDefVarPacking(streamptr, ncvarid, xtype, vlistID, varID);
+  cdfDefVarPacking(streamptr, ncvarID, xtype, vlistID, varID);
 
   if ( dtype == CDI_DATATYPE_UINT8 && xtype == NC_BYTE )
     {
       const int validrange[2] = {0, 255};
-      cdf_put_att_int(fileID, ncvarid, "valid_range", NC_SHORT, 2, validrange);
-      cdf_put_att_text(fileID, ncvarid, "_Unsigned", 4, "true");
+      cdf_put_att_int(fileID, ncvarID, "valid_range", NC_SHORT, 2, validrange);
+      cdf_put_att_text(fileID, ncvarID, "_Unsigned", 4, "true");
     }
 
-  streamptr->vars[varID].ncvarid = ncvarid;
+  streamptr->vars[varID].ncvarid = ncvarID;
 
   if ( vlistInqVarMissvalUsed(vlistID, varID) )
     cdfDefVarMissval(streamptr, varID, vlistInqVarDatatype(vlistID, varID), 0);
 
-  if (zid == CDI_UNDEFID) cdfDefineAttrLeveltype(fileID, ncvarid, zaxisID, zaxistype);
+  if (zid == CDI_UNDEFID) cdfDefineAttrLeveltype(fileID, ncvarID, zaxisID, zaxistype);
 
-  cdfDefineAttrEnsemble(fileID, ncvarid, vlistID, varID);
+  cdfDefineAttrEnsemble(fileID, ncvarID, vlistID, varID);
+
+  // Attribute: cell_methods
+  cdfDefineCellMethods(streamptr, vlistID, varID, fileID, ncvarID);
 
   // Attributes
-  cdfDefineAttributes(vlistID, varID, fileID, ncvarid);
+  cdfDefineAttributes(vlistID, varID, fileID, ncvarID);
 
-  return ncvarid;
+  // Institute
+  if (vlistInqInstitut(vlistID) == CDI_UNDEFID) cdfDefineInstituteName(vlistID, varID, fileID, ncvarID);
+
+  return ncvarID;
 }
 
 
 void cdfEndDef(stream_t *streamptr)
 {
   cdfDefGlobalAtts(streamptr);
-  cdfDefLocalAtts(streamptr);
 
   if ( streamptr->accessmode == 0 )
     {
@@ -801,18 +961,17 @@ void cdfWriteGridTraj(stream_t *streamptr, int gridID)
   const int gridindex = nc_grid_index(streamptr, gridID);
   const int lonID = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
   const int latID = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
-  const int tsID = streamptr->curTsID;
-  size_t index = (size_t)tsID;
+  const size_t index = (size_t)streamptr->curTsID;
 
-  double xlon = gridInqXval(gridID, 0);
-  double xlat = gridInqYval(gridID, 0);
+  const double xlon = gridInqXval(gridID, 0);
+  const double xlat = gridInqYval(gridID, 0);
 
   cdf_put_var1_double(streamptr->fileID, lonID, &index, &xlon);
   cdf_put_var1_double(streamptr->fileID, latID, &index, &xlat);
 }
 
 static
-void cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarid, int dtype, size_t nvals, size_t xsize, size_t ysize,
+void cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarID, int dtype, size_t nvals, size_t xsize, size_t ysize,
                         bool swapxy, size_t *start, size_t *count, int memtype, const void *data, size_t nmiss)
 {
   const double *pdata_dp = (const double *) data;
@@ -907,7 +1066,7 @@ void cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarid, int dty
               if ( dtype == CDI_DATATYPE_UINT8 )
                 {
                   nc_type xtype;
-                  cdf_inq_vartype(fileID, ncvarid, &xtype);
+                  cdf_inq_vartype(fileID, ncvarID, &xtype);
                   if ( xtype == NC_BYTE )
                     {
                       for ( size_t i = 0; i < nvals; ++i )
@@ -929,7 +1088,7 @@ void cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarid, int dty
               if ( dtype == CDI_DATATYPE_UINT8 )
                 {
                   nc_type xtype;
-                  cdf_inq_vartype(fileID, ncvarid, &xtype);
+                  cdf_inq_vartype(fileID, ncvarID, &xtype);
                   if ( xtype == NC_BYTE )
                     {
                       for ( size_t i = 0; i < nvals; ++i )
@@ -977,10 +1136,43 @@ void cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarid, int dty
         }
     }
 
-  if ( memtype == MEMTYPE_FLOAT )
-    cdf_put_vara_float(fileID, ncvarid, start, count, pdata_sp);
+  if (dtype == CDI_DATATYPE_CPX32 || dtype == CDI_DATATYPE_CPX64)
+    {
+      void *pdata = (memtype == MEMTYPE_FLOAT) ? (void*)pdata_sp : (void*)pdata_dp;
+      float *cdata_sp = NULL;
+      double *cdata_dp = NULL;
+      if (memtype == MEMTYPE_FLOAT && dtype == CDI_DATATYPE_CPX64)
+        {
+          cdata_dp = (double *) Malloc(2*nvals*sizeof(double));
+          for (size_t i = 0; i < nvals; i++)
+            {
+              cdata_dp[2*i] = (double)(pdata_sp[2*i]);
+              cdata_dp[2*i+1] = (double)(pdata_sp[2*i+1]);
+            }
+          pdata = cdata_dp;
+        }
+      else if (memtype == MEMTYPE_DOUBLE && dtype == CDI_DATATYPE_CPX32)
+        {
+          cdata_sp = (float *) Malloc(2*nvals*sizeof(float));
+          for (size_t i = 0; i < nvals; i++)
+            {
+              cdata_sp[2*i] = (float)(pdata_dp[2*i]);
+              cdata_sp[2*i+1] = (float)(pdata_dp[2*i+1]);
+            }
+          pdata = cdata_sp;
+        }
+
+      cdf_put_vara(fileID, ncvarID, start, count, pdata);
+      if (cdata_sp) Free(cdata_sp);
+      if (cdata_dp) Free(cdata_dp);
+    }
   else
-    cdf_put_vara_double(fileID, ncvarid, start, count, pdata_dp);
+    {
+      if ( memtype == MEMTYPE_FLOAT )
+        cdf_put_vara_float(fileID, ncvarID, start, count, pdata_sp);
+      else
+        cdf_put_vara_double(fileID, ncvarID, start, count, pdata_dp);
+    }
 
   if ( mdata_dp ) Free(mdata_dp);
   if ( sdata_dp ) Free(sdata_dp);
@@ -994,7 +1186,8 @@ void cdfGetXYZid(stream_t *streamptr, int gridID, int zaxisID, int *xid, int *yi
   *xid = CDI_UNDEFID;
   *yid = CDI_UNDEFID;
 
-  if (gridInqType(gridID) == GRID_TRAJECTORY)
+  const int gridtype = gridInqType(gridID);
+  if (gridtype == GRID_TRAJECTORY)
     {
       cdfWriteGridTraj(streamptr, gridID);
     }
@@ -1002,7 +1195,8 @@ void cdfGetXYZid(stream_t *streamptr, int gridID, int zaxisID, int *xid, int *yi
     {
       const int gridindex = nc_grid_index(streamptr, gridID);
       *xid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
-      *yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
+      if (gridtype != GRID_GAUSSIAN_REDUCED)
+        *yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
     }
 
   const int vlistID = streamptr->vlistID;
@@ -1074,7 +1268,7 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
 
-  const int ncvarid = cdfDefVar(streamptr, varID);
+  const int ncvarID = cdfDefVar(streamptr, varID);
 
   const int gridID   = vlistInqVarGrid(vlistID, varID);
   const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
@@ -1099,7 +1293,7 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
   const size_t nvals = gridInqSize(gridID) * (size_t)(zaxisInqSize(zaxisID));
 
   bool swapxy = false;
-  cdf_write_var_data(fileID, vlistID, varID, ncvarid, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
+  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
 }
 
 static
@@ -1172,7 +1366,7 @@ void cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype,
   const int vlistID = streamInqVlist(streamID);
   const int fileID  = streamInqFileID(streamID);
 
-  const int ncvarid = cdfDefVar(streamptr, varID);
+  const int ncvarID = cdfDefVar(streamptr, varID);
 
   const int gridID   = vlistInqVarGrid(vlistID, varID);
   const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
@@ -1197,7 +1391,7 @@ void cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype,
   const size_t nvals = gridInqSize(gridID) * (size_t)(zaxisInqSize(zaxisID));
 
   bool swapxy = false;
-  cdf_write_var_data(fileID, vlistID, varID, ncvarid, dtype, nvals,
+  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals,
                      xsize, ysize, swapxy, start, count, memtype, data, nmiss);
 }
 
@@ -1252,6 +1446,7 @@ void cdfDefineStartAndCountSlice(stream_t *streamptr, int varID, int levelID, in
       Message("dim = %d  start = %d  count = %d", idim, start[idim], count[idim]);
 }
 
+
 void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtype, const void *data, size_t nmiss)
 {
   if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
@@ -1261,7 +1456,7 @@ void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtyp
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
 
-  const int ncvarid = cdfDefVar(streamptr, varID);
+  const int ncvarID = cdfDefVar(streamptr, varID);
 
   const int gridID   = vlistInqVarGrid(vlistID, varID);
   const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
@@ -1283,7 +1478,7 @@ void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtyp
 
   const size_t nvals = gridInqSize(gridID);
 
-  cdf_write_var_data(fileID, vlistID, varID, ncvarid, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
+  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
 }
 
 
