@@ -19,19 +19,10 @@
 static
 cdi_atts_t *get_attsp(vlist_t *vlistptr, int varID)
 {
-  cdi_atts_t *attsp = NULL;
+  if ( varID == CDI_GLOBAL ) return &vlistptr->atts;
+  else if ( varID >= 0 && varID < vlistptr->nvars ) return &(vlistptr->vars[varID].atts);
 
-  if ( varID == CDI_GLOBAL )
-    {
-      attsp = &vlistptr->atts;
-    }
-  else
-    {
-      if ( varID >= 0 && varID < vlistptr->nvars )
-	attsp = &(vlistptr->vars[varID].atts);
-    }
-
-  return attsp;
+  return NULL;
 }
 
 static
@@ -178,7 +169,7 @@ int cdiInqAtt(int cdiID, int varID, int attnum, char *name, int *typep, int *len
   if ( attnum >= 0 && attnum < (int)attsp->nelems )
     attp = &(attsp->value[attnum]);
 
-  if ( attp != NULL ) /* name in use */
+  if ( attp != NULL && attp->name ) /* name in use */
     {
       memcpy(name, attp->name, attp->namesz+1);
       *typep  = attp->exdtype;
@@ -196,6 +187,44 @@ int cdiInqAtt(int cdiID, int varID, int attnum, char *name, int *typep, int *len
 }
 
 
+int cdiInqAttLen(int cdiID, int varID, const char *name)
+{
+  int length = -1;
+
+  xassert(name != NULL);
+
+  cdi_atts_t *attsp = cdi_get_attsp(cdiID, varID);
+  xassert(attsp != NULL);
+
+  for ( int attid = 0; attid < (int)attsp->nelems; attid++ )
+    {
+      cdi_att_t *attp = &(attsp->value[attid]);
+      if (attp->name && strcmp(attp->name, name) == 0) length = (int)attp->nelems;
+    }
+
+  return length;
+}
+
+
+int cdiInqAttType(int cdiID, int varID, const char *name)
+{
+  int type = -1;
+
+  xassert(name != NULL);
+
+  cdi_atts_t *attsp = cdi_get_attsp(cdiID, varID);
+  xassert(attsp != NULL);
+
+  for ( int attid = 0; attid < (int)attsp->nelems; attid++ )
+    {
+      cdi_att_t *attp = &(attsp->value[attid]);
+      if (attp->name && strcmp(attp->name, name) == 0) type = attp->exdtype;
+    }
+
+  return type;
+}
+
+
 int cdiDeleteAtts(int cdiID, int varID)
 {
   int status = CDI_NOERR;
@@ -206,7 +235,12 @@ int cdiDeleteAtts(int cdiID, int varID)
   for ( int attid = 0; attid < (int)attsp->nelems; attid++ )
     {
       cdi_att_t *attp = &(attsp->value[attid]);
-      if ( attp->name   ) Free(attp->name);
+      if ( attp->name   )
+        {
+          Free(attp->name);
+          attp->name = NULL;
+          attp->namesz = 0;
+        }
       if ( attp->xvalue ) Free(attp->xvalue);
     }
 
@@ -220,11 +254,24 @@ int cdiDelAtt(int cdiID, int varID, const char *name)
 {
   int status = CDI_NOERR;
 
-  UNUSED(cdiID);
-  UNUSED(varID);
-  UNUSED(name);
+  cdi_atts_t *attsp = cdi_get_attsp(cdiID, varID);
+  xassert(attsp != NULL);
 
-  fprintf(stderr, "cdiDelAtt not implemented!\n");
+  for ( int attid = 0; attid < (int)attsp->nelems; attid++ )
+    {
+      cdi_att_t *attp = &(attsp->value[attid]);
+      if (attp->name && strcmp(attp->name, name) == 0)
+        {
+          Free(attp->name);
+          attp->name = NULL;
+          attp->namesz = 0;
+          if ( attp->xvalue )
+            {
+              Free(attp->xvalue);
+              attp->xvalue = NULL;
+            }
+        }
+    }
 
   return status;
 }
@@ -308,7 +355,7 @@ int cdiCopyAtts(int cdiID1, int varID1, int cdiID2, int varID2)
 @Prototype int cdiDefAttInt(int cdiID, int varID, const char *name, int type, int len, const int *ip)
 
 @Parameter
-    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate} or @fref{gridCreate}.
+    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{zaxisCreate}.
     @Item  varID    Variable identifier, or @func{CDI_GLOBAL} for a global attribute.
     @Item  name     Attribute name.
     @Item  type     External data type (@func{CDI_DATATYPE_INT16} or @func{CDI_DATATYPE_INT32}).
@@ -332,7 +379,7 @@ int cdiDefAttInt(int cdiID, int varID, const char *name, int type, int len, cons
 @Prototype int cdiDefAttFlt(int cdiID, int varID, const char *name, int type, int len, const double *dp)
 
 @Parameter
-    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate} or @fref{gridCreate}.
+    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{zaxisCreate}.
     @Item  varID    Variable identifier, or @func{CDI_GLOBAL} for a global attribute.
     @Item  name     Attribute name.
     @Item  type     External data type (@func{CDI_DATATYPE_FLT32} or @func{CDI_DATATYPE_FLT64}).
@@ -356,7 +403,7 @@ int cdiDefAttFlt(int cdiID, int varID, const char *name, int type, int len, cons
 @Prototype int cdiDefAttTxt(int cdiID, int varID, const char *name, int len, const char *tp)
 
 @Parameter
-    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate} or @fref{gridCreate}.
+    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{zaxisCreate}.
     @Item  varID    Variable identifier, or @func{CDI_GLOBAL} for a global attribute.
     @Item  name     Attribute name.
     @Item  len      Number of values provided for the attribute.
@@ -365,6 +412,21 @@ int cdiDefAttFlt(int cdiID, int varID, const char *name, int type, int len, cons
 @Description
 The function @func{cdiDefAttTxt} defines a text attribute.
 
+@Example
+Here is an example using @func{cdiDefAttTxt} to define the attribute "description":
+
+@Source
+#include "cdi.h"
+   ...
+int vlistID, varID, status;
+char text[] = "description of the variable";
+   ...
+vlistID = vlistCreate();
+varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
+   ...
+status = cdiDefAttTxt(vlistID, varID, "description", LEN(text), text);
+   ...
+@EndSource
 @EndFunction
 */
 int cdiDefAttTxt(int cdiID, int varID, const char *name, int len, const char *tp)
@@ -378,7 +440,7 @@ int cdiDefAttTxt(int cdiID, int varID, const char *name, int len, const char *tp
 
 @Prototype int cdiInqAttInt(int cdiID, int varID, const char *name, int mlen, int *ip)
 @Parameter
-    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{streamInqVlist}.
+    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{zaxisCreate}.
     @Item  varID    Variable identifier, or @func{CDI_GLOBAL} for a global attribute.
     @Item  name     Attribute name.
     @Item  mlen     Number of allocated values provided for the attribute.
@@ -400,7 +462,7 @@ int cdiInqAttInt(int cdiID, int varID, const char *name, int mlen, int *ip)
 
 @Prototype int cdiInqAttFlt(int cdiID, int varID, const char *name, int mlen, double *dp)
 @Parameter
-    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{streamInqVlist}.
+    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{zaxisCreate}.
     @Item  varID    Variable identifier, or @func{CDI_GLOBAL} for a global attribute.
     @Item  name     Attribute name.
     @Item  mlen     Number of allocated values provided for the attribute.
@@ -422,7 +484,7 @@ int cdiInqAttFlt(int cdiID, int varID, const char *name, int mlen, double *dp)
 
 @Prototype int cdiInqAttTxt(int cdiID, int varID, const char *name, int mlen, char *tp)
 @Parameter
-    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{streamInqVlist}.
+    @Item  cdiID    CDI ID, from a previous call to @fref{vlistCreate}, @fref{gridCreate} or @fref{zaxisCreate}.
     @Item  varID    Variable identifier, or @func{CDI_GLOBAL} for a global attribute.
     @Item  name     Attribute name.
     @Item  mlen     Number of allocated values provided for the attribute.
@@ -456,46 +518,36 @@ int cdiAttTypeLookup(cdi_att_t *attp)
     type = attp->indtype;
     break;
   default:
-    xabort("Unknown datatype encountered in attribute %s: %d\n",
-            attp->name, attp->indtype);
+    xabort("Unknown datatype encountered in attribute %s: %d\n", attp->name, attp->indtype);
   }
   return type;
 }
 
 
-int cdi_att_compare(vlist_t *a, int varIDA, vlist_t *b, int varIDB, int attnum)
+int cdi_att_compare(cdi_atts_t *attspa, cdi_atts_t *attspb, int attnum)
 {
-  cdi_atts_t *attspa = get_attsp(a, varIDA),
-    *attspb = get_attsp(b, varIDB);
-  if (attspa == NULL && attspb == NULL)
-    return 0;
-  xassert(attnum >= 0 && attnum < (int)attspa->nelems
-          && attnum < (int)attspb->nelems);
+  xassert(attnum >= 0 && attnum < (int)attspa->nelems && attnum < (int)attspb->nelems);
   cdi_att_t *attpa = attspa->value + attnum,
-    *attpb = attspb->value + attnum;
-  size_t len;
-  if ((len = attpa->namesz) != attpb->namesz)
-    return 1;
-  int diff;
-  if ((diff = memcmp(attpa->name, attpb->name, len)))
-    return 1;
+            *attpb = attspb->value + attnum;
+
+  if (attpa->namesz != attpb->namesz) return 1;
+
+  if (attpa->name && attpb->name && memcmp(attpa->name, attpb->name, attpa->namesz)) return 1;
+
   if (attpa->indtype != attpb->indtype
       || attpa->exdtype != attpb->exdtype
       || attpa->nelems != attpb->nelems)
     return 1;
+
   return memcmp(attpa->xvalue, attpb->xvalue, attpa->xsz);
 }
 
 
 static
-int cdiAttGetSize(vlist_t *vlistptr, int varID, int attnum, void *context)
+int cdiAttGetSize(cdi_atts_t *attsp, int attnum, void *context)
 {
-  cdi_atts_t *attsp;
-  cdi_att_t *attp;
-
-  xassert(attsp = get_attsp(vlistptr, varID));
   xassert(attnum >= 0 && attnum < (int)attsp->nelems);
-  attp = &(attsp->value[attnum]);
+  cdi_att_t *attp = &(attsp->value[attnum]);
   int txsize = serializeGetSize(cdi_att_nints, CDI_DATATYPE_INT, context)
     + serializeGetSize((int)attp->namesz, CDI_DATATYPE_TXT, context);
   txsize += serializeGetSize((int)attp->nelems, cdiAttTypeLookup(attp), context);
@@ -505,26 +557,22 @@ int cdiAttGetSize(vlist_t *vlistptr, int varID, int attnum, void *context)
 
 int cdiAttsGetSize(void *vp, int varID, void *context)
 {
-  vlist_t *p = (vlist_t*) vp;
-  cdi_atts_t *attsp = get_attsp(p, varID);
+  cdi_atts_t *attsp;
+  xassert(attsp = get_attsp((vlist_t*) vp, varID));
   int txsize = serializeGetSize(1, CDI_DATATYPE_INT, context);
   size_t numAtts = attsp->nelems;
   for (size_t i = 0; i < numAtts; ++i)
-    txsize += cdiAttGetSize(p, varID, (int)i, context);
+    txsize += cdiAttGetSize(attsp, (int)i, context);
   return txsize;
 }
 
 static
-void cdiAttPack(vlist_t *vlistptr, int varID, int attnum,
-                void *buf, int size, int *position, void *context)
+void cdiAttPack(cdi_atts_t *attsp, int attnum, void *buf, int size, int *position, void *context)
 {
-  cdi_atts_t *attsp;
-  cdi_att_t *attp;
   int tempbuf[cdi_att_nints];
 
-  xassert(attsp = get_attsp(vlistptr, varID));
   xassert(attnum >= 0 && attnum < (int)attsp->nelems);
-  attp = &(attsp->value[attnum]);
+  cdi_att_t *attp = &(attsp->value[attnum]);
   tempbuf[0] = (int)attp->namesz;
   tempbuf[1] = attp->exdtype;
   tempbuf[2] = attp->indtype;
@@ -538,14 +586,14 @@ void cdiAttPack(vlist_t *vlistptr, int varID, int attnum,
 
 void cdiAttsPack(void *vp, int varID, void *buf, int size, int *position, void *context)
 {
-  vlist_t *p = (vlist_t*) vp;
-  cdi_atts_t *attsp = get_attsp(p, varID);
+  cdi_atts_t *attsp;
+  xassert(attsp = get_attsp((vlist_t*) vp, varID));
   size_t numAtts = attsp->nelems;
   int numAttsI = (int)numAtts;
   xassert(numAtts <= INT_MAX);
   serializePack(&numAttsI, 1, CDI_DATATYPE_INT, buf, size, position, context);
   for (size_t i = 0; i < numAtts; ++i)
-    cdiAttPack(p, varID, (int)i, buf, size, position, context);
+    cdiAttPack(attsp, (int)i, buf, size, position, context);
 }
 
 static
