@@ -319,6 +319,9 @@ void cdiInitialize(void)
       gribFixZSE(1);   // 1: Fix ZeroShiftError of simple packed spherical harmonics
       gribSetConst(1); // 1: Don't pack constant fields on regular grids
 #endif
+#ifdef  HAVE_LIBGRIB_API
+      grib_multi_support_off(NULL);
+#endif
 
       value = cdiGetenvInt("CDI_DEBUG");
       if ( value >= 0 ) CDI_Debug = (int) value;
@@ -498,6 +501,21 @@ double cdiInqGridMissval(void)
   return CDI_default_missval;
 }
 
+int cdiBaseFiletype(int filetype)
+{
+  switch (filetype)
+    {
+    case CDI_FILETYPE_NC:
+    case CDI_FILETYPE_NC2:
+    case CDI_FILETYPE_NC4:
+    case CDI_FILETYPE_NC4C:
+    case CDI_FILETYPE_NC5:   return CDI_FILETYPE_NETCDF;
+    default:                 return filetype;
+    }
+
+  return filetype;
+}
+
 /*
  * Local Variables:
  * c-file-style: "Java"
