@@ -53,7 +53,7 @@ const char *gribapiLibraryVersionString(void)
 
 void gribContainersNew(stream_t * streamptr)
 {
-  int editionNumber = (streamptr->filetype == CDI_FILETYPE_GRB) ? 1 : 2;
+  const int editionNumber = (streamptr->filetype == CDI_FILETYPE_GRB) ? 1 : 2;
 
 #ifdef HAVE_LIBCGRIBEX
   if ( editionNumber == 1 && !CDI_gribapi_grib1 )
@@ -62,15 +62,15 @@ void gribContainersNew(stream_t * streamptr)
   else
 #endif
     {
-      int nvars = streamptr->nvars;
+      const int nvars = streamptr->nvars;
 
-#if defined (GRIBCONTAINER2D)
+#ifdef GRIBCONTAINER2D
       gribContainer_t **gribContainers;
       gribContainers = (gribContainer_t **) Malloc(nvars*sizeof(gribContainer_t *));
 
       for ( int varID = 0; varID < nvars; ++varID )
         {
-          int nlevs = streamptr->vars[varID].nlevs;
+          const int nlevs = streamptr->vars[varID].nlevs;
           gribContainers[varID] = (gribContainer_t *) Malloc(nlevs*sizeof(gribContainer_t));
 
           for ( int levelID = 0; levelID < nlevs; ++levelID )
@@ -101,14 +101,14 @@ void gribContainersDelete(stream_t * streamptr)
 {
   if ( streamptr->gribContainers )
     {
-      int nvars = streamptr->nvars;
+      const int nvars = streamptr->nvars;
 
-#if defined (GRIBCONTAINER2D)
+#ifdef GRIBCONTAINER2D
       gribContainer_t **gribContainers = (gribContainer_t **) streamptr->gribContainers;
 
       for ( int varID = 0; varID < nvars; ++varID )
 	{
-          int nlevs = streamptr->vars[varID].nlevs;
+          const int nlevs = streamptr->vars[varID].nlevs;
           for ( int levelID = 0; levelID < nlevs; ++levelID )
             {
               gribHandleDelete(gribContainers[varID][levelID].gribHandle);
