@@ -14,11 +14,12 @@
 
 #define  GRIBAPI_MISSVAL  -9.E33
 
-/* GRIB2 Level Types */
+// GRIB2 Level Types
 #define  GRIB2_LTYPE_SURFACE               1
 #define  GRIB2_LTYPE_CLOUD_BASE            2
 #define  GRIB2_LTYPE_CLOUD_TOP             3
 #define  GRIB2_LTYPE_ISOTHERM0             4
+#define  GRIB2_LTYPE_TROPOPAUSE            7
 #define  GRIB2_LTYPE_TOA                   8
 #define  GRIB2_LTYPE_SEA_BOTTOM            9
 #define  GRIB2_LTYPE_ATMOSPHERE           10
@@ -98,6 +99,7 @@ static inline void *gribHandleNew(int editionNumber)
   if ( gh == NULL ) Error("grib_handle_new_from_samples failed!");
 
   if ( editionNumber == 1 ) GRIB_CHECK(my_grib_set_long(gh, "deleteLocalDefinition", 1L), 0);
+  if ( editionNumber == 2 ) GRIB_CHECK(my_grib_set_long(gh, "grib2LocalSectionPresent", 0L), 0);
   if ( editionNumber == 2 ) GRIB_CHECK(my_grib_set_long(gh, "numberOfValues", 0L), 0);
 
   return gh;
