@@ -251,7 +251,7 @@ static int addToDate(struct tm* me, long long amount, long unit)
     }
 }
 
-static char* makeDateString(struct tm* me)
+static char *makeDateString(struct tm *me)
 {
   char *result= (char *) Malloc(4+1+ 2+1+ 2+1+ 2+1+ 2+1+ 2+ 4+ 1);
   sprintf(result, "%04d-%02d-%02dT%02d:%02d:%02d.000", me->tm_year + 1900, me->tm_mon + 1, me->tm_mday, me->tm_hour, me->tm_min, me->tm_sec);
@@ -269,11 +269,16 @@ static int getAvailabilityOfRelativeTimes(grib_handle* gh, bool* outHaveForecast
         return 0;
 
       //case 55 and case 40455 are the same: 55 is the proposed standard value, 40455 is the value in the local use range that is used by the dwd until the standard is updated.
-      case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 15: case 32: case 33: case 40: case 41: case 44: case 45: case 48: case 51: case 53: case 54: case 55: case 56: case 60: case 1000: case 1002: case 1100: case 40033: case 40455: case 40456:
+      case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7:
+      case 15: case 32: case 33: case 40: case 41: case 44: case 45: case 48:
+      case 51: case 53: case 54: case 55: case 56: case 57: case 58: case 60:
+      case 1000: case 1002: case 1100: case 40033: case 40455: case 40456:
         *outHaveForecastTime = true, *outHaveTimeRange = false;
         return 0;
 
-      case 8: case 9: case 10: case 11: case 12: case 13: case 14: case 34: case 42: case 43: case 46: case 47: case 61: case 91: case 1001: case 1101: case 40034:
+      case 8: case 9: case 10: case 11: case 12: case 13: case 14:
+      case 34: case 42: case 43: case 46: case 47: case 61: case 67: case 68: case 91:
+      case 1001: case 1101: case 40034:
         *outHaveForecastTime = true, *outHaveTimeRange = true;
         return 0;
 
@@ -285,40 +290,41 @@ static int getAvailabilityOfRelativeTimes(grib_handle* gh, bool* outHaveForecast
 
 char* gribMakeTimeString(grib_handle* gh, CdiTimeType timeType)
 {
-  //Get the parts of the reference date.
+  // Get the parts of the reference date.
   struct tm date;
-  date.tm_mon = (int)gribGetLong(gh, "month") - 1;   //months are zero based in struct tm and one based in GRIB
+  date.tm_mon = (int)gribGetLong(gh, "month") - 1;   // months are zero based in struct tm and one based in GRIB
   date.tm_mday = (int)gribGetLong(gh, "day");
   date.tm_hour = (int)gribGetLong(gh, "hour");
   date.tm_min = (int)gribGetLong(gh, "minute");
+  date.tm_isdst = 0;
 
   if(gribEditionNumber(gh) == 1)
     {
-      date.tm_year = (int)gribGetLong(gh, "yearOfCentury");  //years are -1900 based both in struct tm and GRIB1
+      date.tm_year = (int)gribGetLong(gh, "yearOfCentury");  // years are -1900 based both in struct tm and GRIB1
     }
   else
     {
-      date.tm_year = (int)gribGetLong(gh, "year") - 1900;   //years are -1900 based in struct tm and zero based in GRIB2
+      date.tm_year = (int)gribGetLong(gh, "year") - 1900;   // years are -1900 based in struct tm and zero based in GRIB2
       date.tm_sec = (int)gribGetLong(gh, "second");
 
-      //If the start or end time are requested, we need to take the relative times into account.
+      // If the start or end time are requested, we need to take the relative times into account.
       if(timeType != kCdiTimeType_referenceTime)
         {
-          //Determine whether we have a forecast time and a time range.
+          // Determine whether we have a forecast time and a time range.
           bool haveForecastTime, haveTimeRange;
           if(getAvailabilityOfRelativeTimes(gh, &haveForecastTime, &haveTimeRange)) return NULL;
           if(timeType == kCdiTimeType_endTime && !haveTimeRange) return NULL;     //tell the caller that the requested time does not exist
 
-          //If we have relative times, apply the relative times to the date
+          // If we have relative times, apply the relative times to the date
           if(haveForecastTime)
             {
-              long offset = gribGetLongDefault(gh, "forecastTime", 0);  //if(stepUnits == indicatorOfUnitOfTimeRange) assert(startStep == forecastTime)
+              long offset = gribGetLongDefault(gh, "forecastTime", 0);  // if(stepUnits == indicatorOfUnitOfTimeRange) assert(startStep == forecastTime)
               long offsetUnit = gribGetLongDefault(gh, "indicatorOfUnitOfTimeRange", 255);
               if(addToDate(&date, offset, offsetUnit)) return NULL;
               if(timeType == kCdiTimeType_endTime)
                 {
                   assert(haveTimeRange);
-                  long range = gribGetLongDefault(gh, "lengthOfTimeRange", 0);       //if(stepUnits == indicatorOfUnitForTimeRange) assert(endStep == startStep + lengthOfTimeRange)
+                  long range = gribGetLongDefault(gh, "lengthOfTimeRange", 0);       // if(stepUnits == indicatorOfUnitForTimeRange) assert(endStep == startStep + lengthOfTimeRange)
                   long rangeUnit = gribGetLongDefault(gh, "indicatorOfUnitForTimeRange", 255);
                   if(addToDate(&date, range, rangeUnit)) return NULL;
                 }
@@ -527,12 +533,12 @@ void gribapiGetGridGaussianReduced(grib_handle *gh, grid_t *grid, int editionNum
 
   grid->size = numberOfPoints;
 
-  grid->nrowlon = (int)nlat;
-  grid->rowlon = (int *) Malloc(nlat * sizeof(int));
+  grid->reducedPointsSize = (int)nlat;
+  grid->reducedPoints = (int *) Malloc(nlat * sizeof(int));
   long *pl     = (long *) Malloc(nlat * sizeof(long));
   size_t dummy = nlat;
   FAIL_ON_GRIB_ERROR(grib_get_long_array, gh, "pl", pl, &dummy);
-  for ( size_t i = 0; i < nlat; ++i ) grid->rowlon[i] = (int)pl[i];
+  for ( size_t i = 0; i < nlat; ++i ) grid->reducedPoints[i] = (int)pl[i];
   Free(pl);
 
   grid->y.size  = nlat;
@@ -738,9 +744,9 @@ void gribapiGetGridUnstructured(grib_handle *gh, grid_t *grid, size_t numberOfPo
   long lpar;
   if ( grib_get_long(gh, "numberOfGridUsed", &lpar) == 0 )
     {
-      grid->number   = (int)lpar;
+      cdiDefVarKeyInt(&grid->keys, CDI_KEY_NUMBEROFGRIDUSED, (int)lpar);
       if ( grib_get_long(gh, "numberOfGridInReference", &lpar) == 0 )
-        grid->position = (int)lpar;
+        cdiDefVarKeyInt(&grid->keys, CDI_KEY_NUMBEROFGRIDINREFERENCE, (int)lpar);
       /*
         if ( grib_get_string(gh, "gridDescriptionFile", reference_link, &len) == 0 )
         {
@@ -749,7 +755,8 @@ void gribapiGetGridUnstructured(grib_handle *gh, grid_t *grid, size_t numberOfPo
         }
       */
       size_t len = (size_t)CDI_UUID_SIZE;
-      if ( grib_get_bytes(gh, "uuidOfHGrid", uuid, &len) == 0) memcpy(grid->uuid, uuid, CDI_UUID_SIZE);
+      if ( grib_get_bytes(gh, "uuidOfHGrid", uuid, &len) == 0)
+        cdiDefVarKeyBytes(&grid->keys, CDI_KEY_UUID, uuid, CDI_UUID_SIZE);
     }
 }
 
@@ -843,14 +850,13 @@ bool gribapiGetGrid(grib_handle *gh, grid_t *grid)
 
   if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_PROJECTION )
     {
-      long temp;
-      GRIB_CHECK(grib_get_long(gh, "iScansNegatively", &temp), 0);
-      grid->iScansNegatively = (bool)temp;
-      GRIB_CHECK(grib_get_long(gh, "jScansPositively", &temp), 0);
-      grid->jScansPositively = (bool)temp;
-      GRIB_CHECK(grib_get_long(gh, "jPointsAreConsecutive", &temp), 0);
-      grid->jPointsAreConsecutive = (bool)temp;
-      grid->scanningMode = 128*grid->iScansNegatively + 64*grid->jScansPositively + 32*grid->jPointsAreConsecutive;
+      long iScansNegatively, jScansPositively, jPointsAreConsecutive;
+      GRIB_CHECK(grib_get_long(gh, "iScansNegatively", &iScansNegatively), 0);
+      GRIB_CHECK(grib_get_long(gh, "jScansPositively", &jScansPositively), 0);
+      GRIB_CHECK(grib_get_long(gh, "jPointsAreConsecutive", &jPointsAreConsecutive), 0);
+
+      int scanningMode = 128*iScansNegatively + 64*jScansPositively + 32*jPointsAreConsecutive;
+      cdiDefVarKeyInt(&grid->keys, CDI_KEY_SCANNINGMODE, scanningMode);
       /* scanningMode  = 128 * iScansNegatively + 64 * jScansPositively + 32 * jPointsAreConsecutive;
                    64  = 128 * 0                + 64 *        1         + 32 * 0
                    00  = 128 * 0                + 64 *        0         + 32 * 0
@@ -867,9 +873,7 @@ bool gribapiGetGrid(grib_handle *gh, grid_t *grid)
         GRIB_CHECK(grib_get_long(gh, "level", &levelId), 0);
         Message("(param,ltype,level) = (%3d,%3d,%4d); Scanning mode = %02d -> bits:(%1d.%1d.%1d)*32;  uvRelativeToGrid = %02d",\
                 (int)paramId, (int)levelTypeId, (int)levelId,
-                grid->scanningMode,grid->jPointsAreConsecutive,
-                grid->jScansPositively, grid->iScansNegatively,
-                uvRelativeToGrid);
+                scanningMode, jPointsAreConsecutive, jScansPositively, iScansNegatively, uvRelativeToGrid);
       }
 #endif //HIRLAM_EXTENSIONS
     }
