@@ -4,6 +4,7 @@
 #include "cdi.h"
 #include <stdbool.h>
 
+#include "cdi_key.h"
 #include "cdi_att.h"
 
 extern int (*proj_lonlat_to_lcc_func)();
@@ -61,69 +62,51 @@ struct gridVirtTable
 };
 
 struct gridaxis_t {
-  char    name[CDI_MAX_NAME];
-  char    longname[CDI_MAX_NAME];
-  char    units[CDI_MAX_NAME];
-  char    dimname[CDI_MAX_NAME];
-  const char *stdname;
-  size_t  size;                  // number of values
-  short   flag;                  // 0: undefined 1:vals 2:first+inc
-  double  first, last, inc;
-  double *vals;
-  double *bounds;
+  size_t      size;                   // number of values
+  short       flag;                   // 0: undefined 1:vals 2:first+inc
+  double      first, last, inc;
+  double     *vals;
+  double     *bounds;
+  cdi_keys_t  keys;
 #ifndef USE_MPI
-  int clength;
-  char  **cvals;
+  int         clength;
+  char      **cvals;
 #endif
 };
 
 // GME Grid
 struct grid_gme_t {
-  int     nd, ni, ni2, ni3;       /* parameter for GRID_GME         */
+  int     nd, ni, ni2, ni3;           // parameter for GRID_GME
 };
 
 struct grid_t {
-  char    vdimname[CDI_MAX_NAME];
-  char    mapname[CDI_MAX_NAME];
-  char    mapping[CDI_MAX_NAME];
-  char   *name;
-  int     self;
-  size_t  size;
-  int     type;                   /* grid type                      */
-  int     datatype;               /* grid data type                 */
-  int     proj;                   /* grid projection                */
-  int     projtype;               /* grid projection type           */
-  mask_t *mask;
-  mask_t *mask_gme;
-  double *area;
+  char       *name;
+  int         self;
+  size_t      size;
+  int         type;                   // grid type
+  int         datatype;               // grid data type
+  int         proj;                   // grid projection
+  int         projtype;               // grid projection type
+  mask_t     *mask;
+  mask_t     *mask_gme;
+  double     *area;
   struct grid_gme_t  gme;
-  int     number, position;       /* parameter for GRID_REFERENCE   */
-  int     trunc;                  /* parameter for GRID_SPECTEAL    */
-  int     nvertex;
-  char   *reference;
-  unsigned char uuid[CDI_UUID_SIZE]; /* uuid for grid reference        */
-  int    *rowlon;
-  int     nrowlon;
-  int     np;                     /* number of parallels between a pole and the equator */
-  signed char isCyclic;           /* three possible states:
-                                   * -1 if unknown,
-                                   * 0 if found not cyclic, or
-                                   * 1 for global cyclic grids
-                                   */
-  bool    lcomplex;
-  bool    hasdims;
+  int         trunc;                  // parameter for GRID_SPECTRAL
+  int         nvertex;
+  int        *reducedPoints;
+  int         reducedPointsSize;
+  int         np;                     // number of parallels between a pole and the equator
+  signed char isCyclic;               // three possible states:
+                                      // -1 if unknown,
+                                      //  0 if found not cyclic, or
+                                      //  1 for global cyclic grids
+  bool        lcomplex;
+  bool        hasdims;
   struct gridaxis_t x;
   struct gridaxis_t y;
   const struct gridVirtTable *vtable;
-  cdi_atts_t atts;
-  int  scanningMode;
-  bool iScansNegatively, jScansPositively, jPointsAreConsecutive;
-  /* scanningMode  = 128 * iScansNegatively + 64 * jScansPositively + 32 * jPointsAreConsecutive;
-               64  = 128 * 0                + 64 *        1         + 32 * 0
-               00  = 128 * 0                + 64 *        0         + 32 * 0
-               96  = 128 * 0                + 64 *        1         + 32 * 1
-     Default / implicit scanning mode is 64:
-                        i and j scan positively, i points are consecutive (row-major)        */
+  cdi_keys_t  keys;
+  cdi_atts_t  atts;
 };
 
 
@@ -147,8 +130,6 @@ const double *gridInqXboundsPtr(int gridID);
 const double *gridInqYboundsPtr(int gridID);
 const double *gridInqAreaPtr(int gridID);
 
-const char *gridInqReferencePtr(int gridID);
-
 int gridGenerate(const grid_t *grid);
 
 //int gridIsEqual(int gridID1, int gridID2);
@@ -173,7 +154,8 @@ int gridVerifyGribParamLCC(double missval, double *lon_0, double *lat_0, double
 int gridVerifyGribParamSTERE(double missval, double *lon_0, double *lat_ts, double *lat_0,
                              double *a, double *xval_0, double *yval_0, double *x_0, double *y_0);
 
-bool isGaussGrid(size_t ysize, double yinc, const double *yvals);
+bool isGaussianLatitudes(size_t nlats, const double *latitudes);
+void gaussianLatitudes(size_t nlats, double *latitudes, double *weights);
 
 #endif
 /*
