@@ -16,11 +16,9 @@ static int initIegLib      = 0;
 static int iegDefaultDprec = 0;
 
 
-/*
- * A version string.
- */
+// A version string.
 #undef  LIBVERSION
-#define LIBVERSION      1.4.1
+#define LIBVERSION      1.4.2
 #define XSTRING(x)	#x
 #define STRING(x)	XSTRING(x)
 static const char ieg_libvers[] = STRING(LIBVERSION);
@@ -31,7 +29,7 @@ const char *iegLibraryVersion(void)
 }
 
 
-static int IEG_Debug = 0;    /* If set to 1, debugging */
+static int IEG_Debug = 0;    // If set to 1, debugging
 
 static
 void iegLibInit(void)
@@ -42,7 +40,6 @@ void iegLibInit(void)
   if ( envString )
     {
       int nrun = (strlen(envString) == 2) ? 1 : 2;
-
       int pos = 0;
       while ( nrun-- )
 	{
@@ -54,8 +51,7 @@ void iegLibInit(void)
 		  {
 		  case '4': iegDefaultDprec = EXSE_SINGLE_PRECISION; break;
 		  case '8': iegDefaultDprec = EXSE_DOUBLE_PRECISION; break;
-		  default:
-		    Message("Invalid digit in %s: %s", envName, envString);
+		  default: Message("Invalid digit in %s: %s", envName, envString);
 		  }
 		break;
 	      }
@@ -75,8 +71,8 @@ void iegLibInit(void)
 
 void iegDebug(int debug)
 {
+  if (debug) Message("debug level %d", debug);
   IEG_Debug = debug;
-  if (IEG_Debug) Message("debug level %d", debug);
 }
 
 static
@@ -95,7 +91,6 @@ void iegInit(iegrec_t *iegp)
 void iegInitMem(void *ieg)
 {
   iegrec_t *iegp = (iegrec_t *) ieg;
-
   memset(iegp->ipdb, 0, sizeof(iegp->ipdb));
   memset(iegp->igdb, 0, sizeof(iegp->igdb));
   memset(iegp->vct,  0, sizeof(iegp->vct));
@@ -107,7 +102,6 @@ void *iegNew(void)
   if ( ! initIegLib ) iegLibInit();
 
   iegrec_t *iegp = (iegrec_t *) Malloc(sizeof(iegrec_t));
-
   iegInit(iegp);
   iegInitMem(iegp);
 
@@ -136,11 +130,12 @@ int iegCheckFiletype(int fileID, int *swap)
 
   if ( fileRead(fileID, buffer, 4) != 4 ) return 0;
 
-  size_t blocklen  = get_UINT32(buffer);
-  size_t sblocklen = get_SUINT32(buffer);
+  const size_t blocklen  = get_UINT32(buffer);
+  const size_t sblocklen = get_SUINT32(buffer);
 
   if ( IEG_Debug ) Message("blocklen = %d sblocklen = %d", blocklen, sblocklen);
 
+  // clang-format off
   if ( blocklen == 636 || blocklen == 640 )
     {
      *swap = 0;
@@ -177,6 +172,7 @@ int iegCheckFiletype(int fileID, int *swap)
       pbuf = buffer+(37+5)*4;     dimy = (size_t) get_SUINT32(pbuf);
       pbuf = buffer+sblocklen+4;  data = (size_t) get_SUINT32(pbuf);
     }
+  // clang-format on
 
   fileRewind(fileID);
 
@@ -186,7 +182,7 @@ int iegCheckFiletype(int fileID, int *swap)
       Message("dimx = %lu dimy = %lu data = %lu", dimx, dimy, data);
     }
 
-  int found = data && (dimx*dimy*fact == data || dimx*dimy*8 == data);
+  const int found = data && (dimx*dimy*fact == data || dimx*dimy*8 == data);
   return found;
 }
 
@@ -196,7 +192,7 @@ void iegCopyMeta(void *dieg, void *sieg)
   iegrec_t *diegp = (iegrec_t *) dieg;
   iegrec_t *siegp = (iegrec_t *) sieg;
 
-  /*  diegp->byteswap = siegp->byteswap; */
+  // diegp->byteswap = siegp->byteswap;
   diegp->dprec    = siegp->dprec;
   diegp->refval   = siegp->refval;
 
@@ -210,10 +206,10 @@ int iegInqData(void *ieg, int prec, void *data)
 {
   iegrec_t *iegp = (iegrec_t *) ieg;
   int ierr = 0;
-  int byteswap = iegp->byteswap;
-  size_t datasize = iegp->datasize;
+  const int byteswap = iegp->byteswap;
+  const size_t datasize = iegp->datasize;
   void *buffer = iegp->buffer;
-  int dprec = iegp->dprec;
+  const int dprec = iegp->dprec;
 
   switch ( dprec )
     {
@@ -239,6 +235,7 @@ int iegInqData(void *ieg, int prec, void *data)
 	break;
       }
     case EXSE_DOUBLE_PRECISION:
+      {
 	if ( sizeof(FLT64) == 8 )
 	  {
 	    if ( byteswap ) swap8byte(buffer, datasize);
@@ -257,6 +254,7 @@ int iegInqData(void *ieg, int prec, void *data)
 	    Error("not implemented for %d byte float", sizeof(FLT64));
 	  }
 	break;
+      }
     default:
       {
 	Error("unexpected data precision %d", dprec);
@@ -286,19 +284,15 @@ iegDefData(iegrec_t *iegp, int prec, const void *data)
   int dprec = iegDefaultDprec ? iegDefaultDprec : iegp->dprec;
   iegp->dprec = dprec ? dprec : prec;
 
-  size_t datasize = (size_t)IEG_G_NumLon(iegp->igdb) * (size_t)IEG_G_NumLat(iegp->igdb);
-  size_t blocklen = datasize * (size_t)dprec;
+  const size_t datasize = (size_t)IEG_G_NumLon(iegp->igdb) * (size_t)IEG_G_NumLat(iegp->igdb);
+  const size_t blocklen = datasize * (size_t)dprec;
 
   iegp->datasize = datasize;
 
-  void *buffer = iegp->buffer;
-  size_t buffersize = iegp->buffersize;
-  if ( buffersize != blocklen )
+  if ( iegp->buffersize != blocklen )
     {
-      buffersize = blocklen;
-      buffer = Realloc(buffer, buffersize);
-      iegp->buffer = buffer;
-      iegp->buffersize = buffersize;
+      iegp->buffersize = blocklen;
+      iegp->buffer = Realloc(iegp->buffer, iegp->buffersize);
     }
 
   switch ( dprec )
@@ -306,11 +300,11 @@ iegDefData(iegrec_t *iegp, int prec, const void *data)
     case EXSE_SINGLE_PRECISION:
       {
 	if ( dprec == prec )
-	  memcpy(buffer, data, datasize*sizeof(FLT32));
+	  memcpy(iegp->buffer, data, datasize*sizeof(FLT32));
 	else
           {
             const double *restrict p = (const double *)data;
-            float *restrict q = (float *)buffer;
+            float *restrict q = (float *)iegp->buffer;
             for (size_t i = 0; i < datasize; i++) q[i] = (float)p[i];
           }
 	break;
@@ -318,11 +312,11 @@ iegDefData(iegrec_t *iegp, int prec, const void *data)
     case EXSE_DOUBLE_PRECISION:
       {
 	if ( dprec == prec )
-	  memcpy(buffer, data, datasize*sizeof(FLT64));
+	  memcpy(iegp->buffer, data, datasize*sizeof(FLT64));
 	else
           {
             const float *restrict p = (const float *)data;
-            double *restrict q = (double *)buffer;
+            double *restrict q = (double *)iegp->buffer;
             for ( size_t i = 0; i < datasize; i++) q[i] = p[i];
           }
 	break;
@@ -357,20 +351,19 @@ int iegRead(int fileID, void *ieg)
 
   if ( ! iegp->checked )
     {
-      int status = iegCheckFiletype(fileID, &iegp->byteswap);
+      const int status = iegCheckFiletype(fileID, &iegp->byteswap);
       if ( status == 0 ) Error("Not a IEG file!");
       iegp->checked = 1;
     }
 
-  int byteswap = iegp->byteswap;
+  const int byteswap = iegp->byteswap;
 
-  /* read header record */
+  // read header record
   size_t blocklen = binReadF77Block(fileID, byteswap);
 
   if ( fileEOF(fileID) ) return -1;
 
-  if ( IEG_Debug )
-    Message("blocklen = %lu", blocklen);
+  if ( IEG_Debug ) Message("blocklen = %lu", blocklen);
 
   int dprec = 0;
   if ( blocklen == 636 || blocklen == 640 )
@@ -386,10 +379,10 @@ int iegRead(int fileID, void *ieg)
   iegp->dprec = dprec;
 
   binReadInt32(fileID, byteswap, 37, buf.i32);
-  for ( size_t i = 0; i < 37; i++ ) iegp->ipdb[i] = (int)buf.i32[i];
+  for ( int i = 0; i < 37; i++ ) iegp->ipdb[i] = (int)buf.i32[i];
 
   binReadInt32(fileID, byteswap, 18, buf.i32);
-  for ( size_t i = 0; i < 18; i++ ) iegp->igdb[i] = (int)buf.i32[i];
+  for ( int i = 0; i < 18; i++ ) iegp->igdb[i] = (int)buf.i32[i];
 
   if ( blocklen == 636 || blocklen == 1036 )
     {
@@ -405,21 +398,19 @@ int iegRead(int fileID, void *ieg)
     }
 
   binReadInt32(fileID, byteswap, 3, buf.i32);
-  for ( size_t i = 0; i < 3; i++ ) iegp->igdb[18+i] = (int)buf.i32[i];
+  for ( int i = 0; i < 3; i++ ) iegp->igdb[18+i] = (int)buf.i32[i];
 
   if ( dprec == EXSE_SINGLE_PRECISION )
     {
       fileRead(fileID, buf.f, 400);
       if ( byteswap ) swap4byte(buf.f, 100);
-      for ( size_t i = 0; i < 100; i++ )
-	iegp->vct[i] = (double)buf.f[i];
+      for ( int i = 0; i < 100; i++ ) iegp->vct[i] = (double)buf.f[i];
     }
   else
     {
       fileRead(fileID, buf.d, 800);
       if ( byteswap ) swap8byte(buf.d, 100);
-      for ( size_t i = 0; i < 100; i++ )
-	iegp->vct[i] = buf.d[i];
+      for ( int i = 0; i < 100; i++ ) iegp->vct[i] = buf.d[i];
     }
 
   size_t blocklen2 = binReadF77Block(fileID, byteswap);
@@ -430,28 +421,25 @@ int iegRead(int fileID, void *ieg)
       return -1;
     }
 
-  size_t datasize = iegp->datasize
-    = (size_t)IEG_G_NumLon(iegp->igdb) * (size_t)IEG_G_NumLat(iegp->igdb);
+  iegp->datasize = (size_t)IEG_G_NumLon(iegp->igdb) * (size_t)IEG_G_NumLat(iegp->igdb);
 
-  if ( IEG_Debug ) Message("datasize = %lu", iegp->datasize);
+  if ( IEG_Debug ) Message("datasize = %zu", iegp->datasize);
 
   blocklen = binReadF77Block(fileID, byteswap);
 
-  void *buffer = iegp->buffer;
   if ( iegp->buffersize < blocklen )
     {
-      iegp->buffer = buffer = Realloc(buffer, blocklen);
+      iegp->buffer = Realloc(iegp->buffer, blocklen);
       iegp->buffersize = blocklen;
     }
 
-  if ( dprec != (int) (blocklen/datasize) )
+  if ( dprec != (int) (blocklen/iegp->datasize) )
     {
-      Warning("data precision differ! (h = %d; d = %d)",
-	      (int) dprec, (int) (blocklen/datasize));
+      Warning("data precision differ! (h = %d; d = %d)", (int) dprec, (int) (blocklen/iegp->datasize));
       return -1;
     }
 
-  fileRead(fileID, buffer, blocklen);
+  fileRead(fileID, iegp->buffer, blocklen);
 
   blocklen2 = binReadF77Block(fileID, byteswap);
 
@@ -469,18 +457,18 @@ int iegWrite(int fileID, void *ieg)
 {
   iegrec_t *iegp = (iegrec_t *) ieg;
   union { INT32 i32[200]; float fvct[100]; } buf;
-  int dprec  = iegp->dprec;
-  int byteswap = iegp->byteswap;
+  const int dprec  = iegp->dprec;
+  const int byteswap = iegp->byteswap;
 
-  /* write header record */
+  // write header record
   size_t blocklen = ( dprec == EXSE_SINGLE_PRECISION ) ? 636 : 1040;
 
   binWriteF77Block(fileID, byteswap, blocklen);
 
-  for ( size_t i = 0; i < 37; i++ ) buf.i32[i] = (INT32) iegp->ipdb[i];
+  for ( int i = 0; i < 37; i++ ) buf.i32[i] = (INT32) iegp->ipdb[i];
   binWriteInt32(fileID, byteswap, 37, buf.i32);
 
-  for ( size_t i = 0; i < 18; i++ ) buf.i32[i] = (INT32) iegp->igdb[i];
+  for ( int i = 0; i < 18; i++ ) buf.i32[i] = (INT32) iegp->igdb[i];
   binWriteInt32(fileID, byteswap, 18, buf.i32);
 
   FLT64 refval = (FLT64)iegp->refval;
@@ -490,12 +478,12 @@ int iegWrite(int fileID, void *ieg)
   else
     binWriteFlt64(fileID, byteswap, 1, &refval);
 
-  for ( size_t i = 0; i < 3; i++ ) buf.i32[i] = (INT32) iegp->igdb[18+i];
+  for ( int i = 0; i < 3; i++ ) buf.i32[i] = (INT32) iegp->igdb[18+i];
   binWriteInt32(fileID, byteswap, 3, buf.i32);
 
   if ( dprec == EXSE_SINGLE_PRECISION )
     {
-      for ( size_t i = 0; i < 100; i++ ) buf.fvct[i] = (float) iegp->vct[i];
+      for ( int i = 0; i < 100; i++ ) buf.fvct[i] = (float) iegp->vct[i];
       binWriteFlt32(fileID, byteswap, 100, buf.fvct);
     }
   else
@@ -505,25 +493,21 @@ int iegWrite(int fileID, void *ieg)
 
   binWriteF77Block(fileID, byteswap, blocklen);
 
-  size_t datasize = (size_t)iegp->igdb[4] * (size_t)iegp->igdb[5];
-  blocklen = datasize * (size_t)dprec;
+  iegp->datasize = (size_t)iegp->igdb[4] * (size_t)iegp->igdb[5];
+  blocklen = iegp->datasize * (size_t)dprec;
 
   binWriteF77Block(fileID, byteswap, blocklen);
 
-  iegp->datasize = datasize;
-
-  void *buffer = iegp->buffer;
-
   switch ( dprec )
     {
     case EXSE_SINGLE_PRECISION:
       {
-	binWriteFlt32(fileID, byteswap, datasize, (FLT32 *) buffer);
+	binWriteFlt32(fileID, byteswap, iegp->datasize, (FLT32 *) iegp->buffer);
 	break;
       }
     case EXSE_DOUBLE_PRECISION:
       {
-	binWriteFlt64(fileID, byteswap, datasize, (FLT64 *) buffer);
+	binWriteFlt64(fileID, byteswap, iegp->datasize, (FLT64 *) iegp->buffer);
 	break;
       }
     default:
