@@ -247,22 +247,25 @@ int cdiFallbackIterator_levelType(CdiIterator *super, int levelSelector, char **
   CdiFallbackIterator *me = (CdiFallbackIterator*)(void *)super;
   int zaxisId = vlistInqVarZaxis(me->vlistId, me->curVariable);
   (void)levelSelector;
-#define copyString(outPointer, function) do \
+#define copyString(outPointer, key) do \
     { \
       if(outPointer) \
         { \
           char tempBuffer[CDI_MAX_NAME]; \
-          function(zaxisId, tempBuffer); \
+          int length = CDI_MAX_NAME;     \
+          cdiInqKeyString(zaxisId, CDI_GLOBAL, key, tempBuffer, &length); \
           *outPointer = strdup(tempBuffer); \
         } \
     } \
   while(0)
-  copyString(outName, zaxisInqName);    //FIXME: zaxisInqName is unsafe.
-  copyString(outLongName, zaxisInqLongname);    //FIXME: zaxisInqLongname is unsafe.
-  copyString(outStdName, zaxisInqStdname);    //FIXME: zaxisInqStdname is unsafe.
-  copyString(outUnit, zaxisInqUnits);    //FIXME: zaxisInqUnits is unsafe.
+  copyString(outName, CDI_KEY_NAME);
+  copyString(outLongName, CDI_KEY_LONGNAME);
+  copyString(outStdName, CDI_KEY_STDNAME);
+  copyString(outUnit, CDI_KEY_UNITS);
 #undef copyString
-  return zaxisInqLtype(zaxisId);
+  int ltype = 0;
+  cdiInqKeyInt(zaxisId, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
+  return ltype;
 }
 
 int cdiFallbackIterator_level(CdiIterator *super, int levelSelector, double *outValue1, double *outValue2)
@@ -305,10 +308,25 @@ int cdiFallbackIterator_zaxisUuid(CdiIterator *super, int *outVgridNumber, int *
 {
   CdiFallbackIterator *me = (CdiFallbackIterator*)(void *)super;
   int zaxisId = vlistInqVarZaxis(me->vlistId, me->curVariable);
-  if(zaxisInqLtype(zaxisId) != ZAXIS_HYBRID) return CDI_EINVAL;
-  if(outVgridNumber) *outVgridNumber = zaxisInqNumber(zaxisId);
-  if(outLevelCount) *outLevelCount = zaxisInqNlevRef(zaxisId);
-  if(outUuid) zaxisInqUUID(zaxisId, outUuid);
+  int ltype = 0;
+  cdiInqKeyInt(zaxisId, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
+  if (ltype != ZAXIS_HYBRID) return CDI_EINVAL;
+  if (outVgridNumber)
+    {
+      *outVgridNumber = 0;
+      cdiInqKeyInt(zaxisId, CDI_GLOBAL, CDI_KEY_NUMBEROFVGRIDUSED, outVgridNumber);
+    }
+  if (outLevelCount)
+    {
+      *outLevelCount = 0;
+      cdiInqKeyInt(zaxisId, CDI_GLOBAL, CDI_KEY_NLEV, outLevelCount);
+    }
+  if (outUuid)
+    {
+      int length = CDI_UUID_SIZE;
+      memset(outUuid, 0, length);
+      cdiInqKeyBytes(zaxisId, CDI_GLOBAL, CDI_KEY_UUID, outUuid, &length);
+    }
   return CDI_NOERR;
 }
 
