@@ -1,12 +1,11 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #endif
 
 #include "iterator_grib.h"
 
 #include "cdi.h"
 #include "cdi_int.h"
-#include "cgribex.h"
 #include "dmemory.h"
 #include "error.h"
 #include "gribapi.h"
@@ -578,7 +577,7 @@ void cdiGribIterator_readField(CdiIterator *super, double *buffer, size_t *nmiss
 {
   CdiGribIterator *me = (CdiGribIterator*)(void *)super;
 
-  GRIB_CHECK(my_grib_set_double(me->gribHandle, "missingValue", cdiDefaultMissval), 0);
+  GRIB_CHECK(my_grib_set_double(me->gribHandle, "missingValue", CDI_default_missval), 0);
   gribGetDoubleArray(me->gribHandle, "values", buffer);
   long gridType = gribGetLong(me->gribHandle, "gridDefinitionTemplateNumber");
   if(nmiss)
@@ -692,14 +691,16 @@ int cdiGribIterator_getLong(CdiGribIterator *me, const char *key, long *result)
 */
 int cdiGribIterator_getLength(CdiGribIterator *me, const char *key, size_t *result)
 {
+#ifdef HAVE_LIBGRIB_API
 #ifdef HAVE_GRIB_GET_LENGTH
   return grib_get_length(me->gribHandle, key, result);
-#elif defined(HAVE_LIBGRIB_API)
+#else
   (void)me;
   (void)key;
   (void)result;
   Error("grib_get_length() is not available, so cdiGribIterator_getLength() can't be used");
   return -1;
+#endif
 #else
   (void)me;
   (void)key;
