@@ -303,7 +303,7 @@ countVarChunkMissingVals(int vlistID, int varID,
                          struct streamMapping *mapping,
                          int chunkLen, const double *restrict data)
 {
-  int nmiss = 0;
+  size_t nmiss = 0;
   if (mapping->hasMissing[varID])
     {
       double missval = vlistInqVarMissval(vlistID, varID);
@@ -450,7 +450,7 @@ writeNetCDFStream(size_t streamIdx,
           xt_idxlist_delete(preWriteChunk);
         }
         /* count missing values if appropriate */
-        int nmiss
+        size_t nmiss
           = countVarChunkMissingVals(vlistID, varID, mapping,
                                      PPM_extents_size(3, varChunk),
                                      data);
@@ -513,7 +513,7 @@ cdiPioServerCdfDefVars(stream_t *streamptr)
   if (commInqIOMode() == PIO_NONE
       || ((rank = commInqRankColl())
           == (rankOpen = cdiPioSerialOpenFileMap(streamptr->self))))
-    cdfDefVars(streamptr);
+    cdfDefCoordinateVars(streamptr);
 }
 
 static void
@@ -553,7 +553,7 @@ writeNetCDFStream(size_t streamIdx,
         gatherArray(headerIdx, streamIdx, data, dstList, conf);
         if (writerRank == collRank)
           {
-            int nmiss = countVarChunkMissingVals(vlistID, varID,
+            size_t nmiss = countVarChunkMissingVals(vlistID, varID,
                                                  mapping, varSize, data);
             streamWriteVar(streamID, varID, data, nmiss);
           }
@@ -1053,7 +1053,7 @@ writeGribStream(size_t streamIdx,
             varIdx += varID != prevVarID;
             size_t recordSize = writtenRecords[recordIdx].dataSize;
             size_t nvals = recordSize / sizeof (double);
-            int nmiss
+            size_t nmiss
               = countVarChunkMissingVals(vlistID, varID, mapping, (int)nvals,
                                          data + recordDataOfs);
             streamWriteVarSlice(streamID, varID, level, data + recordDataOfs, nmiss);
@@ -1107,18 +1107,14 @@ readGetBuffers(size_t streamIdx, const struct cdiPioConf *conf)
     int currentDataBufSize = 0;
     int filetype = streamInqFiletype(streamID);
 
-    switch (filetype)
+    switch (cdiBaseFiletype(filetype))
       {
       case CDI_FILETYPE_GRB:
       case CDI_FILETYPE_GRB2:
         writeGribStream(streamIdx, map, &data, &currentDataBufSize, conf);
         break;
 #ifdef HAVE_NETCDF4
-      case CDI_FILETYPE_NC:
-      case CDI_FILETYPE_NC2:
-      case CDI_FILETYPE_NC4:
-      case CDI_FILETYPE_NC4C:
-      case CDI_FILETYPE_NC5:
+      case CDI_FILETYPE_NETCDF:
         writeNetCDFStream(streamIdx, map, &data, &currentDataBufSize, conf);
         break;
 #endif
@@ -1257,14 +1253,10 @@ cdiPioServerStreamClose(stream_t *streamptr, int recordBufIsToBeDeleted)
     Warning("File %s not open!", streamptr->filename);
   else
     {
-      switch (filetype)
+      switch (cdiBaseFiletype(filetype))
         {
 #if defined (HAVE_LIBNETCDF) && ! defined (HAVE_PARALLEL_NC4)
-        case CDI_FILETYPE_NC:
-        case CDI_FILETYPE_NC2:
-        case CDI_FILETYPE_NC4:
-        case CDI_FILETYPE_NC4C:
-        case CDI_FILETYPE_NC5:
+        case CDI_FILETYPE_NETCDF:
           {
             int rank, rankOpen = cdiPioSerialOpenFileMap(streamptr->self);
             if (commInqIOMode() == PIO_NONE
