@@ -147,13 +147,13 @@ void printArray ( const char *cdiPioDebugString, const char *ps, const void *arr
 
   switch ( datatype )
     {
-    case DATATYPE_INT:
+    case CDI_DATATYPE_INT:
       iArray = ( int * ) array;
       for ( i = 0; i < n-1; i++ )
 	fprintf ( stdout, "%d ", * ( iArray + i ));
       fprintf ( stdout, "%d\n", * ( iArray + n - 1 ));
       break;
-    case DATATYPE_FLT:
+    case CDI_DATATYPE_FLT:
       dArray = ( double * ) array;
       for ( i = 0; i < n-1; i++ )
 	fprintf ( stdout, "%.2f ", * ( dArray + i ));
@@ -184,7 +184,7 @@ cdiPioQueryVarDims(int varShape[3], int vlistID, int varID)
       varShape[1] = gridInqSize(gridID) / 2;
       break;
     case GRID_GENERIC:
-    case GRID_LCC:
+    case GRID_PROJECTION:
     case GRID_GME:
     case GRID_CURVILINEAR:
     case GRID_UNSTRUCTURED:
