@@ -333,19 +333,15 @@ const char *streamFilesuffix(int filetype)
   static const char *extSuffix = ".ext";
   static const char *iegSuffix = ".ieg";
 
-  switch (filetype)
+  switch (cdiBaseFiletype(filetype))
     {
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:   return ncSuffix;
     case CDI_FILETYPE_GRB:
-    case CDI_FILETYPE_GRB2:  return grbSuffix;
-    case CDI_FILETYPE_SRV:   return srvSuffix;
-    case CDI_FILETYPE_EXT:   return extSuffix;
-    case CDI_FILETYPE_IEG:   return iegSuffix;
-    default:                 return noSuffix;
+    case CDI_FILETYPE_GRB2:    return grbSuffix;
+    case CDI_FILETYPE_SRV:     return srvSuffix;
+    case CDI_FILETYPE_EXT:     return extSuffix;
+    case CDI_FILETYPE_IEG:     return iegSuffix;
+    case CDI_FILETYPE_NETCDF:  return ncSuffix;
+    default:                   return noSuffix;
     }
 }
 
@@ -376,7 +372,7 @@ int cdiInqContents(stream_t *streamptr)
   int status = 0;
   int filetype = streamptr->filetype;
 
-  switch (filetype)
+  switch (cdiBaseFiletype(filetype))
     {
 #ifdef  HAVE_LIBGRIB
     case CDI_FILETYPE_GRB:
@@ -408,11 +404,7 @@ int cdiInqContents(stream_t *streamptr)
       }
 #endif
 #ifdef  HAVE_LIBNETCDF
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:
+    case CDI_FILETYPE_NETCDF:
       {
         status = cdfInqContents(streamptr);
 	break;
@@ -430,8 +422,7 @@ int cdiInqContents(stream_t *streamptr)
 
   if ( status == 0 )
     {
-      int vlistID = streamptr->vlistID;
-      int taxisID = vlistInqTaxis(vlistID);
+      int taxisID = vlistInqTaxis(streamptr->vlistID);
       if ( taxisID != CDI_UNDEFID )
         {
           taxis_t *taxisptr1 = &streamptr->tsteps[0].taxis;
@@ -886,7 +877,6 @@ void streamDefaultValue ( stream_t * streamptr )
   streamptr->tsteps            = NULL;
   streamptr->tstepsTableSize   = 0;
   streamptr->tstepsNextID      = 0;
-  streamptr->historyID         = CDI_UNDEFID;
   streamptr->vlistID           = CDI_UNDEFID;
   streamptr->globalatts        = 0;
   streamptr->localatts         = 0;
@@ -900,6 +890,9 @@ void streamDefaultValue ( stream_t * streamptr )
   basetimeInit(&streamptr->basetime);
 
 #ifdef HAVE_LIBNETCDF
+  streamptr->nc_complex_float_id  = CDI_UNDEFID;
+  streamptr->nc_complex_double_id = CDI_UNDEFID;
+
   for ( int i = 0; i < MAX_ZAXES_PS; i++ ) streamptr->zaxisID[i]  = CDI_UNDEFID;
   for ( int i = 0; i < MAX_ZAXES_PS; i++ ) streamptr->nczvarID[i] = CDI_UNDEFID;
 
@@ -952,7 +945,7 @@ void cdiStreamCloseDefaultDelegate(stream_t *streamptr, int recordBufIsToBeDelet
   if ( fileID == CDI_UNDEFID )
     Warning("File %s not open!", streamptr->filename);
   else
-    switch (filetype)
+    switch (cdiBaseFiletype(filetype))
       {
 #if defined(HAVE_LIBGRIB) && (defined(HAVE_LIBCGRIBEX) || defined(HAVE_LIBGRIB_API))
       case CDI_FILETYPE_GRB:
@@ -996,11 +989,7 @@ void cdiStreamCloseDefaultDelegate(stream_t *streamptr, int recordBufIsToBeDelet
         }
 #endif
 #ifdef  HAVE_LIBNETCDF
-      case CDI_FILETYPE_NC:
-      case CDI_FILETYPE_NC2:
-      case CDI_FILETYPE_NC4:
-      case CDI_FILETYPE_NC4C:
-      case CDI_FILETYPE_NC5:
+      case CDI_FILETYPE_NETCDF:
         {
           cdfClose(fileID);
           if (streamptr->ntsteps == 0)
@@ -1144,14 +1133,10 @@ void cdiStreamSync_(stream_t *streamptr)
     {
       if ( streamptr->filemode == 'w' || streamptr->filemode == 'a' )
 	{
-	  switch (filetype)
+	  switch (cdiBaseFiletype(filetype))
 	    {
 #ifdef  HAVE_LIBNETCDF
-	    case CDI_FILETYPE_NC:
-	    case CDI_FILETYPE_NC2:
-	    case CDI_FILETYPE_NC4:
-	    case CDI_FILETYPE_NC4C:
-	    case CDI_FILETYPE_NC5:
+	    case CDI_FILETYPE_NETCDF:
 	      {
 		void cdf_sync(int ncid);
 		if ( streamptr->ncmode == 2 ) cdf_sync(fileID);
@@ -1227,12 +1212,7 @@ int cdiStreamDefTimestep_(stream_t *streamptr, int tsID)
   streamptr->ntsteps = tsID + 1;
 
 #ifdef HAVE_LIBNETCDF
-  if ((streamptr->filetype == CDI_FILETYPE_NC  ||
-       streamptr->filetype == CDI_FILETYPE_NC2 ||
-       streamptr->filetype == CDI_FILETYPE_NC5 ||
-       streamptr->filetype == CDI_FILETYPE_NC4 ||
-       streamptr->filetype == CDI_FILETYPE_NC4C)
-      && time_is_varying)
+  if (cdiBaseFiletype(streamptr->filetype) == CDI_FILETYPE_NETCDF  && time_is_varying)
     {
       void (*myCdfDefTimestep)(stream_t *streamptr, int tsID)
         = (void (*)(stream_t *, int))
@@ -1309,7 +1289,8 @@ int streamInqTimestep(int streamID, int tsID)
   stream_t *streamptr = stream_to_pointer(streamID);
   int vlistID = streamptr->vlistID;
 
-  if ( tsID < streamptr->rtsteps )
+  if (tsID < streamptr->ntsteps) streamptr->tsteps[tsID].curRecID = CDI_UNDEFID; // fix for netCDF
+  if (tsID < streamptr->rtsteps)
     {
       streamptr->curTsID = tsID;
       nrecs = streamptr->tsteps[tsID].nrecs;
@@ -1322,17 +1303,14 @@ int streamInqTimestep(int streamID, int tsID)
       return nrecs;
     }
 
-  if ( tsID >= streamptr->ntsteps && streamptr->ntsteps != CDI_UNDEFID )
-    {
-      return 0;
-    }
+  if ( tsID >= streamptr->ntsteps && streamptr->ntsteps != CDI_UNDEFID ) return 0;
 
   int filetype = streamptr->filetype;
 
   if ( CDI_Debug )
     Message("streamID = %d  tsID = %d  filetype = %d", streamID, tsID, filetype);
 
-  switch (filetype)
+  switch (cdiBaseFiletype(filetype))
     {
 #ifdef  HAVE_LIBGRIB
     case CDI_FILETYPE_GRB:
@@ -1364,11 +1342,7 @@ int streamInqTimestep(int streamID, int tsID)
       }
 #endif
 #ifdef  HAVE_LIBNETCDF
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:
+    case CDI_FILETYPE_NETCDF:
       {
         nrecs = cdfInqTimestep(streamptr, tsID);
 	break;
@@ -1643,54 +1617,40 @@ void cdiStreamSetupVlist_(stream_t *streamptr, int vlistID)
   if (streamptr->filemode == 'w')
     {
       tstepsNewEntry(streamptr); // timestep 0
-      int vlistID = streamptr->vlistID;
-      int time_is_varying = vlistHasTime(vlistID);
+      int vlistIDw = streamptr->vlistID;
+      int time_is_varying = vlistHasTime(vlistIDw);
       if ( time_is_varying )
         {
-          int taxisID = vlistInqTaxis(vlistID);
+          int taxisID = vlistInqTaxis(vlistIDw);
           if ( taxisID == CDI_UNDEFID )
             {
               Warning("taxisID undefined for fileID = %d! Using absolute time axis.", streamptr->self);
               taxisID = taxisCreate(TAXIS_ABSOLUTE);
-              vlistDefTaxis(vlistID, taxisID);
+              vlistDefTaxis(vlistIDw, taxisID);
             }
 
-          if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
-            switch (streamptr->filetype)
-              {
 #ifdef HAVE_LIBNETCDF
-              case CDI_FILETYPE_NC:
-              case CDI_FILETYPE_NC2:
-              case CDI_FILETYPE_NC4:
-              case CDI_FILETYPE_NC4C:
-              case CDI_FILETYPE_NC5:
-                {
-                  taxis_t *taxisptr = taxisPtr(taxisID);
-                  if ( taxisptr->rdate == -1 )
-                    {
-                      int vdate = taxisInqVdate(taxisID);
-                      if ( vdate == 0 ) vdate = 10101;
-                      taxisDefRdate(taxisID, vdate);
-                    }
-                }
-                break;
-#endif
-              default:
-                ;
+          if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
+            if (cdiBaseFiletype(streamptr->filetype) == CDI_FILETYPE_NETCDF)
+              {
+                taxis_t *taxisptr = taxisPtr(taxisID);
+                if ( taxisptr->rdate == -1 )
+                  {
+                    int vdate = taxisInqVdate(taxisID);
+                    if ( vdate == 0 ) vdate = 10101;
+                    taxisDefRdate(taxisID, vdate);
+                  }
               }
+#endif
           ptaxisCopy(&streamptr->tsteps[0].taxis, taxisPtr(taxisID));
         }
 
-      switch (streamptr->filetype)
+      switch (cdiBaseFiletype(streamptr->filetype))
         {
 #ifdef HAVE_LIBNETCDF
-        case CDI_FILETYPE_NC:
-        case CDI_FILETYPE_NC2:
-        case CDI_FILETYPE_NC4:
-        case CDI_FILETYPE_NC4C:
-        case CDI_FILETYPE_NC5:
+        case CDI_FILETYPE_NETCDF:
           {
-            /* calls cdfDefVars in serial mode but
+            /* calls cdfDefCoordinateVars in serial mode but
              * cdiPioClientStreamNOP (i.e. nothing) on client ranks
              * and cdiPioServerCdfDefVars on server ranks in parallel mode*/
             void (*myCdfDefVars)(stream_t *streamptr)
@@ -1764,9 +1724,9 @@ void streamDestroyP ( void * streamptr )
 }
 
 
-void streamPrintP   ( void * streamptr, FILE * fp )
+void streamPrintP(void *streamptr, FILE *fp)
 {
-  stream_t * sp = ( stream_t * ) streamptr;
+  stream_t *sp = (stream_t *) streamptr;
 
   if ( !sp ) return;
 
@@ -1791,7 +1751,6 @@ void streamPrintP   ( void * streamptr, FILE * fp )
           "tstepsNextID  = %d\n"
           "ncmode        = %d\n"
           "vlistID       = %d\n"
-          "historyID     = %d\n"
           "globalatts    = %d\n"
           "localatts     = %d\n"
           "unreduced     = %d\n"
@@ -1803,7 +1762,7 @@ void streamPrintP   ( void * streamptr, FILE * fp )
           sp->filetype, sp->byteorder, sp->fileID, sp->filemode,
           sp->filename, sp->nrecs, sp->nvars, sp->varsAllocated,
           sp->curTsID, sp->rtsteps, sp->ntsteps, sp->tstepsTableSize,
-          sp->tstepsNextID, sp->ncmode, sp->vlistID, sp->historyID,
+          sp->tstepsNextID, sp->ncmode, sp->vlistID,
           sp->globalatts, sp->localatts, sp->unreduced, sp->sortname,
           sp->have_missval, sp->comptype, sp->complevel);
 }
