@@ -456,6 +456,35 @@ void cgribexGetLevel(int *isec1, int *leveltype, int *level1, int *level2)
   else if ( *leveltype == GRIB1_LTYPE_99 || *leveltype == GRIB1_LTYPE_ISOBARIC_PA ) *leveltype = GRIB1_LTYPE_ISOBARIC;
 }
 
+static
+void cgribexDefProjLCC(int *isec2, int gridID)
+{
+  const bool earthIsOblate = gribbyte_get_bit(ISEC2_ResFlag, 2);
+  const double a = earthIsOblate ? 6378160. : 6367470.;
+  const double rf = earthIsOblate ? 297.0 : 0;
+  const double xval_0 = ISEC2_FirstLon * 0.001;
+  const double yval_0 = ISEC2_FirstLat * 0.001;
+  const double lon_0  = ISEC2_Lambert_Lov * 0.001;
+  double lat_1  = ISEC2_Lambert_LatS1 * 0.001;
+  double lat_2  = ISEC2_Lambert_LatS2 * 0.001;
+  const bool lsouth = gribbyte_get_bit(ISEC2_Lambert_ProjFlag, 1);
+  if ( lsouth ) { lat_1 = -lat_1; lat_2 = -lat_2; }
+
+  const double lat_0 = lat_2;
+  double x_0 = CDI_grid_missval;
+  double y_0 = CDI_grid_missval;
+
+  if ( proj_lonlat_to_lcc_func )
+    {
+      x_0 = xval_0; y_0 = yval_0;
+      proj_lonlat_to_lcc_func(CDI_grid_missval, lon_0, lat_0, lat_1, lat_2, a, rf, (size_t)1, &x_0, &y_0);
+      if ( IS_NOT_EQUAL(x_0, CDI_grid_missval) && IS_NOT_EQUAL(y_0, CDI_grid_missval) )
+        { x_0 = -x_0; y_0 = -y_0; }
+    }
+
+  gridDefParamLCC(gridID, CDI_grid_missval, lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0);
+}
+
 static
 void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, size_t recsize,
                       off_t position, int comptype, int lmv, int iret)
@@ -476,7 +505,6 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
   record_t *record = &streamptr->tsteps[tsID].records[recID];
 
   const int tsteptype = cgribexGetTsteptype(ISEC1_TimeRange);
-  const int numavg    = ISEC1_AvgNum;
 
   int leveltype, level1, level2;
   cgribexGetLevel(isec1, &leveltype, &level1, &level2);
@@ -514,29 +542,7 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
         }
       else if ( gridptr->projtype == CDI_PROJ_LCC )
         {
-          const bool earthIsOblate = gribbyte_get_bit(ISEC2_ResFlag, 2);
-          const double a = earthIsOblate ? 6378160. : 6367470.;
-          const double rf = earthIsOblate ? 297.0 : 0;
-          const double xval_0 = ISEC2_FirstLon * 0.001;
-          const double yval_0 = ISEC2_FirstLat * 0.001;
-          const double lon_0  = ISEC2_Lambert_Lov * 0.001;
-          double lat_1  = ISEC2_Lambert_LatS1 * 0.001;
-          double lat_2  = ISEC2_Lambert_LatS2 * 0.001;
-          const bool lsouth = gribbyte_get_bit(ISEC2_Lambert_ProjFlag, 1);
-          if ( lsouth ) { lat_1 = -lat_1; lat_2 = -lat_2; }
-
-          const double lat_0 = lat_2;
-          double x_0 = CDI_grid_missval;
-          double y_0 = CDI_grid_missval;
-
-          if ( proj_lonlat_to_lcc_func )
-            {
-              x_0 = xval_0; y_0 = yval_0;
-              proj_lonlat_to_lcc_func(CDI_grid_missval, lon_0, lat_0, lat_1, lat_2, a, rf, (size_t)1, &x_0, &y_0);
-              if ( IS_NOT_EQUAL(x_0, CDI_grid_missval) && IS_NOT_EQUAL(y_0, CDI_grid_missval) )
-                { x_0 = -x_0; y_0 = -y_0; }
-            }
-          gridDefParamLCC(gridID, CDI_grid_missval, lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0);
+          cgribexDefProjLCC(isec2, gridID);
         }
     }
   else
@@ -554,11 +560,11 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
 
   int varID = 0, levelID = 0;
   varAddRecord(recID, param, gridID, zaxistype, lbounds, level1, level2, 0, 0,
-	       datatype, &varID, &levelID, tsteptype, numavg, leveltype, -1,
-               NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+	       datatype, &varID, &levelID, tsteptype, leveltype, -1,
+               NULL, NULL, NULL, NULL);
 
   record->varID = (short)varID;
-  record->levelID = (short)levelID;
+  record->levelID = levelID;
 
   varDefCompType(varID, comptype);
 
@@ -968,14 +974,9 @@ int cgribexScanTimestep2(stream_t * streamptr)
       if ( ISEC1_AvgNum )
 	{
 	  if ( taxis->numavg && warn_numavg && (taxis->numavg != ISEC1_AvgNum) )
-	    {
-              // Warning("Changing numavg from %d to %d not supported!", taxis->numavg, ISEC1_AvgNum);
-	      warn_numavg = false;
-	    }
+            warn_numavg = false;
 	  else
-	    {
-	      taxis->numavg = ISEC1_AvgNum;
-	    }
+            taxis->numavg = ISEC1_AvgNum;
 	}
 
       compvar_t compVar = cgribexVarSet(param, level1, level2, leveltype, ISEC1_TimeRange);
@@ -1171,16 +1172,9 @@ int cgribexScanTimestep(stream_t * streamptr)
 	  if ( ISEC1_AvgNum )
 	    {
 	      if (  taxis->numavg && warn_numavg && (taxis->numavg != ISEC1_AvgNum) )
-		{
-	      /*
-	          Warning("Changing numavg from %d to %d not supported!", streamptr->tsteps[tsID].taxis.numavg, ISEC1_AvgNum);
-	      */
-		  warn_numavg = false;
-		}
+                warn_numavg = false;
 	      else
-		{
-		  taxis->numavg = ISEC1_AvgNum;
-		}
+                taxis->numavg = ISEC1_AvgNum;
 	    }
 
 	  datetime.date = vdate;
@@ -1520,7 +1514,6 @@ static
 void cgribexDefTime(int *isec1, int vdate, int vtime, int tsteptype, int numavg, int taxisID)
 {
   int timetype = TAXIS_ABSOLUTE;
-  int timerange = 0;
   int timeunit = TUNIT_HOUR;
 
   if ( taxisID != -1 )
@@ -1546,7 +1539,7 @@ void cgribexDefTime(int *isec1, int vdate, int vtime, int tsteptype, int numavg,
 
       int factor = cgribexDefDateTime(isec1, timeunit, rdate, rtime);
       int ip1 = 0, ip2 = 0;
-      timerange = cgribexDefTimerange(tsteptype, factor, calendar, rdate, rtime, vdate, vtime, sdate, stime, &ip1, &ip2);
+      int timerange = cgribexDefTimerange(tsteptype, factor, calendar, rdate, rtime, vdate, vtime, sdate, stime, &ip1, &ip2);
 
       if ( ip2 > 0xFF && timeunit < TUNIT_YEAR )
         {
@@ -1702,9 +1695,16 @@ void cgribexDefGridLambert(int *isec2, int gridID, int uvRelativeToGrid)
   const int xsize = (int)gridInqXsize(gridID);
   const int ysize = (int)gridInqYsize(gridID);
 
+  double missval = CDI_grid_missval;
   double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
-  gridInqParamLCC(gridID, CDI_grid_missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
+  gridInqParamLCC(gridID, missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
+  if (IS_EQUAL(x_0, missval) && IS_EQUAL(y_0, missval) && (IS_EQUAL(xval_0, missval) || IS_EQUAL(yval_0, missval)))
+    {
+      x_0 = gridInqXval(gridID, 0);
+      y_0 = gridInqYval(gridID, 0);
+    }
   gridVerifyGribParamLCC(CDI_grid_missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
+
   bool lsouth = (lat_1 < 0);
   if ( lsouth ) { lat_1 = -lat_2; lat_2 = -lat_2; }
 
@@ -1846,18 +1846,17 @@ static
 void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int levelID)
 {
   char units[CDI_MAX_NAME];
-  static bool lwarning_vct = true;
 
   int zaxistype = zaxisInqType(zaxisID);
-  int ltype = zaxisInqLtype(zaxisID);
+  int ltype = 0;
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
 
   if ( zaxistype == ZAXIS_GENERIC && ltype == 0 )
     {
-      Message("Changed zaxis type from %s to %s",
-	      zaxisNamePtr(zaxistype), zaxisNamePtr(ZAXIS_PRESSURE));
+      Message("Changed zaxis type from %s to %s", zaxisNamePtr(zaxistype), zaxisNamePtr(ZAXIS_PRESSURE));
       zaxistype = ZAXIS_PRESSURE;
       zaxisChangeType(zaxisID, zaxistype);
-      zaxisDefUnits(zaxisID, "Pa");
+      cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, "Pa");
     }
 
   ISEC2_NumVCP = 0;
@@ -1878,6 +1877,7 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
     case ZAXIS_CLOUD_BASE:
     case ZAXIS_CLOUD_TOP:
     case ZAXIS_ISOTHERM_ZERO:
+    case ZAXIS_TROPOPAUSE:
     case ZAXIS_TOA:
     case ZAXIS_SEA_BOTTOM:
     case ZAXIS_ATMOSPHERE:
@@ -1897,6 +1897,7 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
 	int vctsize = zaxisInqVctSize(zaxisID);
 	if ( vctsize > 255 )
 	  {
+            static bool lwarning_vct = true;
 	    ISEC2_NumVCP = 0;
 	    if ( lwarning_vct )
 	      {
@@ -1916,7 +1917,8 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
 	double level = zaxisInqLevel(zaxisID, levelID);
 	if ( level < 0 ) Warning("Pressure level of %f Pa is below zero!", level);
 
-	zaxisInqUnits(zaxisID, units);
+        int length = CDI_MAX_NAME;
+        cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units, &length);
 	if ( units[0] && (units[0] != 'P') | (units[1] != 'a') ) level *= 100;
 
 	double dum;
@@ -1932,7 +1934,8 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
       {
 	double level = zaxisInqLevel(zaxisID, levelID);
 
-	zaxisInqUnits(zaxisID, units);
+        int length = CDI_MAX_NAME;
+        cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units, &length);
         if ( units[1] == 'm' && !units[2] )
           {
             // clang-format off
@@ -1957,7 +1960,8 @@ void cgribexDefLevel(int *isec1, int *isec2, double *fsec2, int zaxisID, int lev
       }
     case ZAXIS_DEPTH_BELOW_LAND:
       {
-	zaxisInqUnits(zaxisID, units);
+        int length = CDI_MAX_NAME;
+        cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units, &length);
 
 	double factor = 100; // default: meter
         // clang-format off
@@ -2011,13 +2015,12 @@ void cgribexDefaultSec4(int *isec4)
 static
 void cgribexDefEnsembleVar(int *isec1, int vlistID, int varID)
 {
-  /* For Ensemble info  */
+  // For Ensemble info
 
-  //Put1Byte(isec1[36]);        /* MPIM local GRIB use definition identifier  */
-                                /*    (extension identifier)                  */
-  //Put1Byte(isec1[37]);        /* type of ensemble forecast                  */
-  //Put2Byte(isec1[38]);        /* individual ensemble member                 */
-  //Put2Byte(isec1[39]);        /* number of forecasts in ensemble            */
+  //Put1Byte(isec1[36]);        // MPIM local GRIB use definition identifier (extension identifier)
+  //Put1Byte(isec1[37]);        // type of ensemble forecast
+  //Put2Byte(isec1[38]);        // individual ensemble member
+  //Put2Byte(isec1[39]);        // number of forecasts in ensemble
 
   int perturbationNumber, numberOfForecastsInEnsemble, typeOfEnsembleForecast;
   const int r1 = cdiInqKeyInt(vlistID, varID, CDI_KEY_PERTURBATIONNUMBER, &perturbationNumber);
@@ -2030,10 +2033,9 @@ void cgribexDefEnsembleVar(int *isec1, int vlistID, int varID)
         {
           ISEC1_LocalFLag = 1;
           isec1[36] = 1;
-
-          isec1[37] =  typeOfEnsembleForecast;
-          isec1[38] =  perturbationNumber;
-          isec1[39] =  numberOfForecastsInEnsemble;
+          isec1[37] = typeOfEnsembleForecast;
+          isec1[38] = perturbationNumber;
+          isec1[39] = numberOfForecastsInEnsemble;
         }
     }
 }
