@@ -191,11 +191,11 @@ void extAddRecord(stream_t *streamptr, int param, int level, size_t xysize,
 
   int varID, levelID = 0;
   varAddRecord(recID, param, gridID, leveltype, 0, level, 0, 0, 0,
-               datatype, &varID, &levelID, TSTEP_INSTANT, 0, 0, -1,
-               NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+               datatype, &varID, &levelID, TSTEP_INSTANT, 0, -1,
+               NULL, NULL, NULL, NULL);
 
   record->varID   = (short)varID;
-  record->levelID = (short)levelID;
+  record->levelID = levelID;
 
   streamptr->tsteps[tsID].nallrecs++;
   streamptr->nrecs++;
@@ -240,11 +240,13 @@ void extScanTimestep1(stream_t *streamptr)
       const int rlevel  = header[2];
       const int rxysize = header[3];
       const int param   = cdiEncodeParam(rcode, 255, 255);
+      DateTime datetime = { .date = vdate, .time = vtime};
 
       if ( nrecs == 0 )
 	{
-	  datetime0.date = vdate;
-	  datetime0.time = vtime;
+	  datetime0 = datetime;
+          taxis->vdate = vdate;
+          taxis->vtime = vtime;
 	}
       else
 	{
@@ -253,8 +255,7 @@ void extScanTimestep1(stream_t *streamptr)
             if ( param == records[recID].param && rlevel == records[recID].ilevel )
               goto tstepScanLoopFinished;
 
-	  DateTime datetime = { .date = vdate, .time = vtime};
-	  if ( datetimeCmp(datetime, datetime0) )
+	  if ( datetimeDiffer(datetime, datetime0) )
 	    Warning("Inconsistent verification time for code %d level %d", rcode, rlevel);
 	}
 
@@ -273,8 +274,6 @@ void extScanTimestep1(stream_t *streamptr)
 
   const int taxisID = taxisCreate(TAXIS_ABSOLUTE);
   taxis->type  = TAXIS_ABSOLUTE;
-  taxis->vdate = (int)datetime0.date;
-  taxis->vtime = (int)datetime0.time;
   taxis->rdate = taxis->vdate;
   taxis->rtime = taxis->vtime;
 
