@@ -1,7 +1,3 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
-#endif
-
 #include "dmemory.h"
 
 #include "cdi.h"
