@@ -13,7 +13,7 @@
 #include "namespace.h"
 
 
-/* the single image implementation */
+// the single image implementation
 int cdiStreamWriteVar_(int streamID, int varID, int memtype, const void *data, size_t nmiss)
 {
   // May fail if memtype == MEMTYPE_FLOAT and the file format does not support single precision writing.
@@ -31,9 +31,9 @@ int cdiStreamWriteVar_(int streamID, int varID, int memtype, const void *data, s
   // check taxis
   if ( streamptr->curTsID == CDI_UNDEFID ) streamDefTimestep(streamID, 0);
 
-  int filetype = streamptr->filetype;
+  const int filetype = streamptr->filetype;
 
-  switch (filetype)
+  switch (cdiBaseFiletype(filetype))
     {
 #ifdef HAVE_LIBGRIB
     case CDI_FILETYPE_GRB:
@@ -68,11 +68,7 @@ int cdiStreamWriteVar_(int streamID, int varID, int memtype, const void *data, s
       }
 #endif
 #ifdef HAVE_LIBNETCDF
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:
+    case CDI_FILETYPE_NETCDF:
       {
         cdf_write_var(streamptr, varID, memtype, data, nmiss);
 	break;
@@ -138,9 +134,8 @@ void streamWriteVarF(int streamID, int varID, const float *data, size_t nmiss)
   if ( myCdiStreamWriteVar_(streamID, varID, MEMTYPE_FLOAT, (const void *) data, nmiss) )
     {
       // In case the file format does not support single precision writing,
-      // we fall back to double precision writing, converting the data
-      // on the fly.
-      int vlistID = streamInqVlist(streamID);
+      // we fall back to double precision writing, converting the data on the fly.
+      const int vlistID = streamInqVlist(streamID);
       size_t elementCount = gridInqSize(vlistInqVarGrid(vlistID, varID));
       elementCount *= (size_t) zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
       double *conversionBuffer = (double *) Malloc(elementCount*sizeof(*conversionBuffer));
@@ -168,9 +163,9 @@ int cdiStreamWriteVarSlice(int streamID, int varID, int levelID, int memtype, co
   // check taxis
   if ( streamptr->curTsID == CDI_UNDEFID ) streamDefTimestep(streamID, 0);
 
-  int filetype = streamptr->filetype;
+  const int filetype = streamptr->filetype;
 
-  switch (filetype)
+  switch (cdiBaseFiletype(filetype))
     {
 #ifdef HAVE_LIBGRIB
     case CDI_FILETYPE_GRB:
@@ -205,11 +200,7 @@ int cdiStreamWriteVarSlice(int streamID, int varID, int levelID, int memtype, co
       }
 #endif
 #ifdef HAVE_LIBNETCDF
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:
+    case CDI_FILETYPE_NETCDF:
       cdf_write_var_slice(streamptr, varID, levelID, memtype, data, nmiss);
       break;
 #endif
@@ -268,7 +259,7 @@ void streamWriteVarSliceF(int streamID, int varID, int levelID, const float *dat
     {
       // In case the file format does not support single precision writing,
       // we fall back to double precision writing, converting the data on the fly.
-      size_t elementCount = gridInqSize(vlistInqVarGrid(streamInqVlist(streamID), varID));
+      const size_t elementCount = gridInqSize(vlistInqVarGrid(streamInqVlist(streamID), varID));
       double *conversionBuffer = (double *) Malloc(elementCount*sizeof(*conversionBuffer));
       for ( size_t i = elementCount; i--; ) conversionBuffer[i] = (double) data[i];
       streamWriteVarSlice(streamID, varID, levelID, conversionBuffer, nmiss);
@@ -287,7 +278,7 @@ void streamWriteVarChunk(int streamID, int varID,
   myCdiStreamWriteVarChunk_(streamID, varID, MEMTYPE_DOUBLE, rect, data, nmiss);
 }
 
-/* single image implementation */
+// single image implementation
 void cdiStreamWriteVarChunk_(int streamID, int varID, int memtype,
                              const int rect[][2], const void *data, size_t nmiss)
 {
@@ -297,9 +288,9 @@ void cdiStreamWriteVarChunk_(int streamID, int varID, int memtype,
 
   // streamDefineTaxis(streamID);
 
-  int filetype = streamptr->filetype;
+  const int filetype = streamptr->filetype;
 
-  switch (filetype)
+  switch (cdiBaseFiletype(filetype))
     {
 #if defined (HAVE_LIBGRIB)
     case CDI_FILETYPE_GRB:
@@ -316,16 +307,11 @@ void cdiStreamWriteVarChunk_(int streamID, int varID, int memtype,
 #endif
 #if  defined (HAVE_LIBGRIB) || defined (HAVE_LIBSERVICE)      \
   || defined (HAVE_LIBEXTRA) || defined (HAVE_LIBIEG)
-      xabort("streamWriteVarChunk not implemented for filetype %s!",
-             strfiletype(filetype));
+      xabort("streamWriteVarChunk not implemented for filetype %s!", strfiletype(filetype));
       break;
 #endif
 #ifdef HAVE_LIBNETCDF
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:
+    case CDI_FILETYPE_NETCDF:
       cdf_write_var_chunk(streamptr, varID, memtype, rect, data, nmiss);
       break;
 #endif
@@ -346,7 +332,7 @@ int stream_write_record(int streamID, int memtype, const void *data, size_t nmis
 
   stream_t *streamptr = stream_to_pointer(streamID);
 
-  switch (streamptr->filetype)
+  switch (cdiBaseFiletype(streamptr->filetype))
     {
 #ifdef HAVE_LIBGRIB
     case CDI_FILETYPE_GRB:
@@ -373,11 +359,7 @@ int stream_write_record(int streamID, int memtype, const void *data, size_t nmis
       break;
 #endif
 #ifdef HAVE_LIBNETCDF
-    case CDI_FILETYPE_NC:
-    case CDI_FILETYPE_NC2:
-    case CDI_FILETYPE_NC4:
-    case CDI_FILETYPE_NC4C:
-    case CDI_FILETYPE_NC5:
+    case CDI_FILETYPE_NETCDF:
       {
 	cdf_write_record(streamptr, memtype, data, nmiss);
 	break;
@@ -421,8 +403,8 @@ void streamWriteRecordF(int streamID, const float *data, size_t nmiss)
       // In case the file format does not support single precision writing,
       // we fall back to double precision writing, converting the data on the fly.
       stream_t *streamptr = stream_to_pointer(streamID);
-      int varID = streamptr->record->varID;
-      size_t elementCount = gridInqSize(vlistInqVarGrid(streamInqVlist(streamID), varID));
+      const int varID = streamptr->record->varID;
+      const size_t elementCount = gridInqSize(vlistInqVarGrid(streamInqVlist(streamID), varID));
       double *conversionBuffer = (double *) Malloc(elementCount*sizeof(*conversionBuffer));
       for ( size_t i = elementCount; i--; ) conversionBuffer[i] = (double) data[i];
       streamWriteRecord(streamID, conversionBuffer, nmiss);
