@@ -1,19 +1,12 @@
-#if defined (HAVE_CONFIG_H)
-#  include "config.h"
-#endif
-
-#include <ctype.h>
 #include <stddef.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "dmemory.h"
 #include "cdi.h"
 #include "cdi_int.h"
 
 
-/*int TableDefine = 0; */ /* Define new table also if the entry already exist */
-                          /* This is needed for createtable */
-
 #include "tablepar.h"
 #include "table.h"
 
@@ -73,7 +66,7 @@ void tableDefEntry(int tableID, int id, int ltype, const char *name,
     }
 }
 
-static void tableLink(int tableID, const param_type *pars, int npars)
+void tableLink(int tableID, const param_type *pars, int npars)
 {
   for ( int item = 0; item < npars; item++ )
     {
@@ -101,11 +94,8 @@ static void parTableInitEntry(int tableID)
 static void tableGetPath(void)
 {
   char *path = getenv("TABLEPATH");
-
   if ( path ) tablePath = strdupx(path);
-  /*
-  printf("tablePath = %s\n", tablePath);
-  */
+  // printf("tablePath = %s\n", tablePath);
 }
 
 static void parTableFinalize(void)
@@ -133,8 +123,7 @@ static void parTableInit(void)
   ParTableInit = 1;
 
   atexit(parTableFinalize);
-  if ( cdiPartabIntern )
-    tableDefault();
+  if ( cdiPartabIntern ) tableDefault();
 
   tableGetPath();
 }
@@ -142,8 +131,8 @@ static void parTableInit(void)
 static int tableNewEntry()
 {
   int tableID = 0;
-  static int init = 0;
 
+  static int init = 0;
   if ( ! init )
     {
       for ( tableID = 0; tableID < parTableSize; tableID++ )
@@ -151,16 +140,13 @@ static int tableNewEntry()
       init = 1;
     }
 
-  /*
-    Look for a free slot in parTable.
-  */
+  // Look for a free slot in parTable.
   for ( tableID = 0; tableID < parTableSize; tableID++ )
     {
       if ( ! parTable[tableID].used ) break;
     }
 
-  if ( tableID == parTableSize )
-    Error("no more entries!");
+  if ( tableID == parTableSize ) Error("no more entries!");
 
   parTable[tableID].used = true;
   parTableNum++;
@@ -243,7 +229,7 @@ decodeForm1(char *pline, char *name, char *longname, char *units)
 static int
 decodeForm2(char *pline, char *name, char *longname, char *units)
 {
-  /* Format 2 : code | name | longname | units */
+  // Format 2 : code | name | longname | units
   char *pend;
 
   pline = strchr(pline, '|');
@@ -326,7 +312,7 @@ int tableRead(const char *tablefile)
   FILE *tablefp = fopen(tablefile, "r");
   if ( tablefp == NULL ) return tableID;
 
-  char *tablename = (char* )strrchr(tablefile, '/');
+  char *tablename = strrchr(tablefile, '/');
   if ( tablename == 0 ) tablename = (char *) tablefile;
   else                  tablename++;
 
@@ -349,9 +335,7 @@ int tableRead(const char *tablefile)
       if ( len < 4 ) continue;
       while ( isspace((int) *pline) ) pline++;
       id = atoi(pline);
-      /*
-      if ( id > 255 ) id -= 256;
-      */
+      // if ( id > 255 ) id -= 256;
       if ( id == 0 ) continue;
 
       while ( isdigit((int) *pline) ) pline++;
@@ -447,7 +431,6 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 
   if ( tablename )
     {
-      size_t len;
       strcpy(tablefile, tablename);
       /*
       printf("tableInq: tablefile = >%s<\n", tablefile);
@@ -458,7 +441,7 @@ int tableInq(int modelID, int tablenum, const char *tablename)
 	  if ( parTable[tableID].used && parTable[tableID].name )
 	    {
 	      /* len = strlen(parTable[tableID].name); */
-	      len = strlen(tablename);
+	      size_t len = strlen(tablename);
 	      if ( memcmp(parTable[tableID].name, tablename, len) == 0 ) break;
 	    }
 	}
@@ -578,11 +561,9 @@ int tableInqModel(int tableID)
 
 static void partabCheckID(int item)
 {
-  if ( item < 0 || item >= parTableSize )
-    Error("item %d undefined!", item);
+  if ( item < 0 || item >= parTableSize ) Error("item %d undefined!", item);
 
-  if ( ! parTable[item].name )
-    Error("item %d name undefined!", item);
+  if ( ! parTable[item].name ) Error("item %d name undefined!", item);
 }
 
 
@@ -590,8 +571,7 @@ const char *tableInqNamePtr(int tableID)
 {
   const char *tablename = NULL;
 
-  if ( CDI_Debug )
-    Message("tableID = %d", tableID);
+  if ( CDI_Debug ) Message("tableID = %d", tableID);
 
   if ( ! ParTableInit ) parTableInit();
 
