@@ -51,24 +51,25 @@ vlist_compare(vlist_t *a, vlist_t *b)
     | (a->nzaxis != b->nzaxis) | (a->instID != b->instID)
     | (a->modelID != b->modelID) | (a->tableID != b->tableID)
     | (a->ntsteps != b->ntsteps) | (a->atts.nelems != b->atts.nelems);
+
   int nvars = a->nvars;
   for (int varID = 0; varID < nvars; ++varID)
     diff |= vlistVarCompare(a, varID, b, varID);
+
   size_t natts = a->atts.nelems;
   for (size_t attID = 0; attID < natts; ++attID)
-    diff |= cdi_att_compare(a, CDI_GLOBAL, b, CDI_GLOBAL, (int)attID);
+    diff |= cdi_att_compare(&a->atts, &a->atts, (int)attID);
+
   return diff;
 }
 
-static void
-vlistPrintKernel(vlist_t *vlistptr, FILE * fp );
-static void
-vlist_delete(vlist_t *vlistptr);
+static void vlistPrintKernel(vlist_t *vlistptr, FILE *fp);
+static void vlist_delete(vlist_t *vlistptr);
 
-static int  vlistGetSizeP ( void * vlistptr, void *context);
-static void vlistPackP    ( void * vlistptr, void * buff, int size,
-                            int *position, void *context);
-static int  vlistTxCode   ( void );
+static int  vlistGetSizeP (void *vlistptr, void *context);
+static void vlistPackP    (void *vlistptr, void *buff, int size,
+                           int *position, void *context);
+static int  vlistTxCode   (void);
 
 #if !defined(__cplusplus)
 const
@@ -107,6 +108,8 @@ void vlist_init_entry(vlist_t *vlistptr)
   vlistptr->ntsteps        = CDI_UNDEFID;
   vlistptr->keys.nalloc    = MAX_KEYS;
   vlistptr->keys.nelems    = 0;
+  for ( int i = 0; i < MAX_KEYS; ++i )
+    vlistptr->keys.value[i].length = 0;
   vlistptr->atts.nalloc    = MAX_ATTRIBUTES;
   vlistptr->atts.nelems    = 0;
   vlistptr->nsubtypes      = 0;
@@ -226,10 +229,6 @@ void vlist_delete(vlist_t *vlistptr)
   for ( int varID = 0; varID < nvars; varID++ )
     {
       if ( vars[varID].levinfo )  Free(vars[varID].levinfo);
-      if ( vars[varID].name )     Free(vars[varID].name);
-      if ( vars[varID].longname ) Free(vars[varID].longname);
-      if ( vars[varID].stdname )  Free(vars[varID].stdname);
-      if ( vars[varID].units )    Free(vars[varID].units);
 
       if ( vlistptr->vars[varID].opt_grib_kvpair )
         {
@@ -290,11 +289,6 @@ void cdiVlistDestroy_(int vlistID)
 static
 void var_copy_entries(var_t *var2, var_t *var1)
 {
-  if ( var1->name )     var2->name     = strdupx(var1->name);
-  if ( var1->longname ) var2->longname = strdupx(var1->longname);
-  if ( var1->stdname )  var2->stdname  = strdupx(var1->stdname);
-  if ( var1->units )    var2->units    = strdupx(var1->units);
-
   var2->opt_grib_kvpair_size = 0;
   var2->opt_grib_kvpair      = NULL;
   var2->opt_grib_nentries    = 0;
@@ -593,7 +587,6 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
                 double *levels = NULL;
 		double *lbounds = NULL, *ubounds = NULL;
 		const double *vct = NULL;
-                char ctemp[CDI_MAX_NAME];
 
                 if ( !vars1[varID].levinfo ) cdiVlistCreateVarLevInfo(vlistptr1, varID);
 
@@ -601,7 +594,7 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
                 int zaxisType = zaxisInqType(zaxisID);
 
                 int levID2 = 0;
-                for ( int levID = 0; levID < nlevs; ++levID )
+                for ( int levID = 0; levID < nlevs; levID++ )
                   if ( vars1[varID].levinfo[levID].flag )
                     {
                       vars1[varID].levinfo[levID].flevelID = levID2;
@@ -687,23 +680,32 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
                     Free(cvals2);
                   }
 
-                zaxisInqName(zaxisID, ctemp);
-                zaxisDefName(zaxisID2, ctemp);
-                zaxisInqLongname(zaxisID, ctemp);
-                zaxisDefLongname(zaxisID2, ctemp);
-                zaxisInqUnits(zaxisID, ctemp);
-                zaxisDefUnits(zaxisID2, ctemp);
+                char ctemp[CDI_MAX_NAME];
+                int length = CDI_MAX_NAME;
+                cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, ctemp, &length);
+                cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_NAME, ctemp);
+                length = CDI_MAX_NAME;
+                cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, ctemp, &length);
+                cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_LONGNAME, ctemp);
+                length = CDI_MAX_NAME;
+                cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, ctemp, &length);
+                cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_UNITS, ctemp);
+
                 zaxisDefDatatype(zaxisID2, zaxisInqDatatype(zaxisID));
+                zaxisDefPositive(zaxisID2, zaxisInqPositive(zaxisID));
 
                 if ( zaxisType == ZAXIS_CHAR )
                   {
-                    char dimname[CDI_MAX_NAME+3]; dimname[0] = 0;
-                    cdiZaxisInqKeyStr(zaxisID, CDI_KEY_DIMNAME, CDI_MAX_NAME, dimname);
+                    char dimname[CDI_MAX_NAME+3];
+                    length = sizeof(dimname);
+                    cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_DIMNAME, dimname, &length);
                     if ( dimname[0] == 0 ) { memcpy(dimname, "area_type", 10); dimname[10] = 0; }
-                    cdiZaxisDefKeyStr(zaxisID2, CDI_KEY_DIMNAME, CDI_MAX_NAME, dimname);
+                    cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_DIMNAME, dimname);
                   }
 
-                if ( zaxisType == ZAXIS_GENERIC ) zaxisDefLtype(zaxisID2, zaxisInqLtype(zaxisID));
+                if ( zaxisType == ZAXIS_GENERIC ) cdiCopyKey(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, zaxisID2);
+
+                cdiCopyAtts(zaxisID, CDI_GLOBAL, zaxisID2, CDI_GLOBAL);
 
 		zaxisID = zaxisID2;
 		vars2[varID2].zaxisID = zaxisID2;
@@ -720,7 +722,7 @@ void vlistCopyFlag(int vlistID2, int vlistID1)
 	      if ( vars1[varID].levinfo[levID].flag )
 		{
 		  vars2[varID2].levinfo[levID2].flevelID = levID;
-		  vars2[varID2].levinfo[levID2].mlevelID = levID;
+		  vars2[varID2].levinfo[levID2].mlevelID = levID2;
 		  levID2++;
 		}
 
@@ -830,15 +832,21 @@ void vlistMerge(int vlistID2, int vlistID1)
 
   if ( nvars1 == nvars2 )
     {
+      char name1[CDI_MAX_NAME], name2[CDI_MAX_NAME];
       for ( varID = 0; varID < nvars2; varID++ )
 	{
           size_t ngp1 = gridInqSize(vars1[varID].gridID);
           size_t ngp2 = gridInqSize(vars2[varID].gridID);
           if ( ngp1 != ngp2 ) break;
 
-	  if ( vars1[varID].name && vars2[varID].name )
+          int length = CDI_MAX_NAME;
+          (void)cdiInqKeyString(vlistID1, varID, CDI_KEY_NAME, name1, &length);
+          length = CDI_MAX_NAME;
+          (void)cdiInqKeyString(vlistID2, varID, CDI_KEY_NAME, name2, &length);
+
+	  if ( *name1 && *name2 )
 	    {
-	      if ( strcmp(vars1[varID].name, vars2[varID].name) != 0 ) break;
+	      if ( strcmp(name1, name2) != 0 ) break;
 	    }
 	  else
 	    {
@@ -1078,11 +1086,10 @@ int vlistNtsteps(int vlistID)
 static
 void vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
 {
-  char paramstr[32];
+  const int vlistID = vlistptr->self;
+  fprintf ( fp, "#\n# vlistID %d\n#\n", vlistID);
 
-  fprintf ( fp, "#\n# vlistID %d\n#\n", vlistptr->self);
-
-  int nvars = vlistptr->nvars;
+  const int nvars = vlistptr->nvars;
 
   fprintf(fp, "nvars    : %d\n"
           "ngrids   : %d\n"
@@ -1097,8 +1104,7 @@ void vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
 
   if ( nvars > 0 )
     {
-      fprintf(fp, " varID param    gridID zaxisID stypeID tsteptype flag iorank"
-              " name     longname         units\n");
+      fprintf(fp, " varID param    gridID zaxisID stypeID tsteptype flag iorank name     longname         units\n");
       for ( int varID = 0; varID < nvars; varID++ )
         {
           int param = vlistptr->vars[varID].param;
@@ -1106,16 +1112,20 @@ void vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
           int zaxisID = vlistptr->vars[varID].zaxisID;
           int subtypeID = vlistptr->vars[varID].subtypeID;
 	  int tsteptype = vlistptr->vars[varID].tsteptype;
-          const char *name = vlistptr->vars[varID].name;
-          const char *longname = vlistptr->vars[varID].longname;
-          const char *units = vlistptr->vars[varID].units;
+          char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
+          int length = CDI_MAX_NAME;
+          (void)cdiInqKeyString(vlistID, varID, CDI_KEY_NAME, name, &length);
+          length = CDI_MAX_NAME;
+          (void)cdiInqKeyString(vlistID, varID, CDI_KEY_LONGNAME, longname, &length);
+          length = CDI_MAX_NAME;
+          (void)cdiInqKeyString(vlistID, varID, CDI_KEY_UNITS, units, &length);
           int flag = vlistptr->vars[varID].flag;
           int iorank = vlistptr->vars[varID].iorank;
 
+          char paramstr[32];
           cdiParamToString(param, paramstr, sizeof(paramstr));
           fprintf(fp, "%6d %-8s %6d  %6d  %6d  %6d  %5d %6d %-8s %s [%s]\n",
-                  varID, paramstr, gridID, zaxisID, subtypeID, tsteptype, flag, iorank,
-                  name?name:"", longname?longname:"", units?units:"");
+                  varID, paramstr, gridID, zaxisID, subtypeID, tsteptype, flag, iorank, name, longname, units);
         }
 
       fputs("\n"
@@ -1142,8 +1152,7 @@ void vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
               double level = zaxisInqLevels(zaxisID, NULL) ? zaxisInqLevel(zaxisID, levID) : levID+1;
 
               fprintf(fp, "%6d %6d %6d %6d %6d %6d %6d %6d %5d  %.9g\n",
-                      varID, levID, fvarID, flevID, mvarID, mlevID, index,
-                      dtype, flag, level);
+                      varID, levID, fvarID, flevID, mvarID, mlevID, index, dtype, flag, level);
             }
         }
 
@@ -1151,8 +1160,7 @@ void vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
             " varID  size iorank\n", fp);
       for ( int varID = 0; varID < nvars; varID++ )
         fprintf(fp, "%3d %8zu %6d\n", varID,
-                zaxisInqSize(vlistptr->vars[varID].zaxisID)
-                * gridInqSize(vlistptr->vars[varID].gridID),
+                zaxisInqSize(vlistptr->vars[varID].zaxisID) * gridInqSize(vlistptr->vars[varID].gridID),
                 vlistptr->vars[varID].iorank);
     }
 }
@@ -1210,7 +1218,6 @@ The function @func{vlistInqTaxis} returns the time axis of a variable list.
 int vlistInqTaxis(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
-
   return vlistptr->taxisID;
 }
 
@@ -1230,7 +1237,6 @@ void vlistDefTable(int vlistID, int tableID)
 int vlistInqTable(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
-
   return vlistptr->tableID;
 }
 
@@ -1535,12 +1541,11 @@ static int vlistTxCode ( void )
 static
 int  vlistGetSizeP ( void * vlistptr, void *context)
 {
-  int txsize, varID;
   vlist_t *p = (vlist_t*) vlistptr;
-  txsize = serializeGetSize(vlist_nints, CDI_DATATYPE_INT, context);
+  int txsize = serializeGetSize(vlist_nints, CDI_DATATYPE_INT, context);
   txsize += serializeGetSize(1, CDI_DATATYPE_LONG, context);
   txsize += cdiAttsGetSize(p, CDI_GLOBAL, context);
-  for ( varID = 0; varID <  p->nvars; varID++ )
+  for ( int varID = 0; varID <  p->nvars; varID++ )
     txsize += vlistVarGetPackSize(p, varID, context);
   return txsize;
 }
@@ -1550,7 +1555,7 @@ static
 void vlistPackP ( void * vlistptr, void * buf, int size, int *position,
                   void *context )
 {
-  int varID, tempbuf[vlist_nints];
+  int tempbuf[vlist_nints];
   vlist_t *p = (vlist_t*) vlistptr;
   tempbuf[0] = p->self;
   tempbuf[1] = p->nvars;
@@ -1562,7 +1567,7 @@ void vlistPackP ( void * vlistptr, void * buf, int size, int *position,
   serializePack(&p->ntsteps, 1, CDI_DATATYPE_LONG, buf, size, position, context);
 
   cdiAttsPack(p, CDI_GLOBAL, buf, size, position, context);
-  for ( varID = 0; varID < p->nvars; varID++ )
+  for ( int varID = 0; varID < p->nvars; varID++ )
     {
       vlistVarPack(p, varID, (char *)buf, size, position, context);
     }
@@ -1588,19 +1593,16 @@ void vlistUnpack(char * buf, int size, int *position, int originNamespace,
   cdiAttsUnpack(targetID, CDI_GLOBAL, buf, size, position, context);
   for (int varID = 0; varID < nvars; varID++ )
     vlistVarUnpack(targetID, buf, size, position, originNamespace, context);
-  reshSetStatus(targetID, &vlistOps,
-                reshGetStatus(targetID, &vlistOps) & ~RESH_SYNC_BIT);
+  reshSetStatus(targetID, &vlistOps, reshGetStatus(targetID, &vlistOps) & ~RESH_SYNC_BIT);
 }
 
 
 void vlist_check_contents(int vlistID)
 {
-  int zaxisID;
   int nzaxis = vlistNzaxis(vlistID);
-
   for ( int index = 0; index < nzaxis; index++ )
     {
-      zaxisID = vlistZaxis(vlistID, index);
+      int zaxisID = vlistZaxis(vlistID, index);
       if ( zaxisInqType(zaxisID) == ZAXIS_GENERIC )
 	cdiCheckZaxis(zaxisID);
     }
