@@ -66,10 +66,6 @@ typedef struct
   int         xyz;
   bool        missvalused; // true if missval is defined
   bool        lvalidrange;
-  char       *name;
-  char       *longname;
-  char       *stdname;
-  char       *units;
   char       *extra;
   double      missval;
   double      scalefactor;
@@ -118,16 +114,10 @@ typedef struct
 vlist_t;
 
 
-int vlist_key_compare(vlist_t *a, int varIDA, vlist_t *b, int varIDB, int keynum);
-
 vlist_t *vlist_to_pointer(int vlistID);
 void cdiVlistMakeInternal(int vlistID);
 void cdiVlistMakeImmutable(int vlistID);
 void vlistCheckVarID(const char *caller, int vlistID, int varID);
-void     vlistDestroyVarName(int vlistID, int varID);
-void     vlistDestroyVarLongname(int vlistID, int varID);
-void     vlistDestroyVarStdname(int vlistID, int varID);
-void     vlistDestroyVarUnits(int vlistID, int varID);
 void     cdiVlistDestroy_(int vlistID);
 int      vlistInqVarMissvalUsed(int vlistID, int varID);
 int      vlistHasTime(int vlistID);
@@ -143,8 +133,6 @@ int     vlistInqVarValidrange(int vlistID, int varID, double *validrange);
 
 void vlistInqVarDimorder(int vlistID, int varID, int (*outDimorder)[3]);
 
-int cdi_att_compare(vlist_t *a, int varIDA, vlist_t *b, int varIDB, int attnum);
-
 void resize_opt_grib_entries(var_t *var, int nentries);
 
 
