@@ -53,6 +53,7 @@ ZaxistypeEntry[] = {
   { /* 24 */ 0, "mixlayer",          "mix_layer",              "",               ""},
   { /* 25 */ 0, "height",            "generalized_height",     "height",         ""},
   { /* 26 */ 0, "character",         "area_type",              "",               ""},
+  { /* 27 */ 0, "tropopause",        "tropopause",             "",               ""},
 };
 
 enum {
@@ -113,15 +114,6 @@ static
 void zaxis_init(zaxis_t *zaxisptr)
 {
   zaxisptr->self          = CDI_UNDEFID;
-  zaxisptr->name[0]       = 0;
-  zaxisptr->longname[0]   = 0;
-  zaxisptr->stdname[0]    = 0;
-  zaxisptr->dimname[0]    = 0;
-  zaxisptr->vdimname[0]   = 0;
-  zaxisptr->units[0]      = 0;
-  zaxisptr->psname[0]     = 0;
-  zaxisptr->p0name[0]     = 0;
-  zaxisptr->p0value.defined = false;
   zaxisptr->vals          = NULL;
 #ifndef USE_MPI
   zaxisptr->cvals         = NULL;
@@ -131,8 +123,6 @@ void zaxis_init(zaxis_t *zaxisptr)
   zaxisptr->lbounds       = NULL;
   zaxisptr->weights       = NULL;
   zaxisptr->type          = CDI_UNDEFID;
-  zaxisptr->ltype         = 0;
-  zaxisptr->ltype2        = -1;
   zaxisptr->positive      = 0;
   zaxisptr->scalar        = 0;
   zaxisptr->direction     = 0;
@@ -140,11 +130,10 @@ void zaxis_init(zaxis_t *zaxisptr)
   zaxisptr->size          = 0;
   zaxisptr->vctsize       = 0;
   zaxisptr->vct           = NULL;
-  zaxisptr->number        = 0;
-  zaxisptr->nhlev         = 0;
-  memset(zaxisptr->uuid, 0, CDI_UUID_SIZE);
-  zaxisptr->atts.nalloc   = MAX_ATTRIBUTES;
-  zaxisptr->atts.nelems   = 0;
+
+  cdiInitKeys(&zaxisptr->keys);
+  zaxisptr->atts.nalloc    = MAX_ATTRIBUTES;
+  zaxisptr->atts.nelems    = 0;
 }
 
 static
@@ -181,6 +170,8 @@ void zaxis_copy(zaxis_t *zaxisptr2, zaxis_t *zaxisptr1)
   const int zaxisID2 = zaxisptr2->self;
   memcpy(zaxisptr2, zaxisptr1, sizeof(zaxis_t));
   zaxisptr2->self = zaxisID2;
+  cdiInitKeys(&zaxisptr2->keys);
+  cdiCopyVarKeys(&zaxisptr1->keys, &zaxisptr2->keys);
 }
 
 
@@ -199,15 +190,16 @@ int zaxisCreate_(int zaxistype, int size, int id)
   zaxisptr->size = size;
 
   if ( zaxistype >= CDI_NumZaxistype || zaxistype < 0 )
-    Error("Internal problem! zaxistype > CDI_MaxZaxistype");
+    Error("Internal problem! zaxistype=%d out of range (min=0/max=%d)!", zaxistype, CDI_NumZaxistype-1);
 
   const int zaxisID = zaxisptr->self;
-  zaxisDefName(zaxisID, ZaxistypeEntry[zaxistype].name);
+  cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, ZaxistypeEntry[zaxistype].name);
   if ( zaxistype != ZAXIS_GENERIC ) zaxisDefLongname(zaxisID, ZaxistypeEntry[zaxistype].longname);
-  zaxisDefUnits(zaxisID, ZaxistypeEntry[zaxistype].units);
+  cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, ZaxistypeEntry[zaxistype].units);
 
-  if ( *ZaxistypeEntry[zaxistype].stdname )
-    strcpy(zaxisptr->stdname, ZaxistypeEntry[zaxistype].stdname);
+  const char *stdname = ZaxistypeEntry[zaxistype].stdname;
+  if ( *stdname )
+    cdiDefVarKeyBytes(&zaxisptr->keys, CDI_KEY_STDNAME, (const unsigned char*)stdname, (int)strlen(stdname)+1);
 
   zaxisptr->positive = ZaxistypeEntry[zaxistype].positive;
 
@@ -285,9 +277,12 @@ void zaxisDestroyKernel( zaxis_t * zaxisptr )
   if ( zaxisptr->weights ) Free( zaxisptr->weights );
   if ( zaxisptr->vct )     Free( zaxisptr->vct );
 
-  Free( zaxisptr );
+  cdiDeleteKeys(id, CDI_GLOBAL);
+  cdiDeleteAtts(id, CDI_GLOBAL);
 
-  reshRemove ( id, &zaxisOps );
+  Free(zaxisptr);
+
+  reshRemove(id, &zaxisOps);
 }
 
 /*
@@ -328,194 +323,6 @@ void zaxisName(int zaxistype, char *zaxisname)
   strcpy(zaxisname, zaxisNamePtr(zaxistype));
 }
 
-static inline
-void zaxisSetString(char *zaxisstrname, const char *name, size_t len)
-{
-  if ( len > CDI_MAX_NAME ) len = CDI_MAX_NAME;
-  strncpy(zaxisstrname, name, len);
-  zaxisstrname[len-1] = 0;
-}
-
-static inline
-void zaxisGetString(char *name, const char *zaxisstrname, size_t len)
-{
-  size_t slen = strlen(zaxisstrname)+1;
-  if ( slen > len ) slen = len;
-  if ( slen > CDI_MAX_NAME ) slen = CDI_MAX_NAME;
-  strncpy(name, zaxisstrname, slen);
-  name[slen-1] = 0;
-}
-
-static
-void *zaxis_key_to_ptr(zaxis_t *zaxisptr, int key)
-{
-  void *keyptr = NULL;
-
-  switch (key)
-    {
-    case CDI_KEY_NAME:      keyptr = (void*)zaxisptr->name; break;
-    case CDI_KEY_LONGNAME:  keyptr = (void*)zaxisptr->longname; break;
-    case CDI_KEY_UNITS:     keyptr = (void*)zaxisptr->units; break;
-    case CDI_KEY_DIMNAME:   keyptr = (void*)zaxisptr->dimname; break;
-    case CDI_KEY_VDIMNAME:  keyptr = (void*)zaxisptr->vdimname; break;
-    case CDI_KEY_PSNAME:    keyptr = (void*)zaxisptr->psname; break;
-    case CDI_KEY_P0NAME:    keyptr = (void*)zaxisptr->p0name; break;
-    case CDI_KEY_P0VALUE:   keyptr = (void*)&zaxisptr->p0value; break;
-    }
-
-  return keyptr;
-}
-
-/*
-@Function  cdiZaxisDefKeyStr
-@Title     Define a CDI Z-axis string value from a key
-
-@Prototype int cdiZaxisDefKeyStr(int zaxisID, int key, int size, const char *mesg)
-@Parameter
-    @Item  zaxisID  Z-axis ID, from a previous call to @fref{zaxisCreate}.
-    @Item  key      The key to be searched
-    @Item  size     The allocated length of the string on input
-    @Item  mesg     The address of a string where the data will be read
-
-@Description
-The function @func{cdiZaxisDefKeyStr} defines a CDI Z-axis string value from a key.
-
-@Result
-@func{cdiZaxisDefKeyStr} returns 0 if OK and integer value on error.
-
-@EndFunction
-*/
-int cdiZaxisDefKeyStr(int zaxisID, int key, int size, const char *mesg)
-{
-  if ( size < 1 || mesg == NULL ) return -1;
-
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
-  char *keyptr = (char*)zaxis_key_to_ptr(zaxisptr, key);
-  if ( keyptr == NULL)
-    {
-      Warning("CDI zaxis string key %d not supported!", key);
-      return -1;
-    }
-
-  zaxisSetString(keyptr, mesg, (size_t)size);
-  reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
-
-  return 0;
-}
-
-/*
-@Function  cdiZaxisInqKeyStr
-@Title     Get a CDI Z-axis string value from a key
-
-@Prototype int cdiZaxisInqKeyStr(int zaxisID, int key, int size, char *mesg)
-@Parameter
-    @Item  zaxisID  Z-axis ID, from a previous call to @fref{zaxisCreate}.
-    @Item  key      The key to be searched.
-    @Item  size     The allocated length of the string on input.
-    @Item  mesg     The address of a string where the data will be retrieved.
-                    The caller must allocate space for the returned string.
-                    The maximum possible length, in characters, of the string
-                    is given by the predefined constant @func{CDI_MAX_NAME}.
-
-@Description
-The function @func{cdiZaxisInqKeyStr} return a CDI Z-axis string value from a key.
-
-@Result
-@func{cdiZaxisInqKeyStr} returns 0 if OK and integer value on error.
-
-@EndFunction
-*/
-int cdiZaxisInqKeyStr(int zaxisID, int key, int size, char *mesg)
-{
-  if ( size < 1 || mesg == NULL ) return -1;
-
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  const char *keyptr = (const char*)zaxis_key_to_ptr(zaxisptr, key);
-  if ( keyptr == NULL)
-    {
-      Warning("CDI zaxis string key %d not supported!", key);
-      return -1;
-    }
-
-  zaxisGetString(mesg, keyptr, (size_t)size);
-
-  return 0;
-}
-
-
-/*
-@Function  cdiZaxisDefKeyFlt
-@Title     Define a CDI Z-axis floating point value from a key
-
-@Prototype int cdiZaxisDefKeyFlt(int zaxisID, int key, double value)
-@Parameter
-    @Item  zaxisID  Z-axis ID, from a previous call to @fref{zaxisCreate}.
-    @Item  key      The key to be searched
-    @Item  value    A double where the data will be read
-
-@Description
-The function @func{cdiZaxisDefKeyFlt} defines a CDI Z-axis double value from a key.
-
-@Result
-@func{cdiZaxisDefKeyFlt} returns 0 if OK and integer value on error.
-
-@EndFunction
-*/
-int cdiZaxisDefKeyFlt(int zaxisID, int key, double value)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
-  zkey_double_t *keyptr = (zkey_double_t*)zaxis_key_to_ptr(zaxisptr, key);
-  if ( keyptr == NULL)
-    {
-      Warning("CDI zaxis double key %d not supported!", key);
-      return -1;
-    }
-
-  keyptr->value = value;
-  keyptr->defined = true;
-
-  reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
-
-  return 0;
-}
-
-/*
-@Function  cdiZaxisInqKeyFlt
-@Title     Get a CDI Z-axis floating point value from a key
-
-@Prototype int cdiZaxisInqKeyFlt(int zaxisID, int key, double *value)
-@Parameter
-    @Item  zaxisID  Z-axis ID, from a previous call to @fref{zaxisCreate}.
-    @Item  key      The key to be searched.
-    @Item value     The address of a double where the data will be retrieved.
-
-@Description
-The function @func{cdiZaxisInqKeyFlt} return a CDI Z-axis double value from a key.
-
-@Result
-@func{cdiZaxisInqKeyFlt} returns 0 if OK and integer value on error.
-
-@EndFunction
-*/
-int cdiZaxisInqKeyFlt(int zaxisID, int key, double *value)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  zkey_double_t *keyptr = (zkey_double_t*)zaxis_key_to_ptr(zaxisptr, key);
-  if ( keyptr == NULL)
-    {
-      Warning("CDI zaxis double key %d not supported!", key);
-      return -1;
-    }
-
-  if ( !keyptr->defined ) return 1;
-
-  *value = keyptr->value;
-
-  return 0;
-}
-
 /*
 @Function  zaxisDefName
 @Title     Define the name of a Z-axis
@@ -532,7 +339,7 @@ The function @func{zaxisDefName} defines the name of a Z-axis.
 */
 void zaxisDefName(int zaxisID, const char *name)
 {
-  (void)cdiZaxisDefKeyStr(zaxisID, CDI_KEY_NAME, CDI_MAX_NAME, name);
+  (void)cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, name);
 }
 
 /*
@@ -551,7 +358,7 @@ The function @func{zaxisDefLongname} defines the longname of a Z-axis.
 */
 void zaxisDefLongname(int zaxisID, const char *longname)
 {
-  (void)cdiZaxisDefKeyStr(zaxisID, CDI_KEY_LONGNAME, CDI_MAX_NAME, longname);
+  (void)cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, longname);
 }
 
 /*
@@ -570,7 +377,7 @@ The function @func{zaxisDefUnits} defines the units of a Z-axis.
 */
 void zaxisDefUnits(int zaxisID, const char *units)
 {
-  (void)cdiZaxisDefKeyStr(zaxisID, CDI_KEY_UNITS, CDI_MAX_NAME, units);
+  (void)cdiDefKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units);
 }
 
 /*
@@ -594,13 +401,14 @@ The function @func{zaxisInqName} returns the name of a Z-axis.
 */
 void zaxisInqName(int zaxisID, char *name)
 {
-  (void)cdiZaxisInqKeyStr(zaxisID, CDI_KEY_NAME, CDI_MAX_NAME, name);
+  int length = CDI_MAX_NAME;
+  (void)cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, name, &length);
 }
 
 const char *zaxisInqNamePtr(int zaxisID)
 {
   zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  return zaxisptr->name;
+  return cdiInqVarKeyString(&zaxisptr->keys, CDI_KEY_NAME);
 }
 
 /*
@@ -624,7 +432,8 @@ The function @func{zaxisInqLongname} returns the longname of a Z-axis.
 */
 void zaxisInqLongname(int zaxisID, char *longname)
 {
-  (void)cdiZaxisInqKeyStr(zaxisID, CDI_KEY_LONGNAME, CDI_MAX_NAME, longname);
+  int length = CDI_MAX_NAME;
+  (void)cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, longname, &length);
 }
 
 /*
@@ -648,14 +457,15 @@ The function @func{zaxisInqUnits} returns the units of a Z-axis.
 */
 void zaxisInqUnits(int zaxisID, char *units)
 {
-  (void)cdiZaxisInqKeyStr(zaxisID, CDI_KEY_UNITS, CDI_MAX_NAME, units);
+  int length = CDI_MAX_NAME;
+  (void)cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units, &length);
 }
 
 
 void zaxisInqStdname(int zaxisID, char *stdname)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  strcpy(stdname, zaxisptr->stdname);
+  int length = CDI_MAX_NAME;
+  (void)cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_STDNAME, stdname, &length);
 }
 
 
@@ -711,44 +521,6 @@ int zaxisInqScalar(int zaxisID)
   return zaxisptr->scalar;
 }
 
-
-void zaxisDefLtype(int zaxisID, int ltype)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
-  if (zaxisptr->ltype != ltype)
-    {
-      zaxisptr->ltype = ltype;
-      reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
-    }
-}
-
-
-int zaxisInqLtype(int zaxisID)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  return zaxisptr->ltype;
-}
-
-
-void zaxisDefLtype2(int zaxisID, int ltype2)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
-  if ( zaxisptr->ltype2 != ltype2 )
-    {
-      zaxisptr->ltype2 = ltype2;
-      reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
-    }
-}
-
-
-int zaxisInqLtype2(int zaxisID)
-{
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  return zaxisptr->ltype2;
-}
-
 /*
 @Function  zaxisDefLevels
 @Title     Define the levels of a Z-axis
@@ -840,21 +612,17 @@ void zaxisDefLevel(int zaxisID, int levelID, double level)
 }
 
 
-void zaxisDefNlevRef(int zaxisID, int nhlev)
+void zaxisDefNlevRef(int zaxisID, int nlev)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  if (zaxisptr->nhlev != nhlev)
-    {
-      zaxisptr->nhlev = nhlev;
-      reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
-    }
+  cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NLEV, nlev);
 }
 
 
 int zaxisInqNlevRef(int zaxisID)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  return zaxisptr->nhlev;
+  int nlev = 0;
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NLEV, &nlev);
+  return nlev;
 }
 
 /*
@@ -873,12 +641,7 @@ The function @func{zaxisDefNumber} defines the reference number for a generalize
 */
 void zaxisDefNumber(int zaxisID, int number)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  if (zaxisptr->number != number)
-    {
-      zaxisptr->number = number;
-      reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
-    }
+  cdiDefKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NUMBEROFVGRIDUSED, number);
 }
 
 /*
@@ -898,8 +661,9 @@ The function @func{zaxisInqNumber} returns the reference number to a generalized
 */
 int zaxisInqNumber(int zaxisID)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  return zaxisptr->number;
+  int referenceNumber = 0;
+  cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_NUMBEROFVGRIDUSED, &referenceNumber);
+  return referenceNumber;
 }
 
 /*
@@ -918,8 +682,8 @@ The function @func{zaxisDefUUID} defines the UUID for a generalized  Z-axis.
 */
 void zaxisDefUUID(int zaxisID, const unsigned char uuid[CDI_UUID_SIZE])
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  memcpy(zaxisptr->uuid, uuid, CDI_UUID_SIZE);
+  cdiDefKeyBytes(zaxisID, CDI_GLOBAL, CDI_KEY_UUID, uuid, CDI_UUID_SIZE);
+
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
 }
 
@@ -941,8 +705,9 @@ The function @func{zaxisInqUUID} returns the UUID to a generalized Z-axis.
 */
 void zaxisInqUUID(int zaxisID, unsigned char uuid[CDI_UUID_SIZE])
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-  memcpy(uuid, zaxisptr->uuid, CDI_UUID_SIZE);
+  memset(uuid, 0, CDI_UUID_SIZE);
+  int length = CDI_UUID_SIZE;
+  cdiInqKeyBytes(zaxisID, CDI_GLOBAL, CDI_KEY_UUID, uuid, &length);
 }
 
 /*
@@ -963,9 +728,8 @@ The function @func{zaxisInqLevel} returns one level of a Z-axis.
 */
 double zaxisInqLevel(int zaxisID, int levelID)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   double level = 0;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->vals && levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->vals[levelID];
 
@@ -975,9 +739,8 @@ double zaxisInqLevel(int zaxisID, int levelID)
 
 double zaxisInqLbound(int zaxisID, int levelID)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   double level = 0;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->lbounds && levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->lbounds[levelID];
 
@@ -987,9 +750,8 @@ double zaxisInqLbound(int zaxisID, int levelID)
 
 double zaxisInqUbound(int zaxisID, int levelID)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   double level = 0;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->ubounds && levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->ubounds[levelID];
 
@@ -1046,12 +808,12 @@ int zaxisInqLevels(int zaxisID, double *levels)
   return size;
 }
 
+
 int zaxisInqCLen(int zaxisID)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   int clen = 0;
 #ifndef USE_MPI
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->cvals && zaxisptr->clength)
     clen = zaxisptr->clength;
 #endif
@@ -1059,12 +821,12 @@ int zaxisInqCLen(int zaxisID)
   return clen;
 }
 
+
 int zaxisInqCVals(int zaxisID, char ***clevels)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   int size = 0;
 #ifndef USE_MPI
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->cvals )
     {
       size = zaxisptr->size;
@@ -1087,9 +849,8 @@ int zaxisInqCVals(int zaxisID, char ***clevels)
 
 int zaxisInqLbounds(int zaxisID, double *lbounds)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   int size = 0;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->lbounds )
     {
       size = zaxisptr->size;
@@ -1105,9 +866,8 @@ int zaxisInqLbounds(int zaxisID, double *lbounds)
 
 int zaxisInqUbounds(int zaxisID, double *ubounds)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   int size = 0;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->ubounds )
     {
       size = zaxisptr->size;
@@ -1123,9 +883,8 @@ int zaxisInqUbounds(int zaxisID, double *ubounds)
 
 int zaxisInqWeights(int zaxisID, double *weights)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   int size = 0;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->weights )
     {
       size = zaxisptr->size;
@@ -1141,9 +900,8 @@ int zaxisInqWeights(int zaxisID, double *weights)
 
 int zaxisInqLevelID(int zaxisID, double level)
 {
-  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
-
   int levelID = CDI_UNDEFID;
+  zaxis_t *zaxisptr = zaxis_to_pointer(zaxisID);
   if ( zaxisptr->vals )
     {
       const int size = zaxisptr->size;
@@ -1258,7 +1016,7 @@ void zaxisDefVct(int zaxisID, int size, const double *vct)
       zaxisptr->vct = (double *) Realloc(zaxisptr->vct, (size_t)size*sizeof(double));
     }
 
-  memcpy(zaxisptr->vct, vct, (size_t)size*sizeof(double));
+  if (vct) memcpy(zaxisptr->vct, vct, (size_t)size*sizeof(double));
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
 }
 
@@ -1291,13 +1049,13 @@ void zaxisDefLbounds(int zaxisID, const double *lbounds)
   const size_t size = (size_t)zaxisptr->size;
 
   if ( CDI_Debug )
-    if ( zaxisptr->lbounds != NULL )
+    if ( zaxisptr->lbounds )
       Warning("Lower bounds already defined for zaxisID = %d", zaxisID);
 
   if ( zaxisptr->lbounds == NULL )
     zaxisptr->lbounds = (double *) Malloc(size*sizeof(double));
 
-  memcpy(zaxisptr->lbounds, lbounds, size*sizeof(double));
+  if (lbounds) memcpy(zaxisptr->lbounds, lbounds, size*sizeof(double));
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
 }
 
@@ -1309,13 +1067,13 @@ void zaxisDefUbounds(int zaxisID, const double *ubounds)
   const size_t size = (size_t)zaxisptr->size;
 
   if ( CDI_Debug )
-    if ( zaxisptr->ubounds != NULL )
+    if ( zaxisptr->ubounds )
       Warning("Upper bounds already defined for zaxisID = %d", zaxisID);
 
   if ( zaxisptr->ubounds == NULL )
     zaxisptr->ubounds = (double *) Malloc(size*sizeof(double));
 
-  memcpy(zaxisptr->ubounds, ubounds, size*sizeof(double));
+  if (ubounds) memcpy(zaxisptr->ubounds, ubounds, size*sizeof(double));
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
 }
 
@@ -1357,6 +1115,14 @@ void zaxisResize(int zaxisID, int size)
     zaxisptr->vals = (double *) Realloc(zaxisptr->vals, (size_t)size*sizeof(double));
 }
 
+static inline
+void zaxisCopyKeyStr(zaxis_t *zaxisptr1, zaxis_t *zaxisptr2, int key)
+{
+  cdi_key_t *keyp = find_key(&zaxisptr1->keys, key);
+  if (keyp && keyp->type == KEY_BYTES)
+    cdiDefVarKeyBytes(&zaxisptr2->keys, key, (const unsigned char*)keyp->v.s, (int)keyp->length);
+}
+
 
 int zaxisDuplicate(int zaxisID)
 {
@@ -1370,9 +1136,9 @@ int zaxisDuplicate(int zaxisID)
 
   zaxis_copy(zaxisptrnew, zaxisptr);
 
-  strcpy(zaxisptrnew->name, zaxisptr->name);
-  strcpy(zaxisptrnew->longname, zaxisptr->longname);
-  strcpy(zaxisptrnew->units, zaxisptr->units);
+  zaxisCopyKeyStr(zaxisptr, zaxisptrnew, CDI_KEY_NAME);
+  zaxisCopyKeyStr(zaxisptr, zaxisptrnew, CDI_KEY_LONGNAME);
+  zaxisCopyKeyStr(zaxisptr, zaxisptrnew, CDI_KEY_UNITS);
 
   if ( zaxisptr->vals )
     {
@@ -1406,6 +1172,9 @@ int zaxisDuplicate(int zaxisID)
         }
     }
 
+  zaxisptrnew->atts.nelems = 0;
+  cdiCopyAtts(zaxisID, CDI_GLOBAL, zaxisIDnew, CDI_GLOBAL);
+
   return zaxisIDnew;
 }
 
@@ -1420,8 +1189,6 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
 
   const int dig = (datatype == CDI_DATATYPE_FLT64) ? 15 : 7;
 
-  int nbyte;
-  int nbyte0 = 0;
   fprintf(fp, "zaxistype = %s\n", zaxisNamePtr(type));
   fprintf(fp, "size      = %d\n", nlevels);
   if ( nlevels == 1 )
@@ -1429,14 +1196,18 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
       const bool zscalar = (bool)zaxisptr->scalar;
       if ( zscalar ) fprintf(fp, "scalar    = true\n");
     }
-  if ( zaxisptr->name[0]     ) fprintf(fp, "name      = %s\n", zaxisptr->name);
-  if ( zaxisptr->longname[0] ) fprintf(fp, "longname  = %s\n", zaxisptr->longname);
-  if ( zaxisptr->units[0]    ) fprintf(fp, "units     = %s\n", zaxisptr->units);
+
+  const char *string = cdiInqVarKeyString(&zaxisptr->keys, CDI_KEY_NAME);
+  if ( string[0] ) fprintf(fp, "name      = %s\n", string);
+  string = cdiInqVarKeyString(&zaxisptr->keys, CDI_KEY_LONGNAME);
+  if ( string[0] ) fprintf(fp, "longname  = %s\n", string);
+  string = cdiInqVarKeyString(&zaxisptr->keys, CDI_KEY_UNITS);
+  if ( string[0] ) fprintf(fp, "units     = %s\n", string);
 
   if ( zaxisptr->vals )
     {
-      nbyte0 = fprintf(fp, "levels    = ");
-      nbyte = nbyte0;
+      int nbyte0 = fprintf(fp, "levels    = ");
+      int nbyte = nbyte0;
       for ( int levelID = 0; levelID < nlevels; levelID++ )
         {
           if ( nbyte > 80 )
@@ -1452,8 +1223,8 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
 
   if ( zaxisptr->lbounds && zaxisptr->ubounds )
     {
-      nbyte0 = fprintf(fp, "lbounds   = ");
-      nbyte = nbyte0;
+      int nbyte0 = fprintf(fp, "lbounds   = ");
+      int nbyte = nbyte0;
       for ( int levelID = 0; levelID < nlevels; levelID++ )
 	{
 	  if ( nbyte > 80 )
@@ -1488,8 +1259,8 @@ void zaxisPrintKernel(zaxis_t *zaxisptr, FILE *fp)
       fprintf(fp, "vctsize   = %d\n", vctsize);
       if ( vctsize )
         {
-          nbyte0 = fprintf(fp, "vct       = ");
-          nbyte = nbyte0;
+          int nbyte0 = fprintf(fp, "vct       = ");
+          int nbyte = nbyte0;
           for ( int i = 0; i < vctsize; i++ )
             {
               if ( nbyte > 70 || i == vctsize/2 )
@@ -1524,7 +1295,7 @@ int zaxisCompareP(zaxis_t *z1, zaxis_t *z2)
   xassert(z1 && z2);
 
   diff |= (z1->type != z2->type)
-    | (z1->ltype != z2->ltype)
+    | (cdiInqVarKeyInt(&z1->keys, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE) != cdiInqVarKeyInt(&z2->keys, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE))
     | (z1->direction != z2->direction)
     | (z1->datatype != z2->datatype)
     | (z1->size != z2->size)
@@ -1589,11 +1360,11 @@ int zaxisCompareP(zaxis_t *z1, zaxis_t *z2)
   if (anyPresent)
     xassert(size);
 
-  diff |= strcmp(z1->name, z2->name)
-    | strcmp(z1->longname, z2->longname)
-    | strcmp(z1->stdname, z2->stdname)
-    | strcmp(z1->units, z2->units)
-    | memcmp(z1->uuid, z2->uuid, CDI_UUID_SIZE);
+  diff |= strcmp(cdiInqVarKeyString(&z1->keys, CDI_KEY_NAME), cdiInqVarKeyString(&z2->keys, CDI_KEY_NAME))
+    | strcmp(cdiInqVarKeyString(&z1->keys, CDI_KEY_LONGNAME), cdiInqVarKeyString(&z2->keys, CDI_KEY_LONGNAME))
+    | strcmp(cdiInqVarKeyString(&z1->keys, CDI_KEY_STDNAME), cdiInqVarKeyString(&z2->keys, CDI_KEY_STDNAME))
+    | strcmp(cdiInqVarKeyString(&z1->keys, CDI_KEY_UNITS), cdiInqVarKeyString(&z2->keys, CDI_KEY_UNITS));
+
   return diff != 0;
 }
 
@@ -1604,18 +1375,14 @@ int zaxisTxCode(void)
   return ZAXIS;
 }
 
-enum { zaxisNint     = 8,
+enum { zaxisNint     = 7,
        vals     = 1 << 0,
        lbounds  = 1 << 1,
        ubounds  = 1 << 2,
        weights  = 1 << 3,
        vct      = 1 << 4,
-       zaxisHasUUIDFlag = 1 << 5,
 };
 
-#define ZAXIS_STR_SERIALIZE { zaxisP->name, zaxisP->longname, \
-                              zaxisP->stdname, zaxisP->units }
-
 static
 int zaxisGetMemberMask( zaxis_t * zaxisP )
 {
@@ -1626,7 +1393,7 @@ int zaxisGetMemberMask( zaxis_t * zaxisP )
   if ( zaxisP->ubounds )   memberMask |= ubounds;
   if ( zaxisP->weights )   memberMask |= weights;
   if ( zaxisP->vct )       memberMask |= vct;
-  if (!cdiUUIDIsNull(zaxisP->uuid)) memberMask |= zaxisHasUUIDFlag;
+
   return memberMask;
 }
 
@@ -1663,18 +1430,10 @@ zaxisGetPackSize(void * voidP, void *context)
         + serializeGetSize(1, CDI_DATATYPE_UINT32, context);
     }
 
-  {
-    const char *strTab[] = ZAXIS_STR_SERIALIZE;
-    size_t numStr = sizeof (strTab) / sizeof (strTab[0]);
-    packBufferSize
-      += serializeStrTabGetPackSize(strTab, (int)numStr, context);
-  }
+  packBufferSize += serializeKeysGetPackSize(&zaxisP->keys, context);
 
   packBufferSize += serializeGetSize(1, CDI_DATATYPE_UCHAR, context);
 
-  if (!cdiUUIDIsNull(zaxisP->uuid))
-    packBufferSize += serializeGetSize(CDI_UUID_SIZE, CDI_DATATYPE_UCHAR, context);
-
   return packBufferSize;
 }
 
@@ -1702,11 +1461,10 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
 
   zaxisP->datatype  = intBuffer[1];
   zaxisP->type      = intBuffer[2];
-  zaxisP->ltype     = intBuffer[3];
-  zaxisP->size      = intBuffer[4];
-  zaxisP->direction = intBuffer[5];
-  zaxisP->vctsize   = intBuffer[6];
-  memberMask        = intBuffer[7];
+  zaxisP->size      = intBuffer[3];
+  zaxisP->direction = intBuffer[4];
+  zaxisP->vctsize   = intBuffer[5];
+  memberMask        = intBuffer[6];
 
   if (memberMask & vals)
     {
@@ -1773,20 +1531,11 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       xassert(cdiCheckSum(CDI_DATATYPE_FLT64, size, zaxisP->vct) == d);
     }
 
-  {
-    char *strTab[] = ZAXIS_STR_SERIALIZE;
-    int numStr = sizeof (strTab) / sizeof (strTab[0]);
-    serializeStrTabUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                          strTab, numStr, context);
-  }
+  serializeKeysUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos, &zaxisP->keys, context);
 
   serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                   &zaxisP->positive, 1, CDI_DATATYPE_UINT, context);
 
-  if (memberMask & zaxisHasUUIDFlag)
-    serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                    zaxisP->uuid, CDI_UUID_SIZE, CDI_DATATYPE_UCHAR, context);
-
   reshSetStatus(zaxisP->self, &zaxisOps,
                 reshGetStatus(zaxisP->self, &zaxisOps) & ~RESH_SYNC_BIT);
 }
@@ -1803,11 +1552,10 @@ zaxisPack(void * voidP, void * packBuffer, int packBufferSize,
   intBuffer[0]  = zaxisP->self;
   intBuffer[1]  = zaxisP->datatype;
   intBuffer[2]  = zaxisP->type;
-  intBuffer[3]  = zaxisP->ltype;
-  intBuffer[4]  = zaxisP->size;
-  intBuffer[5]  = zaxisP->direction;
-  intBuffer[6]  = zaxisP->vctsize;
-  intBuffer[7]  = memberMask = zaxisGetMemberMask ( zaxisP );
+  intBuffer[3]  = zaxisP->size;
+  intBuffer[4]  = zaxisP->direction;
+  intBuffer[5]  = zaxisP->vctsize;
+  intBuffer[6]  = memberMask = zaxisGetMemberMask ( zaxisP );
 
   serializePack(intBuffer, zaxisNint, CDI_DATATYPE_INT,
                 packBuffer, packBufferSize, packBufferPos, context);
@@ -1869,20 +1617,10 @@ zaxisPack(void * voidP, void * packBuffer, int packBufferSize,
                     packBuffer, packBufferSize, packBufferPos, context);
     }
 
-  {
-    const char *strTab[] = ZAXIS_STR_SERIALIZE;
-    int numStr = sizeof (strTab) / sizeof (strTab[0]);
-    serializeStrTabPack(strTab, numStr,
-                        packBuffer, packBufferSize, packBufferPos, context);
-  }
+  serializeKeysPack(&zaxisP->keys, packBuffer, packBufferSize, packBufferPos, context);
 
   serializePack(&zaxisP->positive, 1, CDI_DATATYPE_UINT,
                 packBuffer, packBufferSize, packBufferPos, context);
-
-  if (memberMask & zaxisHasUUIDFlag)
-    serializePack(zaxisP->uuid, CDI_UUID_SIZE, CDI_DATATYPE_UCHAR,
-                  packBuffer, packBufferSize, packBufferPos, context);
-
 }
 
 
@@ -1891,8 +1629,6 @@ void cdiZaxisGetIndexList(unsigned nzaxis, int *zaxisResHs)
   reshGetResHListOfType(nzaxis, zaxisResHs, &zaxisOps);
 }
 
-#undef ZAXIS_STR_SERIALIZE
-
 /*
  * Local Variables:
  * c-file-style: "Java"
