@@ -99,13 +99,13 @@ static const struct {
   char suffix[4];
   int type, defaultDT, defaultGrid;
 } suffix2type[] = {
-  { "nc", FILETYPE_NC, DATATYPE_FLT64, GRID_LONLAT },
-  { "grb",  FILETYPE_GRB, DATATYPE_PACK24, GRID_LONLAT },
-  { "nc2", FILETYPE_NC2, DATATYPE_FLT64, GRID_LONLAT },
-  { "nc4", FILETYPE_NC4, DATATYPE_FLT64, GRID_LONLAT },
-  { "ext", FILETYPE_EXT, DATATYPE_FLT64, GRID_GENERIC, },
-  { "svc", FILETYPE_SRV, DATATYPE_FLT64, GRID_GENERIC, },
-  { "ieg", FILETYPE_IEG, DATATYPE_FLT64, GRID_LONLAT },
+  { "nc", CDI_FILETYPE_NC, CDI_DATATYPE_FLT64, GRID_LONLAT },
+  { "grb",  CDI_FILETYPE_GRB, CDI_DATATYPE_PACK24, GRID_LONLAT },
+  { "nc2", CDI_FILETYPE_NC2, CDI_DATATYPE_FLT64, GRID_LONLAT },
+  { "nc4", CDI_FILETYPE_NC4, CDI_DATATYPE_FLT64, GRID_LONLAT },
+  { "ext", CDI_FILETYPE_EXT, CDI_DATATYPE_FLT64, GRID_GENERIC, },
+  { "svc", CDI_FILETYPE_SRV, CDI_DATATYPE_FLT64, GRID_GENERIC, },
+  { "ieg", CDI_FILETYPE_IEG, CDI_DATATYPE_FLT64, GRID_LONLAT },
 };
 #endif
 enum {
@@ -130,11 +130,11 @@ main(int argc, char *argv[])
 #ifndef TEST_CHUNK_WRITE
   const char *suffix = "grb", *prefix = "example";
   int grid = GRID_LONLAT;
-  int filetype = FILETYPE_GRB, datatype = DATATYPE_PACK24;
+  int filetype = CDI_FILETYPE_GRB, datatype = CDI_DATATYPE_PACK24;
 #else
   const char *suffix = "nc", *prefix = "example";
   int grid = GRID_LONLAT;
-  int filetype = FILETYPE_NC, datatype = DATATYPE_FLT64;
+  int filetype = CDI_FILETYPE_NC, datatype = CDI_DATATYPE_FLT64;
 #endif
   {
     int opt;
@@ -226,7 +226,7 @@ main(int argc, char *argv[])
 
   /* add uuids to zaxis and grid */
   {
-    unsigned char uuid[16];
+    unsigned char uuid[CDI_UUID_SIZE];
 
     static char gridUUIDTxt[] = "107d7a5b-348c-4d1a-90a9-d745914f2fb6";
 
@@ -241,7 +241,7 @@ main(int argc, char *argv[])
     for (int i = 0; i < 2; ++i)
       {
         cdiStr2UUID(zaxisUUIDTxt[i], uuid);
-        zaxisDefUUID(zaxisID[i], uuid);
+        cdiDefKeyBytes(zaxisID[i], CDI_GLOBAL, CDI_KEY_UUID, uuid, CDI_UUID_SIZE);
       }
   }
 
@@ -319,7 +319,7 @@ main(int argc, char *argv[])
                             / (lons[nlat-1] - lats[0]))
                       ) * mscale)) * mrscale;
 
-        if (filetype == FILETYPE_EXT)
+        if (filetype == CDI_FILETYPE_EXT)
           {
             /* EXTRA doesn't store time, only date
              * set the value to 0 before checksumming, because a
