@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,9 @@
 #include <cdi.h>
 
 #include "cdo_options.h"
-#include "cdo_int.h"
+#include "cdo_zaxis.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "util_string.h"
 
@@ -89,8 +91,8 @@ adipot(const double t, const double s, const double p)
 }
 
 static void
-calc_adisit(size_t gridsize, size_t nlevel, const std::vector<double> &pressure, const FieldVector &tho,
-            const FieldVector &sao, FieldVector &tis)
+calc_adisit(size_t gridsize, size_t nlevel, const Varray<double> &pressure, const FieldVector &tho, const FieldVector &sao,
+            FieldVector &tis)
 {
   // pressure units: hPa
   // tho units:      Celsius
@@ -98,29 +100,23 @@ calc_adisit(size_t gridsize, size_t nlevel, const std::vector<double> &pressure,
 
   for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const auto &thovec = tho[levelID].vec;
-      const auto &saovec = sao[levelID].vec;
-      auto &tisvec = tis[levelID].vec;
-      const double tho_missval = tho[levelID].missval;
-      const double sao_missval = sao[levelID].missval;
-      const double tis_missval = tis[levelID].missval;
+      const auto &thovec = tho[levelID].vec_d;
+      const auto &saovec = sao[levelID].vec_d;
+      auto &tisvec = tis[levelID].vec_d;
+      const auto tho_missval = tho[levelID].missval;
+      const auto sao_missval = sao[levelID].missval;
+      const auto tis_missval = tis[levelID].missval;
       for (size_t i = 0; i < gridsize; ++i)
         {
-          if (DBL_IS_EQUAL(thovec[i], tho_missval) || DBL_IS_EQUAL(saovec[i], sao_missval))
-            {
-              tisvec[i] = tis_missval;
-            }
-          else
-            {
-              tisvec[i] = adisit_1(thovec[i], saovec[i], pressure[levelID]);
-            }
+          const auto is_missing = (DBL_IS_EQUAL(thovec[i], tho_missval) || DBL_IS_EQUAL(saovec[i], sao_missval));
+          tisvec[i] = is_missing ? tis_missval : adisit_1(thovec[i], saovec[i], pressure[levelID]);
         }
     }
 }
 
 static void
-calc_adipot(size_t gridsize, size_t nlevel, const std::vector<double> &pressure, const FieldVector &t,
-            const FieldVector &s, FieldVector &tpot)
+calc_adipot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, const FieldVector &t, const FieldVector &s,
+            FieldVector &tpot)
 {
   // pressure units: hPa
   // t units:        Celsius
@@ -128,22 +124,16 @@ calc_adipot(size_t gridsize, size_t nlevel, const std::vector<double> &pressure,
 
   for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const auto &tvec = t[levelID].vec;
-      const auto &svec = s[levelID].vec;
-      auto &tpotvec = tpot[levelID].vec;
-      const double t_missval = t[levelID].missval;
-      const double s_missval = s[levelID].missval;
-      const double tpot_missval = tpot[levelID].missval;
+      const auto &tvec = t[levelID].vec_d;
+      const auto &svec = s[levelID].vec_d;
+      auto &tpotvec = tpot[levelID].vec_d;
+      const auto t_missval = t[levelID].missval;
+      const auto s_missval = s[levelID].missval;
+      const auto tpot_missval = tpot[levelID].missval;
       for (size_t i = 0; i < gridsize; ++i)
         {
-          if (DBL_IS_EQUAL(tvec[i], t_missval) || DBL_IS_EQUAL(svec[i], s_missval))
-            {
-              tpotvec[i] = tpot_missval;
-            }
-          else
-            {
-              tpotvec[i] = adipot(tvec[i], svec[i], pressure[levelID]);
-            }
+          const auto is_missing = (DBL_IS_EQUAL(tvec[i], t_missval) || DBL_IS_EQUAL(svec[i], s_missval));
+          tpotvec[i] = is_missing ? tpot_missval : adipot(tvec[i], svec[i], pressure[levelID]);
         }
     }
 }
@@ -158,40 +148,38 @@ Adisit(void *process)
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
 
   cdoInitialize(process);
-  const int ADISIT = cdoOperatorAdd("adisit", 1, 1, "");
-  const int ADIPOT = cdoOperatorAdd("adipot", 1, 1, "");
 
-  const int operatorID = cdoOperatorID();
+  const auto ADISIT = cdoOperatorAdd("adisit", 1, 1, "");
+  const auto ADIPOT = cdoOperatorAdd("adipot", 1, 1, "");
 
-  const double pin = (operatorArgc() == 1) ? parameter2double(operatorArgv()[0]) : -1;
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto pin = (operatorArgc() == 1) ? parameter2double(cdoOperatorArgv(0)) : -1.0;
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+
+  const auto nvars = vlistNvars(vlistID1);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int code = vlistInqVarCode(vlistID1, varID);
+      auto code = vlistInqVarCode(vlistID1, varID);
       if (code <= 0)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          vlistInqVarStdname(vlistID1, varID, stdname);
           cstrToLowerCase(varname);
 
-          if (cstrIsEqual(varname, "s") || cstrIsEqual(stdname, "sea_water_salinity"))
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID1, varID, CDI_KEY_STDNAME, stdname, &length);
+          cstrToLowerCase(stdname);
+
+          if (cdo_cmpstr(varname, "s") || cdo_cmpstr(stdname, "sea_water_salinity"))
             code = 5;
-          else if (cstrIsEqual(varname, "t"))
+          else if (cdo_cmpstr(varname, "t"))
             code = 2;
 
-          if (operatorID == ADISIT)
-            {
-              if (cstrIsEqual(stdname, "sea_water_potential_temperature")) code = 2;
-            }
-          else
-            {
-              if (cstrIsEqual(stdname, "sea_water_temperature")) code = 2;
-            }
+          const char *cname = (operatorID == ADISIT) ? "sea_water_potential_temperature" : "sea_water_temperature";
+          if (cdo_cmpstr(stdname, cname)) code = 2;
         }
 
       if (code == 2 || (code == 20 && operatorID == ADIPOT))
@@ -203,33 +191,33 @@ Adisit(void *process)
   if (saoID == -1) cdoAbort("Sea water salinity not found!");
   if (thoID == -1) cdoAbort("%s temperature not found!", (operatorID == ADISIT) ? "Potential" : "Insitu");
 
-  const int gridID = vlistGrid(vlistID1, 0);
-  const size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridID = vlistGrid(vlistID1, 0);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
-  int zaxisID = vlistInqVarZaxis(vlistID1, saoID);
-  const int nlevel1 = zaxisInqSize(zaxisID);
+  auto zaxisID = vlistInqVarZaxis(vlistID1, saoID);
+  const auto nlevels1 = zaxisInqSize(zaxisID);
   zaxisID = vlistInqVarZaxis(vlistID1, thoID);
-  const int nlevel2 = zaxisInqSize(zaxisID);
+  const auto nlevels2 = zaxisInqSize(zaxisID);
 
-  if (nlevel1 != nlevel2) cdoAbort("temperature and salinity have different number of levels!");
-  const int nlevel = nlevel1;
+  if (nlevels1 != nlevels2) cdoAbort("temperature and salinity have different number of levels!");
+  const auto nlevels = nlevels1;
 
-  std::vector<double> pressure(nlevel);
+  Varray<double> pressure(nlevels);
   cdoZaxisInqLevels(zaxisID, pressure.data());
 
   if (pin >= 0)
-    for (i = 0; i < nlevel; ++i) pressure[i] = pin;
+    for (i = 0; i < nlevels; ++i) pressure[i] = pin;
   else
-    for (i = 0; i < nlevel; ++i) pressure[i] /= 10;
+    for (i = 0; i < nlevels; ++i) pressure[i] /= 10;
 
   if (Options::cdoVerbose)
     {
       cdoPrint("Level Pressure");
-      for (i = 0; i < nlevel; ++i) cdoPrint("%5d  %g", i + 1, pressure[i]);
+      for (i = 0; i < nlevels; ++i) cdoPrint("%5d  %g", i + 1, pressure[i]);
     }
 
-  FieldVector tho(nlevel), sao(nlevel), tis(nlevel);
-  for (levelID = 0; levelID < nlevel; ++levelID)
+  FieldVector tho(nlevels), sao(nlevels), tis(nlevels);
+  for (levelID = 0; levelID < nlevels; ++levelID)
     {
       tho[levelID].resize(gridsize);
       sao[levelID].resize(gridsize);
@@ -243,41 +231,41 @@ Adisit(void *process)
   if (vlistInqVarDatatype(vlistID1, thoID) == CDI_DATATYPE_FLT64 && vlistInqVarDatatype(vlistID1, saoID) == CDI_DATATYPE_FLT64)
     datatype = CDI_DATATYPE_FLT64;
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
 
-  const int tisID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
+  const auto tisID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
   if (operatorID == ADISIT)
     {
       vlistDefVarParam(vlistID2, tisID2, cdiEncodeParam(20, 255, 255));
-      vlistDefVarName(vlistID2, tisID2, "to");
-      vlistDefVarLongname(vlistID2, tisID2, "Sea water temperature");
-      vlistDefVarStdname(vlistID2, tisID2, "sea_water_temperature");
+      cdiDefKeyString(vlistID2, tisID2, CDI_KEY_NAME, "to");
+      cdiDefKeyString(vlistID2, tisID2, CDI_KEY_LONGNAME, "Sea water temperature");
+      cdiDefKeyString(vlistID2, tisID2, CDI_KEY_STDNAME, "sea_water_temperature");
     }
   else
     {
       vlistDefVarParam(vlistID2, tisID2, cdiEncodeParam(2, 255, 255));
-      vlistDefVarName(vlistID2, tisID2, "tho");
-      vlistDefVarLongname(vlistID2, tisID2, "Sea water potential temperature");
-      vlistDefVarStdname(vlistID2, tisID2, "sea_water_potential_temperature");
+      cdiDefKeyString(vlistID2, tisID2, CDI_KEY_NAME, "tho");
+      cdiDefKeyString(vlistID2, tisID2, CDI_KEY_LONGNAME, "Sea water potential temperature");
+      cdiDefKeyString(vlistID2, tisID2, CDI_KEY_STDNAME, "sea_water_potential_temperature");
     }
-  vlistDefVarUnits(vlistID2, tisID2, "K");
+  cdiDefKeyString(vlistID2, tisID2, CDI_KEY_UNITS, "K");
   vlistDefVarMissval(vlistID2, tisID2, tis[0].missval);
   vlistDefVarDatatype(vlistID2, tisID2, datatype);
 
-  const int saoID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
+  const auto saoID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
   vlistDefVarParam(vlistID2, saoID2, cdiEncodeParam(5, 255, 255));
-  vlistDefVarName(vlistID2, saoID2, "s");
-  vlistDefVarLongname(vlistID2, saoID2, "Sea water salinity");
-  vlistDefVarStdname(vlistID2, saoID2, "sea_water_salinity");
-  vlistDefVarUnits(vlistID2, saoID2, "psu");
+  cdiDefKeyString(vlistID2, saoID2, CDI_KEY_NAME, "s");
+  cdiDefKeyString(vlistID2, saoID2, CDI_KEY_LONGNAME, "Sea water salinity");
+  cdiDefKeyString(vlistID2, saoID2, CDI_KEY_STDNAME, "sea_water_salinity");
+  cdiDefKeyString(vlistID2, saoID2, CDI_KEY_UNITS, "psu");
   vlistDefVarMissval(vlistID2, saoID2, sao[0].missval);
   vlistDefVarDatatype(vlistID2, saoID2, datatype);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -289,22 +277,22 @@ Adisit(void *process)
       for (int recID = 0; recID < nrecs; ++recID)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          if (varID == thoID) cdoReadRecord(streamID1, tho[levelID].vec.data(), &tho[levelID].nmiss);
-          if (varID == saoID) cdoReadRecord(streamID1, sao[levelID].vec.data(), &sao[levelID].nmiss);
+          if (varID == thoID) cdoReadRecord(streamID1, tho[levelID].vec_d.data(), &tho[levelID].nmiss);
+          if (varID == saoID) cdoReadRecord(streamID1, sao[levelID].vec_d.data(), &sao[levelID].nmiss);
         }
 
       if (operatorID == ADISIT)
-        calc_adisit(gridsize, nlevel, pressure, tho, sao, tis);
+        calc_adisit(gridsize, nlevels, pressure, tho, sao, tis);
       else
-        calc_adipot(gridsize, nlevel, pressure, tho, sao, tis);
+        calc_adipot(gridsize, nlevels, pressure, tho, sao, tis);
 
-      for (levelID = 0; levelID < nlevel; ++levelID)
+      for (levelID = 0; levelID < nlevels; ++levelID)
         {
           cdoDefRecord(streamID2, tisID2, levelID);
-          cdoWriteRecord(streamID2, tis[levelID].vec.data(), fieldNumMiss(tis[levelID]));
+          cdoWriteRecord(streamID2, tis[levelID].vec_d.data(), fieldNumMiss(tis[levelID]));
 
           cdoDefRecord(streamID2, saoID2, levelID);
-          cdoWriteRecord(streamID2, sao[levelID].vec.data(), fieldNumMiss(sao[levelID]));
+          cdoWriteRecord(streamID2, sao[levelID].vec_d.data(), fieldNumMiss(sao[levelID]));
         }
 
       tsID++;
