@@ -98,6 +98,9 @@ Arithdays(void *process)
   const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
@@ -126,9 +129,9 @@ Arithdays(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
-
-          field.grid = vlistInqVarGrid(vlistID1, varID);
-          field.missval = vlistInqVarMissval(vlistID1, varID);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
           vfarcfun(field, rconst, operfunc);
 
