@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,10 +21,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_wtime.h"
-
 #include "process_int.h"
 
 void *
@@ -37,37 +35,37 @@ CDItest(void *process)
 
   cdoInitialize(process);
 
-  bool lcopy = false;
+  const auto lcopy = false;
   // bool lcopy = unchangedRecord();
 
-  const int NCOPY = cdoOperatorAdd("ncopy", 0, 0, nullptr);
-  UNUSED(NCOPY);
+  const auto NCOPY = cdoOperatorAdd("ncopy", 0, 0, nullptr);
+  (void) (NCOPY);  // CDO_UNUSED
 
-  const int operatorID = cdoOperatorID();
-  UNUSED(operatorID);
+  const auto operatorID = cdoOperatorID();
+  (void) (operatorID);  // CDO_UNUSED
 
   //  operatorInputArg("Number of copies");
-  if (operatorArgc() == 1) max_copy = parameter2int(operatorArgv()[0]);
+  if (operatorArgc() == 1) max_copy = parameter2int(cdoOperatorArgv(0));
 
-  const double startTime = cdo_get_wtime();
+  const auto startTime = cdo_get_wtime();
 
   int n = 0;
   while (true)
     {
-      CdoStreamID streamID1 = cdoOpenRead(0);
+      const auto streamID1 = cdoOpenRead(0);
 
-      const int vlistID1 = cdoStreamInqVlist(streamID1);
-      const int taxisID1 = vlistInqTaxis(vlistID1);
+      const auto vlistID1 = cdoStreamInqVlist(streamID1);
+      const auto taxisID1 = vlistInqTaxis(vlistID1);
 
-      const int vlistID2 = vlistDuplicate(vlistID1);
-      const int taxisID2 = taxisDuplicate(taxisID1);
+      const auto vlistID2 = vlistDuplicate(vlistID1);
+      const auto taxisID2 = taxisDuplicate(taxisID1);
       vlistDefTaxis(vlistID2, taxisID2);
 
-      CdoStreamID streamID2 = cdoOpenWrite(1);
+      const auto streamID2 = cdoOpenWrite(1);
       cdoDefVlist(streamID2, vlistID2);
 
-      const size_t gridsize = vlistGridsizeMax(vlistID1);
-      std::vector<double> array(gridsize);
+      const auto gridsize = vlistGridsizeMax(vlistID1);
+      Varray<double> array(gridsize);
 
       int tsID1 = 0;
       int tsID2 = 0;
