@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,24 +23,23 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include "cdo_cmor.h"
-
 #include "pmlist.h"
 #include "convert_units.h"
-#include "cdo_cdiWrapper.h"
-
-int stringToParam(const char *paramstr);
+#include "cdo_cdi_wrapper.h"
+#include "cdi_lockedIO.h"
 
 void
-cdo_define_var_units(CmorVar &var, int vlistID2, int varID, const char *units)
+cdo_define_var_units(CmorVar &var, const int vlistID2, const int varID, const char *const units)
 {
   char units_old[CDI_MAX_NAME];
 
   vlistInqVarUnits(vlistID2, varID, units_old);
-  const size_t len1 = strlen(units_old);
-  const size_t len2 = strlen(units);
+  const auto len1 = strlen(units_old);
+  const auto len2 = strlen(units);
 
   if (strcmp(units, units_old) != 0)
     {
@@ -51,7 +50,7 @@ cdo_define_var_units(CmorVar &var, int vlistID2, int varID, const char *units)
           strcpy(var.units, units);
         }
 
-      vlistDefVarUnits(vlistID2, varID, units);
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, units);
       cdiDefAttTxt(vlistID2, varID, "original_units", (int) strlen(units_old), units_old);
     }
 }
@@ -61,7 +60,7 @@ cmor_check_init(int nvars, std::vector<CmorVar> &vars)
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
-      CmorVar &var = vars[varID];
+      auto &var = vars[varID];
       if (var.checkvalid || var.check_min_mean_abs || var.check_max_mean_abs)
         {
           var.amean = 0;
@@ -73,22 +72,22 @@ cmor_check_init(int nvars, std::vector<CmorVar> &vars)
 }
 
 void
-cmor_check_eval(int vlistID, int nvars, std::vector<CmorVar> &vars)
+cmor_check_eval(int vlistID, int nvars, const std::vector<CmorVar> &vars)
 {
   char varname[CDI_MAX_NAME];
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const CmorVar &var = vars[varID];
+      const auto &var = vars[varID];
       if (var.checkvalid || var.check_min_mean_abs || var.check_max_mean_abs)
         {
-          double amean = var.amean;
-          const long nvals = var.nvals;
+          auto amean = var.amean;
+          const auto nvals = var.nvals;
 
           if (nvals > 0) amean /= nvals;
 
-          const long n_lower_min = var.n_lower_min;
-          const long n_greater_max = var.n_greater_max;
+          const auto n_lower_min = var.n_lower_min;
+          const auto n_greater_max = var.n_greater_max;
 
           vlistInqVarName(vlistID, varID, varname);
 
@@ -127,17 +126,16 @@ cmor_check_eval(int vlistID, int nvars, std::vector<CmorVar> &vars)
 }
 
 void
-cmor_check_prep(CmorVar &var, long gridsize, double missval, double *array)
+cmor_check_prep(CmorVar &var, const long gridsize, const double missval, const double *const array)
 {
   if (var.checkvalid || var.check_min_mean_abs || var.check_max_mean_abs)
     {
-      double aval;
       double amean = 0;
       long nvals = 0;
 
       for (long i = 0; i < gridsize; ++i)
         {
-          aval = array[i];
+          const auto aval = array[i];
           if (!DBL_IS_EQUAL(aval, missval))
             {
               amean += std::fabs(aval);
@@ -148,12 +146,11 @@ cmor_check_prep(CmorVar &var, long gridsize, double missval, double *array)
       var.amean += amean;
       var.nvals += nvals;
 
-      long n_lower_min = 0;
-      long n_greater_max = 0;
+      long n_lower_min = 0, n_greater_max = 0;
 
       for (long i = 0; i < gridsize; ++i)
         {
-          aval = array[i];
+          const auto aval = array[i];
           if (!DBL_IS_EQUAL(aval, missval))
             {
               if (aval < var.valid_min) n_lower_min++;
@@ -178,24 +175,23 @@ applyCmorList(PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &var
   double missval = 0;
 
   {
-    const KVList *kvlist = pmlist.getKVListVentry(hentry);
+    auto kvlist = pmlist.getKVListVentry(hentry);
     if (kvlist)
       {
         for (const auto &kv : *kvlist)
           {
-            const char *key = kv.key.c_str();
-            const char *value = (kv.nvalues == 1) ? kv.values[0].c_str() : nullptr;
-            if (!value || (value && !*value)) continue;
+            const auto &key = kv.key;
+            const auto &value = kv.values[0];
+            if (kv.nvalues != 1 || value.empty()) continue;
 
-            if (cstrIsEqual(key, "missing_value"))
+            if (key == "missing_value")
               {
                 lmissval = true;
-                missval = parameter2double(kv.values[0].c_str());
+                missval = parameter2double(value);
               }
-            else if (cstrIsEqual(key, "table_id") || cstrIsEqual(key, "modeling_realm") || cstrIsEqual(key, "realm")
-                     || cstrIsEqual(key, "project_id") || cstrIsEqual(key, "frequency"))
+            else if (key == "table_id" || key == "modeling_realm" || key == "realm" || key == "project_id" || key == "frequency")
               {
-                cdiDefAttTxt(vlistID2, CDI_GLOBAL, key, (int) strlen(value), value);
+                cdiDefAttTxt(vlistID2, CDI_GLOBAL, key.c_str(), (int) value.size(), value.c_str());
               }
           }
       }
@@ -203,13 +199,13 @@ applyCmorList(PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &var
 
   for (int varID = 0; varID < nvars; varID++)
     {
-      CmorVar &var = vars[varID];
+      auto &var = vars[varID];
       vlistInqVarName(vlistID2, varID, varname);
 
       strcpy(var.name, varname);
       if (lmissval)
         {
-          const double missval_old = vlistInqVarMissval(vlistID2, varID);
+          const auto missval_old = vlistInqVarMissval(vlistID2, varID);
           if (!DBL_IS_EQUAL(missval, missval_old))
             {
               var.changemissval = true;
@@ -218,61 +214,51 @@ applyCmorList(PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &var
             }
         }
 
-      const KVList *kvlist = pmlist.searchKVListVentry("name", varname, ventry);
+      auto kvlist = pmlist.searchKVListVentry("name", varname, ventry);
       if (kvlist)
         {
           bool lvalid_min = false, lvalid_max = false;
 
           for (const auto &kv : *kvlist)
             {
-              const char *key = kv.key.c_str();
-              const char *value = (kv.nvalues == 1) ? kv.values[0].c_str() : nullptr;
-              if (!value || (value && !*value)) continue;
+              const auto &key = kv.key;
+              const auto &value = kv.values[0];
+              if (kv.nvalues != 1 || value.empty()) continue;
+              auto value_cstr = value.c_str();
 
-              // printf("key=%s  value=>%s<\n", key, value);
+              // printf("key=%s  value=>%s<\n", key.c_str(), value.c_str());
 
-              if (cstrIsEqual(key, "standard_name"))
-                vlistDefVarStdname(vlistID2, varID, value);
-              else if (cstrIsEqual(key, "long_name"))
-                vlistDefVarLongname(vlistID2, varID, value);
-              else if (cstrIsEqual(key, "units"))
-                cdo_define_var_units(var, vlistID2, varID, value);
-              else if (cstrIsEqual(key, "name"))
-                // vlistDefVarName(vlistID2, varID, parameter2word(value))
-                ;
-              else if (cstrIsEqual(key, "out_name"))
+              // clang-format off
+              if      (key == "standard_name") cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, value_cstr);
+              else if (key == "long_name") cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, value_cstr);
+              else if (key == "units") cdo_define_var_units(var, vlistID2, varID, value_cstr);
+              else if (key == "name") ; // cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, parameter2word(value.c_str()));
+              else if (key == "out_name")
                 {
-                  const char *outname = parameter2word(value);
-                  if (!cstrIsEqual(var.name, outname))
+                  const auto outname = parameter2word(value.c_str());
+                  if (!cdo_cmpstr(var.name, outname))
                     {
-                      vlistDefVarName(vlistID2, varID, outname);
+                      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, outname);
                       cdiDefAttTxt(vlistID2, varID, "original_name", (int) strlen(var.name), var.name);
                     }
                 }
-              else if (cstrIsEqual(key, "param"))
-                vlistDefVarParam(vlistID2, varID, stringToParam(parameter2word(value)));
-              else if (cstrIsEqual(key, "out_param"))
-                vlistDefVarParam(vlistID2, varID, stringToParam(parameter2word(value)));
-              else if (cstrIsEqual(key, "comment"))
-                cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
-              else if (cstrIsEqual(key, "cell_methods"))
-                cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
-              else if (cstrIsEqual(key, "cell_measures"))
-                cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
-              else if (cstrIsEqual(key, "delete"))
-                var.remove = parameter2bool(value);
-              else if (cstrIsEqual(key, "convert"))
-                var.convert = parameter2bool(value);
-              else if (cstrIsEqual(key, "factor"))
+              else if (key == "param") vlistDefVarParam(vlistID2, varID, stringToParam(parameter2word(value.c_str())));
+              else if (key == "out_param") vlistDefVarParam(vlistID2, varID, stringToParam(parameter2word(value.c_str())));
+              else if (key == "comment") cdiDefAttTxt(vlistID2, varID, key.c_str(), (int) value.size(), value_cstr);
+              else if (key == "cell_methods") cdiDefAttTxt(vlistID2, varID, key.c_str(), (int) value.size(), value_cstr);
+              else if (key == "cell_measures") cdiDefAttTxt(vlistID2, varID, key.c_str(), (int) value.size(), value_cstr);
+              else if (key == "delete") var.remove = parameter2bool(value);
+              else if (key == "convert") var.convert = parameter2bool(value);
+              else if (key == "factor")
                 {
                   var.lfactor = true;
                   var.factor = parameter2double(value);
                   if (Options::cdoVerbose) cdoPrint("%s - scale factor %g", varname, var.factor);
                 }
-              else if (cstrIsEqual(key, "missval") || cstrIsEqual(key, "missing_value"))
+              else if (key == "missval" || key == "missing_value")
                 {
-                  double missval = parameter2double(value);
-                  double missval_old = vlistInqVarMissval(vlistID2, varID);
+                  missval = parameter2double(value);
+                  auto missval_old = vlistInqVarMissval(vlistID2, varID);
                   if (!DBL_IS_EQUAL(missval, missval_old))
                     {
                       if (Options::cdoVerbose) cdoPrint("%s - change missval from %g to %g", varname, missval_old, missval);
@@ -281,35 +267,36 @@ applyCmorList(PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &var
                       vlistDefVarMissval(vlistID2, varID, missval);
                     }
                 }
-              else if (cstrIsEqual(key, "valid_min"))
+              else if (key == "valid_min")
                 {
                   lvalid_min = true;
                   var.valid_min = parameter2double(value);
                 }
-              else if (cstrIsEqual(key, "valid_max"))
+              else if (key == "valid_max")
                 {
                   lvalid_max = true;
                   var.valid_max = parameter2double(value);
                 }
-              else if (cstrIsEqual(key, "ok_min_mean_abs"))
+              else if (key == "ok_min_mean_abs")
                 {
                   var.check_min_mean_abs = true;
                   var.ok_min_mean_abs = parameter2double(value);
                 }
-              else if (cstrIsEqual(key, "ok_max_mean_abs"))
+              else if (key == "ok_max_mean_abs")
                 {
                   var.check_max_mean_abs = true;
                   var.ok_max_mean_abs = parameter2double(value);
                 }
-              else if (cstrIsEqual(key, "datatype") || cstrIsEqual(key, "type"))
+              else if (key == "datatype" || key == "type")
                 {
-                  int datatype = str2datatype(parameter2word(value));
+                  const auto datatype = cdi_str2datatype(parameter2word(value.c_str()));
                   if (datatype != -1) vlistDefVarDatatype(vlistID2, varID, datatype);
                 }
               else
                 {
-                  if (Options::cdoVerbose) cdoPrint("Attribute %s:%s not supported!", varname, key);
+                  if (Options::cdoVerbose) cdoPrint("Attribute %s:%s not supported!", varname, key.c_str());
                 }
+              // clang-format on
             }
 
           if (lvalid_min && lvalid_max) var.checkvalid = true;
@@ -325,7 +312,6 @@ void *
 CMOR_lite(void *process)
 {
   int nrecs;
-  int varID, levelID;
   size_t nmiss;
   bool delvars = false;
 
@@ -336,7 +322,7 @@ CMOR_lite(void *process)
 
   cdoOperatorAdd("cmorlite", 0, 0, "parameter table name");
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
@@ -345,29 +331,27 @@ CMOR_lite(void *process)
   bool convert_data = false;
   if (operatorArgc() == 2)
     {
-      if (strcmp("convert", operatorArgv()[1]) == 0)
+      if (cdoOperatorArgv(1) == "convert")
         convert_data = true;
       else
-        cdoAbort("Unknown parameter: >%s<", operatorArgv()[1]);
+        cdoAbort("Unknown parameter: >%s<", cdoOperatorArgv(1).c_str());
     }
 
   if (operatorArgc() > 2) cdoAbort("Too many arguments!");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
-  /* vlistPrint(vlistID2);*/
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int nvars = vlistNvars(vlistID2);
+  const auto nvars = vlistNvars(vlistID2);
   std::vector<CmorVar> vars(nvars);
-  memset(vars.data(), 0, nvars * sizeof(CmorVar));
 
   if (convert_data)
-    for (varID = 0; varID < nvars; ++varID) vars[varID].convert = true;
+    for (int varID = 0; varID < nvars; ++varID) vars[varID].convert = true;
 
-  const char *filename = operatorArgv()[0];
-  FILE *fp = fopen(filename, "r");
+  const auto filename = cdoOperatorArgv(0).c_str();
+  auto fp = fopen(filename, "r");
   if (fp == nullptr) cdoAbort("Open failed on: %s\n", filename);
 
   PMList pmlist;
@@ -376,6 +360,9 @@ CMOR_lite(void *process)
 
   applyCmorList(pmlist, nvars, vlistID2, vars);
 
+  VarList varList2;
+  varListInit(varList2, vlistID2);
+
   for (int varID = 0; varID < nvars; ++varID)
     if (vars[varID].remove)
       {
@@ -390,9 +377,7 @@ CMOR_lite(void *process)
 
       for (int varID = 0; varID < nvars; varID++)
         {
-          const int zaxisID = vlistInqVarZaxis(vlistID2, varID);
-          const int nlevs = zaxisInqSize(zaxisID);
-          for (int levID = 0; levID < nlevs; levID++)
+          for (int levID = 0; levID < varList2[varID].nlevels; levID++)
             {
               vlistDefFlag(vlistID1, varID, levID, true);
               vlistDefFlag(vlistID2, varID, levID, true);
@@ -404,7 +389,7 @@ CMOR_lite(void *process)
             }
         }
 
-      int vlistIDx = vlistCreate();
+      const auto vlistIDx = vlistCreate();
       cdoVlistCopyFlag(vlistIDx, vlistID2);
 
       vlistDestroy(vlistID2);
@@ -415,23 +400,23 @@ CMOR_lite(void *process)
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      CmorVar &var = vars[varID];
+      auto &var = vars[varID];
       if (!var.convert) var.changeunits = false;
       if (var.changeunits)
         cdoConvertUnits(&var.ut_converter, &var.changeunits, (char *) &var.units, (char *) &var.units_old, var.name);
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   /* vlistPrint(vlistID2);*/
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-  std::vector<double> array(gridsizemax);
+  Varray<double> array(gridsizemax);
 
   int tsID1 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
@@ -444,11 +429,12 @@ CMOR_lite(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          CmorVar &var = vars[varID];
-          int varID2 = varID;
-          int levelID2 = levelID;
+          auto &var = vars[varID];
+          auto varID2 = varID;
+          auto levelID2 = levelID;
 
           if (delvars)
             {
@@ -465,9 +451,9 @@ CMOR_lite(void *process)
 
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
-          const double missval = vlistInqVarMissval(vlistID2, varID2);
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
-          if (vlistInqVarNumber(vlistID2, varID2) != CDI_REAL) gridsize *= 2;
+          const auto missval = varList2[varID2].missval;
+          auto gridsize = varList2[varID2].gridsize;
+          if (varList2[varID2].nwpv != CDI_REAL) gridsize *= 2;
 
           if (nmiss && var.changemissval)
             {
