@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,11 +21,9 @@
 */
 
 #include "cdo_options.h"
-#include "dmemory.h"
 #include "process_int.h"
 #include "pmlist.h"
 
-
 static void
 dump_cmor_table(const PMList &pmlist)
 {
@@ -56,15 +54,15 @@ conv_cmor_table(const PMList &pmlist)
 
   for (const auto &kvlist : pmlist)
     {
-      const char *listname = kvlist.name.c_str();
+      const auto listname = kvlist.name.c_str();
 
       if (strncmp(listname, hname, strlen(hname)) == 0)
         {
           for (const auto &kv : kvlist)
             {
-              const char *ename = kv.key.c_str();
-              const char *evalue = kv.values[0].c_str();
-              size_t len = strlen(ename);
+              const auto ename = kv.key.c_str();
+              const auto evalue = kv.values[0].c_str();
+              const size_t len = strlen(ename);
 
               if (strncmp("missing_value", ename, len) == 0)
                 {
@@ -80,7 +78,7 @@ conv_cmor_table(const PMList &pmlist)
             {
               const char *ename = kv.key.c_str();
               const char *evalue = kv.values[0].c_str();
-              int len = strlen(ename);
+              const int len = strlen(ename);
               int vlen = strlen(evalue);
 
               if (vlen > 1 && evalue[0] == '"' && evalue[vlen - 1] == '"')
@@ -113,7 +111,7 @@ conv_cmor_table(const PMList &pmlist)
                     printf("  %-15s = \"%.*s\"\n", ename, vlen, ovalue);
                 }
 
-              Free(ovalue);
+              free(ovalue);
             }
           if (hasmissval) printf("  %-15s = %g\n", "missing_value", missval);
           printf("/\n");
