@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "timer.h"
 #include "util_files.h"
 #include "progress.h"
@@ -38,47 +40,51 @@ Cat(void *process)
   CdoStreamID streamID2;
   int vlistID2 = CDI_UNDEFID;
   int taxisID2 = CDI_UNDEFID;
-  size_t nmiss;
   double tw0 = 0, tw = 0;
-  std::vector<double> array;
+  Field field;
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  operatorCheckArgc(0);
 
-  const int timer_cat = timer_new("cat");
+  const auto lcopy = unchangedRecord();
+
+  const auto timer_cat = timer_new("cat");
   if (Options::Timer) timer_start(timer_cat);
 
-  const int streamCnt = cdoStreamCnt();
-  const int nfiles = streamCnt - 1;
+  const auto streamCnt = cdoStreamCnt();
+  const auto nfiles = streamCnt - 1;
 
-  Progress::init();
+  progress::init();
 
   for (int indf = 0; indf < nfiles; ++indf)
     {
       if (Options::cdoVerbose) cdoPrint("Process file: %s", cdoGetStreamName(indf));
       if (Options::Timer) tw0 = timer_val(timer_cat);
 
-      CdoStreamID streamID1 = cdoOpenRead((indf));
-      const int vlistID1 = cdoStreamInqVlist(streamID1);
-      const int taxisID1 = vlistInqTaxis(vlistID1);
+      const auto streamID1 = cdoOpenRead(indf);
+      const auto vlistID1 = cdoStreamInqVlist(streamID1);
+      const auto taxisID1 = vlistInqTaxis(vlistID1);
+
+      VarList varList1;
+      varListInit(varList1, vlistID1);
 
       if (indf == 0)
         {
-          int ntsteps = vlistNtsteps(vlistID1);
-          const int nvars = vlistNvars(vlistID1);
+          const auto nvars = vlistNvars(vlistID1);
+
+          auto ntsteps = vlistNtsteps(vlistID1);
           if (ntsteps == 1)
             {
               for (varID = 0; varID < nvars; ++varID)
-                if (vlistInqVarTimetype(vlistID1, varID) != TIME_CONSTANT) break;
-
+                if (varList1[varID].timetype != TIME_CONSTANT) break;
               if (varID == nvars) ntsteps = 0;
             }
 
           const bool file_exists = !Options::cdoOverwriteMode ? fileExists(cdoGetStreamName(nfiles)) : false;
           if (file_exists)
             {
-              streamID2 = cdoOpenAppend((nfiles));
+              streamID2 = cdoOpenAppend(nfiles);
 
               vlistID2 = cdoStreamInqVlist(streamID2);
               taxisID2 = vlistInqTaxis(vlistID2);
@@ -94,7 +100,7 @@ Cat(void *process)
             {
               if (Options::cdoVerbose) cdoPrint("Output file doesn't exist, creating: %s", cdoGetStreamName(nfiles));
 
-              streamID2 = cdoOpenWrite((nfiles));
+              streamID2 = cdoOpenWrite(nfiles);
 
               vlistID2 = vlistDuplicate(vlistID1);
               taxisID2 = taxisDuplicate(taxisID1);
@@ -108,26 +114,19 @@ Cat(void *process)
 
               cdoDefVlist(streamID2, vlistID2);
             }
-
-          if (!lcopy)
-            {
-              size_t gridsizemax = vlistGridsizeMax(vlistID1);
-              if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-              array.resize(gridsizemax);
-            }
         }
       else
         {
           vlistCompare(vlistID1, vlistID2, CMP_ALL);
         }
 
-      const int ntsteps = vlistNtsteps(vlistID1);
+      const auto ntsteps = vlistNtsteps(vlistID1);
 
       int tsID1 = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
         {
           const double fstatus = (ntsteps > 1) ? indf + (tsID1 + 1.) / ntsteps : indf + 1.;
-          if (!Options::cdoVerbose) Progress::update(0, 1, fstatus / nfiles);
+          if (!Options::cdoVerbose) progress::update(0, 1, fstatus / nfiles);
 
           taxisCopyTimestep(taxisID2, taxisID1);
           cdoDefTimestep(streamID2, tsID2);
@@ -137,7 +136,7 @@ Cat(void *process)
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (lconstvars && tsID2 > 0 && tsID1 == 0)
-                if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
+                if (varList1[varID].timetype == TIME_CONSTANT) continue;
 
               cdoDefRecord(streamID2, varID, levelID);
 
@@ -147,8 +146,9 @@ Cat(void *process)
                 }
               else
                 {
-                  cdoReadRecord(streamID1, &array[0], &nmiss);
-                  cdoWriteRecord(streamID2, &array[0], nmiss);
+                  field.init(varList1[varID]);
+                  cdoReadRecord(streamID1, field);
+                  cdoWriteRecord(streamID2, field);
                 }
             }
 
@@ -168,5 +168,5 @@ Cat(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
