@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
 #include "process_int.h"
 #include "cdi_lockedIO.h"
 
-
 void *
 Change_e5slm(void *process)
 {
@@ -71,9 +70,9 @@ Change_e5slm(void *process)
 
   if (nmiss) cdoAbort("SLM with missing values are unsupported!");
 
-  const auto minmax = varrayMinMax(cland);
-  if (minmax.first < 0 || minmax.second > 1)
-    cdoWarning("Values of SLM out of bounds! (minval=%g, maxval=%g)", minmax.first, minmax.second);
+  const auto mm = varrayMinMax(cland);
+  if (mm.min < 0 || mm.max > 1)
+    cdoWarning("Values of SLM out of bounds! (minval=%g, maxval=%g)", mm.min, mm.max);
 
   streamClose(streamIDslm);
 
@@ -92,18 +91,18 @@ Change_e5slm(void *process)
       if (code < 0)
         {
           // clang-format off
-          if      (cstrIsEqual(name, "SLM")      ) code = 172;
-          else if (cstrIsEqual(name, "ALAKE")    ) code = 99;
-          else if (cstrIsEqual(name, "WS")       ) code = 140;
-          else if (cstrIsEqual(name, "AZ0")      ) code = 173;
-          else if (cstrIsEqual(name, "ALB")      ) code = 174;
-          else if (cstrIsEqual(name, "VGRAT")    ) code = 198;
-          else if (cstrIsEqual(name, "FOREST")   ) code = 212;
-          else if (cstrIsEqual(name, "FAO")      ) code = 226;
-          else if (cstrIsEqual(name, "WSMX")     ) code = 229;
-          else if (cstrIsEqual(name, "GLAC")     ) code = 232;
-          else if (cstrIsEqual(name, "VLTCLIM")  ) code = 71;
-          else if (cstrIsEqual(name, "VGRATCLIM")) code = 70;
+          if      (cdo_cmpstr(name, "SLM")      ) code = 172;
+          else if (cdo_cmpstr(name, "ALAKE")    ) code = 99;
+          else if (cdo_cmpstr(name, "WS")       ) code = 140;
+          else if (cdo_cmpstr(name, "AZ0")      ) code = 173;
+          else if (cdo_cmpstr(name, "ALB")      ) code = 174;
+          else if (cdo_cmpstr(name, "VGRAT")    ) code = 198;
+          else if (cdo_cmpstr(name, "FOREST")   ) code = 212;
+          else if (cdo_cmpstr(name, "FAO")      ) code = 226;
+          else if (cdo_cmpstr(name, "WSMX")     ) code = 229;
+          else if (cdo_cmpstr(name, "GLAC")     ) code = 232;
+          else if (cdo_cmpstr(name, "VLTCLIM")  ) code = 71;
+          else if (cdo_cmpstr(name, "VGRATCLIM")) code = 70;
           // clang-format on
         }
 
