@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,77 +15,58 @@
   GNU General Public License for more details.
 */
 
+#include <algorithm> // sort
+
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_rlimit.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
-
 #include <mpim_grid.h>
 #include "util_files.h"
+#include "cdo_options.h"
+#include "cdi_lockedIO.h"
 
 static int globalGridType = CDI_UNDEFID;
 
-struct ensfileType
+struct EnsfileType
 {
   CdoStreamID streamID;
   int vlistID;
-  int gridID;
-  size_t nmiss;
   size_t gridsize;
   std::vector<long> gridindex;
-  std::vector<double> array;
+  VarList varList;
+  Field field;
 };
 
 struct xyinfoType
 {
-  double x, y;
-  int id;
+  double x = 0, y = 0;
+  int id = -1;
 };
 
-static int
-cmpx(const void *a, const void *b)
+static bool
+cmpx(const xyinfoType &a, const xyinfoType &b)
 {
-  const double x = ((const xyinfoType *) a)->x;
-  const double y = ((const xyinfoType *) b)->x;
-  return ((x > y) - (x < y)) * 2 + (x > y) - (x < y);
+  return a.x < b.x;
+}
+
+static bool
+cmpxy_lt(const xyinfoType &a, const xyinfoType &b)
+{
+  return (a.y < b.y || (std::fabs(a.y - b.y) <= 0 && a.x < b.x));
+}
+
+static bool
+cmpxy_gt(const xyinfoType &a, const xyinfoType &b)
+{
+  return (a.y > b.y || (std::fabs(a.y - b.y) <= 0 && a.x < b.x));
 }
 
 static int
-cmpxy_lt(const void *a, const void *b)
-{
-  const double x1 = ((const xyinfoType *) a)->x;
-  const double x2 = ((const xyinfoType *) b)->x;
-  const double y1 = ((const xyinfoType *) a)->y;
-  const double y2 = ((const xyinfoType *) b)->y;
-
-  int cmp = 0;
-  if (y1 < y2 || (std::fabs(y1 - y2) <= 0 && x1 < x2))
-    cmp = -1;
-  else if (y1 > y2 || (std::fabs(y1 - y2) <= 0 && x1 > x2))
-    cmp = 1;
-
-  return cmp;
-}
-
-static int
-cmpxy_gt(const void *a, const void *b)
-{
-  const double x1 = ((const xyinfoType *) a)->x;
-  const double x2 = ((const xyinfoType *) b)->x;
-  const double y1 = ((const xyinfoType *) a)->y;
-  const double y2 = ((const xyinfoType *) b)->y;
-
-  int cmp = 0;
-  if (y1 > y2 || (std::fabs(y1 - y2) <= 0 && x1 < x2))
-    cmp = -1;
-  else if (y1 < y2 || (std::fabs(y1 - y2) <= 0 && x1 > x2))
-    cmp = 1;
-
-  return cmp;
-}
-
-static int
-genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int igrid, long nxblocks)
+genGrid(int ngrids, int nfiles, std::vector<EnsfileType> &ef, bool ginit, int igrid, long nxblocks)
 {
   bool lsouthnorth = true;
   bool lregular = false;
@@ -93,28 +74,25 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
   bool lunstructured = false;
   int gridID2 = -1;
 
-  long nx = -1;
-  if (nxblocks != -1) nx = nxblocks;
+  long nx = (nxblocks != -1) ? nxblocks : -1;
 
-  int gridID = vlistGrid(ef[0].vlistID, igrid);
-  int gridtype = (globalGridType != CDI_UNDEFID) ? globalGridType : gridInqType(gridID);
-  if (ngrids > 1 && gridtype == GRID_GENERIC && gridInqXsize(gridID) == 0 && gridInqYsize(gridID) == 0) return gridID2;
-  if (gridtype == GRID_UNSTRUCTURED) lunstructured = true;
+  auto gridID = vlistGrid(ef[0].vlistID, igrid);
+  auto gridtype0 = (globalGridType != CDI_UNDEFID) ? globalGridType : gridInqType(gridID);
+  if (ngrids > 1 && gridtype0 == GRID_GENERIC && gridInqXsize(gridID) == 0 && gridInqYsize(gridID) == 0) return gridID2;
+  if (gridtype0 == GRID_UNSTRUCTURED) lunstructured = true;
   const int nv = lunstructured ? gridInqNvertex(gridID) : 0;
-  const bool lcenter = globalGridType == CDI_UNDEFID && gridInqXvals(gridID, nullptr) > 0 && gridInqYvals(gridID, nullptr) > 0;
-  const bool lbounds = lunstructured && globalGridType == CDI_UNDEFID && gridInqXbounds(gridID, nullptr) > 0
-                       && gridInqYbounds(gridID, nullptr) > 0;
+  const bool lcenter = globalGridType == CDI_UNDEFID && gridHasCoordinates(gridID);
+  const bool lbounds = lunstructured && globalGridType == CDI_UNDEFID && gridHasBounds(gridID);
 
-  std::vector<long> xsize(nfiles);
-  std::vector<long> ysize(nfiles);
-  xyinfoType *xyinfo = (xyinfoType *) Malloc(nfiles * sizeof(xyinfoType));
-  std::vector<std::vector<double>> xvals(nfiles), yvals(nfiles);
-  std::vector<std::vector<double>> xbounds(nfiles), ybounds(nfiles);
+  std::vector<xyinfoType> xyinfo(nfiles);
+  std::vector<long> xsize(nfiles), ysize(nfiles);
+  Varray2D<double> xvals(nfiles), yvals(nfiles);
+  Varray2D<double> xbounds(nfiles), ybounds(nfiles);
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
       gridID = vlistGrid(ef[fileID].vlistID, igrid);
-      const int gridtype = (globalGridType != CDI_UNDEFID) ? globalGridType : gridInqType(gridID);
+      const auto gridtype = (globalGridType != CDI_UNDEFID) ? globalGridType : gridInqType(gridID);
       if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)
         lregular = true;
       else if (gridtype == GRID_CURVILINEAR)
@@ -165,23 +143,17 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
         }
       // printf("fileID %d, gridID %d\n", fileID, gridID);
 
+      xyinfo[fileID].id = fileID;
       if (lregular)
         {
           xyinfo[fileID].x = xvals[fileID][0];
           xyinfo[fileID].y = yvals[fileID][0];
-          xyinfo[fileID].id = fileID;
 
           if (ysize[fileID] > 1)
             {
               if (yvals[fileID][0] > yvals[fileID][ysize[fileID] - 1]) lsouthnorth = false;
             }
         }
-      else
-        {
-          xyinfo[fileID].x = 0;
-          xyinfo[fileID].y = 0;
-          xyinfo[fileID].id = fileID;
-        }
     }
 
   if (Options::cdoVerbose && lregular)
@@ -189,16 +161,16 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
 
   if (lregular)
     {
-      qsort(xyinfo, nfiles, sizeof(xyinfoType), cmpx);
+      std::sort(xyinfo.begin(), xyinfo.end(), cmpx);
 
       if (Options::cdoVerbose)
         for (int fileID = 0; fileID < nfiles; fileID++)
           printf("2 %d %g %g \n", xyinfo[fileID].id, xyinfo[fileID].x, xyinfo[fileID].y);
 
       if (lsouthnorth)
-        qsort(xyinfo, nfiles, sizeof(xyinfoType), cmpxy_lt);
+        std::sort(xyinfo.begin(), xyinfo.end(), cmpxy_lt);
       else
-        qsort(xyinfo, nfiles, sizeof(xyinfoType), cmpxy_gt);
+        std::sort(xyinfo.begin(), xyinfo.end(), cmpxy_gt);
 
       if (Options::cdoVerbose)
         for (int fileID = 0; fileID < nfiles; fileID++)
@@ -239,8 +211,8 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
       if (ysize[xyinfo[i].id] != ys) cdoAbort("ysize=%ld differ from first file (ysize=%ld)!", ysize[xyinfo[i].id], ys);
   }
 
-  std::vector<double> xvals2, yvals2;
-  std::vector<double> xbounds2, ybounds2;
+  Varray<double> xvals2, yvals2;
+  Varray<double> xbounds2, ybounds2;
   if (lregular)
     {
       xvals2.resize(xsize2);
@@ -260,8 +232,7 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
         }
     }
 
-  std::vector<long> xoff(nx + 1);
-  std::vector<long> yoff(ny + 1);
+  std::vector<long> xoff(nx + 1), yoff(ny + 1);
 
   xoff[0] = 0;
   for (long i = 0; i < nx; ++i)
@@ -315,7 +286,7 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
         }
     }
 
-  gridID2 = gridCreate(gridtype, xsize2 * ysize2);
+  gridID2 = gridCreate(gridtype0, xsize2 * ysize2);
   if (!lunstructured)
     {
       gridDefXsize(gridID2, xsize2);
@@ -340,13 +311,11 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
         }
     }
 
-  Free(xyinfo);
-
   gridID = vlistGrid(ef[0].vlistID, igrid);
 
-  grid_copy_attributes(gridID, gridID2);
+  grid_copy_keys(gridID, gridID2);
 
-  if (gridtype == GRID_PROJECTION) grid_copy_mapping(gridID, gridID2);
+  if (gridtype0 == GRID_PROJECTION) grid_copy_mapping(gridID, gridID2);
 
   return gridID2;
 }
@@ -360,38 +329,40 @@ Collgrid(void *process)
 
   cdoInitialize(process);
 
-  const int nfiles = cdoStreamCnt() - 1;
-  const char *ofilename = cdoGetStreamName(nfiles);
+  const auto nfiles = cdoStreamCnt() - 1;
+  const auto ofilename = cdoGetStreamName(nfiles);
 
   if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
     cdoAbort("Outputfile %s already exists!", ofilename);
 
-  std::vector<ensfileType> ef(nfiles);
+  std::vector<EnsfileType> ef(nfiles);
+
+  cdo::set_numfiles(nfiles + 8);
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      ef[fileID].streamID = cdoOpenRead((fileID));
+      ef[fileID].streamID = cdoOpenRead(fileID);
       ef[fileID].vlistID = cdoStreamInqVlist(ef[fileID].streamID);
+      varListInit(ef[fileID].varList, ef[fileID].vlistID);
     }
 
-  const int vlistID1 = ef[0].vlistID;
+  const auto &varList1 = ef[0].varList;
+  const auto vlistID1 = ef[0].vlistID;
   vlistClearFlag(vlistID1);
 
-  /* check that the contents is always the same */
+  // check that the contents is always the same
   for (int fileID = 1; fileID < nfiles; fileID++) vlistCompare(vlistID1, ef[fileID].vlistID, CMP_NAME | CMP_NLEVEL);
 
-  const int nvars = vlistNvars(vlistID1);
-  std::vector<bool> vars(nvars);
-  for (varID = 0; varID < nvars; varID++) vars[varID] = false;
-  std::vector<bool> vars1(nvars);
-  for (varID = 0; varID < nvars; varID++) vars1[varID] = false;
+  const auto nvars = vlistNvars(vlistID1);
+  std::vector<bool> vars(nvars, false), vars1(nvars, false);
 
-  int nsel = operatorArgc();
+  auto nsel = operatorArgc();
   int noff = 0;
 
   if (nsel > 0)
     {
-      if (strcmp(operatorArgv()[0], "gridtype=unstructured") == 0)
+      auto argument = cdoOperatorArgv(0).c_str();
+      if (strcmp(argument, "gridtype=unstructured") == 0)
         {
           nsel--;
           noff++;
@@ -399,15 +370,15 @@ Collgrid(void *process)
         }
       else
         {
-          int len = (int) strlen(operatorArgv()[0]);
-          while (--len >= 0 && isdigit(operatorArgv()[0][len]))
+          int len = (int) strlen(argument);
+          while (--len >= 0 && isdigit(argument[len]))
             ;
 
           if (len == -1)
             {
               nsel--;
               noff++;
-              nxblocks = parameter2int(operatorArgv()[0]);
+              nxblocks = parameter2int(argument);
             }
         }
     }
@@ -418,22 +389,17 @@ Collgrid(void *process)
     }
   else
     {
-      char **argnames = operatorArgv() + noff;
-
       if (Options::cdoVerbose)
-        for (int i = 0; i < nsel; i++) fprintf(stderr, "name %d = %s\n", i + 1, argnames[i]);
+        for (int i = 0; i < nsel; i++) cdoPrint("name %d = %s\n", i + 1, cdoOperatorArgv(noff + i).c_str());
 
       std::vector<bool> selfound(nsel);
       for (int i = 0; i < nsel; i++) selfound[i] = false;
 
-      char varname[CDI_MAX_NAME];
       for (varID = 0; varID < nvars; varID++)
         {
-          vlistInqVarName(vlistID1, varID, varname);
-
           for (int isel = 0; isel < nsel; isel++)
             {
-              if (strcmp(argnames[isel], varname) == 0)
+              if (cdoOperatorArgv(noff + isel) == varList1[varID].name)
                 {
                   selfound[isel] = true;
                   vars1[varID] = true;
@@ -441,42 +407,43 @@ Collgrid(void *process)
             }
         }
 
+      int err = 0;
       for (int isel = 0; isel < nsel; isel++)
-        if (selfound[isel] == false) cdoAbort("Variable name %s not found!", argnames[isel]);
+        {
+          if (selfound[isel] == false)
+            {
+              err++;
+              cdoWarning("Variable name %s not found!", cdoOperatorArgv(noff + isel).c_str());
+            }
+        }
+      if (err) cdoAbort("Could not find all requested variables: (%d/%d)", nsel - err, nsel);
     }
 
   for (varID = 0; varID < nvars; varID++)
     {
       if (vars1[varID])
         {
-          const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          const int nlevs = zaxisInqSize(zaxisID);
-          for (int levID = 0; levID < nlevs; levID++) vlistDefFlag(vlistID1, varID, levID, true);
+          const auto nlevels = varList1[varID].nlevels;
+          for (int levID = 0; levID < nlevels; levID++) vlistDefFlag(vlistID1, varID, levID, true);
         }
     }
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      const size_t gridsize = vlistGridsizeMax(ef[fileID].vlistID);
+      const auto gridsize = vlistGridsizeMax(ef[fileID].vlistID);
       ef[fileID].gridsize = gridsize;
       ef[fileID].gridindex.resize(gridsize);
-      ef[fileID].array.resize(gridsize);
     }
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
-  /*
-  if ( Options::cdoVerbose )
-    {
-      vlistPrint(vlistID1);
-      vlistPrint(vlistID2);
-    }
-  */
-  // int vlistID2 = vlistDuplicate(vlistID1);
-  const int nvars2 = vlistNvars(vlistID2);
+  // if ( Options::cdoVerbose ) vlistPrint(vlistID1);
+  // if ( Options::cdoVerbose ) vlistPrint(vlistID2);
+  // auto vlistID2 = vlistDuplicate(vlistID1);
+  const auto nvars2 = vlistNvars(vlistID2);
 
-  const int ngrids1 = vlistNgrids(vlistID1);
-  const int ngrids2 = vlistNgrids(vlistID2);
+  const auto ngrids1 = vlistNgrids(vlistID1);
+  const auto ngrids2 = vlistNgrids(vlistID2);
 
   std::vector<int> gridIDs(ngrids2);
 
@@ -487,8 +454,6 @@ Collgrid(void *process)
       for (i1 = 0; i1 < ngrids1; ++i1)
         if (vlistGrid(vlistID1, i1) == vlistGrid(vlistID2, i2)) break;
 
-      //   printf("i1 %d i2 %d\n", i1, i2);
-
       if (!ginit)
         {
           gridIDs[i2] = genGrid(ngrids2, nfiles, ef, ginit, i1, nxblocks);
@@ -498,8 +463,8 @@ Collgrid(void *process)
         gridIDs[i2] = genGrid(ngrids2, nfiles, ef, ginit, i1, nxblocks);
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   size_t gridsize2 = 0;
@@ -513,11 +478,13 @@ Collgrid(void *process)
           vlistChangeGridIndex(vlistID2, i, gridIDs[i]);
         }
     }
+  
+  VarList varList2;
+  varListInit(varList2, vlistID2);
 
   for (varID = 0; varID < nvars2; varID++)
     {
-      const int gridID = vlistInqVarGrid(vlistID2, varID);
-
+      const auto gridID = varList2[varID].gridID;
       for (int i = 0; i < ngrids2; ++i)
         {
           if (gridIDs[i] != -1)
@@ -528,11 +495,10 @@ Collgrid(void *process)
         }
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite((nfiles));
+  const auto streamID2 = cdoOpenWrite(nfiles);
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> array2;
-  if (gridsize2 > 0) array2.resize(gridsize2);
+  Field field2;
 
   int tsID = 0;
   do
@@ -540,7 +506,7 @@ Collgrid(void *process)
       nrecs0 = cdoStreamInqTimestep(ef[0].streamID, tsID);
       for (int fileID = 1; fileID < nfiles; fileID++)
         {
-          const int nrecs = cdoStreamInqTimestep(ef[fileID].streamID, tsID);
+          const auto nrecs = cdoStreamInqTimestep(ef[fileID].streamID, tsID);
           if (nrecs != nrecs0)
             cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(0),
                      cdoGetStreamName(fileID));
@@ -563,24 +529,35 @@ Collgrid(void *process)
 
           if (vlistInqFlag(vlistID1, varID, levelID) == true)
             {
-              const int varID2 = vlistFindVar(vlistID2, varID);
-              const int levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+              const auto varID2 = vlistFindVar(vlistID2, varID);
+              const auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
               if (Options::cdoVerbose && tsID == 0) printf("varID %d %d levelID %d %d\n", varID, varID2, levelID, levelID2);
 
-              const double missval = vlistInqVarMissval(vlistID2, varID2);
-              for (size_t i = 0; i < gridsize2; i++) array2[i] = missval;
+              field2.init(varList2[varID2]);
+
+              const auto missval = varList2[varID2].missval;
+              if (field2.memType == MemType::Float)
+                for (size_t i = 0; i < gridsize2; i++) field2.vec_f[i] = missval;
+              else
+                for (size_t i = 0; i < gridsize2; i++) field2.vec_d[i] = missval;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
               for (int fileID = 0; fileID < nfiles; fileID++)
                 {
-                  cdoReadRecord(ef[fileID].streamID, &ef[fileID].array[0], &ef[fileID].nmiss);
+                  auto &field1 = ef[fileID].field;
+                  field1.init(ef[fileID].varList[varID]);
+                  cdoReadRecord(ef[fileID].streamID, field1);
 
                   if (vars[varID2])
                     {
-                      const size_t gridsize = ef[fileID].gridsize;
-                      for (size_t i = 0; i < gridsize; ++i) array2[ef[fileID].gridindex[i]] = ef[fileID].array[i];
+                      const auto gridsize = ef[fileID].gridsize;
+                      const auto &gindex = ef[fileID].gridindex;
+                      if (field1.memType == MemType::Float)
+                        for (size_t i = 0; i < gridsize; ++i) field2.vec_f[gindex[i]] = field1.vec_f[i];
+                      else
+                        for (size_t i = 0; i < gridsize; ++i) field2.vec_d[gindex[i]] = field1.vec_d[i];
                     }
                 }
 
@@ -588,11 +565,11 @@ Collgrid(void *process)
 
               if (vars[varID2])
                 {
-                  const size_t nmiss = arrayNumMV(gridsize2, &array2[0], missval);
-                  cdoWriteRecord(streamID2, &array2[0], nmiss);
+                  fieldNumMV(field2);
+                  cdoWriteRecord(streamID2, field2);
                 }
               else
-                cdoWriteRecord(streamID2, &ef[0].array[0], ef[0].nmiss);
+                cdoWriteRecord(streamID2, ef[0].field);
             }
         }
 
