@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
 
 void *
@@ -40,25 +41,25 @@ Compc(void *process)
 
   cdoInitialize(process);
 
-  const int EQC = cdoOperatorAdd("eqc", 0, 0, nullptr);
-  const int NEC = cdoOperatorAdd("nec", 0, 0, nullptr);
-  const int LEC = cdoOperatorAdd("lec", 0, 0, nullptr);
-  const int LTC = cdoOperatorAdd("ltc", 0, 0, nullptr);
-  const int GEC = cdoOperatorAdd("gec", 0, 0, nullptr);
-  const int GTC = cdoOperatorAdd("gtc", 0, 0, nullptr);
+  const auto EQC = cdoOperatorAdd("eqc", 0, 0, nullptr);
+  const auto NEC = cdoOperatorAdd("nec", 0, 0, nullptr);
+  const auto LEC = cdoOperatorAdd("lec", 0, 0, nullptr);
+  const auto LTC = cdoOperatorAdd("ltc", 0, 0, nullptr);
+  const auto GEC = cdoOperatorAdd("gec", 0, 0, nullptr);
+  const auto GTC = cdoOperatorAdd("gtc", 0, 0, nullptr);
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg("constant value");
-  const double rc = parameter2double(operatorArgv()[0]);
+  const auto rc = parameter2double(cdoOperatorArgv(0));
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   nospec(vlistID1);
@@ -66,11 +67,10 @@ Compc(void *process)
   VarList varList;
   varListInit(varList, vlistID1);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax), array2(gridsizemax);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -84,13 +84,15 @@ Compc(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          const double missval = varList[varID].missval;
-          const size_t gridsize = varList[varID].gridsize;
-          const int datatype = varList[varID].datatype;
+          const auto missval = varList[varID].missval;
+          const auto gridsize = varList[varID].gridsize;
+          const auto datatype = varList[varID].datatype;
           const double rcv = (datatype == CDI_DATATYPE_FLT32) ? (float) rc : rc;
 
           const bool rc_is_missval = DBL_IS_EQUAL(rc, missval);
 
+          if (nmiss > 0) cdo_check_missval(missval, varList[varID].name);
+
           if (operatorID == EQC)
             {
               for (size_t i = 0; i < gridsize; i++)
@@ -126,7 +128,7 @@ Compc(void *process)
               cdoAbort("Operator not implemented!");
             }
 
-          nmiss = arrayNumMV(gridsize, array2.data(), missval);
+          nmiss = varrayNumMV(gridsize, array2, missval);
           cdoDefRecord(streamID2, varID, levelID);
           cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
