@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,19 +30,16 @@
 #include "cdo_vlist.h"
 #include "cdo_fill.h"
 
-
 static void
 operatorIFTHEN(size_t n, double mv1, double mv2, const Varray<double> &a1, const Varray<double> &a2, Varray<double> &a3)
 {
-  for (size_t i = 0; i < n; i++)
-    a3[i] = !DBL_IS_EQUAL(a1[i], mv1) && !DBL_IS_EQUAL(a1[i], 0.) ? a2[i] : mv2;
+  for (size_t i = 0; i < n; i++) a3[i] = !DBL_IS_EQUAL(a1[i], mv1) && !DBL_IS_EQUAL(a1[i], 0.) ? a2[i] : mv2;
 }
 
 static void
 operatorIFNOTTHEN(size_t n, double mv1, double mv2, const Varray<double> &a1, const Varray<double> &a2, Varray<double> &a3)
 {
-  for (size_t i = 0; i < n; i++)
-    a3[i] = !DBL_IS_EQUAL(a1[i], mv1) && DBL_IS_EQUAL(a1[i], 0.) ? a2[i] : mv2;
+  for (size_t i = 0; i < n; i++) a3[i] = !DBL_IS_EQUAL(a1[i], mv1) && DBL_IS_EQUAL(a1[i], 0.) ? a2[i] : mv2;
 }
 
 void *
@@ -71,6 +68,8 @@ Cond(void *process)
 
   const auto operatorID = cdoOperatorID();
 
+  operatorCheckArgc(0);
+
   const auto streamID1 = cdoOpenRead(0);
   const auto streamID2 = cdoOpenRead(1);
 
@@ -168,6 +167,8 @@ Cond(void *process)
           const auto missval2 = varList2[varID].missval;
           if (recID == 0 || filltype != FILL_REC) missval1 = varList1[varID].missval;
 
+          if (nmiss1 > 0) cdo_check_missval(missval1, varList1[varID].name);
+
           // clang-format off
           if      (operatorID == IFTHEN)    operatorIFTHEN(gridsize, missval1, missval2, array1, array2, array3);
           else if (operatorID == IFNOTTHEN) operatorIFNOTTHEN(gridsize, missval1, missval2, array1, array2, array3);
