@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,6 @@
 #include "cdo_vlist.h"
 #include "cdo_fill.h"
 
-
 void *
 Cond2(void *process)
 {
@@ -45,13 +44,15 @@ Cond2(void *process)
   int varID, levelID;
   double missval1 = -9.E33;
   size_t nmiss1 = 0;
-  std::vector<std::vector<size_t>> varnmiss1;
+  Varray2D<size_t> varnmiss1;
   Varray2D<double> vardata1;
 
   cdoInitialize(process);
 
   cdoOperatorAdd("ifthenelse", 0, 0, nullptr);
 
+  operatorCheckArgc(0);
+
   const auto streamID1 = cdoOpenRead(0);
   const auto streamID2 = cdoOpenRead(1);
   const auto streamID3 = cdoOpenRead(2);
@@ -87,12 +88,16 @@ Cond2(void *process)
   const auto streamID4 = cdoOpenWrite(3);
   cdoDefVlist(streamID4, vlistID4);
 
-  auto gridsize = vlistGridsizeMax(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
 
-  if (filltype == FILL_REC && gridsize != gridInqSize(vlistGrid(vlistID1, 0)))
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+
+  if (filltype == FILL_REC && gridsizemax != gridInqSize(vlistGrid(vlistID1, 0)))
     cdoAbort("Stream1 >%s< has wrong gridsize!", cdoGetStreamName(0));
 
-  Varray<double> array1(gridsize), array2(gridsize), array3(gridsize), array4(gridsize);
+  Varray<double> array1(gridsizemax), array2(gridsizemax), array3(gridsizemax), array4(gridsizemax);
 
   if (Options::cdoVerbose)
     cdoPrint("Number of timesteps: file1 %d, file2 %d, file3 %d", ntsteps1, ntsteps2, vlistNtsteps(vlistID3));
@@ -142,23 +147,25 @@ Cond2(void *process)
 
               if (filltype == FILL_TS)
                 {
-                  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-                  size_t offset = gridsize * levelID;
+                  const auto gridsize = varList1[varID].gridsize;
+                  const auto offset = gridsize * levelID;
                   arrayCopy(gridsize, &array1[0], &vardata1[varID][offset]);
                   varnmiss1[varID][levelID] = nmiss1;
                 }
             }
           else if (filltype == FILL_TS)
             {
-              gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-              size_t offset = gridsize * levelID;
+              const auto gridsize = varList1[varID].gridsize;
+              const auto offset = gridsize * levelID;
               arrayCopy(gridsize, &vardata1[varID][offset], &array1[0]);
               nmiss1 = varnmiss1[varID][levelID];
             }
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-          const auto missval2 = vlistInqVarMissval(vlistID2, varID);
-          if (recID == 0 || filltype != FILL_REC) missval1 = vlistInqVarMissval(vlistID1, varID);
+          const auto gridsize = varList2[varID].gridsize;
+          const auto missval2 = varList2[varID].missval;
+          if (recID == 0 || filltype != FILL_REC) missval1 = varList1[varID].missval;
+
+          if (nmiss1 > 0) cdo_check_missval(missval1, varList1[varID].name);
 
           for (size_t i = 0; i < gridsize; i++)
             array4[i] = DBL_IS_EQUAL(array1[i], missval1) ? missval2 : !DBL_IS_EQUAL(array1[i], 0.) ? array2[i] : array3[i];
