@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -36,7 +36,6 @@
 #include "cdo_vlist.h"
 #include "param_conversion.h"
 
-
 enum
 {
   CONSECSUM,
@@ -47,13 +46,12 @@ enum
 static void
 selEndOfPeriod(Field &periods, const Field &history, const Field &current, int isLastTimestep)
 {
-  size_t i;
   double pmissval = periods.missval;
-  auto &parray = periods.vec;
+  auto &parray = periods.vec_d;
   double hmissval = history.missval;
-  const auto &harray = history.vec;
+  const auto &harray = history.vec_d;
   double cmissval = current.missval;
-  const auto &carray = current.vec;
+  const auto &carray = current.vec_d;
 
   auto len = gridInqSize(periods.grid);
   if (len != gridInqSize(current.grid) || (gridInqSize(current.grid) != gridInqSize(history.grid)))
@@ -66,7 +64,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
-          for (i = 0; i < len; i++)
+          for (size_t i = 0; i < len; i++)
             {
               if (!DBL_IS_EQUAL(harray[i], hmissval))
                 {
@@ -90,7 +88,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
-          for (i = 0; i < len; i++)
+          for (size_t i = 0; i < len; i++)
             parray[i] = (DBL_IS_EQUAL(carray[i], 0.0) && IS_NOT_EQUAL(harray[i], 0.0)) ? harray[i] : pmissval;
         }
     }
@@ -101,7 +99,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
-          for (i = 0; i < len; i++)
+          for (size_t i = 0; i < len; i++)
             if (!DBL_IS_EQUAL(carray[i], cmissval))
               {
                 parray[i] = (DBL_IS_EQUAL(carray[i], 0.0)) ? pmissval : carray[i];
@@ -116,7 +114,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
-          for (i = 0; i < len; i++) parray[i] = DBL_IS_EQUAL(carray[i], 0.0) ? pmissval : carray[i];
+          for (size_t i = 0; i < len; i++) parray[i] = DBL_IS_EQUAL(carray[i], 0.0) ? pmissval : carray[i];
         }
     }
 
@@ -165,7 +163,7 @@ Consecstat(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      vlistDefVarUnits(ovlistID, varID, "steps"); /* TODO */
+      cdiDefKeyString(ovlistID, varID, CDI_KEY_UNITS, "steps"); // TODO
     }
 
   const auto ostreamID = cdoOpenWrite(1);
@@ -198,7 +196,7 @@ Consecstat(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(istreamID, &varID, &levelID);
-          cdoReadRecord(istreamID, field.vec.data(), &field.nmiss);
+          cdoReadRecord(istreamID, field.vec_d.data(), &field.nmiss);
           field.grid = varList[varID].gridID;
           field.size = varList[varID].gridsize;
           field.missval = varList[varID].missval;
@@ -209,20 +207,20 @@ Consecstat(void *process)
             {
             case CONSECSUM:
               cdoDefRecord(ostreamID, varID, levelID);
-              cdoWriteRecord(ostreamID, vars[varID][levelID].vec.data(), vars[varID][levelID].nmiss);
+              cdoWriteRecord(ostreamID, vars[varID][levelID].vec_d.data(), vars[varID][levelID].nmiss);
               break;
             case CONSECTS:
               if (itsID != 0)
                 {
                   selEndOfPeriod(periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], false);
                   cdoDefRecord(ostreamID, varID, levelID);
-                  cdoWriteRecord(ostreamID, periods[varID][levelID].vec.data(), periods[varID][levelID].nmiss);
+                  cdoWriteRecord(ostreamID, periods[varID][levelID].vec_d.data(), periods[varID][levelID].nmiss);
                 }
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
-              for (size_t i = 0; i < vars[varID][levelID].size; i++) hist[varID][levelID].vec[i] = vars[varID][levelID].vec[i];
+              for (size_t i = 0; i < vars[varID][levelID].size; i++) hist[varID][levelID].vec_d[i] = vars[varID][levelID].vec_d[i];
 #else
-              hist[varID][levelID].vec = vars[varID][levelID].vec;
+              hist[varID][levelID].vec_d = vars[varID][levelID].vec_d;
 #endif
               break;
             default: printf(SWITCHWARN, __func__); break;
@@ -248,7 +246,7 @@ Consecstat(void *process)
             {
               selEndOfPeriod(periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], true);
               cdoDefRecord(ostreamID, varID, levelID);
-              cdoWriteRecord(ostreamID, periods[varID][levelID].vec.data(), periods[varID][levelID].nmiss);
+              cdoWriteRecord(ostreamID, periods[varID][levelID].vec_d.data(), periods[varID][levelID].nmiss);
             }
         }
     }
