@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "datetime.h"
 
 void *
@@ -39,38 +39,40 @@ Deltat(void *process)
   cdoOperatorAdd("timederivative",    0,     1, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
   const bool ldivdt = cdoOperatorF2(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int calendar = taxisInqCalendar(taxisID1);
+  const auto calendar = taxisInqCalendar(taxisID1);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   FieldVector2D vars;
   fieldsFromVlist(vlistID1, vars, FIELD_VEC);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsizemax), array2(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax), array2(gridsizemax);
 
   int tsID = 0;
-  int nrecs = cdoStreamInqTimestep(streamID1, tsID);
+  auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
 
-  JulianDate juldate0 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
+  auto juldate0 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
 
   for (int recID = 0; recID < nrecs; ++recID)
     {
       cdoInqRecord(streamID1, &varID, &levelID);
-      cdoReadRecord(streamID1, vars[varID][levelID].vec.data(), &nmiss);
+      cdoReadRecord(streamID1, vars[varID][levelID].vec_d.data(), &nmiss);
       vars[varID][levelID].nmiss = nmiss;
     }
 
@@ -78,7 +80,7 @@ Deltat(void *process)
   int tsID2 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      const JulianDate juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
+      const auto juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
       const double idt_in_sec = ldivdt ? 1. / julianDateToSeconds(julianDateSub(juldate1, juldate0)) : 1;
       juldate0 = juldate1;
 
@@ -90,9 +92,9 @@ Deltat(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          const double missval = vars[varID][levelID].missval;
-          const size_t fieldsize = vars[varID][levelID].size;
-          auto &array0 = vars[varID][levelID].vec;
+          const auto missval = vars[varID][levelID].missval;
+          const auto fieldsize = vars[varID][levelID].size;
+          auto &array0 = vars[varID][levelID].vec_d;
           if (nmiss || vars[varID][levelID].nmiss)
             {
               for (size_t i = 0; i < fieldsize; ++i)
@@ -103,14 +105,14 @@ Deltat(void *process)
                     array2[i] = (array1[i] - array0[i]) * idt_in_sec;
                 }
 
-              nmiss = arrayNumMV(fieldsize, array2.data(), missval);
+              nmiss = varrayNumMV(fieldsize, array2, missval);
             }
           else
             {
               for (size_t i = 0; i < fieldsize; ++i) array2[i] = (array1[i] - array0[i]) * idt_in_sec;
             }
 
-          arrayCopy(fieldsize, array1.data(), array0.data());
+          varrayCopy(fieldsize, array1, array0);
 
           cdoDefRecord(streamID2, varID, levelID);
           cdoWriteRecord(streamID2, array2.data(), nmiss);
@@ -125,5 +127,5 @@ Deltat(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
