@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,9 +17,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "process_int.h"
 #include "util_string.h"
+#include "cdo_options.h"
 
 void *
 Deltime(void *process)
@@ -34,28 +34,29 @@ Deltime(void *process)
 
   cdoInitialize(process);
 
-  bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
   // clang-format off
-  int DELDAY   = cdoOperatorAdd("delday",   0, 0, nullptr);
-  int DEL29FEB = cdoOperatorAdd("del29feb", 0, 0, nullptr);
+  const auto DELDAY   = cdoOperatorAdd("delday",   0, 0, nullptr);
+  const auto DEL29FEB = cdoOperatorAdd("del29feb", 0, 0, nullptr);
   // clang-format on
 
-  UNUSED(DELDAY);
+  (void) (DELDAY);  // CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == DEL29FEB)
     {
       dday = 29;
       dmon = 2;
+      operatorCheckArgc(0);
     }
   else
     {
       int nsel = operatorArgc();
       if (nsel < 1) cdoAbort("Too few arguments!");
       if (nsel > 1) cdoAbort("Too many arguments!");
-      char *sarg = operatorArgv()[0];
+      const char *sarg = cdoOperatorArgv(0).c_str();
       dday = atoi(sarg);
       dmon = 0;
       while (isdigit(*sarg)) sarg++;
@@ -75,23 +76,23 @@ Deltime(void *process)
 
   if (Options::cdoVerbose) cdoPrint("delete day %d%s", dday, cmons[dmon]);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisDefCalendar(taxisID2, CALENDAR_365DAYS);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      const auto gridsizemax = vlistGridsizeMax(vlistID1);
       array.resize(gridsizemax);
     }
 
@@ -100,7 +101,7 @@ Deltime(void *process)
   int tsID2 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
       // vtime = taxisInqVtime(taxisID1);
 
       cdiDecodeDate(vdate, &year, &month, &day);
