@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,80 +17,80 @@
 /*
    This module contains the following operators:
 
-      Derivepar     gheight          geopotential height
+      Derivepar     gheight           Geopotential height
+      Derivepar     sealevelpressure  Sea level pressure
 */
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "after_vertint.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "vertical_interp.h"
 #include "stdnametable.h"
 #include "util_string.h"
 #include "const.h"
 #include "cdo_zaxis.h"
+#include "cdo_options.h"
 
 void MakeGeopotHeight(double *geop, double *gt, double *gq, double *ph, int nhor, int nlev);
 
-void vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, std::vector<double> &vct, int *rnhlevf);
+void vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, Varray<double> &vct, int *rnhlevf);
 
 void *
 Derivepar(void *process)
 {
   ModelMode mode(ModelMode::UNDEF);
-  int nrecs;
-  int i;
-  size_t offset;
   int varID, levelID;
-  int zaxisID;
-  int nlevel;
   int surfaceID = -1;
   int sgeopotID = -1, geopotID = -1, tempID = -1, humID = -1, psID = -1, lnpsID = -1, presID = -1, gheightID = -1;
   int lnpsID2 = -1;
   // int clwcID = -1, ciwcID = -1;
   char paramstr[32];
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
-  double *single2;
   // double *lwater = nullptr, *iwater = nullptr;
   size_t nmiss, nmissout = 0;
-  double minval, maxval;
   gribcode_t gribcodes;
-  memset(&gribcodes, 0, sizeof(gribcode_t));
 
   cdoInitialize(process);
 
   // clang-format off
-  const int GHEIGHT          = cdoOperatorAdd("gheight",            0, 0, nullptr);
-  const int SEALEVELPRESSURE = cdoOperatorAdd("sealevelpressure",   0, 0, nullptr);
+  const auto GHEIGHT          = cdoOperatorAdd("gheight",            0, 0, nullptr);
+  const auto SEALEVELPRESSURE = cdoOperatorAdd("sealevelpressure",   0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  int gridID = vlistGrid(vlistID1, 0);
-  if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
+  auto gridID0 = vlistGrid(vlistID1, 0);
+  if (gridInqType(gridID0) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  auto gridsize = vlist_check_gridsize(vlistID1);
 
   int zaxisIDh = -1;
   int nvct = 0;
   int nhlevf = 0;
-  std::vector<double> vct;
+  Varray<double> vct;
   vlist_hybrid_vct(vlistID1, &zaxisIDh, &nvct, vct, &nhlevf);
 
   if (Options::cdoVerbose)
-    for (i = 0; i < nvct / 2; ++i) cdoPrint("vct: %5d %25.17f %25.17f", i, vct[i], vct[nvct / 2 + i]);
+    for (int i = 0; i < nvct / 2; ++i) cdoPrint("vct: %5d %25.17f %25.17f", i, vct[i], vct[nvct / 2 + i]);
 
   if (zaxisIDh == -1) cdoAbort("No 3D variable with hybrid sigma pressure coordinate found!");
 
-  const int nvars = vlistNvars(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+  varListSetUniqueMemtype(varList1);
 
-  bool useTable = false;
+  const auto nvars = vlistNvars(vlistID1);
+
+  auto useTable = false;
   for (varID = 0; varID < nvars; varID++)
     {
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const auto tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
       if (tableNum > 0 && tableNum != 255)
         {
           useTable = true;
@@ -102,15 +102,15 @@ Derivepar(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const int nlevel = zaxisInqSize(zaxisID);
-      const int instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
-      const int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const auto gridID = varList1[varID].gridID;
+      const auto zaxisID = varList1[varID].zaxisID;
+      const auto nlevels = varList1[varID].nlevels;
+      const auto instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
+      const auto tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
 
-      int code = vlistInqVarCode(vlistID1, varID);
-      int param = vlistInqVarParam(vlistID1, varID);
+      auto code = varList1[varID].code;
 
+      const auto param = varList1[varID].param;
       cdiParamToString(param, paramstr, sizeof(paramstr));
       int pnum, pcat, pdis;
       cdiDecodeParam(param, &pnum, &pcat, &pdis);
@@ -151,36 +151,37 @@ Derivepar(void *process)
           vlistInqVarName(vlistID1, varID, varname);
           cstrToLowerCase(varname);
 
-          vlistInqVarStdname(vlistID1, varID, stdname);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID1, varID, CDI_KEY_STDNAME, stdname, &length);
           cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
           if (code < 0)
             {
               // clang-format off
-              if      (sgeopotID == -1 && cstrIsEqual(varname, "geosp")) code = gribcodes.geopot;
-              else if (psID == -1 && cstrIsEqual(varname, "aps")) code = gribcodes.ps;
-              else if (psID == -1 && cstrIsEqual(varname, "ps")) code = gribcodes.ps;
-              else if (lnpsID == -1 && cstrIsEqual(varname, "lsp")) code = gribcodes.lsp;
-              else if (lnpsID2 == -1 && cstrIsEqual(varname, "lnps")) code = 777;
-              else if (tempID == -1 && cstrIsEqual(varname, "t")) code = gribcodes.temp;
-              else if (humID == -1 && cstrIsEqual(varname, "q")) code = gribcodes.hum;
-              // else if ( geopotID  == -1 && cstrIsEqual(stdname, "geopotential_full")) code = gribcodes.geopot;
-              // else if (cstrIsEqual(varname, "clwc")) code = 246;
-              // else if (cstrIsEqual(varname, "ciwc")) code = 247;
+              if      (sgeopotID == -1 && cdo_cmpstr(varname, "geosp")) code = gribcodes.geopot;
+              else if (psID == -1      && cdo_cmpstr(varname, "aps")) code = gribcodes.ps;
+              else if (psID == -1      && cdo_cmpstr(varname, "ps")) code = gribcodes.ps;
+              else if (lnpsID == -1    && cdo_cmpstr(varname, "lsp")) code = gribcodes.lsp;
+              else if (lnpsID2 == -1   && cdo_cmpstr(varname, "lnps")) code = 777;
+              else if (tempID == -1    && cdo_cmpstr(varname, "t")) code = gribcodes.temp;
+              else if (humID == -1     && cdo_cmpstr(varname, "q")) code = gribcodes.hum;
+              // else if ( geopotID  == -1 && cdo_cmpstr(stdname, "geopotential_full")) code = gribcodes.geopot;
+              // else if (cdo_cmpstr(varname, "clwc")) code = 246;
+              // else if (cdo_cmpstr(varname, "ciwc")) code = 247;
               // clang-format on
             }
         }
 
       // clang-format off
-      if      (code == gribcodes.geopot && nlevel == 1) sgeopotID = varID;
-      else if (code == gribcodes.geopot && nlevel == nhlevf) geopotID = varID;
-      else if (code == gribcodes.temp && nlevel == nhlevf) tempID = varID;
-      else if (code == gribcodes.hum && nlevel == nhlevf) humID = varID;
-      else if (code == gribcodes.ps && nlevel == 1) psID = varID;
-      else if (code == gribcodes.lsp && nlevel == 1) lnpsID = varID;
-      else if (code == 777 && nlevel == 1) lnpsID2 = varID;
-      else if (code == gribcodes.gheight && nlevel == nhlevf) gheightID = varID;
+      if      (code == gribcodes.geopot  && nlevels == 1) sgeopotID = varID;
+      else if (code == gribcodes.geopot  && nlevels == nhlevf) geopotID = varID;
+      else if (code == gribcodes.temp    && nlevels == nhlevf) tempID = varID;
+      else if (code == gribcodes.hum     && nlevels == nhlevf) humID = varID;
+      else if (code == gribcodes.ps      && nlevels == 1) psID = varID;
+      else if (code == gribcodes.lsp     && nlevels == 1) lnpsID = varID;
+      else if (code == 777               && nlevels == 1) lnpsID2 = varID;
+      else if (code == gribcodes.gheight && nlevels == nhlevf) gheightID = varID;
       // else if (code == 246) clwcID    = varID;
       // else if (code == 247) ciwcID    = varID;
       // clang-format on
@@ -196,27 +197,26 @@ Derivepar(void *process)
   if (Options::cdoVerbose)
     {
       cdoPrint("Found:");
-      if (tempID != -1) cdoPrint("  %s", var_stdname(air_temperature));
-      if (psID != -1) cdoPrint("  %s", var_stdname(surface_air_pressure));
-      if (lnpsID != -1) cdoPrint("  LOG(%s)", var_stdname(surface_air_pressure));
-      if (sgeopotID != -1) cdoPrint("  %s", var_stdname(surface_geopotential));
-      if (geopotID != -1) cdoPrint("  %s", var_stdname(geopotential));
-      if (gheightID != -1) cdoPrint("  %s", var_stdname(geopotential_height));
+      // clang-format off
+      if (-1 != humID)     cdoPrint("  %s -> %s", var_stdname(specific_humidity), varList1[humID].name);
+      if (-1 != tempID)    cdoPrint("  %s -> %s", var_stdname(air_temperature), varList1[tempID].name);
+      if (-1 != psID)      cdoPrint("  %s -> %s", var_stdname(surface_air_pressure), varList1[psID].name);
+      if (-1 != lnpsID)    cdoPrint("  LOG(%s) -> %s", var_stdname(surface_air_pressure), varList1[lnpsID].name);
+      if (-1 != sgeopotID) cdoPrint("  %s -> %s", var_stdname(surface_geopotential), varList1[sgeopotID].name);
+      if (-1 != geopotID)  cdoPrint("  %s -> %s", var_stdname(geopotential), varList1[geopotID].name);
+      if (-1 != gheightID) cdoPrint("  %s -> %s", var_stdname(geopotential_height), varList1[gheightID].name);
+      // clang-format on
     }
 
   if (lnpsID != -1 && lnpsID2 != -1) cdoAbort("Found LOG(%s) twice: lsp and lnps!", var_stdname(surface_air_pressure));
 
   if (tempID == -1) cdoAbort("%s not found!", var_stdname(air_temperature));
 
-  std::vector<double> array(gridsize);
-  std::vector<double> sgeopot(gridsize);
-  std::vector<double> ps(gridsize);
-  std::vector<double> temp(gridsize * nhlevf);
-
-  std::vector<double> half_press(gridsize * (nhlevf + 1));
-
-  std::vector<double> hum;
-  std::vector<double> gheight;
+  Varray<double> array(gridsize);
+  Varray<double> sgeopot(gridsize), ps(gridsize);
+  Varray<double> temp(gridsize * nhlevf);
+  Varray<double> half_press(gridsize * (nhlevf + 1));
+  Varray<double> hum, gheight;
   if (operatorID == GHEIGHT)
     {
       if (humID == -1)
@@ -227,8 +227,7 @@ Derivepar(void *process)
       gheight.resize(gridsize * (nhlevf + 1));
     }
 
-  std::vector<double> full_press;
-  std::vector<double> sealevelpressure;
+  Varray<double> full_press, sealevelpressure;
   if (operatorID == SEALEVELPRESSURE)
     {
       full_press.resize(gridsize * nhlevf);
@@ -244,7 +243,7 @@ Derivepar(void *process)
       else
         cdoPrint("%s not found - using bottom layer of %s!", var_stdname(surface_geopotential), var_stdname(geopotential));
 
-      arrayFill(gridsize, sgeopot.data(), 0.0);
+      varrayFill(sgeopot, 0.0);
     }
 
   presID = lnpsID;
@@ -264,65 +263,67 @@ Derivepar(void *process)
         cdoPrint("using %s", var_stdname(surface_air_pressure));
     }
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
 
   int var_id = -1;
 
   if (operatorID == GHEIGHT)
     {
       var_id = geopotential_height;
-      varID = vlistDefVar(vlistID2, gridID, zaxisIDh, TIME_VARYING);
+      varID = vlistDefVar(vlistID2, gridID0, zaxisIDh, TIME_VARYING);
     }
   else if (operatorID == SEALEVELPRESSURE)
     {
       var_id = air_pressure_at_sea_level;
-      varID = vlistDefVar(vlistID2, gridID, surfaceID, TIME_VARYING);
+      varID = vlistDefVar(vlistID2, gridID0, surfaceID, TIME_VARYING);
     }
   else
     cdoAbort("Internal problem, invalid operatorID: %d!", operatorID);
 
   vlistDefVarParam(vlistID2, varID, cdiEncodeParam(var_echamcode(var_id), 128, 255));
-  vlistDefVarName(vlistID2, varID, var_name(var_id));
-  vlistDefVarStdname(vlistID2, varID, var_stdname(var_id));
-  vlistDefVarUnits(vlistID2, varID, var_units(var_id));
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, var_name(var_id));
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, var_stdname(var_id));
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, var_units(var_id));
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          nlevel = zaxisInqSize(zaxisID);
-          offset = gridsize * levelID;
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
+          const auto offset = gridsize * levelID;
+
           if (zaxisIDh != -1)
             {
               if (varID == sgeopotID)
                 {
-                  arrayCopy(gridsize, array.data(), sgeopot.data());
+                  varrayCopy(gridsize, array, sgeopot);
                 }
               else if (varID == geopotID && sgeopotID == -1 && (levelID + 1) == nhlevf)
                 {
-                  arrayCopy(gridsize, array.data(), sgeopot.data());
+                  varrayCopy(gridsize, array, sgeopot);
                 }
               else if (varID == presID)
                 {
                   if (lnpsID != -1)
-                    for (size_t i = 0; i < gridsize; ++i) ps[i] = exp(array[i]);
+                    for (size_t i = 0; i < gridsize; ++i) ps[i] = std::exp(array[i]);
                   else if (psID != -1)
-                    arrayCopy(gridsize, array.data(), ps.data());
+                    varrayCopy(gridsize, array, ps);
                 }
               else if (varID == tempID)
                 arrayCopy(gridsize, array.data(), &temp[offset]);
@@ -334,53 +335,48 @@ Derivepar(void *process)
       if (zaxisIDh != -1)
         {
           // check range of ps_prog
-          arrayMinMax(gridsize, ps.data(), minval, maxval);
-          if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
+          auto mm = varrayMinMax(ps);
+          if (mm.min < MIN_PS || mm.max > MAX_PS)
+            cdoWarning("Surface pressure out of range (min=%g max=%g)!", mm.min, mm.max);
 
           // check range of surface geopot
-          arrayMinMax(gridsize, sgeopot.data(), minval, maxval);
-          if (minval < MIN_FIS || maxval > MAX_FIS) cdoWarning("Orography out of range (min=%g max=%g)!", minval, maxval);
+          mm = varrayMinMax(sgeopot);
+          if (mm.min < MIN_FIS || mm.max > MAX_FIS)
+            cdoWarning("Orography out of range (min=%g max=%g)!", mm.min, mm.max);
         }
 
-      varID = tempID;
-      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      for (levelID = 0; levelID < nlevel; levelID++)
-        {
-          offset = gridsize * levelID;
-          single2 = &temp[offset];
-
-          arrayMinMax(gridsize, single2, minval, maxval);
-          if (minval < MIN_T || maxval > MAX_T)
-            cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
-        }
+      {
+        for (levelID = 0; levelID < varList1[tempID].nlevels; levelID++)
+          {
+            auto single = &temp[gridsize * levelID];
+            const auto mm = varrayMinMax(gridsize, single);
+            if (mm.min < MIN_T || mm.max > MAX_T)
+              cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
+          }
+      }
 
       if (humID != -1)
         {
-          varID = humID;
-          nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevel; levelID++)
+          for (levelID = 0; levelID < varList1[humID].nlevels; levelID++)
             {
-              offset = gridsize * levelID;
-              single2 = &hum[offset];
-
-              // corr_hum(gridsize, single2, MIN_Q);
-
-              arrayMinMax(gridsize, single2, minval, maxval);
-              if (minval < -0.1 || maxval > MAX_Q)
-                cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
+              auto single = &hum[gridsize * levelID];
+              // corr_hum(gridsize, single, MIN_Q);
+              const auto mm = varrayMinMax(gridsize, single);
+              if (mm.min < -0.1 || mm.max > MAX_Q)
+                cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
             }
         }
 
       if (operatorID == GHEIGHT)
         {
-          presh(nullptr, half_press.data(), vct.data(), ps.data(), nhlevf, gridsize);
+          presh((double*)nullptr, half_press.data(), vct.data(), ps.data(), nhlevf, gridsize);
           arrayCopy(gridsize, sgeopot.data(), gheight.data() + gridsize * nhlevf);
           MakeGeopotHeight(gheight.data(), temp.data(), hum.data(), half_press.data(), gridsize, nhlevf);
 
           nmissout = 0;
           varID = 0;
-          nlevel = nhlevf;
-          for (levelID = 0; levelID < nlevel; levelID++)
+          const auto nlevels = nhlevf;
+          for (levelID = 0; levelID < nlevels; levelID++)
             {
               cdoDefRecord(streamID2, varID, levelID);
               cdoWriteRecord(streamID2, gheight.data() + levelID * gridsize, nmissout);
@@ -390,7 +386,7 @@ Derivepar(void *process)
         {
           presh(full_press.data(), half_press.data(), vct.data(), ps.data(), nhlevf, gridsize);
 
-          extra_P(sealevelpressure.data(), &half_press[gridsize * (nhlevf)], &full_press[gridsize * (nhlevf - 1)], sgeopot.data(),
+          extrapolate_P(sealevelpressure.data(), &half_press[gridsize * (nhlevf)], &full_press[gridsize * (nhlevf - 1)], sgeopot.data(),
                   &temp[gridsize * (nhlevf - 1)], gridsize);
 
           cdoDefRecord(streamID2, 0, 0);
