@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,25 +21,101 @@
       Diff       diff            Compare two datasets
 */
 
+#include <map>
+#include <algorithm>
+
 #include <cdi.h>
 
-#include <map>
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
-#include "cdo_math.h"
-
 #include "mpmo_color.h"
 #include "cdo_options.h"
 #include "printinfo.h"
 #include "pmlist.h"
+#include "cdo_zaxis.h"
+
 
 static void
-diffGetParameter(double &abslim, double &abslim2, double &rellim, int &mapflag)
+varListSynchronizeMemtype(VarList &varList1, VarList &varList2, const std::map<int, int> &mapOfVarIDs)
 {
-  const int pargc = operatorArgc();
+  const int nvars = varList1.size();
+  for (int varID1 = 0; varID1 < nvars; varID1++)
+    {
+	  const auto it = mapOfVarIDs.find(varID1);
+      if (it == mapOfVarIDs.end()) continue;
+
+      auto varID2 = it->second;
+
+      if (varList1[varID1].memType == MemType::Float && varList2[varID2].memType == MemType::Double)
+        varList1[varID1].memType = MemType::Double;
+      else if (varList1[varID1].memType == MemType::Double && varList2[varID2].memType == MemType::Float)
+        varList2[varID2].memType = MemType::Double;
+    }
+}
+
+template <typename T>
+static void
+diff_kernel(bool lmiss, const T v1, const T v2, const T missval1, const T missval2,
+            size_t &ndiff, bool &dsgn, bool &zero, double &absm, double &relm)
+{
+  const auto v1isnan = std::isnan(v1);
+  const auto v2isnan = std::isnan(v2);
+  const auto v1ismissval = lmiss ? DBL_IS_EQUAL(v1, missval1) : false;
+  const auto v2ismissval = lmiss ? DBL_IS_EQUAL(v2, missval2) : false;
+  if ((v1isnan && !v2isnan) || (!v1isnan && v2isnan))
+    {
+      ndiff++;
+      relm = 1.0;
+    }
+  else if (lmiss == false || (!v1ismissval && !v2ismissval))
+    {
+      const double absdiff = std::fabs(v1 - v2);
+      if (absdiff > 0.) ndiff++;
+
+      absm = std::max(absm, absdiff);
+
+      const auto vv = v1 * v2;
+      if (vv < 0.)
+        dsgn = true;
+      else if (IS_EQUAL(vv, 0.))
+        zero = true;
+      else
+        relm = std::max(relm, absdiff / std::max(std::fabs(v1), std::fabs(v2)));
+    }
+  else if ((v1ismissval && !v2ismissval) || (!v1ismissval && v2ismissval))
+    {
+      ndiff++;
+      relm = 1.0;
+    }
+}
+
+static void
+diff_kernel(size_t i, bool lmiss, const Field &field1, const Field &field2, size_t &ndiff, bool &dsgn, bool &zero, double &absm, double &relm)
+{
+  if (field1.memType == MemType::Float)
+    diff_kernel(lmiss, field1.vec_f[i], field2.vec_f[i], (float)field1.missval, (float)field2.missval, ndiff, dsgn, zero, absm, relm);
+  else
+    diff_kernel(lmiss, field1.vec_d[i], field2.vec_d[i], field1.missval, field2.missval, ndiff, dsgn, zero, absm, relm);
+}
+
+static void
+use_real_part(const size_t gridsize, Field &field)
+{
+  if (field.memType == MemType::Float)
+    for (size_t i = 0; i < gridsize; ++i) field.vec_f[i] = field.vec_f[i * 2];
+  else
+    for (size_t i = 0; i < gridsize; ++i) field.vec_d[i] = field.vec_d[i * 2];
+}
+
+static void
+diffGetParameter(double &abslim, double &abslim2, double &rellim, int &mapflag, int &maxcount)
+{
+  const auto pargc = operatorArgc();
   if (pargc)
     {
-      const char **pargv = (const char **) operatorArgv();
+      const auto pargv = cdoGetOperArgv();
 
       KVList kvlist;
       kvlist.name = "DIFF";
@@ -48,23 +124,24 @@ diffGetParameter(double &abslim, double &abslim2, double &rellim, int &mapflag)
 
       for (const auto &kv : kvlist)
         {
-          const char *key = kv.key.c_str();
-          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv.values[0].c_str();
+          const auto &key = kv.key;
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          const auto &value = kv.values[0];
 
           // clang-format off
-          if      (cstrIsEqual(key, "abslim"))  abslim = parameter2double(value);
-          else if (cstrIsEqual(key, "abslim2")) abslim2 = parameter2double(value);
-          else if (cstrIsEqual(key, "rellim"))  rellim = parameter2double(value);
-          else if (cstrIsEqual(key, "names"))
+          if      (key == "abslim")   abslim = parameter2double(value);
+          else if (key == "abslim2")  abslim2 = parameter2double(value);
+          else if (key == "rellim")   rellim = parameter2double(value);
+          else if (key == "maxcount") maxcount = parameter2int(value);
+          else if (key == "names")
             {
-              if      (cstrIsEqual(value, "left"))      mapflag = 1;
-              else if (cstrIsEqual(value, "right"))     mapflag = 2;
-              else if (cstrIsEqual(value, "intersect")) mapflag = 3;
-              else cdoAbort("Invalid value for key >%s< (names=<left/right/intersect>)", key, value);
+              if      (value == "left")      mapflag = 1;
+              else if (value == "right")     mapflag = 2;
+              else if (value == "intersect") mapflag = 3;
+              else cdoAbort("Invalid value for key >%s< (names=<left/right/intersect>)", key.c_str(), value.c_str());
             }
-          else cdoAbort("Invalid parameter key >%s<!", key);
+          else cdoAbort("Invalid parameter key >%s<!", key.c_str());
           // clang-format on
         }
     }
@@ -77,39 +154,35 @@ Diff(void *process)
   int nrecs, nrecs2;
   int varID1, varID2 = -1;
   int levelID;
-  size_t nmiss1, nmiss2;
   int ndrec = 0, nd2rec = 0, ngrec = 0;
-  char varname[CDI_MAX_NAME];
   char paramstr[32];
-  char vdatestr[32], vtimestr[32];
-  double abslim = 0., abslim2 = 1.e-3, rellim = 1.0;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int DIFF  = cdoOperatorAdd("diff",  0, 0, nullptr);
-  const int DIFFP = cdoOperatorAdd("diffp", 0, 0, nullptr);
-  const int DIFFN = cdoOperatorAdd("diffn", 0, 0, nullptr);
-  const int DIFFC = cdoOperatorAdd("diffc", 0, 0, nullptr);
+  const auto DIFF  = cdoOperatorAdd("diff",  0, 0, nullptr);
+  const auto DIFFP = cdoOperatorAdd("diffp", 0, 0, nullptr);
+  const auto DIFFN = cdoOperatorAdd("diffn", 0, 0, nullptr);
+  const auto DIFFC = cdoOperatorAdd("diffc", 0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  int mapflag = 0;
-
-  diffGetParameter(abslim, abslim2, rellim, mapflag);
+  int mapflag = 0, maxcount = 0;
+  double abslim = 0., abslim2 = 1.e-3, rellim = 1.0;
+  diffGetParameter(abslim, abslim2, rellim, mapflag, maxcount);
 
   if (rellim < -1.e33 || rellim > 1.e+33) cdoAbort("Rel. limit out of range!");
   if (abslim < -1.e33 || abslim > 1.e+33) cdoAbort("Abs. limit out of range!");
   if (abslim2 < -1.e33 || abslim2 > 1.e+33) cdoAbort("Abs2. limit out of range!");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::map<int, int> mapOfVarIDs;
 
   if (mapflag == 0)
@@ -122,24 +195,24 @@ Diff(void *process)
       vlistMap(vlistID1, vlistID2, CMP_ALL, mapflag, mapOfVarIDs);
     }
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+  varListSynchronizeMemtype(varList1, varList2, mapOfVarIDs);
 
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
-  std::vector<double> work(vlistNumber(vlistID1) != CDI_REAL ? 2 * gridsizemax : 0);
+  Field field1, field2;
 
-  const int taxisID = vlistInqTaxis(vlistID1);
+  const auto taxisID = vlistInqTaxis(vlistID1);
 
   int indg = 0;
   int tsID = 0;
   while (true)
     {
+      bool lstop = false;
+
       nrecs = cdoStreamInqTimestep(streamID1, tsID);
-      if (nrecs > 0)
-        {
-          date2str(taxisInqVdate(taxisID), vdatestr, sizeof(vdatestr));
-          time2str(taxisInqVtime(taxisID), vtimestr, sizeof(vtimestr));
-        }
+      const auto vdateString = dateToString(taxisInqVdate(taxisID));
+      const auto vtimeString = timeToString(taxisInqVtime(taxisID));
 
       nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
 
@@ -173,69 +246,29 @@ Diff(void *process)
 
           indg += 1;
 
-          const int param = vlistInqVarParam(vlistID1, varID1);
-          const int code = vlistInqVarCode(vlistID1, varID1);
-          const int gridID = vlistInqVarGrid(vlistID1, varID1);
-          const int zaxisID = vlistInqVarZaxis(vlistID1, varID1);
-          const size_t gridsize = gridInqSize(gridID);
-          const double missval1 = vlistInqVarMissval(vlistID1, varID1);
-          const double missval2 = vlistInqVarMissval(vlistID2, varID2);
+          const auto gridsize = varList1[varID1].gridsize;
 
           // checkrel = gridInqType(gridID) != GRID_SPECTRAL;
           const bool checkrel = true;
 
-          cdiParamToString(param, paramstr, sizeof(paramstr));
+          cdiParamToString(varList1[varID1].param, paramstr, sizeof(paramstr));
 
-          if (vlistInqVarNumber(vlistID1, varID1) == CDI_COMP)
-            {
-              cdoReadRecord(streamID1, work.data(), &nmiss1);
-              for (size_t i = 0; i < gridsize; ++i) array1[i] = work[i * 2];
-            }
-          else
-            cdoReadRecord(streamID1, array1.data(), &nmiss1);
+          field1.init(varList1[varID1]);
+          cdoReadRecord(streamID1, field1);
+          if (varList1[varID1].nwpv == CDI_COMP) use_real_part(gridsize, field1);
 
-          if (vlistInqVarNumber(vlistID2, varID2) == CDI_COMP)
-            {
-              cdoReadRecord(streamID2, work.data(), &nmiss2);
-              for (size_t i = 0; i < gridsize; ++i) array2[i] = work[i * 2];
-            }
-          else
-            cdoReadRecord(streamID2, array2.data(), &nmiss2);
+          field2.init(varList2[varID2]);
+          cdoReadRecord(streamID2, field2);
+          if (varList2[varID2].nwpv == CDI_COMP) use_real_part(gridsize, field2);
 
-          int ndiff = 0;
-          bool dsgn = false;
-          bool zero = false;
-          double absm = 0.0;
-          double relm = 0.0;
-          double absdiff;
+          const bool lmiss = field1.nmiss || field2.nmiss;
+          size_t ndiff = 0;
+          bool dsgn = false, zero = false;
+          double absm = 0.0, relm = 0.0;
 
           for (size_t i = 0; i < gridsize; i++)
             {
-              if ((DBL_IS_NAN(array1[i]) && !DBL_IS_NAN(array2[i])) || (!DBL_IS_NAN(array1[i]) && DBL_IS_NAN(array2[i])))
-                {
-                  ndiff++;
-                  relm = 1.0;
-                }
-              else if (!DBL_IS_EQUAL(array1[i], missval1) && !DBL_IS_EQUAL(array2[i], missval2))
-                {
-                  absdiff = std::fabs(array1[i] - array2[i]);
-                  if (absdiff > 0.) ndiff++;
-
-                  absm = cdo::max(absm, absdiff);
-
-                  if (array1[i] * array2[i] < 0.)
-                    dsgn = true;
-                  else if (IS_EQUAL(array1[i] * array2[i], 0.))
-                    zero = true;
-                  else
-                    relm = cdo::max(relm, absdiff / cdo::max(std::fabs(array1[i]), std::fabs(array2[i])));
-                }
-              else if ((DBL_IS_EQUAL(array1[i], missval1) && !DBL_IS_EQUAL(array2[i], missval2))
-                       || (!DBL_IS_EQUAL(array1[i], missval1) && DBL_IS_EQUAL(array2[i], missval2)))
-                {
-                  ndiff++;
-                  relm = 1.0;
-                }
+              diff_kernel(i, lmiss, field1, field2, ndiff, dsgn, zero, absm, relm);
             }
 
           if (!Options::silentMode || Options::cdoVerbose)
@@ -246,7 +279,6 @@ Diff(void *process)
                     {
                       lhead = false;
 
-                      set_text_color(stdout, BRIGHT, BLACK);
                       fprintf(stdout, "               Date     Time   Level Gridsize    Miss ");
                       fprintf(stdout, "   Diff ");
                       fprintf(stdout, ": S Z  Max_Absdiff Max_Reldiff : ");
@@ -257,47 +289,36 @@ Diff(void *process)
                         fprintf(stdout, "Parameter ID");
                       else if (operatorID == DIFFC)
                         fprintf(stdout, "Code number");
-                      reset_text_color(stdout);
 
                       fprintf(stdout, "\n");
                     }
 
-                  if (operatorID == DIFFN) vlistInqVarName(vlistID1, varID1, varname);
-
-                  set_text_color(stdout, BRIGHT, BLACK);
                   fprintf(stdout, "%6d ", indg);
-                  reset_text_color(stdout);
-                  set_text_color(stdout, BLACK);
                   fprintf(stdout, ":");
-                  reset_text_color(stdout);
 
                   set_text_color(stdout, MAGENTA);
-                  fprintf(stdout, "%s %s ", vdatestr, vtimestr);
+                  fprintf(stdout, "%s %s ", vdateString.c_str(), vtimeString.c_str());
                   reset_text_color(stdout);
                   set_text_color(stdout, GREEN);
-                  double level = cdoZaxisInqLevel(zaxisID, levelID);
-                  fprintf(stdout, "%7g ", level);
-                  fprintf(stdout, "%8zu %7zu ", gridsize, cdo::max(nmiss1, nmiss2));
-                  fprintf(stdout, "%7d ", ndiff);
+                  fprintf(stdout, "%7g ", cdoZaxisInqLevel(varList1[varID1].zaxisID, levelID));
+                  fprintf(stdout, "%8zu %7zu ", gridsize, std::max(field1.nmiss, field2.nmiss));
+                  fprintf(stdout, "%7zu ", ndiff);
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, BLACK);
                   fprintf(stdout, ":");
-                  reset_text_color(stdout);
                   fprintf(stdout, " %c %c ", dsgn ? 'T' : 'F', zero ? 'T' : 'F');
                   set_text_color(stdout, BLUE);
                   fprintf(stdout, "%#12.5g%#12.5g", absm, relm);
-                  set_text_color(stdout, BLACK);
-                  fprintf(stdout, " : ");
                   reset_text_color(stdout);
+                  fprintf(stdout, " : ");
 
                   set_text_color(stdout, BRIGHT, GREEN);
                   if (operatorID == DIFFN)
-                    fprintf(stdout, "%-11s", varname);
+                    fprintf(stdout, "%-11s", varList1[varID1].name);
                   else if (operatorID == DIFF || operatorID == DIFFP)
                     fprintf(stdout, "%-11s", paramstr);
                   else if (operatorID == DIFFC)
-                    fprintf(stdout, "%4d", code);
+                    fprintf(stdout, "%4d", varList1[varID1].code);
                   reset_text_color(stdout);
 
                   fprintf(stdout, "\n");
@@ -307,7 +328,16 @@ Diff(void *process)
           ngrec++;
           if (absm > abslim || (checkrel && relm >= rellim)) ndrec++;
           if (absm > abslim2 || (checkrel && relm >= rellim)) nd2rec++;
+
+          if (maxcount > 0 && ndrec >= maxcount)
+            {
+              lstop = true;
+              break;
+            }
         }
+
+      if (lstop) break;
+
       tsID++;
     }
 
