@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,23 +15,24 @@
   GNU General Public License for more details.
 */
 
+#include <algorithm>
+
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_rlimit.h"
+#include "process_int.h"
 #include "param_conversion.h"
-#include "cdo_math.h"
 #include <mpim_grid.h>
-
 #include "util_files.h"
 
-#define MAX_BLOCKS 65536
 
 static void
 genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblocks, size_t nyblocks,
          std::vector<std::vector<size_t>> &gridindex, size_t *ogridsize, size_t nsplit)
 {
-  std::vector<double> xpvals, ypvals;
-  int gridtype = gridInqType(gridID1);
+  Varray<double> xpvals, ypvals;
+  auto gridtype = gridInqType(gridID1);
   bool lunstructured = gridtype == GRID_UNSTRUCTURED;
   bool lregular = true;
   if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GENERIC)
@@ -43,15 +44,14 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
   else
     cdoAbort("Unsupported grid type: %s!", gridNamePtr(gridtype));
 
-  const size_t nx = gridInqXsize(gridID1);
+  const auto nx = gridInqXsize(gridID1);
   const size_t ny = lunstructured ? 1 : gridInqYsize(gridID1);
 
   const bool lxcoord = gridInqXvals(gridID1, nullptr) > 0;
   const bool lycoord = gridInqYvals(gridID1, nullptr) > 0;
-  const bool lbounds = !lregular && gridInqXbounds(gridID1, nullptr) > 0 && gridInqYbounds(gridID1, nullptr) > 0;
+  const bool lbounds = !lregular && gridHasBounds(gridID1);
 
-  std::vector<size_t> xlsize(nxblocks);
-  std::vector<size_t> ylsize(nyblocks);
+  std::vector<size_t> xlsize(nxblocks), ylsize(nyblocks);
 
   for (size_t ix = 0; ix < nxblocks; ++ix) xlsize[ix] = nxvals;
   if (nx % nxblocks != 0) xlsize[nxblocks - 1] = nx - (nxblocks - 1) * nxvals;
@@ -63,11 +63,11 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
   if (Options::cdoVerbose)
     for (size_t iy = 0; iy < nyblocks; ++iy) cdoPrint("yblock %zu: %zu", iy, ylsize[iy]);
 
-  const size_t nxvmax = cdo::max(nxvals, xlsize[nxblocks - 1]);
-  const size_t nyvmax = cdo::max(nyvals, ylsize[nyblocks - 1]);
+  const auto nxvmax = std::max(nxvals, xlsize[nxblocks - 1]);
+  const auto nyvmax = std::max(nyvals, ylsize[nyblocks - 1]);
 
-  std::vector<double> xvals, yvals, xbounds, ybounds;
-  std::vector<double> xvals2, yvals2, xbounds2, ybounds2;
+  Varray<double> xvals, yvals, xbounds, ybounds;
+  Varray<double> xvals2, yvals2, xbounds2, ybounds2;
 
   if (lxcoord)
     {
@@ -108,8 +108,8 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
   for (size_t iy = 0; iy < nyblocks; ++iy)
     for (size_t ix = 0; ix < nxblocks; ++ix)
       {
-        const size_t offset = iy * nyvals * nx + ix * nxvals;
-        size_t gridsize2 = xlsize[ix] * ylsize[iy];
+        const auto offset = iy * nyvals * nx + ix * nxvals;
+        auto gridsize2 = xlsize[ix] * ylsize[iy];
         gridindex[index].resize(gridsize2);
 
         gridsize2 = 0;
@@ -135,7 +135,7 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
             }
         // printf("gridsize2 %d\n", gridsize2);
 
-        const int gridID2 = gridCreate(gridtype, gridsize2);
+        const auto gridID2 = gridCreate(gridtype, gridsize2);
         if (gridtype != GRID_UNSTRUCTURED)
           {
             gridDefXsize(gridID2, xlsize[ix]);
@@ -148,7 +148,7 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
 
         gridDefDatatype(gridID2, gridInqDatatype(gridID1));
 
-        grid_copy_attributes(gridID1, gridID2);
+        grid_copy_keys(gridID1, gridID2);
 
         if (gridtype == GRID_PROJECTION) grid_copy_mapping(gridID1, gridID2);
 
@@ -168,14 +168,14 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
               }
           }
 
-        const int projID1 = gridInqProj(gridID1);
+        const auto projID1 = gridInqProj(gridID1);
         if (projID1 != CDI_UNDEFID && gridInqType(projID1) == GRID_PROJECTION)
           {
-            const int projID2 = gridCreate(GRID_PROJECTION, gridsize2);
+            const auto projID2 = gridCreate(GRID_PROJECTION, gridsize2);
             gridDefXsize(projID2, xlsize[ix]);
             gridDefYsize(projID2, ylsize[iy]);
 
-            grid_copy_attributes(projID1, projID2);
+            grid_copy_keys(projID1, projID2);
             grid_copy_mapping(projID1, projID2);
 
             const bool lxpcoord = gridInqXvals(projID1, nullptr) > 0;
@@ -211,14 +211,16 @@ genGrids(int gridID1, int *gridIDs, size_t nxvals, size_t nyvals, size_t nxblock
 }
 
 static void
-window_cell(double *array1, double *array2, size_t gridsize2, size_t *cellidx)
+window_cell(const Field &field1, Field &field2, size_t gridsize2, size_t *cellidx)
 {
-  for (size_t i = 0; i < gridsize2; ++i) array2[i] = array1[cellidx[i]];
+  if (field1.memType == MemType::Float)
+    for (size_t i = 0; i < gridsize2; ++i) field2.vec_f[i] = field1.vec_f[cellidx[i]];
+  else
+    for (size_t i = 0; i < gridsize2; ++i) field2.vec_d[i] = field1.vec_d[cellidx[i]];
 }
 
 struct GridInfo
 {
-  int gridID;
   std::vector<int> gridIDs;
   std::vector<size_t> gridsize;
   std::vector<std::vector<size_t>> gridindex;
@@ -227,49 +229,51 @@ struct GridInfo
 void *
 Distgrid(void *process)
 {
+  constexpr size_t MaxBlocks = 99999;
   int gridID1;
   int varID, levelID;
   int nrecs;
-  char filesuffix[32];
-  char filename[8192];
-  int index;
   int gridtype = -1;
-  size_t nmiss;
 
   cdoInitialize(process);
 
+  if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
+
   operatorInputArg("nxblocks, [nyblocks]");
   if (operatorArgc() < 1) cdoAbort("Too few arguments!");
   if (operatorArgc() > 2) cdoAbort("Too many arguments!");
-  size_t nxblocks = parameter2int(operatorArgv()[0]);
+  size_t nxblocks = parameter2int(cdoOperatorArgv(0));
   size_t nyblocks = 1;
-  if (operatorArgc() == 2) nyblocks = parameter2int(operatorArgv()[1]);
+  if (operatorArgc() == 2) nyblocks = parameter2int(cdoOperatorArgv(1));
 
   if (nxblocks == 0) cdoAbort("nxblocks has to be greater than 0!");
   if (nyblocks == 0) cdoAbort("nyblocks has to be greater than 0!");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
 
-  for (index = 0; index < ngrids; index++)
-    {
-      gridID1 = vlistGrid(vlistID1, index);
-      gridtype = gridInqType(gridID1);
-      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
-          || (gridtype == GRID_GENERIC && gridInqXsize(gridID1) > 0 && gridInqYsize(gridID1) > 0))
-        break;
+  {
+    int index;
+    for (index = 0; index < ngrids; index++)
+      {
+        gridID1 = vlistGrid(vlistID1, index);
+        gridtype = gridInqType(gridID1);
+        if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
+            || (gridtype == GRID_GENERIC && gridInqXsize(gridID1) > 0 && gridInqYsize(gridID1) > 0))
+          break;
+      }
+
+    if (index == ngrids)
+      cdoAbort("No Lon/Lat, Gaussian, curvilinear or generic grid found (%s data unsupported)!", gridNamePtr(gridtype));
     }
 
-  if (index == ngrids)
-    cdoAbort("No Lon/Lat, Gaussian, curvilinear or generic grid found (%s data unsupported)!", gridNamePtr(gridtype));
-
   const bool lunstruct = gridtype == GRID_UNSTRUCTURED;
 
   gridID1 = vlistGrid(vlistID1, 0);
-  const size_t gridsize = gridInqSize(gridID1);
-  const size_t nx = gridInqXsize(gridID1);
+  const auto gridsize = gridInqSize(gridID1);
+  const auto nx = gridInqXsize(gridID1);
   const size_t ny = lunstruct ? 1 : gridInqYsize(gridID1);
   for (int i = 1; i < ngrids; i++)
     {
@@ -288,22 +292,27 @@ Distgrid(void *process)
       nyblocks = ny;
     }
 
-  size_t xinc = nx / nxblocks;
-  size_t yinc = ny / nyblocks;
+  auto xinc = nx / nxblocks;
+  auto yinc = ny / nyblocks;
   if (nx % xinc && nx % (xinc + 1) && nxblocks * (xinc + 1) <= nx) xinc++;
   if (ny % yinc && ny % (yinc + 1) && nyblocks * (yinc + 1) <= ny) yinc++;
 
-  const size_t nsplit = nxblocks * nyblocks;
-  if (nsplit > MAX_BLOCKS) cdoAbort("Too many blocks (max = %d)!", MAX_BLOCKS);
+  const auto nsplit = nxblocks * nyblocks;
+  if (nsplit > MaxBlocks) cdoAbort("Too many blocks (max = %d)!", MaxBlocks);
+
+  cdo::set_numfiles(nsplit + 8);
+
+  Field field1, field2;
+
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
-  std::vector<double> array1(gridsize);
   std::vector<int> vlistIDs(nsplit);
   std::vector<CdoStreamID> streamIDs(nsplit);
 
   std::vector<GridInfo> gridinfo(ngrids);
   for (int i = 0; i < ngrids; i++)
     {
-      gridinfo[i].gridID = vlistGrid(vlistID1, i);
       gridinfo[i].gridIDs.resize(nsplit);
       gridinfo[i].gridsize.resize(nsplit);
       gridinfo[i].gridindex.resize(nsplit);
@@ -327,17 +336,12 @@ Distgrid(void *process)
       for (size_t index = 0; index < nsplit; index++) vlistChangeGridIndex(vlistIDs[index], i, gridinfo[i].gridIDs[index]);
     }
 
-  size_t gridsize2max = 0;
-  for (size_t index = 0; index < nsplit; index++)
-    if (gridinfo[0].gridsize[index] > gridsize2max) gridsize2max = gridinfo[0].gridsize[index];
-
-  std::vector<double> array2(gridsize2max);
-
+  char filename[8192];
   strcpy(filename, cdoGetObase());
   const int nchars = strlen(filename);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   for (size_t index = 0; index < nsplit; index++)
@@ -353,22 +357,26 @@ Distgrid(void *process)
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      for (size_t index = 0; index < nsplit; index++) cdoDefTimestep(streamIDs[index], tsID);
+      for (auto &streamID : streamIDs) cdoDefTimestep(streamID, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, &array1[0], &nmiss);
-
-          const double missval = vlistInqVarMissval(vlistID1, varID);
+          field1.init(varList1[varID]);
+          cdoReadRecord(streamID1, field1);
 
           for (size_t index = 0; index < nsplit; index++)
             {
-              int i = 0;
-              window_cell(&array1[0], &array2[0], gridinfo[i].gridsize[index], gridinfo[i].gridindex[index].data());
+              auto var = varList1[varID];
+              var.gridID = gridinfo[0].gridIDs[index];
+              var.gridsize = gridinfo[0].gridsize[index];
+              field2.init(var);
+
+              window_cell(field1, field2, gridinfo[0].gridsize[index], gridinfo[0].gridindex[index].data());
+              if (field1.nmiss) fieldNumMV(field2);
+
               cdoDefRecord(streamIDs[index], varID, levelID);
-              if (nmiss) nmiss = arrayNumMV(gridinfo[i].gridsize[index], &array2[0], missval);
-              cdoWriteRecord(streamIDs[index], &array2[0], nmiss);
+              cdoWriteRecord(streamIDs[index], field2);
             }
         }
 
@@ -377,11 +385,8 @@ Distgrid(void *process)
 
   cdoStreamClose(streamID1);
 
-  for (size_t index = 0; index < nsplit; index++)
-    {
-      cdoStreamClose(streamIDs[index]);
-      vlistDestroy(vlistIDs[index]);
-    }
+  for (auto &streamID : streamIDs) cdoStreamClose(streamID);
+  for (auto &vlistID : vlistIDs) vlistDestroy(vlistID);
 
   for (int i = 0; i < ngrids; i++)
     for (size_t index = 0; index < nsplit; index++) gridDestroy(gridinfo[i].gridIDs[index]);
