@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,33 +34,35 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include <mpim_grid.h>
 #include "statistic.h"
 #include "timer.h"
 #include "datetime.h"
+#include "eof_mode.h"
 
-// NO MISSING VALUE SUPPORT ADDED SO FAR
+// No missing value support added so far!
 
 static void
-scale_eigvec_grid(double *restrict out, int tsID, size_t npack, const size_t *restrict pack, const double *restrict weight,
-                  double **covar, double sum_w)
+scale_eigvec_grid(Varray<double> &out, int tsID, size_t npack, const std::vector<size_t> &pack, const Varray<double> &weight,
+                  Varray2D<double> &covar, double sum_w)
 {
-  for (size_t i = 0; i < npack; ++i) out[pack[i]] = covar[tsID][i] / sqrt(weight[pack[i]] / sum_w);
+  for (size_t i = 0; i < npack; ++i) out[pack[i]] = covar[tsID][i] / std::sqrt(weight[pack[i]] / sum_w);
 }
 
 static void
-scale_eigvec_time(double *restrict out, int tsID, int nts, size_t npack, const size_t *restrict pack, const double *restrict weight,
-                  double **covar, double **data, double missval, double sum_w)
+scale_eigvec_time(Varray<double> &out, int tsID, int nts, size_t npack, const std::vector<size_t> &pack, const Varray<double> &weight,
+                  const Varray2D<double> &covar, const Varray2D<double> &data, double missval, double sum_w)
 {
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(npack, nts, tsID, pack, data, covar, out)
 #endif
   for (size_t i = 0; i < npack; ++i)
     {
-      double sum = 0;
+      double sum = 0.0;
       for (int j = 0; j < nts; ++j) sum += data[j][i] * covar[tsID][j];
 
       out[pack[i]] = sum;
@@ -74,7 +76,7 @@ scale_eigvec_time(double *restrict out, int tsID, int nts, size_t npack, const s
   */
 
   // Normalizing
-  double sum = 0;
+  double sum = 0.0;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) reduction(+ : sum) shared(out, weight, pack, npack)
@@ -85,9 +87,9 @@ scale_eigvec_time(double *restrict out, int tsID, int nts, size_t npack, const s
       sum += weight[pack[i]] * out[pack[i]] * out[pack[i]];
     }
 
-  if (sum > 0)
+  if (sum > 0.0)
     {
-      sum = sqrt(sum / sum_w);
+      sum = std::sqrt(sum / sum_w);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(npack, pack, sum, out)
 #endif
@@ -102,64 +104,6 @@ scale_eigvec_time(double *restrict out, int tsID, int nts, size_t npack, const s
     }
 }
 
-enum T_EIGEN_MODE
-get_eigenmode(void)
-{
-  enum T_EIGEN_MODE eigen_mode = JACOBI;
-
-  char *envstr = getenv("CDO_SVD_MODE");
-  if (envstr)
-    {
-      if (!strncmp(envstr, "danielson_lanczos", 17))
-        eigen_mode = DANIELSON_LANCZOS;
-      else if (!strncmp(envstr, "jacobi", 6))
-        eigen_mode = JACOBI;
-      else
-        {
-          cdoWarning("Unknown environmental setting %s for CDO_SVD_MODE. Available options are", envstr);
-          cdoWarning("  - 'jacobi' for a one-sided parallelized jacobi algorithm");
-          cdoWarning("  - 'danielson_lanzcos' for the D/L algorithm");
-        }
-    }
-
-  if (Options::cdoVerbose)
-    cdoPrint("Using CDO_SVD_MODE '%s' from %s", eigen_mode == JACOBI ? "jacobi" : "danielson_lanczos",
-             envstr ? "Environment" : " default");
-
-#ifdef _OPENMP
-  if (omp_get_max_threads() > 1 && eigen_mode == DANIELSON_LANCZOS)
-    {
-      cdoWarning("Requested parallel computation with %i Threads ", omp_get_max_threads());
-      cdoWarning("  but environmental setting CDO_SVD_MODE causes sequential ");
-      cdoWarning("  Singular value decomposition");
-    }
-#endif
-
-  return eigen_mode;
-}
-
-enum T_WEIGHT_MODE
-get_weightmode(void)
-{
-  enum T_WEIGHT_MODE weight_mode = WEIGHT_OFF;
-
-  char *envstr = getenv("CDO_WEIGHT_MODE");
-  if (envstr)
-    {
-      if (!strncmp(envstr, "off", 3))
-        weight_mode = WEIGHT_OFF;
-      else if (!strncmp(envstr, "on", 2))
-        weight_mode = WEIGHT_ON;
-      else
-        cdoWarning("Unknown environmental setting %s for CDO_WEIGHT_MODE. Available options are: on/off", envstr);
-    }
-
-  if (Options::cdoVerbose)
-    cdoPrint("Using CDO_WEIGHT_MODE '%s' from %s", weight_mode == WEIGHT_OFF ? "off" : "on", envstr ? "Environment" : " default");
-
-  return weight_mode;
-}
-
 void *
 EOFs(void *process)
 {
@@ -170,24 +114,21 @@ EOFs(void *process)
     EOF_SPATIAL
   };
 
-  int nlevs = 0;
   size_t nmiss;
   int varID, levelID;
-  int nts = 0;
-  size_t n = 0;
+  int nts = 1;
   int grid_space = 0, time_space = 0;
   int timer_cov = 0, timer_eig = 0;
 
-  int calendar = CALENDAR_STANDARD;
+  const auto calendar = CALENDAR_STANDARD;
 
   struct eofdata_t
   {
     bool init;
     bool first_call;
-    double *eig_val;
-    double *covar_array;
-    double **covar;
-    double **data;
+    Varray<double> eig_val;
+    Varray2D<double> covar;
+    Varray2D<double> data;
   };
 
   if (Options::Timer)
@@ -204,35 +145,39 @@ EOFs(void *process)
   cdoOperatorAdd("eofspatial", EOF_SPATIAL, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
   operatorInputArg("Number of eigen functions to write out");
-  int n_eig = parameter2int(operatorArgv()[0]);
+  auto n_eig = parameter2int(cdoOperatorArgv(0));
 
-  enum T_EIGEN_MODE eigen_mode = get_eigenmode();
-  enum T_WEIGHT_MODE weight_mode = get_weightmode();
+  const auto eigen_mode = get_eigenmode();
+  auto weight_mode = get_weightmode();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int gridID1 = vlistInqVarGrid(vlistID1, 0);
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  int nvars = vlistNvars(vlistID1);
-  int nrecs;
+  auto streamID1 = cdoOpenRead(0);
+  auto vlistID1 = cdoStreamInqVlist(streamID1);
+  auto taxisID1 = vlistInqTaxis(vlistID1);
+  
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto gridID1 = varList1[0].gridID;
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
+
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 1; index < ngrids; index++)
     if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) cdoAbort("Too many different grids!");
 
-  /* eigenvalues */
+  // eigenvalues
 
-  /* COUNT NUMBER OF TIMESTEPS if EOF_ or EOF_TIME */
+  // Count number of timesteps if EOF_ or EOF_TIME
   if (operfunc == EOF_ || operfunc == EOF_TIME)
     {
       if (Options::cdoVerbose) cdoPrint("Counting timesteps in ifile");
 
       nts = vlistNtsteps(vlistID1);
+      if (nts == 0) nts = 1;
       if (nts == -1)
         {
           nts = 0;
@@ -249,7 +194,7 @@ EOFs(void *process)
       else if (Options::cdoVerbose)
         cdoPrint("Found %i timeSteps", nts);
 
-      if ((size_t) nts < gridsize || operfunc == EOF_TIME)
+      if ((size_t) nts < gridsizemax || operfunc == EOF_TIME)
         {
           time_space = 1;
           grid_space = 0;
@@ -266,8 +211,8 @@ EOFs(void *process)
       grid_space = 1;
     }
 
-  /* reset the requested number of eigen-function to the maximum if neccessary
-   */
+  // reset the requested number of eigen-function to the maximum if neccessary
+  size_t n = 0;
   if (time_space)
     {
       if (n_eig > nts)
@@ -282,28 +227,28 @@ EOFs(void *process)
     }
   else if (grid_space)
     {
-      if (((double) gridsize) * gridsize > (double) SIZE_MAX) cdoAbort("Grid space too large!");
+      if (((double) gridsizemax) * gridsizemax > (double) SIZE_MAX) cdoAbort("Grid space too large!");
 
-      if ((size_t) n_eig > gridsize)
+      if ((size_t) n_eig > gridsizemax)
         {
           cdoWarning("Solving in spatial space");
           cdoWarning("Number of eigen-functions to write out is bigger than grid size");
-          cdoWarning("Setting n_eig to %zu", gridsize);
+          cdoWarning("Setting n_eig to %zu", gridsizemax);
           cdoWarning("If You want to force a solution in time-space use operator eoftime");
-          n_eig = gridsize;
+          n_eig = gridsizemax;
         }
-      n = gridsize;
+      n = gridsizemax;
     }
 
   if (Options::cdoVerbose)
     cdoPrint("Calculating %d eigenvectors and %zu eigenvalues in %s", n_eig, n, grid_space == 1 ? "grid_space" : "time_space");
 
-  std::vector<double> weight(gridsize);
-  for (size_t i = 0; i < gridsize; ++i) weight[i] = 1.;
+  Varray<double> weight(gridsizemax);
+  for (size_t i = 0; i < gridsizemax; ++i) weight[i] = 1.;
 
   if (weight_mode == WEIGHT_ON)
     {
-      int wstatus = gridWeights(gridID1, weight.data());
+      const auto wstatus = gridWeights(gridID1, weight.data());
       if (wstatus != 0)
         {
           weight_mode = WEIGHT_OFF;
@@ -313,49 +258,44 @@ EOFs(void *process)
 
   // allocation of temporary fields and output structures
   size_t npack = SIZE_MAX;
-  std::vector<size_t> pack(gridsize);
-  double *in = (double *) Malloc(gridsize * sizeof(double));
+  std::vector<size_t> pack(gridsizemax);
+  Varray<double> in(gridsizemax);
   std::vector<std::vector<eofdata_t>> eofdata(nvars);
 
   for (varID = 0; varID < nvars; ++varID)
     {
-      nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      const auto nlevs = varList1[varID].nlevels;
       eofdata[varID].resize(nlevs);
 
       for (levelID = 0; levelID < nlevs; ++levelID)
         {
           eofdata[varID][levelID].init = false;
           eofdata[varID][levelID].first_call = true;
-          eofdata[varID][levelID].eig_val = nullptr;
-          eofdata[varID][levelID].covar_array = nullptr;
-          eofdata[varID][levelID].covar = nullptr;
-          eofdata[varID][levelID].data = nullptr;
 
-          if (time_space) eofdata[varID][levelID].data = (double **) Malloc(nts * sizeof(double *));
+          if (time_space) eofdata[varID][levelID].data.resize(nts);
         }
     }
 
-  if (Options::cdoVerbose) cdoPrint("Allocated eigenvalue/eigenvector structures with nts=%d gridsize=%zu", nts, gridsize);
+  if (Options::cdoVerbose) cdoPrint("Allocated eigenvalue/eigenvector structures with nts=%d gridsize=%zu", nts, gridsizemax);
 
-  double *covar_array = nullptr;
-  double **covar = nullptr;
-  double sum_w = 1.;
+  double sum_w = 1.0;
 
   int tsID = 0;
 
   /* read the data and create covariance matrices for each var & level */
   while (true)
     {
-      nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, in, &nmiss);
+          cdoReadRecord(streamID1, in.data(), &nmiss);
+
+          const auto gridsize = varList1[varID].gridsize;
+          const auto missval = varList1[varID].missval;
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          double missval = vlistInqVarMissval(vlistID1, varID);
           if (npack == SIZE_MAX)
             {
               npack = 0;
@@ -370,40 +310,33 @@ EOFs(void *process)
 
               if (weight_mode == WEIGHT_ON)
                 {
-                  sum_w = 0;
+                  sum_w = 0.0;
                   for (size_t i = 0; i < npack; i++) sum_w += weight[pack[i]];
                 }
             }
 
-          size_t ipack = 0;
-          for (size_t i = 0; i < gridsize; ++i)
-            {
-              if (!DBL_IS_EQUAL(weight[i], 0.0) && !DBL_IS_EQUAL(weight[i], missval) && !DBL_IS_EQUAL(in[i], missval))
-                {
-                  if (pack[ipack] != i) cdoAbort("Missing values unsupported!");
-                  ipack++;
-                }
-            }
-          if (ipack != npack) cdoAbort("Missing values unsupported!");
+          {
+            size_t ipack = 0;
+            for (size_t i = 0; i < gridsize; ++i)
+              {
+                if (!DBL_IS_EQUAL(weight[i], 0.0) && !DBL_IS_EQUAL(weight[i], missval) && !DBL_IS_EQUAL(in[i], missval))
+                  {
+                    if (pack[ipack] != i) cdoAbort("Missing values unsupported!");
+                    ipack++;
+                  }
+              }
+            if (ipack != npack) cdoAbort("Missing values unsupported!");
+          }
 
           if (grid_space)
             {
               if (!eofdata[varID][levelID].init)
                 {
-                  n = npack;
-                  double *covar_array = (double *) Malloc(npack * npack * sizeof(double));
-                  covar = (double **) Malloc(npack * sizeof(double *));
-                  for (size_t i = 0; i < npack; ++i) covar[i] = covar_array + npack * i;
-                  for (size_t i = 0; i < npack; ++i)
-                    for (size_t j = 0; j < npack; ++j) covar[i][j] = 0;
+                  eofdata[varID][levelID].covar.resize(npack);
+                  for (size_t i = 0; i < npack; ++i) eofdata[varID][levelID].covar[i].resize(npack, 0.0);
+                }
 
-                  eofdata[varID][levelID].covar_array = covar_array;
-                  eofdata[varID][levelID].covar = covar;
-                }
-              else
-                {
-                  covar = eofdata[varID][levelID].covar;
-                }
+              auto &covar = eofdata[varID][levelID].covar;
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
@@ -412,8 +345,8 @@ EOFs(void *process)
             }
           else if (time_space)
             {
-              double *data = (double *) Malloc(npack * sizeof(double));
-              eofdata[varID][levelID].data[tsID] = data;
+              eofdata[varID][levelID].data[tsID].resize(npack);
+              auto &data = eofdata[varID][levelID].data[tsID];
 
               for (size_t ipack = 0; ipack < npack; ipack++) data[ipack] = in[pack[ipack]];
             }
@@ -428,30 +361,30 @@ EOFs(void *process)
 
   if (tsID == 1) cdoAbort("File consists of only one timestep!");
 
-  /* write files with eigenvalues (ID3) and eigenvectors (ID2) */
+  // write files with eigenvalues (ID3) and eigenvectors (ID2)
 
-  /* eigenvalues */
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  // eigenvalues
+  const auto streamID2 = cdoOpenWrite(1);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisDefRdate(taxisID2, 0);
   taxisDefRtime(taxisID2, 0);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int gridID2 = gridCreate(GRID_LONLAT, 1);
+  const auto gridID2 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID2, 1);
   gridDefYsize(gridID2, 1);
-  double xvals = 0, yvals = 0;
+  double xvals = 0.0, yvals = 0.0;
   gridDefXvals(gridID2, &xvals);
   gridDefYvals(gridID2, &yvals);
   for (int i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID2, i, gridID2);
 
-  /*  eigenvectors */
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  // eigenvectors
+  const auto streamID3 = cdoOpenWrite(2);
 
-  int vlistID3 = vlistDuplicate(vlistID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   taxisDefRdate(taxisID3, 0);
   taxisDefRtime(taxisID3, 0);
   vlistDefTaxis(vlistID3, taxisID3);
@@ -461,10 +394,9 @@ EOFs(void *process)
 
   int64_t vdate = 10101;
   int vtime = 0;
-  JulianDate juldate = julianDateEncode(calendar, vdate, vtime);
+  auto juldate = julianDateEncode(calendar, vdate, vtime);
 
-  double *out = in;
-  double *eig_val = nullptr;
+  Varray<double> &out = in;
 
   int nts_out = nts;
   if (npack < (size_t) nts) nts_out = npack;
@@ -472,7 +404,7 @@ EOFs(void *process)
   for (tsID = 0; tsID < nts_out; tsID++)
     {
       juldate = julianDateAddSeconds(60, juldate);
-      julianDateDecode(calendar, juldate, &vdate, &vtime);
+      julianDateDecode(calendar, juldate, vdate, vtime);
 
       taxisDefVdate(taxisID2, vdate);
       taxisDefVtime(taxisID2, vtime);
@@ -487,15 +419,15 @@ EOFs(void *process)
 
       for (varID = 0; varID < nvars; varID++)
         {
-          char vname[256];
-          vlistInqVarName(vlistID1, varID, vname);
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const auto vname = varList1[varID].name;
+          const auto gridsize = varList1[varID].gridsize;
+          const auto nlevs = varList1[varID].nlevels;
+          const auto missval = varList1[varID].missval;
 
           for (levelID = 0; levelID < nlevs; levelID++)
             {
-              double **data = eofdata[varID][levelID].data;
+              const auto &data = eofdata[varID][levelID].data;
+              auto &covar = eofdata[varID][levelID].covar;
 
               if (eofdata[varID][levelID].first_call)
                 {
@@ -509,18 +441,11 @@ EOFs(void *process)
 
                   if (grid_space)
                     {
-                      if (npack)
-                        {
-                          eig_val = (double *) Malloc(npack * sizeof(double));
-                          eofdata[varID][levelID].eig_val = eig_val;
-                        }
-
-                      covar = eofdata[varID][levelID].covar;
+                      if (npack) eofdata[varID][levelID].eig_val.resize(npack);
 
                       for (size_t ipack = 0; ipack < npack; ++ipack)
                         {
-                          size_t j;
-                          size_t i = pack[ipack];
+                    	  const size_t i = pack[ipack];
                           for (size_t jpack = 0; jpack < npack; ++jpack)
                             {
                               if (jpack < ipack)
@@ -529,10 +454,10 @@ EOFs(void *process)
                                 }
                               else
                                 {
-                                  j = pack[jpack];
-                                  covar[ipack][jpack] = covar[ipack][jpack] *                        // covariance
-                                                        sqrt(weight[i]) * sqrt(weight[j]) / sum_w /  // weights
-                                                        nts;                                         // number of data contributing
+                                  const auto j = pack[jpack];
+                                  covar[ipack][jpack] = covar[ipack][jpack] *                                  // covariance
+                                                        std::sqrt(weight[i]) * std::sqrt(weight[j]) / sum_w /  // weights
+                                                        nts;  // number of data contributing
                                 }
                             }
                         }
@@ -541,26 +466,22 @@ EOFs(void *process)
                     {
                       if (Options::cdoVerbose) cdoPrint("allocating covar with %i x %i elements | npack=%zu", nts, nts, npack);
 
-                      covar_array = (double *) Malloc(nts * nts * sizeof(double));
-                      covar = (double **) Malloc(nts * sizeof(double *));
-                      for (int i = 0; i < nts; ++i) covar[i] = covar_array + nts * i;
+                      covar.resize(nts);
+                      for (int i = 0; i < nts; ++i) covar[i].resize(nts);
 
-                      eig_val = (double *) Malloc(nts * sizeof(double));
-                      eofdata[varID][levelID].eig_val = eig_val;
-                      eofdata[varID][levelID].covar_array = covar_array;
-                      eofdata[varID][levelID].covar = covar;
+                      eofdata[varID][levelID].eig_val.resize(nts);
 
+                      for (int j1 = 0; j1 < nts; j1++)
+                        {
+                          const auto &df1p = data[j1];
+                          for (int j2 = 0; j2 < j1; j2++) covar[j1][j2] = covar[j2][j1];
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(dynamic)
 #endif
-                      for (int j1 = 0; j1 < nts; j1++)
-                        {
-                          double *df1p = data[j1];
-                          for (int j2 = 0; j2 < j1; j2++) covar[j1][j2] = covar[j2][j1];
                           for (int j2 = j1; j2 < nts; j2++)
                             {
-                              double *df2p = data[j2];
-                              double sum = 0;
+                              const auto &df2p = data[j2];
+                              double sum = 0.0;
                               for (size_t i = 0; i < npack; i++) sum += weight[pack[i]] * df1p[i] * df2p[i];
                               covar[j1][j2] = sum / sum_w / nts;
                             }
@@ -571,40 +492,38 @@ EOFs(void *process)
 
                   if (Options::Timer) timer_stop(timer_cov);
 
-                  /* SOLVE THE EIGEN PROBLEM */
+                  // Solve the eigen problem
                   if (Options::Timer) timer_start(timer_eig);
 
+                  auto &eig_val = eofdata[varID][levelID].eig_val;
                   if (eigen_mode == JACOBI)
                     // TODO: use return status (>0 okay, -1 did not converge at all)
-                    parallel_eigen_solution_of_symmetric_matrix(covar, eig_val, n, __func__);
+                    cdo::parallel_eigen_solution_of_symmetric_matrix(covar, eig_val, n, __func__);
                   else
-                    eigen_solution_of_symmetric_matrix(covar, eig_val, n, __func__);
+                    cdo::eigen_solution_of_symmetric_matrix(covar, eig_val, n, __func__);
 
                   if (Options::Timer) timer_stop(timer_eig);
-                  /* NOW: covar contains the eigenvectors, eig_val the eigenvalues */
+                  // NOW: covar contains the eigenvectors, eig_val the eigenvalues
 
                   for (size_t i = 0; i < gridsize; ++i) out[i] = missval;
 
                   // for ( int i = 0; i < n; i++ ) eig_val[i] *= sum_w;
                 }  // first_call
-              else
-                {
-                  covar = eofdata[varID][levelID].covar;
-                  eig_val = eofdata[varID][levelID].eig_val;
-                }
 
               if (tsID < n_eig)
                 {
                   if (grid_space)
-                    scale_eigvec_grid(out, tsID, npack, pack.data(), weight.data(), covar, sum_w);
+                    scale_eigvec_grid(out, tsID, npack, pack, weight, covar, sum_w);
                   else if (time_space)
-                    scale_eigvec_time(out, tsID, nts, npack, pack.data(), weight.data(), covar, data, missval, sum_w);
+                    scale_eigvec_time(out, tsID, nts, npack, pack, weight, covar, data, missval, sum_w);
 
-                  nmiss = arrayNumMV(gridsize, out, missval);
+                  nmiss = varrayNumMV(gridsize, out, missval);
                   cdoDefRecord(streamID3, varID, levelID);
-                  cdoWriteRecord(streamID3, out, nmiss);
+                  cdoWriteRecord(streamID3, out.data(), nmiss);
                 }  // loop n_eig
 
+              auto eig_val = eofdata[varID][levelID].eig_val.data();
+
               nmiss = (DBL_IS_EQUAL(eig_val[tsID], missval)) ? 1 : 0;
 
               cdoDefRecord(streamID2, varID, levelID);
@@ -613,26 +532,6 @@ EOFs(void *process)
         }      // loop nvars
     }
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-
-      for (levelID = 0; levelID < nlevs; levelID++)
-        {
-          if (eofdata[varID][levelID].eig_val) Free(eofdata[varID][levelID].eig_val);
-          if (eofdata[varID][levelID].covar_array) Free(eofdata[varID][levelID].covar_array);
-          if (eofdata[varID][levelID].covar) Free(eofdata[varID][levelID].covar);
-          if (time_space && eofdata[varID][levelID].data)
-            {
-              for (tsID = 0; tsID < nts; tsID++)
-                if (eofdata[varID][levelID].data[tsID]) Free(eofdata[varID][levelID].data[tsID]);
-              Free(eofdata[varID][levelID].data);
-            }
-        }
-    }
-
-  Free(in);
-
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
