@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,14 @@
 #include <time.h>
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "dmemory.h"
+#include "functs.h"
+#include "process_int.h"
+#include "griddes.h"
 #include "commandline.h"
+#include "cdo_cdi_wrapper.h"
+#include "cdo_default_values.h"
 
 #ifdef HAVE_LIBNETCDF
 #include "netcdf.h"
@@ -121,15 +127,14 @@ import_e5ml(const char *filename, VAR **vars)
   size_t count[3];
   int nlon, nlat, nlev, nlevp1, nvct, nsp, i, iv;
   int gridIDgp, gridIDsp, zaxisIDml, zaxisIDsfc;
-  int gridtype, zaxistype;
+  int gridtype;
   int nc_file_id;
   char filetype[256];
   size_t attlen;
   double *xvals, *yvals, *vct, *levs;
 
-  /* open file and check file type */
-  /* nce(nc_open(filename, NC_NOWRITE, &nc_file_id)); */
-  nc_file_id = cdf_openread(filename);
+  // open file and check file type
+  nc_file_id = cdo_cdf_openread(filename);
 
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "file_type", filetype));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "file_type", &attlen));
@@ -207,9 +212,6 @@ import_e5ml(const char *filename, VAR **vars)
       nvals = 0;
 
       gridtype = (*vars)[iv].gridtype;
-      zaxistype = (*vars)[iv].zaxistype;
-
-      UNUSED(zaxistype);
 
       if (gridtype == GRID_GAUSSIAN)
         {
@@ -273,8 +275,8 @@ import_e5ml(const char *filename, VAR **vars)
   nce(nc_inq_varid(nc_file_id, "STP", &nc_var_id));
   nce(nc_get_vara_double(nc_file_id, nc_var_id, start, count, (*vars)[nvars_ml].ptr));
 
-  /*close input file */
-  nce(nc_close(nc_file_id));
+  // close input file
+  cdo_cdf_close(nc_file_id);
 
   nvars = nvars_ml + 1;
 
@@ -298,7 +300,7 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
   int lon, lat;
   int nlon, nlat, nlev, nlevp1 /*, nvct*/, nsp, n2, i, nvclev;
   int lat_dimid, lon_dimid, nlev_dimid, nlevp1_dimid, nsp_dimid, nvclev_dimid, n2_dimid;
-  int gridIDgp = -1, gridIDsp, zaxisIDml = -1;
+  int gridIDgp = -1, zaxisIDml = -1;
   int gridtype, zaxistype;
   int nc_file_id;
   int nc_stpid, lspid;
@@ -351,8 +353,6 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
         }
       else if (gridtype == GRID_SPECTRAL && nsp == 0)
         {
-          gridIDsp = vars[varid].gridID;
-          UNUSED(gridIDsp);
           nsp = gridInqSize(vars[varid].gridID);
           nsp = nsp / 2;
         }
@@ -637,21 +637,19 @@ export_e5ml(const char *filename, VAR *vars, int nvars, int64_t vdate, int vtime
 static void
 read_gg3d(int nc_file_id, const char *name, VAR *var, int gridID, int zaxisID)
 {
-  int nlev, nlat, nlon, gridsize, i;
-  int gridtype, zaxistype;
   int nc_var_id;
   size_t start[3], count[3];
 
-  gridtype = gridInqType(gridID);
-  zaxistype = zaxisInqType(zaxisID);
+  int gridtype = gridInqType(gridID);
+  int zaxistype = zaxisInqType(zaxisID);
 
   inivar(var, gridtype, zaxistype, 0, name, nullptr, nullptr);
 
-  nlon = gridInqXsize(gridID);
-  nlat = gridInqYsize(gridID);
-  nlev = zaxisInqSize(zaxisID);
+  int nlon = gridInqXsize(gridID);
+  int nlat = gridInqYsize(gridID);
+  int nlev = zaxisInqSize(zaxisID);
 
-  gridsize = nlon * nlat;
+  int gridsize = nlon * nlat;
 
   var->gridID = gridID;
   var->zaxisID = zaxisID;
@@ -660,7 +658,7 @@ read_gg3d(int nc_file_id, const char *name, VAR *var, int gridID, int zaxisID)
 
   var->ptr = (double *) Malloc(nlev * gridsize * sizeof(double));
 
-  for (i = 0; i < nlev; i++)
+  for (int i = 0; i < nlev; i++)
     {
       start[0] = 0;
       start[1] = i;
@@ -741,8 +739,8 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
   int lon_dimid, lat_dimid, nhgl_dimid, nlevp1_dimid, spc_dimid, nvclev_dimid;
   int complex_dimid, nmp1_dimid, belowsurface_dimid, lev_dimid, ilev_dimid;
   int /* surface_dimid, height2m_dimid, height10m_dimid,*/ n2_dimid;
-  int lon, lat, nhgl, nlevp1, spc, nvclev;
-  int complex, nmp1, belowsurface, lev, ilev;
+  int lon, lat, nhgl, nlevp1, nvclev;
+  int complex, nmp1, belowsurface, lev;
   int /* surface, height2m, height10m,*/ n2;
   int iatt;
   int attint;
@@ -753,9 +751,8 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
   const int attstringlen = 8192;
   char attstring[8192];
 
-  /* open file and check file type */
-  /* nce(nc_open(filename, NC_NOWRITE, &nc_file_id)); */
-  nc_file_id = cdf_openread(filename);
+  // open file and check file type
+  nc_file_id = cdo_cdf_openread(filename);
 
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "file_type", filetype));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "file_type", &attlen));
@@ -783,9 +780,6 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
 
   nce(nc_inq_dimid(nc_file_id, "spc", &spc_dimid));
   nce(nc_inq_dimlen(nc_file_id, spc_dimid, &dimlen));
-  spc = (int) dimlen;
-
-  UNUSED(spc);
 
   nce(nc_inq_dimid(nc_file_id, "nvclev", &nvclev_dimid));
   nce(nc_inq_dimlen(nc_file_id, nvclev_dimid, &dimlen));
@@ -809,9 +803,7 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
 
   nce(nc_inq_dimid(nc_file_id, "ilev", &ilev_dimid));
   nce(nc_inq_dimlen(nc_file_id, ilev_dimid, &dimlen));
-  ilev = (int) dimlen;
 
-  UNUSED(ilev);
   /*
   nce(nc_inq_dimid(nc_file_id, "surface", &surface_dimid));
   nce(nc_inq_dimlen(nc_file_id, surface_dimid, &surface));
@@ -1097,8 +1089,8 @@ import_e5res(const char *filename, VAR **vars, ATTS *atts)
         }
     }
 
-  /*close input file */
-  nce(nc_close(nc_file_id));
+  // close input file
+  cdo_cdf_close(nc_file_id);
 
   nvars = varid;
 
@@ -1491,27 +1483,24 @@ export_e5res(const char *filename, VAR *vars, int nvars)
 void *
 Echam5ini(void *process)
 {
-  CdoStreamID streamID1, streamID2 = CDO_STREAM_UNDEF;
   int nrecs = 0;
   int varID, levelID;
-  int vlistID1, vlistID2;
   int nvars = 0;
-  int iv, nlev;
-  size_t gridsize;
+  int iv;
   size_t nmiss;
-  int taxisID, tsID;
+  int tsID;
 
   cdoInitialize(process);
 
   // clang-format off
-  int IMPORT_E5ML  = cdoOperatorAdd("import_e5ml",  func_read,  0, nullptr);
-  int IMPORT_E5RES = cdoOperatorAdd("import_e5res", func_read,  0, nullptr);
-  int EXPORT_E5ML  = cdoOperatorAdd("export_e5ml",  func_write, 0, nullptr);
-  int EXPORT_E5RES = cdoOperatorAdd("export_e5res", func_write, 0, nullptr);
+  const auto IMPORT_E5ML  = cdoOperatorAdd("import_e5ml",  func_read,  0, nullptr);
+  const auto IMPORT_E5RES = cdoOperatorAdd("import_e5res", func_read,  0, nullptr);
+  const auto EXPORT_E5ML  = cdoOperatorAdd("export_e5ml",  func_write, 0, nullptr);
+  const auto EXPORT_E5RES = cdoOperatorAdd("export_e5res", func_write, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
   if (operatorID == EXPORT_E5ML && processSelf().m_ID != 0) cdoAbort("This operator can't be linked with other operators!");
 
@@ -1532,7 +1521,7 @@ Echam5ini(void *process)
 
       if (nvars == 0) cdoAbort("Unsupported file type!");
 
-      vlistID2 = vlistCreate();
+      const auto vlistID2 = vlistCreate();
       vlistDefNtsteps(vlistID2, 0);
 
       for (iv = 0; iv < nvars; iv++)
@@ -1543,9 +1532,9 @@ Echam5ini(void *process)
           */
           varID = vlistDefVar(vlistID2, vars[iv].gridID, vars[iv].zaxisID, TIME_CONSTANT);
           if (vars[iv].code > 0) vlistDefVarCode(vlistID2, varID, vars[iv].code);
-          if (vars[iv].name) vlistDefVarName(vlistID2, varID, vars[iv].name);
-          if (vars[iv].longname) vlistDefVarLongname(vlistID2, varID, vars[iv].longname);
-          if (vars[iv].units) vlistDefVarUnits(vlistID2, varID, vars[iv].units);
+          if (vars[iv].name) cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, vars[iv].name);
+          if (vars[iv].longname) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, vars[iv].longname);
+          if (vars[iv].units) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, vars[iv].units);
           vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
         }
 
@@ -1555,12 +1544,12 @@ Echam5ini(void *process)
           cdiDefAttTxt(vlistID2, CDI_GLOBAL, atts.atxtname[iatt], (int) strlen(atts.atxtentry[iatt]) + 1, atts.atxtentry[iatt]);
         }
 
-      taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
+      const auto taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
       vlistDefTaxis(vlistID2, taxisID);
 
       if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
 
-      streamID2 = cdoOpenWrite(1);
+      const auto streamID2 = cdoOpenWrite(1);
 
       cdoDefVlist(streamID2, vlistID2);
 
@@ -1569,8 +1558,8 @@ Echam5ini(void *process)
 
       for (varID = 0; varID < nvars; varID++)
         {
-          gridsize = vars[varID].gridsize;
-          nlev = vars[varID].nlev;
+          auto gridsize = vars[varID].gridsize;
+          auto nlev = vars[varID].nlev;
 
           for (levelID = 0; levelID < nlev; levelID++)
             {
@@ -1586,18 +1575,18 @@ Echam5ini(void *process)
   else if (operfunc == func_write)
     {
       VAR *vars = nullptr;
-      int code, gridID, zaxisID, gridtype, zaxistype, gridsize, nlev;
+      int code, gridID, zaxisID, gridtype, zaxistype;
       char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
       int64_t vdate;
-      int taxisID, vtime;
+      int vtime;
       int ntr = 0;
 
-      streamID1 = cdoOpenRead(0);
+      const auto streamID1 = cdoOpenRead(0);
 
-      vlistID1 = cdoStreamInqVlist(streamID1);
-      taxisID = vlistInqTaxis(vlistID1);
+      const auto vlistID1 = cdoStreamInqVlist(streamID1);
+      const auto taxisID = vlistInqTaxis(vlistID1);
 
-      nvars = vlistNvars(vlistID1);
+      const auto nvars = vlistNvars(vlistID1);
 
       vars = (VAR *) Malloc(nvars * sizeof(VAR));
 
@@ -1658,8 +1647,8 @@ Echam5ini(void *process)
               ntr = gridInqTrunc(gridID);
             }
 
-          gridsize = gridInqSize(gridID);
-          nlev = zaxisInqSize(zaxisID);
+          auto gridsize = gridInqSize(gridID);
+          auto nlev = zaxisInqSize(zaxisID);
 
           if (zaxistype == ZAXIS_HYBRID && nlev == 1) zaxistype = ZAXIS_SURFACE;
 
@@ -1687,9 +1676,7 @@ Echam5ini(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          gridID = vlistInqVarGrid(vlistID1, varID);
-          gridsize = gridInqSize(gridID);
-
+          auto gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           cdoReadRecord(streamID1, vars[varID].ptr + levelID * gridsize, &nmiss);
         }
 
