@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "griddes.h"
 
 void *
@@ -36,33 +37,33 @@ Enlarge(void *process)
 
   operatorCheckArgc(1);
 
-  const int gridID2 = cdoDefineGrid(operatorArgv()[0]);
-  const size_t xsize2 = gridInqXsize(gridID2);
-  const size_t ysize2 = gridInqYsize(gridID2);
+  const auto gridID2 = cdoDefineGrid(cdoOperatorArgv(0));
+  const auto xsize2 = gridInqXsize(gridID2);
+  const auto ysize2 = gridInqYsize(gridID2);
 
   if (Options::cdoVerbose) fprintf(stderr, "gridID2 %d, xsize2 %zu, ysize2 %zu\n", gridID2, xsize2, ysize2);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const size_t gridsize2 = gridInqSize(gridID2);
+  const auto gridsize2 = gridInqSize(gridID2);
   if (gridsize2 < vlistGridsizeMax(vlistID1)) cdoAbort("Gridsize of input stream is greater than new gridsize!");
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> array1(gridsize2);
-  std::vector<double> array2(gridsize2);
+  Varray<double> array1(gridsize2);
+  Varray<double> array2(gridsize2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -77,12 +78,12 @@ Enlarge(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          const double missval = vlistInqVarMissval(vlistID1, varID);
-          const int gridID1 = vlistInqVarGrid(vlistID1, varID);
-          const size_t gridsize1 = gridInqSize(gridID1);
+          const auto missval = vlistInqVarMissval(vlistID1, varID);
+          const auto gridID1 = vlistInqVarGrid(vlistID1, varID);
+          const auto gridsize1 = gridInqSize(gridID1);
 
-          size_t xsize1 = gridInqXsize(gridID1);
-          size_t ysize1 = gridInqYsize(gridID1);
+          auto xsize1 = gridInqXsize(gridID1);
+          auto ysize1 = gridInqYsize(gridID1);
           if (xsize1 == 0) xsize1 = 1;
           if (ysize1 == 0) ysize1 = 1;
 
@@ -114,7 +115,7 @@ Enlarge(void *process)
             }
           else
             {
-              arrayCopy(gridsize1, array1.data(), array2.data());
+              varrayCopy(gridsize1, array1, array2);
               for (size_t i = gridsize1; i < gridsize2; i++) array2[i] = array1[gridsize1 - 1];
 
               if (nmiss && DBL_IS_EQUAL(array1[gridsize1 - 1], missval)) nmiss += (gridsize2 - gridsize1);
@@ -132,5 +133,5 @@ Enlarge(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
