@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,80 +22,62 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "process_int.h"
 #include <mpim_grid.h>
 #include "griddes.h"
 
 void
-genGridIndex(int gridID1, int gridID2, int *index)
+genGridIndex(int gridID1, int gridID2, std::vector<long> &index)
 {
-  int i1;
+  size_t i1;
 
-  int gridtype1 = gridInqType(gridID1);
-  int gridtype2 = gridInqType(gridID2);
+  const auto gridtype1 = gridInqType(gridID1);
+  const auto gridtype2 = gridInqType(gridID2);
 
-  size_t gridsize2 = gridInqSize(gridID2);
+  const auto gridsize2 = gridInqSize(gridID2);
 
   if (gridtype1 != gridtype2) cdoAbort("Input streams have different grid types!");
 
-  if (index == nullptr) cdoAbort("Internal problem, index not allocated!");
-
   for (size_t i = 0; i < gridsize2; i++) index[i] = -1;
 
   if (gridtype1 == GRID_LONLAT || gridtype1 == GRID_GAUSSIAN)
     {
-      int nlon1 = gridInqXsize(gridID1);
-      int nlat1 = gridInqYsize(gridID1);
+      const auto nlon1 = gridInqXsize(gridID1);
+      const auto nlat1 = gridInqYsize(gridID1);
 
-      int nlon2 = gridInqXsize(gridID2);
-      int nlat2 = gridInqYsize(gridID2);
+      const auto nlon2 = gridInqXsize(gridID2);
+      const auto nlat2 = gridInqYsize(gridID2);
 
-      if (!(gridInqXvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr))) cdoAbort("Grid 1 has no values!");
+      if (!gridHasCoordinates(gridID1)) cdoAbort("Grid 1 has no values!");
+      if (!gridHasCoordinates(gridID2)) cdoAbort("Grid 2 has no values!");
 
-      if (!(gridInqXvals(gridID2, nullptr) && gridInqYvals(gridID2, nullptr))) cdoAbort("Grid 2 has no values!");
-
-      std::vector<double> xvals1(nlon1);
-      std::vector<double> yvals1(nlat1);
-      std::vector<double> xvals2(nlon2);
-      std::vector<double> yvals2(nlat2);
-
-      std::vector<int> xindex(nlon2);
-      std::vector<int> yindex(nlat2);
+      Varray<double> xvals1(nlon1), yvals1(nlat1);
+      Varray<double> xvals2(nlon2), yvals2(nlat2);
+      std::vector<long> xindex(nlon2), yindex(nlat2);
 
       gridInqXvals(gridID1, xvals1.data());
       gridInqYvals(gridID1, yvals1.data());
 
-      /* Convert lat/lon units if required */
-      {
-        char units[CDI_MAX_NAME];
-        gridInqXunits(gridID1, units);
-        grid_to_degree(units, nlon1, xvals1.data(), "grid1 center lon");
-        gridInqYunits(gridID1, units);
-        grid_to_degree(units, nlat1, yvals1.data(), "grid1 center lat");
-      }
+      // Convert lat/lon units if required
+      cdo_grid_to_degree(gridID1, CDI_XAXIS, nlon1, xvals1.data(), "grid1 center lon");
+      cdo_grid_to_degree(gridID1, CDI_YAXIS, nlat1, yvals1.data(), "grid1 center lat");
 
       gridInqXvals(gridID2, xvals2.data());
       gridInqYvals(gridID2, yvals2.data());
 
-      /* Convert lat/lon units if required */
-      {
-        char units[CDI_MAX_NAME];
-        gridInqXunits(gridID2, units);
-        grid_to_degree(units, nlon2, xvals2.data(), "grid2 center lon");
-        gridInqYunits(gridID2, units);
-        grid_to_degree(units, nlat2, yvals2.data(), "grid2 center lat");
-      }
+      // Convert lat/lon units if required
+      cdo_grid_to_degree(gridID2, CDI_XAXIS, nlon2, xvals2.data(), "grid2 center lon");
+      cdo_grid_to_degree(gridID2, CDI_YAXIS, nlat2, yvals2.data(), "grid2 center lat");
 
-      for (int i2 = 0; i2 < nlat2; i2++)
+      for (size_t i2 = 0; i2 < nlat2; i2++)
         {
           for (i1 = 0; i1 < nlat1; i1++)
             if (std::fabs(yvals2[i2] - yvals1[i1]) < 0.001) break;
 
-          yindex[i2] = (i1 == nlat1) ? -1 : i1;
+          yindex[i2] = (i1 == nlat1) ? -1 : (long)i1;
         }
 
-      for (int i2 = 0; i2 < nlon2; i2++)
+      for (size_t i2 = 0; i2 < nlon2; i2++)
         {
           for (i1 = 0; i1 < nlon1; i1++)
             if (std::fabs(xvals2[i2] - xvals1[i1]) < 0.001) break;
@@ -114,7 +96,7 @@ genGridIndex(int gridID1, int gridID2, int *index)
                 }
             }
 
-          xindex[i2] = (i1 == nlon1) ? -1 : i1;
+          xindex[i2] = (i1 == nlon1) ? -1 : (long)i1;
         }
       /*
       for ( i2 = 0; i2 < nlon2; i2++ )
@@ -123,9 +105,9 @@ genGridIndex(int gridID1, int gridID2, int *index)
       for ( i2 = 0; i2 < nlat2; i2++ )
         printf("y %d %d\n", i2, yindex[i2]);
       */
-      int k = 0;
-      for (int j = 0; j < nlat2; j++)
-        for (int i = 0; i < nlon2; i++)
+      size_t k = 0;
+      for (size_t j = 0; j < nlat2; j++)
+        for (size_t i = 0; i < nlon2; i++)
           {
             if (xindex[i] == -1 || yindex[j] == -1)
               index[k++] = -1;
@@ -148,13 +130,13 @@ Enlargegrid(void *process)
   if (operatorArgc() < 1) cdoAbort("Too few arguments!");
   if (operatorArgc() > 2) cdoAbort("Too many arguments!");
 
-  int gridID2 = cdoDefineGrid(operatorArgv()[0]);
+  const auto gridID2 = cdoDefineGrid(cdoOperatorArgv(0));
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
 
   int ndiffgrids = 0;
   for (int index = 1; index < vlistNgrids(vlistID1); index++)
@@ -162,26 +144,22 @@ Enlargegrid(void *process)
 
   if (ndiffgrids > 0) cdoAbort("Too many different grids in %s!", cdoGetStreamName(0));
 
-  int gridID1 = vlistGrid(vlistID1, 0);
+  const auto gridID1 = vlistGrid(vlistID1, 0);
 
-  size_t gridsize1 = gridInqSize(gridID1);
-  size_t gridsize2 = gridInqSize(gridID2);
+  const auto gridsize1 = gridInqSize(gridID1);
+  const auto gridsize2 = gridInqSize(gridID2);
 
-  std::vector<double> array1(gridsize1);
-  std::vector<double> array2(gridsize2);
-  std::vector<int> gindex(gridsize1);
+  Varray<double> array1(gridsize1), array2(gridsize2);
+  std::vector<long> gindex(gridsize1);
 
-  genGridIndex(gridID2, gridID1, gindex.data());
+  genGridIndex(gridID2, gridID1, gindex);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int ngrids = vlistNgrids(vlistID1);
-  for (int index = 0; index < ngrids; index++)
-    {
-      vlistChangeGridIndex(vlistID2, index, gridID2);
-    }
+  const auto ngrids = vlistNgrids(vlistID1);
+  for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   vlistDefTaxis(vlistID2, taxisID2);
   cdoDefVlist(streamID2, vlistID2);
@@ -199,13 +177,13 @@ Enlargegrid(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          double missval1 = vlistInqVarMissval(vlistID1, varID);
+          const auto missval1 = vlistInqVarMissval(vlistID1, varID);
 
           for (size_t i = 0; i < gridsize2; i++) array2[i] = missval1;
           for (size_t i = 0; i < gridsize1; i++)
             if (gindex[i] >= 0) array2[gindex[i]] = array1[i];
 
-          nmiss = arrayNumMV(gridsize2, array2.data(), missval1);
+          nmiss = varrayNumMV(gridsize2, array2, missval1);
           cdoDefRecord(streamID2, varID, levelID);
           cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
