@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 
 #include <cdi.h>
 
-#include "dmemory.h"
 #include "functs.h"
 #include "process_int.h"
 #include "cdo_vlist.h"
@@ -47,20 +46,20 @@ enum TDATA_TYPE
 #define DEBUG_ROC 0
 enum CONTINGENCY_TYPE
 {
-  TP, // TP - True positive  ( event     forecast and     occured)  HIT
-  FP, // FP - False positive ( event     forecast and not occured)  false ALARM
-  FN, // FN - False negative ( event not forecast and     ocurred)  MISSED
-  TN  // TN - True negative  ( event not forecast and not ocurred)  CORRECT REJECTION
+  TP,  // TP - True positive  ( event     forecast and     occured)  HIT
+  FP,  // FP - False positive ( event     forecast and not occured)  false ALARM
+  FN,  // FN - False negative ( event not forecast and     ocurred)  MISSED
+  TN   // TN - True negative  ( event not forecast and not ocurred)  CORRECT REJECTION
 };
 
 enum ROC_ENUM_TYPE
 {
-  TPR, // TPR = True Positive Rate = TP / ( TP + FN )
-  FPR  // FNR = False Negtive Rate = FN / ( FP + TN )
+  TPR,  // TPR = True Positive Rate = TP / ( TP + FN )
+  FPR   // FNR = False Negtive Rate = FN / ( FP + TN )
 };
 
 static double
-roc_curve_integrate(const double **roc, const int n)
+roc_curve_integrate(const Varray2D<double> &roc, const int n)
 {
   double area = 0;
 
@@ -83,7 +82,7 @@ roc_curve_integrate(const double **roc, const int n)
 void *
 Ensstat3(void *process)
 {
-  int i, j;
+  int j;
   int nrecs = 0, nrecs0;
   size_t nmiss = 0;
   int cum;
@@ -93,12 +92,8 @@ Ensstat3(void *process)
   int gridID, gridID2;
   int have_miss = 0;
   CdoStreamID streamID = 0, streamID2 = 0;
-  int **array2 = nullptr;
-  int **ctg_tab = nullptr, *hist = nullptr;  // contingency table and histogram
   double missval;
-  double *dat;                                    // pointer to ensemble data for ROC
-  double *uThresh = nullptr, *lThresh = nullptr;  // thresholds for histograms
-  double **roc = nullptr;                         // receiver operating characteristics table
+  double *dat;                            // pointer to ensemble data for ROC
   double val;
   int ival;
 
@@ -106,7 +101,7 @@ Ensstat3(void *process)
   {
     CdoStreamID streamID;
     int vlistID;
-    double *array;
+    Varray<double> array;
   };
 
   cdoInitialize(process);
@@ -133,12 +128,12 @@ Ensstat3(void *process)
 
   if (Options::cdoVerbose) cdoPrint("Ensemble over %d files.", nfiles);
 
-  const char *ofilename = cdoGetStreamName(nfiles);
+  auto ofilename = cdoGetStreamName(nfiles);
 
   if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
     cdoAbort("Outputfile %s already exists!", ofilename);
 
-  ens_file_t *ef = (ens_file_t *) Malloc(nfiles * sizeof(ens_file_t));
+  std::vector<ens_file_t> ef(nfiles);
 
   /* *************************************************** */
   /* should each thread be allocating memory locally???? */
@@ -146,7 +141,7 @@ Ensstat3(void *process)
   /* --> #pragma omp parallel for ...                    */
   /* *************************************************** */
   FieldVector field(Threading::ompNumThreads);
-  for (i = 0; i < Threading::ompNumThreads; i++)
+  for (int i = 0; i < Threading::ompNumThreads; i++)
     {
       field[i].resize(nfiles);
       field[i].weightv.resize(nfiles);
@@ -155,7 +150,7 @@ Ensstat3(void *process)
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      streamID = cdoOpenRead((fileID));
+      streamID = cdoOpenRead(fileID);
       vlistID = cdoStreamInqVlist(streamID);
       ef[fileID].streamID = streamID;
       ef[fileID].vlistID = vlistID;
@@ -167,13 +162,15 @@ Ensstat3(void *process)
   const auto vlistID1 = ef[0].vlistID;
   const auto vlistID2 = vlistCreate();
   const auto nvars = vlistNvars(vlistID1);
-  int *varID2 = (int *) Malloc(nvars * sizeof(int));
+  std::vector<int> varIDs2(nvars);
 
-  double *levs = (double *) Calloc(nfiles, sizeof(double));
   const auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
-  for (i = 0; i < nfiles; i++) levs[i] = i;
-  zaxisDefLevels(zaxisID2, levs);
-  zaxisDefName(zaxisID2, "histogram_binID");
+  {
+    Varray<double> levs(nfiles, 0);
+    for (int i = 0; i < nfiles; i++) levs[i] = i;
+    zaxisDefLevels(zaxisID2, levs.data());
+    cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_NAME, "histogram_binID");
+  }
 
   int time_mode = datafunc == TIME ? TIME_VARYING : TIME_CONSTANT;
 
@@ -195,7 +192,7 @@ Ensstat3(void *process)
       else  // datafunc == SPACE
         gridID2 = vlistInqVarGrid(vlistID1, varID);
 
-      varID2[varID] = vlistDefVar(vlistID2, gridID2, zaxisID2, time_mode);
+      varIDs2[varID] = vlistDefVar(vlistID2, gridID2, zaxisID2, time_mode);
     }
 
   const int taxisID1 = vlistInqTaxis(vlistID1);
@@ -215,42 +212,47 @@ Ensstat3(void *process)
 
   if (operfunc != func_roc)
     {
-      streamID2 = cdoOpenWrite((nfiles));
+      streamID2 = cdoOpenWrite(nfiles);
       cdoDefVlist(streamID2, vlistID2);
     }
 
   auto gridsize = vlistGridsizeMax(vlistID1);
 
-  for (int fileID = 0; fileID < nfiles; fileID++) ef[fileID].array = (double *) Malloc(gridsize * sizeof(double));
+  for (int fileID = 0; fileID < nfiles; fileID++) ef[fileID].array.resize(gridsize);
 
+  std::vector<std::vector<int>> array2(nfiles + 1);
   if (operfunc == func_rank && datafunc == SPACE)
     {  // need to memorize data for entire grid before writing
-      array2 = (int **) Malloc((nfiles + 1) * sizeof(int *));
-      for (binID = 0; binID < nfiles; binID++) array2[binID] = (int *) Calloc(gridsize, sizeof(int));
+      for (binID = 0; binID < nfiles; binID++) array2[binID].resize(gridsize, 0);
     }
   else if (operfunc == func_rank)
     {  // can process data separately for each timestep and only need to cumulate values over the grid
-      array2 = (int **) Malloc((nfiles + 1) * sizeof(int *));
-      for (binID = 0; binID < nfiles; binID++) array2[binID] = (int *) Calloc(1, sizeof(int));
+      for (binID = 0; binID < nfiles; binID++) array2[binID].resize(1, 0);
     }
 
+  std::vector<int> hist;
+  Varray2D<double> roc;             // receiver operating characteristics table
+  Varray2D<int> ctg_tab;            // contingency table and histogram
+  Varray<double> uThresh, lThresh;  // thresholds for histograms
   if (operfunc == func_roc)
     {
-      ctg_tab = (int **) Malloc((nbins + 1) * sizeof(int *));
-      hist = (int *) Malloc(nbins * sizeof(int));
-      uThresh = (double *) Malloc(nbins * sizeof(double));
-      lThresh = (double *) Malloc(nbins * sizeof(double));
-      roc = (double **) Malloc((nbins + 1) * sizeof(double *));
+      hist.resize(nbins);
 
-      for (i = 0; i < nbins; i++)
+      roc.resize(nbins + 1);
+      ctg_tab.resize(nbins + 1);
+      for (int i = 0; i <= nbins; i++)
+        {
+          roc[i].resize(2, 0);
+          ctg_tab[i].resize(4, 0);
+        }
+
+      uThresh.resize(nbins);
+      lThresh.resize(nbins);
+      for (int i = 0; i < nbins; i++)
         {
-          ctg_tab[i] = (int *) Calloc(4, sizeof(int));
-          roc[i] = (double *) Calloc(2, sizeof(double));
           uThresh[i] = ((double) i + 1) / nbins;
           lThresh[i] = (double) i / nbins;
         }
-      ctg_tab[nbins] = (int *) Calloc(4, sizeof(int));
-      roc[nbins] = (double *) Calloc(2, sizeof(double));
     }
 
   int tsID = 0;
@@ -288,7 +290,7 @@ Ensstat3(void *process)
             {
               streamID = ef[fileID].streamID;
               cdoInqRecord(streamID, &varID, &levelID);
-              cdoReadRecord(streamID, ef[fileID].array, &nmiss);
+              cdoReadRecord(streamID, ef[fileID].array.data(), &nmiss);
             }
 
           gridID = vlistInqVarGrid(vlistID1, varID);
@@ -311,8 +313,8 @@ Ensstat3(void *process)
               have_miss = 0;
               for (int fileID = 0; fileID < nfiles; fileID++)
                 {
-                  field[ompthID].vec[fileID] = ef[fileID].array[i];
-                  if (DBL_IS_EQUAL(field[ompthID].vec[fileID], missval))
+                  field[ompthID].vec_d[fileID] = ef[fileID].array[i];
+                  if (DBL_IS_EQUAL(field[ompthID].vec_d[fileID], missval))
                     {
                       have_miss = 1;
                       break;
@@ -330,7 +332,7 @@ Ensstat3(void *process)
                       /* RANK HISTOGRAM */
                       /* ************** */
                       // for ( j=0; j<nfiles; j++ )
-                      //   fprintf(stderr,"%5.2g ",field[ompthID].vec[j]);
+                      //   fprintf(stderr,"%5.2g ",field[ompthID].vec_d[j]);
 #ifdef _OPENMP
 #pragma omp critical
 #endif
@@ -347,8 +349,8 @@ Ensstat3(void *process)
                       /* ********************************** */
                       /* RECEIVER OPERATING CHARACTERISTICS */
                       /* ********************************** */
-                      dat = &field[ompthID].vec[1];
-                      ival = field[ompthID].vec[0] > 0.5 ? 1 : 0;
+                      dat = &field[ompthID].vec_d[1];
+                      ival = field[ompthID].vec_d[0] > 0.5 ? 1 : 0;
 
                       for (binID = 0; binID < nbins; binID++) hist[binID] = 0;
 
@@ -403,7 +405,7 @@ Ensstat3(void *process)
                 {
                   val = (double) array2[binID][0];
                   //		fprintf(stderr,"%i ",(int)val);
-                  cdoDefRecord(streamID2, varID2[varID], binID);
+                  cdoDefRecord(streamID2, varIDs2[varID], binID);
                   cdoWriteRecord(streamID2, &val, nmiss);
                 }
               // fprintf(stderr,"\n");
@@ -429,7 +431,7 @@ Ensstat3(void *process)
                               ctg_tab[binID][0], ctg_tab[binID][1], ctg_tab[binID][2], ctg_tab[binID][3], tpr, fpr);
                     }
                   fprintf(stderr, "nbins %10i\n", nbins);
-                  fprintf(stderr, "#ROC CurveArea: %10.6f\n", roc_curve_integrate((const double **) roc, nbins));
+                  fprintf(stderr, "#ROC CurveArea: %10.6f\n", roc_curve_integrate(roc, nbins));
                 }  // if ( DEBUG_ROC )
             }      // else if (operfunc == func_roc )
         }          // for ( recID=0; recID<nrecs; recID++ )
@@ -444,9 +446,9 @@ Ensstat3(void *process)
 
       for (binID = 0; binID < nfiles; binID++)
         {
-          for (i = 0; i < osize; i++) tmpdoub[i] = (double) array2[binID][i];
+          for (int i = 0; i < osize; i++) tmpdoub[i] = (double) array2[binID][i];
 
-          cdoDefRecord(streamID2, varID2[varID], binID);
+          cdoDefRecord(streamID2, varIDs2[varID], binID);
           cdoWriteRecord(streamID2, tmpdoub.data(), nmiss);
         }
     }
@@ -454,7 +456,7 @@ Ensstat3(void *process)
     {
       fprintf(stdout, "#             :     TP     FP     FN     TN         TPR        FPR\n");
 
-      for (i = 0; i <= nbins; i++)
+      for (int i = 0; i <= nbins; i++)
         {
           const int p = ctg_tab[i][TP] + ctg_tab[i][FN];
           const int n = ctg_tab[i][FP] + ctg_tab[i][TN];
@@ -471,28 +473,13 @@ Ensstat3(void *process)
                   ctg_tab[i][1], ctg_tab[i][2], ctg_tab[i][3], sum, tpr, fpr);
         }
 
-      fprintf(stdout, "#ROC CurveArea: %10.6f\n", roc_curve_integrate((const double **) roc, nbins));
+      fprintf(stdout, "#ROC CurveArea: %10.6f\n", roc_curve_integrate(roc, nbins));
     }
 
-  for (int fileID = 0; fileID < nfiles; fileID++)
-    {
-      streamID = ef[fileID].streamID;
-      cdoStreamClose(streamID);
-    }
+  for (int fileID = 0; fileID < nfiles; fileID++) cdoStreamClose(ef[fileID].streamID);
 
   if (operfunc != func_roc) cdoStreamClose(streamID2);
 
-  for (int fileID = 0; fileID < nfiles; fileID++)
-    if (ef[fileID].array) Free(ef[fileID].array);
-
-  if (ef) Free(ef);
-
-  if (array2)
-    {
-      for (binID = 0; binID < nfiles; binID++) Free(array2[binID]);
-      Free(array2);
-    }
-
   cdoFinish();
 
   return nullptr;
