@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,13 +25,15 @@
      Score for Ensemble Prediction Systems, in: Weather and Forecasting (15) pp. 559-570
 */
 
+#include <algorithm> // sort
+
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_options.h"
 #include "param_conversion.h"
-#include "cdo_math.h"
-
-#include "merge_sort2.h"
 #include "util_files.h"
 
 enum OPERTYPE
@@ -47,6 +49,7 @@ enum RESTYPE_BRS
   BRS_RESOL,
   BRS_UNCTY
 };
+
 enum RESTYPE_CRPS
 {
   CRPS_RES,
@@ -57,43 +60,34 @@ enum RESTYPE_CRPS
 void *
 Ensval(void *process)
 {
-  int i, k;
-  int nrecs = 0, nrecs0, nostreams = 0, ngrids;
+  int k;
+  int nrecs = 0, nostreams = 0, ngrids;
   size_t nmiss;
   int levelID = -1, varID = -1;
   size_t gridsize = 0;
   int vlistID;
-  int gridID = -1;
   int have_miss = 0;
   int stream = 0;
   CdoStreamID streamID = 0;
-  // int xsize,ysize;
   double missval = 0;
-  double xval = 0;
-  double yval = 0;
-  double xa;
   double sum_weights = 0;
   double crps_reli = 0, crps_pot = 0, crps = 0;
   double heavyside0, heavysideN;
   double brs_reli, brs_resol, brs_uncty, brs_thresh = 0;
-  double g, o, p;
 
   int fileID;
-  char file_suffix[32];
   char type_suffix[10];
 
   struct ens_file_t
   {
     CdoStreamID streamID;
     int vlistID;
-    std::vector<double> array;
+    Varray<double> array;
   };
 
-  // INITIALIZE POINTERS
-  std::vector<double> alpha, beta, alpha_weights, beta_weights;
-  std::vector<double> brs_g, brs_o;
-  std::vector<double> weights;
-  double *x = nullptr;
+  Varray<double> alpha, beta, alpha_weights, beta_weights;
+  Varray<double> brs_g, brs_o;
+  Varray<double> weights;
   // int vlistCheck, gridsizeCheck;
 
   cdoInitialize(process);
@@ -101,11 +95,11 @@ Ensval(void *process)
   cdoOperatorAdd("enscrps", CRPS, 0, nullptr);
   cdoOperatorAdd("ensbrs", BRS, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  int nfiles = cdoStreamCnt() - 1;
-  int nens = nfiles - 1;
+  const auto nfiles = cdoStreamCnt();
+  const auto nens = nfiles - 1;
 
   if (operfunc == CRPS)
     {
@@ -115,22 +109,20 @@ Ensval(void *process)
     {
       operatorInputArg("Threshold for Brier score?");
       operatorCheckArgc(1);
-      brs_thresh = parameter2double(operatorArgv()[0]);
+      brs_thresh = parameter2double(cdoOperatorArgv(0));
       nostreams = 4;
 
       fprintf(stderr, "brs_thres %10.6f\n", brs_thresh);
     }
 
   // allocate array to hold results
-  std::vector<double> r(nostreams);
+  Varray<double> r(nostreams);
 
   // one stream for each value of the decomposition
   std::vector<CdoStreamID> streamID2(nostreams);
-  std::vector<int> vlistID2(nostreams);
-  std::vector<int> taxisID2(nostreams);
-  std::vector<int> zaxisID2(nostreams);
+  std::vector<int> vlistID2(nostreams), taxisID2(nostreams), zaxisID2(nostreams);
 
-  std::vector<double> val(nfiles, 0);
+  Varray<double> val(nens, 0);
 
   if (operfunc == CRPS)
     {
@@ -144,50 +136,52 @@ Ensval(void *process)
       brs_g.resize(nens + 1, 0);
       brs_o.resize(nens + 1, 0);
     }
-  if (Options::cdoVerbose) cdoPrint("Ensemble over %d files (Ensstat5).", nfiles - 1);
+
+  if (Options::cdoVerbose) cdoPrint("Ensemble over %d files.", nens);
 
   std::vector<ens_file_t> ef(nfiles);
 
   for (fileID = 0; fileID < nfiles; fileID++)
     {
-      streamID = cdoOpenRead((fileID));
+      streamID = cdoOpenRead(fileID);
       vlistID = cdoStreamInqVlist(streamID);
 
       ef[fileID].streamID = streamID;
       ef[fileID].vlistID = vlistID;
     }
 
-  CdoStreamID streamID1 = ef[0].streamID;
+  const auto streamID1 = ef[0].streamID;
 
   if (Options::cdoVerbose) cdoPrint("Opened %i Input Files for Ensemble Operator", nfiles);
 
-  /* check for identical contents of all ensemble members */
-  int nvars = vlistNvars(ef[0].vlistID);
+  // check for identical contents of all ensemble members
+  const auto nvars = vlistNvars(ef[0].vlistID);
   if (Options::cdoVerbose) cdoPrint("nvars %i", nvars);
 
   for (fileID = 1; fileID < nfiles; fileID++) vlistCompare(ef[0].vlistID, ef[fileID].vlistID, CMP_ALL);
 
-  int vlistID1 = ef[0].vlistID;
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int zaxisID1 = vlistInqVarZaxis(vlistID1, 0);
+  const auto vlistID1 = ef[0].vlistID;
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto zaxisID1 = vlistInqVarZaxis(vlistID1, 0);
 
-  int gridID2 = gridCreate(GRID_LONLAT, 1);
+  const auto gridID2 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID2, 1);
   gridDefYsize(gridID2, 1);
+  double xval = 0, yval = 0;
   gridDefXvals(gridID2, &xval);
   gridDefYvals(gridID2, &yval);
 
-  const char *ofilebase = cdoGetStreamName(nfiles);
+  auto ofilebase = cdoGetObase();
 
-  const char *refname = cdoGetObase();
-  memset(file_suffix, 0, sizeof(file_suffix));
-  cdoGenFileSuffix(file_suffix, sizeof(file_suffix), cdoInqFiletype(streamID1), vlistID1, refname);
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   for (stream = 0; stream < nostreams; stream++)
     {
-      int namelen = strlen(ofilebase) + 9 /*type_suffix*/
-                    + 32                  /*file_suffix*/
-                    + 3 /*separating dots and EOS*/;
+      int namelen = strlen(ofilebase) + 9 // type_suffix
+                    + 32                  // filesuffix
+                    + 3;                  // separating dots and EOS
 
       switch (operfunc)
         {
@@ -210,9 +204,9 @@ Ensval(void *process)
           break;
         }
 
-      char *ofilename = (char *) Calloc(namelen, sizeof(char));
+      char *ofilename = (char *) calloc(namelen, sizeof(char));
 
-      sprintf(ofilename, "%s.%s%s", ofilebase, type_suffix, file_suffix);
+      sprintf(ofilename, "%s.%s%s", ofilebase, type_suffix, filesuffix);
       // fprintf(stderr, "StreamID %i: %s\n", stream, ofilename);
 
       if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
@@ -220,32 +214,28 @@ Ensval(void *process)
 
       streamID2[stream] = cdoOpenWrite(ofilename);
 
-      Free(ofilename);
+      free(ofilename);
 
       zaxisID2[stream] = zaxisDuplicate(zaxisID1);
       taxisID2[stream] = taxisDuplicate(taxisID1);
       vlistID2[stream] = vlistDuplicate(vlistID1);
 
       ngrids = vlistNgrids(vlistID2[stream]);
-      // fprintf(stderr,"ngrids %i\n",ngrids);
-      for (i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID2[stream], i, gridID2);
+      for (int i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID2[stream], i, gridID2);
 
       vlistDefTaxis(vlistID2[stream], taxisID2[stream]);
       cdoDefVlist(streamID2[stream], vlistID2[stream]);
-
-      // vlistCheck = cdoStreamInqVlist(streamID2[stream]);
-      // gridsizeCheck = vlistGridsizeMax(vlistCheck);
-
-      // fprintf(stderr,"stream %i vlist %3i gridsize
-      // %4i\n",stream,vlistCheck,gridsizeCheck);
     }
 
   if (Options::cdoVerbose) cdoPrint(" sum_weights %10.6f", sum_weights);
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   int tsID = 0;
   do
     {
-      nrecs0 = cdoStreamInqTimestep(ef[0].streamID, tsID);
+      const auto nrecs0 = cdoStreamInqTimestep(ef[0].streamID, tsID);
       for (fileID = 1; fileID < nfiles; fileID++)
         {
           streamID = ef[fileID].streamID;
@@ -270,10 +260,8 @@ Ensval(void *process)
 
               if (fileID == 0)
                 {
-                  gridID = vlistInqVarGrid(vlistID1, varID);
-                  gridsize = gridInqSize(gridID);
-                  missval = vlistInqVarMissval(vlistID1, varID);
-                  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));  // vlistGridsizeMax(vlistID1);
+                  gridsize = varList1[varID].gridsize;
+                  missval = varList1[varID].missval;
                   weights.resize(gridsize);
                 }
 
@@ -285,13 +273,15 @@ Ensval(void *process)
           // xsize = gridInqXsize(gridID);
           // ysize = gridInqYsize(gridID);
 
-          /*	  if ( xsize > 1 && ysize > 1 )  {
+          /*
+          if ( xsize > 1 && ysize > 1 )
+          {
             gridWeights(gridID, weights);
             sum_weights=0;
-            for ( i=0; i<gridsize; i++ )
-              sum_weights += weights[i];
+            for ( i=0; i<gridsize; i++ ) sum_weights += weights[i];
           }
-          else*/
+          else
+          */
           {
             for (size_t i = 0; i < gridsize; i++) weights[i] = 1. / gridsize;
             sum_weights = 1.;
@@ -303,26 +293,29 @@ Ensval(void *process)
 
           for (size_t i = 0; i < gridsize; i++)
             {
+              double xa = 0;
               have_miss = 0;
               for (fileID = 0; fileID < nfiles; fileID++)
                 {
-                  val[fileID] = ef[fileID].array[i];
-                  if (DBL_IS_EQUAL(val[fileID], missval))
+                  auto value = ef[fileID].array[i];
+                  if (fileID == 0)
+                    xa = value;             // 1st file contains reference
+                  else
+                    val[fileID-1] = value;  // Ensembles start at 2nd file
+
+                  if (DBL_IS_EQUAL(value, missval))
                     {
                       have_miss = 1;
                       break;
                     }
                 }
 
-              xa = val[0];                  /* 1st file contains reference  */
-              x = &val[1];                  /* Ensembles start at 2nd   file*/
-              sort_iter_single(nens, x, 1); /* Sort The Ensemble Array      */
-                                            /* to ascending order           */
-              // only process if no missing value in ensemble
+              auto &x = val;             
+              std::sort(x.begin(), x.end()); // Sort The Ensemble Array to ascending order
 
+              // only process if no missing value in ensemble
               if (!have_miss && operfunc == CRPS)
                 {
-
                   if (xa < x[0])
                     { /* Consider outliers            */
                       beta[0] += (x[0] - xa) * weights[i];
@@ -335,23 +328,22 @@ Ensval(void *process)
                       heavysideN += 1.;
                     }
 
-                  /* Loop start at zero ==> 1st ensemble (c-indexing) */
+                  // Loop start at zero ==> 1st ensemble (c-indexing)
                   for (k = 0; k < nens - 1; k++)
-                    {                                                   /* Cumulate alpha and beta      */
-                      if (xa > x[k + 1])                                /* left of heavyside            */
-                        alpha[k + 1] += (x[k + 1] - x[k]) * weights[i]; /*                              */
-                      else if (xa < x[k])                               /* right of heavyside           */
-                        beta[k + 1] += (x[k + 1] - x[k]) * weights[i];  /*                              */
+                    {                                                   // Cumulate alpha and beta
+                      if (xa > x[k + 1])                                // left of heavyside
+                        alpha[k + 1] += (x[k + 1] - x[k]) * weights[i];
+                      else if (xa < x[k])                               // right of heavyside
+                        beta[k + 1] += (x[k + 1] - x[k]) * weights[i];
                       else if (x[k + 1] >= xa && xa >= x[k])
-                        {                                              /* hitting jump pf heavyside    */
-                          alpha[k + 1] += (xa - x[k]) * weights[i];    /* (occurs exactly once!) */
-                          beta[k + 1] += (x[k + 1] - xa) * weights[i]; /* **************************** */
+                        {                                               // hitting jump pf heavyside (occurs exactly once!)
+                          beta[k + 1] += (x[k + 1] - xa) * weights[i];
                         }
                     }
                 }
               else if (operfunc == BRS)
                 {
-                  //  int occ = xa > brs_thresh? 1 : 0;
+                  // int occ = xa > brs_thresh? 1 : 0;
 
                   // brs_g[i] - number of enemble members with rank i that
                   // forecast event
@@ -371,8 +363,7 @@ Ensval(void *process)
                           }
                       }
 
-                  // brs_o[i] - number of times that the obs is between
-                  //            Ensemble i-1 and i
+                  // brs_o[i] - number of times that the obs is between Ensemble i-1 and i
                   if (1)
                     {
                       if (x[0] > xa)
@@ -394,15 +385,10 @@ Ensval(void *process)
 
           if (operfunc == CRPS)
             {
-
               // First Bin
-              p = 0.;
-              g = 0.;
-              o = heavyside0 / gridsize;
-              if (o > 0.)
-                {
-                  g = beta[0] / o;
-                }
+              double p = 0., g = 0.;
+              double o = heavyside0 / gridsize;
+              if (o > 0.) g = beta[0] / o;
               crps_reli = g * (o - p) * (o - p);
               crps_pot = g * o * (1. - o);
               crps = g * ((1. - o) * p * p + o * (1. - p) * (1. - p));
@@ -444,15 +430,11 @@ Ensval(void *process)
             }
           else if (operfunc == BRS)
             {
-              double gsum = 0;
-              double obar = 0;
-              double osum = 0;
-              double o, g, p;
-
               brs_reli = 0;
               brs_resol = 0;
               brs_uncty = 0;
 
+              double gsum = 0, obar = 0, osum = 0;
               for (k = 0; k <= nens; k++)
                 {
                   obar += brs_g[k] * brs_o[k];
@@ -465,9 +447,8 @@ Ensval(void *process)
                   cdoAbort("Internal error - normalization constraint of problem not fulfilled");
                   cdoAbort("This is likely due to missing values");
                 }
-              o = 0;
-              p = 0;
-              g = 0;
+
+              double o = 0, p = 0, g = 0;
               brs_uncty = obar * (1 - obar);
 
               for (k = 0; k <= nens; k++)
@@ -490,7 +471,7 @@ Ensval(void *process)
 
               if (Options::cdoVerbose)
                 {
-                  //	      cdoPrint("Brier score for var %i level %i calculated",varID, levelID);
+                  // cdoPrint("Brier score for var %i level %i calculated",varID, levelID);
                   cdoPrint("BRS: obar %12.6g brs  %12.6g reli %12.6g resol %12.6g u %12.6g", obar, brs_reli - brs_resol + brs_uncty,
                            brs_reli, brs_resol, brs_uncty);
                 }
@@ -502,7 +483,7 @@ Ensval(void *process)
           for (stream = 0; stream < nostreams; stream++)
             {
               cdoDefRecord(streamID2[stream], varID, levelID);
-              if (DBL_IS_NAN(r[stream]))
+              if (std::isnan(r[stream]))
                 {
                   r[stream] = missval;
                   have_miss = 1;
@@ -513,14 +494,14 @@ Ensval(void *process)
           switch (operfunc)
             {
             case (CRPS):
-              arrayFill(nens + 1, alpha.data(), 0.0);
-              arrayFill(nens + 1, beta.data(), 0.0);
+              varrayFill(alpha, 0.0);
+              varrayFill(beta, 0.0);
               heavyside0 = 0;
               heavysideN = 0;
               break;
             case (BRS):
-              arrayFill(nens + 1, brs_o.data(), 0.0);
-              arrayFill(nens + 1, brs_g.data(), 0.0);
+              varrayFill(brs_o, 0.0);
+              varrayFill(brs_g, 0.0);
               break;
             }
         }  // for ( int recID = 0; recID < nrecs; recID++ )
@@ -543,7 +524,6 @@ Ensval(void *process)
       zaxisDestroy(zaxisID2[stream]);
     }
 
-  gridDestroy(gridID);
   gridDestroy(gridID2);
 
   cdoFinish();
