@@ -2,7 +2,7 @@
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate model Data.
 
- Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+ Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
  See COPYING file for copying and redistribution conditions.
 
  This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "util_files.h"
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
@@ -31,9 +33,9 @@
 void *
 Eofcoeff(void *process)
 {
-  char eof_name[16], oname[1024], filesuffix[32];
+  char eof_name[16], oname[1024];
   double missval1 = -999, missval2;
-  int i, varID, levelID;
+  int varID, levelID;
   int nrecs;
   size_t nmiss;
 
@@ -43,36 +45,36 @@ Eofcoeff(void *process)
 
   cdoOperatorAdd("eofcoeff", 0, 0, nullptr);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID2);
 
   // taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = taxisDuplicate(taxisID2);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID2);
 
-  const int gridID1 = vlistInqVarGrid(vlistID1, 0);
-  const int gridID2 = vlistInqVarGrid(vlistID2, 0);
+  const auto gridID1 = vlistInqVarGrid(vlistID1, 0);
+  const auto gridID2 = vlistInqVarGrid(vlistID2, 0);
 
-  const size_t gridsize = vlistGridsizeMax(vlistID1);
+  const auto gridsize = vlistGridsizeMax(vlistID1);
   if (gridsize != vlistGridsizeMax(vlistID2))
     cdoAbort("Gridsize of input files does not match! %zu and %zu", gridsize, vlistGridsizeMax(vlistID2));
 
   if (vlistNgrids(vlistID2) > 1 || vlistNgrids(vlistID1) > 1) cdoAbort("Too many different grids in input!");
 
-  const int nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
-  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
+  const auto nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
+  const auto nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
 
   if (gridID1 != gridID2) cdoCompareGrids(gridID1, gridID2);
 
   strcpy(oname, cdoGetObase());
   int nchars = strlen(oname);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   FieldVector3D eof(nvars);
@@ -95,12 +97,12 @@ Eofcoeff(void *process)
           eof[varID][levelID][eofID].grid = gridID1;
           eof[varID][levelID][eofID].missval = missval1;
           eof[varID][levelID][eofID].resize(gridsize);
-          arrayFill(gridsize, eof[varID][levelID][eofID].vec.data(), missval1);
+          varrayFill(gridsize, eof[varID][levelID][eofID].vec_d, missval1);
 
           if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
           if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
 
-          cdoReadRecord(streamID1, eof[varID][levelID][eofID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, eof[varID][levelID][eofID].vec_d.data(), &nmiss);
           eof[varID][levelID][eofID].nmiss = nmiss;
         }
       eofID++;
@@ -119,8 +121,8 @@ Eofcoeff(void *process)
 
   // Create var-list and time-axis for output
 
-  const int ngrids = vlistNgrids(vlistID3);
-  for (i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID3, i, gridID3);
+  const auto ngrids = vlistNgrids(vlistID3);
+  for (int i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID3, i, gridID3);
 
   vlistDefTaxis(vlistID3, taxisID3);
   for (varID = 0; varID < nvars; varID++) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
@@ -144,7 +146,7 @@ Eofcoeff(void *process)
 
   // ALLOCATE temporary fields for data read and write
   Field in, out;
-  in.resize(gridsize );
+  in.resize(gridsize);
   in.grid = gridID1;
   out.missval = missval1;
   out.nmiss = 0;
@@ -167,34 +169,34 @@ Eofcoeff(void *process)
         {
           cdoInqRecord(streamID2, &varID, &levelID);
           missval2 = vlistInqVarMissval(vlistID2, varID);
-          cdoReadRecord(streamID2, in.vec.data(), &in.nmiss);
+          cdoReadRecord(streamID2, in.vec_d.data(), &in.nmiss);
 
           for (eofID = 0; eofID < neof; eofID++)
             {
               if (recID == 0) cdoDefTimestep(streamIDs[eofID], tsID);
               // if ( recID == 0 ) fprintf(stderr, "ts%i rec%i eof%i\n", tsID, recID, eofID);
-              out.vec[0] = 0;
+              out.vec_d[0] = 0;
               out.grid = gridID3;
               out.missval = missval2;
               for (size_t i = 0; i < gridsize; i++)
                 {
-                  if (!DBL_IS_EQUAL(in.vec[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec[i], missval1))
+                  if (!DBL_IS_EQUAL(in.vec_d[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec_d[i], missval1))
                     {
-                      // out.vec[0] += w[i]*in.vec[i]*eof[varID][levelID][eofID].vec[i];
-                      out.vec[0] += in.vec[i] * eof[varID][levelID][eofID].vec[i];
+                      // out.vec_d[0] += w[i]*in.vec_d[i]*eof[varID][levelID][eofID].vec_d[i];
+                      out.vec_d[0] += in.vec_d[i] * eof[varID][levelID][eofID].vec_d[i];
                     }
                 }
-              if (!DBL_IS_EQUAL(out.vec[0], 0.))
+              if (!DBL_IS_EQUAL(out.vec_d[0], 0.))
                 nmiss = 0;
               else
                 {
                   nmiss = 1;
-                  out.vec[0] = missval2;
+                  out.vec_d[0] = missval2;
                 }
 
               cdoDefRecord(streamIDs[eofID], varID, levelID);
-              // fprintf(stderr, "%d %d %d %d %d %g\n", streamIDs[eofID],tsID, recID, varID, levelID,*out.vec.data());
-              cdoWriteRecord(streamIDs[eofID], out.vec.data(), nmiss);
+              // fprintf(stderr, "%d %d %d %d %d %g\n", streamIDs[eofID],tsID, recID, varID, levelID,*out.vec_d.data());
+              cdoWriteRecord(streamIDs[eofID], out.vec_d.data(), nmiss);
             }
           if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
           if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
