@@ -2,7 +2,7 @@
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate model Data.
 
- Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+ Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
  See COPYING file for copying and redistribution conditions.
 
  This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,9 @@
 
 #include "cdi.h"
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "util_files.h"
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
@@ -31,9 +33,9 @@
 void *
 Eofcoeff3d(void *process)
 {
-  char eof_name[16], oname[1024], filesuffix[32];
+  char eof_name[16], oname[1024];
   double missval1 = -999, missval2 = -999;
-  int i, varID, levelID;
+  int varID, levelID;
   int nrecs;
   size_t nmiss;
 
@@ -43,34 +45,34 @@ Eofcoeff3d(void *process)
 
   cdoOperatorAdd("eofcoeff3d", 0, 0, nullptr);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
 
   // taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = taxisDuplicate(taxisID2);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID2);
 
-  const int gridID1 = vlistInqVarGrid(vlistID1, 0);
-  const int gridID2 = vlistInqVarGrid(vlistID2, 0);
+  const auto gridID1 = vlistInqVarGrid(vlistID1, 0);
+  const auto gridID2 = vlistInqVarGrid(vlistID2, 0);
 
-  const size_t gridsize = vlistGridsizeMax(vlistID1);
+  const auto gridsize = vlistGridsizeMax(vlistID1);
   if (gridsize != vlistGridsizeMax(vlistID2)) cdoAbort("Gridsize of input files does not match!");
 
   if (vlistNgrids(vlistID2) > 1 || vlistNgrids(vlistID1) > 1) cdoAbort("Too many different grids in input");
 
-  const int nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
-  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
+  const auto nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
+  const auto nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
 
   if (gridID1 != gridID2) cdoCompareGrids(gridID1, gridID2);
 
   strcpy(oname, cdoGetObase());
   int nchars = strlen(oname);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   FieldVector3D eof(nvars);
@@ -86,19 +88,16 @@ Eofcoeff3d(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
           missval1 = vlistInqVarMissval(vlistID1, varID);
-          if (eofID == 0)
-            eof[varID][levelID].resize(1);
-          else
-            eof[varID][levelID].resize(eofID + 1);
+          eof[varID][levelID].resize(eofID == 0 ? 1 : eofID + 1);
           eof[varID][levelID][eofID].grid = gridID1;
           eof[varID][levelID][eofID].missval = missval1;
           eof[varID][levelID][eofID].resize(gridsize);
-          arrayFill(gridsize, eof[varID][levelID][eofID].vec.data(), missval1);
+          varrayFill(gridsize, eof[varID][levelID][eofID].vec_d, missval1);
 
           if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
           if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
 
-          cdoReadRecord(streamID1, eof[varID][levelID][eofID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, eof[varID][levelID][eofID].vec_d.data(), &nmiss);
           eof[varID][levelID][eofID].nmiss = nmiss;
         }
       eofID++;
@@ -108,7 +107,7 @@ Eofcoeff3d(void *process)
 
   if (Options::cdoVerbose) cdoPrint("%s contains %i eof's", cdoGetStreamName(0), neof);
   // Create 1x1 Grid for output
-  const int gridID3 = gridCreate(GRID_LONLAT, 1);
+  const auto gridID3 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID3, 1);
   gridDefYsize(gridID3, 1);
   double xvals = 0, yvals = 0;
@@ -116,20 +115,20 @@ Eofcoeff3d(void *process)
   gridDefYvals(gridID3, &yvals);
 
   double zvals = 0.;
-  const int zaxisID3 = zaxisCreate(ZAXIS_GENERIC, 1);
+  const auto zaxisID3 = zaxisCreate(ZAXIS_GENERIC, 1);
   zaxisDefLevels(zaxisID3, &zvals);
-  zaxisDefName(zaxisID3, "zaxis_Reduced");
-  zaxisDefLongname(zaxisID3, "Reduced zaxis from EOF3D - only one coefficient per 3D eigenvector and time step");
+  cdiDefKeyString(zaxisID3, CDI_GLOBAL, CDI_KEY_NAME, "zaxis_Reduced");
+  cdiDefKeyString(zaxisID3, CDI_GLOBAL, CDI_KEY_LONGNAME, "Reduced zaxis from EOF3D - only one coefficient per 3D eigenvector and time step");
 
   // Create var-list and time-axis for output
 
-  const int vlistID3 = vlistDuplicate(vlistID2);
+  const auto vlistID3 = vlistDuplicate(vlistID2);
 
-  const int ngrids = vlistNgrids(vlistID3);
-  for (i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID3, i, gridID3);
+  const auto ngrids = vlistNgrids(vlistID3);
+  for (int i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID3, i, gridID3);
 
-  const int nzaxis = vlistNzaxis(vlistID3);
-  for (i = 0; i < nzaxis; i++) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
+  const auto nzaxis = vlistNzaxis(vlistID3);
+  for (int i = 0; i < nzaxis; i++) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
 
   vlistDefTaxis(vlistID3, taxisID3);
   for (varID = 0; varID < nvars; varID++) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
@@ -176,7 +175,7 @@ Eofcoeff3d(void *process)
       for (varID = 0; varID < nvars; varID++)
         for (eofID = 0; eofID < neof; eofID++)
           {
-            out[varID][eofID].vec[0] = 0;
+            out[varID][eofID].vec_d[0] = 0;
             out[varID][eofID].grid = gridID3;
             out[varID][eofID].missval = missval2;
           }
@@ -186,7 +185,7 @@ Eofcoeff3d(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, in.vec.data(), &in.nmiss);
+          cdoReadRecord(streamID2, in.vec_d.data(), &in.nmiss);
           missval2 = vlistInqVarMissval(vlistID2, varID);
 
           for (eofID = 0; eofID < neof; eofID++)
@@ -196,10 +195,11 @@ Eofcoeff3d(void *process)
               nmiss = 0;
               for (size_t i = 0; i < gridsize; i++)
                 {
-                  if (!DBL_IS_EQUAL(in.vec[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec[i], missval1))
+                  if (!DBL_IS_EQUAL(in.vec_d[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec_d[i], missval1))
                     {
-                      // out[varID][eofID].vec[0] += w[i]*in.vec[i]*eof[varID][levelID][eofID].vec[i];
-                      out[varID][eofID].vec[0] += in.vec[i] * eof[varID][levelID][eofID].vec[i];;
+                      // out[varID][eofID].vec_d[0] += w[i]*in.vec_d[i]*eof[varID][levelID][eofID].vec_d[i];
+                      out[varID][eofID].vec_d[0] += in.vec_d[i] * eof[varID][levelID][eofID].vec_d[i];
+                      ;
                     }
                   else
                     nmiss += 1;
@@ -207,7 +207,7 @@ Eofcoeff3d(void *process)
               /*
               if ( nmiss ) {
                 out[varID][eofID].nmiss=1;
-                out[varID][eofID].vec[0] = missval2;
+                out[varID][eofID].vec_d[0] = missval2;
               }
               */
             }
@@ -221,7 +221,7 @@ Eofcoeff3d(void *process)
           for (varID = 0; varID < nvars; varID++)
             {
               cdoDefRecord(streamIDs[eofID], varID, 0);
-              cdoWriteRecord(streamIDs[eofID], out[varID][eofID].vec.data(), out[varID][eofID].nmiss);
+              cdoWriteRecord(streamIDs[eofID], out[varID][eofID].vec_d.data(), out[varID][eofID].nmiss);
             }
         }
 
