@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_options.h"
 
 void *
 EstFreq(void *process)
@@ -28,27 +29,29 @@ EstFreq(void *process)
 
   cdoInitialize(process);
 
-  bool lcopy = unchangedRecord();
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto lcopy = unchangedRecord();
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  int ntsteps = vlistNtsteps(vlistID1);
+  const auto ntsteps = vlistNtsteps(vlistID1);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      const auto gridsizemax = vlistGridsizeMax(vlistID1);
       array.resize(gridsizemax);
     }
 
@@ -93,7 +96,7 @@ EstFreq(void *process)
       tsID++;
     }
 
-  char frequency[CDI_MAX_NAME];
+  const char *frequency = "";
 
   if (ntsteps > 2)
     {
@@ -106,19 +109,17 @@ EstFreq(void *process)
                " time stamp '%d'.\n",
                ntsteps, lyear, fyear);
       double covered_years = lyear - fyear + 1.0;
-      if (DBL_IS_EQUAL(ntsteps / covered_years, 1.))
-        strcpy(frequency, "yr");
-      else if (DBL_IS_EQUAL(ntsteps / covered_years, 12.))
-        strcpy(frequency, "mon");
-      else if (DBL_IS_EQUAL(ntsteps / covered_years, 365.) || DBL_IS_EQUAL(ntsteps / covered_years, 365.25)
-               || DBL_IS_EQUAL(ntsteps / covered_years, 366.))
-        strcpy(frequency, "day");
-      else if (DBL_IS_EQUAL(ntsteps / covered_years, 365. * 4) || DBL_IS_EQUAL(ntsteps / covered_years, 365.25 * 4)
-               || DBL_IS_EQUAL(ntsteps / covered_years, 366. * 4))
-        strcpy(frequency, "6hr");
-      else if (DBL_IS_EQUAL(ntsteps / covered_years, 365. * 8) || DBL_IS_EQUAL(ntsteps / covered_years, 365.25 * 8)
-               || DBL_IS_EQUAL(ntsteps / covered_years, 366. * 8))
-        strcpy(frequency, "3hr");
+      double freq = ntsteps / covered_years;
+      if (DBL_IS_EQUAL(freq, 1.))
+        frequency = "yr";
+      else if (DBL_IS_EQUAL(freq, 12.))
+        frequency = "mon";
+      else if (IS_EQUAL(freq, 365.) || IS_EQUAL(freq, 365.25) || IS_EQUAL(freq, 366.))
+        frequency = "day";
+      else if (IS_EQUAL(freq, 365. * 4) || IS_EQUAL(freq, 365.25 * 4) || IS_EQUAL(freq, 366. * 4))
+        frequency = "6hr";
+      else if (IS_EQUAL(freq, 365. * 8) || IS_EQUAL(freq, 365.25 * 8) || IS_EQUAL(freq, 366. * 8))
+        frequency = "3hr";
       else
         {
           int covered_months = lmonth - fmonth + 1;
@@ -133,16 +134,14 @@ EstFreq(void *process)
                      step_per_year, fyear);
           else
             {
-              if ((double) step_per_year / (double) covered_months > 31 * 8)
-                cdoAbort("Frequency is sub-3hourly! Not yet enabled.");
-              else if ((double) step_per_year / (double) covered_months > 31 * 4)
-                strcpy(frequency, "3hr");
-              else if ((double) step_per_year / (double) covered_months > 31)
-                strcpy(frequency, "6hr");
-              else if ((double) step_per_year / (double) covered_months > 1)
-                strcpy(frequency, "day");
-              else
-                strcpy(frequency, "mon");
+              freq = (double) step_per_year / (double) covered_months;
+              // clang-format off
+              if      (freq > 31 * 8) cdoAbort("Frequency is sub-3hourly! Not yet enabled.");
+              else if (freq > 31 * 4) frequency = "3hr";
+              else if (freq > 31)     frequency = "6hr";
+              else if (freq > 1)      frequency = "day";
+              else                    frequency = "mon";
+              // clang-format on
             }
         }
     }
@@ -159,5 +158,5 @@ EstFreq(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
