@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,39 +24,46 @@
       Exprf      aexprf          Append evaluated expressions from script file
 */
 /*
-Operatoren: +, -, *, \, ^, ==, !=, >, <, >=, <=, <=>, &&, ||, ?:
-Functions: sqrt, exp, log, log10, sin, cos, tan, asin, acos, atan
-Functions: min, max, avg, std, var
-Constansts: M_PI, M_E
+  Operatoren: +, -, *, \, ^, ==, !=, >, <, >=, <=, <=>, &&, ||, ?:
+  Functions: sqrt, exp, log, log10, sin, cos, tan, asin, acos, atan
+  Functions: min, max, avg, std, var
+  Constansts: M_PI, M_E
 */
 
+#include <algorithm>
 #include <sys/stat.h> /* stat */
+#include <cstdlib>
 #include <cassert>
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "cdo_math.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 #include "gridreference.h"
 #include "expr.h"
 #include "datetime.h"
-
+#include "cdo_zaxis.h"
+#include "cdi_lockedIO.h"
 
 void grid_cell_area(int gridID, double *array);
 int getSurfaceID(int vlistID);
-
+struct yy_buffer_state *yy_scan_string(const char *str, void *scanner);
 
 static char *
-exprsFromArgument(int exprArgc, const char **exprArgv)
+exprsFromArgument(const std::vector<std::string> &exprArgv)
 {
   char *exprs = nullptr;
+  const int exprArgc = exprArgv.size();
+  auto firstArg = exprArgv[0].c_str();
 
   if (exprArgc == 1)
     {
-      size_t slen = strlen(exprArgv[0]);
+      const size_t slen = strlen(firstArg);
       exprs = (char *) Malloc(slen + 2);
-      strcpy(exprs, exprArgv[0]);
+      strcpy(exprs, firstArg);
       if (exprs[slen - 1] != ';')
         {
           exprs[slen] = ';';
@@ -66,15 +73,15 @@ exprsFromArgument(int exprArgc, const char **exprArgv)
   else if (exprArgc > 1)
     {
       size_t slen = exprArgc + 1;
-      for (int i = 0; i < exprArgc; ++i) slen += strlen(exprArgv[i]);
+      for (int i = 0; i < exprArgc; ++i) slen += exprArgv[i].size();
       exprs = (char *) Malloc(slen);
-      strcpy(exprs, exprArgv[0]);
-      char *pexprs = exprs + strlen(exprArgv[0]);
+      strcpy(exprs, firstArg);
+      char *pexprs = exprs + strlen(firstArg);
       for (int i = 1; i < exprArgc; ++i)
         {
           *pexprs++ = ',';
-          strcpy(pexprs, exprArgv[i]);
-          pexprs += strlen(exprArgv[i]);
+          strcpy(pexprs, exprArgv[i].c_str());
+          pexprs += exprArgv[i].size();
         }
       if (*pexprs != ';')
         {
@@ -91,14 +98,15 @@ exprsFromArgument(int exprArgc, const char **exprArgv)
 }
 
 static char *
-exprsFromFile(int exprArgc, const char **exprArgv)
+exprsFromFile(const std::vector<std::string> &exprArgv)
 {
+  const int exprArgc = exprArgv.size();
   if (exprArgc != 1) operatorCheckArgc(1);
-  const char *exprf = exprArgv[0];
+  auto exprf = exprArgv[0].c_str();
 
-  /* Open expr script file for reading */
-  FILE *fp = fopen(exprf, "r");
-  if (fp == nullptr) cdoAbort("Open failed on %s", exprf);
+  // Open expr script file for reading
+  auto fp = fopen(exprf, "r");
+  if (!fp) cdoAbort("Open failed on %s", exprf);
 
   struct stat filestat;
   if (stat(exprf, &filestat) != 0) cdoAbort("Stat failed on %s", exprf);
@@ -124,13 +132,13 @@ str_replace(char *target, const char *needle, const char *replacement)
 {
   char buffer[1024] = { 0 };
   char *insert_point = &buffer[0];
-  const char *tmp = target;
-  const size_t needle_len = strlen(needle);
-  const size_t repl_len = strlen(replacement);
+  auto tmp = target;
+  const auto needle_len = strlen(needle);
+  const auto repl_len = strlen(replacement);
 
-  while (1)
+  while (true)
     {
-      const char *p = strstr(tmp, needle);
+      char *p = strstr(tmp, needle);
 
       // walked past last occurrence of needle; copy remaining part
       if (p == nullptr)
@@ -160,7 +168,7 @@ str_replace(char *target, const char *needle, const char *replacement)
 static char *
 exprsExpand(char *exprs, int vlistID)
 {
-  const size_t len = strlen(exprs);
+  const auto len = strlen(exprs);
   unsigned nequal = 0, nsemi = 0;
   for (size_t i = 0; i < len; ++i)
     {
@@ -172,7 +180,7 @@ exprsExpand(char *exprs, int vlistID)
   if (nequal == 0 && nsemi == 1 && strstr(exprs, needle))
     {
       char varname[CDI_MAX_NAME];
-      const int nvars = vlistNvars(vlistID);
+      const auto nvars = vlistNvars(vlistID);
       const size_t bufsize = 1024;
       char *sbuf = (char *) Malloc(bufsize);
       char *buf = (char *) Malloc(nvars * bufsize);
@@ -194,79 +202,87 @@ exprsExpand(char *exprs, int vlistID)
   return exprs;
 }
 
-static paramType *
-params_new(int vlistID)
+static void
+params_init(std::vector<paramType> &params, const VarList &varList, int vlistID)
 {
-  paramType *params = (paramType *) Malloc(MAX_PARAMS * sizeof(paramType));
-  memset(params, 0, MAX_PARAMS * sizeof(paramType));
+  for (int varID = 0; varID < MAX_PARAMS; varID++)
+    {
+      params[varID].data = nullptr;
+      params[varID].name = nullptr;
+      params[varID].longname = nullptr;
+      params[varID].units = nullptr;
+    }
 
-  const int nvars1 = vlistNvars(vlistID);
-
-  char name[CDI_MAX_NAME];
-  char longname[CDI_MAX_NAME];
-  char units[CDI_MAX_NAME];
+  char longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
 
+  const auto nvars1 = vlistNvars(vlistID);
   for (int varID = 0; varID < nvars1; varID++)
     {
-      const int gridID = vlistInqVarGrid(vlistID, varID);
-      const int zaxisID = vlistInqVarZaxis(vlistID, varID);
-      const int datatype = vlistInqVarDatatype(vlistID, varID);
-      const int steptype = vlistInqVarTimetype(vlistID, varID);
-      const size_t nlat = gridInqYsize(gridID);
-      const size_t ngp = gridInqSize(gridID);
-      const size_t nlev = zaxisInqSize(zaxisID);
-      const double missval = vlistInqVarMissval(vlistID, varID);
-
-      vlistInqVarName(vlistID, varID, name);
       vlistInqVarLongname(vlistID, varID, longname);
       vlistInqVarUnits(vlistID, varID, units);
 
       params[varID].type = PARAM_VAR;
+      params[varID].isValid = true;
       params[varID].select = false;
       params[varID].remove = false;
       params[varID].lmiss = true;
       params[varID].coord = 0;
-      params[varID].gridID = gridID;
-      params[varID].zaxisID = zaxisID;
-      params[varID].datatype = datatype;
-      params[varID].steptype = steptype;
-      params[varID].nlat = nlat;
-      params[varID].ngp = ngp;
-      params[varID].nlev = nlev;
-      params[varID].missval = missval;
+      params[varID].gridID = varList[varID].gridID;
+      params[varID].zaxisID = varList[varID].zaxisID;
+      params[varID].datatype = varList[varID].datatype;
+      params[varID].steptype = varList[varID].timetype;
+      params[varID].nlat = gridInqYsize(varList[varID].gridID);
+      params[varID].ngp = varList[varID].gridsize;
+      params[varID].nlev = varList[varID].nlevels;
+      params[varID].missval = varList[varID].missval;
       params[varID].nmiss = 0;
       params[varID].data = nullptr;
-      params[varID].name = strdup(name);
+      params[varID].name = strdup(varList[varID].name);
       params[varID].longname = strdup(longname);
       params[varID].units = strdup(units);
     }
-
-  return params;
 }
 
 static void
-params_add_coord(parseParamType *parse_arg, int coord, int cdiID, size_t size, const char *units, const char *longname)
+params_delete(std::vector<paramType> &params)
 {
-  const int ncoords = parse_arg->ncoords;
-  if (ncoords >= parse_arg->maxcoords) cdoAbort("Too many coordinates (limit=%d)", parse_arg->maxcoords);
+  for (int varID = 0; varID < MAX_PARAMS; varID++)
+    {
+      if (params[varID].data) Free(params[varID].data);
+      if (params[varID].name) Free(params[varID].name);
+      if (params[varID].longname) Free(params[varID].longname);
+      if (params[varID].units) Free(params[varID].units);
+    }
+}
 
-  parse_arg->coords[ncoords].needed = false;
-  parse_arg->coords[ncoords].coord = coord;
-  parse_arg->coords[ncoords].cdiID = cdiID;
-  parse_arg->coords[ncoords].size = size;
-  parse_arg->coords[ncoords].units = nullptr;
-  parse_arg->coords[ncoords].longname = nullptr;
-  parse_arg->coords[ncoords].data = nullptr;
-  if (units) parse_arg->coords[ncoords].units = strdup(units);
-  if (longname) parse_arg->coords[ncoords].longname = strdup(longname);
+static void
+params_add_coord(parseParamType &parse_arg, int coord, int cdiID, size_t size, const char *units, const char *longname)
+{
+  const auto ncoords = parse_arg.ncoords;
+  if (ncoords >= parse_arg.maxcoords) cdoAbort("Too many coordinates (limit=%d)", parse_arg.maxcoords);
 
-  parse_arg->ncoords++;
+  parse_arg.coords[ncoords].needed = false;
+  parse_arg.coords[ncoords].coord = coord;
+  parse_arg.coords[ncoords].cdiID = cdiID;
+  parse_arg.coords[ncoords].size = size;
+  if (units)
+    {
+      parse_arg.coords[ncoords].units.resize(strlen(units) + 1);
+      strcpy(parse_arg.coords[ncoords].units.data(), units);
+    }
+  if (longname)
+    {
+      parse_arg.coords[ncoords].longname.resize(strlen(longname) + 1);
+      strcpy(parse_arg.coords[ncoords].longname.data(), longname);
+    }
+
+  parse_arg.ncoords++;
 }
 
 int
 params_get_coordID(parseParamType *parse_arg, int coord, int cdiID)
 {
-  const int ncoords = parse_arg->ncoords;
+  const auto ncoords = parse_arg->ncoords;
   for (int coordID = 0; coordID < ncoords; ++coordID)
     {
       if (parse_arg->coords[coordID].coord == coord && parse_arg->coords[coordID].cdiID == cdiID) return coordID;
@@ -278,98 +294,82 @@ params_get_coordID(parseParamType *parse_arg, int coord, int cdiID)
 }
 
 static void
-params_add_coordinates(int vlistID, parseParamType *parse_arg)
+params_add_coordinates(int vlistID, parseParamType &parse_arg)
 {
-  char longname[CDI_MAX_NAME];
-  char units[CDI_MAX_NAME];
+  char longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
 
-  const int ngrids = vlistNgrids(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
-      const int gridID = vlistGrid(vlistID, index);
-      const size_t size = gridInqSize(gridID);
-      gridInqXunits(gridID, units);
+      const auto gridID = vlistGrid(vlistID, index);
+      const auto size = gridInqSize(gridID);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, units, &length);
       params_add_coord(parse_arg, 'x', gridID, size, units, "longitude");
-      gridInqYunits(gridID, units);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, units, &length);
       params_add_coord(parse_arg, 'y', gridID, size, units, "latitude");
 
       params_add_coord(parse_arg, 'a', gridID, size, "m^2", "grid cell area");
       params_add_coord(parse_arg, 'w', gridID, size, nullptr, "grid cell area weights");
     }
 
-  const int nzaxis = vlistNzaxis(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; ++index)
     {
-      const int zaxisID = vlistZaxis(vlistID, index);
-      const int size = zaxisInqSize(zaxisID);
-      zaxisInqUnits(zaxisID, units);
-      zaxisInqLongname(zaxisID, longname);
+      const auto zaxisID = vlistZaxis(vlistID, index);
+      const auto size = zaxisInqSize(zaxisID);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, units, &length);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, longname, &length);
       params_add_coord(parse_arg, 'z', zaxisID, size, units, longname);
     }
 }
 
 static int
-params_add_ts(parseParamType *parse_arg)
+params_add_ts(parseParamType &parse_arg)
 {
-  int varID = -1;
-  paramType *params = parse_arg->params;
-  if (params)
-    {
-      varID = parse_arg->nparams;
-      if (varID >= parse_arg->maxparams) cdoAbort("Too many parameter (limit=%d)", parse_arg->maxparams);
+  auto &params = parse_arg.params;
 
-      params[varID].name = strdup("_ts");
-      params[varID].gridID = parse_arg->pointID;
-      params[varID].zaxisID = parse_arg->surfaceID;
-      params[varID].steptype = TIME_VARYING;
-      params[varID].ngp = CLEN;
-      params[varID].nlev = 1;
+  auto varID = parse_arg.nparams;
+  if (varID >= parse_arg.maxparams) cdoAbort("Too many parameter (limit=%d)", parse_arg.maxparams);
 
-      parse_arg->nparams++;
-      parse_arg->cnparams++;
-    }
+  params[varID].name = strdup("_ts");
+  params[varID].gridID = parse_arg.pointID;
+  params[varID].zaxisID = parse_arg.surfaceID;
+  params[varID].steptype = TIME_VARYING;
+  params[varID].ngp = CLEN;
+  params[varID].nlev = 1;
+
+  parse_arg.nparams++;
+  parse_arg.cnparams++;
 
   return varID;
 }
 
 static void
-params_delete(paramType *params)
+parseParamInit(parseParamType &parse_arg, int vlistID, int pointID, int zonalID, int surfaceID)
 {
-  if (params)
-    {
-      for (int varID = 0; varID < MAX_PARAMS; varID++)
-        {
-          if (params[varID].data) Free(params[varID].data);
-          if (params[varID].name) Free(params[varID].name);
-          if (params[varID].longname) Free(params[varID].longname);
-          if (params[varID].units) Free(params[varID].units);
-        }
-      Free(params);
-    }
-}
+  const auto nvars = vlistNvars(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
+  const auto maxcoords = ngrids * 4 + nzaxis;
 
-static void
-parseParamInit(parseParamType *parse_arg, int vlistID, int pointID, int zonalID, int surfaceID, paramType *params)
-{
-  const int nvars = vlistNvars(vlistID);
-  const int ngrids = vlistNgrids(vlistID);
-  const int nzaxis = vlistNzaxis(vlistID);
-  const int maxcoords = ngrids * 4 + nzaxis;
-
-  parse_arg->maxparams = MAX_PARAMS;
-  parse_arg->nparams = nvars;
-  parse_arg->cnparams = nvars;
-  parse_arg->nvars1 = nvars;
-  parse_arg->init = true;
-  parse_arg->debug = Options::cdoVerbose != 0;
-  parse_arg->params = params;
-  parse_arg->pointID = pointID;
-  parse_arg->zonalID = zonalID;
-  parse_arg->surfaceID = surfaceID;
-  parse_arg->needed = (bool *) Malloc(nvars * sizeof(bool));
-  parse_arg->coords = (coordType *) Malloc(maxcoords * sizeof(coordType));
-  parse_arg->maxcoords = maxcoords;
-  parse_arg->ncoords = 0;
+  parse_arg.maxparams = MAX_PARAMS;
+  parse_arg.params.resize(MAX_PARAMS);
+  parse_arg.nparams = nvars;
+  parse_arg.cnparams = nvars;
+  parse_arg.nvars1 = nvars;
+  parse_arg.init = true;
+  parse_arg.debug = Options::cdoVerbose != 0;
+  parse_arg.pointID = pointID;
+  parse_arg.zonalID = zonalID;
+  parse_arg.surfaceID = surfaceID;
+  parse_arg.needed.resize(nvars);
+  parse_arg.coords.resize(maxcoords);
+  parse_arg.maxcoords = maxcoords;
+  parse_arg.ncoords = 0;
 }
 
 static int
@@ -377,11 +377,11 @@ genZonalID(int vlistID)
 {
   int zonalID = -1;
 
-  const int ngrids = vlistNgrids(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID = vlistGrid(vlistID, index);
-      const int gridtype = gridInqType(gridID);
+      const auto gridID = vlistGrid(vlistID, index);
+      const auto gridtype = gridInqType(gridID);
       if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GENERIC)
         if (gridInqXsize(gridID) > 1 && gridInqYsize(gridID) >= 1)
           {
@@ -393,58 +393,99 @@ genZonalID(int vlistID)
   return zonalID;
 }
 
-void *
-Expr(void *process)
+static
+void setDateAndTime(paramType &varts, int calendar, int tsID, int64_t vdate0, int vtime0, int64_t vdate, int vtime)
 {
-  cdoInitialize(process);
+  double jdelta = 0.0;
 
-  void *scanner;
-  int yy_scan_string(const char *str, void *scanner);
-  yylex_init(&scanner);
+  if (tsID)
+    {
+      const auto juldate0 = julianDateEncode(calendar, vdate0, vtime0);
+      const auto juldate = julianDateEncode(calendar, vdate, vtime);
+      jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
+    }
 
-  parseParamType parse_arg;
-  yyset_extra(&parse_arg, scanner);
+  varts.data[CTIMESTEP] = tsID + 1;
+  varts.data[CDATE] = vdate;
+  varts.data[CTIME] = vtime;
+  varts.data[CDELTAT] = jdelta;
 
+  int year, mon, day;
+  int hour, minute, second;
+  cdiDecodeDate(vdate, &year, &mon, &day);
+  cdiDecodeTime(vtime, &hour, &minute, &second);
+
+  varts.data[CDAY] = day;
+  varts.data[CMONTH] = mon;
+  varts.data[CYEAR] = year;
+  varts.data[CSECOND] = second;
+  varts.data[CMINUTE] = minute;
+  varts.data[CHOUR] = hour;
+}
+
+static void
+addOperators(void)
+{
   // clang-format off
   cdoOperatorAdd("expr",   1, 1, "expressions");
   cdoOperatorAdd("exprf",  1, 0, "expr script filename");
   cdoOperatorAdd("aexpr",  0, 1, "expressions");
   cdoOperatorAdd("aexprf", 0, 0, "expr script filename");
   // clang-format on
+}
 
-  const int operatorID = cdoOperatorID();
+void *
+Expr(void *process)
+{
+  cdoInitialize(process);
+
+  void *scanner = nullptr;
+  yylex_init(&scanner);
+
+  parseParamType parse_arg = {};
+  yyset_extra(&parse_arg, scanner);
+
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
   const bool replacesVariables = cdoOperatorF1(operatorID);
   const bool readsCommandLine = cdoOperatorF2(operatorID);
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  int exprArgc = operatorArgc();
-  const char **exprArgv = (const char **) operatorArgv();
+  auto exprArgv = cdoGetOperArgv();
 
-  char *exprs = readsCommandLine ? exprsFromArgument(exprArgc, exprArgv) : exprsFromFile(exprArgc, exprArgv);
+  auto exprs = readsCommandLine ? exprsFromArgument(exprArgv) : exprsFromFile(exprArgv);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
   exprs = exprsExpand(exprs, vlistID1);
   if (Options::cdoVerbose) cdoPrint(exprs);
 
-  const int nvars1 = vlistNvars(vlistID1);
+  const auto nvars1 = vlistNvars(vlistID1);
 
-  const int pointID = gridCreate(GRID_GENERIC, 1);
-  const int zonalID = genZonalID(vlistID1);
-  const int surfaceID = getSurfaceID(vlistID1);
+  const auto pointID = gridCreate(GRID_GENERIC, 1);
+  const auto zonalID = genZonalID(vlistID1);
+  const auto surfaceID = getSurfaceID(vlistID1);
 
-  paramType *params = params_new(vlistID1);
+  parseParamInit(parse_arg, vlistID1, pointID, zonalID, surfaceID);
 
-  parseParamInit(&parse_arg, vlistID1, pointID, zonalID, surfaceID, params);
+  auto &params = parse_arg.params;
+  params_init(params, varList1, vlistID1);
 
   // Set all input variables to 'needed' if replacing is switched off
   for (int varID = 0; varID < nvars1; varID++) parse_arg.needed[varID] = !replacesVariables;
 
-  const int vartsID = params_add_ts(&parse_arg);
+  // init function rand()
+  std::srand(Options::Random_Seed);
+
+  const auto vartsID = params_add_ts(parse_arg);
   parse_arg.tsID = vartsID;
-  params_add_coordinates(vlistID1, &parse_arg);
+  params_add_coordinates(vlistID1, parse_arg);
 
   CDO_parser_errorno = 0;
   yy_scan_string(exprs, scanner);
@@ -464,11 +505,11 @@ Expr(void *process)
 
   std::vector<int> varIDmap(parse_arg.nparams);
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
+  vlistClearFlag(vlistID1);
   if (!replacesVariables)
     {
-      vlistClearFlag(vlistID1);
       int pidx = 0;
       for (int varID = 0; varID < nvars1; varID++)
         {
@@ -476,13 +517,13 @@ Expr(void *process)
           if (!params[varID].remove)
             {
               varIDmap[pidx++] = varID;
-              const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              // printf("Replace %d nlevs %d\n", varID, nlevs);
-              for (int levID = 0; levID < nlevs; levID++) vlistDefFlag(vlistID1, varID, levID, true);
+              const auto nlevels = varList1[varID].nlevels;
+              // printf("Replace %d nlevs %d\n", varID, nlevels);
+              for (int levID = 0; levID < nlevels; levID++) vlistDefFlag(vlistID1, varID, levID, true);
             }
         }
-      cdoVlistCopyFlag(vlistID2, vlistID1);
     }
+  cdoVlistCopyFlag(vlistID2, vlistID1);  // Copy global attributes
 
   //  printf("parse_arg.nparams %d\n", parse_arg.nparams);
   for (int pidx = 0; pidx < parse_arg.nparams; pidx++)
@@ -497,18 +538,18 @@ Expr(void *process)
         }
 
       // printf("gridID %d zaxisID %d\n",  params[pidx].gridID, params[pidx].zaxisID);
-      const int varID = vlistDefVar(vlistID2, params[pidx].gridID, params[pidx].zaxisID, params[pidx].steptype);
-      vlistDefVarName(vlistID2, varID, params[pidx].name);
+      const auto varID = vlistDefVar(vlistID2, params[pidx].gridID, params[pidx].zaxisID, params[pidx].steptype);
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, params[pidx].name);
       // printf("add: %d %s %d levs %d\n", pidx,  params[pidx].name, varID, zaxisInqSize(params[pidx].zaxisID));
       if (params[pidx].lmiss) vlistDefVarMissval(vlistID2, varID, params[pidx].missval);
-      if (params[pidx].units) vlistDefVarUnits(vlistID2, varID, params[pidx].units);
-      if (params[pidx].longname) vlistDefVarLongname(vlistID2, varID, params[pidx].longname);
-      const size_t len = strlen(params[pidx].name);
+      if (params[pidx].units) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, params[pidx].units);
+      if (params[pidx].longname) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, params[pidx].longname);
+      const auto len = strlen(params[pidx].name);
       if (len > 3 && memcmp(params[pidx].name, "var", 3) == 0)
         {
           if (isdigit(params[pidx].name[3]))
             {
-              const int code = atoi(params[pidx].name + 3);
+              const auto code = atoi(params[pidx].name + 3);
               vlistDefVarCode(vlistID2, varID, code);
             }
         }
@@ -522,25 +563,21 @@ Expr(void *process)
       cdoPrint("vlistNvars(vlistID1)=%d, vlistNvars(vlistID2)=%d", vlistNvars(vlistID1), vlistNvars(vlistID2));
     }
 
-  const int nvars2 = vlistNvars(vlistID2);
+  const auto nvars2 = vlistNvars(vlistID2);
   if (nvars2 == 0) cdoAbort("No output variable found!");
 
   for (int varID = 0; varID < nvars1; varID++)
     {
       if (parse_arg.needed[varID])
         {
-          const size_t ngp = params[varID].ngp;
-          const size_t nlev = params[varID].nlev;
-          const size_t nItems = cdo::max((size_t) 4, ngp * nlev);
+          const auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
           params[varID].data = (double *) Malloc(nItems * sizeof(double));
         }
     }
 
   for (int varID = parse_arg.nvars1; varID < parse_arg.nparams; varID++)
     {
-      const size_t ngp = params[varID].ngp;
-      const size_t nlev = params[varID].nlev;
-      const size_t nItems = cdo::max((size_t) 4, ngp * nlev);
+      const auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
       params[varID].data = (double *) Malloc(nItems * sizeof(double));
     }
 
@@ -548,60 +585,54 @@ Expr(void *process)
     {
       if (parse_arg.coords[i].needed)
         {
-          const int coord = parse_arg.coords[i].coord;
+          const auto coord = parse_arg.coords[i].coord;
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
-              int gridID = parse_arg.coords[i].cdiID;
-              const size_t ngp = parse_arg.coords[i].size;
-              double *data = (double *) Malloc(ngp * sizeof(double));
-              parse_arg.coords[i].data = data;
+              auto gridID = parse_arg.coords[i].cdiID;
+              const auto ngp = parse_arg.coords[i].size;
+              parse_arg.coords[i].data.resize(ngp);
+              auto &cdata = parse_arg.coords[i].data;
               if (coord == 'x' || coord == 'y')
                 {
-                  const int gridtype = gridInqType(gridID);
+                  const auto gridtype = gridInqType(gridID);
                   if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
                   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
 
-                  if (gridtype == GRID_UNSTRUCTURED)
+                  if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID))
                     {
-                      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-                        {
-                          if (gridInqNumber(gridID) > 0)
-                            {
-                              gridID = referenceToGrid(gridID);
-                              if (gridID == -1) cdoAbort("Reference to source grid not found!");
-                            }
-                        }
+                      const auto reference = dereferenceGrid(gridID);
+                      if (reference.isValid) gridID = reference.gridID;
+                      if (reference.notFound) cdoAbort("Reference to source grid not found!");
                     }
 
-                  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-                    cdoAbort("Cell center coordinates missing!");
+                  if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
 
-                  if (coord == 'x') gridInqXvals(gridID, data);
-                  if (coord == 'y') gridInqYvals(gridID, data);
+                  if (coord == 'x') gridInqXvals(gridID, cdata.data());
+                  if (coord == 'y') gridInqYvals(gridID, cdata.data());
 
                   if (gridID != parse_arg.coords[i].cdiID) gridDestroy(gridID);
                 }
               else if (coord == 'a')
                 {
-                  grid_cell_area(gridID, data);
+                  grid_cell_area(gridID, cdata.data());
                 }
               else if (coord == 'w')
                 {
-                  data[0] = 1;
+                  cdata[0] = 1;
                   if (ngp > 1)
                     {
-                      int wstatus = gridWeights(gridID, data);
+                      int wstatus = gridWeights(gridID, cdata.data());
                       if (wstatus) cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
                     }
                 }
             }
           else if (coord == 'z')
             {
-              const int zaxisID = parse_arg.coords[i].cdiID;
-              const size_t nlev = parse_arg.coords[i].size;
-              double *data = (double *) Malloc(nlev * sizeof(double));
-              parse_arg.coords[i].data = data;
-              cdoZaxisInqLevels(zaxisID, data);
+              const auto zaxisID = parse_arg.coords[i].cdiID;
+              const auto nlev = parse_arg.coords[i].size;
+              parse_arg.coords[i].data.resize(nlev);
+              auto &cdata = parse_arg.coords[i].data;
+              cdoZaxisInqLevels(zaxisID, cdata.data());
             }
           else
             cdoAbort("Computation of coordinate %c not implemented!", coord);
@@ -610,93 +641,69 @@ Expr(void *process)
 
   for (int varID = parse_arg.nvars1; varID < parse_arg.nparams; varID++)
     {
-      const int coord = params[varID].coord;
+      const auto coord = params[varID].coord;
       if (coord)
         {
-          char *varname = strdup(params[varID].name);
-          varname[strlen(varname) - 2] = 0;
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
-              const int coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
-              const int gridID = parse_arg.coords[coordID].cdiID;
-              const size_t ngp = parse_arg.coords[coordID].size;
-              double *data = parse_arg.coords[coordID].data;
+              const auto coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
+              const auto gridID = parse_arg.coords[coordID].cdiID;
+              const auto ngp = parse_arg.coords[coordID].size;
+              auto &cdata = parse_arg.coords[coordID].data;
               assert(gridID == params[varID].gridID);
-              assert(data != nullptr);
+              assert(!cdata.empty());
 
-              arrayCopy(ngp, data, params[varID].data);
+              arrayCopy(ngp, cdata.data(), params[varID].data);
             }
           else if (coord == 'z')
             {
-              const int coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
-              const int zaxisID = parse_arg.coords[coordID].cdiID;
-              const size_t nlev = parse_arg.coords[coordID].size;
-              double *data = parse_arg.coords[coordID].data;
+              const auto coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
+              const auto zaxisID = parse_arg.coords[coordID].cdiID;
+              const auto nlev = parse_arg.coords[coordID].size;
+              auto &cdata = parse_arg.coords[coordID].data;
               assert(zaxisID == params[varID].zaxisID);
-              assert(data != nullptr);
+              assert(!cdata.empty());
 
-              arrayCopy(nlev, data, params[varID].data);
+              arrayCopy(nlev, cdata.data(), params[varID].data);
             }
           else
             cdoAbort("Computation of coordinate %c not implemented!", coord);
-
-          free(varname);
         }
     }
 
   if (Options::cdoVerbose) vlistPrint(vlistID2);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int64_t vdate0 = 0;
   int vtime0 = 0;
-  const int calendar = taxisInqCalendar(taxisID1);
+  const auto calendar = taxisInqCalendar(taxisID1);
 
-  int nrecs;
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
 
-      double jdelta = 0;
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
-      if (tsID)
-        {
-          JulianDate juldate0 = julianDateEncode(calendar, vdate0, vtime0);
-          JulianDate juldate = julianDateEncode(calendar, vdate, vtime);
-          jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
-        }
+      setDateAndTime(params[vartsID], calendar, tsID, vdate0, vtime0, vdate, vtime);
 
       vdate0 = vdate;
       vtime0 = vtime;
 
-      params[vartsID].data[CTIMESTEP] = tsID + 1;
-      params[vartsID].data[CDATE] = vdate;
-      params[vartsID].data[CTIME] = vtime;
-      params[vartsID].data[CDELTAT] = jdelta;
-
-      int year, mon, day;
-      int hour, minute, second;
-      cdiDecodeDate(vdate, &year, &mon, &day);
-      cdiDecodeTime(vtime, &hour, &minute, &second);
-
-      params[vartsID].data[CDAY] = day;
-      params[vartsID].data[CMONTH] = mon;
-      params[vartsID].data[CYEAR] = year;
-      params[vartsID].data[CSECOND] = second;
-      params[vartsID].data[CMINUTE] = minute;
-      params[vartsID].data[CHOUR] = hour;
-
       taxisCopyTimestep(taxisID2, taxisID1);
 
       cdoDefTimestep(streamID2, tsID);
 
+      //for (int varID = 0; varID < nvars1; varID++) printf(">>> %s %d\n", params[varID].name, params[varID].isValid);
+      for (int varID = 0; varID < nvars1; varID++) params[varID].isValid = true;
       for (int varID = 0; varID < nvars1; varID++)
         if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].nmiss = 0;
 
@@ -706,23 +713,23 @@ Expr(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           if (parse_arg.needed[varID])
             {
-              const size_t offset = params[varID].ngp * levelID;
-              double *vardata = params[varID].data + offset;
+              const auto offset = params[varID].ngp * levelID;
+              auto vardata = params[varID].data + offset;
               size_t nmiss;
               cdoReadRecord(streamID1, vardata, &nmiss);
               params[varID].nmiss += nmiss;
+
+              if (nmiss > 0) cdo_check_missval(params[varID].missval, params[varID].name);
             }
         }
 
       for (int varID = 0; varID < nvars2; varID++)
         {
-          const int pidx = varIDmap[varID];
+          const auto pidx = varIDmap[varID];
           if (pidx < nvars1) continue;
-          const size_t ngp = params[pidx].ngp;
-          const size_t nlev = params[pidx].nlev;
 
           params[pidx].nmiss = 0;
-          arrayFill(ngp * nlev, params[pidx].data, 0.0);
+          varrayFill(params[pidx].ngp * params[pidx].nlev, params[pidx].data, 0.0);
         }
 
       parse_arg.cnparams = vartsID + 1;
@@ -731,19 +738,19 @@ Expr(void *process)
 
       for (int varID = 0; varID < nvars2; varID++)
         {
-          const int pidx = varIDmap[varID];
+          const auto pidx = varIDmap[varID];
 
           if (tsID > 0 && params[pidx].steptype == TIME_CONSTANT) continue;
 
-          const double missval = vlistInqVarMissval(vlistID2, varID);
+          const auto missval = vlistInqVarMissval(vlistID2, varID);
 
-          const size_t ngp = params[pidx].ngp;
-          const int nlev = (int) params[pidx].nlev;
+          const auto ngp = params[pidx].ngp;
+          const auto nlev = (int) params[pidx].nlev;
           for (int levelID = 0; levelID < nlev; levelID++)
             {
-              const size_t offset = ngp * levelID;
+              const auto offset = ngp * levelID;
               double *vardata = params[pidx].data + offset;
-              const size_t nmiss = arrayNumMV(ngp, vardata, missval);
+              const auto nmiss = arrayNumMV(ngp, vardata, missval);
               cdoDefRecord(streamID2, varID, levelID);
               cdoWriteRecord(streamID2, vardata, nmiss);
             }
@@ -763,18 +770,6 @@ Expr(void *process)
 
   params_delete(params);
 
-  if (parse_arg.needed) Free(parse_arg.needed);
-  if (parse_arg.coords)
-    {
-      for (int i = 0; i < parse_arg.ncoords; i++)
-        {
-          if (parse_arg.coords[i].data) Free(parse_arg.coords[i].data);
-          if (parse_arg.coords[i].units) Free(parse_arg.coords[i].units);
-          if (parse_arg.coords[i].longname) Free(parse_arg.coords[i].longname);
-        }
-      Free(parse_arg.coords);
-    }
-
   gridDestroy(pointID);
   if (zonalID != -1) gridDestroy(zonalID);
 
