@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include "cdo_wtime.h"
 #include <mpim_grid.h>
@@ -31,6 +31,7 @@
 #include "cdo_options.h"
 #include "progress.h"
 #include "cimdOmp.h"
+#include "matrix_view.h"
 
 void
 fillmiss(Field &field1, Field &field2, int nfill)
@@ -43,27 +44,19 @@ fillmiss(Field &field1, Field &field2, int nfill)
   double s1, s2;
   double xr, xu, xl, xo;
 
-  int gridID = field1.grid;
-  size_t nmiss1 = field1.nmiss;
-  double missval = field1.missval;
-  double *array1 = field1.vec.data();
-  double *array2 = field2.vec.data();
+  const auto gridID = field1.grid;
+  const auto nmiss1 = field1.nmiss;
+  const auto missval = field1.missval;
 
-  int nx = gridInqXsize(gridID);
-  int ny = gridInqYsize(gridID);
-  bool globgrid = (bool) gridIsCircular(gridID);
+  const int nx = gridInqXsize(gridID);
+  const int ny = gridInqYsize(gridID);
+  const auto globgrid = (bool) gridIsCircular(gridID);
 
-  int gridtype = gridInqType(gridID);
+  const auto gridtype = gridInqType(gridID);
   if (!(gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)) cdoAbort("Unsupported grid type: %s!", gridNamePtr(gridtype));
 
-  double **matrix1 = (double **) Malloc(ny * sizeof(double *));
-  double **matrix2 = (double **) Malloc(ny * sizeof(double *));
-
-  for (j = 0; j < ny; j++)
-    {
-      matrix1[j] = array1 + j * nx;
-      matrix2[j] = array2 + j * nx;
-    }
+  MatrixView<double> matrix1(field1.vec_d.data(), ny, nx);
+  MatrixView<double> matrix2(field2.vec_d.data(), ny, nx);
 
   for (j = 0; j < ny; j++)
     for (i = 0; i < nx; i++)
@@ -73,7 +66,7 @@ fillmiss(Field &field1, Field &field2, int nfill)
             nmiss2++;
 
             kr = ku = kl = ko = 0;
-            xr = xu = xl = xo = 0.;
+            xr = xu = xl = xo = 0.0;
 
             for (ir = i + 1; ir < nx; ir++)
               if (!DBL_IS_EQUAL(matrix1[j][ir], missval))
@@ -129,52 +122,21 @@ fillmiss(Field &field1, Field &field2, int nfill)
                   break;
                 }
 
-            /*  printf("%d %d %d %d %d %d %g %g %g %g\n",
-             * j,i,kr,kl,ku,ko,xr,xl,xu,xo);*/
+            // printf("%d %d %d %d %d %d %g %g %g %g\n", j,i,kr,kl,ku,ko,xr,xl,xu,xo);
 
             kh = kl + kr;
             kv = ko + ku;
-            if (kh == 0)
-              {
-                s1 = 0.;
-                k1 = 0;
-              }
-            else if (kl == 0)
-              {
-                s1 = xr;
-                k1 = 1;
-              }
-            else if (kr == 0)
-              {
-                s1 = xl;
-                k1 = 1;
-              }
-            else
-              {
-                s1 = xr * kl / kh + xl * kr / kh;
-                k1 = 2;
-              }
+            // clang-format off
+            if      (kh == 0) { k1 = 0; s1 = 0.; }
+            else if (kl == 0) { k1 = 1; s1 = xr; }
+            else if (kr == 0) { k1 = 1; s1 = xl; }
+            else              { k1 = 2; s1 = xr * kl / kh + xl * kr / kh; }
 
-            if (kv == 0)
-              {
-                s2 = 0.;
-                k2 = 0;
-              }
-            else if (ku == 0)
-              {
-                s2 = xo;
-                k2 = 1;
-              }
-            else if (ko == 0)
-              {
-                s2 = xu;
-                k2 = 1;
-              }
-            else
-              {
-                s2 = xu * ko / kv + xo * ku / kv;
-                k2 = 2;
-              }
+            if      (kv == 0) { k2 = 0; s2 = 0.; }
+            else if (ku == 0) { k2 = 1; s2 = xo; }
+            else if (ko == 0) { k2 = 1; s2 = xu; }
+            else              { k2 = 2; s2 = xu * ko / kv + xo * ku / kv; }
+            // clang-format on
 
             kk = k1 + k2;
             if (kk >= nfill)
@@ -191,7 +153,7 @@ fillmiss(Field &field1, Field &field2, int nfill)
             else
               matrix2[j][i] = matrix1[j][i];
 
-            /* matrix1[j][i] = matrix2[j][i]; */
+            // matrix1[j][i] = matrix2[j][i];
           }
         else
           {
@@ -200,41 +162,27 @@ fillmiss(Field &field1, Field &field2, int nfill)
       }
 
   if (nmiss1 != nmiss2) cdoAbort("found only %zu of %zu missing values!", nmiss2, nmiss1);
-
-  Free(matrix2);
-  Free(matrix1);
 }
 
 void
 fillmiss_one_step(Field &field1, Field &field2, int maxfill)
 {
-  int gridID, nx, ny, i, j;
+  int nx, ny, i, j;
   size_t nmiss2 = 0;
   int kr, ku, kl, ko;
   int ir, iu, il, io;
   int kh, kv, k1, k2, kk;
   double s1, s2;
   double xr, xu, xl, xo;
-  double missval;
-  double *array1, *array2;
-  double **matrix1, **matrix2;
 
-  gridID = field1.grid;
-  missval = field1.missval;
-  array1 = field1.vec.data();
-  array2 = field2.vec.data();
+  const auto gridID = field1.grid;
+  const auto missval = field1.missval;
 
   nx = gridInqXsize(gridID);
   ny = gridInqYsize(gridID);
 
-  matrix1 = (double **) Malloc(ny * sizeof(double *));
-  matrix2 = (double **) Malloc(ny * sizeof(double *));
-
-  for (j = 0; j < ny; j++)
-    {
-      matrix1[j] = array1 + j * nx;
-      matrix2[j] = array2 + j * nx;
-    }
+  MatrixView<double> matrix1(field1.vec_d.data(), ny, nx);
+  MatrixView<double> matrix2(field2.vec_d.data(), ny, nx);
 
   for (int fill_iterations = 0; fill_iterations < maxfill; fill_iterations++)
     {
@@ -371,58 +319,45 @@ fillmiss_one_step(Field &field1, Field &field2, int maxfill)
       for (j = 0; j < ny; j++)
         for (i = 0; i < nx; i++) matrix1[j][i] = matrix2[j][i];
     }
-
-  Free(matrix2);
-  Free(matrix1);
 }
 
 static void
 setmisstodis(Field &field1, Field &field2, int numNeighbors)
 {
-  int gridID = field1.grid;
-  const int gridID0 = gridID;
-  const double missval = field1.missval;
-  double *array1 = field1.vec.data();
-  double *array2 = field2.vec.data();
+  auto gridID = field1.grid;
+  const auto gridID0 = gridID;
+  const auto missval = field1.missval;
+  auto array1 = field1.vec_d.data();
+  auto array2 = field2.vec_d.data();
 
-  const int gridtype = gridInqType(gridID);
-  const size_t gridsize = gridInqSize(gridID);
+  const auto gridtype = gridInqType(gridID);
+  const auto gridsize = gridInqSize(gridID);
 
-  size_t nmiss = field1.nmiss;
-  const size_t nvals = gridsize - nmiss;
+  auto nmiss = field1.nmiss;
+  const auto nvals = gridsize - nmiss;
 
   if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
 
-  if (gridtype == GRID_UNSTRUCTURED)
+  if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID))
     {
-      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-        {
-          if (gridInqNumber(gridID) > 0)
-            {
-              gridID = referenceToGrid(gridID);
-              if (gridID == -1) cdoAbort("Reference to source grid not found!");
-            }
-        }
+      const auto reference = dereferenceGrid(gridID);
+      if (reference.isValid) gridID = reference.gridID;
+      if (reference.notFound) cdoAbort("Reference to source grid not found!");
     }
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
+  if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
 
-  std::vector<double> xvals(gridsize), yvals(gridsize);
+  Varray<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, xvals.data());
   gridInqYvals(gridID, yvals.data());
 
   // Convert lat/lon units if required
-  char units[CDI_MAX_NAME];
-  gridInqXunits(gridID, units);
-  grid_to_radian(units, gridsize, &xvals[0], "grid center lon");
-  gridInqYunits(gridID, units);
-  grid_to_radian(units, gridsize, &yvals[0], "grid center lat");
+  cdo_grid_to_radian(gridID, CDI_XAXIS, gridsize, xvals.data(), "grid center lon");
+  cdo_grid_to_radian(gridID, CDI_YAXIS, gridsize, yvals.data(), "grid center lat");
 
-  std::vector<size_t> mindex(nmiss, 1);
-  std::vector<size_t> vindex(nvals, 1);
-  std::vector<double> lons(nvals);
-  std::vector<double> lats(nvals);
+  std::vector<size_t> mindex(nmiss, 1), vindex(nvals, 1);
+  Varray<double> lons(nvals), lats(nvals);
 
   size_t nv = 0, nm = 0;
   for (size_t i = 0; i < gridsize; ++i)
@@ -447,29 +382,29 @@ setmisstodis(Field &field1, Field &field2, int numNeighbors)
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
-  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
-  GridPointSearch *gps = nullptr;
+  GridPointSearch gps;
 
   if (nmiss)
     {
-      const bool xIsCyclic = false;
+      const auto xIsCyclic = false;
       size_t dims[2] = { nvals, 0 };
-      gps = gridPointSearchCreate(xIsCyclic, dims, nvals, &lons[0], &lats[0]);
+      gridPointSearchCreate(gps, xIsCyclic, dims, nvals, lons, lats);
       gridPointSearchExtrapolate(gps);
     }
 
   if (Options::cdoVerbose) cdoPrint("Point search created: %.2f seconds", cdo_get_wtime() - start);
 
-  Progress::init();
+  progress::init();
 
   start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
-  double findex = 0;
+  double findex = 0.0;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  shared(nmiss, knnWeights, findex, mindex, vindex, array1, array2, xvals, yvals, gps, numNeighbors)
+    shared(nmiss, knnWeights, findex, mindex, vindex, array1, array2, xvals, yvals, gps, numNeighbors)
 #endif
   for (size_t i = 0; i < nmiss; ++i)
     {
@@ -477,28 +412,28 @@ setmisstodis(Field &field1, Field &field2, int numNeighbors)
 #pragma omp atomic
 #endif
       findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / nmiss);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / nmiss);
 
-      const int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
       gridSearchPoint(gps, xvals[mindex[i]], yvals[mindex[i]], knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      const size_t nadds = knnWeights[ompthID].computeWeights();
+      const auto nadds = knnWeights[ompthID].computeWeights();
       if (nadds)
         {
-          double result = 0;
+          double result = 0.0;
           for (size_t n = 0; n < nadds; ++n)
             result += array1[vindex[knnWeights[ompthID].m_addr[n]]] * knnWeights[ompthID].m_dist[n];
           array2[mindex[i]] = result;
         }
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
 
-  if (gps) gridPointSearchDelete(gps);
+  gridPointSearchDelete(gps);
 
   if (gridID0 != gridID) gridDestroy(gridID);
 }
@@ -507,69 +442,50 @@ void *
 Fillmiss(void *process)
 {
   int nrecs, varID, levelID;
-  void (*fill_method)(Field &fin, Field &fout, int) = &setmisstodis;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int FILLMISS        = cdoOperatorAdd("fillmiss"   ,   0, 0, "nfill");
-  const int FILLMISSONESTEP = cdoOperatorAdd("fillmiss2"  ,   0, 0, "nfill");
-  const int SETMISSTONN     = cdoOperatorAdd("setmisstonn" ,  0, 0, "");
-  const int SETMISSTODIS    = cdoOperatorAdd("setmisstodis" , 0, 0, "number of neighbors");
+  const auto FILLMISS        = cdoOperatorAdd("fillmiss"   ,   0, 0, "nfill");
+  const auto FILLMISSONESTEP = cdoOperatorAdd("fillmiss2"  ,   0, 0, "nfill");
+  const auto SETMISSTONN     = cdoOperatorAdd("setmisstonn" ,  0, 0, "");
+  const auto SETMISSTODIS    = cdoOperatorAdd("setmisstodis" , 0, 0, "number of neighbors");
+
+  const auto operatorID = cdoOperatorID();
+
+  void (*fill_method)(Field &, Field &, int) = &setmisstodis;
+  if      (operatorID == FILLMISS)        fill_method = &fillmiss;
+  else if (operatorID == FILLMISSONESTEP) fill_method = &fillmiss_one_step;
+  else if (operatorID == SETMISSTONN)     fill_method = &setmisstodis;
+  else if (operatorID == SETMISSTODIS)    fill_method = &setmisstodis;
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  auto nfill = (operatorID == SETMISSTODIS) ? 4 : 1;
 
-  int nfill = 1;
-  if (operatorID == FILLMISS)
+  // Argument handling
+  const auto oargc = operatorArgc();
+  if (oargc == 1)
     {
-      fill_method = &fillmiss;
-    }
-  else if (operatorID == FILLMISSONESTEP)
-    {
-      fill_method = &fillmiss_one_step;
-    }
-  else if (operatorID == SETMISSTONN)
-    {
-      fill_method = &setmisstodis;
-    }
-  else if (operatorID == SETMISSTODIS)
-    {
-      nfill = 4;
-      fill_method = &setmisstodis;
+      nfill = parameter2int(cdoOperatorArgv(0));
+      if (operatorID == FILLMISS && (nfill < 1 || nfill > 4)) cdoAbort("nfill out of range!");
     }
+  else if (oargc > 1)
+    cdoAbort("Too many arguments!");
 
-  /* Argument handling */
-  {
-    const int oargc = operatorArgc();
-    char **oargv = operatorArgv();
+  const auto streamID1 = cdoOpenRead(0);
 
-    if (oargc == 1)
-      {
-        nfill = parameter2int(oargv[0]);
-        if (operatorID == FILLMISS)
-          {
-            if (nfill < 1 || nfill > 4) cdoAbort("nfill out of range!");
-          }
-      }
-    else if (oargc > 1)
-      cdoAbort("Too many arguments!");
-  }
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
-
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
   field1.resize(gridsizemax);
@@ -585,21 +501,21 @@ Fillmiss(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
+          cdoReadRecord(streamID1, field1.vec_d.data(), &field1.nmiss);
 
           cdoDefRecord(streamID2, varID, levelID);
 
           if (field1.nmiss == 0)
             {
-              cdoWriteRecord(streamID2, field1.vec.data(), 0);
+              cdoWriteRecord(streamID2, field1.vec_d.data(), 0);
             }
           else
             {
-              const int gridID = vlistInqVarGrid(vlistID1, varID);
-
+              const auto gridID = vlistInqVarGrid(vlistID1, varID);
+              const auto gridtype = gridInqType(gridID);
               if ((operatorID == FILLMISS || operatorID == FILLMISSONESTEP)
-                  && (gridInqType(gridID) == GRID_GME || gridInqType(gridID) == GRID_UNSTRUCTURED))
-                cdoAbort("%s data unsupported!", gridNamePtr(gridInqType(gridID)));
+                  && (gridtype == GRID_GME || gridtype == GRID_UNSTRUCTURED))
+                cdoAbort("%s data unsupported!", gridNamePtr(gridtype));
 
               field1.grid = gridID;
               field1.size = gridInqSize(field1.grid);
@@ -611,7 +527,7 @@ Fillmiss(void *process)
 
               fill_method(field1, field2, nfill);
 
-              cdoWriteRecord(streamID2, field2.vec.data(), fieldNumMiss(field2));
+              cdoWriteRecord(streamID2, field2.vec_d.data(), fieldNumMiss(field2));
             }
         }
 
