@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,62 +22,65 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 
 void *
 Fldrms(void *process)
 {
   int lastgrid = -1;
-  int index;
   int nrecs;
   int varID, levelID;
 
   cdoInitialize(process);
 
-  const bool needWeights = true;
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto needWeights = true;
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  double slon = 0;
-  double slat = 0;
-  int gridID3 = gridCreate(GRID_LONLAT, 1);
+  double slon = 0.0, slat = 0.0;
+  const auto gridID3 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID3, 1);
   gridDefYsize(gridID3, 1);
   gridDefXvals(gridID3, &slon);
   gridDefYvals(gridID3, &slat);
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   int ndiffgrids = 0;
-  for (index = 1; index < ngrids; index++)
+  for (int index = 1; index < ngrids; index++)
     if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
 
-  index = 0;
-  const int gridID1 = vlistGrid(vlistID1, index);
-  const int gridID2 = vlistGrid(vlistID2, index);
+  const auto gridID1 = vlistGrid(vlistID1, 0);
+  const auto gridID2 = vlistGrid(vlistID2, 0);
 
   if (gridInqSize(gridID1) != gridInqSize(gridID2)) cdoAbort("Fields have different grid size!");
 
   if (needWeights && gridInqType(gridID1) != GRID_LONLAT && gridInqType(gridID1) != GRID_GAUSSIAN)
     cdoAbort("Unsupported gridtype: %s", gridNamePtr(gridInqType(gridID1)));
 
-  for (index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID3, index, gridID3);
+  for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID3, index, gridID3);
 
   if (ndiffgrids > 0) cdoAbort("Too many different grids!");
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
 
   Field field1, field2, field3;
   field1.resize(gridsizemax);
@@ -100,12 +103,12 @@ Fldrms(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
+          cdoReadRecord(streamID1, field1.vec_d.data(), &field1.nmiss);
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, field2.vec.data(), &field2.nmiss);
+          cdoReadRecord(streamID2, field2.vec_d.data(), &field2.nmiss);
 
-          field1.grid = vlistInqVarGrid(vlistID1, varID);
-          field2.grid = vlistInqVarGrid(vlistID2, varID);
+          field1.grid = varList1[varID].gridID;
+          field2.grid = varList2[varID].gridID;
 
           if (needWeights && field1.grid != lastgrid)
             {
@@ -113,24 +116,20 @@ Fldrms(void *process)
               field1.weightv[0] = 1;
               if (field1.size > 1)
                 {
-                  const int wstatus = gridWeights(field1.grid, field1.weightv.data());
+                  const auto wstatus = gridWeights(field1.grid, field1.weightv.data());
                   if (wstatus != 0 && tsID == 0 && levelID == 0)
-                    {
-                      char varname[CDI_MAX_NAME];
-                      vlistInqVarName(vlistID1, varID, varname);
-                      cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
-                    }
+                    cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varList1[varID].name);
                 }
             }
 
-          field1.missval = vlistInqVarMissval(vlistID1, varID);
-          field2.missval = vlistInqVarMissval(vlistID1, varID);
-          field3.missval = vlistInqVarMissval(vlistID1, varID);
+          field1.missval = varList1[varID].missval;
+          field2.missval = varList1[varID].missval;
+          field3.missval = varList1[varID].missval;
 
           vfldrms(field1, field2, field3);
 
           cdoDefRecord(streamID3, varID, levelID);
-          cdoWriteRecord(streamID3, field3.vec.data(), field3.nmiss);
+          cdoWriteRecord(streamID3, field3.vec_d.data(), field3.nmiss);
         }
 
       tsID++;
