@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,17 +23,18 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include <mpim_grid.h>
 
-/* routine corr copied from PINGO */
-/* correclation in space */
+// routine corr copied from PINGO
+// correclation in space
 static double
-correlation_s(const double *restrict in0, const double *restrict in1, const double *restrict weight, double missval1,
+correlation_s(const Varray<double> &in0, const Varray<double> &in1, const Varray<double> &weight, double missval1,
               double missval2, size_t gridsize)
 {
-  double sum0 = 0, sum1 = 0, sum00 = 0, sum01 = 0, sum11 = 0, wsum0 = 0;
+  double sum0 = 0.0, sum1 = 0.0, sum00 = 0.0, sum01 = 0.0, sum11 = 0.0, wsum0 = 0.0;
 
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -48,19 +49,19 @@ correlation_s(const double *restrict in0, const double *restrict in1, const doub
         }
     }
 
-  double out = IS_NOT_EQUAL(wsum0, 0)
+  double out = IS_NOT_EQUAL(wsum0, 0.0)
                    ? DIVMN((sum01 * wsum0 - sum0 * sum1), SQRTMN((sum00 * wsum0 - sum0 * sum0) * (sum11 * wsum0 - sum1 * sum1)))
                    : missval1;
 
   return out;
 }
 
-/* covariance in space */
+// covariance in space
 static double
-covariance_s(const double *restrict in0, const double *restrict in1, const double *restrict weight, double missval1,
+covariance_s(const Varray<double> &in0, const Varray<double> &in1, const Varray<double> &weight, double missval1,
              double missval2, size_t gridsize)
 {
-  double sum0 = 0, sum1 = 0, sum01 = 0, wsum0 = 0, wsum00 = 0;
+  double sum0 = 0.0, sum1 = 0.0, sum01 = 0.0, wsum0 = 0.0, wsum00 = 0.0;
 
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -74,7 +75,7 @@ covariance_s(const double *restrict in0, const double *restrict in1, const doubl
         }
     }
 
-  double out = IS_NOT_EQUAL(wsum0, 0) ? (sum01 * wsum0 - sum0 * sum1) / (wsum0 * wsum0) : missval1;
+  double out = IS_NOT_EQUAL(wsum0, 0.0) ? (sum01 * wsum0 - sum0 * sum1) / (wsum0 * wsum0) : missval1;
 
   return out;
 }
@@ -82,14 +83,12 @@ covariance_s(const double *restrict in0, const double *restrict in1, const doubl
 void *
 Fldstat2(void *process)
 {
-  int gridID, lastgridID = -1;
+  int lastgridID = -1;
   int nrecs;
   int varID, levelID;
   size_t nmiss1, nmiss2;
   bool wstatus = false;
   bool needWeights = true;
-  double sglval = 0;
-  char varname[CDI_MAX_NAME];
 
   cdoInitialize(process);
 
@@ -98,48 +97,50 @@ Fldstat2(void *process)
   cdoOperatorAdd("fldcovar", func_covar, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  double slon = 0;
-  double slat = 0;
-  int gridID3 = gridCreate(GRID_LONLAT, 1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
+  double slon = 0.0, slat = 0.0;
+  const auto gridID3 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID3, 1);
   gridDefYsize(gridID3, 1);
   gridDefXvals(gridID3, &slon);
   gridDefYvals(gridID3, &slat);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
 
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID3, index, gridID3);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
-  std::vector<double> weight;
+  Varray<double> array1(gridsizemax), array2(gridsizemax);
+  Varray<double> weight;
   if (needWeights) weight.resize(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
+      const auto nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
 
       if (nrecs2 == 0)
         {
@@ -157,32 +158,26 @@ Fldstat2(void *process)
           cdoReadRecord(streamID1, array1.data(), &nmiss1);
           cdoReadRecord(streamID2, array2.data(), &nmiss2);
 
-          gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
+          const auto gridID = varList1[varID].gridID;
+          const auto gridsize = varList1[varID].gridsize;
           if (needWeights && gridID != lastgridID)
             {
               lastgridID = gridID;
               wstatus = gridWeights(gridID, weight.data()) != 0;
             }
           if (wstatus && tsID == 0 && levelID == 0)
-            {
-              vlistInqVarName(vlistID1, varID, varname);
-              cdoWarning("Using constant grid cell area weights for variable %s!", varname);
-            }
+            cdoWarning("Using constant grid cell area weights for variable %s!", varList1[varID].name);
 
-          double missval1 = vlistInqVarMissval(vlistID1, varID);
-          double missval2 = vlistInqVarMissval(vlistID2, varID);
+          const auto missval1 = varList1[varID].missval;
+          const auto missval2 = varList2[varID].missval;
 
+          double sglval = 0.0;
           if (operfunc == func_cor)
-            {
-              sglval = correlation_s(array1.data(), array2.data(), weight.data(), missval1, missval2, gridsize);
-            }
+            sglval = correlation_s(array1, array2, weight, missval1, missval2, gridsize);
           else if (operfunc == func_covar)
-            {
-              sglval = covariance_s(array1.data(), array2.data(), weight.data(), missval1, missval2, gridsize);
-            }
+            sglval = covariance_s(array1, array2, weight, missval1, missval2, gridsize);
 
-          size_t nmiss3 = DBL_IS_EQUAL(sglval, missval1) ? 1 : 0;
+          const auto nmiss3 = DBL_IS_EQUAL(sglval, missval1) ? 1 : 0;
 
           cdoDefRecord(streamID3, varID, levelID);
           cdoWriteRecord(streamID3, &sglval, nmiss3);
