@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,12 +27,14 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
-
 #include "util_fileextensions.h"
 #include "util_files.h"
+#include "cdo_zaxis.h"
 
 /*
   Values output into the grib map file:
@@ -64,6 +66,7 @@
                  2 - reference value
 
 */
+
 struct gaindx
 {
   int type;      /* Indexing file type             */
@@ -76,6 +79,7 @@ struct gaindx
   int *intpnt;   /* Pointer to int index values    */
   float *fltpnt; /* Pointer to float index values  */
 };
+
 struct gaindxb
 {
   int bignum;    /* Number of off_t values */
@@ -87,17 +91,14 @@ struct gaindxb
 static void
 gabswp(void *r, int cnt)
 {
-  int i;
-  char *ch1, *ch2, *ch3, *ch4, cc1, cc2;
-
-  ch1 = (char *) r;
-  ch2 = ch1 + 1;
-  ch3 = ch2 + 1;
-  ch4 = ch3 + 1;
-  for (i = 0; i < cnt; i++)
+  char *ch1 = (char *) r;
+  char *ch2 = ch1 + 1;
+  char *ch3 = ch2 + 1;
+  char *ch4 = ch3 + 1;
+  for (int i = 0; i < cnt; i++)
     {
-      cc1 = *ch1;
-      cc2 = *ch2;
+      char cc1 = *ch1;
+      char cc2 = *ch2;
       *ch1 = *ch4;
       *ch2 = *ch3;
       *ch3 = cc2;
@@ -117,26 +118,17 @@ gabswp(void *r, int cnt)
 static float
 ibm2flt(unsigned char *ibm)
 {
-
-  int positive, power;
-  unsigned int abspower;
-  long int mant;
-  double value, exp;
-
-  positive = (ibm[0] & 0x80) == 0;
-  mant = (ibm[1] << 16) + (ibm[2] << 8) + ibm[3];
-  power = (int) (ibm[0] & 0x7f) - 64;
-  abspower = power > 0 ? power : -power;
+  int positive = (ibm[0] & 0x80) == 0;
+  long mant = (ibm[1] << 16) + (ibm[2] << 8) + ibm[3];
+  int power = (int) (ibm[0] & 0x7f) - 64;
+  unsigned int abspower = power > 0 ? power : -power;
 
   /* calc exp */
-  exp = 16.0;
-  value = 1.0;
+  double exp = 16.0;
+  double value = 1.0;
   while (abspower)
     {
-      if (abspower & 1)
-        {
-          value *= exp;
-        }
+      if (abspower & 1) value *= exp;
       exp = exp * exp;
       abspower >>= 1;
     }
@@ -144,6 +136,7 @@ ibm2flt(unsigned char *ibm)
   if (power < 0) value = 1.0 / value;
   value = value * mant / 16777216.0;
   if (positive == 0) value = -value;
+
   return (float) value;
 }
 
@@ -157,9 +150,7 @@ ibm2flt(unsigned char *ibm)
 static int
 flt2ibm(float x, unsigned char *ibm)
 {
-
   int sign, exp, i;
-  double mant;
 
   if (std::fabs((double) x) <= 0)
     {
@@ -176,7 +167,7 @@ flt2ibm(float x, unsigned char *ibm)
   else
     sign = 0;
 
-  mant = frexp((double) x, &exp);
+  double mant = frexp((double) x, &exp);
 
   /* round up by adding 2**-24 */
   /* mant = mant + 1.0/16777216.0; */
@@ -213,16 +204,16 @@ flt2ibm(float x, unsigned char *ibm)
   ibm[0] = sign | exp;
 
   mant = mant * 256.0;
-  i = (int) floor(mant);
+  i = (int) std::floor(mant);
   mant = mant - i;
   ibm[1] = i;
 
   mant = mant * 256.0;
-  i = (int) floor(mant);
+  i = (int) std::floor(mant);
   mant = mant - i;
   ibm[2] = i;
 
-  ibm[3] = (int) floor(mant * 256.0);
+  ibm[3] = (int) std::floor(mant * 256.0);
 
   return 0;
 }
@@ -247,7 +238,6 @@ dumpmap()
   struct gaindx indx;
   struct gaindxb indxb;
   size_t nbytes;
-  FILE *mapfp;
 
   indx.hipnt = nullptr;
   indx.hfpnt = nullptr;
@@ -256,7 +246,7 @@ dumpmap()
   indxb.bigpnt = nullptr;
   indxb.bignum = 0;
 
-  mapfp = fopen(cdoGetStreamName(0), "r");
+  auto mapfp = fopen(cdoGetStreamName(0), "r");
   if (mapfp == nullptr) cdoAbort("Open failed on %s", cdoGetStreamName(0));
 
   /* check the version number */
@@ -367,7 +357,7 @@ dumpmap()
         }
     }
 
-  UNUSED(nbytes);
+  (void) (nbytes);  // CDO_UNUSED
 
   fclose(mapfp);
 
@@ -415,7 +405,6 @@ static void
 ctl_xydef(FILE *gdp, int gridID, bool *yrev)
 {
   int i, j;
-  double xfirst, yfirst, xinc, yinc;
 
   *yrev = false;
 
@@ -434,8 +423,8 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
       int nx = 0, ny = 0, ni;
       double inc[] = { 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001 };
 
-      double xinc = gridInqXinc(gridID);
-      double yinc = gridInqYinc(gridID);
+      auto xinc = gridInqXinc(gridID);
+      auto yinc = gridInqYinc(gridID);
 
       double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
 
@@ -443,8 +432,7 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
       fprintf(gdp, "PDEF %d %d LCCR %g %g 1 1 %g %g %g %g %g\n", xsize, ysize, xval_0, yval_0, lat_1, lat_2, lon_0, xinc, yinc);
 
       gridID = gridToCurvilinear(gridID, 0);
-      std::vector<double> xvals(xsize * ysize);
-      std::vector<double> yvals(xsize * ysize);
+      Varray<double> xvals(xsize * ysize), yvals(xsize * ysize);
       gridInqXvals(gridID, xvals.data());
       gridInqYvals(gridID, yvals.data());
       for (i = 0; i < xsize * ysize; ++i)
@@ -456,8 +444,8 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
           if (yvals[i] > ymax) ymax = yvals[i];
         }
 
-      xfirst = ((int) (xmin - 0.0));
-      yfirst = ((int) (ymin - 0.0));
+      double xfirst = ((int) (xmin - 0.0));
+      double yfirst = ((int) (ymin - 0.0));
       xrange = ((int) (xmax + 1.5)) - xfirst;
       yrange = ((int) (ymax + 1.5)) - yfirst;
 
@@ -479,11 +467,11 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
     }
   else
     {
-      xfirst = gridInqXval(gridID, 0);
-      xinc = gridInqXinc(gridID);
+      auto xfirst = gridInqXval(gridID, 0);
+      auto xinc = gridInqXinc(gridID);
       if (IS_EQUAL(xinc, 0) && gridInqXvals(gridID, nullptr))
         {
-          std::vector<double> xvals(xsize);
+          Varray<double> xvals(xsize);
           gridInqXvals(gridID, xvals.data());
           fprintf(gdp, "XDEF %d LEVELS ", xsize);
           j = 0;
@@ -511,13 +499,13 @@ ctl_xydef(FILE *gdp, int gridID, bool *yrev)
 
   if (!(gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC))
     {
-      yfirst = gridInqYval(gridID, 0);
-      yinc = gridInqYinc(gridID);
+      auto yfirst = gridInqYval(gridID, 0);
+      auto yinc = gridInqYinc(gridID);
       if (gridtype == GRID_GAUSSIAN) yinc = 0;
 
       if (IS_EQUAL(yinc, 0) && gridInqYvals(gridID, nullptr))
         {
-          std::vector<double> yvals(ysize);
+          Varray<double> yvals(ysize);
           gridInqYvals(gridID, yvals.data());
           fprintf(gdp, "YDEF %d LEVELS ", ysize);
           j = 0;
@@ -590,7 +578,7 @@ ctl_zdef(FILE *gdp, int vlistID, bool *zrev)
         }
     }
 
-  std::vector<double> levels(nlevmax);
+  Varray<double> levels(nlevmax);
   cdoZaxisInqLevels(zaxisIDmax, levels.data());
   bool lplev = (zaxisInqType(zaxisIDmax) == ZAXIS_PRESSURE);
   double level0 = levels[0];
@@ -700,7 +688,8 @@ ctl_vars(FILE *gdp, int filetype, int vlistID, int nvarsout, int *vars)
       if (vars[varID] == true)
         {
           zaxisID = vlistInqVarZaxis(vlistID, varID);
-          ltype = zaxisInqLtype(zaxisID);
+          ltype = 0;
+          cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
           nlev = zaxisInqSize(zaxisID);
           vlistInqVarName(vlistID, varID, varname);
 
@@ -787,12 +776,11 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
   int i;
   struct gaindx indx;
   struct gaindxb indxb;
-  FILE *mapfp;
   int hinum[5];
 
   memset(&indx, 0, sizeof(struct gaindx));
 
-  mapfp = fopen(ctlfile, "w");
+  auto mapfp = fopen(ctlfile, "w");
   if (mapfp == nullptr) cdoAbort("Open failed on %s", ctlfile);
 
   indx.type = map_version;
@@ -963,22 +951,22 @@ Gradsdes(void *process)
   std::vector<int> intnum;
   std::vector<float> fltnum;
   std::vector<off_t> bignum;
-  std::vector<double> array;
+  Varray<double> array;
   const char *cmons[] = { "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };
 
   cdoInitialize(process);
 
   // clang-format off
-  int GRADSDES = cdoOperatorAdd("gradsdes",  0, 0, nullptr);
-  int DUMPMAP  = cdoOperatorAdd("dumpmap",   0, 0, nullptr);
+  const auto GRADSDES = cdoOperatorAdd("gradsdes",  0, 0, nullptr);
+  const auto DUMPMAP  = cdoOperatorAdd("dumpmap",   0, 0, nullptr);
   // clang-format on
 
-  UNUSED(GRADSDES);
+  (void) (GRADSDES);  // CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  const char *datfile = cdoGetStreamName(0);
-  size_t len = strlen(datfile);
+  auto datfile = cdoGetStreamName(0);
+  const auto len = strlen(datfile);
   char *ctlfile = (char *) Malloc(len + 10);
   strcpy(ctlfile, datfile);
 
@@ -993,7 +981,7 @@ Gradsdes(void *process)
 
   if (operatorArgc() == 1)
     {
-      map_version = parameter2int(operatorArgv()[0]);
+      map_version = parameter2int(cdoOperatorArgv(0));
       if (map_version != 1 && map_version != 2 && map_version != 4) cdoAbort("map_version=%d unsupported!", map_version);
     }
   else
@@ -1006,16 +994,16 @@ Gradsdes(void *process)
   if (map_version == 4 && sizeof(off_t) != 8)
     cdoAbort("GrADS GRIB map version %d requires size of off_t to be 8! The size of off_t is %ld.", map_version, sizeof(off_t));
 
-  CdoStreamID streamID = cdoOpenRead(0);
+  const auto streamID = cdoOpenRead(0);
 
-  int vlistID = cdoStreamInqVlist(streamID);
+  const auto vlistID = cdoStreamInqVlist(streamID);
 
-  int nvars = vlistNvars(vlistID);
-  int ntsteps = vlistNtsteps(vlistID);
-  int ngrids = vlistNgrids(vlistID);
+  const auto nvars = vlistNvars(vlistID);
+  const auto ntsteps = vlistNtsteps(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
 
-  int filetype = cdoInqFiletype(streamID);
-  int byteorder = cdoInqByteorder(streamID);
+  auto filetype = cdoInqFiletype(streamID);
+  auto byteorder = cdoInqByteorder(streamID);
 
   if (filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C || filetype == CDI_FILETYPE_NC5)
     filetype = CDI_FILETYPE_NC;
@@ -1037,7 +1025,7 @@ Gradsdes(void *process)
     {
       gridID = vlistGrid(vlistID, index);
       gridtype = gridInqType(gridID);
-      int projtype = gridInqProjType(gridID);
+      const auto projtype = gridInqProjType(gridID);
       if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC)
           || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL))
         break;
@@ -1048,8 +1036,7 @@ Gradsdes(void *process)
   /* select all variables with used gridID */
   std::vector<int> vars(nvars);
   std::vector<int> recoffset(nvars);
-  int nvarsout = 0;
-  int nrecsout = 0;
+  int nvarsout = 0, nrecsout = 0;
   for (varID = 0; varID < nvars; varID++)
     {
       if (vlistInqVarGrid(vlistID, varID) == gridID)
@@ -1108,7 +1095,7 @@ Gradsdes(void *process)
   repl_filetypeext(ctlfile, filetypeext(filetype), ".ctl");
 
   /* open ctl file*/
-  FILE *gdp = fopen(ctlfile, "w");
+  auto gdp = fopen(ctlfile, "w");
   if (gdp == nullptr) cdoAbort("Open failed on %s", ctlfile);
 
   /* VERSION */
@@ -1176,15 +1163,15 @@ Gradsdes(void *process)
 
   /* TIME */
 
-  int taxisID = vlistInqTaxis(vlistID);
+  const auto taxisID = vlistInqTaxis(vlistID);
 
   bool cal365day = (taxisInqCalendar(taxisID) == CALENDAR_365DAYS);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID);
-      int vtime = taxisInqVtime(taxisID);
+      const auto vdate = taxisInqVdate(taxisID);
+      const auto vtime = taxisInqVtime(taxisID);
 
       if (tsID == 0)
         {
