@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_cdi_wrapper.h"
 #include "cdo_zaxis.h"
 #include <mpim_grid.h>
 #include "constants.h"
@@ -33,43 +35,41 @@
 static inline double
 orthodrome(double px1, double py1, double px2, double py2)
 {
-  return acos(sin(py1) * sin(py2) + cos(py1) * cos(py2) * cos(px2 - px1));
+  return std::acos(std::sin(py1) * std::sin(py2) + std::cos(py1) * std::cos(py2) * std::cos(px2 - px1));
 }
 
-double
-gridGetPlanetRadius(int gridID);
+double gridGetPlanetRadius(int gridID);
 
 double
 getPlanetRadius(int gridID)
 {
-  static bool radiusFromEnv = true;
+  static auto radiusFromEnv = true;
   if (radiusFromEnv)
     {
       radiusFromEnv = false;
-      char *envstr = getenv("PLANET_RADIUS");
+      auto envstr = getenv("PLANET_RADIUS");
       if (envstr)
         {
-          const double fval = atof(envstr);
+          const auto fval = atof(envstr);
           if (fval > 0) PlanetRadius = fval;
         }
     }
 
-  double planetRadius = PlanetRadius;
+  auto planetRadius = PlanetRadius;
 
-  double gridRadius = gridGetPlanetRadius(gridID);
+  const auto gridRadius = gridGetPlanetRadius(gridID);
   if (gridRadius > 1) planetRadius = gridRadius;
 
   return planetRadius;
 }
-  
+
 void
 grid_cell_area(int gridID, double *array)
 {
-  const int gridtype = gridInqType(gridID);
-  const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  const auto gridtype = gridInqType(gridID);
 
-  if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA
-      || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR
+  if (gridProjIsSupported(gridID) || gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN
+      || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR
       || gridtype == GRID_UNSTRUCTURED || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       if (gridHasArea(gridID))
@@ -85,9 +85,9 @@ grid_cell_area(int gridID, double *array)
           else if (status == 2)
             cdoAbort("%s: Can't compute grid cell area for this grid!", __func__);
 
-          const double planetRadius = getPlanetRadius(gridID);
+          const auto planetRadius = getPlanetRadius(gridID);
           if (Options::cdoVerbose) cdoPrint("Planet radius: %.2f", planetRadius);
-          const size_t ngp = gridInqSize(gridID);
+          const auto ngp = gridInqSize(gridID);
           for (size_t i = 0; i < ngp; ++i) array[i] *= planetRadius * planetRadius;
         }
     }
@@ -106,74 +106,76 @@ Gridcell(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int GRIDAREA    = cdoOperatorAdd("gridarea",     1,  0, nullptr);
-  const int GRIDWGTS    = cdoOperatorAdd("gridweights",  1,  0, nullptr);
-  const int GRIDMASK    = cdoOperatorAdd("gridmask",     0,  0, nullptr);
-  const int GRIDDX      = cdoOperatorAdd("griddx",       1,  0, nullptr);
-  const int GRIDDY      = cdoOperatorAdd("griddy",       1,  0, nullptr);
-  const int GRIDCELLIDX = cdoOperatorAdd("gridcellidx",  0,  0, nullptr);
+  const auto GRIDAREA    = cdoOperatorAdd("gridarea",     1,  0, nullptr);
+  const auto GRIDWGTS    = cdoOperatorAdd("gridweights",  1,  0, nullptr);
+  const auto GRIDMASK    = cdoOperatorAdd("gridmask",     0,  0, nullptr);
+  const auto GRIDDX      = cdoOperatorAdd("griddx",       1,  0, nullptr);
+  const auto GRIDDY      = cdoOperatorAdd("griddy",       1,  0, nullptr);
+  const auto GRIDCELLIDX = cdoOperatorAdd("gridcellidx",  0,  0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  //const bool needRadius = cdoOperatorF1(operatorID) > 0;
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  // const bool needRadius = cdoOperatorF1(operatorID) > 0;
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+
+  const auto ngrids = vlistNgrids(vlistID1);
 
   if (ngrids > 1) cdoWarning("Found more than 1 grid, using the first one!");
 
-  int gridID = vlistGrid(vlistID1, 0);
-  const int zaxisID = zaxisFromName("surface");
+  auto gridID = vlistGrid(vlistID1, 0);
+  const auto zaxisID = zaxisFromName("surface");
 
-  const int vlistID2 = vlistCreate();
-  int varID = vlistDefVar(vlistID2, gridID, zaxisID, TIME_CONSTANT);
+  const auto vlistID2 = vlistCreate();
+  auto varID = vlistDefVar(vlistID2, gridID, zaxisID, TIME_CONSTANT);
   vlistDefNtsteps(vlistID2, 0);
 
   if (operatorID == GRIDAREA)
     {
-      vlistDefVarName(vlistID2, varID, "cell_area");
-      vlistDefVarStdname(vlistID2, varID, "area");
-      vlistDefVarLongname(vlistID2, varID, "area of grid cell");
-      vlistDefVarUnits(vlistID2, varID, "m2");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "cell_area");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, "area");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, "area of grid cell");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, "m2");
       vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
     }
   else if (operatorID == GRIDWGTS)
     {
-      vlistDefVarName(vlistID2, varID, "cell_weights");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "cell_weights");
       vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT64);
     }
   else if (operatorID == GRIDMASK)
     {
-      vlistDefVarName(vlistID2, varID, "grid_mask");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "grid_mask");
       vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_UINT8);
     }
   else if (operatorID == GRIDDX)
     {
-      vlistDefVarName(vlistID2, varID, "dx");
-      vlistDefVarLongname(vlistID2, varID, "delta x");
-      vlistDefVarUnits(vlistID2, varID, "m");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "dx");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, "delta x");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, "m");
     }
   else if (operatorID == GRIDDY)
     {
-      vlistDefVarName(vlistID2, varID, "dy");
-      vlistDefVarLongname(vlistID2, varID, "delta y");
-      vlistDefVarUnits(vlistID2, varID, "m");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "dy");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, "delta y");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, "m");
     }
   else if (operatorID == GRIDCELLIDX)
     {
-      vlistDefVarName(vlistID2, varID, "gridcellidx");
-      vlistDefVarLongname(vlistID2, varID, "grid cell index");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "gridcellidx");
+      cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, "grid cell index");
       vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_INT32);
     }
 
-  const int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
+  const auto taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID);
 
-  size_t gridsize = gridInqSize(gridID);
-  std::vector<double> array(gridsize);
+  auto gridsize = gridInqSize(gridID);
+  Varray<double> array(gridsize);
 
   if (operatorID == GRIDAREA)
     {
@@ -181,7 +183,7 @@ Gridcell(void *process)
     }
   else if (operatorID == GRIDWGTS)
     {
-      int status = gridWeights(gridID, &array[0]);
+      auto status = gridWeights(gridID, &array[0]);
       if (status != 0) cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
     }
   else if (operatorID == GRIDMASK)
@@ -204,35 +206,29 @@ Gridcell(void *process)
     }
   else if (operatorID == GRIDDX || operatorID == GRIDDY)
     {
-      const int gridtype = gridInqType(gridID);
-      const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_LCC || projtype == CDI_PROJ_STERE
-          || gridtype == GRID_CURVILINEAR)
+      const auto gridtype = gridInqType(gridID);
+      if (gridProjIsSupported(gridID) || gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR)
         {
-          double len1 = 0, len2 = 0;
-          char units[CDI_MAX_NAME];
-
           if (gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 1);
 
           gridsize = gridInqSize(gridID);
-          const size_t xsize = gridInqXsize(gridID);
-          const size_t ysize = gridInqYsize(gridID);
+          const auto xsize = gridInqXsize(gridID);
+          const auto ysize = gridInqYsize(gridID);
 
-          if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-            cdoAbort("Cell center coordinates missing!");
+          if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
 
-          std::vector<double> xv(gridsize), yv(gridsize);
+          Varray<double> xv(gridsize), yv(gridsize);
           gridInqXvals(gridID, xv.data());
           gridInqYvals(gridID, yv.data());
 
           // Convert lat/lon units if required
-          gridInqXunits(gridID, units);
-          grid_to_radian(units, gridsize, &xv[0], "grid longitudes");
-          grid_to_radian(units, gridsize, &yv[0], "grid latitudes");
+          cdo_grid_to_radian(gridID, CDI_XAXIS, gridsize, &xv[0], "grid longitudes");
+          cdo_grid_to_radian(gridID, CDI_YAXIS, gridsize, &yv[0], "grid latitudes");
 
-          const double planetRadius = getPlanetRadius(gridID);
+          const auto planetRadius = getPlanetRadius(gridID);
           if (Options::cdoVerbose) cdoPrint("Planet radius: %.2f", planetRadius);
 
+          double len1 = 0, len2 = 0;
           if (operatorID == GRIDDX)
             {
               for (size_t j = 0; j < ysize; ++j)
@@ -291,7 +287,7 @@ Gridcell(void *process)
         }
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
   cdoDefTimestep(streamID2, 0);
   cdoDefRecord(streamID2, 0, 0);
