@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
-#include "listarray.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "process_int.h"
+#include "param_conversion.h"
 
 void *
 Histogram(void *process)
@@ -35,22 +36,21 @@ Histogram(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int HISTCOUNT = cdoOperatorAdd("histcount", 0, 0, nullptr);
-  int HISTSUM   = cdoOperatorAdd("histsum",   0, 0, nullptr);
-  int HISTMEAN  = cdoOperatorAdd("histmean",  0, 0, nullptr);
-  int HISTFREQ  = cdoOperatorAdd("histfreq",  0, 0, nullptr);
+  const auto HISTCOUNT = cdoOperatorAdd("histcount", 0, 0, nullptr);
+  const auto HISTSUM   = cdoOperatorAdd("histsum",   0, 0, nullptr);
+  const auto HISTMEAN  = cdoOperatorAdd("histmean",  0, 0, nullptr);
+  const auto HISTFREQ  = cdoOperatorAdd("histfreq",  0, 0, nullptr);
   // clang-format on
 
-  UNUSED(HISTSUM);
+  (void) (HISTSUM);  // CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg("bins");
 
-  ListArray<double> listArrayFlt;
-  int nbins = listArrayFlt.argvToFlt(operatorArgc(), operatorArgv()) - 1;
+  const auto fltarr = cdoArgvToFlt(cdoGetOperArgv());
+  int nbins = fltarr.size() - 1;
   if (nbins < 1) cdoAbort("Too few arguments!");
-  double *fltarr = listArrayFlt.data();
 
   if (Options::cdoVerbose)
     {
@@ -58,60 +58,57 @@ Histogram(void *process)
       for (int i = 0; i < nbins; i++) printf("flt %d = %g\n", i + 1, fltarr[i]);
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
   /* create zaxis for output bins */
-  int zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nbins);
+  const auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nbins);
 
-  {
-    std::vector<double> bins(nbins);
-    /* for ( int i = 0; i < nbins; i++ ) bins[i] = (fltarr[i]+fltarr[i+1])/2; */
-    for (int i = 0; i < nbins; i++) bins[i] = fltarr[i];
-    zaxisDefLevels(zaxisID2, bins.data());
-  }
+  zaxisDefLevels(zaxisID2, &fltarr[0]);
+  zaxisDefLbounds(zaxisID2, &fltarr[0]);
+  zaxisDefUbounds(zaxisID2, &fltarr[1]);
 
-  zaxisDefLbounds(zaxisID2, fltarr);
-  zaxisDefUbounds(zaxisID2, fltarr + 1);
-  zaxisDefName(zaxisID2, "bin");
-  zaxisDefLongname(zaxisID2, "histogram bins");
-  zaxisDefUnits(zaxisID2, "level");
+  cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_NAME, "bin");
+  cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_LONGNAME, "histogram bins");
+  cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_UNITS, "level");
 
   /* check zaxis: only 2D fields allowed */
-  int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (int index = 0; index < nzaxis; index++)
     {
-      int zaxisID = vlistZaxis(vlistID1, index);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, index);
+      const auto nlevel = zaxisInqSize(zaxisID);
       if (nlevel > 1) cdoAbort("Found 3D field with %d levels. Only 2D fields allowed!", nlevel);
       vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   cdoDefVlist(streamID2, vlistID2);
+  
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
 
-  int nvars = vlistNvars(vlistID2);
-  std::vector<std::vector<double>> vardata(nvars);
-  std::vector<std::vector<double>> varcount(nvars);
-  std::vector<std::vector<double>> vartcount(nvars);
+  const auto nvars = vlistNvars(vlistID2);
+  Varray2D<double> vardata(nvars), varcount(nvars), vartcount(nvars);
   for (varID = 0; varID < nvars; varID++)
     {
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+      const auto gridsize = varList1[varID].gridsize;
       vardata[varID].resize(nbins * gridsize, 0);
       varcount[varID].resize(nbins * gridsize, 0);
       vartcount[varID].resize(gridsize, 0);
     }
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
 
   int tsID1 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
@@ -122,9 +119,9 @@ Histogram(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array.data(), &nmiss);
-          double missval = vlistInqVarMissval(vlistID1, varID);
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const auto gridsize = varList1[varID].gridsize;
+          const auto missval = varList1[varID].missval;
 
           nmiss = 0;
           for (size_t i = 0; i < gridsize; i++)
@@ -135,7 +132,7 @@ Histogram(void *process)
                   int index = 0;
                   while (index < nbins)
                     {
-                      size_t offset = gridsize * index;
+                      const auto offset = gridsize * index;
                       if (!DBL_IS_EQUAL(vardata[varID][offset + i], missval) && array[i] >= fltarr[index]
                           && array[i] < fltarr[index + 1])
                         {
@@ -147,11 +144,12 @@ Histogram(void *process)
                     }
                 }
               else
-                { /* missing value */
-                  nmiss++;
+                { 
+                  nmiss++; // missing value
                 }
             }
         }
+
       tsID1++;
     }
 
@@ -159,15 +157,15 @@ Histogram(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      double missval = vlistInqVarMissval(vlistID2, varID);
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+      const auto gridsize = varList2[varID].gridsize;
+      const auto missval = varList2[varID].missval;
 
-      /* fix mising values */
+      // fix missing values
 
       for (int index = 0; index < nbins; index++)
         {
           nmiss = 0;
-          size_t offset = gridsize * index;
+          const auto offset = gridsize * index;
 
           for (size_t i = 0; i < gridsize; i++)
             {
