@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_cdi_wrapper.h"
 #include "printinfo.h"
+#include "gaussian_latitudes.h"
 
 extern "C"
 {
@@ -73,52 +76,21 @@ rev_vals(double *vals, int n)
     }
 }
 
-static bool
-y_is_gauss(std::vector<double> &gridyvals, int ysize)
-{
-  bool lgauss = false;
-  int i;
-
-  if (ysize > 2)
-    {
-      std::vector<double> yvals(ysize);
-      std::vector<double> yw(ysize);
-      gaussianLatitudes(yvals.data(), yw.data(), (size_t) ysize);
-      for (i = 0; i < ysize; i++) yvals[i] = asin(yvals[i]) / M_PI * 180.0;
-
-      for (i = 0; i < ysize; i++)
-        if (std::fabs(yvals[i] - gridyvals[i]) > ((yvals[0] - yvals[1]) / 500)) break;
-
-      if (i == ysize) lgauss = true;
-
-      // check S->N
-      if (lgauss == false)
-        {
-          for (i = 0; i < ysize; i++)
-            if (std::fabs(yvals[i] - gridyvals[ysize - i - 1]) > ((yvals[0] - yvals[1]) / 500)) break;
-
-          if (i == ysize) lgauss = true;
-        }
-    }
-
-  return lgauss;
-}
-
 static int
 define_grid(dsets_t *pfi)
 {
   const int nx = pfi->dnum[0];
   const int ny = pfi->dnum[1];
 
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
+  Varray<double> xvals(nx);
+  Varray<double> yvals(ny);
 
   get_dim_vals(pfi, xvals.data(), nx, 0);
   get_dim_vals(pfi, yvals.data(), ny, 1);
 
   if (pfi->yrflg) rev_vals(yvals.data(), ny);
 
-  const bool lgauss = (pfi->linear[1] == 0) ? y_is_gauss(yvals, (size_t) ny) : false;
+  const bool lgauss = (pfi->linear[1] == 0) ? is_gaussian_latitudes((size_t) ny, yvals.data()) : false;
 
   const int gridtype = lgauss ? GRID_GAUSSIAN : GRID_LONLAT;
 
@@ -140,21 +112,19 @@ define_level(dsets_t *pfi, int nlev)
 
   if (nz)
     {
-      double *zvals = (double *) Malloc(nz * sizeof(double));
+      Varray<double> zvals(nz);
 
-      get_dim_vals(pfi, zvals, nz, 2);
+      get_dim_vals(pfi, zvals.data(), nz, 2);
 
       if (nz == 1 && IS_EQUAL(zvals[0], 0))
         zaxisID = zaxisCreate(ZAXIS_SURFACE, nz);
       else
         {
           if (nlev > 0 && nlev < nz) nz = nlev;
-          if (pfi->zrflg) rev_vals(zvals, nz);
+          if (pfi->zrflg) rev_vals(zvals.data(), nz);
           zaxisID = zaxisCreate(ZAXIS_GENERIC, nz);
         }
-      zaxisDefLevels(zaxisID, zvals);
-
-      Free(zvals);
+      zaxisDefLevels(zaxisID, zvals.data());
     }
   else
     {
@@ -175,28 +145,23 @@ Importbinary(void *process)
   size_t nmiss = 0, n_nan;
   int ivar;
   int varID = -1, levelID, tsID;
-  size_t gridsize;
   int datatype;
   dsets_t pfi;
-  int64_t vdate;
-  int vtime;
   int tcur, told, fnum;
   int tmin = 0, tmax = 0;
   char *ch = nullptr;
   int nlevels;
   int e, flag;
-  size_t rc, recsize;
-  int recoffset;
-  char *rec = nullptr;
+  size_t rc;
   struct dt dtim, dtimi;
   double missval;
   double fmin, fmax;
-  double *array;
   double sfclevel = 0;
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
+  operatorCheckArgc(0);
+
   dsets_init(&pfi);
 
   int status = read_gradsdes((char *) cdoGetStreamName(0), &pfi);
@@ -210,16 +175,16 @@ Importbinary(void *process)
 
   if (nvars == 0) cdoAbort("No variables found!");
 
-  int gridID = define_grid(&pfi);
-  int zaxisID = define_level(&pfi, 0);
+  const auto gridID = define_grid(&pfi);
+  const auto zaxisID = define_level(&pfi, 0);
 
-  int zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
+  const auto zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(zaxisIDsfc, &sfclevel);
 
-  int vlistID = vlistCreate();
+  const auto vlistID = vlistCreate();
 
-  int *var_zaxisID = (int *) Malloc(nvars * sizeof(int));
-  int *var_dfrm = (int *) Malloc(nrecs * sizeof(int));
+  Varray<int> var_zaxisID(nvars);
+  Varray<int> var_dfrm(nrecs);
   std::vector<RecordInfo> recList(nrecs);
 
   int recID = 0;
@@ -261,17 +226,17 @@ Importbinary(void *process)
 
       var_zaxisID[varID] = vlistInqVarZaxis(vlistID, varID);
 
-      vlistDefVarName(vlistID, varID, pvar->abbrv);
+      cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, pvar->abbrv);
       {
         char *longname = pvar->varnm;
-        size_t len = strlen(longname);
+        const auto len = strlen(longname);
         if (longname[0] == '\'' && longname[len - 1] == '\'')
           {
             longname[len - 1] = 0;
             longname++;
           }
         if (longname[0] == '\t') longname++;
-        vlistDefVarLongname(vlistID, varID, longname);
+        cdiDefKeyString(vlistID, varID, CDI_KEY_LONGNAME, longname);
       }
 
       missval = pfi.undef;
@@ -315,27 +280,23 @@ Importbinary(void *process)
       pvar++;
     }
 
-  int calendar = CALENDAR_STANDARD;
-  int taxisID = cdoTaxisCreate(TAXIS_RELATIVE);
+  const auto calendar = CALENDAR_STANDARD;
+  const auto taxisID = cdoTaxisCreate(TAXIS_RELATIVE);
   taxisDefCalendar(taxisID, calendar);
   vlistDefTaxis(vlistID, taxisID);
 
-  CdoStreamID streamID = cdoOpenWrite(1);
+  const auto streamID = cdoOpenWrite(1);
   cdoDefVlist(streamID, vlistID);
 
-  gridsize = pfi.dnum[0] * pfi.dnum[1];
-  if (pfi.flt64)
-    recoffset = pfi.xyhdr * 8;
-  else
-    recoffset = pfi.xyhdr * 4;
-
+  const size_t gridsize = pfi.dnum[0] * pfi.dnum[1];
+  int recoffset = pfi.xyhdr * (pfi.flt64 ? 8 : 4);
   if (pfi.seqflg) recoffset += 4;
 
   // recsize = pfi.gsiz*4;
-  recsize = pfi.gsiz * 8;
-  rec = (char *) Malloc(recsize);
+  size_t recsize = pfi.gsiz * 8;
+  Varray<char> rec(recsize);
 
-  array = (double *) Malloc(gridsize * sizeof(double));
+  Varray<double> array(gridsize);
 
   /*
   if (pfi.tmplat)
@@ -421,7 +382,6 @@ Importbinary(void *process)
               cdoAbort("Could not open file: %s", ch);
             }
         }
-      // if (pfi.tmplat) gree(ch, "312"); //US: ch is a local stack variable, which is not memory allocated by malloc()
 
       /* file header */
       if (pfi.fhdr > 0) fseeko(pfi.infile, pfi.fhdr, SEEK_SET);
@@ -438,19 +398,16 @@ Importbinary(void *process)
       for (tsID = tmin - 1; tsID < tmax; ++tsID)
         {
           gr2t(pfi.grvals[3], (gadouble)(tsID + 1), &dtim);
-          vdate = cdiEncodeDate(dtim.yr, dtim.mo, dtim.dy);
-          vtime = cdiEncodeTime(dtim.hr, dtim.mn, 0);
-
-          date2str(vdate, vdatestr, sizeof(vdatestr));
-          time2str(vtime, vtimestr, sizeof(vtimestr));
-
-          if (Options::cdoVerbose) cdoPrint(" Reading timestep: %3d %s %s", tsID + 1, vdatestr, vtimestr);
+          const auto vdate = cdiEncodeDate(dtim.yr, dtim.mo, dtim.dy);
+          const auto vtime = cdiEncodeTime(dtim.hr, dtim.mn, 0);
+          if (Options::cdoVerbose)
+            cdoPrint(" Reading timestep: %3d %s %s", tsID + 1, dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
           cdoDefTimestep(streamID, tsID);
 
-          for (int recID = 0; recID < nrecs; ++recID)
+          for (recID = 0; recID < nrecs; ++recID)
             {
               /* record size depends on data type */
               if (var_dfrm[recID] == 1)
@@ -466,30 +423,30 @@ Importbinary(void *process)
                   recsize = pfi.flt64 ? pfi.gsiz * 8 : pfi.gsiz * 4;
                 }
 
-              rc = fread(rec, 1, recsize, pfi.infile);
+              rc = fread(rec.data(), 1, recsize, pfi.infile);
               if (rc < recsize) cdoAbort("I/O error reading record=%d of timestep=%d!", recID + 1, tsID + 1);
 
               /* convert */
               if (var_dfrm[recID] == 1)
                 {
-                  unsigned char *carray = (unsigned char *) (rec + recoffset);
+                  unsigned char *carray = (unsigned char *) &rec[recoffset];
                   for (i = 0; i < gridsize; ++i) array[i] = (double) carray[i];
                 }
               else if (var_dfrm[recID] == 2)
                 {
-                  unsigned short *sarray = (unsigned short *) (rec + recoffset);
+                  unsigned short *sarray = (unsigned short *) &rec[recoffset];
                   if (pfi.bswap) gabswp2(sarray, gridsize);
                   for (i = 0; i < gridsize; ++i) array[i] = (double) sarray[i];
                 }
               else if (var_dfrm[recID] == -2)
                 {
-                  short *sarray = (short *) (rec + recoffset);
+                  short *sarray = (short *) &rec[recoffset];
                   if (pfi.bswap) gabswp2(sarray, gridsize);
                   for (i = 0; i < gridsize; ++i) array[i] = (double) sarray[i];
                 }
               else if (var_dfrm[recID] == 4)
                 {
-                  int *iarray = (int *) (rec + recoffset);
+                  int *iarray = (int *) &rec[recoffset];
                   if (pfi.bswap) gabswp(iarray, gridsize);
                   for (i = 0; i < gridsize; ++i) array[i] = (double) iarray[i];
                 }
@@ -497,13 +454,13 @@ Importbinary(void *process)
                 {
                   if (pfi.flt64)
                     {
-                      double *darray = (double *) (rec + recoffset);
+                      double *darray = (double *) &rec[recoffset];
                       if (pfi.bswap) gabswp(darray, gridsize);
                       for (i = 0; i < gridsize; ++i) array[i] = darray[i];
                     }
                   else
                     {
-                      float *farray = (float *) (rec + recoffset);
+                      float *farray = (float *) &rec[recoffset];
                       if (pfi.bswap) gabswp(farray, gridsize);
                       for (i = 0; i < gridsize; ++i) array[i] = (double) farray[i];
                     }
@@ -520,7 +477,7 @@ Importbinary(void *process)
                       array[i] = pfi.undef;
                       nmiss++;
                     }
-                  else if (DBL_IS_NAN(array[i]))
+                  else if (std::isnan(array[i]))
                     {
                       array[i] = pfi.undef;
                       nmiss++;
@@ -528,8 +485,8 @@ Importbinary(void *process)
                     }
                   else
                     {
-                      if (array[i] < fmin) fmin = array[i];
-                      if (array[i] > fmax) fmax = array[i];
+                      fmin = std::min(fmin, array[i]);
+                      fmax = std::max(fmax, array[i]);
                     }
                 }
               /*
@@ -540,7 +497,7 @@ Importbinary(void *process)
               varID = recList[recID].varID;
               levelID = recList[recID].levelID;
               cdoDefRecord(streamID, varID, levelID);
-              cdoWriteRecord(streamID, array, nmiss);
+              cdoWriteRecord(streamID, array.data(), nmiss);
             }
         }
 
@@ -558,11 +515,7 @@ Importbinary(void *process)
   zaxisDestroy(zaxisID);
   taxisDestroy(taxisID);
 
-  Free(array);
-  Free(rec);
-
-  if (var_zaxisID) Free(var_zaxisID);
-  if (var_dfrm) Free(var_dfrm);
+  if (pfi.infile) fclose(pfi.infile);
 
   cdoFinish();
 
