@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "array.h"
+#include "process_int.h"
+#include "cdo_cdi_wrapper.h"
+#include "cdo_default_values.h"
 #include <mpim_grid.h>
 
 #define MAX_DSETS 1024
@@ -99,20 +104,20 @@ get_grid_info(double c0, double re, int *nrxp, int *nryp, double *r0p, double *s
   double r0, s0, c;
   int nrx, nry;
 
-  git = 2. * pi * re * cos(pi / 6.) / c0;
+  git = 2. * pi * re * std::cos(pi / 6.) / c0;
   /* number of longitude pixels */
-  nrx = 2 * (int) lround(0.5 * git);
+  nrx = 2 * (int) std::lround(0.5 * git);
 
   /* central index in longitude */
   r0 = nrx / 2 + 0.5;
 
   /* resolution in km */
-  c = 2. * pi * re * cos(30. * pi / 180.) / nrx;
+  c = 2. * pi * re * std::cos(30. * pi / 180.) / nrx;
 
   phi = pi / 2.;
-  s90 = re / c * sin(phi) / cos(30. * pi / 180.);
+  s90 = re / c * std::sin(phi) / std::cos(30. * pi / 180.);
 
-  nry = (int) floor(s90);
+  nry = (int) std::floor(s90);
   /* central index in latitude */
   s0 = nry + 0.5;
   /* number of latitude pixels */
@@ -130,7 +135,7 @@ det_lon_atovs(double r, double r0, double lts, double c, double re)
 {
   const double pi = M_PI;
 
-  double xla = (r - r0) * c / re / cos(lts * pi / 180.); /* longitude */
+  double xla = (r - r0) * c / re / std::cos(lts * pi / 180.); /* longitude */
   xla = 180. * xla / pi;
 
   return xla;
@@ -141,8 +146,8 @@ det_lat_atovs(double s, double s0, double lts, double c, double re)
 {
   const double pi = M_PI;
 
-  double siphi = (s - s0) * c * cos(lts * pi / 180.) / re;
-  double phi = 180. * asin(siphi) / pi; /* latitude */
+  double siphi = (s - s0) * c * std::cos(lts * pi / 180.) / re;
+  double phi = 180. * std::asin(siphi) / pi; /* latitude */
 
   return phi;
 }
@@ -165,10 +170,8 @@ defLonLatGrid(int nx, int ny, double c0, double lts, double re)
       return -1;
     }
 
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
-  std::vector<double> xbounds(nx * 2);
-  std::vector<double> ybounds(ny * 2);
+  Varray<double> xvals(nx), yvals(ny);
+  Varray<double> xbounds(nx * 2), ybounds(ny * 2);
 
   for (i = 0; i < nx; ++i)
     {
@@ -208,17 +211,13 @@ defLonLatGrid(int nx, int ny, double c0, double lts, double re)
 }
 
 static int
-defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax, double dx, double dy, double p1, double p2,
-                  double p3, double p4)
+defSinusoidalGrid(int nx, int ny, double xmin, double ymax, double dx, double dy, double p1, double p2, double p3, double p4)
 {
-  UNUSED(p1);
-  UNUSED(p2);
-  UNUSED(p3);
-  UNUSED(p4);
-  UNUSED(xmax);
-  UNUSED(ymin);
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
+  (void) (p1);  // CDO_UNUSED
+  (void) (p2);  // CDO_UNUSED
+  (void) (p3);  // CDO_UNUSED
+  (void) (p4);  // CDO_UNUSED
+  Varray<double> xvals(nx), yvals(ny);
 
   for (int i = 0; i < nx; ++i) xvals[i] = xmin + i * dx + dx / 2;
 
@@ -237,13 +236,9 @@ defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, double
 }
 
 static int
-defLaeaGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax, double dx, double dy, double a, double lon0,
-            double lat0)
+defLaeaGrid(int nx, int ny, double xmin, double ymax, double dx, double dy, double a, double lon0, double lat0)
 {
-  UNUSED(xmax);
-  UNUSED(ymin);
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
+  Varray<double> xvals(nx), yvals(ny);
 
   for (int i = 0; i < nx; ++i) xvals[i] = xmin + i * dx + dx / 2;
   for (int i = 0; i < ny; ++i) yvals[i] = ymax - i * dy - dy / 2;
@@ -327,18 +322,18 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   int xsize, ysize;
   struct proj_t
   {
-    char name[64];
-    char ellipsoid[64];
-    float parameter[10];
+    char name[64] = { 0 };
+    char ellipsoid[64] = { 0 };
+    float parameter[10] = { 0.0 };
   };
   struct region_t
   {
-    float xmin;
-    float xmax;
-    float ymin;
-    float ymax;
-    float dx;
-    float dy;
+    float xmin = 0.0;
+    float xmax = 0.0;
+    float ymin = 0.0;
+    float ymax = 0.0;
+    float dx = 0.0;
+    float dy = 0.0;
   };
 
   proj_t proj;
@@ -390,10 +385,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       }
   }
   */
-  if (proj_id < 0)
-    memset(&proj, 0, sizeof(proj_t));
-  else
-    H5Dread(proj_id, proj_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &proj);
+  if (proj_id >= 0) H5Dread(proj_id, proj_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &proj);
 
   H5Dclose(proj_id);
   H5Tclose(proj_tid);
@@ -415,10 +407,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   region_id = H5Dopen(grp_id, "Region");
   if (region_id < 0) region_id = H5Dopen(grp_id, "region");
 
-  if (region_id < 0)
-    memset(&region, 0, sizeof(region_t));
-  else
-    H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
+  if (region_id >= 0) H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
 
   H5Dclose(region_id);
   H5Tclose(region_tid);
@@ -438,8 +427,8 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   H5Gclose(grp_id);
 
   /* check region */
-  xsize = (int) lround((region.xmax - region.xmin) / region.dx);
-  ysize = (int) lround((region.ymax - region.ymin) / region.dy);
+  xsize = (int) std::lround((region.xmax - region.xmin) / region.dx);
+  ysize = (int) std::lround((region.ymax - region.ymin) / region.dy);
 
   if (Options::cdoVerbose) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
 
@@ -457,8 +446,8 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
         cdoPrint("  Corrected region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g", region.xmin, region.xmax, region.ymin,
                  region.ymax, region.dx, region.dy);
 
-      xsize = (int) lround((region.xmax - region.xmin) / region.dx);
-      ysize = (int) lround((region.ymax - region.ymin) / region.dy);
+      xsize = (int) std::lround((region.xmax - region.xmin) / region.dx);
+      ysize = (int) std::lround((region.ymax - region.ymin) / region.dy);
       if (Options::cdoVerbose) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
     }
 
@@ -472,8 +461,8 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       cdoPrint("  Corrected region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g", region.xmin, region.xmax, region.ymin,
                region.ymax, region.dx, region.dy);
 
-      xsize = (int) lround((region.xmax - region.xmin) / region.dx);
-      ysize = (int) lround((region.ymax - region.ymin) / region.dy);
+      xsize = (int) std::lround((region.xmax - region.xmin) / region.dx);
+      ysize = (int) std::lround((region.ymax - region.ymin) / region.dy);
       if (Options::cdoVerbose) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
     }
 
@@ -511,20 +500,19 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
 
   if (nx == xsize && ny == ysize && strcmp(proj.name, "sinusoidal") == 0 && strcmp(proj.ellipsoid, "WGS-84") == 0)
     {
-      gridID = defSinusoidalGrid(nx, ny, region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy,
-                                 proj.parameter[0], proj.parameter[1], proj.parameter[2], proj.parameter[3]);
+      gridID = defSinusoidalGrid(nx, ny, region.xmin, region.ymax, region.dx, region.dy, proj.parameter[0], proj.parameter[1],
+                                 proj.parameter[2], proj.parameter[3]);
     }
   /* modification by Frank Kaspar */
   else if (nx == xsize && ny == ysize && strcmp(proj.name, "Lambert Azimuthal Equal Area") == 0
            && memcmp(proj.ellipsoid, "Sphere", 6) == 0)
     {
       double a = (proj.parameter[4] < 0) ? 6370997.0 : proj.parameter[4];
-      gridID = defLaeaGrid(nx, ny, region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy, a, proj.parameter[2],
-                           proj.parameter[3]);
+      gridID = defLaeaGrid(nx, ny, region.xmin, region.ymax, region.dx, region.dy, a, proj.parameter[2], proj.parameter[3]);
     }
   else if (memcmp(proj.name, "Cylindrical Equal Area", 22) == 0 && memcmp(proj.ellipsoid, "Sphere", 6) == 0)
     {
-      double c0 = 0.001 * sqrt(proj.parameter[5]); /* nominal spatial resolution */
+      double c0 = 0.001 * std::sqrt(proj.parameter[5]); /* nominal spatial resolution */
       double lts = proj.parameter[3];
       double re = proj.parameter[4] / 1000; /* Earth radius [km]*/
       if (Options::cdoVerbose) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
@@ -609,7 +597,7 @@ read_region(hid_t loc_id, int nx, int ny)
   }
   */
   herr_t status = H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
-  UNUSED(status);
+  (void) (status);  // CDO_UNUSED
 
   if (Options::cdoVerbose)
     {
@@ -666,7 +654,7 @@ read_region(hid_t loc_id, int nx, int ny)
 
   if (nfound == 4 && nx == region.xsize && ny == region.ysize && strcmp(proj, "laea") == 0)
     {
-      gridID = defLaeaGrid(nx, ny, xmin, xmax, ymin, ymax, dx, dy, a, lon0, lat0);
+      gridID = defLaeaGrid(nx, ny, xmin, ymax, dx, dy, a, lon0, lat0);
     }
 
   return gridID;
@@ -692,13 +680,11 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
   double addoffset = 0, scalefactor = 1, missval = cdiInqMissval();
   int laddoffset = 0, lscalefactor = 0, lmissval = 0;
   int nset;
-  int i, k;
   int ftype = 0;
   int len;
   int dtype = CDI_DATATYPE_FLT32;
   char attstring[4096]; /* Buffer to read string attribute back */
   char varname[CDI_MAX_NAME];
-  double minval, maxval;
   size_t nmiss;
   int num_attrs;
 
@@ -831,7 +817,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
       if (Options::cdoVerbose) print_filter(dset_id, varname);
 
       num_attrs = H5Aget_num_attrs(dset_id);
-      for (i = 0; i < num_attrs; i++)
+      for (int i = 0; i < num_attrs; i++)
         {
           attr = H5Aopen_idx(dset_id, i);
           atype = H5Aget_type(attr);
@@ -844,7 +830,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
           atype_class = H5Tget_class(atype);
 
           len = strlen(attname);
-          for (k = 0; k < len; ++k) attname[k] = tolower(attname[k]);
+          for (int k = 0; k < len; ++k) attname[k] = tolower(attname[k]);
 
           if (strcmp(attname, "intercept") == 0 || strcmp(attname, "offset") == 0)
             {
@@ -1015,7 +1001,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
         }
       else
         {
-          std::vector<int> iarray(gridsize * nt);
+          Varray<int> iarray(gridsize * nt);
           status = H5Dread(dset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, iarray.data());
           if (status < 0) cdoAbort("Reading of NATIVE_INT variable %s failed!", varname);
           for (size_t i = 0; i < gridsize * nt; ++i) array[i] = iarray[i];
@@ -1048,21 +1034,22 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
       nmiss = 0;
 
-      arrayMinMax(gridsize * nt, array, minval, maxval);
+      const auto mm = varrayMinMax(gridsize * nt, array);
 
       if (Options::cdoVerbose)
         cdoPrint("Dataset %s: missval = %g  addoffset = %g  scalefactor = %g", varname, missval, addoffset, scalefactor);
 
       if (Options::cdoVerbose)
-        cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, minval, maxval, missval);
+        cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, mm.min, mm.max,
+                 missval);
 
       if (dtype == CDI_DATATYPE_UINT8)
         {
-          if (minval >= 0 && maxval <= 127) dtype = CDI_DATATYPE_INT8;
+          if (mm.min >= 0 && mm.max <= 127) dtype = CDI_DATATYPE_INT8;
         }
       else if (dtype == CDI_DATATYPE_UINT16)
         {
-          if (minval >= 0 && maxval <= 32767) dtype = CDI_DATATYPE_INT16;
+          if (mm.min >= 0 && mm.max <= 32767) dtype = CDI_DATATYPE_INT16;
         }
 
       laddoffset = IS_NOT_EQUAL(addoffset, 0);
@@ -1085,13 +1072,13 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
               }
         }
 
-      minval = 1e35;
-      maxval = -1e35;
+      double minval = 1e35;
+      double maxval = -1e35;
       for (size_t i = 0; i < gridsize * nt; i++)
         if (mask[i] == false)
           {
-            if (array[i] < minval) minval = array[i];
-            if (array[i] > maxval) maxval = array[i];
+            minval = std::min(minval, array[i]);
+            maxval = std::max(maxval, array[i]);
           }
 
       if (Options::cdoVerbose)
@@ -1312,34 +1299,30 @@ void *
 Importcmsaf(void *process)
 {
 #ifdef HAVE_LIBHDF5
-  CdoStreamID streamID;
-  int gridID = -1, zaxisID, taxisID, vlistID;
+  int gridID = -1, zaxisID;
   int i, offset;
   size_t nmiss;
   int ivar;
   int varID, levelID, tsID;
-  double *array;
-  double missval, minval, maxval;
-  hid_t file_id; /* HDF5 File ID	        	*/
-  herr_t status; /* Generic return value		*/
+  double missval;
+  herr_t status; // Generic return value
   datasets_t dsets;
-  int64_t vdate;
-  int vtime;
-  int *vtimes = nullptr;
 #endif
 
   cdoInitialize(process);
 
+  operatorCheckArgc(0);
+
   if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
 
 #ifdef HAVE_LIBHDF5
   dsets_init(&dsets);
 
-  /* Open an existing file. */
-  file_id = H5Fopen(cdoGetStreamName(0), H5F_ACC_RDONLY, H5P_DEFAULT);
+  // Open an existing file.
+  hid_t file_id = H5Fopen(cdoGetStreamName(0), H5F_ACC_RDONLY, H5P_DEFAULT);
   if (file_id < 0) cdoAbort("H5Fopen failed on %s", cdoGetStreamName(0));
 
-  /* cmsaf_type = get_cmsaf_type(file_id); */
+  // cmsaf_type = get_cmsaf_type(file_id);
 
   H5Giterate(file_id, "/", nullptr, obj_info, (void *) &dsets);
 
@@ -1358,26 +1341,25 @@ Importcmsaf(void *process)
         break;
       }
 
+  Varray<int> vtimes;
   if (nt > 1)
     {
-      vtimes = (int *) Malloc(nt * sizeof(int));
+      vtimes.resize(nt);
 
       for (i = 0; i < nt; ++i) vtimes[i] = i * 10000 + 45 * 100;
 
       if (dsets.obj[ivar].time)
         {
-          long itime;
           char *pline = dsets.obj[ivar].time;
-
           for (i = 0; i < nt; ++i)
             {
-              itime = ((int) strtol(pline, &pline, 10)) * 100;
+              int itime = ((int) strtol(pline, &pline, 10)) * 100;
               if (itime < 0 || itime > 240000)
                 {
                   cdoWarning("Wrong time string!");
                   break;
                 }
-              vtimes[i] = (int) itime;
+              vtimes[i] = itime;
             }
         }
     }
@@ -1412,29 +1394,26 @@ Importcmsaf(void *process)
     zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
   else
     {
-      std::vector<double> levels(nz);
+      Varray<double> levels(nz);
       for (i = 0; i < nz; ++i) levels[i] = i + 1;
       zaxisID = zaxisCreate(ZAXIS_GENERIC, nz);
       zaxisDefLevels(zaxisID, levels.data());
     }
 
-  vlistID = vlistCreate();
+  const auto vlistID = vlistCreate();
 
-  taxisID = cdoTaxisCreate(nt > 1 ? TAXIS_RELATIVE : TAXIS_ABSOLUTE);
+  const auto taxisID = cdoTaxisCreate(nt > 1 ? TAXIS_RELATIVE : TAXIS_ABSOLUTE);
   taxisDefCalendar(taxisID, CALENDAR_STANDARD);
   vlistDefTaxis(vlistID, taxisID);
 
   for (ivar = 0; ivar < dsets.nsets; ++ivar)
     {
       varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
-      vlistDefVarName(vlistID, varID, dsets.obj[ivar].name);
-      if (dsets.obj[ivar].description) vlistDefVarLongname(vlistID, varID, dsets.obj[ivar].description);
-      if (dsets.obj[ivar].units) vlistDefVarUnits(vlistID, varID, dsets.obj[ivar].units);
+      cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, dsets.obj[ivar].name);
+      if (dsets.obj[ivar].description) cdiDefKeyString(vlistID, varID, CDI_KEY_LONGNAME, dsets.obj[ivar].description);
+      if (dsets.obj[ivar].units) cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, dsets.obj[ivar].units);
       if (dsets.obj[ivar].title) cdiDefAttTxt(vlistID, varID, "title", (int) strlen(dsets.obj[ivar].title), dsets.obj[ivar].title);
 
-      /*
-      vlistDefVarUnits(vlistID, varID, units[i]);
-      */
       vlistDefVarDatatype(vlistID, varID, dsets.obj[ivar].dtype);
       if (dsets.obj[ivar].lmissval) vlistDefVarMissval(vlistID, varID, dsets.obj[ivar].missval);
       if (dsets.obj[ivar].lscale) vlistDefVarScalefactor(vlistID, varID, dsets.obj[ivar].scale);
@@ -1444,18 +1423,17 @@ Importcmsaf(void *process)
   get_global_att(file_id, "/", vlistID);
   if (dsets.lmetadata) get_global_att(file_id, "Metadata", vlistID);
 
-  vdate = get_vdate(vlistID);
+  int vdate = get_vdate(vlistID);
   if (vdate == 0) vdate = 10101;
 
-  streamID = cdoOpenWrite(1);
+  const auto streamID = cdoOpenWrite(1);
 
   cdoDefVlist(streamID, vlistID);
 
   for (tsID = 0; tsID < nt; ++tsID)
     {
       taxisDefVdate(taxisID, vdate);
-      vtime = 0;
-      if (vtimes) vtime = vtimes[tsID];
+      const int vtime = (vtimes.empty()) ? 0 : vtimes[tsID];
       taxisDefVtime(taxisID, vtime);
       cdoDefTimestep(streamID, tsID);
 
@@ -1472,29 +1450,24 @@ Importcmsaf(void *process)
             {
               offset = gridsize * levelID;
               if (nz == 1) offset = gridsize * tsID;
-              array = dsets.obj[ivar].array + offset;
+              double *array = dsets.obj[ivar].array + offset;
 
-              arrayMinMaxMV(gridsize, array, missval, minval, maxval);
-
-              nmiss = arrayNumMV(gridsize, array, missval);
+              const auto mm = varrayMinMaxMV(gridsize, array, missval);
+              nmiss = gridsize - mm.n;
 
               if (Options::cdoVerbose)
                 cdoPrint(" Write var %d,  level %d, nmiss %zu, missval %g, minval %g, maxval %g", varID, levelID, nmiss, missval,
-                         minval, maxval);
-              /*
-                if ( ! (missval < minval || missval > maxval) )
-                cdoWarning(" Missval is inside of valid values! Name: %s  Range:
-                %g - %g  Missval: %g\n", dsets.obj[ivar].name, minval, maxval, missval);
-              */
+                         mm.min, mm.max);
+
               cdoDefRecord(streamID, varID, levelID);
               cdoWriteRecord(streamID, array, nmiss);
             }
         }
     }
 
-  /* Close file */
+  // Close file
   status = H5Fclose(file_id);
-  UNUSED(status);
+  (void) (status);  // CDO_UNUSED
 
   processDefVarNum(vlistNvars(vlistID));
 
@@ -1507,8 +1480,6 @@ Importcmsaf(void *process)
 
   for (ivar = 0; ivar < dsets.nsets; ++ivar) Free(dsets.obj[ivar].array);
 
-  if (vtimes) Free(vtimes);
-
   cdoFinish();
 #else
   cdoAbort("HDF5 support not compiled in!");
