@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,16 +18,14 @@
 #include <cdi.h>
 
 #include "cdo_options.h"
-#include "dmemory.h"
 #include "process_int.h"
+#include "array.h"
 #include "readline.h"
-#include "cdo_cdiWrapper.h"
+#include "cdo_cdi_wrapper.h"
 #include <mpim_grid.h>
 #include "griddes.h"
 
 
-#define MAX_VARS 6
-
 static void
 init_vars(int vlistID, int gridID, int zaxisID, int nvars)
 {
@@ -37,16 +35,16 @@ init_vars(int vlistID, int gridID, int zaxisID, int nvars)
 
   for (int i = 0; i < nvars; ++i)
     {
-      int varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
+      const auto varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
       vlistDefVarParam(vlistID, varID, cdiEncodeParam(code[i], 255, 255));
-      vlistDefVarName(vlistID, varID, name[i]);
-      vlistDefVarUnits(vlistID, varID, units[i]);
+      cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, name[i]);
+      cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, units[i]);
       vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_FLT32);
     }
 }
 
 static void
-init_data(int vlistID, int nvars, double *data[])
+init_data(int vlistID, int nvars, Varray2D<double> &data)
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
@@ -57,15 +55,15 @@ init_data(int vlistID, int nvars, double *data[])
 }
 
 static void
-write_data(CdoStreamID streamID, int vlistID, int nvars, double *data[])
+write_data(CdoStreamID streamID, int vlistID, int nvars, Varray2D<double> &data)
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
       const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
       const auto missval = vlistInqVarMissval(vlistID, varID);
-      const auto nmiss = arrayNumMV(gridsize, data[varID], missval);
+      const auto nmiss = varrayNumMV(gridsize, data[varID], missval);
       cdoDefRecord(streamID, varID, 0);
-      cdoWriteRecord(streamID, data[varID], nmiss);
+      cdoWriteRecord(streamID, data[varID].data(), nmiss);
     }
 }
 
@@ -83,17 +81,14 @@ void *
 Importobs(void *process)
 {
   char line[MAX_LINE_LEN];
-  int i, j;
-  int nvars = MAX_VARS;
+  size_t i, j;
+  long index;
+  constexpr int nvars = 6;
   int vtime = 0;
-  double *data[MAX_VARS];
   char dummy[32], station[32], datetime[32];
   float lat, lon, height1, pressure, height2, value;
   double latmin = 90, latmax = -90, lonmin = 360, lonmax = -360;
   int code;
-  int index;
-  double dx, dy;
-  char *pstation;
 
   cdoInitialize(process);
 
@@ -108,27 +103,18 @@ Importobs(void *process)
   if (gridInqType(gridID) != GRID_LONLAT) cdoAbort("Unsupported grid type: %s", gridNamePtr(gridInqType(gridID)));
 
   const auto gridsize = gridInqSize(gridID);
-  const int xsize = gridInqXsize(gridID);
-  const int ysize = gridInqYsize(gridID);
-
-  // printf("gridsize=%d, xsize=%d, ysize=%d\n", gridsize, xsize, ysize);
-
-  Varray<double> xvals(gridsize);
-  Varray<double> yvals(gridsize);
+  const auto xsize = gridInqXsize(gridID);
+  const auto ysize = gridInqYsize(gridID);
 
+  Varray<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, xvals.data());
   gridInqYvals(gridID, yvals.data());
 
   // Convert lat/lon units if required
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridID, units);
-    grid_to_degree(units, gridsize, xvals.data(), "grid center lon");
-    gridInqYunits(gridID, units);
-    grid_to_degree(units, gridsize, yvals.data(), "grid center lat");
-  }
+  cdo_grid_to_degree(gridID, CDI_XAXIS, gridsize, xvals.data(), "grid center lon");
+  cdo_grid_to_degree(gridID, CDI_YAXIS, gridsize, yvals.data(), "grid center lat");
 
-  FILE *fp = fopen(cdoGetStreamName(0), "r");
+  auto fp = fopen(cdoGetStreamName(0), "r");
   if (fp == nullptr)
     {
       perror(cdoGetStreamName(0));
@@ -147,7 +133,8 @@ Importobs(void *process)
   const auto taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID, taxisID);
 
-  for (i = 0; i < nvars; ++i) data[i] = (double *) Malloc(gridsize * sizeof(double));
+  Varray2D<double> data(nvars);
+  for (i = 0; i < nvars; ++i) data[i].resize(gridsize);
 
   init_vars(vlistID, gridID, zaxisID, nvars);
 
@@ -157,7 +144,7 @@ Importobs(void *process)
   int vtime0 = 0;
   // ntime = 0;
   int tsID = 0;
-  while (readline(fp, line, MAX_LINE_LEN))
+  while (cdo::readline(fp, line, MAX_LINE_LEN))
     {
       sscanf(line, "%s %s %s %g %g %g %d %g %g %g", dummy, station, datetime, &lat, &lon, &height1, &code, &pressure, &height2,
              &value);
@@ -186,11 +173,11 @@ Importobs(void *process)
       if (lat < latmin) latmin = lat;
       if (lat > latmax) latmax = lat;
 
-      dy = yvals[1] - yvals[0];
+      const auto dy = yvals[1] - yvals[0];
       for (j = 0; j < ysize; ++j)
         if (lat >= (yvals[j] - dy / 2) && lat < (yvals[j] + dy / 2)) break;
 
-      dx = xvals[1] - xvals[0];
+      const auto dx = xvals[1] - xvals[0];
       if (lon < (xvals[0] - dx / 2) && lon < 0) lon += 360;
       for (i = 0; i < xsize; ++i)
         if (lon >= (xvals[i] - dx / 2) && lon < (xvals[i] + dx / 2)) break;
@@ -204,7 +191,7 @@ Importobs(void *process)
       // printf("%d %d %d %g %g %g %g\n", i, j, index, dx, dy, lon, lat);
       if (i < xsize && j < ysize && index >= 0)
         {
-          pstation = station;
+          char *pstation = station;
           while (isalpha(*pstation)) pstation++;
           // printf("station %s %d\n", pstation, atoi(pstation));
           data[index][j * xsize + i] = value;
@@ -220,8 +207,6 @@ Importobs(void *process)
 
   write_data(streamID, vlistID, nvars, data);
 
-  for (i = 0; i < nvars; ++i) Free(data[i]);
-
   if (Options::cdoVerbose) printf("lonmin=%g, lonmax=%g, latmin=%g, latmax=%g\n", lonmin, lonmax, latmin, latmax);
 
   processDefVarNum(vlistNvars(vlistID));
