@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,12 +26,12 @@
 #include <cdi.h>
 
 #include <mpim_grid.h>
-#include "cdo_cdiWrapper.h"
+#include "cdo_cdi_wrapper.h"
 #include "process_int.h"
 #include "griddes.h"
 #include "cdo_zaxis.h"
 #include "grid_read_pingo.h"
-#include "cdo_defaultValues.h"
+#include "cdo_default_values.h"
 
 
 static size_t
@@ -41,7 +41,7 @@ input_iarray(size_t nval, int *array)
 
   for (size_t i = 0; i < nval; i++)
     {
-      const size_t n = scanf("%d", &array[i]);
+      const auto n = scanf("%d", &array[i]);
       if (n != 1) break;
 
       ival++;
@@ -62,7 +62,6 @@ Input(void *process)
   int output_filetype = CDI_FILETYPE_GRB;
   int ihead[8];
   double missval = 0;
-  double levels[1];
   Varray<double> array;
 
   cdoInitialize(process);
@@ -88,11 +87,14 @@ Input(void *process)
       gridID = cdoDefineGrid(cdoOperatorArgv(0));
       if (operatorArgc() == 2) zaxisID = cdoDefineZaxis(cdoOperatorArgv(1));
     }
+  else
+    {
+      operatorCheckArgc(0);
+    }
 
-  int nlevs = 1;
-  if (zaxisID != -1) nlevs = zaxisInqSize(zaxisID);
+  const int nlevs = (zaxisID == -1) ? 1 : zaxisInqSize(zaxisID);
 
-  levels[0] = 0;
+  double dlevel = 0;
   int nrecs = 0;
 
   int tsID = 0;
@@ -111,7 +113,7 @@ Input(void *process)
 
           cdoPrint("Enter all %zu elements of timestep %d!", gridsize * nlevs, nrecs + 1);
 
-          const size_t rval = input_darray(stdin, gridsize * nlevs, array.data());
+          const auto rval = input_darray(stdin, gridsize * nlevs, array);
 
           if (nrecs > 0 && rval == 0) break;
 
@@ -126,7 +128,7 @@ Input(void *process)
 
           cdoPrint("Enter header (code,level,date,time,nlon,nlat,dispo1,dispo2) of record %d (or EOF(=^D))!", nrecs + 1);
 
-          int rval = input_iarray(4, ihead);
+          auto rval = input_iarray(4, ihead);
           if (feof(stdin) && nrecs == 0) cdoAbort("Too few header elements (%d of %d)!", rval, 4);
           if (feof(stdin)) break;
           if (rval != 4) cdoAbort("Invalid header input!");
@@ -140,7 +142,7 @@ Input(void *process)
 
           if (nrecs == 0)
             {
-              levels[0] = level;
+              dlevel = level;
               gridsize0 = gridsize;
               array.resize(gridsize);
               gridID = gridCreate(GRID_GENERIC, gridsize);
@@ -152,8 +154,8 @@ Input(void *process)
 
           cdoPrint("Enter all %zu elements of record %d!", gridsize, nrecs + 1);
 
-          rval = input_darray(stdin, gridsize, array.data());
-          if (rval != (int) gridsize) cdoAbort("Invalid data input!");
+          rval = input_darray(stdin, gridsize, array);
+          if (rval != gridsize) cdoAbort("Invalid data input!");
         }
       else if (operatorID == INPUTSRV)
         {
@@ -162,7 +164,7 @@ Input(void *process)
 
           cdoPrint("Enter header (code,level,date,time,nlon,nlat,dispo1,dispo2) of record %d (or EOF(=^D))!", nrecs + 1);
 
-          int rval = input_iarray(8, ihead);
+          auto rval = input_iarray(8, ihead);
           if (feof(stdin) && nrecs == 0) cdoAbort("Too few header elements (%d of %d)!", rval, 8);
           if (feof(stdin)) break;
           if (rval != 8) cdoAbort("Invalid header input!");
@@ -178,7 +180,7 @@ Input(void *process)
 
           if (nrecs == 0)
             {
-              levels[0] = level;
+              dlevel = level;
               gridsize0 = gridsize;
               array.resize(gridsize);
               gridID = gridCreate(GRID_GENERIC, gridsize);
@@ -192,8 +194,8 @@ Input(void *process)
 
           cdoPrint("Enter all %zu elements of record %d!", gridsize, nrecs + 1);
 
-          rval = input_darray(stdin, gridsize, array.data());
-          if (rval != (int) gridsize) cdoAbort("Invalid data input!");
+          rval = input_darray(stdin, gridsize, array);
+          if (rval != gridsize) cdoAbort("Invalid data input!");
         }
 
       if (nrecs == 0)
@@ -201,7 +203,7 @@ Input(void *process)
           if (zaxisID == -1)
             {
               zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
-              zaxisDefLevels(zaxisID, levels);
+              zaxisDefLevels(zaxisID, &dlevel);
             }
 
           vlistID = vlistCreate();
@@ -213,12 +215,12 @@ Input(void *process)
           taxisID = cdoTaxisCreate(TAXIS_RELATIVE);
           vlistDefTaxis(vlistID, taxisID);
 
-          streamID = cdoOpenWrite((0), output_filetype);
+          streamID = cdoOpenWrite(0, output_filetype);
           cdoDefVlist(streamID, vlistID);
         }
 
-      auto vdate = date;
-      auto vtime = time;
+      const auto vdate = date;
+      const auto vtime = time;
       taxisDefVdate(taxisID, vdate);
       taxisDefVtime(taxisID, vtime);
       cdoDefTimestep(streamID, tsID);
