@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,17 +30,12 @@
 #include "datetime.h"
 #include "printinfo.h"
 
-
 size_t inttime(double fac1, double fac2, size_t gridsize, const double *single1, const double *single2, Varray<double> &array,
                bool withMissval, double missval1, double missval2);
 
 void *
 Intntime(void *process)
 {
-  int varID, levelID;
-  int64_t vdate;
-  int vtime;
-
   cdoInitialize(process);
 
   operatorInputArg("number of timesteps between 2 timesteps");
@@ -73,7 +68,7 @@ Intntime(void *process)
   vardata[0].resize(nvars);
   vardata[1].resize(nvars);
 
-  for (varID = 0; varID < nvars; varID++)
+  for (int varID = 0; varID < nvars; varID++)
     {
       const auto gridsize = varList1[varID].gridsize;
       const auto nlevel = varList1[varID].nlevels;
@@ -92,7 +87,7 @@ Intntime(void *process)
 
   cdoDefVlist(streamID2, vlistID2);
 
-  auto calendar = taxisInqCalendar(taxisID1);
+  const auto calendar = taxisInqCalendar(taxisID1);
 
   int curFirst = 0, curSecond = 1;
 
@@ -107,9 +102,10 @@ Intntime(void *process)
   cdoDefTimestep(streamID2, tsIDo++);
   for (int recID = 0; recID < nrecs; recID++)
     {
+      int varID, levelID;
       cdoInqRecord(streamID1, &varID, &levelID);
       const auto offset = varList1[varID].gridsize * levelID;
-      double *single1 = &vardata[curFirst][varID][offset];
+      auto single1 = &vardata[curFirst][varID][offset];
       cdoReadRecord(streamID1, single1, &nmiss[curFirst][varID][levelID]);
 
       cdoDefRecord(streamID2, varID, levelID);
@@ -120,29 +116,31 @@ Intntime(void *process)
     {
       const auto vdate2 = taxisInqVdate(taxisID1);
       const auto vtime2 = taxisInqVtime(taxisID1);
-      auto juldate2 = julianDateEncode(calendar, vdate2, vtime2);
+      const auto juldate2 = julianDateEncode(calendar, vdate2, vtime2);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           recList[recID].varID = varID;
           recList[recID].levelID = levelID;
 
           const auto offset = varList1[varID].gridsize * levelID;
-          double *single2 = &vardata[curSecond][varID][offset];
+          auto single2 = &vardata[curSecond][varID][offset];
           cdoReadRecord(streamID1, single2, &nmiss[curSecond][varID][levelID]);
         }
 
       for (int it = 1; it < numts; it++)
         {
           const double seconds = it * julianDateToSeconds(julianDateSub(juldate2, juldate1)) / numts;
-          auto juldate = julianDateAddSeconds(lround(seconds), juldate1);
+          const auto juldate = julianDateAddSeconds(lround(seconds), juldate1);
 
+          int64_t vdate;
+          int vtime;
           julianDateDecode(calendar, juldate, vdate, vtime);
 
-          if (Options::cdoVerbose)
-            cdoPrint("%s %s", dateToString(vdate).c_str(), timeToString(vtime).c_str());
+          if (Options::cdoVerbose) cdoPrint("%s %s", dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
           taxisDefVdate(taxisID2, vdate);
           taxisDefVtime(taxisID2, vtime);
@@ -154,16 +152,16 @@ Intntime(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              varID = recList[recID].varID;
-              levelID = recList[recID].levelID;
+              const auto varID = recList[recID].varID;
+              const auto levelID = recList[recID].levelID;
               const auto gridsize = varList1[varID].gridsize;
               const auto offset = gridsize * levelID;
-              const double *single1 = &vardata[curFirst][varID][offset];
-              const double *single2 = &vardata[curSecond][varID][offset];
+              const auto single1 = &vardata[curFirst][varID][offset];
+              const auto single2 = &vardata[curSecond][varID][offset];
 
               const bool withMissval = nmiss[curFirst][varID][levelID] || nmiss[curSecond][varID][levelID];
-              const size_t nmiss3 = inttime(fac1, fac2, gridsize, single1, single2, array, withMissval, varList1[varID].missval,
-                                            varList2[varID].missval);
+              const auto nmiss3 = inttime(fac1, fac2, gridsize, single1, single2, array, withMissval, varList1[varID].missval,
+                                          varList2[varID].missval);
 
               cdoDefRecord(streamID2, varID, levelID);
               cdoWriteRecord(streamID2, array.data(), nmiss3);
@@ -175,10 +173,10 @@ Intntime(void *process)
       cdoDefTimestep(streamID2, tsIDo++);
       for (int recID = 0; recID < nrecs; recID++)
         {
-          varID = recList[recID].varID;
-          levelID = recList[recID].levelID;
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
           const auto offset = varList2[varID].gridsize * levelID;
-          double *single2 = &vardata[curSecond][varID][offset];
+          auto single2 = &vardata[curSecond][varID][offset];
 
           cdoDefRecord(streamID2, varID, levelID);
           cdoWriteRecord(streamID2, single2, nmiss[curSecond][varID][levelID]);
@@ -193,5 +191,5 @@ Intntime(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
