@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,16 +23,19 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "param_conversion.h"
+#include <utility>
 
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "param_conversion.h"
 #include "datetime.h"
 #include "printinfo.h"
 
-int get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit);
+int get_tunits(const char *unit, int &incperiod, int &incunit, int &tunit);
 
 size_t
-inttime(double fac1, double fac2, size_t gridsize, const double *single1, const double *single2, std::vector<double> &array,
+inttime(double fac1, double fac2, size_t gridsize, const double *single1, const double *single2, Varray<double> &array,
         bool withMissval, double missval1, double missval2)
 {
   size_t nmiss = 0;
@@ -69,23 +72,13 @@ juldateAddIncrement(JulianDate &juldate, int64_t ijulinc, int calendar, int tuni
     {
       int64_t vdate;
       int vtime;
-      julianDateDecode(calendar, juldate, &vdate, &vtime);
+      julianDateDecode(calendar, juldate, vdate, vtime);
 
       int year, month, day;
       cdiDecodeDate(vdate, &year, &month, &day);
 
       month += (int) ijulinc;
-
-      while (month > 12)
-        {
-          month -= 12;
-          year++;
-        }
-      while (month < 1)
-        {
-          month += 12;
-          year--;
-        }
+      adjustMonthAndYear(month, year);
 
       vdate = cdiEncodeDate(year, month, day);
 
@@ -101,7 +94,6 @@ void *
 Inttime(void *process)
 {
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
-  int varID, levelID;
   int64_t vdate;
   int vtime;
   int incperiod = 0, incunit = 3600, tunit = TUNIT_HOUR;
@@ -112,8 +104,8 @@ Inttime(void *process)
   operatorInputArg("date,time<,increment> (format YYYY-MM-DD,hh:mm:ss)");
   if (operatorArgc() < 2) cdoAbort("Too few arguments!");
 
-  const char *datestr = operatorArgv()[0];
-  const char *timestr = operatorArgv()[1];
+  const auto datestr = cdoOperatorArgv(0).c_str();
+  const auto timestr = cdoOperatorArgv(1).c_str();
 
   if (strchr(datestr, '-'))
     {
@@ -143,60 +135,60 @@ Inttime(void *process)
 
   if (operatorArgc() == 3)
     {
-      const char *timeunits = operatorArgv()[2];
+      auto timeunits = cdoOperatorArgv(2).c_str();
       incperiod = (int) strtol(timeunits, nullptr, 10);
       if (timeunits[0] == '-' || timeunits[0] == '+') timeunits++;
       while (isdigit((int) *timeunits)) timeunits++;
 
-      get_tunits(timeunits, &incperiod, &incunit, &tunit);
+      get_tunits(timeunits, incperiod, incunit, tunit);
     }
   /* increment in seconds */
   const int64_t ijulinc = (int64_t) incperiod * incunit;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
   VarList varList1, varList2;
   varListInit(varList1, vlistID1);
   varListInit(varList2, vlistID2);
-  
+
   if (ijulinc == 0) vlistDefNtsteps(vlistID2, 1);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
 
-  std::vector<std::vector<std::vector<size_t>>> nmiss(2);
+  Varray3D<size_t> nmiss(2);
   nmiss[0].resize(nvars);
   nmiss[1].resize(nvars);
-  std::vector<std::vector<std::vector<double>>> vardata(2);
+  Varray3D<double> vardata(2);
   vardata[0].resize(nvars);
   vardata[1].resize(nvars);
 
-  for (varID = 0; varID < nvars; varID++)
+  for (int varID = 0; varID < nvars; varID++)
     {
-      const size_t gridsize = varList1[varID].gridsize;
-      const int nlevel = varList1[varID].nlevels;
+      const auto gridsize = varList1[varID].gridsize;
+      const auto nlevel = varList1[varID].nlevels;
       nmiss[0][varID].resize(nlevel);
       nmiss[1][varID].resize(nlevel);
       vardata[0][varID].resize(gridsize * nlevel);
       vardata[1][varID].resize(gridsize * nlevel);
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID2)) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int calendar = taxisInqCalendar(taxisID1);
+  const auto calendar = taxisInqCalendar(taxisID1);
 
-  JulianDate juldate = julianDateEncode(calendar, vdate, vtime);
+  auto juldate = julianDateEncode(calendar, vdate, vtime);
 
   if (Options::cdoVerbose)
     {
@@ -208,13 +200,14 @@ Inttime(void *process)
   int curFirst = 0, curSecond = 1;
 
   int tsID = 0;
-  int nrecs = cdoStreamInqTimestep(streamID1, tsID++);
-  JulianDate juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
+  auto nrecs = cdoStreamInqTimestep(streamID1, tsID++);
+  auto juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
   for (int recID = 0; recID < nrecs; recID++)
     {
+      int varID, levelID;
       cdoInqRecord(streamID1, &varID, &levelID);
-      const size_t offset = varList1[varID].gridsize * levelID;
-      double *single1 = &vardata[curFirst][varID][offset];
+      const auto offset = varList1[varID].gridsize * levelID;
+      auto single1 = &vardata[curFirst][varID][offset];
       cdoReadRecord(streamID1, single1, &nmiss[curFirst][varID][levelID]);
     }
 
@@ -232,7 +225,7 @@ Inttime(void *process)
       nrecs = cdoStreamInqTimestep(streamID1, tsID++);
       if (nrecs == 0) break;
 
-      JulianDate juldate2 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
+      auto juldate2 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
       if (Options::cdoVerbose)
         {
           cdoPrint("date %ld  time %d", taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
@@ -241,13 +234,14 @@ Inttime(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           recList[recID].varID = varID;
           recList[recID].levelID = levelID;
           recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
-          const size_t offset = varList1[varID].gridsize * levelID;
+          const auto offset = varList1[varID].gridsize * levelID;
           double *single2 = &vardata[curSecond][varID][offset];
           cdoReadRecord(streamID1, single2, &nmiss[curSecond][varID][levelID]);
         }
@@ -257,15 +251,11 @@ Inttime(void *process)
           if (julianDateToSeconds(juldate) >= julianDateToSeconds(juldate1)
               && julianDateToSeconds(juldate) <= julianDateToSeconds(juldate2))
             {
-              julianDateDecode(calendar, juldate, &vdate, &vtime);
+              julianDateDecode(calendar, juldate, vdate, vtime);
 
               if (Options::cdoVerbose)
-                {
-                  char vdatestr[32], vtimestr[32];
-                  date2str(vdate, vdatestr, sizeof(vdatestr));
-                  time2str(vtime, vtimestr, sizeof(vtimestr));
-                  cdoPrint("%s %s  %f  %d", vdatestr, vtimestr, julianDateToSeconds(juldate), calendar);
-                }
+                cdoPrint("%s %s  %f  %d", dateToString(vdate).c_str(), timeToString(vtime).c_str(), julianDateToSeconds(juldate),
+                         calendar);
 
               if (streamID2 == CDO_STREAM_UNDEF)
                 {
@@ -277,22 +267,22 @@ Inttime(void *process)
               taxisDefVtime(taxisID2, vtime);
               cdoDefTimestep(streamID2, tsIDo++);
 
-              const double diff = julianDateToSeconds(julianDateSub(juldate2, juldate1));
-              const double fac1 = julianDateToSeconds(julianDateSub(juldate2, juldate)) / diff;
-              const double fac2 = julianDateToSeconds(julianDateSub(juldate, juldate1)) / diff;
+              const auto diff = julianDateToSeconds(julianDateSub(juldate2, juldate1));
+              const auto fac1 = julianDateToSeconds(julianDateSub(juldate2, juldate)) / diff;
+              const auto fac2 = julianDateToSeconds(julianDateSub(juldate, juldate1)) / diff;
 
               for (int recID = 0; recID < nrecs; recID++)
                 {
-                  varID = recList[recID].varID;
-                  levelID = recList[recID].levelID;
-                  const size_t gridsize = varList1[varID].gridsize;
-                  const size_t offset = gridsize * levelID;
-                  const double *single1 = &vardata[curFirst][varID][offset];
-                  const double *single2 = &vardata[curSecond][varID][offset];
+                  const auto varID = recList[recID].varID;
+                  const auto levelID = recList[recID].levelID;
+                  const auto gridsize = varList1[varID].gridsize;
+                  const auto offset = gridsize * levelID;
+                  const auto single1 = &vardata[curFirst][varID][offset];
+                  const auto single2 = &vardata[curSecond][varID][offset];
 
                   const bool withMissval = nmiss[curFirst][varID][levelID] || nmiss[curSecond][varID][levelID];
-                  const size_t nmiss3 = inttime(fac1, fac2, gridsize, single1, single2, array,
-                                                withMissval, varList1[varID].missval, varList2[varID].missval);
+                  const auto nmiss3 = inttime(fac1, fac2, gridsize, single1, single2, array, withMissval, varList1[varID].missval,
+                                              varList2[varID].missval);
 
                   cdoDefRecord(streamID2, varID, levelID);
                   cdoWriteRecord(streamID2, array.data(), nmiss3);
