@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,18 +28,19 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
 
 static void
 invertLonDes(int vlistID)
 {
-  const int ngrids = vlistNgrids(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID, index);
-      const int gridID2 = gridDuplicate(gridID1);
+      const auto gridID1 = vlistGrid(vlistID, index);
+      const auto gridID2 = gridDuplicate(gridID1);
 
-      const int gridtype = gridInqType(gridID1);
+      const auto gridtype = gridInqType(gridID1);
 
       if (!(gridtype == GRID_GENERIC || gridtype == GRID_GAUSSIAN || gridtype == GRID_PROJECTION || gridtype == GRID_LONLAT
             || gridtype == GRID_CURVILINEAR))
@@ -47,12 +48,11 @@ invertLonDes(int vlistID)
 
       if (gridInqXvals(gridID1, nullptr))
         {
-          const size_t nlon = gridInqXsize(gridID1);
-          const size_t nlat = gridInqYsize(gridID1);
-          const size_t size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlon;
+          const auto nlon = gridInqXsize(gridID1);
+          const auto nlat = gridInqYsize(gridID1);
+          const auto size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlon;
 
-          std::vector<double> xv1(size);
-          std::vector<double> xv2(size);
+          Varray<double> xv1(size), xv2(size);
 
           gridInqXvals(gridID1, xv1.data());
 
@@ -71,13 +71,12 @@ invertLonDes(int vlistID)
 
       if (gridInqXbounds(gridID1, nullptr))
         {
-          const size_t nlon = gridInqXsize(gridID1);
-          const size_t nlat = gridInqYsize(gridID1);
-          const int nv = gridInqNvertex(gridID1);
-          const size_t size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlon;
+          const auto nlon = gridInqXsize(gridID1);
+          const auto nlat = gridInqYsize(gridID1);
+          const auto nv = gridInqNvertex(gridID1);
+          const auto size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlon;
 
-          std::vector<double> xb1(size);
-          std::vector<double> xb2(size);
+          Varray<double> xb1(size), xb2(size);
 
           gridInqXbounds(gridID1, xb1.data());
 
@@ -107,16 +106,15 @@ invertLonDes(int vlistID)
 static void
 invertLatCoord(int gridID)
 {
-  const int gridtype = gridInqType(gridID);
+  const auto gridtype = gridInqType(gridID);
 
   if (gridInqYvals(gridID, nullptr))
     {
-      const size_t nlon = gridInqXsize(gridID);
-      const size_t nlat = gridInqYsize(gridID);
-      const size_t size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlat;
+      const auto nlon = gridInqXsize(gridID);
+      const auto nlat = gridInqYsize(gridID);
+      const auto size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlat;
 
-      std::vector<double> yv1(size);
-      std::vector<double> yv2(size);
+      Varray<double> yv1(size), yv2(size);
 
       if (gridtype == GRID_CURVILINEAR)
         {
@@ -146,13 +144,12 @@ invertLatCoord(int gridID)
 
   if (gridInqYbounds(gridID, nullptr))
     {
-      const size_t nlon = gridInqXsize(gridID);
-      const size_t nlat = gridInqYsize(gridID);
-      const int nv = gridInqNvertex(gridID);
-      const size_t size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlat;
+      const auto nlon = gridInqXsize(gridID);
+      const auto nlat = gridInqYsize(gridID);
+      const auto nv = gridInqNvertex(gridID);
+      const auto size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlat;
 
-      std::vector<double> yb1(size);
-      std::vector<double> yb2(size);
+      Varray<double> yb1(size), yb2(size);
 
       gridInqYbounds(gridID, yb1.data());
 
@@ -179,13 +176,13 @@ invertLatCoord(int gridID)
 static void
 invertLatDes(int vlistID)
 {
-  const int ngrids = vlistNgrids(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID, index);
-      const int gridID2 = gridDuplicate(gridID1);
+      const auto gridID1 = vlistGrid(vlistID, index);
+      const auto gridID2 = gridDuplicate(gridID1);
 
-      const int gridtype = gridInqType(gridID1);
+      const auto gridtype = gridInqType(gridID1);
 
       if (!(gridtype == GRID_GENERIC || gridtype == GRID_GAUSSIAN || gridtype == GRID_PROJECTION || gridtype == GRID_LONLAT
             || gridtype == GRID_CURVILINEAR))
@@ -193,7 +190,7 @@ invertLatDes(int vlistID)
 
       invertLatCoord(gridID2);
 
-      const int projID = gridInqProj(gridID2);
+      const auto projID = gridInqProj(gridID2);
       if (projID != CDI_UNDEFID) invertLatCoord(projID);
 
       vlistChangeGrid(vlistID, gridID1, gridID2);
@@ -203,13 +200,12 @@ invertLatDes(int vlistID)
 static void
 invertLonData(double *array1, double *array2, int gridID1)
 {
-  const size_t nlon = gridInqXsize(gridID1);
-  const size_t nlat = gridInqYsize(gridID1);
+  const auto nlon = gridInqXsize(gridID1);
+  const auto nlat = gridInqYsize(gridID1);
 
   if (nlat > 0)
     {
-      double **field1 = (double **) Malloc(nlat * sizeof(double *));
-      double **field2 = (double **) Malloc(nlat * sizeof(double *));
+      std::vector<double *> field1(nlat), field2(nlat);
 
       for (size_t ilat = 0; ilat < nlat; ilat++)
         {
@@ -219,9 +215,6 @@ invertLonData(double *array1, double *array2, int gridID1)
 
       for (size_t ilat = 0; ilat < nlat; ilat++)
         for (size_t ilon = 0; ilon < nlon; ilon++) field2[ilat][nlon - ilon - 1] = field1[ilat][ilon];
-
-      if (field1) Free(field1);
-      if (field2) Free(field2);
     }
   else
     {
@@ -232,13 +225,12 @@ invertLonData(double *array1, double *array2, int gridID1)
 static void
 invertLatData(double *array1, double *array2, int gridID1)
 {
-  const size_t nlon = gridInqXsize(gridID1);
-  const size_t nlat = gridInqYsize(gridID1);
+  const auto nlon = gridInqXsize(gridID1);
+  const auto nlat = gridInqYsize(gridID1);
 
   if (nlat > 0)
     {
-      double **field1 = (double **) Malloc(nlat * sizeof(double *));
-      double **field2 = (double **) Malloc(nlat * sizeof(double *));
+      std::vector<double *> field1(nlat), field2(nlat);
 
       for (size_t ilat = 0; ilat < nlat; ilat++)
         {
@@ -247,9 +239,6 @@ invertLatData(double *array1, double *array2, int gridID1)
         }
 
       for (size_t ilat = 0; ilat < nlat; ilat++) arrayCopy(nlon, field1[ilat], field2[nlat - ilat - 1]);
-
-      if (field1) Free(field1);
-      if (field2) Free(field2);
     }
   else
     {
@@ -276,17 +265,19 @@ Invert(void *process)
   cdoOperatorAdd("invertlondata", func_fld, func_lon, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc1 = cdoOperatorF1(operatorID);
-  const int operfunc2 = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc1 = cdoOperatorF1(operatorID);
+  const auto operfunc2 = cdoOperatorF2(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   if (operfunc1 == func_all || operfunc1 == func_hrd)
@@ -297,14 +288,12 @@ Invert(void *process)
         invertLonDes(vlistID2);
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  const size_t gridsize = vlistGridsizeMax(vlistID1);
-
-  std::vector<double> array1(gridsize);
-  std::vector<double> array2(gridsize);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax), array2(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
