@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,25 +23,26 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 
 static void
 invertLevDes(int vlistID)
 {
-  const int nzaxis = vlistNzaxis(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; index++)
     {
-      const int zaxisID1 = vlistZaxis(vlistID, index);
-      const int zaxisID2 = zaxisDuplicate(zaxisID1);
-      const int zaxistype = zaxisInqType(zaxisID1);
+      const auto zaxisID1 = vlistZaxis(vlistID, index);
+      const auto zaxisID2 = zaxisDuplicate(zaxisID1);
+      const auto zaxistype = zaxisInqType(zaxisID1);
 
-      const int nlev = zaxisInqSize(zaxisID1);
+      const auto nlev = zaxisInqSize(zaxisID1);
       if (nlev <= 1) continue;
 
       if (zaxisInqLevels(zaxisID1, nullptr))
         {
-          std::vector<double> yv1(nlev);
-          std::vector<double> yv2(nlev);
+          Varray<double> yv1(nlev), yv2(nlev);
           zaxisInqLevels(zaxisID1, yv1.data());
           for (int ilev = 0; ilev < nlev; ++ilev) yv2[nlev - ilev - 1] = yv1[ilev];
           zaxisDefLevels(zaxisID2, yv2.data());
@@ -49,8 +50,7 @@ invertLevDes(int vlistID)
 
       if (zaxisInqLbounds(zaxisID1, nullptr) && zaxisInqUbounds(zaxisID1, nullptr))
         {
-          std::vector<double> yb1(nlev);
-          std::vector<double> yb2(nlev);
+          Varray<double> yb1(nlev), yb2(nlev);
           zaxisInqLbounds(zaxisID1, yb1.data());
           for (int ilev = 0; ilev < nlev; ++ilev) yb2[nlev - ilev - 1] = yb1[ilev];
           zaxisDefLbounds(zaxisID2, yb2.data());
@@ -65,8 +65,7 @@ invertLevDes(int vlistID)
           const int vctsize = zaxisInqVctSize(zaxisID1);
           if (vctsize && vctsize % 2 == 0)
             {
-              std::vector<double> vct1(vctsize);
-              std::vector<double> vct2(vctsize);
+              Varray<double> vct1(vctsize), vct2(vctsize);
               zaxisInqVct(zaxisID1, vct1.data());
               for (int i = 0; i < vctsize / 2; ++i)
                 {
@@ -87,54 +86,53 @@ Invertlev(void *process)
   int nrecs;
   int varID, levelID;
   size_t nmiss;
-  int nlev, nlevel;
-  int gridID, zaxisID;
   bool linvert = false;
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
   cdoOperatorAdd("invertlev", func_all, 0, nullptr);
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   if (operfunc == func_all || operfunc == func_hrd) invertLevDes(vlistID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  std::vector<std::vector<double>> vardata(nvars);
+  Varray2D<double> vardata(nvars);
   std::vector<std::vector<size_t>> varnmiss(nvars);
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   for (varID = 0; varID < nvars; varID++)
     {
-      gridID = vlistInqVarGrid(vlistID1, varID);
-      zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const size_t gridsize = gridInqSize(gridID);
-      nlev = zaxisInqSize(zaxisID);
-
-      if (nlev > 1)
+      const auto nlevels = varList1[varID].nlevels;
+      if (nlevels > 1)
         {
           linvert = true;
-          vardata[varID].resize(gridsize * nlev);
-          varnmiss[varID].resize(nlev);
+          vardata[varID].resize(varList1[varID].gridsize * nlevels);
+          varnmiss[varID].resize(nlevels);
         }
     }
 
@@ -153,10 +151,7 @@ Invertlev(void *process)
 
           if (vardata[varID].size())
             {
-              gridID = vlistInqVarGrid(vlistID1, varID);
-              const size_t gridsize = gridInqSize(gridID);
-              const size_t offset = gridsize * levelID;
-
+              const auto offset = varList1[varID].gridsize * levelID;
               cdoReadRecord(streamID1, &vardata[varID][offset], &nmiss);
               varnmiss[varID][levelID] = nmiss;
             }
@@ -179,16 +174,14 @@ Invertlev(void *process)
         {
           if (vardata[varID].size())
             {
-              gridID = vlistInqVarGrid(vlistID1, varID);
-              zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              const size_t gridsize = gridInqSize(gridID);
-              nlevel = zaxisInqSize(zaxisID);
-              for (levelID = 0; levelID < nlevel; levelID++)
+              const auto gridsize = varList1[varID].gridsize;
+              const auto nlevels = varList1[varID].nlevels;
+              for (levelID = 0; levelID < nlevels; levelID++)
                 {
                   cdoDefRecord(streamID2, varID, levelID);
 
-                  size_t offset = gridsize * (nlevel - levelID - 1);
-                  nmiss = varnmiss[varID][nlevel - levelID - 1];
+                  const auto offset = gridsize * (nlevels - levelID - 1);
+                  nmiss = varnmiss[varID][nlevels - levelID - 1];
 
                   cdoWriteRecord(streamID2, &vardata[varID][offset], nmiss);
                 }
