@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,10 +15,11 @@
   GNU General Public License for more details.
 */
 
+#include <cstdlib>
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "cdo_options.h"
+#include "process_int.h"
 #include "color.h"
 
 CPT cpt;
@@ -68,7 +69,7 @@ NormalizVectrs(int n_xres, int n_yres, float *pVectr)
     for (int i = 0; i < n_xres; i++)
       {
         int index = (j * n_xres + i) << 1;
-        float vcMag = (float) (sqrt((double) (pVectr[index] * pVectr[index] + pVectr[index + 1] * pVectr[index + 1])));
+        float vcMag = (float) (std::sqrt((double) (pVectr[index] * pVectr[index] + pVectr[index + 1] * pVectr[index + 1])));
 
         float scale = (vcMag == 0.0f) ? 0.0f : 1.0f / vcMag;
         pVectr[index] *= scale;
@@ -83,7 +84,7 @@ MakeWhiteNoise(int n_xres, int n_yres, unsigned char *pNoise)
   for (int j = 0; j < n_yres; j++)
     for (int i = 0; i < n_xres; i++)
       {
-        int r = rand();
+        int r = std::rand();
         r = ((r & 0xff) + ((r & 0xff00) >> 8)) & 0xff;
         pNoise[j * n_xres + i] = (unsigned char) r;
       }
@@ -97,7 +98,7 @@ GenBoxFiltrLUT(int LUTsiz, float *p_LUT0, float *p_LUT1)
 }
 
 // write the LIC image to a PPM file
-
+/*
 static void
 WriteImage2PPM(int n_xres, int n_yres, unsigned char *pImage, const char *f_name)
 {
@@ -119,6 +120,7 @@ WriteImage2PPM(int n_xres, int n_yres, unsigned char *pImage, const char *f_name
 
   fclose(o_file);
 }
+*/
 
 // write the LIC image to a PNG file
 
@@ -391,7 +393,7 @@ lic1(const char *obasename, int num, int n_xres, int n_yres, float *pVectr)
 #pragma omp parallel for default(none) shared(mag, pVectr, n)
 #endif
   for (int i = 0; i < n; ++i)
-    mag[i] = (float) sqrt((double) pVectr[i * 2] * pVectr[i * 2] + (double) pVectr[i * 2 + 1] * pVectr[i * 2 + 1]);
+    mag[i] = (float) std::sqrt((double) pVectr[i * 2] * pVectr[i * 2] + (double) pVectr[i * 2 + 1] * pVectr[i * 2 + 1]);
 
   if (Options::cdoVerbose)
     {
@@ -444,38 +446,38 @@ Lic(void *process)
   // int operatorID = cdoOperatorID();
 
   if (operatorArgc() < 1 || operatorArgc() > 2) operatorCheckArgc(1);
-  char *cpt_file = operatorArgv()[0];
-  int nstart = (operatorArgc() == 2) ? atoi(operatorArgv()[1]) : 0;
-  FILE *cpt_fp = fopen(cpt_file, "r");
+  const char *cpt_file = cdoOperatorArgv(0).c_str();
+  int nstart = (operatorArgc() == 2) ? atoi(cdoOperatorArgv(1).c_str()) : 0;
+  auto cpt_fp = fopen(cpt_file, "r");
   if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
 
   int status = cptRead(cpt_fp, &cpt);
   if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
 
-  const char *obasename = cdoGetStreamName(1);
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto obasename = cdoGetStreamName(1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
   // int taxisID1 = vlistInqTaxis(vlistID1);
 
   /* find variables */
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   if (nvars != 2) cdoAbort("Need 2 input variable, found %d\n", nvars);
   int varID1 = 0;
   int varID2 = 1;
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   if (ngrids != 1) cdoAbort("Need 1 grid, found %d\n", ngrids);
 
-  int gridID = vlistInqVarGrid(vlistID1, varID1);
+  const auto gridID = vlistInqVarGrid(vlistID1, varID1);
   // int zaxisID = vlistInqVarZaxis(vlistID1, varID1);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsize);
+  auto gridsize = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsize);
   std::vector<float> varray(2 * gridsize);
 
-  std::vector<double> ivar1, ivar2, ovar1;
+  Varray<double> ivar1, ivar2, ovar1;
   if (varID1 != -1 && varID2 != -1)
     {
       nlev = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
@@ -488,14 +490,14 @@ Lic(void *process)
       ovar1.resize(nlev * gridsize);
     }
 
-  int gridtype = gridInqType(gridID);
+  const auto gridtype = gridInqType(gridID);
   if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN) cdoAbort("Unsupported grid!\n");
 
-  size_t nx = gridInqXsize(gridID);
-  size_t ny = gridInqYsize(gridID);
+  const auto nx = gridInqXsize(gridID);
+  const auto ny = gridInqYsize(gridID);
 
-  std::vector<double> xvals(nx);
-  std::vector<double> yvals(ny);
+  Varray<double> xvals(nx);
+  Varray<double> yvals(ny);
   gridInqYvals(gridID, yvals.data());
   bool invertlat = (yvals[0] < yvals[ny - 1]);
 
