@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,19 +18,22 @@
 #include "config.h" /* HAVE_LIBMAGICS */
 #endif
 
-#include <limits.h>
+#include <climits>
 #include <cdi.h>
+#include <cctype>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 
 #ifdef HAVE_LIBMAGICS
 
 #include "magics_api.h"
-
 #include "magics_template_parser.h"
 #include "results_template_parser.h"
-#include "StringUtilities.h"
+#include "string_utilities.h"
+#include "util_string.h"
+#include "cdo_vlist.h"
+#include "functs.h"
 #include "printinfo.h"
 
 #define DBG 0
@@ -96,40 +99,70 @@ const char *graph_params[] = { "ymin", "ymax", "sigma", "stat", "obsv", "device"
 int graph_param_count = sizeof(graph_params) / sizeof(char *);
 int num_colours = sizeof(line_colours) / sizeof(char *);
 
-int compareDateOrTimeStr(char *datetimestr1, char *datetimestr2, const char *sep_char);
-
 extern int checkdevice(char *device_in);
 
 extern char *DEVICE;
 extern char *DEVICE_TABLE;
 extern int DEVICE_COUNT;
 
+static int
+compareDate(int64_t date1, int64_t date2)
+{
+  int c1[3], c2[3];
+  cdiDecodeDate(date1, c1, c1 + 1, c1 + 2);
+  cdiDecodeDate(date2, c2, c2 + 1, c2 + 2);
+
+  for (int i = 0; i < 3; i++)
+    {
+      int flag = c1[i] - c2[i];
+      if (flag > 0)
+        return 1;
+      else if (flag < 0)
+        return -1;
+      else
+        continue;
+    }
+
+  return 0;
+}
+
+static int
+compareTime(int time1, int time2)
+{
+  int c1[3], c2[3];
+  cdiDecodeTime(time1, c1, c1 + 1, c1 + 2);
+  cdiDecodeTime(time2, c2, c2 + 1, c2 + 2);
+
+  for (int i = 0; i < 3; i++)
+    {
+      int flag = c1[i] - c2[i];
+      if (flag > 0)
+        return 1;
+      else if (flag < 0)
+        return -1;
+      else
+        continue;
+    }
+
+  return 0;
+}
+
 static void
-maggraph(const char *plotfile, const char *varname, const char *varunits, long nfiles, long *nts, int64_t **vdate, int **vtime,
-         double **datatab, int nparam, char **params)
+maggraph(const char *plotfile, const char *varname, const char *varunits, long nfiles, std::vector<long> nts,
+         std::vector<std::vector<int64_t>> vdates, std::vector<std::vector<int>> vtimes, std::vector<std::vector<double>> datatab,
+         int nparam, std::vector<std::string> &params)
 {
-  char *lines[1];
-  char *temp_str;
-  char **split_str = nullptr;
-  const char *sep_char = "=";
-  char **date_time_str[nfiles];
   char min_date_time_str[1024], max_date_time_str[1024];
   int min_index = 0, max_index = 0;
-  char vdatestr[32], vtimestr[32], legend_text_data[256];
-  char vdatestr1[32], vtimestr1[32];
-  char vdatestr2[32], vtimestr2[32];
+  char legend_text_data[256];
   int num_sigma = 2;
   bool stat = false, obsv = false;
-  int split_str_count;
   int file_begin = 0;
   int count;
   int num_years = 0, num_months = 0, num_days = 0;
   int ret;
   long tsID, fileID, i, ntime_steps = 0;
-  double *date_time = nullptr;
   double min_val = 1.0e+200, max_val = -1.0e+200;
-  double *mean_val = nullptr, *std_dev_val = nullptr;
-  double *spread_min = nullptr, *spread_max = nullptr;
   double y_min_val = 1.0e+200, y_max_val = -1.0e+200;
   int linewidth_val = 8;
 
@@ -137,20 +170,23 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
     {
       fprintf(stderr, "Num params %d\n", nparam);
 
-      for (i = 0; i < nparam; i++) fprintf(stderr, "Param %s\n", params[i]);
+      for (i = 0; i < nparam; i++) fprintf(stderr, "Param %s\n", params[i].c_str());
     }
 
+  int split_str_count;
+  const char *sep_char = "=";
+  std::string temp_str;
   for (i = 0; i < nparam; ++i)
     {
+      std::vector<std::string> split_str;
       split_str_count = 0;
       sep_char = "=";
-      split_str_count = cstrSplitWithSeperator(params[i], sep_char, &split_str);
+      split_str_count = cstrSplitWithSeperator(params[i].c_str(), sep_char, split_str);
 
-      if (cstrIsEqual(split_str[0], "obsv"))
+      if (split_str[0] == "obsv")
         {
-          temp_str = strdup(split_str[1]);
-          cstrToUpperCase(temp_str);
-          if (cstrIsEqual(temp_str, "true"))
+          temp_str = stringToLower(split_str[1]);
+          if (temp_str == "true")
             {
               obsv = true;
               file_begin = 1;
@@ -158,53 +194,48 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
             }
         }
 
-      if (cstrIsEqual(split_str[0], "stat"))
+      if (split_str[0] == "stat")
         {
-          temp_str = strdup(split_str[1]);
-          cstrToUpperCase(temp_str);
-
-          if (cstrIsEqual(temp_str, "true"))
+          temp_str = stringToLower(split_str[1]);
+          if (temp_str == "true")
             {
               stat = true;
               if (DBG) fprintf(stderr, "STAT true\n");
             }
         }
 
-      if (cstrIsEqual(split_str[0], "ymin"))
+      if (split_str[0] == "ymin")
         {
-          y_min_val = atof(split_str[1]);
+          y_min_val = std::stod(split_str[1]);
           if (DBG) fprintf(stderr, "Y min Val %g\n", y_min_val);
         }
 
-      if (cstrIsEqual(split_str[0], "ymax"))
+      if (split_str[0] == "ymax")
         {
-          y_max_val = atof(split_str[1]);
+          y_max_val = std::stod(split_str[1]);
           if (DBG) fprintf(stderr, "Y max Val %g\n", y_max_val);
         }
 
-      if (cstrIsEqual(split_str[0], "linewidth"))
+      if (split_str[0] == "linewidth")
         {
-          linewidth_val = atoi(split_str[1]);
+          linewidth_val = std::stoi(split_str[1]);
           if (DBG) fprintf(stderr, "linewidth Val %d\n", linewidth_val);
         }
 
-      if (cstrIsEqual(split_str[0], "sigma"))
+      if (split_str[0] == "sigma")
         {
-          num_sigma = atof(split_str[1]);
+          num_sigma = std::stod(split_str[1]);
           if (DBG) fprintf(stderr, "SIGMA %d\n", num_sigma);
         }
 
-      if (cstrIsEqual(split_str[0], "device"))
+      if (split_str[0] == "device")
         {
-          temp_str = strdup(split_str[1]);
-          cstrToUpperCase(temp_str);
-          DEVICE = temp_str;
+          temp_str = stringToUpper(split_str[1]);
+          DEVICE = strdup(temp_str.c_str());
           if (DBG) fprintf(stderr, "DEVICE %s\n", DEVICE);
 
           mag_setc("output_format", DEVICE);
         }
-
-      Free(split_str);
     }
 
   if (DBG)
@@ -217,13 +248,6 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
   if (stat)
     {
       ntime_steps = nts[0];
-      /* First date & time of first file */
-      date2str(vdate[0][0], vdatestr1, sizeof(vdatestr));
-      date2str(vdate[0][nts[0] - 1], vdatestr2, sizeof(vdatestr));
-
-      /* Last date & time of first file */
-      time2str(vtime[0][0], vtimestr1, sizeof(vtimestr));
-      time2str(vtime[0][nts[0] - 1], vtimestr2, sizeof(vtimestr));
 
       for (fileID = 1; fileID < nfiles; fileID++)
         {
@@ -235,10 +259,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
             }
 
           /* First date & time of the present file */
-          date2str(vdate[fileID][0], vdatestr, sizeof(vdatestr));
-          sep_char = "-";
-          ret = compareDateOrTimeStr(vdatestr, vdatestr1, sep_char);
-          if (ret)
+          if (compareDate(vdates[0][0], vdates[fileID][0]))
             {
               cdoWarning("  Incosistent start date! Statistics disabled.");
               stat = false;
@@ -246,10 +267,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
             }
 
           /* First time of the present file */
-          time2str(vtime[fileID][0], vtimestr, sizeof(vtimestr));
-          sep_char = ":";
-          ret = compareDateOrTimeStr(vtimestr, vtimestr1, sep_char);
-          if (ret)
+          if (compareTime(vtimes[0][0], vtimes[fileID][0]))
             {
               cdoWarning("  Incosistent start time! Statistics disabled.");
               stat = false;
@@ -257,10 +275,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
             }
 
           /* Last date of the present file */
-          date2str(vdate[fileID][nts[fileID] - 1], vdatestr, sizeof(vdatestr));
-          sep_char = "-";
-          ret = compareDateOrTimeStr(vdatestr, vdatestr2, sep_char);
-          if (ret)
+          if (compareDate(vdates[fileID][nts[fileID] - 1], vdates[0][nts[0] - 1]))
             {
               cdoWarning("  Incosistent end date! Statistics disabled.");
               stat = false;
@@ -268,10 +283,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
             }
 
           /* Last time of the present file */
-          time2str(vtime[fileID][nts[fileID] - 1], vtimestr, sizeof(vtimestr));
-          sep_char = ":";
-          ret = compareDateOrTimeStr(vtimestr, vtimestr2, sep_char);
-          if (ret)
+          if (compareTime(vtimes[fileID][nts[fileID] - 1], vtimes[0][nts[0] - 1]))
             {
               cdoWarning("  Incosistent end time! Statistics disabled.");
               stat = false;
@@ -285,32 +297,35 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
       fprintf(stderr, "STAT  %d\n", (int) stat);
     }
 
+  char ***date_time_str = (char ***) malloc(nfiles * sizeof(char **));
+
+  std::vector<double> date_time;
+  std::vector<double> mean_val, std_dev_val;
+  std::vector<double> spread_min, spread_max;
+
   if (stat)
     {
-      /* if all files are of same number of steps, only one date_time_str array
-       * is being used */
-      date_time_str[0] = (char **) Malloc(ntime_steps * sizeof(char *));
+      /* if all files are of same number of steps, only one date_time_str array is being used */
+      date_time_str[0] = (char **) malloc(ntime_steps * sizeof(char *));
 
-      date_time = (double *) Malloc(ntime_steps * sizeof(double));
-      mean_val = (double *) Malloc(ntime_steps * sizeof(double));
-      std_dev_val = (double *) Malloc(ntime_steps * sizeof(double));
-      spread_min = (double *) Malloc(ntime_steps * sizeof(double));
-      spread_max = (double *) Malloc(ntime_steps * sizeof(double));
+      date_time.resize(ntime_steps);
+      mean_val.resize(ntime_steps);
+      std_dev_val.resize(ntime_steps);
+      spread_min.resize(ntime_steps);
+      spread_max.resize(ntime_steps);
 
       for (tsID = 0; tsID < ntime_steps; ++tsID)
         {
           date_time[tsID] = tsID + 1;
-          date2str(vdate[0][tsID], vdatestr, sizeof(vdatestr));
-          time2str(vtime[0][tsID], vtimestr, sizeof(vtimestr));
-          date_time_str[0][tsID] = (char *) Malloc(256);
-          sprintf(date_time_str[0][tsID], "%s %s", vdatestr, vtimestr);
+          date_time_str[0][tsID] = (char *) malloc(256);
+          sprintf(date_time_str[0][tsID], "%s %s", dateToString(vdates[0][tsID]).c_str(), timeToString(vtimes[0][tsID]).c_str());
           mean_val[tsID] = 0.;
           std_dev_val[tsID] = 0.;
 
           if (DBG)
             {
               fprintf(stderr, "%ld: %s\n", tsID, date_time_str[0][tsID]);
-              fprintf(stderr, "%6ld %6d", (long) vdate[0][tsID], vtime[0][tsID]);
+              fprintf(stderr, "%6ld %6d", (long) vdates[0][tsID], vtimes[0][tsID]);
             }
 
           for (fileID = 0; fileID < nfiles; ++fileID)
@@ -344,7 +359,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
               std_dev_val[tsID] += (datatab[fileID][tsID] - mean_val[tsID]) * (datatab[fileID][tsID] - mean_val[tsID]);
             }
           std_dev_val[tsID] /= (double) nfiles;
-          std_dev_val[tsID] = pow(std_dev_val[tsID], 0.5);
+          std_dev_val[tsID] = std::pow(std_dev_val[tsID], 0.5);
 
           if (DBG) fprintf(stderr, " Mean : %g Std Dev: %g\n", mean_val[tsID], std_dev_val[tsID]);
 
@@ -382,23 +397,23 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
       for (fileID = 0; fileID < nfiles; fileID++)
         {
           if (DBG) fprintf(stderr, "FILE  %ld\n", fileID);
-          date_time = (double *) Malloc(nts[fileID] * sizeof(double));
-          date_time_str[fileID] = (char **) Malloc(nts[fileID] * sizeof(char *));
+          date_time.resize(nts[fileID]);
+          date_time_str[fileID] = (char **) malloc(nts[fileID] * sizeof(char *));
 
           for (tsID = 0; tsID < nts[fileID]; ++tsID)
             {
               date_time[tsID] = tsID + 1;
-              date2str(vdate[fileID][tsID], vdatestr, sizeof(vdatestr));
-              time2str(vtime[fileID][tsID], vtimestr, sizeof(vtimestr));
 
-              date_time_str[fileID][tsID] = (char *) Malloc(256);
-              sprintf(date_time_str[fileID][tsID], "%s %s", vdatestr, vtimestr);
-              if (DBG) fprintf(stderr, "%s %s %s\n", vdatestr, vtimestr, date_time_str[fileID][tsID]);
+              date_time_str[fileID][tsID] = (char *) malloc(256);
+              sprintf(date_time_str[fileID][tsID], "%s %s", dateToString(vdates[fileID][tsID]).c_str(),
+                      timeToString(vtimes[fileID][tsID]).c_str());
+              if (DBG)
+                fprintf(stderr, "%s %s %s\n", dateToString(vdates[fileID][tsID]).c_str(),
+                        timeToString(vtimes[fileID][tsID]).c_str(), date_time_str[fileID][tsID]);
 
               if (datatab[fileID][tsID] < min_val) min_val = datatab[fileID][tsID];
               if (datatab[fileID][tsID] > max_val) max_val = datatab[fileID][tsID];
             }
-          Free(date_time);
 
           if (fileID == 0)
             {
@@ -408,22 +423,12 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
             }
           else
             {
-              if (DBG) fprintf(stderr, "compareDateOrTimeStr %s\n", date_time_str[fileID][0]);
-              date2str(vdate[min_index][0], vdatestr1, sizeof(vdatestr));
-              date2str(vdate[fileID][0], vdatestr2, sizeof(vdatestr));
-              sep_char = "-";
-              ret = compareDateOrTimeStr(vdatestr1, vdatestr2, sep_char);
-              if (ret == -999)
-                cdoAbort("Error in input Date Time");
-              else if (ret == 1)
+              ret = compareDate(vdates[min_index][0], vdates[fileID][0]);
+              if (ret == 1)
                 min_index = fileID;
               else if (!ret)
                 {
-                  time2str(vtime[min_index][0], vtimestr1, sizeof(vtimestr));
-                  time2str(vtime[fileID][0], vtimestr2, sizeof(vtimestr));
-                  sep_char = ":";
-                  ret = compareDateOrTimeStr(vtimestr1, vtimestr2, sep_char);
-
+                  ret = compareTime(vtimes[min_index][0], vtimes[fileID][0]);
                   if (ret == -999)
                     cdoAbort("Error in input Date Time");
                   else if (ret == 1)
@@ -431,24 +436,14 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
                 }
               if (DBG) fprintf(stderr, "Min File ID %d\n", min_index);
 
-              if (DBG) fprintf(stderr, "compareDateOrTimeStr  %s\n", date_time_str[fileID][nts[fileID] - 1]);
+              if (DBG) fprintf(stderr, "compareDateOrTime  %s\n", date_time_str[fileID][nts[fileID] - 1]);
 
-              date2str(vdate[max_index][nts[max_index] - 1], vdatestr1, sizeof(vdatestr));
-              date2str(vdate[fileID][nts[fileID] - 1], vdatestr2, sizeof(vdatestr));
-              sep_char = "-";
-              ret = compareDateOrTimeStr(vdatestr1, vdatestr2, sep_char);
-
-              if (ret == -999)
-                cdoAbort("Error in input Date Time");
-              else if (ret == -1)
+              ret = compareDate(vdates[max_index][nts[max_index] - 1], vdates[fileID][nts[fileID] - 1]);
+              if (ret == -1)
                 max_index = fileID;
               else if (!ret)
                 {
-                  time2str(vtime[max_index][nts[max_index] - 1], vtimestr1, sizeof(vtimestr));
-                  time2str(vtime[fileID][nts[fileID] - 1], vtimestr2, sizeof(vtimestr));
-                  sep_char = ":";
-                  ret = compareDateOrTimeStr(vtimestr1, vtimestr2, sep_char);
-
+                  ret = compareTime(vtimes[max_index][nts[max_index] - 1], vtimes[fileID][nts[fileID] - 1]);
                   if (ret == -999)
                     cdoAbort("Error in input Date Time");
                   else if (ret == -1)
@@ -466,31 +461,30 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
 
   if (DBG) fprintf(stderr, "%s %s\n", min_date_time_str, max_date_time_str);
 
+  std::vector<std::string> split_str;
   split_str_count = 0;
   sep_char = "-";
-  split_str_count = cstrSplitWithSeperator(max_date_time_str, sep_char, &split_str);
+  split_str_count = cstrSplitWithSeperator(max_date_time_str, sep_char, split_str);
   (void) split_str_count;
-  num_years = atoi(split_str[0]);
-  num_months = atoi(split_str[1]);
-  num_days = atoi(split_str[2]);
-  Free(split_str);
+  num_years = std::stoi(split_str[0]);
+  num_months = std::stoi(split_str[1]);
+  num_days = std::stoi(split_str[2]);
 
-  split_str_count = cstrSplitWithSeperator(min_date_time_str, sep_char, &split_str);
-  num_years -= atoi(split_str[0]);
+  split_str_count = cstrSplitWithSeperator(min_date_time_str, sep_char, split_str);
+  num_years -= std::stoi(split_str[0]);
 
   if (num_years <= 1)
     {
       if (num_years == 1)
-        num_months += (12 - atoi(split_str[1]));
+        num_months += (12 - std::stoi(split_str[1]));
       else
-        num_months -= (atoi(split_str[1]));
+        num_months -= (std::stoi(split_str[1]));
 
       if (!num_months)
-        num_days -= atoi(split_str[2]);
+        num_days -= std::stoi(split_str[2]);
       else if (num_months == 1)
-        num_days += (31 - atoi(split_str[2]));
+        num_days += (31 - std::stoi(split_str[2]));
     }
-  Free(split_str);
 
   if (DBG) fprintf(stderr, " %d %d\n", num_years, num_months);
 
@@ -597,11 +591,11 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
         mag_set1c("graph_curve_date_x_values", (const char **) date_time_str[i], nts[i]);
 
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  begin**/
-      mag_setr("graph_x_suppress_below", LLONG_MIN);
-      mag_setr("graph_x_suppress_above", LLONG_MAX);
+      mag_setr("graph_x_suppress_below", (double) LLONG_MIN);
+      mag_setr("graph_x_suppress_above", (double) LLONG_MAX);
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  end  **/
 
-      mag_set1r("graph_curve_y_values", datatab[i], nts[i]);
+      mag_set1r("graph_curve_y_values", datatab[i].data(), nts[i]);
       mag_graph();
     }
 
@@ -613,11 +607,11 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
       mag_set1c("graph_curve_date_x_values", (const char **) date_time_str[0], nts[0]);
 
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  begin**/
-      mag_setr("graph_x_suppress_below", LLONG_MIN);
-      mag_setr("graph_x_suppress_above", LLONG_MAX);
+      mag_setr("graph_x_suppress_below", (double) LLONG_MIN);
+      mag_setr("graph_x_suppress_above", (double) LLONG_MAX);
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  end  **/
 
-      mag_set1r("graph_curve_y_values", datatab[0], nts[0]);
+      mag_set1r("graph_curve_y_values", datatab[0].data(), nts[0]);
       mag_setc("graph_line_style", "dot");
       mag_seti("graph_line_thickness", linewidth_val + 2);
       mag_graph();
@@ -635,11 +629,11 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
       mag_set1c("graph_curve_date_x_values", (const char **) date_time_str[0], ntime_steps);
 
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  begin**/
-      mag_setr("graph_x_suppress_below", LLONG_MIN);
-      mag_setr("graph_x_suppress_above", LLONG_MAX);
+      mag_setr("graph_x_suppress_below", (double) LLONG_MIN);
+      mag_setr("graph_x_suppress_above", (double) LLONG_MAX);
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  end  **/
 
-      mag_set1r("graph_curve_y_values", mean_val, ntime_steps);
+      mag_set1r("graph_curve_y_values", mean_val.data(), ntime_steps);
       sprintf(legend_text_data, "Mean");
       mag_setc("legend_user_text", legend_text_data);
       mag_graph();
@@ -650,29 +644,28 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
       mag_setc("graph_shade_style", "dot");
       mag_setr("graph_shade_dot_size", 1.);
       mag_set1c("graph_curve2_date_x_values", (const char **) date_time_str[0], ntime_steps);
-      mag_set1r("graph_curve2_y_values", spread_max, ntime_steps);
+      mag_set1r("graph_curve2_y_values", spread_max.data(), ntime_steps);
       mag_set1c("graph_curve_date_x_values", (const char **) date_time_str[0], ntime_steps);
-      mag_set1r("graph_curve_y_values", spread_min, ntime_steps);
+      mag_set1r("graph_curve_y_values", spread_min.data(), ntime_steps);
       mag_setc("graph_shade_colour", "grey");
       sprintf(legend_text_data, "%dSigma", num_sigma);
       mag_setc("legend_user_text", legend_text_data);
 
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  begin**/
-      mag_setr("graph_x_suppress_below", LLONG_MIN);
-      mag_setr("graph_x_suppress_above", LLONG_MAX);
+      mag_setr("graph_x_suppress_below", (double) LLONG_MIN);
+      mag_setr("graph_x_suppress_above", (double) LLONG_MAX);
       /* TEMPORARY FIX, UNITL NEW MAGICS LIBRARY RELEASE *  end  **/
 
       mag_graph();
     }
 
-  lines[0] = (char *) Malloc(1024);
+  char *lines[1];
+  lines[0] = (char *) malloc(1024);
   /* To be obtained from Meta Data */
-  /*sprintf( lines[0],"%s","ExpID : " );*/
-  /*sprintf( lines[0],"%sxxxx  Variable : %s[%s]",lines[0], varname, varunits
-   * );*/
+  /* sprintf( lines[0],"%s","ExpID : " );*/
+  /* sprintf( lines[0],"%sxxxx  Variable : %s[%s]",lines[0], varname, varunits );*/
   // sprintf( lines[0],"Variable : %s[%s]",varname, varunits );
-  // sprintf( lines[0],"%s  Date : %s --%s",lines[0], min_date_time_str,
-  // max_date_time_str );
+  // sprintf( lines[0],"%s  Date : %s --%s",lines[0], min_date_time_str, max_date_time_str );
   sprintf(lines[0], "Variable : %s[%s]  Date : %s --%s", varname, varunits, min_date_time_str, max_date_time_str);
   mag_set1c("text_lines", (const char **) lines, 1);
 
@@ -688,63 +681,11 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
   mag_setc("text_justification", "left");
   mag_text();
 
-  if (stat)
-    {
-      Free(date_time);
-      Free(mean_val);
-      Free(std_dev_val);
-      Free(spread_min);
-      Free(spread_max);
-    }
+  free(date_time_str);
 
   if (DBG) fprintf(stderr, "%s\n", lines[0]);
-}
 
-int
-compareDateOrTimeStr(char *datetimestr1, char *datetimestr2, const char *sep_char)
-{
-  int split_str_count1, split_str_count2;
-  int i, flag[3]; /*  '3' since, three fields are expected in the input strings */
-  char **split_str1 = nullptr;
-  char **split_str2 = nullptr;
-
-  if (DBG) fprintf(stderr, "Inside compareDateOrTimeStr %s %s\n", datetimestr1, datetimestr2);
-  split_str_count1 = cstrSplitWithSeperator(datetimestr1, sep_char, &split_str1);
-
-  if (split_str_count1)
-    split_str_count2 = cstrSplitWithSeperator(datetimestr2, sep_char, &split_str2);
-  else
-    {
-      Free(split_str1);
-      return -999;
-    }
-
-  if (split_str_count2 && !(split_str_count1 - split_str_count2))
-    {
-      flag[0] = atoi(split_str1[0]) - atoi(split_str2[0]);
-      flag[1] = atoi(split_str1[1]) - atoi(split_str2[1]);
-      flag[2] = atoi(split_str1[2]) - atoi(split_str2[2]);
-    }
-  else
-    {
-      Free(split_str1);
-      Free(split_str2);
-      return -999;
-    }
-
-  Free(split_str1);
-  Free(split_str2);
-
-  for (i = 0; i < 3; i++)
-    {
-      if (flag[i] > 0)
-        return 1;
-      else if (flag[i] < 0)
-        return -1;
-      else
-        continue;
-    }
-  return 0;
+  free(lines[0]);
 }
 
 static void
@@ -753,11 +694,9 @@ init_MAGICS()
   setenv("MAGPLUS_QUIET", "1", 1); /* To suppress magics messages */
   mag_open();
 
-  /* Some standard parameters affectng the magics environment, moved from the
-   * xml file  ** begin ** */
+  /* Some standard parameters affectng the magics environment, moved from the xml file  ** begin ** */
   mag_setc("page_id_line", "off");
-  /* Some standard parameters affectng the magics environment, moved from the
-   * xml file  ** end ** */
+  /* Some standard parameters affectng the magics environment, moved from the xml file  ** end ** */
 }
 
 static void
@@ -768,7 +707,7 @@ quit_MAGICS()
 }
 
 static void
-VerifyGraphParameters(int num_param, char **param_names)
+VerifyGraphParameters(int num_param, std::vector<std::string> &param_names)
 {
   int i, j;
   bool found = false, syntax = true, halt_flag = false;
@@ -782,51 +721,51 @@ VerifyGraphParameters(int num_param, char **param_names)
       split_str_count = 0;
       found = false;
       syntax = true;
-      split_str_count = cstrSplitWithSeperator(param_names[i], sep_char, &split_str);
+      split_str_count = cstrSplitWithSeperator(param_names[i].c_str(), sep_char, &split_str);
       if (split_str_count > 1)
         {
           for (j = 0; j < graph_param_count; ++j)
             {
-              if (cstrIsEqual(split_str[0], graph_params[j]))
+              if (cdo_cmpstr(split_str[0], graph_params[j]))
                 {
                   found = true;
-                  if (cstrIsEqual(split_str[0], "obsv") || cstrIsEqual(split_str[0], "stat"))
+                  if (cdo_cmpstr(split_str[0], "obsv") || cdo_cmpstr(split_str[0], "stat"))
                     {
                       if (cstrIsNumeric(split_str[1]))
                         syntax = false;
                       else
                         {
                           temp_str = strdup(split_str[1]);
-                          cstrToUpperCase(temp_str);
+                          cstrToLowerCase(temp_str);
                           if (strcmp(temp_str, "true") && strcmp(temp_str, "false")) syntax = false;
                         }
                     }
 
-                  if (cstrIsEqual(split_str[0], "ymin") || cstrIsEqual(split_str[0], "ymax") || cstrIsEqual(split_str[0], "sigma")
-                      || cstrIsEqual(split_str[0], "linewidth"))
+                  if (cdo_cmpstr(split_str[0], "ymin") || cdo_cmpstr(split_str[0], "ymax") || cdo_cmpstr(split_str[0], "sigma")
+                      || cdo_cmpstr(split_str[0], "linewidth"))
                     {
                       if (!cstrIsNumeric(split_str[1])) syntax = false;
                     }
 
-                  if (cstrIsEqual(split_str[0], "device"))
+                  if (cdo_cmpstr(split_str[0], "device"))
                     {
                       if (cstrIsNumeric(split_str[1]))
                         syntax = false;
                       else
                         {
-                          if (cstrIsEqual(split_str[0], "device"))
+                          if (cdo_cmpstr(split_str[0], "device"))
                             {
                               if (DBG) fprintf(stderr, "Parameter value '%s'\n", split_str[1]);
                               if (checkdevice(split_str[1])) syntax = false;
 
                               /* Graph not supported in google earth format */
-                              if (cstrIsEqual(split_str[1], "GIF_ANIMATION") || cstrIsEqual(split_str[1], "gif_animation"))
+                              if (cdo_cmpstr(split_str[1], "GIF_ANIMATION") || cdo_cmpstr(split_str[1], "gif_animation"))
                                 {
                                   syntax = false;
                                   fprintf(stderr, "Animation not supported for Graph!\n");
                                   if (DBG) fprintf(stderr, "Parameter value '%s'\n", split_str[1]);
                                 }
-                              if (cstrIsEqual(split_str[1], "KML") || cstrIsEqual(split_str[1], "kml"))
+                              if (cdo_cmpstr(split_str[1], "KML") || cdo_cmpstr(split_str[1], "kml"))
                                 {
                                   syntax = false;
                                   fprintf(stderr, " 'kml' format not supported "
@@ -838,24 +777,20 @@ VerifyGraphParameters(int num_param, char **param_names)
                     }
 
                   /*
-                                    if( cstrIsEqual( split_str[0],"xml" ) )
-                                      {
-                                        if( ( fp = fopen( split_str[1],"r") ) ==
-                     nullptr )
-                                          {
-                                            fprintf( stderr,"Input XML File not
-                     found in specified path '%s'\n", split_str[1] ); halt_flag
-                     = true;
-                                          }
-                                        else
-                                          {
-                                            // HARDCODED THE FILE NAME .. TO BE
-                     SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR
-                                            fclose(fp);
-                                            init_XMLtemplate_parser(
-                     split_str[1] ); updatemagics_and_results_nodes( );
-                                          }
-                                      }
+                    if( cdo_cmpstr( split_str[0],"xml" ) )
+                      {
+                        if( ( fp = fopen( split_str[1],"r") ) == nullptr )
+                          {
+                            fprintf( stderr,"Input XML File not found in specified path '%s'\n", split_str[1] );
+                            halt_flag = true;
+                          }
+                        else
+                          {
+                            // HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR
+                            fclose(fp);
+                            init_XMLtemplate_parser( split_str[1] ); updatemagics_and_results_nodes( );
+                          }
+                      }
                   */
                 }
             }
@@ -868,22 +803,20 @@ VerifyGraphParameters(int num_param, char **param_names)
       if (!found)
         {
           halt_flag = true;
-          fprintf(stderr, "Unknown parameter  '%s'!\n", param_names[i]);
+          fprintf(stderr, "Unknown parameter  '%s'!\n", param_names[i].c_str());
         }
       if (found && !syntax)
         {
           halt_flag = true;
-          fprintf(stderr, "Invalid parameter specification  '%s'!\n", param_names[i]);
+          fprintf(stderr, "Invalid parameter specification  '%s'!\n", param_names[i].c_str());
         }
-      Free(split_str);
+      free(split_str);
     }
 
   if (halt_flag) exit(0);
 }
 #endif
 
-#define NINC_ALLOC 1024
-
 void *
 Maggraph(void *process)
 {
@@ -896,10 +829,10 @@ Maggraph(void *process)
   int nrecs;
   int vlistID0 = -1;
   size_t nmiss;
-  long nts_alloc;
+  size_t nts_alloc;
 
   int nparam = operatorArgc();
-  char **pnames = operatorArgv();
+  auto pnames = cdoGetOperArgv();
 
   if (nparam) VerifyGraphParameters(nparam, pnames);
 
@@ -912,27 +845,19 @@ Maggraph(void *process)
       fprintf(stderr, " files %s\n", ofilename);
     }
 
-  double **datatab = (double **) Malloc(nfiles * sizeof(double *));
-  int64_t **vdate = (int64_t **) Malloc(nfiles * sizeof(int64_t *));
-  int **vtime = (int **) Malloc(nfiles * sizeof(int *));
-  long *nts = (long *) Malloc(nfiles * sizeof(long));
-
-  for (int fileID = 0; fileID < nfiles; fileID++)
-    {
-      datatab[fileID] = nullptr;
-      vdate[fileID] = nullptr;
-      vtime[fileID] = nullptr;
-      nts[fileID] = 0;
-    }
+  std::vector<std::vector<double>> datatab(nfiles);
+  std::vector<std::vector<int64_t>> vdates(nfiles);
+  std::vector<std::vector<int>> vtimes(nfiles);
+  std::vector<long> nts(nfiles, 0);
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
       if (DBG) fprintf(stderr, " file %d is %s\n", fileID, cdoGetStreamName(fileID));
 
-      CdoStreamID streamID = cdoOpenRead((fileID));
+      const auto streamID = cdoOpenRead(fileID);
 
-      int vlistID = cdoStreamInqVlist(streamID);
-      int taxisID = vlistInqTaxis(vlistID);
+      const auto vlistID = cdoStreamInqVlist(streamID);
+      const auto taxisID = vlistInqTaxis(vlistID);
 
       vlistInqVarUnits(vlistID, 0, units);
       if (DBG) fprintf(stderr, " %s\n", units);
@@ -941,8 +866,8 @@ Maggraph(void *process)
           vlistInqVarName(vlistID, 0, varname);
 
           gridID = vlistInqVarGrid(vlistID, 0);
-          int zaxisID = vlistInqVarZaxis(vlistID, 0);
-          int nvars = vlistNvars(vlistID);
+          const auto zaxisID = vlistInqVarZaxis(vlistID, 0);
+          const auto nvars = vlistNvars(vlistID);
 
           if (nvars > 1) cdoAbort("Input stream has more than on variable!");
           if (gridInqSize(gridID) != 1) cdoAbort("Variable has more than one grid point!");
@@ -961,30 +886,23 @@ Maggraph(void *process)
         {
           if (nrecs != 1) cdoAbort("Input stream has more than one point in time!");
 
-          if (tsID == 0)
+          if ((size_t) tsID >= nts_alloc)
             {
-              nts_alloc += NINC_ALLOC;
-              datatab[fileID] = (double *) Malloc(nts_alloc * sizeof(double));
-              vdate[fileID] = (int64_t *) Malloc(nts_alloc * sizeof(int64_t));
-              vtime[fileID] = (int *) Malloc(nts_alloc * sizeof(int));
+              constexpr size_t NALLOC_INC = 1024;
+              nts_alloc += NALLOC_INC;
+              datatab[fileID].resize(nts_alloc);
+              vdates[fileID].resize(nts_alloc);
+              vtimes[fileID].resize(nts_alloc);
             }
 
           nts[fileID]++;
 
-          if (nts[fileID] > nts_alloc)
-            {
-              nts_alloc += NINC_ALLOC;
-              datatab[fileID] = (double *) Realloc(datatab[fileID], nts_alloc * sizeof(double));
-              vdate[fileID] = (int64_t *) Realloc(vdate[fileID], nts_alloc * sizeof(int64_t));
-              vtime[fileID] = (int *) Realloc(vtime[fileID], nts_alloc * sizeof(int));
-            }
-
           double val;
           cdoInqRecord(streamID, &varID, &levelID);
           cdoReadRecord(streamID, &val, &nmiss);
           datatab[fileID][tsID] = val;
-          vdate[fileID][tsID] = taxisInqVdate(taxisID);
-          vtime[fileID][tsID] = taxisInqVtime(taxisID);
+          vdates[fileID][tsID] = taxisInqVdate(taxisID);
+          vtimes[fileID][tsID] = taxisInqVtime(taxisID);
 
           if (DBG) fprintf(stderr, "%f %f\n", datatab[fileID][tsID], val);
           tsID++;
@@ -992,8 +910,7 @@ Maggraph(void *process)
       cdoStreamClose(streamID);
     }
 
-  /* HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE
-   * MAGICS OPERATOR */
+  /* HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR */
   /*
   init_XMLtemplate_parser( Filename );
   updatemagics_and_results_nodes( );
@@ -1006,9 +923,10 @@ Maggraph(void *process)
     {
       fprintf(stderr, "Num params %d\n", nparam);
 
-      for (int i = 0; i < nparam; i++) fprintf(stderr, "Param %s\n", pnames[i]);
+      for (int i = 0; i < nparam; i++) fprintf(stderr, "Param %s\n", pnames[i].c_str());
     }
-  maggraph(ofilename, varname, units, nfiles, nts, vdate, vtime, datatab, nparam, pnames);
+
+  maggraph(ofilename, varname, units, nfiles, nts, vdates, vtimes, datatab, nparam, pnames);
 
   /* quit_XMLtemplate_parser( ); */
 
@@ -1016,16 +934,6 @@ Maggraph(void *process)
 
   if (vlistID0 != -1) vlistDestroy(vlistID0);
 
-  for (int fileID = 0; fileID < nfiles; fileID++)
-    {
-      if (datatab[fileID]) Free(datatab[fileID]);
-    }
-
-  Free(datatab);
-
-  if (vdate) Free(vdate);
-  if (vtime) Free(vtime);
-
 #else
 
   cdoAbort("MAGICS support not compiled in!");
