@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "cdi_lockedIO.h"
 #include <mpim_grid.h>
 #include "griddes.h"
@@ -28,7 +28,7 @@ read_first_record(const char *filename, double *field)
 {
   size_t nmiss;
   int varID, levelID;
-  int streamID = streamOpenReadLocked(filename);
+  const auto streamID = streamOpenReadLocked(filename);
   streamInqTimestep(streamID, 0);
   streamInqRecord(streamID, &varID, &levelID);
   streamReadRecord(streamID, field, &nmiss);
@@ -65,22 +65,22 @@ MapReduce(void *process)
   if (operatorArgc() < 1) cdoAbort("Too few arguments!");
 
   /* check input grid type and size - this will be used for selecting relevant variables from the input file*/
-  const char *maskfilename = operatorArgv()[0];
-  int inputGridID = cdoDefineGrid(maskfilename);
-  size_t inputGridSize = gridInqSize(inputGridID);
-  int inputGridType = gridInqType(inputGridID);
-  Debug(cdoDebug,"MapReduce: input gridSize: %zu", inputGridSize);
+  const char *maskfilename = cdoOperatorArgv(0).c_str();
+  auto inputGridID = cdoDefineGrid(maskfilename);
+  auto inputGridSize = gridInqSize(inputGridID);
+  auto inputGridType = gridInqType(inputGridID);
+  Debug(cdoDebug, "MapReduce: input gridSize: %zu", inputGridSize);
 
   /* create an index list of the relevant locations */
   std::vector<int> maskIndexList;
   size_t maskSize;
   {
-    std::vector<double> inputMaskField(inputGridSize);
+    Varray<double> inputMaskField(inputGridSize);
     read_first_record(maskfilename, inputMaskField.data());
 
     /* non-zero values mark the relevant points */
     maskSize = countMask(inputMaskField.data(), inputGridSize, 0.0);
-    Debug(cdoDebug,"MapReduce: maskSize = %zu", maskSize);
+    Debug(cdoDebug, "MapReduce: maskSize = %zu", maskSize);
 
     maskIndexList.resize(maskSize);
 
@@ -94,11 +94,10 @@ MapReduce(void *process)
   }
 
   /* check if coordinated bounds shound not be created */
-  bool nobounds = false;
-  bool nocoords = false;
+  bool nobounds = false, nocoords = false;
   if (2 <= operatorArgc())
     {
-      char *coordinatesLimitation = operatorArgv()[1];
+      const char *coordinatesLimitation = cdoOperatorArgv(1).c_str();
       if (0 == strncmp("nobounds", coordinatesLimitation, 8)) nobounds = true;
       if (0 == strncmp("nocoords", coordinatesLimitation, 8)) nocoords = true;
     }
@@ -108,10 +107,10 @@ MapReduce(void *process)
   /* create output vlist: Only variabes which have the same gridtype and
    * gridsize as the input mask should be proessed. Everything else is ignoreds
    * {{{ */
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars, false);
 
   /* use vlist flags for marking the corresponding variables */
@@ -131,24 +130,23 @@ MapReduce(void *process)
         }
     }
 
-  int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
   /* }}} */
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   /* use the new selection grid for all output variables */
-  int ngrids = vlistNgrids(vlistID2);
+  const auto ngrids = vlistNgrids(vlistID2);
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, outputGridID);
 
   /* loop over input fields and mask the data values {{{ */
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> arrayIn(inputGridSize);
-  std::vector<double> arrayOut(maskSize);
+  Varray<double> arrayIn(inputGridSize), arrayOut(maskSize);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -161,8 +159,8 @@ MapReduce(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           if (vars[varID])
             {
-              int varID2 = vlistFindVar(vlistID2, varID);
-              int levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+              const auto varID2 = vlistFindVar(vlistID2, varID);
+              const auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
               cdoReadRecord(streamID1, arrayIn.data(), &nmiss);
 
@@ -181,5 +179,5 @@ MapReduce(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
