@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,18 +25,22 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include <mpim_grid.h>
+#include <utility>
 
-#define MAX_LINE 256
+#include "cdo_options.h"
+#include "process_int.h"
+#include "readline.h"
+#include <mpim_grid.h>
+#include "selboxinfo.h"
 
 static int
 ReadCoords(double *xvals, double *yvals, const char *polyfile, FILE *fp)
 {
   int z = 0, number = 0, jumpedlines = 0;
+  constexpr size_t MAX_LINE = 256;
   char line[MAX_LINE];
 
-  while (readline(fp, line, MAX_LINE))
+  while (cdo::readline(fp, line, MAX_LINE))
     {
       int i = 0;
       if (line[0] == '#' || line[0] == '\0')
@@ -48,7 +52,7 @@ ReadCoords(double *xvals, double *yvals, const char *polyfile, FILE *fp)
 
       char *linep = &line[0];
 
-      const double xcoord = strtod(linep, &linep);
+      const auto xcoord = strtod(linep, &linep);
 
       if (std::fabs(xcoord) <= 0) jumpedlines++;
 
@@ -67,7 +71,7 @@ ReadCoords(double *xvals, double *yvals, const char *polyfile, FILE *fp)
         }
       if ((i >= 63) && (number != 0)) cdoAbort("Wrong value format in file %s at line %d", polyfile, (number + jumpedlines + 1));
 
-      const double ycoord = strtod(linep, nullptr);
+      const auto ycoord = strtod(linep, nullptr);
       xvals[number] = xcoord;
       yvals[number] = ycoord;
       number++;
@@ -88,16 +92,17 @@ ReadCoords(double *xvals, double *yvals, const char *polyfile, FILE *fp)
   return number;
 }
 
-void genlonlatbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
-
-void genindexbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
-
 static void
-maskbox(std::vector<bool> &mask, const int gridID, const long lat1, const long lat2, const long lon11, const long lon12,
-        const long lon21, const long lon22)
+maskbox(std::vector<bool> &mask, const int gridID, const SelboxInfo &sbox)
 {
-  const long nlon = (long) gridInqXsize(gridID);
-  const long nlat = (long) gridInqYsize(gridID);
+  const auto &lat1 = sbox.lat1;
+  const auto &lat2 = sbox.lat2;
+  const auto &lon11 = sbox.lon11;
+  const auto &lon12 = sbox.lon12;
+  const auto &lon21 = sbox.lon21;
+  const auto &lon22 = sbox.lon22;
+  const long nlon = gridInqXsize(gridID);
+  const long nlat = gridInqYsize(gridID);
 
   for (long ilat = 0; ilat < nlat; ilat++)
     for (long ilon = 0; ilon < nlon; ilon++)
@@ -113,21 +118,15 @@ maskbox_cell(std::vector<bool> &mask, const int gridID)
   double xlon1 = 0, xlon2 = 0, xlat1 = 0, xlat2 = 0;
   getlonlatparams(0, xlon1, xlon2, xlat1, xlat2);
 
-  const size_t gridsize = gridInqSize(gridID);
+  const auto gridsize = gridInqSize(gridID);
 
-  std::vector<double> xvals(gridsize);
-  std::vector<double> yvals(gridsize);
+  Varray<double> xvals(gridsize), yvals(gridsize);
+  gridInqXvals(gridID, xvals.data());
+  gridInqYvals(gridID, yvals.data());
 
-  gridInqXvals(gridID, &xvals[0]);
-  gridInqYvals(gridID, &yvals[0]);
-
-  char xunits[CDI_MAX_NAME];
-  char yunits[CDI_MAX_NAME];
-  gridInqXunits(gridID, xunits);
-  gridInqYunits(gridID, yunits);
-
-  const double xfact = (strncmp(xunits, "radian", 6) == 0) ? RAD2DEG : 1;
-  const double yfact = (strncmp(yunits, "radian", 6) == 0) ? RAD2DEG : 1;
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID, CDI_XAXIS, gridsize, xvals.data(), "grid center lon");
+  cdo_grid_to_degree(gridID, CDI_YAXIS, gridsize, yvals.data(), "grid center lat");
 
   if (xlon1 > xlon2) cdoAbort("The second longitude have to be greater than the first one!");
 
@@ -137,8 +136,8 @@ maskbox_cell(std::vector<bool> &mask, const int gridID)
     {
       mask[i] = true;
 
-      const double xval = xfact * xvals[i];
-      const double yval = yfact * yvals[i];
+      const auto xval = xvals[i];
+      const auto yval = yvals[i];
       if (yval >= xlat1 && yval <= xlat2)
         {
           if (((xval >= xlon1 && xval <= xlon2) || (xval - 360 >= xlon1 && xval - 360 <= xlon2)
@@ -151,55 +150,45 @@ maskbox_cell(std::vector<bool> &mask, const int gridID)
 }
 
 static void
-maskregion(std::vector<bool> &mask, const int gridID, const std::vector<double> &xcoords, const std::vector<double> &ycoords,
+maskregion(std::vector<bool> &mask, const int gridID, const Varray<double> &xcoords, const Varray<double> &ycoords,
            const size_t nofcoords)
 {
-  size_t nlon = gridInqXsize(gridID);
-  size_t nlat = gridInqYsize(gridID);
+  const auto nlon = gridInqXsize(gridID);
+  const auto nlat = gridInqYsize(gridID);
 
-  std::vector<double> xvals(nlon);
-  std::vector<double> yvals(nlat);
+  Varray<double> xvals(nlon), yvals(nlat);
+  gridInqXvals(gridID, xvals.data());
+  gridInqYvals(gridID, yvals.data());
 
-  gridInqXvals(gridID, &xvals[0]);
-  gridInqYvals(gridID, &yvals[0]);
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID, CDI_XAXIS, nlon, xvals.data(), "grid center lon");
+  cdo_grid_to_degree(gridID, CDI_YAXIS, nlat, yvals.data(), "grid center lat");
 
-  /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridID, units);
-    grid_to_degree(units, nlon, &xvals[0], "grid center lon");
-    gridInqYunits(gridID, units);
-    grid_to_degree(units, nlat, &yvals[0], "grid center lat");
-  }
-
-  double xmin, xmax;
-  arrayMinMax(nlon, &xvals[0], xmin, xmax);
-
-  double ymin, ymax;
-  arrayMinMax(nlat, &yvals[0], ymin, ymax);
+  const auto xmm = varrayMinMax(xvals);
+  const auto ymm = varrayMinMax(yvals);
 
   for (size_t ilat = 0; ilat < nlat; ilat++)
     {
-      double yval = yvals[ilat];
+      const auto yval = yvals[ilat];
       for (size_t ilon = 0; ilon < nlon; ilon++)
         {
-          size_t i, j;
           int c = 0;
-          const double xval = xvals[ilon];
-          if (!(((xval > xmin) || (xval < xmax)) || ((yval > ymin) || (yval < ymax)))) c = !c;
+          const auto xval = xvals[ilon];
+          if (!(((xval > xmm.min) || (xval < xmm.max)) || ((yval > ymm.min) || (yval < ymm.max)))) c = !c;
 
           if (c == 0)
             {
-              for (i = 0, j = nofcoords - 1; i < nofcoords; j = i++)
-
-                if ((((ycoords[i] <= yval) && (yval < ycoords[j])) || ((ycoords[j] <= yval) && (yval < ycoords[i])))
-                    && ((xval) < (xcoords[j] - (xcoords[i])) * (yval - ycoords[i]) / (ycoords[j] - ycoords[i]) + (xcoords[i])))
-                  c = !c;
+              for (size_t i = 0, j = nofcoords - 1; i < nofcoords; j = i++)
+                {
+                  if ((((ycoords[i] <= yval) && (yval < ycoords[j])) || ((ycoords[j] <= yval) && (yval < ycoords[i])))
+                      && ((xval) < (xcoords[j] - (xcoords[i])) * (yval - ycoords[i]) / (ycoords[j] - ycoords[i]) + (xcoords[i])))
+                    c = !c;
+                }
             }
 
           if (c == 0)
             {
-              for (i = 0, j = nofcoords - 1; i < nofcoords; j = i++)
+              for (size_t i = 0, j = nofcoords - 1; i < nofcoords; j = i++)
                 {
                   if (xvals[ilon] > 180)
                     {
@@ -213,7 +202,7 @@ maskregion(std::vector<bool> &mask, const int gridID, const std::vector<double>
 
           if (c == 0)
             {
-              for (i = 0, j = nofcoords - 1; i < nofcoords; j = i++)
+              for (size_t i = 0, j = nofcoords - 1; i < nofcoords; j = i++)
                 {
                   if (xval < 0)
                     {
@@ -225,7 +214,7 @@ maskregion(std::vector<bool> &mask, const int gridID, const std::vector<double>
                 }
             }
 
-          if (c != 0) mask[nlon * ilat + ilon] = false;
+          if (c) mask[nlon * ilat + ilon] = false;
         }
     }
 }
@@ -236,33 +225,33 @@ Maskbox(void *process)
   int nrecs;
   int varID, levelID;
   int index, gridtype = -1;
-  long lat1, lat2, lon11, lon12, lon21, lon22;
+  SelboxInfo sbox;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int MASKLONLATBOX = cdoOperatorAdd("masklonlatbox", 0, 0, "western and eastern longitude and southern and northern latitude");
-  const int MASKINDEXBOX  = cdoOperatorAdd("maskindexbox",  0, 0, "index of first and last longitude and index of first and last latitude");
-  const int MASKREGION    = cdoOperatorAdd("maskregion",    0, 0, "limiting coordinates of the region");
+  const auto MASKLONLATBOX = cdoOperatorAdd("masklonlatbox", 0, 0, "western and eastern longitude and southern and northern latitude");
+  const auto MASKINDEXBOX  = cdoOperatorAdd("maskindexbox",  0, 0, "index of first and last longitude and index of first and last latitude");
+  const auto MASKREGION    = cdoOperatorAdd("maskregion",    0, 0, "limiting coordinates of the region");
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars, false);
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   int ndiffgrids = 0;
   for (index = 1; index < ngrids; index++)
     if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
@@ -286,18 +275,14 @@ Maskbox(void *process)
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      vars[varID] = (gridID == vlistInqVarGrid(vlistID1, varID));
-    }
+  for (varID = 0; varID < nvars; varID++) vars[varID] = (gridID == vlistInqVarGrid(vlistID1, varID));
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = gridInqSize(gridID);
-  std::vector<double> array(gridsize);
-  std::vector<bool> mask(gridsize);
-  for (size_t i = 0; i < gridsize; ++i) mask[i] = true;
+  const auto gridsize = gridInqSize(gridID);
+  Varray<double> array(gridsize);
+  std::vector<bool> mask(gridsize, true);
 
   if (operatorID == MASKLONLATBOX)
     {
@@ -307,31 +292,30 @@ Maskbox(void *process)
         }
       else
         {
-          genlonlatbox(0, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
-          maskbox(mask, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+          genlonlatbox(0, gridID, sbox);
+          maskbox(mask, gridID, sbox);
         }
     }
   else if (operatorID == MASKINDEXBOX)
     {
-      genindexbox(0, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
-      maskbox(mask, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+      genindexbox(0, gridID, sbox);
+      maskbox(mask, gridID, sbox);
     }
   else if (operatorID == MASKREGION)
     {
-      const size_t maxVals = 1048576;
-      std::vector<double> xcoords(maxVals);
-      std::vector<double> ycoords(maxVals);
-      const int nfiles = operatorArgc();
+      constexpr size_t maxVals = 1048576;
+      Varray<double> xcoords(maxVals), ycoords(maxVals);
+      const auto nfiles = operatorArgc();
 
       for (int i = 0; i < nfiles; i++)
         {
-          const char *polyfile = operatorArgv()[i];
-          FILE *fp = fopen(polyfile, "r");
+          const auto polyfile = cdoOperatorArgv(i).c_str();
+          auto fp = fopen(polyfile, "r");
           if (fp == nullptr) cdoAbort("Open failed on %s", polyfile);
 
           while (true)
             {
-              const int number = ReadCoords(xcoords.data(), ycoords.data(), polyfile, fp);
+              const auto number = ReadCoords(xcoords.data(), ycoords.data(), polyfile, fp);
               if (number == 0) break;
               if (number < 3) cdoAbort("Too less values in file %s", polyfile);
               maskregion(mask, gridID, xcoords, ycoords, number);
@@ -355,14 +339,13 @@ Maskbox(void *process)
               size_t nmiss;
               cdoReadRecord(streamID1, &array[0], &nmiss);
 
-              const double missval = vlistInqVarMissval(vlistID1, varID);
+              const auto missval = vlistInqVarMissval(vlistID1, varID);
               for (size_t i = 0; i < gridsize; i++)
                 if (mask[i]) array[i] = missval;
 
-              nmiss = arrayNumMV(gridsize, &array[0], missval);
-
+              nmiss = varrayNumMV(gridsize, array, missval);
               cdoDefRecord(streamID2, varID, levelID);
-              cdoWriteRecord(streamID2, &array[0], nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
