@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,53 +23,52 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "array.h"
+#include "process_int.h"
 #include <mpim_grid.h>
+#include "cdo_zaxis.h"
 
 static void
-mastrfu(int gridID, int zaxisID, std::vector<std::vector<double>> &field1, std::vector<std::vector<double>> &field2, size_t nmiss,
-        double missval)
+mastrfu(int gridID, int zaxisID, Varray2D<double> &field1, Varray2D<double> &field2, size_t nmiss, double missval)
 {
-  size_t ilat;
-  int ilev, n;
-  double fact = 4 * atan(1.0) * 6371000 / 9.81;
-  char units[CDI_MAX_NAME];
+  const double fact = 4 * std::atan(1.0) * 6371000 / 9.81;
 
-  size_t nlat = gridInqSize(gridID);
-  int nlev = zaxisInqSize(zaxisID);
-  std::vector<double> phi(nlat);
-  std::vector<double> cosphi(nlat);
-  std::vector<double> plevel(nlev);
+  const auto nlat = gridInqSize(gridID);
+  const auto nlev = zaxisInqSize(zaxisID);
+  Varray<double> phi(nlat), cosphi(nlat), plevel(nlev);
 
   cdoZaxisInqLevels(zaxisID, plevel.data());
 
   gridInqYvals(gridID, phi.data());
-  gridInqYunits(gridID, units);
+
+  char units[CDI_MAX_NAME];
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, units, &length);
 
   if (memcmp(units, "degree", 6) == 0)
-    for (ilat = 0; ilat < nlat; ilat++) phi[ilat] *= DEG2RAD;
+    for (size_t ilat = 0; ilat < nlat; ilat++) phi[ilat] *= DEG2RAD;
 
-  for (ilat = 0; ilat < nlat; ilat++) phi[ilat] = sin(phi[ilat]);
+  for (size_t ilat = 0; ilat < nlat; ilat++) phi[ilat] = std::sin(phi[ilat]);
 
-  for (ilat = 0; ilat < nlat; ilat++) cosphi[ilat] = sqrt(1.0 - phi[ilat] * phi[ilat]);
+  for (size_t ilat = 0; ilat < nlat; ilat++) cosphi[ilat] = std::sqrt(1.0 - phi[ilat] * phi[ilat]);
 
-  for (ilev = 0; ilev < nlev; ilev++)
-    for (ilat = 0; ilat < nlat; ilat++) field2[ilev][ilat] = 0.0;
+  for (int ilev = 0; ilev < nlev; ilev++)
+    for (size_t ilat = 0; ilat < nlat; ilat++) field2[ilev][ilat] = 0.0;
 
   if (nmiss == 0)
     {
-      for (ilev = nlev - 1; ilev >= 0; ilev--)
-        for (n = ilev; n < nlev - 1; n++)
-          for (ilat = 0; ilat < nlat; ilat++)
+      for (int ilev = nlev - 1; ilev >= 0; ilev--)
+        for (int n = ilev; n < nlev - 1; n++)
+          for (size_t ilat = 0; ilat < nlat; ilat++)
             {
               field2[ilev][ilat] += fact * (field1[n][ilat] + field1[n + 1][ilat]) * cosphi[ilat] * (plevel[n] - plevel[n + 1]);
             }
     }
   else
     {
-      for (ilat = 0; ilat < nlat; ilat++)
-        for (ilev = nlev - 1; ilev >= 0; ilev--)
-          for (n = ilev; n < nlev - 1; n++)
+      for (size_t ilat = 0; ilat < nlat; ilat++)
+        for (int ilev = nlev - 1; ilev >= 0; ilev--)
+          for (int n = ilev; n < nlev - 1; n++)
             {
               if (DBL_IS_EQUAL(field1[n][ilat], missval))
                 {
@@ -91,49 +90,52 @@ Mastrfu(void *process)
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  operatorCheckArgc(0);
 
-  int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+
+  const auto nvars = vlistNvars(vlistID1);
   if (nvars != 1) cdoAbort("This operator works only with one variable!");
 
-  int code = vlistInqVarCode(vlistID1, 0);
+  const auto code = vlistInqVarCode(vlistID1, 0);
   if (code > 0 && code != 132) cdoWarning("Unexpected code %d!", code);
 
-  double missval = vlistInqVarMissval(vlistID1, 0);
+  const auto missval = vlistInqVarMissval(vlistID1, 0);
 
-  int zaxisID = vlistInqVarZaxis(vlistID1, 0);
+  const auto zaxisID = vlistInqVarZaxis(vlistID1, 0);
   if (zaxisInqType(zaxisID) != ZAXIS_PRESSURE && zaxisInqType(zaxisID) != ZAXIS_GENERIC)
     {
       char longname[CDI_MAX_NAME];
-      zaxisInqLongname(zaxisID, longname);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, longname, &length);
       cdoWarning("Unexpected vertical grid %s!", longname);
     }
 
-  int gridID = vlistInqVarGrid(vlistID1, 0);
+  const auto gridID = vlistInqVarGrid(vlistID1, 0);
   if (gridInqXsize(gridID) > 1) cdoAbort("Grid must be a zonal mean!");
 
-  size_t gridsize = gridInqSize(gridID);
-  int nlev = zaxisInqSize(zaxisID);
+  const auto gridsize = gridInqSize(gridID);
+  const auto nlev = zaxisInqSize(zaxisID);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   vlistDefVarCode(vlistID2, 0, 272);
-  vlistDefVarName(vlistID2, 0, "mastrfu");
-  vlistDefVarLongname(vlistID2, 0, "mass stream function");
-  vlistDefVarUnits(vlistID2, 0, "kg/s");
+  cdiDefKeyString(vlistID2, 0, CDI_KEY_NAME, "mastrfu");
+  cdiDefKeyString(vlistID2, 0, CDI_KEY_LONGNAME, "mass stream function");
+  cdiDefKeyString(vlistID2, 0, CDI_KEY_UNITS, "kg/s");
   vlistDefVarDatatype(vlistID2, 0, CDI_DATATYPE_FLT32);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  VECTOR_2D(double, array1, nlev, gridsize);
-  VECTOR_2D(double, array2, nlev, gridsize);
+  Varray2D<double> array1(nlev, Varray<double>(gridsize));
+  Varray2D<double> array2(nlev, Varray<double>(gridsize));
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -167,5 +169,5 @@ Mastrfu(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
