@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_rlimit.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "util_files.h"
 #include "printinfo.h"
 
@@ -36,25 +40,24 @@ struct StreamFile
   int64_t vdate;
   int vtime;
   int nrecs;
+  VarList varList;
 };
 
-static bool
+bool
 getenvSkipSameTime()
 {
-  bool skipSameTime = false;
-
-  const char *envstr = getenv("SKIP_SAME_TIME");
+  const auto envstr = getenv("SKIP_SAME_TIME");
   if (envstr)
     {
-      const int ival = atoi(envstr);
+      const auto ival = atoi(envstr);
       if (ival == 1)
         {
-          skipSameTime = true;
           if (Options::cdoVerbose) cdoPrint("Set SKIP_SAME_TIME to %d", ival);
+          return true;
         }
     }
 
-  return skipSameTime;
+  return false;
 }
 
 static void
@@ -64,9 +67,11 @@ openAllFiles(int nfiles, std::vector<StreamFile> &sf)
     {
       if (Options::cdoVerbose) cdoPrint("process: %s", cdoGetStreamName(fileID));
 
-      sf[fileID].streamID = cdoOpenRead((fileID));
-      sf[fileID].vlistID = cdoStreamInqVlist(sf[fileID].streamID);
-      sf[fileID].taxisID = vlistInqTaxis(sf[fileID].vlistID);
+      auto &s = sf[fileID];
+      s.streamID = cdoOpenRead(fileID);
+      s.vlistID = cdoStreamInqVlist(s.streamID);
+      s.taxisID = vlistInqTaxis(s.vlistID);
+      varListInit(s.varList, s.vlistID);
     }
 }
 
@@ -75,17 +80,18 @@ readFirstTimestep(int nfiles, std::vector<StreamFile> &sf)
 {
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      sf[fileID].tsID = 0;
-      sf[fileID].nrecs = cdoStreamInqTimestep(sf[fileID].streamID, sf[fileID].tsID);
-      if (sf[fileID].nrecs == 0)
+      auto &s = sf[fileID];
+      s.tsID = 0;
+      s.nrecs = cdoStreamInqTimestep(s.streamID, s.tsID);
+      if (s.nrecs == 0)
         {
-          cdoStreamClose(sf[fileID].streamID);
-          sf[fileID].streamID = CDO_STREAM_UNDEF;
+          cdoStreamClose(s.streamID);
+          s.streamID = CDO_STREAM_UNDEF;
         }
       else
         {
-          sf[fileID].vdate = taxisInqVdate(sf[fileID].taxisID);
-          sf[fileID].vtime = taxisInqVtime(sf[fileID].taxisID);
+          s.vdate = taxisInqVdate(s.taxisID);
+          s.vtime = taxisInqVtime(s.taxisID);
         }
     }
 }
@@ -100,13 +106,17 @@ Mergetime(void *process)
 
   cdoInitialize(process);
 
-  const bool skipSameTime = getenvSkipSameTime();
+  operatorCheckArgc(0);
 
-  const bool lcopy = unchangedRecord();
+  const auto skipSameTime = getenvSkipSameTime();
 
-  const int nfiles = cdoStreamCnt() - 1;
+  const auto lcopy = unchangedRecord();
+
+  const auto nfiles = cdoStreamCnt() - 1;
   std::vector<StreamFile> sf(nfiles);
 
+  cdo::set_numfiles(nfiles + 8);
+
   openAllFiles(nfiles, sf);
 
   // check that the contents is always the same
@@ -115,18 +125,13 @@ Mergetime(void *process)
   // read the first time step
   readFirstTimestep(nfiles, sf);
 
-  const char *ofilename = cdoGetStreamName(nfiles);
+  auto ofilename = cdoGetStreamName(nfiles);
   if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
     cdoAbort("Outputfile %s already exists!", ofilename);
 
-  CdoStreamID streamID2 = cdoOpenWrite((nfiles));
+  const auto streamID2 = cdoOpenWrite(nfiles);
 
-  std::vector<double> array;
-  if (!lcopy)
-    {
-      size_t gridsizemax = vlistGridsizeMax(sf[0].vlistID);
-      array.resize(gridsizemax);
-    }
+  Field field;
 
   while (true)
     {
@@ -146,28 +151,26 @@ Mergetime(void *process)
               }
         }
 
-      const int fileID = next_fileID;
+      const auto fileID = next_fileID;
       if (Options::cdoVerbose) cdoPrint("nextstep = %d  vdate = %ld  vtime = %d", fileID, vdate, vtime);
       if (fileID == -1) break;
 
-      if (skipSameTime)
-        if (vdate == last_vdate && vtime == last_vtime)
-          {
-            char vdatestr[32], vtimestr[32];
-            date2str(vdate, vdatestr, sizeof(vdatestr));
-            time2str(vtime, vtimestr, sizeof(vtimestr));
-            cdoPrint("Timestep %4d in stream %d (%s %s) already exists, skipped!", sf[fileID].tsID + 1, sf[fileID].streamID->getID(),
-                     vdatestr, vtimestr);
-            process_timestep = false;
-          }
+      auto &s = sf[fileID];
+
+      if (skipSameTime && vdate == last_vdate && vtime == last_vtime)
+        {
+          cdoPrint("Timestep %4d in stream %d (%s %s) already exists, skipped!", s.tsID + 1,
+                   s.streamID->getID(), dateToString(vdate).c_str(), timeToString(vtime).c_str());
+          process_timestep = false;
+        }
 
       if (process_timestep)
         {
           if (tsID2 == 0)
             {
-              const int vlistID1 = sf[fileID].vlistID;
-              const int vlistID2 = vlistDuplicate(vlistID1);
-              const int taxisID1 = vlistInqTaxis(vlistID1);
+              const auto vlistID1 = s.vlistID;
+              const auto vlistID2 = vlistDuplicate(vlistID1);
+              const auto taxisID1 = vlistInqTaxis(vlistID1);
               taxisID2 = taxisDuplicate(taxisID1);
               vlistDefTaxis(vlistID2, taxisID2);
 
@@ -177,44 +180,43 @@ Mergetime(void *process)
           last_vdate = vdate;
           last_vtime = vtime;
 
-          taxisCopyTimestep(taxisID2, sf[fileID].taxisID);
+          taxisCopyTimestep(taxisID2, s.taxisID);
           cdoDefTimestep(streamID2, tsID2);
 
-          for (int recID = 0; recID < sf[fileID].nrecs; recID++)
+          for (int recID = 0; recID < s.nrecs; recID++)
             {
               int varID, levelID;
-              cdoInqRecord(sf[fileID].streamID, &varID, &levelID);
+              cdoInqRecord(s.streamID, &varID, &levelID);
 
-              if (tsID2 > 0 && sf[fileID].tsID == 0)
-                if (vlistInqVarTimetype(sf[fileID].vlistID, varID) == TIME_CONSTANT) continue;
+              if (tsID2 > 0 && s.tsID == 0 && s.varList[varID].timetype == TIME_CONSTANT) continue;
 
               cdoDefRecord(streamID2, varID, levelID);
 
               if (lcopy)
                 {
-                  cdoCopyRecord(streamID2, sf[fileID].streamID);
+                  cdoCopyRecord(streamID2, s.streamID);
                 }
               else
                 {
-                  size_t nmiss;
-                  cdoReadRecord(sf[fileID].streamID, array.data(), &nmiss);
-                  cdoWriteRecord(streamID2, array.data(), nmiss);
+                  field.init(s.varList[varID]);
+                  cdoReadRecord(s.streamID, field);
+                  cdoWriteRecord(streamID2, field);
                 }
             }
 
           tsID2++;
         }
 
-      sf[fileID].nrecs = cdoStreamInqTimestep(sf[fileID].streamID, ++sf[fileID].tsID);
-      if (sf[fileID].nrecs == 0)
+      s.nrecs = cdoStreamInqTimestep(s.streamID, ++s.tsID);
+      if (s.nrecs == 0)
         {
-          cdoStreamClose(sf[fileID].streamID);
-          sf[fileID].streamID = CDO_STREAM_UNDEF;
+          cdoStreamClose(s.streamID);
+          s.streamID = CDO_STREAM_UNDEF;
         }
       else
         {
-          sf[fileID].vdate = taxisInqVdate(sf[fileID].taxisID);
-          sf[fileID].vtime = taxisInqVtime(sf[fileID].taxisID);
+          s.vdate = taxisInqVdate(s.taxisID);
+          s.vtime = taxisInqVtime(s.taxisID);
         }
     }
 
