@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,30 +25,22 @@
       Merstat    mermean         Meridional mean
       Merstat    meravg          Meridional average
       Merstat    merstd          Meridional standard deviation
-      Merstat    merstd          Meridional standard deviation [Normalize by (n-1)]
+      Merstat    merstd1         Meridional standard deviation [Normalize by (n-1)]
       Merstat    mervar          Meridional variance
-      Merstat    mervar          Meridional variance [Normalize by (n-1)]
+      Merstat    mervar1         Meridional variance [Normalize by (n-1)]
       Merstat    merpctl         Meridional percentiles
 */
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
-#include "percentiles.h"
 
-void *
-Merstat(void *process)
+static void
+addOperators(void)
 {
-  int gridID1, gridID2 = -1, lastgrid = -1;
-  int index;
-  int nrecs;
-  int varID, levelID;
-  char varname[CDI_MAX_NAME];
-
-  cdoInitialize(process);
-
   // clang-format off
   cdoOperatorAdd("merrange", func_range, 0, nullptr);
   cdoOperatorAdd("mermin",   func_min,   0, nullptr);
@@ -60,31 +52,49 @@ Merstat(void *process)
   cdoOperatorAdd("mervar1",  func_var1w, 1, nullptr);
   cdoOperatorAdd("merstd",   func_stdw,  1, nullptr);
   cdoOperatorAdd("merstd1",  func_std1w, 1, nullptr);
+  cdoOperatorAdd("merskew",  func_skew,  0, nullptr);
+  cdoOperatorAdd("merkurt",  func_kurt,  0, nullptr);
   cdoOperatorAdd("merpctl",  func_pctl,  0, nullptr);
   // clang-format on
+}
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+void *
+Merstat(void *process)
+{
+  int gridID1, gridID2 = -1, lastgrid = -1;
+  int index;
+  int nrecs;
+  int varID, levelID;
+
+  cdoInitialize(process);
+
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
   const bool needWeights = cdoOperatorF2(operatorID) != 0;
 
   double pn = 0;
   if (operfunc == func_pctl)
     {
       operatorInputArg("percentile number");
-      pn = parameter2double(operatorArgv()[0]);
-      percentile_check_number(pn);
+      pn = parameter2double(cdoOperatorArgv(0));
+    }
+  else
+    {
+      operatorCheckArgc(0);
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   int ndiffgrids = 0;
   for (index = 1; index < ngrids; index++)
     if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
@@ -105,12 +115,12 @@ Merstat(void *process)
 
   vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   gridID1 = vlistInqVarGrid(vlistID1, 0);
-  const int nlonmax = gridInqXsize(gridID1); /* max nlon ? */
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const int nlonmax = gridInqXsize(gridID1); // max nlon?
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
   field1.resize(gridsizemax);
@@ -119,6 +129,9 @@ Merstat(void *process)
   field2.resize(nlonmax);
   field2.grid = gridID2;
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
@@ -128,8 +141,11 @@ Merstat(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
-          field1.grid = vlistInqVarGrid(vlistID1, varID);
+          cdoReadRecord(streamID1, field1.vec_d.data(), &field1.nmiss);
+
+          field1.grid = varList1[varID].gridID;
+          field1.missval = varList1[varID].missval;
+          field2.missval = varList1[varID].missval;
 
           bool wstatus = false;
           if (needWeights && field1.grid != lastgrid)
@@ -139,21 +155,13 @@ Merstat(void *process)
             }
 
           if (wstatus != 0 && tsID == 0 && levelID == 0)
-            {
-              vlistInqVarName(vlistID1, varID, varname);
-              cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
-            }
+            cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!",
+                       varList1[varID].name);
 
-          field1.missval = vlistInqVarMissval(vlistID1, varID);
-          field2.missval = vlistInqVarMissval(vlistID1, varID);
-
-          if (operfunc == func_pctl)
-            merpctl(field1, field2, pn);
-          else
-            merfun(field1, field2, operfunc);
+          (operfunc == func_pctl) ? fieldMerPctl(field1, field2, pn) : fieldMerFunction(field1, field2, operfunc);
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, field2.vec.data(), field2.nmiss);
+          cdoWriteRecord(streamID2, field2.vec_d.data(), field2.nmiss);
         }
 
       tsID++;
