@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,70 +23,68 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
 #include <mpim_grid.h>
+#include "array.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdi_lockedIO.h"
 
 void
-rotate_uv(double *u_i, double *v_j, int ix, int iy, double *lon, double *lat, double *u_lon, double *v_lat)
+rotate_uv(double *u_i, double *v_j, long ix, long iy, double *lon, double *lat, double *u_lon, double *v_lat)
 {
   /*
-    real,intent(in)      :: u_i(ix,iy,iz),v_j(ix,iy,iz)      ! vector components in i-j-direction real,intent(out)     ::
-                            u_lon(ix,iy,iz),v_lat(ix,iy,iz)  ! vector components in lon-lat direction
-    real,intent(in)      :: lat(ix,iy),lon(ix,iy)            ! latitudes and longitudes
+    in      :: u_i(ix,iy),v_j(ix,iy)      vector components in i-j-direction
+    in      :: lat(ix,iy),lon(ix,iy)      latitudes and longitudes
+    out     :: u_lon(ix,iy),v_lat(ix,iy)  vector components in lon-lat direction
   */
-  double dlat_i, dlat_j, dlon_i, dlon_j, dist_i, dist_j;
-  double lat_factor;
-  double absold, absnew; /* velocity vector lengths */
-  int i, j, ip1, im1, jp1, jm1;
   double pi = 3.14159265359;
 
-  /* specification whether change in sign is needed for the input arrays */
+  // specification whether change in sign is needed for the input arrays
   bool change_sign_u = false;
   bool change_sign_v = true;
 
-  /* initialization */
-  for (i = 0; i < ix * iy; i++)
+  // initialization
+  for (long i = 0; i < ix * iy; i++)
     {
       v_lat[i] = 0;
       u_lon[i] = 0;
     }
 
-  /* rotation */
-  for (j = 0; j < iy; j++)
-    for (i = 0; i < ix; i++)
+  // rotation
+  for (long j = 0; j < iy; j++)
+    for (long i = 0; i < ix; i++)
       {
-        ip1 = i + 1;
-        im1 = i - 1;
-        jp1 = j + 1;
-        jm1 = j - 1;
-        if (ip1 >= ix) ip1 = 0; /* the 0-meridian */
+        auto ip1 = i + 1;
+        auto im1 = i - 1;
+        auto jp1 = j + 1;
+        auto jm1 = j - 1;
+        if (ip1 >= ix) ip1 = 0; // the 0-meridian
         if (im1 < 0) im1 = ix - 1;
-        if (jp1 >= iy) jp1 = j; /* treatment of the last..  */
-        if (jm1 < 0) jm1 = j;   /* .. and the fist grid-row */
+        if (jp1 >= iy) jp1 = j; // treatment of the last..
+        if (jm1 < 0) jm1 = j;   // .. and the fist grid-row
 
-        /* difference in latitudes */
-        dlat_i = lat[IX2D(j, ip1, ix)] - lat[IX2D(j, im1, ix)];
-        dlat_j = lat[IX2D(jp1, i, ix)] - lat[IX2D(jm1, i, ix)];
+        // difference in latitudes
+        auto dlat_i = lat[IX2D(j, ip1, ix)] - lat[IX2D(j, im1, ix)];
+        auto dlat_j = lat[IX2D(jp1, i, ix)] - lat[IX2D(jm1, i, ix)];
 
-        /* difference in longitudes */
-        dlon_i = lon[IX2D(j, ip1, ix)] - lon[IX2D(j, im1, ix)];
+        // difference in longitudes
+        auto dlon_i = lon[IX2D(j, ip1, ix)] - lon[IX2D(j, im1, ix)];
         if (dlon_i > pi) dlon_i -= 2 * pi;
         if (dlon_i < (-pi)) dlon_i += 2 * pi;
-        dlon_j = lon[IX2D(jp1, i, ix)] - lon[IX2D(jm1, i, ix)];
+        auto dlon_j = lon[IX2D(jp1, i, ix)] - lon[IX2D(jm1, i, ix)];
         if (dlon_j > pi) dlon_j -= 2 * pi;
         if (dlon_j < (-pi)) dlon_j += 2 * pi;
 
-        lat_factor = cos(lat[IX2D(j, i, ix)]);
+        const auto lat_factor = std::cos(lat[IX2D(j, i, ix)]);
         dlon_i = dlon_i * lat_factor;
         dlon_j = dlon_j * lat_factor;
 
-        /* projection by scalar product */
+        // projection by scalar product
         u_lon[IX2D(j, i, ix)] = u_i[IX2D(j, i, ix)] * dlon_i + v_j[IX2D(j, i, ix)] * dlat_i;
         v_lat[IX2D(j, i, ix)] = u_i[IX2D(j, i, ix)] * dlon_j + v_j[IX2D(j, i, ix)] * dlat_j;
 
-        dist_i = sqrt(dlon_i * dlon_i + dlat_i * dlat_i);
-        dist_j = sqrt(dlon_j * dlon_j + dlat_j * dlat_j);
+        auto dist_i = std::sqrt(dlon_i * dlon_i + dlat_i * dlat_i);
+        auto dist_j = std::sqrt(dlon_j * dlon_j + dlat_j * dlat_j);
 
         if (std::fabs(dist_i) > 0 && std::fabs(dist_j) > 0)
           {
@@ -99,8 +97,9 @@ rotate_uv(double *u_i, double *v_j, int ix, int iy, double *lon, double *lat, do
             v_lat[IX2D(j, i, ix)] = 0.0;
           }
 
-        absold = sqrt(u_i[IX2D(j, i, ix)] * u_i[IX2D(j, i, ix)] + v_j[IX2D(j, i, ix)] * v_j[IX2D(j, i, ix)]);
-        absnew = sqrt(u_lon[IX2D(j, i, ix)] * u_lon[IX2D(j, i, ix)] + v_lat[IX2D(j, i, ix)] * v_lat[IX2D(j, i, ix)]);
+        // velocity vector lengths
+        auto absold = std::sqrt(u_i[IX2D(j, i, ix)] * u_i[IX2D(j, i, ix)] + v_j[IX2D(j, i, ix)] * v_j[IX2D(j, i, ix)]);
+        auto absnew = std::sqrt(u_lon[IX2D(j, i, ix)] * u_lon[IX2D(j, i, ix)] + v_lat[IX2D(j, i, ix)] * v_lat[IX2D(j, i, ix)]);
 
         u_lon[IX2D(j, i, ix)] *= absold;
         v_lat[IX2D(j, i, ix)] *= absold;
@@ -116,38 +115,36 @@ rotate_uv(double *u_i, double *v_j, int ix, int iy, double *lon, double *lat, do
             v_lat[IX2D(j, i, ix)] = 0.0;
           }
 
-        /* change sign */
+        // change sign
         if (change_sign_u) u_lon[IX2D(j, i, ix)] *= -1;
         if (change_sign_v) v_lat[IX2D(j, i, ix)] *= -1;
 
         if (Options::cdoVerbose)
           {
-            absold = sqrt(u_i[IX2D(j, i, ix)] * u_i[IX2D(j, i, ix)] + v_j[IX2D(j, i, ix)] * v_j[IX2D(j, i, ix)]);
-            absnew = sqrt(u_lon[IX2D(j, i, ix)] * u_lon[IX2D(j, i, ix)] + v_lat[IX2D(j, i, ix)] * v_lat[IX2D(j, i, ix)]);
+            absold = std::sqrt(u_i[IX2D(j, i, ix)] * u_i[IX2D(j, i, ix)] + v_j[IX2D(j, i, ix)] * v_j[IX2D(j, i, ix)]);
+            absnew = std::sqrt(u_lon[IX2D(j, i, ix)] * u_lon[IX2D(j, i, ix)] + v_lat[IX2D(j, i, ix)] * v_lat[IX2D(j, i, ix)]);
 
             if (i % 20 == 0 && j % 20 == 0 && absold > 0)
               {
-                printf("(absold,absnew) %d %d %g %g %g %g %g %g\n", j + 1, i + 1, absold, absnew, u_i[IX2D(j, i, ix)],
+                printf("(absold,absnew) %ld %ld %g %g %g %g %g %g\n", j + 1, i + 1, absold, absnew, u_i[IX2D(j, i, ix)],
                        v_j[IX2D(j, i, ix)], u_lon[IX2D(j, i, ix)], v_lat[IX2D(j, i, ix)]);
 
-                /* test orthogonality */
+                // test orthogonality
                 if ((dlon_i * dlon_j + dlat_j * dlat_i) > 0.1)
-                  fprintf(stderr, "orthogonal? %d %d %g\n", j + 1, i + 1, (dlon_i * dlon_j + dlat_j * dlat_i));
+                  fprintf(stderr, "orthogonal? %ld %ld %g\n", j + 1, i + 1, (dlon_i * dlon_j + dlat_j * dlat_i));
               }
           }
       }
 }
 
 void
-p_to_uv_grid(int nlon, int nlat, double *grid1x, double *grid1y, double *gridux, double *griduy, double *gridvx, double *gridvy)
+p_to_uv_grid(long nlon, long nlat, double *grid1x, double *grid1y, double *gridux, double *griduy, double *gridvx, double *gridvy)
 {
-  int i, j, jp1, ip1;
-
-  /* interpolate scalar to u points */
-  for (j = 0; j < nlat; j++)
-    for (i = 0; i < nlon; i++)
+  // interpolate scalar to u points
+  for (long j = 0; j < nlat; j++)
+    for (long i = 0; i < nlon; i++)
       {
-        ip1 = i + 1;
+        auto ip1 = i + 1;
         if (ip1 > nlon - 1) ip1 = 0;
 
         gridux[IX2D(j, i, nlon)] = (grid1x[IX2D(j, i, nlon)] + grid1x[IX2D(j, ip1, nlon)]) * 0.5;
@@ -163,11 +160,11 @@ p_to_uv_grid(int nlon, int nlat, double *grid1x, double *grid1y, double *gridux,
         griduy[IX2D(j, i, nlon)] = (grid1y[IX2D(j, i, nlon)] + grid1y[IX2D(j, ip1, nlon)]) * 0.5;
       }
 
-  /* interpolate scalar to v points */
-  for (j = 0; j < nlat; j++)
-    for (i = 0; i < nlon; i++)
+  // interpolate scalar to v points
+  for (long j = 0; j < nlat; j++)
+    for (long i = 0; i < nlon; i++)
       {
-        jp1 = j + 1;
+        auto jp1 = j + 1;
         if (jp1 > nlat - 1) jp1 = nlat - 1;
 
         gridvx[IX2D(j, i, nlon)] = (grid1x[IX2D(j, i, nlon)] + grid1x[IX2D(jp1, i, nlon)]) * 0.5;
@@ -195,11 +192,13 @@ Mrotuv(void *process)
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+
+  const auto nvars = vlistNvars(vlistID1);
   for (varid = 0; varid < nvars; varid++)
     {
       int code = vlistInqVarCode(vlistID1, varid);
@@ -218,12 +217,12 @@ Mrotuv(void *process)
         cdoAbort("U and V not found in %s", cdoGetStreamName(0));
     }
 
-  int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, uid));
+  const auto nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, uid));
   if (nlevs != zaxisInqSize(vlistInqVarZaxis(vlistID1, vid))) cdoAbort("U and V have different number of levels!");
 
-  int gridID1 = vlistInqVarGrid(vlistID1, uid);
-  int gridID2 = vlistInqVarGrid(vlistID1, vid);
-  size_t gridsize = gridInqSize(gridID1);
+  auto gridID1 = vlistInqVarGrid(vlistID1, uid);
+  const auto gridID2 = vlistInqVarGrid(vlistID1, vid);
+  const auto gridsize = gridInqSize(gridID1);
   if (gridID1 != gridID2) cdoAbort("Input grids differ!");
 
   if (gridInqType(gridID1) != GRID_LONLAT && gridInqType(gridID1) != GRID_GAUSSIAN && gridInqType(gridID1) != GRID_CURVILINEAR)
@@ -233,40 +232,32 @@ Mrotuv(void *process)
 
   if (gridsize != gridInqSize(gridID1)) cdoAbort("Internal problem: gridsize changed!");
 
-  size_t nlon = gridInqXsize(gridID1);
-  size_t nlat = gridInqYsize(gridID1);
+  const auto nlon = gridInqXsize(gridID1);
+  const auto nlat = gridInqYsize(gridID1);
 
-  std::vector<double> grid1x(gridsize);
-  std::vector<double> grid1y(gridsize);
-  std::vector<double> gridux(gridsize);
-  std::vector<double> griduy(gridsize);
-  std::vector<double> gridvx(gridsize);
-  std::vector<double> gridvy(gridsize);
+  Varray<double> grid1x(gridsize), grid1y(gridsize);
+  Varray<double> gridux(gridsize), griduy(gridsize);
+  Varray<double> gridvx(gridsize), gridvy(gridsize);
 
-  size_t gridsizex = (nlon + 2) * nlat;
+  const auto gridsizex = (nlon + 2) * nlat;
 
   gridInqXvals(gridID1, grid1x.data());
   gridInqYvals(gridID1, grid1y.data());
 
-  /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridID1, units);
-    grid_to_degree(units, gridsize, grid1x.data(), "grid center lon");
-    gridInqYunits(gridID1, units);
-    grid_to_degree(units, gridsize, grid1y.data(), "grid center lat");
-  }
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID1, CDI_XAXIS, gridsize, grid1x.data(), "grid center lon");
+  cdo_grid_to_degree(gridID1, CDI_YAXIS, gridsize, grid1y.data(), "grid center lat");
 
   p_to_uv_grid(nlon, nlat, grid1x.data(), grid1y.data(), gridux.data(), griduy.data(), gridvx.data(), gridvy.data());
 
-  int gridIDu = gridCreate(GRID_CURVILINEAR, nlon * nlat);
+  const auto gridIDu = gridCreate(GRID_CURVILINEAR, nlon * nlat);
   gridDefDatatype(gridIDu, gridInqDatatype(gridID1));
   gridDefXsize(gridIDu, nlon);
   gridDefYsize(gridIDu, nlat);
   gridDefXvals(gridIDu, gridux.data());
   gridDefYvals(gridIDu, griduy.data());
 
-  int gridIDv = gridCreate(GRID_CURVILINEAR, nlon * nlat);
+  const auto gridIDv = gridCreate(GRID_CURVILINEAR, nlon * nlat);
   gridDefDatatype(gridIDv, gridInqDatatype(gridID1));
   gridDefXsize(gridIDv, nlon);
   gridDefYsize(gridIDv, nlat);
@@ -281,44 +272,41 @@ Mrotuv(void *process)
 
   vlistClearFlag(vlistID1);
   for (int lid = 0; lid < nlevs; lid++) vlistDefFlag(vlistID1, uid, lid, true);
-  int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
   vlistChangeVarGrid(vlistID2, 0, gridIDu);
 
   vlistClearFlag(vlistID1);
   for (int lid = 0; lid < nlevs; lid++) vlistDefFlag(vlistID1, vid, lid, true);
-  int vlistID3 = vlistCreate();
+  const auto vlistID3 = vlistCreate();
   cdoVlistCopyFlag(vlistID3, vlistID1);
   vlistChangeVarGrid(vlistID3, 0, gridIDv);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID2 = cdoOpenWrite(1);
+  const auto streamID3 = cdoOpenWrite(2);
 
   cdoDefVlist(streamID2, vlistID2);
   cdoDefVlist(streamID3, vlistID3);
 
-  double missval1 = vlistInqVarMissval(vlistID1, uid);
-  double missval2 = vlistInqVarMissval(vlistID1, vid);
+  const auto missval1 = vlistInqVarMissval(vlistID1, uid);
+  const auto missval2 = vlistInqVarMissval(vlistID1, vid);
 
-  std::vector<double> ufield(gridsize);
-  std::vector<double> vfield(gridsize);
+  Varray<double> ufield(gridsize), vfield(gridsize);
 
-  std::vector<std::vector<double>> urfield(nlevs);
-  std::vector<std::vector<double>> vrfield(nlevs);
+  Varray2D<double> urfield(nlevs), vrfield(nlevs);
   for (int lid = 0; lid < nlevs; lid++)
     {
       urfield[lid].resize(gridsize);
       vrfield[lid].resize(gridsize);
     }
 
-  std::vector<double> uhelp(gridsizex);
-  std::vector<double> vhelp(gridsizex);
+  Varray<double> uhelp(gridsizex), vhelp(gridsizex);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -338,7 +326,7 @@ Mrotuv(void *process)
 
       for (levelID = 0; levelID < nlevs; levelID++)
         {
-          /* remove missing values */
+          // remove missing values
           if (nmiss1 || nmiss2)
             {
               for (size_t i = 0; i < gridsize; i++)
@@ -348,12 +336,11 @@ Mrotuv(void *process)
                 }
             }
 
-          /* rotate*/
-
+          // rotate
           rotate_uv(urfield[levelID].data(), vrfield[levelID].data(), nlon, nlat, grid1x.data(), grid1y.data(), ufield.data(),
                     vfield.data());
 
-          /* load to a help field */
+          // load to a help field
           for (size_t j = 0; j < nlat; j++)
             for (size_t i = 0; i < nlon; i++)
               {
@@ -361,7 +348,7 @@ Mrotuv(void *process)
                 vhelp[IX2D(j, i + 1, nlon + 2)] = vfield[IX2D(j, i, nlon)];
               }
 
-          /* make help field cyclic */
+          // make help field cyclic
           for (size_t j = 0; j < nlat; j++)
             {
               uhelp[IX2D(j, 0, nlon + 2)] = uhelp[IX2D(j, nlon, nlon + 2)];
@@ -370,7 +357,7 @@ Mrotuv(void *process)
               vhelp[IX2D(j, nlon + 1, nlon + 2)] = vhelp[IX2D(j, 1, nlon + 2)];
             }
 
-          /* interpolate on u/v points */
+          // interpolate on u/v points
           for (size_t j = 0; j < nlat; j++)
             for (size_t i = 0; i < nlon; i++)
               {
@@ -403,5 +390,5 @@ Mrotuv(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
