@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "array.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 
 /*
@@ -41,83 +42,76 @@
 !
 !           if this routine is used to rotate data of mpi-om, the
 !           logical change_sign_v needs to be true.
-!j. jungclaus: 22.01.04:
-!note here for the coupling fields u-i,v_j are on the non-verlapping
-! (ie-2=ix) grid, furthermore, the velocity fields were previously
-! interpolated onto the scalar points !
-!
-!h.haak: 07.10.2005 vectorisation and omp directives
-!malte: use outside mpiom 02.06.2006
+!  j. jungclaus: 22.01.04:
+!    note here for the coupling fields u-i,v_j are on the non-verlapping
+!    (ie-2=ix) grid, furthermore, the velocity fields were previously
+!    interpolated onto the scalar points !
 !----------------------------------------------------------------------
 */
 
 void
-rotate_uv2(double *u_i, double *v_j, int ix, int iy, double *lon, double *lat, double *u_lon, double *v_lat)
+rotate_uv2(double *u_i, double *v_j, long  ix, long iy, double *lon, double *lat, double *u_lon, double *v_lat)
 {
   /*
-    real,intent(in)      :: u_i(ix,iy,iz),v_j(ix,iy,iz)       ! vector components in i-j-direction
-    real,intent(out)     :: u_lon(ix,iy,iz),v_lat(ix,iy,iz)   ! vector components in lon-lat direction
-    real,intent(in)      :: lat(ix,iy),lon(ix,iy)             ! latitudes and longitudes
+    in      :: u_i(ix,iy),v_j(ix,iy)       vector components in i-j-direction
+    in      :: lat(ix,iy),lon(ix,iy)       latitudes and longitudes
+    out     :: u_lon(ix,iy),v_lat(ix,iy)   vector components in lon-lat direction
   */
-  double dlat_i, dlat_j, dlon_i, dlon_j, dist_i, dist_j;
-  double lat_factor;
-  double absold, absnew; /* velocity vector lengths */
-  int i, j, ip1, im1, jp1, jm1;
   double pi = 3.14159265359;
 
-  /* specification whether change in sign is needed for the input arrays */
+  // specification whether change in sign is needed for the input arrays
   bool change_sign_u = false;
   bool change_sign_v = true;
 
-  /* initialization */
-  for (i = 0; i < ix * iy; i++)
+  // initialization
+  for (long i = 0; i < ix * iy; i++)
     {
       v_lat[i] = 0;
       u_lon[i] = 0;
     }
 
-  /* change sign */
+  // change sign
   if (change_sign_u)
-    for (i = 0; i < ix * iy; i++) u_i[i] *= -1;
+    for (long i = 0; i < ix * iy; i++) u_i[i] *= -1;
 
   if (change_sign_v)
-    for (i = 0; i < ix * iy; i++) v_j[i] *= -1;
+    for (long i = 0; i < ix * iy; i++) v_j[i] *= -1;
 
-  /* rotation */
-  for (j = 0; j < iy; j++)
-    for (i = 0; i < ix; i++)
+  // rotation
+  for (long j = 0; j < iy; j++)
+    for (long i = 0; i < ix; i++)
       {
-        ip1 = i + 1;
-        im1 = i - 1;
-        jp1 = j + 1;
-        jm1 = j - 1;
-        if (ip1 >= ix) ip1 = 0; /* the 0-meridian */
+        auto ip1 = i + 1;
+        auto im1 = i - 1;
+        auto jp1 = j + 1;
+        auto jm1 = j - 1;
+        if (ip1 >= ix) ip1 = 0; // the 0-meridian
         if (im1 < 0) im1 = ix - 1;
-        if (jp1 >= iy) jp1 = j; /* treatment of the last..  */
-        if (jm1 < 0) jm1 = j;   /* .. and the fist grid-row */
+        if (jp1 >= iy) jp1 = j; // treatment of the last..
+        if (jm1 < 0) jm1 = j;   // .. and the fist grid-row
 
-        /* difference in latitudes */
-        dlat_i = lat[IX2D(j, ip1, ix)] - lat[IX2D(j, im1, ix)];
-        dlat_j = lat[IX2D(jp1, i, ix)] - lat[IX2D(jm1, i, ix)];
+        // difference in latitudes
+        auto dlat_i = lat[IX2D(j, ip1, ix)] - lat[IX2D(j, im1, ix)];
+        auto dlat_j = lat[IX2D(jp1, i, ix)] - lat[IX2D(jm1, i, ix)];
 
-        /* difference in longitudes */
-        dlon_i = lon[IX2D(j, ip1, ix)] - lon[IX2D(j, im1, ix)];
+        // difference in longitudes
+        auto dlon_i = lon[IX2D(j, ip1, ix)] - lon[IX2D(j, im1, ix)];
         if (dlon_i > pi) dlon_i -= 2 * pi;
         if (dlon_i < (-pi)) dlon_i += 2 * pi;
-        dlon_j = lon[IX2D(jp1, i, ix)] - lon[IX2D(jm1, i, ix)];
+        auto dlon_j = lon[IX2D(jp1, i, ix)] - lon[IX2D(jm1, i, ix)];
         if (dlon_j > pi) dlon_j -= 2 * pi;
         if (dlon_j < (-pi)) dlon_j += 2 * pi;
 
-        lat_factor = cos(lat[IX2D(j, i, ix)]);
+        auto lat_factor = std::cos(lat[IX2D(j, i, ix)]);
         dlon_i = dlon_i * lat_factor;
         dlon_j = dlon_j * lat_factor;
 
-        /* projection by scalar product */
+        // projection by scalar product
         u_lon[IX2D(j, i, ix)] = u_i[IX2D(j, i, ix)] * dlon_i + v_j[IX2D(j, i, ix)] * dlat_i;
         v_lat[IX2D(j, i, ix)] = u_i[IX2D(j, i, ix)] * dlon_j + v_j[IX2D(j, i, ix)] * dlat_j;
 
-        dist_i = sqrt(dlon_i * dlon_i + dlat_i * dlat_i);
-        dist_j = sqrt(dlon_j * dlon_j + dlat_j * dlat_j);
+        auto dist_i = std::sqrt(dlon_i * dlon_i + dlat_i * dlat_i);
+        auto dist_j = std::sqrt(dlon_j * dlon_j + dlat_j * dlat_j);
 
         if (std::fabs(dist_i) > 0 && std::fabs(dist_j) > 0)
           {
@@ -132,17 +126,18 @@ rotate_uv2(double *u_i, double *v_j, int ix, int iy, double *lon, double *lat, d
 
         if (Options::cdoVerbose)
           {
-            absold = sqrt(u_i[IX2D(j, i, ix)] * u_i[IX2D(j, i, ix)] + v_j[IX2D(j, i, ix)] * v_j[IX2D(j, i, ix)]);
-            absnew = sqrt(u_lon[IX2D(j, i, ix)] * u_lon[IX2D(j, i, ix)] + v_lat[IX2D(j, i, ix)] * v_lat[IX2D(j, i, ix)]);
+            // velocity vector lengths
+            auto absold = std::sqrt(u_i[IX2D(j, i, ix)] * u_i[IX2D(j, i, ix)] + v_j[IX2D(j, i, ix)] * v_j[IX2D(j, i, ix)]);
+            auto absnew = std::sqrt(u_lon[IX2D(j, i, ix)] * u_lon[IX2D(j, i, ix)] + v_lat[IX2D(j, i, ix)] * v_lat[IX2D(j, i, ix)]);
 
             if (i % 20 == 0 && j % 20 == 0 && absold > 0)
               {
-                printf("(absold,absnew) %d %d %g %g %g %g %g %g\n", j + 1, i + 1, absold, absnew, u_i[IX2D(j, i, ix)],
+                printf("(absold,absnew) %ld %ld %g %g %g %g %g %g\n", j + 1, i + 1, absold, absnew, u_i[IX2D(j, i, ix)],
                        v_j[IX2D(j, i, ix)], u_lon[IX2D(j, i, ix)], v_lat[IX2D(j, i, ix)]);
 
-                /* test orthogonality */
+                // test orthogonality
                 if ((dlon_i * dlon_j + dlat_j * dlat_i) > 0.1)
-                  fprintf(stderr, "orthogonal? %d %d %g\n", j + 1, i + 1, (dlon_i * dlon_j + dlat_j * dlat_i));
+                  fprintf(stderr, "orthogonal? %ld %ld %g\n", j + 1, i + 1, (dlon_i * dlon_j + dlat_j * dlat_i));
               }
           }
       }
@@ -152,13 +147,10 @@ static void
 uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *grid2x, double *grid2y, double *grid3x,
              double *grid3y)
 {
-  double gx, gy;
-  double gx2, gy2;
+  const auto gridsizex = (nlon + 2) * nlat;
+  Varray<double> gxhelp(gridsizex), gyhelp(gridsizex);
 
-  size_t gridsizex = (nlon + 2) * nlat;
-  std::vector<double> gxhelp(gridsizex), gyhelp(gridsizex);
-
-  /* load to a help field */
+  // load to a help field
   for (size_t j = 0; j < nlat; j++)
     for (size_t i = 0; i < nlon; i++)
       {
@@ -166,7 +158,7 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
         gyhelp[IX2D(j, i + 1, nlon + 2)] = grid1y[IX2D(j, i, nlon)];
       }
 
-  /* make help field cyclic */
+  // make help field cyclic
   for (size_t j = 0; j < nlat; j++)
     {
       gxhelp[IX2D(j, 0, nlon + 2)] = gxhelp[IX2D(j, nlon, nlon + 2)];
@@ -175,7 +167,7 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
       gyhelp[IX2D(j, nlon + 1, nlon + 2)] = gyhelp[IX2D(j, 1, nlon + 2)];
     }
 
-  /* interpolate u to scalar points */
+  // interpolate u to scalar points
   for (size_t j = 0; j < nlat; j++)
     for (size_t i = 0; i < nlon; i++)
       {
@@ -192,7 +184,7 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
         grid3y[IX2D(j, i, nlon)] = (gyhelp[IX2D(j, i, nlon + 2)] + gyhelp[IX2D(j, i + 1, nlon + 2)]) * 0.5;
       }
 
-  /* load to a help field */
+  // load to a help field
   for (size_t j = 0; j < nlat; j++)
     for (size_t i = 0; i < nlon; i++)
       {
@@ -200,7 +192,7 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
         gyhelp[IX2D(j, i + 1, nlon + 2)] = grid2y[IX2D(j, i, nlon)];
       }
 
-  /* make help field cyclic */
+  // make help field cyclic
   for (size_t j = 0; j < nlat; j++)
     {
       gxhelp[IX2D(j, 0, nlon + 2)] = gxhelp[IX2D(j, nlon, nlon + 2)];
@@ -209,11 +201,11 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
       gyhelp[IX2D(j, nlon + 1, nlon + 2)] = gyhelp[IX2D(j, 1, nlon + 2)];
     }
 
-  /* interpolate v to scalar points */
+  // interpolate v to scalar points
   for (size_t j = 1; j < nlat - 1; j++)
     for (size_t i = 0; i < nlon; i++)
       {
-        gx = (gxhelp[IX2D(j, i + 1, nlon + 2)] + gxhelp[IX2D(j - 1, i + 1, nlon + 2)]) * 0.5;
+        auto gx = (gxhelp[IX2D(j, i + 1, nlon + 2)] + gxhelp[IX2D(j - 1, i + 1, nlon + 2)]) * 0.5;
         if ((gxhelp[IX2D(j, i + 1, nlon + 2)] > 340 && gxhelp[IX2D(j - 1, i + 1, nlon + 2)] < 20)
             || (gxhelp[IX2D(j, i + 1, nlon + 2)] < 20 && gxhelp[IX2D(j - 1, i + 1, nlon + 2)] > 340))
           {
@@ -223,12 +215,11 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
               gx -= 180;
           }
 
-        gy = (gyhelp[IX2D(j, i + 1, nlon + 2)] + gyhelp[IX2D(j - 1, i + 1, nlon + 2)]) * 0.5;
+        auto gy = (gyhelp[IX2D(j, i + 1, nlon + 2)] + gyhelp[IX2D(j - 1, i + 1, nlon + 2)]) * 0.5;
 
-        /* printf("%d %d %g %g %g %g \n", j, i, gx, gy, grid3x[IX2D(j,i,nlon)],
-         * grid3y[IX2D(j,i,nlon)]); */
+        // printf("%d %d %g %g %g %g \n", j, i, gx, gy, grid3x[IX2D(j,i,nlon)], grid3y[IX2D(j,i,nlon)]);
 
-        gx2 = (gx + grid3x[IX2D(j, i, nlon)]) * 0.5;
+        auto gx2 = (gx + grid3x[IX2D(j, i, nlon)]) * 0.5;
         if ((gx > 340 && grid3x[IX2D(j, i, nlon)] < 20) || (gx < 20 && grid3x[IX2D(j, i, nlon)] > 340))
           {
             if (gx2 < 180)
@@ -237,13 +228,12 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
               gx2 -= 180;
           }
 
-        gy2 = (gy + grid3y[IX2D(j, i, nlon)]) * 0.5;
+        auto gy2 = (gy + grid3y[IX2D(j, i, nlon)]) * 0.5;
 
         grid3x[IX2D(j, i, nlon)] = gx2;
         grid3y[IX2D(j, i, nlon)] = gy2;
 
-        /* printf("%d %d %g %g %g %g \n", j, i, gx2, gy2,
-         * grid3x[IX2D(j,i,nlon)], grid3y[IX2D(j,i,nlon)]); */
+        // printf("%d %d %g %g %g %g \n", j, i, gx2, gy2, grid3x[IX2D(j,i,nlon)], grid3y[IX2D(j,i,nlon)]);
       }
 }
 
@@ -259,22 +249,22 @@ Mrotuvb(void *process)
   cdoInitialize(process);
 
   if (operatorArgc() == 1)
-    if (strcmp(operatorArgv()[0], "noint") == 0) gpint = false;
+    if (cdoOperatorArgv(0) == "noint") gpint = false;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
 
-  int nvars = vlistNvars(vlistID1);
+  auto nvars = vlistNvars(vlistID1);
   if (nvars > 1) cdoAbort("More than one variable found in %s", cdoGetStreamName(0));
   nvars = vlistNvars(vlistID2);
   if (nvars > 1) cdoAbort("More than one variable found in %s", cdoGetStreamName(1));
 
-  int gridID1 = vlistGrid(vlistID1, 0);
-  int gridID2 = vlistGrid(vlistID2, 0);
-  const size_t gridsize = gridInqSize(gridID1);
+  auto gridID1 = vlistGrid(vlistID1, 0);
+  auto gridID2 = vlistGrid(vlistID2, 0);
+  const auto gridsize = gridInqSize(gridID1);
   if (gpint && gridID1 == gridID2) cdoAbort("Input grids are the same, use parameter >noint< to disable interpolation!");
   if (!gpint && gridID1 != gridID2) cdoAbort("Input grids are not the same!");
   if (gridsize != gridInqSize(gridID2)) cdoAbort("Grids have different size!");
@@ -290,36 +280,26 @@ Mrotuvb(void *process)
 
   if (gridsize != gridInqSize(gridID2)) cdoAbort("Internal problem: gridsize changed!");
 
-  const size_t nlon = gridInqXsize(gridID1);
-  const size_t nlat = gridInqYsize(gridID1);
+  const auto nlon = gridInqXsize(gridID1);
+  const auto nlat = gridInqYsize(gridID1);
 
-  std::vector<double> grid1x(gridsize), grid1y(gridsize);
-  std::vector<double> grid2x(gridsize), grid2y(gridsize);
-  std::vector<double> grid3x(gridsize), grid3y(gridsize);
+  Varray<double> grid1x(gridsize), grid1y(gridsize);
+  Varray<double> grid2x(gridsize), grid2y(gridsize);
+  Varray<double> grid3x(gridsize), grid3y(gridsize);
 
   gridInqXvals(gridID1, grid1x.data());
   gridInqYvals(gridID1, grid1y.data());
 
-  /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridID1, units);
-    grid_to_degree(units, gridsize, grid1x.data(), "grid1 center lon");
-    gridInqYunits(gridID1, units);
-    grid_to_degree(units, gridsize, grid1y.data(), "grid1 center lat");
-  }
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID1, CDI_XAXIS, gridsize, grid1x.data(), "grid1 center lon");
+  cdo_grid_to_degree(gridID1, CDI_YAXIS, gridsize, grid1y.data(), "grid1 center lat");
 
   gridInqXvals(gridID2, grid2x.data());
   gridInqYvals(gridID2, grid2y.data());
 
-  /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridID2, units);
-    grid_to_degree(units, gridsize, grid2x.data(), "grid2 center lon");
-    gridInqYunits(gridID2, units);
-    grid_to_degree(units, gridsize, grid2y.data(), "grid2 center lat");
-  }
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID2, CDI_XAXIS, gridsize, grid2x.data(), "grid2 center lon");
+  cdo_grid_to_degree(gridID2, CDI_YAXIS, gridsize, grid2y.data(), "grid2 center lat");
 
   if (gpint)
     {
@@ -331,7 +311,7 @@ Mrotuvb(void *process)
       grid3y = grid1y;
     }
 
-  const int gridID3 = gridCreate(GRID_CURVILINEAR, gridsize);
+  const auto gridID3 = gridCreate(GRID_CURVILINEAR, gridsize);
   gridDefDatatype(gridID3, gridInqDatatype(gridID1));
   gridDefXsize(gridID3, nlon);
   gridDefYsize(gridID3, nlat);
@@ -344,34 +324,34 @@ Mrotuvb(void *process)
       grid3y[i] *= DEG2RAD;
     }
 
-  int vlistID3 = vlistCreate();
+  const auto vlistID3 = vlistCreate();
   vlistCopy(vlistID3, vlistID1);
   vlistCat(vlistID3, vlistID2);
 
-  int code1 = vlistInqVarCode(vlistID1, 0);
-  int code2 = vlistInqVarCode(vlistID2, 0);
+  const auto code1 = vlistInqVarCode(vlistID1, 0);
+  const auto code2 = vlistInqVarCode(vlistID2, 0);
 
   if (code1 == code2) vlistDefVarCode(vlistID3, 1, code1 + 1);
 
   vlistChangeGrid(vlistID3, gridID1, gridID3);
   vlistChangeGrid(vlistID3, gridID2, gridID3);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
   if (Options::cdoVerbose) vlistPrint(vlistID3);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  double missval1 = vlistInqVarMissval(vlistID1, 0);
-  double missval2 = vlistInqVarMissval(vlistID2, 0);
+  const auto missval1 = vlistInqVarMissval(vlistID1, 0);
+  const auto missval2 = vlistInqVarMissval(vlistID2, 0);
 
-  std::vector<double> ufield(gridsize), vfield(gridsize);
-  std::vector<double> urfield(gridsize), vrfield(gridsize);
+  Varray<double> ufield(gridsize), vfield(gridsize);
+  Varray<double> urfield(gridsize), vrfield(gridsize);
 
-  std::vector<double> uhelp, vhelp;
+  Varray<double> uhelp, vhelp;
   if (gpint)
     {
       size_t gridsizex = (nlon + 2) * nlat;
@@ -398,7 +378,7 @@ Mrotuvb(void *process)
           cdoReadRecord(streamID1, ufield.data(), &nmiss1);
           cdoReadRecord(streamID2, vfield.data(), &nmiss2);
 
-          /* remove missing values */
+          // remove missing values
           if (nmiss1 || nmiss2)
             {
               for (size_t i = 0; i < gridsize; i++)
@@ -410,7 +390,7 @@ Mrotuvb(void *process)
 
           if (gpint)
             {
-              /* load to a help field */
+              // load to a help field
               for (size_t j = 0; j < nlat; j++)
                 for (size_t i = 0; i < nlon; i++)
                   {
@@ -418,7 +398,7 @@ Mrotuvb(void *process)
                     vhelp[IX2D(j, i + 1, nlon + 2)] = vfield[IX2D(j, i, nlon)];
                   }
 
-              /* make help field cyclic */
+              // make help field cyclic
               for (size_t j = 0; j < nlat; j++)
                 {
                   uhelp[IX2D(j, 0, nlon + 2)] = uhelp[IX2D(j, nlon, nlon + 2)];
@@ -427,7 +407,7 @@ Mrotuvb(void *process)
                   vhelp[IX2D(j, nlon + 1, nlon + 2)] = vhelp[IX2D(j, 1, nlon + 2)];
                 }
 
-              /* interpolate on pressure points */
+              // interpolate on pressure points
               for (size_t j = 1; j < nlat; j++)
                 for (size_t i = 0; i < nlon; i++)
                   {
@@ -442,11 +422,10 @@ Mrotuvb(void *process)
               vfield[IX2D(0, i, nlon)] = 0;
             }
 
-          /* rotate*/
-
+          // rotate
           rotate_uv2(ufield.data(), vfield.data(), nlon, nlat, grid3x.data(), grid3y.data(), urfield.data(), vrfield.data());
 
-          /* calc lat, lon, Auv and alpha */
+          // calc lat, lon, Auv and alpha
           /*
           {
           double lat, lon, auv, alpha;
@@ -455,7 +434,7 @@ Mrotuvb(void *process)
               {
                 lat = grid3y[IX2D(j,i,nlon)]*RAD2DEG;
                 lon = grid3x[IX2D(j,i,nlon)]*RAD2DEG;
-                auv = sqrt(urfield[IX2D(j,i,nlon)]*urfield[IX2D(j,i,nlon)] +
+                auv = std::sqrt(urfield[IX2D(j,i,nlon)]*urfield[IX2D(j,i,nlon)] +
                            vrfield[IX2D(j,i,nlon)]*vrfield[IX2D(j,i,nlon)]);
                 alpha = atan2(vrfield[IX2D(j,i,nlon)], urfield[IX2D(j,i,nlon)]);
                 alpha = 90. - alpha*RAD2DEG;
@@ -484,5 +463,5 @@ Mrotuvb(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
