@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,14 +16,13 @@
 */
 
 #include <algorithm>
-
-#include <limits.h>
+#include <climits>
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "libncl.h"
 #include "pmlist.h"
 
@@ -47,8 +46,8 @@ uv2dv_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
       double *tmp_v = v + k * gridsize_uv;
       double *tmp_div = div + k * gridsize_uv;
       // Init output array.
-      arrayFill(gridsize_uv, tmp_div, 0.0);
-// Call the Fortran routine.
+      varrayFill(gridsize_uv, tmp_div, 0.0);
+      // Call the Fortran routine.
 #ifdef HAVE_CF_INTERFACE
       DDVFIDF(tmp_u, tmp_v, lat, lon, inlon, inlat, missval, boundOpt, tmp_div, ierror);
 #else
@@ -77,8 +76,8 @@ uv2vr_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
       double *tmp_v = v + k * gridsize_uv;
       double *tmp_vort = vort + k * gridsize_uv;
       // Init output array.
-      arrayFill(gridsize_uv, tmp_vort, 0.0);
-// Call the Fortran routine.
+      varrayFill(gridsize_uv, tmp_vort, 0.0);
+      // Call the Fortran routine.
 #ifdef HAVE_CF_INTERFACE
       DVRFIDF(tmp_u, tmp_v, lat, lon, inlon, inlat, missval, boundOpt, tmp_vort, ierror);
 #else
@@ -92,11 +91,11 @@ find_name(int vlistID, char *name)
 {
   char varname[CDI_MAX_NAME];
 
-  int nvars = vlistNvars(vlistID);
+  const auto nvars = vlistNvars(vlistID);
   for (int varID = 0; varID < nvars; ++varID)
     {
       vlistInqVarName(vlistID, varID, varname);
-      if (cstrIsEqual(name, varname)) return varID;
+      if (cdo_cmpstr(name, varname)) return varID;
     }
 
   return CDI_UNDEFID;
@@ -127,10 +126,10 @@ set_parameter(void)
   strcpy(name_u, "u");
   strcpy(name_v, "v");
 
-  const int pargc = operatorArgc();
+  const auto pargc = operatorArgc();
   if (pargc)
     {
-      const char **pargv = (const char **) operatorArgv();
+      auto pargv = cdoGetOperArgv();
 
       KVList kvlist;
       kvlist.name = "PARAMETER";
@@ -139,30 +138,24 @@ set_parameter(void)
 
       for (const auto &kv : kvlist)
         {
-          const char *key = kv.key.c_str();
-          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv.values[0].c_str();
+          const auto &key = kv.key;
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          const auto &value = kv.values[0];
 
-          if (cstrIsEqual(key, "u"))
-            strcpy(name_u, value);
-          else if (cstrIsEqual(key, "v"))
-            strcpy(name_v, value);
-          else if (cstrIsEqual(key, "boundOpt"))
-            boundOpt = parameter2int(value);
-          else if (cstrIsEqual(key, "outMode"))
+          // clang-format off
+          if      (key == "u") strcpy(name_u, value.c_str());
+          else if (key == "v") strcpy(name_v, value.c_str());
+          else if (key == "boundOpt") boundOpt = parameter2int(value);
+          else if (key == "outMode")
             {
-              if (cstrIsEqual(value, "new"))
-                outMode = OutMode::NEW;
-              else if (cstrIsEqual(value, "append"))
-                outMode = OutMode::APPEND;
-              else if (cstrIsEqual(value, "replace"))
-                outMode = OutMode::REPLACE;
-              else
-                cdoAbort("Invalid parameter key value: outMode=%s (valid are: new/append/replace)", value);
+              if      (value == "new")     outMode = OutMode::NEW;
+              else if (value == "append")  outMode = OutMode::APPEND;
+              else if (value == "replace") outMode = OutMode::REPLACE;
+              else cdoAbort("Invalid parameter key value: outMode=%s (valid are: new/append/replace)", value.c_str());
             }
-          else
-            cdoAbort("Invalid parameter key >%s<!", key);
+          else cdoAbort("Invalid parameter key >%s<!", key.c_str());
+          // clang-format on
         }
     }
 
@@ -177,16 +170,16 @@ NCL_wind(void *process)
 
   cdoInitialize(process);
 
-  int UV2DV_CFD = cdoOperatorAdd("uv2dv_cfd", 0, 0, "[u, v, boundsOpt, outMode]");
-  int UV2VR_CFD = cdoOperatorAdd("uv2vr_cfd", 0, 0, "[u, v, boundsOpt, outMode]");
+  const auto UV2DV_CFD = cdoOperatorAdd("uv2dv_cfd", 0, 0, "[u, v, boundsOpt, outMode]");
+  const auto UV2VR_CFD = cdoOperatorAdd("uv2vr_cfd", 0, 0, "[u, v, boundsOpt, outMode]");
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   set_parameter();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = CDI_UNDEFID;
   if (outMode == OutMode::NEW)
     vlistID2 = vlistCreate();
@@ -195,16 +188,16 @@ NCL_wind(void *process)
   else
     cdoAbort("outMode=%d unsupported!", outMode);
 
-  int varIDu = find_name(vlistID1, name_u);
-  int varIDv = find_name(vlistID1, name_v);
+  const auto varIDu = find_name(vlistID1, name_u);
+  const auto varIDv = find_name(vlistID1, name_v);
 
   if (varIDu == CDI_UNDEFID) cdoAbort("%s not found!", name_u);
   if (varIDv == CDI_UNDEFID) cdoAbort("%s not found!", name_v);
 
-  int gridIDu = vlistInqVarGrid(vlistID1, varIDu);
-  int gridIDv = vlistInqVarGrid(vlistID1, varIDv);
-  int gridtype = gridInqType(gridIDu);
-  size_t gridsizeuv = gridInqSize(gridIDu);
+  const auto gridIDu = vlistInqVarGrid(vlistID1, varIDu);
+  const auto gridIDv = vlistInqVarGrid(vlistID1, varIDv);
+  const auto gridtype = gridInqType(gridIDu);
+  const auto gridsizeuv = gridInqSize(gridIDu);
 
   if (!((gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN) && gridtype == gridInqType(gridIDv)))
     cdoAbort("u and v must be on a regular lonlat or Gaussian grid!");
@@ -215,53 +208,51 @@ NCL_wind(void *process)
   if (Options::cdoVerbose) print_parameter();
   if (boundOpt < 0 || boundOpt > 3) cdoAbort("Parameter boundOpt=%d out of bounds (0-3)!", boundOpt);
 
-  size_t nlon = gridInqXsize(gridIDu);
-  size_t nlat = gridInqYsize(gridIDu);
+  const auto nlon = gridInqXsize(gridIDu);
+  const auto nlat = gridInqYsize(gridIDu);
 
-  int zaxisIDu = vlistInqVarZaxis(vlistID1, varIDu);
-  int nlev = zaxisInqSize(zaxisIDu);
+  const auto zaxisIDu = vlistInqVarZaxis(vlistID1, varIDu);
+  const auto nlev = zaxisInqSize(zaxisIDu);
 
   if (nlev != zaxisInqSize(vlistInqVarZaxis(vlistID1, varIDv))) cdoAbort("u and v must have the same number of level!");
 
-  double missvalu = vlistInqVarMissval(vlistID1, varIDu);
-  double missvalv = vlistInqVarMissval(vlistID1, varIDv);
+  const auto missvalu = vlistInqVarMissval(vlistID1, varIDu);
+  const auto missvalv = vlistInqVarMissval(vlistID1, varIDv);
 
-  int timetype = vlistInqVarTimetype(vlistID1, varIDu);
-  int varIDo = vlistDefVar(vlistID2, gridIDu, zaxisIDu, timetype);
+  const auto timetype = vlistInqVarTimetype(vlistID1, varIDu);
+  const auto varIDo = vlistDefVar(vlistID2, gridIDu, zaxisIDu, timetype);
   if (operatorID == UV2DV_CFD)
     {
-      vlistDefVarName(vlistID2, varIDo, "d");
-      vlistDefVarLongname(vlistID2, varIDo, "divergence");
-      vlistDefVarUnits(vlistID2, varIDo, "1/s");
+      cdiDefKeyString(vlistID2, varIDo, CDI_KEY_NAME, "d");
+      cdiDefKeyString(vlistID2, varIDo, CDI_KEY_LONGNAME, "divergence");
+      cdiDefKeyString(vlistID2, varIDo, CDI_KEY_UNITS, "1/s");
     }
   else if (operatorID == UV2VR_CFD)
     {
-      vlistDefVarName(vlistID2, varIDo, "vo");
-      vlistDefVarLongname(vlistID2, varIDo, "vorticity");
-      vlistDefVarUnits(vlistID2, varIDo, "1/s");
+      cdiDefKeyString(vlistID2, varIDo, CDI_KEY_NAME, "vo");
+      cdiDefKeyString(vlistID2, varIDo, CDI_KEY_LONGNAME, "vorticity");
+      cdiDefKeyString(vlistID2, varIDo, CDI_KEY_UNITS, "1/s");
     }
 
   vlistDefVarMissval(vlistID2, varIDo, missvalu);
 
-  std::vector<double> lon(nlon);
-  std::vector<double> lat(nlat);
-
+  Varray<double> lon(nlon), lat(nlat);
   gridInqXvals(gridIDu, &lon[0]);
   gridInqYvals(gridIDu, &lat[0]);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
-  std::vector<double> arrayu(nlev * gridsizeuv);
-  std::vector<double> arrayv(nlev * gridsizeuv);
-  std::vector<double> arrayo(nlev * gridsizeuv);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
+  Varray<double> arrayu(nlev * gridsizeuv);
+  Varray<double> arrayv(nlev * gridsizeuv);
+  Varray<double> arrayo(nlev * gridsizeuv);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -305,7 +296,7 @@ NCL_wind(void *process)
             {
               for (levelID = 0; levelID < nlev; ++levelID)
                 {
-                  double *parray = &arrayv[levelID * gridsizeuv];
+                  auto parray = &arrayv[levelID * gridsizeuv];
                   for (size_t i = 0; i < gridsizeuv; ++i)
                     if (DBL_IS_EQUAL(parray[i], missvalv)) parray[i] = missvalu;
                 }
@@ -319,8 +310,8 @@ NCL_wind(void *process)
 
       for (levelID = 0; levelID < nlev; ++levelID)
         {
-          double *parray = &arrayo[levelID * gridsizeuv];
-          size_t nmiss = arrayNumMV(gridsizeuv, parray, missvalu);
+          auto parray = &arrayo[levelID * gridsizeuv];
+          auto nmiss = arrayNumMV(gridsizeuv, parray, missvalu);
           cdoDefRecord(streamID2, varIDo, levelID);
           cdoWriteRecord(streamID2, parray, nmiss);
         }
@@ -335,5 +326,5 @@ NCL_wind(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
