@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,8 @@
 
 #include "cdi.h"
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "parse_literals.h"
 #include "pmlist.h"
 #include "util_string.h"
@@ -33,23 +34,23 @@ printValues(const int nvalues, const std::vector<std::string> &values)
   char fltstr[128];
   if (nvalues)
     {
-      const int dtype = literalsFindDatatype(nvalues, values);
+      const auto dtype = literalsFindDatatype(nvalues, values);
       for (int i = 0; i < nvalues; ++i)
         {
-          const char *value = values[i].c_str();
+          const auto &value = values[i];
           if (i) printf(", ");
           switch (dtype)
             {
-            case CDI_DATATYPE_INT8: printf("%db", literal_to_int(value)); break;
-            case CDI_DATATYPE_INT16: printf("%ds", literal_to_int(value)); break;
-            case CDI_DATATYPE_INT32: printf("%d", literal_to_int(value)); break;
+            case CDI_DATATYPE_INT8: printf("%db", literal2int(value)); break;
+            case CDI_DATATYPE_INT16: printf("%ds", literal2int(value)); break;
+            case CDI_DATATYPE_INT32: printf("%d", literal2int(value)); break;
             case CDI_DATATYPE_FLT32:
-              printf("%sf", doubleToAttstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), literal_to_double(value)));
+              printf("%sf", doubleToAttstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), literal2double(value)));
               break;
             case CDI_DATATYPE_FLT64:
-              printf("%s", doubleToAttstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), literal_to_double(value)));
+              printf("%s", doubleToAttstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), literal2double(value)));
               break;
-            default: printf("\"%s\"", value);
+            default: printf("\"%s\"", value.c_str());
             }
         }
     }
@@ -60,17 +61,17 @@ kvldump(const PMList &pmlist)
 {
   for (const auto &kvlist : pmlist)
     {
-      const char *listname = kvlist.name.c_str();
-      if (listname) printf("&%s\n", listname);
+      const auto &listname = kvlist.name;
+      if (!listname.empty()) printf("&%s\n", listname.c_str());
       for (const auto &kv : kvlist)
         {
-          const char *key = kv.key.c_str();
-          if (listname) printf("  ");
-          printf("%s = ", key);
+          const auto &key = kv.key;
+          if (!listname.empty()) printf("  ");
+          printf("%s = ", key.c_str());
           printValues(kv.nvalues, kv.values);
           printf("\n");
         }
-      if (listname) printf("/\n");
+      if (!listname.empty()) printf("/\n");
     }
 }
 
@@ -79,10 +80,12 @@ Nmldump(void *process)
 {
   cdoInitialize(process);
 
-  const int NMLDUMP = cdoOperatorAdd("nmldump", 0, 0, nullptr);
-  const int KVLDUMP = cdoOperatorAdd("kvldump", 0, 0, nullptr);
+  const auto NMLDUMP = cdoOperatorAdd("nmldump", 0, 0, nullptr);
+  const auto KVLDUMP = cdoOperatorAdd("kvldump", 0, 0, nullptr);
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
+
+  operatorCheckArgc(0);
 
   PMList pmlist;
   pmlist.readNamelist(stdin, "STDIN");
