@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,14 +28,16 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
 #include "gridreference.h"
 #include "printinfo.h"
+#include "cdo_zaxis.h"
 
 static void
-outputarr(int dig, size_t gridsize, std::vector<double> &array)
+outputarr(int dig, size_t gridsize, Varray<double> &array)
 {
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -44,11 +46,10 @@ outputarr(int dig, size_t gridsize, std::vector<double> &array)
 }
 
 static void
-outputsp(size_t gridsize, std::vector<double> &array, long ntr)
+outputsp(size_t gridsize, Varray<double> &array, long ntr)
 {
-  double minval, maxval;
-  arrayMinMax(gridsize, array.data(), minval, maxval);
-  if (/* T11 */ minval >= -1 && maxval <= 12)
+  const auto mm = varrayMinMax(gridsize, array);
+  if (/* T11 */ mm.min >= -1 && mm.max <= 12)
     {
       double *spc = array.data();
       for (long m = 0; m <= ntr; m++)
@@ -64,7 +65,7 @@ outputsp(size_t gridsize, std::vector<double> &array, long ntr)
 }
 
 static void
-output(size_t gridsize, std::vector<double> &array)
+output(size_t gridsize, Varray<double> &array)
 {
   int nout = 0;
   for (size_t i = 0; i < gridsize; i++)
@@ -81,8 +82,8 @@ output(size_t gridsize, std::vector<double> &array)
 }
 
 static void
-outputxyz(size_t gridsize, std::vector<double> &array, double missval, size_t nlon, size_t nlat, std::vector<double> &lon,
-          std::vector<double> &lat)
+outputxyz(size_t gridsize, Varray<double> &array, double missval, size_t nlon, size_t nlat, Varray<double> &lon,
+          Varray<double> &lat)
 {
   double fmin = 0;
   double x, y, z;
@@ -93,7 +94,7 @@ outputxyz(size_t gridsize, std::vector<double> &array, double missval, size_t nl
         fprintf(stdout, "%g\t%g\t%g\t%g\n", lon[i], lat[i], array[i], array[i]);
       }
   const char *fname = "frontplane.xyz";
-  FILE *fp = fopen(fname, "w");
+  auto fp = fopen(fname, "w");
   if (fp == nullptr) cdoAbort("Open failed on %s", fname);
   // first front plane
   double dx = (lon[1] - lon[0]);
@@ -159,7 +160,6 @@ void *
 Output(void *process)
 {
   int varID;
-  int gridID;
   int nrecs;
   int levelID;
   size_t nmiss;
@@ -168,9 +168,7 @@ Output(void *process)
   int index;
   const char *format = nullptr;
   char paramstr[32];
-  char vdatestr[32], vtimestr[32];
-  std::vector<double> grid_center_lon, grid_center_lat;
-  char name[CDI_MAX_NAME];
+  Varray<double> grid_center_lon, grid_center_lat;
   int year, month, day;
   std::vector<int> keys;
   int nkeys = 0, k;
@@ -184,22 +182,22 @@ Output(void *process)
 
   cdoInitialize(process);
 
-  int OUTPUT    = cdoOperatorAdd("output",    0, 1, nullptr);
-  int OUTPUTINT = cdoOperatorAdd("outputint", 0, 0, nullptr);
-  int OUTPUTSRV = cdoOperatorAdd("outputsrv", 0, 0, nullptr);
-  int OUTPUTEXT = cdoOperatorAdd("outputext", 0, 0, nullptr);
-  int OUTPUTF   = cdoOperatorAdd("outputf",   0, 0, nullptr);
-  int OUTPUTTS  = cdoOperatorAdd("outputts",  0, 0, nullptr);
-  int OUTPUTFLD = cdoOperatorAdd("outputfld", 0, 0, nullptr);
-  int OUTPUTARR = cdoOperatorAdd("outputarr", 0, 0, nullptr);
-  int OUTPUTXYZ = cdoOperatorAdd("outputxyz", 0, 0, nullptr);
-  int OUTPUTTAB = cdoOperatorAdd("outputtab", 0, 0, nullptr);
+  const auto OUTPUT    = cdoOperatorAdd("output",    0, 1, nullptr);
+  const auto OUTPUTINT = cdoOperatorAdd("outputint", 0, 0, nullptr);
+  const auto OUTPUTSRV = cdoOperatorAdd("outputsrv", 0, 0, nullptr);
+  const auto OUTPUTEXT = cdoOperatorAdd("outputext", 0, 0, nullptr);
+  const auto OUTPUTF   = cdoOperatorAdd("outputf",   0, 0, nullptr);
+  const auto OUTPUTTS  = cdoOperatorAdd("outputts",  0, 0, nullptr);
+  const auto OUTPUTFLD = cdoOperatorAdd("outputfld", 0, 0, nullptr);
+  const auto OUTPUTARR = cdoOperatorAdd("outputarr", 0, 0, nullptr);
+  const auto OUTPUTXYZ = cdoOperatorAdd("outputxyz", 0, 0, nullptr);
+  const auto OUTPUTTAB = cdoOperatorAdd("outputtab", 0, 0, nullptr);
   // clang-format on
 
-  UNUSED(OUTPUT);
+  (void) (OUTPUT);  // CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
-  bool opercplx = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const bool opercplx = cdoOperatorF2(operatorID);
 
   if (operatorID == OUTPUTF)
     {
@@ -207,8 +205,8 @@ Output(void *process)
 
       if (operatorArgc() < 1) cdoAbort("Too few arguments!");
 
-      format = operatorArgv()[0];
-      if (operatorArgc() == 2) nelem = parameter2int(operatorArgv()[1]);
+      format = cdoOperatorArgv(0).c_str();
+      if (operatorArgc() == 2) nelem = parameter2int(cdoOperatorArgv(1));
     }
   else if (operatorID == OUTPUTTAB)
     {
@@ -216,8 +214,8 @@ Output(void *process)
 
       operatorInputArg("keys to print");
 
-      int npar = operatorArgc();
-      char **parnames = operatorArgv();
+      const auto npar = operatorArgc();
+      auto parnames = cdoGetOperArgv();
 
       if (Options::cdoVerbose)
         for (int i = 0; i < npar; i++) cdoPrint("key %d = %s", i + 1, parnames[i]);
@@ -227,30 +225,31 @@ Output(void *process)
       nKeys = sizeof(Keynames) / sizeof(char *);
       for (int i = 0; i < npar; i++)
         {
+          const char *currentName = parnames[i].c_str();
           for (k = 0; k < nKeys; ++k)
             {
               //	      len = strlen(parnames[i]);
               len = strlen(Keynames[k]);
               if (len < 3) len = 3;
-              if (strncmp(parnames[i], Keynames[k], len) == 0)
+              if (strncmp(currentName, Keynames[k], len) == 0)
                 {
-                  int len2 = strlen(parnames[i]);
-                  if (len2 > len && parnames[i][len] != ':')
-                    cdoAbort("Key parameter >%s< contains invalid character at position %d!", parnames[i], len + 1);
+                  int len2 = strlen(currentName);
+                  if (len2 > len && currentName[len] != ':')
+                    cdoAbort("Key parameter >%s< contains invalid character at position %d!", currentName, len + 1);
 
                   if (k == knohead)
                     lhead = false;
                   else
                     {
                       keys[nkeys++] = k;
-                      if (len2 > len && parnames[i][len] == ':' && isdigit(parnames[i][len + 1]))
-                        Keylen[k] = atoi(&parnames[i][len + 1]);
+                      if (len2 > len && currentName[len] == ':' && isdigit(currentName[len + 1]))
+                        Keylen[k] = atoi(&currentName[len + 1]);
                     }
                   break;
                 }
             }
 
-          if (k == nKeys) cdoAbort("Key %s unsupported!", parnames[i]);
+          if (k == nKeys) cdoAbort("Key %s unsupported!", currentName);
         }
 
       if (Options::cdoVerbose)
@@ -269,70 +268,67 @@ Output(void *process)
           fprintf(stdout, "\n");
         }
     }
+  else
+    {
+      operatorCheckArgc(0);
+    }
 
   for (int indf = 0; indf < cdoStreamCnt(); indf++)
     {
-      CdoStreamID streamID = cdoOpenRead((indf));
+      const auto streamID = cdoOpenRead(indf);
+      const auto vlistID = cdoStreamInqVlist(streamID);
 
-      int vlistID = cdoStreamInqVlist(streamID);
+      VarList varList;
+      varListInit(varList, vlistID);
 
-      int ngrids = vlistNgrids(vlistID);
+      const auto ngrids = vlistNgrids(vlistID);
       int ndiffgrids = 0;
       for (index = 1; index < ngrids; index++)
         if (vlistGrid(vlistID, 0) != vlistGrid(vlistID, index)) ndiffgrids++;
 
       if (ndiffgrids > 0) cdoAbort("Too many different grids!");
 
-      gridID = vlistGrid(vlistID, 0);
-      int gridtype = gridInqType(gridID);
-      size_t gridsize = gridInqSize(gridID);
+      auto gridID0 = vlistGrid(vlistID, 0);
+      auto gridtype = gridInqType(gridID0);
+      auto gridsize = gridInqSize(gridID0);
       size_t nwpv = (vlistNumber(vlistID) == CDI_COMP) ? 2 : 1;
       if (nwpv == 2 && !opercplx) cdoAbort("Fields with complex numbers are not supported by this operator!");
-      size_t gridsizemax = nwpv * gridInqSize(gridID);
-      std::vector<double> array(gridsizemax);
+      auto gridsizemax = nwpv * gridInqSize(gridID0);
+      Varray<double> array(gridsizemax);
 
       if (operatorID == OUTPUTFLD || operatorID == OUTPUTXYZ || operatorID == OUTPUTTAB)
         {
-          if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
+          if (gridtype == GRID_GME) gridID0 = gridToUnstructured(gridID0, 0);
 
-          if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
+          if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID0 = gridToCurvilinear(gridID0, 0);
 
-          if (gridtype == GRID_UNSTRUCTURED)
+          if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID0))
             {
-              if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-                {
-                  if (gridInqNumber(gridID) > 0)
-                    {
-                      gridID = referenceToGrid(gridID);
-                      if (gridID == -1) cdoAbort("Reference to source grid not found!");
-                    }
-                }
+              const auto reference = dereferenceGrid(gridID0);
+              if (reference.isValid) gridID0 = reference.gridID;
+              if (reference.notFound) cdoAbort("Reference to source grid not found!");
             }
 
-          if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-            cdoAbort("Cell center coordinates missing!");
+          if (!gridHasCoordinates(gridID0)) cdoAbort("Cell center coordinates missing!");
 
           grid_center_lon.resize(gridsize);
           grid_center_lat.resize(gridsize);
-          gridInqXvals(gridID, grid_center_lon.data());
-          gridInqYvals(gridID, grid_center_lat.data());
+          gridInqXvals(gridID0, grid_center_lon.data());
+          gridInqYvals(gridID0, grid_center_lat.data());
 
           // Convert lat/lon units if required
-          char units[CDI_MAX_NAME];
-          gridInqXunits(gridID, units);
-          grid_to_degree(units, gridsize, grid_center_lon.data(), "grid center lon");
-          gridInqYunits(gridID, units);
-          grid_to_degree(units, gridsize, grid_center_lat.data(), "grid center lat");
+          cdo_grid_to_degree(gridID0, CDI_XAXIS, gridsize, grid_center_lon.data(), "grid center lon");
+          cdo_grid_to_degree(gridID0, CDI_YAXIS, gridsize, grid_center_lat.data(), "grid center lat");
         }
 
       int tsID = 0;
-      int taxisID = vlistInqTaxis(vlistID);
+      auto taxisID = vlistInqTaxis(vlistID);
       while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
         {
-          int64_t vdate = taxisInqVdate(taxisID);
-          int vtime = taxisInqVtime(taxisID);
-          date2str(vdate, vdatestr, sizeof(vdatestr));
-          time2str(vtime, vtimestr, sizeof(vtimestr));
+          const auto vdate = taxisInqVdate(taxisID);
+          const auto vtime = taxisInqVtime(taxisID);
+          const auto vdateString = dateToString(vdate);
+          const auto vtimeString = timeToString(vtime);
 
           cdiDecodeDate(vdate, &year, &month, &day);
 
@@ -340,21 +336,18 @@ Output(void *process)
             {
               cdoInqRecord(streamID, &varID, &levelID);
 
-              vlistInqVarName(vlistID, varID, name);
-              const int param = vlistInqVarParam(vlistID, varID);
-              const int code = vlistInqVarCode(vlistID, varID);
-              const int gridID = vlistInqVarGrid(vlistID, varID);
-              const int zaxisID = vlistInqVarZaxis(vlistID, varID);
-              const int datatype = vlistInqVarDatatype(vlistID, varID);
+              const auto code = varList[varID].code;
+              const auto gridID = varList[varID].gridID;
+              const auto zaxisID = varList[varID].zaxisID;
+              const auto datatype = varList[varID].datatype;
               const int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
-              const size_t nwpv = (vlistInqVarNumber(vlistID, varID) == CDI_COMP) ? 2 : 1;
-              const size_t gridsize = nwpv * gridInqSize(gridID);
-              size_t nlon = gridInqXsize(gridID);
-              size_t nlat = gridInqYsize(gridID);
-              const double level = cdoZaxisInqLevel(zaxisID, levelID);
-              const double missval = vlistInqVarMissval(vlistID, varID);
+              gridsize = varList[varID].nwpv * varList[varID].gridsize;
+              auto nlon = gridInqXsize(gridID);
+              auto nlat = gridInqYsize(gridID);
+              const auto level = cdoZaxisInqLevel(zaxisID, levelID);
+              const auto missval = varList[varID].missval;
 
-              cdiParamToString(param, paramstr, sizeof(paramstr));
+              cdiParamToString(varList[varID].param, paramstr, sizeof(paramstr));
 
               if (nlon * nlat != gridsize)
                 {
@@ -365,9 +358,9 @@ Output(void *process)
               cdoReadRecord(streamID, array.data(), &nmiss);
 
               if (operatorID == OUTPUTSRV)
-                fprintf(stdout, "%4d %8g %8ld %4d %8zu %8zu %d %d\n", code, level, (long)vdate, vtime, nlon, nlat, 0, 0);
+                fprintf(stdout, "%4d %8g %8ld %4d %8zu %8zu %d %d\n", code, level, (long) vdate, vtime, nlon, nlat, 0, 0);
 
-              if (operatorID == OUTPUTEXT) fprintf(stdout, "%8lld %4d %8g %8zu\n", vdate, code, level, gridsize);
+              if (operatorID == OUTPUTEXT) fprintf(stdout, "%8ld %4d %8g %8zu\n", (long) vdate, code, level, gridsize);
 
               if (operatorID == OUTPUTINT)
                 {
@@ -401,14 +394,9 @@ Output(void *process)
                 }
               else if (operatorID == OUTPUTTS)
                 {
-                  char vdatestr[32], vtimestr[32];
-
                   if (gridsize > 1) cdoAbort("operator works only with one gridpoint!");
 
-                  date2str(vdate, vdatestr, sizeof(vdatestr));
-                  time2str(vtime, vtimestr, sizeof(vtimestr));
-
-                  fprintf(stdout, "%s %s %.*g\n", vdatestr, vtimestr, dig, array[0]);
+                  fprintf(stdout, "%s %s %.*g\n", vdateString.c_str(), vtimeString.c_str(), dig, array[0]);
                 }
               else if (operatorID == OUTPUTFLD)
                 {
@@ -434,8 +422,8 @@ Output(void *process)
                           yind /= xsize;
                           xind -= yind * xsize;
                         }
-                      const double lon = grid_center_lon[i];
-                      const double lat = grid_center_lat[i];
+                      const auto lon = grid_center_lon[i];
+                      const auto lat = grid_center_lat[i];
 
                       for (k = 0; k < nkeys; ++k)
                         {
@@ -445,7 +433,7 @@ Output(void *process)
                             case kvalue: fprintf(stdout, "%*.*g ", len, dig, array[i]); break;
                             case kparam: fprintf(stdout, "%*s ", len, paramstr); break;
                             case kcode: fprintf(stdout, "%*d ", len, code); break;
-                            case kname: fprintf(stdout, "%*s ", len, name); break;
+                            case kname: fprintf(stdout, "%*s ", len, varList[varID].name); break;
                             case klon: fprintf(stdout, "%*g ", len, lon); break;
                             case klat: fprintf(stdout, "%*g ", len, lat); break;
                             case klev: fprintf(stdout, "%*g ", len, level); break;
@@ -453,8 +441,8 @@ Output(void *process)
                             case kxind: fprintf(stdout, "%*d ", len, xind + 1); break;
                             case kyind: fprintf(stdout, "%*d ", len, yind + 1); break;
                             case ktimestep: fprintf(stdout, "%*d ", len, tsID + 1); break;
-                            case kdate: fprintf(stdout, "%*s ", len, vdatestr); break;
-                            case ktime: fprintf(stdout, "%*s ", len, vtimestr); break;
+                            case kdate: fprintf(stdout, "%*s ", len, vdateString.c_str()); break;
+                            case ktime: fprintf(stdout, "%*s ", len, vtimeString.c_str()); break;
                             case kyear: fprintf(stdout, "%*d ", len, year); break;
                             case kmonth: fprintf(stdout, "%*d ", len, month); break;
                             case kday: fprintf(stdout, "%*d ", len, day); break;
