@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,18 +32,21 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "array.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
 #include "gridreference.h"
 #include "color.h"
 #include "printinfo.h"
 #include "interpol.h"
+#include "cdo_zaxis.h"
 
 static int
 check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
 {
-  int ncorner_new = ncorner;
+  auto ncorner_new = ncorner;
   int k;
 
   for (k = ncorner - 1; k > 0; --k)
@@ -55,30 +58,100 @@ check_ncorner(int ncorner, const double *lon_bounds, const double *lat_bounds)
 }
 
 static void
-make_cyclic(double *array1, double *array2, int nlon, int nlat)
+make_cyclic(double *array1, double *array2, long nlon, long nlat)
 {
-  int ij1, ij2;
-
-  for (int j = 0; j < nlat; ++j)
-    for (int i = 0; i < nlon; ++i)
+  for (long j = 0; j < nlat; ++j)
+    for (long i = 0; i < nlon; ++i)
       {
-        ij1 = j * nlon + i;
-        ij2 = j * (nlon + 1) + i;
+        const long ij1 = j * nlon + i;
+        const long ij2 = j * (nlon + 1) + i;
         array2[ij2] = array1[ij1];
       }
 
-  for (int j = 0; j < nlat; ++j)
+  for (long j = 0; j < nlat; ++j)
     {
-      ij2 = j * (nlon + 1);
+      const long ij2 = j * (nlon + 1);
       array2[ij2 + nlon] = array2[ij2];
     }
 }
 
 static void
-output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval, CPT *cpt)
+output_zon(double levmin, double levmax, const double *cell_corner_lat)
 {
-  double minval, maxval, meanval;
-  arrayMinMaxMeanMV(ngp, array, missval, minval, maxval, meanval);
+  double xlev[4], xlat[4];
+  auto latmin = cell_corner_lat[0];
+  auto latmax = cell_corner_lat[0];
+  for (int ic = 1; ic < 4; ic++) latmin = std::min(latmin, cell_corner_lat[ic]);
+  for (int ic = 1; ic < 4; ic++) latmax = std::max(latmax, cell_corner_lat[ic]);
+  xlev[0] = levmin;
+  xlev[1] = levmax;
+  xlev[2] = levmax;
+  xlev[3] = levmin;
+  xlat[0] = latmin;
+  xlat[1] = latmin;
+  xlat[2] = latmax;
+  xlat[3] = latmax;
+  for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlat[ic], xlev[ic]);
+  fprintf(stdout, "   %g  %g\n", xlat[0], xlev[0]);
+}
+
+static void
+output_mer(double levmin, double levmax, const double *cell_corner_lon)
+{
+  double xlev[4], xlon[4];
+  auto lonmin = cell_corner_lon[0];
+  auto lonmax = cell_corner_lon[0];
+  for (int ic = 1; ic < 4; ic++) lonmin = std::min(lonmin, cell_corner_lon[ic]);
+  for (int ic = 1; ic < 4; ic++) lonmax = std::max(lonmax, cell_corner_lon[ic]);
+  xlev[0] = levmin;
+  xlev[1] = levmin;
+  xlev[2] = levmax;
+  xlev[3] = levmax;
+  xlon[0] = lonmin;
+  xlon[1] = lonmax;
+  xlon[2] = lonmax;
+  xlon[3] = lonmin;
+  for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlon[ic], xlev[ic]);
+  fprintf(stdout, "   %g  %g\n", xlon[0], xlev[0]);
+}
+
+static void
+output_boundscpt(double value, double missval, const CPT &cpt)
+{
+  int r = 0, g = 0, b = 0, n;
+
+  if (!DBL_IS_EQUAL(value, missval))
+    {
+      for (n = 0; n < cpt.ncolors; n++)
+        if (value > cpt.lut[n].z_low && value <= cpt.lut[n].z_high) break;
+
+      if (n == cpt.ncolors)
+        {
+          r = cpt.bfn[0].rgb[0];
+          g = cpt.bfn[0].rgb[1];
+          b = cpt.bfn[0].rgb[2];
+        }
+      else
+        {
+          r = cpt.lut[n].rgb_high[0];
+          g = cpt.lut[n].rgb_high[1];
+          b = cpt.lut[n].rgb_high[2];
+        }
+    }
+  else
+    {
+      r = cpt.bfn[2].rgb[0];
+      g = cpt.bfn[2].rgb[1];
+      b = cpt.bfn[2].rgb[2];
+    }
+
+  fprintf(stdout, " -G%d/%d/%d", r, g, b);
+}
+                 
+static void
+output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double missval, const CPT &cpt)
+{
+  const auto mm = varrayMinMaxMV(ngp, array, missval);
 
   double dx = 10. / nlon;
 
@@ -113,44 +186,43 @@ output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval,
   printf("      geometry ElevationGrid {\n");
   printf("        colorPerVertex true\n");
   printf("        solid false\n");
-  printf("        xDimension %d\n", nlon);
-  printf("        zDimension %d\n", nlat);
+  printf("        xDimension %ld\n", nlon);
+  printf("        zDimension %ld\n", nlat);
   printf("        xSpacing %g\n", dx);
   printf("        zSpacing %g\n", dx);
   printf("        color Color {\n");
   printf("          color [\n");
-  for (int j = nlat - 1; j >= 0; --j)
-    for (int i = 0; i < nlon; ++i)
+  for (long j = nlat - 1; j >= 0; --j)
+    for (long i = 0; i < nlon; ++i)
       {
         int r = 0, g = 0, b = 0;
-        double val = array[j * nlon + i];
+        const auto val = array[j * nlon + i];
 
         if (!DBL_IS_EQUAL(val, missval))
           {
             int n;
-            for (n = 0; n < cpt->ncolors; n++)
-              if (val > cpt->lut[n].z_low && val <= cpt->lut[n].z_high) break;
+            for (n = 0; n < cpt.ncolors; n++)
+              if (val > cpt.lut[n].z_low && val <= cpt.lut[n].z_high) break;
 
-            if (n == cpt->ncolors)
+            if (n == cpt.ncolors)
               {
-                r = cpt->bfn[0].rgb[0];
-                g = cpt->bfn[0].rgb[1];
-                b = cpt->bfn[0].rgb[2];
+                r = cpt.bfn[0].rgb[0];
+                g = cpt.bfn[0].rgb[1];
+                b = cpt.bfn[0].rgb[2];
               }
             else
               {
-                //  r = cpt->lut[n].rgb_high[0];  g = cpt->lut[n].rgb_high[1];
-                //  b = cpt->lut[n].rgb_high[2];
-                r = intlin(val, cpt->lut[n].rgb_low[0], cpt->lut[n].z_low, cpt->lut[n].rgb_high[0], cpt->lut[n].z_high);
-                g = intlin(val, cpt->lut[n].rgb_low[1], cpt->lut[n].z_low, cpt->lut[n].rgb_high[1], cpt->lut[n].z_high);
-                b = intlin(val, cpt->lut[n].rgb_low[2], cpt->lut[n].z_low, cpt->lut[n].rgb_high[2], cpt->lut[n].z_high);
+                //  r = cpt.lut[n].rgb_high[0];  g = cpt.lut[n].rgb_high[1];  b = cpt.lut[n].rgb_high[2];
+                r = intlin(val, cpt.lut[n].rgb_low[0], cpt.lut[n].z_low, cpt.lut[n].rgb_high[0], cpt.lut[n].z_high);
+                g = intlin(val, cpt.lut[n].rgb_low[1], cpt.lut[n].z_low, cpt.lut[n].rgb_high[1], cpt.lut[n].z_high);
+                b = intlin(val, cpt.lut[n].rgb_low[2], cpt.lut[n].z_low, cpt.lut[n].rgb_high[2], cpt.lut[n].z_high);
               }
           }
         else
           {
-            r = cpt->bfn[2].rgb[0];
-            g = cpt->bfn[2].rgb[1];
-            b = cpt->bfn[2].rgb[2];
+            r = cpt.bfn[2].rgb[0];
+            g = cpt.bfn[2].rgb[1];
+            b = cpt.bfn[2].rgb[2];
           }
         printf(" %.3g %.3g %.3g,\n", r / 255., g / 255., b / 255.);
       }
@@ -158,8 +230,8 @@ output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval,
   printf("        }\n");
   printf("        height [\n");
 
-  for (int j = nlat - 1; j >= 0; --j)
-    for (int i = 0; i < nlon; ++i) printf("%g,\n", array[j * nlon + i]);
+  for (long j = nlat - 1; j >= 0; --j)
+    for (long i = 0; i < nlon; ++i) printf("%g,\n", array[j * nlon + i]);
 
   printf("        ]\n");
   printf("      }\n");
@@ -167,10 +239,33 @@ output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval,
   printf("  ]\n");
   printf("  translation -5 0 %g\n", -5. * nlat / nlon);
   printf("  rotation 0.0 0.0 0.0 0.0\n");
-  printf("  scale 1.0 %g 1.0\n", 0.5 / (maxval - minval));
+  printf("  scale 1.0 %g 1.0\n", 0.5 / (mm.max - mm.min));
   printf("}\n");
 }
 
+static void
+output_vector(long nlon, long nlat, int ninc, const Varray<double> &lon, const Varray<double> &lat, const Varray<double> &uf, const Varray<double> &vf)
+{
+  for (long j = 0; j < nlat; j += ninc)
+    for (long i = 0; i < nlon; i += ninc)
+      {
+        const auto idx = j * nlon + i;
+
+        // compute length of velocity vector
+        const auto auv = std::sqrt(uf[idx] * uf[idx] + vf[idx] * vf[idx]);
+
+        auto alpha = atan2(vf[idx], uf[idx]);
+        alpha = 90. - alpha * RAD2DEG;
+
+        if (alpha < 0) alpha += 360;
+        if (alpha > 360) alpha -= 360;
+
+        if (std::fabs(auv) > 0) fprintf(stdout, " %g  %g  %g  %g\n", lon[idx], lat[idx], alpha, auv);
+      }
+
+  fprintf(stdout, "#\n");
+}
+
 void *
 Outputgmt(void *process)
 {
@@ -181,35 +276,33 @@ Outputgmt(void *process)
   size_t nmiss;
   int ninc = 1;
   bool lzon = false, lmer = false, lhov = false;
-  bool lgrid_gen_bounds = false, luse_grid_corner = false;
+  bool luse_grid_corner = false;
   char varname[CDI_MAX_NAME];
-  std::vector<double> grid_center_lat2, grid_center_lon2;
-  std::vector<double> grid_corner_lat, grid_corner_lon;
+  Varray<double> grid_center_lat2, grid_center_lon2;
+  Varray<double> grid_corner_lat, grid_corner_lon;
   std::vector<int> grid_mask;
   CPT cpt;
-  char units[CDI_MAX_NAME];
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
   // clang-format off
-  const int OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
-  const int OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
-  const int OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
-  const int OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
-  const int OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
-  const int OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
-  const int OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
-  const int OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
+  const auto OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
+  const auto OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
+  const auto OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
+  const auto OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
+  const auto OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
+  const auto OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
+  const auto OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
+  const auto OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == OUTPUTVECTOR)
     {
       operatorInputArg("increment");
       operatorCheckArgc(1);
-      ninc = parameter2int(operatorArgv()[0]);
+      ninc = parameter2int(cdoOperatorArgv(0));
       if (ninc < 1) cdoAbort("Increment must be greater than 0!");
     }
 
@@ -218,32 +311,32 @@ Outputgmt(void *process)
   if (operatorID == OUTPUTCENTERCPT || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTVRML)
     {
       operatorCheckArgc(1);
-      char *cpt_file = operatorArgv()[0];
+      auto cpt_file = cdoOperatorArgv(0).c_str();
 
-      FILE *cpt_fp = fopen(cpt_file, "r");
+      auto cpt_fp = fopen(cpt_file, "r");
       if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
 
-      const int status = cptRead(cpt_fp, &cpt);
+      const auto status = cptRead(cpt_fp, &cpt);
       if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
 
       if (Options::cdoVerbose) cptWrite(stderr, cpt);
     }
 
-  CdoStreamID streamID = cdoOpenRead(0);
+  const auto streamID = cdoOpenRead(0);
 
-  const int vlistID = cdoStreamInqVlist(streamID);
-  const int taxisID = vlistInqTaxis(vlistID);
+  const auto vlistID = cdoStreamInqVlist(streamID);
+  const auto taxisID = vlistInqTaxis(vlistID);
 
   int varID = 0;
   vlistInqVarName(vlistID, varID, varname);
-  const int code = vlistInqVarCode(vlistID, varID);
-  int gridID = vlistInqVarGrid(vlistID, varID);
-  const int zaxisID = vlistInqVarZaxis(vlistID, varID);
-  const double missval = vlistInqVarMissval(vlistID, varID);
+  const auto code = vlistInqVarCode(vlistID, varID);
+  auto gridID = vlistInqVarGrid(vlistID, varID);
+  const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
+  const auto missval = vlistInqVarMissval(vlistID, varID);
 
-  const size_t gridsize = gridInqSize(gridID);
+  const auto gridsize = gridInqSize(gridID);
 
-  int gridtype = gridInqType(gridID);
+  auto gridtype = gridInqType(gridID);
   if (gridtype == GRID_GME || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       gridID = gridToUnstructured(gridID, 1);
@@ -254,26 +347,20 @@ Outputgmt(void *process)
     {
       gridID = gridToCurvilinear(gridID, 1);
       gridtype = gridInqType(gridID);
-      lgrid_gen_bounds = true;
     }
 
-  if (gridtype == GRID_UNSTRUCTURED)
+  if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID))
     {
-      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-        {
-          if (gridInqNumber(gridID) > 0)
-            {
-              gridID = referenceToGrid(gridID);
-              if (gridID == -1) cdoAbort("Reference to source grid not found!");
-            }
-        }
+      const auto reference = dereferenceGrid(gridID);
+      if (reference.isValid) gridID = reference.gridID;
+      if (reference.notFound) cdoAbort("Reference to source grid not found!");
     }
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
+  if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
 
-  int nlon = gridInqXsize(gridID);
-  int nlat = gridInqYsize(gridID);
-  int nlev = zaxisInqSize(zaxisID);
+  long nlon = gridInqXsize(gridID);
+  long nlat = gridInqYsize(gridID);
+  long nlev = zaxisInqSize(zaxisID);
 
   if (gridInqMaskGME(gridID, nullptr))
     {
@@ -311,17 +398,15 @@ Outputgmt(void *process)
 
   const bool grid_is_circular = gridIsCircular(gridID);
 
-  std::vector<double> grid_center_lat(gridsize), grid_center_lon(gridsize);
+  Varray<double> grid_center_lat(gridsize), grid_center_lon(gridsize);
   gridInqYvals(gridID, grid_center_lat.data());
   gridInqXvals(gridID, grid_center_lon.data());
 
-  /* Convert lat/lon units if required */
-  gridInqXunits(gridID, units);
-  grid_to_degree(units, gridsize, grid_center_lon.data(), "grid center lon");
-  gridInqYunits(gridID, units);
-  grid_to_degree(units, gridsize, grid_center_lat.data(), "grid center lat");
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridID, CDI_XAXIS, gridsize, grid_center_lon.data(), "grid center lon");
+  cdo_grid_to_degree(gridID, CDI_YAXIS, gridsize, grid_center_lat.data(), "grid center lat");
 
-  int nvals = gridsize;
+  long nvals = gridsize;
   double *plon = grid_center_lon.data();
   double *plat = grid_center_lat.data();
 
@@ -335,9 +420,9 @@ Outputgmt(void *process)
       make_cyclic(grid_center_lat.data(), grid_center_lat2.data(), nlon, nlat);
       make_cyclic(grid_center_lon.data(), grid_center_lon2.data(), nlon, nlat);
 
-      for (int j = 0; j < nlat; ++j)
+      for (long j = 0; j < nlat; ++j)
         {
-          const int ij2 = j * (nlon + 1);
+          const long ij2 = j * (nlon + 1);
           grid_center_lon2[ij2 + nlon] += 360;
         }
 
@@ -346,9 +431,7 @@ Outputgmt(void *process)
       plat = grid_center_lat2.data();
     }
 
-  std::vector<double> zaxis_center_lev(nlev);
-  std::vector<double> zaxis_lower_lev(nlev);
-  std::vector<double> zaxis_upper_lev(nlev);
+  Varray<double> zaxis_center_lev(nlev), zaxis_lower_lev(nlev), zaxis_upper_lev(nlev);
 
   cdoZaxisInqLevels(zaxisID, zaxis_center_lev.data());
 
@@ -359,29 +442,13 @@ Outputgmt(void *process)
       grid_corner_lat.resize(nalloc);
       grid_corner_lon.resize(nalloc);
 
-      if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
-        {
-          gridInqYbounds(gridID, grid_corner_lat.data());
-          gridInqXbounds(gridID, grid_corner_lon.data());
-        }
-      else
-        {
-          if (lgrid_gen_bounds)
-            {
-              char xunitstr[CDI_MAX_NAME];
-              char yunitstr[CDI_MAX_NAME];
-              gridInqXunits(gridID, xunitstr);
-              gridInqYunits(gridID, yunitstr);
-              if (!lzon) grid_cell_center_to_bounds_X2D(xunitstr, nlon, nlat, grid_center_lon.data(), grid_corner_lon.data(), 0);
-              if (!lmer) grid_cell_center_to_bounds_Y2D(yunitstr, nlon, nlat, grid_center_lat.data(), grid_corner_lat.data());
-            }
-          else
-            cdoAbort("Cell corner coordinates missing!");
-        }
+      if (!gridHasBounds(gridID)) cdoAbort("Cell corner coordinates missing!");
 
-      /* Note: using units from latitude instead from bounds */
-      grid_to_degree(units, ncorner * gridsize, grid_corner_lon.data(), "grid corner lon");
-      grid_to_degree(units, ncorner * gridsize, grid_corner_lat.data(), "grid corner lat");
+      gridInqYbounds(gridID, grid_corner_lat.data());
+      gridInqXbounds(gridID, grid_corner_lon.data());
+
+      cdo_grid_to_degree(gridID, CDI_XAXIS, ncorner * gridsize, grid_corner_lon.data(), "grid corner lon");
+      cdo_grid_to_degree(gridID, CDI_YAXIS, ncorner * gridsize, grid_corner_lat.data(), "grid corner lat");
 
       if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
         {
@@ -402,38 +469,35 @@ Outputgmt(void *process)
         }
     }
 
-  std::vector<double> array(gridsize);
+  Varray<double> array(gridsize);
   double *parray = array.data();
 
-  std::vector<double> array2;
+  Varray<double> array2;
   if (operatorID == OUTPUTCENTER2 && grid_is_circular)
     {
       array2.resize(nlat * (nlon + 1));
       parray = array2.data();
     }
 
-  std::vector<double> uf, vf, alpha, auv;
+  Varray<double> uf, vf;
   if (operatorID == OUTPUTVECTOR)
     {
       uf.resize(gridsize);
       vf.resize(gridsize);
-      alpha.resize(gridsize);
-      auv.resize(gridsize);
     }
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
     {
-      const int64_t vdate = taxisInqVdate(taxisID);
-      const int vtime = taxisInqVtime(taxisID);
-
-      date2str(vdate, vdatestr, sizeof(vdatestr));
-      time2str(vtime, vtimestr, sizeof(vtimestr));
+      const auto vdate = taxisInqVdate(taxisID);
+      const auto vtime = taxisInqVtime(taxisID);
+      const auto vdateString = dateToString(vdate);
+      const auto vtimeString = timeToString(vtime);
 
       if (tsID == 0 && operatorID != OUTPUTTRI)
         {
           if (operatorID == OUTPUTVRML) printf("#VRML V2.0 utf8\n\n");
-#if defined(VERSION)
+#ifdef VERSION
           fprintf(stdout, "# Generated by CDO version %s\n", VERSION);
           fprintf(stdout, "#\n");
 #endif
@@ -450,8 +514,8 @@ Outputgmt(void *process)
           if (operatorID == OUTPUTVECTOR) fprintf(stdout, "# Increment = %d\n", ninc);
           fprintf(stdout, "#\n");
           fprintf(stdout, "# Stream = %s\n", cdoGetStreamName(0));
-          fprintf(stdout, "# Date   = %s\n", vdatestr);
-          fprintf(stdout, "# Time   = %s\n", vtimestr);
+          fprintf(stdout, "# Date   = %s\n", vdateString.c_str());
+          fprintf(stdout, "# Time   = %s\n", vtimeString.c_str());
           fprintf(stdout, "# Name   = %s\n", varname);
           fprintf(stdout, "# Code   = %d\n", code);
         }
@@ -469,7 +533,7 @@ Outputgmt(void *process)
 
           if (operatorID == OUTPUTCENTER2 && grid_is_circular) make_cyclic(array.data(), array2.data(), nlon, nlat);
 
-          const double level = zaxis_center_lev[levelID];
+          const auto level = zaxis_center_lev[levelID];
 
           if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level  = %g\n", level);
           if (lhov) fprintf(stdout, "# Timestep = %d\n", tsID + 1);
@@ -480,15 +544,14 @@ Outputgmt(void *process)
             {
               if (Options::cdoVerbose)
                 {
-                  double minval, maxval, meanval;
-                  arrayMinMaxMeanMV(gridsize, array.data(), missval, minval, maxval, meanval);
-                  const double range = maxval - minval;
-                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
+                  const auto mm = varrayMinMaxMV(gridsize, array, missval);
+                  const auto range = mm.max - mm.min;
+                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", mm.min, mm.max, range / 20);
                   fprintf(stderr, "pscontour -K -JQ0/10i -Rd -I -Cgmt.cpt data.gmt > gmtplot.ps\n");
                   fprintf(stderr, "pscoast -O -J -R -Dc -W -B40g20 >> gmtplot.ps\n");
                 }
 
-              for (int i = 0; i < nvals; i++)
+              for (long i = 0; i < nvals; i++)
                 {
                   if (grid_mask.size() && grid_mask[i] == 0) continue;
 
@@ -523,10 +586,10 @@ Outputgmt(void *process)
             {
               if (gridInqType(gridID) != GRID_CURVILINEAR) cdoAbort("Unsupported grid!");
 
-              const int mlon = nlon - 1;
+              const long mlon = nlon - 1;
               // if ( gridIsCircular(gridID) ) mlon = nlon;
-              for (int j = 0; j < nlat - 1; ++j)
-                for (int i = 0; i < mlon; ++i)
+              for (long j = 0; j < nlat - 1; ++j)
+                for (long i = 0; i < mlon; ++i)
                   {
                     int ip1 = i + 1;
                     if (i == nlon - 1) ip1 = 0;
@@ -544,43 +607,25 @@ Outputgmt(void *process)
             {
               if (nrecs < 2) cdoAbort("Too few fields!");
 
-              arrayCopy(gridsize, array.data(), uf.data());
+              varrayCopy(gridsize, array, uf);
               cdoInqRecord(streamID, &varID, &levelID);
               cdoReadRecord(streamID, vf.data(), &nmiss);
 
-              for (int j = 0; j < nlat; j += ninc)
-                for (int i = 0; i < nlon; i += ninc)
-                  {
-                    // compute length of velocity vector
-                    auv[IX2D(j, i, nlon)]
-                        = sqrt(uf[IX2D(j, i, nlon)] * uf[IX2D(j, i, nlon)] + vf[IX2D(j, i, nlon)] * vf[IX2D(j, i, nlon)]);
+              output_vector(nlon, nlat, ninc, grid_center_lon, grid_center_lat, uf, vf);
 
-                    alpha[IX2D(j, i, nlon)] = atan2(vf[IX2D(j, i, nlon)], uf[IX2D(j, i, nlon)]);
-                    alpha[IX2D(j, i, nlon)] = 90. - alpha[IX2D(j, i, nlon)] * RAD2DEG;
-
-                    if (alpha[IX2D(j, i, nlon)] < 0) alpha[IX2D(j, i, nlon)] += 360;
-                    if (alpha[IX2D(j, i, nlon)] > 360) alpha[IX2D(j, i, nlon)] -= 360;
-
-                    if (std::fabs(auv[IX2D(j, i, nlon)]) > 0)
-                      fprintf(stdout, " %g  %g  %g  %g\n", grid_center_lon[IX2D(j, i, nlon)], grid_center_lat[IX2D(j, i, nlon)],
-                              alpha[IX2D(j, i, nlon)], auv[IX2D(j, i, nlon)]);
-                  }
-
-              fprintf(stdout, "#\n");
               break;
             }
           else if (operatorID == OUTPUTVRML)
             {
-              output_vrml(nlon, nlat, gridsize, array.data(), missval, &cpt);
+              output_vrml(nlon, nlat, gridsize, array, missval, cpt);
             }
           else if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT)
             {
               if (Options::cdoVerbose)
                 {
-                  double minval, maxval, meanval;
-                  arrayMinMaxMeanMV(gridsize, array.data(), missval, minval, maxval, meanval);
-                  const double range = maxval - minval;
-                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
+                  const auto mm = varrayMinMaxMV(gridsize, array, missval);
+                  const auto range = mm.max - mm.min;
+                  fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", mm.min, mm.max, range / 20);
                   fprintf(stderr, "psxy -K -JQ0/10i -Rd -L -Cgmt.cpt -m data.gmt > gmtplot.ps\n");
                   // fprintf(stderr, "psxy -K -Jx0.028id -Rd -L -Cgmt.cpt -m
                   // data.gmt > gmtplot.ps\n"); fprintf(stderr, "psxy -K
@@ -600,83 +645,18 @@ Outputgmt(void *process)
 
                   if (operatorID == OUTPUTBOUNDSCPT)
                     {
-                      int r = 0, g = 0, b = 0, n;
-
-                      if (!DBL_IS_EQUAL(array[i], missval))
-                        {
-                          for (n = 0; n < cpt.ncolors; n++)
-                            if (array[i] > cpt.lut[n].z_low && array[i] <= cpt.lut[n].z_high) break;
-
-                          if (n == cpt.ncolors)
-                            {
-                              r = cpt.bfn[0].rgb[0];
-                              g = cpt.bfn[0].rgb[1];
-                              b = cpt.bfn[0].rgb[2];
-                            }
-                          else
-                            {
-                              r = cpt.lut[n].rgb_high[0];
-                              g = cpt.lut[n].rgb_high[1];
-                              b = cpt.lut[n].rgb_high[2];
-                            }
-                        }
-                      else
-                        {
-                          r = cpt.bfn[2].rgb[0];
-                          g = cpt.bfn[2].rgb[1];
-                          b = cpt.bfn[2].rgb[2];
-                        }
-
-                      fprintf(stdout, " -G%d/%d/%d", r, g, b);
+                      output_boundscpt(array[i], missval, cpt);
                     }
 
                   fprintf(stdout, "\n");
 
                   if (lzon)
                     {
-                      double xlev[4], xlat[4];
-                      double levmin = zaxis_lower_lev[levelID];
-                      double levmax = zaxis_upper_lev[levelID];
-                      double latmin = grid_corner_lat[i * 4];
-                      double latmax = grid_corner_lat[i * 4];
-                      for (int ic = 1; ic < 4; ic++)
-                        {
-                          if (grid_corner_lat[i * 4 + ic] < latmin) latmin = grid_corner_lat[i * 4 + ic];
-                          if (grid_corner_lat[i * 4 + ic] > latmax) latmax = grid_corner_lat[i * 4 + ic];
-                        }
-                      xlev[0] = levmin;
-                      xlev[1] = levmax;
-                      xlev[2] = levmax;
-                      xlev[3] = levmin;
-                      xlat[0] = latmin;
-                      xlat[1] = latmin;
-                      xlat[2] = latmax;
-                      xlat[3] = latmax;
-                      for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlat[ic], xlev[ic]);
-                      fprintf(stdout, "   %g  %g\n", xlat[0], xlev[0]);
+                      output_zon(zaxis_lower_lev[levelID], zaxis_upper_lev[levelID], &grid_corner_lat[i * 4]);
                     }
                   else if (lmer)
                     {
-                      double xlev[4], xlon[4];
-                      double levmin = zaxis_lower_lev[levelID];
-                      double levmax = zaxis_upper_lev[levelID];
-                      double lonmin = grid_corner_lon[i * 4];
-                      double lonmax = grid_corner_lon[i * 4];
-                      for (int ic = 1; ic < 4; ic++)
-                        {
-                          if (grid_corner_lon[i * 4 + ic] < lonmin) lonmin = grid_corner_lon[i * 4 + ic];
-                          if (grid_corner_lon[i * 4 + ic] > lonmax) lonmax = grid_corner_lon[i * 4 + ic];
-                        }
-                      xlev[0] = levmin;
-                      xlev[1] = levmin;
-                      xlev[2] = levmax;
-                      xlev[3] = levmax;
-                      xlon[0] = lonmin;
-                      xlon[1] = lonmax;
-                      xlon[2] = lonmax;
-                      xlon[3] = lonmin;
-                      for (int ic = 0; ic < 4; ic++) fprintf(stdout, "   %g  %g\n", xlon[ic], xlev[ic]);
-                      fprintf(stdout, "   %g  %g\n", xlon[0], xlev[0]);
+                      output_mer(zaxis_lower_lev[levelID], zaxis_upper_lev[levelID], &grid_corner_lon[i * 4]);
                     }
                   else if (lhov)
                     {
