@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,16 +25,18 @@
 #include <omp.h>
 #endif
 
-#include <limits.h>
+#include <climits>
 
 #include <cdi.h>
-#include "cdo_int.h"
-#include "datetime.h"
 
-#define NALLOC_INC 1024
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "cdo_default_values.h"
+
 
 static int
-get_type_values(int datatype, double &tmin, double &tmax)
+get_type_values(const int datatype, double &tmin, double &tmax)
 {
   int status = 0;
 
@@ -55,7 +57,7 @@ get_type_values(int datatype, double &tmin, double &tmax)
 }
 
 static int
-computeScaleAndOffset(int datatype, double fmin, double fmax, double &scale_factor, double &add_offset)
+computeScaleAndOffset(const int datatype, const double fmin, const double fmax, double &scale_factor, double &add_offset)
 {
   scale_factor = 1.0;
   add_offset = 0.0;
@@ -82,45 +84,46 @@ Pack(void *process)
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   VarList varList;
   varListInit(varList, vlistID1);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   FieldVector3D vars;
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      if ((size_t)tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
+      constexpr size_t NALLOC_INC = 1024;
+      if ((size_t) tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
 
       dtlist.taxisInqTimestep(taxisID1, tsID);
 
-      fieldsFromVlist(vlistID1, vars[tsID], FIELD_NONE);
+      fieldsFromVlist(vlistID1, vars[tsID]);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          const size_t gridsize = varList[varID].gridsize;
-          vars[tsID][varID][levelID].resize(gridsize);
-          size_t nmiss;
-          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
-          vars[tsID][varID][levelID].nmiss = nmiss;
+          auto &field = vars[tsID][varID][levelID];
+          field.init(varList[varID]);
+          cdoReadRecord(streamID1, field);
         }
 
       tsID++;
     }
 
-  int nts = tsID;
+  const auto nts = tsID;
 
   if (CdoDefault::DataType != CDI_UNDEFID)
     {
@@ -139,39 +142,37 @@ Pack(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      double fmin = 1.e300;
-      double fmax = -1.e300;
+      double fmin = 1.e300, fmax = -1.e300;
       size_t nmisspv = 0;
 
-      const int timetype = varList[varID].timetype;
-      const double missval1 = varList[varID].missval;
-      const size_t gridsize = varList[varID].gridsize;
-      const int nlevels = varList[varID].nlevels;
+      const auto timetype = varList[varID].timetype;
+      const auto missval1 = varList[varID].missval;
+      const auto gridsize = varList[varID].gridsize;
+      const auto nlevels = varList[varID].nlevels;
       for (levelID = 0; levelID < nlevels; levelID++)
         {
-          for (int tsID = 0; tsID < nts; tsID++)
+          for (int t = 0; t < nts; t++)
             {
-              double zmin, zmax;
-              if (tsID > 0 && timetype == TIME_CONSTANT) continue;
-              const double *array = vars[tsID][varID][levelID].vec.data();
-              const size_t nmiss = vars[tsID][varID][levelID].nmiss;
-              if (nmiss)
-                {
-                  nmisspv += nmiss;
-                  arrayMinMaxMV(gridsize, array, missval1, zmin, zmax);
-                }
-              else
-                {
-                  arrayMinMax(gridsize, array, zmin, zmax);
-                }
+              if (t > 0 && timetype == TIME_CONSTANT) continue;
 
-              if (zmin < fmin) fmin = zmin;
-              if (zmax > fmax) fmax = zmax;
+              auto &field = vars[t][varID][levelID];
+              const auto nmiss = field.nmiss;
+
+              if (nmiss) nmisspv += nmiss;
+
+              MinMax mm;
+              if (field.memType == MemType::Float)
+                mm = nmiss ? varrayMinMaxMV(gridsize, field.vec_f, (float)missval1) : varrayMinMax(gridsize, field.vec_f);
+              else
+                mm = nmiss ? varrayMinMaxMV(gridsize, field.vec_d, missval1) : varrayMinMax(gridsize, field.vec_d);
+
+              fmin = std::min(fmin, mm.min);
+              fmax = std::max(fmax, mm.max);
             }
         }
 
       vlistDefVarDatatype(vlistID2, varID, datatype);
-      const double missval2 = vlistInqVarMissval(vlistID2, varID);
+      const auto missval2 = vlistInqVarMissval(vlistID2, varID);
 
       if (nmisspv)
         {
@@ -183,52 +184,65 @@ Pack(void *process)
 
               for (levelID = 0; levelID < nlevels; levelID++)
                 {
-                  for (int tsID = 0; tsID < nts; tsID++)
+                  for (int t = 0; t < nts; t++)
                     {
-                      if (tsID > 0 && timetype == TIME_CONSTANT) continue;
-                      auto &array = vars[tsID][varID][levelID].vec;
-                      const size_t nmiss = vars[tsID][varID][levelID].nmiss;
-                      if (nmiss)
-                        for (size_t i = 0; i < gridsize; ++i)
-                          if (DBL_IS_EQUAL(array[i], missval1)) array[i] = missval2;
+                      if (t > 0 && timetype == TIME_CONSTANT) continue;
+
+                      auto &field = vars[t][varID][levelID];
+                      if (field.nmiss)
+                        {
+                          if (field.memType == MemType::Float)
+                            {
+                              auto &v = field.vec_f;
+                              for (size_t i = 0; i < gridsize; ++i)
+                                if (DBL_IS_EQUAL(v[i], missval1)) v[i] = missval2;
+                            }
+                          else
+                            {
+                              auto &v = field.vec_d;
+                              for (size_t i = 0; i < gridsize; ++i)
+                                if (DBL_IS_EQUAL(v[i], missval1)) v[i] = missval2;
+                            }
+                        }
                     }
                 }
             }
         }
 
-      // printf("fmin %g fmax %g missval %g\n", fmin, fmax,
-      // vlistInqVarMissval(vlistID2, varID));
+      // printf("fmin %g fmax %g missval %g\n", fmin, fmax, vlistInqVarMissval(vlistID2, varID));
       double sf, ao;
       if (!computeScaleAndOffset(datatype, fmin, fmax, sf, ao))
         {
+          if (varList[varID].memType == MemType::Float)
+            {
+              sf = (float)sf;
+              ao = (float)ao;
+            }
           // printf("sf = %g ao = %g \n", sf, ao);
           // printf("smin %g smax %g\n", (fmin - ao)/sf, (fmax -ao)/sf);
-
           vlistDefVarScalefactor(vlistID2, varID, sf);
           vlistDefVarAddoffset(vlistID2, varID, ao);
         }
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  for (int tsID = 0; tsID < nts; tsID++)
+  for (tsID = 0; tsID < nts; tsID++)
     {
       dtlist.taxisDefTimestep(taxisID2, tsID);
       cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
-          const int timetype = varList[varID].timetype;
-          if (tsID > 0 && timetype == TIME_CONSTANT) continue;
-          const int nlevels = varList[varID].nlevels;
-          for (levelID = 0; levelID < nlevels; levelID++)
+          if (tsID > 0 && varList[varID].timetype == TIME_CONSTANT) continue;
+          for (levelID = 0; levelID < varList[varID].nlevels; levelID++)
             {
-              if (!vars[tsID][varID][levelID].empty())
+              auto &field = vars[tsID][varID][levelID];
+              if (field.hasData())
                 {
-                  const size_t nmiss = vars[tsID][varID][levelID].nmiss;
                   cdoDefRecord(streamID2, varID, levelID);
-                  cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
+                  cdoWriteRecord(streamID2, field);
                 }
             }
         }
