@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_vlist.h"
+#include "process_int.h"
 #include "param_conversion.h"
 
 void *
@@ -32,18 +33,15 @@ Pardup(void *process)
 {
   int nrecs;
   int varID, varID2, levelID;
-  size_t offset;
   int nmul = 0;
   size_t nmiss;
-  int nlevel;
-  double *single;
 
   cdoInitialize(process);
 
-  int PARDUP = cdoOperatorAdd("pardup", 0, 0, nullptr);
-  int PARMUL = cdoOperatorAdd("parmul", 0, 0, nullptr);
+  const auto PARDUP = cdoOperatorAdd("pardup", 0, 0, nullptr);
+  const auto PARMUL = cdoOperatorAdd("parmul", 0, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == PARDUP)
     {
@@ -52,36 +50,39 @@ Pardup(void *process)
   else if (operatorID == PARMUL)
     {
       operatorInputArg("number of multiply");
-      nmul = parameter2int(operatorArgv()[0]);
+      nmul = parameter2int(cdoOperatorArgv(0));
     }
   else
     cdoAbort("operator not implemented!");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
+
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsize);
-  std::vector<std::vector<double>> vardata(nvars);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
+  Varray2D<double> vardata(nvars);
   std::vector<std::vector<size_t>> varnmiss(nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      vardata[varID].resize(gridsize * nlevel);
-      varnmiss[varID].resize(nlevel);
+      const auto gridsize = varList1[varID].gridsize;
+      const auto nlevels = varList1[varID].nlevels;
+      vardata[varID].resize(gridsize * nlevels);
+      varnmiss[varID].resize(nlevels);
     }
 
   for (int i = 1; i < nmul; i++)
@@ -91,7 +92,7 @@ Pardup(void *process)
         vlistDefVarParam(vlistID2, varID + nvars * i, cdiEncodeParam(-(varID + nvars * i + 1), 255, 255));
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -108,9 +109,9 @@ Pardup(void *process)
           recList[recID].levelID = levelID;
           recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          offset = gridsize * levelID;
-          single = &vardata[varID][offset];
+          const auto gridsize = varList1[varID].gridsize;
+          const auto offset = gridsize * levelID;
+          auto single = &vardata[varID][offset];
 
           cdoReadRecord(streamID1, single, &nmiss);
           varnmiss[varID][levelID] = nmiss;
@@ -123,9 +124,9 @@ Pardup(void *process)
             levelID = recList[recID].levelID;
             varID2 = varID + i * nvars;
 
-            gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-            offset = gridsize * levelID;
-            single = &vardata[varID][offset];
+            const auto gridsize = varList1[varID].gridsize;
+            const auto offset = gridsize * levelID;
+            auto single = &vardata[varID][offset];
             nmiss = varnmiss[varID][levelID];
 
             arrayCopy(gridsize, single, array.data());
