@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,8 +25,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "after_vertint.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "vertical_interp.h"
 #include "stdnametable.h"
 #include "util_string.h"
 #include "const.h"
@@ -45,37 +47,35 @@ Pressure(void *process)
   int psID = -1, lnpsID = -1;
   char paramstr[32];
   char varname[CDI_MAX_NAME];
-  double minval, maxval;
   double *pout = nullptr;
   gribcode_t gribcodes;
-  memset(&gribcodes, 0, sizeof(gribcode_t));
 
   cdoInitialize(process);
 
   // clang-format off
-  int PRESSURE_FL = cdoOperatorAdd("pressure_fl", 0, 0, nullptr);
-  int PRESSURE_HL = cdoOperatorAdd("pressure_hl", 0, 0, nullptr);
-  int DELTAP      = cdoOperatorAdd("deltap",      0, 0, nullptr);
+  const auto PRESSURE_FL = cdoOperatorAdd("pressure_fl", 0, 0, nullptr);
+  const auto PRESSURE_HL = cdoOperatorAdd("pressure_hl", 0, 0, nullptr);
+  const auto DELTAP      = cdoOperatorAdd("deltap",      0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
   int nhlev;
-  double *vct = vlist_read_vct(vlistID1, &zaxisIDh, &nvct, &nhlev, &nhlevf, &nhlevh);
+  Varray<double> vct;
+  vlist_read_vct(vlistID1, &zaxisIDh, &nvct, &nhlev, &nhlevf, &nhlevh, vct);
 
   bool l3Dvars = (zaxisIDh != -1 && gridsize > 0);
   if (!l3Dvars) cdoAbort("No 3D variable with hybrid sigma pressure coordinate found!");
 
-  std::vector<double> ps_prog(gridsize);
-  std::vector<double> deltap(gridsize * nhlevf);
-  std::vector<double> full_press(gridsize * nhlevf);
-  std::vector<double> half_press(gridsize * nhlevh);
+  Varray<double> ps_prog(gridsize);
+  Varray<double> deltap(gridsize * nhlevf), full_press(gridsize * nhlevf), half_press(gridsize * nhlevh);
 
   if (operatorID == PRESSURE_FL || operatorID == DELTAP)
     {
@@ -88,18 +88,18 @@ Pressure(void *process)
       zaxisIDp = zaxisCreate(ZAXIS_HYBRID_HALF, nhlevh);
     }
 
-  std::vector<double> level(nhlevh);
+  Varray<double> level(nhlevh);
   for (int l = 0; l < nhlevh; l++) level[l] = l + 1;
   zaxisDefLevels(zaxisIDp, level.data());
 
-  zaxisDefVct(zaxisIDp, 2 * nhlevh, vct);
+  zaxisDefVct(zaxisIDp, 2 * nhlevh, vct.data());
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   bool useTable = false;
   for (varID = 0; varID < nvars; varID++)
     {
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const auto tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
       if (tableNum > 0 && tableNum != 255)
         {
           useTable = true;
@@ -111,13 +111,13 @@ Pressure(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      int nlevel = zaxisInqSize(zaxisID);
-      int instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+      const auto nlevels = zaxisInqSize(zaxisID);
+      const auto instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
+      const auto tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
 
-      int code = vlistInqVarCode(vlistID1, varID);
-      int param = vlistInqVarParam(vlistID1, varID);
+      auto code = vlistInqVarCode(vlistID1, varID);
+      auto param = vlistInqVarParam(vlistID1, varID);
 
       cdiParamToString(param, paramstr, sizeof(paramstr));
 
@@ -164,36 +164,36 @@ Pressure(void *process)
           cstrToLowerCase(varname);
 
           //                       ECHAM                         ECMWF
-          if (cstrIsEqual(varname, "geosp") || cstrIsEqual(varname, "z"))
+          if (cdo_cmpstr(varname, "geosp") || cdo_cmpstr(varname, "z"))
             code = 129;
-          else if (cstrIsEqual(varname, "st") || cstrIsEqual(varname, "t"))
+          else if (cdo_cmpstr(varname, "st") || cdo_cmpstr(varname, "t"))
             code = 130;
-          else if (cstrIsEqual(varname, "aps") || cstrIsEqual(varname, "sp"))
+          else if (cdo_cmpstr(varname, "aps") || cdo_cmpstr(varname, "sp"))
             code = 134;
-          else if (cstrIsEqual(varname, "ps"))
+          else if (cdo_cmpstr(varname, "ps"))
             code = 134;
-          else if (cstrIsEqual(varname, "lsp") || cstrIsEqual(varname, "lnsp"))
+          else if (cdo_cmpstr(varname, "lsp") || cdo_cmpstr(varname, "lnsp"))
             code = 152;
-          /* else if (cstrIsEqual(varname, "geopoth") == 0 ) code = 156; */
+          /* else if (cdo_cmpstr(varname, "geopoth") == 0 ) code = 156; */
         }
 
       if (mode == ModelMode::ECHAM)
         {
-          if (code == gribcodes.ps && nlevel == 1)
+          if (code == gribcodes.ps && nlevels == 1)
             psID = varID;
-          else if (code == gribcodes.lsp && nlevel == 1)
+          else if (code == gribcodes.lsp && nlevels == 1)
             lnpsID = varID;
         }
       else if (mode == ModelMode::WMO || mode == ModelMode::HIRLAM)
         {
-          if (code == gribcodes.ps && nlevel == 1) psID = varID;
+          if (code == gribcodes.ps && nlevels == 1) psID = varID;
         }
     }
 
-  int pvarID = lnpsID;
+  auto pvarID = lnpsID;
   if (zaxisIDh != -1 && lnpsID != -1)
     {
-      int gridID = vlistInqVarGrid(vlistID1, lnpsID);
+      const auto gridID = vlistInqVarGrid(vlistID1, lnpsID);
       if (gridInqType(gridID) == GRID_SPECTRAL)
         {
           lnpsID = -1;
@@ -208,24 +208,24 @@ Pressure(void *process)
       if (psID == -1) cdoAbort("%s not found!", var_stdname(surface_air_pressure));
     }
 
-  int gridID = vlistInqVarGrid(vlistID1, pvarID);
+  const auto gridID = vlistInqVarGrid(vlistID1, pvarID);
   if (gridInqType(gridID) == GRID_SPECTRAL)
     cdoAbort("%s on spectral representation not supported!", var_stdname(surface_air_pressure));
 
-  std::vector<double> array(gridsize);
+  Varray<double> array(gridsize);
 
-  int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   varID = vlistDefVar(vlistID2, gridID, zaxisIDp, TIME_VARYING);
   vlistDefVarParam(vlistID2, varID, cdiEncodeParam(1, 255, 255));
-  vlistDefVarName(vlistID2, varID, "pressure");
-  vlistDefVarStdname(vlistID2, varID, "air_pressure");
-  vlistDefVarUnits(vlistID2, varID, "Pa");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "pressure");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, "air_pressure");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, "Pa");
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -248,15 +248,16 @@ Pressure(void *process)
       if (zaxisIDh != -1)
         {
           if (lnpsID != -1)
-            for (size_t i = 0; i < gridsize; i++) ps_prog[i] = exp(array[i]);
+            for (size_t i = 0; i < gridsize; i++) ps_prog[i] = std::exp(array[i]);
           else if (psID != -1)
-            arrayCopy(gridsize, array.data(), ps_prog.data());
+            varrayCopy(gridsize, array, ps_prog);
 
           // check range of ps_prog
-          arrayMinMax(gridsize, ps_prog.data(), minval, maxval);
-          if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
+          const auto mm = varrayMinMax(ps_prog);
+          if (mm.min < MIN_PS || mm.max > MAX_PS)
+            cdoWarning("Surface pressure out of range (min=%g max=%g)!", mm.min, mm.max);
 
-          presh(full_press.data(), half_press.data(), vct, ps_prog.data(), nhlevf, gridsize);
+          presh(full_press.data(), half_press.data(), vct.data(), ps_prog.data(), nhlevf, gridsize);
         }
 
       if (operatorID == PRESSURE_FL)
@@ -269,9 +270,7 @@ Pressure(void *process)
           nlevel = nhlevf;
           for (k = 0; k < nhlevf; ++k)
             for (size_t i = 0; i < gridsize; ++i)
-              {
-                deltap[k * gridsize + i] = half_press[(k + 1) * gridsize + i] - half_press[k * gridsize + i];
-              }
+              deltap[k * gridsize + i] = half_press[(k + 1) * gridsize + i] - half_press[k * gridsize + i];
 
           pout = deltap.data();
         }
@@ -285,7 +284,7 @@ Pressure(void *process)
       for (levelID = 0; levelID < nlevel; levelID++)
         {
           cdoDefRecord(streamID2, varID, levelID);
-          size_t offset = levelID * gridsize;
+          const auto offset = levelID * gridsize;
           cdoWriteRecord(streamID2, pout + offset, 0);
         }
 
@@ -295,8 +294,6 @@ Pressure(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (vct) Free(vct);
-
   cdoFinish();
 
   return nullptr;
