@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 
 void *
 Recttocomplex(void *process)
@@ -28,33 +28,36 @@ Recttocomplex(void *process)
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID2 = cdoOpenRead(1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const int vlistID3 = vlistDuplicate(vlistID1);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  const int nvars = vlistNvars(vlistID3);
+  const auto nvars = vlistNvars(vlistID3);
   for (varID = 0; varID < nvars; ++varID)
     {
-      int datatype = vlistInqVarDatatype(vlistID2, varID);
+      auto datatype = vlistInqVarDatatype(vlistID2, varID);
       datatype = (datatype == CDI_DATATYPE_FLT64) ? CDI_DATATYPE_CPX64 : CDI_DATATYPE_CPX32;
       vlistDefVarDatatype(vlistID3, varID, datatype);
     }
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
-  std::vector<double> array3(2 * gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax), array2(gridsizemax), array3(2 * gridsizemax);
+  
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -69,11 +72,10 @@ Recttocomplex(void *process)
 
           cdoInqRecord(streamID2, &varID, &levelID);
 
-          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-
           cdoReadRecord(streamID1, array1.data(), &nmiss);
           cdoReadRecord(streamID2, array2.data(), &nmiss);
 
+          const auto gridsize = varList1[varID].gridsize;
           for (size_t i = 0; i < gridsize; ++i)
             {
               array3[2 * i] = array1[i];
@@ -94,5 +96,5 @@ Recttocomplex(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
