@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,7 @@ enum
   REMAPDIS,
   REMAPNN,
   REMAPLAF,
-  REMAPSUM,
+  REMAPAVG,
   GENSCON,
   GENCON2,
   GENBIL,
@@ -102,9 +102,9 @@ getMaptype(int operfunc, RemapMethod &mapType, SubmapType &submapType, int &numN
       mapType = RemapMethod::CONSERV;
       submapType = SubmapType::LAF;
       break;
-    case REMAPSUM:
-      mapType = RemapMethod::CONSERV_SCRIP;
-      submapType = SubmapType::SUM;
+    case REMAPAVG:
+      mapType = RemapMethod::CONSERV;
+      submapType = SubmapType::AVG;
       break;
     case REMAPBIL:
     case GENBIL: mapType = RemapMethod::BILINEAR; break;
@@ -152,15 +152,16 @@ remapPrintInfo(int operfunc, bool remap_genweights, RemapGrid &src_grid, RemapGr
   line[0] = 0;
 
   // clang-format off
-  if      (operfunc == REMAPBIL || operfunc == GENBIL) strcpy(line, "Bilinear");
-  else if (operfunc == REMAPBIC || operfunc == GENBIC) strcpy(line, "Bicubic");
-  else if (operfunc == REMAPNN || operfunc == GENNN) strcpy(line, "Nearest neighbor");
-  else if (operfunc == REMAPDIS || operfunc == GENDIS) strcpy(line, "Distance-weighted average");
-  else if (operfunc == REMAPSCON || operfunc == GENSCON) strcpy(line, "SCRIP first order conservative");
-  else if (operfunc == REMAPCON2 || operfunc == GENCON2) strcpy(line, "SCRIP second order conservative");
-  else if (operfunc == REMAPLAF || operfunc == GENLAF) strcpy(line, "YAC largest area fraction");
-  else if (operfunc == REMAPCON || operfunc == GENCON) strcpy(line, "YAC first order conservative");
+  if      (operfunc == REMAPBIL   || operfunc == GENBIL)   strcpy(line, "Bilinear");
+  else if (operfunc == REMAPBIC   || operfunc == GENBIC)   strcpy(line, "Bicubic");
+  else if (operfunc == REMAPNN    || operfunc == GENNN)    strcpy(line, "Nearest neighbor");
+  else if (operfunc == REMAPDIS   || operfunc == GENDIS)   strcpy(line, "Distance-weighted average");
+  else if (operfunc == REMAPSCON  || operfunc == GENSCON)  strcpy(line, "SCRIP first order conservative");
+  else if (operfunc == REMAPCON2  || operfunc == GENCON2)  strcpy(line, "SCRIP second order conservative");
+  else if (operfunc == REMAPLAF   || operfunc == GENLAF)   strcpy(line, "YAC largest area fraction");
+  else if (operfunc == REMAPCON   || operfunc == GENCON)   strcpy(line, "YAC first order conservative");
   else if (operfunc == REMAPYCON2 || operfunc == GENYCON2) strcpy(line, "YAC second order conservative");
+  else if (operfunc == REMAPAVG)   strcpy(line, "Average");
   else strcpy(line, "Unknown");
   // clang-format on
 
@@ -193,10 +194,9 @@ remapPrintInfo(int operfunc, bool remap_genweights, RemapGrid &src_grid, RemapGr
 static void
 remapPrintWarning(const char *remap_file, int operfunc, RemapGrid &src_grid, size_t nmiss)
 {
-  char line[256];
-  char tmpstr[256];
-
+  char line[256], tmpstr[256];
   line[0] = 0;
+
   (void) operfunc;
 
   strcat(line, "Remap weights from ");
@@ -221,17 +221,17 @@ remapPrintWarning(const char *remap_file, int operfunc, RemapGrid &src_grid, siz
   cdoWarning(line);
 }
 
-double remap_threshhold = 2;
-double pointSearchRadius = 180;
+double remap_threshhold = 2.0;
+double pointSearchRadius = 180.0;
 int remap_test = 0;
 int remap_non_global = false;
 int remap_num_srch_bins = 180;
 static bool lremap_num_srch_bins = false;
 static bool remap_extrapolate = false;
 static bool lextrapolate = false;
-int max_remaps = -1;
+static int MaxRemaps = -1;
 int sort_mode = HEAP_SORT;
-double remap_frac_min = 0;
+double remap_frac_min = 0.0;
 
 static void
 remapGetenv(void)
@@ -244,8 +244,8 @@ remapGetenv(void)
       const auto ival = atoi(envstr);
       if (ival > 0)
         {
-          max_remaps = ival;
-          if (Options::cdoVerbose) cdoPrint("Set MAX_REMAPS to %d", max_remaps);
+          MaxRemaps = ival;
+          if (Options::cdoVerbose) cdoPrint("Set MAX_REMAPS to %d", MaxRemaps);
         }
     }
 
@@ -290,7 +290,7 @@ remapGetenv(void)
   if (envstr && *envstr)
     {
       const auto fval = atof(envstr);
-      if (fval > 0)
+      if (fval > 0.0)
         {
           remap_threshhold = fval;
           if (Options::cdoVerbose) cdoPrint("Set REMAP_THRESHHOLD to %g", remap_threshhold);
@@ -303,7 +303,7 @@ remapGetenv(void)
   if (envstr && *envstr)
     {
       const auto fval = radius_str_to_deg(envstr);
-      if (fval < 0 || fval > 180) cdoAbort("%s=%g out of bounds (0-180 deg)!", "CDO_REMAP_RADIUS", fval);
+      if (fval < 0.0 || fval > 180.0) cdoAbort("%s=%g out of bounds (0-180 deg)!", "CDO_REMAP_RADIUS", fval);
       pointSearchRadius = fval;
       if (Options::cdoVerbose) cdoPrint("Set CDO_REMAP_RADIUS to %g", pointSearchRadius);
     }
@@ -312,7 +312,7 @@ remapGetenv(void)
   if (envstr && *envstr)
     {
       const auto fval = radius_str_to_deg(envstr);
-      if (fval < 0 || fval > 180) cdoAbort("%s=%g out of bounds (0-180 deg)!", "CDO_GRIDSEARCH_RADIUS", fval);
+      if (fval < 0.0 || fval > 180.0) cdoAbort("%s=%g out of bounds (0-180 deg)!", "CDO_GRIDSEARCH_RADIUS", fval);
       pointSearchRadius = fval;
       if (Options::cdoVerbose) cdoPrint("Set CDO_GRIDSEARCH_RADIUS to %g", pointSearchRadius);
     }
@@ -323,7 +323,7 @@ remapGetenv(void)
   if (envstr && *envstr)
     {
       const auto fval = atof(envstr);
-      if (fval > 0)
+      if (fval > 0.0)
         {
           remap_frac_min = fval;
           if (Options::cdoVerbose) cdoPrint("Set REMAP_AREA_MIN to %g", remap_frac_min);
@@ -381,7 +381,7 @@ remapGetenv(void)
         Options::REMAP_genweights = false;
       else
         cdoWarning("Environment variable CDO_REMAP_GENWEIGHTS has wrong value!");
-      
+
       if (Options::cdoVerbose) cdoPrint("Generation of weights %s!", Options::REMAP_genweights ? "enabled" : "disabled");
     }
 }
@@ -393,9 +393,7 @@ gridIsGlobal(int gridID)
   const bool non_global = remap_non_global || !gridIsCircular(gridID);
 
   const auto gridtype = gridInqType(gridID);
-  const auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-  if ((projtype == CDI_PROJ_RLL) || (projtype == CDI_PROJ_LAEA) || (projtype == CDI_PROJ_SINU) || (projtype == CDI_PROJ_LCC)
-      || (projtype == CDI_PROJ_STERE) || (gridtype == GRID_LONLAT && non_global) || (gridtype == GRID_CURVILINEAR && non_global))
+  if (gridProjIsSupported(gridID) || (gridtype == GRID_LONLAT && non_global) || (gridtype == GRID_CURVILINEAR && non_global))
     global_grid = false;
 
   return global_grid;
@@ -405,10 +403,10 @@ static void
 scaleGridboxArea(size_t gridsize, const double *restrict array1, size_t gridsize2, double *restrict array2,
                  const double *restrict grid2_area)
 {
-  double array1sum = 0;
+  double array1sum = 0.0;
   for (size_t i = 0; i < gridsize; i++) array1sum += array1[i];
 
-  double array2sum = 0;
+  double array2sum = 0.0;
   for (size_t i = 0; i < gridsize2; i++) array2sum += grid2_area[i];
   for (size_t i = 0; i < gridsize2; i++) array2[i] = grid2_area[i] / array2sum * array1sum;
 
@@ -423,7 +421,7 @@ scaleGridboxArea(size_t gridsize, const double *restrict array1, size_t gridsize
 static int
 set_remapgrids(int filetype, int vlistID, int ngrids, std::vector<bool> &remapgrids)
 {
-  (void)filetype;
+  (void) filetype;
   int index;
   for (index = 0; index < ngrids; index++)
     {
@@ -431,12 +429,11 @@ set_remapgrids(int filetype, int vlistID, int ngrids, std::vector<bool> &remapgr
 
       const auto gridID = vlistGrid(vlistID, index);
       const auto gridtype = gridInqType(gridID);
-      const auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
       const bool lprojparams = (gridtype == GRID_PROJECTION) && grid_has_proj_params(gridID);
 
-      if (gridtype != GRID_LONLAT && !lprojparams && projtype != CDI_PROJ_RLL && projtype != CDI_PROJ_LAEA
-          && projtype != CDI_PROJ_SINU && projtype != CDI_PROJ_LCC && projtype != CDI_PROJ_STERE && gridtype != GRID_GAUSSIAN
-          && gridtype != GRID_GME && gridtype != GRID_CURVILINEAR && gridtype != GRID_UNSTRUCTURED && gridtype != GRID_GAUSSIAN_REDUCED)
+      if (!gridProjIsSupported(gridID) && !lprojparams && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN
+          && gridtype != GRID_GME && gridtype != GRID_CURVILINEAR && gridtype != GRID_UNSTRUCTURED
+          && gridtype != GRID_GAUSSIAN_REDUCED)
         {
           /*
           if (gridtype == GRID_GAUSSIAN_REDUCED)
@@ -504,7 +501,7 @@ getNormOpt(void)
 {
   NormOpt normOpt(NormOpt::FRACAREA);
 
-  char *envstr = getenv("CDO_REMAP_NORMALIZE_OPT");  // obsolate
+  char *envstr = getenv("CDO_REMAP_NORMALIZE_OPT");  // obsolete
   if (envstr && *envstr)
     {
       if (memcmp(envstr, "frac", 4) == 0)
@@ -546,7 +543,7 @@ getNormOpt(void)
 static void
 remapNormalizeField(NormOpt normOpt, size_t gridsize, double *array, double missval, RemapGrid *tgt_grid)
 {
-  // used only to check the result of remapcon
+  // used to check the result of remapcon
 
   if (normOpt == NormOpt::NONE)
     {
@@ -581,13 +578,10 @@ remapNormalizeField(NormOpt normOpt, size_t gridsize, double *array, double miss
 static void
 remapSetFracMin(size_t gridsize, double *array, double missval, RemapGrid *tgt_grid)
 {
-  if (remap_frac_min > 0)
+  if (remap_frac_min > 0.0)
     {
       for (size_t i = 0; i < gridsize; i++)
-        {
-          // printf("%zu %g %g\n", i, remaps[r].tgt_grid.cell_frac[i], remaps[r].tgt_grid.cell_area[i]);
-          if (tgt_grid->cell_frac[i] < remap_frac_min) array[i] = missval;
-        }
+        if (tgt_grid->cell_frac[i] < remap_frac_min) array[i] = missval;
     }
 }
 
@@ -710,20 +704,8 @@ remapGenNumBins(int ysize)
   return num_srch_bins;
 }
 
-static void
-remapSum(RemapGrid *remapGrid, size_t gridsize, double *array, const char *tag)
-{
-  for (size_t i = 0; i < gridsize; i++)
-    printf("1 %zd %g %g %g %g\n", i, array[i], remapGrid->cell_frac[i], remapGrid->cell_area[i], remapGrid->cell_frac[i]);
-
-  double sum = 0;
-  for (size_t i = 0; i < gridsize; i++) sum += remapGrid->cell_area[i];
-
-  printf("%s array sum %g\n", tag, sum);
-}
-
 void
-remapInit(remapType &remap)
+remapInit(RemapType &remap)
 {
   remap.nused = 0;
   remap.gridID = -1;
@@ -732,28 +714,28 @@ remapInit(remapType &remap)
 }
 
 static void
-remapGenWeights(RemapMethod mapType, remapType *remap, int numNeighbors)
+remapGenWeights(RemapMethod mapType, RemapType &remap, int numNeighbors)
 {
   // clang-format off
-  if      (mapType == RemapMethod::CONSERV_SCRIP) remapConservWeightsScrip(remap->search, remap->vars);
-  else if (mapType == RemapMethod::BILINEAR)      remapBilinearWeights(remap->search, remap->vars);
-  else if (mapType == RemapMethod::BICUBIC)       remapBicubicWeights(remap->search, remap->vars);
-  else if (mapType == RemapMethod::DISTWGT)       remapDistwgtWeights(numNeighbors, remap->search, remap->vars);
-  else if (mapType == RemapMethod::CONSERV)       remapConservWeights(remap->search, remap->vars);
+  if      (mapType == RemapMethod::CONSERV_SCRIP) remapConservWeightsScrip(remap.search, remap.vars);
+  else if (mapType == RemapMethod::BILINEAR)      remapBilinearWeights(remap.search, remap.vars);
+  else if (mapType == RemapMethod::BICUBIC)       remapBicubicWeights(remap.search, remap.vars);
+  else if (mapType == RemapMethod::DISTWGT)       remapDistwgtWeights(numNeighbors, remap.search, remap.vars);
+  else if (mapType == RemapMethod::CONSERV)       remapConservWeights(remap.search, remap.vars);
   // clang-format on
 
-  if (remap->vars.sort_add) remapSortAddr(remap->vars);
-  if (remap->vars.links_per_value == -1) remapLinksPerValue(remap->vars);
+  if (remap.vars.sort_add) remapSortAddr(remap.vars);
+  if (remap.vars.links_per_value == -1) remapLinksPerValue(remap.vars);
 }
 
 static void
-remapField(RemapMethod mapType, remapType *remap, int numNeighbors, double *array1, double *array2, double missval)
+remapField(RemapMethod mapType, RemapType &remap, int numNeighbors, double *array1, double *array2, double missval)
 {
   // clang-format off
-  if      (mapType == RemapMethod::BILINEAR) remapBilinear(remap->search, array1, array2, missval);
-  else if (mapType == RemapMethod::BICUBIC)  remapBicubic(remap->search, array1, array2, missval);
-  else if (mapType == RemapMethod::DISTWGT)  remapDistwgt(numNeighbors, remap->search, array1, array2, missval);
-  else if (mapType == RemapMethod::CONSERV)  remapConserv(remap->vars.normOpt, remap->search, array1, array2, missval);
+  if      (mapType == RemapMethod::BILINEAR) remapBilinear(remap.search, array1, array2, missval);
+  else if (mapType == RemapMethod::BICUBIC)  remapBicubic(remap.search, array1, array2, missval);
+  else if (mapType == RemapMethod::DISTWGT)  remapDistwgt(numNeighbors, remap.search, array1, array2, missval);
+  else if (mapType == RemapMethod::CONSERV)  remapConserv(remap.vars.normOpt, remap.search, array1, array2, missval);
   // clang-format on
 }
 
@@ -771,7 +753,7 @@ addOperators(void)
   cdoOperatorAdd("remapdis",     REMAPDIS,     0, nullptr);
   cdoOperatorAdd("remapnn",      REMAPNN,      0, nullptr);
   cdoOperatorAdd("remaplaf",     REMAPLAF,     0, nullptr);
-  cdoOperatorAdd("remapsum",     REMAPSUM,     0, nullptr);
+  cdoOperatorAdd("remapavgtest", REMAPAVG,     0, nullptr);
   cdoOperatorAdd("gencon",       GENCON,       1, nullptr);
   cdoOperatorAdd("genycon2",     GENYCON2,     1, nullptr);
   cdoOperatorAdd("genscon",      GENSCON,      1, nullptr);
@@ -854,26 +836,29 @@ Remap(void *argument)
   const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   const auto ngrids = vlistNgrids(vlistID1);
   std::vector<bool> remapgrids(ngrids);
-  const auto index = set_remapgrids(filetype, vlistID1, ngrids, remapgrids);
-  const auto gridID1 = vlistGrid(vlistID1, index);
+  const auto gridID1 = vlistGrid(vlistID1, set_remapgrids(filetype, vlistID1, ngrids, remapgrids));
 
   for (int index = 0; index < ngrids; index++)
     if (remapgrids[index]) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  if (max_remaps == -1) max_remaps = set_max_remaps(vlistID1);
-  if (max_remaps < 1) cdoAbort("max_remaps out of range (>0)!");
+  if (MaxRemaps == -1) MaxRemaps = set_max_remaps(vlistID1);
+  if (MaxRemaps < 1) cdoAbort("max_remaps out of range (>0)!");
 
-  std::vector<remapType> remaps(max_remaps);
-  for (int r = 0; r < max_remaps; r++) remapInit(remaps[r]);
+  std::vector<RemapType> remaps(MaxRemaps);
+  // for (int r = 0; r < MaxRemaps; r++) remapInit(remaps[r]);
+  for (auto &rmap : remaps) remapInit(rmap);
 
   if (writeRemapWeightsOnly || lremap) remap_genweights = true;
 
   int remapOrder = 0;
   if (lremap)
     {
-      remapReadDataScrip(remap_file, gridID1, gridID2, &mapType, &submapType, &numNeighbors, remapOrder, remaps[0].src_grid,
+      remapReadDataScrip(remap_file, gridID1, gridID2, mapType, submapType, numNeighbors, remapOrder, remaps[0].src_grid,
                          remaps[0].tgt_grid, remaps[0].vars);
 
       if (remaps[0].vars.links_per_value == 0) remapLinksPerValue(remaps[0].vars);
@@ -908,9 +893,7 @@ Remap(void *argument)
             if (remaps[0].tgt_grid.vgpm[i]) isize++;
           gridsize2 = isize;
         }
-      /*
-      printf("grid2 %zu %d %zu\n", gridsize2, remaps[0].tgt_grid.nvgp, remaps[0].tgt_grid.size);
-      */
+      // printf("grid2 %zu %d %zu\n", gridsize2, remaps[0].tgt_grid.nvgp, remaps[0].tgt_grid.size);
       if (remaps[0].tgt_grid.size != gridsize2) cdoAbort("Size of target grid and weights from %s differ!", remap_file);
 
       operfunc = maptype2operfunc(mapType, submapType, numNeighbors, remapOrder);
@@ -922,19 +905,19 @@ Remap(void *argument)
       getMaptype(operfunc, mapType, submapType, numNeighbors, remapOrder);
     }
 
-  //if (!remap_genweights && (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV)) remap_genweights = true;
+  // if (!remap_genweights && (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV)) remap_genweights = true;
   if (!remap_genweights && mapType == RemapMethod::CONSERV_SCRIP) remap_genweights = true;
 
   bool useMask = !(!remap_genweights
-                   && (mapType == RemapMethod::BILINEAR || mapType == RemapMethod::BICUBIC
-                       || mapType == RemapMethod::DISTWGT || mapType == RemapMethod::CONSERV));
+                   && (mapType == RemapMethod::BILINEAR || mapType == RemapMethod::BICUBIC || mapType == RemapMethod::DISTWGT
+                       || mapType == RemapMethod::CONSERV));
 
   remap_set_int(REMAP_GENWEIGHTS, (int) remap_genweights);
 
   NormOpt normOpt(NormOpt::NONE);
   if (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV) normOpt = getNormOpt();
 
-  const size_t grid1sizemax = vlistGridsizeMax(vlistID1);
+  const auto grid1sizemax = vlistGridsizeMax(vlistID1);
 
   bool needGradients = (mapType == RemapMethod::BICUBIC);
   if ((mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV) && remapOrder == 2)
@@ -949,8 +932,7 @@ Remap(void *argument)
   Varray<double> array1(grid1sizemax);
   Varray<int> imask(grid1sizemax);
 
-  auto gridsize = gridInqSize(gridID2);
-  Varray<double> array2(gridsize);
+  Varray<double> array2(gridInqSize(gridID2));
 
   if (!writeRemapWeightsOnly)
     {
@@ -971,12 +953,12 @@ Remap(void *argument)
           cdoReadRecord(streamID1, &array1[0], &nmiss1);
           if (!useMask) nmiss1 = 0;
 
-          const auto gridID1 = vlistInqVarGrid(vlistID1, varID);
-          const auto missval = vlistInqVarMissval(vlistID1, varID);
-          auto gridsize = gridInqSize(gridID1);
+          const auto gridID = varList1[varID].gridID;
+          const auto missval = varList1[varID].missval;
+          auto gridsize = varList1[varID].gridsize;
 
           bool skipVar = false;
-          if (!remapgrids[vlistGridIndex(vlistID1, gridID1)])
+          if (!remapgrids[vlistGridIndex(vlistID1, gridID)])
             {
               if (writeRemapWeightsOnly)
                 continue;
@@ -990,17 +972,17 @@ Remap(void *argument)
 
           if (!skipVar)
             {
-              if (mapType != RemapMethod::CONSERV_SCRIP && mapType != RemapMethod::CONSERV && gridInqType(gridID1) == GRID_GME
-                  && gridInqType(gridID2) == GRID_GME)
+              if (mapType != RemapMethod::CONSERV_SCRIP && mapType != RemapMethod::CONSERV && gridInqType(gridID) == GRID_GME
+                  && gridInqType(gridID) == GRID_GME)
                 cdoAbort("Only conservative remapping is available to remap between GME grids!");
 
-              if (gridIsCircular(gridID1) && !lextrapolate) remap_extrapolate = true;
+              if (gridIsCircular(gridID) && !lextrapolate) remap_extrapolate = true;
 
               for (size_t i = 0; i < gridsize; i++) imask[i] = useMask ? !DBL_IS_EQUAL(array1[i], missval) : 1;
 
               for (r = nremaps - 1; r >= 0; r--)
                 {
-                  if (gridID1 == remaps[r].gridID)
+                  if (gridID == remaps[r].gridID)
                     {
                       if ((useMask && nmiss1 == remaps[r].nmiss && imask == remaps[r].src_grid.mask) || !useMask)
                         {
@@ -1014,40 +996,38 @@ Remap(void *argument)
 
               if (r < 0)
                 {
-                  if (nremaps < max_remaps)
+                  if (nremaps < MaxRemaps)
                     {
                       r = nremaps;
                       nremaps++;
                     }
                   else
                     {
-                      const int n0 = (max_remaps > 1 && remaps[0].nused > remaps[1].nused);
+                      const int n0 = (MaxRemaps > 1 && remaps[0].nused > remaps[1].nused);
                       remapVarsFree(remaps[n0].vars);
                       remapGridFree(remaps[n0].src_grid);
                       remapGridFree(remaps[n0].tgt_grid);
                       remapSearchFree(remaps[n0].search);
-                      // for (r = n0 + 1; r < nremaps; r++) memcpy(&remaps[r - 1], &remaps[r], sizeof(remapType));
-                      // r = nremaps - 1;
                       r = n0;
                       remapInit(remaps[r]);
                     }
 
-                  if (remaps[r].gridID != gridID1)
+                  if (remaps[r].gridID != gridID)
                     {
-                      if (gridIsCircular(gridID1) && !lextrapolate) remap_extrapolate = true;
+                      if (gridIsCircular(gridID) && !lextrapolate) remap_extrapolate = true;
                       remaps[r].src_grid.non_global = false;
-                      if (mapType == RemapMethod::DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1
-                          && !gridIsGlobal(gridID1))
+                      if (mapType == RemapMethod::DISTWGT && !remap_extrapolate && gridInqSize(gridID) > 1
+                          && !gridIsGlobal(gridID))
                         remaps[r].src_grid.non_global = true;
-                      /*
-                        remaps[r].src_grid.luse_cell_area = false;
-                        remaps[r].tgt_grid.luse_cell_area = false;
-                      */
-                      if (gridInqType(gridID1) != GRID_UNSTRUCTURED && !lremap_num_srch_bins)
+
+                      //  remaps[r].src_grid.luse_cell_area = false;
+                      //  remaps[r].tgt_grid.luse_cell_area = false;
+
+                      if (gridInqType(gridID) != GRID_UNSTRUCTURED && !lremap_num_srch_bins)
                         {
                           remap_num_srch_bins = (!remap_extrapolate && mapType == RemapMethod::DISTWGT)
                                                     ? 1
-                                                    : remapGenNumBins(gridInqYsize(gridID1));
+                                                    : remapGenNumBins(gridInqYsize(gridID));
                         }
 
                       remap_set_int(REMAP_NUM_SRCH_BINS, remap_num_srch_bins);
@@ -1056,20 +1036,20 @@ Remap(void *argument)
                       remaps[r].vars.pinit = false;
 
                       if ((mapType == RemapMethod::BILINEAR || mapType == RemapMethod::BICUBIC)
-                          && (gridInqType(gridID1) == GRID_GME || gridInqType(gridID1) == GRID_UNSTRUCTURED))
+                          && (gridInqType(gridID) == GRID_GME || gridInqType(gridID) == GRID_UNSTRUCTURED))
                         cdoAbort("Bilinear/bicubic interpolation doesn't support unstructured source grids!");
 
                       // Initialize grid information for both grids
                       if (Options::Timer) timer_start(timer_remap_init);
-                      remapInitGrids(mapType, remap_extrapolate, gridID1, remaps[r].src_grid, gridID2, remaps[r].tgt_grid);
+                      remapInitGrids(mapType, remap_extrapolate, gridID, remaps[r].src_grid, gridID2, remaps[r].tgt_grid);
                       remapSearchInit(mapType, remaps[r].search, remaps[r].src_grid, remaps[r].tgt_grid);
                       if (Options::Timer) timer_stop(timer_remap_init);
                     }
 
-                  remaps[r].gridID = gridID1;
+                  remaps[r].gridID = gridID;
                   remaps[r].nmiss = nmiss1;
 
-                  if (gridInqType(gridID1) == GRID_GME)
+                  if (gridInqType(gridID) == GRID_GME)
                     {
                       for (size_t i = 0, j = 0; i < gridsize; i++)
                         if (remaps[r].src_grid.vgpm[i]) imask[j++] = imask[i];
@@ -1092,7 +1072,7 @@ Remap(void *argument)
 
                   if (remap_genweights)
                     {
-                      remapGenWeights(mapType, &remaps[r], numNeighbors);
+                      remapGenWeights(mapType, remaps[r], numNeighbors);
 
                       if (writeRemapWeightsOnly) goto WRITE_REMAP;
 
@@ -1100,7 +1080,7 @@ Remap(void *argument)
                     }
                 }
 
-              if (gridInqType(gridID1) == GRID_GME)
+              if (gridInqType(gridID) == GRID_GME)
                 {
                   for (size_t i = 0, j = 0; i < gridsize; i++)
                     if (remaps[r].src_grid.vgpm[i]) array1[j++] = array1[i];
@@ -1121,15 +1101,15 @@ Remap(void *argument)
                     }
 
                   if (operfunc == REMAPLAF)
-                    remap_laf(&array2[0], missval, gridsize2, remaps[r].vars, array1.data());
-                  else if (operfunc == REMAPSUM)
-                    remap_sum(&array2[0], missval, gridsize2, remaps[r].vars, array1.data());
+                    remap_laf(array2.data(), missval, gridsize2, remaps[r].vars, array1.data());
+                  else if (operfunc == REMAPAVG)
+                    remap_avg(array2.data(), missval, gridsize2, remaps[r].vars, array1.data());
                   else
-                    remap(&array2[0], missval, gridsize2, remaps[r].vars, array1.data(), gradients);
+                    remap(array2.data(), missval, gridsize2, remaps[r].vars, array1.data(), gradients);
                 }
               else
                 {
-                  remapField(mapType, &remaps[r], numNeighbors, array1.data(), array2.data(), missval);
+                  remapField(mapType, remaps[r], numNeighbors, array1.data(), array2.data(), missval);
                 }
 
               if (operfunc == REMAPSCON || operfunc == REMAPCON2 || operfunc == REMAPCON || operfunc == REMAPYCON2)
@@ -1140,17 +1120,9 @@ Remap(void *argument)
                   remapSetFracMin(gridsize2, array2.data(), missval, &remaps[r].tgt_grid);
                 }
 
-              if (operfunc == REMAPSUM)
-                {
-                  remapSum(&remaps[r].src_grid, gridsize, array1.data(), "src");
-                  remapSum(&remaps[r].tgt_grid, gridsize2, array2.data(), "tgt");
-                }
-
               if (operfunc == REMAPSCON || operfunc == REMAPCON2 || operfunc == REMAPCON || operfunc == REMAPYCON2)
                 {
-                  char varname[CDI_MAX_NAME];
-                  vlistInqVarName(vlistID1, varID, varname);
-                  if (strcmp(varname, "gridbox_area") == 0)
+                  if (strcmp(varList1[varID].name, "gridbox_area") == 0)
                     scaleGridboxArea(gridsize, array1.data(), gridsize2, array2.data(), remaps[r].tgt_grid.cell_area.data());
                 }
 
@@ -1193,7 +1165,7 @@ WRITE_REMAP:
   if (lremap && remap_genweights && remaps[0].nused == 0)
     remapPrintWarning(remap_file, operfunc, remaps[0].src_grid, remaps[0].nmiss);
 
-  for (int r = 0; r < nremaps; r++)
+  for (r = 0; r < nremaps; r++)
     {
       remapVarsFree(remaps[r].vars);
       remapGridFree(remaps[r].src_grid);
