@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "readline.h"
 #include "hetaeta.h"
-#include "after_vertint.h"
+#include "vertical_interp.h"
 #include "stdnametable.h"
 #include "util_string.h"
 #include "timer.h"
@@ -34,13 +36,11 @@
 #include "cdi_lockedIO.h"
 
 static void
-setmissval(long nvals, const int *imiss, double missval, double *array)
+setmissval(long nvals, const Varray<int> &imiss, double missval, double *array)
 {
-  if (imiss)
-    {
-      for (long i = 0; i < nvals; ++i)
-        if (imiss[i]) array[i] = missval;
-    }
+  if (!imiss.empty())
+    for (long i = 0; i < nvals; ++i)
+      if (imiss[i]) array[i] = missval;
 }
 
 static void
@@ -64,16 +64,15 @@ ncctop(double cptop, long nlev, long nlevp1, const double *vct_a, const double *
     E. Roeckner, MPI, October 2001
   */
   long nctop = 0;
-  double za, zb;
-  std::vector<double> zph(nlevp1), zp(nlev);
+  Varray<double> zph(nlevp1), zp(nlev);
   // double    cptop  =  1000.;   /* min. pressure level for cond. */
 
   /* half level pressure values, assuming 101320. Pa surface pressure */
 
   for (long jk = 0; jk < nlevp1; ++jk)
     {
-      za = vct_a[jk];
-      zb = vct_b[jk];
+      const double za = vct_a[jk];
+      const double zb = vct_b[jk];
       zph[jk] = za + zb * 101320.;
     }
 
@@ -93,13 +92,13 @@ ncctop(double cptop, long nlev, long nlevp1, const double *vct_a, const double *
 }
 
 static void
-vctFromFile(const char *filename, int *nvct, std::vector<double> &vct2)
+vctFromFile(const char *filename, int *nvct, Varray<double> &vct2)
 {
   char line[1024], *pline;
   int i = 0;
   constexpr int maxvct = 8192;
 
-  FILE *fp = fopen(filename, "r");
+  auto fp = fopen(filename, "r");
   if (fp == nullptr)
     {
       perror(filename);
@@ -108,7 +107,7 @@ vctFromFile(const char *filename, int *nvct, std::vector<double> &vct2)
 
   vct2.resize(maxvct);
 
-  while (readline(fp, line, 1024))
+  while (cdo::readline(fp, line, 1024))
     {
       if (line[0] == '#' || line[0] == '\0') continue;
 
@@ -140,7 +139,7 @@ vctFromFile(const char *filename, int *nvct, std::vector<double> &vct2)
 }
 
 static void
-vertSum(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t nlevel)
+vertSum(Varray<double> &sum, const Varray<double> &var3d, size_t gridsize, size_t nlevel)
 {
   for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
@@ -152,7 +151,7 @@ vertSum(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t n
 }
 
 static void
-vertSumw(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t nlevel, const std::vector<double> &deltap)
+vertSumw(Varray<double> &sum, const Varray<double> &var3d, size_t gridsize, size_t nlevel, const Varray<double> &deltap)
 {
   for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
@@ -164,7 +163,7 @@ vertSumw(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t
 }
 
 void
-vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, std::vector<double> &vct, int *rnhlevf)
+vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, Varray<double> &vct, int *rnhlevf)
 {
   int zaxisIDh = -1;
   int nhlevf = 0;
@@ -174,8 +173,8 @@ vlist_hybrid_vct(int vlistID, int *rzaxisIDh, int *rnvct, std::vector<double> &v
   const int nzaxis = vlistNzaxis(vlistID);
   for (int i = 0; i < nzaxis; i++)
     {
-      const int zaxisID = vlistZaxis(vlistID, i);
-      const int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID, i);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
       if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel > 1)
         {
@@ -213,25 +212,18 @@ Remapeta(void *process)
 {
   size_t nfis2gp = 0;
   int nrecs;
-  int i;
   int iv;
   int varID, levelID;
   int nvars3D = 0;
   int sgeopotID = -1, tempID = -1, sqID = -1, psID = -1, lnpsID = -1;
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
-  double *single2;
-  std::vector<double> fis2;
-  double *t1 = nullptr, *q1 = nullptr;
-  double *t2 = nullptr, *q2 = nullptr;
-  double *tscor = nullptr, *pscor = nullptr, *secor = nullptr;
+  Varray<double> fis2;
   size_t nmiss, nmissout = 0;
   bool ltq = false;
   bool lfis2 = false;
   int varids[MAX_VARS3D];
-  int *imiss = nullptr;
+  Varray<int> imiss;
   int timer_hetaeta = 0;
-  double **vars1 = nullptr, **vars2 = nullptr;
-  double minval, maxval;
   double missval = 0;
   double cconst = 1.E-6;
   double cptop = 0; /* min. pressure level for cond. */
@@ -241,9 +233,9 @@ Remapeta(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int REMAPETA  = cdoOperatorAdd("remapeta",   0, 0, "VCT file name");
-  const int REMAPETAS = cdoOperatorAdd("remapeta_s", 0, 0, "VCT file name");
-  const int REMAPETAZ = cdoOperatorAdd("remapeta_z", 0, 0, "VCT file name");
+  const auto REMAPETA  = cdoOperatorAdd("remapeta",   0, 0, "VCT file name");
+  const auto REMAPETAS = cdoOperatorAdd("remapeta_s", 0, 0, "VCT file name");
+  const auto REMAPETAZ = cdoOperatorAdd("remapeta_z", 0, 0, "VCT file name");
   // clang-format on
 
   const int operatorID = cdoOperatorID();
@@ -253,7 +245,7 @@ Remapeta(void *process)
   const char *envstr = getenv("REMAPETA_PTOP");
   if (envstr)
     {
-      double fval = atof(envstr);
+      const auto fval = atof(envstr);
       if (fval > 0)
         {
           cptop = fval;
@@ -263,28 +255,28 @@ Remapeta(void *process)
     }
 
   int nvct2 = 0;
-  std::vector<double> vct2;
-  vctFromFile(operatorArgv()[0], &nvct2, vct2);
+  Varray<double> vct2;
+  vctFromFile(cdoOperatorArgv(0).c_str(), &nvct2, vct2);
   const int nhlevf2 = nvct2 / 2 - 1;
 
   const double *a2 = vct2.data();
   const double *b2 = vct2.data() + nvct2 / 2;
 
   if (Options::cdoVerbose)
-    for (i = 0; i < nhlevf2 + 1; ++i) cdoPrint("vct2: %5d %25.17f %25.17f", i, vct2[i], vct2[nvct2 / 2 + i]);
+    for (int i = 0; i < nhlevf2 + 1; ++i) cdoPrint("vct2: %5d %25.17f %25.17f", i, vct2[i], vct2[nvct2 / 2 + i]);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
   if (operatorArgc() == 2)
     {
       lfis2 = true;
 
-      const char *fname = operatorArgv()[1];
+      const char *fname = cdoOperatorArgv(1).c_str();
       int streamID = streamOpenReadLocked(fname);
-      const int vlistID1 = streamInqVlist(streamID);
+      const auto vlistID1 = streamInqVlist(streamID);
 
       streamInqRecord(streamID, &varID, &levelID);
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
       nfis2gp = gridInqSize(gridID);
 
       fis2.resize(nfis2gp);
@@ -294,41 +286,38 @@ Remapeta(void *process)
       if (nmiss)
         {
           missval = vlistInqVarMissval(vlistID1, varID);
-          imiss = (int *) Malloc(nfis2gp * sizeof(int));
-          for (size_t i = 0; i < nfis2gp; ++i)
-            {
-              imiss[i] = (DBL_IS_EQUAL(fis2[i], missval));
-            }
+          imiss.resize(nfis2gp);
+          for (size_t i = 0; i < nfis2gp; ++i) imiss[i] = DBL_IS_EQUAL(fis2[i], missval);
 
           nmissout = nmiss;
         }
 
       // check range of surface_geopotential
-      arrayMinMaxMask(nfis2gp, fis2.data(), imiss, minval, maxval);
-      if (minval < MIN_FIS || maxval > MAX_FIS)
-        cdoWarning("%s out of range (min=%g max=%g)!", var_stdname(surface_geopotential), minval, maxval);
+      auto mm = arrayMinMaxMask(nfis2gp, fis2.data(), imiss);
+      if (mm.min < MIN_FIS || mm.max > MAX_FIS)
+        cdoWarning("%s out of range (min=%g max=%g)!", var_stdname(surface_geopotential), mm.min, mm.max);
 
-      if (minval < -1.e10 || maxval > 1.e10) cdoAbort("%s out of range!", var_stdname(surface_geopotential));
+      if (mm.min < -1.e10 || mm.max > 1.e10) cdoAbort("%s out of range!", var_stdname(surface_geopotential));
 
       streamClose(streamID);
     }
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int gridID = vlistGrid(vlistID1, 0);
-  if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
+  const auto gridID0 = vlistGrid(vlistID1, 0);
+  if (gridInqType(gridID0) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-  const size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
-  const int zaxisID2 = zaxisCreate(ZAXIS_HYBRID, nhlevf2);
+  const auto zaxisID2 = zaxisCreate(ZAXIS_HYBRID, nhlevf2);
 
   {
-    std::vector<double> lev2(nhlevf2);
+    Varray<double> lev2(nhlevf2);
     for (int i = 0; i < nhlevf2; ++i) lev2[i] = i + 1;
     zaxisDefLevels(zaxisID2, lev2.data());
   }
@@ -336,63 +325,63 @@ Remapeta(void *process)
   if (nvct2 == 0) cdoAbort("Internal problem, vct2 undefined!");
   zaxisDefVct(zaxisID2, nvct2, vct2.data());
 
-  int surfaceID = zaxisFromName("surface");
+  auto surfaceID = zaxisFromName("surface");
 
   int zaxisIDh = -1;
   int nvct1 = 0;
   int nhlevf1 = 0;
-  std::vector<double> vct1;
+  Varray<double> vct1;
   vlist_hybrid_vct(vlistID1, &zaxisIDh, &nvct1, vct1, &nhlevf1);
 
   vlist_change_hybrid_zaxis(vlistID1, vlistID2, zaxisIDh, zaxisID2);
 
-  const int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (int i = 0; i < nzaxis; i++)
     {
-      const int zaxisID = vlistZaxis(vlistID1, i);
-      const int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, i);
+      const auto nlevel = zaxisInqSize(zaxisID);
       if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel == 1) vlistChangeZaxisIndex(vlistID2, i, surfaceID);
     }
 
   const double *a1 = vct1.data();
   const double *b1 = vct1.data() + nvct1 / 2;
   if (Options::cdoVerbose)
-    for (i = 0; i < nvct1 / 2; ++i) cdoPrint("vct1: %5d %25.17f %25.17f", i, vct1[i], vct1[nvct1 / 2 + i]);
+    for (int i = 0; i < nvct1 / 2; ++i) cdoPrint("vct1: %5d %25.17f %25.17f", i, vct1[i], vct1[nvct1 / 2 + i]);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   if (zaxisIDh == -1) cdoWarning("No 3D variable with hybrid sigma pressure coordinate found!");
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const int nlevel = zaxisInqSize(zaxisID);
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
-      int code = vlistInqVarCode(vlistID1, varID);
+      auto code = vlistInqVarCode(vlistID1, varID);
       /* code = -1; */
       if (code <= 0 || code == 255)
         {
           vlistInqVarName(vlistID1, varID, varname);
           cstrToLowerCase(varname);
 
-          vlistInqVarStdname(vlistID1, varID, stdname);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID1, varID, CDI_KEY_STDNAME, stdname, &length);
           cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
-
           if (code == -1)
             {
               //                                            ECHAM                         ECMWF
               // clang-format off
-              if (sgeopotID == -1 && (cstrIsEqual(varname, "geosp") || cstrIsEqual(varname, "z"))) code = 129;
-              else if (tempID == -1 && (cstrIsEqual(varname, "st") || cstrIsEqual(varname, "t"))) code = 130;
-              else if (psID == -1 && (cstrIsEqual(varname, "aps") || cstrIsEqual(varname, "ps"))) code = 134;
-              else if (lnpsID == -1 && (cstrIsEqual(varname, "lsp") || cstrIsEqual(varname, "lnsp"))) code = 152;
-              else if (sqID == -1 && (cstrIsEqual(varname, "q"))) code = 133;
+              if      (sgeopotID == -1 && (cdo_cmpstr(varname, "geosp") || cdo_cmpstr(varname, "z")))    code = 129;
+              else if (tempID == -1    && (cdo_cmpstr(varname, "st")    || cdo_cmpstr(varname, "t")))    code = 130;
+              else if (psID == -1      && (cdo_cmpstr(varname, "aps")   || cdo_cmpstr(varname, "ps")))   code = 134;
+              else if (lnpsID == -1    && (cdo_cmpstr(varname, "lsp")   || cdo_cmpstr(varname, "lnsp"))) code = 152;
+              else if (sqID == -1      && (cdo_cmpstr(varname, "q"))) code = 133;
               // clang-format on
             }
         }
@@ -450,15 +439,15 @@ Remapeta(void *process)
     {
     }
 
-  std::vector<double> sum1, sum2;
+  Varray<double> sum1, sum2;
   if (operatorID == REMAPETAS || operatorID == REMAPETAZ)
     {
       sum1.resize(gridsize);
       sum2.resize(gridsize);
     }
 
-  std::vector<double> deltap1, deltap2;
-  std::vector<double> half_press1, half_press2;
+  Varray<double> deltap1, deltap2;
+  Varray<double> half_press1, half_press2;
   if (operatorID == REMAPETAZ)
     {
       deltap1.resize(gridsize * nhlevf1);
@@ -467,46 +456,46 @@ Remapeta(void *process)
       half_press2.resize(gridsize * (nhlevf2 + 1));
     }
 
-  std::vector<double> array(gridsize);
+  Varray<double> array(gridsize);
 
-  std::vector<double> fis1(gridsize);
-  std::vector<double> ps1(gridsize);
+  Varray<double> fis1(gridsize);
+  Varray<double> ps1(gridsize);
 
   if (!lfis2) fis2.resize(gridsize);
   if (lfis2 && gridsize != nfis2gp) cdoAbort("Orographies have different grid size!");
 
-  std::vector<double> ps2(gridsize);
+  Varray<double> ps2(gridsize);
 
+  Varray<double> t1, q1;
+  Varray<double> t2, q2;
+  Varray<double> tscor, pscor, secor;
   if (ltq)
     {
-      tscor = (double *) Malloc(gridsize * sizeof(double));
-      pscor = (double *) Malloc(gridsize * sizeof(double));
-      secor = (double *) Malloc(gridsize * sizeof(double));
+      tscor.resize(gridsize);
+      pscor.resize(gridsize);
+      secor.resize(gridsize);
 
-      t1 = (double *) Malloc(gridsize * nhlevf1 * sizeof(double));
-      q1 = (double *) Malloc(gridsize * nhlevf1 * sizeof(double));
+      t1.resize(gridsize * nhlevf1);
+      q1.resize(gridsize * nhlevf1);
 
-      t2 = (double *) Malloc(gridsize * nhlevf2 * sizeof(double));
-      q2 = (double *) Malloc(gridsize * nhlevf2 * sizeof(double));
+      t2.resize(gridsize * nhlevf2);
+      q2.resize(gridsize * nhlevf2);
     }
 
+  Varray2D<double> vars1, vars2;
   if (nvars3D)
     {
-      vars1 = (double **) Malloc(nvars * sizeof(double *));
-      vars2 = (double **) Malloc(nvars * sizeof(double *));
-
-      for (varID = 0; varID < nvars3D; ++varID)
-        {
-          vars1[varID] = (double *) Malloc(gridsize * nhlevf1 * sizeof(double));
-          vars2[varID] = (double *) Malloc(gridsize * nhlevf2 * sizeof(double));
-        }
+      vars1.resize(nvars);
+      vars2.resize(nvars);
+      for (varID = 0; varID < nvars3D; ++varID) vars1[varID].resize(gridsize * nhlevf1);
+      for (varID = 0; varID < nvars3D; ++varID) vars2[varID].resize(gridsize * nhlevf2);
     }
 
   if (zaxisIDh != -1 && sgeopotID == -1)
     {
       if (ltq) cdoWarning("%s not found - set to zero!", var_stdname(surface_geopotential));
 
-      arrayFill(gridsize, fis1.data(), 0.0);
+      varrayFill(fis1, 0.0);
     }
 
   int presID = lnpsID;
@@ -537,35 +526,36 @@ Remapeta(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          const int nlevel = zaxisInqSize(zaxisID);
-          const size_t offset = gridsize * levelID;
+          const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+          const auto nlevel = zaxisInqSize(zaxisID);
+          const auto offset = gridsize * levelID;
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
           if (zaxisIDh != -1)
             {
               if (varID == sgeopotID)
-                arrayCopy(gridsize, array.data(), fis1.data());
+                varrayCopy(gridsize, array, fis1);
               else if (varID == presID)
                 {
                   if (lnpsID != -1)
-                    for (size_t i = 0; i < gridsize; ++i) ps1[i] = exp(array[i]);
+                    for (size_t i = 0; i < gridsize; ++i) ps1[i] = std::exp(array[i]);
                   else if (psID != -1)
-                    arrayCopy(gridsize, array.data(), ps1.data());
+                    varrayCopy(gridsize, array, ps1);
                 }
               else if (ltq && varID == tempID)
-                arrayCopy(gridsize, array.data(), t1 + offset);
+                arrayCopy(gridsize, array.data(), &t1[offset]);
               else if (ltq && varID == sqID)
-                arrayCopy(gridsize, array.data(), q1 + offset);
+                arrayCopy(gridsize, array.data(), &q1[offset]);
               /* else if ( zaxisID == zaxisIDh ) */
               else if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevel == nhlevf1)
                 {
+                  int i;
                   for (i = 0; i < nvars3D; ++i)
                     if (varID == varids[i]) break;
 
                   if (i == nvars3D) cdoAbort("Internal error, 3D variable not found!");
 
-                  arrayCopy(gridsize, array.data(), vars1[i] + offset);
+                  arrayCopy(gridsize, array.data(), &vars1[i][offset]);
                 }
               else
                 {
@@ -583,12 +573,12 @@ Remapeta(void *process)
       if (zaxisIDh != -1)
         {
           // check range of ps_prog
-          arrayMinMaxMask(gridsize, ps1.data(), imiss, minval, maxval);
-          if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
+          auto mm = arrayMinMaxMask(gridsize, ps1.data(), imiss);
+          if (mm.min < MIN_PS || mm.max > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", mm.min, mm.max);
 
           // check range of geop
-          arrayMinMaxMask(gridsize, fis1.data(), imiss, minval, maxval);
-          if (minval < MIN_FIS || maxval > MAX_FIS) cdoWarning("Orography out of range (min=%g max=%g)!", minval, maxval);
+          mm = arrayMinMaxMask(gridsize, fis1.data(), imiss);
+          if (mm.min < MIN_FIS || mm.max > MAX_FIS) cdoWarning("Orography out of range (min=%g max=%g)!", mm.min, mm.max);
         }
 
       if (!lfis2)
@@ -600,34 +590,30 @@ Remapeta(void *process)
           int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              const size_t offset = gridsize * levelID;
-              single2 = t1 + offset;
-
-              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
-              if (minval < MIN_T || maxval > MAX_T)
-                cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
+              const auto offset = gridsize * levelID;
+              auto mm = arrayMinMaxMask(gridsize, &t1[offset], imiss);
+              if (mm.min < MIN_T || mm.max > MAX_T)
+                cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
             }
 
           varID = sqID;
           nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              const size_t offset = gridsize * levelID;
-              single2 = q1 + offset;
+              const auto offset = gridsize * levelID;
+              corr_hum(gridsize, &q1[offset], MIN_Q);
 
-              corr_hum(gridsize, single2, MIN_Q);
-
-              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
-              if (minval < MIN_Q || maxval > MAX_Q)
-                cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
+              auto mm = arrayMinMaxMask(gridsize, &q1[offset], imiss);
+              if (mm.min < MIN_Q || mm.max > MAX_Q)
+                cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
             }
         }
 
       if (nvars3D || ltq)
         {
           if (Options::Timer) timer_start(timer_hetaeta);
-          hetaeta(ltq, gridsize, imiss, nhlevf1, a1, b1, fis1.data(), ps1.data(), t1, q1, nhlevf2, a2, b2, fis2.data(), ps2.data(),
-                  t2, q2, nvars3D, vars1, vars2, tscor, pscor, secor);
+          hetaeta(ltq, gridsize, imiss.data(), nhlevf1, a1, b1, fis1, ps1, t1, q1, nhlevf2, a2, b2, fis2,
+                  ps2, t2, q2, nvars3D, vars1, vars2, tscor, pscor, secor);
           if (Options::Timer) timer_stop(timer_hetaeta);
         }
 
@@ -643,7 +629,7 @@ Remapeta(void *process)
         }
 
       if (zaxisIDh != -1 && lnpsID != -1)
-        for (size_t i = 0; i < gridsize; ++i) ps2[i] = log(ps2[i]);
+        for (size_t i = 0; i < gridsize; ++i) ps2[i] = std::log(ps2[i]);
 
       if (zaxisIDh != -1 && presID != -1)
         {
@@ -660,12 +646,12 @@ Remapeta(void *process)
           int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              const size_t offset = gridsize * levelID;
-              single2 = t2 + offset;
+              const auto offset = gridsize * levelID;
+              const auto single2 = &t2[offset];
 
-              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
-              if (minval < MIN_T || maxval > MAX_T)
-                cdoWarning("Output temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
+              auto mm = arrayMinMaxMask(gridsize, single2, imiss);
+              if (mm.min < MIN_T || mm.max > MAX_T)
+                cdoWarning("Output temperature at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
 
               setmissval(gridsize, imiss, missval, single2);
               cdoDefRecord(streamID2, varID, levelID);
@@ -676,17 +662,17 @@ Remapeta(void *process)
           nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              const size_t offset = gridsize * levelID;
-              single2 = q2 + offset;
+              const auto offset = gridsize * levelID;
+              const auto single2 = &q2[offset];
 
               corr_hum(gridsize, single2, MIN_Q);
 
               if (levelID < nctop)
                 for (size_t i = 0; i < gridsize; ++i) single2[i] = cconst;
 
-              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
-              if (minval < MIN_Q || maxval > MAX_Q)
-                cdoWarning("Output humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
+              auto mm = arrayMinMaxMask(gridsize, single2, imiss);
+              if (mm.min < MIN_Q || mm.max > MAX_Q)
+                cdoWarning("Output humidity at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
 
               setmissval(gridsize, imiss, missval, single2);
               cdoDefRecord(streamID2, varID, levelID);
@@ -698,7 +684,7 @@ Remapeta(void *process)
         {
           varID = varids[iv];
 
-          const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+          const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
 
           if (operatorID == REMAPETAS)
             {
@@ -707,40 +693,32 @@ Remapeta(void *process)
             }
           else if (operatorID == REMAPETAZ)
             {
-              presh(nullptr, half_press1.data(), vct1.data(), ps1.data(), nhlevf1, gridsize);
+              presh((double*)nullptr, half_press1.data(), vct1.data(), ps1.data(), nhlevf1, gridsize);
               for (int k = 0; k < nhlevf1; ++k)
                 for (size_t i = 0; i < gridsize; ++i)
                   {
                     deltap1[k * gridsize + i] = half_press1[(k + 1) * gridsize + i] - half_press1[k * gridsize + i];
-                    deltap1[k * gridsize + i] = log(deltap1[k * gridsize + i]);
+                    deltap1[k * gridsize + i] = std::log(deltap1[k * gridsize + i]);
                   }
               vertSumw(sum1, vars1[iv], gridsize, nhlevf1, deltap1);
 
-              presh(nullptr, half_press2.data(), vct2.data(), ps1.data(), nhlevf2, gridsize);
+              presh((double*)nullptr, half_press2.data(), vct2.data(), ps1.data(), nhlevf2, gridsize);
               for (int k = 0; k < nhlevf2; ++k)
                 for (size_t i = 0; i < gridsize; ++i)
                   {
                     deltap2[k * gridsize + i] = half_press2[(k + 1) * gridsize + i] - half_press2[k * gridsize + i];
-                    deltap2[k * gridsize + i] = log(deltap2[k * gridsize + i]);
+                    deltap2[k * gridsize + i] = std::log(deltap2[k * gridsize + i]);
                   }
               vertSumw(sum2, vars2[iv], gridsize, nhlevf2, deltap2);
             }
 
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              const size_t offset = gridsize * levelID;
-              single2 = vars2[iv] + offset;
+              const auto offset = gridsize * levelID;
+              const auto single2 = &vars2[iv][offset];
 
               if (operatorID == REMAPETAS || operatorID == REMAPETAZ)
-                {
-                  /*
-                  for ( size_t i = 0; i < gridsize; ++i )
-                    if ( i %100 == 0 )
-                      printf("%d %g %g %g %g %g\n",i, single2[i], sum1[i],
-                  sum2[i], sum1[i]/sum2[i], single2[i]*sum1[i]/sum2[i]);
-                  */
-                  for (size_t i = 0; i < gridsize; ++i) single2[i] = single2[i] * sum1[i] / sum2[i];
-                }
+                for (size_t i = 0; i < gridsize; ++i) single2[i] = single2[i] * sum1[i] / sum2[i];
 
               setmissval(gridsize, imiss, missval, single2);
               cdoDefRecord(streamID2, varID, levelID);
@@ -754,30 +732,6 @@ Remapeta(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (nvars3D)
-    {
-      for (varID = 0; varID < nvars3D; varID++)
-        {
-          Free(vars2[varID]);
-          Free(vars1[varID]);
-        }
-      Free(vars2);
-      Free(vars1);
-    }
-
-  if (ltq)
-    {
-      Free(q2);
-      Free(t2);
-      Free(q1);
-      Free(t1);
-      Free(secor);
-      Free(pscor);
-      Free(tscor);
-    }
-
-  if (imiss) Free(imiss);
-
   cdoFinish();
 
   return nullptr;
