@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,13 @@
 #include "netcdf.h"
 #endif
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "griddes.h"
 #include <mpim_grid.h>
 #include "remap_vars.h"
 #include "remap.h"  // SubmapType
+#include "cdo_zaxis.h"
 
 #ifdef HAVE_LIBNETCDF
 static void
@@ -130,13 +133,13 @@ struct RemapGridW
 
   std::vector<int> mask;  // flag which cells participate
 
-  std::vector<double> cell_center_lon;  // lon/lat coordinates for
-  std::vector<double> cell_center_lat;  // each grid center in radians
-  std::vector<double> cell_corner_lon;  // lon/lat coordinates for
-  std::vector<double> cell_corner_lat;  // each grid corner in radians
+  Varray<double> cell_center_lon;  // lon/lat coordinates for
+  Varray<double> cell_center_lat;  // each grid center in radians
+  Varray<double> cell_corner_lon;  // lon/lat coordinates for
+  Varray<double> cell_corner_lat;  // each grid corner in radians
 
-  std::vector<double> cell_area;  // total area of each grid cell
-  std::vector<double> cell_frac;  // fractional area of grid cells participating in remapping
+  Varray<double> cell_area;  // total area of each grid cell
+  Varray<double> cell_frac;  // fractional area of grid cells participating in remapping
 
   RemapGridW() : rank(0), ncells(0), ncorners(0)
   {
@@ -178,17 +181,15 @@ remapGridWAlloc(RemapMethod mapType, RemapGridW &grid)
 
 struct RemapVarsW
 {
-  bool sort_add;
-  RemapMethod mapType;  // identifier for remapping method
-  NormOpt normOpt;      // option for normalization (conserv only)
-  size_t num_links;     // number of links for remapping
-  size_t num_wts;       // num of weights used in remapping
+  bool sort_add = false;
+  RemapMethod mapType{RemapMethod::UNDEF}; // identifier for remapping method
+  NormOpt normOpt{NormOpt::NONE};          // option for normalization (conserv only)
+  size_t num_links = 0;                    // number of links for remapping
+  size_t num_wts = 0;                      // num of weights used in remapping
 
   std::vector<size_t> src_cell_add;  // source grid address for each link
   std::vector<size_t> tgt_cell_add;  // target grid address for each link
-  std::vector<double> wts;           // map weights for each link [num_links*num_wts]
-
-  RemapVarsW() : sort_add(false), num_links(0), num_wts(0) {}
+  Varray<double> wts;                // map weights for each link [num_links*num_wts]
 };
 
 static void
@@ -206,9 +207,9 @@ remapGetNormOpt(const std::string &normOptStr)
 {
   NormOpt normOpt(NormOpt::NONE);
   // clang-format off
-  if      (cstrIsEqual(normOptStr.c_str(), "none"))     normOpt = NormOpt::NONE;
-  else if (cstrIsEqual(normOptStr.c_str(), "fracarea")) normOpt = NormOpt::FRACAREA;
-  else if (cstrIsEqual(normOptStr.c_str(), "destarea")) normOpt = NormOpt::DESTAREA;
+  if      (normOptStr == "none")     normOpt = NormOpt::NONE;
+  else if (normOptStr == "fracarea") normOpt = NormOpt::FRACAREA;
+  else if (normOptStr == "destarea") normOpt = NormOpt::DESTAREA;
   else
     {
       cdoPrint("normalize_opt = %s", normOptStr.c_str());
@@ -221,10 +222,10 @@ remapGetNormOpt(const std::string &normOptStr)
   return normOpt;
 }
 
-RemapMethod getMapType(int ncfileid, SubmapType *submapType, int *numNeighbors, int &remapOrder);
+RemapMethod getMapType(int ncfileid, SubmapType &submapType, int &numNeighbors, int &remapOrder);
 
 static void
-readRemapGridScrip(const int ncfileid, const std::string prefix, bool lgridarea, RemapGridW &grid)
+readRemapGridScrip(const int ncfileid, const std::string &prefix, bool lgridarea, RemapGridW &grid)
 {
   // Read all variables of the grid
   cdfReadVarSize(ncfileid, (prefix + "_dims").c_str(), 2, grid.dims);
@@ -264,8 +265,7 @@ readRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMetho
   // The routine reads a NetCDF file to extract remapping info in SCRIP format
 
   // Open file and read some global information
-  // nce(nc_open(remapFile, NC_NOWRITE, &ncfileid));
-  const int ncfileid = cdf_openread(remapFile);
+  const int ncfileid = cdo_cdf_openread(remapFile);
 
   cdfReadAttText(ncfileid, NC_GLOBAL, "history", remapAtts.history);
 
@@ -279,7 +279,7 @@ readRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMetho
     }
 
   // Map Type
-  mapType = getMapType(ncfileid, &submapType, &numNeighbors, remapOrder);
+  mapType = getMapType(ncfileid, submapType, numNeighbors, remapOrder);
   const bool lgridarea = (mapType == RemapMethod::CONSERV || mapType == RemapMethod::CONSERV_SCRIP);
 
   remapVarsWInit(mapType, remapOrder, rv);
@@ -295,10 +295,10 @@ readRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMetho
   // File convention
   std::string convention;  // character string for output convention
   cdfReadAttText(ncfileid, NC_GLOBAL, "conventions", convention);
-  if (!cstrIsEqual(convention.c_str(), "SCRIP"))
+  if (convention != "SCRIP")
     {
       cdoPrint("convention = %s", convention.c_str());
-      if (cstrIsEqual(convention.c_str(), "NCAR-CSM"))
+      if (convention == "NCAR-CSM")
         cdoAbort("Unsupported file convention: %s!", convention.c_str());
       else
         cdoAbort("Unknown file convention!");
@@ -320,9 +320,7 @@ readRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMetho
   tgt_grid.setNumCorners(cdfReadDimlen(ncfileid, "dst_grid_corners"));
 
   src_grid.rank = cdfReadDimlen(ncfileid, "src_grid_rank");
-  ;
   tgt_grid.rank = cdfReadDimlen(ncfileid, "dst_grid_rank");
-  ;
 
   remapGridWAlloc(rv.mapType, src_grid);
   remapGridWAlloc(rv.mapType, tgt_grid);
@@ -345,8 +343,8 @@ readRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMetho
   if (rv.num_links > 0) readRemapWeightsScrip(ncfileid, rv);
 
   // Close input file
-  nce(nc_close(ncfileid));
-}  // remapReadDataScrip
+  cdo_cdf_close(ncfileid);
+}  // readRemapFileScrip
 
 static int
 cdfDefDim(int ncfileid, const char *name, size_t len)
@@ -402,7 +400,7 @@ cdfWriteVarSize(int ncfileid, int ncvarid, nc_type sizetype, size_t len, size_t
 }
 
 static void
-checkRemapFilesize(RemapGridW &src_grid, RemapGridW &tgt_grid, RemapVarsW &rv, int &writemode, nc_type &sizetype)
+checkRemapFilesize(const RemapGridW &src_grid, const RemapGridW &tgt_grid, const RemapVarsW &rv, int &writemode, nc_type &sizetype)
 {
   const size_t nlinks = rv.num_links;
   const size_t nele1 = 4 * 8 + 4 + src_grid.ncorners * 2 * 8;
@@ -418,7 +416,7 @@ checkRemapFilesize(RemapGridW &src_grid, RemapGridW &tgt_grid, RemapVarsW &rv, i
   if (filesize > 0x7FFFFC00)  // 2**31 - 1024 (<2GB)
     {
       const size_t maxlinks = 0x3FFFFFFF;  // 1GB
-      const size_t gridsizemax = (src_grid.ncells > tgt_grid.ncells) ? src_grid.ncells : tgt_grid.ncells;
+      const auto gridsizemax = (src_grid.ncells > tgt_grid.ncells) ? src_grid.ncells : tgt_grid.ncells;
       if (nlinks > maxlinks || filesize > 8 * maxlinks || gridsizemax > 0x7FFFFC00)
         {
 #ifdef HAVE_NETCDF4
@@ -457,7 +455,7 @@ struct CDFgrid
 };
 
 static CDFgrid
-defineRemapGridScrip(const int ncfileid, const std::string prefix, nc_type sizetype, bool lgridarea, const RemapGridW &grid)
+defineRemapGridScrip(const int ncfileid, const std::string &prefix, nc_type sizetype, bool lgridarea, const RemapGridW &grid)
 {
   CDFgrid cdfGrid;
 
@@ -510,7 +508,7 @@ defineRemapGridScrip(const int ncfileid, const std::string prefix, nc_type sizet
 }
 
 static void
-writeRemapGridScrip(const int ncfileid, CDFgrid &cdfGrid, bool lgridarea, RemapGridW &grid)
+writeRemapGridScrip(const int ncfileid, const CDFgrid &cdfGrid, bool lgridarea, RemapGridW &grid)
 {
   int dims[2];
   dims[0] = (int) grid.dims[0];
@@ -550,7 +548,7 @@ remapSetMapMethod(RemapMethod mapType, SubmapType submapType, int numNeighbors)
       mapMethod = "Conservative remapping using clipping on sphere";
       break;
     case RemapMethod::BILINEAR: mapMethod = "Bilinear remapping"; break;
-    case RemapMethod::BICUBIC: mapMethod = "Bicubic remapping"; break;
+    case RemapMethod::BICUBIC:  mapMethod = "Bicubic remapping"; break;
     case RemapMethod::DISTWGT:
       mapMethod = (numNeighbors == 1) ? "Nearest neighbor" : "Distance weighted avg of nearest neighbors";
       break;
@@ -563,7 +561,7 @@ remapSetMapMethod(RemapMethod mapType, SubmapType submapType, int numNeighbors)
 }
 
 static void
-writeRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMethod mapType, SubmapType submapType, int numNeighbors,
+writeRemapFileScrip(const char *remapFile, const RemapAttributes &remapAtts, RemapMethod mapType, SubmapType submapType, int numNeighbors,
                     int remapOrder, RemapGridW &src_grid, RemapGridW &tgt_grid, RemapVarsW &rv)
 {
   // Writes remap data to a NetCDF file using SCRIP conventions
@@ -592,7 +590,7 @@ writeRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMeth
   // clang-format off
   switch (rv.normOpt)
     {
-    case NormOpt::NONE:     normalizeOptStr = "none"; break;
+    case NormOpt::NONE:     normalizeOptStr = "none";     break;
     case NormOpt::FRACAREA: normalizeOptStr = "fracarea"; break;
     case NormOpt::DESTAREA: normalizeOptStr = "destarea"; break;
     default: normalizeOptStr = "unknown";
@@ -661,10 +659,10 @@ writeRemapFileScrip(const char *remapFile, RemapAttributes &remapAtts, RemapMeth
 }  // remapWriteDataScrip
 
 static void
-checkAreas(size_t n_a, const std::vector<double> &area_a, const std::vector<double> &area_b, size_t n_s, const size_t *col,
-           const size_t *row, const double *S)
+checkAreas(size_t n_a, const Varray<double> &area_a, const Varray<double> &area_b, size_t n_s, const size_t *col, const size_t *row,
+           const double *S)
 {
-  std::vector<double> sum(n_a, 0.0);
+  Varray<double> sum(n_a, 0.0);
 
   // sum weighted ratio of true areas
   //      ’a’ is source; ’b’ is destination
@@ -680,9 +678,6 @@ checkAreas(size_t n_a, const std::vector<double> &area_a, const std::vector<doub
 static void
 verifyWeights(const char *remapFile)
 {
-  // int ncfileid = cdf_openread(weights_file);
-  // nce(nc_close(ncfileid));
-
   RemapMethod mapType(RemapMethod::UNDEF);
   SubmapType submapType(SubmapType::NONE);
   int numNeighbors = 0;
@@ -710,14 +705,16 @@ writeRemapScrip(const char *remapFileIn, const char *remapFileOut)
 
   readRemapFileScrip(remapFileIn, remapAtts, mapType, submapType, numNeighbors, remapOrder, src_grid, tgt_grid, remapVars);
 
+  /*
   char history[1024] = "date and time";
   time_t date_and_time_in_sec = time(NULL);
   if (date_and_time_in_sec != -1)
     {
-      // struct tm *date_and_time = localtime(&date_and_time_in_sec);
-      // (void) strftime(history, 1024, "%d %b %Y : ", date_and_time);
-      // strcat(history, commandLine());
+      struct tm *date_and_time = localtime(&date_and_time_in_sec);
+      (void) strftime(history, 1024, "%d %b %Y : ", date_and_time);
+      strcat(history, commandLine());
     }
+    */
   if (Options::VersionInfo) remapAtts.cdo_version = cdoComment();
   writeRemapFileScrip(remapFileOut, remapAtts, mapType, submapType, numNeighbors, remapOrder, src_grid, tgt_grid, remapVars);
 }
@@ -729,17 +726,17 @@ Remapweights(void *argument)
   cdoInitialize(argument);
 
 #ifdef HAVE_LIBNETCDF
-  const int VERIFYWEIGHTS = cdoOperatorAdd("verifyweights", 0, 1, "remap file name");
-  const int WRITEREMAPSCRIP = cdoOperatorAdd("writeremapscrip", 0, 2, "input and output remap file name");
+  const auto VERIFYWEIGHTS = cdoOperatorAdd("verifyweights", 0, 1, "remap file name");
+  const auto WRITEREMAPSCRIP = cdoOperatorAdd("writeremapscrip", 0, 2, "input and output remap file name");
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
-  const int nfiles = cdoOperatorF2(operatorID);
+  const auto nfiles = cdoOperatorF2(operatorID);
   operatorCheckArgc(nfiles);
 
-  const char *remapFileIn = operatorArgv()[0];
-  const char *remapFileOut = (nfiles == 2) ? operatorArgv()[1] : nullptr;
+  const char *remapFileIn = cdoOperatorArgv(0).c_str();
+  const char *remapFileOut = (nfiles == 2) ? cdoOperatorArgv(1).c_str() : nullptr;
 
   // clang-format off
   if      (operatorID == VERIFYWEIGHTS) verifyWeights(remapFileIn);
