@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,37 +23,41 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_zaxis.h"
 
-#define MAX_VARS 1024
 
 void *
 Replace(void *process)
 {
+  constexpr int MaxVars = 1024;
   int varID, varID1, varID2;
   int nrecs = 0;
   int levelID, levelID2;
   int nrecs2;
   int nchvars = 0;
   int idx;
-  char varname1[CDI_MAX_NAME], varname2[CDI_MAX_NAME];
   size_t nmiss = 0;
-  int varlist1[MAX_VARS], varlist2[MAX_VARS];
+  int vars1[MaxVars], vars2[MaxVars];
   std::vector<std::vector<int>> varlevel;
   std::vector<std::vector<size_t>> varnmiss2;
-  std::vector<std::vector<double>> vardata2;
+  Varray2D<double> vardata2;
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
 
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto streamID2 = cdoOpenRead(1);
+
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
 
   VarList varList1, varList2;
   varListInit(varList1, vlistID1);
@@ -61,41 +65,38 @@ Replace(void *process)
 
   // compare all variables in vlistID2
 
-  const int nvars1 = vlistNvars(vlistID1);
-  const int nvars2 = vlistNvars(vlistID2);
+  const auto nvars1 = vlistNvars(vlistID1);
+  const auto nvars2 = vlistNvars(vlistID2);
 
   for (varID2 = 0; varID2 < nvars2; varID2++)
     {
-      vlistInqVarName(vlistID2, varID2, varname2);
-
       for (varID1 = 0; varID1 < nvars1; varID1++)
         {
-          vlistInqVarName(vlistID1, varID1, varname1);
-          if (strcmp(varname1, varname2) == 0) break;
+          if (cdo_cmpstr(varList1[varID1].name, varList2[varID2].name)) break;
         }
 
       if (varID1 < nvars1)
         {
-          const size_t gridsize1 = varList1[varID1].gridsize;
-          const int nlevel1 = varList1[varID1].nlevels;
+          const auto gridsize1 = varList1[varID1].gridsize;
+          const auto nlevel1 = varList1[varID1].nlevels;
 
-          const size_t gridsize2 = varList2[varID2].gridsize;
-          const int nlevel2 = varList2[varID2].nlevels;
+          const auto gridsize2 = varList2[varID2].gridsize;
+          const auto nlevel2 = varList2[varID2].nlevels;
 
           if (gridsize1 != gridsize2) cdoAbort("Variables have different gridsize!");
 
           if (nlevel1 < nlevel2) cdoAbort("Variables have different number of levels!");
 
-          if (Options::cdoVerbose) cdoPrint("Variable %s replaced.", varname1);
+          if (Options::cdoVerbose) cdoPrint("Variable %s replaced.", varList1[varID1].name);
 
-          varlist1[nchvars] = varID1;
-          varlist2[nchvars] = varID2;
+          vars1[nchvars] = varID1;
+          vars2[nchvars] = varID2;
           nchvars++;
-          if (nchvars > MAX_VARS) cdoAbort("Internal problem - too many variables!");
+          if (nchvars > MaxVars) cdoAbort("Internal problem - too many variables!");
         }
       else
         {
-          cdoPrint("Variable %s not found!", varname2);
+          cdoPrint("Variable %s not found!", varList2[varID2].name);
         }
     }
 
@@ -106,11 +107,11 @@ Replace(void *process)
       varlevel.resize(nchvars);
       for (idx = 0; idx < nchvars; idx++)
         {
-          varID1 = varlist1[idx];
-          varID2 = varlist2[idx];
-          const int nlevel1 = varList1[varID1].nlevels;
-          const int nlevel2 = varList2[varID2].nlevels;
-          const size_t gridsize = varList2[varID2].gridsize;
+          varID1 = vars1[idx];
+          varID2 = vars2[idx];
+          const auto nlevel1 = varList1[varID1].nlevels;
+          const auto nlevel2 = varList2[varID2].nlevels;
+          const auto gridsize = varList2[varID2].gridsize;
           vardata2[idx].resize(nlevel2 * gridsize);
           varnmiss2[idx].resize(nlevel2);
           varlevel[idx].resize(nlevel1);
@@ -120,8 +121,7 @@ Replace(void *process)
           */
           if (nlevel2 <= nlevel1)
             {
-              std::vector<double> level1(nlevel1);
-              std::vector<double> level2(nlevel2);
+              Varray<double> level1(nlevel1), level2(nlevel2);
               cdoZaxisInqLevels(vlistInqVarZaxis(vlistID1, varID1), level1.data());
               cdoZaxisInqLevels(vlistInqVarZaxis(vlistID2, varID2), level2.data());
 
@@ -137,23 +137,24 @@ Replace(void *process)
                         break;
                       }
 
-                  if (l1 == nlevel1) cdoWarning("Level %g not found!", level2[l2]);
+                  if (l1 == nlevel1)
+                    cdoWarning("Variable %s on level %g not found!", varList2[varID2].name, level2[l2]);
                 }
             }
         }
     }
 
-  const int vlistID3 = vlistDuplicate(vlistID1);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
 
   vlistDefTaxis(vlistID3, taxisID3);
   cdoDefVlist(streamID3, vlistID3);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsize);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
 
-  const int nts2 = vlistNtsteps(vlistID2);
+  const auto nts2 = vlistNtsteps(vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -170,9 +171,9 @@ Replace(void *process)
               cdoInqRecord(streamID2, &varID, &levelID);
 
               for (idx = 0; idx < nchvars; idx++)
-                if (varlist2[idx] == varID)
+                if (vars2[idx] == varID)
                   {
-                    const size_t offset = varList2[varID].gridsize * levelID;
+                    const auto offset = varList2[varID].gridsize * levelID;
                     cdoReadRecord(streamID2, &vardata2[idx][offset], &nmiss);
                     varnmiss2[idx][levelID] = nmiss;
                     break;
@@ -186,15 +187,15 @@ Replace(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          double *parray = array.data();
+          auto parray = array.data();
 
           for (idx = 0; idx < nchvars; idx++)
-            if (varlist1[idx] == varID)
+            if (vars1[idx] == varID)
               {
                 levelID2 = varlevel[idx][levelID];
                 if (levelID2 != -1)
                   {
-                    const size_t offset = varList1[varID].gridsize * levelID2;
+                    const auto offset = varList1[varID].gridsize * levelID2;
                     parray = &vardata2[idx][offset];
                     nmiss = varnmiss2[idx][levelID2];
                     break;
@@ -216,5 +217,5 @@ Replace(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
