@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,11 +25,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
 
-#include "listarray.h"
-
 void *
 Replacevalues(void *process)
 {
@@ -37,62 +35,64 @@ Replacevalues(void *process)
   int varID, levelID;
   size_t nmiss;
   int nvals = 0;
-  ListArray<double> listArrayFlt;
-  double *fltarr = nullptr;
+  std::vector<double> fltarr;
   double rmin = 0, rmax = 0;
   double newval = 0, newval2 = 0;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int SETVALS  = cdoOperatorAdd("setvals",  0, 0, "I1,O1,...,In,On");
-  const int SETRTOC  = cdoOperatorAdd("setrtoc",  0, 0, "range (min, max), value");
-  const int SETRTOC2 = cdoOperatorAdd("setrtoc2", 0, 0, "range (min, max), value1, value2");
+  const auto SETVALS  = cdoOperatorAdd("setvals",  0, 0, "I1,O1,...,In,On");
+  const auto SETRTOC  = cdoOperatorAdd("setrtoc",  0, 0, "range (min, max), value");
+  const auto SETRTOC2 = cdoOperatorAdd("setrtoc2", 0, 0, "range (min, max), value1, value2");
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   if (operatorID == SETVALS)
     {
-      nvals = listArrayFlt.argvToFlt(operatorArgc(), operatorArgv());
+      fltarr = cdoArgvToFlt(cdoGetOperArgv());
+      nvals = fltarr.size();
       if (nvals < 2) cdoAbort("Too few arguments!");
       if (nvals % 2 != 0) cdoAbort("Need pairs of arguments!");
-      fltarr = listArrayFlt.data();
       nvals = nvals / 2;
     }
   else if (operatorID == SETRTOC)
     {
       operatorCheckArgc(3);
-      rmin = parameter2double(operatorArgv()[0]);
-      rmax = parameter2double(operatorArgv()[1]);
-      newval = parameter2double(operatorArgv()[2]);
+      rmin = parameter2double(cdoOperatorArgv(0));
+      rmax = parameter2double(cdoOperatorArgv(1));
+      newval = parameter2double(cdoOperatorArgv(2));
     }
   else if (operatorID == SETRTOC2)
     {
       operatorCheckArgc(4);
-      rmin = parameter2double(operatorArgv()[0]);
-      rmax = parameter2double(operatorArgv()[1]);
-      newval = parameter2double(operatorArgv()[2]);
-      newval2 = parameter2double(operatorArgv()[3]);
+      rmin = parameter2double(cdoOperatorArgv(0));
+      rmax = parameter2double(cdoOperatorArgv(1));
+      newval = parameter2double(cdoOperatorArgv(2));
+      newval2 = parameter2double(cdoOperatorArgv(3));
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
+
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -105,21 +105,19 @@ Replacevalues(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
-          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          const double missval = vlistInqVarMissval(vlistID1, varID);
+          const auto gridsize = varList1[varID].gridsize;
+          const auto missval = varList1[varID].missval;
 
           if (operatorID == SETVALS)
             {
               for (size_t i = 0; i < gridsize; i++)
                 if (!DBL_IS_EQUAL(array[i], missval))
                   {
-                    // printf("\nelem %d val %f ",i,array[i]);
                     for (int j = 0; j < nvals; j++)
                       {
                         if (DBL_IS_EQUAL(array[i], fltarr[j * 2]))
                           {
                             array[i] = fltarr[j * 2 + 1];
-                            // printf("j=%d %f %f",j,fltarr[j*2],fltarr[j*2+1]);
                             break;
                           }
                       }
