@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,13 +21,16 @@
       Rhopot      rhopot          potential density
 */
 
+#include <algorithm>
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
-#include "cdo_math.h"
 #include <mpim_grid.h>
 #include "util_string.h"
+#include "cdo_zaxis.h"
 
 /*
 !>
@@ -65,8 +68,8 @@ potrho_1(const double t, const double sal, const double p)
          r_ak0 = 8.50935e-5, r_ak1 = -6.12293e-6, r_ak2 = 5.2787e-8,
          r_am0 = -9.9348e-7, r_am1 = 2.0816e-8, r_am2 = 9.1697e-10;
 
-  double s = cdo::max(sal, 0.0);
-  double s3h = sqrt(s*s*s);
+  double s = std::max(sal, 0.0);
+  double s3h = std::sqrt(s*s*s);
 
   double rho = (r_a0 + t * (r_a1 + t * (r_a2 + t * (r_a3 + t * (r_a4 + t * r_a5))))
             + s * (r_b0 + t * (r_b1 + t * (r_b2 + t * (r_b3 + t * r_b4))))    
@@ -123,7 +126,8 @@ r[i]-x[i]);
 */
 
 static void
-calc_rhopot(size_t gridsize, size_t nlevel, const std::vector<double> &pressure, const FieldVector &to, const FieldVector &sao, FieldVector &rho)
+calc_rhopot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, const FieldVector &to, const FieldVector &sao,
+            FieldVector &rho)
 {
   // pressure units: hPa
   // to units:       Celsius
@@ -131,12 +135,12 @@ calc_rhopot(size_t gridsize, size_t nlevel, const std::vector<double> &pressure,
 
   for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const auto &tovec = to[levelID].vec;
-      const auto &saovec = sao[levelID].vec;
-      auto &rhovec = rho[levelID].vec;
-      const double to_missval = to[levelID].missval;
-      const double sao_missval = sao[levelID].missval;
-      const double rho_missval = rho[levelID].missval;
+      const auto &tovec = to[levelID].vec_d;
+      const auto &saovec = sao[levelID].vec_d;
+      auto &rhovec = rho[levelID].vec_d;
+      const auto to_missval = to[levelID].missval;
+      const auto sao_missval = sao[levelID].missval;
+      const auto rho_missval = rho[levelID].missval;
       for (size_t i = 0; i < gridsize; ++i)
         {
           if (DBL_IS_EQUAL(tovec[i], to_missval) || DBL_IS_EQUAL(saovec[i], sao_missval))
@@ -163,12 +167,12 @@ Rhopot(void *process)
 
   cdoInitialize(process);
 
-  if (operatorArgc() == 1) pin = parameter2double(operatorArgv()[0]);
+  if (operatorArgc() == 1) pin = parameter2double(cdoOperatorArgv(0));
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   for (varID = 0; varID < nvars; varID++)
     {
@@ -176,22 +180,25 @@ Rhopot(void *process)
       if (code <= 0)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          vlistInqVarStdname(vlistID1, varID, stdname);
           cstrToLowerCase(varname);
 
-          if (cstrIsEqual(varname, "to"))
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID1, varID, CDI_KEY_STDNAME, stdname, &length);
+          cstrToLowerCase(stdname);
+
+          if (cdo_cmpstr(varname, "to"))
             code = 20;
-          else if (cstrIsEqual(varname, "sao"))
+          else if (cdo_cmpstr(varname, "sao"))
             code = 5;
-          else if (cstrIsEqual(varname, "tho"))
+          else if (cdo_cmpstr(varname, "tho"))
             code = 2;
-          else if (cstrIsEqual(varname, "s"))
+          else if (cdo_cmpstr(varname, "s"))
             code = 5;
-          else if (cstrIsEqual(varname, "t"))
+          else if (cdo_cmpstr(varname, "t"))
             code = 2;
-          else if (cstrIsEqual(stdname, "sea_water_salinity"))
+          else if (cdo_cmpstr(stdname, "sea_water_salinity"))
             code = 5;
-          else if (cstrIsEqual(stdname, "sea_water_potential_temperature"))
+          else if (cdo_cmpstr(stdname, "sea_water_potential_temperature"))
             code = 2;
         }
 
@@ -212,18 +219,18 @@ Rhopot(void *process)
     }
   if (toID == -1) cdoAbort("In-situ temperature not found!");
 
-  const int gridID = vlistGrid(vlistID1, 0);
-  const size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridID = vlistGrid(vlistID1, 0);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
   zaxisID = vlistInqVarZaxis(vlistID1, saoID);
-  const int nlevel1 = zaxisInqSize(zaxisID);
+  const auto nlevel1 = zaxisInqSize(zaxisID);
   zaxisID = vlistInqVarZaxis(vlistID1, toID);
-  const int nlevel2 = zaxisInqSize(zaxisID);
+  const auto nlevel2 = zaxisInqSize(zaxisID);
 
   if (nlevel1 != nlevel2) cdoAbort("temperature and salinity have different number of levels!");
-  const int nlevel = nlevel1;
+  const auto nlevel = nlevel1;
 
-  std::vector<double> pressure(nlevel);
+  Varray<double> pressure(nlevel);
   cdoZaxisInqLevels(zaxisID, pressure.data());
 
   if (pin >= 0)
@@ -252,21 +259,21 @@ Rhopot(void *process)
   if (vlistInqVarDatatype(vlistID1, toID) == CDI_DATATYPE_FLT64 && vlistInqVarDatatype(vlistID1, saoID) == CDI_DATATYPE_FLT64)
     datatype = CDI_DATATYPE_FLT64;
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   varID = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
   vlistDefVarParam(vlistID2, varID, cdiEncodeParam(18, 255, 255));
-  vlistDefVarName(vlistID2, varID, "rhopoto");
-  vlistDefVarLongname(vlistID2, varID, "Sea water potential density");
-  vlistDefVarStdname(vlistID2, varID, "sea_water_potential_density");
-  vlistDefVarUnits(vlistID2, varID, "kg m-3");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, "rhopoto");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, "Sea water potential density");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, "sea_water_potential_density");
+  cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, "kg m-3");
   vlistDefVarMissval(vlistID2, varID, rho[0].missval);
   vlistDefVarDatatype(vlistID2, varID, datatype);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -278,8 +285,8 @@ Rhopot(void *process)
       for (int recID = 0; recID < nrecs; ++recID)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          if (varID == toID) cdoReadRecord(streamID1, to[levelID].vec.data(), &to[levelID].nmiss);
-          if (varID == saoID) cdoReadRecord(streamID1, sao[levelID].vec.data(), &sao[levelID].nmiss);
+          if (varID == toID) cdoReadRecord(streamID1, to[levelID].vec_d.data(), &to[levelID].nmiss);
+          if (varID == saoID) cdoReadRecord(streamID1, sao[levelID].vec_d.data(), &sao[levelID].nmiss);
         }
 
       calc_rhopot(gridsize, nlevel, pressure, to, sao, rho);
@@ -287,7 +294,7 @@ Rhopot(void *process)
       for (levelID = 0; levelID < nlevel; ++levelID)
         {
           cdoDefRecord(streamID2, 0, levelID);
-          cdoWriteRecord(streamID2, rho[levelID].vec.data(), fieldNumMiss(rho[levelID]));
+          cdoWriteRecord(streamID2, rho[levelID].vec_d.data(), fieldNumMiss(rho[levelID]));
         }
 
       tsID++;
@@ -300,5 +307,5 @@ Rhopot(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
