@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,6 @@
 #include <mpim_grid.h>
 #include "grid_define.h"
 
-
 static void
 sampleData(const double *array1, int gridID1, double *array2, int gridID2, int resampleFactor)
 {
@@ -74,13 +73,10 @@ Samplegrid(void *process)
   int varID, levelID;
   int resampleFactor = 0;
   int subI0 = 0, subI1 = 0, subJ0 = 0, subJ1 = 0;
-  int index;
   size_t nmiss;
   struct sbox_t
   {
     int gridSrcID, gridIDsampled;
-    int *cellidx, nvals;
-    int subI0, subI1, subJ0, subJ1;
   };
 
   cdoInitialize(process);
@@ -203,6 +199,7 @@ Samplegrid(void *process)
             {
               const auto gridSrcID = vlistInqVarGrid(vlistID1, varID);
 
+              int index;
               for (index = 0; index < ngrids; index++)
                 if (gridSrcID == sbox[index].gridSrcID) break;
 
