@@ -23,8 +23,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
 #include "cdo_season.h"
+#include "datetime.h"
+#include "process_int.h"
 
 void *
 Seascount(void *process)
@@ -32,32 +33,33 @@ Seascount(void *process)
   int64_t vdate0 = 0;
   int vtime0 = 0;
   int nrecs;
-  int varID, levelID;
-  int year, month, day, seas0 = 0;
+  int seas0 = 0;
   int oldmon = 0;
 
   cdoInitialize(process);
 
   cdoOperatorAdd("seascount", 0, 0, nullptr);
 
-  const int season_start = get_season_start();
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto season_start = getSeasonStart();
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
 
   Field field;
@@ -74,14 +76,14 @@ Seascount(void *process)
       bool newseas = false;
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
         {
-          const int64_t vdate = taxisInqVdate(taxisID1);
-          const int vtime = taxisInqVtime(taxisID1);
-          cdiDecodeDate(vdate, &year, &month, &day);
+          const auto vdate = taxisInqVdate(taxisID1);
+          const auto vtime = taxisInqVtime(taxisID1);
 
-          int newmon = month;
+          const auto month = decodeMonth(vdate);
+          auto newmon = month;
           if (season_start == START_DEC && newmon == 12) newmon = 0;
 
-          const int seas = month_to_season(month);
+          const auto seas = monthToSeason(month);
 
           if (nsets == 0)
             {
@@ -97,6 +99,7 @@ Seascount(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
+              int varID, levelID;
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
@@ -110,11 +113,12 @@ Seascount(void *process)
 
               if (nsets == 0)
                 {
-                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec[i] = vars1[varID][levelID].missval;
+                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec_d[i] = vars1[varID][levelID].missval;
                   vars1[varID][levelID].nmiss = fieldsize;
                 }
 
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+              field.size = vars1[varID][levelID].size;
               field.grid = vars1[varID][levelID].grid;
               field.missval = vars1[varID][levelID].missval;
 
@@ -137,10 +141,10 @@ Seascount(void *process)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
+          cdoWriteRecord(streamID2, vars1[varID][levelID].vec_d.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
