@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,26 +32,16 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
 #include "datetime.h"
 #include "printinfo.h"
 #include "cdo_season.h"
 
-void *
-Seasstat(void *process)
+static void
+addOperators(void)
 {
-  TimeStat timestat_date = TimeStat::MEAN;
-  int64_t vdate0 = 0, vdate1 = 0;
-  int vtime0 = 0, vtime1 = 0;
-  int nrecs;
-  int varID, levelID;
-  int year, month, day, seas0 = 0;
-  int oldmon = 0;
-  int nseason = 0;
-  const char *seas_name[4];
-
-  cdoInitialize(process);
-
   // clang-format off
   cdoOperatorAdd("seasrange", func_range, 0, nullptr);
   cdoOperatorAdd("seasmin",   func_min,   0, nullptr);
@@ -63,49 +53,68 @@ Seasstat(void *process)
   cdoOperatorAdd("seasvar1",  func_var1,  0, nullptr);
   cdoOperatorAdd("seasstd",   func_std,   0, nullptr);
   cdoOperatorAdd("seasstd1",  func_std1,  0, nullptr);
-
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
-  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
   // clang-format on
+}
+
+void *
+Seasstat(void *process)
+{
+  TimeStat timestat_date = TimeStat::MEAN;
+  int64_t vdate0 = 0, vdate1 = 0;
+  int vtime0 = 0, vtime1 = 0;
+  int nrecs;
+  int seas0 = 0;
+  int oldmon = 0;
+  int nseason = 0;
+  const char *seas_name[4];
+
+  cdoInitialize(process);
+
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+
+  operatorCheckArgc(0);
+
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
   const bool lvars2 = lvarstd || lrange;
 
-  const int season_start = get_season_start();
-  get_season_name(seas_name);
+  const auto season_start = getSeasonStart();
+  getSeasonName(seas_name);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   FieldVector2D samp1, vars1, vars2;
-  fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
+  fieldsFromVlist(vlistID1, samp1);
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
   if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
@@ -118,16 +127,14 @@ Seasstat(void *process)
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
         {
           dtlist.taxisInqTimestep(taxisID1, nsets);
-          int64_t vdate = dtlist.getVdate(nsets);
-          int vtime = dtlist.getVtime(nsets);
-
-          cdiDecodeDate(vdate, &year, &month, &day);
+          const auto vdate = dtlist.getVdate(nsets);
+          const auto vtime = dtlist.getVtime(nsets);
 
+          const auto month = decodeMonth(vdate);
           int newmon = month;
-
           if (season_start == START_DEC && newmon == 12) newmon = 0;
 
-          int seas = month_to_season(month);
+          const auto seas = monthToSeason(month);
 
           if (nsets == 0)
             {
@@ -146,6 +153,7 @@ Seasstat(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
+              int varID, levelID;
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
@@ -155,31 +163,30 @@ Seasstat(void *process)
                   recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
-              Field &rsamp1 = samp1[varID][levelID];
-              Field &rvars1 = vars1[varID][levelID];
+              auto &rsamp1 = samp1[varID][levelID];
+              auto &rvars1 = vars1[varID][levelID];
 
               const size_t fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+                  cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
                   if (lrange)
                     {
                       vars2[varID][levelID].nmiss = rvars1.nmiss;
-                      vars2[varID][levelID].vec = rvars1.vec;
+                      vars2[varID][levelID].vec_d = rvars1.vec_d;
                     }
 
                   if (rvars1.nmiss || !rsamp1.empty())
                     {
                       if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                      for (size_t i = 0; i < fieldsize; i++)
-                        rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                      for (size_t i = 0; i < fieldsize; i++) rsamp1.vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                     }
                 }
               else
                 {
-                  cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                  cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
                   field.size = fieldsize;
                   field.grid = rvars1.grid;
                   field.missval = rvars1.missval;
@@ -189,7 +196,7 @@ Seasstat(void *process)
                       if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
                       for (size_t i = 0; i < fieldsize; i++)
-                        if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
+                        if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i]++;
                     }
 
                   if (lvarstd)
@@ -214,8 +221,8 @@ Seasstat(void *process)
               {
                 if (recList[recID].lconst) continue;
 
-                const int varID = recList[recID].varID;
-                const int levelID = recList[recID].levelID;
+                const auto varID = recList[recID].varID;
+                const auto levelID = recList[recID].levelID;
                 vfarmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
@@ -231,10 +238,10 @@ Seasstat(void *process)
         {
           if (recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rsamp1 = samp1[varID][levelID];
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rsamp1 = samp1[varID][levelID];
+          auto &rvars1 = vars1[varID][levelID];
 
           if (lmean)
             {
@@ -245,7 +252,7 @@ Seasstat(void *process)
             }
           else if (lvarstd)
             {
-              Field &rvars2 = vars2[varID][levelID];
+              auto &rvars2 = vars2[varID][levelID];
               if (!rsamp1.empty())
                 {
                   if (lstd)
@@ -269,37 +276,27 @@ Seasstat(void *process)
         }
 
       if (Options::cdoVerbose)
-        {
-          char vdatestr0[32], vtimestr0[32];
-          char vdatestr1[32], vtimestr1[32];
-          date2str(vdate0, vdatestr0, sizeof(vdatestr0));
-          time2str(vtime0, vtimestr0, sizeof(vtimestr0));
-          date2str(vdate1, vdatestr1, sizeof(vdatestr1));
-          time2str(vtime1, vtimestr1, sizeof(vtimestr1));
-          cdoPrint("season: %3d %3s  start: %s %s  end: %s %s ntimesteps: %ld", nseason, seas_name[seas0], vdatestr0, vtimestr0,
-                   vdatestr1, vtimestr1, nsets);
-        }
+        cdoPrint("season: %3d %3s  start: %s %s  end: %s %s ntimesteps: %ld", nseason, seas_name[seas0],
+                 dateToString(vdate0).c_str(), timeToString(vtime0).c_str(), dateToString(vdate1).c_str(),
+                 timeToString(vtime1).c_str(), nsets);
 
       dtlist.statTaxisDefTimestep(taxisID2, nsets);
       cdoDefTimestep(streamID2, otsID);
 
       if (nsets < 3)
-        {
-          char vdatestr[32];
-          date2str(vdate0, vdatestr, sizeof(vdatestr));
-          cdoWarning("Season %3d (%s) has only %d input time step%s!", otsID + 1, vdatestr, nsets, nsets == 1 ? "" : "s");
-        }
+        cdoWarning("Season %3d (%s) has only %d input time step%s!", otsID + 1, dateToString(vdate0).c_str(), nsets,
+                   nsets == 1 ? "" : "s");
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rvars1 = vars1[varID][levelID];
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
         }
 
       if (nrecs == 0) break;
