@@ -17,9 +17,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
-#include "listarray.h"
+#include "process_int.h"
+#include "cdo_zaxis.h"
+#include "readline.h"
+#include "cdi_lockedIO.h"
 
 // NOTE: All operators in this module works only on GRIB edition 1 files!
 
@@ -123,13 +124,13 @@ ECMWF_H11_test4.grb ECMWF_H11_test.grb
 
 struct TUPLEREC
 {
-  ListArray<int> *codeLST;
+  std::vector<int> codeLST;
   int ncodes;
 
-  ListArray<int> *levelTypeLST;
+  std::vector<int> levelTypeLST;
   int nlevelTypes;
 
-  ListArray<int> *levelLST;
+  std::vector<int> levelLST;
   int nlevels;
   int sel_or_del_or_change;  // sel_or_del_or_change:  0:  operator decides,
                              // 1:select , 2:delete, 3:change
@@ -142,9 +143,7 @@ struct TUPLEREC
   int changedLevel;
 };
 
-int push_backIntList(int value, ListArray<int> *list, int arraylen);
-int checkListContainsInt(int value, ListArray<int> *list, int arraylen);
-int getIntFromList(int index, ListArray<int> *list, int arraylen);
+int checkListContainsInt(int value, std::vector<int> &list, int arraylen);
 
 #define MAX_TUPLES 1000
 static int NUMTUPLES = 0;
@@ -180,23 +179,23 @@ Selmulti(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int SELMULTI    = cdoOperatorAdd("selmulti",    0, 0, "filename/string with selection specification ");
-  int DELMULTI    = cdoOperatorAdd("delmulti",    0, 0, "filename/string with selection specification ");
-  int CHANGEMULTI = cdoOperatorAdd("changemulti", 0, 0, "filename/string with selection specification ");
+  const auto SELMULTI    = cdoOperatorAdd("selmulti",    0, 0, "filename/string with selection specification ");
+  const auto DELMULTI    = cdoOperatorAdd("delmulti",    0, 0, "filename/string with selection specification ");
+  const auto CHANGEMULTI = cdoOperatorAdd("changemulti", 0, 0, "filename/string with selection specification ");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  char *filenameOrString;
+  const char *filenameOrString;
 
   // operatorCheckArgc(1);
-  filenameOrString = operatorArgv()[0];
+  filenameOrString = cdoOperatorArgv(0).c_str();
   if (cdoDebugExt)
     {
       printf("Given operator arguments (nr=%d): \n", operatorArgc());
-      for (int i = 0; i < operatorArgc(); i++) printf("%s", operatorArgv()[i]);
+      for (int i = 0; i < operatorArgc(); i++) printf("%s", cdoOperatorArgv(i).c_str());
       printf("\n");
     }
   if (!multiSelectionParser(filenameOrString))
@@ -204,33 +203,30 @@ Selmulti(void *process)
 
   if (operatorID == SELMULTI)
     if (getNumberOfSelectionTuples() == 0)
-      cdoAbort("Error! You must provide at lease ONE selection "
-               "tuple!\nNotations: 'SELECT,  .. or sel(/;;) or (/;;)'\nCheck "
-               "the file: %s",
+      cdoAbort("Error! You must provide at lease ONE selection tuple!\n"
+               "Notations: 'SELECT,  .. or sel(/;;) or (/;;)'\nCheck the file: %s",
                filenameOrString);
 
   if (operatorID == DELMULTI)
     if (getNumberOfDeleteSelectionTuples() == 0)
-      cdoAbort("Error! You must provide at lease ONE selection "
-               "tuple!\nNotations: 'DELETE,  .. or del(/;;) or (/;;)'\nCheck "
-               "the file: %s",
+      cdoAbort("Error! You must provide at lease ONE selection tuple!\n"
+               "Notations: 'DELETE,  .. or del(/;;) or (/;;)'\nCheck the file: %s",
                filenameOrString);
 
   if (operatorID == CHANGEMULTI)
     if (getNumberOfSelectionTuples() == 0)
-      cdoAbort("Error! You must provide at lease ONE selection "
-               "tuple!\nNotations: 'CHANGE,  .. or (/;;|;;;)'\nCheck the file: "
-               "%s",
+      cdoAbort("Error! You must provide at lease ONE selection tuple!\n"
+               "Notations: 'CHANGE,  .. or (/;;|;;;)'\nCheck the file: %s",
                filenameOrString);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
   vlistClearFlag(vlistID1);
-  int nvars = vlistNvars(vlistID1);
+  auto nvars = vlistNvars(vlistID1);
 
-  Debug(cdoDebugExt," Total number of variables: %d", nvars);
+  Debug(cdoDebugExt, " Total number of variables: %d", nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
@@ -241,7 +237,7 @@ Selmulti(void *process)
 
       for (levelID = 0; levelID < nlevs; levelID++)
         {
-          double level = zaxisInqLevel(zaxisID, levelID);
+          const double level = zaxisInqLevel(zaxisID, levelID);
 
           if (operatorID == DELMULTI)
             vlistDefFlag(vlistID1, varID, levelID, true);  // set initially, override bellow if in selection
@@ -360,9 +356,9 @@ Selmulti(void *process)
         }      // end for ( levelID
     }          // end for ( varID
 
-  Debug(cdoDebugExt," Writing the selected fields ...");
+  Debug(cdoDebugExt, " Writing the selected fields ...");
 
-  int vlistID2 = vlistCreate();
+  auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
 
   nvars = vlistNvars(vlistID2);
@@ -370,22 +366,23 @@ Selmulti(void *process)
     if (vlistInqVarTimetype(vlistID2, varID) != TIME_CONSTANT) break;
   if (varID == nvars) vlistDefNtsteps(vlistID2, 0);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int nrecs = vlistNrecs(vlistID2);
-
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  auto gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-  double *array = (double *) malloc(gridsize * sizeof(double));
+  Varray<double> array(gridsize);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
@@ -401,7 +398,7 @@ Selmulti(void *process)
               offset = 0.0;
               code = vlistInqVarCode(vlistID1, varID);
               zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              double level = zaxisInqLevel(zaxisID, levelID);
+              const auto level = zaxisInqLevel(zaxisID, levelID);
               ltype = zaxis2ltype(zaxisID);
               for (int ii = 0; ii < NUMTUPLES; ii++)
                 {
@@ -440,8 +437,7 @@ Selmulti(void *process)
                                      "selltype %d, sellevel %d)",
                                      code, ltype, (int) level, nvars, varID, selcode, selltype, sellevel);
                         }
-                      simpleMath = tuplerec->simpleMath;  // 1:  simple array
-                                                          // arithmetics ( *,+),
+                      simpleMath = tuplerec->simpleMath;  // 1:  simple array arithmetics ( *,+),
                                                           // 0: do nothing
                       if (simpleMath)
                         {
@@ -475,14 +471,14 @@ Selmulti(void *process)
                 }
               else
                 {
-                  cdoReadRecord(streamID1, array, &nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
 
                   if (!simpleMath)
                     {
                       if (cdoDebugExt)
                         cdoPrint(" Writing record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", recID,
                                  code, ltype, (int) (level), varID, levelID);
-                      cdoWriteRecord(streamID2, array, nmiss);
+                      cdoWriteRecord(streamID2, array.data(), nmiss);
                     }
                   else  // 1:  simple array arithmetics ( *,+)
                     {
@@ -497,41 +493,25 @@ Selmulti(void *process)
                             // If SCALE and/or OFFSET are defined, then the data
                             // values are scaled as SCALE*(VALUE-OFFSET).
                           }
-                      cdoWriteRecord(streamID2, array, nmiss);
+                      cdoWriteRecord(streamID2, array.data(), nmiss);
                     }
                 }  // end of else ( lcopy )
             }      // end if ( vlistInqFlag(vlistID1, varID, levelID) == true )
         }          // end for ( recID ..
 
-      /*if ( vlistInqFlag(vlistID1, varID, levelID) == false )
-        if (operatorID == DELMULTI)
-        {
-        code    = vlistInqVarCode(vlistID1, varID);
-        zaxisID = vlistInqVarZaxis(vlistID1, varID);
-        level = zaxisInqLevel(zaxisID, levelID);
-        ltype = zaxis2ltype(zaxisID);
-        if ( cdoDebugExt ) cdoPrint(" Removing record [%4d] with (code
-        %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]",recID, code,
-        ltype, (int)(level), varID, levelID);
-        }
-        else*/
-
       tsID++;
-    }  // end while ( (nrecs
+    }  // end while
 
   cdoStreamClose(streamID1);
   cdoStreamClose(streamID2);
 
   vlistDestroy(vlistID2);
 
-  if (!lcopy)
-    if (array) free(array);
-
   cdoFinish();
 
   cdoDebugExt = 0;
 
-  return 0;
+  return nullptr;
 }
 
 TUPLEREC *
@@ -539,13 +519,10 @@ TUPLERECNew()
 {
   TUPLEREC *tpl = (TUPLEREC *) malloc(sizeof(TUPLEREC));
 
-  tpl->codeLST = new ListArray<int>;
   tpl->ncodes = 0;
 
-  tpl->levelTypeLST = new ListArray<int>;
   tpl->nlevelTypes = 0;
 
-  tpl->levelLST = new ListArray<int>;
   tpl->nlevels = 0;
   tpl->sel_or_del_or_change = 0;  // sel_or_del_or_change:  0:  operator
                                   // decides, 1:select , 2:delete, 3:change
@@ -582,19 +559,10 @@ getSelTuple(int index)
 }
 
 int
-push_backIntList(int value, ListArray<int> *list, int arraylen)
-{
-  list->setValue(arraylen, value);
-  arraylen += 1;
-  return arraylen;
-}
-
-int
-checkListContainsInt(int value, ListArray<int> *list, int arraylen)
+checkListContainsInt(int value, std::vector<int> &list, int arraylen)
 // Note: When the list is Empty it also returns true !
 {
   int i;
-  int *array = list->data();
 
   if (arraylen == 0)
     {
@@ -603,14 +571,14 @@ checkListContainsInt(int value, ListArray<int> *list, int arraylen)
     }
   for (i = 0; i < arraylen; i++)
     {
-      if (array[i] == -1)
+      if (list[i] == -1)
         {  // this is for '*' selection; can be any code, any level or any
            // level-type
           // if ( cdoDebugExt ) cdoPrint(" value=%d: found.");
           return 1;
         }
 
-      if (array[i] == value)
+      if (list[i] == value)
         {
           // if ( cdoDebugExt ) cdoPrint(" value=%d: found.");
           return 1;
@@ -620,16 +588,6 @@ checkListContainsInt(int value, ListArray<int> *list, int arraylen)
   return 0;
 }
 
-int
-getIntFromList(int index, ListArray<int> *list, int arraylen)
-{
-  int *array = list->data();
-  if (index < arraylen)
-    return array[index];
-  else
-    return -999;
-}
-
 #define MAX_LINE_LEN 65536
 
 static char *
@@ -670,7 +628,7 @@ goToNextSeparator(char *pline)
         }
     }
   if (separatorFound) pline++;
-  Debug(cdoDebugExt >= 100,"goToNextSeparator():  pline= ('%s') ", pline);
+  Debug(cdoDebugExt >= 100, "goToNextSeparator():  pline= ('%s') ", pline);
   // while ( isspace((int) *pline) ) pline++;
   pline = removeSpaces(pline);
   return pline;
@@ -696,12 +654,12 @@ strContains(char *str, const char *substr)
   char *rv = strstr(str, substr);
   if (rv)
     {
-      Debug(cdoDebugExt >= 100,"strContains():  substr('%s') FOUND in str('%s')", substr, str);
+      Debug(cdoDebugExt >= 100, "strContains():  substr('%s') FOUND in str('%s')", substr, str);
       return (rv + lensub);  // points after subStr ..
     }
   else
     {
-      Debug(cdoDebugExt >= 100,"strContains():  substr('%s') NOT found in str('%s')", substr, str);
+      Debug(cdoDebugExt >= 100, "strContains():  substr('%s') NOT found in str('%s')", substr, str);
       return rv;
     }
 }
@@ -719,10 +677,10 @@ findParamEnd(char *str)
   if (ptrEnding != nullptr)
     {
       ptrEnding = removeSpaces(ptrEnding);
-      Debug(cdoDebugExt >= 100," ptrEnding='%s'", ptrEnding);
+      Debug(cdoDebugExt >= 100, " ptrEnding='%s'", ptrEnding);
       return ptrEnding;
     }
-  Debug(cdoDebugExt >= 100," ptrEnding=end-of-string");
+  Debug(cdoDebugExt >= 100, " ptrEnding=end-of-string");
   size_t lenstr = strlen(str);
   ptrEnding = str + lenstr;
   ptrEnding = removeSpaces(ptrEnding);
@@ -742,10 +700,10 @@ findTupleEnd(char *str)
   if (ptrEnding != nullptr)
     {
       ptrEnding = removeSpaces(ptrEnding);
-      Debug(cdoDebugExt >= 100," findTupleEnd='%s'", ptrEnding);
+      Debug(cdoDebugExt >= 100, " findTupleEnd='%s'", ptrEnding);
       return ptrEnding;
     }
-  Debug(cdoDebugExt >= 100," findTupleEnd=end-of-string");
+  Debug(cdoDebugExt >= 100, " findTupleEnd=end-of-string");
   size_t lenstr = strlen(str);
   ptrEnding = str + lenstr;
   ptrEnding = removeSpaces(ptrEnding);
@@ -757,7 +715,7 @@ readlineForParsing(FILE *gfp, char *strToParsePtr, char *line)
 {
   if (gfp != nullptr)  // file is open => we parse text from a file
     {
-      int status = readline(gfp, line, MAX_LINE_LEN);
+      int status = cdo::readline(gfp, line, MAX_LINE_LEN);
       return (status == 0) ? nullptr : line;
     }
   else if (strToParsePtr != nullptr)  // we parse a given string
@@ -767,14 +725,14 @@ readlineForParsing(FILE *gfp, char *strToParsePtr, char *line)
       if (strlen(strToParsePtr) > 0) tpEnd = findTupleEnd(strToParsePtr);
       if (tpEnd == nullptr)
         {
-          Debug(cdoDebugExt >= 100,"%s(): End of selection string reached.", __func__);
+          Debug(cdoDebugExt >= 100, "%s(): End of selection string reached.", __func__);
           return nullptr;
         }
       else
         {
           tpEnd[0] = 0;
           if (strlen(strToParsePtr) <= MAX_LINE_LEN) strcpy(line, strToParsePtr);
-          Debug(cdoDebugExt >= 100,"%s(): Current selection line=%s", __func__, line);
+          Debug(cdoDebugExt >= 100, "%s(): Current selection line=%s", __func__, line);
           strToParsePtr = tpEnd + 1;
           return strToParsePtr;
         }
@@ -817,12 +775,12 @@ multiSelectionParser(const char *filenameOrString)
       if (strToParsePtr[0] == '{') strToParsePtr++;
       int strLn = strlen(strToParsePtr);
       if (strToParsePtr[strLn - 1] == '}') strToParsePtr[strLn - 1] = 0;
-      Debug(cdoDebugExt," Parsing selection string:  %s", strToParsePtr);
+      Debug(cdoDebugExt, " Parsing selection string:  %s", strToParsePtr);
     }
   else
     {
       gfp = fopen(filenameOrString, "r");
-      Debug(cdoDebugExt," Parsing file:  %s", filenameOrString);
+      Debug(cdoDebugExt, " Parsing file:  %s", filenameOrString);
       if (gfp == nullptr)
         {
           cdoAbort(" Missing file:  %s", filenameOrString);
@@ -856,7 +814,7 @@ multiSelectionParser(const char *filenameOrString)
         }
       if (strpos != nullptr)  // we have SELECT ..
         {
-          Debug(cdoDebugExt," Parsing notation SELECT: %s", strpos);
+          Debug(cdoDebugExt, " Parsing notation SELECT: %s", strpos);
           pline = strpos;
           tuplerec = TUPLERECNew();
           tuplerec->sel_or_del_or_change = selectionRec;
@@ -864,16 +822,15 @@ multiSelectionParser(const char *filenameOrString)
           // SELECT, PARAMETER=11/17, LEVTYPE=105, LEVEL=2
           while ((pline != nullptr) && (strlen(pline) != 0))
             {
-              Debug(cdoDebugExt >= 100,"pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "pline='%s'", pline);
               strpos = strContains(pline, "PARAMETER=");
               if ((strpos) != nullptr)
                 {
-                  Debug(cdoDebugExt >= 100,": PARAMETER=%s", strpos);
+                  Debug(cdoDebugExt >= 100, ": PARAMETER=%s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
-                  if ((!parEnd) && (!pline))
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
@@ -881,24 +838,24 @@ multiSelectionParser(const char *filenameOrString)
                         val = -1;
                       else
                         val = atoi(pline);
-                      Debug(cdoDebugExt >= 100,"code=%d", val);
-                      tuplerec->ncodes = push_backIntList(val, tuplerec->codeLST, tuplerec->ncodes);
+                      Debug(cdoDebugExt >= 100, "code=%d", val);
+                      tuplerec->codeLST.push_back(val);
+                      tuplerec->ncodes = tuplerec->codeLST.size();
                       strpos = goToNextSeparator(pline);
                       pline = strpos;
                       if (!strpos) break;
                       pline = skipSeparator(strpos);
                     }
                 }
-              Debug(cdoDebugExt >= 100,"pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "pline='%s'", pline);
               strpos = strContains(pline, "LEVTYPE=");
               if ((strpos) != nullptr)
                 {
-                  Debug(cdoDebugExt >= 100,": LEVTYPE=%s", strpos);
+                  Debug(cdoDebugExt >= 100, ": LEVTYPE=%s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
-                  if ((!parEnd) && (!pline))
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
@@ -915,24 +872,24 @@ multiSelectionParser(const char *filenameOrString)
                         }
                       else
                         convertLevelsHPa2Pa = 0;
-                      Debug(cdoDebugExt >= 100,"levelType=%d", val);
-                      tuplerec->nlevelTypes = push_backIntList(val, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
+                      Debug(cdoDebugExt >= 100, "levelType=%d", val);
+                      tuplerec->levelTypeLST.push_back(val);
+                      tuplerec->nlevelTypes = tuplerec->levelTypeLST.size();
                       strpos = goToNextSeparator(pline);
                       pline = strpos;
                       if (!strpos) break;
                       pline = skipSeparator(strpos);
                     }
                 }
-              Debug(cdoDebugExt >= 100,"pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "pline='%s'", pline);
               strpos = strContains(pline, "LEVEL=");
               if ((strpos) != nullptr)
                 {
-                  Debug(cdoDebugExt >= 100,": LEVEL=%s", strpos);
+                  Debug(cdoDebugExt >= 100, ": LEVEL=%s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
-                  if ((!parEnd) && (!pline))
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
@@ -944,29 +901,29 @@ multiSelectionParser(const char *filenameOrString)
                         {
                           val *= 100;
                         }
-                      Debug(cdoDebugExt >= 100,"level=%d", val);
-                      tuplerec->nlevels = push_backIntList(val, tuplerec->levelLST, tuplerec->nlevels);
+                      Debug(cdoDebugExt >= 100, "level=%d", val);
+                      tuplerec->levelLST.push_back(val);
+                      tuplerec->nlevels = tuplerec->levelLST.size();
                       strpos = goToNextSeparator(pline);
                       pline = strpos;
                       if (!strpos) break;
                       pline = skipSeparator(strpos);
                     }
                 }
-              Debug(cdoDebugExt >= 100,"pline='%s' (check SCALE=...)", pline);
+              Debug(cdoDebugExt >= 100, "pline='%s' (check SCALE=...)", pline);
               strpos = strContains(pline, "SCALE=");
               if ((strpos) != nullptr)
                 {
-                  Debug(cdoDebugExt >= 100,": SCALE= %s", strpos);
+                  Debug(cdoDebugExt >= 100, ": SCALE= %s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
-                  if ((!parEnd) && (!pline))
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       floatval = atof(pline);
-                      Debug(cdoDebugExt >= 100,"scale=%f", floatval);
+                      Debug(cdoDebugExt >= 100, "scale=%f", floatval);
                       tuplerec->simpleMath = 1;    // 1:  simple array arithmetics
                                                    // ( *,+), 0: do nothing
                       tuplerec->scale = floatval;  // tuplerec->offset = 0.0;
@@ -976,21 +933,20 @@ multiSelectionParser(const char *filenameOrString)
                       pline = skipSeparator(strpos);
                     }
                 }
-              Debug(cdoDebugExt >= 100,"pline='%s' (check OFFSET=...)", pline);
+              Debug(cdoDebugExt >= 100, "pline='%s' (check OFFSET=...)", pline);
               strpos = strContains(pline, "OFFSET=");
               if ((strpos) != nullptr)
                 {
-                  Debug(cdoDebugExt >= 100,": OFFSET= %s", strpos);
+                  Debug(cdoDebugExt >= 100, ": OFFSET= %s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
-                  if ((!parEnd) && (!pline))
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       floatval = atof(pline);
-                      Debug(cdoDebugExt >= 100,"offset=%f", floatval);
+                      Debug(cdoDebugExt >= 100, "offset=%f", floatval);
                       tuplerec->simpleMath = 1;     // 1:  simple array arithmetics
                                                     // ( *,+), 0: do nothing
                       tuplerec->offset = floatval;  // tuplerec->scale = 1.0;
@@ -1039,13 +995,12 @@ multiSelectionParser(const char *filenameOrString)
           // (33/34; 105; 10)
           while ((pline != nullptr) && (strlen(pline) != 0) && (pline[0] != ')'))
             {
-              Debug(cdoDebugExt >= 100,"[1]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "[1]: pline='%s'", pline);
               // 1st is code
               {
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
-                if ((!parEnd) && (!pline))
-                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1053,8 +1008,9 @@ multiSelectionParser(const char *filenameOrString)
                       val = -1;
                     else
                       val = atoi(pline);
-                    Debug(cdoDebugExt >= 100,"code=%d", val);
-                    tuplerec->ncodes = push_backIntList(val, tuplerec->codeLST, tuplerec->ncodes);
+                    Debug(cdoDebugExt >= 100, "code=%d", val);
+                    tuplerec->codeLST.push_back(val);
+                    tuplerec->ncodes = tuplerec->codeLST.size();
                     strpos = goToNextSeparator(pline);
                     if (!strpos)
                       {
@@ -1067,12 +1023,11 @@ multiSelectionParser(const char *filenameOrString)
                   }
               }
               // 2nd is level type
-              Debug(cdoDebugExt >= 100,"[2]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "[2]: pline='%s'", pline);
               {
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
-                if ((!parEnd) && (!pline))
-                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1090,8 +1045,9 @@ multiSelectionParser(const char *filenameOrString)
                     else
                       convertLevelsHPa2Pa = 0;
 
-                    Debug(cdoDebugExt >= 100,"levelType=%d", val);
-                    tuplerec->nlevelTypes = push_backIntList(val, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
+                    Debug(cdoDebugExt >= 100, "levelType=%d", val);
+                    tuplerec->levelTypeLST.push_back(val);
+                    tuplerec->nlevelTypes = tuplerec->levelTypeLST.size();
                     strpos = goToNextSeparator(pline);
                     if (!strpos)
                       {
@@ -1104,12 +1060,11 @@ multiSelectionParser(const char *filenameOrString)
                   }
               }
               // 3rd is level
-              Debug(cdoDebugExt >= 100,"[3]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "[3]: pline='%s'", pline);
               {
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
-                if ((!parEnd) && (!pline))
-                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1121,11 +1076,12 @@ multiSelectionParser(const char *filenameOrString)
                       {
                         val *= 100;
                       }
-                    Debug(cdoDebugExt >= 100,"level=%d", val);
-                    tuplerec->nlevels = push_backIntList(val, tuplerec->levelLST, tuplerec->nlevels);
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100, "level=%d", val);
+                    tuplerec->levelLST.push_back(val);
+                    tuplerec->nlevels = tuplerec->levelLST.size();
+                    Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     strpos = goToNextSeparator(pline);
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (!strpos)
                       {
                         strpos = strContains(pline, "|");  // compact notation for  changemulti
@@ -1133,7 +1089,7 @@ multiSelectionParser(const char *filenameOrString)
                           pline = strpos - 1;  // strContains returns character after...
                         else
                           pline = parEnd;
-                        Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                        Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                         break;
                       }
                     else
@@ -1146,7 +1102,7 @@ multiSelectionParser(const char *filenameOrString)
               // cdo
               // changemulti,'{(134;1;*|1;105;*);{(6;1;*|6;105;*)};{(246;*;*|76;*;*)};{(247;*;*|58;*;*)};{(248;*;*|71;*;*)}'
               // fileIN fileOUT
-              Debug(cdoDebugExt >= 100,"[OPT]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100, "[OPT]: pline='%s'", pline);
               {
                 pline = removeSpaces(pline);
                 // pline points to: "=1;105;*);....."
@@ -1158,13 +1114,13 @@ multiSelectionParser(const char *filenameOrString)
                     parEnd = findParamEnd(pline);
                     if ((!parEnd) || (pline[0] == 0))
                       cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (pline[0] == '*')
                       val = -1;
                     else
                       val = atoi(pline);
                     tuplerec->changedCode = val;
-                    Debug(cdoDebugExt >= 100,"changedCode=%d", val);
+                    Debug(cdoDebugExt >= 100, "changedCode=%d", val);
                     strpos = goToNextSeparator(pline);
                     if (!strpos) cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     pline = skipSeparator(strpos);
@@ -1172,13 +1128,13 @@ multiSelectionParser(const char *filenameOrString)
                     parEnd = findParamEnd(pline);
                     if ((!parEnd) || (pline[0] == 0))
                       cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (pline[0] == '*')
                       val = -1;
                     else
                       val = atoi(pline);
                     tuplerec->changedLevelType = val;
-                    Debug(cdoDebugExt >= 100,"changedLevelType=%d", val);
+                    Debug(cdoDebugExt >= 100, "changedLevelType=%d", val);
                     strpos = goToNextSeparator(pline);
                     if (!strpos) cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     pline = skipSeparator(strpos);
@@ -1186,18 +1142,17 @@ multiSelectionParser(const char *filenameOrString)
                     parEnd = findParamEnd(pline);
                     if ((!parEnd) || (pline[0] == 0))
                       cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (pline[0] == '*')
                       val = -1;
                     else
                       val = atoi(pline);
                     tuplerec->changedLevel = val;
-                    Debug(cdoDebugExt >= 100,"changedLevel=%d", val);
+                    Debug(cdoDebugExt >= 100, "changedLevel=%d", val);
                     pline = parEnd;
                   }  // changemulti specification
                 parEnd = findParamEnd(pline);
-                if ((!parEnd) && (!pline))
-                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1209,8 +1164,9 @@ multiSelectionParser(const char *filenameOrString)
                       {
                         val *= 100;
                       }
-                    Debug(cdoDebugExt >= 100,"level=%d", val);
-                    tuplerec->nlevels = push_backIntList(val, tuplerec->levelLST, tuplerec->nlevels);
+                    Debug(cdoDebugExt >= 100, "level=%d", val);
+                    tuplerec->levelLST.push_back(val);
+                    tuplerec->nlevels = tuplerec->levelLST.size();
                     strpos = goToNextSeparator(pline);
                     if (!strpos)
                       {
@@ -1225,7 +1181,7 @@ multiSelectionParser(const char *filenameOrString)
 
             }  // end while pline
         }      // end if "("
-    }          // end while ( readline(gfp, line, MAX_LINE_LEN) )
+    }          // end while ( cdo::readline(gfp, line, MAX_LINE_LEN) )
   if (gfp != nullptr) fclose(gfp);
 
   printSelectionTuples();
@@ -1235,7 +1191,7 @@ multiSelectionParser(const char *filenameOrString)
 void
 printSelectionTuples()
 {
-  Debug(cdoDebugExt," Printing selection tuples:");
+  Debug(cdoDebugExt, " Printing selection tuples:");
 
   int ii, ri;
   for (ii = 0; ii < NUMTUPLES; ii++)
@@ -1250,7 +1206,7 @@ printSelectionTuples()
                  tuplerec->nlevels);
       for (ri = 0; ri < tuplerec->ncodes; ri++)
         {
-          sprintf(bff, "%d", tuplerec->codeLST->getValue(ri));
+          sprintf(bff, "%d", tuplerec->codeLST[ri]);
           strcat(strval, bff);
           if ((ri + 1) < tuplerec->ncodes)
             strcat(strval, "/");
@@ -1259,7 +1215,7 @@ printSelectionTuples()
         }
       for (ri = 0; ri < tuplerec->nlevelTypes; ri++)
         {
-          sprintf(bff, "%d", tuplerec->levelTypeLST->getValue(ri));
+          sprintf(bff, "%d", tuplerec->levelTypeLST[ri]);
           strcat(strval, bff);
           if ((ri + 1) < tuplerec->nlevelTypes)
             strcat(strval, "/");
@@ -1268,7 +1224,7 @@ printSelectionTuples()
         }
       for (ri = 0; ri < tuplerec->nlevels; ri++)
         {
-          sprintf(bff, "%d", tuplerec->levelLST->getValue(ri));
+          sprintf(bff, "%d", tuplerec->levelLST[ri]);
           strcat(strval, bff);
           if ((ri + 1) < tuplerec->nlevels)
             strcat(strval, "/");
@@ -1286,11 +1242,11 @@ printSelectionTuples()
       // 3:change
       if (tuplerec->sel_or_del_or_change == 1)
         {
-          Debug(cdoDebugExt," Selection tuple [%d] = %s (select)", ii, strval);
+          Debug(cdoDebugExt, " Selection tuple [%d] = %s (select)", ii, strval);
         }
       else if (tuplerec->sel_or_del_or_change == 2)
         {
-          Debug(cdoDebugExt," Selection tuple [%d] = %s (delete)", ii, strval);
+          Debug(cdoDebugExt, " Selection tuple [%d] = %s (delete)", ii, strval);
         }
       if (tuplerec->sel_or_del_or_change == 3)
         {
@@ -1306,9 +1262,8 @@ printSelectionTuples()
 int
 getNumberOfSelectionTuples()
 {
-  int ii;
   int nn = 0;
-  for (ii = 0; ii < NUMTUPLES; ii++)
+  for (int ii = 0; ii < NUMTUPLES; ii++)
     {
       TUPLEREC *tuplerec = getSelTuple(ii);
       if (tuplerec->sel_or_del_or_change != 2) nn++;
