@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,16 +23,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h" /* processSelf */
-
-#include "listarray.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "param_conversion.h"
 
 void *
 Selrec(void *process)
 {
   int nrecs;
   int varID, levelID;
-  ListArray<int> listArrayInt;
 
   cdoInitialize(process);
 
@@ -40,31 +39,30 @@ Selrec(void *process)
 
   operatorInputArg("records");
 
-  const int nsel = listArrayInt.argvToInt(operatorArgc(), operatorArgv());
-
-  int *intarr = listArrayInt.data();
+  const auto intarr = cdoArgvToInt(cdoGetOperArgv());
+  const int nsel = intarr.size();
 
   if (Options::cdoVerbose)
     {
       for (int i = 0; i < nsel; i++) cdoPrint("intarr entry: %d %d", i, intarr[i]);
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int filetype = cdoInqFiletype(streamID1);
+  const auto filetype = cdoInqFiletype(streamID1);
 
   if (filetype == CDI_FILETYPE_NC || filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C
       || filetype == CDI_FILETYPE_NC5)
     cdoAbort("This operator does not work on NetCDF data!");
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int recordID = 0;
