@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,33 +35,33 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
-#include "listarray.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_zaxis.h"
 #include "util_wildcards.h"
+#include "cdi_lockedIO.h"
+#include "param_conversion.h"
 
 void *
 Selvar(void *process)
 {
   int varID2, levelID2;
   int varID, levelID;
-  int *intarr = nullptr, nsel = 0;
-  double *fltarr = nullptr;
+  int nsel = 0;
   char paramstr[32];
   char varname[CDI_MAX_NAME];
   char stdname[CDI_MAX_NAME];
   char gridname[CDI_MAX_NAME];
   char zaxistypename[CDI_MAX_NAME];
   char zaxisname[CDI_MAX_NAME];
-  char **argnames = nullptr;
   size_t nmiss;
-  int gridnum = 0;
-  ListArray<int> listArrayInt;
-  ListArray<double> listArrayFlt;
+  std::vector<int> intarr;
+  std::vector<double> fltarr;
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
 #define INVERTS_SELECTION(id) (cdoOperatorF2(id) & 1)
 #define TAKES_STRINGS(id) (cdoOperatorF2(id) & 2)
@@ -69,50 +69,50 @@ Selvar(void *process)
 #define TAKES_FLOATS(id) (cdoOperatorF2(id) & 8)
 
   // clang-format off
-  const int SELPARAM     = cdoOperatorAdd("selparam",     0, 2,   "parameters");
-  const int SELCODE      = cdoOperatorAdd("selcode",      0, 4,   "code numbers");
-  const int SELNAME      = cdoOperatorAdd("selname",      0, 2,   "variable names");
-  const int SELSTDNAME   = cdoOperatorAdd("selstdname",   0, 2,   "standard names");
-  const int SELLEVEL     = cdoOperatorAdd("sellevel",     0, 8,   "levels");
-  const int SELLEVIDX    = cdoOperatorAdd("sellevidx",    0, 4,   "index of levels");
-  const int SELGRID      = cdoOperatorAdd("selgrid",      0, 4|2, "list of grid names or numbers");
-  const int SELZAXIS     = cdoOperatorAdd("selzaxis",     0, 4|2, "list of zaxis types or numbers");
-  const int SELZAXISNAME = cdoOperatorAdd("selzaxisname", 0, 2,   "list of zaxis names");
-  const int SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 4,   "table numbers");
-  const int DELPARAM     = cdoOperatorAdd("delparam",     1, 2|1, "parameter");
-  const int DELCODE      = cdoOperatorAdd("delcode",      1, 1,   "code numbers");
-  const int DELNAME      = cdoOperatorAdd("delname",      1, 2|1, "variable names");
-  const int SELLTYPE     = cdoOperatorAdd("selltype",     0, 4,   "GRIB level types");
+  const auto SELPARAM     = cdoOperatorAdd("selparam",     0, 2,   "parameters");
+  const auto SELCODE      = cdoOperatorAdd("selcode",      0, 4,   "code numbers");
+  const auto SELNAME      = cdoOperatorAdd("selname",      0, 2,   "variable names");
+  const auto SELSTDNAME   = cdoOperatorAdd("selstdname",   0, 2,   "standard names");
+  const auto SELLEVEL     = cdoOperatorAdd("sellevel",     0, 8,   "levels");
+  const auto SELLEVIDX    = cdoOperatorAdd("sellevidx",    0, 4,   "index of levels");
+  const auto SELGRID      = cdoOperatorAdd("selgrid",      0, 4|2, "list of grid names or numbers");
+  const auto SELZAXIS     = cdoOperatorAdd("selzaxis",     0, 4|2, "list of zaxis types or numbers");
+  const auto SELZAXISNAME = cdoOperatorAdd("selzaxisname", 0, 2,   "list of zaxis names");
+  const auto SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 4,   "table numbers");
+  const auto DELPARAM     = cdoOperatorAdd("delparam",     1, 2|1, "parameter");
+  const auto DELCODE      = cdoOperatorAdd("delcode",      1, 1,   "code numbers");
+  const auto DELNAME      = cdoOperatorAdd("delname",      1, 2|1, "variable names");
+  const auto SELLTYPE     = cdoOperatorAdd("selltype",     0, 4,   "GRIB level types");
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   const bool ldelete = (cdoOperatorF1(operatorID) == 1);
 
-  int args_are_numeric = operatorArgc() > 0 && isdigit(*operatorArgv()[0]);
+  int args_are_numeric = operatorArgc() > 0 && isdigit(cdoOperatorArgv(0)[0]);
 
+  auto argnames = cdoGetOperArgv();
   if (TAKES_STRINGS(operatorID) && !(TAKES_INTEGERS(operatorID) && args_are_numeric))
     {
       nsel = operatorArgc();
-      argnames = operatorArgv();
 
       if (Options::cdoVerbose)
         for (int i = 0; i < nsel; i++) cdoPrint("name %d = %s", i + 1, argnames[i]);
     }
   else if (TAKES_FLOATS(operatorID))
     {
-      nsel = listArrayFlt.argvToFlt(operatorArgc(), operatorArgv());
-      fltarr = listArrayFlt.data();
+      fltarr = cdoArgvToFlt(argnames);
+      nsel = fltarr.size();
 
       if (Options::cdoVerbose)
         for (int i = 0; i < nsel; i++) cdoPrint("flt %d = %g", i + 1, fltarr[i]);
     }
   else
     {
-      nsel = listArrayInt.argvToInt(operatorArgc(), operatorArgv());
-      intarr = listArrayInt.data();
+      intarr = cdoArgvToInt(argnames);
+      nsel = intarr.size();
 
       if (Options::cdoVerbose)
         for (int i = 0; i < nsel; i++) cdoPrint("int %d = %d", i + 1, intarr[i]);
@@ -125,19 +125,16 @@ Selvar(void *process)
       for (int i = 0; i < nsel; i++) selfound[i] = false;
     }
 
-  /*
-  if ( nsel == 0 )
-    cdoAbort("missing code argument!");
-  */
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars);
 
-  if (operatorID == SELGRID && !args_are_numeric && nsel == 1 && strncmp(argnames[0], "var=", 4) == 0)
+  if (operatorID == SELGRID && !args_are_numeric && nsel == 1 && argnames[0].compare(0, 4, "var=") == 0)
     {
-      const char *gridvarname = argnames[0] + 4;
+      int gridnum = 0;
+      const char *gridvarname = &argnames[0][4];
       if (*gridvarname == 0) cdoAbort("Variable name missing!");
 
       for (varID = 0; varID < nvars; varID++)
@@ -145,10 +142,10 @@ Selvar(void *process)
           vlistInqVarName(vlistID1, varID, varname);
           if (strcmp(varname, gridvarname) == 0)
             {
-              int gridID = vlistInqVarGrid(vlistID1, varID);
+              const auto gridID = vlistInqVarGrid(vlistID1, varID);
               gridnum = 1 + vlistGridIndex(vlistID1, gridID);
               args_are_numeric = true;
-              intarr = &gridnum;
+              intarr.push_back(gridnum);
               break;
             }
         }
@@ -162,24 +159,26 @@ Selvar(void *process)
       vars[varID] = ldelete;
 
       vlistInqVarName(vlistID1, varID, varname);
-      vlistInqVarStdname(vlistID1, varID, stdname);
-      const int param = vlistInqVarParam(vlistID1, varID);
-      const int code = vlistInqVarCode(vlistID1, varID);
-      const int tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const int grididx = vlistGridIndex(vlistID1, gridID);
-      const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const int zaxisidx = vlistZaxisIndex(vlistID1, zaxisID);
-      const int nlevs = zaxisInqSize(zaxisID);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(vlistID1, varID, CDI_KEY_STDNAME, stdname, &length);
+      const auto param = vlistInqVarParam(vlistID1, varID);
+      const auto code = vlistInqVarCode(vlistID1, varID);
+      const auto tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto grididx = vlistGridIndex(vlistID1, gridID);
+      const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+      const auto zaxisidx = vlistZaxisIndex(vlistID1, zaxisID);
+      const auto nlevs = zaxisInqSize(zaxisID);
       gridName(gridInqType(gridID), gridname);
-      zaxisInqName(zaxisID, zaxisname);
+      length = CDI_MAX_NAME;
+      cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, zaxisname, &length);
       zaxisName(zaxisInqType(zaxisID), zaxistypename);
 
       cdiParamToString(param, paramstr, sizeof(paramstr));
 
       for (int levID = 0; levID < nlevs; levID++)
         {
-          const double level = cdoZaxisInqLevel(zaxisID, levID);
+          const auto level = cdoZaxisInqLevel(zaxisID, levID);
 
           if (ldelete) vlistDefFlag(vlistID1, varID, levID, true);
 
@@ -201,11 +200,11 @@ Selvar(void *process)
               else if (operatorID == SELGRID && args_are_numeric)
                 found = intarr[isel] == (grididx + 1);
               else if (operatorID == SELGRID && !args_are_numeric)
-                found = memcmp(argnames[isel], gridname, strlen(argnames[isel])) == 0;
+                found = memcmp(argnames[isel].c_str(), gridname, argnames[isel].size()) == 0;
               else if (operatorID == SELZAXIS && args_are_numeric)
                 found = intarr[isel] == (zaxisidx + 1);
               else if (operatorID == SELZAXIS && !args_are_numeric)
-                found = memcmp(argnames[isel], zaxistypename, strlen(argnames[isel])) == 0;
+                found = memcmp(argnames[isel].c_str(), zaxistypename, argnames[isel].size()) == 0;
               else if (operatorID == SELZAXISNAME)
                 found = wildcardmatch(argnames[isel], zaxisname) == 0;
               else if (operatorID == SELTABNUM)
@@ -215,7 +214,7 @@ Selvar(void *process)
               else if (operatorID == DELNAME)
                 found = wildcardmatch(argnames[isel], varname) == 0;
               else if (operatorID == DELPARAM)
-                found = strcmp(argnames[isel], paramstr) == 0;
+                found = strcmp(argnames[isel].c_str(), paramstr) == 0;
               else if (operatorID == SELLTYPE)
                 found = intarr[isel] == zaxis2ltype(zaxisID);
 
@@ -237,10 +236,10 @@ Selvar(void *process)
     {
       if (vars[varID])
         {
-          const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
+          const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
           if (zaxisInqType(zaxisID) == ZAXIS_HYBRID)
             {
-              int psvarid = vlist_get_psvarid(vlistID1, zaxisID);
+              const auto psvarid = vlist_get_psvarid(vlistID1, zaxisID);
               if (psvarid != -1 && !vars[psvarid])
                 {
                   vars[psvarid] = true;
@@ -285,25 +284,27 @@ Selvar(void *process)
 
   if (npar == 0) cdoAbort("No variables selected!");
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
 
+  if (Options::cdoVerbose) vlistPrint(vlistID2);
+
   nvars = vlistNvars(vlistID2);
   for (varID = 0; varID < nvars; ++varID)
     if (vlistInqVarTimetype(vlistID2, varID) != TIME_CONSTANT) break;
   if (varID == nvars) vlistDefNtsteps(vlistID2, 0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      auto gridsizemax = vlistGridsizeMax(vlistID1);
       if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
       array.resize(gridsizemax);
     }
