@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,13 +25,10 @@
 #include "util_wildcards.h"
 
 static void
-setAttributes(KVList &kvlist, int vlistID)
+setAttributes(const KVList &kvlist, int vlistID)
 {
-  enum
-  {
-    Undefined = -99
-  };
-  const int delim = '@';
+  constexpr int Undefined = -99;
+  constexpr int delim = '@';
   const auto nvars = vlistNvars(vlistID);
   const auto ngrids = vlistNgrids(vlistID);
   const auto nzaxis = vlistNzaxis(vlistID);
@@ -48,7 +45,7 @@ setAttributes(KVList &kvlist, int vlistID)
     {
       char *varname = nullptr, *attname = nullptr;
       strcpy(buffer, kv.key.c_str());
-      char *result = strrchr(buffer, delim);
+      char *const result = strrchr(buffer, delim);
       if (result == nullptr)
         {
           attname = buffer;
@@ -78,17 +75,20 @@ setAttributes(KVList &kvlist, int vlistID)
 
           if (cdiID == Undefined)
             {
+              int length = CDI_MAX_NAME;
               /*
               for ( int idx = 0; idx < ngrids; idx++ )
                 {
                   int gridID = vlistGrid(vlistID, idx);
-                  gridInqXname(gridID, name);
+                  length = CDI_MAX_NAME;
+                  cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, name, &length);
                   if (wildcardmatch(varname, name) == 0)
                     {
                       cdiID = gridID;
                       varIDs[nv++] = CDI_GLOBAL;
                     }
-                  gridInqYname(gridID, name);
+                  length = CDI_MAX_NAME;
+                  cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, name, &length);
                   if (wildcardmatch(varname, name) == 0)
                     {
                       cdiID = gridID;
@@ -98,8 +98,9 @@ setAttributes(KVList &kvlist, int vlistID)
               */
               for (int idx = 0; idx < nzaxis; idx++)
                 {
-                  const int zaxisID = vlistZaxis(vlistID, idx);
-                  zaxisInqName(zaxisID, name);
+                  const auto zaxisID = vlistZaxis(vlistID, idx);
+                  length = CDI_MAX_NAME;
+                  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, name, &length);
                   if (wildcardmatch(varname, name) == 0)
                     {
                       cdiID = zaxisID;
@@ -118,7 +119,7 @@ setAttributes(KVList &kvlist, int vlistID)
                       wname[i] = strdup(varname);
                       break;
                     }
-                  if (cstrIsEqual(wname[i], varname))
+                  if (cdo_cmpstr(wname[i], varname))
                     {
                       lwarn = false;
                       break;
@@ -140,38 +141,46 @@ setAttributes(KVList &kvlist, int vlistID)
           int nvalues = kv.nvalues;
           if (nvalues == 1 && value.empty()) nvalues = 0;
 
-          const int dtype = literalsFindDatatype(nvalues, values);
+          const auto dtype = literalsFindDatatype(nvalues, values);
 
           for (int idx = 0; idx < nv; ++idx)
             {
               const auto varID = varIDs[idx];
               // if ( Options::cdoVerbose ) printf("varID, cdiID, attname %d %d %s %d\n", varID, cdiID, attname,
               // (int)strlen(attname));
-              if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
+              if (nvalues == 0)
                 {
-                  std::vector<int> ivals(nvalues);
-                  for (int i = 0; i < nvalues; ++i) ivals[i] = literal2int(values[i]);
-                  cdiDefAttInt(cdiID, varID, attname, dtype, nvalues, ivals.data());
-                }
-              else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
-                {
-                  Varray<double> dvals(nvalues);
-                  for (int i = 0; i < nvalues; ++i) dvals[i] = literal2double(values[i]);
-                  cdiDefAttFlt(cdiID, varID, attname, dtype, nvalues, dvals.data());
+                  cdiDelAtt(cdiID, varID, attname);
                 }
               else
                 {
-                  const int len = (int)value.size();
-                  int outlen = 0;
-                  std::vector<char> outvalue(len);
-                  for (int i = 0; i < len; ++i)
+                  if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
                     {
-                      if (i > 0 && value[i - 1] == '\\' && value[i] == 'n')
-                        outvalue[outlen - 1] = '\n';
-                      else
-                        outvalue[outlen++] = value[i];
+                      std::vector<int> ivals(nvalues);
+                      for (int i = 0; i < nvalues; ++i) ivals[i] = literal2int(values[i]);
+                      cdiDefAttInt(cdiID, varID, attname, dtype, nvalues, ivals.data());
+                    }
+                  else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
+                    {
+                      Varray<double> dvals(nvalues);
+                      for (int i = 0; i < nvalues; ++i) dvals[i] = literal2double(values[i]);
+                      cdiDefAttFlt(cdiID, varID, attname, dtype, nvalues, dvals.data());
+                    }
+                  else
+                    {
+                      if (nvalues > 1) cdoAbort("Multidimensional string attributes not supported! %s=\"%s\"", attname, values[1].c_str());
+                      const auto len = (int) value.size();
+                      int outlen = 0;
+                      std::vector<char> outvalue(len);
+                      for (int i = 0; i < len; ++i)
+                        {
+                          if (i > 0 && value[i - 1] == '\\' && value[i] == 'n')
+                            outvalue[outlen - 1] = '\n';
+                          else
+                            outvalue[outlen++] = value[i];
+                        }
+                      cdiDefAttTxt(cdiID, varID, attname, outlen, outvalue.data());
                     }
-                  cdiDefAttTxt(cdiID, varID, attname, outlen, outvalue.data());
                 }
             }
         }
@@ -202,7 +211,7 @@ Setattribute(void *process)
 
   PMList pmlist;
   KVList kvlist;
-  kvlist.name = "SETATTRIBUTES";
+  kvlist.name = "SETATTRIBUTE";
   if (kvlist.parseArguments(natts, cdoGetOperArgv()) != 0) cdoAbort("Parse error!");
   if (Options::cdoVerbose) kvlist.print();
 
@@ -214,7 +223,7 @@ Setattribute(void *process)
         {
           if (Options::cdoVerbose) cdoPrint("Reading attributes from: %s", kv.values[0].c_str());
           auto filename = parameter2word(kv.values[0].c_str());
-          auto *fp = fopen(filename, "r");
+          auto fp = fopen(filename, "r");
           if (fp == nullptr) cdoAbort("Open failed on: %s\n", filename);
           pmlist.readNamelist(fp, filename);
           pkvlist = &pmlist.front();
