@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,20 +24,23 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include <mpim_grid.h>
+#include "selboxinfo.h"
 
-void genlonlatbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
-
-void genindexbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
 
 static void
-setcbox(double constant, double *array, int gridID, long lat1, long lat2, long lon11, long lon12, long lon21, long lon22)
+setcbox(double constant, double *array, int gridID, const SelboxInfo &sbox)
 {
-  long nlon = gridInqXsize(gridID);
-  long nlat = gridInqYsize(gridID);
+  const auto &lat1 = sbox.lat1;
+  const auto &lat2 = sbox.lat2;
+  const auto &lon11 = sbox.lon11;
+  const auto &lon12 = sbox.lon12;
+  const auto &lon21 = sbox.lon21;
+  const auto &lon22 = sbox.lon22;
+  const long nlon = gridInqXsize(gridID);
+  const long nlat = gridInqYsize(gridID);
 
   for (long ilat = 0; ilat < nlat; ilat++)
     for (long ilon = 0; ilon < nlon; ilon++)
@@ -55,26 +58,25 @@ Setbox(void *process)
   int gridID = -1;
   int index, gridtype;
   size_t nmiss;
-  long lat1, lat2, lon11, lon12, lon21, lon22;
+  SelboxInfo sbox;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int SETCLONLATBOX = cdoOperatorAdd("setclonlatbox", 0, 0, "constant, western and eastern longitude and southern and northern latitude");
-  const int SETCINDEXBOX = cdoOperatorAdd("setcindexbox", 0, 0, "constant, index of first and last longitude and index of first and last latitude");
+  const auto SETCLONLATBOX = cdoOperatorAdd("setclonlatbox", 0, 0, "constant, western and eastern longitude and southern and northern latitude");
+  const auto SETCINDEXBOX = cdoOperatorAdd("setcindexbox", 0, 0, "constant, index of first and last longitude and index of first and last latitude");
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  const double constant = parameter2double(operatorArgv()[0]);
+  const auto constant = parameter2double(cdoOperatorArgv(0));
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   int ndiffgrids = 0;
   for (index = 1; index < ngrids; index++)
     if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
@@ -97,26 +99,26 @@ Setbox(void *process)
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   if (operatorID == SETCLONLATBOX)
-    genlonlatbox(1, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+    genlonlatbox(1, gridID, sbox);
   else
-    genindexbox(1, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+    genindexbox(1, gridID, sbox);
 
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars);
   for (varID = 0; varID < nvars; varID++) vars[varID] = (gridID == vlistInqVarGrid(vlistID1, varID));
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  const size_t gridsize = gridInqSize(gridID);
-  std::vector<double> array(gridsize);
+  const auto gridsize = gridInqSize(gridID);
+  Varray<double> array(gridsize);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -132,10 +134,10 @@ Setbox(void *process)
             {
               cdoReadRecord(streamID1, array.data(), &nmiss);
 
-              setcbox(constant, array.data(), gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+              setcbox(constant, array.data(), gridID, sbox);
 
-              const double missval = vlistInqVarMissval(vlistID1, varID);
-              nmiss = arrayNumMV(gridsize, array.data(), missval);
+              const auto missval = vlistInqVarMissval(vlistID1, varID);
+              nmiss = varrayNumMV(gridsize, array, missval);
               cdoDefRecord(streamID2, varID, levelID);
               cdoWriteRecord(streamID2, array.data(), nmiss);
             }
