@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "cdi_lockedIO.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
@@ -37,31 +38,31 @@
 static void
 changeGrid(const int vlistID1, const int vlistID2, const int gridID2)
 {
+  const auto gridsize2 = gridInqSize(gridID2);
   int found = 0;
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID1, index);
-      if (gridInqSize(gridID1) == gridInqSize(gridID2))
+      if (gridsize2 == gridInqSize(vlistGrid(vlistID1, index)))
         {
           vlistChangeGridIndex(vlistID2, index, gridID2);
           found++;
         }
     }
-  if (!found) cdoWarning("No horizontal grid with %zu cells found!", gridInqSize(gridID2));
+  if (!found) cdoWarning("No horizontal grid with %zu cells found!", gridsize2);
 }
 
 static void
-setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const char *gridname, const bool lregular,
+setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const std::string &gridname, const bool lregular,
             const bool lregularnn, const bool ldereference, const bool lbounds, std::vector<int> &grid2_vgpm)
 {
   int gridID2;
-  bool lrgrid = false;
-  const int ngrids = vlistNgrids(vlistID1);
+  auto lrgrid = false;
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID1, index);
-      const int gridtype1 = gridInqType(gridID1);
+      const auto gridID1 = vlistGrid(vlistID1, index);
+      const auto gridtype1 = gridInqType(gridID1);
       gridID2 = -1;
 
       if (gridtype1 == GRID_GENERIC && gridInqSize(gridID1) == 1) continue;
@@ -72,8 +73,9 @@ setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const ch
         }
       else if (ldereference)
         {
-          gridID2 = referenceToGrid(gridID1);
-          if (gridID2 == -1) cdoAbort("Reference to horizontal grid not found!");
+          const auto reference = dereferenceGrid(gridID1);
+          if (reference.isValid) gridID2 = reference.gridID;
+          if (reference.notFound) cdoAbort("Reference to horizontal grid not found!");
         }
       else
         {
@@ -84,10 +86,9 @@ setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const ch
           else if (gridtype == GRID_UNSTRUCTURED)
             {
               gridID2 = gridToUnstructured(gridID1, 1);
-              bool ligme = (gridtype1 == GRID_GME);
-              if (ligme)
+              if (gridtype1 == GRID_GME)
                 {
-                  const size_t grid2_nvgp = gridInqSize(gridID2);
+                  const auto grid2_nvgp = gridInqSize(gridID2);
                   grid2_vgpm.resize(grid2_nvgp);
                   gridInqMaskGME(gridID2, grid2_vgpm.data());
                   gridCompress(gridID2);
@@ -112,8 +113,19 @@ setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const ch
             {
               gridID2 = gridID1;
             }
+          else if (gridtype == GRID_PROJECTION)
+            {
+              if (gridInqType(gridID1) == GRID_PROJECTION)
+                gridID2 = gridID1;
+              else
+                {
+                  const int projID = gridInqProj(gridID1);
+                  if (projID != CDI_UNDEFID && gridInqType(projID) == GRID_PROJECTION) gridID2 = projID;
+                }
+              if (gridID2 == -1) cdoWarning("Projection not found!");
+            }
           else
-            cdoAbort("Unsupported grid name: %s", gridname);
+            cdoAbort("Unsupported grid name: %s", gridname.c_str());
         }
 
       if (gridID2 == -1)
@@ -132,18 +144,18 @@ setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const ch
 }
 
 static void
-setGridcellArea(const int vlistID1, const int vlistID2, std::vector<double> &gridcellArea)
+setGridcellArea(const int vlistID1, const int vlistID2, Varray<double> &gridcellArea)
 {
-  const size_t areasize = gridcellArea.size();
+  const auto areasize = gridcellArea.size();
   int numGridsChanges = 0;
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID1, index);
-      const size_t gridsize = gridInqSize(gridID1);
+      const auto gridID1 = vlistGrid(vlistID1, index);
+      const auto gridsize = gridInqSize(gridID1);
       if (gridsize == areasize)
         {
-          const int gridID2 = gridDuplicate(gridID1);
+          const auto gridID2 = gridDuplicate(gridID1);
           gridDefArea(gridID2, gridcellArea.data());
           vlistChangeGridIndex(vlistID2, index, gridID2);
           numGridsChanges++;
@@ -153,22 +165,22 @@ setGridcellArea(const int vlistID1, const int vlistID2, std::vector<double> &gri
 }
 
 static void
-setGridMask(const int vlistID1, const int vlistID2, std::vector<double> &gridmask)
+setGridMask(const int vlistID1, const int vlistID2, Varray<double> &gridmask)
 {
-  const size_t masksize = gridmask.size();
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto masksize = gridmask.size();
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID1, index);
-      const size_t gridsize = gridInqSize(gridID1);
+      const auto gridID1 = vlistGrid(vlistID1, index);
+      const auto gridsize = gridInqSize(gridID1);
       if (gridsize == masksize)
         {
           std::vector<int> mask(masksize);
           for (size_t i = 0; i < masksize; i++)
             {
-              mask[i] = (gridmask[i] < 0 || gridmask[i] > 255) ? 0 : (int) lround(gridmask[i]);
+              mask[i] = (gridmask[i] < 0 || gridmask[i] > 255) ? 0 : (int) std::lround(gridmask[i]);
             }
-          const int gridID2 = gridDuplicate(gridID1);
+          const auto gridID2 = gridDuplicate(gridID1);
           gridDefMask(gridID2, mask.data());
           vlistChangeGridIndex(vlistID2, index, gridID2);
         }
@@ -178,26 +190,42 @@ setGridMask(const int vlistID1, const int vlistID2, std::vector<double> &gridmas
 static void
 unsetGridMask(const int vlistID1, const int vlistID2)
 {
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridID1 = vlistGrid(vlistID1, index);
-      const int gridID2 = gridDuplicate(gridID1);
+      const auto gridID1 = vlistGrid(vlistID1, index);
+      const auto gridID2 = gridDuplicate(gridID1);
       gridDefMask(gridID2, nullptr);
       vlistChangeGridIndex(vlistID2, index, gridID2);
     }
 }
 
 static void
-readAreaFromFile(const char *areafile, std::vector<double> &gridcellArea)
+setProjParams(const int vlistID1, const int vlistID2, const char *projparams)
 {
-  bool searchName = false;
+  const auto ngrids = vlistNgrids(vlistID1);
+  for (int index = 0; index < ngrids; index++)
+    {
+      const auto gridID1 = vlistGrid(vlistID1, index);
+      if (gridInqType(gridID1) == GRID_PROJECTION)
+        {
+          const auto gridID2 = gridDuplicate(gridID1);
+          cdiDefAttTxt(gridID2, CDI_GLOBAL, "proj4_params", (int) (strlen(projparams)), projparams);
+          vlistChangeGridIndex(vlistID2, index, gridID2);
+        }
+    }
+}
+
+static void
+readAreaFromFile(const std::string &areafile, Varray<double> &gridcellArea)
+{
+  auto searchName = false;
   std::string filename = areafile;
   std::string varname;
 
-  if (!fileExists(areafile))
+  if (!fileExists(areafile.c_str()))
     {
-      auto const pos = filename.find_last_of(':');
+      const auto pos = filename.find_last_of(':');
       if (pos > 1 && pos < (filename.size() - 1))
         {
           varname = filename.substr(pos + 1);
@@ -206,19 +234,19 @@ readAreaFromFile(const char *areafile, std::vector<double> &gridcellArea)
         }
     }
 
-  int streamID = streamOpenReadLocked(filename.c_str());
-  const int vlistID = streamInqVlist(streamID);
+  const auto streamID = streamOpenReadLocked(filename.c_str());
+  const auto vlistID = streamInqVlist(streamID);
 
   int svarID = 0;
   if (searchName)
     {
       char name[CDI_MAX_NAME];
-      const int nvars = vlistNvars(vlistID);
+      const auto nvars = vlistNvars(vlistID);
       int varID;
       for (varID = 0; varID < nvars; ++varID)
         {
           vlistInqVarName(vlistID, varID, name);
-          if (cstrIsEqual(name, varname.c_str()))
+          if (cdo_cmpstr(name, varname.c_str()))
             {
               svarID = varID;
               break;
@@ -227,15 +255,15 @@ readAreaFromFile(const char *areafile, std::vector<double> &gridcellArea)
       if (varID == nvars) cdoAbort("Variable %s not found in %s\n", varname.c_str(), filename.c_str());
     }
 
-  int nrecs = streamInqTimestep(streamID, 0);
-  int varID, levelID;
+  const auto nrecs = streamInqTimestep(streamID, 0);
   for (int recID = 0; recID < nrecs; ++recID)
     {
+      int varID, levelID;
       streamInqRecord(streamID, &varID, &levelID);
       if (varID == svarID)
         {
-          const int gridID = vlistInqVarGrid(vlistID, varID);
-          const size_t areasize = gridInqSize(gridID);
+          const auto gridID = vlistInqVarGrid(vlistID, varID);
+          const auto areasize = gridInqSize(gridID);
           gridcellArea.resize(areasize);
 
           size_t nmiss;
@@ -260,81 +288,82 @@ Setgrid(void *process)
   bool ldereference = false;
   int number = 0, position = 0;
   bool lbounds = true;
-  char *gridname = nullptr;
-  char *griduri = nullptr;
+  std::string gridname;
+  const char *griduri = nullptr;
+  const char *projparams = nullptr;
   std::vector<int> grid2_vgpm;
-  std::vector<double> gridmask, gridcellArea;
+  Varray<double> gridmask, gridcellArea;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int SETGRID       = cdoOperatorAdd("setgrid",       0, 0, "grid description file or name");
-  const int SETGRIDTYPE   = cdoOperatorAdd("setgridtype",   0, 0, "grid type");
-  const int SETGRIDAREA   = cdoOperatorAdd("setgridarea",   0, 0, "filename with area weights");
-  const int SETGRIDMASK   = cdoOperatorAdd("setgridmask",   0, 0, "filename with grid mask");
-  const int UNSETGRIDMASK = cdoOperatorAdd("unsetgridmask", 0, 0, nullptr);
-  const int SETGRIDNUMBER = cdoOperatorAdd("setgridnumber", 0, 0, "grid number and optionally grid position");
-  const int SETGRIDURI    = cdoOperatorAdd("setgriduri",    0, 0, "reference URI of the horizontal grid");
-  const int USEGRIDNUMBER = cdoOperatorAdd("usegridnumber", 0, 0, "use existing grid identified by grid number");
+  const auto SETGRID       = cdoOperatorAdd("setgrid",       0, 0, "grid description file or name");
+  const auto SETGRIDTYPE   = cdoOperatorAdd("setgridtype",   0, 0, "grid type");
+  const auto SETGRIDAREA   = cdoOperatorAdd("setgridarea",   0, 0, "filename with area weights");
+  const auto SETGRIDMASK   = cdoOperatorAdd("setgridmask",   0, 0, "filename with grid mask");
+  const auto UNSETGRIDMASK = cdoOperatorAdd("unsetgridmask", 0, 0, nullptr);
+  const auto SETGRIDNUMBER = cdoOperatorAdd("setgridnumber", 0, 0, "grid number and optionally grid position");
+  const auto SETGRIDURI    = cdoOperatorAdd("setgriduri",    0, 0, "reference URI of the horizontal grid");
+  const auto USEGRIDNUMBER = cdoOperatorAdd("usegridnumber", 0, 0, "use existing grid identified by grid number");
+  const auto SETPROJPARAMS = cdoOperatorAdd("setprojparams", 0, 0, "proj library parameter (e.g.: +init=EPSG:3413)");
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID != UNSETGRIDMASK) operatorInputArg(cdoOperatorEnter(operatorID));
 
   if (operatorID == SETGRID)
     {
       operatorCheckArgc(1);
-      gridID2 = cdoDefineGrid(operatorArgv()[0]);
+      gridID2 = cdoDefineGrid(cdoOperatorArgv(0));
     }
   else if (operatorID == SETGRIDTYPE)
     {
       operatorCheckArgc(1);
-      gridname = operatorArgv()[0];
+      gridname = cdoOperatorArgv(0);
 
       // clang-format off
-      if      (cstrIsEqual(gridname, "curvilinear0"))  {gridtype = GRID_CURVILINEAR; lbounds = false;}
-      else if (cstrIsEqual(gridname, "curvilinear"))   {gridtype = GRID_CURVILINEAR; lbounds = true;}
-      else if (cstrIsEqual(gridname, "cell"))           gridtype = GRID_UNSTRUCTURED;
-      else if (cstrIsEqual(gridname, "unstructured0")) {gridtype = GRID_UNSTRUCTURED; lbounds = false;}
-      else if (cstrIsEqual(gridname, "unstructured"))  {gridtype = GRID_UNSTRUCTURED; lbounds = true;}
-      else if (cstrIsEqual(gridname, "generic"))        gridtype = GRID_GENERIC;
-      else if (cstrIsEqual(gridname, "dereference"))    ldereference = true;
-      else if (cstrIsEqual(gridname, "lonlat"))         gridtype = GRID_LONLAT;
-      else if (cstrIsEqual(gridname, "gaussian"))       gridtype = GRID_GAUSSIAN;
-      else if (cstrIsEqual(gridname, "regularnn"))     {gridtype = GRID_GAUSSIAN; lregularnn = true;}
-      else if (cstrIsEqual(gridname, "regular"))       {gridtype = GRID_GAUSSIAN; lregular = true;}
-      else cdoAbort("Unsupported grid name: %s", gridname);
+      if      (gridname == "curvilinear0")  {gridtype = GRID_CURVILINEAR; lbounds = false;}
+      else if (gridname == "curvilinear")   {gridtype = GRID_CURVILINEAR; lbounds = true;}
+      else if (gridname == "cell")           gridtype = GRID_UNSTRUCTURED;
+      else if (gridname == "unstructured0") {gridtype = GRID_UNSTRUCTURED; lbounds = false;}
+      else if (gridname == "unstructured")  {gridtype = GRID_UNSTRUCTURED; lbounds = true;}
+      else if (gridname == "generic")        gridtype = GRID_GENERIC;
+      else if (gridname == "dereference")    ldereference = true;
+      else if (gridname == "lonlat")         gridtype = GRID_LONLAT;
+      else if (gridname == "gaussian")       gridtype = GRID_GAUSSIAN;
+      else if (gridname == "regularnn")     {gridtype = GRID_GAUSSIAN; lregularnn = true;}
+      else if (gridname == "regular")       {gridtype = GRID_GAUSSIAN; lregular = true;}
+      else if (gridname == "projection")    {gridtype = GRID_PROJECTION;}
+      else cdoAbort("Unsupported grid name: %s", gridname.c_str());
       // clang-format on
     }
   else if (operatorID == SETGRIDAREA)
     {
       operatorCheckArgc(1);
 
-      readAreaFromFile(operatorArgv()[0], gridcellArea);
+      readAreaFromFile(cdoOperatorArgv(0), gridcellArea);
 
       if (Options::cdoVerbose)
         {
-          const size_t areasize = gridcellArea.size();
-
-          double arrmean, arrmin, arrmax;
-          arrayMinMaxMean(areasize, gridcellArea.data(), arrmin, arrmax, arrmean);
-          cdoPrint("gridcellAreas: %zu %#12.5g%#12.5g%#12.5g", areasize, arrmin, arrmean, arrmax);
+          const auto areasize = gridcellArea.size();
+          auto mmm = varrayMinMaxMean(areasize, gridcellArea);
+          cdoPrint("gridcellAreas: %zu %#12.5g%#12.5g%#12.5g", areasize, mmm.min, mmm.mean, mmm.max);
         }
     }
   else if (operatorID == SETGRIDMASK)
     {
       operatorCheckArgc(1);
-      const char *maskfile = operatorArgv()[0];
-      int streamID = streamOpenReadLocked(maskfile);
-      const int vlistID = streamInqVlist(streamID);
+      const auto maskfile = cdoOperatorArgv(0).c_str();
+      const auto streamID = streamOpenReadLocked(maskfile);
+      const auto vlistID = streamInqVlist(streamID);
 
-      nrecs = streamInqTimestep(streamID, 0);
+      (void) streamInqTimestep(streamID, 0);
       streamInqRecord(streamID, &varID, &levelID);
 
-      const double missval = vlistInqVarMissval(vlistID, varID);
-      const int gridID = vlistInqVarGrid(vlistID, varID);
-      const size_t masksize = gridInqSize(gridID);
+      const auto missval = vlistInqVarMissval(vlistID, varID);
+      const auto gridID = vlistInqVarGrid(vlistID, varID);
+      const auto masksize = gridInqSize(gridID);
       gridmask.resize(masksize);
 
       streamReadRecord(streamID, gridmask.data(), &nmiss);
@@ -346,14 +375,14 @@ Setgrid(void *process)
   else if (operatorID == USEGRIDNUMBER)
     {
       operatorCheckArgc(1);
-      number = parameter2int(operatorArgv()[0]);
+      number = parameter2int(cdoOperatorArgv(0));
     }
   else if (operatorID == SETGRIDNUMBER)
     {
       if (operatorArgc() >= 1 && operatorArgc() <= 2)
         {
-          number = parameter2int(operatorArgv()[0]);
-          if (operatorArgc() == 2) position = parameter2int(operatorArgv()[1]);
+          number = parameter2int(cdoOperatorArgv(0));
+          if (operatorArgc() == 2) position = parameter2int(cdoOperatorArgv(1));
         }
       else
         {
@@ -363,16 +392,21 @@ Setgrid(void *process)
   else if (operatorID == SETGRIDURI)
     {
       operatorCheckArgc(1);
-      griduri = operatorArgv()[0];
+      griduri = cdoOperatorArgv(0).c_str();
+    }
+  else if (operatorID == SETPROJPARAMS)
+    {
+      operatorCheckArgc(1);
+      projparams = cdoOperatorArgv(0).c_str();
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   if (operatorID == SETGRID)
@@ -381,29 +415,29 @@ Setgrid(void *process)
     }
   else if (operatorID == SETGRIDNUMBER || operatorID == SETGRIDURI || operatorID == USEGRIDNUMBER)
     {
-      int gridID2 = -1;
+      int gridID2x = -1;
       if (operatorID == SETGRIDNUMBER)
         {
-          const int gridID1 = vlistGrid(vlistID1, 0);
-          gridID2 = gridCreate(GRID_UNSTRUCTURED, gridInqSize(gridID1));
-          gridDefNumber(gridID2, number);
-          gridDefPosition(gridID2, position);
+          const auto gridID1 = vlistGrid(vlistID1, 0);
+          gridID2x = gridCreate(GRID_UNSTRUCTURED, gridInqSize(gridID1));
+          cdiDefKeyInt(gridID2x, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDUSED, number);
+          cdiDefKeyInt(gridID2x, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDINREFERENCE, position);
         }
       else if (operatorID == USEGRIDNUMBER)
         {
           if (number < 1 || number > vlistNgrids(vlistID1))
             cdoAbort("Invalid grid number: %d (max = %d)!", number, vlistNgrids(vlistID1));
 
-          gridID2 = vlistGrid(vlistID1, number - 1);
+          gridID2x = vlistGrid(vlistID1, number - 1);
         }
       else
         {
-          const int gridID1 = vlistGrid(vlistID1, 0);
-          gridID2 = gridDuplicate(gridID1);
-          gridDefReference(gridID2, griduri);
+          const auto gridID1 = vlistGrid(vlistID1, 0);
+          gridID2x = gridDuplicate(gridID1);
+          cdiDefKeyString(gridID2x, CDI_GLOBAL, CDI_KEY_REFERENCEURI, griduri);
         }
 
-      changeGrid(vlistID1, vlistID2, gridID2);
+      changeGrid(vlistID1, vlistID2, gridID2x);
     }
   else if (operatorID == SETGRIDTYPE)
     {
@@ -421,16 +455,20 @@ Setgrid(void *process)
     {
       unsetGridMask(vlistID1, vlistID2);
     }
+  else if (operatorID == SETPROJPARAMS)
+    {
+      setProjParams(vlistID1, vlistID2, projparams);
+    }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
   // vlistPrint(vlistID2);
 
-  size_t gridsize = (lregular || lregularnn) ? vlistGridsizeMax(vlistID2) : vlistGridsizeMax(vlistID1);
+  auto gridsizemax = (lregular || lregularnn) ? vlistGridsizeMax(vlistID2) : vlistGridsizeMax(vlistID1);
 
-  if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-  std::vector<double> array(gridsize);
+  if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
+  Varray<double> array(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -445,20 +483,19 @@ Setgrid(void *process)
 
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
-          const int gridID1 = vlistInqVarGrid(vlistID1, varID);
+          const auto gridID1 = vlistInqVarGrid(vlistID1, varID);
           if (lregular || lregularnn)
             {
-              const int gridID2 = vlistInqVarGrid(vlistID2, varID);
               if (gridInqType(gridID1) == GRID_GAUSSIAN_REDUCED)
                 {
-                  const double missval = vlistInqVarMissval(vlistID1, varID);
+                  const auto missval = vlistInqVarMissval(vlistID1, varID);
                   const int lnearst = lregularnn ? 1 : 0;
-                  field2regular(gridID1, gridID2, missval, array.data(), nmiss, lnearst);
+                  field2regular(gridID1, vlistInqVarGrid(vlistID2, varID), missval, array.data(), nmiss, lnearst);
                 }
             }
           else if (gridInqType(gridID1) == GRID_GME)
             {
-              const size_t gridsize = gridInqSize(gridID1);
+              const auto gridsize = gridInqSize(gridID1);
               size_t j = 0;
               for (size_t i = 0; i < gridsize; i++)
                 if (grid2_vgpm[i]) array[j++] = array[i];
