@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,43 +23,25 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "table.h"
 #include "param_conversion.h"
 #include "cdo_cmor.h"
-
 #include "pmlist.h"
+#include "mapping.h"
 #include "convert_units.h"
 #include "util_files.h"
-#include "parse_literals.h"
-#include "cdo_cdiWrapper.h"
+#include "cdi_lockedIO.h"
 
-int stringToParam(const char *paramstr);
 
-void
-paramToString(int param, char *paramstr, int maxlen)
-{
-  int dis, cat, num;
-  cdiDecodeParam(param, &num, &cat, &dis);
-
-  const size_t umaxlen = maxlen >= 0 ? (unsigned) maxlen : 0U;
-  int len;
-  if (dis == 255 && (cat == 255 || cat == 0))
-    len = snprintf(paramstr, umaxlen, "%d", num);
-  else if (dis == 255)
-    len = snprintf(paramstr, umaxlen, "%d.%d", num, cat);
-  else
-    len = snprintf(paramstr, umaxlen, "%d.%d.%d", num, cat, dis);
-
-  if (len >= maxlen || len < 0) fprintf(stderr, "Internal problem (%s): size of input string is too small!\n", __func__);
-}
-
-typedef enum
+enum pt_mode_t
 {
   CODE_NUMBER,
   PARAMETER_ID,
   VARIABLE_NAME,
   STANDARD_NAME
-} pt_mode_t;
+};
 
 static void
 applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &vars)
@@ -76,10 +58,10 @@ applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, st
   double missval = 0;
 
   {
-    const KVList *kvlist = pmlist.getKVListVentry(hentry);
+    auto kvlist = pmlist.getKVListVentry(hentry);
     if (kvlist)
       {
-        const KeyValues *kv = kvlist->search("missing_value");
+        const auto kv = kvlist->search("missing_value");
         if (kv && kv->nvalues > 0)
           {
             lmissval = true;
@@ -90,13 +72,13 @@ applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, st
 
   for (int varID = 0; varID < nvars; varID++)
     {
-      CmorVar &var = vars[varID];
+      auto &var = vars[varID];
       vlistInqVarName(vlistID2, varID, varname);
 
       strcpy(var.name, varname);
       if (lmissval)
         {
-          const double missval_old = vlistInqVarMissval(vlistID2, varID);
+          const auto missval_old = vlistInqVarMissval(vlistID2, varID);
           if (!DBL_IS_EQUAL(missval, missval_old))
             {
               var.changemissval = true;
@@ -105,25 +87,26 @@ applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, st
             }
         }
 
-      KVList *kvlist = nullptr;
+      const KVList *kvlist = nullptr;
       if (ptmode == CODE_NUMBER)
         {
           codenum = vlistInqVarCode(vlistID2, varID);
           snprintf(valstr, sizeof(valstr), "%d", codenum);
-          kvlist = (KVList *) pmlist.searchKVListVentry("code", valstr, ventry);
+          kvlist = pmlist.searchKVListVentry("code", valstr, ventry);
           if (kvlist)
             {
-              const int tableID = vlistInqVarTable(vlistID2, varID);
-              const int tabnum = tableInqNum(tableID);
-              const int levtype = zaxisInqLtype(vlistInqVarZaxis(vlistID2, varID));
-              int table = tabnum;
-              int ltype = levtype;
+              const auto tableID = vlistInqVarTable(vlistID2, varID);
+              const auto tabnum = tableInqNum(tableID);
+              int levtype = 0;
+              cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &levtype);
+              auto table = tabnum;
+              auto ltype = levtype;
               {
-                const KeyValues *kv = kvlist->search("table");
+                const auto kv = kvlist->search("table");
                 if (kv && kv->nvalues == 1) table = parameter2int(kv->values[0].c_str());
               }
               {
-                const KeyValues *kv = kvlist->search("ltype");
+                const auto kv = kvlist->search("ltype");
                 if (kv && kv->nvalues == 1) ltype = parameter2int(kv->values[0].c_str());
               }
               if (!(tabnum == table && levtype == ltype)) kvlist = nullptr;
@@ -131,21 +114,22 @@ applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, st
         }
       else if (ptmode == PARAMETER_ID)
         {
-          const int param = vlistInqVarParam(vlistID2, varID);
+          const auto param = vlistInqVarParam(vlistID2, varID);
           paramToString(param, paramstr, sizeof(paramstr));
           snprintf(valstr, sizeof(valstr), "%s", paramstr);
-          kvlist = (KVList *) pmlist.searchKVListVentry("param", valstr, ventry);
+          kvlist = pmlist.searchKVListVentry("param", valstr, ventry);
           if (kvlist)
             {
-              const int levtype = zaxisInqLtype(vlistInqVarZaxis(vlistID2, varID));
-              const KeyValues *kv = kvlist->search("ltype");
-              const int ltype = (kv && kv->nvalues == 1) ? parameter2int(kv->values[0].c_str()) : levtype;
+              int levtype = 0;
+              cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &levtype);
+              const auto kv = kvlist->search("ltype");
+              const auto ltype = (kv && kv->nvalues == 1) ? parameter2int(kv->values[0].c_str()) : levtype;
               if (levtype != ltype) kvlist = nullptr;
             }
         }
       else if (ptmode == VARIABLE_NAME)
         {
-          kvlist = (KVList *) pmlist.searchKVListVentry("name", varname, ventry);
+          kvlist = pmlist.searchKVListVentry("name", varname, ventry);
         }
 
       if (kvlist)
@@ -156,125 +140,9 @@ applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, st
 
           for (const auto &kv : *kvlist)
             {
-              const char *key = kv.key.c_str();
-              const char *value = (kv.nvalues > 0) ? kv.values[0].c_str() : nullptr;
-              const bool lv1 = (kv.nvalues == 1);
-
-              // printf("key=%s  value=%s\n", key, value ? value : "");
-
-              if (lv1 && cstrIsEqual(key, "standard_name"))
-                vlistDefVarStdname(vlistID2, varID, value);
-              else if (lv1 && cstrIsEqual(key, "long_name"))
-                vlistDefVarLongname(vlistID2, varID, value);
-              else if (lv1 && cstrIsEqual(key, "units"))
-                cdo_define_var_units(var, vlistID2, varID, value);
-              else if (lv1 && cstrIsEqual(key, "name"))
-                {
-                  if (ptmode != VARIABLE_NAME) vlistDefVarName(vlistID2, varID, parameter2word(value));
-                }
-              else if (lv1 && cstrIsEqual(key, "out_name"))
-                {
-                  const char *outname = parameter2word(value);
-                  if (!cstrIsEqual(var.name, outname))
-                    {
-                      vlistDefVarName(vlistID2, varID, outname);
-                      cdiDefAttTxt(vlistID2, varID, "original_name", (int) strlen(var.name), var.name);
-                    }
-                }
-              else if (lv1 && cstrIsEqual(key, "param"))
-                vlistDefVarParam(vlistID2, varID, stringToParam(parameter2word(value)));
-              else if (lv1 && cstrIsEqual(key, "out_param"))
-                vlistDefVarParam(vlistID2, varID, stringToParam(parameter2word(value)));
-              else if (lv1 && cstrIsEqual(key, "code"))
-                vlistDefVarParam(vlistID2, varID, cdiEncodeParam(parameter2int(value), ptab, 255));
-              else if (lv1 && cstrIsEqual(key, "out_code"))
-                vlistDefVarParam(vlistID2, varID, cdiEncodeParam(parameter2int(value), ptab, 255));
-              else if (lv1 && cstrIsEqual(key, "uvRelativeToGrid"))
-                cdiDefKeyInt(vlistID2, varID, CDI_KEY_UVRELATIVETOGRID, parameter2bool(value));
-              else if (lv1 && cstrIsEqual(key, "comment"))
-                cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
-              else if (lv1 && cstrIsEqual(key, "chunktype"))
-                ;
-              else if (lv1 && cstrIsEqual(key, "cell_methods"))
-                cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
-              else if (lv1 && cstrIsEqual(key, "cell_measures"))
-                cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
-              else if (lv1 && cstrIsEqual(key, "delete"))
-                var.remove = parameter2bool(value);
-              else if (lv1 && cstrIsEqual(key, "convert"))
-                var.convert = parameter2bool(value);
-              else if (lv1 && cstrIsEqual(key, "factor"))
-                {
-                  var.lfactor = true;
-                  var.factor = parameter2double(value);
-                  if (Options::cdoVerbose) cdoPrint("%s - scale factor %g", varname, var.factor);
-                }
-              else if (lv1 && (cstrIsEqual(key, "missval") || cstrIsEqual(key, "missing_value")))
-                {
-                  const double missval = parameter2double(value);
-                  const double missval_old = vlistInqVarMissval(vlistID2, varID);
-                  if (!DBL_IS_EQUAL(missval, missval_old))
-                    {
-                      if (Options::cdoVerbose) cdoPrint("%s - change missval from %g to %g", varname, missval_old, missval);
-                      var.changemissval = true;
-                      var.missval_old = missval_old;
-                      vlistDefVarMissval(vlistID2, varID, missval);
-                    }
-                }
-              else if (lv1 && cstrIsEqual(key, "valid_min"))
-                {
-                  lvalid_min = true;
-                  var.valid_min = parameter2double(value);
-                }
-              else if (lv1 && cstrIsEqual(key, "valid_max"))
-                {
-                  lvalid_max = true;
-                  var.valid_max = parameter2double(value);
-                }
-              else if (lv1 && cstrIsEqual(key, "ok_min_mean_abs"))
-                {
-                  var.check_min_mean_abs = true;
-                  var.ok_min_mean_abs = parameter2double(value);
-                }
-              else if (lv1 && cstrIsEqual(key, "ok_max_mean_abs"))
-                {
-                  var.check_max_mean_abs = true;
-                  var.ok_max_mean_abs = parameter2double(value);
-                }
-              else if (lv1 && (cstrIsEqual(key, "datatype") || cstrIsEqual(key, "type")))
-                {
-                  const int datatype = str2datatype(parameter2word(value));
-                  if (datatype != -1) vlistDefVarDatatype(vlistID2, varID, datatype);
-                }
-              else if (lv1 && cstrIsEqual(key, "dimensions"))
-                {
-                }
-              else
-                {
-                  int nvalues = kv.nvalues;
-                  if (nvalues == 1 && !*value) nvalues = 0;
-                  const int dtype = literalsFindDatatype(nvalues, kv.values);
-
-                  if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
-                    {
-                      std::vector<int> ivals(nvalues);
-                      for (int i = 0; i < nvalues; ++i) ivals[i] = literal_to_int(kv.values[i].c_str());
-                      cdiDefAttInt(vlistID2, varID, key, dtype, nvalues, ivals.data());
-                    }
-                  else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
-                    {
-                      std::vector<double> dvals(nvalues);
-                      for (int i = 0; i < nvalues; ++i) dvals[i] = literal_to_double(kv.values[i].c_str());
-                      cdiDefAttFlt(vlistID2, varID, key, dtype, nvalues, dvals.data());
-                    }
-                  else
-                    {
-                      const int len = (value && *value) ? (int) strlen(value) : 0;
-                      cdiDefAttTxt(vlistID2, varID, key, len, value);
-                    }
-                }
+              const auto &key = kv.key;
+              mapvar(vlistID2, varID, kv, key, &var, lvalid_min, lvalid_max, ptab, (ptmode != VARIABLE_NAME));
             }
-
           if (lvalid_min && lvalid_max) var.checkvalid = true;
         }
       else
@@ -293,7 +161,6 @@ void *
 Setpartab(void *process)
 {
   int nrecs;
-  int varID, levelID;
   size_t nmiss;
   int tableID = -1;
   int tableformat = 0;
@@ -301,12 +168,12 @@ Setpartab(void *process)
 
   cdoInitialize(process);
 
-  const int SETCODETAB = cdoOperatorAdd("setcodetab", 0, 0, "parameter code table name");
-  const int SETPARTABC = cdoOperatorAdd("setpartabc", 0, 0, "parameter table name");
-  const int SETPARTABP = cdoOperatorAdd("setpartabp", 0, 0, "parameter table name");
-  const int SETPARTABN = cdoOperatorAdd("setpartabn", 0, 0, "parameter table name");
+  const auto SETCODETAB = cdoOperatorAdd("setcodetab", 0, 0, "parameter code table name");
+  const auto SETPARTABC = cdoOperatorAdd("setpartabc", 0, 0, "parameter table name");
+  const auto SETPARTABP = cdoOperatorAdd("setpartabp", 0, 0, "parameter table name");
+  const auto SETPARTABN = cdoOperatorAdd("setpartabn", 0, 0, "parameter table name");
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
@@ -315,10 +182,10 @@ Setpartab(void *process)
   bool convert_data = false;
   if (operatorArgc() == 2)
     {
-      if (strcmp("convert", operatorArgv()[1]) == 0)
+      if (cdoOperatorArgv(1) == "convert")
         convert_data = true;
       else
-        cdoAbort("Unknown parameter: >%s<", operatorArgv()[1]);
+        cdoAbort("Unknown parameter: >%s<", cdoOperatorArgv(1).c_str());
     }
 
   if (operatorArgc() > 2) cdoAbort("Too many arguments!");
@@ -333,13 +200,12 @@ Setpartab(void *process)
 
   if (ptmode == CODE_NUMBER)
     {
-      char *partab = operatorArgv()[0];
-      FILE *fp = nullptr;
-      if (fileExists(partab)) fp = fopen(partab, "r");
+      auto partab = cdoOperatorArgv(0).c_str();
+      FILE *fp = fileExists(partab) ? fopen(partab, "r") : nullptr;
       if (fp != nullptr)
         {
           fseek(fp, 0L, SEEK_END);
-          size_t fsize = (size_t) ftell(fp);
+          const auto fsize = (size_t) ftell(fp);
           std::vector<char> parbuf(fsize + 1);
           fseek(fp, 0L, SEEK_SET);
           fread(parbuf.data(), fsize, 1, fp);
@@ -364,41 +230,41 @@ Setpartab(void *process)
 
   if (Options::cdoVerbose) cdoPrint("Table format version %d", tableformat);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  auto vlistID2 = vlistDuplicate(vlistID1);
   // vlistPrint(vlistID2);
 
-  const int nvars = vlistNvars(vlistID2);
+  const auto nvars = vlistNvars(vlistID2);
   std::vector<CmorVar> vars(nvars);
-  memset(vars.data(), 0, nvars * sizeof(CmorVar));
 
   if (convert_data)
-    for (varID = 0; varID < nvars; ++varID) vars[varID].convert = true;
+    for (int varID = 0; varID < nvars; ++varID) vars[varID].convert = true;
 
   if (tableformat == 0)
     {
-      // for ( int varID = 0; varID < nvars; varID++ ) vlistDefVarTable(vlistID2, varID, tableID);
+      // for (int varID = 0; varID < nvars; varID++) vlistDefVarTable(vlistID2, varID, tableID);
       char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
       for (int varID = 0; varID < nvars; varID++)
         {
-          const int param = vlistInqVarParam(vlistID2, varID);
+          const auto param = vlistInqVarParam(vlistID2, varID);
           int pdis, pcat, pnum;
           cdiDecodeParam(param, &pnum, &pcat, &pdis);
           if (pdis == 255)
             {
-              const int code = pnum;
-              const int ltype = zaxisInqLtype(vlistInqVarZaxis(vlistID2, varID));
+              const auto code = pnum;
+              int ltype = 0;
+              cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
               name[0] = 0;
               longname[0] = 0;
               units[0] = 0;
               tableInqEntry(tableID, code, ltype, name, longname, units);
               if (name[0])
                 {
-                  vlistDefVarName(vlistID2, varID, name);
-                  if (longname[0]) vlistDefVarLongname(vlistID2, varID, longname);
-                  if (units[0]) vlistDefVarUnits(vlistID2, varID, units);
+                  cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, name);
+                  if (longname[0]) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, longname);
+                  if (units[0]) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, units);
                 }
             }
           vlistDefVarTable(vlistID2, varID, tableID);
@@ -407,8 +273,8 @@ Setpartab(void *process)
   else
     {
       {
-        const char *filename = operatorArgv()[0];
-        FILE *fp = fopen(filename, "r");
+        const auto filename = cdoOperatorArgv(0).c_str();
+        auto fp = fopen(filename, "r");
         if (fp == nullptr) cdoAbort("Open failed on: %s\n", filename);
 
         PMList pmlist;
@@ -432,8 +298,8 @@ Setpartab(void *process)
 
           for (int varID = 0; varID < nvars; varID++)
             {
-              const int zaxisID = vlistInqVarZaxis(vlistID2, varID);
-              const int nlevs = zaxisInqSize(zaxisID);
+              const auto zaxisID = vlistInqVarZaxis(vlistID2, varID);
+              const auto nlevs = zaxisInqSize(zaxisID);
               for (int levID = 0; levID < nlevs; levID++)
                 {
                   vlistDefFlag(vlistID1, varID, levID, true);
@@ -446,7 +312,7 @@ Setpartab(void *process)
                 }
             }
 
-          int vlistIDx = vlistCreate();
+          auto vlistIDx = vlistCreate();
           cdoVlistCopyFlag(vlistIDx, vlistID2);
 
           vlistDestroy(vlistID2);
@@ -457,24 +323,27 @@ Setpartab(void *process)
 
       for (int varID = 0; varID < nvars; ++varID)
         {
-          CmorVar &var = vars[varID];
+          auto &var = vars[varID];
           if (!var.convert) var.changeunits = false;
           if (var.changeunits)
             cdoConvertUnits(&var.ut_converter, &var.changeunits, (char *) &var.units, (char *) &var.units_old, var.name);
         }
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   // vlistPrint(vlistID2);
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-  std::vector<double> array(gridsizemax);
+  Varray<double> array(gridsizemax);
+
+  VarList varList2;
+  varListInit(varList2, vlistID2);
 
   int tsID1 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
@@ -486,11 +355,12 @@ Setpartab(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          CmorVar &var = vars[varID];
-          int varID2 = varID;
-          int levelID2 = levelID;
+          auto &var = vars[varID];
+          auto varID2 = varID;
+          auto levelID2 = levelID;
 
           if (delvars)
             {
@@ -507,9 +377,8 @@ Setpartab(void *process)
 
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
-          const double missval = vlistInqVarMissval(vlistID2, varID2);
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
-          if (vlistInqVarNumber(vlistID2, varID2) != CDI_REAL) gridsize *= 2;
+          const auto missval = varList2[varID2].missval;
+          const auto gridsize = varList2[varID2].nwpv * varList2[varID2].gridsize;
 
           if (nmiss && var.changemissval)
             {
