@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "readline.h"
+#include "cdo_zaxis.h"
 
 #define MAX_LINE_LEN 4096
 
@@ -26,33 +28,33 @@ Setrcaname(void *process)
 {
   int nrecs;
   int varID, levelID;
-  char **rcsnames;
+  const char *rcsnames;
   char line[MAX_LINE_LEN];
   char sname[CDI_MAX_NAME], sdescription[CDI_MAX_NAME], sunits[CDI_MAX_NAME];
   int scode, sltype, slevel;
   int zaxisID, ltype, code, nlev;
   int level;
-  size_t gridsize, nmiss;
-  double *array = nullptr;
+  size_t nmiss;
+  Varray<double> array;
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
   operatorInputArg("file name with RCA names");
-  rcsnames = operatorArgv();
+  rcsnames = cdoOperatorArgv(0).c_str();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int nvars = vlistNvars(vlistID2);
+  const auto nvars = vlistNvars(vlistID2);
 
-  FILE *fp = fopen(rcsnames[0], "r");
+  auto fp = fopen(rcsnames, "r");
   if (fp != nullptr)
     {
-      while (readline(fp, line, MAX_LINE_LEN))
+      while (cdo::readline(fp, line, MAX_LINE_LEN))
         {
           sscanf(line, "%d\t%d\t%d\t%s\t%s\t%s", &scode, &sltype, &slevel, sname, sdescription, sunits);
           /*
@@ -81,17 +83,17 @@ Setrcaname(void *process)
                       level = (int) cdoZaxisInqLevel(zaxisID, 0);
                       if (sltype == 105 && slevel == level)
                         {
-                          vlistDefVarName(vlistID2, varID, sname);
-                          vlistDefVarLongname(vlistID2, varID, sdescription);
-                          vlistDefVarUnits(vlistID2, varID, sunits);
+                          cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, sname);
+                          cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, sdescription);
+                          cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, sunits);
                           break;
                         }
                     }
                   else if (sltype != 105)
                     {
-                      vlistDefVarName(vlistID2, varID, sname);
-                      vlistDefVarLongname(vlistID2, varID, sdescription);
-                      vlistDefVarUnits(vlistID2, varID, sunits);
+                      cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, sname);
+                      cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, sdescription);
+                      cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, sunits);
                       break;
                     }
                 }
@@ -102,21 +104,21 @@ Setrcaname(void *process)
     }
   else
     {
-      perror(rcsnames[0]);
+      perror(rcsnames);
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
   if (!lcopy)
     {
-      gridsize = vlistGridsizeMax(vlistID1);
-      array = (double *) Malloc(gridsize * sizeof(double));
+      const auto gridsizemax = vlistGridsizeMax(vlistID1);
+      array.resize(gridsizemax);
     }
 
   int tsID = 0;
@@ -136,8 +138,8 @@ Setrcaname(void *process)
             }
           else
             {
-              cdoReadRecord(streamID1, array, &nmiss);
-              cdoWriteRecord(streamID2, array, nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
@@ -149,9 +151,6 @@ Setrcaname(void *process)
 
   vlistDestroy(vlistID2);
 
-  if (!lcopy)
-    if (array) Free(array);
-
   cdoFinish();
 
   return nullptr;
