@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
 
 #include "cdo_zaxis.h"
@@ -31,12 +32,13 @@
 void genLayerBounds(int nlev, double *levels, double *lbounds, double *ubounds);
 
 int
-getkeyval_dp(const char *keyval, const char *key, double *val)
+getkeyval_dp(const std::string &p_keyval, const char *p_key, double *val)
 {
   int status = 0;
-  size_t keylen = strlen(key);
+  const char *keyval = p_keyval.c_str();
+  size_t keylen = strlen(p_key);
 
-  if (strncmp(keyval, key, keylen) == 0)
+  if (strncmp(keyval, p_key, keylen) == 0)
     {
       const char *pkv = keyval + keylen;
       if (pkv[0] == '=' && pkv[1] != 0)
@@ -67,8 +69,8 @@ Setzaxis(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int SETZAXIS       = cdoOperatorAdd("setzaxis",        0, 0, "zaxis description file");
-  const int GENLEVELBOUNDS = cdoOperatorAdd("genlevelbounds",  0, 0, nullptr);
+  const auto SETZAXIS       = cdoOperatorAdd("setzaxis",        0, 0, "zaxis description file");
+  const auto GENLEVELBOUNDS = cdoOperatorAdd("genlevelbounds",  0, 0, nullptr);
   // clang-format on
 
   int operatorID = cdoOperatorID();
@@ -77,14 +79,14 @@ Setzaxis(void *process)
     {
       operatorInputArg(cdoOperatorEnter(operatorID));
       operatorCheckArgc(1);
-      zaxisID2 = cdoDefineZaxis(operatorArgv()[0]);
+      zaxisID2 = cdoDefineZaxis(cdoOperatorArgv(0));
     }
   else if (operatorID == GENLEVELBOUNDS)
     {
-      unsigned npar = operatorArgc();
-      char **parnames = operatorArgv();
+      const auto npar = operatorArgc();
+      auto parnames = cdoGetOperArgv();
 
-      for (unsigned i = 0; i < npar; i++)
+      for (int i = 0; i < npar; i++)
         {
           if (Options::cdoVerbose) cdoPrint("keyval[%d]: %s", i + 1, parnames[i]);
 
@@ -97,16 +99,16 @@ Setzaxis(void *process)
         }
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   if (operatorID == SETZAXIS)
     {
@@ -130,12 +132,12 @@ Setzaxis(void *process)
       for (index = 0; index < nzaxis; index++)
         {
           zaxisID1 = vlistZaxis(vlistID1, index);
-          int nlev = zaxisInqSize(zaxisID1);
+          auto nlev = zaxisInqSize(zaxisID1);
           if (nlev > 1)
             {
-              std::vector<double> levels(nlev);
-              std::vector<double> lbounds(nlev);
-              std::vector<double> ubounds(nlev);
+              Varray<double> levels(nlev);
+              Varray<double> lbounds(nlev);
+              Varray<double> ubounds(nlev);
 
               cdoZaxisInqLevels(zaxisID1, levels.data());
               zaxisID2 = zaxisDuplicate(zaxisID1);
@@ -156,7 +158,7 @@ Setzaxis(void *process)
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-  std::vector<double> array(gridsize);
+  Varray<double> array(gridsize);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
