@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -31,13 +31,12 @@
       Showinfo   showformat      Show file format
 */
 
-#include <stdio.h>
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "process_int.h"
 #include "Showattribute.h"
 #include "printinfo.h"
+#include "cdo_zaxis.h"
 
 void *
 Showinfo(void *process)
@@ -49,33 +48,34 @@ Showinfo(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int SHOWYEAR      = cdoOperatorAdd("showyear",      0, 0, nullptr);
-  const int SHOWMON       = cdoOperatorAdd("showmon",       0, 0, nullptr);
-  const int SHOWDATE      = cdoOperatorAdd("showdate",      0, 0, nullptr);
-  const int SHOWTIME      = cdoOperatorAdd("showtime",      0, 0, nullptr);
-  const int SHOWTIMESTAMP = cdoOperatorAdd("showtimestamp", 0, 0, nullptr);
-  const int SHOWCODE      = cdoOperatorAdd("showcode",      0, 0, nullptr);
-  const int SHOWUNIT      = cdoOperatorAdd("showunit",      0, 0, nullptr);
-  const int SHOWPARAM     = cdoOperatorAdd("showparam",     0, 0, nullptr);
-  const int SHOWNAME      = cdoOperatorAdd("showname",      0, 0, nullptr);
-  const int SHOWSTDNAME   = cdoOperatorAdd("showstdname",   0, 0, nullptr);
-  const int SHOWLEVEL     = cdoOperatorAdd("showlevel",     0, 0, nullptr);
-  const int SHOWLTYPE     = cdoOperatorAdd("showltype",     0, 0, nullptr);
-  const int SHOWFORMAT    = cdoOperatorAdd("showformat",    0, 0, nullptr);
-  const int SHOWGRID      = cdoOperatorAdd("showgrid",      0, 0, nullptr); 
-  const int SHOWATTS      = cdoOperatorAdd("showatts",      0, 0, nullptr);
-  const int SHOWATTSGLOB  = cdoOperatorAdd("showattsglob",  0, 0, nullptr);
+  const auto SHOWYEAR      = cdoOperatorAdd("showyear",      0, 0, nullptr);
+  const auto SHOWMON       = cdoOperatorAdd("showmon",       0, 0, nullptr);
+  const auto SHOWDATE      = cdoOperatorAdd("showdate",      0, 0, nullptr);
+  const auto SHOWTIME      = cdoOperatorAdd("showtime",      0, 0, nullptr);
+  const auto SHOWTIMESTAMP = cdoOperatorAdd("showtimestamp", 0, 0, nullptr);
+  const auto SHOWCODE      = cdoOperatorAdd("showcode",      0, 0, nullptr);
+  const auto SHOWUNIT      = cdoOperatorAdd("showunit",      0, 0, nullptr);
+  const auto SHOWPARAM     = cdoOperatorAdd("showparam",     0, 0, nullptr);
+  const auto SHOWNAME      = cdoOperatorAdd("showname",      0, 0, nullptr);
+  const auto SHOWSTDNAME   = cdoOperatorAdd("showstdname",   0, 0, nullptr);
+  const auto SHOWLEVEL     = cdoOperatorAdd("showlevel",     0, 0, nullptr);
+  const auto SHOWLTYPE     = cdoOperatorAdd("showltype",     0, 0, nullptr);
+  const auto SHOWFORMAT    = cdoOperatorAdd("showformat",    0, 0, nullptr);
+  const auto SHOWGRID      = cdoOperatorAdd("showgrid",      0, 0, nullptr); 
+  const auto SHOWATTS      = cdoOperatorAdd("showatts",      0, 0, nullptr);
+  const auto SHOWATTSGLOB  = cdoOperatorAdd("showattsglob",  0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID = cdoStreamInqVlist(streamID);
+  const auto streamID = cdoOpenRead(0);
+  const auto vlistID = cdoStreamInqVlist(streamID);
 
-  const int nvars = vlistNvars(vlistID);
-  const int taxisID = vlistInqTaxis(vlistID);
-  const int ntsteps = vlistNtsteps(vlistID);
+  const auto nvars = vlistNvars(vlistID);
+  const auto taxisID = vlistInqTaxis(vlistID);
+  const auto ntsteps = vlistNtsteps(vlistID);
 
   if (operatorID == SHOWYEAR)
     {
@@ -83,8 +83,7 @@ Showinfo(void *process)
       if (ntsteps != 0)
         while (cdoStreamInqTimestep(streamID, tsID))
           {
-            const int64_t vdate = taxisInqVdate(taxisID);
-
+            const auto vdate = taxisInqVdate(taxisID);
             cdiDecodeDate(vdate, &year, &month, &day);
 
             if (tsID == 0 || year0 != year)
@@ -103,8 +102,7 @@ Showinfo(void *process)
       if (ntsteps != 0)
         while (cdoStreamInqTimestep(streamID, tsID))
           {
-            const int64_t vdate = taxisInqVdate(taxisID);
-
+            const auto vdate = taxisInqVdate(taxisID);
             cdiDecodeDate(vdate, &year, &month, &day);
 
             if (tsID == 0 || month0 != month)
@@ -119,19 +117,15 @@ Showinfo(void *process)
     }
   else if (operatorID == SHOWDATE)
     {
-      char vdatestr[32];
       int tsID = 0;
       if (ntsteps != 0)
         while (cdoStreamInqTimestep(streamID, tsID))
           {
-            const int64_t vdate = taxisInqVdate(taxisID);
-
-            date2str(vdate, vdatestr, sizeof(vdatestr));
-
+            const auto vdate = taxisInqVdate(taxisID);
             if (tsID == 0 || date0 != vdate)
               {
                 date0 = vdate;
-                fprintf(stdout, " %s", vdatestr);
+                fprintf(stdout, " %s", dateToString(vdate).c_str());
               }
 
             tsID++;
@@ -140,15 +134,12 @@ Showinfo(void *process)
     }
   else if (operatorID == SHOWTIME)
     {
-      char vtimestr[32];
       int tsID = 0;
       if (ntsteps != 0)
         while (cdoStreamInqTimestep(streamID, tsID))
           {
-            const int vtime = taxisInqVtime(taxisID);
-
-            time2str(vtime, vtimestr, sizeof(vtimestr));
-            fprintf(stdout, " %s", vtimestr);
+            const auto vtime = taxisInqVtime(taxisID);
+            fprintf(stdout, " %s", timeToString(vtime).c_str());
 
             tsID++;
           }
@@ -156,16 +147,13 @@ Showinfo(void *process)
     }
   else if (operatorID == SHOWTIMESTAMP)
     {
-      char vdatetimestr[64];
       int tsID = 0;
       if (ntsteps != 0)
         while (cdoStreamInqTimestep(streamID, tsID))
           {
-            const int64_t vdate = taxisInqVdate(taxisID);
-            const int vtime = taxisInqVtime(taxisID);
-
-            datetime2str(vdate, vtime, vdatetimestr, sizeof(vdatetimestr));
-            fprintf(stdout, " %s", vdatetimestr);
+            const auto vdate = taxisInqVdate(taxisID);
+            const auto vtime = taxisInqVtime(taxisID);
+            fprintf(stdout, " %s", datetimeToString(vdate, vtime).c_str());
 
             tsID++;
           }
@@ -184,8 +172,8 @@ Showinfo(void *process)
       fprintf(stdout, "# param nr | grid nr | z-axis nr:   /* Use in combination with operatores: griddes and zaxisdes */\n");
       for (int varID = 0; varID < nvars; varID++)
         {
-          const int gridID = vlistInqVarGrid(vlistID, varID);
-          const int zaxisID = vlistInqVarZaxis(vlistID, varID);
+          const auto gridID = vlistInqVarGrid(vlistID, varID);
+          const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
 
           fprintf(stdout, "      %3d     %3d      %3d\n", vlistInqVarCode(vlistID, varID), vlistGridIndex(vlistID, gridID) + 1,
                   vlistZaxisIndex(vlistID, zaxisID) + 1);
@@ -207,7 +195,7 @@ Showinfo(void *process)
       char paramstr[32];
       for (int varID = 0; varID < nvars; varID++)
         {
-          const int param = vlistInqVarParam(vlistID, varID);
+          const auto param = vlistInqVarParam(vlistID, varID);
           cdiParamToString(param, paramstr, sizeof(paramstr));
 
           fprintf(stdout, " %s", paramstr);
@@ -229,7 +217,8 @@ Showinfo(void *process)
       char stdname[CDI_MAX_NAME];
       for (int varID = 0; varID < nvars; varID++)
         {
-          vlistInqVarStdname(vlistID, varID, stdname);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID, varID, CDI_KEY_STDNAME, stdname, &length);
           fprintf(stdout, " %s", stdname[0] != 0 ? stdname : "unknown");
         }
       fprintf(stdout, "\n");
@@ -238,19 +227,19 @@ Showinfo(void *process)
     {
       for (int varID = 0; varID < nvars; varID++)
         {
-          const int zaxisID = vlistInqVarZaxis(vlistID, varID);
-          const int nlevs = zaxisInqSize(zaxisID);
+          const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
+          const auto nlevs = zaxisInqSize(zaxisID);
           for (int levelID = 0; levelID < nlevs; levelID++) fprintf(stdout, " %.9g", cdoZaxisInqLevel(zaxisID, levelID));
           fprintf(stdout, "\n");
         }
     }
   else if (operatorID == SHOWLTYPE)
     {
-      const int nzaxis = vlistNzaxis(vlistID);
+      const auto nzaxis = vlistNzaxis(vlistID);
       for (int index = 0; index < nzaxis; index++)
         {
-          const int zaxisID = vlistZaxis(vlistID, index);
-          const int ltype = zaxis2ltype(zaxisID);
+          const auto zaxisID = vlistZaxis(vlistID, index);
+          const auto ltype = zaxis2ltype(zaxisID);
 
           if (ltype != -1) fprintf(stdout, " %d", ltype);
         }
@@ -265,7 +254,8 @@ Showinfo(void *process)
       char stdname[CDI_MAX_NAME];
       for (int varID = 0; varID < nvars; varID++)
         {
-          vlistInqVarStdname(vlistID, varID, stdname);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID, varID, CDI_KEY_STDNAME, stdname, &length);
           fprintf(stdout, " %s", stdname[0] != 0 ? stdname : "unknown");
         }
       fprintf(stdout, "\n");
@@ -274,8 +264,6 @@ Showinfo(void *process)
     {
       if (operatorID == SHOWATTS)
         {
-          const int vlistID = cdoStreamInqVlist(streamID);
-          const int nvars = vlistNvars(vlistID);
           for (int varID = 0; varID < nvars; varID++)
             {
               char varname[CDI_MAX_NAME];
@@ -292,6 +280,7 @@ Showinfo(void *process)
       cdiInqNatts(vlistID, CDI_GLOBAL, &natts);
       printAtts(vlistID, CDI_GLOBAL, natts, nullptr);
     }
+
   cdoStreamClose(streamID);
 
   cdoFinish();
