@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,70 +21,65 @@
       Sort sortcode  Sort by code number
 */
 
+#include <algorithm> // sort
+
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "process_int.h"
 #include "param_conversion.h"
+#include "cdo_zaxis.h"
 
-struct levinfo_t
+struct LevInfo
 {
   int levelID;
   size_t nmiss;
   double level;
 };
 
-struct varinfo_t
+struct VarInfo
 {
   int varID;
   int nlevs;
   int code;
   char param[CDI_MAX_NAME];
   char name[CDI_MAX_NAME];
-  levinfo_t *levInfo;
+  std::vector<LevInfo> levInfo;
 };
 
-static int
-cmpvarcode(const void *a, const void *b)
+static bool
+cmpvarcode(const VarInfo &a, const VarInfo &b)
 {
-  const int x = ((const varinfo_t *) a)->code;
-  const int y = ((const varinfo_t *) b)->code;
-  return ((x > y) - (x < y)) * 2 + (x > y) - (x < y);
+  return a.code < b.code;
 }
 
-static int
-cmpvarparam(const void *a, const void *b)
+static bool
+cmpvarparam(const VarInfo &a, const VarInfo &b)
 {
-  const char *x = ((const varinfo_t *) a)->param;
-  const char *y = ((const varinfo_t *) b)->param;
-  return strcmp(x, y);
+  return strcmp(a.param, b.param) < 0;
 }
 
-static int
-cmpvarname(const void *a, const void *b)
+static bool
+cmpvarname(const VarInfo &a, const VarInfo &b)
 {
-  const char *x = ((const varinfo_t *) a)->name;
-  const char *y = ((const varinfo_t *) b)->name;
-  return strcmp(x, y);
+  return strcmp(a.name, b.name) < 0;
 }
 
-static int
-cmpvarlevel(const void *a, const void *b)
+static bool
+cmpvarlevel(const LevInfo &a, const LevInfo &b)
 {
-  const double x = ((const levinfo_t *) a)->level;
-  const double y = ((const levinfo_t *) b)->level;
-  return ((x > y) - (x < y)) * 2 + (x > y) - (x < y);
+  return a.level < b.level;
 }
 
-static int
-cmpvarlevelrev(const void *a, const void *b)
+static bool
+cmpvarlevelrev(const LevInfo &a, const LevInfo &b)
 {
-  const double x = ((const levinfo_t *) a)->level;
-  const double y = ((const levinfo_t *) b)->level;
-  return -1 * (((x > y) - (x < y)) * 2 + (x > y) - (x < y));
+  return a.level > b.level;
 }
 
 static void
-setNmiss(int varID, int levelID, int nvars, varinfo_t *varInfo, size_t nmiss)
+setNmiss(int varID, int levelID, int nvars, std::vector<VarInfo> &varInfo, size_t nmiss)
 {
   int vindex, lindex;
 
@@ -93,7 +88,7 @@ setNmiss(int varID, int levelID, int nvars, varinfo_t *varInfo, size_t nmiss)
 
   if (vindex == nvars) cdoAbort("Internal problem; varID not found!");
 
-  int nlevs = varInfo[vindex].nlevs;
+  const auto nlevs = varInfo[vindex].nlevs;
   for (lindex = 0; lindex < nlevs; lindex++)
     if (varInfo[vindex].levInfo[lindex].levelID == levelID) break;
 
@@ -102,62 +97,41 @@ setNmiss(int varID, int levelID, int nvars, varinfo_t *varInfo, size_t nmiss)
   varInfo[vindex].levInfo[lindex].nmiss = nmiss;
 }
 
-void
-paramToStringLong(int param, char *paramstr, int maxlen)
-{
-  int len;
-
-  int dis, cat, num;
-  cdiDecodeParam(param, &num, &cat, &dis);
-
-  size_t umaxlen = maxlen >= 0 ? (unsigned) maxlen : 0U;
-  if (dis == 255 && (cat == 255 || cat == 0))
-    len = snprintf(paramstr, umaxlen, "%03d", num);
-  else if (dis == 255)
-    len = snprintf(paramstr, umaxlen, "%03d.%03d", num, cat);
-  else
-    len = snprintf(paramstr, umaxlen, "%03d.%03d.%03d", num, cat, dis);
-
-  if (len >= maxlen || len < 0) fprintf(stderr, "Internal problem (%s): size of input string is too small!\n", __func__);
-}
-
 void *
 Sort(void *process)
 {
   int varID, levelID, zaxisID;
   int vindex, lindex;
-  int nrecs, nlevs, offset;
-  size_t gridsize;
+  int nlevs;
   size_t nmiss;
-  double *single;
-  int (*cmpvarlev)(const void *, const void *) = cmpvarlevel;
+  bool (*cmpvarlev)(const LevInfo &a, const LevInfo &b) = cmpvarlevel;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int SORTCODE  = cdoOperatorAdd("sortcode",  0, 0, nullptr);
-  const int SORTPARAM = cdoOperatorAdd("sortparam", 0, 0, nullptr);
-  const int SORTNAME  = cdoOperatorAdd("sortname",  0, 0, nullptr);
-  const int SORTLEVEL = cdoOperatorAdd("sortlevel", 0, 0, nullptr);
+  const auto SORTCODE  = cdoOperatorAdd("sortcode",  0, 0, nullptr);
+  const auto SORTPARAM = cdoOperatorAdd("sortparam", 0, 0, nullptr);
+  const auto SORTNAME  = cdoOperatorAdd("sortname",  0, 0, nullptr);
+  const auto SORTLEVEL = cdoOperatorAdd("sortlevel", 0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorArgc() > 1) cdoAbort("Too many arguments!");
 
   if (operatorID == SORTLEVEL && operatorArgc() == 1)
     {
-      int iarg = parameter2int(operatorArgv()[0]);
+      auto iarg = parameter2int(cdoOperatorArgv(0));
       if (iarg < 0) cmpvarlev = cmpvarlevelrev;
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
   /*
   if ( operatorID == SORTCODE )
@@ -168,31 +142,34 @@ Sort(void *process)
       ;
   */
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
-  varinfo_t *varInfo = (varinfo_t *) Malloc(nvars * sizeof(varinfo_t));
+  const auto nvars = vlistNvars(vlistID1);
+
+  std::vector<VarInfo> varInfo(nvars);
   for (varID = 0; varID < nvars; ++varID)
     {
       nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
       varInfo[varID].nlevs = nlevs;
-      varInfo[varID].levInfo = (levinfo_t *) Malloc(nlevs * sizeof(levinfo_t));
+      varInfo[varID].levInfo.resize(nlevs);
     }
 
-  double **vardata = (double **) Malloc(nvars * sizeof(double *));
-
+  Varray2D<double> vardata(nvars);
   for (varID = 0; varID < nvars; varID++)
     {
-      gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-      nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      vardata[varID] = (double *) Malloc(gridsize * nlevs * sizeof(double));
+      vardata[varID].resize(varList1[varID].gridsize * varList1[varID].nlevels);
     }
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
@@ -204,17 +181,16 @@ Sort(void *process)
             {
               varInfo[varID].varID = varID;
               varInfo[varID].code = vlistInqVarCode(vlistID1, varID);
-              const int iparam = vlistInqVarParam(vlistID1, varID);
-              paramToStringLong(iparam, varInfo[varID].param, sizeof(varInfo[varID].param));
+              const auto iparam = vlistInqVarParam(vlistID1, varID);
+              paramToString(iparam, varInfo[varID].param, sizeof(varInfo[varID].param));
               vlistInqVarName(vlistID1, varID, varInfo[varID].name);
               zaxisID = vlistInqVarZaxis(vlistID1, varID);
               varInfo[varID].levInfo[levelID].levelID = levelID;
               varInfo[varID].levInfo[levelID].level = cdoZaxisInqLevel(zaxisID, levelID);
             }
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          offset = gridsize * levelID;
-          single = vardata[varID] + offset;
+          const auto offset = varList1[varID].gridsize * levelID;
+          auto single = &vardata[varID][offset];
 
           cdoReadRecord(streamID1, single, &nmiss);
 
@@ -234,18 +210,15 @@ Sort(void *process)
               }
 
           if (operatorID == SORTCODE)
-            qsort(varInfo, nvars, sizeof(varinfo_t), cmpvarcode);
+            std::sort(varInfo.begin(), varInfo.end(), cmpvarcode);
           else if (operatorID == SORTPARAM)
-            qsort(varInfo, nvars, sizeof(varinfo_t), cmpvarparam);
+            std::sort(varInfo.begin(), varInfo.end(), cmpvarparam);
           else if (operatorID == SORTNAME)
-            qsort(varInfo, nvars, sizeof(varinfo_t), cmpvarname);
+            std::sort(varInfo.begin(), varInfo.end(), cmpvarname);
           else if (operatorID == SORTLEVEL)
             {
               for (vindex = 0; vindex < nvars; vindex++)
-                {
-                  nlevs = varInfo[vindex].nlevs;
-                  qsort(varInfo[vindex].levInfo, nlevs, sizeof(levinfo_t), cmpvarlev);
-                }
+                std::sort(varInfo[vindex].levInfo.begin(), varInfo[vindex].levInfo.end(), cmpvarlev);
             }
 
           if (Options::cdoVerbose)
@@ -269,9 +242,8 @@ Sort(void *process)
 
               if (tsID == 0 || vlistInqVarTimetype(vlistID1, varID) != TIME_CONSTANT)
                 {
-                  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-                  offset = gridsize * levelID;
-                  single = vardata[varID] + offset;
+                  const auto offset = varList1[varID].gridsize * levelID;
+                  auto single = &vardata[varID][offset];
 
                   cdoDefRecord(streamID2, varID, levelID);
                   cdoWriteRecord(streamID2, single, nmiss);
@@ -285,12 +257,6 @@ Sort(void *process)
   cdoStreamClose(streamID1);
   cdoStreamClose(streamID2);
 
-  for (varID = 0; varID < nvars; varID++) Free(vardata[varID]);
-  Free(vardata);
-
-  for (vindex = 0; vindex < nvars; vindex++) Free(varInfo[vindex].levInfo);
-  Free(varInfo);
-
   cdoFinish();
 
   return nullptr;
