@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,25 +21,25 @@
       Specinfo specinfo  Spectral information
 */
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 
 #define NTR2NSP(ntr) ((ntr + 1) * (ntr + 2))
-#define NSP2NTR(nsp) ((int) ((((sqrt((double) (4 * nsp + 1))) - 3) / 2)))
-#define NGP2NLEVEL(ngp) ((int) (log10(((double) ngp) / 80.) / log10(4.)))
-#define NGP_ICON(nrooti, nlevel) ((int) (20 * nrooti * nrooti * ipow(4, nlevel)))
+#define NSP2NTR(nsp) ((long) ((((std::sqrt((double) (4 * nsp + 1))) - 3) / 2)))
+#define NGP2NLEVEL(ngp) ((long) (log10(((double) ngp) / 80.) / std::log10(4.)))
+#define NGP_ICON(nrooti, nlevel) ((long) (20 * nrooti * nrooti * ipow(4, nlevel)))
 /*#define NGP_GME(ni)           ((ni+1)*(ni+1)*10)*/
 #define NGP_GME(ni) (2 + ni * ni * 10)
-#define NGP2NI(ngp) ((int) sqrt((double) ngp / 10.) - 1)
+#define NGP2NI(ngp) ((long) std::sqrt((double) ngp / 10.) - 1)
 
 static void
-fac(int nlonin, int *nlonout, int *ierr)
+fac(long nlonin, long *nlonout, int *ierr)
 {
-  int n2 = 0;
-  int n3 = 0;
-  int n5 = 0;
+  long n2 = 0;
+  long n3 = 0;
+  long n5 = 0;
 
-  int m = nlonin;
+  long m = nlonin;
 
   while (m % 2 == 0)
     {
@@ -71,14 +71,15 @@ fac(int nlonin, int *nlonout, int *ierr)
   return;
 }
 
-static int
-nlat2nlon(int nlat)
+static long
+nlat2nlon(long nlat)
 {
   if (nlat == 0) cdoAbort("nlat = 0!");
 
-  int nlon = 2 * nlat;
+  long nlon = 2 * nlat;
 
-  int m, ierr;
+  long m;
+  int ierr;
   fac(nlon, &m, &ierr);
   /* adjust till fft is possible */
   while (ierr != 0)
@@ -92,57 +93,68 @@ nlat2nlon(int nlat)
   return nlon;
 }
 
-int
-ngp2ntr(int ngp)
+long
+ngp2ntr(long ngp)
 {
-  int ntr = (int) lround(sqrt(0.25 + ngp) - 1.5);
-  int nlonl = nlat_to_nlon(ntr_to_nlat_linear(ntr));
-  int nlatl = nlonl / 2;
+  long ntr = (long) std::lround(std::sqrt(0.25 + ngp) - 1.5);
+  long nlonl = nlat_to_nlon(ntr_to_nlat_linear(ntr));
+  long nlatl = nlonl / 2;
 
   ntr = (2 * nlatl - 1) / 2;
 
   return ntr;
 }
 
-int
-ipow(int i1, int i2)
+static long
+ipow(long i1, long i2)
 {
-  int i3 = 1;
+  long i3 = 1;
 
-  for (int i = 0; i < i2; ++i) i3 *= i1;
+  for (long i = 0; i < i2; ++i) i3 *= i1;
 
   return i3;
 }
 
-void
-lookup_ni(int nsp, int *nroot, int *ni)
-{
-  int tbl2[12], tbl3[12], tbl5[12];
-  int i, d, d2, n2, d3, n3, d5, n5;
+static constexpr long NiMax = 12;
 
-  for (i = 0; i < 12; ++i)
+static void
+lookup_ni(long nsp, long *nroot, long *ni)
+{
+  long tbl2[NiMax], tbl3[NiMax], tbl5[NiMax];
+  long d2 = 0, n2 = 0, d3 = 0, n3 = 0, d5 = 0, n5 = 0;
+
+  for (long i = 0; i < NiMax; ++i)
     {
       tbl2[i] = 10 * 2 * 2 * ipow(4, (i + 1)) + 2;
       tbl3[i] = 10 * 3 * 3 * ipow(4, (i + 1)) + 2;
       tbl5[i] = 10 * 5 * 5 * ipow(4, (i + 1)) + 2;
     }
 
-  for (i = 0; i < 12; ++i)
-    if (tbl2[i] >= nsp) break;
-  n2 = i;
-  d2 = tbl2[n2] - nsp;
+  for (long i = 0; i < NiMax; ++i)
+    if (tbl2[i] >= nsp)
+      {
+        n2 = i;
+        d2 = tbl2[n2] - nsp;
+        break;
+      }
 
-  for (i = 0; i < 12; ++i)
-    if (tbl3[i] >= nsp) break;
-  n3 = i;
-  d3 = tbl3[n3] - nsp;
+  for (long i = 0; i < NiMax; ++i)
+    if (tbl3[i] >= nsp)
+      {
+        n3 = i;
+        d3 = tbl3[n3] - nsp;
+        break;
+      }
 
-  for (i = 0; i < 12; ++i)
-    if (tbl5[i] >= nsp) break;
-  n5 = i;
-  d5 = tbl5[n5] - nsp;
+  for (long i = 0; i < NiMax; ++i)
+    if (tbl5[i] >= nsp)
+      {
+        n5 = i;
+        d5 = tbl5[n5] - nsp;
+        break;
+      }
 
-  d = d2;
+  long d = d2;
   if (d3 < d) d = d3;
   if (d5 < d) d = d5;
 
@@ -163,35 +175,44 @@ lookup_ni(int nsp, int *nroot, int *ni)
     }
 }
 
-void
-lookup_rl(int nsp, int *nroot, int *nlevel)
+static void
+lookup_rl(long nsp, long *nroot, long *nlevel)
 {
-  int tbl2[12], tbl3[12], tbl5[12];
-  int i, d, d2, n2, d3, n3, d5, n5;
+  long tbl2[NiMax], tbl3[NiMax], tbl5[NiMax];
+  long d2 = 0, n2 = 0, d3 = 0, n3 = 0, d5 = 0, n5 = 0;
 
-  for (i = 0; i < 12; ++i)
+  for (long i = 0; i < NiMax; ++i)
     {
       tbl2[i] = 20 * 2 * 2 * ipow(4, (i + 1));
       tbl3[i] = 20 * 3 * 3 * ipow(4, (i + 1));
       tbl5[i] = 20 * 5 * 5 * ipow(4, (i + 1));
     }
 
-  for (i = 0; i < 12; ++i)
-    if (tbl2[i] >= nsp) break;
-  n2 = i;
-  d2 = tbl2[n2] - nsp;
+  for (long i = 0; i < NiMax; ++i)
+    if (tbl2[i] >= nsp)
+      {
+        n2 = i;
+        d2 = tbl2[n2] - nsp;
+        break;
+      }
 
-  for (i = 0; i < 12; ++i)
-    if (tbl3[i] >= nsp) break;
-  n3 = i;
-  d3 = tbl3[n3] - nsp;
+  for (long i = 0; i < NiMax; ++i)
+    if (tbl3[i] >= nsp)
+      {
+        n3 = i;
+        d3 = tbl3[n3] - nsp;
+        break;
+      }
 
-  for (i = 0; i < 12; ++i)
-    if (tbl5[i] >= nsp) break;
-  n5 = i;
-  d5 = tbl5[n5] - nsp;
+  for (long i = 0; i < NiMax; ++i)
+    if (tbl5[i] >= nsp)
+      {
+        n5 = i;
+        d5 = tbl5[n5] - nsp;
+        break;
+      }
 
-  d = d2;
+  long d = d2;
   if (d3 < d) d = d3;
   if (d5 < d) d = d5;
 
@@ -217,24 +238,24 @@ Specinfo(void *process)
 {
   char arg[128], *parg;
   bool nout1 = false, nout2 = false, nout3 = false;
-  int i;
-  int ntr1 = 0, nsp1 = 0, nlat1 = 0, nlon1 = 0, ngp1 = 0, ni1 = 0, ngp_gme1 = 0;
-  int ntr2 = 0, nsp2 = 0, nlat2 = 0, nlon2 = 0, ngp2 = 0, ni2 = 0, ngp_gme2 = 0;
-  int ntr3 = 0, nsp3 = 0, nlat3 = 0, nlon3 = 0, ngp3 = 0, ni3 = 0, ngp_gme3 = 0;
-  int nlevel1 = 0, nlevel2 = 0, nlevel3 = 0;
-  int ngp_icon1 = 0, ngp_icon2 = 0, ngp_icon3 = 0;
-  int nrootg1 = 0, nrootg2 = 0, nrootg3 = 0;
-  int nrooti1 = 0, nrooti2 = 0, nrooti3 = 0;
+  long i;
+  long ntr1 = 0, nsp1 = 0, nlat1 = 0, nlon1 = 0, ngp1 = 0, ni1 = 0, ngp_gme1 = 0;
+  long ntr2 = 0, nsp2 = 0, nlat2 = 0, nlon2 = 0, ngp2 = 0, ni2 = 0, ngp_gme2 = 0;
+  long ntr3 = 0, nsp3 = 0, nlat3 = 0, nlon3 = 0, ngp3 = 0, ni3 = 0, ngp_gme3 = 0;
+  long nlevel1 = 0, nlevel2 = 0, nlevel3 = 0;
+  long ngp_icon1 = 0, ngp_icon2 = 0, ngp_icon3 = 0;
+  long nrootg1 = 0, nrootg2 = 0, nrootg3 = 0;
+  long nrooti1 = 0, nrooti2 = 0, nrooti3 = 0;
 
   cdoInitialize(process);
 
   operatorInputArg("Txx, TLxx, NLON=xx, NLAT=xx, NIxx or ICONRyyLxx");
 
-  const int len = strlen(operatorArgv()[0]);
+  const long len = cdoOperatorArgv(0).size();
 
   if ((len + 1) >= 128) cdoAbort("Parameter string too large!");
 
-  for (i = 0; i < len; i++) arg[i] = toupper(operatorArgv()[0][i]);
+  for (i = 0; i < len; i++) arg[i] = toupper(cdoOperatorArgv(0)[i]);
   arg[len] = 0;
 
   if (arg[0] == 'T' && arg[1] == 'L')
@@ -484,7 +505,7 @@ Specinfo(void *process)
       nrooti1 = atoi(parg);
       nrooti2 = nrooti1;
       while (isdigit((int) *parg)) parg++;
-      if (*parg != 'L') cdoAbort("Wrong parameter: %s", arg);
+      if (*parg != 'B') cdoAbort("Wrong parameter: %s", arg);
       parg++;
       if (!isdigit((int) *parg)) cdoAbort("Wrong parameter: %s", arg);
       nlevel1 = atoi(parg);
@@ -535,15 +556,15 @@ Specinfo(void *process)
   fprintf(stdout, "truncation     nsp  nlon  nlat      ngp  gme    ngp_gme  icon   ngp_icon\n");
 
   if (nout2)
-    fprintf(stdout, "   TL%-4d %8d %5d %5d %8d  ni%d %8d  R%dB%02d  %8d\n", ntr2, nsp2, nlon2, nlat2, ngp2, ni2, ngp_gme2, nrooti2,
+    fprintf(stdout, "   TL%-4ld %8ld %5ld %5ld %8ld  ni%ld %8ld  R%ldB%02ld  %8ld\n", ntr2, nsp2, nlon2, nlat2, ngp2, ni2, ngp_gme2, nrooti2,
             nlevel2, ngp_icon2);
 
   if (nout1)
-    fprintf(stdout, "   TQ%-4d %8d %5d %5d %8d  ni%d %8d  R%dB%02d  %8d\n", ntr1, nsp1, nlon1, nlat1, ngp1, ni1, ngp_gme1, nrooti1,
+    fprintf(stdout, "   TQ%-4ld %8ld %5ld %5ld %8ld  ni%ld %8ld  R%ldB%02ld  %8ld\n", ntr1, nsp1, nlon1, nlat1, ngp1, ni1, ngp_gme1, nrooti1,
             nlevel1, ngp_icon1);
 
   if (nout3)
-    fprintf(stdout, "   TC%-4d %8d %5d %5d %8d  ni%d %8d  R%dB%02d  %8d\n", ntr3, nsp3, nlon3, nlat3, ngp3, ni3, ngp_gme3, nrooti3,
+    fprintf(stdout, "   TC%-4ld %8ld %5ld %5ld %8ld  ni%ld %8ld  R%ldB%02ld  %8ld\n", ntr3, nsp3, nlon3, nlat3, ngp3, ni3, ngp_gme3, nrooti3,
             nlevel3, ngp_icon3);
 
   cdoFinish();
