@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_cdi_wrapper.h"
 #include "param_conversion.h"
 #include "statistic.h"
 
-#define NALLOC_INC 1024
 
 static void
 spectrum(int nrec, double *data, double *spectrum, double *real, double *imag, const double *window, double wssum, int detrend,
@@ -62,7 +63,7 @@ spectrum(int nrec, double *data, double *spectrum, double *real, double *imag, c
       for (k = 0; k < nrec; k++) data[k] -= a + b * k;
     }
 
-  std::vector<double> work_r, work_i;
+  Varray<double> work_r, work_i;
   if (bit != 1)
     {
       work_r.resize(seg_l);
@@ -116,10 +117,10 @@ spectrum(int nrec, double *data, double *spectrum, double *real, double *imag, c
       else
         for (k = 0; k < seg_l; k++) imag[k] = 0;
 
-      if (bit == 1) /* seg_l is a power of 2 */
-        fft(real, imag, seg_l, 1);
+      if (bit == 1) // seg_l is a power of 2
+        cdo::fft(real, imag, seg_l, 1);
       else
-        ft_r(real, imag, seg_l, 1, work_r.data(), work_i.data());
+        cdo::ft_r(real, imag, seg_l, 1, work_r.data(), work_i.data());
 
       spectrum[0] += real[0] * real[0] + imag[0] * imag[0];
 
@@ -140,7 +141,6 @@ spectrum(int nrec, double *data, double *spectrum, double *real, double *imag, c
 void *
 Spectrum(void *process)
 {
-  int nrecs;
   int varID, levelID;
   int k;
   int nalloc = 0;
@@ -149,31 +149,35 @@ Spectrum(void *process)
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = cdoTaxisCreate(TAXIS_ABSOLUTE);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   VarList varList;
   varListInit(varList, vlistID1);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   FieldVector3D vars;
   std::vector<int64_t> vdate;
   std::vector<int> vtime;
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       if (tsID >= nalloc)
         {
+          constexpr int NALLOC_INC = 1024;
           nalloc += NALLOC_INC;
           vdate.resize(nalloc);
           vtime.resize(nalloc);
@@ -183,13 +187,13 @@ Spectrum(void *process)
       vdate[tsID] = taxisInqVdate(taxisID1);
       vtime[tsID] = taxisInqVtime(taxisID1);
 
-      fieldsFromVlist(vlistID1, vars[tsID], FIELD_NONE);
+      fieldsFromVlist(vlistID1, vars[tsID]);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
           vars[tsID][varID][levelID].resize(varList[varID].gridsize);
-          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec_d.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
 
           if (nmiss) cdoAbort("Missing values are not allowed!");
@@ -212,10 +216,10 @@ Spectrum(void *process)
 
   operatorCheckArgc(4);
 
-  const int detrend = parameter2int(operatorArgv()[0]);
-  const int seg_l = parameter2int(operatorArgv()[1]);
-  const int seg_n = parameter2int(operatorArgv()[2]);
-  const int which_window = parameter2int(operatorArgv()[3]);
+  const auto detrend = parameter2int(cdoOperatorArgv(0));
+  const auto seg_l = parameter2int(cdoOperatorArgv(1));
+  const auto seg_n = parameter2int(cdoOperatorArgv(2));
+  const auto which_window = parameter2int(cdoOperatorArgv(3));
 
   if (detrend < 0 || detrend > 3) cdoAbort("Illegal value for detrend (=%d)!", detrend);
 
@@ -229,11 +233,11 @@ Spectrum(void *process)
   FieldVector3D vars2(nfreq);
   for (freq = 0; freq < nfreq; freq++) fieldsFromVlist(vlistID1, vars2[freq], FIELD_VEC);
 
-  std::vector<double> array1(nts);
-  std::vector<double> array2(nfreq);
-  std::vector<double> real(seg_l);
-  std::vector<double> imag(seg_l);
-  std::vector<double> window(seg_l);
+  Varray<double> array1(nts);
+  Varray<double> array2(nfreq);
+  Varray<double> real(seg_l);
+  Varray<double> imag(seg_l);
+  Varray<double> window(seg_l);
 
   switch (which_window)
     {
@@ -241,7 +245,7 @@ Spectrum(void *process)
       for (k = 0; k < seg_l; k++) window[k] = 1;
       break;
     case 1:
-      for (k = 0; k < seg_l; k++) window[k] = 1 - cos(2 * M_PI * (k + 1) / (seg_l + 1));
+      for (k = 0; k < seg_l; k++) window[k] = 1 - std::cos(2 * M_PI * (k + 1) / (seg_l + 1));
       break;
     case 2:
       for (k = 0; k < seg_l / 2; k++) window[k] = window[seg_l - 1 - k] = k;
@@ -267,13 +271,13 @@ Spectrum(void *process)
         {
           for (size_t i = 0; i < gridsize; i++)
             {
-              for (tsID = 0; tsID < nts; tsID++) array1[tsID] = vars[tsID][varID][levelID].vec[i];
+              for (tsID = 0; tsID < nts; tsID++) array1[tsID] = vars[tsID][varID][levelID].vec_d[i];
 
               for (freq = 0; freq < nfreq; freq++) array2[freq] = 0;
 
               spectrum(nts, array1.data(), array2.data(), real.data(), imag.data(), window.data(), wssum, detrend, seg_n, seg_l);
 
-              for (freq = 0; freq < nfreq; freq++) vars2[freq][varID][levelID].vec[i] = array2[freq];
+              for (freq = 0; freq < nfreq; freq++) vars2[freq][varID][levelID].vec_d[i] = array2[freq];
             }
         }
     }
@@ -286,14 +290,14 @@ Spectrum(void *process)
 
       for (varID = 0; varID < nvars; varID++)
         {
-          const int nlevels = varList[varID].nlevels;
+          const auto nlevels = varList[varID].nlevels;
           for (levelID = 0; levelID < nlevels; levelID++)
             {
               if (!vars2[tsID][varID][levelID].empty())
                 {
                   nmiss = vars2[tsID][varID][levelID].nmiss;
                   cdoDefRecord(streamID2, varID, levelID);
-                  cdoWriteRecord(streamID2, vars2[tsID][varID][levelID].vec.data(), 0);
+                  cdoWriteRecord(streamID2, vars2[tsID][varID][levelID].vec_d.data(), 0);
                 }
             }
         }
