@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,10 +29,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "process_int.h"
 #include "cdo_history.h"
 #include "util_files.h"
+#include "cdo_zaxis.h"
+#include "cdi_lockedIO.h"
 
 static void
 gen_filename(char *filename, bool swap_obase, const char *obase, const char *suffix)
@@ -48,11 +49,8 @@ Split(void *process)
   int varID;
   int levelID, levID;
   int varID2, levelID2;
-  int vlistID2;
   int itmp[999];
   double ftmp[999];
-  char filesuffix[32];
-  char filename[8192];
   int nsplit = 0;
   size_t nmiss;
   bool swap_obase = false;
@@ -64,43 +62,47 @@ Split(void *process)
 
   if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
 
-  const bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
   // clang-format off
-  const int SPLITCODE   = cdoOperatorAdd("splitcode",   0, 0, nullptr);
-  const int SPLITPARAM  = cdoOperatorAdd("splitparam",  0, 0, nullptr);
-  const int SPLITNAME   = cdoOperatorAdd("splitname",   0, 0, nullptr);
-  const int SPLITLEVEL  = cdoOperatorAdd("splitlevel",  0, 0, nullptr);
-  const int SPLITGRID   = cdoOperatorAdd("splitgrid",   0, 0, nullptr);
-  const int SPLITZAXIS  = cdoOperatorAdd("splitzaxis",  0, 0, nullptr);
-  const int SPLITTABNUM = cdoOperatorAdd("splittabnum", 0, 0, nullptr);
+  const auto SPLITCODE   = cdoOperatorAdd("splitcode",   0, 0, nullptr);
+  const auto SPLITPARAM  = cdoOperatorAdd("splitparam",  0, 0, nullptr);
+  const auto SPLITNAME   = cdoOperatorAdd("splitname",   0, 0, nullptr);
+  const auto SPLITLEVEL  = cdoOperatorAdd("splitlevel",  0, 0, nullptr);
+  const auto SPLITGRID   = cdoOperatorAdd("splitgrid",   0, 0, nullptr);
+  const auto SPLITZAXIS  = cdoOperatorAdd("splitzaxis",  0, 0, nullptr);
+  const auto SPLITTABNUM = cdoOperatorAdd("splittabnum", 0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   for (int i = 0; i < operatorArgc(); ++i)
     {
-      if (strcmp("swap", operatorArgv()[i]) == 0)
+      if (cdoOperatorArgv(i) == "swap")
         swap_obase = true;
-      else if (strncmp("uuid=", operatorArgv()[i], 5) == 0)
-        uuid_attribute = operatorArgv()[i] + 5;
+      else if (cdoOperatorArgv(i).find("uuid=") == 0)
+        uuid_attribute = &cdoOperatorArgv(i)[0] + 5;
       else
-        cdoAbort("Unknown parameter: >%s<", operatorArgv()[0]);
+        cdoAbort("Unknown parameter: >%s<", cdoOperatorArgv(0).c_str());
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const int nvars = vlistNvars(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
+  const auto nvars = vlistNvars(vlistID1);
+
+  char filename[8192];
   if (!swap_obase)
     {
       strcpy(filename, cdoGetObase());
       nchars = strlen(filename);
     }
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   if (operatorID == SPLITCODE)
@@ -108,10 +110,10 @@ Split(void *process)
       nsplit = 0;
       for (varID = 0; varID < nvars; varID++)
         {
-          const int code = vlistInqVarCode(vlistID1, varID);
+          const auto code = varList1[varID].code;
           int index;
           for (index = 0; index < varID; index++)
-            if (code == vlistInqVarCode(vlistID1, index)) break;
+            if (code == varList1[index].code) break;
 
           if (index == varID)
             {
@@ -130,11 +132,10 @@ Split(void *process)
           vlistClearFlag(vlistID1);
           for (varID = 0; varID < nvars; varID++)
             {
-              const int code = vlistInqVarCode(vlistID1, varID);
+              const auto code = varList1[varID].code;
               if (codes[index] == code)
                 {
-                  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  for (levID = 0; levID < nlevs; levID++)
+                  for (levID = 0; levID <  varList1[varID].nlevels; levID++)
                     {
                       vlistDefIndex(vlistID1, varID, levID, index);
                       vlistDefFlag(vlistID1, varID, levID, true);
@@ -142,7 +143,7 @@ Split(void *process)
                 }
             }
 
-          vlistID2 = vlistCreate();
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
@@ -171,10 +172,10 @@ Split(void *process)
       nsplit = 0;
       for (varID = 0; varID < nvars; varID++)
         {
-          const int param = vlistInqVarParam(vlistID1, varID);
+          const auto param = varList1[varID].param;
           int index;
           for (index = 0; index < varID; index++)
-            if (param == vlistInqVarParam(vlistID1, index)) break;
+            if (param == varList1[index].param) break;
 
           if (index == varID)
             {
@@ -193,11 +194,10 @@ Split(void *process)
           vlistClearFlag(vlistID1);
           for (varID = 0; varID < nvars; varID++)
             {
-              const int param = vlistInqVarParam(vlistID1, varID);
+              const auto param = varList1[varID].param;
               if (params[index] == param)
                 {
-                  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  for (levID = 0; levID < nlevs; levID++)
+                  for (levID = 0; levID < varList1[varID].nlevels; levID++)
                     {
                       vlistDefIndex(vlistID1, varID, levID, index);
                       vlistDefFlag(vlistID1, varID, levID, true);
@@ -205,7 +205,7 @@ Split(void *process)
                 }
             }
 
-          vlistID2 = vlistCreate();
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
@@ -223,7 +223,7 @@ Split(void *process)
       nsplit = 0;
       for (varID = 0; varID < nvars; varID++)
         {
-          const int tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+          const auto tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
           int index;
           for (index = 0; index < varID; index++)
             if (tabnum == tableInqNum(vlistInqVarTable(vlistID1, index))) break;
@@ -245,18 +245,18 @@ Split(void *process)
           vlistClearFlag(vlistID1);
           for (varID = 0; varID < nvars; varID++)
             {
-              const int tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+              const auto tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
               if (tabnums[index] == tabnum)
                 {
-                  int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  for (levID = 0; levID < nlevs; levID++)
+                  for (levID = 0; levID < varList1[varID].nlevels; levID++)
                     {
                       vlistDefIndex(vlistID1, varID, levID, index);
                       vlistDefFlag(vlistID1, varID, levID, true);
                     }
                 }
             }
-          vlistID2 = vlistCreate();
+
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
@@ -268,7 +268,6 @@ Split(void *process)
     }
   else if (operatorID == SPLITNAME)
     {
-      char varname[CDI_MAX_NAME];
       nsplit = nvars;
 
       vlistIDs.resize(nsplit);
@@ -278,20 +277,18 @@ Split(void *process)
         {
           vlistClearFlag(vlistID1);
           varID = index;
-          const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levID = 0; levID < nlevs; levID++)
+          for (levID = 0; levID < varList1[varID].nlevels; levID++)
             {
               vlistDefIndex(vlistID1, varID, levID, index);
               vlistDefFlag(vlistID1, varID, levID, true);
             }
 
-          vlistID2 = vlistCreate();
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
           filename[nchars] = '\0';
-          vlistInqVarName(vlistID1, varID, varname);
-          strcat(filename, varname);
+          strcat(filename, varList1[varID].name);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
           streamIDs[index] = cdoOpenWrite(filename);
@@ -299,16 +296,15 @@ Split(void *process)
     }
   else if (operatorID == SPLITLEVEL)
     {
-      double level;
-      const int nzaxis = vlistNzaxis(vlistID1);
+      const auto nzaxis = vlistNzaxis(vlistID1);
       nsplit = 0;
       for (int index = 0; index < nzaxis; index++)
         {
-          const int zaxisID = vlistZaxis(vlistID1, index);
-          const int nlevs = zaxisInqSize(zaxisID);
+          const auto zaxisID = vlistZaxis(vlistID1, index);
+          const auto nlevs = zaxisInqSize(zaxisID);
           for (levID = 0; levID < nlevs; levID++)
             {
-              level = cdoZaxisInqLevel(zaxisID, levID);
+              const auto level = cdoZaxisInqLevel(zaxisID, levID);
               int i;
               for (i = 0; i < nsplit; i++)
                 if (IS_EQUAL(level, ftmp[i])) break;
@@ -318,7 +314,7 @@ Split(void *process)
 
       vlistIDs.resize(nsplit);
       streamIDs.resize(nsplit);
-      std::vector<double> levels(nsplit);
+      Varray<double> levels(nsplit);
       for (int index = 0; index < nsplit; ++index) levels[index] = ftmp[index];
 
       for (int index = 0; index < nsplit; ++index)
@@ -326,11 +322,10 @@ Split(void *process)
           vlistClearFlag(vlistID1);
           for (varID = 0; varID < nvars; varID++)
             {
-              const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              const int nlevs = zaxisInqSize(zaxisID);
-              for (levID = 0; levID < nlevs; levID++)
+              const auto zaxisID = varList1[varID].zaxisID;
+              for (levID = 0; levID < varList1[varID].nlevels; levID++)
                 {
-                  level = cdoZaxisInqLevel(zaxisID, levID);
+                  const auto level = cdoZaxisInqLevel(zaxisID, levID);
                   if (IS_EQUAL(levels[index], level))
                     {
                       vlistDefIndex(vlistID1, varID, levID, index);
@@ -338,7 +333,8 @@ Split(void *process)
                     }
                 }
             }
-          vlistID2 = vlistCreate();
+
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
@@ -364,18 +360,18 @@ Split(void *process)
           vlistClearFlag(vlistID1);
           for (varID = 0; varID < nvars; varID++)
             {
-              gridID = vlistInqVarGrid(vlistID1, varID);
+              gridID = varList1[varID].gridID;
               if (gridIDs[index] == gridID)
                 {
-                  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  for (levID = 0; levID < nlevs; levID++)
+                  for (levID = 0; levID < varList1[varID].nlevels; levID++)
                     {
                       vlistDefIndex(vlistID1, varID, levID, index);
                       vlistDefFlag(vlistID1, varID, levID, true);
                     }
                 }
             }
-          vlistID2 = vlistCreate();
+
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
@@ -399,18 +395,18 @@ Split(void *process)
           vlistClearFlag(vlistID1);
           for (varID = 0; varID < nvars; varID++)
             {
-              int zaxisID = vlistInqVarZaxis(vlistID1, varID);
+              const auto zaxisID = varList1[varID].zaxisID;
               if (zaxisIDs[index] == zaxisID)
                 {
-                  int nlevs = zaxisInqSize(zaxisID);
-                  for (levID = 0; levID < nlevs; levID++)
+                  for (levID = 0; levID < varList1[varID].nlevels; levID++)
                     {
                       vlistDefIndex(vlistID1, varID, levID, index);
                       vlistDefFlag(vlistID1, varID, levID, true);
                     }
                 }
             }
-          vlistID2 = vlistCreate();
+
+          const auto vlistID2 = vlistCreate();
           cdoVlistCopyFlag(vlistID2, vlistID1);
           vlistIDs[index] = vlistID2;
 
@@ -432,10 +428,10 @@ Split(void *process)
       cdoDefVlist(streamIDs[index], vlistIDs[index]);
     }
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      auto gridsizemax = vlistGridsizeMax(vlistID1);
       if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
       array.resize(gridsizemax);
     }
@@ -450,14 +446,12 @@ Split(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          const int index = vlistInqIndex(vlistID1, varID, levelID);
-          vlistID2 = vlistIDs[index];
+          const auto index = vlistInqIndex(vlistID1, varID, levelID);
+          const auto vlistID2 = vlistIDs[index];
           varID2 = vlistFindVar(vlistID2, varID);
           levelID2 = vlistFindLevel(vlistID2, varID, levelID);
-          /*
-            printf("%d %d %d %d %d %d\n", index, vlistID2, varID, levelID,
-            varID2, levelID2);
-          */
+          // printf("%d %d %d %d %d %d\n", index, vlistID2, varID, levelID, varID2, levelID2);
+
           cdoDefRecord(streamIDs[index], varID2, levelID2);
           if (lcopy)
             {
@@ -475,11 +469,8 @@ Split(void *process)
 
   cdoStreamClose(streamID1);
 
-  for (int index = 0; index < nsplit; index++)
-    {
-      cdoStreamClose(streamIDs[index]);
-      vlistDestroy(vlistIDs[index]);
-    }
+  for (auto &streamID : streamIDs) cdoStreamClose(streamID);
+  for (auto &vlistID : vlistIDs) vlistDestroy(vlistID);
 
   cdoFinish();
 
