@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,41 +23,41 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "cdo_options.h"
+#include "process_int.h"
 #include "util_files.h"
+#include "cdi_lockedIO.h"
 
 void *
 Splitrec(void *process)
 {
-  int varID;
-  int levelID;
-  char filesuffix[32];
-  char filename[8192];
+  int nrecs;
+  int varID, levelID;
   size_t nmiss;
 
   cdoInitialize(process);
 
   if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
 
-  const bool lcopy = unchangedRecord();
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto lcopy = unchangedRecord();
 
-  int nrecs = vlistNrecs(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
+  char filename[8192];
   strcpy(filename, cdoGetObase());
   int nchars = strlen(filename);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      auto gridsizemax = vlistGridsizeMax(vlistID1);
       if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
       array.resize(gridsizemax);
     }
@@ -82,12 +82,12 @@ Splitrec(void *process)
 
           if (Options::cdoVerbose) cdoPrint("create file %s", filename);
 
-          CdoStreamID streamID2 = cdoOpenWrite(filename);
+          const auto streamID2 = cdoOpenWrite(filename);
 
           cdoDefVlist(streamID2, vlistID2);
 
-          const int varID2 = vlistFindVar(vlistID2, varID);
-          const int levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+          const auto varID2 = vlistFindVar(vlistID2, varID);
+          const auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
           cdoDefTimestep(streamID2, 0);
           cdoDefRecord(streamID2, varID2, levelID2);
