@@ -2,11 +2,11 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
+  it under the terms of the GNU General Public Liccense as published by
   the Free Software Foundation; version 2 of the License.
 
   This program is distributed in the hope that it will be useful,
@@ -23,7 +23,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include "util_files.h"
 
@@ -34,60 +35,67 @@ Splitsel(void *process)
   int varID, levelID;
   int tsID;
   size_t nmiss;
-  int i2 = 0;
-  char filesuffix[32];
-  char filename[8192];
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
+
+  const auto lcopy = unchangedRecord();
 
   cdoOperatorAdd("splitsel", 0, 0, nullptr);
 
-  /*  operatorInputArg("nsets <noffset <nskip>>"); */
+  // operatorInputArg("nsets <noffset <nskip>>");
 
-  const int nargc = operatorArgc();
+  const auto nargc = operatorArgc();
   if (nargc < 1) cdoAbort("Too few arguments! Need %d found %d.", 1, nargc);
 
-  /*   ndates = parameter2int(operatorArgv()[0]); */
-  /*   if ( nargc > 1 ) noffset = parameter2int(operatorArgv()[1]); */
-  /*   if ( nargc > 2 ) nskip   = parameter2int(operatorArgv()[2]); */
-  /*   printf("%s %s %s\n", operatorArgv()[0],operatorArgv()[1],operatorArgv()[2]); */
-  const double ndates = parameter2double(operatorArgv()[0]);
-  const double noffset = (nargc > 1) ? parameter2double(operatorArgv()[1]) : 0.0;
-  const double nskip = (nargc > 2) ? parameter2double(operatorArgv()[2]) : 0.0;
+  const auto nsets = parameter2int(cdoOperatorArgv(0));
+  const auto noffset = (nargc > 1) ? parameter2int(cdoOperatorArgv(1)) : 0;
+  const auto nskip = (nargc > 2) ? parameter2int(cdoOperatorArgv(2)) : 0;
 
-  if (Options::cdoVerbose) cdoPrint("nsets = %f, noffset = %f, nskip = %f", ndates, noffset, nskip);
+  if (Options::cdoVerbose) cdoPrint("nsets = %d, noffset = %d, nskip = %d", nsets, noffset, nskip);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  if (nsets < 1) cdoAbort("nsets must be greater than 0!");
+  if (noffset < 0) cdoAbort("noffset must be greater or equal 0!");
+  if (nskip < 0)
+    {
+      if (-nskip >= nsets) cdoAbort("Absolute value of negative nskip must be less than nsets!");
+      if (cdoAssertFilesOnly() == false) cdoAbort("Negative nskip not allowed in combination with other operators!");
+    }
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
   // int taxisID2 = cdoTaxisCreate(TAXIS_ABSOLUTE);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  strcpy(filename, cdoGetObase());
-  const int nchars = strlen(filename);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  char filename[8192];
+  strcpy(filename, cdoGetObase());
+  const auto nchars = strlen(filename);
+
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
-  std::vector<double> array;
+  Varray<double> array;
   //  if ( ! lcopy )
   {
-    size_t gridsizemax = vlistGridsizeMax(vlistID1);
+    auto gridsizemax = vlistGridsizeMax(vlistID1);
     if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
     array.resize(gridsizemax);
   }
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   int nconst = 0;
   for (varID = 0; varID < nvars; varID++)
-    if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
+    if (varList1[varID].timetype == TIME_CONSTANT) nconst++;
 
   FieldVector2D vars;
   if (nconst)
@@ -96,15 +104,15 @@ Splitsel(void *process)
 
       for (varID = 0; varID < nvars; varID++)
         {
-          if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
+          if (varList1[varID].timetype == TIME_CONSTANT)
             {
-              const int gridID = vlistInqVarGrid(vlistID1, varID);
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              const size_t gridsize = gridInqSize(gridID);
+              const auto gridID = varList1[varID].gridID;
+              const auto gridsize = varList1[varID].gridsize;
+              const auto nlevels = varList1[varID].nlevels;
 
-              vars[varID].resize(nlevel);
+              vars[varID].resize(nlevels);
 
-              for (levelID = 0; levelID < nlevel; levelID++)
+              for (levelID = 0; levelID < nlevels; levelID++)
                 {
                   vars[varID][levelID].grid = gridID;
                   vars[varID][levelID].resize(gridsize);
@@ -114,9 +122,8 @@ Splitsel(void *process)
     }
 
   int index = 0;
-  int nsets = 0;
 
-  /* offset */
+  // offset
   for (tsID = 0; tsID < noffset; tsID++)
     {
       nrecs = cdoStreamInqTimestep(streamID1, tsID);
@@ -130,9 +137,9 @@ Splitsel(void *process)
         for (int recID = 0; recID < nrecs; recID++)
           {
             cdoInqRecord(streamID1, &varID, &levelID);
-            if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
+            if (varList1[varID].timetype == TIME_CONSTANT)
               {
-                cdoReadRecord(streamID1, vars[varID][levelID].vec.data(), &nmiss);
+                cdoReadRecord(streamID1, vars[varID][levelID].vec_d.data(), &nmiss);
                 vars[varID][levelID].nmiss = nmiss;
               }
           }
@@ -148,7 +155,7 @@ Splitsel(void *process)
       CdoStreamID streamID2 = CDO_STREAM_UNDEF;
       int tsID2 = 0;
 
-      for (; nsets < (int) (ndates * (index + 1)); nsets++)
+      for (int i = 0; i < nsets; ++i)
         {
           nrecs = cdoStreamInqTimestep(streamID1, tsID);
           if (nrecs == 0) break;
@@ -167,14 +174,14 @@ Splitsel(void *process)
             {
               for (varID = 0; varID < nvars; varID++)
                 {
-                  if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
+                  if (varList1[varID].timetype == TIME_CONSTANT)
                     {
-                      const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                      const auto nlevels = varList1[varID].nlevels;
                       for (levelID = 0; levelID < nlevels; levelID++)
                         {
                           cdoDefRecord(streamID2, varID, levelID);
                           nmiss = vars[varID][levelID].nmiss;
-                          cdoWriteRecord(streamID2, vars[varID][levelID].vec.data(), nmiss);
+                          cdoWriteRecord(streamID2, vars[varID][levelID].vec_d.data(), nmiss);
                         }
                     }
                 }
@@ -182,7 +189,6 @@ Splitsel(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-
               cdoInqRecord(streamID1, &varID, &levelID);
               cdoDefRecord(streamID2, varID, levelID);
               if (lcopy && !(tsID == 0 && nconst))
@@ -196,11 +202,9 @@ Splitsel(void *process)
 
                   if (tsID == 0 && nconst)
                     {
-                      if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
+                      if (varList1[varID].timetype == TIME_CONSTANT)
                         {
-                          int gridID = vlistInqVarGrid(vlistID1, varID);
-                          size_t gridsize = gridInqSize(gridID);
-                          arrayCopy(gridsize, array.data(), vars[varID][levelID].vec.data());
+                          varrayCopy(varList1[varID].gridsize, array, vars[varID][levelID].vec_d);
                           vars[varID][levelID].nmiss = nmiss;
                         }
                     }
@@ -214,18 +218,15 @@ Splitsel(void *process)
       cdoStreamClose(streamID2);
       if (nrecs == 0) break;
 
-      nrecs = cdoStreamInqTimestep(streamID1, tsID);
-      if (nrecs == 0) break;
+      if (cdoStreamInqTimestep(streamID1, tsID) == 0) break;
 
-      for (; i2 < (int) (nskip * (index + 1)); i2++)
-        {
-          nrecs = cdoStreamInqTimestep(streamID1, tsID);
-          if (nrecs == 0) break;
-          tsID++;
-        }
+      // skip
+      for (int i = 0; i < nskip; ++i)
+        if (cdoStreamInqTimestep(streamID1, tsID + i) == 0) break;
 
-      nrecs = cdoStreamInqTimestep(streamID1, tsID);
-      if (nrecs == 0) break;
+      tsID += nskip;
+
+      if (cdoStreamInqTimestep(streamID1, tsID) == 0) break;
 
       index++;
     }
