@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,11 +27,13 @@
 #include <time.h>
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
 #include "cdo_season.h"
 #include "util_files.h"
 
-#define MAX_STREAMS 32
+constexpr int MaxStreams = 32;
 
 struct tm
 datetime_to_tm(int64_t date, int time)
@@ -60,65 +62,67 @@ Splittime(void *process)
   int varID;
   int nrecs;
   int levelID;
-  CdoStreamID streamIDs[MAX_STREAMS];
-  int tsIDs[MAX_STREAMS];
+  CdoStreamID streamIDs[MaxStreams];
+  int tsIDs[MaxStreams];
   int index = 0;
   size_t nmiss;
-  char filesuffix[32];
-  char filename[8192];
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
+
+  const auto lcopy = unchangedRecord();
 
   // clang-format off
-  const int SPLITHOUR = cdoOperatorAdd("splithour", func_time, 10000, nullptr);
-  const int SPLITDAY  = cdoOperatorAdd("splitday",  func_date,     1, nullptr);
-  const int SPLITMON  = cdoOperatorAdd("splitmon",  func_date,   100, nullptr);
-  const int SPLITSEAS = cdoOperatorAdd("splitseas", func_date,   100, nullptr);
+  const auto SPLITHOUR = cdoOperatorAdd("splithour", func_time, 10000, nullptr);
+  const auto SPLITDAY  = cdoOperatorAdd("splitday",  func_date,     1, nullptr);
+  const auto SPLITMON  = cdoOperatorAdd("splitmon",  func_date,   100, nullptr);
+  const auto SPLITSEAS = cdoOperatorAdd("splitseas", func_date,   100, nullptr);
   // clang-format on
 
-  UNUSED(SPLITDAY);
-  UNUSED(SPLITHOUR);
+  (void) (SPLITDAY);   // CDO_UNUSED
+  (void) (SPLITHOUR);  // CDO_UNUSED
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-  const int operintval = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+  const auto operintval = cdoOperatorF2(operatorID);
 
   const char *format = nullptr;
-  if (operatorID == SPLITMON && operatorArgc() == 1) format = operatorArgv()[0];
+  if (operatorID == SPLITMON && operatorArgc() == 1) format = cdoOperatorArgv(0).c_str();
+  else  operatorCheckArgc(0);
 
   const char *seas_name[4];
-  get_season_name(seas_name);
+  getSeasonName(seas_name);
 
-  for (int i = 0; i < MAX_STREAMS; i++) streamIDs[i] = CDO_STREAM_UNDEF;
-  for (int i = 0; i < MAX_STREAMS; i++) tsIDs[i] = 0;
+  for (int i = 0; i < MaxStreams; i++) streamIDs[i] = CDO_STREAM_UNDEF;
+  for (int i = 0; i < MaxStreams; i++) tsIDs[i] = 0;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
+  char filename[8192];
   strcpy(filename, cdoGetObase());
   const int nchars = strlen(filename);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
-  std::vector<double> array;
+  Varray<double> array;
   //  if ( ! lcopy )
   {
-    int gridsizemax = vlistGridsizeMax(vlistID1);
+    auto gridsizemax = vlistGridsizeMax(vlistID1);
     if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
     array.resize(gridsizemax);
   }
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   int nconst = 0;
   for (varID = 0; varID < nvars; varID++)
     if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
@@ -132,9 +136,9 @@ Splittime(void *process)
         {
           if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
             {
-              const int gridID = vlistInqVarGrid(vlistID1, varID);
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              const size_t gridsize = gridInqSize(gridID);
+              const auto gridID = vlistInqVarGrid(vlistID1, varID);
+              const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+              const auto gridsize = gridInqSize(gridID);
 
               vars[varID].resize(nlevel);
 
@@ -151,22 +155,22 @@ Splittime(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       if (operfunc == func_date)
         {
           index = (vdate / operintval) % 100;
           if (index < 0) index = -index;
 
-          if (operatorID == SPLITSEAS) index = month_to_season(index);
+          if (operatorID == SPLITSEAS) index = monthToSeason(index);
         }
       else if (operfunc == func_time)
         {
           index = (vtime / operintval) % 100;
         }
 
-      if (index < 0 || index >= MAX_STREAMS) cdoAbort("Index out of range!");
+      if (index < 0 || index >= MaxStreams) cdoAbort("Index out of range!");
 
       streamID2 = streamIDs[index];
       if (streamID2 == CDO_STREAM_UNDEF)
@@ -185,7 +189,7 @@ Splittime(void *process)
               if (operatorID == SPLITMON && format)
                 {
                   char sbuf[32];
-                  struct tm stime = datetime_to_tm(vdate, vtime);
+                  auto stime = datetime_to_tm(vdate, vtime);
                   slen = strftime(sbuf, 32, format, &stime);
 
                   if (slen) strcpy(oformat, sbuf);
@@ -210,12 +214,12 @@ Splittime(void *process)
             {
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                 {
-                  const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                  const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
                   for (levelID = 0; levelID < nlevels; levelID++)
                     {
                       cdoDefRecord(streamID2, varID, levelID);
                       nmiss = vars[varID][levelID].nmiss;
-                      cdoWriteRecord(streamID2, vars[varID][levelID].vec.data(), nmiss);
+                      cdoWriteRecord(streamID2, vars[varID][levelID].vec_d.data(), nmiss);
                     }
                 }
             }
@@ -239,9 +243,9 @@ Splittime(void *process)
                 {
                   if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                     {
-                      const int gridID = vlistInqVarGrid(vlistID1, varID);
-                      const size_t gridsize = gridInqSize(gridID);
-                      arrayCopy(gridsize, array.data(), vars[varID][levelID].vec.data());
+                      const auto gridID = vlistInqVarGrid(vlistID1, varID);
+                      const auto gridsize = gridInqSize(gridID);
+                      varrayCopy(gridsize, array, vars[varID][levelID].vec_d);
                       vars[varID][levelID].nmiss = nmiss;
                     }
                 }
@@ -254,7 +258,7 @@ Splittime(void *process)
 
   cdoStreamClose(streamID1);
 
-  for (index = 0; index < MAX_STREAMS; index++)
+  for (index = 0; index < MaxStreams; index++)
     {
       if (streamIDs[index] != CDO_STREAM_UNDEF) cdoStreamClose(streamIDs[index]);
     }
