@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,11 +22,11 @@
      Splityear  splityearmon    Split in years and month
 */
 
-#include <limits.h>
-
+#include <climits>
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "util_files.h"
 
 #define MAX_YEARS 99999
@@ -40,10 +40,7 @@ Splityear(void *process)
   int levelID;
   int day;
   int ic = 0;
-  int cyear[MAX_YEARS];
   size_t nmiss;
-  char filesuffix[32];
-  char filename[8192];
 
   cdoInitialize(process);
 
@@ -52,43 +49,49 @@ Splityear(void *process)
   const bool lcopy = unchangedRecord();
 
   // clang-format off
-  const int SPLITYEAR    = cdoOperatorAdd("splityear",     0, 10000, nullptr);
-  const int SPLITYEARMON = cdoOperatorAdd("splityearmon",  0,   100, nullptr);
+  const auto SPLITYEAR    = cdoOperatorAdd("splityear",     0, 10000, nullptr);
+  const auto SPLITYEARMON = cdoOperatorAdd("splityearmon",  0,   100, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
-  const int operintval = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operintval = cdoOperatorF2(operatorID);
 
-  arrayFill(MAX_YEARS, cyear, 0);
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  Varray<int> cyear(MAX_YEARS, 0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
+  char filename[8192];
   strcpy(filename, cdoGetObase());
   const int nchars = strlen(filename);
 
-  const char *refname = cdoGetObase();
-  filesuffix[0] = 0;
+  auto refname = cdoGetStreamName(0);
+  char filesuffix[32] = { 0 };
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
-  std::vector<double> array;
+  Varray<double> array;
   // if ( ! lcopy )
   {
-    int gridsizemax = vlistGridsizeMax(vlistID1);
+    auto gridsizemax = vlistGridsizeMax(vlistID1);
     if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
     array.resize(gridsizemax);
   }
 
-  const int nvars = vlistNvars(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
+  const auto nvars = vlistNvars(vlistID1);
   int nconst = 0;
   for (varID = 0; varID < nvars; varID++)
-    if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
+    if (varList1[varID].timetype == TIME_CONSTANT) nconst++;
 
   FieldVector2D vars;
   if (nconst)
@@ -99,13 +102,13 @@ Splityear(void *process)
         {
           if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
             {
-              const int gridID = vlistInqVarGrid(vlistID1, varID);
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              const size_t gridsize = gridInqSize(gridID);
+              const auto gridID = varList1[varID].gridID;
+              const auto gridsize = varList1[varID].gridsize;
+              const auto nlevels = varList1[varID].nlevels;
 
-              vars[varID].resize(nlevel);
+              vars[varID].resize(nlevels);
 
-              for (levelID = 0; levelID < nlevel; levelID++)
+              for (levelID = 0; levelID < nlevels; levelID++)
                 {
                   vars[varID][levelID].grid = gridID;
                   vars[varID][levelID].resize(gridsize);
@@ -124,7 +127,7 @@ Splityear(void *process)
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      const int64_t vdate = taxisInqVdate(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
       cdiDecodeDate(vdate, &year2, &mon2, &day);
 
       if (operatorID == SPLITYEAR)
@@ -186,12 +189,12 @@ Splityear(void *process)
             {
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                 {
-                  const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                  const auto nlevels = varList1[varID].nlevels;
                   for (levelID = 0; levelID < nlevels; levelID++)
                     {
                       nmiss = vars[varID][levelID].nmiss;
                       cdoDefRecord(streamID2, varID, levelID);
-                      cdoWriteRecord(streamID2, vars[varID][levelID].vec.data(), nmiss);
+                      cdoWriteRecord(streamID2, vars[varID][levelID].vec_d.data(), nmiss);
                     }
                 }
             }
@@ -213,11 +216,9 @@ Splityear(void *process)
 
               if (tsID == 0 && nconst)
                 {
-                  if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
+                  if (varList1[varID].timetype == TIME_CONSTANT)
                     {
-                      const int gridID = vlistInqVarGrid(vlistID1, varID);
-                      const size_t gridsize = gridInqSize(gridID);
-                      arrayCopy(gridsize, array.data(), vars[varID][levelID].vec.data());
+                      varrayCopy(varList1[varID].gridsize, array, vars[varID][levelID].vec_d);
                       vars[varID][levelID].nmiss = nmiss;
                     }
                 }
