@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,42 +17,35 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "array.h"
 
 void *
 Test(void *process)
 {
-  /*
-  CdoStreamID streamID1, streamID2;
-  */
-
   cdoInitialize(process);
 
   /*
-  streamID1 = cdoOpenRead(0);
-  streamID2 = cdoOpenWrite(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
   */
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
 
 void *
 Test2(void *process)
 {
-  /*
-  CdoStreamID streamID1, streamID2, streamID3;
-  */
-
   cdoInitialize(process);
 
   /*
-  streamID1 = cdoOpenRead(0);
-  streamID2 = cdoOpenRead(1);
-  streamID3 = cdoOpenWrite(2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto streamID3 = cdoOpenWrite(2);
 
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
@@ -60,7 +53,7 @@ Test2(void *process)
   */
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
 
 void *
@@ -72,29 +65,31 @@ Testdata(void *process)
 
   cdoInitialize(process);
 
+  operatorCheckArgc(0);
+
   int tsID2 = 0;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  double *array = (double *) Malloc(gridsize * sizeof(double));
-  float *fval = (float *) Malloc(gridsize * sizeof(float));
-  int *ival = (int *) Malloc(gridsize * sizeof(int));
-  unsigned char *cval = (unsigned char *) Malloc(gridsize * sizeof(unsigned char) * 4);
-  unsigned char *cval2 = (unsigned char *) Malloc(gridsize * sizeof(unsigned char) * 4);
+  auto gridsize = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsize);
+  Varray<float> fval(gridsize);
+  Varray<int> ival(gridsize);
+  Varray<unsigned char> cval(gridsize * 4);
+  Varray<unsigned char> cval2(gridsize * 4);
 
-  FILE *fp = fopen("testdata", "w");
+  auto fp = fopen("testdata", "w");
 
   int tsID1 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
@@ -107,7 +102,7 @@ Testdata(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoDefRecord(streamID2, varID, levelID);
 
-          cdoReadRecord(streamID1, array, &nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           for (size_t i = 0; i < gridsize; ++i)
@@ -123,13 +118,13 @@ Testdata(void *process)
               cval2[i + gridsize * 3] = cval[i * 4 + 3];
 
               if (tsID1 == 0 && recID == 0)
-                printf("%4zu %3d %3d %3d %3d %d %g\n", i, (unsigned int) cval[4 * i + 0], (unsigned int) cval[4 * i + 1],
+                printf("%4zu %3u %3u %3u %3u %d %g\n", i, (unsigned int) cval[4 * i + 0], (unsigned int) cval[4 * i + 1],
                        (unsigned int) cval[4 * i + 2], (unsigned int) cval[4 * i + 3], ival[i], fval[i]);
             }
 
-          cdoWriteRecord(streamID2, array, nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
 
-          fwrite(cval, 4, gridsize, fp);
+          fwrite(cval.data(), 4, gridsize, fp);
         }
 
       tsID1++;
@@ -140,9 +135,7 @@ Testdata(void *process)
   cdoStreamClose(streamID1);
   cdoStreamClose(streamID2);
 
-  if (array) Free(array);
-
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
