@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,9 +17,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "statistic.h"
 
 void *
@@ -29,19 +28,18 @@ Tests(void *process)
   int varID, levelID;
   size_t nmiss;
   double degree_of_freedom = 0, p = 0, q = 0, n = 0, d = 0;
-  double missval;
 
   cdoInitialize(process);
 
   // clang-format off
-  int NORMAL    = cdoOperatorAdd("normal",    0, 0, nullptr);
-  int STUDENTT  = cdoOperatorAdd("studentt",  0, 0, "degree of freedom");
-  int CHISQUARE = cdoOperatorAdd("chisquare", 0, 0, "degree of freedom");
-  int BETA      = cdoOperatorAdd("beta",      0, 0, "p and q");
-  int FISHER    = cdoOperatorAdd("fisher",    0, 0, "degree of freedom of nominator and of denominator");
+  const auto  NORMAL    = cdoOperatorAdd("normal",    0, 0, nullptr);
+  const auto  STUDENTT  = cdoOperatorAdd("studentt",  0, 0, "degree of freedom");
+  const auto  CHISQUARE = cdoOperatorAdd("chisquare", 0, 0, "degree of freedom");
+  const auto  BETA      = cdoOperatorAdd("beta",      0, 0, "p and q");
+  const auto  FISHER    = cdoOperatorAdd("fisher",    0, 0, "degree of freedom of nominator and of denominator");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == STUDENTT || operatorID == CHISQUARE)
     {
@@ -49,7 +47,7 @@ Tests(void *process)
 
       operatorCheckArgc(1);
 
-      degree_of_freedom = parameter2double(operatorArgv()[0]);
+      degree_of_freedom = parameter2double(cdoOperatorArgv(0));
 
       if (degree_of_freedom <= 0) cdoAbort("degree of freedom must be positive!");
     }
@@ -59,8 +57,8 @@ Tests(void *process)
 
       operatorCheckArgc(2);
 
-      p = parameter2double(operatorArgv()[0]);
-      q = parameter2double(operatorArgv()[1]);
+      p = parameter2double(cdoOperatorArgv(0));
+      q = parameter2double(cdoOperatorArgv(1));
 
       if (p <= 0 || q <= 0) cdoAbort("p and q must be positive!");
     }
@@ -70,27 +68,29 @@ Tests(void *process)
 
       operatorCheckArgc(2);
 
-      n = parameter2double(operatorArgv()[0]);
-      d = parameter2double(operatorArgv()[1]);
+      n = parameter2double(cdoOperatorArgv(0));
+      d = parameter2double(cdoOperatorArgv(1));
 
       if (n <= 0 || d <= 0) cdoAbort("both degrees must be positive!");
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  double *array1 = (double *) Malloc(gridsize * sizeof(double));
-  double *array2 = (double *) Malloc(gridsize * sizeof(double));
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax), array2(gridsizemax);
+
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -101,27 +101,27 @@ Tests(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, array1, &nmiss);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          missval = vlistInqVarMissval(vlistID1, varID);
+          const auto gridsize = varList1[varID].gridsize;
+          const auto missval = varList1[varID].missval;
 
           if (operatorID == NORMAL)
             {
               for (size_t i = 0; i < gridsize; i++)
-                array2[i] = DBL_IS_EQUAL(array1[i], missval) ? missval : normal(array1[i], processInqPrompt());
+                array2[i] = DBL_IS_EQUAL(array1[i], missval) ? missval : cdo::normal(array1[i], processInqPrompt());
             }
           else if (operatorID == STUDENTT)
             {
               for (size_t i = 0; i < gridsize; i++)
                 array2[i]
-                    = DBL_IS_EQUAL(array1[i], missval) ? missval : student_t(degree_of_freedom, array1[i], processInqPrompt());
+                  = DBL_IS_EQUAL(array1[i], missval) ? missval : cdo::student_t(degree_of_freedom, array1[i], processInqPrompt());
             }
           else if (operatorID == CHISQUARE)
             {
               for (size_t i = 0; i < gridsize; i++)
                 array2[i]
-                    = DBL_IS_EQUAL(array1[i], missval) ? missval : chi_square(degree_of_freedom, array1[i], processInqPrompt());
+                    = DBL_IS_EQUAL(array1[i], missval) ? missval : cdo::chi_square(degree_of_freedom, array1[i], processInqPrompt());
             }
           else if (operatorID == BETA)
             {
@@ -129,13 +129,13 @@ Tests(void *process)
                 {
                   if (array1[i] < 0 || array1[i] > 1) cdoAbort("Value out of range (0-1)!");
 
-                  array2[i] = DBL_IS_EQUAL(array1[i], missval) ? missval : beta_distr(p, q, array1[i], processInqPrompt());
+                  array2[i] = DBL_IS_EQUAL(array1[i], missval) ? missval : cdo::beta_distr(p, q, array1[i], processInqPrompt());
                 }
             }
           else if (operatorID == FISHER)
             {
               for (size_t i = 0; i < gridsize; i++)
-                array2[i] = DBL_IS_EQUAL(array1[i], missval) ? missval : fisher(n, d, array1[i], processInqPrompt());
+                array2[i] = DBL_IS_EQUAL(array1[i], missval) ? missval : cdo::fisher(n, d, array1[i], processInqPrompt());
             }
           else
             {
@@ -143,7 +143,7 @@ Tests(void *process)
             }
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, array2, nmiss);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
@@ -154,9 +154,6 @@ Tests(void *process)
 
   vlistDestroy(vlistID2);
 
-  if (array1) Free(array1);
-  if (array2) Free(array2);
-
   cdoFinish();
 
   return nullptr;
