@@ -27,7 +27,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "util_date.h"
 
 void *
 Timcount(void *process)
@@ -35,8 +36,6 @@ Timcount(void *process)
   char indate1[DATE_LEN + 1], indate2[DATE_LEN + 1];
   int64_t vdate0 = 0;
   int vtime0 = 0;
-  int nrecs;
-  int varID, levelID;
 
   cdoInitialize(process);
 
@@ -48,31 +47,33 @@ Timcount(void *process)
   cdoOperatorAdd("hourcount", 0,  4, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   const int cmplen = DATE_LEN - cdoOperatorF2(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int nvars = vlistNvars(vlistID1);
-  for (varID = 0; varID < nvars; varID++) vlistDefVarUnits(vlistID2, varID, "No.");
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto nvars = vlistNvars(vlistID1);
+  for (int varID = 0; varID < nvars; varID++) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, "No.");
 
   if (cdoOperatorF2(operatorID) == 16) vlistDefNtsteps(vlistID2, 1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
 
   Field field;
@@ -85,11 +86,15 @@ Timcount(void *process)
   int otsID = 0;
   while (true)
     {
+      int nrecs = 0;
       int nsets = 0;
-      while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+      while (true)
         {
-          const int64_t vdate = taxisInqVdate(taxisID1);
-          const int vtime = taxisInqVtime(taxisID1);
+          nrecs = cdoStreamInqTimestep(streamID1, tsID);
+          if (nrecs == 0) break;
+
+          const auto vdate = taxisInqVdate(taxisID1);
+          const auto vtime = taxisInqVtime(taxisID1);
 
           if (nsets == 0) SET_DATE(indate2, vdate, vtime);
           SET_DATE(indate1, vdate, vtime);
@@ -98,6 +103,7 @@ Timcount(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
+              int varID, levelID;
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
@@ -111,11 +117,12 @@ Timcount(void *process)
 
               if (nsets == 0)
                 {
-                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec[i] = vars1[varID][levelID].missval;
+                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec_d[i] = vars1[varID][levelID].missval;
                   vars1[varID][levelID].nmiss = fieldsize;
                 }
 
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+              field.size = vars1[varID][levelID].size;
               field.grid = vars1[varID][levelID].grid;
               field.missval = vars1[varID][levelID].missval;
 
@@ -138,10 +145,10 @@ Timcount(void *process)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
+          cdoWriteRecord(streamID2, vars1[varID][levelID].vec_d.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
