@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,29 +23,30 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 
 void *
 Timcumsum(void *process)
 {
-  int nrecs;
   int varID, levelID;
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
 
   Field field;
@@ -55,8 +56,11 @@ Timcumsum(void *process)
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
@@ -64,33 +68,33 @@ Timcumsum(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          Field &rvars1 = vars1[varID][levelID];
+          auto &rvars1 = vars1[varID][levelID];
 
-          const size_t fieldsize = rvars1.size;
+          const auto fieldsize = rvars1.size;
 
           if (tsID == 0)
             {
-              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+              cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
               if (rvars1.nmiss)
                 for (size_t i = 0; i < fieldsize; ++i)
-                  if (DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval)) rvars1.vec[i] = 0;
+                  if (DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval)) rvars1.vec_d[i] = 0;
             }
           else
             {
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
               field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
 
               if (field.nmiss)
                 for (size_t i = 0; i < fieldsize; ++i)
-                  if (DBL_IS_EQUAL(field.vec[i], rvars1.missval)) field.vec[i] = 0;
+                  if (DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) field.vec_d[i] = 0;
 
               vfarsum(rvars1, field);
             }
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
         }
 
       tsID++;
