@@ -23,10 +23,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "percentiles_hist.h"
-#include "percentiles.h"
 #include "datetime.h"
 
 void *
@@ -34,10 +36,8 @@ Timselpctl(void *process)
 {
   TimeStat timestat_date = TimeStat::MEAN;
   int nrecs = 0;
-  int gridID, varID, levelID;
   int tsID;
   size_t nmiss;
-  int nlevels;
 
   cdoInitialize(process);
 
@@ -45,66 +45,65 @@ Timselpctl(void *process)
 
   operatorInputArg("percentile number, nsets <,noffset <,nskip>>");
 
-  const int nargc = operatorArgc();
+  const auto nargc = operatorArgc();
   if (nargc < 2) cdoAbort("Too few arguments! Need %d found %d.", 2, nargc);
 
-  const double pn = parameter2double(operatorArgv()[0]);
-  percentile_check_number(pn);
-  const int ndates = parameter2int(operatorArgv()[1]);
+  const auto pn = parameter2double(cdoOperatorArgv(0));
+  const auto ndates = parameter2int(cdoOperatorArgv(1));
   int noffset = 0, nskip = 0;
-  if (nargc > 2) noffset = parameter2int(operatorArgv()[2]);
-  if (nargc > 3) nskip = parameter2int(operatorArgv()[3]);
+  if (nargc > 2) noffset = parameter2int(cdoOperatorArgv(2));
+  if (nargc > 3) nskip = parameter2int(cdoOperatorArgv(3));
 
   if (Options::cdoVerbose) cdoPrint("nsets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
-  CdoStreamID streamID3 = cdoOpenRead(2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto streamID3 = cdoOpenRead(2);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = cdoStreamInqVlist(streamID3);
-  const int vlistID4 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = cdoStreamInqVlist(streamID3);
+  const auto vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = vlistInqTaxis(vlistID3);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  const int taxisID4 = taxisDuplicate(taxisID1);
+  const auto taxisID4 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  CdoStreamID streamID4 = cdoOpenWrite(3);
+  const auto streamID4 = cdoOpenWrite(3);
   cdoDefVlist(streamID4, vlistID4);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   FieldVector2D vars1;
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
-  HISTOGRAM_SET *hset = hsetCreate(nvars);
+  HistogramSet hset(nvars);
 
-  for (varID = 0; varID < nvars; varID++)
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
+  for (int varID = 0; varID < nvars; varID++)
     {
-      gridID = vlistInqVarGrid(vlistID1, varID);
-      nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-
-      hsetCreateVarLevels(hset, varID, nlevels, gridID);
+      hset.createVarLevels(varID, varList1[varID].nlevels, varList1[varID].gridsize);
     }
 
   for (tsID = 0; tsID < noffset; tsID++)
@@ -114,13 +113,14 @@ Timselpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
               recList[recID].varID = varID;
               recList[recID].levelID = levelID;
-              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
             }
         }
     }
@@ -138,28 +138,31 @@ Timselpctl(void *process)
       if (nrecs != cdoStreamInqTimestep(streamID3, otsID))
         cdoAbort("Number of records at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      const int64_t vdate2 = taxisInqVdate(taxisID2);
-      const int vtime2 = taxisInqVtime(taxisID2);
-      const int64_t vdate3 = taxisInqVdate(taxisID3);
-      const int vtime3 = taxisInqVtime(taxisID3);
+      const auto vdate2 = taxisInqVdate(taxisID2);
+      const auto vtime2 = taxisInqVtime(taxisID2);
+      const auto vdate3 = taxisInqVdate(taxisID3);
+      const auto vtime3 = taxisInqVtime(taxisID3);
       if (vdate2 != vdate3 || vtime2 != vtime3)
         cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars1[varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars1[varID][levelID].vec_d.data(), &nmiss);
           vars1[varID][levelID].nmiss = nmiss;
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID3, &varID, &levelID);
-          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
-          field.grid = vars1[varID][levelID].grid;
-          field.missval = vars1[varID][levelID].missval;
+          cdoReadRecord(streamID3, field.vec_d.data(), &field.nmiss);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
-          hsetDefVarLevelBounds(hset, varID, levelID, &vars1[varID][levelID], &field);
+          hset.defVarLevelBounds(varID, levelID, vars1[varID][levelID], field);
         }
 
       int nsets = 0;
@@ -173,19 +176,20 @@ Timselpctl(void *process)
 
             for (int recID = 0; recID < nrecs; recID++)
               {
+                int varID, levelID;
                 cdoInqRecord(streamID1, &varID, &levelID);
 
                 if (tsID == 0)
                   {
                     recList[recID].varID = varID;
                     recList[recID].levelID = levelID;
-                    recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                    recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
                   }
 
-                cdoReadRecord(streamID1, vars1[varID][levelID].vec.data(), &nmiss);
+                cdoReadRecord(streamID1, vars1[varID][levelID].vec_d.data(), &nmiss);
                 vars1[varID][levelID].nmiss = nmiss;
 
-                hsetAddVarLevelValues(hset, varID, levelID, &vars1[varID][levelID]);
+                hset.addSubVarLevelValues(varID, levelID, vars1[varID][levelID], 1, FIELD_VEC);
               }
 
             tsID++;
@@ -193,13 +197,12 @@ Timselpctl(void *process)
 
       if (nrecs == 0 && nsets == 0) break;
 
-      for (varID = 0; varID < nvars; varID++)
+      for (int varID = 0; varID < nvars; varID++)
         {
-          if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-          nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+          if (varList1[varID].timetype == TIME_CONSTANT) continue;
 
-          for (levelID = 0; levelID < nlevels; levelID++)
-            hsetGetVarLevelPercentiles(&vars1[varID][levelID], hset, varID, levelID, pn);
+          const auto nlevels = varList1[varID].nlevels;
+          for (int levelID = 0; levelID < nlevels; levelID++) hset.getVarLevelPercentiles(vars1[varID][levelID], varID, levelID, pn, FIELD_VEC);
         }
 
       dtlist.statTaxisDefTimestep(taxisID4, nsets);
@@ -209,10 +212,10 @@ Timselpctl(void *process)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
           cdoDefRecord(streamID4, varID, levelID);
-          cdoWriteRecord(streamID4, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
+          cdoWriteRecord(streamID4, vars1[varID][levelID].vec_d.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
@@ -230,8 +233,6 @@ Timselpctl(void *process)
 
 LABEL_END:
 
-  hsetDestroy(hset);
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
