@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,22 +32,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "datetime.h"
 
-void *
-Timselstat(void *process)
+static void
+addOperators(void)
 {
-  TimeStat timestat_date = TimeStat::MEAN;
-  int nrecs = 0;
-  int varID, levelID;
-  int tsID;
-  int nsets;
-
-  cdoInitialize(process);
-
   // clang-format off
   cdoOperatorAdd("timselrange", func_range, 0, nullptr);
   cdoOperatorAdd("timselmin",   func_min,   0, nullptr);
@@ -59,54 +52,67 @@ Timselstat(void *process)
   cdoOperatorAdd("timselvar1",  func_var1,  0, nullptr);
   cdoOperatorAdd("timselstd",   func_std,   0, nullptr);
   cdoOperatorAdd("timselstd1",  func_std1,  0, nullptr);
+  // clang-format on
+}
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+void *
+Timselstat(void *process)
+{
+  TimeStat timestat_date = TimeStat::MEAN;
+  int nrecs = 0;
+  int tsID;
+  int nsets;
 
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
+  cdoInitialize(process);
+
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
   const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
 
   operatorInputArg("nsets <noffset <nskip>>");
 
-  const int nargc  = operatorArgc();
-  const int ndates = parameter2int(operatorArgv()[0]);
-  const int noffset = (nargc > 1) ? parameter2int(operatorArgv()[1]) : 0;
-  const int nskip   = (nargc > 2) ? parameter2int(operatorArgv()[2]) : 0;
-  // clang-format on
+  const auto nargc = operatorArgc();
+  const auto ndates = parameter2int(cdoOperatorArgv(0));
+  const auto noffset = (nargc > 1) ? parameter2int(cdoOperatorArgv(1)) : 0;
+  const auto nskip = (nargc > 2) ? parameter2int(cdoOperatorArgv(2)) : 0;
   const bool lvars2 = lvarstd || lrange;
 
   if (Options::cdoVerbose) cdoPrint("nsets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   FieldVector2D samp1, vars1, vars2;
-  fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
+  fieldsFromVlist(vlistID1, samp1);
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
   if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
@@ -117,6 +123,7 @@ Timselstat(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
@@ -146,6 +153,7 @@ Timselstat(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
+              int varID, levelID;
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
@@ -155,31 +163,31 @@ Timselstat(void *process)
                   recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
-              Field &rsamp1 = samp1[varID][levelID];
-              Field &rvars1 = vars1[varID][levelID];
+              auto &rsamp1 = samp1[varID][levelID];
+              auto &rvars1 = vars1[varID][levelID];
 
-              const size_t fieldsize = rvars1.size;
+              const auto fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+                  cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
                   if (lrange)
                     {
                       vars2[varID][levelID].nmiss = rvars1.nmiss;
-                      vars2[varID][levelID].vec = rvars1.vec;
+                      vars2[varID][levelID].vec_d = rvars1.vec_d;
                     }
 
                   if (rvars1.nmiss || !rsamp1.empty())
                     {
                       if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                      for (size_t i = 0; i < fieldsize; i++)
-                        rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                      for (size_t i = 0; i < fieldsize; i++) rsamp1.vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                     }
                 }
               else
                 {
-                  cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                  cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+                  field.size = fieldsize;
                   field.grid = rvars1.grid;
                   field.missval = rvars1.missval;
 
@@ -188,7 +196,7 @@ Timselstat(void *process)
                       if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
                       for (size_t i = 0; i < fieldsize; i++)
-                        if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
+                        if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i]++;
                     }
 
                   if (lvarstd)
@@ -213,9 +221,8 @@ Timselstat(void *process)
               {
                 if (recList[recID].lconst) continue;
 
-                const int varID = recList[recID].varID;
-                const int levelID = recList[recID].levelID;
-
+                const auto varID = recList[recID].varID;
+                const auto levelID = recList[recID].levelID;
                 vfarmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
@@ -228,10 +235,10 @@ Timselstat(void *process)
         {
           if (recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rsamp1 = samp1[varID][levelID];
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rsamp1 = samp1[varID][levelID];
+          auto &rvars1 = vars1[varID][levelID];
 
           if (lmean)
             {
@@ -242,7 +249,7 @@ Timselstat(void *process)
             }
           else if (lvarstd)
             {
-              Field &rvars2 = vars2[varID][levelID];
+              auto &rvars2 = vars2[varID][levelID];
               if (!rsamp1.empty())
                 {
                   if (lstd)
@@ -252,7 +259,6 @@ Timselstat(void *process)
                 }
               else
                 {
-                  Field &rvars2 = vars2[varID][levelID];
                   if (lstd)
                     vfarcstd(rvars1, rvars2, nsets, divisor);
                   else
@@ -261,7 +267,7 @@ Timselstat(void *process)
             }
           else if (lrange)
             {
-              Field &rvars2 = vars2[varID][levelID];
+              auto &rvars2 = vars2[varID][levelID];
               vfarsub(rvars1, rvars2);
             }
         }
@@ -273,12 +279,12 @@ Timselstat(void *process)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rvars1 = vars1[varID][levelID];
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
         }
 
       if (nrecs == 0) break;
