@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -72,9 +72,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "functs.h"
+#include "process_int.h"
 #include "datetime.h"
 #include "printinfo.h"
+#include "util_date.h"
+#include "param_conversion.h"
+#include "progress.h"
 
 enum
 {
@@ -85,7 +91,7 @@ enum
 };
 
 static void
-vlistSetFrequency(int vlistID, int comparelen)
+vlistSetFrequency(const int vlistID, const int comparelen)
 {
   const char *freq = nullptr;
   // clang-format off
@@ -160,11 +166,10 @@ addOperators(void)
 void *
 Timstat(void *argument)
 {
-  TimeStat timestat_date = TimeStat::MEAN;
+  const TimeStat timestat_date = TimeStat::MEAN;
   int64_t vdate0 = 0;
   int vtime0 = 0;
   int nrecs;
-  int varID, levelID;
   CdoStreamID streamID3;
   int vlistID3, taxisID3 = -1;
   bool lvfrac = false;
@@ -175,9 +180,9 @@ Timstat(void *argument)
 
   addOperators();
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-  const int comparelen = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+  const auto comparelen = cdoOperatorF2(operatorID);
 
   const bool lrange = operfunc == func_range;
   const bool lminidx = operfunc == func_minidx;
@@ -189,46 +194,49 @@ Timstat(void *argument)
 
   if (operfunc == func_mean)
     {
-      const int oargc = operatorArgc();
-      char **oargv = operatorArgv();
-
+      const auto oargc = operatorArgc();
       if (oargc == 1)
         {
           lvfrac = true;
-          vfrac = atof(oargv[0]);
+          vfrac = parameter2double(cdoOperatorArgv(0));
           if (Options::cdoVerbose) cdoPrint("Set vfrac to %g", vfrac);
           if (vfrac < 0 || vfrac > 1) cdoAbort("vfrac out of range!");
         }
       else if (oargc > 1)
         cdoAbort("Too many arguments!");
     }
+  else
+    {
+      operatorCheckArgc(0);
+    }
 
   const int cmplen = DATE_LEN - comparelen;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  vlistDefineTimestepType(vlistID2, operfunc);
 
   if (cmplen == 0) vlistDefNtsteps(vlistID2, 1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   if (lminidx || lmaxidx)
     {
-      for (varID = 0; varID < nvars; ++varID)
-        vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_INT32);
+      for (int varID = 0; varID < nvars; ++varID) vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_INT32);
     }
 
   vlistSetFrequency(vlistID2, comparelen);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   if (Options::cdoDiag)
@@ -241,11 +249,11 @@ Timstat(void *argument)
 
       vlistID3 = vlistDuplicate(vlistID1);
 
-      for (varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < nvars; ++varID)
         {
           vlistDefVarDatatype(vlistID3, varID, CDI_DATATYPE_INT32);
           vlistDefVarMissval(vlistID3, varID, -1);
-          vlistDefVarUnits(vlistID3, varID, "");
+          cdiDefKeyString(vlistID3, varID, CDI_KEY_UNITS, "");
           vlistDefVarAddoffset(vlistID3, varID, 0);
           vlistDefVarScalefactor(vlistID3, varID, 1);
         }
@@ -257,22 +265,22 @@ Timstat(void *argument)
       cdoDefVlist(streamID3, vlistID3);
     }
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
 
-  int FIELD_MEMTYPE = 0;
-  if (operfunc == func_mean && Options::CDO_Memtype == MEMTYPE_FLOAT) FIELD_MEMTYPE = MEMTYPE_FLOAT;
+  MemType FIELD_MEMTYPE(MemType::Native);
+  if (operfunc == func_mean && Options::CDO_Memtype == MemType::Float) FIELD_MEMTYPE = MemType::Float;
 
   Field field;
-  field.memtype = FIELD_MEMTYPE;
-  if (FIELD_MEMTYPE == MEMTYPE_FLOAT)
+  field.memType = FIELD_MEMTYPE;
+  if (FIELD_MEMTYPE == MemType::Float)
     field.resizef(gridsizemax);
   else
     field.resize(gridsizemax);
@@ -280,20 +288,29 @@ Timstat(void *argument)
   const bool lvars2 = lvarstd || lrange || lminidx || lmaxidx;
 
   FieldVector2D samp1, vars1, vars2;
-  fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
+  fieldsFromVlist(vlistID1, samp1);
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
   if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
+  const auto ntsteps1 = vlistNtsteps(vlistID1);
+
+  if (!Options::cdoVerbose && ntsteps1 > 1) progress::init();
+
   int tsID = 0;
   int otsID = 0;
   while (true)
     {
       int nsets = 0;
-      while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+      while (true)
         {
+          nrecs = cdoStreamInqTimestep(streamID1, tsID);
+          if (nrecs == 0) break;
+
+          if (!Options::cdoVerbose && ntsteps1 > 1) progress::update(0, 1, (tsID + 1.) / ntsteps1);
+
           dtlist.taxisInqTimestep(taxisID1, nsets);
-          const int64_t vdate = dtlist.getVdate(nsets);
-          const int vtime = dtlist.getVtime(nsets);
+          const auto vdate = dtlist.getVdate(nsets);
+          const auto vtime = dtlist.getVtime(nsets);
 
           if (nsets == 0) SET_DATE(indate2, vdate, vtime);
           SET_DATE(indate1, vdate, vtime);
@@ -302,6 +319,7 @@ Timstat(void *argument)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
+              int varID, levelID;
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
@@ -311,39 +329,38 @@ Timstat(void *argument)
                   recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
-              Field &rsamp1 = samp1[varID][levelID];
-              Field &rvars1 = vars1[varID][levelID];
+              auto &rsamp1 = samp1[varID][levelID];
+              auto &rvars1 = vars1[varID][levelID];
 
-              const size_t fieldsize = rvars1.size;
+              const auto fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+                  cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
                   if (lrange)
                     {
                       vars2[varID][levelID].nmiss = rvars1.nmiss;
-                      vars2[varID][levelID].vec = rvars1.vec;
+                      vars2[varID][levelID].vec_d = rvars1.vec_d;
                     }
                   else if (lminidx || lmaxidx)
                     {
                       vars2[varID][levelID].nmiss = rvars1.nmiss;
-                      vars2[varID][levelID].vec = rvars1.vec;
+                      vars2[varID][levelID].vec_d = rvars1.vec_d;
                       fieldFill(rvars1, 0);
                     }
 
                   if (rvars1.nmiss || !rsamp1.empty())
                     {
                       if (rsamp1.empty()) rsamp1.resize(fieldsize);
-                      for (size_t i = 0; i < fieldsize; i++)
-                        rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                      for (size_t i = 0; i < fieldsize; i++) rsamp1.vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                     }
                 }
               else
                 {
-                  if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                    cdoReadRecordF(streamID1, field.vecf.data(), &field.nmiss);
+                  if (FIELD_MEMTYPE == MemType::Float)
+                    cdoReadRecordF(streamID1, field.vec_f.data(), &field.nmiss);
                   else
-                    cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                    cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
                   field.size = fieldsize;
                   field.grid = rvars1.grid;
                   field.missval = rvars1.missval;
@@ -352,15 +369,15 @@ Timstat(void *argument)
                     {
                       if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
-                      if (Options::CDO_Memtype == MEMTYPE_FLOAT)
+                      if (FIELD_MEMTYPE == MemType::Float)
                         {
                           for (size_t i = 0; i < fieldsize; i++)
-                            if (!DBL_IS_EQUAL(field.vecf[i], rvars1.missval)) rsamp1.vec[i]++;
+                            if (!DBL_IS_EQUAL(field.vec_f[i], rvars1.missval)) rsamp1.vec_d[i]++;
                         }
                       else
                         {
                           for (size_t i = 0; i < fieldsize; i++)
-                            if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
+                            if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i]++;
                         }
                     }
 
@@ -394,8 +411,8 @@ Timstat(void *argument)
               {
                 if (recList[recID].lconst) continue;
 
-                const int varID = recList[recID].varID;
-                const int levelID = recList[recID].levelID;
+                const auto varID = recList[recID].varID;
+                const auto levelID = recList[recID].levelID;
                 vfarmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
@@ -411,10 +428,10 @@ Timstat(void *argument)
         {
           if (recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rsamp1 = samp1[varID][levelID];
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          const auto &rsamp1 = samp1[varID][levelID];
+          auto &rvars1 = vars1[varID][levelID];
 
           if (lmean)
             {
@@ -425,7 +442,7 @@ Timstat(void *argument)
             }
           else if (lvarstd)
             {
-              Field &rvars2 = vars2[varID][levelID];
+              const auto &rvars2 = vars2[varID][levelID];
               if (!rsamp1.empty())
                 {
                   if (lstd)
@@ -443,39 +460,34 @@ Timstat(void *argument)
             }
           else if (lrange)
             {
-              Field &rvars2 = vars2[varID][levelID];
+              const auto &rvars2 = vars2[varID][levelID];
               vfarsub(rvars1, rvars2);
             }
         }
 
       if (Options::cdoVerbose)
-        {
-          char vdatestr[32], vtimestr[32];
-          date2str(vdate0, vdatestr, sizeof(vdatestr));
-          time2str(vtime0, vtimestr, sizeof(vtimestr));
-          cdoPrint("%s %s  vfrac = %g, nsets = %d", vdatestr, vtimestr, vfrac, nsets);
-        }
+        cdoPrint("%s %s  vfrac = %g, nsets = %d", dateToString(vdate0).c_str(), timeToString(vtime0).c_str(), vfrac, nsets);
 
       if (lvfrac && operfunc == func_mean)
         for (int recID = 0; recID < maxrecs; recID++)
           {
             if (recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
-            Field &rsamp1 = samp1[varID][levelID];
-            Field &rvars1 = vars1[varID][levelID];
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
+            auto &rsamp1 = samp1[varID][levelID];
+            auto &rvars1 = vars1[varID][levelID];
 
-            const size_t fieldsize = rvars1.size;
-            const double missval = rvars1.missval;
+            const auto fieldsize = rvars1.size;
+            const auto missval = rvars1.missval;
             if (!rsamp1.empty())
               {
                 int irun = 0;
                 for (size_t i = 0; i < fieldsize; ++i)
                   {
-                    if ((rsamp1.vec[i] / nsets) < vfrac)
+                    if ((rsamp1.vec_d[i] / nsets) < vfrac)
                       {
-                        rvars1.vec[i] = missval;
+                        rvars1.vec_d[i] = missval;
                         irun++;
                       }
                   }
@@ -497,23 +509,22 @@ Timstat(void *argument)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rsamp1 = samp1[varID][levelID];
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rsamp1 = samp1[varID][levelID];
+          auto &rvars1 = vars1[varID][levelID];
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
 
           if (Options::cdoDiag)
             {
-              double *sampptr = field.vec.data();
+              auto sampptr = field.vec_d.data();
               if (!rsamp1.empty())
-                sampptr = rsamp1.vec.data();
+                sampptr = rsamp1.vec_d.data();
               else
                 {
-                  const size_t fieldsize = rvars1.size;
-                  for (size_t i = 0; i < fieldsize; ++i) sampptr[i] = nsets;
+                  for (size_t i = 0; i < rvars1.size; ++i) sampptr[i] = nsets;
                 }
 
               cdoDefRecord(streamID3, varID, levelID);
@@ -525,6 +536,8 @@ Timstat(void *argument)
       otsID++;
     }
 
+  if (!Options::cdoVerbose && ntsteps1 > 1) progress::update(0, 1, 1);
+
   if (Options::cdoDiag) cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
