@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "calendar.h"
-
 #include "datetime.h"
 #include "printinfo.h"
 #include "util_string.h"
@@ -34,87 +34,9 @@
 #define MAX_NTSM 128
 #define LIM_NTSM 1024
 
-enum
-{
-  TU_SECONDS = 0,
-  TU_MINUTES,
-  TU_HOURS,
-  TU_DAYS,
-  TU_MONTHS,
-  TU_YEARS
-};
 const char *tunits[] = { "second", "minute", "hour", "day", "month", "year" };
 int iunits[] = { 1, 60, 3600, 86400, 1, 12 };
 
-void
-getTimeInc(double jdelta, int64_t vdate0, int64_t vdate1, int64_t *incperiod, int *incunit)
-{
-  *incperiod = 0;
-  *incunit = 0;
-
-  int64_t lperiod = (jdelta < 0) ? (int64_t)(jdelta - 0.5) : (int64_t)(jdelta + 0.5);
-
-  int sign = 1;
-  if (lperiod < 0)
-    {
-      int64_t tmp = vdate1;
-      vdate1 = vdate0;
-      vdate0 = tmp;
-      lperiod = -lperiod;
-      sign = -1;
-    }
-
-  // printf("\n%d %d %d\n",lperiod, vdate0, vdate1);
-
-  int year0, month0, day0;
-  cdiDecodeDate(vdate0, &year0, &month0, &day0);
-  int year1, month1, day1;
-  cdiDecodeDate(vdate1, &year1, &month1, &day1);
-
-  int deltay = year1 - year0;
-  int deltam = deltay * 12 + (month1 - month0);
-  if (deltay == 0) deltay = 1;
-  if (deltam == 0) deltam = 1;
-
-  if (lperiod / 60 >= 1 && lperiod / 60 < 60)
-    {
-      *incperiod = lperiod / 60;
-      *incunit = TU_MINUTES;
-    }
-  else if (lperiod / 3600 >= 1 && lperiod / 3600 < 24)
-    {
-      *incperiod = lperiod / 3600;
-      *incunit = TU_HOURS;
-    }
-  else if (lperiod / (3600 * 24) >= 1 && lperiod / (3600 * 24) < 32)
-    {
-      *incperiod = lperiod / (3600 * 24);
-      *incunit = TU_DAYS;
-      if (*incperiod >= 27 && deltam == 1)
-        {
-          *incperiod = 1;
-          *incunit = TU_MONTHS;
-        }
-    }
-  else if (lperiod / (3600 * 24 * 30) >= 1 && lperiod / (3600 * 24 * 30) < 12)
-    {
-      *incperiod = deltam;
-      *incunit = TU_MONTHS;
-    }
-  else if (lperiod / (3600 * 24 * 30 * 12) >= 1)
-    {
-      *incperiod = deltay;
-      *incunit = TU_YEARS;
-    }
-  else
-    {
-      *incperiod = lperiod;
-      *incunit = TU_SECONDS;
-    }
-
-  *incperiod *= sign;
-}
-
 static void
 printBounds(int taxisID, int calendar)
 {
@@ -123,45 +45,38 @@ printBounds(int taxisID, int calendar)
   int vtime0, vtime1;
   taxisInqVtimeBounds(taxisID, &vtime0, &vtime1);
 
-  char vdatestr[32], vtimestr[32];
-  date2str(vdate0, vdatestr, sizeof(vdatestr));
-  time2str(vtime0, vtimestr, sizeof(vtimestr));
-  fprintf(stdout, " %s %s", vdatestr, vtimestr);
+  fprintf(stdout, " %s %s", dateToString(vdate0).c_str(), timeToString(vtime0).c_str());
+  fprintf(stdout, " %s %s", dateToString(vdate1).c_str(), timeToString(vtime1).c_str());
 
-  date2str(vdate1, vdatestr, sizeof(vdatestr));
-  time2str(vtime1, vtimestr, sizeof(vtimestr));
-  fprintf(stdout, " %s %s", vdatestr, vtimestr);
+  const auto juldate0 = julianDateEncode(calendar, vdate0, vtime0);
+  const auto juldate1 = julianDateEncode(calendar, vdate1, vtime1);
+  const auto jdelta = julianDateToSeconds(julianDateSub(juldate1, juldate0));
 
-  JulianDate juldate0 = julianDateEncode(calendar, vdate0, vtime0);
-  JulianDate juldate1 = julianDateEncode(calendar, vdate1, vtime1);
-  double jdelta = julianDateToSeconds(julianDateSub(juldate1, juldate0));
+  const auto timeIncr = getTimeIncrement(jdelta, vdate0, vdate1);
 
-  int64_t incperiod = 0;
-  int incunit = 0;
-  getTimeInc(jdelta, vdate0, vdate1, &incperiod, &incunit);
-
-  /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), incperiod%3600);*/
-  int len = fprintf(stdout, " %3ld %s%s", (long)incperiod, tunits[incunit], labs(incperiod) != 1 ? "s" : "");
+  /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), timeIncr.period%3600);*/
+  int len = fprintf(stdout, " %3ld %s%s", (long) timeIncr.period, tunits[(int) timeIncr.unit],
+                    std::abs(timeIncr.period) != 1 ? "s" : "");
   for (int i = 0; i < 11 - len; ++i) fprintf(stdout, " ");
 }
 
 static int
 fill_gap(int ngaps, int ntsm[MAX_NTSM], int rangetsm[MAX_GAPS][2], int64_t vdatem[MAX_GAPS][MAX_NTSM],
-         int vtimem[MAX_GAPS][MAX_NTSM], int tsID, int64_t incperiod0, int incunit0, int64_t vdate, int vdate0, int vtime0,
+         int vtimem[MAX_GAPS][MAX_NTSM], int tsID, int64_t incperiod0, TimeUnit incunit0, int64_t vdate, int vdate0, int vtime0,
          int calendar, int day0, JulianDate juldate, JulianDate juldate0)
 {
   int its = 0;
   int year, month, day;
   int64_t ndate;
   int ntime;
-  int64_t ijulinc = incperiod0 * iunits[incunit0];
+  int64_t ijulinc = incperiod0 * iunits[(int) incunit0];
 
   if (ijulinc > 0 && ngaps < MAX_GAPS)
     {
       rangetsm[ngaps][0] = tsID;
       rangetsm[ngaps][1] = tsID + 1;
 
-      if (incunit0 == TU_MONTHS || incunit0 == TU_YEARS)
+      if (incunit0 == TimeUnit::MONTHS || incunit0 == TimeUnit::YEARS)
         {
           its = 0;
           ndate = vdate0;
@@ -171,17 +86,7 @@ fill_gap(int ngaps, int ntsm[MAX_NTSM], int rangetsm[MAX_GAPS][2], int64_t vdate
               cdiDecodeDate(ndate, &year, &month, &day);
 
               month += (int) ijulinc;
-
-              while (month > 12)
-                {
-                  month -= 12;
-                  year++;
-                }
-              while (month < 1)
-                {
-                  month += 12;
-                  year--;
-                }
+              adjustMonthAndYear(month, year);
 
               if (day0 == 31) day = days_per_month(calendar, year, month);
 
@@ -206,7 +111,7 @@ fill_gap(int ngaps, int ntsm[MAX_NTSM], int rangetsm[MAX_GAPS][2], int64_t vdate
           juldate0 = julianDateAddSeconds(ijulinc, juldate0);
           while (julianDateToSeconds(juldate0) < julianDateToSeconds(juldate))
             {
-              julianDateDecode(calendar, juldate0, &ndate, &ntime);
+              julianDateDecode(calendar, juldate0, ndate, ntime);
               juldate0 = julianDateAddSeconds(ijulinc, juldate0);
               if (its < MAX_NTSM)
                 {
@@ -235,7 +140,7 @@ Tinfo(void *process)
   int year, month, day;
   bool lforecast = false;
   int64_t incperiod0 = 0, incperiod = 0;
-  int incunit0 = 0, incunit = 0;
+  TimeUnit incunit0 = TimeUnit::SECONDS, incunit = TimeUnit::SECONDS;
   int its = 0, igap;
   int ngaps = 0;
   int ntsm[MAX_GAPS];
@@ -246,18 +151,18 @@ Tinfo(void *process)
   double jdelta = 0, jdelta0 = 0;
   int arrow = 0;
   int i, len;
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
-  CdoStreamID streamID = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  int vlistID = cdoStreamInqVlist(streamID);
+  const auto streamID = cdoOpenRead(0);
+  const auto vlistID = cdoStreamInqVlist(streamID);
 
   fprintf(stdout, "\n");
 
-  int taxisID = vlistInqTaxis(vlistID);
-  int ntsteps = vlistNtsteps(vlistID);
+  const auto taxisID = vlistInqTaxis(vlistID);
+  const auto ntsteps = vlistNtsteps(vlistID);
 
   if (ntsteps != 0)
     {
@@ -272,16 +177,12 @@ Tinfo(void *process)
             {
               vdate = taxisInqRdate(taxisID);
               vtime = taxisInqRtime(taxisID);
+              fprintf(stdout, "     RefTime = %s %s", dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
-              date2str(vdate, vdatestr, sizeof(vdatestr));
-              time2str(vtime, vtimestr, sizeof(vtimestr));
-
-              fprintf(stdout, "     RefTime = %s %s", vdatestr, vtimestr);
-
-              int unit = taxisInqTunit(taxisID);
+              auto unit = taxisInqTunit(taxisID);
               if (unit != CDI_UNDEFID) fprintf(stdout, "  Units = %s", tunitToCstr(unit));
 
-              int calendar = taxisInqCalendar(taxisID);
+              const auto calendar = taxisInqCalendar(taxisID);
               if (calendar != CDI_UNDEFID) fprintf(stdout, "  Calendar = %s", calendarToCstr(calendar));
 
               if (taxisHasBounds(taxisID)) fprintf(stdout, "  Bounds = true");
@@ -290,13 +191,9 @@ Tinfo(void *process)
 
               if (taxisInqType(taxisID) == TAXIS_FORECAST)
                 {
-                  int64_t fdate = taxisInqFdate(taxisID);
-                  int ftime = taxisInqFtime(taxisID);
-
-                  date2str(fdate, vdatestr, sizeof(vdatestr));
-                  time2str(ftime, vtimestr, sizeof(vtimestr));
-
-                  fprintf(stdout, "     Forecast RefTime = %s %s", vdatestr, vtimestr);
+                  const auto fdate = taxisInqFdate(taxisID);
+                  const auto ftime = taxisInqFtime(taxisID);
+                  fprintf(stdout, "     Forecast RefTime = %s %s", dateToString(fdate).c_str(), timeToString(ftime).c_str());
 
                   unit = taxisInqForecastTunit(taxisID);
                   if (unit != CDI_UNDEFID) fprintf(stdout, "  Units = %s", tunitToCstr(unit));
@@ -308,7 +205,7 @@ Tinfo(void *process)
             }
         }
 
-      int calendar = taxisInqCalendar(taxisID);
+      const auto calendar = taxisInqCalendar(taxisID);
 
       fprintf(stdout, "\n");
       fprintf(stdout, "         Verification Time              ");
@@ -329,10 +226,7 @@ Tinfo(void *process)
 
           cdiDecodeDate(vdate, &year, &month, &day);
 
-          date2str(vdate, vdatestr, sizeof(vdatestr));
-          time2str(vtime, vtimestr, sizeof(vtimestr));
-
-          fprintf(stdout, "%6d  %s %s", tsID + 1, vdatestr, vtimestr);
+          fprintf(stdout, "%6d  %s %s", tsID + 1, dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
           if (tsID)
             {
@@ -342,10 +236,12 @@ Tinfo(void *process)
               juldate = julianDateEncode(calendar, vdate, vtime);
               jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
 
-              getTimeInc(jdelta, vdate0, vdate, &incperiod, &incunit);
+              const auto timeIncr = getTimeIncrement(jdelta, vdate0, vdate);
+              incperiod = timeIncr.period;
+              incunit = timeIncr.unit;
 
               /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), incperiod%3600);*/
-              len = fprintf(stdout, " %3ld %s%s", (long)incperiod, tunits[incunit], labs(incperiod) != 1 ? "s" : "");
+              len = fprintf(stdout, " %3ld %s%s", (long) incperiod, tunits[(int) incunit], std::abs(incperiod) != 1 ? "s" : "");
               for (i = 0; i < 11 - len; ++i) fprintf(stdout, " ");
             }
           else
@@ -357,15 +253,11 @@ Tinfo(void *process)
 
           if (lforecast)
             {
-              int64_t fdate = taxisInqFdate(taxisID);
-              int ftime = taxisInqFtime(taxisID);
+              const auto fdate = taxisInqFdate(taxisID);
+              const auto ftime = taxisInqFtime(taxisID);
+              fprintf(stdout, " %s %s", dateToString(fdate).c_str(), timeToString(ftime).c_str());
 
-              date2str(fdate, vdatestr, sizeof(vdatestr));
-              time2str(ftime, vtimestr, sizeof(vtimestr));
-
-              fprintf(stdout, " %s %s", vdatestr, vtimestr);
-
-              double fc_period = taxisInqForecastPeriod(taxisID);
+              const auto fc_period = taxisInqForecastPeriod(taxisID);
               fprintf(stdout, " %7g", fc_period);
             }
 
@@ -432,15 +324,10 @@ Tinfo(void *process)
 
   fprintf(stdout, "\n");
 
-  date2str(vdate_first, vdatestr, sizeof(vdatestr));
-  time2str(vtime_first, vtimestr, sizeof(vtimestr));
-  fprintf(stdout, " Start date          : %s %s\n", vdatestr, vtimestr);
+  fprintf(stdout, " Start date          : %s %s\n", dateToString(vdate_first).c_str(), timeToString(vtime_first).c_str());
+  fprintf(stdout, " End date            : %s %s\n", dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
-  date2str(vdate, vdatestr, sizeof(vdatestr));
-  time2str(vtime, vtimestr, sizeof(vtimestr));
-  fprintf(stdout, " End date            : %s %s\n", vdatestr, vtimestr);
-
-  fprintf(stdout, " Increment           : %3ld %s%s\n", (long)incperiod0, tunits[incunit0], incperiod0 != 1 ? "s" : "");
+  fprintf(stdout, " Increment           : %3ld %s%s\n", (long) incperiod0, tunits[(int) incunit0], incperiod0 != 1 ? "s" : "");
   fprintf(stdout, " Number of timesteps : %d\n", tsID);
   fprintf(stdout, " Gaps identified     : %d\n", ngaps);
 
@@ -475,10 +362,7 @@ Tinfo(void *process)
 
               vdate = vdatem[igap][its];
               vtime = vtimem[igap][its];
-
-              date2str(vdate, vdatestr, sizeof(vdatestr));
-              time2str(vtime, vtimestr, sizeof(vtimestr));
-              fprintf(stdout, "  %s %s", vdatestr, vtimestr);
+              fprintf(stdout, "  %s %s", dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
               ntimeout++;
               tsID++;
@@ -489,5 +373,5 @@ Tinfo(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
