@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,9 +17,8 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
-#include "cdo_defaultValues.h"  // Namespace CdoDefault
+#include "process_int.h"
+#include "cdo_default_values.h"  // Namespace CdoDefault
 
 void *
 Tocomplex(void *process)
@@ -30,35 +29,39 @@ Tocomplex(void *process)
 
   cdoInitialize(process);
 
-  int RETOCOMPLEX = cdoOperatorAdd("retocomplex", 0, 0, nullptr);
-  int IMTOCOMPLEX = cdoOperatorAdd("imtocomplex", 0, 0, nullptr);
+  const auto RETOCOMPLEX = cdoOperatorAdd("retocomplex", 0, 0, nullptr);
+  const auto IMTOCOMPLEX = cdoOperatorAdd("imtocomplex", 0, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int nvars = vlistNvars(vlistID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto nvars = vlistNvars(vlistID2);
   for (varID = 0; varID < nvars; ++varID)
     {
-      int datatype = vlistInqVarDatatype(vlistID2, varID);
+      auto datatype = vlistInqVarDatatype(vlistID2, varID);
       datatype = (datatype == CDI_DATATYPE_FLT64) ? CDI_DATATYPE_CPX64 : CDI_DATATYPE_CPX32;
       vlistDefVarDatatype(vlistID2, varID, datatype);
     }
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   // if (CdoDefault::FileType != CDI_FILETYPE_EXT) cdoAbort("Complex numbers need EXTRA format; used CDO option -f ext!");
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  std::vector<double> array1(gridsize);
-  std::vector<double> array2(2 * gridsize);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array1(gridsizemax), array2(2 * gridsizemax);
+
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
   int tsID = 0;
   int tsID2 = 0;
@@ -72,10 +75,9 @@ Tocomplex(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoDefRecord(streamID2, varID, levelID);
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-
           cdoReadRecord(streamID1, array1.data(), &nmiss);
 
+          const auto gridsize = varList1[varID].gridsize;
           if (operatorID == RETOCOMPLEX)
             {
               for (size_t i = 0; i < gridsize; ++i)
