@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,50 +17,48 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 
 void *
 Unpack(void *process)
 {
   int nrecs;
-  int varID, levelID;
-  size_t nmiss;
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto nvars = vlistNvars(vlistID1);
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const int datatype = vlistInqVarDatatype(vlistID1, varID);
-      const double addoffset = vlistInqVarAddoffset(vlistID1, varID);
-      const double scalefactor = vlistInqVarScalefactor(vlistID1, varID);
+      const auto datatype = vlistInqVarDatatype(vlistID1, varID);
+      const auto addoffset = vlistInqVarAddoffset(vlistID1, varID);
+      const auto scalefactor = vlistInqVarScalefactor(vlistID1, varID);
       const bool laddoffset = IS_NOT_EQUAL(addoffset, 0);
       const bool lscalefactor = IS_NOT_EQUAL(scalefactor, 1);
       if (laddoffset || lscalefactor)
         {
-          if (datatype != CDI_DATATYPE_FLT64)
-            {
-              vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT32);
-            }
           vlistDefVarAddoffset(vlistID2, varID, 0);
           vlistDefVarScalefactor(vlistID2, varID, 1);
+          if (datatype != CDI_DATATYPE_FLT64)
+            vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT32);
         }
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  Varray<double> array(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -70,9 +68,11 @@ Unpack(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoDefRecord(streamID2, varID, levelID);
 
+          size_t nmiss;
           cdoReadRecord(streamID1, array.data(), &nmiss);
           cdoWriteRecord(streamID2, array.data(), nmiss);
         }
@@ -87,5 +87,5 @@ Unpack(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
