@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,14 +17,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "functs.h"
+#include "process_int.h"
 
 static void
 checkUniqueZaxis(int vlistID)
 {
-  const int nzaxis = vlistNzaxis(vlistID);
-  const int zaxisID = vlistZaxis(vlistID, 0);
-  const int nlevels = zaxisInqSize(zaxisID);
+  const auto nzaxis = vlistNzaxis(vlistID);
+  const auto zaxisID = vlistZaxis(vlistID, 0);
+  const auto nlevels = zaxisInqSize(zaxisID);
   for (int index = 1; index < nzaxis; ++index)
     {
       if (nlevels != zaxisInqSize(vlistZaxis(vlistID, index))) cdoAbort("Number of level differ!");
@@ -34,9 +35,9 @@ checkUniqueZaxis(int vlistID)
 static void
 checkUniqueGridsize(int vlistID)
 {
-  const int ngrids = vlistNgrids(vlistID);
-  const int gridID = vlistGrid(vlistID, 0);
-  const size_t gridsize = gridInqSize(gridID);
+  const auto ngrids = vlistNgrids(vlistID);
+  const auto gridID = vlistGrid(vlistID, 0);
+  const auto gridsize = gridInqSize(gridID);
   for (int index = 0; index < ngrids; index++)
     {
       if (gridsize != gridInqSize(vlistGrid(vlistID, index))) cdoAbort("Horizontal gridsize differ!");
@@ -44,18 +45,18 @@ checkUniqueGridsize(int vlistID)
 }
 
 static void
-setAttributes(int vlistID1, int vlistID2, int varID2, int operatorID)
+setAttributes(const VarList &varList1, int vlistID1, int vlistID2, int varID2, int operatorID)
 {
   bool paramIsEqual = true;
   char name[CDI_MAX_NAME], zname[CDI_MAX_NAME];
-  const int param = vlistInqVarParam(vlistID1, 0);
-  vlistInqVarName(vlistID1, 0, name);
-  int nvars = vlistNvars(vlistID1);
+  strcpy(name, varList1[0].name);
+  const auto param = varList1[0].param;
+  const auto nvars = vlistNvars(vlistID1);
   for (int varID = 1; varID < nvars; ++varID)
     {
-      const int zparam = vlistInqVarParam(vlistID1, varID);
-      vlistInqVarName(vlistID1, varID, zname);
-      if (param != zparam || !cstrIsEqual(name, zname))
+      strcpy(zname, varList1[varID].name);
+      const auto zparam = varList1[varID].param;
+      if (param != zparam || !cdo_cmpstr(name, zname))
         {
           paramIsEqual = false;
           break;
@@ -63,16 +64,16 @@ setAttributes(int vlistID1, int vlistID2, int varID2, int operatorID)
     }
 
   if (!paramIsEqual) strcpy(name, cdoOperatorName(operatorID));
-  vlistDefVarName(vlistID2, varID2, name);
+  cdiDefKeyString(vlistID2, varID2, CDI_KEY_NAME, name);
   if (paramIsEqual)
     {
       if (param >= 0) vlistDefVarParam(vlistID2, varID2, param);
       zname[0] = 0;
       vlistInqVarLongname(vlistID1, 0, zname);
-      if (*zname) vlistDefVarUnits(vlistID2, varID2, zname);
+      if (*zname) cdiDefKeyString(vlistID2, varID2, CDI_KEY_LONGNAME, zname);
       zname[0] = 0;
       vlistInqVarUnits(vlistID1, 0, zname);
-      if (*zname) vlistDefVarUnits(vlistID2, varID2, zname);
+      if (*zname) cdiDefKeyString(vlistID2, varID2, CDI_KEY_UNITS, zname);
     }
 }
 
@@ -103,57 +104,58 @@ Varsstat(void *process)
 
   addOperators();
 
-  // clang-format off
-  const int operatorID = cdoOperatorID();
-  const int operfunc   = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
+  operatorCheckArgc(0);
+
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
   const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
-  // clang-format on
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
   checkUniqueZaxis(vlistID1);
-  const int zaxisID = vlistZaxis(vlistID1, 0);
-  const int nlevels = zaxisInqSize(zaxisID);
+  const auto zaxisID = vlistZaxis(vlistID1, 0);
+  const auto nlevels = zaxisInqSize(zaxisID);
 
   checkUniqueGridsize(vlistID1);
-  const int gridID = vlistGrid(vlistID1, 0);
-  const size_t gridsize = gridInqSize(gridID);
+  const auto gridID = vlistGrid(vlistID1, 0);
+  const auto gridsize = gridInqSize(gridID);
 
-  const int timetype = vlistInqVarTimetype(vlistID1, 0);
-  const int nvars = vlistNvars(vlistID1);
+  const auto timetype = varList1[0].timetype;
+  const auto nvars = vlistNvars(vlistID1);
   for (varID = 1; varID < nvars; ++varID)
     {
-      if (timetype != vlistInqVarTimetype(vlistID1, varID)) cdoAbort("Number of timesteps differ!");
+      if (timetype != varList1[varID].timetype) cdoAbort("Number of timesteps differ!");
     }
 
-  const int vlistID2 = vlistCreate();
-  const int varID2 = vlistDefVar(vlistID2, gridID, zaxisID, timetype);
-  setAttributes(vlistID1, vlistID2, varID2, operatorID);
+  const auto vlistID2 = vlistCreate();
+  const auto varID2 = vlistDefVar(vlistID2, gridID, zaxisID, timetype);
+  setAttributes(varList1, vlistID1, vlistID2, varID2, operatorID);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   Field field;
   field.resize(gridsize);
 
-  FieldVector vars1(nlevels);
-  FieldVector samp1(nlevels);
-  FieldVector vars2;
+  FieldVector vars1(nlevels), samp1(nlevels), vars2;
   if (lvarstd || lrange) vars2.resize(nlevels);
 
   for (levelID = 0; levelID < nlevels; levelID++)
     {
-      const double missval = vlistInqVarMissval(vlistID1, 0);
+      const auto missval = varList1[0].missval;
 
       vars1[levelID].grid = gridID;
       vars1[levelID].missval = missval;
@@ -183,11 +185,11 @@ Varsstat(void *process)
 
           if (varID == 0)
             {
-              cdoReadRecord(streamID1, vars1[levelID].vec.data(), &vars1[levelID].nmiss);
+              cdoReadRecord(streamID1, vars1[levelID].vec_d.data(), &vars1[levelID].nmiss);
               if (lrange)
                 {
                   vars2[levelID].nmiss = vars1[levelID].nmiss;
-                  vars2[levelID].vec = vars1[levelID].vec;
+                  vars2[levelID].vec_d = vars1[levelID].vec_d;
                 }
 
               if (lvarstd) vfarmoq(vars2[levelID], vars1[levelID]);
@@ -197,12 +199,13 @@ Varsstat(void *process)
                   if (samp1[levelID].empty()) samp1[levelID].resize(gridsize);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    samp1[levelID].vec[i] = !DBL_IS_EQUAL(vars1[levelID].vec[i], vars1[levelID].missval);
+                    samp1[levelID].vec_d[i] = !DBL_IS_EQUAL(vars1[levelID].vec_d[i], vars1[levelID].missval);
                 }
             }
           else
             {
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+              field.size = vars1[levelID].size;
               field.grid = vars1[levelID].grid;
               field.missval = vars1[levelID].missval;
 
@@ -211,7 +214,7 @@ Varsstat(void *process)
                   if (samp1[levelID].empty()) samp1[levelID].resize(gridsize, vars1[levelID].nsamp);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (!DBL_IS_EQUAL(field.vec[i], field.missval)) samp1[levelID].vec[i]++;
+                    if (!DBL_IS_EQUAL(field.vec_d[i], field.missval)) samp1[levelID].vec_d[i]++;
                 }
 
               if (lvarstd)
@@ -265,7 +268,7 @@ Varsstat(void *process)
                 }
 
               cdoDefRecord(streamID2, 0, levelID);
-              cdoWriteRecord(streamID2, vars1[levelID].vec.data(), vars1[levelID].nmiss);
+              cdoWriteRecord(streamID2, vars1[levelID].vec_d.data(), vars1[levelID].nmiss);
               vars1[levelID].nsamp = 0;
             }
         }
