@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,19 +24,17 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 
 #define IS_SURFACE_LEVEL(zaxisID) (zaxisInqType(zaxisID) == ZAXIS_SURFACE && zaxisInqSize(zaxisID) == 1)
 
 static void
-add_vars_mv(size_t gridsize, double missval, const std::vector<double> &var1, const std::vector<double> &var2, std::vector<double> &var3)
+add_vars_mv(size_t gridsize, double missval, const Varray<double> &var1, const Varray<double> &var2, Varray<double> &var3)
 {
-  const double missval1 = missval;
-  const double missval2 = missval;
-  /*
-  for ( size_t i = 0; i < gridsize; ++i )
-    var3[i] = ADDMN(var2[i], var1[i]);
-  */
+  const auto missval1 = missval;
+  const auto missval2 = missval;
+  // for ( size_t i = 0; i < gridsize; ++i ) var3[i] = ADDMN(var2[i], var1[i]);
   for (size_t i = 0; i < gridsize; ++i)
     {
       var3[i] = var2[i];
@@ -56,7 +54,6 @@ Vertcum(void *process)
   int nrecs;
   int nlevshl = 0;
   int varID, levelID;
-  size_t nmiss;
 
   cdoInitialize(process);
 
@@ -65,24 +62,26 @@ Vertcum(void *process)
   int VERTCUMHL = cdoOperatorAdd("vertcumhl",  0,  0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
   int zaxisIDhl = -1;
 
   if (operatorID == VERTCUMHL)
     {
-      double *vct = nullptr;
+      std::vector<double> vct;
       bool lhybrid = false;
-      const int nzaxis = vlistNzaxis(vlistID1);
+      const auto nzaxis = vlistNzaxis(vlistID1);
       for (int i = 0; i < nzaxis; ++i)
         {
-          const int zaxisID = vlistZaxis(vlistID1, i);
-          const int nlevs = zaxisInqSize(zaxisID);
+          const auto zaxisID = vlistZaxis(vlistID1, i);
+          const auto nlevs = zaxisInqSize(zaxisID);
 
           if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevs > 1)
             {
@@ -94,42 +93,39 @@ Vertcum(void *process)
                       lhybrid = true;
                       nlevshl = nlevs + 1;
 
-                      double *vct = (double *) Malloc(nvct * sizeof(double));
-                      zaxisInqVct(zaxisID, vct);
+                      vct.resize(nvct);
+                      zaxisInqVct(zaxisID, vct.data());
 
                       zaxisIDhl = zaxisCreate(ZAXIS_HYBRID_HALF, nlevshl);
-                      double *levels = (double *) Malloc(nlevshl * sizeof(double));
+                      std::vector<double> levels(nlevshl);
                       for (levelID = 0; levelID < nlevshl; ++levelID) levels[levelID] = levelID + 1;
-                      zaxisDefLevels(zaxisIDhl, levels);
-                      Free(levels);
-                      zaxisDefVct(zaxisIDhl, nvct, vct);
+                      zaxisDefLevels(zaxisIDhl, levels.data());
+                      zaxisDefVct(zaxisIDhl, nvct, vct.data());
                       vlistChangeZaxisIndex(vlistID2, i, zaxisIDhl);
                     }
-                  else if (vct)
+                  else if (vct.size())
                     {
-                      if (memcmp(vct, zaxisInqVctPtr(zaxisID), nvct * sizeof(double)) == 0)
+                      if (memcmp(vct.data(), zaxisInqVctPtr(zaxisID), nvct * sizeof(double)) == 0)
                         vlistChangeZaxisIndex(vlistID2, i, zaxisIDhl);
                     }
                 }
             }
         }
-      if (vct) Free(vct);
     }
 
   VarList varList1, varList2;
   varListInit(varList1, vlistID1);
   varListInit(varList2, vlistID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<std::vector<size_t>> varnmiss(nvars);
-  std::vector<std::vector<std::vector<double>>> vardata1(nvars);
-  std::vector<std::vector<std::vector<double>>> vardata2(nvars);
+  Varray3D<double> vardata1(nvars), vardata2(nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      const size_t gridsize = varList1[varID].gridsize;
-      const int nlevs = varList1[varID].nlevels;
-      const int nlevs2 = varList2[varID].nlevels;
+      const auto gridsize = varList1[varID].gridsize;
+      const auto nlevs = varList1[varID].nlevels;
+      const auto nlevs2 = varList2[varID].nlevels;
 
       varnmiss[varID].resize(nlevs);
       vardata1[varID].resize(nlevs);
@@ -138,11 +134,11 @@ Vertcum(void *process)
       for (levelID = 0; levelID < nlevs2; ++levelID) vardata2[varID][levelID].resize(gridsize);
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -159,9 +155,9 @@ Vertcum(void *process)
 
       for (varID = 0; varID < nvars; ++varID)
         {
-          const double missval = varList2[varID].missval;
-          const size_t gridsize = varList2[varID].gridsize;
-          const int nlevs2 = varList2[varID].nlevels;
+          const auto missval = varList2[varID].missval;
+          const auto gridsize = varList2[varID].gridsize;
+          const auto nlevs2 = varList2[varID].nlevels;
 
           if (operatorID == VERTCUMHL && nlevs2 == nlevshl)
             {
@@ -183,10 +179,10 @@ Vertcum(void *process)
 
           if (operatorID == VERTCUMHL && nlevs2 == nlevshl)
             {
-              std::vector<double> &var1 = vardata2[varID][nlevs2 - 1];
+              const auto &var1 = vardata2[varID][nlevs2 - 1];
               for (levelID = 0; levelID < nlevs2; ++levelID)
                 {
-                  std::vector<double> &var2 = vardata2[varID][levelID];
+                  auto &var2 = vardata2[varID][levelID];
                   for (size_t i = 0; i < gridsize; ++i)
                     {
                       if (IS_NOT_EQUAL(var1[i], 0))
@@ -200,15 +196,15 @@ Vertcum(void *process)
 
       for (varID = 0; varID < nvars; ++varID)
         {
-          const double missval = varList2[varID].missval;
-          const size_t gridsize = varList2[varID].gridsize;
-          const int nlevs2 = varList2[varID].nlevels;
+          const auto missval = varList2[varID].missval;
+          const auto gridsize = varList2[varID].gridsize;
+          const auto nlevs2 = varList2[varID].nlevels;
           for (levelID = 0; levelID < nlevs2; ++levelID)
             {
-              double *single = vardata2[varID][levelID].data();
-              nmiss = arrayNumMV(gridsize, single, missval);
+              auto &single = vardata2[varID][levelID];
+              const auto nmiss = varrayNumMV(gridsize, single, missval);
               cdoDefRecord(streamID2, varID, levelID);
-              cdoWriteRecord(streamID2, single, nmiss);
+              cdoWriteRecord(streamID2, single.data(), nmiss);
             }
         }
 
