@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,10 +33,14 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "cdo_zaxis.h"
 #include "param_conversion.h"
 #include "pmlist.h"
+#include "cdi_lockedIO.h"
 
 #define IS_SURFACE_LEVEL(zaxisID) (zaxisInqType(zaxisID) == ZAXIS_SURFACE && zaxisInqSize(zaxisID) == 1)
 
@@ -45,10 +49,10 @@ getSurfaceID(int vlistID)
 {
   int surfID = -1;
 
-  const int nzaxis = vlistNzaxis(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; ++index)
     {
-      const int zaxisID = vlistZaxis(vlistID, index);
+      const auto zaxisID = vlistZaxis(vlistID, index);
       if (IS_SURFACE_LEVEL(zaxisID))
         {
           surfID = vlistZaxis(vlistID, index);
@@ -64,10 +68,10 @@ getSurfaceID(int vlistID)
 static void
 setSurfaceID(const int vlistID, const int surfID)
 {
-  const int nzaxis = vlistNzaxis(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; ++index)
     {
-      const int zaxisID = vlistZaxis(vlistID, index);
+      const auto zaxisID = vlistZaxis(vlistID, index);
       if (zaxisID != surfID || !IS_SURFACE_LEVEL(zaxisID)) vlistChangeZaxisIndex(vlistID, index, surfID);
     }
 }
@@ -77,8 +81,8 @@ genLayerBounds(int nlev, double *levels, double *lbounds, double *ubounds)
 {
   if (nlev == 1)
     {
-      lbounds[0] = 0.;
-      ubounds[0] = 1.;
+      lbounds[0] = 0.0;
+      ubounds[0] = 1.0;
     }
   else
     {
@@ -86,7 +90,7 @@ genLayerBounds(int nlev, double *levels, double *lbounds, double *ubounds)
       ubounds[nlev - 1] = levels[nlev - 1];
       for (int i = 0; i < nlev - 1; ++i)
         {
-          double bound = 0.5 * (levels[i] + levels[i + 1]);
+          const auto bound = 0.5 * (levels[i] + levels[i + 1]);
           lbounds[i + 1] = bound;
           ubounds[i] = bound;
         }
@@ -97,9 +101,7 @@ int
 getLayerThickness(bool useweights, bool genbounds, int index, int zaxisID, int nlev, double *thickness, double *weights)
 {
   int status = 0;
-  std::vector<double> levels(nlev);
-  std::vector<double> lbounds(nlev);
-  std::vector<double> ubounds(nlev);
+  Varray<double> levels(nlev), lbounds(nlev), ubounds(nlev);
 
   cdoZaxisInqLevels(zaxisID, levels.data());
   if (genbounds)
@@ -124,8 +126,7 @@ getLayerThickness(bool useweights, bool genbounds, int index, int zaxisID, int n
 
   for (int i = 0; i < nlev; ++i) thickness[i] = std::fabs(ubounds[i] - lbounds[i]);
 
-  double lsum = 0;
-  double wsum = 0;
+  double lsum = 0, wsum = 0;
   for (int i = 0; i < nlev; ++i) lsum += thickness[i];
   for (int i = 0; i < nlev; ++i) weights[i] = thickness[i];
   for (int i = 0; i < nlev; ++i) weights[i] /= (lsum / nlev);
@@ -145,10 +146,10 @@ getLayerThickness(bool useweights, bool genbounds, int index, int zaxisID, int n
 static void
 vertstatGetParameter(bool *weights, bool *genbounds)
 {
-  const int pargc = operatorArgc();
+  const auto pargc = operatorArgc();
   if (pargc)
     {
-      const char **pargv = (const char **) operatorArgv();
+      const auto pargv = cdoGetOperArgv();
 
       KVList kvlist;
       kvlist.name = "VERTSTAT";
@@ -157,17 +158,16 @@ vertstatGetParameter(bool *weights, bool *genbounds)
 
       for (const auto &kv : kvlist)
         {
-          const char *key = kv.key.c_str();
-          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv.values[0].c_str();
+          const auto &key = kv.key;
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          const auto &value = kv.values[0];
 
-          if (cstrIsEqual(key, "weights"))
-            *weights = parameter2bool(value);
-          else if (cstrIsEqual(key, "genbounds"))
-            *genbounds = parameter2bool(value);
-          else
-            cdoAbort("Invalid parameter key >%s<!", key);
+          // clang-format off
+          if      (key == "weights")   *weights = parameter2bool(value);
+          else if (key == "genbounds") *genbounds = parameter2bool(value);
+          else cdoAbort("Invalid parameter key >%s<!", key.c_str());
+          // clang-format on
         }
     }
 }
@@ -175,15 +175,14 @@ vertstatGetParameter(bool *weights, bool *genbounds)
 void *
 Vertstat(void *process)
 {
-  int nrecs;
   int varID, levelID;
   struct VertInfo
   {
     int zaxisID;
     int status;
     int numlevel;
-    std::vector<double> thickness;
-    std::vector<double> weights;
+    Varray<double> thickness;
+    Varray<double> weights;
   };
 
   cdoInitialize(process);
@@ -201,8 +200,8 @@ Vertstat(void *process)
                  cdoOperatorAdd("vertstd",   func_std,   1, nullptr);
                  cdoOperatorAdd("vertstd1",  func_std1,  1, nullptr);
 
-  const int operatorID   = cdoOperatorID();
-  const int operfunc     = cdoOperatorF1(operatorID);
+  const auto operatorID  = cdoOperatorID();
+  const auto operfunc    = cdoOperatorF1(operatorID);
   const bool needWeights = cdoOperatorF2(operatorID);
 
   const bool lrange  = operfunc == func_range;
@@ -214,24 +213,24 @@ Vertstat(void *process)
 
   // int applyWeights = lmean;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
   vlistClearFlag(vlistID1);
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; varID++) vlistDefFlag(vlistID1, varID, 0, true);
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int surfID = getSurfaceID(vlistID1);
+  const auto surfID = getSurfaceID(vlistID1);
   setSurfaceID(vlistID2, surfID);
 
-  const int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   std::vector<VertInfo> vert(nzaxis);
   if (needWeights)
     {
@@ -247,8 +246,8 @@ Vertstat(void *process)
 
       for (int index = 0; index < nzaxis; ++index)
         {
-          const int zaxisID = vlistZaxis(vlistID1, index);
-          const int nlev = zaxisInqSize(zaxisID);
+          const auto zaxisID = vlistZaxis(vlistID1, index);
+          const auto nlev = zaxisInqSize(zaxisID);
           vert[index].numlevel = 0;
           vert[index].status = 0;
           vert[index].zaxisID = zaxisID;
@@ -264,20 +263,18 @@ Vertstat(void *process)
         }
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   VarList varList;
   varListInit(varList, vlistID1);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
-  FieldVector vars1(nvars);
-  FieldVector samp1(nvars);
-  FieldVector vars2;
+  FieldVector vars1(nvars), samp1(nvars), vars2;
   if (lvarstd || lrange) vars2.resize(nvars);
 
   for (varID = 0; varID < nvars; varID++)
@@ -296,8 +293,11 @@ Vertstat(void *process)
     }
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
@@ -308,9 +308,9 @@ Vertstat(void *process)
           vars1[varID].nsamp++;
           if (lrange) vars2[varID].nsamp++;
 
-          const size_t gridsize = varList[varID].gridsize;
-          const int zaxisID = varList[varID].zaxisID;
-          const int nlev = varList[varID].nlevels;
+          const auto gridsize = varList[varID].gridsize;
+          const auto zaxisID = varList[varID].zaxisID;
+          const auto nlev = varList[varID].nlevels;
 
           double layer_weight = 1.0;
           double layer_thickness = 1.0;
@@ -321,9 +321,8 @@ Vertstat(void *process)
                   {
                     if (vert[index].status == 0 && tsID == 0 && levelID == 0 && nlev > 1)
                       {
-                        char varname[CDI_MAX_NAME];
-                        vlistInqVarName(vlistID1, varID, varname);
-                        cdoWarning("Layer bounds not available, using constant vertical weights for variable %s!", varname);
+                        cdoWarning("Layer bounds not available, using constant vertical weights for variable %s!",
+                                   varList[varID].name);
                       }
                     else
                       {
@@ -337,11 +336,11 @@ Vertstat(void *process)
 
           if (levelID == 0)
             {
-              cdoReadRecord(streamID1, vars1[varID].vec.data(), &vars1[varID].nmiss);
+              cdoReadRecord(streamID1, vars1[varID].vec_d.data(), &vars1[varID].nmiss);
               if (lrange)
                 {
                   vars2[varID].nmiss = vars1[varID].nmiss;
-                  vars2[varID].vec = vars1[varID].vec;
+                  vars2[varID].vec_d = vars1[varID].vec_d;
                 }
 
               if (operatorID == VERTINT && IS_NOT_EQUAL(layer_thickness, 1.0)) vfarcmul(vars1[varID], layer_thickness);
@@ -365,15 +364,15 @@ Vertstat(void *process)
                   if (samp1[varID].empty()) samp1[varID].resize(gridsize);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (DBL_IS_EQUAL(vars1[varID].vec[i], vars1[varID].missval))
-                      samp1[varID].vec[i] = 0.;
+                    if (DBL_IS_EQUAL(vars1[varID].vec_d[i], vars1[varID].missval))
+                      samp1[varID].vec_d[i] = 0.0;
                     else
-                      samp1[varID].vec[i] = layer_weight;
+                      samp1[varID].vec_d[i] = layer_weight;
                 }
             }
           else
             {
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
               field.grid = vars1[varID].grid;
               field.size = vars1[varID].size;
               field.missval = vars1[varID].missval;
@@ -386,7 +385,7 @@ Vertstat(void *process)
                   if (samp1[varID].empty()) samp1[varID].resize(gridsize, vars1[varID].nsamp);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (!DBL_IS_EQUAL(field.vec[i], vars1[varID].missval)) samp1[varID].vec[i] += layer_weight;
+                    if (!DBL_IS_EQUAL(field.vec_d[i], vars1[varID].missval)) samp1[varID].vec_d[i] += layer_weight;
                 }
 
               if (lvarstd)
@@ -448,7 +447,7 @@ Vertstat(void *process)
                 }
 
               cdoDefRecord(streamID2, varID, 0);
-              cdoWriteRecord(streamID2, vars1[varID].vec.data(), vars1[varID].nmiss);
+              cdoWriteRecord(streamID2, vars1[varID].vec_d.data(), vars1[varID].nmiss);
               vars1[varID].nsamp = 0;
             }
         }
