@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
-#include "after_vertint.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdi_lockedIO.h"
+#include "vertical_interp.h"
 #include "util_string.h"
+#include "cdo_zaxis.h"
 
 #define R 287.07  /* spezielle Gaskonstante fuer Luft */
 #define G 9.80665 /* Erdbeschleunigung */
@@ -34,20 +36,20 @@
 void *
 Vertwind(void *process)
 {
-  int nrecs;
   int varID, levelID;
   int nvct = 0;
   size_t nmiss;
   int tempID = -1, sqID = -1, psID = -1, omegaID = -1;
   char varname[CDI_MAX_NAME];
-  std::vector<double> vct;
-  std::vector<double> hpress, ps_prog;
+  Varray<double> vct;
+  Varray<double> hpress, ps_prog;
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
   vlist_check_gridsize(vlistID1);
 
@@ -91,32 +93,32 @@ Vertwind(void *process)
     }
 
   /* Get missing values */
-  double missval_t = vlistInqVarMissval(vlistID1, tempID);
-  double missval_sq = vlistInqVarMissval(vlistID1, sqID);
-  double missval_wap = vlistInqVarMissval(vlistID1, omegaID);
-  double missval_out = missval_wap;
+  const auto missval_t = vlistInqVarMissval(vlistID1, tempID);
+  const auto missval_sq = vlistInqVarMissval(vlistID1, sqID);
+  const auto missval_wap = vlistInqVarMissval(vlistID1, omegaID);
+  const auto missval_out = missval_wap;
 
-  int gridID = vlistInqVarGrid(vlistID1, omegaID);
-  int zaxisID = vlistInqVarZaxis(vlistID1, omegaID);
+  const auto gridID = vlistInqVarGrid(vlistID1, omegaID);
+  const auto zaxisID = vlistInqVarZaxis(vlistID1, omegaID);
 
   if (psID == -1 && zaxisInqType(zaxisID) == ZAXIS_HYBRID) cdoAbort("Surface pressure (code 134) not found!");
 
-  size_t gridsize = gridInqSize(gridID);
-  int nlevel = zaxisInqSize(zaxisID);
-  std::vector<double> level(nlevel);
+  const auto gridsize = gridInqSize(gridID);
+  const auto nlevel = zaxisInqSize(zaxisID);
+  Varray<double> level(nlevel);
   cdoZaxisInqLevels(zaxisID, level.data());
 
-  std::vector<double> temp(gridsize * nlevel);
-  std::vector<double> sq(gridsize * nlevel);
-  std::vector<double> omega(gridsize * nlevel);
-  std::vector<double> wms(gridsize * nlevel);
-  std::vector<double> fpress(gridsize * nlevel);
+  Varray<double> temp(gridsize * nlevel);
+  Varray<double> sq(gridsize * nlevel);
+  Varray<double> omega(gridsize * nlevel);
+  Varray<double> wms(gridsize * nlevel);
+  Varray<double> fpress(gridsize * nlevel);
 
   if (zaxisInqType(zaxisID) == ZAXIS_PRESSURE)
     {
       for (levelID = 0; levelID < nlevel; ++levelID)
         {
-          size_t offset = (size_t) levelID * gridsize;
+          const auto offset = (size_t) levelID * gridsize;
           for (size_t i = 0; i < gridsize; ++i) fpress[offset + i] = level[levelID];
         }
     }
@@ -140,25 +142,28 @@ Vertwind(void *process)
   vlistClearFlag(vlistID1);
   for (levelID = 0; levelID < nlevel; ++levelID) vlistDefFlag(vlistID1, omegaID, levelID, true);
 
-  int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
   vlistDefVarCode(vlistID2, 0, 40);
-  vlistDefVarName(vlistID2, 0, "W");
-  vlistDefVarLongname(vlistID2, 0, "Vertical velocity");
-  vlistDefVarUnits(vlistID2, 0, "m/s");
+  cdiDefKeyString(vlistID2, 0, CDI_KEY_NAME, "W");
+  cdiDefKeyString(vlistID2, 0, CDI_KEY_LONGNAME, "Vertical velocity");
+  cdiDefKeyString(vlistID2, 0, CDI_KEY_UNITS, "m/s");
   vlistDefVarMissval(vlistID2, 0, missval_out);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
@@ -166,7 +171,7 @@ Vertwind(void *process)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          size_t offset = (size_t) levelID * gridsize;
+          const auto offset = (size_t) levelID * gridsize;
 
           if (varID == tempID)
             cdoReadRecord(streamID1, &temp[offset], &nmiss);
@@ -182,7 +187,7 @@ Vertwind(void *process)
 
       for (levelID = 0; levelID < nlevel; ++levelID)
         {
-          size_t offset = (size_t) levelID * gridsize;
+          const auto offset = (size_t) levelID * gridsize;
 
           for (size_t i = 0; i < gridsize; ++i)
             {
@@ -194,10 +199,10 @@ Vertwind(void *process)
               else
                 {
                   // Virtuelle Temperatur bringt die Feuchteabhaengigkeit hinein
-                  double tv = temp[offset + i] * (1. + 0.608 * sq[offset + i]);
+                  const auto tv = temp[offset + i] * (1. + 0.608 * sq[offset + i]);
 
                   // Die Dichte erhaelt man nun mit der Gasgleichung rho=p/(R*tv) Level in Pa!
-                  double rho = fpress[offset + i] / (R * tv);
+                  const auto rho = fpress[offset + i] / (R * tv);
                   /*
                     Nun daraus die Vertikalgeschwindigkeit im m/s, indem man die Vertikalgeschwindigkeit
                     in Pa/s durch die Erdbeschleunigung und die Dichte teilt
@@ -209,7 +214,7 @@ Vertwind(void *process)
 
       for (levelID = 0; levelID < nlevel; ++levelID)
         {
-          size_t offset = (size_t) levelID * gridsize;
+          const auto offset = (size_t) levelID * gridsize;
 
           size_t nmiss_out = 0;
           for (size_t i = 0; i < gridsize; i++)
@@ -229,5 +234,5 @@ Vertwind(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
