@@ -23,14 +23,18 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_cdi_wrapper.h"
 
 static const char WCT_NAME[] = "wind_chill_temperature";
 static const char WCT_LONGNAME[] = "Windchill temperature describes the fact that low temperatures are felt "
                                    "to be even lower in case of wind. It is based on the rate of heat loss "
                                    "from exposed skin caused by wind and cold. It is calculated according "
                                    "to the empirical formula: 33 + (T - 33) * (0.478 + 0.237 * ( "
-                                   "SQRTMN(ff*3.6) - 0.0124 * ff * 3.6)) with T  = air temperature in "
+                                   "SQRT(ff*3.6) - 0.0124 * ff * 3.6)) with T  = air temperature in "
                                    "degree Celsius, ff = 10 m wind speed in m/s. Windchill temperature is "
                                    "only defined for temperatures at or below 33 degree Celsius and wind "
                                    "speeds above 1.39 m/s. It is mainly used for freezing temperatures.";
@@ -41,32 +45,32 @@ static const int FIRST_VAR = 0;
 static double
 windchillTemperature(double t, double ff, double missval)
 {
-  static const double tmax = 33.0;
-  static const double vmin = 1.39; /* minimum wind speed (m/s) */
+  constexpr double tmax = 33.0;
+  constexpr double vmin = 1.39; /* minimum wind speed (m/s) */
 
-  return ff < vmin || t > tmax ? missval : tmax + (t - tmax) * (0.478 + 0.237 * (sqrt(ff * 3.6) - 0.0124 * ff * 3.6));
+  return ff < vmin || t > tmax ? missval : tmax + (t - tmax) * (0.478 + 0.237 * (std::sqrt(ff * 3.6) - 0.0124 * ff * 3.6));
 }
 
 static void
 farexpr(Field &field1, Field &field2, double (*expression)(double, double, double))
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
     {
       for (size_t i = 0; i < len; i++)
-        if (DBL_IS_EQUAL(field1.vec[i], missval1) || DBL_IS_EQUAL(field2.vec[i], missval2))
-          field1.vec[i] = missval1;
+        if (DBL_IS_EQUAL(field1.vec_d[i], missval1) || DBL_IS_EQUAL(field2.vec_d[i], missval2))
+          field1.vec_d[i] = missval1;
         else
-          field1.vec[i] = expression(field1.vec[i], field2.vec[i], missval1);
+          field1.vec_d[i] = expression(field1.vec_d[i], field2.vec_d[i], missval1);
     }
   else
     {
-      for (size_t i = 0; i < len; i++) field1.vec[i] = expression(field1.vec[i], field2.vec[i], missval1);
+      for (size_t i = 0; i < len; i++) field1.vec_d[i] = expression(field1.vec_d[i], field2.vec_d[i], missval1);
     }
 
   field1.nmiss = fieldNumMiss(field1);
@@ -82,17 +86,19 @@ Wct(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("wct", 0, 0, nullptr);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_DIM);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
   field1.resize(gridsizemax);
@@ -100,23 +106,23 @@ Wct(void *process)
 
   if (Options::cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d", vlistNtsteps(vlistID1), vlistNtsteps(vlistID2));
 
-  const int vlistID3 = vlistCreate();
-  const int gridID = vlistInqVarGrid(vlistID1, FIRST_VAR);
-  const int zaxisID = vlistInqVarZaxis(vlistID1, FIRST_VAR);
-  const int varID3 = vlistDefVar(vlistID3, gridID, zaxisID, TIME_VARYING);
+  const auto vlistID3 = vlistCreate();
+  const auto gridID = vlistInqVarGrid(vlistID1, FIRST_VAR);
+  const auto zaxisID = vlistInqVarZaxis(vlistID1, FIRST_VAR);
+  const auto varID3 = vlistDefVar(vlistID3, gridID, zaxisID, TIME_VARYING);
 
-  const int taxisID3 = cdoTaxisCreate(TAXIS_RELATIVE);
+  const auto taxisID3 = cdoTaxisCreate(TAXIS_RELATIVE);
   taxisDefTunit(taxisID3, TUNIT_MINUTE);
   taxisDefCalendar(taxisID3, CALENDAR_STANDARD);
   taxisDefRdate(taxisID3, 19550101);
   taxisDefRtime(taxisID3, 0);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  vlistDefVarName(vlistID3, varID3, WCT_NAME);
-  vlistDefVarLongname(vlistID3, varID3, WCT_LONGNAME);
-  vlistDefVarUnits(vlistID3, varID3, WCT_UNITS);
+  cdiDefKeyString(vlistID3, varID3, CDI_KEY_NAME, WCT_NAME);
+  cdiDefKeyString(vlistID3, varID3, CDI_KEY_LONGNAME, WCT_LONGNAME);
+  cdiDefKeyString(vlistID3, varID3, CDI_KEY_UNITS, WCT_UNITS);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
   int tsID = 0;
@@ -131,10 +137,10 @@ Wct(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID1, &levelID1);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
+          cdoReadRecord(streamID1, field1.vec_d.data(), &field1.nmiss);
 
           cdoInqRecord(streamID2, &varID2, &levelID2);
-          cdoReadRecord(streamID2, field2.vec.data(), &field2.nmiss);
+          cdoReadRecord(streamID2, field2.vec_d.data(), &field2.nmiss);
 
           if (varID1 != varID2 || levelID1 != levelID2) cdoAbort("Input streams have different structure!");
 
@@ -146,7 +152,7 @@ Wct(void *process)
           farexpr(field1, field2, windchillTemperature);
 
           cdoDefRecord(streamID3, varID3, levelID1);
-          cdoWriteRecord(streamID3, field1.vec.data(), field1.nmiss);
+          cdoWriteRecord(streamID3, field1.vec_d.data(), field1.nmiss);
         }
 
       tsID++;
