@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,60 +21,62 @@
       Writerandom writerandom
 */
 
+#include <cstdlib>
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+
 
 void *
 Writerandom(void *process)
 {
-  size_t gridsize;
   int nrecs;
   int varID, levelID;
-  int rindex;
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
 
   cdoDefVlist(streamID2, vlistID2);
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
-      std::vector<std::vector<double>> recdata(nrecs);
-      std::vector<int> recvarID(nrecs);
-      std::vector<int> reclevelID(nrecs);
+      Varray2D<double> recdata(nrecs);
+      std::vector<int> recvarID(nrecs), reclevelID(nrecs), recindex(nrecs);
       std::vector<size_t> recnmiss(nrecs);
-      std::vector<int> recindex(nrecs);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           recvarID[recID] = varID;
           reclevelID[recID] = levelID;
-          recdata[recID].resize(gridsize);
+          recdata[recID].resize(varList1[varID].gridsize);
           cdoReadRecord(streamID1, recdata[recID].data(), &recnmiss[recID]);
         }
 
       for (int recID = 0; recID < nrecs; recID++) recindex[recID] = -1;
 
-      for (rindex = nrecs - 1; rindex >= 0; rindex--)
+      for (int rindex = nrecs - 1; rindex >= 0; rindex--)
         {
-          int index = (int) (rindex * ((double) rand()) / ((double) RAND_MAX));
+          const auto index = (int) (rindex * ((double) std::rand()) / ((double) RAND_MAX));
           /*	printf("rindex %d %d\n", rindex, index); */
           int ipos = -1;
           for (int recID = 0; recID < nrecs; recID++)
@@ -97,10 +99,9 @@ Writerandom(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          rindex = recindex[recID];
+          const auto rindex = recindex[recID];
           varID = recvarID[rindex];
           levelID = reclevelID[rindex];
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
           cdoDefRecord(streamID2, varID, levelID);
           cdoWriteRecord(streamID2, recdata[rindex].data(), recnmiss[rindex]);
         }
@@ -113,5 +114,5 @@ Writerandom(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
