@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -67,13 +67,20 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "cdo_task.h"
+#include <utility>
 
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_task.h"
 #include "timer.h"
 #include "datetime.h"
 #include "printinfo.h"
-//#include "pstream_write.h"
+#include "util_date.h"
+#include "param_conversion.h"
+
+#define USE_CDI_STREAM 1
 
 enum
 {
@@ -83,16 +90,19 @@ enum
   YEAR_LEN = 10
 };
 
-class ReadArguments
+struct ReadArguments
 {
-public:
-  ReadArguments(FieldVector2D &input_vars) : vars(input_vars){};
-  ~ReadArguments(){};
   int tsIDnext;
+#ifdef USE_CDI_STREAM
+  int streamID;
+#else
   CdoStreamID streamID;
-    int nrecs;
+#endif
+  int nrecs;
   RecordInfo *recList;
   FieldVector2D &vars;
+
+  ReadArguments(FieldVector2D &input_vars) : tsIDnext(0), nrecs(0), recList(nullptr), vars(input_vars) {};
 };
 
 static int num_recs = 0;
@@ -100,12 +110,10 @@ static int num_recs = 0;
 static void *
 cdoReadTimestep(void *rarg)
 {
-  int varID, levelID;
-  size_t nmiss;
   ReadArguments *readarg = (ReadArguments *) rarg;
-  FieldVector2D &input_vars = readarg->vars;
+  auto &input_vars = readarg->vars;
   RecordInfo *recList = readarg->recList;
-  CdoStreamID streamID = readarg->streamID;
+  const auto streamID = readarg->streamID;
   int tsIDnext = readarg->tsIDnext;
   int nrecs = readarg->nrecs;
 
@@ -113,7 +121,12 @@ cdoReadTimestep(void *rarg)
 
   for (int recID = 0; recID < nrecs; ++recID)
     {
+      int varID, levelID;
+#ifdef USE_CDI_STREAM
+      streamInqRecord(streamID, &varID, &levelID);
+#else
       cdoInqRecord(streamID, &varID, &levelID);
+#endif
 
       if (tsIDnext == 1 && recList)
         {
@@ -121,18 +134,29 @@ cdoReadTimestep(void *rarg)
           recList[recID].levelID = levelID;
         }
 
-      if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-        cdoReadRecordF(streamID, input_vars[varID][levelID].vecf.data(), &nmiss);
-      else
-        cdoReadRecord(streamID, input_vars[varID][levelID].vec.data(), &nmiss);
+      size_t nmiss;
 
+#ifdef USE_CDI_STREAM
+      if (Options::CDO_Memtype == MemType::Float)
+        streamReadRecordF(streamID, input_vars[varID][levelID].vec_f.data(), &nmiss);
+      else
+        streamReadRecord(streamID, input_vars[varID][levelID].vec_d.data(), &nmiss);
+#else
+      if (Options::CDO_Memtype == MemType::Float)
+        cdoReadRecordF(streamID, input_vars[varID][levelID].vec_f.data(), &nmiss);
+      else
+        cdoReadRecord(streamID, input_vars[varID][levelID].vec_d.data(), &nmiss);
+#endif
       input_vars[varID][levelID].nmiss = nmiss;
     }
 
-  // timer_stop(timer_read);
+    // timer_stop(timer_read);
 
+#ifdef USE_CDI_STREAM
+  num_recs = streamInqTimestep(streamID, tsIDnext);
+#else
   num_recs = cdoStreamInqTimestep(streamID, tsIDnext);
-  //num_recs = cdoStreamInqTimestep(streamID, tsIDnext);
+#endif
 
   return ((void *) &num_recs);
 }
@@ -189,7 +213,6 @@ XTimstat(void *process)
   TimeStat timestat_date = TimeStat::MEAN;
   int64_t vdate = 0, vdate0 = 0;
   int vtime = 0, vtime0 = 0;
-  int varID;
   CdoStreamID streamID3 = CDO_STREAM_UNDEF;
   int vlistID3, taxisID3 = -1;
   bool lvfrac = false;
@@ -201,9 +224,9 @@ XTimstat(void *process)
   addOperators();
 
   // clang-format off
-  int operatorID = cdoOperatorID();
-  int operfunc   = cdoOperatorF1(operatorID);
-  int comparelen = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  auto operfunc = cdoOperatorF1(operatorID);
+  const auto comparelen = cdoOperatorF2(operatorID);
 
   bool lmean   = operfunc == func_mean || operfunc == func_avg;
   bool lstd    = operfunc == func_std || operfunc == func_std1;
@@ -214,41 +237,48 @@ XTimstat(void *process)
 
   if (operfunc == func_mean)
     {
-      int oargc = operatorArgc();
-      char **oargv = operatorArgv();
-
+      const auto oargc = operatorArgc();
       if (oargc == 1)
         {
           lvfrac = true;
-          vfrac = atof(oargv[0]);
+          vfrac = parameter2double(cdoOperatorArgv(0));
           if (Options::cdoVerbose) cdoPrint("Set vfrac to %g", vfrac);
           if (vfrac < 0 || vfrac > 1) cdoAbort("vfrac out of range!");
         }
       else if (oargc > 1)
         cdoAbort("Too many arguments!");
     }
+  else
+    {
+      operatorCheckArgc(0);
+    }
 
-  int cmplen = DATE_LEN - comparelen;
+  const int cmplen = DATE_LEN - comparelen;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  // CdoStreamID streamID1 = cdoOpenRead(cdoGetStreamName(0));
+#ifdef USE_CDI_STREAM
+  const auto streamID1 = streamOpenRead(cdoGetStreamName(0));
+  auto vlistID1 = streamInqVlist(streamID1);
+#else
+  const auto streamID1 = cdoOpenRead(0);
+  auto vlistID1 = cdoStreamInqVlist(streamID1);
+#endif
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+  vlistDefineTimestepType(vlistID2, operfunc);
 
   if (cmplen == 0) vlistDefNtsteps(vlistID2, 1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   vlistSetFrequency(vlistID2, comparelen);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   if (Options::cdoDiag)
@@ -261,11 +291,11 @@ XTimstat(void *process)
 
       vlistID3 = vlistDuplicate(vlistID1);
 
-      for (varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < nvars; ++varID)
         {
           vlistDefVarDatatype(vlistID3, varID, CDI_DATATYPE_INT32);
           vlistDefVarMissval(vlistID3, varID, -1);
-          vlistDefVarUnits(vlistID3, varID, "");
+          cdiDefKeyString(vlistID3, varID, CDI_KEY_UNITS, "");
           vlistDefVarAddoffset(vlistID3, varID, 0);
           vlistDefVarScalefactor(vlistID3, varID, 1);
         }
@@ -282,38 +312,36 @@ XTimstat(void *process)
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
   int FIELD_MEMTYPE = 0;
-  if (operfunc == func_mean && Options::CDO_Memtype == MEMTYPE_FLOAT) FIELD_MEMTYPE = FIELD_FLT;
+  if (operfunc == func_mean && Options::CDO_Memtype == MemType::Float) FIELD_MEMTYPE = FIELD_FLT;
   FieldVector3D input_vars(2);
   fieldsFromVlist(vlistID1, input_vars[0], FIELD_VEC | FIELD_MEMTYPE);
   fieldsFromVlist(vlistID1, input_vars[1], FIELD_VEC | FIELD_MEMTYPE);
   FieldVector2D samp1, vars1, vars2;
-  fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
+  fieldsFromVlist(vlistID1, samp1);
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
   if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
-  int curFirst = 0, curSecond = 1;
+  int curFirst = 0;
+  int curSecond = 1;
+  (void) curSecond;
 
   ReadArguments readarg(input_vars[curFirst]);
   readarg.streamID = streamID1;
 
   bool lparallelread = Options::CDO_Parallel_Read > 0;
   bool ltsfirst = true;
-  void *read_task = nullptr;
+  cdo::Task *read_task = nullptr;
   void *readresult = nullptr;
 
-  if (lparallelread)
-    {
-      read_task = cdo_task_new();
-      if (read_task == nullptr)
-        {
-          lparallelread = false;
-          cdoWarning("CDO tasks not available!");
-        }
-    }
+  if (lparallelread) read_task = new cdo::Task;
 
   int tsID = 0;
   int otsID = 0;
-  int nrecs = cdoStreamInqTimestep(streamID1, tsID);
+#ifdef USE_CDI_STREAM
+  auto nrecs = streamInqTimestep(streamID1, tsID);
+#else
+  auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+#endif
   int maxrecs = nrecs;
   std::vector<RecordInfo> recList(maxrecs);
 
@@ -344,7 +372,7 @@ XTimstat(void *process)
             }
           else
             {
-              readresult = cdo_task_wait(read_task);
+              readresult = read_task->wait();
             }
 
           nrecs = *(int *) readresult;
@@ -355,7 +383,7 @@ XTimstat(void *process)
             {
               readarg.vars = input_vars[curFirst];
               readarg.tsIDnext = tsID + 1;
-              cdo_task_start(read_task, cdoReadTimestep, &readarg);
+              read_task->start(cdoReadTimestep, &readarg);
             }
 
           if (nsets == 0)
@@ -365,24 +393,23 @@ XTimstat(void *process)
 #endif
               for (int recID = 0; recID < maxrecs; recID++)
                 {
-                  int varID = recList[recID].varID;
-                  int levelID = recList[recID].levelID;
+                  const auto varID = recList[recID].varID;
+                  const auto levelID = recList[recID].levelID;
 
-                  Field &rvars1 = vars1[varID][levelID];
-                  Field &rinput_var = input_vars[curFirst][varID][levelID];
+                  auto &rvars1 = vars1[varID][levelID];
+                  auto &rinput_var = input_vars[curFirst][varID][levelID];
 
-                  size_t fieldsize = rvars1.size;
-                  size_t nmiss = rinput_var.nmiss;
+                  const auto nmiss = rinput_var.nmiss;
 
                   vfarcpy(rvars1, rinput_var);
                   rvars1.nmiss = nmiss;
                   if (nmiss || !samp1[varID][levelID].empty())
                     {
-                      if (samp1[varID][levelID].empty())
-                        samp1[varID][levelID].resize(fieldsize);
+                      const auto fieldsize = rvars1.size;
+                      if (samp1[varID][levelID].empty()) samp1[varID][levelID].resize(fieldsize);
 
                       for (size_t i = 0; i < fieldsize; i++)
-                        samp1[varID][levelID].vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                        samp1[varID][levelID].vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                     }
                 }
             }
@@ -390,25 +417,25 @@ XTimstat(void *process)
             {
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  shared(lvarstd, nsets, maxrecs, recList, curFirst, input_vars, vars1, samp1, vars2, operfunc) if (maxrecs > 1)
+    shared(lvarstd, nsets, maxrecs, recList, curFirst, input_vars, vars1, samp1, vars2, operfunc) if (maxrecs > 1)
 #endif
               for (int recID = 0; recID < maxrecs; recID++)
                 {
-                  int varID = recList[recID].varID;
-                  int levelID = recList[recID].levelID;
+                  const auto varID = recList[recID].varID;
+                  const auto levelID = recList[recID].levelID;
 
-                  Field &rvars1 = vars1[varID][levelID];
-                  Field &rinput_var = input_vars[curFirst][varID][levelID];
+                  auto &rvars1 = vars1[varID][levelID];
+                  auto &rinput_var = input_vars[curFirst][varID][levelID];
 
-                  size_t fieldsize = rvars1.size;
-                  size_t nmiss = rinput_var.nmiss;
+                  const auto nmiss = rinput_var.nmiss;
 
                   if (nmiss || !samp1[varID][levelID].empty())
                     {
+                      const auto fieldsize = rvars1.size;
                       if (samp1[varID][levelID].empty()) samp1[varID][levelID].resize(fieldsize, nsets);
 
                       for (size_t i = 0; i < fieldsize; i++)
-                        if (!DBL_IS_EQUAL(rinput_var.vec[i], rvars1.missval)) samp1[varID][levelID].vec[i]++;
+                        if (!DBL_IS_EQUAL(rinput_var.vec_d[i], rvars1.missval)) samp1[varID][levelID].vec_d[i]++;
                     }
 
                   if (lvarstd)
@@ -426,8 +453,8 @@ XTimstat(void *process)
           if (nsets == 0 && lvarstd)
             for (int recID = 0; recID < maxrecs; recID++)
               {
-                int varID = recList[recID].varID;
-                int levelID = recList[recID].levelID;
+                const auto varID = recList[recID].varID;
+                const auto levelID = recList[recID].levelID;
 
                 if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
@@ -449,8 +476,8 @@ XTimstat(void *process)
 #endif
           for (int recID = 0; recID < maxrecs; recID++)
             {
-              int varID = recList[recID].varID;
-              int levelID = recList[recID].levelID;
+              const auto varID = recList[recID].varID;
+              const auto levelID = recList[recID].levelID;
 
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
@@ -464,14 +491,14 @@ XTimstat(void *process)
         {
           for (int recID = 0; recID < maxrecs; recID++)
             {
-              int varID = recList[recID].varID;
-              int levelID = recList[recID].levelID;
+              const auto varID = recList[recID].varID;
+              const auto levelID = recList[recID].levelID;
 
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-              Field &rsamp1 = samp1[varID][levelID];
-              Field &rvars1 = vars1[varID][levelID];
-              Field &rvars2 = vars2[varID][levelID];
+              auto &rsamp1 = samp1[varID][levelID];
+              auto &rvars1 = vars1[varID][levelID];
+              auto &rvars2 = vars2[varID][levelID];
 
               if (!rsamp1.empty())
                 {
@@ -491,32 +518,27 @@ XTimstat(void *process)
         }
 
       if (Options::cdoVerbose)
-        {
-          char vdatestr[32], vtimestr[32];
-          date2str(vdate0, vdatestr, sizeof(vdatestr));
-          time2str(vtime0, vtimestr, sizeof(vtimestr));
-          cdoPrint("%s %s  vfrac = %g, nsets = %d", vdatestr, vtimestr, vfrac, nsets);
-        }
+        cdoPrint("%s %s  vfrac = %g, nsets = %d", dateToString(vdate0).c_str(), timeToString(vtime0).c_str(), vfrac, nsets);
 
       if (lvfrac && operfunc == func_mean)
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            int varID = recList[recID].varID;
-            int levelID = recList[recID].levelID;
-            Field &rvars1 = vars1[varID][levelID];
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
+            auto &rvars1 = vars1[varID][levelID];
 
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-            size_t fieldsize = rvars1.size;
-            double missval = rvars1.missval;
+            const auto missval = rvars1.missval;
             if (!samp1[varID][levelID].empty())
               {
+                const auto fieldsize = rvars1.size;
                 size_t irun = 0;
                 for (size_t i = 0; i < fieldsize; ++i)
                   {
-                    if ((samp1[varID][levelID].vec[i] / nsets) < vfrac)
+                    if ((samp1[varID][levelID].vec_d[i] / nsets) < vfrac)
                       {
-                        rvars1.vec[i] = missval;
+                        rvars1.vec_d[i] = missval;
                         irun++;
                       }
                   }
@@ -536,21 +558,21 @@ XTimstat(void *process)
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          int varID = recList[recID].varID;
-          int levelID = recList[recID].levelID;
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rvars1 = vars1[varID][levelID];
 
           if (otsID && vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
 
           if (Options::cdoDiag)
             {
               if (!samp1[varID][levelID].empty())
                 {
                   cdoDefRecord(streamID3, varID, levelID);
-                  cdoWriteRecord(streamID3, samp1[varID][levelID].vec.data(), 0);
+                  cdoWriteRecord(streamID3, samp1[varID][levelID].vec_d.data(), 0);
                 }
             }
         }
@@ -561,7 +583,13 @@ XTimstat(void *process)
 
   if (Options::cdoDiag) cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
+#ifdef USE_CDI_STREAM
+  streamClose(streamID1);
+#else
   cdoStreamClose(streamID1);
+#endif
+
+  if (read_task) delete read_task;
 
   cdoFinish();
 
