@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,24 +26,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
-#define MAX_DOY 373
-
-static int
-getDayOfYearIndex(int64_t vdate)
-{
-  int year, month, day;
-  cdiDecodeDate(vdate, &year, &month, &day);
-  const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
-  if (dayoy < 0 || dayoy >= MAX_DOY) cdoAbort("Day of year %d out of range (date=%d)!", dayoy, vdate);
-  return dayoy;
-}
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 
 void *
 Ydayarith(void *process)
 {
-  int nrecs;
+  constexpr int MaxDays = 373;
   int varID, levelID;
 
   cdoInitialize(process);
@@ -53,42 +44,48 @@ Ydayarith(void *process)
   cdoOperatorAdd("ydaymul", func_mul, 0, nullptr);
   cdoOperatorAdd("ydaydiv", func_div, 0, nullptr);
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
   VarList varList1;
   varListInit(varList1, vlistID1);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
-  Field field1, field2;
-  field1.resize(gridsizemax);
-  field2.resize(gridsizemax);
+  Field field;
+  field.resize(gridsizemax);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  FieldVector2D vars2[MAX_DOY];
+  FieldVector2D vars2[MaxDays];
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
+  while (true)
     {
-      const int dayoy = getDayOfYearIndex(taxisInqVdate(taxisID2));
-      if (vars2[dayoy].size() > 0) cdoAbort("Day of year index %d already allocated (date=%d)!", dayoy);
+      const auto nrecs = cdoStreamInqTimestep(streamID2, tsID);
+      if (nrecs == 0) break;
+
+      const auto vdate = taxisInqVdate(taxisID2);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day of year %d out of range (date=%d)!", dayoy, vdate);
+      if (vars2[dayoy].size() > 0) cdoAbort("Day of year index %d already allocated (date=%d)!", dayoy, vdate);
 
       fieldsFromVlist(vlistID2, vars2[dayoy], FIELD_VEC);
 
@@ -96,7 +93,7 @@ Ydayarith(void *process)
         {
           size_t nmiss;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars2[dayoy][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars2[dayoy][varID][levelID].vec_d.data(), &nmiss);
           vars2[dayoy][varID][levelID].nmiss = nmiss;
         }
 
@@ -104,10 +101,15 @@ Ydayarith(void *process)
     }
 
   tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int dayoy = getDayOfYearIndex(taxisInqVdate(taxisID1));
-      if (vars2[dayoy].size() == 0) cdoAbort("Day of year index %d not found (date=%d)!", dayoy);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day of year %d out of range (date=%d)!", dayoy, vdate);
+      if (vars2[dayoy].size() == 0) cdoAbort("Day of year index %d not found (date=%d)!", dayoy, vdate);
 
       taxisCopyTimestep(taxisID3, taxisID1);
       cdoDefTimestep(streamID3, tsID);
@@ -115,14 +117,15 @@ Ydayarith(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
-          field1.grid = varList1[varID].gridID;
-          field1.missval = varList1[varID].missval;
+          cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
-          vfarfun(field1, vars2[dayoy][varID][levelID], operfunc);
+          vfarfun(field, vars2[dayoy][varID][levelID], operfunc);
 
           cdoDefRecord(streamID3, varID, levelID);
-          cdoWriteRecord(streamID3, field1.vec.data(), field1.nmiss);
+          cdoWriteRecord(streamID3, field.vec_d.data(), field.nmiss);
         }
       tsID++;
     }
