@@ -23,146 +23,135 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "percentiles_hist.h"
-#include "percentiles.h"
-
-#define NDAY 373
-
-int getmonthday(int64_t date);
 
 void *
 Ydaypctl(void *process)
 {
-  int varID;
-  int gridID;
-  int year, month, day;
-  int nrecs;
-  int levelID;
+  constexpr int MaxDays = 373;
   size_t nmiss;
-  int nlevels;
-  int64_t vdates1[NDAY], vdates2[NDAY];
-  int vtimes1[NDAY];
-  long nsets[NDAY];
-  FieldVector2D vars1[NDAY];
-  HISTOGRAM_SET *hsets[NDAY];
+  int64_t vdates1[MaxDays] = { 0 }, vdates2[MaxDays] = { 0 };
+  int vtimes1[MaxDays] = { 0 };
+  long nsets[MaxDays] = { 0 };
+  FieldVector2D vars1[MaxDays];
+  HistogramSet hsets[MaxDays];
 
   cdoInitialize(process);
   cdoOperatorAdd("ydaypctl", func_pctl, 0, nullptr);
 
   operatorInputArg("percentile number");
-  const double pn = parameter2double(operatorArgv()[0]);
-  percentile_check_number(pn);
+  const auto pn = parameter2double(cdoOperatorArgv(0));
 
-  for (int dayoy = 0; dayoy < NDAY; dayoy++)
-    {
-      hsets[dayoy] = nullptr;
-      nsets[dayoy] = 0;
-    }
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto streamID3 = cdoOpenRead(2);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
-  CdoStreamID streamID3 = cdoOpenRead(2);
-
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = cdoStreamInqVlist(streamID3);
-  const int vlistID4 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = cdoStreamInqVlist(streamID3);
+  const auto vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = vlistInqTaxis(vlistID3);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  const int taxisID4 = taxisDuplicate(taxisID1);
+  const auto taxisID4 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID4)) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  CdoStreamID streamID4 = cdoOpenWrite(3);
+  const auto streamID4 = cdoOpenWrite(3);
   cdoDefVlist(streamID4, vlistID4);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID2, tsID);
+      if (nrecs == 0) break;
+
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
         cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      const int64_t vdate = taxisInqVdate(taxisID2);
-      const int vtime = taxisInqVtime(taxisID2);
+      const auto vdate = taxisInqVdate(taxisID2);
+      const auto vtime = taxisInqVtime(taxisID2);
 
       if (vdate != taxisInqVdate(taxisID3))
         cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-
-      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
-      if (dayoy < 0 || dayoy >= NDAY) cdoAbort("Day %d out of range!", dayoy);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day %d out of range!", dayoy);
 
       vdates2[dayoy] = vdate;
 
       if (!vars1[dayoy].size())
         {
           fieldsFromVlist(vlistID1, vars1[dayoy], FIELD_VEC);
-          hsets[dayoy] = hsetCreate(nvars);
+          hsets[dayoy].create(nvars);
 
-          for (varID = 0; varID < nvars; varID++)
-            {
-              gridID = vlistInqVarGrid(vlistID1, varID);
-              nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-
-              hsetCreateVarLevels(hsets[dayoy], varID, nlevels, gridID);
-            }
+          for (int varID = 0; varID < nvars; varID++)
+            hsets[dayoy].createVarLevels(varID, varList1[varID].nlevels, varList1[varID].gridsize);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars1[dayoy][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars1[dayoy][varID][levelID].vec_d.data(), &nmiss);
           vars1[dayoy][varID][levelID].nmiss = nmiss;
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID3, &varID, &levelID);
-          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
-          field.grid = vars1[dayoy][varID][levelID].grid;
-          field.missval = vars1[dayoy][varID][levelID].missval;
+          cdoReadRecord(streamID3, field.vec_d.data(), &field.nmiss);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
-          hsetDefVarLevelBounds(hsets[dayoy], varID, levelID, &vars1[dayoy][varID][levelID], &field);
+          hsets[dayoy].defVarLevelBounds(varID, levelID, vars1[dayoy][varID][levelID], field);
         }
 
       tsID++;
     }
 
   tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-
-      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
-      if (dayoy < 0 || dayoy >= NDAY) cdoAbort("Day %d out of range!", dayoy);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day %d out of range!", dayoy);
 
       vdates1[dayoy] = vdate;
       vtimes1[dayoy] = vtime;
@@ -171,19 +160,20 @@ Ydaypctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
               recList[recID].varID = varID;
               recList[recID].levelID = levelID;
-              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
             }
 
-          cdoReadRecord(streamID1, vars1[dayoy][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, vars1[dayoy][varID][levelID].vec_d.data(), &nmiss);
           vars1[dayoy][varID][levelID].nmiss = nmiss;
 
-          hsetAddVarLevelValues(hsets[dayoy], varID, levelID, &vars1[dayoy][varID][levelID]);
+          hsets[dayoy].addSubVarLevelValues(varID, levelID, vars1[dayoy][varID][levelID], 1, FIELD_VEC);
         }
 
       nsets[dayoy]++;
@@ -191,20 +181,20 @@ Ydaypctl(void *process)
     }
 
   int otsID = 0;
-  for (int dayoy = 0; dayoy < NDAY; dayoy++)
+  for (int dayoy = 0; dayoy < MaxDays; dayoy++)
     if (nsets[dayoy])
       {
-        if (getmonthday(vdates1[dayoy]) != getmonthday(vdates2[dayoy]))
+        if (decodeMonthAndDay(vdates1[dayoy]) != decodeMonthAndDay(vdates2[dayoy]))
           cdoAbort("Verification dates for the day %d of %s and %s are different!", dayoy, cdoGetStreamName(0),
                    cdoGetStreamName(1));
 
-        for (varID = 0; varID < nvars; varID++)
+        for (int varID = 0; varID < nvars; varID++)
           {
-            if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-            nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+            if (varList1[varID].timetype == TIME_CONSTANT) continue;
 
-            for (levelID = 0; levelID < nlevels; levelID++)
-              hsetGetVarLevelPercentiles(&vars1[dayoy][varID][levelID], hsets[dayoy], varID, levelID, pn);
+            const auto nlevels = varList1[varID].nlevels;
+            for (int levelID = 0; levelID < nlevels; levelID++)
+              hsets[dayoy].getVarLevelPercentiles(vars1[dayoy][varID][levelID], varID, levelID, pn, FIELD_VEC);
           }
 
         taxisDefVdate(taxisID4, vdates1[dayoy]);
@@ -215,23 +205,15 @@ Ydaypctl(void *process)
           {
             if (otsID && recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
             cdoDefRecord(streamID4, varID, levelID);
-            cdoWriteRecord(streamID4, vars1[dayoy][varID][levelID].vec.data(), vars1[dayoy][varID][levelID].nmiss);
+            cdoWriteRecord(streamID4, vars1[dayoy][varID][levelID].vec_d.data(), vars1[dayoy][varID][levelID].nmiss);
           }
 
         otsID++;
       }
 
-  for (int dayoy = 0; dayoy < NDAY; dayoy++)
-    {
-      if (vars1[dayoy].size())
-        {
-          hsetDestroy(hsets[dayoy]);
-        }
-    }
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
