@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,22 +32,23 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "pmlist.h"
 
-#define MAX_DOY 373
-
 int yearMode = 0;
 
 static void
-set_parameter(void)
+setParameter(void)
 {
-  const int pargc = operatorArgc();
+  const auto pargc = operatorArgc();
   if (pargc)
     {
-      const char **pargv = (const char **) operatorArgv();
+      auto pargv = cdoGetOperArgv();
 
       KVList kvlist;
       kvlist.name = "YDAYSTAT";
@@ -56,34 +57,22 @@ set_parameter(void)
 
       for (const auto &kv : kvlist)
         {
-          const char *key = kv.key.c_str();
-          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv.values[0].c_str();
+          const auto &key = kv.key;
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          const auto &value = kv.values[0];
 
-          if (cstrIsEqual(key, "yearMode"))
+          if (key == "yearMode")
             yearMode = parameter2int(value);
           else
-            cdoAbort("Invalid parameter key >%s<!", key);
+            cdoAbort("Invalid parameter key >%s<!", key.c_str());
         }
     }
 }
 
-void *
-Ydaystat(void *process)
+static void
+addOperators(void)
 {
-  int varID, levelID;
-  int year, month, day;
-  int nrecs;
-  int dayoy_nsets[MAX_DOY];
-  int64_t vdates[MAX_DOY];
-  int vtimes[MAX_DOY];
-  FieldVector2D vars1[MAX_DOY], vars2[MAX_DOY], samp1[MAX_DOY];
-
-  cdoInitialize(process);
-
-  set_parameter();
-
   // clang-format off
   cdoOperatorAdd("ydayrange", func_range, 0, nullptr);
   cdoOperatorAdd("ydaymin",   func_min,   0, nullptr);
@@ -95,101 +84,119 @@ Ydaystat(void *process)
   cdoOperatorAdd("ydayvar1",  func_var1,  0, nullptr);
   cdoOperatorAdd("ydaystd",   func_std,   0, nullptr);
   cdoOperatorAdd("ydaystd1",  func_std1,  0, nullptr);
-
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
-  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
   // clang-format on
+}
+
+void *
+Ydaystat(void *process)
+{
+  constexpr int MaxDays = 373;
+  int dayoy_nsets[MaxDays] = { 0 };
+  int64_t vdates[MaxDays] = { 0 };
+  int vtimes[MaxDays] = { 0 };
+  FieldVector2D vars1[MaxDays], vars2[MaxDays], samp1[MaxDays];
+
+  cdoInitialize(process);
+
+  setParameter();
+
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
   const bool lvars2 = lvarstd || lrange;
 
-  for (int dayoy = 0; dayoy < MAX_DOY; ++dayoy) dayoy_nsets[dayoy] = 0;
+  const auto streamID1 = cdoOpenRead(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
-
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID2)) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   int tsID = 0;
   int otsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-
-      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
-      if (dayoy < 0 || dayoy >= MAX_DOY) cdoAbort("Day of year %d out of range (date=%d)!", dayoy, vdate);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day of year %d out of range (date=%d)!", dayoy, vdate);
 
       vdates[dayoy] = vdate;
       vtimes[dayoy] = vtime;
 
       if (!vars1[dayoy].size())
         {
-          fieldsFromVlist(vlistID1, samp1[dayoy], FIELD_NONE);
+          fieldsFromVlist(vlistID1, samp1[dayoy]);
           fieldsFromVlist(vlistID1, vars1[dayoy], FIELD_VEC);
           if (lvars2) fieldsFromVlist(vlistID1, vars2[dayoy], FIELD_VEC);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
               recList[recID].varID = varID;
               recList[recID].levelID = levelID;
-              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
             }
 
-          Field &rsamp1 = samp1[dayoy][varID][levelID];
-          Field &rvars1 = vars1[dayoy][varID][levelID];
+          auto &rsamp1 = samp1[dayoy][varID][levelID];
+          auto &rvars1 = vars1[dayoy][varID][levelID];
 
-          const int nsets = dayoy_nsets[dayoy];
-          const size_t fieldsize = rvars1.size;
+          const auto nsets = dayoy_nsets[dayoy];
+          const auto fieldsize = rvars1.size;
 
           if (nsets == 0)
             {
-              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+              cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
               if (lrange)
                 {
                   vars2[dayoy][varID][levelID].nmiss = rvars1.nmiss;
-                  vars2[dayoy][varID][levelID].vec = rvars1.vec;
+                  vars2[dayoy][varID][levelID].vec_d = rvars1.vec_d;
                 }
 
               if (rvars1.nmiss || !rsamp1.empty())
                 {
                   if (rsamp1.empty()) rsamp1.resize(fieldsize);
-                  for (size_t i = 0; i < fieldsize; i++)
-                    rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                  for (size_t i = 0; i < fieldsize; i++) rsamp1.vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                 }
             }
           else
             {
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
               field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
@@ -199,7 +206,7 @@ Ydaystat(void *process)
                   if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
                   for (size_t i = 0; i < fieldsize; i++)
-                    if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
+                    if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i]++;
                 }
 
               if (lvarstd)
@@ -224,8 +231,8 @@ Ydaystat(void *process)
           {
             if (recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
             vfarmoq(vars2[dayoy][varID][levelID], vars1[dayoy][varID][levelID]);
           }
 
@@ -236,35 +243,35 @@ Ydaystat(void *process)
   // set the year to the minimum of years found on output timestep
   if (yearMode)
     {
+      int year, month, day;
       int outyear = 1e9;
-      for (int dayoy = 0; dayoy < MAX_DOY; dayoy++)
+      for (int dayoy = 0; dayoy < MaxDays; dayoy++)
         if (dayoy_nsets[dayoy])
           {
             cdiDecodeDate(vdates[dayoy], &year, &month, &day);
             if (year < outyear) outyear = year;
           }
-      for (int dayoy = 0; dayoy < MAX_DOY; dayoy++)
+      for (int dayoy = 0; dayoy < MaxDays; dayoy++)
         if (dayoy_nsets[dayoy])
           {
             cdiDecodeDate(vdates[dayoy], &year, &month, &day);
             if (year > outyear) vdates[dayoy] = cdiEncodeDate(outyear, month, day);
-            //  printf("vdates[%d] = %d  nsets = %d\n", dayoy, vdates[dayoy],
-            //  nsets[dayoy]);
+            //  printf("vdates[%d] = %d  nsets = %d\n", dayoy, vdates[dayoy], nsets[dayoy]);
           }
     }
 
-  for (int dayoy = 0; dayoy < MAX_DOY; dayoy++)
+  for (int dayoy = 0; dayoy < MaxDays; dayoy++)
     if (dayoy_nsets[dayoy])
       {
-        const int nsets = dayoy_nsets[dayoy];
+        const auto nsets = dayoy_nsets[dayoy];
         for (int recID = 0; recID < maxrecs; recID++)
           {
             if (recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
-            Field &rsamp1 = samp1[dayoy][varID][levelID];
-            Field &rvars1 = vars1[dayoy][varID][levelID];
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
+            auto &rsamp1 = samp1[dayoy][varID][levelID];
+            auto &rvars1 = vars1[dayoy][varID][levelID];
 
             if (lmean)
               {
@@ -275,7 +282,7 @@ Ydaystat(void *process)
               }
             else if (lvarstd)
               {
-                Field &rvars2 = vars2[dayoy][varID][levelID];
+                auto &rvars2 = vars2[dayoy][varID][levelID];
                 if (!rsamp1.empty())
                   {
                     if (lstd)
@@ -293,7 +300,7 @@ Ydaystat(void *process)
               }
             else if (lrange)
               {
-                Field &rvars2 = vars2[dayoy][varID][levelID];
+                auto &rvars2 = vars2[dayoy][varID][levelID];
                 vfarsub(rvars1, rvars2);
               }
           }
@@ -306,12 +313,12 @@ Ydaystat(void *process)
           {
             if (otsID && recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
-            Field &rvars1 = vars1[dayoy][varID][levelID];
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
+            auto &rvars1 = vars1[dayoy][varID][levelID];
 
             cdoDefRecord(streamID2, varID, levelID);
-            cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+            cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
           }
 
         otsID++;
