@@ -32,33 +32,44 @@
 #include "calendar.h"
 #include "percentiles_hist.h"
 #include "percentiles.h"
-
+#include "pmlist.h"
 
 void *
 Ydrunpctl(void *process)
 {
   constexpr int MaxDays = 373;
-  int varID;
-  int nrecs;
-  int levelID;
-  int inp, its;
+  int its;
   size_t nmiss;
-  int64_t vdates1[MaxDays], vdates2[MaxDays];
-  int vtimes1[MaxDays] /*, vtimes2[MaxDays]*/;
-  int nsets[MaxDays];
+  int64_t vdates1[MaxDays] = { 0 }, vdates2[MaxDays] = { 0 };
+  int vtimes1[MaxDays] = { 0 } /*, vtimes2[MaxDays]*/;
+  int nsets[MaxDays] = { 0 };
   FieldVector2D vars2[MaxDays];
   HistogramSet hsets[MaxDays];
 
   cdoInitialize(process);
   cdoOperatorAdd("ydrunpctl", func_pctl, 0, nullptr);
 
-  operatorInputArg("percentile number, number of timesteps");
-  operatorCheckArgc(2);
   const auto pn = parameter2double(cdoOperatorArgv(0));
-  percentile_check_number(pn);
   const auto ndates = parameter2int(cdoOperatorArgv(1));
 
-  for (int dayoy = 0; dayoy < MaxDays; dayoy++) nsets[dayoy] = 0;
+  char *percMethod = nullptr, *readMethod = nullptr;
+  if (operatorArgc() > 2)
+    {
+      auto params = cdoGetOperArgv();
+      params = std::vector<std::string>(params.begin() + 2, params.end());
+      KVList kvlist;
+      if (kvlist.parseArguments(operatorArgc()-2, params) != 0) cdoAbort("Argument parse error!"); 
+      auto kv = kvlist.search("pm");
+      if (kv && kv->nvalues > 0) percMethod = strdup(kv->values[0].c_str());
+      kv = kvlist.search("rm");
+      if (kv && kv->nvalues > 0) readMethod = strdup(kv->values[0].c_str());
+    }
+
+  if (percMethod)
+    {
+      if (strcmp(percMethod,"r8") == 0) percentile_set_method("rtype8");
+      free(percMethod);
+    }
 
   const auto streamID1 = cdoOpenRead(0);
   const auto streamID2 = cdoOpenRead(1);
@@ -72,6 +83,10 @@ Ydrunpctl(void *process)
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
   const auto taxisID1 = vlistInqTaxis(vlistID1);
   const auto taxisID2 = vlistInqTaxis(vlistID2);
   const auto taxisID3 = vlistInqTaxis(vlistID3);
@@ -102,8 +117,12 @@ Ydrunpctl(void *process)
   for (its = 0; its < ndates; its++) fieldsFromVlist(vlistID1, vars1[its], FIELD_VEC);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
+  int startYear = 0, endYear = 0, mon = 0, day = 0;
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID2, tsID);
+      if (nrecs == 0) break;
+
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
         cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
@@ -126,27 +145,26 @@ Ydrunpctl(void *process)
           fieldsFromVlist(vlistID2, vars2[dayoy], FIELD_VEC);
           hsets[dayoy].create(nvars);
 
-          for (varID = 0; varID < nvars; varID++)
-            {
-              auto gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-              auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-              hsets[dayoy].createVarLevels(varID, nlevels, gridsize);
-            }
+          for (int varID = 0; varID < nvars; varID++)
+            hsets[dayoy].createVarLevels(varID, varList2[varID].nlevels, varList2[varID].gridsize);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars2[dayoy][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars2[dayoy][varID][levelID].vec_d.data(), &nmiss);
           vars2[dayoy][varID][levelID].nmiss = nmiss;
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID3, &varID, &levelID);
-          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
-          field.grid = vars2[dayoy][varID][levelID].grid;
-          field.missval = vars2[dayoy][varID][levelID].missval;
+          cdoReadRecord(streamID3, field.vec_d.data(), &field.nmiss);
+          field.size = varList2[varID].gridsize;
+          field.grid = varList2[varID].gridID;
+          field.missval = varList2[varID].missval;
 
           hsets[dayoy].defVarLevelBounds(varID, levelID, vars2[dayoy][varID][levelID], field);
         }
@@ -156,7 +174,7 @@ Ydrunpctl(void *process)
 
   for (tsID = 0; tsID < ndates; tsID++)
     {
-      nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
       if (nrecs == 0) cdoAbort("File has less then %d timesteps!", ndates);
 
       datetime[tsID].date = taxisInqVdate(taxisID1);
@@ -164,16 +182,17 @@ Ydrunpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
               recList[recID].varID = varID;
               recList[recID].levelID = levelID;
-              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
             }
 
-          cdoReadRecord(streamID1, vars1[tsID][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, vars1[tsID][varID][levelID].vec_d.data(), &nmiss);
           vars1[tsID][varID][levelID].nmiss = nmiss;
         }
     }
@@ -193,25 +212,25 @@ Ydrunpctl(void *process)
 
       if (!vars2[dayoy].size()) cdoAbort("No data for day %d in %s and %s", dayoy, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      for (varID = 0; varID < nvars; varID++)
+      for (int varID = 0; varID < nvars; varID++)
         {
           if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-          const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevels; levelID++)
-            for (inp = 0; inp < ndates; inp++) hsets[dayoy].addVarLevelValues(varID, levelID, vars1[inp][varID][levelID]);
+          const auto nlevels = varList1[varID].nlevels;
+          for (int levelID = 0; levelID < nlevels; levelID++)
+            for (int inp = 0; inp < ndates; inp++) hsets[dayoy].addSubVarLevelValues(varID, levelID, vars1[inp][varID][levelID], 1, FIELD_VEC);
         }
 
       datetime[ndates] = datetime[0];
       vars1[ndates] = vars1[0];
 
-      for (inp = 0; inp < ndates; inp++)
+      for (int inp = 0; inp < ndates; inp++)
         {
           datetime[inp] = datetime[inp + 1];
           vars1[inp] = vars1[inp + 1];
         }
 
-      nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       datetime[ndates - 1].date = taxisInqVdate(taxisID1);
@@ -219,15 +238,78 @@ Ydrunpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
-          cdoReadRecord(streamID1, vars1[ndates - 1][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, vars1[ndates - 1][varID][levelID].vec_d.data(), &nmiss);
           vars1[ndates - 1][varID][levelID].nmiss = nmiss;
         }
 
       nsets[dayoy] += ndates;
       tsID++;
     }
+
+  if (readMethod && readMethod[0] == 'c' && cdoAssertFilesOnly() )
+    {
+      cdiDecodeDate(datetime[ndates - 1].date, &endYear, &mon, &day);
+      const auto cdiStream = streamOpenRead(cdoGetStreamName(0));
+      const auto cdiVlistID = streamInqVlist(cdiStream);
+      const auto cdiTaxisID = vlistInqTaxis(cdiVlistID);
+      int missTimes = 0;
+      for ( missTimes = 0; missTimes < ndates-1; missTimes++ )
+        {
+	  const auto nrecs = streamInqTimestep(cdiStream, missTimes);
+          if (nrecs == 0) break;
+
+          int year;
+          cdiDecodeDate(taxisInqVdate(cdiTaxisID), &year, &mon, &day);
+          datetime[ndates - 1].date = cdiEncodeDate(endYear+1, mon, day);
+          datetime[ndates - 1].time = taxisInqVtime(cdiTaxisID);
+
+          for (int recID = 0; recID < nrecs; recID++)
+            {
+              int varID, levelID;
+              streamInqRecord(cdiStream, &varID, &levelID);
+              streamReadRecord(cdiStream, vars1[ndates - 1][varID][levelID].vec_d.data(), &nmiss);
+              vars1[ndates - 1][varID][levelID].nmiss = nmiss;
+            }
+
+          datetime_avg(dpy, ndates, datetime.data());
+          cdiDecodeDate(datetime[ndates].date, &year, &mon, &day);
+          const auto vdate = (year > endYear) ? cdiEncodeDate(startYear, mon, day) : datetime[ndates].date;
+
+          const auto dayoy = decodeDayOfYear(vdate);
+          if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day %d out of range!", dayoy);
+
+          nsets[dayoy] += ndates;
+
+          for (int varID = 0; varID < nvars; varID++)
+            {
+              if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
+
+              const auto nlevels = varList1[varID].nlevels;
+              for (int levelID = 0; levelID < nlevels; levelID++)
+                for (int inp = 0; inp < ndates; inp++) hsets[dayoy].addSubVarLevelValues(varID, levelID, vars1[inp][varID][levelID], 1, FIELD_VEC);
+            }
+
+          datetime[ndates] = datetime[0];
+          vars1[ndates] = vars1[0];
+
+          for (int inp = 0; inp < ndates; inp++)
+            {
+              datetime[inp] = datetime[inp + 1];
+              vars1[inp] = vars1[inp + 1];
+	    }
+        }
+      if ( missTimes != ndates-1 )
+        cdoAbort("Addding the missing values when using the 'readMethodar' method was not possible");
+      streamClose(cdiStream);    
+    }
+  else if (readMethod && readMethod[0] == 'c' )
+    cdoWarning("Operators cannot be piped in circular mode");
+
+  if (readMethod) free(readMethod);
+
   /*
   int outyear = 1e9;
   for ( dayoy = 0; dayoy < MaxDays; dayoy++ )
@@ -254,13 +336,13 @@ Ydrunpctl(void *process)
           cdoAbort("Verification dates for day %d of %s, %s and %s are different!", dayoy, cdoGetStreamName(0),
                    cdoGetStreamName(1));
 
-        for (varID = 0; varID < nvars; varID++)
+        for (int varID = 0; varID < nvars; varID++)
           {
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-            const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-            for (levelID = 0; levelID < nlevels; levelID++)
-              hsets[dayoy].getVarLevelPercentiles(vars2[dayoy][varID][levelID], varID, levelID, pn);
+            const auto nlevels = varList1[varID].nlevels;
+            for (int levelID = 0; levelID < nlevels; levelID++)
+              hsets[dayoy].getVarLevelPercentiles(vars2[dayoy][varID][levelID], varID, levelID, pn, FIELD_VEC);
           }
 
         taxisDefVdate(taxisID4, vdates1[dayoy]);
@@ -274,7 +356,7 @@ Ydrunpctl(void *process)
             const auto varID = recList[recID].varID;
             const auto levelID = recList[recID].levelID;
             cdoDefRecord(streamID4, varID, levelID);
-            cdoWriteRecord(streamID4, vars2[dayoy][varID][levelID].vec.data(), vars2[dayoy][varID][levelID].nmiss);
+            cdoWriteRecord(streamID4, vars2[dayoy][varID][levelID].vec_d.data(), vars2[dayoy][varID][levelID].nmiss);
           }
 
         otsID++;
