@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "functs.h"
+#include "process_int.h"
 #include "calendar.h"
 #include "datetime.h"
 #include "printinfo.h"
@@ -35,11 +38,8 @@ Yearmonstat(void *process)
   TimeStat timestat_date = TimeStat::MEAN;
   int64_t vdate0 = 0;
   int vtime0 = 0;
-  int nrecs;
-  int varID, levelID;
   int year0 = 0, month0 = 0;
   int year, month, day;
-  char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
 
@@ -48,52 +48,61 @@ Yearmonstat(void *process)
   cdoOperatorAdd("yearmonavg",   func_avg,  0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const int calendar = taxisInqCalendar(taxisID1);
+  const auto calendar = taxisInqCalendar(taxisID1);
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(calendar);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   FieldVector2D samp1, vars1;
-  fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
+  fieldsFromVlist(vlistID1, samp1);
   fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
 
   int tsID = 0;
   int otsID = 0;
   while (true)
     {
+      int nrecs = 0;
       long nsets = 0;
-      double dsets = 0;
-      while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+      double dsets = 0.0;
+      while (true)
         {
+          nrecs = cdoStreamInqTimestep(streamID1, tsID);
+          if (nrecs == 0) break;
+
           dtlist.taxisInqTimestep(taxisID1, nsets);
-          const int64_t vdate = dtlist.getVdate(nsets);
-          const int vtime = dtlist.getVtime(nsets);
+          const auto vdate = dtlist.getVdate(nsets);
+          const auto vtime = dtlist.getVtime(nsets);
           cdiDecodeDate(vdate, &year, &month, &day);
 
           if (nsets == 0) year0 = year;
@@ -102,36 +111,33 @@ Yearmonstat(void *process)
 
           if (nsets > 0 && month == month0)
             {
-              date2str(vdate0, vdatestr, sizeof(vdatestr));
-              time2str(vtime0, vtimestr, sizeof(vtimestr));
-              cdoWarning("   last timestep: %s %s", vdatestr, vtimestr);
-              date2str(vdate, vdatestr, sizeof(vdatestr));
-              time2str(vtime, vtimestr, sizeof(vtimestr));
-              cdoWarning("current timestep: %s %s", vdatestr, vtimestr);
+              cdoWarning("   last timestep: %s %s", dateToString(vdate0).c_str(), timeToString(vtime0).c_str());
+              cdoWarning("current timestep: %s %s", dateToString(vdate).c_str(), timeToString(vtime).c_str());
               cdoAbort("Month does not change!");
             }
 
-          const int dpm = days_per_month(calendar, year, month);
+          const auto dpm = days_per_month(calendar, year, month);
 
           for (int recID = 0; recID < nrecs; recID++)
             {
+              int varID, levelID;
               cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
                 {
                   recList[recID].varID = varID;
                   recList[recID].levelID = levelID;
-                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
                 }
 
-              Field &rsamp1 = samp1[varID][levelID];
-              Field &rvars1 = vars1[varID][levelID];
+              auto &rsamp1 = samp1[varID][levelID];
+              auto &rvars1 = vars1[varID][levelID];
 
-              const size_t fieldsize = rvars1.size;
+              const auto fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+                  cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
 
                   vfarcmul(rvars1, dpm);
 
@@ -140,15 +146,12 @@ Yearmonstat(void *process)
                       if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
                       for (size_t i = 0; i < fieldsize; i++)
-                        if (DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval))
-                          rsamp1.vec[i] = 0;
-                        else
-                          rsamp1.vec[i] = dpm;
+                        rsamp1.vec_d[i] = (DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval)) ? 0 : dpm;
                     }
                 }
               else
                 {
-                  cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                  cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
                   field.size = fieldsize;
                   field.grid = rvars1.grid;
                   field.missval = rvars1.missval;
@@ -160,7 +163,7 @@ Yearmonstat(void *process)
                       if (rsamp1.empty()) rsamp1.resize(fieldsize, dsets);
 
                       for (size_t i = 0; i < fieldsize; i++)
-                        if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i] += dpm;
+                        if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i] += dpm;
                     }
 
                   vfarfun(rvars1, field, operfunc);
@@ -177,11 +180,11 @@ Yearmonstat(void *process)
 
       if (nrecs == 0 && nsets == 0) break;
 
-      for (varID = 0; varID < nvars; varID++)
+      for (int varID = 0; varID < nvars; varID++)
         {
-          if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-          int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevels; levelID++)
+          if (varList1[varID].timetype == TIME_CONSTANT) continue;
+          const auto nlevels = varList1[varID].nlevels;
+          for (int levelID = 0; levelID < nlevels; levelID++)
             {
               if (samp1[varID][levelID].empty())
                 vfarcdiv(vars1[varID][levelID], dsets);
@@ -192,9 +195,7 @@ Yearmonstat(void *process)
 
       if (Options::cdoVerbose)
         {
-          date2str(vdate0, vdatestr, sizeof(vdatestr));
-          time2str(vtime0, vtimestr, sizeof(vtimestr));
-          cdoPrint("%s %s  nsets = %ld", vdatestr, vtimestr, nsets);
+          cdoPrint("%s %s  nsets = %ld", dateToString(vdate0).c_str(), timeToString(vtime0).c_str(), nsets);
         }
 
       dtlist.statTaxisDefTimestep(taxisID2, nsets);
@@ -204,11 +205,11 @@ Yearmonstat(void *process)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rvars1 = vars1[varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rvars1 = vars1[varID][levelID];
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
         }
 
       if (nrecs == 0) break;
