@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,32 +26,21 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 #include "printinfo.h"
 
-#define MAX_HOUR 9301 /* 31*12*25 + 1 */
+constexpr int MaxHours = 9301;  // 31*12*25 + 1
 
 static int
 getHourOfYearIndex(int64_t vdate, int vtime)
 {
-  int year, month, day, houroy;
-  int hour, minute, second;
+  const auto houroy = decodeHourOfYear(vdate, vtime);
 
-  cdiDecodeDate(vdate, &year, &month, &day);
-  cdiDecodeTime(vtime, &hour, &minute, &second);
-
-  if (month >= 1 && month <= 12 && day >= 1 && day <= 31 && hour >= 0 && hour < 24)
-    houroy = ((month - 1) * 31 + day - 1) * 25 + hour + 1;
-  else
-    houroy = 0;
-
-  if (houroy < 0 || houroy >= MAX_HOUR)
-    {
-      char vdatestr[32], vtimestr[32];
-      date2str(vdate, vdatestr, sizeof(vdatestr));
-      time2str(vtime, vtimestr, sizeof(vtimestr));
-      cdoAbort("Hour of year %d out of range (%s %s)!", houroy, vdatestr, vtimestr);
-    }
+  if (houroy < 0 || houroy >= MaxHours)
+    cdoAbort("Hour of year %d out of range (%s %s)!", houroy, dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
   return houroy;
 }
@@ -59,7 +48,6 @@ getHourOfYearIndex(int64_t vdate, int vtime)
 void *
 Yhourarith(void *process)
 {
-  int nrecs;
   int varID, levelID;
 
   cdoInitialize(process);
@@ -69,41 +57,45 @@ Yhourarith(void *process)
   cdoOperatorAdd("yhourmul", func_mul, 0, nullptr);
   cdoOperatorAdd("yhourdiv", func_div, 0, nullptr);
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
   VarList varList1;
   varListInit(varList1, vlistID1);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
-  Field field1, field2;
-  field1.resize(gridsizemax);
-  field2.resize(gridsizemax);
+  Field field;
+  field.resize(gridsizemax);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  FieldVector2D vars2[MAX_HOUR];
+  FieldVector2D vars2[MaxHours];
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
+  while (true)
     {
-      const int houroy = getHourOfYearIndex(taxisInqVdate(taxisID2), taxisInqVtime(taxisID2));
+      const auto nrecs = cdoStreamInqTimestep(streamID2, tsID);
+      if (nrecs == 0) break;
+
+      const auto houroy = getHourOfYearIndex(taxisInqVdate(taxisID2), taxisInqVtime(taxisID2));
       if (vars2[houroy].size() > 0) cdoAbort("Hour of year index %d already allocated!", houroy);
 
       fieldsFromVlist(vlistID2, vars2[houroy], FIELD_VEC);
@@ -112,7 +104,7 @@ Yhourarith(void *process)
         {
           size_t nmiss;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars2[houroy][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars2[houroy][varID][levelID].vec_d.data(), &nmiss);
           vars2[houroy][varID][levelID].nmiss = nmiss;
         }
 
@@ -120,9 +112,12 @@ Yhourarith(void *process)
     }
 
   tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int houroy = getHourOfYearIndex(taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      const auto houroy = getHourOfYearIndex(taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
       if (vars2[houroy].size() == 0) cdoAbort("Hour of year index %d not found!", houroy);
 
       taxisCopyTimestep(taxisID3, taxisID1);
@@ -131,14 +126,15 @@ Yhourarith(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
-          field1.grid = varList1[varID].gridID;
-          field1.missval = varList1[varID].missval;
+          cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
-          vfarfun(field1, vars2[houroy][varID][levelID], operfunc);
+          vfarfun(field, vars2[houroy][varID][levelID], operfunc);
 
           cdoDefRecord(streamID3, varID, levelID);
-          cdoWriteRecord(streamID3, field1.vec.data(), field1.nmiss);
+          cdoWriteRecord(streamID3, field.vec_d.data(), field.nmiss);
         }
 
       tsID++;
