@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -43,41 +43,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
 #include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 #include "printinfo.h"
 
-
-static int
-hour_of_year(int64_t vdate, int vtime)
-{
-  int year, month, day;
-  int hour, minute, second;
-  cdiDecodeDate(vdate, &year, &month, &day);
-  cdiDecodeTime(vtime, &hour, &minute, &second);
-
-  int houroy = 0;
-  if (month >= 1 && month <= 12 && day >= 1 && day <= 31 && hour >= 0 && hour < 24)
-    houroy = ((month - 1) * 31 + day - 1) * 25 + hour + 1;
-
-  return houroy;
-}
-
-static int
-hour_of_day(int64_t vdate, int vtime)
-{
-  int year, month, day;
-  int hour, minute, second;
-  cdiDecodeDate(vdate, &year, &month, &day);
-  cdiDecodeTime(vtime, &hour, &minute, &second);
-
-  int hourod = 0;
-  if (month >= 1 && month <= 12 && day >= 1 && day <= 31 && hour >= 0 && hour < 24)
-    hourod = hour + 1;
-
-  return hourod;
-}
-
 static void
 addOperators(void)
 {
@@ -110,41 +81,41 @@ void *
 Yhourstat(void *process)
 {
   TimeStat timestat_date = TimeStat::LAST;
-  int varID, levelID;
-  int nrecs;
 
   cdoInitialize(process);
 
   addOperators();
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  const bool ldaily  = cdoOperatorF2(operatorID) == 1;
+  const bool ldaily = cdoOperatorF2(operatorID) == 1;
 
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
   const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
-  const bool lvars2  = lvarstd || lrange;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
+  const bool lvars2 = lvarstd || lrange;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  operatorCheckArgc(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
 
-  const int maxHours = ldaily ? 25 : 9301;  // year: 31*12*25 + 1
-  std::vector<int> hourot_nsets(maxHours, 0);  // hour of time
-  FieldVector3D vars1(maxHours), vars2(maxHours), samp1(maxHours);
+  const int MaxHours = ldaily ? 25 : 9301;     // year: 31*12*25 + 1
+  std::vector<int> hourot_nsets(MaxHours, 0);  // hour of time
+  FieldVector3D vars1(MaxHours), vars2(MaxHours), samp1(MaxHours);
 
-  std::vector<DateTimeList> dtlist(maxHours);
-  for (int hourot = 0; hourot < maxHours; ++hourot)
+  std::vector<DateTimeList> dtlist(MaxHours);
+  for (int hourot = 0; hourot < MaxHours; ++hourot)
     {
       dtlist[hourot].setStat(timestat_date);
       dtlist[hourot].setCalendar(taxisInqCalendar(taxisID1));
@@ -152,46 +123,45 @@ Yhourstat(void *process)
 
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   int tsID = 0;
   int otsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      const int hourot = ldaily ? hour_of_day(vdate, vtime) : hour_of_year(vdate, vtime);
-      if (hourot < 0 || hourot >= maxHours)
-        {
-          char vdatestr[32], vtimestr[32];
-          date2str(vdate, vdatestr, sizeof(vdatestr));
-          time2str(vtime, vtimestr, sizeof(vtimestr));
-          cdoAbort("Hour of year %d out of range (%s %s)!", hourot, vdatestr, vtimestr);
-        }
+      const auto hourot = ldaily ? decodeHourOfDay(vdate, vtime) : decodeHourOfYear(vdate, vtime);
+      if (hourot < 0 || hourot >= MaxHours)
+        cdoAbort("Hour of year %d out of range (%s %s)!", hourot, dateToString(vdate).c_str(), timeToString(vtime).c_str());
 
       dtlist[hourot].taxisInqTimestep(taxisID1, hourot_nsets[hourot]);
 
       if (!vars1[hourot].size())
         {
-          fieldsFromVlist(vlistID1, samp1[hourot], FIELD_NONE);
+          fieldsFromVlist(vlistID1, samp1[hourot]);
           fieldsFromVlist(vlistID1, vars1[hourot], FIELD_VEC);
           if (lvars2) fieldsFromVlist(vlistID1, vars2[hourot], FIELD_VEC);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
@@ -201,32 +171,31 @@ Yhourstat(void *process)
               recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
-          Field &rsamp1 = samp1[hourot][varID][levelID];
-          Field &rvars1 = vars1[hourot][varID][levelID];
+          auto &rsamp1 = samp1[hourot][varID][levelID];
+          auto &rvars1 = vars1[hourot][varID][levelID];
 
-          const int nsets = hourot_nsets[hourot];
-          const size_t fieldsize = rvars1.size;
+          const auto nsets = hourot_nsets[hourot];
+          const auto fieldsize = rvars1.size;
 
           if (nsets == 0)
             {
-              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+              cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
               if (lrange)
                 {
                   vars2[hourot][varID][levelID].nmiss = rvars1.nmiss;
-                  vars2[hourot][varID][levelID].vec = rvars1.vec;
+                  vars2[hourot][varID][levelID].vec_d = rvars1.vec_d;
                 }
 
               if (rvars1.nmiss || !rsamp1.empty())
                 {
                   if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                  for (size_t i = 0; i < fieldsize; i++)
-                    rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                  for (size_t i = 0; i < fieldsize; i++) rsamp1.vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                 }
             }
           else
             {
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
               field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
@@ -236,7 +205,7 @@ Yhourstat(void *process)
                   if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
                   for (size_t i = 0; i < fieldsize; i++)
-                    if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
+                    if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i]++;
                 }
 
               if (lvarstd)
@@ -261,8 +230,8 @@ Yhourstat(void *process)
           {
             if (recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
             vfarmoq(vars2[hourot][varID][levelID], vars1[hourot][varID][levelID]);
           }
 
@@ -270,18 +239,18 @@ Yhourstat(void *process)
       tsID++;
     }
 
-  for (int hourot = 0; hourot < maxHours; ++hourot)
+  for (int hourot = 0; hourot < MaxHours; ++hourot)
     if (hourot_nsets[hourot])
       {
-        const int nsets = hourot_nsets[hourot];
+        const auto nsets = hourot_nsets[hourot];
         for (int recID = 0; recID < maxrecs; recID++)
           {
             if (recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
-            Field &rsamp1 = samp1[hourot][varID][levelID];
-            Field &rvars1 = vars1[hourot][varID][levelID];
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
+            auto &rsamp1 = samp1[hourot][varID][levelID];
+            auto &rvars1 = vars1[hourot][varID][levelID];
 
             if (lmean)
               {
@@ -292,7 +261,7 @@ Yhourstat(void *process)
               }
             else if (lvarstd)
               {
-                Field &rvars2 = vars2[hourot][varID][levelID];
+                auto &rvars2 = vars2[hourot][varID][levelID];
                 if (!rsamp1.empty())
                   {
                     if (lstd)
@@ -310,7 +279,7 @@ Yhourstat(void *process)
               }
             else if (lrange)
               {
-                Field &rvars2 = vars2[hourot][varID][levelID];
+                auto &rvars2 = vars2[hourot][varID][levelID];
                 vfarsub(rvars1, rvars2);
               }
           }
@@ -322,12 +291,12 @@ Yhourstat(void *process)
           {
             if (otsID && recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
-            Field &rvars1 = vars1[hourot][varID][levelID];
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
+            auto &rvars1 = vars1[hourot][varID][levelID];
 
             cdoDefRecord(streamID2, varID, levelID);
-            cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+            cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
           }
 
         otsID++;
