@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,53 +30,53 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_vlist.h"
 #include "cdo_season.h"
+#include "functs.h"
+#include "process_int.h"
 
-#define MAX_MON 12
+constexpr int MaxMonths = 12;
+
+static const char *mon_names[] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
 
 static int
 getMonthIndex(const int64_t vdate)
 {
   int year, mon, day;
   cdiDecodeDate(vdate, &year, &mon, &day);
-  if (mon < 1 || mon > MAX_MON) cdoAbort("Month %d out of range!", mon);
+  if (mon < 1 || mon > MaxMonths) cdoAbort("Month %d out of range!", mon);
   mon--;
   return mon;
 }
 
 static void
-alreadyAllocated(bool isSeasonal, int mon, const char *seas_name[4])
+alreadyAllocated(bool isSeasonal, int mon, const char *seas_names[4])
 {
   if (isSeasonal)
-    cdoAbort("Season index %s already allocated!", seas_name[mon]);
+    cdoAbort("Season %s already allocated!", seas_names[mon]);
   else
-    cdoAbort("Month index %d already allocated!", mon);
+    cdoAbort("%s already allocated! The second input file must contain monthly mean values for a maximum of one year.",
+             mon_names[mon]);
 }
 
 static void
-notFound(bool isSeasonal, int mon, const char *seas_name[4])
+notFound(bool isSeasonal, int mon, const char *seas_names[4])
 {
   if (isSeasonal)
-    cdoAbort("Season index %s not found!", seas_name[mon]);
+    cdoAbort("Season %s not found!", seas_names[mon]);
   else
-    cdoAbort("Month index %d not found!", mon);
+    cdoAbort("%s not found! The second input file must contain monthly mean values for a maximum of one year.", mon_names[mon]);
 }
 
-void *
-Ymonarith(void *process)
+enum
 {
-  enum
-  {
-    MONTHLY,
-    SEASONAL
-  };
-  int nrecs;
-  int varID, levelID;
-  const char *seas_name[4];
-
-  cdoInitialize(process);
+  MONTHLY,
+  SEASONAL
+};
 
+static void
+addOperators(void)
+{
   // clang-format off
   cdoOperatorAdd("ymonadd",  func_add, MONTHLY, nullptr);
   cdoOperatorAdd("ymonsub",  func_sub, MONTHLY, nullptr);
@@ -87,47 +87,62 @@ Ymonarith(void *process)
   cdoOperatorAdd("yseasmul", func_mul, SEASONAL, nullptr);
   cdoOperatorAdd("yseasdiv", func_div, SEASONAL, nullptr);
   // clang-format on
+}
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-  const int opertype = cdoOperatorF2(operatorID);
+void *
+Ymonarith(void *process)
+{
+  int varID, levelID;
+  const char *seas_names[4];
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  cdoInitialize(process);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID1);
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+  const auto opertype = cdoOperatorF2(operatorID);
+
+  operatorCheckArgc(0);
+
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
   VarList varList1;
   varListInit(varList1, vlistID1);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
-  Field field1, field2;
-  field1.resize(gridsizemax);
-  field2.resize(gridsizemax);
+  Field field;
+  field.resize(gridsizemax);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  if (opertype == SEASONAL) get_season_name(seas_name);
+  if (opertype == SEASONAL) getSeasonName(seas_names);
+
+  FieldVector2D vars2[MaxMonths];
 
-  FieldVector2D vars2[MAX_MON];
-  
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
+  while (true)
     {
-      int mon = getMonthIndex(taxisInqVdate(taxisID2));
-      if (opertype == SEASONAL) mon = month_to_season(mon + 1);
-      if (vars2[mon].size()) alreadyAllocated(opertype == SEASONAL, mon, seas_name);
+      const auto nrecs = cdoStreamInqTimestep(streamID2, tsID);
+      if (nrecs == 0) break;
+
+      auto mon = getMonthIndex(taxisInqVdate(taxisID2));
+      if (opertype == SEASONAL) mon = monthToSeason(mon + 1);
+      if (vars2[mon].size()) alreadyAllocated(opertype == SEASONAL, mon, seas_names);
 
       fieldsFromVlist(vlistID2, vars2[mon], FIELD_VEC);
 
@@ -135,7 +150,7 @@ Ymonarith(void *process)
         {
           size_t nmiss;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars2[mon][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars2[mon][varID][levelID].vec_d.data(), &nmiss);
           vars2[mon][varID][levelID].nmiss = nmiss;
         }
 
@@ -143,11 +158,14 @@ Ymonarith(void *process)
     }
 
   tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      int mon = getMonthIndex(taxisInqVdate(taxisID1));
-      if (opertype == SEASONAL) mon = month_to_season(mon + 1);
-      if (vars2[mon].size() == 0) notFound(opertype == SEASONAL, mon, seas_name);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      auto mon = getMonthIndex(taxisInqVdate(taxisID1));
+      if (opertype == SEASONAL) mon = monthToSeason(mon + 1);
+      if (vars2[mon].size() == 0) notFound(opertype == SEASONAL, mon, seas_names);
 
       taxisCopyTimestep(taxisID3, taxisID1);
       cdoDefTimestep(streamID3, tsID);
@@ -155,14 +173,15 @@ Ymonarith(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
-          field1.grid = varList1[varID].gridID;
-          field1.missval = varList1[varID].missval;
+          cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
-          vfarfun(field1, vars2[mon][varID][levelID], operfunc);
+          vfarfun(field, vars2[mon][varID][levelID], operfunc);
 
           cdoDefRecord(streamID3, varID, levelID);
-          cdoWriteRecord(streamID3, field1.vec.data(), field1.nmiss);
+          cdoWriteRecord(streamID3, field.vec_d.data(), field.nmiss);
         }
 
       tsID++;
