@@ -30,20 +30,15 @@
 #include "process_int.h"
 #include "param_conversion.h"
 #include "percentiles_hist.h"
-#include "percentiles.h"
-
 
 void *
 Ymonpctl(void *process)
 {
   constexpr int MaxMonths = 17;
-  int varID;
-  int levelID;
   size_t nmiss;
-  int nrecs;
-  int64_t vdates1[MaxMonths], vdates2[MaxMonths];
-  int vtimes1[MaxMonths];
-  long nsets[MaxMonths];
+  int64_t vdates1[MaxMonths] = { 0 }, vdates2[MaxMonths] = { 0 };
+  int vtimes1[MaxMonths] = { 0 };
+  long nsets[MaxMonths] = { 0 };
   FieldVector2D vars1[MaxMonths];
   HistogramSet hsets[MaxMonths];
 
@@ -52,9 +47,6 @@ Ymonpctl(void *process)
 
   operatorInputArg("percentile number");
   const auto pn = parameter2double(cdoOperatorArgv(0));
-  percentile_check_number(pn);
-
-  for (int month = 0; month < MaxMonths; month++) nsets[month] = 0;
 
   const auto streamID1 = cdoOpenRead(0);
   const auto streamID2 = cdoOpenRead(1);
@@ -68,6 +60,9 @@ Ymonpctl(void *process)
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
   const auto taxisID1 = vlistInqTaxis(vlistID1);
   const auto taxisID2 = vlistInqTaxis(vlistID2);
   const auto taxisID3 = vlistInqTaxis(vlistID3);
@@ -91,8 +86,11 @@ Ymonpctl(void *process)
   field.resize(gridsizemax);
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID2, tsID);
+      if (nrecs == 0) break;
+
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
         cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
@@ -114,27 +112,26 @@ Ymonpctl(void *process)
           fieldsFromVlist(vlistID1, vars1[month], FIELD_VEC);
           hsets[month].create(nvars);
 
-          for (varID = 0; varID < nvars; varID++)
-            {
-              auto gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-              auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              hsets[month].createVarLevels(varID, nlevels, gridsize);
-            }
+          for (int varID = 0; varID < nvars; varID++)
+            hsets[month].createVarLevels(varID, varList1[varID].nlevels, varList1[varID].gridsize);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID2, &varID, &levelID);
-          cdoReadRecord(streamID2, vars1[month][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID2, vars1[month][varID][levelID].vec_d.data(), &nmiss);
           vars1[month][varID][levelID].nmiss = nmiss;
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID3, &varID, &levelID);
-          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
-          field.grid = vars1[month][varID][levelID].grid;
-          field.missval = vars1[month][varID][levelID].missval;
+          cdoReadRecord(streamID3, field.vec_d.data(), &field.nmiss);
+          field.size = varList1[varID].gridsize;
+          field.grid = varList1[varID].gridID;
+          field.missval = varList1[varID].missval;
 
           hsets[month].defVarLevelBounds(varID, levelID, vars1[month][varID][levelID], field);
         }
@@ -143,8 +140,11 @@ Ymonpctl(void *process)
     }
 
   tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       const auto vdate = taxisInqVdate(taxisID1);
       const auto vtime = taxisInqVtime(taxisID1);
 
@@ -160,6 +160,7 @@ Ymonpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
@@ -169,10 +170,10 @@ Ymonpctl(void *process)
               recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
-          cdoReadRecord(streamID1, vars1[month][varID][levelID].vec.data(), &nmiss);
+          cdoReadRecord(streamID1, vars1[month][varID][levelID].vec_d.data(), &nmiss);
           vars1[month][varID][levelID].nmiss = nmiss;
 
-          hsets[month].addVarLevelValues(varID, levelID, vars1[month][varID][levelID]);
+          hsets[month].addSubVarLevelValues(varID, levelID, vars1[month][varID][levelID], 1, FIELD_VEC);
         }
 
       nsets[month]++;
@@ -187,13 +188,13 @@ Ymonpctl(void *process)
           cdoAbort("Verification dates for the month %d of %s and %s are different!", month, cdoGetStreamName(0),
                    cdoGetStreamName(1));
 
-        for (varID = 0; varID < nvars; varID++)
+        for (int varID = 0; varID < nvars; varID++)
           {
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
             const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-            for (levelID = 0; levelID < nlevels; levelID++)
-              hsets[month].getVarLevelPercentiles(vars1[month][varID][levelID], varID, levelID, pn);
+            for (int levelID = 0; levelID < nlevels; levelID++)
+              hsets[month].getVarLevelPercentiles(vars1[month][varID][levelID], varID, levelID, pn, FIELD_VEC);
           }
 
         taxisDefVdate(taxisID4, vdates1[month]);
@@ -207,7 +208,7 @@ Ymonpctl(void *process)
             const auto varID = recList[recID].varID;
             const auto levelID = recList[recID].levelID;
             cdoDefRecord(streamID4, varID, levelID);
-            cdoWriteRecord(streamID4, vars1[month][varID][levelID].vec.data(), vars1[month][varID][levelID].nmiss);
+            cdoWriteRecord(streamID4, vars1[month][varID][levelID].vec_d.data(), vars1[month][varID][levelID].nmiss);
           }
 
         otsID++;
