@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -32,36 +32,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 
-#define NMONTH 17
 
-/*
-static
-int compareInt(const void *a, const void *b)
+static void
+addOperators(void)
 {
-  const int *x = (const int*) a;
-  const int *y = (const int*) b;
-  return ((*x > *y) - (*x < *y)) * 2 + (*x > *y) - (*x < *y);
-}
-*/
-
-void *
-Ymonstat(void *process)
-{
-  int varID;
-  int year, month, day;
-  int nrecs;
-  int levelID;
-  int month_nsets[NMONTH];
-  int64_t vdates[NMONTH];
-  int vtimes[NMONTH];
-  int mon[NMONTH];
-  int nmon = 0;
-  FieldVector2D vars1[NMONTH], vars2[NMONTH], samp1[NMONTH];
-
-  cdoInitialize(process);
-
   // clang-format off
   cdoOperatorAdd("ymonrange", func_range, 0, nullptr);
   cdoOperatorAdd("ymonmin",   func_min,   0, nullptr);
@@ -73,67 +52,92 @@ Ymonstat(void *process)
   cdoOperatorAdd("ymonvar1",  func_var1,  0, nullptr);
   cdoOperatorAdd("ymonstd",   func_std,   0, nullptr);
   cdoOperatorAdd("ymonstd1",  func_std1,  0, nullptr);
-
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
-  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
   // clang-format on
+}
+
+void *
+Ymonstat(void *process)
+{
+  const TimeStat timestat_date = TimeStat::LAST;
+  constexpr int MaxMonths = 17;
+  int month_nsets[MaxMonths] = { 0 };
+  int mon[MaxMonths] = { 0 };
+  int nmon = 0;
+  FieldVector2D vars1[MaxMonths], vars2[MaxMonths], samp1[MaxMonths];
+
+  cdoInitialize(process);
+
+  addOperators();
+
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
   const bool lvars2 = lvarstd || lrange;
 
-  for (month = 0; month < NMONTH; month++) month_nsets[month] = 0;
+  operatorCheckArgc(0);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
-  if (taxisHasBounds(taxisID2)) taxisDeleteBounds(taxisID2);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
+  taxisWithBounds(taxisID2);
+  if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  DateTimeList dtlists[MaxMonths];
+  for (int month = 0; month < MaxMonths; ++ month)
+    {
+      dtlists[month].setStat(timestat_date);
+      dtlists[month].setCalendar(taxisInqCalendar(taxisID1));
+    }
+
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
 
   int tsID = 0;
   int otsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-      if (month < 0 || month >= NMONTH) cdoAbort("month %d out of range!", month);
+      const auto month = decodeMonth(vdate);
+      if (month < 0 || month >= MaxMonths) cdoAbort("Month %d out of range!", month);
 
-      vdates[month] = vdate;
-      vtimes[month] = vtime;
-      // mon[month] = vdate;
+      dtlists[month].taxisSetNextTimestep(taxisID1);
 
       if (!vars1[month].size())
         {
           mon[nmon++] = month;
-          fieldsFromVlist(vlistID1, samp1[month], FIELD_NONE);
+          fieldsFromVlist(vlistID1, samp1[month]);
           fieldsFromVlist(vlistID1, vars1[month], FIELD_VEC);
           if (lvars2) fieldsFromVlist(vlistID1, vars2[month], FIELD_VEC);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
+          int varID, levelID;
           cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
@@ -143,32 +147,31 @@ Ymonstat(void *process)
               recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
-          Field &rsamp1 = samp1[month][varID][levelID];
-          Field &rvars1 = vars1[month][varID][levelID];
+          auto &rsamp1 = samp1[month][varID][levelID];
+          auto &rvars1 = vars1[month][varID][levelID];
 
-          const int nsets = month_nsets[month];
-          const size_t fieldsize = rvars1.size;
+          const auto nsets = month_nsets[month];
+          const auto fieldsize = rvars1.size;
 
           if (nsets == 0)
             {
-              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+              cdoReadRecord(streamID1, rvars1.vec_d.data(), &rvars1.nmiss);
               if (lrange)
                 {
                   vars2[month][varID][levelID].nmiss = rvars1.nmiss;
-                  vars2[month][varID][levelID].vec = rvars1.vec;
+                  vars2[month][varID][levelID].vec_d = rvars1.vec_d;
                 }
 
               if (rvars1.nmiss || !rsamp1.empty())
                 {
                   if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                  for (size_t i = 0; i < fieldsize; i++)
-                    rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
+                  for (size_t i = 0; i < fieldsize; i++) rsamp1.vec_d[i] = !DBL_IS_EQUAL(rvars1.vec_d[i], rvars1.missval);
                 }
             }
           else
             {
-              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              cdoReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
               field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
@@ -178,7 +181,7 @@ Ymonstat(void *process)
                   if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
                   for (size_t i = 0; i < fieldsize; i++)
-                    if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
+                    if (!DBL_IS_EQUAL(field.vec_d[i], rvars1.missval)) rsamp1.vec_d[i]++;
                 }
 
               if (lvarstd)
@@ -203,8 +206,8 @@ Ymonstat(void *process)
           {
             if (recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
             vfarmoq(vars2[month][varID][levelID], vars1[month][varID][levelID]);
           }
 
@@ -215,45 +218,26 @@ Ymonstat(void *process)
   if (nmon == 12)
     {
       int smon = 0;
-      for (month = 1; month <= 12; month++)
+      for (int month = 1; month <= 12; month++)
         if (month_nsets[month]) smon++;
       if (smon == 12)
-        for (month = 1; month <= 12; month++) mon[month - 1] = month;
+        for (int month = 1; month <= 12; month++) mon[month - 1] = month;
     }
 
-  /* sort output time steps */
-  /*
-  nmon = 0;
-  for ( month = 0; month < NMONTH; month++ )
-    {
-      if ( month_nsets[month] == 0 )
-        for ( i = month+1; i < NMONTH; i++ ) mon[i-1] = mon[i];
-      else
-        nmon++;
-    }
-
-  qsort(mon, nmon, sizeof(int), compareInt);
-
-  for ( i = 0; i < nmon; i++ )
-    {
-      cdiDecodeDate(mon[i], &year, &month, &day);
-      mon[i] = month;
-    }
-  */
   for (int i = 0; i < nmon; i++)
     {
-      month = mon[i];
-      const int nsets = month_nsets[month];
+      const auto month = mon[i];
+      const auto nsets = month_nsets[month];
       if (nsets == 0) cdoAbort("Internal problem, nsets[%d] not defined!", month);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
           if (recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rsamp1 = samp1[month][varID][levelID];
-          Field &rvars1 = vars1[month][varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rsamp1 = samp1[month][varID][levelID];
+          auto &rvars1 = vars1[month][varID][levelID];
 
           if (lmean)
             {
@@ -264,7 +248,7 @@ Ymonstat(void *process)
             }
           else if (lvarstd)
             {
-              Field &rvars2 = vars2[month][varID][levelID];
+              auto &rvars2 = vars2[month][varID][levelID];
               if (!rsamp1.empty())
                 {
                   if (lstd)
@@ -282,25 +266,24 @@ Ymonstat(void *process)
             }
           else if (lrange)
             {
-              Field &rvars2 = vars2[month][varID][levelID];
+              auto &rvars2 = vars2[month][varID][levelID];
               vfarsub(rvars1, rvars2);
             }
         }
 
-      taxisDefVdate(taxisID2, vdates[month]);
-      taxisDefVtime(taxisID2, vtimes[month]);
+      dtlists[month].statTaxisDefTimestep(taxisID2);
       cdoDefTimestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
           if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recList[recID].varID;
-          const int levelID = recList[recID].levelID;
-          Field &rvars1 = vars1[month][varID][levelID];
+          const auto varID = recList[recID].varID;
+          const auto levelID = recList[recID].levelID;
+          auto &rvars1 = vars1[month][varID][levelID];
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
+          cdoWriteRecord(streamID2, rvars1.vec_d.data(), rvars1.nmiss);
         }
 
       otsID++;
