@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.1
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,11 +33,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include <mpim_grid.h>
-#include "percentiles.h"
 
 static void
 addOperators(void)
@@ -53,6 +54,8 @@ addOperators(void)
   cdoOperatorAdd("zonvar1",  func_var1,  0, nullptr);
   cdoOperatorAdd("zonstd",   func_std,   0, nullptr);
   cdoOperatorAdd("zonstd1",  func_std1,  0, nullptr);
+  cdoOperatorAdd("zonskew",  func_skew,  0, nullptr);
+  cdoOperatorAdd("zonkurt",  func_kurt,  0, nullptr);
   cdoOperatorAdd("zonpctl",  func_pctl,  0, nullptr);
   // clang-format on
 }
@@ -63,68 +66,71 @@ Zonstat(void *process)
   int gridID1 = -1, gridID2 = -1;
   int zongridID = -1;
   int index;
-  int nrecs;
   int varID, levelID;
 
   cdoInitialize(process);
 
   addOperators();
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
 
-  double pn = 0;
+  double pn = 0.0;
   if (operfunc == func_pctl)
     {
       operatorInputArg("percentile number");
-      pn = parameter2double(operatorArgv()[0]);
-      percentile_check_number(pn);
+      pn = parameter2double(cdoOperatorArgv(0));
+    }
+  else
+    {
+      operatorCheckArgc(0);
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (index = 0; index < ngrids; index++)
     {
-      const int gridID = vlistGrid(vlistID1, index);
-      if (gridInqXsize(gridID) > 1 || gridInqType(gridID) == GRID_GAUSSIAN_REDUCED)
+      const auto gridID = vlistGrid(vlistID1, index);
+      const auto xsize = gridInqXsize(gridID);
+      const auto ysize = gridInqYsize(gridID);
+      const auto gridtype = gridInqType(gridID);
+      if (xsize > 1 || gridtype == GRID_GAUSSIAN_REDUCED)
         {
           if (gridID1 == -1) gridID1 = gridID;
         }
       else
         {
-          if (zongridID == -1) zongridID = gridID;
+          if (ysize > 1 && zongridID == -1) zongridID = gridID;
         }
     }
 
   int ndiffgrids = 0;
   for (index = 0; index < ngrids; index++)
     {
-      if (zongridID != -1 && zongridID == vlistGrid(vlistID1, index)) continue;
-      if (gridID1 != vlistGrid(vlistID1, index)) ndiffgrids++;
+      const auto gridID = vlistGrid(vlistID1, index);
+      if (zongridID != -1 && zongridID == gridID) continue;
+      if (gridID1 != gridID) ndiffgrids++;
     }
   if (ndiffgrids) cdoAbort("Too many different grids!");
 
   if (gridID1 != -1)
     {
-      const int gridtype = gridInqType(gridID1);
+      const auto gridtype = gridInqType(gridID1);
       if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED || gridtype == GRID_GENERIC)
         {
-          if (zongridID != -1 && gridInqYsize(zongridID) == gridInqYsize(gridID1))
-            gridID2 = zongridID;
-          else
-            gridID2 = gridToZonal(gridID1);
+          gridID2 = (zongridID != -1 && gridInqYsize(zongridID) == gridInqYsize(gridID1)) ? zongridID : gridToZonal(gridID1);
         }
       else
         {
-          cdoAbort("Unsupported gridtype: %s", gridNamePtr(gridInqType(gridID1)));
+          cdoAbort("Unsupported gridtype: %s", gridNamePtr(gridtype));
         }
     }
   else
@@ -137,17 +143,20 @@ Zonstat(void *process)
 
   if (Options::cdoChunkType == CDI_UNDEFID)
     {
-      const int nvars = vlistNvars(vlistID2);
+      const auto nvars = vlistNvars(vlistID2);
       for (varID = 0; varID < nvars; ++varID) vlistDefVarChunkType(vlistID2, varID, CDI_CHUNK_AUTO);
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  gridID1 = vlistInqVarGrid(vlistID1, 0);
-  const int nlatmax = gridInqYsize(gridID1); /* max nlat ? */
+  VarList varList1;
+  varListInit(varList1, vlistID1);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  gridID1 = vlistInqVarGrid(vlistID1, 0);
+  const auto nlatmax = gridInqYsize(gridID1); // max nlat?
+
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
   field1.resize(gridsizemax);
@@ -155,35 +164,35 @@ Zonstat(void *process)
   field2.grid = gridID2;
 
   int tsID = 0;
-  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
+  while (true)
     {
+      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      if (nrecs == 0) break;
+
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
+          cdoReadRecord(streamID1, field1.vec_d.data(), &field1.nmiss);
 
-          field1.grid = vlistInqVarGrid(vlistID1, varID);
-          field1.missval = vlistInqVarMissval(vlistID1, varID);
-          field2.missval = vlistInqVarMissval(vlistID1, varID);
+          field1.grid = varList1[varID].gridID;
+          field1.missval = varList1[varID].missval;
+          field2.missval = varList1[varID].missval;
 
           if (zongridID != -1 && zongridID == field1.grid)
             {
-              arrayCopy(nlatmax, field1.vec.data(), field2.vec.data());
+              varrayCopy(nlatmax, field1.vec_d, field2.vec_d);
               field2.nmiss = field1.nmiss;
             }
           else
             {
-              if (operfunc == func_pctl)
-                zonpctl(field1, field2, pn);
-              else
-                zonfun(field1, field2, operfunc);
+              (operfunc == func_pctl) ? fieldZonPctl(field1, field2, pn) : fieldZonFunction(field1, field2, operfunc);
             }
 
           cdoDefRecord(streamID2, varID, levelID);
-          cdoWriteRecord(streamID2, field2.vec.data(), field2.nmiss);
+          cdoWriteRecord(streamID2, field2.vec_d.data(), field2.nmiss);
         }
 
       tsID++;
