@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -129,7 +129,7 @@ dv2uv_kernel(const double *d, const double *o, double *restrict u, double *restr
 }
 
 void
-dv2uv(const double *d, const double *o, double *u, double *v, double *f, double *g, long nt, long nsp, long nlev)
+dv2uv(const double *d, const double *o, double *u, double *v, const double *f, const double *g, long nt, long nsp, long nlev)
 {
   // d(nsp,nlev), o(nsp,nlev)     ! divergence, vorticity
   // u(nsp,nlev), v(nsp,nlev)     ! zonal wind, meridional wind
@@ -176,42 +176,36 @@ scaluv(double *fu, const double *rclat, long nlat, long lot)
 }
 
 void
-uv2dv(double *fu, double *fv, double *sd, double *sv, double *pol2, double *pol3, long klev, long nlat, long nt)
+uv2dv(const double *fu, const double *fv, double *sd, double *sv, const double *pol2, const double *pol3, long klev, long nlat,
+      long nt)
 {
-  long lev, jmm, jfc, lat;
-  double dir, dii, vor, voi;
-  double *ufr, *ufi, *vfr, *vfi;
-  double *ful, *fvl, *sdl, *svl;
-  double *po2, *po3;
-
   long nsp2 = (nt + 1) * (nt + 2);
   long nfc = (nt + 1) * 2;
 
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(jmm, jfc, lat, po2, po3, ful, fvl, sdl, svl, ufr, ufi, vfr, vfi, dir, dii, vor, \
-                                                 voi)
+#pragma omp parallel for default(shared)
 #endif
-  for (lev = 0; lev < klev; lev++)
+  for (long lev = 0; lev < klev; lev++)
     {
-      po2 = pol2;
-      po3 = pol3;
-      ful = fu + lev * nfc * nlat;
-      fvl = fv + lev * nfc * nlat;
-      sdl = sd + lev * nsp2;
-      svl = sv + lev * nsp2;
-      for (jmm = 0; jmm <= nt; jmm++)
+      auto po2 = pol2;
+      auto po3 = pol3;
+      auto ful = fu + lev * nfc * nlat;
+      auto fvl = fv + lev * nfc * nlat;
+      auto sdl = sd + lev * nsp2;
+      auto svl = sv + lev * nsp2;
+      for (long jmm = 0; jmm <= nt; jmm++)
         {
-          for (jfc = jmm; jfc <= nt; jfc++)
+          for (long jfc = jmm; jfc <= nt; jfc++)
             {
-              ufr = ful;
-              ufi = ful + nlat;
-              vfr = fvl;
-              vfi = fvl + nlat;
-              dir = 0.0;
-              dii = 0.0;
-              vor = 0.0;
-              voi = 0.0;
-              for (lat = 0; lat < nlat; lat++)
+              auto ufr = ful;
+              auto ufi = ful + nlat;
+              auto vfr = fvl;
+              auto vfi = fvl + nlat;
+              double dir = 0.0;
+              double dii = 0.0;
+              double vor = 0.0;
+              double voi = 0.0;
+              for (long lat = 0; lat < nlat; lat++)
                 {
                   dir += vfr[lat] * po2[lat] - ufi[lat] * po3[lat];
                   dii += vfi[lat] * po2[lat] + ufr[lat] * po3[lat];
@@ -234,18 +228,15 @@ uv2dv(double *fu, double *fv, double *sd, double *sv, double *pol2, double *pol3
 void
 geninx(long ntr, double *f, double *g)
 {
-  long m2, n2;
-  long m, n;
-
-  for (m = 0; m <= ntr; m++)
+  for (long m = 0; m <= ntr; m++)
     {
-      m2 = m * m;
-      for (n = m; n <= ntr; n++)
+      long m2 = m * m;
+      for (long n = m; n <= ntr; n++)
         {
-          n2 = n * n;
+          long n2 = n * n;
           if (n)
             {
-              *g++ = -PlanetRadius / n * sqrt((double) (n2 - m2) / (double) (4 * n2 - 1));
+              *g++ = -PlanetRadius / n * std::sqrt((double) (n2 - m2) / (double) (4 * n2 - 1));
               *f++ = -PlanetRadius * m / (double) (n2 + n);
             }
           else
