@@ -24,10 +24,8 @@
 #include "cdo_options.h"
 #include "cimdOmp.h"
 #include "constants.h"
-#include "mpmo.h"
 #include "array.h"
-
-extern "C" void gaussianLatitudes(double *latitudes, double *weights, size_t nlat);
+#include "gaussian_latitudes.h"
 
 static void
 jspleg1(double *pleg, double plat, long ktrunc, double *work)
@@ -286,7 +284,7 @@ after_legini_full(long ntr, long nlat, double *restrict poli, double *restrict p
   long dimsp = (ntr + 1) * (ntr + 2); // used in omp loop
 
   Varray<double> gmu(nlat), gwt(nlat);
-  gaussianLatitudes(gmu.data(), gwt.data(), nlat);
+  gaussian_latitudes(nlat, gmu.data(), gwt.data());
 
 #ifndef _OPENMP
   Varray<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
@@ -309,10 +307,10 @@ after_legini_full(long ntr, long nlat, double *restrict poli, double *restrict p
       const double zrafgmusqr = 1. / (PlanetRadius * gmusq);
       const double zradsqrtgmusqr = 1. / (-PlanetRadius * std::sqrt(gmusq));
 
-      const int lpold = pold != nullptr;
-      const int lpdev = pdev != nullptr;
-      const int lpol2 = pol2 != nullptr;
-      const int lpol3 = pol3 != nullptr;
+      const bool lpold = pold != nullptr;
+      const bool lpdev = pdev != nullptr;
+      const bool lpol2 = pol2 != nullptr;
+      const bool lpol3 = pol3 != nullptr;
 
       long jsp = jgl;
       for (long jm = 0; jm < waves; ++jm)
@@ -344,7 +342,7 @@ after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold,
 #endif
 
   Varray<double> gmu(nlat), gwt(nlat);
-  gaussianLatitudes(gmu.data(), gwt.data(), nlat);
+  gaussian_latitudes(nlat, gmu.data(), gwt.data());
   for (long jgl = 0; jgl < nlat; ++jgl) gwt[jgl] *= 0.5;
 
   for (long jgl = 0; jgl < nlat; ++jgl) coslat[jgl] = std::sqrt(1.0 - gmu[jgl] * gmu[jgl]);
