@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
 #include <string.h>
 #include <stdarg.h>
 
-#include "error.h"
 #include "dmemory.h"
 #include "process_int.h"
 #include "transform.h"
@@ -47,7 +46,7 @@ struct Date
   int mn;
 };
 
-struct Control
+struct AfterControl
 {
   int Verbose;
 
@@ -61,11 +60,11 @@ struct Control
   int OutputInterval;
   int EndOfInterval;
 
-  int AnalysisData; /* 0 = ECHAM Data, 1 = ECMWF Spectral Analyses */
-  int DayIn;        /* day increment of infiles if Multi = true    */
+  int AnalysisData; // 0 = ECHAM Data, 1 = ECMWF Spectral Analyses
+  int DayIn;        // day increment of infiles if Multi = true
   int Debug;
-  int Extrapolate;
-  int Szip;
+  bool extrapolate;
+  bool szip;
 
   int istreamID;
   CdoStreamID ostreamID;
@@ -130,17 +129,17 @@ struct Control
 
 struct Variable
 {
-  int needed0;  /* var needed for process  */
-  int needed;   /* var needed for process  */
-  int selected; /* var selected for output */
-  int detected; /* var detected in input   */
-  int comp;     /* compute var if selected and not detected */
+  int needed0;  // var needed for process
+  int needed;   // var needed for process
+  int selected; // var selected for output
+  int detected; // var detected in input
+  int comp;     // compute var if selected and not detected
   int sfit;
   int hlev;
   int plev;
   int ivarID;
-  int ovarID;  /* 1st variable ID */
-  int ovarID2; /* 2nd variable ID used for variance */
+  int ovarID;   // 1st variable ID
+  int ovarID2;  // 2nd variable ID used for variance
   int tableID;
   int igridID;
   int ogridID;
@@ -219,36 +218,35 @@ struct Variable
 
 void after_read_vct(const char *vctfile, double **vct, int *nvct);
 
-void after_gp2sp(struct Control *globs, struct Variable *vars, int ccode);
+void after_gp2sp(const AfterControl &globs, struct Variable *vars, int ccode);
 void after_GP2FC(double *gp, double *fc, long nlat, long nlon, long nlev, long nfc);
 void after_FC2GP(double *fc, double *gp, long nlat, long nlon, long nlev, long nfc);
-void after_FCrh2FCsh(struct Control *globs, struct Variable *vars);
-void after_SPuv2SPdv(struct Control *globs, struct Variable *vars);
-void after_FCsh2FCrh(struct Control *globs, struct Variable *vars);
+void after_FCrh2FCsh(const AfterControl &globs, struct Variable *vars);
+void after_SPuv2SPdv(const AfterControl &globs, struct Variable *vars);
+void after_FCsh2FCrh(const AfterControl &globs, struct Variable *vars);
 
-void after_EchamCompGP(struct Control *globs, struct Variable *vars);
-void after_processPL(struct Control *globs, struct Variable *vars);
-void after_processML(struct Control *globs, struct Variable *vars);
+void after_EchamCompGP(const AfterControl &globs, struct Variable *vars);
+void after_processPL(AfterControl &globs, struct Variable *vars);
+void after_processML(AfterControl &globs, struct Variable *vars);
 
-void after_AnalysisAddRecord(struct Control *globs, struct Variable *vars, int code, int gridID, int zaxisID, int levelID,
+void after_AnalysisAddRecord(const AfterControl *globs, struct Variable *vars, int code, int gridID, int zaxisID, int levelID,
                              size_t nmiss);
 double *after_get_dataptr(struct Variable *vars, int code, int gridID, int zaxisID, int levelID);
-void after_EchamAddRecord(struct Control *globs, struct Variable *vars, int code, int gridID, int zaxisID, int levelID,
+void after_EchamAddRecord(const AfterControl *globs, struct Variable *vars, int code, int gridID, int zaxisID, int levelID,
                           size_t nmiss);
 
 void after_AnalysisDependencies(struct Variable *vars, int ncodes);
 void after_EchamDependencies(struct Variable *vars, int ncodes, int type, int source);
 
-void after_legini_setup(struct Control *globs, struct Variable *vars);
-
+void after_legini_setup(AfterControl &globs, struct Variable *vars);
 
 template <typename... Args>
 void
 afterAbort(const std::string &format, Args const &... args)
 {
-  extern void *afterReadTask;
+  extern cdo::Task *afterReadTask;
   extern bool afterReadAsync;
-  if (afterReadAsync) cdo_task_wait(afterReadTask);
+  if (afterReadAsync && afterReadTask) afterReadTask->wait();
   cdoAbort(format, args...);
 }
 
