@@ -104,8 +104,6 @@ extern "C" void streamGrbDefDataScanningMode(int scanmode);
 
 void setPointSearchMethod(const std::string &methodstr);
 
-#define ITSME (cdo_cmpstr(cdo::Username, "\x6d\x32\x31\x34\x30\x30\x33"))
-
 static void
 cdo_stackframe()
 {
@@ -269,6 +267,7 @@ cdo_usage()
   fprintf(stderr,
           "                   F<XXX>, t<RES>, tl<RES>, global_<DXY>, r<NX>x<NY>, g<NX>x<NY>, gme<NI>, lon=<LON>/lat=<LAT>\n");
   fprintf(stderr, "    -h, --help     Help information for the operators\n");
+  fprintf(stderr, "    --ignore_time_bounds  Ignores time bounds for time range statistics\n");
   fprintf(stderr, "    --no_history   Do not append to NetCDF \"history\" global attribute\n");
   fprintf(stderr, "    --netcdf_hdr_pad, --hdr_pad, --header_pad <nbr>\n");
   fprintf(stderr, "                   Pad NetCDF output header with nbr bytes\n");
@@ -335,7 +334,7 @@ cdo_usage()
   fprintf(stderr, "    Use option --operators for a list of all operators.\n");
 
   fprintf(stderr, "\n");
-  fprintf(stderr, "  CDO version %s, Copyright (C) 2003-2020 Uwe Schulzweida\n", VERSION);
+  fprintf(stderr, "  CDO version %s, Copyright (C) 2003-2021 Uwe Schulzweida\n", VERSION);
   fprintf(stderr, "  This is free software and comes with ABSOLUTELY NO WARRANTY\n");
   fprintf(stderr, "  Report bugs to <https://mpimet.mpg.de/cdo>\n");
 }
@@ -853,7 +852,7 @@ print_system_info()
 #ifdef _OPENMP
   fprintf(stderr, "OPENMP VERSION      = %d\n", _OPENMP);
 #endif
-  fprintf(stderr, "__cplusplus         = %ld\n", __cplusplus);
+  fprintf(stderr, "__cplusplus         = %ld\n", (long)__cplusplus);
 #ifdef __GNUC__
   fprintf(stderr, "GNUC VERSION        = %d\n", __GNUC__);
 #endif
@@ -1001,6 +1000,7 @@ parseOptionsLong(int argc, char *argv[])
   int leccodes;
   int lenableexcept;
   int lgridsearchradius;
+  int lignore_time_bounds;
   int lnetcdf_hdr_pad;
   int lno_remap_weights;
   int lpercentile;
@@ -1033,6 +1033,7 @@ parseOptionsLong(int argc, char *argv[])
       { "enableexcept"               , required_argument , &lenableexcept               , 1             },
       { "timestat_date"              , required_argument , &ltimestat_date              , 1             },
       { "no_remap_weights"           , no_argument       , &lno_remap_weights           , 1             },
+      { "ignore_time_bounds"         , no_argument       , &lignore_time_bounds         , 1             },
       { "use_time_bounds"            , no_argument       , &luse_time_bounds            , 1             },
       { "cmor"                       , no_argument       , &Options::CMOR_Mode          , 1             },
       { "reduce_dim"                 , no_argument       , &Options::CDO_Reduce_Dim     , 1             },
@@ -1095,6 +1096,7 @@ parseOptionsLong(int argc, char *argv[])
       lsortparam = 0;
       ltimestat_date = 0;
       luse_fftw = 0;
+      lignore_time_bounds = 0;
       luse_time_bounds = 0;
 
       int c = cdo_getopt_long(argc, argv, "f:b:e:P:g:i:k:l:m:n:t:D:z:aC:AcdhLMOpQRrsSTuVvWwXZ", opt_long, nullptr);
@@ -1155,6 +1157,11 @@ parseOptionsLong(int argc, char *argv[])
             {
               setTimestatDate(CDO_optarg);
             }
+          else if (lignore_time_bounds)
+            {
+              extern bool CDO_Ignore_Time_Bounds;
+              CDO_Ignore_Time_Bounds = true;
+            }
           else if (luse_time_bounds)
             {
               extern bool CDO_Use_Time_Bounds;
@@ -1716,26 +1723,23 @@ print_openmp_info()
 }
 #endif
 
-extern "C" int (*proj_lonlat_to_lcc_func)(double, double, double, double, double, double, double, size_t, double*, double*);
-extern "C" int (*proj_lcc_to_lonlat_func)(double, double, double, double, double, double, double, double, double, size_t, double*, double*);
-extern "C" int (*proj_lonlat_to_stere_func)(double, double, double, double, double, size_t, double*, double*);
-extern "C" int (*proj_stere_to_lonlat_func)(double, double, double, double, double, double, double, size_t, double*, double*);
-
 static void
 set_external_proj_func(void)
 {
+#ifdef HAVE_CDI_PROJ_FUNCS
   proj_lonlat_to_lcc_func = proj_lonlat_to_lcc;
   proj_lcc_to_lonlat_func = proj_lcc_to_lonlat;
 
   proj_lonlat_to_stere_func = proj_lonlat_to_stere;
   proj_stere_to_lonlat_func = proj_stere_to_lonlat;
+#endif
 }
 
 const char *
 getProgname(char *string)
 {
 #ifdef _WIN32
-  /*  progname = strrchr(string, '\\'); */
+  //  progname = strrchr(string, '\\');
   char *progname = " cdo";
 #else
   char *progname = strrchr(string, '/');
