@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,18 +26,19 @@
 #include "cdo_options.h"
 
 static int nextCdoStreamID = 1;
-static int createdCdoStreams = 0;
 
 CdoStream::~CdoStream() {}
 CdoStream::CdoStream() : m_cdoStreamID(nextCdoStreamID++)
 {
   isopen = false;
+  ispipe = false;
   m_vlistID = -1;
   m_tsID = -1;
   m_filetype = CDI_UNDEFID;
   m_name = "";
   m_varID = -1;
   m_nvals = 0;
+  m_fileID = 0;
 }
 
 int
@@ -51,11 +52,13 @@ CdoStream::getTsID()
 {
   return m_tsID;
 }
+
 int
 CdoStream::getVarID()
 {
   return m_varID;
 }
+
 int
 CdoStream::getVlistID()
 {
@@ -63,33 +66,33 @@ CdoStream::getVlistID()
 }
 
 void
-CdoStream::defVarList(int p_vlistID)
+CdoStream::defDatarangeList(int p_vlistID)
 {
-  int filetype = m_filetype;
+  const auto filetype = m_filetype;
 
   if (m_vlistID != -1) cdoAbort("Internal problem, vlist already defined!");
 
-  if (m_varlist.size() != 0) cdoAbort("Internal problem, varlist already allocated!");
+  if (m_datarangelist.size() != 0) cdoAbort("Internal problem, datarangelist already allocated!");
 
-  const int nvars = vlistNvars(p_vlistID);
+  const auto nvars = vlistNvars(p_vlistID);
   assert(nvars > 0);
 
-  m_varlist.resize(nvars);
+  m_datarangelist.resize(nvars);
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      m_varlist[varID].gridsize = gridInqSize(vlistInqVarGrid(p_vlistID, varID));
-      m_varlist[varID].datatype = vlistInqVarDatatype(p_vlistID, varID);
-      m_varlist[varID].missval = vlistInqVarMissval(p_vlistID, varID);
-      m_varlist[varID].addoffset = vlistInqVarAddoffset(p_vlistID, varID);
-      m_varlist[varID].scalefactor = vlistInqVarScalefactor(p_vlistID, varID);
+      m_datarangelist[varID].gridsize = gridInqSize(vlistInqVarGrid(p_vlistID, varID));
+      m_datarangelist[varID].datatype = vlistInqVarDatatype(p_vlistID, varID);
+      m_datarangelist[varID].missval = vlistInqVarMissval(p_vlistID, varID);
+      m_datarangelist[varID].addoffset = vlistInqVarAddoffset(p_vlistID, varID);
+      m_datarangelist[varID].scalefactor = vlistInqVarScalefactor(p_vlistID, varID);
 
-      m_varlist[varID].check_datarange = false;
+      m_datarangelist[varID].check_datarange = false;
 
-      const bool laddoffset = IS_NOT_EQUAL(m_varlist[varID].addoffset, 0);
-      const bool lscalefactor = IS_NOT_EQUAL(m_varlist[varID].scalefactor, 1);
+      const bool laddoffset = IS_NOT_EQUAL(m_datarangelist[varID].addoffset, 0);
+      const bool lscalefactor = IS_NOT_EQUAL(m_datarangelist[varID].scalefactor, 1);
 
-      int datatype = m_varlist[varID].datatype;
+      auto datatype = m_datarangelist[varID].datatype;
 
       if (filetype == CDI_FILETYPE_NC || filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4
           || filetype == CDI_FILETYPE_NC4C || filetype == CDI_FILETYPE_NC5)
@@ -98,28 +101,28 @@ CdoStream::defVarList(int p_vlistID)
               && (filetype == CDI_FILETYPE_NC || filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC5))
             {
               datatype = CDI_DATATYPE_INT16;
-              m_varlist[varID].datatype = datatype;
+              m_datarangelist[varID].datatype = datatype;
             }
 
           if (datatype == CDI_DATATYPE_UINT16
               && (filetype == CDI_FILETYPE_NC || filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC5))
             {
               datatype = CDI_DATATYPE_INT32;
-              m_varlist[varID].datatype = datatype;
+              m_datarangelist[varID].datatype = datatype;
             }
 
           if (laddoffset || lscalefactor)
             {
               if (datatype == CDI_DATATYPE_INT8 || datatype == CDI_DATATYPE_UINT8 || datatype == CDI_DATATYPE_INT16
                   || datatype == CDI_DATATYPE_UINT16)
-                m_varlist[varID].check_datarange = true;
+                m_datarangelist[varID].check_datarange = true;
             }
           else if (Options::CheckDatarange)
             {
-              m_varlist[varID].check_datarange = true;
+              m_datarangelist[varID].check_datarange = true;
             }
         }
     }
 
-  m_vlistID = p_vlistID; /* used for -r/-a */
+  m_vlistID = p_vlistID; // used for -r/-a
 }
