@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,18 +22,19 @@
 #include "config.h" /* _FILE_OFFSET_BITS influence off_t */
 #endif
 
-#include "varlist.h"
-
 #include <sys/types.h> /* off_t */
 #include <vector>
 #include <memory>
 
+#include "datarangelist.h"
+#include "field.h"
+
 class FileStream;
 
 class CdoStream;
 
-typedef int CdiStreamID;
-typedef std::shared_ptr<CdoStream> CdoStreamID;
+using CdiStreamID = int;
+using CdoStreamID = std::shared_ptr<CdoStream>;
 #define CDO_STREAM_UNDEF nullptr
 
 class CdoStream
@@ -50,10 +51,14 @@ public:
   virtual void inqRecord(int *varID, int *levelID) = 0;
   virtual void defRecord(int varID, int levelID) = 0;
 
-  virtual void readRecord(double *data, size_t *nmiss) = 0;
-  virtual void readRecord(float *data, size_t *nmiss) = 0;
-  virtual void writeRecord(double *data, size_t nmiss) = 0;
-  virtual void writeRecord(float *data, size_t nmiss) = 0;
+  virtual void readRecord(float *p_data, size_t *nmiss) = 0;
+  virtual void readRecord(double *p_data, size_t *nmiss) = 0;
+  virtual void readRecord(Field *p_field, size_t *nmiss) = 0;
+
+  virtual void writeRecord(float *p_data, size_t nmiss) = 0;
+  virtual void writeRecord(double *p_data, size_t nmiss) = 0;
+  virtual void writeRecord(Field *p_field, size_t nmiss) = 0;
+
   virtual void copyRecord(CdoStreamID dest) = 0;
 
   virtual int inqTimestep(int tsID) = 0;
@@ -77,11 +82,11 @@ public:
   bool isopen;
 
   std::string m_name;
-  std::vector<varlist_t> m_varlist;
+  std::vector<Datarange> m_datarangelist;
 
   int m_vlistID;
   int m_tsID;
-  int m_varID; /* next varID defined with streamDefVar */
+  int m_varID; // next varID defined with streamDefVar
 
   // to be removed or to be moved to FileStream! // some operators need some refactoring for these to be able to be moved
   int m_fileID;
@@ -90,9 +95,9 @@ public:
 protected:
   CdoStream();
   virtual ~CdoStream() = 0;
-  void defVarList(int p_vlistID);
+  void defDatarangeList(int p_vlistID);
 
 private:
 };
 
-#endif /* PSTREAM_H */
+#endif /* CDOSTREAM_H */
