@@ -1,3 +1,6 @@
+#ifndef CDO_APPLY_H
+#define CDO_APPLY_H
+
 #include <vector>
 
 /*
@@ -18,7 +21,23 @@
 /*
  * supports:
  * operators with number of inputs = 1 (no others)
- * operators with arguments: written as -apply,OPERNAME,arguments
+ * operators with arguments: written as -apply,-OPERNAME,arguments
  */
+enum class ApplyStatus : int
+{
+  OK = 0,
+  ARG_TOO_MANY_OUT = 1,
+  ARG_NO_INPUT = 2,
+  ARG_VARIABLE_INPUT = 3,
+  BRACKET_USED = 4,
+  OPER_WITH_INPUT_USED = 5,
+  MISSING_CLOSING_BRACKET = 6,
+  APPLY_USED_FIRST = 7,
+  MISSING_ARG = 8,
+  MISSING_BRACKET = 9,
+  MISSING_PIPE_SYM = 10
+};
 
-std::vector<std::string> expandApply(std::vector<std::string> p_argv);
+std::vector<std::string> expandApply(const std::vector<std::string> &p_argv, ApplyStatus &p_status);
+
+#endif
