@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -36,8 +36,8 @@
 #include <curl/curl.h>
 #endif
 
-#ifdef HAVE_PROJ_API_H
-#include <proj_api.h>
+#ifdef HAVE_PROJ_H
+#include <proj.h>
 #endif
 
 #ifdef HAVE_LIBCMOR
@@ -49,8 +49,10 @@ extern "C"
 
 #include <stdio.h>
 
-#include "cdo_int.h"  // HAVE_OPENMP4
+// HAVE_OPENMP4
+#include "process_int.h"
 #include "cdo_features.h"
+#include "cimdOmp.h"
 
 #include <thread>  // std::thread::hardware_concurrency()
 
@@ -64,15 +66,6 @@ printFeatures(void)
   if (memory_size > 0) fprintf(stderr, " %zuGB", memory_size);
   fprintf(stderr, " %uthreads", std::thread::hardware_concurrency());
   fprintf(stderr, " C++%d", (int) ((__cplusplus - (__cplusplus / 10000) * 10000) / 100));
-#ifdef HAVE_CF_INTERFACE
-  fprintf(stderr, " Fortran");
-#endif
-#ifdef ENABLE_DATA
-  fprintf(stderr, " DATA");
-#endif
-#ifdef HAVE_LIBPTHREAD
-  fprintf(stderr, " PTHREADS");
-#endif
 #ifdef _OPENMP
   fprintf(stderr, " OpenMP");
 #if defined(HAVE_OPENMP45)
@@ -83,6 +76,12 @@ printFeatures(void)
   fprintf(stderr, "3");
 #endif
 #endif
+#ifdef HAVE_CF_INTERFACE
+  fprintf(stderr, " Fortran");
+#endif
+#ifdef HAVE_LIBPTHREAD
+  fprintf(stderr, " PTHREADS");
+#endif
 #ifdef HAVE_LIBHDF5
   fprintf(stderr, " HDF5");
 #endif
@@ -110,7 +109,7 @@ fprintf(stderr, " Z");
   fprintf(stderr, " UDUNITS2");
 #endif
 #ifdef HAVE_LIBPROJ
-  fprintf(stderr, " PROJ.4");
+  fprintf(stderr, " PROJ");
 #endif
 #ifdef HAVE_LIBXML2
   fprintf(stderr, " XML2");
@@ -178,8 +177,8 @@ printLibraries(void)
 */
 #ifdef HAVE_LIBPROJ
   fprintf(stderr, " proj");
-#ifdef PJ_VERSION
-  fprintf(stderr, "/%g", PJ_VERSION * 0.01);
+#ifdef PROJ_VERSION_MAJOR
+  fprintf(stderr, "/%u.%u.%u", PROJ_VERSION_MAJOR, PROJ_VERSION_MINOR, PROJ_VERSION_PATCH);
 #endif
 #endif
 
@@ -199,7 +198,7 @@ printLibraries(void)
 
 #ifdef HAVE_LIBCURL
   {
-    curl_version_info_data *version_data = curl_version_info(CURLVERSION_NOW);
+    const auto version_data = curl_version_info(CURLVERSION_NOW);
     fprintf(stderr, " curl/%s", version_data->version);
 #ifdef LIBCURL_VERSION
     if (strcmp(LIBCURL_VERSION, version_data->version) != 0) fprintf(stderr, "(h%s)", LIBCURL_VERSION);
@@ -213,7 +212,7 @@ printLibraries(void)
 }
 
 void
-cdoConfig(const char *option)
+cdoConfig(const std::string &option)
 {
   std::map<std::string, std::pair<std::string, bool>> configMap;
 
@@ -266,7 +265,7 @@ cdoConfig(const char *option)
   configMap["has-wordexp"].second = true;
 #endif
 
-  if (cstrIsEqual("all-json", option) || cstrIsEqual("all", option))
+  if ("all-json" == option || "all" == option)
     {
       std::cout << "{";
       int i = 0;
@@ -282,7 +281,7 @@ cdoConfig(const char *option)
       bool foundOption = false;
       for (const auto &entry : configMap)
         {
-          if (cstrIsEqual(entry.first.c_str(), option))
+          if (entry.first == option)
             {
               foundOption = true;
               std::cout << (entry.second.second ? "yes" : "no") << "\n";
@@ -291,7 +290,7 @@ cdoConfig(const char *option)
 
       if (!foundOption)
         {
-          fprintf(stdout, "unknown config option: %s\n", option);
+          fprintf(stdout, "unknown config option: %s\n", option.c_str());
           fprintf(stdout, "\n");
           fprintf(stdout, "Available config option:\n");
           fprintf(stdout, "\n");
