@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -37,9 +37,9 @@ struct cdo_option
   int val;
 };
 
-#define no_argument 1        // no argument to the option is expect
-#define required_argument 2  // an argument to the option is required
-#define optional_argument 3  // an argument to the option may be presented.
+constexpr int no_argument = 1;        // no argument to the option is expect
+constexpr int required_argument = 2;  // an argument to the option is required
+constexpr int optional_argument = 3;  // an argument to the option may be presented.
 
 int cdo_getopt(int argc, char *const *argv, const char *optstring);
 int cdo_getopt_long(int argc, char *const *argv, const char *optstring, const struct cdo_option *longopts, int *longindex);
