@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,37 +15,32 @@
   GNU General Public License for more details.
 */
 
+#include <vector>
+#include <string>
 #include <string.h>
 #include <time.h>
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
 #include "cdi_uuid.h"
 
-static char *ghistory = NULL;
-static size_t ghistorysize = 0;
-
 static char strtime[32];
 static char datetimestr[32];
 
 static void
 init_strtime()
 {
-  time_t tp;
-  struct tm *ltime;
-
-  tp = time(NULL);
-
+  auto tp = time(nullptr);
   if (tp != -1)
     {
-      ltime = localtime(&tp);
+      auto ltime = localtime(&tp);
       (void) strftime(strtime, sizeof(strtime), "%a %b %d %H:%M:%S %Y: ", ltime);
       (void) strftime(datetimestr, sizeof(datetimestr), "%Y-%m-%dT%H:%M:%SZ", ltime);
     }
 }
 
-static char *
+static const char *
 get_strtimeptr()
 {
   if (strlen(strtime) == 0) init_strtime();
@@ -54,68 +49,47 @@ get_strtimeptr()
 }
 
 void
-cdoInqHistory(int fileID)
+cdo_append_history(int vlistID, const char *histstring)
 {
-  if (ghistory) return;
-  /*
-    {
-      Free(ghistory);
-      ghistorysize = 0;
-      ghistory = NULL;
-    }
-  */
+  static const char *historyAttrName = "history";
 
-  ghistorysize = streamInqHistorySize(fileID);
-  if (ghistorysize > 0)
+  if (Options::CDO_Reset_History) cdiDelAtt(vlistID, CDI_GLOBAL, historyAttrName);
+
+  if (!Options::CDO_Append_History) return;
+
+  std::vector<char> ghistory;
+  const auto atttype = cdiInqAttType(vlistID, CDI_GLOBAL, historyAttrName);
+  if (atttype == CDI_DATATYPE_TXT)
     {
-      size_t len;
-      ghistory = (char *) Malloc(ghistorysize + 1);
-      ghistory[ghistorysize] = 0;
-      streamInqHistoryString(fileID, ghistory);
-      len = strlen(ghistory);
-      if (len < ghistorysize)
+      int ghistorysize = cdiInqAttLen(vlistID, CDI_GLOBAL, historyAttrName);
+      if (ghistorysize < 0) ghistorysize = 0;
+      if (ghistorysize > 0)
         {
-          /* printf("%d %d\n", len, ghistorysize); */
-          ghistorysize = len;
+          ghistorysize++;
+          ghistory.resize(ghistorysize);
+          cdiInqAttTxt(vlistID, CDI_GLOBAL, historyAttrName, ghistorysize, ghistory.data());
+          ghistory[ghistorysize-1] = 0;
+          ghistorysize = strlen(ghistory.data());
         }
     }
-}
-
-void
-cdoDefHistory(int fileID, char *histstring)
-{
-  char *strtimeptr = NULL;
-  size_t historysize = 0;
-
-  if (!CDO_Reset_History) historysize += ghistorysize + 1;
-
-  if (CDO_Append_History)
+  else if (atttype != -1)
     {
-      strtimeptr = get_strtimeptr();
-      historysize += strlen(strtimeptr) + strlen(histstring) + 1;
+      return;
     }
 
-  if (historysize)
+
+  std::string history;
+  auto strtimeptr = get_strtimeptr();
+  if (strtimeptr) history = strtimeptr;
+  history += histstring;
+
+  if (!ghistory.empty())
     {
-      char *history = (char *) Malloc(historysize);
-      history[0] = 0;
-
-      if (CDO_Append_History)
-        {
-          if (strtimeptr) strcpy(history, strtimeptr);
-          strcat(history, histstring);
-        }
-
-      if (!CDO_Reset_History)
-        if (ghistory)
-          {
-            if (CDO_Append_History) strcat(history, "\n");
-            strcat(history, ghistory);
-          }
-
-      streamDefHistory(fileID, strlen(history), history);
-      Free(history);
+      history += "\n";
+      history += ghistory.data();
     }
+
+  cdiDefAttTxt(vlistID, CDI_GLOBAL, historyAttrName, strlen(history.c_str()), history.c_str());
 }
 
 void
@@ -125,7 +99,7 @@ cdo_def_creation_date(int vlistID)
   cdiDefAttTxt(vlistID, CDI_GLOBAL, "creation_date", (int) strlen(datetimestr), datetimestr);
 }
 
-#define UUIDSTR_SIZE (CDI_UUID_SIZE * 2 + 4)
+constexpr int UUIDSTR_SIZE = (CDI_UUID_SIZE * 2 + 4);
 
 static void
 get_uuid(char uuidstr[UUIDSTR_SIZE])
