@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,20 +15,23 @@
   GNU General Public License for more details.
 */
 
+#include "config.h"
+
 #include <cdi.h>
 #include "cdo_options.h"
 #include "cdo_output.h"
-#include "config.h"
 
 #include <cstring>
 
-namespace Cdo
+namespace cdo
 {
 const char *progname;
 char File_Suffix[32];
-const char *Version = "Climate Data Operators version " VERSION " (http://mpimet.mpg.de/cdo)";
+const char *Version = "Climate Data Operators version " VERSION " (https://mpimet.mpg.de/cdo)";
 const char *Username;
-}  // namespace Cdo
+const char *DownloadPath;
+const char *IconGrids;
+}  // namespace cdo
 
 namespace Options
 {
@@ -55,19 +58,24 @@ int CMOR_Mode = false;
 
 bool cdoDiag = false;
 
-int CDO_Memtype = MEMTYPE_DOUBLE;
+MemType CDO_Memtype(MemType::Native);
 bool CDO_Parallel_Read = false;
 
 int CDO_Reduce_Dim = false;
 int CDO_Append_History = true;
 bool CDO_Reset_History = false;
+bool CDO_task = false;
+
+unsigned Random_Seed = 1;
 
 int cdoChunkType = CDI_UNDEFID;
 bool cdoOverwriteMode = false;
 bool cdoParIO = false;
 bool cdoRegulargrid = false;
-int cdoNumVarnames = 0;
-char **cdoVarnames = nullptr;
+std::vector<std::string> cdoVarnames;
+size_t cdoNumVarnames(){
+    return cdoVarnames.size();
+}
 
 bool REMAP_genweights = true;
 
@@ -83,23 +91,11 @@ bool cdoLockIO = false;
 const char *
 cdoComment(void)
 {
-  static char comment[256];
-  static bool init = false;
-
-  if (!init)
-    {
-      init = true;
-
-      int size = strlen(Cdo::Version);
-      strncat(comment, Cdo::Version, size);
-      comment[size] = 0;
-    }
-
-  return comment;
+  return cdo::Version;
 }
 
 void
-set_comp(int fileID, int filetype)
+setCompression(int fileID, int filetype)
 {
   if (Options::cdoCompress)
     {
