@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,16 +14,30 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#ifndef CDOOPTIONS_H
-#define CDOOPTIONS_H
+#ifndef CDO_OPTIONS_H
+#define CDO_OPTIONS_H
 
-namespace Cdo
+#include <vector>
+#include <string>
+
+#define ITSME (cdo_cmpstr(cdo::Username, "\x6d\x32\x31\x34\x30\x30\x33"))
+
+namespace cdo
 {
 extern const char *progname;
-extern char File_Suffix[32];  // refactor: added keyword extern
+extern char File_Suffix[32];
 extern const char *Version;
 extern const char *Username;
-}  // namespace Cdo
+extern const char *DownloadPath;
+extern const char *IconGrids;
+}  // namespace cdo
+
+enum class MemType
+{
+  Native,
+  Float,
+  Double
+};
 
 namespace Options
 {
@@ -50,21 +64,23 @@ extern int CMOR_Mode;
 
 extern bool cdoDiag;
 
-extern int CDO_Memtype;
-#define MEMTYPE_DOUBLE 1
-#define MEMTYPE_FLOAT 2
+extern MemType CDO_Memtype;
+
 extern bool CDO_Parallel_Read;
+extern bool CDO_task;
 
 extern int CDO_Reduce_Dim;
 extern int CDO_Append_History;
 extern bool CDO_Reset_History;
 
+extern unsigned Random_Seed;
+
 extern int cdoChunkType;
 extern bool cdoOverwriteMode;
 extern bool cdoParIO;
 extern bool cdoRegulargrid;
-extern int cdoNumVarnames;
-extern char **cdoVarnames;
+size_t  cdoNumVarnames();
+extern std::vector<std::string> cdoVarnames;
 
 extern bool REMAP_genweights;
 
@@ -78,7 +94,6 @@ extern bool cdoLockIO;
 
 const char *cdoComment(void);
 
-void
-set_comp(int fileID, int filetype);
+void setCompression(int fileID, int filetype);
 
 #endif
