@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,9 +34,8 @@ int cdoDebugExt = 0;  //  Debug level for the KNMI extensions
 int PSTREAM = 0;
 int PROCESS = 0;
 int PIPE = 0;
-int PIPE_STREAM = 0; 
+int PIPE_STREAM = 0;
 int FILE_STREAM = 0;
-int ARGUMENT = 0;
 int PTHREAD = 0;
 int PROCESS_MANAGER = 0;
 
@@ -44,49 +43,40 @@ void
 printDebugOptions()
 {
   std::cout << "DebugLevels:" << std::endl
-            << "   0: off " << std::endl
-            << "   1: on : std debug: 2 cdi,32 cdo, 512 processes, 1024 process manager" << std::endl
-            << "   2: cdi" << std::endl
-            << "   4: memory" << std::endl
-            << "   8: file" << std::endl
-            << "  16: format" << std::endl
-            << "  32: cdo" << std::endl
-            << "  64: stream" << std::endl
-            << " 128: pipe" << std::endl
-            << " 256: pthread" << std::endl
-            << " 512: process" << std::endl
-            << " 1024: process manager" << std::endl;
+            << "  Cdi:" << std::endl
+            << "     0: off " << std::endl
+            << "     1: on : std debug: 2 cdi,32 cdo, 512 processes, 1024 process manager" << std::endl
+            << "     2: cdi" << std::endl
+            << "     3: memory" << std::endl
+            << "     4: file" << std::endl
+            << "     5: format" << std::endl
+            << "  Cdo:" << std::endl
+            << "     6: cdo" << std::endl
+            << "     7: pipeStream" << std::endl
+            << "     8: fileStream" << std::endl
+            << "     9: pipe" << std::endl
+            << "    10: pthread" << std::endl
+            << "    11: process" << std::endl
+            << "    12: process manager" << std::endl;
 }
 
+namespace cdo
+{
 void
-SetDebug(int p_debug_level)
+setDebug(int p_debug_level)
 {
-  /*
-  p_debug_level   0: off
-  p_debug_level   1: on
-  p_debug_level   2: cdi
-  p_debug_level   4: memory
-  p_debug_level   8: file
-  p_debug_level  16: format
-  p_debug_level  32: cdo
-  p_debug_level  64: stream
-  p_debug_level 128: pipe
-  p_debug_level 256: pthread
-  p_debug_level 512: process
-  */
   MpMO::DebugLevel = p_debug_level;
   if (p_debug_level == 1 || (p_debug_level & 32)) cdoDebug = 1;
-  if (p_debug_level == 1 || (p_debug_level & 64)) PSTREAM = 1;
-  if (p_debug_level == 1 || (p_debug_level & 512)) PROCESS = 1;
-  if (p_debug_level == 1 || (p_debug_level & 1024)) PROCESS_MANAGER = 1;
+  if (p_debug_level == 1 || (p_debug_level & 64)) FILE_STREAM = 1;
+  if (p_debug_level == 1 || (p_debug_level & 128)) PIPE_STREAM = 1;
 #ifdef HAVE_LIBPTHREAD
-  if ((p_debug_level & 128)) PIPE = 1;
-  if ((p_debug_level & 256)) PTHREAD = 1;
+  if ((p_debug_level & 256)) PIPE = 1;
+  if ((p_debug_level & 512)) PTHREAD = 1;
 #endif
+  if (p_debug_level == 1 || (p_debug_level & 1024)) PROCESS = 1;
+  if (p_debug_level == 1 || (p_debug_level & 2048)) PROCESS_MANAGER = 1;
 }
 
-namespace CdoOut
-{
 void (*exitProgram)(void);
 const char *(*getContext)(void);
 
@@ -101,7 +91,7 @@ setContextFunction(const char *(*func)(void) )
 {
   getContext = func;
 }
-}  // namespace CdoOut
+}  // namespace cdo
 
 char *
 getGRB2ErrStr(const char *progname)
@@ -122,7 +112,7 @@ getNCErrString(int filetype, const char *progname)
                         : ((filetype == CDI_FILETYPE_NC2) ? "2" : ((filetype == CDI_FILETYPE_NC5) ? "5" : ""));
 #ifdef HAVE_LIBNETCDF
   const char *format = "%s was build with a NetCDF version which doesn't support NetCDF%s data!";
-  size_t finalSize = snprintf(nullptr, 0, format, progname, ncv, ncv);
+  size_t finalSize = snprintf(nullptr, 0, format, progname, ncv);
   char *errStr = (char *) malloc(finalSize + 1);
   sprintf(errStr, format, progname, ncv);
 #else
@@ -169,7 +159,7 @@ checkForMissingLib(int filetype, const char *progname)
 void
 cdiOpenError(int cdiErrno, const std::string &format, const char *path)
 {
-  std::string context = CdoOut::getContext();
+  std::string context = cdo::getContext();
   MpMO::PrintCerr(Red("%s: ") + format + "\n" + std::string(context.size() + 2, ' ') + "%s", context, path,
                   cdiStringError(cdiErrno));
   if (cdiErrno == CDI_ELIBNAVAIL)
@@ -186,7 +176,7 @@ cdiOpenError(int cdiErrno, const std::string &format, const char *path)
 
   if (MpMO::exitOnError)
     {
-      CdoOut::exitProgram();
+      cdo::exitProgram();
     }
 }
 
@@ -209,12 +199,14 @@ query_user_exit(const char *argument)
     {
       if (nbr_itr++ > USR_RPL_MAX_NBR)
         {
-          (void) fprintf(stdout, "\n%s: ERROR %d failed attempts to obtain valid interactive input.\n", CdoOut::getContext(), nbr_itr - 1);
+          (void) fprintf(stdout, "\n%s: ERROR %d failed attempts to obtain valid interactive input.\n", cdo::getContext(),
+                         nbr_itr - 1);
           exit(EXIT_FAILURE);
         }
 
-      if (nbr_itr > 1) (void) fprintf(stdout, "%s: ERROR Invalid response.\n", CdoOut::getContext());
-      (void) fprintf(stdout, "%s: %s exists ---`e'xit, or `o'verwrite (delete existing file) (e/o)? ", CdoOut::getContext(), argument);
+      if (nbr_itr > 1) (void) fprintf(stdout, "%s: ERROR Invalid response.\n", cdo::getContext());
+      (void) fprintf(stdout, "%s: %s exists ---`e'xit, or `o'verwrite (delete existing file) (e/o)? ", cdo::getContext(),
+                     argument);
       (void) fflush(stdout);
       if (fgets(usr_rpl, USR_RPL_MAX_LNG, stdin) == nullptr) continue;
 
