@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,32 +15,31 @@
   GNU General Public License for more details.
 */
 #include <cdi.h>
-#include "cdo_options.h"
-#include <vector>
 
+#include "cdo_options.h"
 #include "cdo_output.h"
-#include "dmemory.h"
 #include "compare.h"
 #include "cdi_lockedIO.h"
+#include "array.h"
 
-bool *
-cdo_read_timestepmask(const char *maskfile, int *n)
+int
+cdo_read_timestepmask(const char *maskfile, std::vector<bool> &imask)
 {
-  *n = 0;
+  int n = -1;
 
-  int streamID = streamOpenReadLocked(maskfile);
-  int vlistID = streamInqVlist(streamID);
+  auto streamID = streamOpenReadLocked(maskfile);
+  const auto vlistID = streamInqVlist(streamID);
 
-  int nvars = vlistNvars(vlistID);
+  const auto nvars = vlistNvars(vlistID);
   if (nvars > 1) cdoAbort("timestepmask %s contains more than one variable!", maskfile);
 
-  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
+  const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
   if (gridsize > 1) cdoAbort("timestepmask %s has more than one gridpoint!", maskfile);
 
-  int nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
+  const auto nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
   if (nlev > 1) cdoAbort("timestepmask %s has more than one level!", maskfile);
 
-  int nts = vlistNtsteps(vlistID);
+  auto nts = vlistNtsteps(vlistID);
   if (nts == -1)
     {
       nts = 0;
@@ -54,13 +53,15 @@ cdo_read_timestepmask(const char *maskfile, int *n)
   else if (Options::cdoVerbose)
     cdoPrint("%s: found %i timeSteps in %s", __func__, nts, maskfile);
 
-  *n = nts;
-  bool *imask = (bool *) Malloc(nts * sizeof(bool));
+  n = nts;
+  imask.resize(nts);
 
-  int nrecs;
   int tsID = 0;
-  while ((nrecs = streamInqTimestep(streamID, tsID)))
+  while (true)
     {
+      auto nrecs = streamInqTimestep(streamID, tsID);
+      if (nrecs == 0) break;
+
       if (nrecs != 1) cdoAbort("Internal error; unexprected number of records!");
 
       int varID, levelID;
@@ -76,30 +77,27 @@ cdo_read_timestepmask(const char *maskfile, int *n)
 
   streamClose(streamID);
 
-  return imask;
+  return n;
 }
 
-bool *
-cdo_read_mask(const char *maskfile, size_t *n)
+size_t
+cdo_read_mask(const char *maskfile, std::vector<bool> &imask)
 {
-  *n = 0;
+  const auto streamID = streamOpenReadLocked(maskfile);
+  const auto vlistID = streamInqVlist(streamID);
 
-  int streamID = streamOpenReadLocked(maskfile);
-  int vlistID = streamInqVlist(streamID);
-
-  int nvars = vlistNvars(vlistID);
+  const auto nvars = vlistNvars(vlistID);
   if (nvars > 1) cdoAbort("Mask %s contains more than one variable!", maskfile);
 
-  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
+  const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
 
-  int nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
+  const auto nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
   if (nlev > 1) cdoAbort("Mask %s has more than one level!", maskfile);
 
-  *n = gridsize;
-  bool *imask = (bool *) Malloc(gridsize * sizeof(bool));
-  std::vector<double> array(gridsize);
+  imask.resize(gridsize);
+  Varray<double> array(gridsize);
 
-  int nrecs = streamInqTimestep(streamID, 0);
+  const auto nrecs = streamInqTimestep(streamID, 0);
   if (nrecs != 1) cdoAbort("Internal error; unexprected number of records!");
 
   int varID, levelID;
@@ -110,30 +108,27 @@ cdo_read_mask(const char *maskfile, size_t *n)
 
   for (size_t i = 0; i < gridsize; ++i) imask[i] = IS_NOT_EQUAL(array[i], 0);
 
-  return imask;
+  return gridsize;
 }
 
-int *
-cdo_read_index(const char *indexfile, size_t *n)
+size_t
+cdo_read_index(const char *indexfile, std::vector<int> &index)
 {
-  *n = 0;
+  const auto streamID = streamOpenReadLocked(indexfile);
+  const auto vlistID = streamInqVlist(streamID);
 
-  int streamID = streamOpenReadLocked(indexfile);
-  int vlistID = streamInqVlist(streamID);
-
-  int nvars = vlistNvars(vlistID);
+  const auto nvars = vlistNvars(vlistID);
   if (nvars > 1) cdoAbort("Index file %s contains more than one variable!", indexfile);
 
-  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
+  const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
 
-  int nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
+  const auto nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
   if (nlev > 1) cdoAbort("Index file %s has more than one level!", indexfile);
 
-  *n = gridsize;
-  int *index = (int *) Malloc(gridsize * sizeof(int));
-  std::vector<double> array(gridsize);
+  index.resize(gridsize);
+  Varray<double> array(gridsize);
 
-  int nrecs = streamInqTimestep(streamID, 0);
+  const auto nrecs = streamInqTimestep(streamID, 0);
   if (nrecs != 1) cdoAbort("Internal error; unexprected number of records!");
 
   int varID, levelID;
@@ -142,7 +137,7 @@ cdo_read_index(const char *indexfile, size_t *n)
   streamReadRecord(streamID, array.data(), &nmiss);
   streamClose(streamID);
 
-  for (size_t i = 0; i < gridsize; ++i) index[i] = (int) lround(array[i]) - 1;
+  for (size_t i = 0; i < gridsize; ++i) index[i] = (int) std::lround(array[i]) - 1;
 
-  return index;
+  return gridsize;
 }
