@@ -8,11 +8,10 @@
 const char *seas_name_dec[4] = { "DJF", "MAM", "JJA", "SON" };
 const char *seas_name_jan[4] = { "JFM", "AMJ", "JAS", "OND" };
 
-static int season_start = START_DEC;
-
 int
-get_season_start(void)
+getSeasonStart(void)
 {
+  static int season_start = START_DEC;
   static bool lgetenv = true;
 
   if (lgetenv)
@@ -22,9 +21,9 @@ get_season_start(void)
       char *envstr = getenv("CDO_SEASON_START");
       if (envstr)
         {
-          if (cstrIsEqual(envstr, "DEC"))
+          if (cdo_cmpstr(envstr, "DEC"))
             season_start = START_DEC;
-          else if (cstrIsEqual(envstr, "JAN"))
+          else if (cdo_cmpstr(envstr, "JAN"))
             season_start = START_JAN;
 
           if (Options::cdoVerbose)
@@ -41,18 +40,18 @@ get_season_start(void)
 }
 
 void
-get_season_name(const char *seas_name[])
+getSeasonName(const char *seas_name[])
 {
-  if (get_season_start() == START_DEC)
+  if (getSeasonStart() == START_DEC)
     for (int i = 0; i < 4; ++i) seas_name[i] = seas_name_dec[i];
   else
     for (int i = 0; i < 4; ++i) seas_name[i] = seas_name_jan[i];
 }
 
 int
-month_to_season(int month)
+monthToSeason(int month)
 {
-  int season_start = get_season_start();
+  const auto season_start = getSeasonStart();
   int seas = -1;
 
   if (month < 0 || month > 16) cdoAbort("Month %d out of range!", month);
