@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,37 +16,47 @@
 */
 
 #include <algorithm>
-#include <vector>
 
 #include <cdi.h>
 #include "cdo_options.h"
 #include "util_string.h"
 #include "cdo_vlist.h"
-#include "dmemory.h"
 #include "compare.h"
 #include "cdo_output.h"
 #include "functs.h"
+#include "array.h"
 
-char *
-cdoVlistInqVarName(int vlistID, int varID, char *name)
+void
+vlistDefineTimestepType(int vlistID, int operfunc)
 {
-  vlistInqVarName(vlistID, varID, name);
-  return name;
+  int stepType = -1;
+  // clang-format off
+  if      (operfunc == func_mean)  stepType = TSTEP_AVG;
+  else if (operfunc == func_avg)   stepType = TSTEP_AVG;
+  else if (operfunc == func_sum)   stepType = TSTEP_SUM;
+  else if (operfunc == func_range) stepType = TSTEP_RANGE;
+  else if (operfunc == func_min)   stepType = TSTEP_MIN;
+  else if (operfunc == func_max)   stepType = TSTEP_MAX;
+  // clang-format on
+
+  if (stepType != -1)
+    {
+      const auto nvars = vlistNvars(vlistID);
+      for (int varID = 0; varID < nvars; ++varID) vlistDefVarTsteptype(vlistID, varID, stepType);
+    }
 }
 
 double
 cdoZaxisInqLevel(int zaxisID, int levelID)
 {
-  int zaxistype = zaxisInqType(zaxisID);
-  double level
-      = zaxisInqLevels(zaxisID, nullptr) ? zaxisInqLevel(zaxisID, levelID) : (zaxistype == ZAXIS_SURFACE) ? 0 : levelID + 1;
-  return level;
+  const auto zaxistype = zaxisInqType(zaxisID);
+  return zaxisInqLevels(zaxisID, nullptr) ? zaxisInqLevel(zaxisID, levelID) : (zaxistype == ZAXIS_SURFACE) ? 0.0 : levelID + 1.0;
 }
 
 int
 cdoZaxisInqLevels(int zaxisID, double *levels)
 {
-  int size = zaxisInqLevels(zaxisID, nullptr);
+  auto size = zaxisInqLevels(zaxisID, nullptr);
 
   if (levels)
     {
@@ -58,7 +68,7 @@ cdoZaxisInqLevels(int zaxisID, double *levels)
           if (size == 1 && zaxisInqType(zaxisID) == ZAXIS_SURFACE)
             levels[0] = 0;
           else
-            for (int i = 0; i < size; ++i) levels[i] = i + 1;
+            for (int i = 0; i < size; ++i) levels[i] = i + 1.0;
         }
     }
 
@@ -70,18 +80,17 @@ compare_lat_reg2d(size_t ysize, int gridID1, int gridID2)
 {
   if (ysize > 1)
     {
-      std::vector<double> yvals1(ysize);
-      std::vector<double> yvals2(ysize);
-
-      gridInqYvals(gridID1, &yvals1[0]);
-      gridInqYvals(gridID2, &yvals2[0]);
+      Varray<double> yvals1(ysize), yvals2(ysize);
+      const auto ny1 = gridInqYvals(gridID1, &yvals1[0]);
+      const auto ny2 = gridInqYvals(gridID2, &yvals2[0]);
+      if (ny1 == 0 || ny2 == 0) return;
 
       if (IS_EQUAL(yvals1[0], yvals2[ysize - 1]) && IS_EQUAL(yvals1[ysize - 1], yvals2[0]))
         {
           if (yvals1[0] > yvals2[0])
-            cdoWarning("Latitude orientation differ! First grid: N->S; second grid: S->N");
+            cdoAbort("Latitude orientation differ! First grid: N->S; second grid: S->N");
           else
-            cdoWarning("Latitude orientation differ! First grid: S->N; second grid: N->S");
+            cdoAbort("Latitude orientation differ! First grid: S->N; second grid: N->S");
         }
       else
         {
@@ -100,11 +109,10 @@ compare_lon_reg2d(size_t xsize, int gridID1, int gridID2)
 {
   if (xsize > 1)
     {
-      std::vector<double> xvals1(xsize);
-      std::vector<double> xvals2(xsize);
-
-      gridInqXvals(gridID1, &xvals1[0]);
-      gridInqXvals(gridID2, &xvals2[0]);
+      Varray<double> xvals1(xsize), xvals2(xsize);
+      const auto nx1 = gridInqXvals(gridID1, &xvals1[0]);
+      const auto nx2 = gridInqXvals(gridID2, &xvals2[0]);
+      if (nx1 == 0 || nx2 == 0) return;
 
       for (size_t i = 0; i < xsize; ++i)
         if (std::fabs(xvals1[i] - xvals2[i]) > 3.e-5)
@@ -121,25 +129,21 @@ compare_grid_unstructured(int gridID1, int gridID2)
   if (gridInqXvals(gridID1, nullptr) && gridInqXvals(gridID1, nullptr) == gridInqXvals(gridID2, nullptr)
       && gridInqYvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr) == gridInqYvals(gridID2, nullptr))
     {
-      size_t gridsize = gridInqSize(gridID1);
-      std::vector<double> xvals1(gridsize);
-      std::vector<double> xvals2(gridsize);
-      std::vector<double> yvals1(gridsize);
-      std::vector<double> yvals2(gridsize);
-
-      gridInqXvals(gridID1, &xvals1[0]);
-      gridInqXvals(gridID2, &xvals2[0]);
-      gridInqYvals(gridID1, &yvals1[0]);
-      gridInqYvals(gridID2, &yvals2[0]);
+      const auto gridsize = gridInqSize(gridID1);
+      Varray<double> xvals1(gridsize), xvals2(gridsize), yvals1(gridsize), yvals2(gridsize);
+      gridInqXvals(gridID1, xvals1.data());
+      gridInqXvals(gridID2, xvals2.data());
+      gridInqYvals(gridID1, yvals1.data());
+      gridInqYvals(gridID2, yvals2.data());
 
       size_t inc = gridsize > 10000 ? gridsize / 1000 : 1;
       for (size_t i = 0; i < gridsize; i += inc)
         if (std::fabs(xvals1[i] - xvals2[i]) > 2.e-5 || std::fabs(yvals1[i] - yvals2[i]) > 2.e-5)
           {
-            // printf("%d %g %g %g %g %g %g\n", i, xvals1[i], xvals2[i],
-            // yvals1[i], yvals2[i], xvals1[i] - xvals2[i], yvals1[i] -
-            // yvals2[i]);
             cdoWarning("Geographic location of some grid points differ!");
+            if (Options::cdoVerbose)
+              printf("cell=%zu x1=%g x2=%g y1=%g y2=%g dx=%g dy=%g\n", i + 1, xvals1[i], xvals2[i],
+                     yvals1[i], yvals2[i], xvals1[i] - xvals2[i], yvals1[i] - yvals2[i]);
             break;
           }
     }
@@ -154,8 +158,8 @@ cdoCompareGrids(int gridID1, int gridID2)
     {
       if (gridInqType(gridID1) == GRID_GAUSSIAN || gridInqType(gridID1) == GRID_LONLAT)
         {
-          size_t xsize = gridInqXsize(gridID1);
-          size_t ysize = gridInqYsize(gridID1);
+          const auto xsize = gridInqXsize(gridID1);
+          const auto ysize = gridInqYsize(gridID1);
 
           if (ysize == gridInqYsize(gridID2))
             compare_lat_reg2d(ysize, gridID1, gridID2);
@@ -195,16 +199,15 @@ zaxisCheckLevels(int zaxisID1, int zaxisID2)
 {
   if (zaxisID1 != zaxisID2)
     {
-      int nlev1 = zaxisInqSize(zaxisID1);
-      int nlev2 = zaxisInqSize(zaxisID2);
+      const auto nlev1 = zaxisInqSize(zaxisID1);
+      const auto nlev2 = zaxisInqSize(zaxisID2);
       if (nlev1 != nlev2) cdoAbort("Number of levels of the input parameters do not match!");
 
-      std::vector<double> lev1(nlev1);
-      std::vector<double> lev2(nlev1);
+      Varray<double> lev1(nlev1), lev2(nlev1);
       cdoZaxisInqLevels(zaxisID1, &lev1[0]);
       cdoZaxisInqLevels(zaxisID2, &lev2[0]);
 
-      bool ldiffer = false;
+      auto ldiffer = false;
       for (int i = 0; i < nlev1; ++i)
         if (IS_NOT_EQUAL(lev1[i], lev2[i]))
           {
@@ -237,12 +240,11 @@ static void
 vlistCheckNames(int vlistID1, int vlistID2)
 {
   int varID;
-  int nvars = vlistNvars(vlistID1);
+  auto nvars = vlistNvars(vlistID1);
 
   // std::vector<std::array<char,CDI_MAX_NAME>> names1(nvars); C++14?
   // std::vector<std::array<char,CDI_MAX_NAME>> names2(nvars);
-  std::vector<std::vector<char>> names1(nvars);
-  std::vector<std::vector<char>> names2(nvars);
+  std::vector<std::vector<char>> names1(nvars), names2(nvars);
   for (varID = 0; varID < nvars; varID++) names1[varID].resize(CDI_MAX_NAME);
   for (varID = 0; varID < nvars; varID++) names2[varID].resize(CDI_MAX_NAME);
   for (varID = 0; varID < nvars; varID++) vlistInqVarName(vlistID1, varID, names1[varID].data());
@@ -252,7 +254,7 @@ vlistCheckNames(int vlistID1, int vlistID2)
   std::sort(names2.begin(), names2.end());
 
   for (varID = 0; varID < nvars; varID++)
-    if (!cstrIsEqual(names1[varID].data(), names2[varID].data())) break;
+    if (!cdo_cmpstr(names1[varID].data(), names2[varID].data())) break;
 
   if (varID == nvars) cdoPrint("Use CDO option --sortname to sort the parameter by name (NetCDF only)!");
 }
@@ -260,17 +262,16 @@ vlistCheckNames(int vlistID1, int vlistID2)
 void
 vlistCompare(int vlistID1, int vlistID2, int flag)
 {
-  int varID;
-  bool lchecknames = false;
+  auto lchecknames = false;
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
   if (nvars != vlistNvars(vlistID2)) cdoAbort("Input streams have different number of variables per timestep!");
 
   if (vlistNrecs(vlistID1) != vlistNrecs(vlistID2))
     cdoAbort("Input streams have different number of %s per timestep!", nvars == 1 ? "layers" : "records");
 
-  for (varID = 0; varID < nvars; varID++)
+  for (int varID = 0; varID < nvars; varID++)
     {
       if (nvars > 1)
         {
@@ -289,21 +290,25 @@ vlistCompare(int vlistID1, int vlistID2, int flag)
       if (flag & CMP_GRIDSIZE)
         {
           if (gridInqSize(vlistInqVarGrid(vlistID1, varID)) != gridInqSize(vlistInqVarGrid(vlistID2, varID)))
-            cdoAbort("Grid size of the input parameters do not match!");
+            {
+              char name[CDI_MAX_NAME];
+              vlistInqVarName(vlistID1, varID, name);
+              cdoAbort("Grid size of the input parameter %s do not match!", name);
+            }
         }
 
       if (flag & CMP_NLEVEL)
         {
-          int zaxisID1 = vlistInqVarZaxis(vlistID1, varID);
-          int zaxisID2 = vlistInqVarZaxis(vlistID2, varID);
+          const auto zaxisID1 = vlistInqVarZaxis(vlistID1, varID);
+          const auto zaxisID2 = vlistInqVarZaxis(vlistID2, varID);
           if (zaxisCheckLevels(zaxisID1, zaxisID2) != 0) break;
         }
     }
 
   if (flag & CMP_GRID)
     {
-      int gridID1 = vlistInqVarGrid(vlistID1, 0);
-      int gridID2 = vlistInqVarGrid(vlistID2, 0);
+      const auto gridID1 = vlistInqVarGrid(vlistID1, 0);
+      const auto gridID2 = vlistInqVarGrid(vlistID2, 0);
       cdoCompareGrids(gridID1, gridID2);
     }
 
@@ -313,9 +318,9 @@ vlistCompare(int vlistID1, int vlistID2, int flag)
 int
 vlistCompareX(int vlistID1, int vlistID2, int flag)
 {
-  int nvars = vlistNvars(vlistID1);
-  int nvars2 = vlistNvars(vlistID2);
-  int nlevels2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, 0));
+  const auto nvars = vlistNvars(vlistID1);
+  const auto nvars2 = vlistNvars(vlistID2);
+  const auto nlevels2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, 0));
 
   if (nvars2 != 1) cdoAbort("Internal problem, vlistCompareX() called with unexpected vlistID2 argument!");
 
@@ -336,8 +341,8 @@ vlistCompareX(int vlistID1, int vlistID2, int flag)
 
   if (flag & CMP_GRID)
     {
-      int gridID1 = vlistInqVarGrid(vlistID1, 0);
-      int gridID2 = vlistInqVarGrid(vlistID2, 0);
+      const auto gridID1 = vlistInqVarGrid(vlistID1, 0);
+      const auto gridID2 = vlistInqVarGrid(vlistID2, 0);
       cdoCompareGrids(gridID1, gridID2);
     }
 
@@ -348,11 +353,10 @@ void
 vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &mapOfVarIDs)
 {
   int varID1, varID2;
-  int nvars1 = vlistNvars(vlistID1);
-  int nvars2 = vlistNvars(vlistID2);
+  const auto nvars1 = vlistNvars(vlistID1);
+  const auto nvars2 = vlistNvars(vlistID2);
 
-  std::vector<std::vector<char>> names1(nvars1);
-  std::vector<std::vector<char>> names2(nvars2);
+  std::vector<std::vector<char>> names1(nvars1), names2(nvars2);
   for (varID1 = 0; varID1 < nvars1; varID1++) names1[varID1].resize(CDI_MAX_NAME);
   for (varID2 = 0; varID2 < nvars2; varID2++) names2[varID2].resize(CDI_MAX_NAME);
   for (varID1 = 0; varID1 < nvars1; varID1++) vlistInqVarName(vlistID1, varID1, names1[varID1].data());
@@ -364,7 +368,7 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           for (varID1 = 0; varID1 < nvars1; varID1++)
             {
-              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;
+              if (cdo_cmpstr(names1[varID1].data(), names2[varID2].data())) break;
             }
           if (varID1 == nvars1)
             {
@@ -382,7 +386,7 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           for (varID2 = 0; varID2 < nvars2; varID2++)
             {
-              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;
+              if (cdo_cmpstr(names1[varID1].data(), names2[varID2].data())) break;
             }
           if (varID2 == nvars2)
             {
@@ -431,15 +435,15 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
 
       if (flag & CMP_NLEVEL)
         {
-          int zaxisID1 = vlistInqVarZaxis(vlistID1, varID1);
-          int zaxisID2 = vlistInqVarZaxis(vlistID2, varID2);
+          const auto zaxisID1 = vlistInqVarZaxis(vlistID1, varID1);
+          const auto zaxisID2 = vlistInqVarZaxis(vlistID2, varID2);
           if (zaxisCheckLevels(zaxisID1, zaxisID2) != 0) break;
         }
 
       if (flag & CMP_GRID && varID1 == mapOfVarIDs.begin()->first)
         {
-          int gridID1 = vlistInqVarGrid(vlistID1, varID1);
-          int gridID2 = vlistInqVarGrid(vlistID2, varID2);
+          const auto gridID1 = vlistInqVarGrid(vlistID1, varID1);
+          const auto gridID2 = vlistInqVarGrid(vlistID2, varID2);
           cdoCompareGrids(gridID1, gridID2);
         }
     }
@@ -448,28 +452,22 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
 bool
 vlistIsSzipped(int vlistID)
 {
-  bool lszip = false;
-
-  const int nvars = vlistNvars(vlistID);
+  const auto nvars = vlistNvars(vlistID);
   for (int varID = 0; varID < nvars; varID++)
     {
-      const int comptype = vlistInqVarCompType(vlistID, varID);
-      if (comptype == CDI_COMPRESS_SZIP)
-        {
-          lszip = true;
-          break;
-        }
+      const auto comptype = vlistInqVarCompType(vlistID, varID);
+      if (comptype == CDI_COMPRESS_SZIP) return true;
     }
 
-  return lszip;
+  return false;
 }
 
 int
 vlistInqNWPV(int vlistID, int varID)
 {
-  int datatype = vlistInqVarDatatype(vlistID, varID);
+  const auto datatype = vlistInqVarDatatype(vlistID, varID);
   // number of words per value; real:1  complex:2
-  int nwpv = (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64) ? 2 : 1;
+  const auto nwpv = (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64) ? 2 : 1;
 
   return nwpv;
 }
@@ -477,14 +475,14 @@ vlistInqNWPV(int vlistID, int varID)
 size_t
 vlist_check_gridsize(int vlistID)
 {
-  bool lerror = false;
-  const size_t ngp = gridInqSize(vlistGrid(vlistID, 0));
+  auto lerror = false;
+  const auto ngp = gridInqSize(vlistGrid(vlistID, 0));
 
   // check gridsize
-  const int ngrids = vlistNgrids(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
-      int gridID = vlistGrid(vlistID, index);
+      const auto gridID = vlistGrid(vlistID, index);
       if (ngp != gridInqSize(gridID))
         {
           lerror = true;
@@ -498,7 +496,7 @@ vlist_check_gridsize(int vlistID)
       cdoPrint("Horizontal grids found:");
       for (int index = 0; index < ngrids; ++index)
         {
-          int gridID = vlistGrid(vlistID, index);
+          const auto gridID = vlistGrid(vlistID, index);
           cdoPrint("  grid=%d  type=%s  points=%zu", index + 1, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID));
         }
       cdoAbort("The input stream contains variables on different horizontal grids!");
@@ -507,33 +505,32 @@ vlist_check_gridsize(int vlistID)
   return ngp;
 }
 
-double *
-vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlevf, int *rnhlevh)
+void
+vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlevf, int *rnhlevh, Varray<double> &vct)
 {
-  double *vct = nullptr;
   int zaxisIDh = -1;
   int nhlev = 0, nhlevf = 0, nhlevh = 0;
   int nvct = 0;
 
-  bool lhavevct = false;
-  int nzaxis = vlistNzaxis(vlistID);
+  auto lhavevct = false;
+  auto nzaxis = vlistNzaxis(vlistID);
   for (int iz = 0; iz < nzaxis; ++iz)
     {
       // bool mono_level = false;
-      bool mono_level = true;
-      int zaxisID = vlistZaxis(vlistID, iz);
-      int nlevel = zaxisInqSize(zaxisID);
-      int zaxistype = zaxisInqType(zaxisID);
+      auto mono_level = true;
+      auto zaxisID = vlistZaxis(vlistID, iz);
+      auto nlevel = zaxisInqSize(zaxisID);
+      auto zaxistype = zaxisInqType(zaxisID);
 
       if (Options::cdoVerbose)
-        cdoPrint("ZAXIS_HYBRID = %d ZAXIS_HYBRID_HALF=%d nlevel=%d mono_level=%d", zaxisInqType(zaxisID) == ZAXIS_HYBRID,
+        cdoPrint("ZAXIS_HYBRID=%d ZAXIS_HYBRID_HALF=%d nlevel=%d mono_level=%d", zaxisInqType(zaxisID) == ZAXIS_HYBRID,
                  zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF, nlevel, mono_level);
 
       if ((zaxistype == ZAXIS_HYBRID || zaxistype == ZAXIS_HYBRID_HALF) && nlevel > 1 && !mono_level)
         {
-          int l;
-          std::vector<double> level(nlevel);
+          Varray<double> level(nlevel);
           cdoZaxisInqLevels(zaxisID, &level[0]);
+          int l;
           for (l = 0; l < nlevel; l++)
             {
               if ((l + 1) != (int) (level[l] + 0.5)) break;
@@ -554,8 +551,8 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
                   nhlevf = nhlev;
                   nhlevh = nhlevf + 1;
 
-                  vct = (double *) Malloc(nvct * sizeof(double));
-                  zaxisInqVct(zaxisID, vct);
+                  vct.resize(nvct);
+                  zaxisInqVct(zaxisID, vct.data());
                   if (Options::cdoVerbose)
                     cdoPrint("Detected half-level model definition : nlevel == (nvct/2 - 1) (nlevel: %d, nvct: %d, nhlevf: %d, "
                              "nhlevh: %d) ",
@@ -572,8 +569,8 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
                   nhlevf = nhlev - 1;
                   nhlevh = nhlev;
 
-                  vct = (double *) Malloc(nvct * sizeof(double));
-                  zaxisInqVct(zaxisID, vct);
+                  vct.resize(nvct);
+                  zaxisInqVct(zaxisID, vct.data());
                   if (Options::cdoVerbose)
                     cdoPrint(
                         "Detected full-level model definition : nlevel == (nvct/2) (nlevel: %d, nvct: %d, nhlevf: %d, nhlevh: %d) ",
@@ -584,12 +581,10 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
             {
               if (!lhavevct)
                 {
-                  int vctsize;
+                  Varray<double> rvct(nvct);
+                  zaxisInqVct(zaxisID, rvct.data());
+
                   int voff = 4;
-
-                  std::vector<double> rvct(nvct);
-                  zaxisInqVct(zaxisID, &rvct[0]);
-
                   if ((int) (rvct[0] + 0.5) == 100000 && rvct[voff] < rvct[voff + 1])
                     {
                       lhavevct = true;
@@ -598,10 +593,10 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
                       nhlevf = nhlev;
                       nhlevh = nhlev + 1;
 
-                      vctsize = 2 * nhlevh;
-                      vct = (double *) Malloc(vctsize * sizeof(double));
+                      int vctsize = 2 * nhlevh;
+                      vct.resize(vctsize);
 
-                      /* calculate VCT for LM */
+                      // calculate VCT for LM
 
                       for (int i = 0; i < vctsize / 2; i++)
                         {
@@ -633,25 +628,23 @@ vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlev
   *rnhlev = nhlev;
   *rnhlevf = nhlevf;
   *rnhlevh = nhlevh;
-
-  return vct;
 }
 
 void
 vlist_change_hybrid_zaxis(int vlistID1, int vlistID2, int zaxisID1, int zaxisID2)
 {
   int nvct0 = 0;
-  std::vector<double> vct;
+  Varray<double> vct;
 
-  int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (int i = 0; i < nzaxis; ++i)
     {
-      int zaxisID = vlistZaxis(vlistID1, i);
-      int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, i);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
       if (zaxisID == zaxisID1 && nlevel > 1)
         {
-          int nvct = zaxisInqVctSize(zaxisID);
+          const auto nvct = zaxisInqVctSize(zaxisID);
           if (!vct.size())
             {
               nvct0 = nvct;
@@ -672,44 +665,77 @@ vlist_change_hybrid_zaxis(int vlistID1, int vlistID2, int zaxisID1, int zaxisID2
 int
 vlist_get_psvarid(int vlistID, int zaxisID)
 {
-  int psvarid = -1;
-  char name[CDI_MAX_NAME];
   char psname[CDI_MAX_NAME];
-  psname[0] = 0;
-  cdiZaxisInqKeyStr(zaxisID, CDI_KEY_PSNAME, CDI_MAX_NAME, psname);
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_PSNAME, psname, &length);
 
   if (psname[0])
     {
-      int nvars = vlistNvars(vlistID);
+      char name[CDI_MAX_NAME];
+      const auto nvars = vlistNvars(vlistID);
       for (int varID = 0; varID < nvars; ++varID)
         {
           vlistInqVarName(vlistID, varID, name);
-          if (cstrIsEqual(name, psname))
-            {
-              psvarid = varID;
-              break;
-            }
+          if (cdo_cmpstr(name, psname)) return varID;
         }
-      if (Options::cdoVerbose && psvarid == -1) cdoWarning("Surface pressure variable not found - %s", psname);
+      if (Options::cdoVerbose) cdoWarning("Surface pressure variable not found - %s", psname);
     }
 
-  return psvarid;
+  return -1;
 }
 
-
-void varListInit(VarList &vl, int vlistID)
+int
+vlistGetFirstSpectralGrid(int vlistID)
 {
-  const int nvars = vlistNvars(vlistID);
-  vl.resize(nvars);
-
-  for (int varID = 0; varID < nvars; ++varID)
+  // find first spectral grid
+  const auto ngrids = vlistNgrids(vlistID);
+  for (int index = 0; index < ngrids; index++)
     {
-      vl[varID].gridID = vlistInqVarGrid(vlistID, varID);
-      vl[varID].zaxisID = vlistInqVarZaxis(vlistID, varID);
-      vl[varID].gridsize = gridInqSize(vl[varID].gridID);
-      vl[varID].nlevels = zaxisInqSize(vl[varID].zaxisID);
-      vl[varID].datatype = vlistInqVarDatatype(vlistID, varID);
-      vl[varID].missval = vlistInqVarMissval(vlistID, varID);
-      vl[varID].timetype = vlistInqVarTimetype(vlistID, varID);
+      const auto gridID = vlistGrid(vlistID, index);
+      if (gridInqType(gridID) == GRID_SPECTRAL) return gridID;
+    }
+
+  return -1;
+}
+
+int
+vlistGetFirstGaussianGrid(int vlistID)
+{
+  // find first gaussian grid
+  const auto ngrids = vlistNgrids(vlistID);
+  for (int index = 0; index < ngrids; index++)
+    {
+      const auto gridID = vlistGrid(vlistID, index);
+      if (gridInqType(gridID) == GRID_GAUSSIAN) return gridID;
+    }
+
+  return -1;
+}
+
+int
+vlistGetFirstFourierGrid(int vlistID)
+{
+  // find first fourier grid
+  const auto ngrids = vlistNgrids(vlistID);
+  for (int index = 0; index < ngrids; index++)
+    {
+      const auto gridID = vlistGrid(vlistID, index);
+      if (gridInqType(gridID) == GRID_FOURIER) return gridID;
+    }
+
+  return -1;
+}
+
+void
+cdo_check_missval(double missval, const char *varname)
+{
+  if (DBL_IS_EQUAL(0, missval) || DBL_IS_EQUAL(1, missval))
+    {
+      static bool lwarn = true;
+      if (lwarn)
+        {
+          lwarn = false;
+          cdoWarning("Variable %s has a missing value of %g, this can lead to incorrect results with this operator!", varname, missval);
+        }
     }
 }
