@@ -20,12 +20,15 @@
 #include <sys/stat.h>
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_cdi_wrapper.h"
 
 #define TO_KELVIN(x) ((x) + 273.15)
 #define MISSVAL -9.0E33
 
-typedef std::array<std::string, 2> cdoCommand;
+using cdoCommand = std::array<std::string, 2>;
 
 static int
 equals(double expected, double actual, double eps)
@@ -42,7 +45,7 @@ double humidityIndex(double t, double p, double r, double missval)
   if ( t < tmin || r < rmin )
     return missval;
 
-  return t + (5.0 / 9.0) * ((0.01 * r * p * 6.112 * pow(10.0, (7.5 * t) / (237.7
+  return t + (5.0 / 9.0) * ((0.01 * r * p * 6.112 * std::pow(10.0, (7.5 * t) / (237.7
 + t))) - 10.0);
 }
 */
@@ -108,7 +111,7 @@ writeNcFile(const char path[], const double array[], int length)
   vlistID = vlistCreate();
 
   varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
-  vlistDefVarName(vlistID, varID, "test_values");
+  cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, "test_values");
   vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_FLT64);
   vlistDefVarMissval(vlistID, varID, MISSVAL);
 
@@ -206,17 +209,17 @@ static int
 submitCdoCommand(const char *argument)
 {
   const char *cdoPath = getCdoPath();
-  char *cdoCommand = (char *) Malloc(strlen(cdoPath) + strlen(argument) + 8);
+  char *cdo_command = (char *) Malloc(strlen(cdoPath) + strlen(argument) + 8);
 
   int status;
 
-  cdoCommand[0] = '\0';
-  strcat(cdoCommand, cdoPath);
-  strcat(cdoCommand, " -b 64 ");
-  strcat(cdoCommand, argument);
+  cdo_command[0] = '\0';
+  strcat(cdo_command, cdoPath);
+  strcat(cdo_command, " -b 64 ");
+  strcat(cdo_command, argument);
 
-  status = system(cdoCommand);
-  Free(cdoCommand);
+  status = system(cdo_command);
+  Free(cdo_command);
 
   return status;
 }
