@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,19 +14,19 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include "cdo_int.h"
+
+#include "cdo_output.h"
 #include "pmlist.h"
 #include "json/jsmn.h"
 
 static char *
 readLineFromBuffer(char *buffer, size_t *buffersize, char *line, size_t len)
 {
-  int ichar;
   size_t ipos = 0;
 
   while (*buffersize)
     {
-      ichar = *buffer;
+      int ichar = *buffer;
       (*buffersize)--;
       buffer++;
       if (ichar == '\r')
@@ -72,7 +72,7 @@ static char *
 getElementName(char *pline, char *name)
 {
   while (isspace((int) *pline)) pline++;
-  size_t len = strlen(pline);
+  const auto len = strlen(pline);
   size_t pos = 0;
   while (pos < len && !isspace((int) *(pline + pos)) && *(pline + pos) != '=' && *(pline + pos) != ':') pos++;
 
@@ -87,7 +87,7 @@ static char *
 getElementValue(char *pline)
 {
   while (isspace((int) *pline)) pline++;
-  size_t len = strlen(pline);
+  auto len = strlen(pline);
   if (*pline != '"' && *pline != '\'')
     for (size_t i = 1; i < len; ++i)
       if (pline[i] == '!')
@@ -108,9 +108,7 @@ getElementValue(char *pline)
 static void
 parseCmortablebuf(PMList &pmlist, size_t buffersize, char *buffer)
 {
-  char line[4096];
-  char name[256];
-  char *pline;
+  char line[4096], name[256];
   const char *listentry[] = { "axis_entry", "variable_entry" };
   const int nentry = sizeof(listentry) / sizeof(listentry[0]);
   int linenumber = 0;
@@ -119,7 +117,7 @@ parseCmortablebuf(PMList &pmlist, size_t buffersize, char *buffer)
   while ((buffer = readLineFromBuffer(buffer, &buffersize, line, sizeof(line))))
     {
       linenumber++;
-      pline = line;
+      auto pline = line;
       while (isspace((int) *pline)) pline++;
       if (*pline == '#' || *pline == '!' || *pline == '\0') continue;
       //  len = (int) strlen(pline);
@@ -143,7 +141,7 @@ parseCmortablebuf(PMList &pmlist, size_t buffersize, char *buffer)
           pline = skipSeparator(pline);
           pline = getElementValue(pline);
 
-          if (*pline) kvlist.append("name", (const char **) &pline, 1);
+          if (*pline) kvlist.append("name", &pline, 1);
         }
       else
         {
@@ -153,7 +151,7 @@ parseCmortablebuf(PMList &pmlist, size_t buffersize, char *buffer)
 
           if (kvlist.size() == 0) kvlist.name = "Header";
 
-          if (*pline) kvlist.append(name, (const char **) &pline, 1);
+          if (*pline) kvlist.append(name, &pline, 1);
         }
     }
 
@@ -164,7 +162,6 @@ parseCmortablebuf(PMList &pmlist, size_t buffersize, char *buffer)
 int
 dump_json(const char *js, jsmntok_t *t, size_t count, int level)
 {
-  int i, j, k;
   if (count == 0) return 0;
 
   if (t->type == JSMN_PRIMITIVE)
@@ -182,10 +179,10 @@ dump_json(const char *js, jsmntok_t *t, size_t count, int level)
       printf("\n");
       //  printf("Object: size %d\n", t->size);
       printf("Object: size %d count %d level %d\n", t->size, (int) count, level);
-      j = 0;
-      for (i = 0; i < t->size; i++)
+      int j = 0;
+      for (int i = 0; i < t->size; i++)
         {
-          for (k = 0; k < level; k++) printf("  ");
+          for (int k = 0; k < level; k++) printf("  ");
           j += dump_json(js, t + 1 + j, count - j, level + 1);
           printf(": ");
           j += dump_json(js, t + 1 + j, count - j, level + 1);
@@ -195,17 +192,18 @@ dump_json(const char *js, jsmntok_t *t, size_t count, int level)
     }
   else if (t->type == JSMN_ARRAY)
     {
-      j = 0;
+      int j = 0;
       printf("\n");
-      for (i = 0; i < t->size; i++)
+      for (int i = 0; i < t->size; i++)
         {
-          for (k = 0; k < level - 1; k++) printf("  ");
+          for (int k = 0; k < level - 1; k++) printf("  ");
           printf("   - ");
           j += dump_json(js, t + 1 + j, count - j, level + 1);
           printf("\n");
         }
       return j + 1;
     }
+
   return 0;
 }
 
@@ -218,7 +216,7 @@ KVList_append_json(KVList &kvlist, const char *key, const char *js, jsmntok_t *t
   kv.values.resize(nvalues);
   for (int i = 0; i < nvalues; ++i)
     {
-      size_t len = t[i].end - t[i].start;
+      const auto len = t[i].end - t[i].start;
       std::vector<char> value(len + 1);
       snprintf(value.data(), len + 1, "%.*s", (int) len, js + t[i].start);
       value[len] = 0;
@@ -242,7 +240,7 @@ addTokensJson(PMList &pmlist, const char *js, jsmntok_t *t, int count)
       while (nobj--)
         {
           ++i;
-          int pmlname = i;
+          auto pmlname = i;
           if (debug) printf("  object: %.*s\n", t[i].end - t[i].start, js + t[i].start);
           ++i;
           if (t[i].type == JSMN_OBJECT)
@@ -312,19 +310,19 @@ addTokensJson(PMList &pmlist, const char *js, jsmntok_t *t, int count)
 static void
 parseCmortablebufJson(PMList &pmlist, size_t buffersize, char *buffer, const char *filename)
 {
-  /* Prepare parser */
-  jsmn_parser *p = jsmn_new();
+  // Prepare parser
+  auto p = jsmn_new();
 
-  int status = jsmn_parse(p, buffer, buffersize);
+  auto status = jsmn_parse(p, buffer, buffersize);
   if (status != 0)
     {
       switch (status)
         {
         case JSMN_ERROR_INVAL:
-          fprintf(stderr, "JSON error: Invalid character in %s (line=%d character='%c')!\n", filename, p->lineno, buffer[p->pos]);
+          fprintf(stderr, "JSON error: Invalid character in %s (line=%u character='%c')!\n", filename, p->lineno, buffer[p->pos]);
           break;
-        case JSMN_ERROR_PART: fprintf(stderr, "JSON error: End of string not found in %s (line=%d)!\n", filename, p->lineno); break;
-        default: fprintf(stderr, "JSON error in %s (line=%d)\n", filename, p->lineno); break;
+        case JSMN_ERROR_PART: fprintf(stderr, "JSON error: End of string not found in %s (line=%u)!\n", filename, p->lineno); break;
+        default: fprintf(stderr, "JSON error in %s (line=%u)\n", filename, p->lineno); break;
         }
     }
 
@@ -350,7 +348,8 @@ PMList::readCmortable(FILE *fp, const char *name)
     }
   else if (buffer0 == '&' || buffer0 == '#')
     {
-      const int status = parseListBuffer(*this, listBuffer);
+      NamelistParser p;
+      const auto status = parseListBuffer(p, listBuffer);
       if (status) cdoAbort("Namelist not found!");
     }
   else
