@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,19 +15,16 @@
   GNU General Public License for more details.
 */
 
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <cmath>
 
-#include "dmemory.h"
 #include "color.h"
 
 #define RGB 0
 #define HSV 1
 #define CMYK 2
 
-double rint(double x);
-
 #define irint(x) ((int) rint(x))
 
 int
@@ -57,7 +54,7 @@ hsv_to_rgb(int rgb[], double h, double s, double v)
   double f, p, q, t, rr = 0, gg = 0, bb = 0;
 
   if (std::fabs(s) <= 0)
-    rgb[0] = rgb[1] = rgb[2] = (int) floor(255.999 * v);
+    rgb[0] = rgb[1] = rgb[2] = (int) std::floor(255.999 * v);
   else
     {
       while (h >= 360.0) h -= 360.0;
@@ -101,9 +98,9 @@ hsv_to_rgb(int rgb[], double h, double s, double v)
           break;
         }
 
-      rgb[0] = (rr < 0.0) ? 0 : (int) floor(rr * 255.999);
-      rgb[1] = (gg < 0.0) ? 0 : (int) floor(gg * 255.999);
-      rgb[2] = (bb < 0.0) ? 0 : (int) floor(bb * 255.999);
+      rgb[0] = (rr < 0.0) ? 0 : (int) std::floor(rr * 255.999);
+      rgb[1] = (gg < 0.0) ? 0 : (int) std::floor(gg * 255.999);
+      rgb[2] = (bb < 0.0) ? 0 : (int) std::floor(bb * 255.999);
     }
 }
 
@@ -116,7 +113,7 @@ cmyk_to_rgb(int rgb[], double cmyk[])
 
   /* CMYK is in 0-100, RGB will be in 0-255 range */
 
-  for (i = 0; i < 3; i++) rgb[i] = (int) floor((100.0 - cmyk[i] - cmyk[3]) * 2.55999);
+  for (i = 0; i < 3; i++) rgb[i] = (int) std::floor((100.0 - cmyk[i] - cmyk[3]) * 2.55999);
 }
 
 int
@@ -204,13 +201,13 @@ cptRead(FILE *fp, CPT *cpt)
   bool gap;
   int error = 0;
   char T0[64], T1[64], T2[64], T3[64], T4[64], T5[64], T6[64], T7[64], T8[64], T9[64];
-  char line[BUFSIZ], option[64], c;
+  char line[BUFSIZ], option[260], c;
 
   if (fp == nullptr) return (READERR);
 
   cptInit(cpt);
 
-  cpt->lut = (LUT *) Calloc(1, n_alloc * sizeof(LUT));
+  cpt->lut = std::vector<LUT>(1);
 
   /* Save the original setting since it may be modified by settings in the CPT
    * file */
@@ -371,7 +368,7 @@ cptRead(FILE *fp, CPT *cpt)
         {
           i = n_alloc;
           n_alloc += small_chunk;
-          cpt->lut = (LUT *) Realloc((void *) cpt->lut, (size_t) n_alloc * sizeof(LUT));
+          cpt->lut.resize(n_alloc);
           memset((void *) &cpt->lut[i], 0, (size_t)(small_chunk * sizeof(LUT))); /* Initialize new structs to zero */
         }
     }
@@ -390,7 +387,7 @@ cptRead(FILE *fp, CPT *cpt)
       return (READERR);
     }
 
-  cpt->lut = (LUT *) Realloc((void *) cpt->lut, (size_t) n * sizeof(LUT));
+  cpt->lut.resize(n);
   ncolors = n;
   for (i = annot = 0, gap = false; i < ncolors - 1; i++)
     {
