@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 #define COLOR_H
 
 #include <cstdio>
+#include <vector>
 
 struct LUT
 {
@@ -36,7 +37,7 @@ struct BFN_COLOR
 struct CPT
 {
   int ncolors;
-  LUT *lut;
+  std::vector<LUT> lut;
   BFN_COLOR bfn[3];
 };
 
