@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,10 +30,6 @@ constexpr double C_EARTH_RD = 1000. * C_R / C_RMD;
 constexpr double C_EARTH_RADIUS = 6371000.0;  // radius of the Earth in m
 constexpr double C_EARTH_GRAV = 9.80665;
 
-constexpr double C_MARS_RD = 189.0;
-constexpr double C_MARS_RADIUS = 3400000.0;  // radius of the Mars in m
-constexpr double C_MARS_GRAV = 3.7;
-
 #define C_RG (1.0 / PlanetGrav)
 
 constexpr double C_RCPV = 4.0 * C_RV;
