@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,20 +22,11 @@
 #include "cdo_output.h"
 #include "cdo_options.h"
 
-
 #ifdef HAVE_UDUNITS2
 
-#ifdef HAVE_LIBPTHREAD
-
 #include <mutex>
 static std::mutex udunitsMutex;
-#define UDUNITS_LOCK()  std::unique_lock<std::mutex> locked_mutex(udunitsMutex);
-
-#else
-
-#define UDUNITS_LOCK()
-
-#endif
+#define UDUNITS_LOCK() std::unique_lock<std::mutex> locked_mutex(udunitsMutex);
 
 static ut_system *ut_read = nullptr;
 
@@ -145,6 +136,8 @@ cdoConvertDestroy()
 void
 cdoConvertUnits(void **ut_converter, bool *changeunits, char *units, char *units_old, const char *name)
 {
+  (void) ut_converter; // removes wrong warning, caused by ifdef
+
   if (*changeunits)
     {
 #ifdef HAVE_UDUNITS2
