@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 #ifndef __CONVERT_UNITS_H_
 #define __CONVERT_UNITS_H_
 
-#if defined(HAVE_CONFIG_H)
+#ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
@@ -36,8 +36,6 @@ void cdoConvertFree(void *ut_converter);
 void cdoConvertDestroy();
 #endif
 
-#include "cdo_int.h"
-
 void cdoConvertUnits(void **ut_converter, bool *changeunits, char *units, char *units_old, const char *name);
 
 #endif  // __CONVERT_UNITS_H_
