@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,14 +17,25 @@
 #ifndef COUNTER_H
 #define COUNTER_H
 
-struct counter_t
+namespace cdo
 {
-  double cputime;
-  char mark[32];
-};
+class Counter
+{
+public:
+  Counter() : m_cputime(0) {}
 
-void counter_start(counter_t *counter);
-void counter_stop(counter_t *counter);
-double counter_cputime(counter_t counter);
+  void start();
+  void stop();
+  double
+  cputime()
+  {
+    return m_cputime;
+  }
+
+private:
+  double m_cputime = 0;
+  char mark[32] = { 0 };
+};
+}  // namespace cdo
 
 #endif /* COUNTER_H */
