@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,19 +27,20 @@
 #endif
 
 #ifdef __cplusplus
-extern "C" {
+extern "C"
+{
 #endif
 
-extern size_t memTotal(void);
-extern void memDebug(int debug);
-extern void memExitOnError(void);
+  extern size_t memTotal(void);
+  extern void memDebug(int debug);
+  extern void memExitOnError(void);
 
 #if defined DEBUG_MEMORY
 
-extern void *memRealloc(void *ptr, size_t size, const char *file, const char *functionname, int line);
-extern void *memCalloc(size_t nmemb, size_t size, const char *file, const char *functionname, int line);
-extern void *memMalloc(size_t size, const char *file, const char *functionname, int line);
-extern void memFree(void *ptr, const char *file, const char *functionname, int line);
+  extern void *memRealloc(void *ptr, size_t size, const char *file, const char *functionname, int line);
+  extern void *memCalloc(size_t nmemb, size_t size, const char *file, const char *functionname, int line);
+  extern void *memMalloc(size_t size, const char *file, const char *functionname, int line);
+  extern void memFree(void *ptr, const char *file, const char *functionname, int line);
 
 #if defined(__cplusplus)
 }
