@@ -20,17 +20,17 @@
 
 #include "field.h"
 
-typedef enum
+enum ECA_EPILOG
 {
   ECA_NONE,
   MEAN,
   PERCENT_OF_TIME,
   PERCENT_OF_TOTAL_AMOUNT
-} ECA_EPILOG;
+};
 
-typedef void (*ECA_FUNC_1)(Field &, double);
-typedef void (*ECA_FUNC_2)(Field &, const Field &);
-typedef void (*ECA_FUNC_3)(Field &, const Field &, double);
+using ECA_FUNC_1 = void (*)(Field &, double);
+using ECA_FUNC_2 = void (*)(Field &, const Field &);
+using ECA_FUNC_3 = void (*)(Field &, const Field &, double);
 
 /**
  * Structure describing a processing request of the form
@@ -55,16 +55,17 @@ typedef void (*ECA_FUNC_3)(Field &, const Field &, double);
  */
 struct ECA_MAJOR_REQUEST_ELEMENT_1
 {
-  const char *name;
-  const char *longname;
-  const char *units;
-  ECA_FUNC_1 f1;
-  double f1arg;
-  ECA_FUNC_2 f2;
-  ECA_FUNC_2 f3;
-  double mulc;
-  double addc;
-  ECA_EPILOG epilog;
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  int refdate;
+  ECA_FUNC_1 f1 = nullptr;
+  double f1arg = 1.0;
+  ECA_FUNC_2 f2 = nullptr;
+  ECA_FUNC_2 f3 = nullptr;
+  double mulc = 0.0;
+  double addc = 0.0;
+  ECA_EPILOG epilog = ECA_NONE;
 };
 
 /**
@@ -86,13 +87,13 @@ struct ECA_MAJOR_REQUEST_ELEMENT_1
  */
 struct ECA_MINOR_REQUEST_ELEMENT_1
 {
-  const char *name;
-  const char *longname;
-  const char *units;
-  ECA_FUNC_1 h1;
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  ECA_FUNC_1 h1 = nullptr;
   double h1arg;
-  ECA_FUNC_2 h2;
-  ECA_FUNC_2 h3;
+  ECA_FUNC_2 h2 = nullptr;
+  ECA_FUNC_2 h3 = nullptr;
 };
 
 struct ECA_REQUEST_1
@@ -126,18 +127,19 @@ struct ECA_REQUEST_1
  */
 struct ECA_MAJOR_REQUEST_ELEMENT_2
 {
-  const char *name;
-  const char *longname;
-  const char *units;
-  ECA_FUNC_1 f1;
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  int refdate;
+  ECA_FUNC_1 f1 = nullptr;
   double f1arg;
-  ECA_FUNC_1 f2;
+  ECA_FUNC_1 f2 = nullptr;
   double f2arg;
-  ECA_FUNC_2 f3;
-  ECA_FUNC_2 f4;
-  ECA_FUNC_3 f5;
+  ECA_FUNC_2 f3 = nullptr;
+  ECA_FUNC_2 f4 = nullptr;
+  ECA_FUNC_3 f5 = nullptr;
   double f5arg;
-  ECA_EPILOG epilog;
+  ECA_EPILOG epilog = ECA_NONE;
 };
 
 /**
@@ -159,12 +161,12 @@ struct ECA_MAJOR_REQUEST_ELEMENT_2
  */
 struct ECA_MINOR_REQUEST_ELEMENT_2
 {
-  const char *name;
-  const char *longname;
-  const char *units;
-  ECA_FUNC_1 h1;
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  ECA_FUNC_1 h1 = nullptr;
   double h1arg;
-  ECA_FUNC_2 h2;
+  ECA_FUNC_2 h2 = nullptr;
 };
 
 struct ECA_REQUEST_2
@@ -192,12 +194,13 @@ struct ECA_REQUEST_2
  */
 struct ECA_REQUEST_3
 {
-  const char *name;
-  const char *longname;
-  const char *units;
-  ECA_FUNC_2 f1;
-  ECA_FUNC_2 f2;
-  ECA_FUNC_2 f3;
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  int refdate;
+  ECA_FUNC_2 f1 = nullptr;
+  ECA_FUNC_2 f2 = nullptr;
+  ECA_FUNC_2 f3 = nullptr;
 };
 
 /**
@@ -221,21 +224,47 @@ struct ECA_REQUEST_3
  */
 struct ECA_REQUEST_4
 {
-  const char *name;
-  const char *longname;
-  const char *units;
-  const char *name2;
-  const char *longname2;
-  const char *units2;
-  ECA_FUNC_1 s1;
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  const char *name2 = nullptr;
+  const char *longname2 = nullptr;
+  const char *units2 = nullptr;
+  ECA_FUNC_1 s1 = nullptr;
   double s1arg;
-  ECA_FUNC_1 s2;
+  ECA_FUNC_1 s2 = nullptr;
   double s2arg;
-  ECA_FUNC_1 s3;
+  ECA_FUNC_1 s3 = nullptr;
   double s3arg;
   int consecutiveDays;
 };
 
+/**
+ * Structure describing an ETCCDI index processing request. The structure
+ * contains the following elements:
+ *
+ * name       the name of the output variable
+ * longname   the longname of the output variable
+ * units      the units of the output variable
+ * pn         the percentile for the threshold
+ * ndates     the window size
+ * startboot  the begin year of the bootstrapping interval
+ * endboot    the end year of the bootstrapping interval
+ * func2      either func_sum or func_average
+ */
+
+struct ETCCDI_REQUEST
+{
+  const char *name = nullptr;
+  const char *longname = nullptr;
+  const char *units = nullptr;
+  int pn;
+  int ndates;
+  int startboot;
+  int endboot;
+  int func2;
+};
+
 /**
  * Function processing a request of type 1.
  *
@@ -264,4 +293,12 @@ void eca3(const ECA_REQUEST_3 *request);
  */
 void eca4(const ECA_REQUEST_4 *request);
 
+/**
+ * Function processing a etccdi request.
+ *
+ * @param request the processing request
+ */
+
+void etccdi_op(ETCCDI_REQUEST *request);
+
 #endif /* ECA_H_ */
