@@ -15,13 +15,14 @@
   GNU General Public License for more details.
 */
 
+#include <algorithm>
 #include <assert.h>
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "cdo_math.h"
+#include "process_int.h"
 #include "timebase.h"
+#include "datetime.h"
 #include "ecautil.h"
 
 /**
@@ -83,7 +84,11 @@ gregdate_to_julday(int year,  /* Gregorian year */
     {
       double xi = 365.25 * jy;
 
-      if ((int) xi != xi) xi -= 1;
+      {
+        int temp = xi * 100;
+        if (temp % 100 != 0) xi -= 1;
+      }
+      // if (((int) xi) != xi) xi -= 1; // old version, caused warning (25.08.2020 O.Heidmann)
       julday += (int) xi;
     }
   julday += 1720995;
@@ -146,8 +151,8 @@ count(Field &field1, const Field &field2, double mode)
 {
   const double missval1 = field1.missval;
   const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   size_t len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
@@ -164,10 +169,10 @@ count(Field &field1, const Field &field2, double mode)
 
           if (!DBL_IS_EQUAL(array1[i], missval1))
             {
-              if (IS_EQUAL(mode, 0.0) || IS_EQUAL(mode, 1.0) || array2[i] > mode)
+              if (IS_EQUAL(mode, 0.0) || IS_EQUAL(mode, 1.0))
                 array1[i] += 1.0;
-              else if (DBL_IS_EQUAL(array2[i], mode))
-                array1[i] += mode;
+              else if (DBL_IS_EQUAL(array2[i], mode) || array2[i] > mode)
+                array1[i] += array2[i];
             }
           else
             {
@@ -192,10 +197,10 @@ count(Field &field1, const Field &field2, double mode)
               continue;
             }
 
-          if (IS_EQUAL(mode, 0.0) || IS_EQUAL(mode, 1.0) || array2[i] > mode)
+          if (IS_EQUAL(mode, 0.0) || IS_EQUAL(mode, 1.0))
             array1[i] += 1.0;
-          else if (DBL_IS_EQUAL(array2[i], mode))
-            array1[i] += mode;
+          else if (DBL_IS_EQUAL(array2[i], mode) || array2[i] > mode)
+            array1[i] += array2[i];
         }
     }
 }
@@ -220,8 +225,8 @@ selcomp(Field &field1, const Field &field2, int (*compare)(double, double))
 {
   const double missval1 = field1.missval;
   const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   size_t len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
@@ -260,7 +265,7 @@ static void
 selcompc(Field &field, double c, int (*compare)(double, double))
 {
   const double missval = field.missval;
-  auto &array = field.vec;
+  auto &array = field.vec_d;
 
   size_t len = field.size;
 
@@ -341,8 +346,8 @@ vfarsel(Field &field1, const Field &field2)
 {
   const double missval1 = field1.missval;
   const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   const size_t len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different gridsize (%s)", __func__);
@@ -440,17 +445,51 @@ updateHist(FieldVector2D &field, int nlevels, size_t gridsize, std::vector<doubl
     for (size_t i = 0; i < gridsize; i++)
       if (onlyNorth)
         {
-          if (yvals[i] >= 0.0) field[1][levelID].vec[i] = field[0][levelID].vec[i];
+          if (yvals[i] >= 0.0) field[1][levelID].vec_d[i] = field[0][levelID].vec_d[i];
         }
       else
-        field[1][levelID].vec[i] = field[0][levelID].vec[i];
+        field[1][levelID].vec_d[i] = field[0][levelID].vec_d[i];
 }
 
 void
-adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, int64_t ovdate, FieldVector2D &startDateWithHist,
-              FieldVector2D &endDateWithHist)
+defineMidOfTime(int frequency, int taxisID, int year, int month, int MaxMonths)
 {
-  const int64_t ovdateSouth = cdo::min(cdiEncodeDate(ovdate / 10000, 6, 30), ovdate);
+  int64_t vdate, vdateb, vdatebp1;
+  int vtime;
+  int64_t vtime0 = cdiEncodeTime(0, 0, 0);
+
+  const auto calendar = taxisInqCalendar(taxisID);
+
+  if (frequency == 8)
+    {
+      vdateb = cdiEncodeDate(year, month, 1);
+
+      int boundmonth = (month + 1 <= MaxMonths) ? month + 1 : 1;
+      int boundyear = (boundmonth != 1) ? year : year + 1;
+      vdatebp1 = cdiEncodeDate(boundyear, boundmonth, 1);
+    }
+  else
+    {
+      vdateb = cdiEncodeDate(year, 1, 1);
+      vdatebp1 = cdiEncodeDate(year + 1, 1, 1);
+    }
+
+  const auto juldate1 = julianDateEncode(calendar, vdateb, vtime0);
+  const auto juldate2 = julianDateEncode(calendar, vdatebp1, vtime0);
+
+  const auto seconds = julianDateToSeconds(julianDateSub(juldate2, juldate1)) / 2;
+  const auto juldatem = julianDateAddSeconds(lround(seconds), juldate1);
+  julianDateDecode(calendar, juldatem, vdate, vtime);
+
+  taxisDefVdate(taxisID, vdate);
+  taxisDefVtime(taxisID, vtime);
+}
+
+void
+adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, int64_t ovdate,
+              FieldVector2D &startDateWithHist, FieldVector2D &endDateWithHist)
+{
+  const int64_t ovdateSouth = std::min(cdiEncodeDate(ovdate / 10000, 6, 30), ovdate);
 
   for (int levelID = 0; levelID < nlevels; levelID++)
     {
@@ -459,27 +498,27 @@ adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double m
           /* start with southern sphere */
           if (yvals[i] < 0)
             {
-              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].vec[i], missval))
+              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].vec_d[i], missval))
                 {
-                  endDateWithHist[0][levelID].vec[i] = missval;
+                  endDateWithHist[0][levelID].vec_d[i] = missval;
                   continue;
                 }
-              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].vec[i], missval))
+              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].vec_d[i], missval))
                 {
-                  endDateWithHist[0][levelID].vec[i] = ovdateSouth;
+                  endDateWithHist[0][levelID].vec_d[i] = ovdateSouth;
                 }
             }
           else
             {
-              if (DBL_IS_EQUAL(startDateWithHist[0][levelID].vec[i], missval))
+              if (DBL_IS_EQUAL(startDateWithHist[0][levelID].vec_d[i], missval))
                 {
-                  endDateWithHist[0][levelID].vec[i] = missval;
+                  endDateWithHist[0][levelID].vec_d[i] = missval;
                   continue;
                 }
 
-              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].vec[i], missval))
+              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].vec_d[i], missval))
                 {
-                  endDateWithHist[0][levelID].vec[i] = ovdate;
+                  endDateWithHist[0][levelID].vec_d[i] = ovdate;
                 }
             }
         }
@@ -487,8 +526,8 @@ adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double m
 }
 
 void
-computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, FieldVector2D &startDateWithHist, FieldVector2D &endDateWithHist,
-           FieldVector &gslDuration, FieldVector &gslFirstDay, bool useCurrentYear)
+computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, FieldVector2D &startDateWithHist,
+           FieldVector2D &endDateWithHist, FieldVector &gslDuration, FieldVector &gslFirstDay, bool useCurrentYear)
 {
   int levelID;
   double firstDay, duration;
@@ -502,22 +541,22 @@ computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double miss
               /* start with southern sphere */
               if (yvals[i] < 0.0)
                 {
-                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].vec[i])
-                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].vec[i]));
+                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].vec_d[i])
+                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].vec_d[i]));
                 }
               else
                 {
-                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[1][levelID].vec[i])
-                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].vec[i]));
+                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[1][levelID].vec_d[i])
+                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].vec_d[i]));
                 }
 
-              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].vec[i], missval))
+              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].vec_d[i], missval))
                 firstDay = missval;
               else
-                firstDay = (double) day_of_year((int64_t) startDateWithHist[1][levelID].vec[i]);
+                firstDay = (double) day_of_year((int64_t) startDateWithHist[1][levelID].vec_d[i]);
 
-              gslDuration[levelID].vec[i] = duration;
-              gslFirstDay[levelID].vec[i] = firstDay;
+              gslDuration[levelID].vec_d[i] = duration;
+              gslFirstDay[levelID].vec_d[i] = firstDay;
             }
         }
     }
@@ -531,21 +570,21 @@ computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double miss
               /* start with southern sphere */
               if (yvals[i] < 0.0)
                 {
-                  gslDuration[levelID].vec[i] = missval;
-                  gslFirstDay[levelID].vec[i] = missval;
+                  gslDuration[levelID].vec_d[i] = missval;
+                  gslFirstDay[levelID].vec_d[i] = missval;
                 }
               else
                 {
-                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].vec[i])
-                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[0][levelID].vec[i]));
+                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].vec_d[i])
+                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[0][levelID].vec_d[i]));
 
-                  if (DBL_IS_EQUAL(startDateWithHist[0][levelID].vec[i], missval))
+                  if (DBL_IS_EQUAL(startDateWithHist[0][levelID].vec_d[i], missval))
                     firstDay = missval;
                   else
-                    firstDay = (double) day_of_year((int64_t) startDateWithHist[0][levelID].vec[i]);
+                    firstDay = (double) day_of_year((int64_t) startDateWithHist[0][levelID].vec_d[i]);
 
-                  gslDuration[levelID].vec[i] = duration;
-                  gslFirstDay[levelID].vec[i] = firstDay;
+                  gslDuration[levelID].vec_d[i] = duration;
+                  gslFirstDay[levelID].vec_d[i] = firstDay;
                 }
             }
         }
@@ -572,11 +611,11 @@ writeGslStream(CdoStreamID ostreamID, int otaxisID, int otsID, int ovarID1, int
   for (int levelID = 0; levelID < nlevels; levelID++)
     {
       cdoDefRecord(ostreamID, ovarID1, levelID);
-      cdoWriteRecord(ostreamID, gslDuration[levelID].vec.data(), gslDuration[levelID].nmiss);
+      cdoWriteRecord(ostreamID, gslDuration[levelID].vec_d.data(), gslDuration[levelID].nmiss);
     }
   for (int levelID = 0; levelID < nlevels; levelID++)
     {
       cdoDefRecord(ostreamID, ovarID2, levelID);
-      cdoWriteRecord(ostreamID, gslFirstDay[levelID].vec.data(), gslFirstDay[levelID].nmiss);
+      cdoWriteRecord(ostreamID, gslFirstDay[levelID].vec_d.data(), gslFirstDay[levelID].nmiss);
     }
 }
