@@ -317,8 +317,8 @@ void updateHist(FieldVector2D &field, int nlevels, size_t gridsize, std::vector<
  *                             gsl can only be computed for the northern
  *                             hemisphere (see definition of GSL: EcaGsl()
  */
-void computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, FieldVector2D &startDateWithHist, FieldVector2D &endDateWithHist,
-                FieldVector &gslDuration, FieldVector &gslFirstDay, bool useCurrentYear);
+void computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, FieldVector2D &startDateWithHist,
+                FieldVector2D &endDateWithHist, FieldVector &gslDuration, FieldVector &gslFirstDay, bool useCurrentYear);
 
 /*
  * Adjust the endDates found in the current year:
@@ -326,8 +326,12 @@ void computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double
  * should be the last day of the corresponding year for norther and June, 30th
  * for southern hemisphere
  */
-void adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, int64_t ovdate, FieldVector2D &startDateWithHist,
-                   FieldVector2D &endDateWithHist);
+void adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, int64_t ovdate,
+                   FieldVector2D &startDateWithHist, FieldVector2D &endDateWithHist);
+/*
+ * Calculates the mid of the year or month
+ */
+void defineMidOfTime(int frequency, int taxisID, int year, int month, int MaxMonths);
 /*
  * Write GSL related fields to an output stream
  */
