@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,8 @@
   GNU General Public License for more details.
 */
 
-#include <stdio.h>
+#include <cstdio>
+#include <vector>
 
 extern int CDO_parser_errorno;
 
@@ -106,6 +107,7 @@ enum
 struct paramType
 {
   int type;
+  bool isValid;
   bool select;
   bool remove;
   bool lmiss;
@@ -147,25 +149,25 @@ typedef struct nodeTypeTag
   } u;
 } nodeType;
 
-struct coordType
+struct CoordType
 {
-  bool needed;
+  std::vector<double> data;
+  std::vector<char> units;
+  std::vector<char> longname;
+  size_t size;
   int coord;
   int cdiID;
-  size_t size;
-  char *units;
-  char *longname;
-  double *data;
+  bool needed;
 };
 
 struct parseParamType
 {
   bool init;
   bool debug;
-  bool *needed;
+  std::vector<bool> needed;
   int maxparams;
   int nparams;
-  int cnparams;    // current number of valid params
+  int cnparams;  // current number of valid params
   int nvars1;
   int ncoords;
   int maxcoords;
@@ -173,8 +175,8 @@ struct parseParamType
   int pointID;
   int zonalID;
   int surfaceID;
-  coordType *coords;
-  paramType *params;
+  std::vector<CoordType> coords;
+  std::vector<paramType> params;
 };
 
 typedef union
