@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 
 void
@@ -24,8 +24,10 @@ fld_field_init(Field &field, size_t nmiss, double missval, size_t ngp, double *a
   field.size = ngp;
   field.nmiss = nmiss;
   field.missval = missval;
-  if (array != nullptr) for (size_t i = 0; i < ngp; ++i) field.vec[i] = array[i];
-  if (w != nullptr) for (size_t i = 0; i < ngp; ++i) field.weightv[i] = w[i];
+  if (array != nullptr)
+    for (size_t i = 0; i < ngp; ++i) field.vec_d[i] = array[i];
+  if (w != nullptr)
+    for (size_t i = 0; i < ngp; ++i) field.weightv[i] = w[i];
 }
 /*
 double *
@@ -51,7 +53,7 @@ fld_weights(int gridID, size_t ngp)
 int getLayerThickness(bool useweights, bool genbounds, int index, int zaxisID, int nlev, double *thickness, double *weights);
 
 void
-vert_weights(int zaxisID, size_t nlev, std::vector<double> &weights)
+vert_weights(int zaxisID, size_t nlev, Varray<double> &weights)
 {
   weights.resize(nlev);
   std::vector<double> thickness(nlev);
