@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,6 @@
 #include "field.h"
 
 void fld_field_init(Field &field, size_t nmiss, double missval, size_t ngp, double *array, double *w);
-double *fld_weights(int gridID, size_t ngp);
-void vert_weights(int zaxisID, size_t nlev, std::vector<double> &weights);
+void vert_weights(int zaxisID, size_t nlev, Varray<double> &weights);
 
 #endif
