@@ -35,7 +35,7 @@ enum field_flag
 
 class // Field
 #ifdef __GNUG__
-__attribute__((warn_unused))
+[[gnu::warn_unused]]
 #endif
 Field
 {
@@ -76,7 +76,7 @@ private:
 
 class // Field3D
 #ifdef __GNUG__
-__attribute__((warn_unused))
+[[gnu::warn_unused]]
 #endif
 Field3D : public Field
 {
