@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,19 +15,20 @@
   GNU General Public License for more details.
 */
 
+#include <algorithm>
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "cdo_math.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
 #include "array.h"
 
-
 static int
-vfarsetnmiss(size_t len, std::vector<double> &array, double missval)
+vfarsetnmiss(const size_t len, Varray<double> &array, const double missval)
 {
   size_t nmiss = 0;
 
-  if (DBL_IS_NAN(missval))
+  if (std::isnan(missval))
     {
       for (size_t i = 0; i < len; i++)
         if (DBL_IS_EQUAL(array[i], missval) || array[i] < 0)
@@ -52,19 +53,19 @@ vfarsetnmiss(size_t len, std::vector<double> &array, double missval)
 void
 vfarcpy(Field &field1, const Field &field2)
 {
-  size_t size1 = field1.size;
-  size_t size2 = field2.size;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
-  const auto &array2f = field2.vecf;
+  auto size1 = field1.size;
+  auto size2 = field2.size;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
+  const auto &array2f = field2.vec_f;
 
   if (size1 == 0) size1 = gridInqSize(field1.grid);
   if (size2 == 0) size2 = gridInqSize(field2.grid);
 
-  const size_t len = size1;
+  const auto len = size1;
   if (len != size2) cdoAbort("Fields have different size (%s)", __func__);
 
-  if (field2.memtype == MEMTYPE_FLOAT)
+  if (field2.memType == MemType::Float)
     for (size_t i = 0; i < len; i++) array1[i] = array2f[i];
   else
     for (size_t i = 0; i < len; i++) array1[i] = array2[i];
@@ -73,13 +74,13 @@ vfarcpy(Field &field1, const Field &field2)
 void
 vfaradd(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
-  const auto &array2f = field2.vecf;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
+  const auto &array2f = field2.vec_f;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -89,7 +90,7 @@ vfaradd(Field &field1, const Field &field2)
     }
   else
     {
-      if (field2.memtype == MEMTYPE_FLOAT)
+      if (field2.memType == MemType::Float)
         {
           for (size_t i = 0; i < len; i++) array1[i] += array2f[i];
         }
@@ -103,24 +104,24 @@ vfaradd(Field &field1, const Field &field2)
 void
 vfarsum(Field &field1, const Field &field2)
 {
-  size_t size1 = field1.size;
-  size_t size2 = field2.size;
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
-  const auto &array2f = field2.vecf;
+  auto size1 = field1.size;
+  auto size2 = field2.size;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
+  const auto &array2f = field2.vec_f;
 
   if (size1 == 0) size1 = gridInqSize(field1.grid);
   if (size2 == 0) size2 = gridInqSize(field2.grid);
 
-  const size_t len = size1;
+  const auto len = size1;
 
   if (len != size2) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
     {
-      if (field2.memtype == MEMTYPE_FLOAT)
+      if (field2.memType == MemType::Float)
         {
           for (size_t i = 0; i < len; i++)
             if (!DBL_IS_EQUAL(array2f[i], missval2))
@@ -140,7 +141,7 @@ vfarsum(Field &field1, const Field &field2)
     }
   else
     {
-      if (field2.memtype == MEMTYPE_FLOAT)
+      if (field2.memType == MemType::Float)
         {
           for (size_t i = 0; i < len; i++) array1[i] += array2f[i];
         }
@@ -154,12 +155,12 @@ vfarsum(Field &field1, const Field &field2)
 void
 vfarsumw(Field &field1, const Field &field2, double w)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  size_t len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -188,18 +189,18 @@ vfarsumw(Field &field1, const Field &field2, double w)
  * Compute the occurrence of values in field, if they do not equal refval.
  * This can be used to compute the lengths of multiple periods in a timeseries.
  * Missing field values are handled like refval, i.e. they stop a running period.
- * If there is missing data in the occurence field, missing fields values do not 
+ * If there is missing data in the occurence field, missing fields values do not
  * change anything (they do not start a non-period by setting occurrence to zero).
  */
 void
 vfarsumtr(Field &occur, const Field &field, const double refval)
 {
-  const double omissval = occur.missval;
-  const double fmissval = field.missval;
-  auto &oarray = occur.vec;
-  const auto &farray = field.vec;
+  auto omissval = occur.missval;
+  auto fmissval = field.missval;
+  auto &oarray = occur.vec_d;
+  const auto &farray = field.vec_d;
 
-  const size_t len = occur.size;
+  auto len = occur.size;
   if (len != field.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (occur.nmiss || field.nmiss)
@@ -234,13 +235,13 @@ vfarsumtr(Field &occur, const Field &field, const double refval)
 void
 vfarsumq(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
-  const auto &array2f = field2.vecf;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
+  const auto &array2f = field2.vec_f;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -258,7 +259,7 @@ vfarsumq(Field &field1, const Field &field2)
     }
   else
     {
-      if (field2.memtype == MEMTYPE_FLOAT)
+      if (field2.memType == MemType::Float)
         {
           for (size_t i = 0; i < len; i++) array1[i] += ((double) array2f[i]) * array2f[i];
         }
@@ -270,14 +271,14 @@ vfarsumq(Field &field1, const Field &field2)
 }
 
 void
-vfarsumqw(Field &field1, const Field &field2, double w)
+vfarsumqw(Field &field1, const Field &field2, const double w)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -302,12 +303,12 @@ vfarsumqw(Field &field1, const Field &field2, double w)
 void
 vfarsub(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -325,12 +326,12 @@ vfarsub(Field &field1, const Field &field2)
 void
 vfarmul(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -348,12 +349,12 @@ vfarmul(Field &field1, const Field &field2)
 void
 vfardiv(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   for (size_t i = 0; i < len; i++) array1[i] = DIVMN(array1[i], array2[i]);
@@ -364,12 +365,12 @@ vfardiv(Field &field1, const Field &field2)
 void
 vfaratan2(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   for (size_t i = 0; i < len; i++)
@@ -381,11 +382,11 @@ vfaratan2(Field &field1, const Field &field2)
 void
 vfarsetmiss(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss)
@@ -399,12 +400,12 @@ vfarsetmiss(Field &field1, const Field &field2)
 void
 vfarmin(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -413,26 +414,26 @@ vfarmin(Field &field1, const Field &field2)
         {
           array1[i] = DBL_IS_EQUAL(array2[i], missval2)
                           ? array1[i]
-                          : DBL_IS_EQUAL(array1[i], missval1) ? array2[i] : cdo::min(array1[i], array2[i]);
+                          : DBL_IS_EQUAL(array1[i], missval1) ? array2[i] : std::min(array1[i], array2[i]);
         }
 
       field1.nmiss = fieldNumMiss(field1);
     }
   else
     {
-      for (size_t i = 0; i < len; i++) array1[i] = cdo::min(array1[i], array2[i]);
+      for (size_t i = 0; i < len; i++) array1[i] = std::min(array1[i], array2[i]);
     }
 }
 
 void
 vfarmax(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -441,28 +442,28 @@ vfarmax(Field &field1, const Field &field2)
         {
           array1[i] = DBL_IS_EQUAL(array2[i], missval2)
                           ? array1[i]
-                          : DBL_IS_EQUAL(array1[i], missval1) ? array2[i] : cdo::max(array1[i], array2[i]);
+                          : DBL_IS_EQUAL(array1[i], missval1) ? array2[i] : std::max(array1[i], array2[i]);
         }
 
       field1.nmiss = fieldNumMiss(field1);
     }
   else
     {
-      for (size_t i = 0; i < len; i++) array1[i] = cdo::max(array1[i], array2[i]);
+      for (size_t i = 0; i < len; i++) array1[i] = std::max(array1[i], array2[i]);
     }
 }
 
 void
-vfarminidx(Field &field1, Field &field2, const Field &field3, int idx)
+vfarminidx(Field &field1, Field &field2, const Field &field3, const int idx)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  const double missval3 = field3.missval;
-  auto &array1 = field1.vec;
-  auto &array2 = field2.vec;
-  const auto &array3 = field3.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  const auto missval3 = field3.missval;
+  auto &array1 = field1.vec_d;
+  auto &array2 = field2.vec_d;
+  const auto &array3 = field3.vec_d;
 
-  const size_t len = field3.size;
+  const auto len = field3.size;
   if (len != field3.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field2.nmiss || field3.nmiss)
@@ -505,16 +506,16 @@ vfarminidx(Field &field1, Field &field2, const Field &field3, int idx)
 }
 
 void
-vfarmaxidx(Field &field1, Field &field2, const Field &field3, int idx)
+vfarmaxidx(Field &field1, Field &field2, const Field &field3, const int idx)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  const double missval3 = field3.missval;
-  auto &array1 = field1.vec;
-  auto &array2 = field2.vec;
-  const auto &array3 = field3.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  const auto missval3 = field3.missval;
+  auto &array1 = field1.vec_d;
+  auto &array2 = field2.vec_d;
+  const auto &array3 = field3.vec_d;
 
-  const size_t len = field2.size;
+  const auto len = field2.size;
   if (len != field3.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field2.nmiss || field3.nmiss)
@@ -557,16 +558,16 @@ vfarmaxidx(Field &field1, Field &field2, const Field &field3, int idx)
 }
 
 void
-vfarvar(Field &field1, const Field &field2, const Field &field3, int divisor)
+vfarvar(Field &field1, const Field &field2, const Field &field3, const int divisor)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
-  const auto &array3 = field3.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
+  const auto &array3 = field3.vec_d;
   double temp;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
@@ -592,12 +593,12 @@ vfarvar(Field &field1, const Field &field2, const Field &field3, int divisor)
 }
 
 void
-vfarstd(Field &field1, const Field &field2, const Field &field3, int divisor)
+vfarstd(Field &field1, const Field &field2, const Field &field3, const int divisor)
 {
-  const double missval1 = field1.missval;
-  auto &array1 = field1.vec;
+  const auto missval1 = field1.missval;
+  auto &array1 = field1.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   vfarvar(field1, field2, field3, divisor);
@@ -611,22 +612,22 @@ vfarstd(Field &field1, const Field &field2, const Field &field3, int divisor)
       }
     else
       {
-        array1[i] = IS_NOT_EQUAL(array1[i], 0) ? sqrt(array1[i]) : 0;
+        array1[i] = IS_NOT_EQUAL(array1[i], 0) ? std::sqrt(array1[i]) : 0;
       }
   field1.nmiss = nmiss;
 }
 
 void
-vfarcvar(Field &field1, const Field &field2, int nsets, int divisor)
+vfarcvar(Field &field1, const Field &field2, const int nsets, const int divisor)
 {
   const int nsetx = nsets - divisor;
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
   double temp;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (nsetx == 0)
@@ -656,12 +657,12 @@ vfarcvar(Field &field1, const Field &field2, int nsets, int divisor)
 }
 
 void
-vfarcstd(Field &field1, const Field &field2, int nsets, int divisor)
+vfarcstd(Field &field1, const Field &field2, const int nsets, const int divisor)
 {
-  const double missval1 = field1.missval;
-  auto &array1 = field1.vec;
+  const auto missval1 = field1.missval;
+  auto &array1 = field1.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   vfarcvar(field1, field2, nsets, divisor);
@@ -675,7 +676,7 @@ vfarcstd(Field &field1, const Field &field2, int nsets, int divisor)
       }
     else
       {
-        array1[i] = IS_NOT_EQUAL(array1[i], 0) ? sqrt(array1[i]) : 0;
+        array1[i] = IS_NOT_EQUAL(array1[i], 0) ? std::sqrt(array1[i]) : 0;
       }
 
   field1.nmiss = nmiss;
@@ -684,12 +685,12 @@ vfarcstd(Field &field1, const Field &field2, int nsets, int divisor)
 void
 vfarmoq(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field2.nmiss)
@@ -709,14 +710,14 @@ vfarmoq(Field &field1, const Field &field2)
 }
 
 void
-vfarmoqw(Field &field1, const Field &field2, double w)
+vfarmoqw(Field &field1, const Field &field2, const double w)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field2.nmiss)
@@ -751,12 +752,12 @@ vfarmoqw(Field &field1, const Field &field2, double w)
 void
 vfarcount(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
-  const size_t len = field1.size;
+  const auto len = field1.size;
   if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
   if (field1.nmiss || field2.nmiss)
