@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,21 +16,23 @@
 */
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "functs.h"
+#include "process_int.h"
 #include "array.h"
 
 static void
 faraddcplx(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
   if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  const size_t gridsize = gridInqSize(field1.grid);
+  const auto gridsize = gridInqSize(field1.grid);
   if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   for (size_t i = 0; i < gridsize; i++)
@@ -43,15 +45,15 @@ faraddcplx(Field &field1, const Field &field2)
 static void
 farsubcplx(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
   if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  const size_t gridsize = gridInqSize(field1.grid);
+  const auto gridsize = gridInqSize(field1.grid);
   if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   for (size_t i = 0; i < gridsize; i++)
@@ -64,22 +66,22 @@ farsubcplx(Field &field1, const Field &field2)
 static void
 farmulcplx(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
   if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  const size_t gridsize = gridInqSize(field1.grid);
+  const auto gridsize = gridInqSize(field1.grid);
   if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   // z1 x z2 = (x1x2 - y1y2) + i(x1y2 + x2y1)
   for (size_t i = 0; i < gridsize; i++)
     {
-      double a1r = array1[2 * i];
-      double a1i = array1[2 * i + 1];
+      const auto a1r = array1[2 * i];
+      const auto a1i = array1[2 * i + 1];
       array1[2 * i] = SUBMN(MULMN(a1r, array2[2 * i]), MULMN(a1i, array2[2 * i + 1]));
       array1[2 * i + 1] = ADDMN(MULMN(a1r, array2[2 * i + 1]), MULMN(a1i, array2[2 * i]));
     }
@@ -88,23 +90,23 @@ farmulcplx(Field &field1, const Field &field2)
 static void
 fardivcplx(Field &field1, const Field &field2)
 {
-  const double missval1 = field1.missval;
-  const double missval2 = field2.missval;
-  auto &array1 = field1.vec;
-  const auto &array2 = field2.vec;
+  const auto missval1 = field1.missval;
+  const auto missval2 = field2.missval;
+  auto &array1 = field1.vec_d;
+  const auto &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
   if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  const size_t gridsize = gridInqSize(field1.grid);
+  const auto gridsize = gridInqSize(field1.grid);
   if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   // z1 / z2 = (x1x2 + y1y2) / (x2x2 + y2y2) + i (y1x2 - x1y2) / (x2x2 + y2y2)
   for (size_t i = 0; i < gridsize; i++)
     {
-      const double a1r = array1[2 * i];
-      const double a1i = array1[2 * i + 1];
-      const double denominator = ADDMN(MULMN(array2[2 * i], array2[2 * i]), MULMN(array2[2 * i + 1], array2[2 * i + 1]));
+      const auto a1r = array1[2 * i];
+      const auto a1i = array1[2 * i + 1];
+      const auto denominator = ADDMN(MULMN(array2[2 * i], array2[2 * i]), MULMN(array2[2 * i + 1], array2[2 * i + 1]));
       array1[2 * i] = DIVMN(ADDMN(MULMN(a1r, array2[2 * i]), MULMN(a1i, array2[2 * i + 1])), denominator);
       array1[2 * i + 1] = DIVMN(SUBMN(MULMN(a1i, array2[2 * i]), MULMN(a1r, array2[2 * i + 1])), denominator);
     }
