@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,16 +15,19 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+#include "cdo_output.h"
+#include "field.h"
+#include "array.h"
+#include "functs.h"
 
 void
-vfarcmul(Field &field, double rconst)
+vfarcmul(Field &field, const double rconst)
 {
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t len = field.size;
+  const auto len = field.size;
 
   if (field.nmiss)
     {
@@ -32,23 +35,18 @@ vfarcmul(Field &field, double rconst)
     }
   else
     {
-      /*
-#ifdef  _OPENMP
-#pragma omp parallel for default(shared) private(i)
-#endif
-      */
       for (size_t i = 0; i < len; i++) array[i] *= rconst;
     }
 }
 
 void
-vfarcdiv(Field &field, double rconst)
+vfarcdiv(Field &field, const double rconst)
 {
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t len = field.size;
+  const auto len = field.size;
 
   if (field.nmiss || IS_EQUAL(rconst, 0))
     {
@@ -63,13 +61,13 @@ vfarcdiv(Field &field, double rconst)
 }
 
 void
-vfarcadd(Field &field, double rconst)
+vfarcadd(Field &field, const double rconst)
 {
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t len = field.size;
+  const auto len = field.size;
 
   if (field.nmiss)
     {
@@ -82,18 +80,18 @@ vfarcadd(Field &field, double rconst)
 }
 
 void
-vfarcsub(Field &field, double rconst)
+vfarcsub(Field &field, const double rconst)
 {
   vfarcadd(field, -rconst);
 }
 
 void
-vfarcmin(Field &field, double rconst)
+vfarcmin(Field &field, const double rconst)
 {
-  const double missval1 = field.missval;
-  auto &array = field.vec;
+  const auto missval1 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t len = field.size;
+  const auto len = field.size;
 
   if (field.nmiss)
     {
@@ -108,12 +106,12 @@ vfarcmin(Field &field, double rconst)
 }
 
 void
-vfarcmax(Field &field, double rconst)
+vfarcmax(Field &field, const double rconst)
 {
-  const double missval1 = field.missval;
-  auto &array = field.vec;
+  const auto missval1 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t len = field.size;
+  const auto len = field.size;
 
   if (field.nmiss)
     {
@@ -128,12 +126,12 @@ vfarcmax(Field &field, double rconst)
 }
 
 void
-vfarmod(Field &field, double divisor)
+vfarmod(Field &field, const double divisor)
 {
-  const double missval1 = field.missval;
-  auto &array = field.vec;
+  const auto missval1 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t len = field.size;
+  const auto len = field.size;
 
   for (size_t i = 0; i < len; i++)
     {
@@ -142,7 +140,7 @@ vfarmod(Field &field, double divisor)
 }
 
 void
-vfarcfun(Field &field, double rconst, int function)
+vfarcfun(Field &field, const double rconst, const int function)
 {
   switch (function)
     {
