@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,23 +16,25 @@
 */
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "functs.h"
+#include "process_int.h"
 
 static void
 vfarcmulcplx(Field &field, const double rconst[2])
 {
-  const int grid = field.grid;
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto grid = field.grid;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t gridsize = gridInqSize(grid);
+  const auto gridsize = gridInqSize(grid);
 
   // z1 x z2 = (x1x2 - y1y2) + i(x1y2 + x2y1)
   for (size_t i = 0; i < gridsize; i++)
     {
-      double a1r = array[2 * i];
-      double a1i = array[2 * i + 1];
+      const auto a1r = array[2 * i];
+      const auto a1i = array[2 * i + 1];
       array[2 * i] = SUBMN(MULMN(a1r, rconst[0]), MULMN(a1i, rconst[1]));
       array[2 * i + 1] = ADDMN(MULMN(a1r, rconst[1]), MULMN(a1i, rconst[0]));
     }
@@ -41,19 +43,19 @@ vfarcmulcplx(Field &field, const double rconst[2])
 static void
 vfarcdivcplx(Field &field, const double rconst[2])
 {
-  const int grid = field.grid;
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto grid = field.grid;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t gridsize = gridInqSize(grid);
+  const auto gridsize = gridInqSize(grid);
 
   // z1 / z2 = (x1x2 + y1y2) / (x2x2 + y2y2) + i (y1x2 - x1y2) / (x2x2 + y2y2)
   for (size_t i = 0; i < gridsize; i++)
     {
-      double a1r = array[2 * i];
-      double a1i = array[2 * i + 1];
-      double denominator = ADDMN(MULMN(rconst[0], rconst[0]), MULMN(rconst[1], rconst[1]));
+      const auto a1r = array[2 * i];
+      const auto a1i = array[2 * i + 1];
+      const auto denominator = ADDMN(MULMN(rconst[0], rconst[0]), MULMN(rconst[1], rconst[1]));
       array[2 * i] = DIVMN(ADDMN(MULMN(a1r, rconst[0]), MULMN(a1i, rconst[1])), denominator);
       array[2 * i + 1] = DIVMN(SUBMN(MULMN(a1i, rconst[0]), MULMN(a1r, rconst[1])), denominator);
     }
@@ -62,12 +64,12 @@ vfarcdivcplx(Field &field, const double rconst[2])
 static void
 vfarcaddcplx(Field &field, const double rconst[2])
 {
-  const int grid = field.grid;
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto grid = field.grid;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t gridsize = gridInqSize(grid);
+  const auto gridsize = gridInqSize(grid);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -79,12 +81,12 @@ vfarcaddcplx(Field &field, const double rconst[2])
 static void
 vfarcsubcplx(Field &field, const double rconst[2])
 {
-  const int grid = field.grid;
-  const double missval1 = field.missval;
-  const double missval2 = field.missval;
-  auto &array = field.vec;
+  const auto grid = field.grid;
+  const auto missval1 = field.missval;
+  const auto missval2 = field.missval;
+  auto &array = field.vec_d;
 
-  const size_t gridsize = gridInqSize(grid);
+  const auto gridsize = gridInqSize(grid);
 
   for (size_t i = 0; i < gridsize; i++)
     {
