@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,34 +16,36 @@
 */
 
 #include <cdi.h>
-#include <cdo_int.h>
+
 #include "field.h"
+#include "cdo_vlist.h"
+#include "cdo_options.h"
 
 void
 fieldsFromVlistKernel(const int vlistID, FieldVector2D &field2D, int ptype, bool lfill, double fillValue)
 {
-  const int nvars = vlistNvars(vlistID);
+  const auto nvars = vlistNvars(vlistID);
   field2D.resize(nvars);
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const int gridID = vlistInqVarGrid(vlistID, varID);
-      const size_t gridsize = gridInqSize(gridID);
+      const auto gridID = vlistInqVarGrid(vlistID, varID);
+      const auto gridsize = gridInqSize(gridID);
       const size_t nwpv = vlistInqNWPV(vlistID, varID);  // number of words per value; real:1  complex:2
-      const size_t size = gridsize * nwpv;
-      const int zaxisID = vlistInqVarZaxis(vlistID, varID);
-      const int nlevel = zaxisInqSize(zaxisID);
-      const double missval = vlistInqVarMissval(vlistID, varID);
+      const auto size = gridsize * nwpv;
+      const auto zaxisID = vlistInqVarZaxis(vlistID, varID);
+      const auto nlevels = zaxisInqSize(zaxisID);
+      const auto missval = vlistInqVarMissval(vlistID, varID);
 
-      field2D[varID].resize(nlevel);
+      field2D[varID].resize(nlevels);
 
-      for (int levelID = 0; levelID < nlevel; ++levelID)
+      for (int levelID = 0; levelID < nlevels; ++levelID)
         {
-          Field &field = field2D[varID][levelID];
+          auto &field = field2D[varID][levelID];
           field.nwpv = nwpv;
           field.grid = gridID;
           field.size = size;
-          if (ptype & FIELD_FLT) field.memtype = MEMTYPE_FLOAT;
+          field.memType = (ptype & FIELD_FLT) ? MemType::Float : MemType::Native;
           field.missval = missval;
 
           if (ptype & FIELD_VEC)
@@ -54,7 +56,7 @@ fieldsFromVlistKernel(const int vlistID, FieldVector2D &field2D, int ptype, bool
                     field.resizef(size, (float) fillValue);
                   else
                     field.resizef(size);
-                 }
+                }
               else
                 {
                   if (lfill)
@@ -67,6 +69,12 @@ fieldsFromVlistKernel(const int vlistID, FieldVector2D &field2D, int ptype, bool
     }
 }
 
+void
+fieldsFromVlist(const int vlistID, FieldVector2D &field2D)
+{
+  fieldsFromVlistKernel(vlistID, field2D, 0, false, 0);
+}
+
 void
 fieldsFromVlist(const int vlistID, FieldVector2D &field2D, int ptype)
 {
