@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@ class FileStream : public CdoStream
 {
 public:
   // Constructors
-  FileStream(std::string p_fileStream);
+  FileStream(const std::string &p_fileStream);
   // ---
 
   // CdoStream Interface functions
@@ -39,10 +39,14 @@ public:
   void inqRecord(int *varID, int *levelID) override;
   void defRecord(int varID, int levelID) override;
 
-  void readRecord(double *data, size_t *nmiss) override;
-  void readRecord(float *data, size_t *nmiss) override;
-  void writeRecord(double *data, size_t nmiss) override;
-  void writeRecord(float *data, size_t nmiss) override;
+  void readRecord(float *p_data, size_t *nmiss) override;
+  void readRecord(double *p_data, size_t *nmiss) override;
+  void readRecord(Field *p_field, size_t *nmiss) override;
+
+  void writeRecord(float *p_data, size_t nmiss) override;
+  void writeRecord(double *p_data, size_t nmiss) override;
+  void writeRecord(Field *p_field, size_t nmiss) override;
+
   void copyRecord(CdoStreamID p_fileStream) override;
 
   int inqTimestep(int tsID) override;
@@ -59,9 +63,9 @@ public:
   // FileStreamOnly
   int getFileID();
   static void
-  enableTimers(bool p_enable)
+  enableTimers(const bool p_enable)
   {
-      FileStream::TimerEnabled = p_enable;
+    FileStream::TimerEnabled = p_enable;
   }
   static bool
   timersEnabled()
@@ -75,8 +79,8 @@ protected:
 
 private:
   FileStream() = delete;
-  void checkDatarange(int varID, double *array, size_t nmiss);
   std::string m_filename;
+  void checkDatarange(int varID, double *array, size_t nmiss);
 };
 
 #endif
