@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,14 +27,14 @@
 static void
 lonlat_to_xyz(double lon, double lat, double *xyz)
 {
-  double coslat = cos(lat);
-  xyz[0] = coslat * cos(lon);
-  xyz[1] = coslat * sin(lon);
-  xyz[2] = sin(lat);
+  const auto coslat = std::cos(lat);
+  xyz[0] = coslat * std::cos(lon);
+  xyz[1] = coslat * std::sin(lon);
+  xyz[2] = std::sin(lat);
 }
 
 static void
-cross_product(const double *restrict a, const double *restrict b, double *restrict c)
+cross_product(const double *a, const double *b, double *c)
 {
   c[0] = a[1] * b[2] - a[2] * b[1];
   c[1] = a[2] * b[0] - a[0] * b[2];
@@ -43,14 +43,14 @@ cross_product(const double *restrict a, const double *restrict b, double *restri
 
 /*
 static
-double scalar_product(const double *restrict a, const double *restrict b)
+double scalar_product(const double *a, const double *b)
 {
   return (a[0]*b[0] + a[1]*b[1] + a[2]*b[2]);
 }
 */
 
 static double
-norm(const double *restrict a)
+norm(const double *a)
 {
   return (a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
 }
@@ -96,7 +96,7 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double
   if ( std::fabs(area) > 0.0 )
     {
       for ( m = 0; m < M; m++ )
-        s[m] = sqrt(s[m]);
+        s[m] = std::sqrt(s[m]);
 
       for ( m = 0; m < M; m++ )
         for ( i = 0; i < 3; i++ )
@@ -120,7 +120,7 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double
           ca[m] = - scalar_product(u[m], u[(m+1)%M]);
           if ( ca[m] < -1.0 ) ca[m] = -1.0;
           if ( ca[m] >  1.0 ) ca[m] =  1.0;
-          a[m] = acos(ca[m]);
+          a[m] = std::acos(ca[m]);
         }
 
       //  Compute areas = a1 + a2 + a3 - (M-2) * pi. here for a unit sphere:
@@ -152,27 +152,27 @@ double mod_cell_area(int num_corners, double *cell_corner_lon, double
  * Licensed under the University of Illinois-NCSA License.
  */
 static double
-mod_tri_area(const double *restrict u, const double *restrict v, const double *restrict w)
+mod_tri_area(const double *u, const double *v, const double *w)
 {
   double tmp_vec[3];
 
   cross_product(u, v, tmp_vec);
-  double sina = sqrt(norm(tmp_vec));
-  double a = asin(sina);
+  double sina = std::sqrt(norm(tmp_vec));
+  double a = std::asin(sina);
 
   cross_product(u, w, tmp_vec);
-  double sinb = sqrt(norm(tmp_vec));
-  double b = asin(sinb);
+  double sinb = std::sqrt(norm(tmp_vec));
+  double b = std::asin(sinb);
 
   cross_product(w, v, tmp_vec);
-  double sinc = sqrt(norm(tmp_vec));
-  double c = asin(sinc);
+  double sinc = std::sqrt(norm(tmp_vec));
+  double c = std::asin(sinc);
 
   double s = 0.5 * (a + b + c);
 
-  double t = tan(s * 0.5) * tan((s - a) * 0.5) * tan((s - b) * 0.5) * tan((s - c) * 0.5);
+  double t = std::tan(s * 0.5) * std::tan((s - a) * 0.5) * std::tan((s - b) * 0.5) * std::tan((s - c) * 0.5);
 
-  double area = std::fabs(4.0 * atan(sqrt(std::fabs(t))));
+  double area = std::fabs(4.0 * std::atan(std::sqrt(std::fabs(t))));
 
   return area;
 }
@@ -262,8 +262,8 @@ mod_huiliers_area2(int num_corners, double *cell_corner_lon, double *cell_corner
 static void
 getLonLatCorner(size_t nx, size_t idx, const double *grid_corner_lon, const double *grid_corner_lat, double *lons, double *lats)
 {
-  size_t j = idx / nx;
-  size_t i = idx - j * nx;
+  const auto j = idx / nx;
+  const auto i = idx - j * nx;
 
   lons[0] = grid_corner_lon[2 * i];
   lons[1] = grid_corner_lon[2 * i + 1];
@@ -293,26 +293,25 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
 
   int status = 0;
 
-  if (gridInqXvals(gridID, nullptr) == 0 && gridInqYvals(gridID, nullptr) == 0 && gridInqXbounds(gridID, nullptr) == 0
-      && gridInqYbounds(gridID, nullptr) == 0)
+  if (!gridHasCoordinates(gridID) && !gridHasBounds(gridID))
     {
       cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
       return 1;
     }
 
   char unitstr[CDI_MAX_NAME];
-  size_t gridsize = gridInqSize(gridID);
-  size_t nlon = gridInqXsize(gridID);
-  size_t nlat = gridInqYsize(gridID);
+  auto gridsize = gridInqSize(gridID);
+  auto nlon = gridInqXsize(gridID);
+  auto nlat = gridInqYsize(gridID);
   if (!nlon) nlon = 1;
   if (!nlat) nlat = 1;
-  std::vector<double> grid_corner_lon(nlon * 2);
-  std::vector<double> grid_corner_lat(nlat * 2);
+  std::vector<double> grid_corner_lon(nlon * 2), grid_corner_lat(nlat * 2);
 
   if (gridInqXbounds(gridID, nullptr))
     {
       gridInqXbounds(gridID, grid_corner_lon.data());
-      gridInqXunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
   else if (lweights && nlon == 1)
     {
@@ -329,13 +328,15 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
       std::vector<double> grid_center_lon(nlon);
       gridInqXvals(gridID, grid_center_lon.data());
       grid_gen_bounds(nlon, grid_center_lon, grid_corner_lon);
-      gridInqXunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
 
   if (gridInqYbounds(gridID, nullptr))
     {
       gridInqYbounds(gridID, grid_corner_lat.data());
-      gridInqYunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
   else if (lweights && nlat == 1)
     {
@@ -353,7 +354,8 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
       gridInqYvals(gridID, grid_center_lat.data());
       grid_gen_bounds(nlat, grid_center_lat, grid_corner_lat);
       grid_check_lat_borders(nlat * 2, grid_corner_lat.data());
-      gridInqYunits(gridID, unitstr);
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, unitstr, &length);
     }
 
   grid_to_radian(unitstr, nlon * 2, grid_corner_lon.data(), "grid cell corner longitudes");
@@ -362,7 +364,7 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
   int nv = 4;
   double findex = 0;
 
-  Progress::init();
+  progress::init();
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(gridsize, nlon, findex, area, nv, grid_corner_lon, grid_corner_lat)
@@ -373,15 +375,14 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
 #pragma omp atomic
 #endif
       findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / gridsize);
 
       double lons[4], lats[4];
       getLonLatCorner(nlon, i, grid_corner_lon.data(), grid_corner_lat.data(), lons, lats);
-      // area[i] = mod_cell_area(nv, grid_corner_lon+i*nv, grid_corner_lat+i*nv);
       area[i] = mod_huiliers_area(nv, lons, lats);
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   return status;
 }
@@ -397,11 +398,10 @@ static int
 gridGenAreaUnstruct(int gridID, double *area)
 {
   int status = 0;
-  bool lgrid_gen_bounds = false;
   bool lgriddestroy = false;
 
-  size_t gridsize = gridInqSize(gridID);
-  int gridtype = gridInqType(gridID);
+  auto gridsize = gridInqSize(gridID);
+  auto gridtype = gridInqType(gridID);
 
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
@@ -414,24 +414,21 @@ gridGenAreaUnstruct(int gridID, double *area)
         {
           lgriddestroy = true;
           gridID = gridToCurvilinear(gridID, 1);
-          lgrid_gen_bounds = true;
         }
     }
 
-  if (gridtype == GRID_UNSTRUCTURED)
+  if (gridtype == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID))
     {
-      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+      const auto reference = dereferenceGrid(gridID);
+      if (reference.exists)
         {
-          if (gridInqNumber(gridID) > 0)
+          if (!reference.isValid)
             {
-              lgriddestroy = true;
-              gridID = referenceToGrid(gridID);
-              if (gridID == -1)
-                {
-                  cdoWarning("Reference to source grid not found!");
-                  return 1;
-                }
+              cdoWarning("Reference to source grid not found!");
+              return 1;
             }
+          gridID = reference.gridID;
+          lgriddestroy = true;
         }
     }
 
@@ -439,7 +436,7 @@ gridGenAreaUnstruct(int gridID, double *area)
 
   size_t nv = (gridtype == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+  if (!gridHasCoordinates(gridID))
     {
       cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
       return 1;
@@ -451,57 +448,32 @@ gridGenAreaUnstruct(int gridID, double *area)
       return 1;
     }
 
-  char xunitstr[CDI_MAX_NAME];
-  char yunitstr[CDI_MAX_NAME];
-  gridInqXunits(gridID, xunitstr);
-  gridInqYunits(gridID, yunitstr);
-
-  std::vector<double> grid_center_lon(gridsize);
-  std::vector<double> grid_center_lat(gridsize);
-
+  std::vector<double> grid_center_lon(gridsize), grid_center_lat(gridsize);
   gridInqXvals(gridID, grid_center_lon.data());
   gridInqYvals(gridID, grid_center_lat.data());
 
-  std::vector<double> grid_corner_lon(nv * gridsize);
-  std::vector<double> grid_corner_lat(nv * gridsize);
+  std::vector<double> grid_corner_lon(nv * gridsize), grid_corner_lat(nv * gridsize);
 
-  if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
-    {
-      gridInqXbounds(gridID, grid_corner_lon.data());
-      gridInqYbounds(gridID, grid_corner_lat.data());
-    }
-  else
-    {
-      if (lgrid_gen_bounds)
-        {
-          const size_t nlon = gridInqXsize(gridID);
-          const size_t nlat = gridInqYsize(gridID);
-          const double dlon = (nlon == 1) ? 1 : 0;
-          grid_cell_center_to_bounds_X2D(xunitstr, nlon, nlat, grid_center_lon.data(), grid_corner_lon.data(), dlon);
-          grid_cell_center_to_bounds_Y2D(yunitstr, nlon, nlat, grid_center_lat.data(), grid_corner_lat.data());
-        }
-      else
-        {
-          status = 1;
-          return status;
-        }
-    }
+  if (!gridHasBounds(gridID)) return 1;
 
-  grid_to_radian(xunitstr, gridsize, grid_center_lon.data(), "grid1 center longitudes");
-  grid_to_radian(xunitstr, gridsize * nv, grid_corner_lon.data(), "grid1 corner longitudes");
+  gridInqXbounds(gridID, grid_corner_lon.data());
+  gridInqYbounds(gridID, grid_corner_lat.data());
 
-  grid_to_radian(yunitstr, gridsize, grid_center_lat.data(), "grid1 center latitudes");
-  grid_to_radian(yunitstr, gridsize * nv, grid_corner_lat.data(), "grid1 corner latitudes");
+  cdo_grid_to_radian(gridID, CDI_XAXIS, gridsize, grid_center_lon.data(), "grid1 center longitudes");
+  cdo_grid_to_radian(gridID, CDI_XAXIS, gridsize * nv, grid_corner_lon.data(), "grid1 corner longitudes");
+
+  cdo_grid_to_radian(gridID, CDI_YAXIS, gridsize, grid_center_lat.data(), "grid1 center latitudes");
+  cdo_grid_to_radian(gridID, CDI_YAXIS, gridsize * nv, grid_corner_lat.data(), "grid1 corner latitudes");
 
   if (lgriddestroy) gridDestroy(gridID);
 
   double findex = 0;
 
-  Progress::init();
+  progress::init();
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  shared(gridsize, nv, findex, area, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat)
+    shared(gridsize, nv, findex, area, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -509,9 +481,8 @@ gridGenAreaUnstruct(int gridID, double *area)
 #pragma omp atomic
 #endif
       findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / gridsize);
 
-      // area[i] = mod_cell_area(nv, grid_corner_lon+i*nv, grid_corner_lat+i*nv);
       if (nv <= 4)
         area[i] = mod_huiliers_area(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv]);
       else
@@ -519,7 +490,7 @@ gridGenAreaUnstruct(int gridID, double *area)
             = mod_huiliers_area2(nv, &grid_corner_lon[i * nv], &grid_corner_lat[i * nv], grid_center_lon[i], grid_center_lat[i]);
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   return status;
 }
@@ -529,18 +500,16 @@ gridGenArea(const int gridID, double *area)
 {
   int status = 0;
 
-  const size_t gridsize = gridInqSize(gridID);
-  const int gridtype = gridInqType(gridID);
-  const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  const auto gridsize = gridInqSize(gridID);
+  const auto gridtype = gridInqType(gridID);
 
   if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)
     {
       bool lweights = false;
       status = gridGenAreaReg2D(gridID, area, lweights);
     }
-  else if (projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC
-           || projtype == CDI_PROJ_STERE || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
-           || gridtype == GRID_GAUSSIAN_REDUCED)
+  else if (gridProjIsSupported(gridID)|| gridtype == GRID_GME || gridtype == GRID_CURVILINEAR
+           || gridtype == GRID_UNSTRUCTURED || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       status = gridGenAreaUnstruct(gridID, area);
     }
@@ -560,9 +529,6 @@ gridGenArea(const int gridID, double *area)
     {
       double total_area = 0;
       for (size_t i = 0; i < gridsize; ++i) total_area += area[i];
-      int nzero = 0;
-      for (size_t i = 0; i < gridsize; ++i)
-        if (IS_EQUAL(area[i], 0.)) nzero++;
       if (IS_EQUAL(total_area, 0.)) status = 2;
     }
 
