@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,12 +14,15 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include <cdi.h>
-#include "cdo_int.h"
-#include <mpim_grid.h>
 
 #include <cstring>
 
+#include <cdi.h>
+
+#include "process_int.h"
+#include "cdo_cdi_wrapper.h"
+#include <mpim_grid.h>
+
 /*
 @Function  cdo_define_destagered_grid
 @Title     Define a de-staggered grid for U and V
@@ -41,18 +44,20 @@ int
 cdo_define_destagered_grid(int gridID_u_stag, int gridID_v_stag, double *destagGridOffsets)
 {
   /* Example of horizontal grids (Hirlam LAMH_D11):
+
        U : lonlat       > size      : dim = 399300  nlon = 726  nlat = 550
-                          rlon      : first = -30.15  last = 42.35  inc = 0.1
-  degrees rlat      : first = -30.8  last = 24.1  inc = 0.1  degrees northpole :
-  lon = -195  lat = 30 V : lonlat       > size      : dim = 399300  nlon = 726
-  nlat = 550 rlon      : first = -30.2  last = 42.3  inc = 0.1  degrees rlat
-  : first = -30.75  last = 24.15  inc = 0.1  degrees northpole : lon = -195  lat
-  = 30
+                          rlon      : first = -30.15  last = 42.35  inc = 0.1 degrees
+                          rlat      : first = -30.8  last = 24.1  inc = 0.1  degrees
+                          northpole : lon = -195  lat = 30
+       V : lonlat       > size      : dim = 399300  nlon = 726  nlat = 550
+                          rlon      : first = -30.2  last = 42.3  inc = 0.1  degrees
+                          rlat      : first = -30.75  last = 24.15  inc = 0.1  degrees
+                          northpole : lon = -195  lat = 30
   =>   RESULT:
        R : lonlat       > size      : dim = 399300  nlon = 726  nlat = 550
-                          rlon      : first = -30.2  last = 42.3  inc = 0.1
-  degrees rlat      : first = -30.8  last = 24.1  inc = 0.1  degrees northpole :
-  lon = -195  lat = 30
+                          rlon      : first = -30.2  last = 42.3  inc = 0.1 degrees
+                          rlat      : first = -30.8  last = 24.1  inc = 0.1  degrees
+                          northpole : lon = -195  lat = 30
   */
   if (cdoDebugExt)
     cdoPrint("%s(gridID_u=%d,gridID_v=%d,destagGridOffsets(%02.1f,%02.1f)) ...\n", __func__, gridID_u_stag, gridID_v_stag,
@@ -190,7 +195,7 @@ cdo_define_sample_grid(int gridSrcID, int sampleFactor)
               longitudeOfFirstGridPointInDegrees = -7.89
               latitudeOfFirstGridPointInDegrees = 42.935
   */
-  Debug(cdoDebugExt,"%s(gridSrcID=%d, sampleFactor=%d) ...", __func__, gridSrcID, sampleFactor);
+  Debug(cdoDebugExt, "%s(gridSrcID=%d, sampleFactor=%d) ...", __func__, gridSrcID, sampleFactor);
 
   int gridtype = gridInqType(gridSrcID);
   if (!(gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_PROJECTION || gridtype == GRID_CURVILINEAR
@@ -218,27 +223,26 @@ cdo_define_sample_grid(int gridSrcID, int sampleFactor)
   gridDefNP(gridID_sampled, gridInqNP(gridSrcID));
   gridDefDatatype(gridID_sampled, gridInqDatatype(gridSrcID));
 
-  grid_copy_attributes(gridSrcID, gridID_sampled);
+  grid_copy_keys(gridSrcID, gridID_sampled);
 
   if (gridtype == GRID_PROJECTION) grid_copy_mapping(gridSrcID, gridID_sampled);
 
-  if (gridInqXvals(gridSrcID, nullptr) && gridInqYvals(gridSrcID, nullptr))
+  if (gridHasCoordinates(gridSrcID))
     {
       if (gridtype == GRID_CURVILINEAR)
         {
-          double *vals = (double *) Malloc(gridXsize * gridYsize * sizeof(double));
-          gridInqXvals(gridSrcID, vals);
-          double *pvals = vals;
+          std::vector<double> vals(gridXsize * gridYsize);
+          gridInqXvals(gridSrcID, vals.data());
+          double *pvals = vals.data();
           for (int j = 0; j < gridYsize; j += sampleFactor)
             for (int i = 0; i < gridXsize; i += sampleFactor) *pvals++ = vals[j * gridXsize + i];
-          gridDefXvals(gridID_sampled, vals);
+          gridDefXvals(gridID_sampled, vals.data());
 
-          gridInqYvals(gridSrcID, vals);
-          pvals = vals;
+          gridInqYvals(gridSrcID, vals.data());
+          pvals = vals.data();
           for (int j = 0; j < gridYsize; j += sampleFactor)
             for (int i = 0; i < gridXsize; i += sampleFactor) *pvals++ = vals[j * gridXsize + i];
-          gridDefYvals(gridID_sampled, vals);
-          Free(vals);
+          gridDefYvals(gridID_sampled, vals.data());
         }
       else
         {
@@ -381,10 +385,9 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
   gridDefXsize(gridID_sampled, xsize);
   gridDefYsize(gridID_sampled, ysize);
 
-  if (gridInqXvals(gridSrcID, nullptr) && gridInqYvals(gridSrcID, nullptr))
+  if (gridHasCoordinates(gridSrcID))
     {
-      std::vector<double> xvals(gridXsize);
-      std::vector<double> yvals(gridYsize);
+      std::vector<double> xvals(gridXsize), yvals(gridYsize);
       gridInqXvals(gridSrcID, xvals.data());
       gridInqYvals(gridSrcID, yvals.data());
       gridDefXvals(gridID_sampled, xvals.data());
@@ -394,7 +397,7 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
   gridDefNP(gridID_sampled, gridInqNP(gridSrcID));
   gridDefDatatype(gridID_sampled, gridInqDatatype(gridSrcID));
 
-  grid_copy_attributes(gridSrcID, gridID_sampled);
+  grid_copy_keys(gridSrcID, gridID_sampled);
 
   xval_0 = gridInqXval(gridIDcurvl, subJ0 * gridXsize + subI0);
   yval_0 = gridInqYval(gridIDcurvl, subJ0 * gridXsize + subI0);
