@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,13 +15,13 @@
   GNU General Public License for more details.
 */
 
-#include <valarray>
-
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "process_int.h"
 #include <mpim_grid.h>
 #include "griddes.h"
 #include "util_string.h"
+#include "gaussian_latitudes.h"
 
 size_t genIcosphereCoords(int subdivisions, bool lbounds, std::vector<double> &xvals, std::vector<double> &yvals,
                           std::vector<double> &xbounds, std::vector<double> &ybounds);
@@ -77,7 +77,7 @@ gen_grid_icosphere(GridDesciption &grid, const char *pline)
   grid.type = gridtype;
   if (lbounds) grid.nvertex = 3;
 
-  const size_t ncells = genIcosphereCoords(b + 1, lbounds, grid.xvals, grid.yvals, grid.xbounds, grid.ybounds);
+  const auto ncells = genIcosphereCoords(b + 1, lbounds, grid.xvals, grid.yvals, grid.xbounds, grid.ybounds);
   grid.xsize = ncells;
   grid.ysize = ncells;
   strcpy(grid.xname, "clon");
@@ -95,7 +95,7 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
   if (*pline != 0 && (*pline == '+' || *pline == '-') && (isdigit((int) *(pline + 1)) || ispunct((int) *(pline + 1))))
     {
       char *endptr = (char *) pline;
-      const double off = strtod(pline, &endptr);
+      const auto off = strtod(pline, &endptr);
       pline = endptr;
 
       lon1 -= off;
@@ -158,8 +158,8 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
 
   if (lon1 >= lon2 || lat1 >= lat2) cdoAbort("Invalid grid box: lon1=%g lon2=%g lat1=%g lat2=%g", lon1, lon2, lat1, lat2);
 
-  const size_t nlon = (size_t)((lon2 - lon1) / inc + 0.5);
-  const size_t nlat = (size_t)((lat2 - lat1) / inc + 0.5);
+  const auto nlon = (size_t)((lon2 - lon1) / inc + 0.5);
+  const auto nlat = (size_t)((lat2 - lat1) / inc + 0.5);
 
   grid.xvals.resize(nlon);
   grid.yvals.resize(nlat);
@@ -176,7 +176,7 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
     {
       std::vector<double> yvals(nlat);
       for (size_t j = 0; j < nlat; ++j) yvals[j] = grid.yvals[j];
-      const size_t gridsize = nlon * nlat;
+      const auto gridsize = nlon * nlat;
       grid.xvals.resize(gridsize);
       grid.yvals.resize(gridsize);
       for (size_t j = 0; j < nlat; ++j)
@@ -232,25 +232,25 @@ gen_grid_gme(GridDesciption &grid, const char *pline)
     }
 }
 
-static void
+void
 gaussianLatitudesInDegrees(std::vector<double> &lats, std::vector<double> &lat_bounds, size_t nlat)
 {
   // lats(nlat)
   // lat_bounds(nlat+1)
   std::vector<double> latw(nlat), latw_cumsum(nlat);
 
-  gaussianLatitudes(&lats[0], &latw[0], nlat);
+  gaussian_latitudes(nlat, lats.data(), latw.data());
 
-  for (size_t j = 0; j < nlat; ++j) lats[j] = RAD2DEG*std::asin(lats[j]);
+  for (size_t j = 0; j < nlat; ++j) lats[j] = RAD2DEG * std::asin(lats[j]);
 
   latw_cumsum[0] = latw[0];
-  for (size_t j = 1; j < nlat; ++j) latw_cumsum[j] = latw_cumsum[j-1] + latw[j];
+  for (size_t j = 1; j < nlat; ++j) latw_cumsum[j] = latw_cumsum[j - 1] + latw[j];
 
   lat_bounds[0] = 1.0;
-  for (size_t j = 1; j < nlat; ++j) lat_bounds[j] = 1.0 - latw_cumsum[j-1];
+  for (size_t j = 1; j < nlat; ++j) lat_bounds[j] = 1.0 - latw_cumsum[j - 1];
   lat_bounds[nlat] = -1.0;
-  
-  for (size_t j = 0; j < nlat+1; ++j) lat_bounds[j] = RAD2DEG*std::asin(lat_bounds[j]);
+
+  for (size_t j = 0; j < nlat + 1; ++j) lat_bounds[j] = RAD2DEG * std::asin(lat_bounds[j]);
 }
 
 static void
@@ -258,69 +258,68 @@ gen_grid_gea(GridDesciption &grid, const char *pline)
 {
   if (isdigit((int) *pline))
     {
-      char *endptr = (char *) pline;
-      const double dx = strtod(pline, &endptr);
-      const double dy = dx;
-      if (*endptr == 0)
+      auto endptr = (char *) pline;
+      const auto dx = strtod(pline, &endptr);
+      if (*endptr != 0) return;
+
+      const auto dy = dx;
+      constexpr auto re = 6378.137;
+      constexpr auto f = 1.0 / 298.257223563;
+      constexpr auto rp = re * (1.0 - f);
+      constexpr auto polar_circumference = 2.0 * M_PI * rp;
+      constexpr auto equator_circumference = 2.0 * M_PI * re;
+
+      size_t nlat = 0.5 * polar_circumference / dy;
+      if (nlat % 2) nlat++;
+
+      std::vector<double> lats(nlat), lat_bounds(nlat + 1);
+      gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+
+      std::vector<double> cell_height(nlat);
+      for (size_t j = 0; j < nlat; ++j) cell_height[j] = 0.25 * polar_circumference * (lat_bounds[j] - lat_bounds[j + 1]) / 90.0;
+
+      size_t nlone = equator_circumference / dx;
+      if (nlone % 2) nlone++;
+
+      std::vector<int> reducedPoints(nlat);
+      size_t ncells = 0;
+      for (size_t j = 0; j < nlat; ++j)
         {
-          constexpr double re = 6378.137;
-          constexpr double f = 1.0 / 298.257223563;
-          constexpr double rp = re * (1.0 - f);
-          constexpr double polar_circumference = 2.0 * M_PI * rp;
-          constexpr double equator_circumference = 2.0 * M_PI * re;
+          const auto rlat = re * std::cos(DEG2RAD * lats[j]);
+          const auto circumference = 2.0 * M_PI * rlat;
+          const auto dx_to_use = dx * dy / cell_height[j];
 
-          size_t nlat = 0.5 * polar_circumference / dy;
-          if (nlat&2) nlat = nlat + 1;
+          size_t nlon = std::max((int) std::lround(circumference / dx_to_use), 1);
+          if (nlon % 2) nlon++;
 
-          std::vector<double> lats(nlat), lat_bounds(nlat+1);
-          gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
-
-          std::vector<double> cell_height(nlat);
-          for (size_t j = 0; j < nlat; ++j) cell_height[j] = 0.25 * polar_circumference * (lat_bounds[j]-lat_bounds[j+1]) / 90.0;
-
-          size_t nlone = equator_circumference / dx;
-          if (nlone%2) nlone++;
-
-          std::vector<int> rowlon(nlat);
-          size_t ncells = 0;
-          for (size_t j = 0; j < nlat; ++j)
-            {
-              const double rlat = re*std::cos(DEG2RAD*lats[j]);
-              const double circumference = 2.0 * M_PI * rlat;
-              const double dx_to_use = dx * dy / cell_height[j];
-    
-              size_t nlon = std::max((int)std::lround(circumference / dx_to_use), 1);
-              if (nlon%2) nlon++;    
-
-              rowlon[j] = nlon;
-              ncells += nlon;
-            }
-
-          // printf("%zu %zu %zu %zu %g\n", ncells, nlone, nlat, nlone*nlat, 100.0*ncells/(nlone*nlat));
-
-          std::vector<double> lons(ncells);
-          size_t ij = 0;
-          for (size_t j = 0; j < nlat; ++j)
-            {
-              const size_t nlon = rowlon[j];
-              for (size_t i = 0; i < nlon; ++i) lons[ij++] = i * 360. / nlon;
-            }
-
-          grid.type = GRID_GAUSSIAN_REDUCED;
-          grid.size = ncells;
-          grid.xsize = ncells;
-          grid.ysize = nlat;
-          grid.np = nlat/2;
-          grid.xvals.resize(ncells);
-          grid.yvals.resize(nlat);
-          grid.ybounds.resize(nlat*2);
-          grid.rowlon.resize(nlat);
-          for (size_t i = 0; i < ncells; ++i) grid.xvals[i] = lons[i];
-          for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
-          for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2+1] = lat_bounds[j];
-          for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2] = lat_bounds[j+1];
-          for (size_t j = 0; j < nlat; ++j) grid.rowlon[j] = rowlon[j];
+          reducedPoints[j] = nlon;
+          ncells += nlon;
         }
+
+      // printf("%zu %zu %zu %zu %g\n", ncells, nlone, nlat, nlone*nlat, 100.0*ncells/(nlone*nlat));
+
+      std::vector<double> lons(ncells);
+      size_t ij = 0;
+      for (size_t j = 0; j < nlat; ++j)
+        {
+          const size_t nlon = reducedPoints[j];
+          for (size_t i = 0; i < nlon; ++i) lons[ij++] = i * 360. / nlon;
+        }
+
+      grid.type = GRID_GAUSSIAN_REDUCED;
+      grid.size = ncells;
+      grid.xsize = ncells;
+      grid.ysize = nlat;
+      grid.numLPE = nlat / 2;
+      grid.xvals.resize(ncells);
+      grid.yvals.resize(nlat);
+      grid.ybounds.resize(nlat * 2);
+      grid.reducedPoints.resize(nlat);
+      for (size_t i = 0; i < ncells; ++i) grid.xvals[i] = lons[i];
+      for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
+      for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2 + 1] = lat_bounds[j];
+      for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2] = lat_bounds[j + 1];
+      for (size_t j = 0; j < nlat; ++j) grid.reducedPoints[j] = reducedPoints[j];
     }
 }
 
@@ -330,7 +329,7 @@ gen_grid_reg2d(GridDesciption &grid, const char *pline)
   if (isdigit((int) *pline))
     {
       grid.type = GRID_LONLAT;
-      grid.xsize = atoi(pline);
+      grid.xsize = atol(pline);
       while (isdigit((int) *pline)) pline++;
       if (*pline == 'x' || *pline == '/' || *pline == '_')
         pline++;
@@ -339,11 +338,11 @@ gen_grid_reg2d(GridDesciption &grid, const char *pline)
           grid.type = CDI_UNDEFID;
           return;
         }
-      grid.ysize = atoi(pline);
+      grid.ysize = atol(pline);
       while (isdigit((int) *pline)) pline++;
 
-      grid.def_xfirst = true;
-      grid.def_yfirst = true;
+      grid.xfirst = 0.0;
+      grid.yfirst = 0.0;
     }
 }
 
@@ -403,13 +402,13 @@ gridFromName(const char *gridnameptr)
       pline = &gridname[off + 1];
       if (isdigit((int) *pline))
         {
-          grid.ntr = atoi(pline);
+          grid.ntr = atol(pline);
           while (isdigit((int) *pline)) pline++;
           // clang-format off
-          if      (cmpstrlen(pline, "grid", len) == 0) grid.type = GRID_GAUSSIAN;
-          else if (cmpstrlen(pline, "zon", len) == 0)  grid.type = GRID_GAUSSIAN;
-          else if (cmpstrlen(pline, "spec", len) == 0) grid.type = GRID_SPECTRAL;
-          else if (cmpstrlen(pline, "", len) == 0)     grid.type = GRID_SPECTRAL;
+          if      (cdo_cmpstrLenRhs(pline, "grid", len)) grid.type = GRID_GAUSSIAN;
+          else if (cdo_cmpstrLenRhs(pline, "zon", len))  grid.type = GRID_GAUSSIAN;
+          else if (cdo_cmpstrLenRhs(pline, "spec", len)) grid.type = GRID_SPECTRAL;
+          else if (cdo_cmpstrLenRhs(pline, "", len))     grid.type = GRID_SPECTRAL;
           // clang-format on
 
           if (pline[len] != 0) return gridID;
@@ -423,23 +422,23 @@ gridFromName(const char *gridnameptr)
               else
                 grid.ysize = ntr_to_nlat(grid.ntr);
 
-              grid.np = grid.ysize / 2;
-              grid.xsize = (cmpstrlen(pline, "zon", len) == 0)
+              grid.numLPE = grid.ysize / 2;
+              grid.xsize = (cdo_cmpstrLenRhs(pline, "zon"))
                                ? 1
                                : ((type == 'c') ? nlat_to_nlon_cubic(grid.ysize) : nlat_to_nlon(grid.ysize));
 
-              grid.def_xfirst = true;
-              grid.def_yfirst = true;
+              grid.xfirst = 0.0;
+              grid.yfirst = 0.0;
               grid.yvals.resize(grid.ysize);
-              grid.ybounds.resize(grid.ysize*2);
+              grid.ybounds.resize(grid.ysize * 2);
 
-              size_t nlat = grid.ysize;
-              std::vector<double> lats(nlat), lat_bounds(nlat+1);
+              const auto nlat = grid.ysize;
+              std::vector<double> lats(nlat), lat_bounds(nlat + 1);
               gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
 
               for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
-              for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2+1] = lat_bounds[j];
-              for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2] = lat_bounds[j+1];
+              for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2 + 1] = lat_bounds[j];
+              for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2] = lat_bounds[j + 1];
             }
         }
     }
@@ -448,7 +447,7 @@ gridFromName(const char *gridnameptr)
       pline = &gridname[1];
       gen_grid_reg2d(grid, pline);
     }
-  else if (cmpstrlen(gridname, "lon=", len) == 0)  // lon=<LON>_lat=<LAT>; one gridpoint
+  else if (cdo_cmpstrLenRhs(gridname, "lon=", len))  // lon=<LON>_lat=<LAT>; one gridpoint
     {
       pline = &gridname[len];
       gen_grid_point(grid, pline);
@@ -463,71 +462,79 @@ gridFromName(const char *gridnameptr)
       pline = &gridname[2];
       gen_grid_gme(grid, pline);
     }
-  else if (gridname[0] == 'g' && gridname[1] == 'e' && gridname[2] == 'a')  // gea<DX>: gaussian reduced equal area
+  else if (gridname[0] == 'g' && gridname[1] == 'e' && gridname[2] == 'a')
     {
+      // gea<DX>: gaussian reduced equal area; DX in km
       pline = &gridname[3];
       gen_grid_gea(grid, pline);
     }
-  else if (gridname[0] == 'n')  // N<xxx>
+  else if ((gridname[0] == 'f' || gridname[0] == 'n') && isdigit((int) gridname[1]))
     {
+      // FXXX - full (regular) Gaussian grid with XXX latitude lines between the pole and equator
       pline = &gridname[1];
-      if (isdigit((int) *pline))
+      const auto numLPE = strtol(pline, &endptr, 10);
+      pline = endptr;
+
+      if (cdo_cmpstrLenRhs(pline, "zon", len))
         {
-          const long np = strtol(pline, &endptr, 10);
-          pline = endptr;
+          grid.xsize = 1;
+          pline += 3;
+        }
+      else if (*pline == 'b')
+        {
+          grid.genBounds = true;
+          pline++;
+        }
 
-          if (cmpstrlen(pline, "zon", len) == 0)
-            {
-              grid.xsize = 1;
-              pline += 3;
-            }
-          else if (*pline == 'b')
-            {
-              grid.genBounds = true;
-              pline++;
-            }
+      if (*pline == 0)
+        {
+          grid.type = GRID_GAUSSIAN;
+          grid.numLPE = numLPE;
+          grid.ysize = numLPE * 2;
+          if (!grid.xsize) grid.xsize = nlat_to_nlon(grid.ysize);
 
-          if (*pline == 0)
-            {
-              grid.type = GRID_GAUSSIAN;
-              grid.np = np;
-              grid.ysize = np * 2;
-              if (!grid.xsize) grid.xsize = nlat_to_nlon(grid.ysize);
+          grid.xfirst = 0.0;
+          grid.yfirst = 0.0;
+          /* this will change the result of remapcon
+          grid.yvals.resize(grid.ysize);
+          grid.ybounds.resize(grid.ysize * 2);
 
-              grid.def_xfirst = true;
-              grid.def_yfirst = true;
-            }
+          size_t nlat = grid.ysize;
+          std::vector<double> lats(nlat), lat_bounds(nlat + 1);
+          gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+
+          for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
+          for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2 + 1] = lat_bounds[j];
+          for (size_t j = 0; j < nlat; ++j) grid.ybounds[j * 2] = lat_bounds[j + 1];
+          */
         }
     }
-  else if (gridname[0] == 'o')  // O<xxx>
+  else if (gridname[0] == 'o' && isdigit((int) gridname[1]))  // O<xxx>
     {
       pline = &gridname[1];
-      if (isdigit((int) *pline))
+      const auto numLPE = strtol(pline, &endptr, 10);
+      pline = endptr;
+
+      if (cdo_cmpstrLenRhs(pline, "zon", len))
         {
-          const long np = strtol(pline, &endptr, 10);
-          pline = endptr;
+          grid.xsize = 1;
+          pline += 3;
+        }
+      else if (*pline == 'b')
+        {
+          grid.genBounds = true;
+          pline++;
+        }
 
-          if (cmpstrlen(pline, "zon", len) == 0)
-            {
-              grid.xsize = 1;
-              pline += 3;
-            }
-          else if (*pline == 'b')
-            {
-              grid.genBounds = true;
-              pline++;
-            }
+      if (*pline == 0)
+        {
+          grid.type = GRID_GAUSSIAN;
+          grid.numLPE = numLPE;
+          grid.ysize = numLPE * 2;
+          if (!grid.xsize) grid.xsize = nlat_to_nlon(grid.ysize) + 16;
 
-          if (*pline == 0)
-            {
-              grid.type = GRID_GAUSSIAN;
-              grid.np = np;
-              grid.ysize = np * 2;
-              if (!grid.xsize) grid.xsize = nlat_to_nlon(grid.ysize) + 16;
-
-              grid.def_xfirst = true;
-              grid.def_yfirst = true;
-            }
+          grid.xfirst = 0.0;
+          grid.yfirst = 0.0;
         }
     }
   else if (gridname[0] == 'g' && isdigit(gridname[1]))  // g<LON>x<LAT> or g<SIZE>
@@ -536,12 +543,12 @@ gridFromName(const char *gridnameptr)
       if (isdigit((int) *pline))
         {
           grid.type = GRID_GENERIC;
-          grid.xsize = atoi(pline);
+          grid.xsize = atol(pline);
           while (isdigit((int) *pline)) pline++;
           if (*pline)
             {
               pline++;
-              grid.ysize = atoi(pline);
+              grid.ysize = atol(pline);
               while (isdigit((int) *pline)) pline++;
             }
           else if (grid.xsize == 1)
@@ -551,43 +558,27 @@ gridFromName(const char *gridnameptr)
             }
         }
     }
-  else if (cmpstrlen(gridname, "germany", len) == 0)  // germany_Xdeg
+  else if (cdo_cmpstrLenRhs(gridname, "germany", len))  // germany_Xdeg
     {
-      const double lon1 = 5.6, lon2 = 15.2;
-      const double lat1 = 47.1, lat2 = 55.1;
-      const double dll = 0.1;
-
       pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, dll, lon1, lon2, lat1, lat2);
+      gen_grid_lonlat(grid, pline, 0.1, 5.6, 15.2, 47.1, 55.1);
     }
-  else if (cmpstrlen(gridname, "europe", len) == 0)  // europe_Xdeg
+  else if (cdo_cmpstrLenRhs(gridname, "europe", len))  // europe_Xdeg
     {
-      const double lon1 = -30, lon2 = 60;
-      const double lat1 = 30, lat2 = 80;
-      const double dll = 1;
-
       pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, dll, lon1, lon2, lat1, lat2);
+      gen_grid_lonlat(grid, pline, 1, -30, 60, 30, 80);
     }
-  else if (cmpstrlen(gridname, "africa", len) == 0)  // africa_Xdeg
+  else if (cdo_cmpstrLenRhs(gridname, "africa", len))  // africa_Xdeg
     {
-      const double lon1 = -20, lon2 = 60;
-      const double lat1 = -40, lat2 = 40;
-      const double dll = 1;
-
       pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, dll, lon1, lon2, lat1, lat2);
+      gen_grid_lonlat(grid, pline, 1, -20, 60, -40, 40);
     }
-  else if (cmpstrlen(gridname, "global", len) == 0)  // global_Xdeg
+  else if (cdo_cmpstrLenRhs(gridname, "global", len))  // global_Xdeg
     {
-      const double lon1 = -180, lon2 = 180;
-      const double lat1 = -90, lat2 = 90;
-      const double dll = 1;
-
       pline = &gridname[len];
-      gen_grid_lonlat(grid, pline, dll, lon1, lon2, lat1, lat2);
+      gen_grid_lonlat(grid, pline, 1, -180, 180, -90, 90);
     }
-  else if (cmpstrlen(gridname, "ico", len) == 0)  // icoR02BXX
+  else if (cdo_cmpstrLenRhs(gridname, "ico", len))  // icoR02BXX
     {
       pline = &gridname[len];
       gen_grid_icosphere(grid, pline);
