@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,9 +16,9 @@
 */
 
 #include <vector>
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cmath>
 
 struct dimension
 {
@@ -30,7 +30,6 @@ struct dimension
 struct array
 {
   char *addr;
-  int rank;
   int offset;
   struct dimension dim[3];
 };
@@ -185,7 +184,7 @@ circum_center(struct cart *v0, struct cart *v1, struct cart *v2)
   cu.x[1] = e1.x[2] * e2.x[0] - e1.x[0] * e2.x[2];
   cu.x[2] = e1.x[0] * e2.x[1] - e1.x[1] * e2.x[0];
 
-  cnorm = sqrt(cu.x[0] * cu.x[0] + cu.x[1] * cu.x[1] + cu.x[2] * cu.x[2]);
+  cnorm = std::sqrt(cu.x[0] * cu.x[0] + cu.x[1] * cu.x[1] + cu.x[2] * cu.x[2]);
 
   ptmp1 = ((double *) center.x);
   ptmp2 = ((double *) cu.x);
@@ -206,13 +205,12 @@ circum_center(struct cart *v0, struct cart *v1, struct cart *v2)
 static struct cart
 gc2cc(struct geo *position)
 {
+  const auto sln = std::sin(position->lon);
+  const auto cln = std::cos(position->lon);
+  const auto slt = std::sin(position->lat);
+  const auto clt = std::cos(position->lat);
+
   struct cart x;
-
-  double sln = sin(position->lon);
-  double cln = cos(position->lon);
-  double slt = sin(position->lat);
-  double clt = cos(position->lat);
-
   x.x[0] = cln * clt;
   x.x[1] = sln * clt;
   x.x[2] = slt;
@@ -227,51 +225,28 @@ cc2gc(struct cart *x)
 {
   struct geo position;
 
-  double tln;
-  double tlt;
-
   if (fabs(x->x[0]) <= 0.0)
     {
-      if (x->x[1] >= 0.0)
-        {
-          position.lon = 0.5 * M_PI;
-        }
-      else
-        {
-          position.lon = 1.5 * M_PI;
-        }
+      position.lon = (x->x[1] >= 0.0) ? 0.5 * M_PI : 1.5 * M_PI;
     }
   else
     {
-      tln = x->x[1] / x->x[0];
-      position.lon = atan(tln);
-      if (x->x[0] < 0.0)
-        {
-          position.lon = position.lon + M_PI;
-        }
-      if (position.lon < 0.0)
-        {
-          position.lon = position.lon + 2 * M_PI;
-        }
+      const auto tln = x->x[1] / x->x[0];
+      position.lon = std::atan(tln);
+      if (x->x[0] < 0.0) position.lon += M_PI;
+      if (position.lon < 0.0) position.lon += 2 * M_PI;
     }
 
-  double r = sqrt(x->x[0] * x->x[0] + x->x[1] * x->x[1]);
+  const auto r = std::sqrt(x->x[0] * x->x[0] + x->x[1] * x->x[1]);
 
   if (fabs(r) <= 0.0)
     {
-      if (x->x[2] > 0.0)
-        {
-          position.lat = 0.5 * M_PI;
-        }
-      else
-        {
-          position.lat = -0.5 * M_PI;
-        }
+      position.lat = (x->x[2] > 0.0) ? 0.5 * M_PI : -0.5 * M_PI;
     }
   else
     {
-      tlt = x->x[2] / r;
-      position.lat = atan(tlt);
+      const auto tlt = x->x[2] / r;
+      position.lat = std::atan(tlt);
     }
 
   return position;
@@ -296,7 +271,6 @@ boundary(struct polygon *poly, int kip1s, int kip1e, int kip2s, int kip2e, int k
 
   int tmp1, tmp2, tmp3, tmp4, tmp5;
 
-  polyinfo.rank = 3;
   polyinfo.offset = 0;
   polyinfo.dim[0].lower = kip1s;
   tmp1 = kip1e - polyinfo.dim[0].lower + 1;
@@ -371,7 +345,6 @@ neighbours(double *px1, double *px2, int kipx1s, int kipx1e, int kipx2s, int kip
   int tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, tmp9;
   int tmp10, tmp11, tmp12, tmp13, tmp14, tmp15;
 
-  px1info.rank = 3;
   px1info.offset = 0;
   px1info.dim[0].lower = kipx1s;
   tmp1 = kipx1e - px1info.dim[0].lower + 1;
@@ -397,7 +370,6 @@ neighbours(double *px1, double *px2, int kipx1s, int kipx1e, int kipx2s, int kip
   tmp5 = px1info.dim[2].mult;
   tmp3 = px1info.offset;
 
-  px2info.rank = 3;
   px2info.offset = 0;
   px2info.dim[0].lower = kipx1s;
   tmp6 = kipx1e - px2info.dim[0].lower + 1;
@@ -423,7 +395,6 @@ neighbours(double *px1, double *px2, int kipx1s, int kipx1e, int kipx2s, int kip
   tmp10 = px2info.dim[2].mult;
   tmp8 = px2info.offset;
 
-  polyinfo.rank = 3;
   polyinfo.offset = 0;
   polyinfo.dim[0].lower = kip1s;
   tmp11 = kip1e - polyinfo.dim[0].lower + 1;
@@ -565,7 +536,6 @@ xd(double *p, int kip1s, int kip1e, int kip2s, int kip2e, int knd, double *px, i
 
   int tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, tmp9, tmp10;
 
-  pinfo.rank = 3;
   pinfo.offset = 0;
   pinfo.dim[0].lower = kip1s;
   tmp1 = kip1e - pinfo.dim[0].lower + 1;
@@ -591,7 +561,6 @@ xd(double *p, int kip1s, int kip1e, int kip2s, int kip2e, int knd, double *px, i
   tmp5 = pinfo.dim[2].mult;
   tmp3 = pinfo.offset;
 
-  pxinfo.rank = 3;
   pxinfo.offset = 0;
   pxinfo.dim[0].lower = kipx1s;
   tmp6 = kipx1e - pxinfo.dim[0].lower + 1;
@@ -701,7 +670,7 @@ tricntr(double *pxn, int kig1s, int kig1e, int kig2s, int kig2e, int knd, int kj
       r1 = pxn[mi1 + id1 * (mi1 + 1) + id2 * 1 + id3 * kjd + ioffset];
       r2 = pxn[mi1 + id1 * (mi1 + 1) + id2 * 2 + id3 * kjd + ioffset];
       r3 = pxn[mi1 + id1 * (mi1 + 1) + id2 * 3 + id3 * kjd + ioffset];
-      zxnorm = 1.0 / sqrt(r1 * r1 + r2 * r2 + r3 * r3);
+      zxnorm = 1.0 / std::sqrt(r1 * r1 + r2 * r2 + r3 * r3);
 
       pxn[mi1 + id1 * (mi1 + 1) + id2 * 1 + id3 * kjd + ioffset]
           = zxnorm * pxn[mi1 + id1 * (mi1 + 1) + id2 * 1 + id3 * kjd + ioffset];
@@ -733,15 +702,15 @@ gcpt(double *pxn, int kig1s, int kig1e, int kig2s, int kig2e, int knd, int kjd,
   double r2 = (pxn[ki2 + id1 * kj2 + id2 * 2 + id3 * kjd + ioffset] - pxn[ki1 + id1 * kj1 + id2 * 2 + id3 * kjd + ioffset]);
   double r3 = (pxn[ki2 + id1 * kj2 + id2 * 3 + id3 * kjd + ioffset] - pxn[ki1 + id1 * kj1 + id2 * 3 + id3 * kjd + ioffset]);
 
-  double zchord = sqrt((r1 * r1) + (r2 * r2) + (r3 * r3));
+  double zchord = std::sqrt((r1 * r1) + (r2 * r2) + (r3 * r3));
 
   /* Calculate "ztheta", the great circle angle between x1 and x2 */
-  double ztheta = 2.0 * asin(zchord * 0.5);
+  double ztheta = 2.0 * std::asin(zchord * 0.5);
 
   /* Calculate the weighting factors which follow from the condition */
   /* that x is a point on the unit-sphere, too. */
-  double zbeta = sin(pgamma * ztheta) / sin(ztheta);
-  double zalpha = sin((1.0 - pgamma) * ztheta) / sin(ztheta);
+  double zbeta = std::sin(pgamma * ztheta) / std::sin(ztheta);
+  double zalpha = std::sin((1.0 - pgamma) * ztheta) / std::sin(ztheta);
 
   /* Store the (x,y,z) coordinates of the point x into the array pxn */
   pxn[ki + id1 * kj + id2 * 1 + id3 * kjd + ioffset] = zalpha * pxn[ki1 + id1 * kj1 + id2 * 1 + id3 * kjd + ioffset]
@@ -782,9 +751,9 @@ glo_coor(double *pxn, double *prlon, double *prlat, int kig1s, int kig1e, int ki
 
   /* Compute angles associated with the icosahedron. */
 
-  double zw = acos(1.0 / (sin(pid5) * 2.0)) * 2.0;
-  double zcosw = cos(zw);
-  double zsinw = sin(zw);
+  double zw = std::acos(1.0 / (std::sin(pid5) * 2.0)) * 2.0;
+  double zcosw = std::cos(zw);
+  double zsinw = std::sin(zw);
   int mni = pow_ii(2, kni2) * pow_ii(3, kni3);
 
   /*     Compute the local array mcosv, i.e. the meridian angle locations */
@@ -833,21 +802,21 @@ glo_coor(double *pxn, double *prlon, double *prlat, int kig1s, int kig1e, int ki
       /*     Now initialize the (x,y,z) coordinates of the "home" vertex, */
       /*     which defines which diamond we are talking about, and label  */
       /*     this point (mni,1,,). */
-      pxn[mni + id1 * 1 + id2 * 1 + id3 * jd + ioffset] = zsinw * cos(zrlon);
-      pxn[mni + id1 * 1 + id2 * 2 + id3 * jd + ioffset] = zsinw * sin(zrlon);
+      pxn[mni + id1 * 1 + id2 * 1 + id3 * jd + ioffset] = zsinw * std::cos(zrlon);
+      pxn[mni + id1 * 1 + id2 * 2 + id3 * jd + ioffset] = zsinw * std::sin(zrlon);
       pxn[mni + id1 * 1 + id2 * 3 + id3 * jd + ioffset] = zcosw * zsgn;
 
       /*     Next initialize the (x,y,z) coordinates for the corner of the */
       /*     diamond on the same latitude as the (mni,1,,) vertex, which   */
       /*     is (0,mni+1,,) */
-      pxn[0 + id1 * (mni + 1) + id2 * 1 + id3 * jd + ioffset] = zsinw * cos(zrlon + 2 * pid5);
-      pxn[0 + id1 * (mni + 1) + id2 * 2 + id3 * jd + ioffset] = zsinw * sin(zrlon + 2 * pid5);
+      pxn[0 + id1 * (mni + 1) + id2 * 1 + id3 * jd + ioffset] = zsinw * std::cos(zrlon + 2 * pid5);
+      pxn[0 + id1 * (mni + 1) + id2 * 2 + id3 * jd + ioffset] = zsinw * std::sin(zrlon + 2 * pid5);
       pxn[0 + id1 * (mni + 1) + id2 * 3 + id3 * jd + ioffset] = zcosw * zsgn;
 
       /*     Initialize the last diamond vertex, which is located */
       /*     in the opposite hemisphere as (mni,mni+1,,)          */
-      pxn[mni + id1 * (mni + 1) + id2 * 1 + id3 * jd + ioffset] = zsinw * cos(zrlon + pid5);
-      pxn[mni + id1 * (mni + 1) + id2 * 2 + id3 * jd + ioffset] = zsinw * sin(zrlon + pid5);
+      pxn[mni + id1 * (mni + 1) + id2 * 1 + id3 * jd + ioffset] = zsinw * std::cos(zrlon + pid5);
+      pxn[mni + id1 * (mni + 1) + id2 * 2 + id3 * jd + ioffset] = zsinw * std::sin(zrlon + pid5);
       pxn[mni + id1 * (mni + 1) + id2 * 3 + id3 * jd + ioffset] = -(zcosw * zsgn);
 
       /***********************************************************************/
@@ -976,7 +945,7 @@ glo_coor(double *pxn, double *prlon, double *prlat, int kig1s, int kig1e, int ki
             {
               prlon[j1 + id1 * j2 + id2 * jd + joffset] = atan2(pxn[j1 + id1 * j2 + id2 * 2 + id3 * jd + ioffset],
                                                                 pxn[j1 + id1 * j2 + id2 * 1 + id3 * jd + ioffset] + 1.0e-20);
-              prlat[j1 + id1 * j2 + id2 * jd + joffset] = asin(pxn[j1 + id1 * j2 + id2 * 3 + id3 * jd + ioffset]);
+              prlat[j1 + id1 * j2 + id2 * jd + joffset] = std::asin(pxn[j1 + id1 * j2 + id2 * 3 + id3 * jd + ioffset]);
             }
         }
     }
@@ -997,7 +966,6 @@ initmask(int *mask, int ni, int nd)
   struct array section;
   struct array maskinfo;
 
-  maskinfo.rank = 3;
   maskinfo.offset = 0;
   maskinfo.dim[0].lower = 0;
   tmp1 = ni + 1;
@@ -1230,7 +1198,6 @@ gme_grid_restore(double *p, int ni, int nd)
 
   struct array pinfo;
 
-  pinfo.rank = 3;
   pinfo.offset = 0;
   pinfo.dim[0].lower = 0;
   int tmp1 = ni + 1;
