@@ -3,11 +3,14 @@
 
 #include "vector3d.h"
 
-#include <stdio.h>
+#include <utility>
+#include <limits>
+#include <cstdio>
 #include <iostream>
 #include <vector>
 #include <array>
 #include <map>
+#include <unordered_map>
 
 using Index = size_t;
 using Vertex = Vector3d;
@@ -15,9 +18,12 @@ using Triangle = std::array<Index, 3>;
 using TriangleList = std::vector<Triangle>;
 using VertexList = std::vector<Vertex>;
 
+using Lookup = std::map<std::pair<Index, Index>, Index>;
+
 // icosahedron from ICON
 namespace icosahedron
 {
+
 // Northern hemisphere are the first 6 elements of vertices[0:5]
 // Southern hemisphere are the other 6 elements of vertices[6:11]
 // 12 vertices
@@ -28,7 +34,7 @@ init(void)
 {
   constexpr double pi_5 = M_PI * 0.2;
   // first define the vertices of the icosahedron
-  const double z_w = 2.0 * acos(1.0 / (2.0 * sin(pi_5)));
+  const double z_w = 2.0 * std::acos(1.0 / (2.0 * std::sin(pi_5)));
 
   // set poles first - it is simple
   vertices[0] = Vertex{ 0.0, 0.0, 1.0 };
@@ -50,7 +56,7 @@ init(void)
       // compute the meridian angle for the base vertex.
       const double z_rlon = (1.0 + i_mdist[j - 1]) * pi_5;
       // now initialize the coordinates
-      vertices[j] = Vertex{ sin(z_w) * cos(z_rlon), sin(z_w) * sin(z_rlon), cos(z_w) * i_msgn };
+      vertices[j] = Vertex{ std::sin(z_w) * std::cos(z_rlon), std::sin(z_w) * std::sin(z_rlon), std::cos(z_w) * i_msgn };
     }
 }
 
@@ -60,9 +66,8 @@ static const TriangleList triangles
         { { 6, 2, 1 } },  { { 7, 3, 2 } },  { { 8, 4, 3 } },  { { 9, 5, 4 } },   { { 10, 1, 5 } },
         { { 2, 6, 7 } },  { { 3, 7, 8 } },  { { 4, 8, 9 } },  { { 5, 9, 10 } },  { { 1, 10, 6 } },
         { { 11, 7, 6 } }, { { 11, 8, 7 } }, { { 11, 9, 8 } }, { { 11, 10, 9 } }, { { 11, 6, 10 } } };
-}  // namespace icosahedron
 
-using Lookup = std::map<std::pair<Index, Index>, Index>;
+}  // namespace icosahedron
 
 static Index
 vertexForEdge(Lookup &lookup, VertexList &vertices, Index first, Index second)
@@ -82,13 +87,16 @@ subdivide(VertexList &vertices, TriangleList triangles)
   Lookup lookup;
   Triangle mid;
 
-  const size_t n = triangles.size();
+  const auto n = triangles.size();
   TriangleList result(4 * n);
   for (size_t i = 0; i < n; ++i)
     {
       const auto &each = triangles[i];
 
-      for (int edge = 0; edge < 3; ++edge) mid[edge] = vertexForEdge(lookup, vertices, each[edge], each[(edge + 1) % 3]);
+      for (int edge = 0; edge < 3; edge++)
+        {
+          mid[edge] = vertexForEdge(lookup, vertices, each[edge], each[(edge + 1) % 3]);
+        }
 
       result[i * 4 + 0] = { { each[0], mid[0], mid[2] } };
       result[i * 4 + 1] = { { each[1], mid[1], mid[0] } };
@@ -105,8 +113,8 @@ static IndexedMesh
 makeIcosphere(int subdivisions)
 {
   icosahedron::init();
-  VertexList vertices = icosahedron::vertices;
-  TriangleList triangles = icosahedron::triangles;
+  auto vertices = icosahedron::vertices;
+  auto triangles = icosahedron::triangles;
 
   for (int i = 0; i < subdivisions; ++i) triangles = subdivide(vertices, triangles);
 
@@ -116,7 +124,7 @@ makeIcosphere(int subdivisions)
 static inline void
 d_normalize(Vertex &v)
 {
-  const double dnorm = sqrt(v * v);
+  const auto dnorm = std::sqrt(v * v);
   v /= dnorm;
 }
 
@@ -158,11 +166,11 @@ size_t
 genIcosphereCoords(int subdivisions, bool lbounds, std::vector<double> &xvals, std::vector<double> &yvals,
                    std::vector<double> &xbounds, std::vector<double> &ybounds)
 {
-  IndexedMesh mesh = makeIcosphere(subdivisions);
-  VertexList &vertices = mesh.first;
-  TriangleList &triangles = mesh.second;
+  auto mesh = makeIcosphere(subdivisions);
+  auto &vertices = mesh.first;
+  auto &triangles = mesh.second;
 
-  const size_t ncells = triangles.size();
+  const auto ncells = triangles.size();
   xvals.resize(ncells);
   yvals.resize(ncells);
   if (lbounds)
@@ -174,7 +182,7 @@ genIcosphereCoords(int subdivisions, bool lbounds, std::vector<double> &xvals, s
   size_t i = 0;
   for (const Triangle &t : triangles)
     {
-      Vertex center = circumCenterMean(vertices[t[0]], vertices[t[1]], vertices[t[2]]);
+      const auto center = circumCenterMean(vertices[t[0]], vertices[t[1]], vertices[t[2]]);
       xvals[i] = center.longitude();
       yvals[i] = center.latitude();
       if (lbounds)
@@ -191,25 +199,55 @@ genIcosphereCoords(int subdivisions, bool lbounds, std::vector<double> &xvals, s
 }
 
 #ifdef TEST_ICO
+#include <chrono>
 static inline double
 rad2deg(double v)
 {
   return v * 180. / M_PI;
 }
 
+void
+subDivdeAndVertexForEdgeTest(int subdivisions)
+{
+  icosahedron::init();
+  auto vertices = icosahedron::vertices;
+  auto triangles = icosahedron::triangles;
+
+  auto start = std::chrono::steady_clock::now();
+  for (int i = 0; i < subdivisions; ++i) triangles = subdivide(vertices, triangles);
+  auto end = std::chrono::steady_clock::now();
+  auto elapsedM = std::chrono::duration_cast<std::chrono::nanoseconds>(end - start).count();
+
+  size_t expectedNumTriangles = std::pow(4, subdivisions) * 20;
+  size_t expectedNumVerticies = std::pow(4, subdivisions) * 10 + 2;
+  if (triangles.size() != (size_t) expectedNumTriangles)
+    {
+      std::cout << "ERROR: unexpected number of triangles for T: std::map, expected " << expectedNumTriangles << " got "
+                << triangles.size() << std::endl;
+    }
+  if (vertices.size() != expectedNumVerticies)
+    {
+      std::cout << "ERROR: unexpected number of verticies for T: std::map, expected " << expectedNumVerticies << " got "
+                << vertices.size() << std::endl;
+    }
+}
+
 int
 main(void)
 {
-  IndexedMesh mesh = makeIcosphere(0);
-  VertexList &vertices = mesh.first;
-  TriangleList &triangles = mesh.second;
+  int numSubdivisions = 0;
+  subDivdeAndVertexForEdgeTest(numSubdivisions);
+
+  auto mesh = makeIcosphere(numSubdivisions);
+  auto &vertices = mesh.first;
+  auto &triangles = mesh.second;
 
   if (1)
     {
       for (Vertex &v : vertices)
         {
-          double lon = v.longitude();
-          double lat = v.latitude();
+          auto lon = v.longitude();
+          auto lat = v.latitude();
           fprintf(stderr, "xyz:%g %g %g   lon:%g lat:%g\n", v[0], v[1], v[2], rad2deg(lon), rad2deg(lat));
         }
       for (Triangle &t : triangles)
@@ -218,9 +256,9 @@ main(void)
         }
       for (Triangle &t : triangles)
         {
-          Vertex center = circumCenterMean(vertices[t[0]], vertices[t[1]], vertices[t[2]]);
-          double lon = center.longitude();
-          double lat = center.latitude();
+          auto center = circumCenterMean(vertices[t[0]], vertices[t[1]], vertices[t[2]]);
+          auto lon = center.longitude();
+          auto lat = center.latitude();
           fprintf(stderr, "center: %g %g\n", rad2deg(lon), rad2deg(lat));
         }
       for (Triangle &t : triangles)
