@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,14 +16,17 @@
 */
 #include <cdi.h>
 #include "cdi_uuid.h"
-#include "cdo_int.h"
+
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include "util_string.h"
 
 static void
 printDblsPrefixAutoBrk(FILE *fp, int dig, const char prefix[], int nbyte0, size_t n, const double vals[])
 {
   fputs(prefix, fp);
-  int nbyte = nbyte0;
+  auto nbyte = nbyte0;
   for (size_t i = 0; i < n; i++)
     {
       if (nbyte > 80)
@@ -40,7 +43,7 @@ static void
 printIntsPrefixAutoBrk(FILE *fp, const char prefix[], int nbyte0, size_t n, const int vals[])
 {
   fputs(prefix, fp);
-  int nbyte = nbyte0;
+  auto nbyte = nbyte0;
   for (size_t i = 0; i < n; i++)
     {
       if (nbyte > 80)
@@ -69,7 +72,7 @@ static void
 printMask(FILE *fp, const char prefix[], size_t nbyte0, size_t n, const int mask[])
 {
   fputs(prefix, fp);
-  size_t nbyte = nbyte0;
+  auto nbyte = nbyte0;
   for (size_t i = 0; i < n; i++)
     {
       if (nbyte > 80)
@@ -110,15 +113,14 @@ grid_print_attributes(FILE *fp, int gridID)
   for (int iatt = 0; iatt < natts; ++iatt)
     {
       cdiInqAtt(cdiID, varID, iatt, attname, &atttype, &attlen);
-
       if (attlen == 0) continue;
 
-      if (cstrIsEqual(attname, "grid_mapping_name")) continue;
+      if (cdo_cmpstr(attname, "grid_mapping_name")) continue;
 
       if (atttype == CDI_DATATYPE_TXT)
         {
-          const size_t attSize = (size_t)(attlen + 1) * sizeof(char);
-          char *atttxt = (char *) resizeBuffer(&attBuf, &attBufSize, attSize);
+          const auto attSize = (size_t)(attlen + 1) * sizeof(char);
+          auto atttxt = (char *) resizeBuffer(&attBuf, &attBufSize, attSize);
           cdiInqAttTxt(cdiID, varID, attname, attlen, atttxt);
           atttxt[attlen] = 0;
           if (strchr(atttxt, '"'))
@@ -129,8 +131,8 @@ grid_print_attributes(FILE *fp, int gridID)
       else if (atttype == CDI_DATATYPE_INT8 || atttype == CDI_DATATYPE_UINT8 || atttype == CDI_DATATYPE_INT16
                || atttype == CDI_DATATYPE_UINT16 || atttype == CDI_DATATYPE_INT32 || atttype == CDI_DATATYPE_UINT32)
         {
-          const size_t attSize = (size_t) attlen * sizeof(int);
-          int *attint = (int *) resizeBuffer(&attBuf, &attBufSize, attSize);
+          const auto attSize = (size_t) attlen * sizeof(int);
+          auto attint = (int *) resizeBuffer(&attBuf, &attBufSize, attSize);
           cdiInqAttInt(cdiID, varID, attname, attlen, &attint[0]);
           fprintf(fp, "%s =", attname);
           for (int i = 0; i < attlen; ++i) fprintf(fp, " %d", attint[i]);
@@ -138,8 +140,8 @@ grid_print_attributes(FILE *fp, int gridID)
         }
       else if (atttype == CDI_DATATYPE_FLT32 || atttype == CDI_DATATYPE_FLT64)
         {
-          const size_t attSize = (size_t) attlen * sizeof(double);
-          double *attflt = (double *) resizeBuffer(&attBuf, &attBufSize, attSize);
+          const auto attSize = (size_t) attlen * sizeof(double);
+          auto attflt = (double *) resizeBuffer(&attBuf, &attBufSize, attSize);
           cdiInqAttFlt(cdiID, varID, attname, attlen, attflt);
           fprintf(fp, "%s =", attname);
           if (atttype == CDI_DATATYPE_FLT32)
@@ -159,21 +161,20 @@ static void
 grid_print_kernel(int gridID, int opt, FILE *fp)
 {
   size_t xdim, ydim;
-  char attstr[CDI_MAX_NAME];
-  char attstr2[CDI_MAX_NAME];
-  const size_t nxvals = gridInqXvals(gridID, nullptr);
-  const size_t nyvals = gridInqYvals(gridID, nullptr);
-  const size_t nxbounds = gridInqXbounds(gridID, nullptr);
-  const size_t nybounds = gridInqYbounds(gridID, nullptr);
+  char attstr[CDI_MAX_NAME], attstr2[CDI_MAX_NAME];
+  const auto nxvals = gridInqXvals(gridID, nullptr);
+  const auto nyvals = gridInqYvals(gridID, nullptr);
+  const auto nxbounds = gridInqXbounds(gridID, nullptr);
+  const auto nybounds = gridInqYbounds(gridID, nullptr);
 
-  const size_t gridsize = gridInqSize(gridID);
-  const size_t xsize = gridInqXsize(gridID);
-  const size_t ysize = gridInqYsize(gridID);
-  const int type = gridInqType(gridID);
-  const int nvertex = gridInqNvertex(gridID);
-  const int datatype = gridInqDatatype(gridID);
-  const int xstrlen = gridInqXIsc(gridID);
-  const int ystrlen = gridInqYIsc(gridID);
+  const auto gridsize = gridInqSize(gridID);
+  const auto xsize = gridInqXsize(gridID);
+  const auto ysize = gridInqYsize(gridID);
+  const auto type = gridInqType(gridID);
+  const auto nvertex = gridInqNvertex(gridID);
+  const auto datatype = gridInqDatatype(gridID);
+  const auto xstrlen = gridInqXIsc(gridID);
+  const auto ystrlen = gridInqYIsc(gridID);
 
   char **xcvals = nullptr;
   if (xstrlen)
@@ -225,40 +226,40 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
       if (nxvals > 0 || xcvals)
         {
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_XNAME, CDI_MAX_NAME, attstr);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, attstr, &length);
           if (attstr[0]) fprintf(fp, "xname     = %s\n", attstr);
-          attstr2[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_XDIMNAME, CDI_MAX_NAME, attstr2);
-          if (attstr2[0] && !cstrIsEqual(attstr, attstr2)) fprintf(fp, "xdimname  = %s\n", attstr2);
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_XLONGNAME, CDI_MAX_NAME, attstr);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_DIMNAME, attstr2, &length);
+          if (attstr2[0] && !cdo_cmpstr(attstr, attstr2)) fprintf(fp, "xdimname  = %s\n", attstr2);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_LONGNAME, attstr, &length);
           if (attstr[0]) fprintf(fp, "xlongname = \"%s\"\n", attstr);
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_XUNITS, CDI_MAX_NAME, attstr);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, attstr, &length);
           if (attstr[0]) fprintf(fp, "xunits    = \"%s\"\n", attstr);
         }
 
       if (nyvals > 0 || ycvals)
         {
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_YNAME, CDI_MAX_NAME, attstr);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, attstr, &length);
           if (attstr[0]) fprintf(fp, "yname     = %s\n", attstr);
-          attstr2[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_YDIMNAME, CDI_MAX_NAME, attstr2);
-          if (attstr2[0] && !cstrIsEqual(attstr, attstr2)) fprintf(fp, "ydimname  = %s\n", attstr2);
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_YLONGNAME, CDI_MAX_NAME, attstr);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_DIMNAME, attstr2, &length);
+          if (attstr2[0] && !cdo_cmpstr(attstr, attstr2)) fprintf(fp, "ydimname  = %s\n", attstr2);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_LONGNAME, attstr, &length);
           if (attstr[0]) fprintf(fp, "ylongname = \"%s\"\n", attstr);
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_YUNITS, CDI_MAX_NAME, attstr);
+          length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, attstr, &length);
           if (attstr[0]) fprintf(fp, "yunits    = \"%s\"\n", attstr);
         }
 
       if (type == GRID_UNSTRUCTURED || type == GRID_CURVILINEAR)
         {
-          attstr[0] = 0;
-          cdiGridInqKeyStr(gridID, CDI_KEY_VDIMNAME, CDI_MAX_NAME, attstr);
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_VDIMNAME, attstr, &length);
           if (attstr[0]) fprintf(fp, "vdimname  = %s\n", attstr);
         }
       if (type == GRID_UNSTRUCTURED && nvertex > 0) fprintf(fp, "nvertex   = %d\n", nvertex);
@@ -276,7 +277,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
     case GRID_TRAJECTORY:
     case GRID_CHARXY:
       {
-        if (type == GRID_GAUSSIAN || type == GRID_GAUSSIAN_REDUCED) fprintf(fp, "np        = %d\n", gridInqNP(gridID));
+        if (type == GRID_GAUSSIAN || type == GRID_GAUSSIAN_REDUCED) fprintf(fp, "numLPE    = %d\n", gridInqNP(gridID));
 
         if (type == GRID_CURVILINEAR || type == GRID_UNSTRUCTURED)
           {
@@ -296,19 +297,21 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
         if (type == GRID_UNSTRUCTURED)
           {
-            const int number = gridInqNumber(gridID);
-            const int position = gridInqPosition(gridID);
-            // const unsigned char *d;
+            int number = 0;
+            cdiInqKeyInt(gridID, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDUSED, &number);
             if (number > 0)
               {
                 fprintf(fp, "number    = %d\n", number);
+                int position = 0;
+                cdiInqKeyInt(gridID, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDINREFERENCE, &position);
                 if (position >= 0) fprintf(fp, "position  = %d\n", position);
               }
 
-            if (gridInqReference(gridID, nullptr))
+            int length = 0;
+            if (CDI_NOERR == cdiInqKeyLen(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, &length))
               {
                 char reference_link[8192];
-                gridInqReference(gridID, reference_link);
+                cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, reference_link, &length);
                 fprintf(fp, "uri       = %s\n", reference_link);
               }
           }
@@ -347,8 +350,8 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
         if (xcvals)
           {
-            attstr[0] = 0;
-            cdiGridInqKeyStr(gridID, CDI_KEY_XNAME, CDI_MAX_NAME, attstr);
+            int length = CDI_MAX_NAME;
+            cdiInqKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, attstr, &length);
             if (attstr[0])
               fprintf(fp, "x%ss  = \"%.*s\"", attstr, xstrlen, xcvals[0]);
             else
@@ -394,8 +397,8 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
         if (ycvals)
           {
-            attstr[0] = 0;
-            cdiGridInqKeyStr(gridID, CDI_KEY_YNAME, CDI_MAX_NAME, attstr);
+            int length = CDI_MAX_NAME;
+            cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, attstr, &length);
             if (attstr[0])
               fprintf(fp, "x%ss  = \"%.*s\"", attstr, ystrlen, ycvals[0]);
             else
@@ -417,26 +420,27 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
         if (type == GRID_GAUSSIAN_REDUCED)
           {
-            static const char prefix[] = "rowlon    = ";
-            std::vector<int> rowlon(ysize);
-            gridInqRowlon(gridID, rowlon.data());
-            printIntsPrefixAutoBrk(fp, prefix, (int) sizeof(prefix) - 1, (size_t)(ysize > 0 ? ysize : 0), rowlon.data());
+            std::vector<int> reducedPoints(ysize);
+            gridInqReducedPoints(gridID, reducedPoints.data());
+            static const char prefix[] = "reducedPoints = ";
+            printIntsPrefixAutoBrk(fp, prefix, (int) sizeof(prefix) - 1, (size_t)(ysize > 0 ? ysize : 0), reducedPoints.data());
           }
 
 #ifdef HIRLAM_EXTENSIONS
         {
-          int scanningMode = gridInqScanningMode(gridID);
+          int scanningMode = 0;
+          cdiInqKeyInt(gridID, CDI_GLOBAL, CDI_KEY_SCANNINGMODE, &scanningMode);
           fprintf(fp, "scanningMode = %d\n", scanningMode);
         }
 #endif  // HIRLAM_EXTENSIONS
 
         if (type == GRID_PROJECTION)
           {
-            attstr[0] = 0;
-            cdiGridInqKeyStr(gridID, CDI_KEY_MAPPING, CDI_MAX_NAME, attstr);
+            int length = CDI_MAX_NAME;
+            cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, attstr, &length);
             if (attstr[0]) fprintf(fp, "grid_mapping = %s\n", attstr);
-            attstr[0] = 0;
-            cdiGridInqKeyStr(gridID, CDI_KEY_MAPNAME, CDI_MAX_NAME, attstr);
+            length = CDI_MAX_NAME;
+            cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, attstr, &length);
             if (attstr[0]) fprintf(fp, "grid_mapping_name = %s\n", attstr);
             grid_print_attributes(fp, gridID);
           }
@@ -468,8 +472,9 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
       }
     }
 
-  unsigned char uuid[CDI_UUID_SIZE];
-  gridInqUUID(gridID, uuid);
+  unsigned char uuid[CDI_UUID_SIZE] = { 0 };
+  int length = CDI_UUID_SIZE;
+  cdiInqKeyBytes(gridID, CDI_GLOBAL, CDI_KEY_UUID, uuid, &length);
   if (!cdiUUIDIsNull(uuid))
     {
       char uuidStr[37];
@@ -485,7 +490,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
       printMask(fp, prefix, sizeof(prefix) - 1, (size_t)(gridsize > 0 ? gridsize : 0), mask.data());
     }
 
-  const int projID = gridInqProj(gridID);
+  const auto projID = gridInqProj(gridID);
   if (projID != CDI_UNDEFID && gridInqType(projID) == GRID_PROJECTION) grid_print_kernel(projID, opt, fp);
 }
 
