@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,22 +25,19 @@
 #include "parse_literals.h"
 #include "pmlist.h"
 
-#define MAX_LINE_LEN 65536
-
 void
 cdo_read_field(const char *name, char *pline, int size, double *field, int *lineno, FILE *fp, const char *dname)
 {
+  constexpr int MAX_LINE_LEN = 65536;
   char line[MAX_LINE_LEN];
-  double fval;
-  char *endptr;
   for (int i = 0; i < size; i++)
     {
-      endptr = pline;
-      fval = strtod(pline, &endptr);
+      auto endptr = pline;
+      auto fval = strtod(pline, &endptr);
       if (pline == endptr)
         {
           (*lineno)++;
-          if (!readline(fp, line, MAX_LINE_LEN)) cdoAbort("Incomplete command: >%s< (line: %d file: %s)", name, *lineno, dname);
+          if (!cdo::readline(fp, line, MAX_LINE_LEN)) cdoAbort("Incomplete command: >%s< (line: %d file: %s)", name, *lineno, dname);
           pline = line;
           fval = strtod(pline, &endptr);
         }
@@ -66,7 +63,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
 
       const auto kv = kvmap[ik].kv;
       const auto &key = kv->key;
-      size_t nvalues = kv->nvalues;
+      const size_t nvalues = kv->nvalues;
       if (nvalues == 0) continue;
       const auto &values = kv->values;
       const auto &value = kv->values[0];
@@ -95,7 +92,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
           else if (gridtype == "generic")          grid.type = GRID_GENERIC;
           else cdoAbort("Invalid gridtype: %s (grid description file: %s)", gridtype.c_str(), dname);
 
-          if (grid.type == GRID_LONLAT || grid.type == GRID_GAUSSIAN)
+          if (grid.type == GRID_LONLAT || grid.type == GRID_GAUSSIAN || grid.type == GRID_GAUSSIAN_REDUCED)
             grid.nvertex = 2;
           else if (grid.type == GRID_CURVILINEAR)
             grid.nvertex = 4;
@@ -103,110 +100,81 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
       else if (key == "datatype")
         {
           auto datatype = parameter2word(value);
-
           if      (datatype == "double") grid.datatype = CDI_DATATYPE_FLT64;
           else if (datatype == "float")  grid.datatype = CDI_DATATYPE_FLT32;
           else cdoAbort("Invalid datatype: %s (zaxis description file: %s)", datatype, dname);
         }
-      else if (key == "gridsize") grid.size = parameter2sizet(value);
-      else if (key == "xsize") grid.xsize = parameter2sizet(value);
-      else if (key == "nlon") grid.xsize = parameter2sizet(value);
-      else if (key == "ysize") grid.ysize = parameter2sizet(value);
-      else if (key == "nlat") grid.ysize = parameter2sizet(value);
+      else if (key == "gridsize")   grid.size = parameter2sizet(value);
+      else if (key == "xsize")      grid.xsize = parameter2sizet(value);
+      else if (key == "nlon")       grid.xsize = parameter2sizet(value);
+      else if (key == "ysize")      grid.ysize = parameter2sizet(value);
+      else if (key == "nlat")       grid.ysize = parameter2sizet(value);
       else if (key == "truncation") grid.ntr = parameter2int(value);
-      else if (key == "numLPE") grid.numLPE = parameter2int(value);
+      else if (key == "numLPE")     grid.numLPE = parameter2int(value);
+      else if (key == "np")         grid.numLPE = parameter2int(value); // np: obsolete
+      else if (key == "nvertex")    grid.nvertex = parameter2int(value);
       else if (key == "complexpacking") grid.lcomplex = parameter2int(value);
-      else if (key == "nvertex") grid.nvertex = parameter2int(value);
       else if (key == "ni")
         {
           grid.ni = parameter2int(value);
           grid.nd = 10;
         }
       else if (key == "position") grid.position = parameter2int(value);
-      else if (key == "number") grid.number = parameter2int(value);
+      else if (key == "number")   grid.number = parameter2int(value);
       else if (key == "scanningMode")
         {
-          int scmode = parameter2int(value);
-          if ((scmode == 0) || (scmode == 64) || (scmode == 96))
-            {
-              grid.scanningMode = scmode;  // -1: not used; allowed modes: <0,
-                                           // 64, 96>; Default is 64
-            }
+          grid.scanningMode = 64;
+          const auto scmode = parameter2int(value);
+          if (scmode == 0 || scmode == 64 || scmode == 96)
+            grid.scanningMode = scmode; // -1: not used; allowed modes: <0, 64, 96>; Default is 64
           else
-            {
-              cdoWarning("Warning: %d not in allowed modes: <0, 64, 96>; Using default: 64\n", scmode);
-              grid.scanningMode = 64;
-            }
+            cdoWarning("Warning: %d not in allowed modes: <0, 64, 96>; Using default: 64\n", scmode);
         }
-      else if (key == "xname") strcpy(grid.xname, parameter2word(value.c_str()));
-      else if (key == "yname") strcpy(grid.yname, parameter2word(value.c_str()));
-      else if (key == "xdimname") strcpy(grid.xdimname, parameter2word(value.c_str()));
-      else if (key == "ydimname") strcpy(grid.ydimname, parameter2word(value.c_str()));
-      else if (key == "vdimname") strcpy(grid.vdimname, parameter2word(value.c_str()));
+      else if (key == "xname")     strcpy(grid.xname, parameter2word(value.c_str()));
+      else if (key == "yname")     strcpy(grid.yname, parameter2word(value.c_str()));
+      else if (key == "xdimname")  strcpy(grid.xdimname, parameter2word(value.c_str()));
+      else if (key == "ydimname")  strcpy(grid.ydimname, parameter2word(value.c_str()));
+      else if (key == "vdimname")  strcpy(grid.vdimname, parameter2word(value.c_str()));
       else if (key == "xlongname") strcpy(grid.xlongname, value.c_str());
       else if (key == "ylongname") strcpy(grid.ylongname, value.c_str());
-      else if (key == "xunits") strcpy(grid.xunits, value.c_str());
-      else if (key == "yunits") strcpy(grid.yunits, value.c_str());
-      else if (key == "path") strcpy(grid.path, value.c_str());
+      else if (key == "xunits")    strcpy(grid.xunits, value.c_str());
+      else if (key == "yunits")    strcpy(grid.yunits, value.c_str());
+      else if (key == "path")      strcpy(grid.path, value.c_str());
       else if (key == "uuid")
         {
           strcpy(uuidStr, value.c_str());
           cdiStr2UUID(uuidStr, grid.uuid);
         }
-      else if (key == "xfirst")
-        {
-          grid.xfirst = parameter2double(value);
-          grid.def_xfirst = true;
-        }
-      else if (key == "yfirst")
-        {
-          grid.yfirst = parameter2double(value);
-          grid.def_yfirst = true;
-        }
-      else if (key == "xlast")
-        {
-          grid.xlast = parameter2double(value);
-          grid.def_xlast = true;
-        }
-      else if (key == "ylast")
-        {
-          grid.ylast = parameter2double(value);
-          grid.def_ylast = true;
-        }
-      else if (key == "xinc")
-        {
-          grid.xinc = parameter2double(value);
-          grid.def_xinc = true;
-        }
-      else if (key == "yinc")
-        {
-          grid.yinc = parameter2double(value);
-          grid.def_yinc = true;
-        }
-      else if (key == "a") grid.a = parameter2double(value);
+      else if (key == "xfirst") grid.xfirst = parameter2double(value);
+      else if (key == "yfirst") grid.yfirst = parameter2double(value);
+      else if (key == "xlast")  grid.xlast = parameter2double(value);
+      else if (key == "ylast")  grid.ylast = parameter2double(value);
+      else if (key == "xinc")   grid.xinc = parameter2double(value);
+      else if (key == "yinc")   grid.yinc = parameter2double(value);
+      else if (key == "a")      grid.a = parameter2double(value);
       else if (key == "xvals")
         {
-          size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
+          auto size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
           if (size == 0) cdoAbort("xsize or gridsize undefined (grid description file: %s)!", dname);
           if (size != nvalues)
-            cdoAbort("xsize=%zu and size of xvals=%zu differ (grid description file: %s)!", nvalues, size, dname);
+            cdoAbort("Number of xvals=%zu and size of xvals=%zu differ (grid description file: %s)!", nvalues, size, dname);
 
           grid.xvals.resize(size);
           for (size_t i = 0; i < size; ++i) grid.xvals[i] = parameter2double(values[i]);
         }
       else if (key == "yvals")
         {
-          size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.ysize;
+          auto size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.ysize;
           if (size == 0) cdoAbort("ysize or gridsize undefined (grid description file: %s)!", dname);
           if (size != nvalues)
-            cdoAbort("ysize=%zu and size of yvals=%zu differ (grid description file: %s)!", nvalues, size, dname);
+            cdoAbort("Number of yvals=%zu and size of yvals=%zu differ (grid description file: %s)!", nvalues, size, dname);
 
           grid.yvals.resize(size);
           for (size_t i = 0; i < size; ++i) grid.yvals[i] = parameter2double(values[i]);
         }
       else if (key == "xbounds")
         {
-          size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
+          auto size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
           if (size == 0) cdoAbort("xsize or gridsize undefined (grid description file: %s)!", dname);
           if (grid.nvertex == 0) cdoAbort("nvertex undefined (grid description file: %s)!", dname);
           if (grid.nvertex * size != nvalues)
@@ -218,7 +186,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
         }
       else if (key == "ybounds")
         {
-          size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.ysize;
+          auto size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.ysize;
           if (size == 0) cdoAbort("ysize or gridsize undefined (grid description file: %s)!", dname);
           if (grid.nvertex == 0) cdoAbort("nvertex undefined (grid description file: %s)!", dname);
           if (grid.nvertex * size != nvalues)
@@ -245,7 +213,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
         }
       else if (key == "mask")
         {
-          size_t size = grid.size;
+          auto size = grid.size;
           if (grid.size == 0) cdoAbort("gridsize undefined (grid description file: %s)!", dname);
           if (size != nvalues)
             cdoAbort("Number of mask values=%zu and size of grid=%zu differ (grid description file: %s)!", nvalues, size, dname);
@@ -264,7 +232,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
         }
       else if (key == "reducedPoints")
         {
-          size_t size = grid.ysize;
+          auto size = grid.ysize;
           if (size == 0) cdoAbort("ysize undefined (grid description file: %s)!", dname);
           grid.reducedPoints.resize(size);
           for (size_t i = 0; i < size; ++i) grid.reducedPoints[i] = parameter2int(values[i]);
@@ -280,7 +248,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
           break;
         }
       else
-        cdoAbort("Invalid key word >%s< (grid description file: %s)", key.c_str(), dname);
+        cdoAbort("Invalid keyword >%s< (grid description file: %s)", key.c_str(), dname);
       // clang-format on
     }
 }
@@ -288,27 +256,39 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
 static void
 grid_read_mapping(size_t igmap, size_t nkv, KVMap *kvmap, int gridID)
 {
+  bool hasGridmapVarname = false;
   for (size_t ik = igmap; ik < nkv; ++ik)
     {
       if (!kvmap[ik].isValid) continue;
 
       const auto kv = kvmap[ik].kv;
       const auto &key = kv->key;
-      size_t nvalues = kv->nvalues;
+      const size_t nvalues = kv->nvalues;
       if (nvalues == 0) continue;
       const auto &values = kv->values;
       const auto &value = kv->values[0];
 
       if (key == "grid_mapping")
         {
-          cdiGridDefKeyStr(gridID, CDI_KEY_MAPPING, (int) value.size() + 1, value.c_str());
+          hasGridmapVarname = true;
+          cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, value.c_str());
           continue;
         }
 
-      if (key == "grid_mapping_name") cdiGridDefKeyStr(gridID, CDI_KEY_MAPNAME, (int) value.size() + 1, value.c_str());
+      if (key == "grid_mapping_name")
+        {
+          if (!hasGridmapVarname)
+            {
+              if (value == "rotated_latitude_longitude")
+                cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, "rotated_pole");
+              else
+                cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, "crs");
+            }
 
-      int dtype = literalsFindDatatype(nvalues, values);
+          cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, value.c_str());
+        }
 
+      const auto dtype = literalsFindDatatype(nvalues, values);
       if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
         {
           std::vector<int> ivals(nvalues);
@@ -323,7 +303,7 @@ grid_read_mapping(size_t igmap, size_t nkv, KVMap *kvmap, int gridID)
         }
       else
         {
-          const int len = (int)value.size();
+          const auto len = (int) value.size();
           cdiDefAttTxt(gridID, CDI_GLOBAL, key.c_str(), len, value.c_str());
         }
     }
@@ -337,7 +317,7 @@ grid_read(FILE *gfp, const char *dname)
   if (pmlist.size() == 0) return -1;
   KVList &kvlist = pmlist.front();
 
-  size_t nkv = kvlist.size();
+  const size_t nkv = kvlist.size();
   if (nkv == 0) return -1;
 
   std::vector<KVMap> kvmap(nkv);
@@ -348,11 +328,11 @@ grid_read(FILE *gfp, const char *dname)
   for (auto &kv : kvlist)
     {
       if (ik == 0 && kv.key != firstKey)
-        cdoAbort("First grid description key word must be >%s< (found: %s)!", firstKey, kv.key.c_str());
+        cdoAbort("First grid description keyword must be >%s< (found: %s)!", firstKey, kv.key.c_str());
 
       if (kv.nvalues == 0)
         {
-          cdoWarning("Grid description key word %s has no values, skipped!", kv.key.c_str());
+          cdoWarning("Grid description keyword %s has no values, skipped!", kv.key.c_str());
         }
       else
         {
@@ -367,18 +347,18 @@ grid_read(FILE *gfp, const char *dname)
   GridDesciption grid;
   grid_read_data(0, nkv, kvmap.data(), grid, &iproj, &igmap, dname);
 
-  int gridID = (grid.type == CDI_UNDEFID) ? CDI_UNDEFID : gridDefine(grid);
+  const auto gridID = (grid.type == CDI_UNDEFID) ? CDI_UNDEFID : gridDefine(grid);
 
   if (gridID != CDI_UNDEFID)
     {
-      int gridprojID = gridID;
+      auto gridprojID = gridID;
 
       if (iproj > 0)
         {
           GridDesciption proj;
           grid_read_data(iproj, nkv, kvmap.data(), proj, &iproj, &igmap, dname);
 
-          int projID = (proj.type == CDI_UNDEFID) ? CDI_UNDEFID : gridDefine(proj);
+          const auto projID = (proj.type == CDI_UNDEFID) ? CDI_UNDEFID : gridDefine(proj);
           if (projID != CDI_UNDEFID)
             {
               gridDefProj(gridID, projID);
