@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
   GNU General Public License for more details.
 */
 
-#include <ctype.h>
+#include "grid_read_pingo.h"
 
 #include <cdi.h>
 
@@ -23,7 +23,7 @@
 #include <mpim_grid.h>
 #include "cdo_output.h"
 #include "compare.h"
-
+#include "gaussian_latitudes.h"
 
 static void
 skip_nondigit_lines(FILE *gfp)
@@ -48,20 +48,19 @@ skip_nondigit_lines(FILE *gfp)
 }
 
 static int
-input_ival(FILE *gfp, int *ival)
+input_ival(FILE *gfp, int &ival)
 {
   skip_nondigit_lines(gfp);
 
   if (feof(gfp)) return 0;
 
-  *ival = 0;
-  int read_items = fscanf(gfp, "%d", ival);
+  ival = 0;
 
-  return read_items;
+  return fscanf(gfp, "%d", &ival);
 }
 
 size_t
-input_darray(FILE *gfp, size_t n_values, double *array)
+input_darray(FILE *gfp, size_t n_values, std::vector<double> &array)
 {
   if (n_values <= 0) return 0;
 
@@ -84,27 +83,28 @@ int
 grid_read_pingo(FILE *gfp)
 {
   int gridID = -1;
-  int i;
+
+  int nlon, nlat;
+  if (!input_ival(gfp, nlon)) return gridID;
+  if (!input_ival(gfp, nlat)) return gridID;
 
   GridDesciption grid;
 
-  int nlon, nlat;
-  if (!input_ival(gfp, &nlon)) return gridID;
-  if (!input_ival(gfp, &nlat)) return gridID;
-
-  if (nlon > 0 && nlon < 9999 && nlat > 0 && nlat < 9999)
+  if (nlon > 0 && nlon < 99999 && nlat > 0 && nlat < 99999)
     {
+      int i;
+
       grid.xsize = nlon;
       grid.ysize = nlat;
 
       grid.xvals.resize(grid.xsize);
       grid.yvals.resize(grid.ysize);
 
-      if (!input_ival(gfp, &nlon)) return gridID;
+      if (!input_ival(gfp, nlon)) return gridID;
       if (nlon == 2)
         {
-          if (input_darray(gfp, 2, grid.xvals.data()) != 2) return gridID;
-          grid.xvals[1] -= 360 * floor((grid.xvals[1] - grid.xvals[0]) / 360);
+          if (input_darray(gfp, 2, grid.xvals) != 2) return gridID;
+          grid.xvals[1] -= 360 * std::floor((grid.xvals[1] - grid.xvals[0]) / 360);
 
           if (grid.xsize > 1)
             if (IS_EQUAL(grid.xvals[0], grid.xvals[1])) grid.xvals[1] += 360;
@@ -113,7 +113,7 @@ grid_read_pingo(FILE *gfp)
         }
       else if (nlon == (int) grid.xsize)
         {
-          if (input_darray(gfp, nlon, grid.xvals.data()) != (size_t) nlon) return gridID;
+          if (input_darray(gfp, nlon, grid.xvals) != (size_t) nlon) return gridID;
           for (i = 0; i < nlon - 1; i++)
             if (grid.xvals[i + 1] <= grid.xvals[i]) break;
 
@@ -130,15 +130,15 @@ grid_read_pingo(FILE *gfp)
       else
         return gridID;
 
-      if (!input_ival(gfp, &nlat)) return gridID;
+      if (!input_ival(gfp, nlat)) return gridID;
       if (nlat == 2)
         {
-          if (input_darray(gfp, 2, grid.yvals.data()) != 2) return gridID;
+          if (input_darray(gfp, 2, grid.yvals) != 2) return gridID;
           for (i = 0; i < (int) grid.ysize; i++) grid.yvals[i] = grid.yvals[0] + i * (grid.yvals[1] - grid.yvals[0]);
         }
       else if (nlat == (int) grid.ysize)
         {
-          if (input_darray(gfp, nlat, grid.yvals.data()) != (size_t) nlat) return gridID;
+          if (input_darray(gfp, nlat, grid.yvals) != (size_t) nlat) return gridID;
         }
       else
         return gridID;
@@ -157,22 +157,7 @@ grid_read_pingo(FILE *gfp)
             return gridID;
           }
 
-      bool lgauss = false;
-      if (nlat > 2) // check if gaussian
-        {
-          std::vector<double> yvals(grid.ysize);
-          std::vector<double> yw(grid.ysize);
-          gaussianLatitudes(yvals.data(), yw.data(), grid.ysize);
-
-          for (i = 0; i < (int) grid.ysize; i++) yvals[i] = asin(yvals[i]) * RAD2DEG;
-
-          for (i = 0; i < (int) grid.ysize; i++)
-            if (std::fabs(yvals[i] - grid.yvals[i]) > ((yvals[0] - yvals[1]) / 500)) break;
-
-          if (i == (int) grid.ysize) lgauss = true;
-        }
-
-      grid.type = lgauss ? GRID_GAUSSIAN : GRID_LONLAT;
+      grid.type = is_gaussian_latitudes(nlat, grid.yvals.data()) ? GRID_GAUSSIAN : GRID_LONLAT;
     }
 
   if (grid.type != CDI_UNDEFID) gridID = gridDefine(grid);
