@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,74 +18,25 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <limits.h>
+#include <climits>
 
 #include <cdi.h>
+
 #include "cdo_options.h"
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_cdi_wrapper.h"
 #include "griddes.h"
 #include "util_wildcards.h"
 #include "grid_read_pingo.h"
+#include "cdi_lockedIO.h"
 
 
-void
-GridDesciption::init()
-{
-  this->type = CDI_UNDEFID;
-  this->datatype = CDI_UNDEFID;
-  this->size = 0;
-  this->xsize = 0;
-  this->ysize = 0;
-  this->np = 0;
-  this->lcomplex = 1;
-  this->ntr = 0;
-  this->nvertex = 0;
-  this->genBounds = false;
-  this->def_xfirst = false;
-  this->def_yfirst = false;
-  this->def_xlast = false;
-  this->def_ylast = false;
-  this->def_xinc = false;
-  this->def_yinc = false;
-  this->xfirst = 0;
-  this->yfirst = 0;
-  this->xlast = 0;
-  this->ylast = 0;
-  this->xinc = 0;
-  this->yinc = 0;
-  this->nd = 0;
-  this->ni = 0;
-  this->ni2 = 0;
-  this->ni3 = 0;
-  this->number = 0;
-  this->position = 0;
-  this->uuid[0] = 0;
-  this->path[0] = 0;
-  this->xname[0] = 0;
-  this->xlongname[0] = 0;
-  this->xunits[0] = 0;
-  this->yname[0] = 0;
-  this->ylongname[0] = 0;
-  this->yunits[0] = 0;
-  this->xdimname[0] = 0;
-  this->ydimname[0] = 0;
-  this->vdimname[0] = 0;
-  this->scanningMode = 64;
-  /*
-    scanningMode  = 128 * iScansNegatively + 64 * jScansPositively + 32 * jPointsAreConsecutive;
-              64  = 128 * 0                + 64 *        1         + 32 * 0
-              00  = 128 * 0                + 64 *        0         + 32 * 0
-              96  = 128 * 0                + 64 *        1         + 32 * 1
-    Default  implicit scanning mode is 64: i and j scan positively, i points are consecutive (row-major)
-  */
-}
-
 int
 getoptname(char *optname, const char *optstring, int nopt)
 {
   int nerr = 0;
-  const char *pname = optstring;
-  const char *pend = optstring;
+  auto pname = optstring;
+  auto pend = optstring;
 
   for (int i = 0; i < nopt; i++)
     {
@@ -99,7 +50,7 @@ getoptname(char *optname, const char *optstring, int nopt)
   if (pend)
     {
       pend = strchr(pname, ',');
-      size_t namelen = (pend == nullptr) ? strlen(pname) : pend - pname;
+      const size_t namelen = (pend == nullptr) ? strlen(pname) : pend - pname;
       memcpy(optname, pname, namelen);
       optname[namelen] = '\0';
     }
@@ -116,8 +67,9 @@ genXboundsRegular(GridDesciption &grid)
   grid.xbounds.resize(grid.xsize * grid.nvertex);
   for (size_t i = 0; i < grid.xsize - 1; ++i)
     {
-      grid.xbounds[2 * i + 1] = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
-      grid.xbounds[2 * (i + 1)] = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
+      const auto value = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
+      grid.xbounds[2 * i + 1]   = value;
+      grid.xbounds[2 * (i + 1)] = value;
     }
   grid.xbounds[0] = 2 * grid.xvals[0] - grid.xbounds[1];
   grid.xbounds[2 * grid.xsize - 1] = 2 * grid.xvals[grid.xsize - 1] - grid.xbounds[2 * (grid.xsize - 1)];
@@ -126,23 +78,33 @@ genXboundsRegular(GridDesciption &grid)
 static void
 genYboundsRegular(GridDesciption &grid)
 {
+  const auto lrev = (grid.yvals[0] > grid.yvals[grid.ysize - 1]);
   grid.nvertex = 2;
   grid.ybounds.resize(grid.ysize * grid.nvertex);
   for (size_t i = 0; i < grid.ysize - 1; ++i)
     {
-      grid.ybounds[2 * i + 1] = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
-      grid.ybounds[2 * (i + 1)] = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
+      const auto value = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
+      if (lrev)
+        {
+          grid.ybounds[2 * i] = value;
+          grid.ybounds[2 * (i + 1) + 1] = value;
+        }
+      else
+        {
+          grid.ybounds[2 * i + 1] = value;
+          grid.ybounds[2 * (i + 1)] = value;
+        }
     }
 
-  if (grid.yvals[0] > grid.yvals[grid.ysize - 1])
+  if (lrev)
     {
-      grid.ybounds[0] = 90;
-      grid.ybounds[grid.ysize * grid.nvertex - 1] = -90;
+      grid.ybounds[1] = 90;
+      grid.ybounds[2 * grid.ysize - 2] = -90;
     }
   else
     {
       grid.ybounds[0] = -90;
-      grid.ybounds[grid.ysize * grid.nvertex - 1] = 90;
+      grid.ybounds[2 * grid.ysize - 1] = 90;
     }
 }
 
@@ -168,8 +130,8 @@ gridDefine(GridDesciption &grid)
         if (grid.size == 0) grid.size = grid.xsize * grid.ysize;
 
         if (grid.type != GRID_GAUSSIAN_REDUCED && grid.xsize && grid.size != grid.xsize * grid.ysize)
-          cdoAbort("Inconsistent grid declaration: xsize*ysize!=gridsize (xsize=%zu ysize=%zu gridsize=%zu)", grid.xsize, grid.ysize,
-                grid.size);
+          cdoAbort("Inconsistent grid declaration: xsize*ysize!=gridsize (xsize=%zu ysize=%zu gridsize=%zu)", grid.xsize,
+                   grid.ysize, grid.size);
 
         // if ( grid.size < 0 || grid.size > INT_MAX ) cdoAbort("grid size (%ld) out of bounds (0 - %d)!", grid.size, INT_MAX);
 
@@ -177,25 +139,45 @@ gridDefine(GridDesciption &grid)
 
         if (grid.xsize > 0) gridDefXsize(gridID, grid.xsize);
         if (grid.ysize > 0) gridDefYsize(gridID, grid.ysize);
-        if (grid.np > 0) gridDefNP(gridID, grid.np);
+        if (grid.numLPE > 0) gridDefNP(gridID, grid.numLPE);
 
         if (grid.nvertex) gridDefNvertex(gridID, grid.nvertex);
 
-        if ((grid.def_xfirst || grid.def_xlast || grid.def_xinc) && grid.xvals.size() == 0)
+        const auto def_xfirst = IS_NOT_EQUAL(grid.xfirst, undef_grid_value);
+        const auto def_yfirst = IS_NOT_EQUAL(grid.yfirst, undef_grid_value);
+        const auto def_xlast = IS_NOT_EQUAL(grid.xlast, undef_grid_value);
+        const auto def_ylast = IS_NOT_EQUAL(grid.ylast, undef_grid_value);
+        const auto def_xinc = IS_NOT_EQUAL(grid.xinc, undef_grid_value);
+        const auto def_yinc = IS_NOT_EQUAL(grid.yinc, undef_grid_value);
+        if ((def_xfirst || def_xlast || def_xinc) && grid.xvals.size() == 0)
           {
+            const auto xfirst = def_xfirst ? grid.xfirst : 0.0;
+            const auto xlast  = def_xlast  ? grid.xlast  : 0.0;
+            const auto xinc   = def_xinc   ? grid.xinc   : 0.0;
             grid.xvals.resize(grid.xsize);
-            gridGenXvals(grid.xsize, grid.xfirst, grid.xlast, grid.xinc, grid.xvals.data());
-
+            gridGenXvals(grid.xsize, xfirst, xlast, xinc, grid.xvals.data());
             if (grid.genBounds && grid.xbounds.size() == 0 && grid.xsize > 1) genXboundsRegular(grid);
           }
 
-        if ((grid.def_yfirst || grid.def_ylast || grid.def_yinc) && grid.yvals.size() == 0)
+        if ((def_yfirst || def_ylast || def_yinc) && grid.yvals.size() == 0)
           {
-            if (!grid.def_ylast) grid.ylast = grid.yfirst;
+            const auto yfirst = def_yfirst ? grid.yfirst : 0.0;
+            const auto ylast  = def_ylast  ? grid.ylast  : yfirst;
+            const auto yinc   = def_yinc   ? grid.yinc   : 0.0;
             grid.yvals.resize(grid.ysize);
-            gridGenYvals(grid.type, grid.ysize, grid.yfirst, grid.ylast, grid.yinc, grid.yvals.data());
-
-            if (grid.genBounds && grid.ybounds.size() == 0 && grid.ysize > 1) genYboundsRegular(grid);
+            if (grid.type == GRID_GAUSSIAN && grid.genBounds)
+              {
+                grid.ybounds.resize(grid.ysize * 2);
+                std::vector<double> lat_bounds(grid.ysize + 1);
+                gaussianLatitudesInDegrees(grid.yvals, lat_bounds, grid.ysize);
+                for (size_t i = 0; i < grid.ysize; ++i) grid.ybounds[2 * i] = lat_bounds[i + 1];
+                for (size_t i = 0; i < grid.ysize; ++i) grid.ybounds[2 * i + 1] = lat_bounds[i];
+              }
+            else
+              {
+                gridGenYvals(grid.type, grid.ysize, yfirst, ylast, yinc, grid.yvals.data());
+                if (grid.genBounds && grid.ybounds.size() == 0 && grid.ysize > 1) genYboundsRegular(grid);
+              }
           }
 
         if (grid.xvals.size()) gridDefXvals(gridID, grid.xvals.data());
@@ -203,7 +185,7 @@ gridDefine(GridDesciption &grid)
         if (grid.xbounds.size()) gridDefXbounds(gridID, grid.xbounds.data());
         if (grid.ybounds.size()) gridDefYbounds(gridID, grid.ybounds.data());
         if (grid.mask.size()) gridDefMask(gridID, grid.mask.data());
-        if (grid.rowlon.size()) gridDefRowlon(gridID, grid.ysize, grid.rowlon.data());
+        if (grid.reducedPoints.size()) gridDefReducedPoints(gridID, grid.ysize, grid.reducedPoints.data());
 
         break;
       }
@@ -226,10 +208,10 @@ gridDefine(GridDesciption &grid)
             if (grid.nvertex > 0) gridDefNvertex(gridID, grid.nvertex);
             if (grid.number > 0)
               {
-                gridDefNumber(gridID, grid.number);
-                if (grid.position >= 0) gridDefPosition(gridID, grid.position);
+                cdiDefKeyInt(gridID, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDUSED, grid.number);
+                if (grid.position >= 0) cdiDefKeyInt(gridID, CDI_GLOBAL, CDI_KEY_NUMBEROFGRIDINREFERENCE, grid.position);
               }
-            if (*grid.path) gridDefReference(gridID, grid.path);
+            if (*grid.path) cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_REFERENCEURI, grid.path);
           }
 
         if (grid.xvals.size()) gridDefXvals(gridID, grid.xvals.data());
@@ -280,17 +262,17 @@ gridDefine(GridDesciption &grid)
 
   if (grid.datatype != CDI_UNDEFID) gridDefDatatype(gridID, grid.datatype);
 
-  if (grid.uuid[0]) gridDefUUID(gridID, grid.uuid);
+  if (grid.uuid[0]) cdiDefKeyBytes(gridID, CDI_GLOBAL, CDI_KEY_UUID, grid.uuid, CDI_UUID_SIZE);
 
-  if (grid.xname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_XNAME, strlen(grid.xname) + 1, grid.xname);
-  if (grid.xlongname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_XLONGNAME, strlen(grid.xlongname) + 1, grid.xlongname);
-  if (grid.xunits[0]) cdiGridDefKeyStr(gridID, CDI_KEY_XUNITS, strlen(grid.xunits) + 1, grid.xunits);
-  if (grid.yname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_YNAME, strlen(grid.yname) + 1, grid.yname);
-  if (grid.ylongname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_YLONGNAME, strlen(grid.ylongname) + 1, grid.ylongname);
-  if (grid.yunits[0]) cdiGridDefKeyStr(gridID, CDI_KEY_YUNITS, strlen(grid.yunits) + 1, grid.yunits);
-  if (grid.xdimname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_XDIMNAME, strlen(grid.xdimname) + 1, grid.xdimname);
-  if (grid.ydimname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_YDIMNAME, strlen(grid.ydimname) + 1, grid.ydimname);
-  if (grid.vdimname[0]) cdiGridDefKeyStr(gridID, CDI_KEY_VDIMNAME, strlen(grid.vdimname) + 1, grid.vdimname);
+  if (grid.xname[0]) cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_NAME, grid.xname);
+  if (grid.xlongname[0]) cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_LONGNAME, grid.xlongname);
+  if (grid.xunits[0]) cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_UNITS, grid.xunits);
+  if (grid.yname[0]) cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_NAME, grid.yname);
+  if (grid.ylongname[0]) cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_LONGNAME, grid.ylongname);
+  if (grid.yunits[0]) cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, grid.yunits);
+  if (grid.xdimname[0]) cdiDefKeyString(gridID, CDI_XAXIS, CDI_KEY_DIMNAME, grid.xdimname);
+  if (grid.ydimname[0]) cdiDefKeyString(gridID, CDI_YAXIS, CDI_KEY_DIMNAME, grid.ydimname);
+  if (grid.vdimname[0]) cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_VDIMNAME, grid.vdimname);
 
   return gridID;
 }
@@ -301,30 +283,26 @@ gridFromMPAS(const char *filename)
   int mpasGridID = -1;
 
   openLock();
-  CdiStreamID streamID = streamOpenRead(filename);
+  const auto streamID = streamOpenRead(filename);
   openUnlock();
   if (streamID >= 0)
     {
-      int vlistID = streamInqVlist(streamID);
-      int nvars = vlistNvars(vlistID);
+      const auto vlistID = streamInqVlist(streamID);
+      const auto nvars = vlistNvars(vlistID);
       int latCellID = -1, lonCellID = -1, areaCellID = -1;
       int latVertexID = -1, lonVertexID = -1, verticesOnCellID = -1;
       char varname[CDI_MAX_NAME];
       for (int varID = 0; varID < nvars; ++varID)
         {
           vlistInqVarName(vlistID, varID, varname);
-          if (cstrIsEqual(varname, "latCell"))
-            latCellID = varID;
-          else if (cstrIsEqual(varname, "lonCell"))
-            lonCellID = varID;
-          else if (cstrIsEqual(varname, "latVertex"))
-            latVertexID = varID;
-          else if (cstrIsEqual(varname, "lonVertex"))
-            lonVertexID = varID;
-          else if (cstrIsEqual(varname, "areaCell"))
-            areaCellID = varID;
-          else if (cstrIsEqual(varname, "verticesOnCell"))
-            verticesOnCellID = varID;
+          // clang-format off
+          if      (cdo_cmpstr(varname, "latCell"))   latCellID = varID;
+          else if (cdo_cmpstr(varname, "lonCell"))   lonCellID = varID;
+          else if (cdo_cmpstr(varname, "latVertex")) latVertexID = varID;
+          else if (cdo_cmpstr(varname, "lonVertex")) lonVertexID = varID;
+          else if (cdo_cmpstr(varname, "areaCell"))  areaCellID = varID;
+          else if (cdo_cmpstr(varname, "verticesOnCell")) verticesOnCellID = varID;
+          // clang-format on
         }
 
       // printf("%d %d %d %d %d %d\n", latCellID, lonCellID, areaCellID, latVertexID, lonVertexID, verticesOnCellID);
@@ -335,17 +313,13 @@ gridFromMPAS(const char *filename)
           return -1;
         }
 
-      const size_t nv = 6;
-      const size_t ncells = gridInqSize(vlistInqVarGrid(vlistID, latCellID));
-      std::vector<double> latCell(ncells);
-      std::vector<double> lonCell(ncells);
-      std::vector<double> areaCell(ncells);
-      const size_t nvertices = gridInqSize(vlistInqVarGrid(vlistID, latVertexID));
-      std::vector<double> latVertex(nvertices);
-      std::vector<double> lonVertex(nvertices);
+      constexpr size_t nv = 6;
+      const auto ncells = gridInqSize(vlistInqVarGrid(vlistID, latCellID));
+      std::vector<double> latCell(ncells), lonCell(ncells), areaCell(ncells);
+      const auto nvertices = gridInqSize(vlistInqVarGrid(vlistID, latVertexID));
+      std::vector<double> latVertex(nvertices), lonVertex(nvertices);
       std::vector<double> verticesOnCell(nv * ncells);
-      std::vector<double> xbounds(nv * ncells);
-      std::vector<double> ybounds(nv * ncells);
+      std::vector<double> xbounds(nv * ncells), ybounds(nv * ncells);
 
       const int nrecs = streamInqTimestep(streamID, 0);
       for (int recID = 0; recID < nrecs; recID++)
@@ -353,33 +327,29 @@ gridFromMPAS(const char *filename)
           size_t nmiss;
           int varID, levelID;
           streamInqRecord(streamID, &varID, &levelID);
-          if (varID == latCellID)
-            streamReadRecord(streamID, latCell.data(), &nmiss);
-          else if (varID == lonCellID)
-            streamReadRecord(streamID, lonCell.data(), &nmiss);
-          else if (varID == latVertexID)
-            streamReadRecord(streamID, latVertex.data(), &nmiss);
-          else if (varID == lonVertexID)
-            streamReadRecord(streamID, lonVertex.data(), &nmiss);
-          else if (varID == areaCellID)
-            streamReadRecord(streamID, areaCell.data(), &nmiss);
-          else if (varID == verticesOnCellID)
-            streamReadRecord(streamID, verticesOnCell.data(), &nmiss);
+          // clang-format off
+          if      (varID == latCellID)   streamReadRecord(streamID, latCell.data(), &nmiss);
+          else if (varID == lonCellID)   streamReadRecord(streamID, lonCell.data(), &nmiss);
+          else if (varID == latVertexID) streamReadRecord(streamID, latVertex.data(), &nmiss);
+          else if (varID == lonVertexID) streamReadRecord(streamID, lonVertex.data(), &nmiss);
+          else if (varID == areaCellID)  streamReadRecord(streamID, areaCell.data(), &nmiss);
+          else if (varID == verticesOnCellID) streamReadRecord(streamID, verticesOnCell.data(), &nmiss);
+          // clang-format on
         }
       streamClose(streamID);
 
       mpasGridID = gridCreate(GRID_UNSTRUCTURED, ncells);
-      gridDefXvals(mpasGridID, lonCell.data());  // CDIcall
-      gridDefYvals(mpasGridID, latCell.data());  // CDIcall
-      gridDefXunits(mpasGridID, "radians");      // CDIcall
-      gridDefYunits(mpasGridID, "radians");      // CDIcall
-      gridDefArea(mpasGridID, areaCell.data());  // CDIcall
+      gridDefXvals(mpasGridID, lonCell.data());
+      gridDefYvals(mpasGridID, latCell.data());
+      cdiDefKeyString(mpasGridID, CDI_XAXIS, CDI_KEY_UNITS, "radians");
+      cdiDefKeyString(mpasGridID, CDI_YAXIS, CDI_KEY_UNITS, "radians");
+      gridDefArea(mpasGridID, areaCell.data());
       bool lbounds = true;
       double xc[nv], yc[nv];
       for (size_t i = 0; i < ncells; ++i)
         for (size_t k = 0; k < nv; ++k)
           {
-            long cellIndex = lround(verticesOnCell[i * nv + k]);
+            long cellIndex = std::lround(verticesOnCell[i * nv + k]);
             if (cellIndex < 0 || cellIndex > (long) nvertices)
               {
                 lbounds = false;
@@ -416,7 +386,7 @@ gridFromMPAS(const char *filename)
   return mpasGridID;
 }
 
-int
+static int
 cdoDefineGrid(const char *pgridfile)
 {
   int gridID = -1;
@@ -424,8 +394,7 @@ cdoDefineGrid(const char *pgridfile)
   bool lalloc = false;
   bool lmpas = false;
 
-  size_t len = strlen(pgridfile);
-
+  auto len = strlen(pgridfile);
   if (len > 5 && strncmp(pgridfile, "mpas:", 5) == 0)
     {
       lmpas = true;
@@ -448,7 +417,7 @@ cdoDefineGrid(const char *pgridfile)
   else
     filename = gridfile;
 
-  int fileno = open(filename, O_RDONLY);
+  auto fileno = open(filename, O_RDONLY);
   if (fileno >= 0)
     {
       struct stat filestat;
@@ -465,7 +434,7 @@ cdoDefineGrid(const char *pgridfile)
     }
   else if (lmpas)
     {
-      Debug(cdoDebug,"Grid from MPAS file");
+      Debug(cdoDebug, "Grid from MPAS file");
       gridID = gridFromMPAS(filename);
       if (gridID == -1) cdoAbort("Unsupported MPAS grid file %s!", filename);
     }
@@ -476,40 +445,40 @@ cdoDefineGrid(const char *pgridfile)
 
       close(fileno);
 
-      if (cmpstrlen(buffer, "CDF", len) == 0)
+      if (cdo_cmpstrLenRhs(buffer, "CDF"))
         {
-          Debug(cdoDebug,"Grid from NetCDF file");
+          Debug(cdoDebug, "Grid from NetCDF file");
           gridID = gridFromNCfile(filename);
         }
 
       if (gridID == -1)
         {
-          if (cmpstrlen(buffer + 1, "HDF", len) == 0)
+          if (cdo_cmpstrLenRhs(buffer + 1, "HDF"))
             {
-              Debug(cdoDebug,"Grid from HDF5 file");
+              Debug(cdoDebug, "Grid from HDF5 file");
               gridID = gridFromH5file(filename);
             }
         }
 
       if (gridID == -1)
         {
-          if (cmpstrlen(buffer + 1, "HDF", len) == 0)
+          if (cdo_cmpstrLenRhs(buffer + 1, "HDF"))
             {
-              Debug(cdoDebug,"Grid from NetCDF4 file");
+              Debug(cdoDebug, "Grid from NetCDF4 file");
               gridID = gridFromNCfile(filename);
             }
         }
 
       if (gridID == -1)
         {
-          Debug(cdoDebug,"Grid from CDI file");
+          Debug(cdoDebug, "Grid from CDI file");
           openLock();
-          int streamID = streamOpenRead(filename);
+          const auto streamID = streamOpenRead(filename);
           openUnlock();
           if (streamID >= 0)
             {
-              int vlistID = streamInqVlist(streamID);
-              int ngrids = vlistNgrids(vlistID);
+              const auto vlistID = streamInqVlist(streamID);
+              const auto ngrids = vlistNgrids(vlistID);
               if (gridno < 1 || gridno > ngrids) cdoAbort("Grid number %d not available in %s!", gridno, filename);
               gridID = vlistGrid(vlistID, gridno - 1);
               streamClose(streamID);
@@ -518,20 +487,16 @@ cdoDefineGrid(const char *pgridfile)
 
       if (gridID == -1)
         {
-          Debug(cdoDebug,"grid from ASCII file");
-          FILE *gfp = fopen(filename, "r");
-          // size_t buffersize = 20*1024*1024;
-          // char *buffer = (char*) Malloc(buffersize);
-          // setvbuf(gfp, buffer, _IOFBF, buffersize);
+          Debug(cdoDebug, "grid from ASCII file");
+          auto gfp = fopen(filename, "r");
           gridID = grid_read(gfp, filename);
           fclose(gfp);
-          // free(buffer);
         }
 
       if (gridID == -1)
         {
-          Debug(cdoDebug,"grid from PINGO file");
-          FILE *gfp = fopen(filename, "r");
+          Debug(cdoDebug, "grid from PINGO file");
+          auto gfp = fopen(filename, "r");
           gridID = grid_read_pingo(gfp);
           fclose(gfp);
         }
@@ -539,12 +504,18 @@ cdoDefineGrid(const char *pgridfile)
       if (gridID == -1) cdoAbort("Invalid grid description file %s!", filename);
     }
 
-  if (lalloc) Free(filename);
+  if (lalloc) free(filename);
   free(gridfile);
 
   return gridID;
 }
 
+int
+cdoDefineGrid(const std::string &gridfile)
+{
+  return cdoDefineGrid(gridfile.c_str());
+}
+
 void
 cdo_set_grids(const char *gridarg)
 {
