@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,8 @@
 
 #include <cdi.h>
 #include "cdo_options.h"
-#include "cdo_int.h"
+
+#include "cdo_output.h"
 #include "griddes.h"
 
 #ifdef HAVE_LIBHDF5
@@ -73,6 +74,7 @@ h5find_object(hid_t file_id, const char *name)
 }
 #endif
 
+#ifdef HAVE_LIBHDF5
 static void
 fill_gridvals(size_t xsize, size_t ysize, double *xvals, double *yvals)
 {
@@ -245,7 +247,6 @@ int
 gridFromH5file(const char *gridfile)
 {
   int gridID = -1;
-#ifdef HAVE_LIBHDF5
   hid_t lon_id = -1; /* Dataset ID	        	*/
   hid_t lat_id = -1; /* Dataset ID	        	*/
   hid_t att_id;
@@ -382,7 +383,6 @@ gridFromH5file(const char *gridfile)
       double xscale = 1, yscale = 1;
       double xoffset = 0, yoffset = 0;
       hid_t grp_id;
-      size_t i;
 
       grp_id = H5Gopen(file_id, "/where/lon/what");
       if (grp_id >= 0)
@@ -513,8 +513,8 @@ gridFromH5file(const char *gridfile)
           status = H5Dclose(lon_id);
           status = H5Dclose(lat_id);
 
-          for (i = 0; i < grid.size; ++i) grid.xvals[i] = grid.xvals[i] * xscale + xoffset;
-          for (i = 0; i < grid.size; ++i) grid.yvals[i] = grid.yvals[i] * yscale + yoffset;
+          for (size_t i = 0; i < grid.size; ++i) grid.xvals[i] = grid.xvals[i] * xscale + xoffset;
+          for (size_t i = 0; i < grid.size; ++i) grid.yvals[i] = grid.yvals[i] * yscale + yoffset;
 
           grid.type = GRID_CURVILINEAR;
           grid.datatype = CDI_DATATYPE_FLT32;
@@ -527,14 +527,18 @@ RETURN:
 
   /* Close file */
   if (file_id >= 0) status = H5Fclose(file_id);
-
   (void) status;
 
   if (gridID != -1 && Options::cdoVerbose) cdoPrint("%s: grid created.", __func__);
 
-#else
-  cdoWarning("HDF5 support not compiled in!");
-#endif
-
   return gridID;
 }
+#else
+int
+gridFromH5file(const char *gridfile)
+{
+  (void) gridfile;
+  cdoWarning("HDF5 support not compiled in!");
+  return -1;
+}
+#endif
