@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,11 @@
 #endif
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "cdo_options.h"
+#include "cdo_output.h"
 #include "griddes.h"
+#include "cdi_lockedIO.h"
 
 #ifdef HAVE_LIBNETCDF
 static void
@@ -36,16 +39,17 @@ nce(int istat)
 #endif
 
 int
-cdf_openread(const char *filename)
+cdo_cdf_openread(const char *filename)
 {
   int fileID = -1;
 #ifdef HAVE_LIBNETCDF
-  int nc_file_id; /* NetCDF grid file id           */
+  int nc_file_id;  // NetCDF grid file id
 
   openLock();
-  int istat = nc_open(filename, NC_NOWRITE, &nc_file_id);
+  const auto istat = nc_open(filename, NC_NOWRITE, &nc_file_id);
   openUnlock();
-  if (istat != NC_NOERR) cdoAbort("Open failed on %s! %s", filename, nc_strerror(istat));
+
+  if (istat != NC_NOERR) cdoAbort("nc_open failed on %s! %s", filename, nc_strerror(istat));
   fileID = nc_file_id;
 #else
   cdoWarning("NetCDF support not compiled in!");
@@ -54,20 +58,33 @@ cdf_openread(const char *filename)
   return fileID;
 }
 
+void
+cdo_cdf_close(int nc_file_id)
+{
+#ifdef HAVE_LIBNETCDF
+  openLock();
+  const auto istat = nc_close(nc_file_id);
+  openUnlock();
+  if (istat != NC_NOERR) cdoAbort("nc_close failed! %s", nc_strerror(istat));
+#else
+  cdoWarning("NetCDF support not compiled in!");
+#endif
+}
+
 int
 gridFromNCfile(const char *gridfile)
 {
   int gridID = -1;
 #ifdef HAVE_LIBNETCDF
-  int nc_gridsize_id; /* NetCDF grid size dim id       */
-  int nc_gridcorn_id; /* NetCDF grid corner dim id     */
-  int nc_gridrank_id; /* NetCDF grid rank dim id       */
-  int nc_griddims_id; /* NetCDF grid dimension size id */
-  int nc_gridclat_id; /* NetCDF grid corner lat var id */
-  int nc_gridclon_id; /* NetCDF grid corner lon var id */
-  int nc_gridlat_id;  /* NetCDF grid center lat var id */
-  int nc_gridlon_id;  /* NetCDF grid center lon var id */
-  int nc_gridmask_id; /* NetCDF grid mask id           */
+  int nc_gridsize_id; // NetCDF grid size dim id
+  int nc_gridcorn_id; // NetCDF grid corner dim id
+  int nc_gridrank_id; // NetCDF grid rank dim id
+  int nc_griddims_id; // NetCDF grid dimension size id
+  int nc_gridclat_id; // NetCDF grid corner lat var id
+  int nc_gridclon_id; // NetCDF grid corner lon var id
+  int nc_gridlat_id;  // NetCDF grid center lat var id
+  int nc_gridlon_id;  // NetCDF grid center lon var id
+  int nc_gridmask_id; // NetCDF grid mask id
 
   nc_type xtype;
   size_t attlen;
@@ -77,7 +94,7 @@ gridFromNCfile(const char *gridfile)
 
   // open grid file and read grid size/name data
 
-  int nc_file_id = cdf_openread(gridfile);
+  const auto nc_file_id = cdo_cdf_openread(gridfile);
 
   if (nc_inq_dimid(nc_file_id, "grid_size", &nc_gridsize_id) == NC_NOERR
       && nc_inq_dimid(nc_file_id, "grid_rank", &nc_gridrank_id) == NC_NOERR
@@ -89,7 +106,7 @@ gridFromNCfile(const char *gridfile)
       nce(nc_inq_dimlen(nc_file_id, nc_gridcorn_id, &grid_nvertex));
       grid.nvertex = (int) grid_nvertex;
 
-      /* check variables */
+      // check variables
       if (nc_inq_varid(nc_file_id, "grid_dims", &nc_griddims_id) != NC_NOERR
           || nc_inq_varid(nc_file_id, "grid_center_lat", &nc_gridlat_id) != NC_NOERR
           || nc_inq_varid(nc_file_id, "grid_center_lon", &nc_gridlon_id) != NC_NOERR
@@ -114,7 +131,7 @@ gridFromNCfile(const char *gridfile)
           if ((size_t) grid_dims[0] * grid_dims[1] != grid_size) return gridID;
         }
 
-      /* allocate grid coordinates and read data */
+      // allocate grid coordinates and read data
 
       grid.xvals.resize(grid.size);
       grid.yvals.resize(grid.size);
@@ -154,7 +171,7 @@ gridFromNCfile(const char *gridfile)
       gridID = gridDefine(grid);
     }
 
-  nce(nc_close(nc_file_id));
+  cdo_cdf_close(nc_file_id);
 
 #else
   cdoWarning("NetCDF support not compiled in!");
@@ -167,26 +184,25 @@ void
 writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
 {
 #ifdef HAVE_LIBNETCDF
-  int nc_file_id;     /* NetCDF grid file id           */
-  int nc_gridsize_id; /* NetCDF grid size dim id       */
-  int nc_gridcorn_id; /* NetCDF grid corner dim id     */
-  int nc_gridrank_id; /* NetCDF grid rank dim id       */
-  int nc_griddims_id; /* NetCDF grid dimension size id */
-  int nc_gridclat_id; /* NetCDF grid corner lat var id */
-  int nc_gridclon_id; /* NetCDF grid corner lon var id */
-  int nc_gridlat_id;  /* NetCDF grid center lat var id */
-  int nc_gridlon_id;  /* NetCDF grid center lon var id */
+  int nc_file_id;     // NetCDF grid file id
+  int nc_gridsize_id; // NetCDF grid size dim id
+  int nc_gridcorn_id; // NetCDF grid corner dim id
+  int nc_gridrank_id; // NetCDF grid rank dim id
+  int nc_griddims_id; // NetCDF grid dimension size id
+  int nc_gridclat_id; // NetCDF grid corner lat var id
+  int nc_gridclon_id; // NetCDF grid corner lon var id
+  int nc_gridlat_id;  // NetCDF grid center lat var id
+  int nc_gridlon_id;  // NetCDF grid center lon var id
   int nc_gridxsize_id = -1, nc_gridysize_id = -1, nc_grdimask_id;
 
   size_t grid_rank = 0, len;
   int grid_dims[2];
   int nc_dims_id[3], ndims;
-  char units[CDI_MAX_NAME];
 
   int gridtype = gridInqType(gridID);
-  size_t gridsize = gridInqSize(gridID);
+  auto gridsize = gridInqSize(gridID);
 
-  nc_type xtype = (gridInqDatatype(gridID) == CDI_DATATYPE_FLT64) ? NC_DOUBLE : NC_FLOAT;
+  const auto xtype = (gridInqDatatype(gridID) == CDI_DATATYPE_FLT64) ? NC_DOUBLE : NC_FLOAT;
 
   if (gridtype == GRID_CURVILINEAR)
     {
@@ -203,7 +219,9 @@ writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
     {
     }
 
-  gridInqYunits(gridID, units);
+  char units[CDI_MAX_NAME];
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(gridID, CDI_YAXIS, CDI_KEY_UNITS, units, &length);
   if (memcmp(units, "degrees", 7) == 0)
     units[7] = 0;
   else if (memcmp(units, "radian", 6) == 0)
@@ -211,7 +229,7 @@ writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
   else
     cdoWarning("Unknown units supplied for grid!");
 
-  /* create NetCDF dataset for this grid */
+  // create NetCDF dataset for this grid
 
   nce(nc_create(gridfile, NC_CLOBBER, &nc_file_id));
 
@@ -221,7 +239,7 @@ writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
 
   if (Options::VersionInfo) nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "CDO", (int) strlen(cdoComment()) + 1, cdoComment()));
 
-  /* define grid size dimension */
+  // define grid size dimension
 
   nce(nc_def_dim(nc_file_id, "grid_size", gridsize, &nc_gridsize_id));
   if (gridtype == GRID_CURVILINEAR)
@@ -230,19 +248,19 @@ writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
       nce(nc_def_dim(nc_file_id, "grid_ysize", gridInqYsize(gridID), &nc_gridysize_id));
     }
 
-  /* define grid corner dimension */
+  // define grid corner dimension
 
   nce(nc_def_dim(nc_file_id, "grid_corners", gridInqNvertex(gridID), &nc_gridcorn_id));
 
-  /* define grid rank dimension */
+  // define grid rank dimension
 
   nce(nc_def_dim(nc_file_id, "grid_rank", grid_rank, &nc_gridrank_id));
 
-  /* define grid dimension size array */
+  // define grid dimension size array
 
   nce(nc_def_var(nc_file_id, "grid_dims", NC_INT, 1, &nc_gridrank_id, &nc_griddims_id));
 
-  /* define grid center latitude array */
+  // define grid center latitude array
 
   if (gridtype == GRID_CURVILINEAR)
     {
@@ -261,21 +279,21 @@ writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
   nce(nc_put_att_text(nc_file_id, nc_gridlat_id, "units", strlen(units), units));
   nce(nc_put_att_text(nc_file_id, nc_gridlat_id, "bounds", 15, "grid_corner_lat"));
 
-  /* define grid center longitude array */
+  // define grid center longitude array
 
   nce(nc_def_var(nc_file_id, "grid_center_lon", xtype, ndims, nc_dims_id, &nc_gridlon_id));
 
   nce(nc_put_att_text(nc_file_id, nc_gridlon_id, "units", strlen(units), units));
   nce(nc_put_att_text(nc_file_id, nc_gridlon_id, "bounds", 15, "grid_corner_lon"));
 
-  /* define grid mask */
+  // define grid mask
 
   nce(nc_def_var(nc_file_id, "grid_imask", NC_INT, ndims, nc_dims_id, &nc_grdimask_id));
 
   nce(nc_put_att_text(nc_file_id, nc_grdimask_id, "units", 8, "unitless"));
   nce(nc_put_att_text(nc_file_id, nc_grdimask_id, "coordinates", 31, "grid_center_lon grid_center_lat"));
 
-  /* define grid corner latitude array */
+  // define grid corner latitude array
 
   if (gridtype == GRID_CURVILINEAR)
     {
@@ -295,17 +313,17 @@ writeNCgrid(const char *gridfile, int gridID, int *grid_imask)
 
   nce(nc_put_att_text(nc_file_id, nc_gridclat_id, "units", strlen(units), units));
 
-  /* define grid corner longitude array */
+  // define grid corner longitude array
 
   nce(nc_def_var(nc_file_id, "grid_corner_lon", xtype, ndims, nc_dims_id, &nc_gridclon_id));
 
   nce(nc_put_att_text(nc_file_id, nc_gridclon_id, "units", strlen(units), units));
 
-  /* end definition stage */
+  // end definition stage
 
   nce(nc_enddef(nc_file_id));
 
-  /*  write grid data */
+  //  write grid data
 
   nce(nc_put_var_int(nc_file_id, nc_griddims_id, grid_dims));
 
