@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,24 +27,37 @@
 #include "cdo_options.h"
 #include "cimdOmp.h"
 
-
-static constexpr double apr = 101325.0; // reference pressure
+static constexpr double apr = 101325.0;  // reference pressure
 static constexpr double aipr = 1.0 / 101325.0;
 
-static constexpr double p_firef = 40000.0; // pressure of reference geopotential
+static constexpr double p_firef = 40000.0;  // pressure of reference geopotential
 
 static constexpr double epsilon = 0.622;
 static constexpr double rair = 287.04;
 static constexpr double cpair = 1004.6;
 
-static constexpr double eta_pbl = 0.8; // upper limit of BPL eta coordiante
+static constexpr double eta_pbl = 0.8;  // upper limit of BPL eta coordiante
 
-#if defined(OUTPUT)
-static FILE *old, *new;
+#ifdef  OUTPUT
+static FILE *fpold, *fpnew;
+#endif
+
+#ifdef  TEST_HETAETA
+// g++ -DTEST_HETAETA -Drestrict= -I../libcdi/src -g -Wall -fopenmp hetaeta.cc 
+int Threading::ompNumThreads = 2;
+int
+cdo_omp_get_thread_num()
+{
+#ifdef _OPENMP
+  return omp_get_thread_num();
+#else
+  return 0;
+#endif
+}
 #endif
 
 static long
-int_index(long n, double *x1, double x2)
+int_index(const long n, const Varray<double> &x1, const double x2)
 {
   long k;
   long klo = 0;
@@ -86,21 +99,21 @@ esat(double temperature)
 
 #define MAX_VARS 512
 
-/* Source from INTERA */
+// Source from INTERA
 
 static void
-hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long nvars, const double *restrict af1,
-           const double *restrict bf1, const double *restrict etah2, const double *restrict af2, const double *restrict bf2,
-           const double *restrict w1, const double *restrict w2, const long *restrict jl1, const long *restrict jl2,
-           const double *restrict ah1, const double *restrict bh1, const double *restrict ps1, double epsm1i,
-           const double *restrict q1, const double *restrict t1, const double *restrict fis1, const double *restrict fis2,
-           double *restrict ps2, const double *restrict ah2, const double *restrict bh2, double *restrict *restrict vars1,
-           double *restrict *restrict vars2, double *restrict t2, double *restrict q2, double *restrict tscor,
-           double *restrict pscor, double *restrict secor, long jblt, double *restrict ph1, double *restrict lnph1,
-           double *restrict fi1, double *restrict pf1, double *restrict lnpf1, double *restrict tv1, double *restrict theta1,
-           double *restrict rh1, double *restrict zvar, double *restrict ph2, double *restrict lnph2, double *restrict fi2,
-           double *restrict pf2, double *restrict rh2, double *restrict wgt, long *restrict idx, double *restrict rh_pbl,
-           double *restrict theta_pbl, double *restrict *restrict vars_pbl, double *restrict zt2, double *restrict zq2)
+hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long nvars, const Varray<double> &af1,
+           const Varray<double> &bf1, const Varray<double> &etah2, const Varray<double> &af2, const Varray<double> &bf2,
+           const Varray<double> &w1, const Varray<double> &w2, const Varray<long> &jl1, const Varray<long> &jl2,
+           const double *restrict ah1, const double *restrict bh1, const Varray<double> &ps1, double epsm1i,
+           const Varray<double> &q1, const Varray<double> &t1, const Varray<double> &fis1, const Varray<double> &fis2,
+           Varray<double> &ps2, const double *restrict ah2, const double *restrict bh2, const Varray2D<double> &vars1,
+           Varray2D<double> &vars2, Varray<double> &t2, Varray<double> &q2, Varray<double> &tscor,
+           Varray<double> &pscor, Varray<double> &secor, long jblt, Varray<double> &ph1, Varray<double> &lnph1,
+           Varray<double> &fi1, Varray<double> &pf1, Varray<double> &lnpf1, Varray<double> &tv1, Varray<double> &theta1,
+           Varray<double> &rh1, Varray<double> &zvar, Varray<double> &ph2, Varray<double> &lnph2, Varray<double> &fi2,
+           Varray<double> &pf2, Varray<double> &rh2, Varray<double> &wgt, Varray<long> &idx, Varray<double> &rh_pbl,
+           Varray<double> &theta_pbl, Varray2D<double> &vars_pbl, Varray<double> &zt2, Varray<double> &zq2)
 {
   long ijk, ijk1, ijk2;
   long jlev = 0, jlevr, jnop;
@@ -116,7 +129,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
 
   /******* initialise atmospheric fields in old system */
 
-  /* pressure */
+  // pressure
   ph1[0] = 0.0;
   lnph1[0] = -1.0;
   for (int k = 1; k < nlev1p1; ++k)
@@ -131,7 +144,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
       lnpf1[k] = std::log(pf1[k]);
     }
 
-  /* virtual temperature, relative humidity, potential temperature */
+  // virtual temperature, relative humidity, potential temperature
   if (ltq)
     for (int k = 0; k < nlev1; ++k)
       {
@@ -153,7 +166,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
           fi1[k] = fi1[k + 1] + rair * tv1[k] * (lnph1[k + 1] - lnph1[k]);
         }
     }
-#if defined(OUTPUT)
+#ifdef  OUTPUT
   if (ij == OPOINT)
     for (int k = nlev1 - 1; k >= 0; --k)
       {
@@ -170,30 +183,28 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
             q = 0;
             fi = 0;
           }
-        fprintf(old, "%3d %18.10f %18.10f %18.10f %18.10f", k, fi / g, pf1[k], t, q);
-        for (int iv = 0; iv < nvars; ++iv) fprintf(old, " %18.10f", vars1[iv][ijk]);
-        fprintf(old, "\n");
+        fprintf(fpold, "%3d %18.10f %18.10f %18.10f %18.10f", k, fi / g, pf1[k], t, q);
+        for (int iv = 0; iv < nvars; ++iv) fprintf(fpold, " %18.10f", vars1[iv][ijk]);
+        fprintf(fpold, "\n");
       }
 #endif
 
   /******* find new surface pressure
            extra-/interpolate to new orography
            linear regression works not well for extrapolation
-           separation necessary
-  */
+           separation necessary */
   if (ltq)
     {
       for (int k = nlev1 - 2; k > 0; --k)
         {
-          /* find index for regression, 1 <= jlev <= nlevec-2 */
+          // find index for regression, 1 <= jlev <= nlevec-2
           jlev = k;
           if (fis2[ij] < fi1[k]) break;
         }
 
       double zdff = fi1[jlev + 1] - fis2[ij];
 
-      /* get the number of points used for estimation of regression coefficients
-       */
+      // get the number of points used for estimation of regression coefficients
       jlevr = 0;
       for (int k = jlev - 1; k > 0; --k)
         {
@@ -208,10 +219,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
         get coefficients of regression between Tv and lnP ::Tv = B*lnP + C
         using three levels surounding new orography geopotential
       */
-      double zsumt = 0.0;
-      double zsump = 0.0;
-      double zsumpp = 0.0;
-      double zsumtp = 0.0;
+      double zsumt = 0.0, zsump = 0.0, zsumpp = 0.0, zsumtp = 0.0;
 
       for (int k = jlevr; k <= jlev + 1; ++k)
         {
@@ -221,23 +229,23 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
           zsumtp = zsumtp + tv1[k] * lnpf1[k];
         }
 
-      /* final regression coefficients */
+      // final regression coefficients
       double zb = jnop * zsumpp - zsump * zsump;
       double zc = (zsumt * zsumpp - zsump * zsumtp) / zb;
       zb = (jnop * zsumtp - zsump * zsumt) / zb;
 
-      /* calculate preliminary surface pressure, adjust to middle level */
+      // calculate preliminary surface pressure, adjust to middle level
       double zps = lnph1[jlev];
 
-      /* calculate preliminary pressure */
+      // calculate preliminary pressure
       if (std::fabs(zb) < 1.0e-20)
         {
-          /* constant virtual temperature near new surface */
+          // constant virtual temperature near new surface
           ps2[ij] = std::exp(zps + (fi1[jlev] - fis2[ij]) / (zc * rair));
         }
       else
         {
-          /* virtual temperatur not constant near new surface */
+          // virtual temperatur not constant near new surface
           double zbb = zc * zc + zb * (zps * (zb * zps + 2.0 * zc) + 2.0 * (fi1[jlev] - fis2[ij]) / rair);
           ps2[ij] = std::exp((std::sqrt(zbb) - zc) / zb);
         }
@@ -258,7 +266,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   for (int k = 0; k < nlev2; ++k)
     {
       pf2[k] = af2[k] + bf2[k] * ps2[ij];
-      /* lnpf2[k] = std::log(pf2[k]); */
+      // lnpf2[k] = std::log(pf2[k]);
     }
 
   /******* find reference geopotential,  */
@@ -278,12 +286,12 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
 
   /******* find the new boundary layer top */
 
-  /* using the pressure from the old system */
+  // using the pressure from the old system
   double pbl_lim = ps1[ij] * eta_pbl;
   jjblt = nlev2 - 1;
   for (int k = nlev2 - 1; k > 0; --k)
     {
-      /* find the next upper level in new system */
+      // find the next upper level in new system
       double pbl_lim_need = ps2[ij] * etah2[k];
       if (pbl_lim > pbl_lim_need) break;
       jjblt = jjblt - 1;
@@ -293,7 +301,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (jblt < jjblt) jjblt = jblt;
 
   /******* PBL profile interpolation */
-  /* tension spline interpolation with full eta levels */
+  // tension spline interpolation with full eta levels
 
   if (ltq)
     for (int k = jjblt; k < nlev2; ++k)
@@ -354,14 +362,14 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
         }
     }
 
-  /******* merge boundary layer and free atmosphere */
+  // ******* merge boundary layer and free atmosphere
 
   if (ltq)
     {
-      /* correction of potential temperature at top of PBL */
+      // correction of potential temperature at top of PBL
       dteta = zt2[jjblt] * std::pow(apr / pf2[jjblt], rair_d_cpair) - theta_pbl[jjblt];
 
-      /* merge top layer values */
+      // merge top layer values
       rh2[jjblt] = 0.5 * (rh2[jjblt] + rh_pbl[jjblt]);
     }
 
@@ -371,7 +379,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
       vars2[iv][ijk] = 0.5 * (vars2[iv][ijk] + vars_pbl[iv][jjblt]);
     }
 
-  /* correct boundary profile values */
+  // correct boundary profile values
   if (ltq)
     for (int k = jjblt + 1; k < nlev2; ++k)
       {
@@ -392,34 +400,35 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
         zq2[k] = rh2[k] * epsilon * esat(zt2[k]) / pf2[k];
       }
 
-  /******* reference level correction */
+  // reference level correction
   if (lpsmod && ltq)
     {
-      /* integrate hydrostatic equation with preliminary temperature and
-       * pressure */
+      // integrate hydrostatic equation with preliminary temperature and pressure
       fi2[nlev2] = fis2[ij];
-      fi2[0] = -1.0; /* top not defined, infinity */
+      fi2[0] = -1.0; // top not defined, infinity
 
-      /* problem at top level, top pressure is zero per definition */
+      // problem at top level, top pressure is zero per definition
       for (int k = nlev2 - 1; k > 0; --k)
         {
           fi2[k] = fi2[k + 1] + rair * zt2[k] * (lnph2[k + 1] - lnph2[k]) * (1.0 + epsm1i * zq2[k]);
         }
 
-      /* search next level above reference level in new system */
+      // search next level above reference level in new system
       for (int k = nlev2 - 1; k > 0; --k)
         {
           jlev = k;
           if (ph2[k] < p_firef) break;
         }
 
-      /* correct surface pressure */
-      dfi = fiadj - (fi2[jlev + 1] + (fi2[jlev] - fi2[jlev + 1]) * std::log(ph2[jlev + 1] / p_firef) / std::log(ph2[jlev + 1] / ph2[jlev]));
+      // correct surface pressure
+      dfi = fiadj
+            - (fi2[jlev + 1]
+               + (fi2[jlev] - fi2[jlev + 1]) * std::log(ph2[jlev + 1] / p_firef) / std::log(ph2[jlev + 1] / ph2[jlev]));
       double ztv = (1.0 + epsm1i * zq2[nlev2 - 1]) * zt2[nlev2 - 1];
       ps2[ij] = ps2[ij] * std::exp(dfi / (rair * ztv));
     }
 
-  /******* final calculation of specific humidity profiles */
+  // final calculation of specific humidity profiles
   if (ltq)
     {
       for (int k = 0; k < nlev2; ++k) pf2[k] = af2[k] + bf2[k] * ps2[ij];
@@ -427,7 +436,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
       for (int k = 0; k < nlev2; ++k) zq2[k] = rh2[k] * epsilon * esat(zt2[k]) / pf2[k];
     }
 
-#if defined(OUTPUT)
+#ifdef  OUTPUT
   if (ij == OPOINT)
     for (int k = nlev2 - 1; k >= 0; --k)
       {
@@ -444,21 +453,21 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
             q = 0;
             fi = 0;
           }
-        fprintf(new, "%3d %18.10f %18.10f %18.10f %18.10f", k, fi / g, pf2[k], t, q);
-        for (int iv = 0; iv < nvars; ++iv) fprintf(new, " %18.10f", vars2[iv][ijk]);
-        fprintf(new, "\n");
+        fprintf(fpnew, "%3d %18.10f %18.10f %18.10f %18.10f", k, fi / g, pf2[k], t, q);
+        for (int iv = 0; iv < nvars; ++iv) fprintf(fpnew, " %18.10f", vars2[iv][ijk]);
+        fprintf(fpnew, "\n");
       }
 #endif
 
   if (ltq)
     {
-      /* calculate surface temperature correction (old version) */
+      // calculate surface temperature correction (old version)
       tscor[ij] = dteta * std::pow(ps2[ij] / apr, rair_d_cpair);
       pscor[ij] = std::pow(ps2[ij] / ps1[ij], rair_d_cpair);
 
-      /* correction term of static energy of lowest layer */
-      secor[ij]
-          = tv1[nlev1 - 1] * (cpair + rair * (1.0 - ph1[nlev1 - 1] / (ps1[ij] - ph1[nlev1 - 1]) * std::log(ps1[ij] / ph1[nlev1 - 1])));
+      // correction term of static energy of lowest layer
+      secor[ij] = tv1[nlev1 - 1]
+                  * (cpair + rair * (1.0 - ph1[nlev1 - 1] / (ps1[ij] - ph1[nlev1 - 1]) * std::log(ps1[ij] / ph1[nlev1 - 1])));
     }
 
   if (ltq)
@@ -471,48 +480,48 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
 }
 
 void
-hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1, const double *fis1, const double *ps1,
-        const double *t1, const double *q1, int nlev2, const double *ah2, const double *bh2, const double *fis2, double *ps2,
-        double *t2, double *q2, int nvars, double **vars1, double **vars2, double *tscor, double *pscor, double *secor)
+hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1, const Varray<double> &fis1, const Varray<double> &ps1,
+        const Varray<double> &t1, const Varray<double> &q1, int nlev2, const double *ah2, const double *bh2, const Varray<double> &fis2, Varray<double> &ps2,
+        Varray<double> &t2, Varray<double> &q2, int nvars, const Varray2D<double> &vars1, Varray2D<double> &vars2, Varray<double> &tscor, Varray<double> &pscor, Varray<double> &secor)
 {
   long jblt;
   long jlev = 0;
   int lpsmod = 1;
-  double *vars_pbl[MAX_VARS];
-#if defined(OUTPUT)
+#ifdef  OUTPUT
   double t, q, fi;
 
-  old = fopen("old.dat", "w");
-  new = fopen("new.dat", "w");
+  fpold = fopen("old.dat", "w");
+  fpnew = fopen("new.dat", "w");
 #endif
 
   long nlev1p1 = nlev1 + 1;
   long nlev2p1 = nlev2 + 1;
 
-  VARRAY_2D(double, ph1, Threading::ompNumThreads, nlev1p1);
-  VARRAY_2D(double, lnph1, Threading::ompNumThreads, nlev1p1);
-  VARRAY_2D(double, fi1, Threading::ompNumThreads, nlev1p1);
+  const auto nthreads = Threading::ompNumThreads;
+  Varray2D<double> ph1(nthreads, Varray<double>(nlev1p1));
+  Varray2D<double> lnph1(nthreads, Varray<double>(nlev1p1));
+  Varray2D<double> fi1(nthreads, Varray<double>(nlev1p1));
 
-  VARRAY_2D(double, pf1, Threading::ompNumThreads, nlev1);
-  VARRAY_2D(double, lnpf1, Threading::ompNumThreads, nlev1);
-  VARRAY_2D(double, tv1, Threading::ompNumThreads, nlev1);
-  VARRAY_2D(double, theta1, Threading::ompNumThreads, nlev1);
-  VARRAY_2D(double, rh1, Threading::ompNumThreads, nlev1);
-  VARRAY_2D(double, zvar, Threading::ompNumThreads, nlev1);
+  Varray2D<double> pf1(nthreads, Varray<double>(nlev1));
+  Varray2D<double> lnpf1(nthreads, Varray<double>(nlev1));
+  Varray2D<double> tv1(nthreads, Varray<double>(nlev1));
+  Varray2D<double> theta1(nthreads, Varray<double>(nlev1));
+  Varray2D<double> rh1(nthreads, Varray<double>(nlev1));
+  Varray2D<double> zvar(nthreads, Varray<double>(nlev1));
 
-  VARRAY_2D(double, ph2, Threading::ompNumThreads, nlev2p1);
-  VARRAY_2D(double, lnph2, Threading::ompNumThreads, nlev2p1);
-  VARRAY_2D(double, fi2, Threading::ompNumThreads, nlev2p1);
+  Varray2D<double> ph2(nthreads, Varray<double>(nlev2p1));
+  Varray2D<double> lnph2(nthreads, Varray<double>(nlev2p1));
+  Varray2D<double> fi2(nthreads, Varray<double>(nlev2p1));
 
-  VARRAY_2D(double, pf2, Threading::ompNumThreads, nlev2);
-  VARRAY_2D(double, rh2, Threading::ompNumThreads, nlev2);
-  VARRAY_2D(double, wgt, Threading::ompNumThreads, nlev2);
-  VARRAY_2D(long, idx, Threading::ompNumThreads, nlev2);
+  Varray2D<double> pf2(nthreads, Varray<double>(nlev2));
+  Varray2D<double> rh2(nthreads, Varray<double>(nlev2));
+  Varray2D<double> wgt(nthreads, Varray<double>(nlev2));
+  Varray2D<long> idx(nthreads, Varray<long>(nlev2));
 
-  VARRAY_2D(double, zt2, Threading::ompNumThreads, ltq ? nlev2 : 0);
-  VARRAY_2D(double, zq2, Threading::ompNumThreads, ltq ? nlev2 : 0);
-  VARRAY_2D(double, rh_pbl, Threading::ompNumThreads, ltq ? nlev2 : 0);
-  VARRAY_2D(double, theta_pbl, Threading::ompNumThreads, ltq ? nlev2 : 0);
+  Varray2D<double> zt2(nthreads, Varray<double>(ltq ? nlev2 : 0));
+  Varray2D<double> zq2(nthreads, Varray<double>(ltq ? nlev2 : 0));
+  Varray2D<double> rh_pbl(nthreads, Varray<double>(ltq ? nlev2 : 0));
+  Varray2D<double> theta_pbl(nthreads, Varray<double>(ltq ? nlev2 : 0));
 
   if (nvars > MAX_VARS)
     {
@@ -520,36 +529,27 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
       exit(-1);
     }
 
-  Varray3D<double> vars_pbl_2(Threading::ompNumThreads);
+  Varray3D<double> vars_pbl(nthreads);
   if (nvars > 0)
     {
-      for (int i = 0; i < Threading::ompNumThreads; i++)
+      for (int i = 0; i < nthreads; i++)
         {
-          vars_pbl_2[i].resize(nvars);
-          for (int iv = 0; iv < nvars; ++iv) vars_pbl_2[i][iv].resize(nlev2);
+          vars_pbl[i].resize(nvars);
+          for (int iv = 0; iv < nvars; ++iv) vars_pbl[i][iv].resize(nlev2);
         }
     }
 
-  std::vector<double> af1(nlev1);
-  std::vector<double> bf1(nlev1);
-  std::vector<double> etaf1(nlev1);
-
-  std::vector<double> etah2(nlev2p1);
-
-  std::vector<double> af2(nlev2);
-  std::vector<double> bf2(nlev2);
-  std::vector<double> etaf2(nlev2);
-
-  std::vector<double> w1(nlev2);
-  std::vector<double> w2(nlev2);
-  std::vector<long> jl1(nlev2);
-  std::vector<long> jl2(nlev2);
+  Varray<double> af1(nlev1), bf1(nlev1), etaf1(nlev1);
+  Varray<double> etah2(nlev2p1);
+  Varray<double> af2(nlev2), bf2(nlev2), etaf2(nlev2);
+  Varray<double> w1(nlev2), w2(nlev2);
+  Varray<long> jl1(nlev2), jl2(nlev2);
 
   /******* set coordinate system ETA's, A's, B's
            calculate half and full level ETA
            set the boundary layer index */
 
-  /* input system */
+  // input system
 
   for (int k = 0; k < nlev1; ++k)
     {
@@ -564,9 +564,9 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
       etaf1[k] = af1[k] * aipr + bf1[k];
     }
 
-  /* output system */
+  // output system
 
-  /* calculates full level VCT */
+  // calculates full level VCT
   for (int k = 0; k < nlev2; ++k)
     {
       af2[k] = 0.5 * (ah2[k] + ah2[k + 1]);
@@ -582,20 +582,20 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
       if (etah2[k] > eta_pbl) jblt = k;
     }
 
-  /* calculate weights for PBL interpolation */
+  // calculate weights for PBL interpolation
   for (int k = 0; k < nlev2; ++k)
     {
-      /* scan through new vertical levels set changes outside the full level eta's of old system to constant */
+      // scan through new vertical levels set changes outside the full level eta's of old system to constant
       if (etaf2[k] <= etaf1[0])
         {
-          /* at top of atmosphere */
+          // at top of atmosphere
           jl1[k] = 0;
           jl2[k] = 1;
           w2[k] = 0.0;
         }
       else if (etaf2[k] >= etaf1[nlev1 - 1])
         {
-          /* at surface of atmosphere */
+          // at surface of atmosphere
           jl1[k] = nlev1 - 2;
           jl2[k] = nlev1 - 1;
           w2[k] = 1.0;
@@ -604,7 +604,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
         {
           for (jlev = nlev1 - 2; jlev >= 1; jlev--)
             {
-              jl1[k] = jlev; /* find nearest eta level below */
+              jl1[k] = jlev; // find nearest eta level below
               if (etaf2[k] > etaf1[jlev]) break;
             }
           jl2[k] = jl1[k] + 1;
@@ -616,43 +616,43 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
   double epsm1i = 1.0 / epsilon - 1.0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) firstprivate(lpsmod) private(vars_pbl) schedule(dynamic, 1)                                \
+#pragma omp parallel for default(none) firstprivate(lpsmod) schedule(dynamic, 1)                                \
     shared(ngp, ph1, lnph1, fi1, pf1, lnpf1, tv1, theta1, rh1, zvar, ph2, lnph2, fi2, pf2, rh_pbl, zt2, zq2, theta_pbl, rh2, wgt, \
-           idx, vars_pbl_2, af1, bf1, etah2, af2, bf2, w1, w2, jl1, jl2, ltq, nvars, imiss, ah1, bh1, ps1, nlev1, epsm1i, q1, t1, \
+           idx, vars_pbl, af1, bf1, etah2, af2, bf2, w1, w2, jl1, jl2, ltq, nvars, imiss, ah1, bh1, ps1, nlev1, epsm1i, q1, t1, \
            fis1, fis2, ps2, ah2, bh2, nlev2, vars1, vars2, t2, q2, tscor, pscor, secor, jblt)
 #endif
   for (int ij = 0; ij < ngp; ++ij)
     {
-      auto ompthID = cdo_omp_get_thread_num();
-
-      for (int iv = 0; iv < nvars; ++iv) vars_pbl[iv] = &vars_pbl_2[ompthID][iv][0];
-
       if (imiss && imiss[ij]) continue;
 
-      hetaeta_sc(ltq, lpsmod, ij, ngp, nlev1, nlev2, nvars, &af1[0], &bf1[0], &etah2[0], &af2[0], &bf2[0], &w1[0], &w2[0], &jl1[0],
-                 &jl2[0], ah1, bh1, ps1, epsm1i, q1, t1, fis1, fis2, ps2, ah2, bh2, vars1, vars2, t2, q2, tscor, pscor, secor, jblt,
-                 &ph1[ompthID][0], &lnph1[ompthID][0], &fi1[ompthID][0], &pf1[ompthID][0], &lnpf1[ompthID][0], &tv1[ompthID][0],
-                 &theta1[ompthID][0], &rh1[ompthID][0], &zvar[ompthID][0], &ph2[ompthID][0], &lnph2[ompthID][0], &fi2[ompthID][0],
-                 &pf2[ompthID][0], &rh2[ompthID][0], &wgt[ompthID][0], &idx[ompthID][0], &rh_pbl[ompthID][0],
-                 &theta_pbl[ompthID][0], vars_pbl, &zt2[ompthID][0], &zq2[ompthID][0]);
+      const auto ompthID = cdo_omp_get_thread_num();
+      hetaeta_sc(ltq, lpsmod, ij, ngp, nlev1, nlev2, nvars, af1, bf1, etah2, af2, bf2, w1, w2, jl1,
+                 jl2, ah1, bh1, ps1, epsm1i, q1, t1, fis1, fis2, ps2, ah2, bh2, vars1, vars2, t2, q2, tscor, pscor, secor, jblt,
+                 ph1[ompthID], lnph1[ompthID], fi1[ompthID], pf1[ompthID], lnpf1[ompthID], tv1[ompthID],
+                 theta1[ompthID], rh1[ompthID], zvar[ompthID], ph2[ompthID], lnph2[ompthID], fi2[ompthID],
+                 pf2[ompthID], rh2[ompthID], wgt[ompthID], idx[ompthID], rh_pbl[ompthID],
+                 theta_pbl[ompthID], vars_pbl[ompthID], zt2[ompthID], zq2[ompthID]);
 
-    } /* end for ij */
+    }
 
-#if defined(OUTPUT)
-  fclose(old);
-  fclose(new);
+#ifdef  OUTPUT
+  fclose(fpold);
+  fclose(fpnew);
 #endif
 
   return;
 }
 
-#if defined(TEST_HETAETA)
+#ifdef  TEST_HETAETA
 #define NGP (512)
-int Threading::ompNumThreads = 2;
 
 int
 main(int argc, char *argv[])
 {
+#ifdef _OPENMP
+  omp_set_num_threads(Threading::ompNumThreads);
+#endif
+
   printf("NGP = %d\n", NGP);
 
   double a2[41] = { 0.00000000000000000,     2000.00000000000000000,  4000.00000000000000000,  6000.00000000000000000,
@@ -711,74 +711,40 @@ main(int argc, char *argv[])
 
   double ifis1 = 9.121094, ips1 = 102511.8;
 
-  double *vars1[5];
-  double *vars2[5];
-
-  int ij, k;
   bool ltq = true;
 
-  double *fis1 = (double *) Malloc(NGP * sizeof(double));
-  double *ps1 = (double *) Malloc(NGP * sizeof(double));
-  double *fis2 = (double *) Malloc(NGP * sizeof(double));
-  double *ps2 = (double *) Malloc(NGP * sizeof(double));
+  Varray<double> fis1(NGP), ps1(NGP), fis2(NGP), ps2(NGP);
+  Varray<double> tscor(NGP), pscor(NGP), secor(NGP);
 
-  double *tscor = (double *) Malloc(NGP * sizeof(double));
-  double *pscor = (double *) Malloc(NGP * sizeof(double));
-  double *secor = (double *) Malloc(NGP * sizeof(double));
+  Varray<double> t1(NGP * 19), q1(NGP * 19);
+  Varray2D<double> vars1(5);
+  for (int i = 0; i < 5; ++i) vars1[i].resize(NGP * 19);
 
-  double *t1 = (double *) Malloc(NGP * 19 * sizeof(double));
-  double *q1 = (double *) Malloc(NGP * 19 * sizeof(double));
-  double *u1 = (double *) Malloc(NGP * 19 * sizeof(double));
-  double *v1 = (double *) Malloc(NGP * 19 * sizeof(double));
-  double *cl1 = (double *) Malloc(NGP * 19 * sizeof(double));
-  double *ci1 = (double *) Malloc(NGP * 19 * sizeof(double));
-  double *cc1 = (double *) Malloc(NGP * 19 * sizeof(double));
+  Varray<double> t2(NGP * 40), q2(NGP * 40);
+  Varray2D<double> vars2(5);
+  for (int i = 0; i < 5; ++i) vars2[i].resize(NGP * 40);
 
-  double *t2 = (double *) Malloc(NGP * 40 * sizeof(double));
-  double *q2 = (double *) Malloc(NGP * 40 * sizeof(double));
-  double *u2 = (double *) Malloc(NGP * 40 * sizeof(double));
-  double *v2 = (double *) Malloc(NGP * 40 * sizeof(double));
-  double *cl2 = (double *) Malloc(NGP * 40 * sizeof(double));
-  double *ci2 = (double *) Malloc(NGP * 40 * sizeof(double));
-  double *cc2 = (double *) Malloc(NGP * 40 * sizeof(double));
-
-  for (ij = 0; ij < NGP; ++ij)
+  for (int ij = 0; ij < NGP; ++ij)
     {
       ps1[ij] = ips1;
       fis1[ij] = ifis1;
     }
 
-  for (k = 0; k < 19; ++k)
-    for (ij = 0; ij < NGP; ++ij)
+  for (int k = 0; k < 19; ++k)
+    for (int ij = 0; ij < NGP; ++ij)
       {
         t1[k * NGP + ij] = it1[k];
         q1[k * NGP + ij] = iq1[k];
-        u1[k * NGP + ij] = iu1[k];
-        v1[k * NGP + ij] = iv1[k];
-        cl1[k * NGP + ij] = icl1[k];
-        ci1[k * NGP + ij] = ici1[k];
-        cc1[k * NGP + ij] = icc1[k];
+        vars1[0][k * NGP + ij] = iu1[k];
+        vars1[1][k * NGP + ij] = iv1[k];
+        vars1[2][k * NGP + ij] = icl1[k];
+        vars1[3][k * NGP + ij] = ici1[k];
+        vars1[4][k * NGP + ij] = icc1[k];
       }
 
-  vars1[0] = u1;
-  vars1[1] = v1;
-  vars1[2] = cl1;
-  vars1[3] = ci1;
-  vars1[4] = cc1;
+  for (int ij = 0; ij < NGP; ++ij) fis2[ij] = fis1[ij];
 
-  vars2[0] = u2;
-  vars2[1] = v2;
-  vars2[2] = cl2;
-  vars2[3] = ci2;
-  vars2[4] = cc2;
-
-  for (ij = 0; ij < NGP; ++ij) fis2[ij] = fis1[ij];
-
-  if (ltq)
-    hetaeta(ltq, NGP, NULL, 19, a1, b1, fis1, ps1, t1, q1, 40, a2, b2, fis2, ps2, t2, q2, 5, vars1, vars2, tscor, pscor, secor);
-  else
-    hetaeta(ltq, NGP, NULL, 19, a1, b1, fis1, ps1, NULL, NULL, 40, a2, b2, fis2, ps2, NULL, NULL, 5, vars1, vars2, NULL, NULL,
-            NULL);
+  hetaeta(ltq, NGP, NULL, 19, a1, b1, fis1, ps1, t1, q1, 40, a2, b2, fis2, ps2, t2, q2, 5, vars1, vars2, tscor, pscor, secor);
 
   double ot2[40] = { 224.226, 212.75,  209.616, 208.895, 210.468, 213.454, 218.091, 220.49,  220.977, 221.114,
                      220.731, 220.803, 223.595, 226.547, 230.566, 234.99,  239.411, 243.722, 248.125, 252.164,
@@ -786,11 +752,11 @@ main(int argc, char *argv[])
                      274.932, 276.493, 278.199, 279.638, 280.811, 281.746, 282.613, 283.479, 284.087, 284.343 };
 
   int lerror = 0;
-  for (k = 0; k < 40; ++k)
+  for (int k = 0; k < 40; ++k)
     if (std::fabs(t2[k * NGP] - ot2[k]) > 0.001) lerror = 1;
 
   if (lerror)
-    for (k = 0; k < 40; ++k) printf("%d %g %g %g\n", k, t2[k * NGP], ot2[k], std::fabs(t2[k * NGP] - ot2[k]));
+    for (int k = 0; k < 40; ++k) printf("%d %g %g %g\n", k, t2[k * NGP], ot2[k], std::fabs(t2[k * NGP] - ot2[k]));
 
   return 0;
 }
