@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,15 +17,9 @@
 #ifndef _HETAETA_H
 #define _HETAETA_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <stdbool.h>
-
-void hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1, const double *fis1,
-             const double *ps1, const double *t1, const double *q1, int nlev2, const double *ah2, const double *bh2,
-             const double *fis2, double *ps2, double *t2, double *q2, int nvars, double **vars1, double **vars2, double *tscor,
-             double *pscor, double *secor);
+void hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1, const Varray<double> &fis1,
+             const Varray<double> &ps1, const Varray<double> &t1, const Varray<double> &q1, int nlev2, const double *ah2, const double *bh2,
+             const Varray<double> &fis2, Varray<double> &ps2, Varray<double> &t2, Varray<double> &q2, int nvars, const Varray2D<double> &vars1, Varray2D<double> &vars2, Varray<double> &scor,
+             Varray<double> &pscor, Varray<double> &secor);
 
 #endif /* _HETAETA_H */
