@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,44 +16,36 @@
 */
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "process_int.h"
+#include "readline.h"
+#include "cdo_default_values.h"
 
 static int
 readInstitution(const char *instfile)
 {
-  int instID = CDI_UNDEFID;
-  char line[1024], *pline;
+  char line[1024];
   int lnr = 0;
   int nvar = 0, maxvar = 4;
   char name[1024], longname[1024];
   int center = CDI_UNDEFID, subcenter = CDI_UNDEFID;
 
-  FILE *instfp = fopen(instfile, "r");
+  auto instfp = fopen(instfile, "r");
+  if (instfp == nullptr) return CDI_UNDEFID;
 
-  if (instfp == nullptr) return instID;
-
-  while (readline(instfp, line, 1024))
+  while (cdo::readline(instfp, line, 1024))
     {
       lnr++;
       if (line[0] == '#') continue;
       if (nvar == maxvar) break;
       nvar++;
 
-      pline = line;
+      char *pline = line;
       while (isspace((int) *pline)) pline++;
 
-      if (nvar == 1)
-        {
-          if (isdigit((int) pline[0]))
-            maxvar = 4;
-          else
-            maxvar = 2;
-        }
+      if (nvar == 1) maxvar = isdigit((int) pline[0]) ? 4 : 2;
 
-      if (nvar == 1 && maxvar == 4)
-        {
-          center = atoi(pline);
-        }
+      if (nvar == 1 && maxvar == 4) center = atoi(pline);
 
       if (nvar == 2 && maxvar == 4)
         {
@@ -62,20 +54,14 @@ readInstitution(const char *instfile)
           subcenter = atoi(pline);
         }
 
-      if ((nvar == 3 && maxvar == 4) || (nvar == 1 && maxvar == 2))
-        {
-          strcpy(name, pline);
-        }
+      if ((nvar == 3 && maxvar == 4) || (nvar == 1 && maxvar == 2)) strcpy(name, pline);
 
-      if ((nvar == 4 && maxvar == 4) || (nvar == 2 && maxvar == 2))
-        {
-          strcpy(longname, pline);
-        }
+      if ((nvar == 4 && maxvar == 4) || (nvar == 2 && maxvar == 2)) strcpy(longname, pline);
     }
 
   fclose(instfp);
 
-  instID = institutInq(center, subcenter, name, longname);
+  auto instID = institutInq(center, subcenter, name, longname);
   if (instID == CDI_UNDEFID) instID = institutDef(center, subcenter, name, longname);
 
   return instID;
