@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,15 +14,18 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
+
+#include <algorithm>
+
 #include <cdi.h>
 
-#include "cdo_int.h"
-#include "cdo_math.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 #include "cdo_options.h"
 #include "interpol.h"
 #include "progress.h"
 #include "cimdOmp.h"
+#include "matrix_view.h"
 
 /**
 * Find the interval i-1 .. i in which an element x fits and return i, the
@@ -35,34 +38,30 @@
 * @param x      the element to find a position for
 */
 static long
-find_element(double x, long nelem, const double *restrict array)
+find_element(double x, long nelem, const Varray<double> &v)
 {
   long ii;
   long mid = 0;
   long first = 1;
   long last = nelem;
 
-  if (array[0] < array[nelem - 1])  // ascending order
+  if (v[0] < v[nelem - 1])  // ascending order
     {
       // return the length of the array if x is out of bounds
-      if (x < array[0] || x > array[nelem - 1]) return nelem;
+      if (x < v[0] || x > v[nelem - 1]) return nelem;
 
       // search for the interval in which x fits
       // implementation: binary search algorithm
       for (ii = 1; ii < nelem; ++ii)
         {
           // binary search: divide search room in the middle
-          // mid = first + ((last - first) >> 1);
-          // faster!
           mid = (first + last) >> 1;
 
           // return the bigger interval border of the interval in which x fits
-          // if ( x >= array[mid-1] && x <= array[mid] ) break;
-          // faster!
-          if (!(x < array[mid - 1] || x > array[mid])) break;
+          if (!(x < v[mid - 1] || x > v[mid])) break;
 
           // binary search: ignore half of the search room
-          if (x > array[mid])
+          if (x > v[mid])
             first = mid;
           else
             last = mid;
@@ -71,31 +70,27 @@ find_element(double x, long nelem, const double *restrict array)
   else
     {
       // return the length of the array if x is out of bounds
-      if (x < array[nelem - 1] || x > array[0]) return nelem;
+      if (x < v[nelem - 1] || x > v[0]) return nelem;
 
       // search for the interval in which x fits
       // implementation: binary search algorithm
       for (ii = 1; ii < nelem; ++ii)
         {
           // binary search: divide search room in the middle
-          // mid = first + ((last - first) >> 1);
-          // faster!
           mid = (first + last) >> 1;
 
           // return the bigger interval border of the interval in which x fits
-          // if ( x >= array[mid] && x <= array[mid-1] ) break;
-          // faster!
-          if (!(x < array[mid] || x > array[mid - 1])) break;
+          if (!(x < v[mid] || x > v[mid - 1])) break;
 
           // binary search: ignore half of the search room
-          if (x < array[mid])
+          if (x < v[mid])
             first = mid;
           else
             last = mid;
         }
     }
 
-  if (mid > 1 && IS_EQUAL(x, array[mid - 1])) mid--;
+  if (mid > 1 && IS_EQUAL(x, v[mid - 1])) mid--;
 
   return mid;
 }
@@ -121,32 +116,32 @@ long find_element(double x, long nelem, const double *array)
 */
 
 int
-rect_grid_search(size_t *ii, size_t *jj, double x, double y, size_t nxm, size_t nym, const double *restrict xm,
-                 const double *restrict ym)
+rect_grid_search(size_t &ii, size_t &jj, double x, double y, size_t nxm, size_t nym, const Varray<double> &xm,
+                 const Varray<double> &ym)
 {
   constexpr double rtol = 1.e-12;
   int lfound = 0;
 
-  *jj = find_element(y, nym, ym);
-  if (*jj >= nym && std::fabs(ym[0] - y) < rtol) *jj = 1;  // fix rounding errors
+  jj = find_element(y, nym, ym);
+  if (jj >= nym && std::fabs(ym[0] - y) < rtol) jj = 1;  // fix rounding errors
 
-  if (*jj < nym)
+  if (jj < nym)
     {
-      *ii = find_element(x, nxm, xm);
-      if (*ii >= nxm && std::fabs(xm[0] - x) < rtol) *ii = 1;  // fix rounding errors
+      ii = find_element(x, nxm, xm);
+      if (ii >= nxm && std::fabs(xm[0] - x) < rtol) ii = 1;  // fix rounding errors
 
-      if (*ii < nxm) lfound = 1;
+      if (ii < nxm) lfound = 1;
     }
 
   return lfound;
 }
 
 int
-rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm, const double *restrict xm)
+rect_grid_search2(long &imin, long &imax, double xmin, double xmax, long nxm, const Varray<double> &xm)
 {
   int lfound = 0;
-  *imin = nxm;
-  *imax = -1;
+  imin = nxm;
+  imax = -1;
 
   const bool lascend = (xm[0] < xm[nxm - 1]);
 
@@ -160,14 +155,14 @@ rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm, co
       if (lascend)
         {
           if (i1 > 1 && xmin <= xm[i1 - 1]) i1--;
-          *imin = i1 - 1;
-          *imax = i1 - 1;
+          imin = i1 - 1;
+          imax = i1 - 1;
         }
       else
         {
           if (i1 < nxm - 1 && xmin <= xm[i1]) i1++;
-          *imin = i1 - 1;
-          *imax = i1 - 1;
+          imin = i1 - 1;
+          imax = i1 - 1;
         }
     }
 
@@ -178,28 +173,28 @@ rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm, co
       if (lascend)
         {
           if (i2 < nxm - 1 && xmax >= xm[i2]) i2++;
-          *imax = i2 - 1;
-          if (*imin == nxm) *imin = *imax;
+          imax = i2 - 1;
+          if (imin == nxm) imin = imax;
         }
       else
         {
           if (i2 > 1 && xmax >= xm[i2 - 1]) i2--;
-          *imin = i2 - 1;
-          if (*imax == -1) *imax = *imin;
+          imin = i2 - 1;
+          if (imax == -1) imax = imin;
         }
     }
 
   return lfound;
 }
 
-double
-intlinarr2p(long nxm, long nym, double **fieldm, const double *xm, const double *ym, double x, double y)
+static double
+intlinarr2p(long nxm, long nym, double **fieldm, const Varray<double> &xm, const Varray<double> &ym, double x, double y)
 {
   long ii, jj;
   double value = 0;
 
   for (jj = 1; jj < nym; jj++)
-    if (y >= cdo::min(ym[jj - 1], ym[jj]) && y <= cdo::max(ym[jj - 1], ym[jj])) break;
+    if (y >= std::min(ym[jj - 1], ym[jj]) && y <= std::max(ym[jj - 1], ym[jj])) break;
 
   for (ii = 1; ii < nxm; ii++)
     if (x >= xm[ii - 1] && x <= xm[ii]) break;
@@ -215,11 +210,13 @@ intlinarr2p(long nxm, long nym, double **fieldm, const double *xm, const double
   return value;
 }
 
+template <typename T>
 static void
-intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const double *restrict fieldm, const std::vector<double> &xm,
-           const std::vector<double> &ym, size_t gridsize2, double *field, const std::vector<double> &x, const std::vector<double> &y)
+intlinarr2(T missval, int lon_is_circular, size_t nxm, size_t nym, const Varray<T> &varray1,
+           const Varray<double> &xm, const Varray<double> &ym, size_t gridsize2, Varray<T> &varray2,
+           const Varray<double> &x, const Varray<double> &y)
 {
-  size_t nlon1 = nxm;
+  auto nlon1 = nxm;
   double findex = 0;
 
   if (lon_is_circular) nlon1--;
@@ -227,28 +224,28 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
 
   std::vector<bool> grid1_mask(gridsize1);
   for (size_t jj = 0; jj < nym; ++jj)
-    for (size_t ii = 0; ii < nlon1; ++ii) grid1_mask[jj * nlon1 + ii] = !DBL_IS_EQUAL(fieldm[jj * nlon1 + ii], missval);
+    for (size_t ii = 0; ii < nlon1; ++ii) grid1_mask[jj * nlon1 + ii] = !DBL_IS_EQUAL(varray1[jj * nlon1 + ii], missval);
 
-  Progress::init();
+  progress::init();
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-    shared(findex, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, lon_is_circular, grid1_mask)
+  shared(findex, varray1, varray2, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, lon_is_circular, grid1_mask)
 #endif
   for (size_t i = 0; i < gridsize2; ++i)
     {
-      size_t src_add[4]; /*  address for the four source points    */
+      size_t src_add[4]; // address for the four source points
 
-      field[i] = missval;
+      varray2[i] = missval;
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize2);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / gridsize2);
 
       size_t ii, jj;
-      int lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm.data(), ym.data());
+      auto lfound = rect_grid_search(ii, jj, x[i], y[i], nxm, nym, xm, ym);
 
       if (lfound)
         {
@@ -259,7 +256,7 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
           src_add[2] = (jj) *nlon1 + (ii - 1);
           src_add[3] = (jj) *nlon1 + (iix);
 
-          /* Check to see if points are missing values */
+          // Check to see if points are missing values
           for (int n = 0; n < 4; ++n)
             if (!grid1_mask[src_add[n]]) lfound = 0;
         }
@@ -275,12 +272,13 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
           // printf("%2ld %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f %6.4f\n",
           // dst_add, plon, plat, wgts[0], wgts[1], wgts[2], wgts[3], iw, jw);
 
-          field[i] = 0;
-          for (int n = 0; n < 4; ++n) field[i] += fieldm[src_add[n]] * wgts[n];
+          double fwsum = 0.0;
+          for (int n = 0; n < 4; ++n) fwsum += varray1[src_add[n]] * wgts[n];
+          varray2[i] = fwsum;
         }
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 }
 
 void
@@ -299,217 +297,139 @@ intlinarr(long nxm, double *ym, double *xm, int nx, double *y, double *x)
 void
 intgridbil(Field &field1, Field &field2)
 {
-  char xunits[CDI_MAX_NAME];
-  int gridID1 = field1.grid;
-  int gridID2 = field2.grid;
-  double *array1 = field1.vec.data();
-  double *array2 = field2.vec.data();
-  const double missval = field1.missval;
+  auto gridID1 = field1.grid;
+  auto gridID2 = field2.grid;
+  const auto missval = field1.missval;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
+  auto nlon1 = gridInqXsize(gridID1);
+  auto nlat1 = gridInqYsize(gridID1);
 
   int lon_is_circular = 0;
 
   bool lgeorefgrid = true;
   if (grid_is_distance_generic(gridID1) && grid_is_distance_generic(gridID2)) lgeorefgrid = false;
 
-  double **array1_2D = (double **) Malloc(nlat1 * sizeof(double *));
-  for (size_t ilat = 0; ilat < nlat1; ilat++) array1_2D[ilat] = array1 + ilat * nlon1;
-
   if (lgeorefgrid)
     {
-      if (!(gridInqXvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr))) cdoAbort("Source grid has no coordinate values!");
+      if (!gridHasCoordinates(gridID1)) cdoAbort("Source grid has no coordinate values!");
 
       lon_is_circular = gridIsCircular(gridID1);
-
       if (lon_is_circular) nlon1 += 1;
     }
 
-  std::vector<double> lon1(nlon1);
-  std::vector<double> lat1(nlat1);
-  gridInqXvals(gridID1, lon1.data());
-  gridInqYvals(gridID1, lat1.data());
+  Varray<double> lons1(nlon1), lats1(nlat1);
+  gridInqXvals(gridID1, lons1.data());
+  gridInqYvals(gridID1, lats1.data());
 
   if (lgeorefgrid)
     {
-      if (lon_is_circular) lon1[nlon1 - 1] = 0;
+      if (lon_is_circular) lons1[nlon1 - 1] = 0;
 
-      gridInqXunits(gridID1, xunits);
+      cdo_grid_to_radian(gridID1, CDI_XAXIS, nlon1, lons1.data(), "grid1 center lon");
+      cdo_grid_to_radian(gridID1, CDI_YAXIS, nlat1, lats1.data(), "grid1 center lat");
 
-      grid_to_radian(xunits, nlon1, lon1.data(), "grid1 center lon");
-      grid_to_radian(xunits, nlat1, lat1.data(), "grid1 center lat");
-
-      if (lon_is_circular) lon1[nlon1 - 1] = lon1[0] + 2 * M_PI;
+      if (lon_is_circular) lons1[nlon1 - 1] = lons1[0] + 2 * M_PI;
     }
 
-  const size_t xsize2 = gridInqXsize(gridID2);
-  const size_t ysize2 = gridInqYsize(gridID2);
+  const auto xsize2 = gridInqXsize(gridID2);
+  const auto ysize2 = gridInqYsize(gridID2);
 
-  if (xsize2 == 1 && ysize2 == 1)
+  if (lgeorefgrid)
     {
-      double lon2, lat2;
-      gridInqXvals(gridID2, &lon2);
-      gridInqYvals(gridID2, &lat2);
+      if (gridInqType(gridID2) == GRID_GME) gridID2 = gridToUnstructured(gridID2, 0);
 
-      gridInqXunits(gridID2, xunits);
+      if (gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR)
+        gridID2 = gridToCurvilinear(gridID2, 0);
 
-      grid_to_radian(xunits, xsize2, &lon2, "grid2 center lon");
-      grid_to_radian(xunits, ysize2, &lat2, "grid2 center lat");
+      if (!gridHasCoordinates(gridID2)) cdoAbort("Target grid has no coordinate values!");
+    }
 
-      if (lon2 < lon1[0]) lon2 += 2 * M_PI;
+  const auto gridsize2 = gridInqSize(gridID2);
 
-      if (lon2 > lon1[nlon1 - 1])
+  Varray<double> xvals2(gridsize2), yvals2(gridsize2);
+
+  if (lgeorefgrid)
+    {
+      gridInqXvals(gridID2, xvals2.data());
+      gridInqYvals(gridID2, yvals2.data());
+
+      cdo_grid_to_radian(gridID2, CDI_XAXIS, gridsize2, xvals2.data(), "grid2 center lon");
+      cdo_grid_to_radian(gridID2, CDI_YAXIS, gridsize2, yvals2.data(), "grid2 center lat");
+
+      for (size_t i = 0; i < gridsize2; ++i)
         {
-          double **field = array1_2D;
-          array1_2D = (double **) Malloc(nlat1 * sizeof(double *));
-          lon1.resize(nlon1 + 1);
-          double *array = (double *) Malloc(nlat1 * (nlon1 + 1) * sizeof(double));
-
-          for (size_t ilat = 0; ilat < nlat1; ilat++)
-            {
-              array1_2D[ilat] = array + ilat * (nlon1 + 1);
-              memcpy(array1_2D[ilat], field[ilat], nlon1 * sizeof(double));
-              array1_2D[ilat][nlon1] = array1_2D[ilat][0];
-              lon1[nlon1] = lon1[0] + 2 * M_PI;
-            }
-          nlon1++;
-          Free(field);
-          Free(array);
+          if (xvals2[i] < lons1[0]) xvals2[i] += 2 * M_PI;
+          if (xvals2[i] > lons1[nlon1 - 1]) xvals2[i] -= 2 * M_PI;
         }
-
-      if (lon2 < lon1[0] || lon2 > lon1[nlon1 - 1])
-        cdoAbort("Longitude %f out of bounds (%f to %f)!", lon2, lon1[0], lon1[nlon1 - 1]);
-
-      if (lat2 < cdo::min(lat1[0], lat1[nlat1 - 1]) || lat2 > cdo::max(lat1[0], lat1[nlat1 - 1]))
-        cdoAbort("Latitude %f out of bounds (%f to %f)!", lat2, lat1[0], lat1[nlat1 - 1]);
-
-      *array2 = intlinarr2p(nlon1, nlat1, array1_2D, lon1.data(), lat1.data(), lon2, lat2);
-      /*
-      printf("%5d %f %f %f\n", index++, lon2, lat2, *array2);
-      */
     }
   else
     {
-      if (lgeorefgrid)
-        {
-          if (gridInqType(gridID2) == GRID_GME) gridID2 = gridToUnstructured(gridID2, 0);
+      Varray<double> xcoord(xsize2), ycoord(ysize2);
+      gridInqXvals(gridID2, xcoord.data());
+      gridInqYvals(gridID2, ycoord.data());
 
-          if (gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR)
-            gridID2 = gridToCurvilinear(gridID2, 0);
-
-          if (!(gridInqXvals(gridID2, nullptr) && gridInqYvals(gridID2, nullptr)))
-            cdoAbort("Target grid has no coordinate values!");
-        }
-
-      const size_t gridsize2 = gridInqSize(gridID2);
-
-      std::vector<double> xvals2(gridsize2);
-      std::vector<double> yvals2(gridsize2);
-
-      if (lgeorefgrid)
-        {
-          gridInqXvals(gridID2, xvals2.data());
-          gridInqYvals(gridID2, yvals2.data());
-
-          gridInqXunits(gridID2, xunits);
-
-          grid_to_radian(xunits, gridsize2, xvals2.data(), "grid2 center lon");
-          grid_to_radian(xunits, gridsize2, yvals2.data(), "grid2 center lat");
-
-          for (size_t i = 0; i < gridsize2; ++i)
-            {
-              if (xvals2[i] < lon1[0]) xvals2[i] += 2 * M_PI;
-              if (xvals2[i] > lon1[nlon1 - 1]) xvals2[i] -= 2 * M_PI;
-            }
-        }
-      else
-        {
-          std::vector<double> xcoord(xsize2);
-          std::vector<double> ycoord(ysize2);
-
-          gridInqXvals(gridID2, xcoord.data());
-          gridInqYvals(gridID2, ycoord.data());
-
-          for (size_t j = 0; j < ysize2; ++j)
-            for (size_t i = 0; i < xsize2; ++i)
-              {
-                xvals2[j * xsize2 + i] = xcoord[i];
-                yvals2[j * xsize2 + i] = ycoord[j];
-              }
-        }
-
-      intlinarr2(missval, lon_is_circular, nlon1, nlat1, array1, lon1, lat1, gridsize2, array2, xvals2, yvals2);
-
-      field2.nmiss = arrayNumMV(gridsize2, array2, missval);
+      for (size_t j = 0; j < ysize2; ++j)
+        for (size_t i = 0; i < xsize2; ++i)
+          {
+            xvals2[j * xsize2 + i] = xcoord[i];
+            yvals2[j * xsize2 + i] = ycoord[j];
+          }
     }
 
-  Free(array1_2D);
+  if (field2.grid != gridID2) gridDestroy(gridID2);
+
+  if (field1.memType == MemType::Float)
+    intlinarr2((float)missval, lon_is_circular, nlon1, nlat1, field1.vec_f, lons1, lats1, gridsize2, field2.vec_f, xvals2, yvals2);
+  else
+    intlinarr2(missval, lon_is_circular, nlon1, nlat1, field1.vec_d, lons1, lats1, gridsize2, field2.vec_d, xvals2, yvals2);
+
+  fieldNumMV(field2);
 }
 
-/* source code from pingo */
+// source code from pingo
 void
 interpolate(Field &field1, Field &field2)
 {
-  int i;
-  double *lono_array, *lato_array, *lono, *lato;
-  double *lon_array, *lat_array, *lon, *lat;
-  // size_t gridsize_i
-  size_t gridsize_o;
-  int gridIDi;
-  double *arrayIn;
-  int gridIDo;
-  double *arrayOut;
-  double missval;
-  long ilon, ilat, nxlon, nxlat, olon, olat;
+  long i;
+  long ilon, ilat, olon, olat;
   long l11, l12, l21, l22, l1, l2;
   double volon1, volon2, volat1, volat2;
-  double vilon1, vilon2, vilat1, vilat2;
+  double vilat1, vilat2;
   double vlon1, vlon2, vlat1, vlat2;
   long ilon1, ilon2;
-  double sum, wsum;
-  int k, n;
-  double *xin_array;
-  double **in0, **xin, **xout;
+  long k, n;
   int wrap_around, xlat_is_ascending;
-  double a11, a12, a21, a22, b11, b12, b21, b22, t;
+  double a11, a12, a21, a22, b11, b12, b21, b22;
   double faclon1, faclon2, faclat1, faclat2;
-  int nlon, nlat, out_nlon, out_nlat;
 
-  gridIDi = field1.grid;
-  gridIDo = field2.grid;
-  arrayIn = field1.vec.data();
-  arrayOut = field2.vec.data();
-  missval = field1.missval;
+  const auto gridIDi = field1.grid;
+  const auto gridIDo = field2.grid;
+  auto &arrayIn = field1.vec_d;
+  auto &arrayOut = field2.vec_d;
+  const auto missval = field1.missval;
 
   // gridsize_i = gridInqSize(gridIDi);
-  gridsize_o = gridInqSize(gridIDo);
+  size_t gridsize_o = gridInqSize(gridIDo);
 
-  nlon = gridInqXsize(gridIDi);
-  nlat = gridInqYsize(gridIDi);
-  out_nlon = gridInqXsize(gridIDo);
-  out_nlat = gridInqYsize(gridIDo);
+  long nlon = gridInqXsize(gridIDi);
+  long nlat = gridInqYsize(gridIDi);
+  long out_nlon = gridInqXsize(gridIDo);
+  long out_nlat = gridInqYsize(gridIDo);
 
-  lon_array = (double *) Malloc((nlon + 2) * sizeof(double));
-  lat_array = (double *) Malloc((nlat + 2) * sizeof(double));
-  lon = lon_array + 1;
-  lat = lat_array + 1;
+  Varray<double> lon_array(nlon + 2);
+  Varray<double> lat_array(nlat + 2);
+  double *lon = lon_array.data() + 1;
+  double *lat = lat_array.data() + 1;
 
-  if (!(gridInqXvals(gridIDi, nullptr) && gridInqYvals(gridIDi, nullptr))) cdoAbort("Source grid has no values");
-
-  if (!(gridInqXvals(gridIDo, nullptr) && gridInqYvals(gridIDo, nullptr))) cdoAbort("Target grid has no values");
+  if (!gridHasCoordinates(gridIDi)) cdoAbort("Source grid has no values");
+  if (!gridHasCoordinates(gridIDo)) cdoAbort("Target grid has no values");
 
   gridInqXvals(gridIDi, lon);
   gridInqYvals(gridIDi, lat);
 
-  /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridIDi, units);
-    grid_to_degree(units, nlon, lon, "grid1 center lon");
-    gridInqYunits(gridIDi, units);
-    grid_to_degree(units, nlat, lat, "grid1 center lat");
-  }
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridIDi, CDI_XAXIS, nlon, lon, "grid1 center lon");
+  cdo_grid_to_degree(gridIDi, CDI_YAXIS, nlat, lat, "grid1 center lat");
 
   if (nlon > 1)
     {
@@ -538,22 +458,17 @@ interpolate(Field &field1, Field &field2)
   if (lat[nlat] < -90) lat[nlat] = -99;
   if (lat[nlat] > 90) lat[nlat] = 99;
 
-  lono_array = (double *) Malloc((out_nlon < 2 ? 4 : out_nlon + 2) * sizeof(double));
-  lono = lono_array + 1;
-  lato_array = (double *) Malloc((out_nlat < 2 ? 4 : out_nlat + 2) * sizeof(double));
-  lato = lato_array + 1;
+  Varray<double> lono_array(out_nlon < 2 ? 4 : out_nlon + 2);
+  Varray<double> lato_array(out_nlat < 2 ? 4 : out_nlat + 2);
+  double *lono = lono_array.data() + 1;
+  double *lato = lato_array.data() + 1;
 
   gridInqXvals(gridIDo, lono);
   gridInqYvals(gridIDo, lato);
 
-  /* Convert lat/lon units if required */
-  {
-    char units[CDI_MAX_NAME];
-    gridInqXunits(gridIDo, units);
-    grid_to_degree(units, out_nlon, lono, "grid2 center lon");
-    gridInqYunits(gridIDo, units);
-    grid_to_degree(units, out_nlat, lato, "grid2 center lat");
-  }
+  // Convert lat/lon units if required
+  cdo_grid_to_degree(gridIDo, CDI_XAXIS, out_nlon, lono, "grid2 center lon");
+  cdo_grid_to_degree(gridIDo, CDI_YAXIS, out_nlat, lato, "grid2 center lat");
 
   for (i = 0; i < out_nlon - 1; i++)
     if (lono[i + 1] <= lono[i]) break;
@@ -605,14 +520,12 @@ interpolate(Field &field1, Field &field2)
   if (lato[out_nlat] < -90) lato[out_nlat] = -99;
   if (lato[out_nlat] > 90) lato[out_nlat] = 99;
 
-  nxlon = 2 * nlon + 1;
-  nxlat = 2 * nlat + 1;
-  xin_array = (double *) Malloc(nxlon * nxlat * sizeof(double));
-  xin = (double **) Malloc(nxlat * sizeof(double *));
-
-  for (ilat = 0; ilat < nxlat; ilat++) xin[ilat] = xin_array + ilat * nxlon;
-
-  std::vector<double> xlon(nxlon);
+  const auto nxlon = 2 * nlon + 1;
+  const auto nxlat = 2 * nlat + 1;
+  Varray<double> xin_array(nxlon * nxlat);
+  MatrixView<double> xin(xin_array.data(), nxlat, nxlon);
+  
+  Varray<double> xlon(nxlon);
   for (ilon = 0; ilon < nlon; ilon++)
     {
       xlon[2 * ilon + 1] = lon[ilon];
@@ -620,7 +533,7 @@ interpolate(Field &field1, Field &field2)
     }
   xlon[2 * nlon] = (lon[nlon - 1] + lon[nlon]) / 2;
 
-  std::vector<double> xlat(2 * nlat + 1);
+  Varray<double> xlat(2 * nlat + 1);
   for (ilat = 0; ilat < nlat; ilat++)
     {
       xlat[2 * ilat + 1] = lat[ilat];
@@ -628,25 +541,18 @@ interpolate(Field &field1, Field &field2)
     }
   xlat[2 * nlat] = (lat[nlat - 1] + lat[nlat]) / 2;
 
-  in0 = (double **) Malloc(nlat * sizeof(double *));
-  for (ilat = 0; ilat < nlat; ilat++) in0[ilat] = arrayIn + ilat * nlon;
+  MatrixView<double> in0(arrayIn.data(), nlat, nlon);
 
-  std::vector<long> ilon11(out_nlon);
-  std::vector<long> ilon12(out_nlon);
-  std::vector<long> ilon21(out_nlon);
-  std::vector<long> ilon22(out_nlon);
-  std::vector<double> volon11(out_nlon);
-  std::vector<double> volon12(out_nlon);
-  std::vector<double> volon21(out_nlon);
-  std::vector<double> volon22(out_nlon);
+  Varray<long> ilon11(out_nlon), ilon12(out_nlon), ilon21(out_nlon), ilon22(out_nlon);
+  Varray<double> volon11(out_nlon), volon12(out_nlon), volon21(out_nlon), volon22(out_nlon);
 
   for (olon = 0; olon < out_nlon; olon++)
     {
       volon1 = (lono[olon - 1] + lono[olon]) / 2;
       volon2 = (lono[olon] + lono[olon + 1]) / 2;
       if (IS_EQUAL(volon1, volon2)) volon2 += 360;
-      volon2 -= 360 * floor((volon1 - xlon[0]) / 360);
-      volon1 -= 360 * floor((volon1 - xlon[0]) / 360);
+      volon2 -= 360 * std::floor((volon1 - xlon[0]) / 360);
+      volon1 -= 360 * std::floor((volon1 - xlon[0]) / 360);
       volon21[olon] = volon1;
       volon22[olon] = volon2;
       for (l21 = 0; l21 < nxlon && xlon[l21] < volon1; l21++)
@@ -667,8 +573,7 @@ interpolate(Field &field1, Field &field2)
       ilon22[olon] = l22;
     }
 
-  std::vector<long> ilat1(out_nlat);
-  std::vector<long> ilat2(out_nlat);
+  Varray<long> ilat1(out_nlat), ilat2(out_nlat);
 
   xlat_is_ascending = xlat[0] <= xlat[nxlat - 1];
   for (olat = 0; olat < out_nlat; olat++)
@@ -714,8 +619,7 @@ interpolate(Field &field1, Field &field2)
       ilat2[olat] = l2;
     }
 
-  xout = (double **) Malloc(out_nlat * sizeof(double *));
-  for (olat = 0; olat < out_nlat; olat++) xout[olat] = arrayOut + olat * out_nlon;
+  MatrixView<double> xout(arrayOut.data(), out_nlat, out_nlon);
 
   wrap_around = nlon > 1 && (lon[nlon - 1] >= lon[-1] + 360 - 0.001 || lon[nlon] >= lon[0] + 360 - 0.001);
 
@@ -725,7 +629,7 @@ interpolate(Field &field1, Field &field2)
   for (ilat = 0; ilat < nxlat; ilat += 2)
     for (ilon = 1; ilon < nxlon; ilon += 2)
       {
-        sum = 0;
+        double sum = 0.0;
         n = 0;
         if (ilat > 0 && !DBL_IS_EQUAL(xin[ilat - 1][ilon], missval))
           {
@@ -743,7 +647,7 @@ interpolate(Field &field1, Field &field2)
   for (ilat = 1; ilat < nxlat; ilat += 2)
     for (ilon = 0; ilon < nxlon; ilon += 2)
       {
-        sum = 0;
+        double sum = 0.0;
         n = 0;
         if (ilon > 0 && !DBL_IS_EQUAL(xin[ilat][ilon - 1], missval))
           {
@@ -771,7 +675,7 @@ interpolate(Field &field1, Field &field2)
   for (ilat = 0; ilat < nxlat; ilat += 2)
     for (ilon = 0; ilon < nxlon; ilon += 2)
       {
-        sum = 0;
+        double sum = 0.0;
         n = 0;
         if (ilon > 0 && !DBL_IS_EQUAL(xin[ilat][ilon - 1], missval))
           {
@@ -821,8 +725,8 @@ interpolate(Field &field1, Field &field2)
 
       for (olon = 0; olon < out_nlon; olon++)
         {
-          sum = 0;
-          wsum = 0;
+          double sum = 0.0;
+          double wsum = 0.0;
           for (k = 0; k < 2; k++)
             {
               if (k == 0)
@@ -843,8 +747,8 @@ interpolate(Field &field1, Field &field2)
               for (ilon = ilon1; ilon <= ilon2; ilon++)
                 {
                   if (ilon == 0 || ilon == nxlon) continue;
-                  vilon1 = xlon[ilon - 1];
-                  vilon2 = xlon[ilon];
+                  const auto vilon1 = xlon[ilon - 1];
+                  const auto vilon2 = xlon[ilon];
                   for (ilat = ilat1[olat]; ilat <= ilat2[olat]; ilat++)
                     {
                       if (ilat == 0 || ilat == nxlat) continue;
@@ -905,9 +809,9 @@ interpolate(Field &field1, Field &field2)
                           b21 = a11 + (a12 - a11) * faclon1 + (a21 - a11) * faclat2 + (a22 - a12 - a21 + a11) * faclon1 * faclat2;
                           b22 = a11 + (a12 - a11) * faclon2 + (a21 - a11) * faclat2 + (a22 - a12 - a21 + a11) * faclon2 * faclat2;
                         }
-                      wsum += (vlon2 - vlon1) * (sin(vlat2) - sin(vlat1));
-                      t = 2 * sin((vlat2 + vlat1) / 2) * sin((vlat2 - vlat1) / 2) / (vlat2 - vlat1);
-                      sum += (vlon2 - vlon1) / 2 * ((b11 + b12) * (t - sin(vlat1)) + (b21 + b22) * (sin(vlat2) - t));
+                      wsum += (vlon2 - vlon1) * (std::sin(vlat2) - std::sin(vlat1));
+                      const auto t = 2.0 * std::sin((vlat2 + vlat1) / 2.0) * std::sin((vlat2 - vlat1) / 2.0) / (vlat2 - vlat1);
+                      sum += (vlon2 - vlon1) / 2 * ((b11 + b12) * (t - std::sin(vlat1)) + (b21 + b22) * (std::sin(vlat2) - t));
                     }
                 }
             }
@@ -915,14 +819,5 @@ interpolate(Field &field1, Field &field2)
         }
     }
 
-  field2.nmiss = arrayNumMV(gridsize_o, arrayOut, missval);
-
-  Free(lon_array);
-  Free(lat_array);
-  Free(lono_array);
-  Free(lato_array);
-  Free(xin);
-  Free(xin_array);
-  Free(in0);
-  Free(xout);
+  field2.nmiss = varrayNumMV(gridsize_o, arrayOut, missval);
 }
