@@ -1412,14 +1412,11 @@ static size_t remove_zero_length_edges(struct point_list * list) {
               curr->vec_coords, curr->next->vec_coords),
            SIN_COS_LOW_TOL) <= 0)) {
       curr->to_be_removed = 1;
-    } else {
-      double temp_a = atan2(curr->vec_coords[1], curr->vec_coords[0]);
-      double temp_b = atan2(curr->next->next->vec_coords[1],
-                            curr->next->next->vec_coords[0]);
-      if (curr->edge_type == LAT_CIRCLE &&
-          curr->next->edge_type == LAT_CIRCLE &&
-          (fabs(get_angle(temp_a, temp_b)) < M_PI_2))
-      curr->next->to_be_removed = 1;
+    } else if (curr->edge_type == LAT_CIRCLE && curr->next->edge_type == LAT_CIRCLE) {
+        double temp_a = atan2(curr->vec_coords[1], curr->vec_coords[0]);
+        double temp_b = atan2(curr->next->next->vec_coords[1], curr->next->next->vec_coords[0]);
+        if (fabs(get_angle(temp_a, temp_b)) < M_PI_2)
+          curr->next->to_be_removed = 1;
     }
 
     curr = curr->next;
