@@ -40,7 +40,7 @@ yac_realloc_array(void **array, size_t elem_size, size_t *curr_array_size,
   *curr_array_size = array_inc_size
     * ((requested_size + array_inc_size) / array_inc_size);
   *array = realloc(*array, *curr_array_size * elem_size);
-  if (*array == NULL && (*curr_array_size * elem_size)) {
+  if ((*array == NULL) && ((*curr_array_size * elem_size) != 0)) {
      fprintf(stderr, "error in realloc\n");
      exit(EXIT_FAILURE);
   }
