@@ -62,28 +62,24 @@
 #define M_SQRT3_4 (0.866025403784438646764) /* sqrt(0.75) */
 #endif
 
+// angle tolerance
+
+#define yac_angle_tol         (1e-9)
+#define yac_cos_angle_tol     (0.9999999999999999995)
+#define yac_angle_low_tol     (1e-11)
+#define yac_cos_angle_low_tol (1.0)
+
 struct sin_cos_angle {
   double sin, cos;
 };
 
-#define SIN_COS_M_PI_2 ((struct sin_cos_angle){.sin = 1.0, .cos = 0.0}) /* PI/2 */
-#define SIN_COS_M_PI ((struct sin_cos_angle){.sin = 0.0, .cos = -1.0}) /* PI */
-#define SIN_COS_7_M_PI_4 \
-  ((struct sin_cos_angle){.sin = -0.707106781186547524401, \
-                          .cos = +0.707106781186547524401}) /* (7 * PI)/4 */
-//#define SIN_COS_ZERO ((struct sin_cos_angle){.sin = 0.0, .cos = 1.0}) /* 0.0 */
-static const struct sin_cos_angle SIN_COS_ZERO = {0.0, 1.0}; /* 0.0 */
-#define SIN_COS_TOL \
-  ((struct sin_cos_angle){.sin = yac_angle_tol, \
-                          .cos = yac_cos_angle_tol}) /* yac_angle_tol */
-#define SIN_COS_LOW_TOL \
-  ((struct sin_cos_angle){.sin = yac_angle_low_tol, \
-                          .cos = yac_cos_angle_low_tol}) /* yac_angle_low_tol */
+static const struct sin_cos_angle SIN_COS_ZERO = {0.0, 1.0};
+static const struct sin_cos_angle SIN_COS_TOL = {yac_angle_tol, yac_cos_angle_tol};
+static const struct sin_cos_angle SIN_COS_LOW_TOL = {yac_angle_low_tol, yac_cos_angle_low_tol};
 
-extern const double yac_angle_tol;
-extern const double yac_cos_angle_tol;
-extern const double yac_angle_low_tol;
-extern const double yac_cos_angle_low_tol;
+static const struct sin_cos_angle SIN_COS_M_PI_2 = {1.0, 0.0}; /* PI/2 */
+static const struct sin_cos_angle SIN_COS_M_PI = {0.0, -1.0}; /* PI */
+static const struct sin_cos_angle SIN_COS_7_M_PI_4 = {-0.707106781186547524401, +0.707106781186547524401}; /* (7 * PI)/4 */
 
 struct line {
    struct {
@@ -712,17 +708,16 @@ static inline double get_vector_angle(double a[3], double b[3]) {
    */
 }
 
+static inline double clamp_abs_one(double val) {
+  return val > -1.0 ? (val < 1.0 ? val : 1.0) : -1.0;
+}
+
 static inline struct sin_cos_angle sin_cos_angle_new(double sin, double cos) {
 
   struct sin_cos_angle angle;
 
-  if (fabs(sin) <= 1.0) angle.sin = sin;
-  else if (sin < 0.0) angle.sin = -1.0;
-  else angle.sin = 1.0;
-
-  if (fabs(cos) <= 1.0) angle.cos = cos;
-  else if (cos < 0.0) angle.cos = -1.0;
-  else angle.cos = 1.0;
+  angle.sin = clamp_abs_one(sin);
+  angle.cos = clamp_abs_one(cos);
 
   return angle;
 }
