@@ -47,6 +47,7 @@
 #include "dep_list.h"
 #include "math.h"
 #include "grid_cell.h"
+#include "communicator.h"
 
 #ifndef  M_PI
 #define  M_PI        3.14159265358979323846264338327950288  /* pi */
@@ -56,17 +57,20 @@
 #define  M_PI_2      1.57079632679489661923132169163975144  /* pi/2 */
 #endif
 
-#define EARTH_RADIUS 6371.2290
-
-static double const EarthRadius  = EARTH_RADIUS;
-static double const rad          = M_PI / 180.0;
-static double const deg          = 180.0 / M_PI;
-static double const EarthRadius2 = EARTH_RADIUS * EARTH_RADIUS / 2.0 ;
+#define YAC_EARTH_RADIUS (6371.2290)
+#define YAC_EARTH_RADIUS2 ((6371.2290 * 6371.2290) * 0.5)
+#define YAC_RAD (0.01745329251994329576923690768489) // M_PI / 180
 
 // forward declaration required by grid_vtable
 struct grid;
 struct bounding_circle;
 
+enum yac_grid_type {
+  YAC_REG2D,
+  YAC_CURVE2D,
+  YAC_UNSTRUCT,
+};
+
 struct grid_vtable {
 
    struct grid * (*copy)(struct grid *);
@@ -102,14 +106,14 @@ struct grid_vtable {
    unsigned (*get_corner_x_coord_index) (struct grid *, unsigned);
    unsigned (*get_corner_y_coord_index) (struct grid *, unsigned);
    int (*get_aux_grid_cell)(struct grid *, unsigned, unsigned *, enum yac_edge_type *);
-   struct dep_list (*get_cell_neigh_dep_list)(struct grid *);
+   struct dep_list * (*get_cell_neigh_dep_list)(struct grid *);
    void (*get_boundary_corners) (struct grid *, unsigned *, unsigned *);
    struct grid * (*generate_cell_grid)(struct grid *, double *, double *);
    struct grid * (*generate_edge_grid)(struct grid *, double *, double *);
    struct grid * (*generate_subgrid)(struct grid *, unsigned *, unsigned,
                                      unsigned **, unsigned **, unsigned **);
-   void (*pack_grid)(struct grid *, double **, unsigned, unsigned *, unsigned *,
-                     unsigned **, unsigned, unsigned *, unsigned *);
+   void (*pack_grid)(struct grid *, void *, int, int *, struct communicator *);
+   int (*grid_pack_size)(struct grid *, struct communicator *);
    struct grid_search * (*get_grid_search)(struct grid * grid);
    int (*cell_contains_non_gc_edge)(struct grid *, unsigned);
    void (*grid_delete)(struct grid *);
@@ -118,6 +122,7 @@ struct grid_vtable {
 struct grid {
 
    struct grid_vtable *vtable;
+   enum yac_grid_type grid_type;
 };
 
 /**
@@ -379,7 +384,7 @@ int yac_get_aux_grid_cell(struct grid * grid, unsigned corner_index,
  * @param[in] grid
  * @return dependency list that contains for each local cell id the local ids of all neighbour cells
  */
-struct dep_list yac_get_cell_neigh_dep_list(struct grid * grid);
+struct dep_list * yac_get_cell_neigh_dep_list(struct grid * grid);
 
 /**
  * makes a complete copy of a grid
@@ -448,36 +453,42 @@ struct grid * yac_generate_subgrid(struct grid * grid, unsigned * selected_local
                                    unsigned ** local_corner_ids, unsigned ** local_edge_ids);
 
 /**
- * packs given grid into a double and unsigned buffer
- * @param[in] grid
- * @param[in,out] dble_buf pointer to double buffer (buffer is reallocated by this routine if necessary)
- * @param[in] dble_buf_offset number of elements already in the buffer (this data is not overwritten by this routine)
- * @param[out] dble_buf_data_size number of elements added to the buffer by this routine
- * @param[in,out] dble_buf_size size of dble_buf (this is adjusted in case this routine reallocates dble_buf)
- * @param[in,out] uint_buf pointer to double buffer (buffer is reallocated by this routine if necessary)
- * @param[in] uint_buf_offset number of elements already in the buffer (this data is not overwritten by this routine)
- * @param[out] uint_buf_data_size number of elements added to the buffer by this routine
- * @param[in,out] uint_buf_size size of uint_buf (this is adjusted in case this routine reallocates uint_buf)
+ * packs given grid into a buffer
+ * @param[in]     grid
+ * @param[in,out] buffer      pointer to buffer into which the grid will be
+ *                            packed
+ * @param[in,out] buffer_size size of the pack buffer
+ * @param[in,out] position    current write position in the buffer
+ * @param[in]     comm
  * @see \ref yac_unpack_grid
+ * @see \ref yac_grid_pack_size
+ * \remark the size of the buffer provided must be sufficiently large to hold
+ *         the packed grid
  */
-void yac_pack_grid(struct grid * grid, double ** dble_buf,
-                   unsigned dble_buf_offset, unsigned * dble_buf_data_size,
-                   unsigned * dble_buf_size, unsigned ** uint_buf,
-                   unsigned uint_buf_offset, unsigned * uint_buf_data_size,
-                   unsigned * uint_buf_size);
+void yac_pack_grid(struct grid * grid, void * buffer, int buffer_size,
+                   int * position, struct communicator * comm);
 
 /**
- * unpacks a grid from the provided double and unsigned buffers; the unpacked grid is identical
- * to the one packed into these buffers by \ref yac_pack_grid
- * @param[in] dble_buf
- * @param[out] dble_buf_data_size number of elements extracted from dble_buf
- * @param[in] uint_buf
- * @param[out] uint_buf_data_size number of elements extracted from uint_buf
+ * unpacks a grid from the provided buffer
+ * @param[in]     buffer      unpack buffer
+ * @param[in]     buffer_size size of unpack buffer
+ * @param[in,out] position    current read position in the buffer
+ * @param[in]     comm
  * @returns unpacked grid
  * @see \ref yac_pack_grid
  */
-struct grid * yac_unpack_grid(double * dble_buf, unsigned * dble_buf_data_size,
-                              unsigned * uint_buf, unsigned * uint_buf_data_size);
+struct grid * yac_unpack_grid(void * buffer, int buffer_size, int * position,
+                              struct communicator * comm);
+
+/**
+ * computes the number of bytes required in a pack buffer by
+ * \ref yac_pack_grid to pack the provided grid
+ * @param[in] grid
+ * @param[in] comm
+ * @return number of bytes required for packing
+ * @see \ref yac_pack_grid
+ */
+int yac_grid_pack_size(struct grid * grid, struct communicator * comm);
 
 /**
  * generates a grid search object for the given grid
@@ -506,6 +517,5 @@ int yac_grid_cell_contains_non_gc_edge(
  */
 void yac_delete_grid(struct grid * grid);
 
-#undef EARTH_RADIUS
-
 #endif // GRID_H
+
