@@ -36,6 +36,7 @@
 #include "dep_list.h"
 #include "grid_unstruct.h"
 #include "utils.h"
+#include "grid_reg2d_common.h"
 
 int yac_get_aux_grid_cell_reg2d_common(
   unsigned num_cells[2], unsigned cyclic, unsigned corner_index,
@@ -101,12 +102,12 @@ unsigned yac_get_num_corner_edges_reg2d_common(
   return num_edges;
 }
 
-void yac_generate_cell_neigh_dep_reg2d_common(
-  struct dep_list * cell_to_neigh, unsigned num_cells[2], unsigned cyclic[2]) {
+struct dep_list * yac_generate_cell_neigh_dep_reg2d_common(
+   unsigned num_cells[2], unsigned cyclic[2]) {
 
    unsigned total_num_neighs;
    unsigned num_grid_cells;
-   unsigned * num_neigh_per_cell;
+   size_t * num_neigh_per_cell;
    unsigned * cell_neigh_dependencies;
    unsigned * curr_cell_neigh_dep;
    unsigned curr_cell_index;
@@ -279,9 +280,11 @@ void yac_generate_cell_neigh_dep_reg2d_common(
       }
    }
 
-   yac_set_dependencies(
-      cell_to_neigh, num_grid_cells, num_neigh_per_cell,
-      cell_neigh_dependencies);
+   struct dep_list * cell_to_neigh =
+      yac_dep_list_new(
+         (size_t)num_grid_cells, num_neigh_per_cell, cell_neigh_dependencies);
+
+   return cell_to_neigh;
 }
 
 void yac_get_boundary_corners_reg2d_common(
@@ -605,7 +608,7 @@ struct grid * yac_generate_edge_grid_reg2d_common(
   unsigned total_num_corners_per_edge_grid_cell =
     num_edge_grid_cells * 4;
 
-  unsigned * num_corners_per_edge_grid_cell =
+  size_t * num_corners_per_edge_grid_cell =
     malloc(num_edge_grid_cells * sizeof(*num_corners_per_edge_grid_cell));
   unsigned * corners_of_edge_grid_cells =
     malloc(total_num_corners_per_edge_grid_cell *
@@ -652,10 +655,10 @@ struct grid * yac_generate_edge_grid_reg2d_common(
           4 * sizeof(*corners_of_edge_grid_cells));
   }
 
-  struct dep_list edge_grid_cell_to_vertex;
-  yac_set_dependencies(
-    &edge_grid_cell_to_vertex, num_edge_grid_cells,
-    num_corners_per_edge_grid_cell, corners_of_edge_grid_cells);
+  struct dep_list * edge_grid_cell_to_vertex =
+    yac_dep_list_new(
+      (size_t)num_edge_grid_cells, num_corners_per_edge_grid_cell,
+      corners_of_edge_grid_cells);
 
   unsigned num_edge_grid_edges =
     (num_cells[0] + ((cyclic[0])?0:1)) * num_cells[1] +
@@ -669,3 +672,4 @@ struct grid * yac_generate_edge_grid_reg2d_common(
 
   return grid;
 }
+
