@@ -127,6 +127,7 @@ struct scrip_grid {
 
 static unsigned get_num_cell_corners_scrip (struct grid * grid, unsigned cell_index) {
 
+   (void) cell_index;
    struct scrip_grid * scrip_grid = (struct scrip_grid *)grid;
 
    return scrip_grid->max_corners;
@@ -285,7 +286,8 @@ static struct grid_search * get_grid_search_scrip(struct grid * grid) {
    return scrip_grid->grid_search;
 }
 
-static int cell_contains_non_gc_edge_scrip(
-  struct grid * grid, unsigned local_cell_id) {
+static int cell_contains_non_gc_edge_scrip(struct grid * grid, unsigned local_cell_id) {
+  (void)grid;
+  (void)local_cell_id;
   return 0;
 }
