@@ -42,11 +42,7 @@
 #include "geometry.h"
 
 // angle tolerance
-const double yac_angle_tol = 1e-9;
-const double yac_cos_angle_tol = 0.9999999999999999995; // cos(yac_angle_tol)
                                                         // this is actually 1.0
-const double yac_angle_low_tol = 1e-11;
-const double yac_cos_angle_low_tol = 1.0;
 static double const tol = 1.0e-10;
 
 //! computes square of the lenght of the vector ab
