@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,14 +33,14 @@ double, int, double *, int *);
 #ifdef HAVE_CF_INTERFACE
 
 PROTOCCALLSFSUB10(DDVFIDF, ddvfidf, DOUBLEV, DOUBLEV, DOUBLEV, DOUBLEV, INT, INT, DOUBLE, INT, DOUBLEV, PINT)
-#define DDVFIDF(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10)                                                                       \
-  CCALLSFSUB10(DDVFIDF, ddvfidf, DOUBLEV, DOUBLEV, DOUBLEV, DOUBLEV, INT, INT, DOUBLE, INT, DOUBLEV, PINT, A1, A2, A3, A4, A5, \
-               A6, A7, A8, A9, A10)
+#define DDVFIDF(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10)                                                                           \
+  CCALLSFSUB10(DDVFIDF, ddvfidf, DOUBLEV, DOUBLEV, DOUBLEV, DOUBLEV, INT, INT, DOUBLE, INT, DOUBLEV, PINT, A1, A2, A3, A4, A5, A6, \
+               A7, A8, A9, A10)
 
 PROTOCCALLSFSUB10(DVRFIDF, dvrfidf, DOUBLEV, DOUBLEV, DOUBLEV, DOUBLEV, INT, INT, DOUBLE, INT, DOUBLEV, PINT)
-#define DVRFIDF(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10)                                                                       \
-  CCALLSFSUB10(DVRFIDF, dvrfidf, DOUBLEV, DOUBLEV, DOUBLEV, DOUBLEV, INT, INT, DOUBLE, INT, DOUBLEV, PINT, A1, A2, A3, A4, A5, \
-               A6, A7, A8, A9, A10)
+#define DVRFIDF(A1, A2, A3, A4, A5, A6, A7, A8, A9, A10)                                                                           \
+  CCALLSFSUB10(DVRFIDF, dvrfidf, DOUBLEV, DOUBLEV, DOUBLEV, DOUBLEV, INT, INT, DOUBLE, INT, DOUBLEV, PINT, A1, A2, A3, A4, A5, A6, \
+               A7, A8, A9, A10)
 
 #endif
 
