@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,32 +24,33 @@
 
 class ListBuffer
 {
- public:
+public:
   std::vector<char> buffer;
   std::string name;
 
   ListBuffer() {}
   ~ListBuffer() {}
 
-  int read(FILE *fp, const char *name)
+  int
+  read(FILE *fp, const char *_name)
   {
     const int filedes = fileno(fp);
     struct stat buf;
     size_t filesize = (fstat(filedes, &buf) == 0) ? (size_t) buf.st_size : 0;
     if (filesize == 0)
       {
-        fprintf(stderr, "ListBuffer: empty stream: %s\n", name);
+        fprintf(stderr, "ListBuffer: empty stream: %s\n", _name);
         return -1;
       }
 
     this->buffer.resize(filesize);
     if (fread(buffer.data(), 1, filesize, fp) != filesize)
       {
-        fprintf(stderr, "ListBuffer: read failed on %s!\n", name);
+        fprintf(stderr, "ListBuffer: read failed on %s!\n", _name);
         return -1;
       }
 
-    if (name) this->name = name;
+    if (_name) this->name = _name;
 
     return 0;
   }
