@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,9 +18,10 @@
 #include "config.h" /* HAVE_LIBMAGICS */
 #endif
 
-#include "cdo_int.h"
+#include "cdo_output.h"
+#include "compare.h"
 #include "magics_template_parser.h"
-#include "StringUtilities.h"
+#include "string_utilities.h"
 
 #ifdef HAVE_LIBXML2
 #include <libxml/parser.h>
@@ -39,10 +40,10 @@ extern void *magics_node;
 
 /* Recursive function that sets the Magics parameters from the XML structure */
 
+#ifdef HAVE_LIBXML2
 int
 magics_template_parser(void *node)
 {
-#ifdef HAVE_LIBXML2
   xmlNode *a_node = (xmlNode *) node;
   int param_set_flag;
   xmlNode *cur_node = nullptr;
@@ -54,7 +55,7 @@ magics_template_parser(void *node)
     fprintf( stdout,"Parsing the magics Node \n");
 #endif
 
-  if (cstrIsEqual((const char *) a_node->name, "magics"))
+  if (cdo_cmpstr((const char *) a_node->name, "magics"))
     {
       const char *value = (const char *) xmlGetProp(a_node, (const xmlChar *) "version");
 
@@ -107,14 +108,20 @@ magics_template_parser(void *node)
             }
         }
     }
-#else
-
-  cdoAbort("XML2 support not compiled in!");
-
-#endif
 
   return 0;
 }
+#else
+
+int
+magics_template_parser(void *node)
+{
+  (void) node;
+  cdoAbort("XML2 support not compiled in!");
+  return 0;
+}
+
+#endif
 
 #ifdef HAVE_LIBMAGICS
 int
@@ -131,7 +138,6 @@ SetMagicsParameterValue(const char *, const char *, const char *)
   char **split_str = nullptr;
   const char *sep_char = ",";
   const char *search_char = ";";
-  double *float_param_list = nullptr;
 
   if (param_name == nullptr)
     {
@@ -142,11 +148,11 @@ SetMagicsParameterValue(const char *, const char *, const char *)
   if (param_value == nullptr) ret_flag = 2;
 
   /*   MAGICS++ ENV RELATED PARAMETERS   */
-  if (cstrIsEqual(param_type, "environvar"))
+  if (cdo_cmpstr(param_type, "environvar"))
     {
-      if (cstrIsEqual(param_name, "quiet_option"))
+      if (cdo_cmpstr(param_name, "quiet_option"))
         {
-          if (cstrIsEqual(param_value, "off") || cstrIsEqual(param_value, "OFF"))
+          if (cdo_cmpstr(param_value, "off") || cdo_cmpstr(param_value, "OFF"))
             {
 #if 0
 				printf( "Quiet Option %s \n", param_value );
@@ -159,7 +165,7 @@ SetMagicsParameterValue(const char *, const char *, const char *)
                 fprintf(stderr, "Quiet Option %s COULDN'T be UNSET!!!\n", param_value);
             }
 
-          if (cstrIsEqual(param_value, "on") || cstrIsEqual(param_value, "ON"))
+          if (cdo_cmpstr(param_value, "on") || cdo_cmpstr(param_value, "ON"))
             {
 #if 0
 				printf( "Quiet Option %s \n", param_value );
@@ -175,13 +181,13 @@ SetMagicsParameterValue(const char *, const char *, const char *)
     }
 
   /*   MAGICS++ FLOAT TYPE PARAMETERS   */
-  else if (cstrIsEqual(param_type, "float"))
+  else if (cdo_cmpstr(param_type, "float"))
     {
       mag_setr(param_name, atof(param_value));
     }
 
   /*   MAGICS++ FLOAT ARRAY  TYPE    PARAMETERS   */
-  else if (cstrIsEqual(param_type, "floatarray"))
+  else if (cdo_cmpstr(param_type, "floatarray"))
     {
 
 #if 0
@@ -191,52 +197,50 @@ SetMagicsParameterValue(const char *, const char *, const char *)
       split_str_count = cstrSplitWithSeperator(param_value, sep_char, &split_str);
       if (split_str_count)
         {
-          float_param_list = (double *) Malloc(sizeof(double) * split_str_count);
+          std::vector<double> float_param_list(split_str_count);
           for (i = 0; i < split_str_count; i++)
             {
 #if 0
-			        fprintf(stderr, "%d %d %s\n", i, split_str_count, split_str[i]);
+	      fprintf(stderr, "%d %d %s\n", i, split_str_count, split_str[i]);
 #endif
               float_param_list[i] = atof(split_str[i]);
             }
-          mag_set1r(param_name, float_param_list, split_str_count);
-          Free(float_param_list);
-          Free(split_str);
+          mag_set1r(param_name, float_param_list.data(), split_str_count);
+          free(split_str);
         }
     }
 
   /*   MAGICS++ INT TYPE    PARAMETERS   */
-  else if (cstrIsEqual(param_type, "int"))
+  else if (cdo_cmpstr(param_type, "int"))
     {
       mag_seti(param_name, atoi(param_value));
     }
 
   /*   MAGICS++ INT ARRAY  TYPE    PARAMETERS   */
-  else if (cstrIsEqual(param_type, "intarray"))
+  else if (cdo_cmpstr(param_type, "intarray"))
     {
       if (strchr(param_value, ';')) sep_char = ";";
       split_str_count = cstrSplitWithSeperator(param_value, sep_char, &split_str);
       if (split_str_count)
         {
-          int *int_param_list = (int *) Malloc(sizeof(int) * split_str_count);
+          std::vector<int> int_param_list(split_str_count);
           for (i = 0; i < split_str_count; i++)
             {
               int_param_list[i] = atoi(split_str[i]);
             }
-          mag_set1i(param_name, int_param_list, split_str_count);
-          Free(int_param_list);
-          Free(split_str);
+          mag_set1i(param_name, int_param_list.data(), split_str_count);
+          free(split_str);
         }
     }
 
   /*   MAGICS++ STRING TYPE    PARAMETERS   */
-  else if (cstrIsEqual(param_type, "string"))
+  else if (cdo_cmpstr(param_type, "string"))
     {
       mag_setc(param_name, param_value);
     }
 
   /*   MAGICS++ STRINGARRAY  TYPE    PARAMETERS   */
-  else if (cstrIsEqual(param_type, "stringarray"))
+  else if (cdo_cmpstr(param_type, "stringarray"))
     {
       if (DBG) fprintf(stderr, "Input strarr is %s  Sep char is %s Search char is %s\n", param_value, sep_char, search_char);
       if (strstr(param_value, ";"))
@@ -250,7 +254,7 @@ SetMagicsParameterValue(const char *, const char *, const char *)
       if (DBG) fprintf(stderr, "Input strarr is %s split str count is %d Sep char is %s\n", param_value, split_str_count, sep_char);
 
       mag_set1c(param_name, (const char **) split_str, split_str_count);
-      Free(split_str);
+      free(split_str);
     }
   else
     {
