@@ -6,7 +6,7 @@
 template <class T, int R = 0, int C = 0>
 class // MatrixView
 #ifdef __GNUG__
-__attribute__((warn_unused))
+[[gnu::warn_unused]]
 #endif
 MatrixView
 {
