@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,6 @@
   GNU General Public License for more details.
 */
 #include <cdi.h>
-#include <signal.h>
 
 #include "process_int.h"
 #include "param_conversion.h"
@@ -43,22 +42,23 @@ MergeVarsOnAxis::check_axissize_consistency(std::vector<int> axissize)
       if (axissize[0] != (int) gridInqXsize(this->inputKeys[i].gridID))
         cdoAbort("ERROR (infile: '%s')! In merging variables to a variable with a character coordinate:\n          Size of x-axis: "
                  "'%d' of variable '%s'\n          differ from x-axis size of variable '%s': '%d'.",
-                 cdoGetStreamName(0), gridInqXsize(this->inputKeys[i].gridID), this->inputNames.values[i].c_str(), this->inputNames.values[0].c_str(), axissize[0]);
+                 cdoGetStreamName(0), gridInqXsize(this->inputKeys[i].gridID), this->inputNames.values[i].c_str(),
+                 this->inputNames.values[0].c_str(), axissize[0]);
       if (axissize[1] != (int) gridInqYsize(this->inputKeys[i].gridID))
         cdoAbort("ERROR (infile: '%s')! In merging variables to a variable with a character coordinate:\n          Size of y-axis: "
                  "'%d' of variable '%s'\n          differ from y-axis size of variable '%s': '%d'.",
-                 cdoGetStreamName(0), gridInqYsize(this->inputKeys[i].gridID), this->inputNames.values[i].c_str(), this->inputNames.values[0].c_str(), axissize[1]);
+                 cdoGetStreamName(0), gridInqYsize(this->inputKeys[i].gridID), this->inputNames.values[i].c_str(),
+                 this->inputNames.values[0].c_str(), axissize[1]);
       if (axissize[2] != zaxisInqSize(this->inputKeys[i].zaxisID))
         cdoAbort("ERROR (infile: '%s')! In merging variables to a variable with a character coordinate:\n          Size of z-axis: "
                  "'%d' of variable '%s'\n          differ from z-axis size of variable '%s': '%d'.",
-                 cdoGetStreamName(0), zaxisInqSize(this->inputKeys[i].zaxisID), this->inputNames.values[i].c_str(), this->inputNames.values[0].c_str(), axissize[2]);
+                 cdoGetStreamName(0), zaxisInqSize(this->inputKeys[i].zaxisID), this->inputNames.values[i].c_str(),
+                 this->inputNames.values[0].c_str(), axissize[2]);
     }
 }
 
-/** The next function will define
-*** output.gridID, output.zaxisID
-*** and reset
-*** axissize
+/** The next function will define output.gridID, output.zaxisID
+*** and reset axissize
 ***
 *** The axis that has size=1 will be used as the merge axis
 */
@@ -100,7 +100,7 @@ MergeVarsOnAxis::define_new_axes(std::vector<int> axissize)
       zvals = subsvals;
       axissize[2] = this->inputNames.nvalues;
       int gridType = gridInqType(this->inputKeys[0].gridID);
-      if ( axissize[0] && axissize[1] )
+      if (axissize[0] && axissize[1])
         {
           xvals = (double *) Malloc(axissize[0] * sizeof(double));
           yvals = (double *) Malloc(axissize[1] * sizeof(double));
@@ -115,17 +115,19 @@ MergeVarsOnAxis::define_new_axes(std::vector<int> axissize)
 
   gridDefXsize(this->output.gridID, axissize[0]);
   gridDefYsize(this->output.gridID, axissize[1]);
-  if ( axissize[0] == 0 )
+  if (axissize[0] == 0)
     axissize[0] = 1;
   else
     gridDefXvals(this->output.gridID, xvals);
-  if ( axissize[1] == 0 )
+  if (axissize[1] == 0)
     axissize[1] = 1;
   else
     gridDefYvals(this->output.gridID, yvals);
   zaxisDefLevels(this->output.zaxisID, zvals);
-  Free(xvals); Free(yvals); Free(zvals);
-  std::vector<int> newaxissize = {axissize[0], axissize[1], axissize[2]};
+  Free(xvals);
+  Free(yvals);
+  Free(zvals);
+  std::vector<int> newaxissize = { axissize[0], axissize[1], axissize[2] };
   return newaxissize;
 }
 
@@ -141,12 +143,15 @@ MergeVarsOnAxis::define_var_structure(int vlistID, int ntsteps, std::vector<int>
 {
   this->output.vlistID = vlistID;
   this->output.varID = vlistDefVar(this->output.vlistID, this->output.gridID, this->output.zaxisID, TIME_VARYING);
-  vlistDefVarName(this->output.vlistID, this->inputKeys[0].varID, "ChangedForMap");
-  vlistDefVarName(this->output.vlistID, this->output.varID, this->inputNames.values[0].c_str());
-  int datatype = vlistInqVarDatatype(this->output.vlistID, this->inputKeys[0].varID);
+  int oldcode = vlistInqVarCode(this->output.vlistID, this->inputKeys[0].varID);
+  vlistDefVarCode(this->output.vlistID, this->inputKeys[0].varID, 1);
+  vlistDefVarCode(this->output.vlistID, this->output.varID, oldcode);
+  cdiDefKeyString(this->output.vlistID, this->inputKeys[0].varID, CDI_KEY_NAME, "ChangedForMap");
+  cdiDefKeyString(this->output.vlistID, this->output.varID, CDI_KEY_NAME, this->inputNames.values[0].c_str());
+  auto datatype = vlistInqVarDatatype(this->output.vlistID, this->inputKeys[0].varID);
   vlistDefVarDatatype(this->output.vlistID, this->output.varID, datatype);
-  const double mv = vlistInqVarMissval(this->output.vlistID, this->inputKeys[0].varID);
-  if ( datatype == CDI_DATATYPE_FLT64 )
+  const auto mv = vlistInqVarMissval(this->output.vlistID, this->inputKeys[0].varID);
+  if (datatype == CDI_DATATYPE_FLT64)
     {
       this->output.datatype = 'd';
       vlistDefVarMissval(this->output.vlistID, this->output.varID, mv);
@@ -168,11 +173,14 @@ MergeVarsOnAxis::define_var_structure(int vlistID, int ntsteps, std::vector<int>
 void
 MergeVarsOnAxis::read_cmor_charvar(std::vector<int> axissize, int streamID, int oldgridsize)
 {
-  double *buffer_old = (double *) Malloc(oldgridsize * sizeof(double));
+  Varray<double> buffer_old(oldgridsize);
 
-  int tsID = 0, nrecs = 0;
-  while ((nrecs = streamInqTimestep(streamID, tsID)))
+  int tsID = 0;
+  while (true)
     {
+      auto nrecs = streamInqTimestep(streamID, tsID);
+      if (nrecs == 0) break;
+
       while (nrecs--)
         {
           int varIDrw, levelIDrw;
@@ -181,31 +189,30 @@ MergeVarsOnAxis::read_cmor_charvar(std::vector<int> axissize, int streamID, int
           for (int i = 0; i < this->inputNames.nvalues; i++)
             if (varIDrw == this->inputKeys[i].varID)
               {
-                streamReadRecord(streamID, buffer_old, &nmiss);
+                streamReadRecord(streamID, buffer_old.data(), &nmiss);
                 int newIndex;
                 for (int j = 0; j < oldgridsize; j++)
                   {
-          // (lev x lat, basin )
+                    // (lev x lat, basin )
 
                     /* (lev x lat, basin )
                                 newIndex = j * levdim + levelID; */
-/* Use this index because z-axis is registered at the end and rearranged by CMOR */
+                    /* Use this index because z-axis is registered at the end and rearranged by CMOR */
                     if (oldgridsize == axissize[0] * axissize[1])
                       newIndex = tsID * axissize[2] * axissize[0] * axissize[1] + j * axissize[2] + i;
                     else if (axissize[0] == this->inputNames.nvalues)
-                      newIndex = tsID * axissize[2] * axissize[0] * axissize[1] + i * axissize[1] * axissize[2]
-                                 + j * axissize[2] + levelIDrw;
+                      newIndex = tsID * axissize[2] * axissize[0] * axissize[1] + i * axissize[1] * axissize[2] + j * axissize[2]
+                                 + levelIDrw;
                     else
                       newIndex = tsID * axissize[2] * axissize[0] * axissize[1] + levelIDrw * axissize[0] * axissize[1]
                                  + i * axissize[0] * axissize[1] / oldgridsize + j;
-                    if ( this->output.datatype == 'd' )
-                      ((double *) this->data)[newIndex] = (double) buffer_old[j]; 
+                    if (this->output.datatype == 'd')
+                      ((double *) this->data)[newIndex] = (double) buffer_old[j];
                     else
-                      ((float *) this->data)[newIndex] = (float) buffer_old[j]; 
+                      ((float *) this->data)[newIndex] = (float) buffer_old[j];
                   }
               }
         }
       tsID++;
     }
-  Free(buffer_old);
 }
