@@ -2,15 +2,13 @@
 #define MODULE_LIST_H
 
 #include "cdi.h"
+#include "modules.h"
 #include "module_definitions.h"
 #include "operator_help.h"
-#include "operator_definitions.h"
-
-#define OBASE -1 // Obase uses input name as base name for files e.g 'test' gets used as test_001 test_002 ...
-#define INTERNAL 0
-#define EXPOSED 1
 
 // clang-format off
+
+
 //                                             FuncName        Help                OperatorList               internal                    RestrictionType
 //                                             |               |                    |                         |          cdi-numbertype          |
 //                                             |               |                    |                         |          |         in   out      |
@@ -21,6 +19,7 @@ static const module_t module_Arith          = {Arith          , ArithHelp
 static const module_t module_Arithc         = {Arithc         , ArithcHelp        , ArithcOperators         , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Arithdays      = {Arithdays      , ArithdaysHelp     , ArithdaysOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Arithlat       = {Arithlat       , {}                , ArithlatOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Apply          = {nullptr        , ApplyHelp         , {"apply"}               , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Cat            = {Cat            , CopyHelp          , CatOperators            , EXPOSED  , CDI_REAL , -1 , 1     , NoRestriction };
 static const module_t module_CDItest        = {CDItest        , {}                , CDItestOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_CDIread        = {CDIread        , {}                , CDIreadOperators        , EXPOSED  , CDI_REAL , 1  , 0     , NoRestriction };
@@ -41,7 +40,7 @@ static const module_t module_Cond2          = {Cond2          , Cond2Help
 static const module_t module_Condc          = {Condc          , CondcHelp         , CondcOperators          , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Consecstat     = {Consecstat     , ConsecstatHelp    , ConsecstatOperators     , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Copy           = {Copy           , CopyHelp          , CopyOperators           , EXPOSED  , CDI_REAL , -1 , 1     , NoRestriction };
-static const module_t module_Deltat         = {Deltat         , {}                , DeltatOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Deltat         = {Deltat         , DeltatHelp        , DeltatOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Deltime        = {Deltime        , {}                , DeltimeOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Derivepar      = {Derivepar      , DeriveparHelp     , DeriveparOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Detrend        = {Detrend        , DetrendHelp       , DetrendOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -58,9 +57,9 @@ static const module_t module_Eofcoeff       = {Eofcoeff       , EofcoeffHelp
 static const module_t module_Eofcoeff3d     = {Eofcoeff3d     , EofcoeffHelp      , Eofcoeff3dOperators     , EXPOSED  , CDI_REAL , 2  , OBASE , NoRestriction };
 static const module_t module_EOFs           = {EOFs           , EOFsHelp          , EOFsOperators           , EXPOSED  , CDI_REAL , 1  , 2     , OnlyFirst     };
 static const module_t module_EOF3d          = {EOF3d          , EOFsHelp          , EOF3dOperators          , EXPOSED  , CDI_REAL , 1  , 2     , OnlyFirst     };
-static const module_t module_EstFreq        = {EstFreq        , {}                , EstFreqOperators        , INTERNAL  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_EstFreq        = {EstFreq        , {}                , EstFreqOperators        , INTERNAL , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Expr           = {Expr           , ExprHelp          , ExprOperators           , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
-static const module_t module_FC             = {FC             , {}                , FCOperators             , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_FC             = {FC             , {}                , FCOperators             , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Filedes        = {Filedes        , FiledesHelp       , FiledesOperators        , EXPOSED  , CDI_BOTH , 1  , 0     , NoRestriction };
 static const module_t module_Fillmiss       = {Fillmiss       , {}                , FillmissOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Filter         = {Filter         , FilterHelp        , FilterOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -92,7 +91,7 @@ static const module_t module_Intntime       = {Intntime       , InttimeHelp
 static const module_t module_Intyear        = {Intyear        , IntyearHelp       , IntyearOperators        , EXPOSED  , CDI_REAL , 2  , OBASE , NoRestriction };
 static const module_t module_Invert         = {Invert         , InvertHelp        , InvertOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Invertlev      = {Invertlev      , InvertlevHelp     , InvertlevOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
-static const module_t module_Isosurface     = {Isosurface     , {}                , IsosurfaceOperators     , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Isosurface     = {Isosurface     , IsosurfaceHelp    , IsosurfaceOperators     , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Lic            = {Lic            , {}                , LicOperators            , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_MapReduce      = {MapReduce      , MapReduceHelp     , MapReduceOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Maskbox        = {Maskbox        , MaskboxHelp       , MaskboxOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -112,7 +111,7 @@ static const module_t module_Nmldump        = {Nmldump        , {}
 static const module_t module_Output         = {Output         , OutputHelp        , OutputOperators         , EXPOSED  , CDI_BOTH , -1 , 0     , NoRestriction };
 static const module_t module_Outputtab      = {Output         , OutputtabHelp     , OutputtabOperators      , EXPOSED  , CDI_REAL , -1 , 0     , NoRestriction };
 static const module_t module_Outputgmt      = {Outputgmt      , OutputgmtHelp     , OutputgmtOperators      , EXPOSED  , CDI_REAL , 1  , 0     , NoRestriction };
-static const module_t module_Pack           = {Pack           , {}                , PackOperators           , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Pack           = {Pack           , PackHelp          , PackOperators           , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Pardup         = {Pardup         , {}                , PardupOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Pinfo          = {Pinfo          , {}                , PinfoOperators          , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Pressure       = {Pressure       , {}                , PressureOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -128,9 +127,10 @@ static const module_t module_Remapycon2     = {Remap          , Remapcon2Help
 static const module_t module_Remapscon      = {Remap          , {}                , RemapsconOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Remapcon2      = {Remap          , Remapcon2Help     , Remapcon2Operators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Remaplaf       = {Remap          , RemaplafHelp      , RemaplafOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
-static const module_t module_Remapgrid      = {Remap          , {}                , RemapgridOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Remapavg       = {Remap          , {}                , RemapavgOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Remapweights   = {Remapweights   , {}                , RemapweightsOperators   , EXPOSED  , CDI_REAL , 0  , 0     , NoRestriction };
 static const module_t module_Remapeta       = {Remapeta       , RemapetaHelp      , RemapetaOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Remapstat      = {Remapstat      , {}                , RemapstatOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Replace        = {Replace        , ReplaceHelp       , ReplaceOperators        , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Replacevalues  = {Replacevalues  , ReplacevaluesHelp , ReplacevaluesOperators  , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Rhopot         = {Rhopot         , RhopotHelp        , RhopotOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -148,7 +148,7 @@ static const module_t module_Selvar         = {Selvar         , SelvarHelp
 static const module_t module_Selrec         = {Selrec         , SelvarHelp        , SelrecOperators         , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Seloperator    = {Seloperator    , {}                , SeloperatorOperators    , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Seltime        = {Seltime        , SeltimeHelp       , SeltimeOperators        , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
-static const module_t module_Selyearidx     = {Selyearidx     , {}                , SelyearidxOperators     , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
+static const module_t module_Selyearidx     = {Selyearidx     , SelyearidxHelp    , SelyearidxOperators     , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Set            = {Set            , SetHelp           , SetOperators            , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Setattribute   = {Setattribute   , SetattributeHelp  , SetattributeOperators   , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Setbox         = {Setbox         , SetboxHelp        , SetboxOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -178,7 +178,6 @@ static const module_t module_Splitrec       = {Splitrec       , SplitHelp
 static const module_t module_Splitsel       = {Splitsel       , SplitselHelp      , SplitselOperators       , EXPOSED  , CDI_BOTH , 1  , OBASE , OnlyFirst };
 static const module_t module_Splittime      = {Splittime      , SplittimeHelp     , SplittimeOperators      , EXPOSED  , CDI_BOTH , 1  , OBASE , OnlyFirst };
 static const module_t module_Splityear      = {Splityear      , SplittimeHelp     , SplityearOperators      , EXPOSED  , CDI_BOTH , 1  , OBASE , NoRestriction };
-static const module_t module_Subtrend       = {Subtrend       , SubtrendHelp      , SubtrendOperators       , EXPOSED  , CDI_REAL , 3  , 1     , NoRestriction };
 static const module_t module_Tee            = {Tee            , TeeHelp           , TeeOperators            , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Template1      = {Template1      , {}                , Template1Operators      , INTERNAL , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Template2      = {Template2      , {}                , Template2Operators      , INTERNAL , CDI_REAL , 1  , 1     , NoRestriction };
@@ -207,27 +206,31 @@ static const module_t module_Monstat        = {Timstat        , MonstatHelp
 static const module_t module_Daystat        = {Timstat        , DaystatHelp       , DaystatOperators        , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Hourstat       = {Timstat        , HourstatHelp      , HourstatOperators       , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Timcor         = {Timstat2       , TimcorHelp        , TimcorOperators         , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
-static const module_t module_Timscorvar     = {Timstat2       , TimcovarHelp      , TimcovarOperators       , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
+static const module_t module_Timcorvar      = {Timstat2       , TimcovarHelp      , TimcovarOperators       , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
+static const module_t module_Timrmsd        = {Timstat2       , {}                , TimrmsdOperators        , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Timstat3       = {Timstat3       , {}                , Timstat3Operators       , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Tinfo          = {Tinfo          , {}                , TinfoOperators          , EXPOSED  , CDI_BOTH , 1  , 0     , NoRestriction };
 static const module_t module_Tocomplex      = {Tocomplex      , {}                , TocomplexOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Transpose      = {Transpose      , {}                , TransposeOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Trend          = {Trend          , TrendHelp         , TrendOperators          , EXPOSED  , CDI_REAL , 1  , 2     , OnlyFirst     };
+static const module_t module_Trendarith     = {Trendarith     , TrendarithHelp    , TrendarithOperators     , EXPOSED  , CDI_REAL , 3  , 1     , NoRestriction };
 static const module_t module_Tstepcount     = {Tstepcount     , {}                , TstepcountOperators     , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Unpack         = {Unpack         , {}                , UnpackOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Vargen         = {Vargen         , VargenHelp        , VargenOperators         , EXPOSED  , CDI_REAL , 0  , 1     , NoRestriction };
 static const module_t module_Varrms         = {Varrms         , {}                , VarrmsOperators         , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Varsstat       = {Varsstat       , VarsstatHelp      , VarsstatOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
-static const module_t module_Vertintml      = {Vertintml      , VertintmlHelp     , VertintmlOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Vertintap      = {Vertintap      , VertintapHelp     , VertintapOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Vertintgh      = {Vertintgh      , VertintghHelp     , VertintghOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Vertintml      = {Vertintml      , VertintmlHelp     , VertintmlOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Vertstat       = {Vertstat       , VertstatHelp      , VertstatOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Vertcum        = {Vertcum        , {}                , VertcumOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Vertwind       = {Vertwind       , {}                , VertwindOperators       , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
-static const module_t module_Verifygrid     = {Verifygrid     , {}                , VerifygridOperators     , EXPOSED  , CDI_REAL , 1  , 0     , NoRestriction };
+static const module_t module_Verifygrid     = {Verifygrid     , VerifygridHelp    , VerifygridOperators     , EXPOSED  , CDI_REAL , 1  , 0     , NoRestriction };
 static const module_t module_Wind           = {Wind           , WindHelp          , WindOperators           , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Wind2          = {Wind           , Wind2Help         , Wind2Operators          , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Writegrid      = {Writegrid      , {}                , WritegridOperators      , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction }; // no cdi output
 static const module_t module_Writerandom    = {Writerandom    , {}                , WriterandomOperators    , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
+static const module_t module_Yeararith      = {Yeararith      , YeararithHelp     , YeararithOperators      , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Yearmonstat    = {Yearmonstat    , YearmonstatHelp   , YearmonstatOperators    , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Ydayarith      = {Ydayarith      , YdayarithHelp     , YdayarithOperators      , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Ydaypctl       = {Ydaypctl       , YdaypctlHelp      , YdaypctlOperators       , EXPOSED  , CDI_REAL , 3  , 1     , NoRestriction };
@@ -249,6 +252,7 @@ static const module_t module_EcaCsu         = {EcaCsu         , EcaCsuHelp
 static const module_t module_EcaCwdi        = {EcaCwdi        , EcaCwdiHelp       , EcaCwdiOperators        , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_EcaCwfi        = {EcaCwfi        , EcaCwfiHelp       , EcaCwfiOperators        , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_EcaEtr         = {EcaEtr         , EcaEtrHelp        , EcaEtrOperators         , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
+static const module_t module_EcaEtccdi      = {EcaEtccdi      , EcaEtccdiHelp     , EcaEtccdiOperators      , EXPOSED  , CDI_REAL , 3  , 1     , NoRestriction };
 static const module_t module_EcaFd          = {EcaFd          , EcaFdHelp         , EcaFdOperators          , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_EcaGsl         = {EcaGsl         , EcaGslHelp        , EcaGslOperators         , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_EcaHd          = {EcaHd          , EcaHdHelp         , EcaHdOperators          , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
