@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,24 +23,62 @@
 #include <algorithm>  // for std::sort()
 
 /* removes '-' from operator string and returns copy of parameter operatorCommand */
+/***
+  vector for library handles for loaded custom modules
+  */
+std::vector<void *> custom_modules_lib_handles;
+
+/***
+  Maps operator names to their module names
+ */
+std::map<std::string, std::string> modules_map;
+
+/***
+  Contains added modules as values and their names as key
+  */
+std::map<std::string, module_t> modules;
+
+/***
+  Key: operator alias / Value: operator original name
+ */
+std::map<std::string, std::string> aliases;
+
+std::string
+module_t::toString()
+{
+  std::string inp = streamInCnt >= 0 ? std::to_string(streamInCnt) : "Arbitrary";
+  std::string out = streamOutCnt >= 0 ? std::to_string(streamOutCnt) : "Output base";
+  std::string restriction = "none";
+
+  if (restrictions == OnlyFirst) restriction = "Can only be the first operator";
+  if (restrictions == FilesOnly)
+    restriction != Green("none") ? restriction = Yellow("Can only use files as input.")
+                                 : restriction += Yellow(", Can only use files as input");
+
+  std::string desc = "Input: " + inp + ", Ouput: " + out + ", Restricton: " + restriction;
+
+  return desc;
+}
+
 static const char *
 getOperatorName(const char *operatorCommand)
 {
-  char *operatorName = nullptr;
-
   if (operatorCommand)
     {
       if (operatorCommand[0] == '-') operatorCommand++;
       const char *commapos = strchr(operatorCommand, ',');
-      const size_t len = (commapos != nullptr) ? (size_t)(commapos - operatorCommand) : strlen(operatorCommand);
+      const auto len = (commapos != nullptr) ? (size_t)(commapos - operatorCommand) : strlen(operatorCommand);
 
-      operatorName = (char *) malloc(len + 1);
-
-      memcpy(operatorName, operatorCommand, len);
-      operatorName[len] = '\0';
+      char *operatorName = (char *) malloc(len + 1);
+      if (operatorName != nullptr)
+        {
+          memcpy(operatorName, operatorCommand, len);
+          operatorName[len] = '\0';
+          return operatorName;
+        }
     }
 
-  return operatorName;
+  return nullptr;
 }
 
 /**
@@ -64,9 +102,9 @@ similar(const char *a, const char *b, unsigned long alen, unsigned long blen)
       b++;
     }
   if (!*a && !*b) return true;
-  /*
-    printf("%d %d %s %s\n", alen, blen, a, b);
-  */
+
+  //  printf("%d %d %s %s\n", alen, blen, a, b);
+
   if (alen >= 2 && blen >= 1 && *a && similar(a + 1, b, alen - 2, blen - 1)) return true;
 
   if (alen >= 1 && blen >= 2 && *b && similar(a, b + 1, alen - 1, blen - 2)) return true;
@@ -130,15 +168,15 @@ module_map_contains(std::string moduleName)
  * with a max lenght of 75 characters
  */
 static std::string
-find_similar(std::string operatorName)
+find_similar(const std::string &operatorName)
 {
   std::string found_similar_operators = "";
-  unsigned long lines = 1;
-  unsigned long line_length = 105;
+  size_t lines = 1;
+  constexpr size_t line_length = 105;
 
   if (operatorName != "")
     {
-      // Searching for simlar operator names in operator to module map
+      // Searching for similar operator names in operator to module map
       for (const auto &str : modules_map)
         {
           if (similar(stringToLower(operatorName), str.first))
@@ -199,7 +237,7 @@ check_operator(const std::string &operatorName)
   else
     {
       // Checking if the operatorname is an existing file name
-      FILE *fp = fopen(operatorName.c_str(), "r");
+      auto fp = fopen(operatorName.c_str(), "r");
       if (fp)
         {
           fclose(fp);
@@ -210,7 +248,7 @@ check_operator(const std::string &operatorName)
       // Checking for similar operators
       fprintf(stderr, "Operator >%s< not found!\n", operatorName.c_str());
       fprintf(stderr, "Similar operators are:\n");
-      std::string found_similar_operators = find_similar(operatorName);
+      const auto found_similar_operators = find_similar(operatorName);
 
       if (found_similar_operators.size() > 0)
         {
@@ -227,6 +265,25 @@ check_operator(const std::string &operatorName)
   return false;
 }
 
+void
+registerOperatorsFromModules()
+{
+  for (auto &x : modules)
+    {
+      for (auto y : x.second.operators)
+        {
+          if (!operator_name_exists(y))
+            {
+              modules_map[y] = x.first;
+            }
+          else
+            {
+              cdoAbort("Tried to add operator but the operator name already exists");
+            }
+        }
+    }
+}
+
 /***
  * Adds a module and its operators to cdo.
  * Adds the module to modules
@@ -235,12 +292,12 @@ check_operator(const std::string &operatorName)
  * @note: if an error happens while adding the new module cdo will exit.
  */
 void
-add_module(const std::string &module_name, const module_t new_module)
+add_module(const std::string &module_name, const module_t &new_module)
 {
   if (modules.find(module_name) == modules.end())
     {
       modules[module_name] = new_module;
-      for (const std::string &operatorName : new_module.operators)
+      for (const auto &operatorName : new_module.operators)
         {
           // if the operator name is not already in the map or in the aliases
           if (!operator_name_exists(operatorName))
@@ -272,19 +329,19 @@ add_alias(const std::string &alias, const std::string &original)
 
   if (iter_alias != aliases.end())
     {
-      printf("WARNING: alias %s could not be added: it already exists", alias.c_str());
+      printf("WARNING: alias %s could not be added: it already exists\n", alias.c_str());
       return -1;
     }
 
   if (iter_original == modules_map.end())
     {
-      printf("ERROR: alias %s could not be added: operator %s does not exist", alias.c_str(), original.c_str());
+      printf("ERROR: alias %s could not be added: operator %s does not exist\n", alias.c_str(), original.c_str());
       return -2;
     }
 
   if (modules_map.find(alias) != modules_map.end())
     {
-      printf("ERROR alias %s could not be added: alias name already exists as an operator", alias.c_str());
+      printf("ERROR alias %s could not be added: alias name already exists as an operator\n", alias.c_str());
       return -3;
     }
 
@@ -301,7 +358,7 @@ add_alias(const std::string &alias, const std::string &original)
  * return value:
  *      std::string -> name of the operators module
  */
-static std::string
+std::string
 get_module_name_to(const std::string &operatorName)
 {
   // if not true the programm will exit in function check_operator
@@ -316,12 +373,6 @@ get_module_name_to(const std::string &operatorName)
           return modules_map[aliases[operatorName]];
         }
     }
-  else
-    {
-      // TODO: see if depricated since no operator can be added without a
-      // module
-      cdoAbort("Module for >%s< not installed", operatorName.c_str());
-    }
   // Only to quell the warning. Function should never reach this.
   return "";
 }
@@ -343,11 +394,11 @@ void *(*operatorModule(const char *operatorName))(void *)
 
 /***
  * returns help for given operator name
- * if there is no help a empty vector will be returned
+ * if there is no help nullptr will be returned
  * @param operatorName operator name
  * @return vector of strings containing the help
  */
-std::vector<std::string>
+const char **
 operatorHelp(const std::string &operatorName)
 {
   return modules[get_module_name_to(operatorName)].help;
@@ -399,12 +450,12 @@ operatorStreamNumber(const char *operatorName)
  * @return a sorted std::vector containing all operator names and aliases
  * excluding all operators which modules are marked as internal
  */
-static std::vector<std::string>
+std::vector<std::string>
 get_sorted_operator_name_list()
 {
 
   std::vector<std::string> names;
-  for (const std::pair<std::string, std::string> &operator_module_names_pair : modules_map)
+  for (const auto &operator_module_names_pair : modules_map)
     {
       if (modules[operator_module_names_pair.second].mode == 1)
         {
@@ -412,7 +463,7 @@ get_sorted_operator_name_list()
         }
     }
   // adding operators names from alias_map
-  for (const std::pair<std::string, std::string> &alias : aliases)
+  for (const auto &alias : aliases)
     {
       names.push_back(alias.first);
     }
@@ -425,7 +476,7 @@ std::vector<std::string>
 get_no_output_operator_list()
 {
   std::vector<std::string> names;
-  for (const std::pair<std::string, std::string> &operator_module_names_pair : modules_map)
+  for (const auto &operator_module_names_pair : modules_map)
     {
       if (modules[operator_module_names_pair.second].mode == 1 && modules[operator_module_names_pair.second].streamOutCnt == 0)
         {
@@ -433,10 +484,9 @@ get_no_output_operator_list()
         }
     }
   // adding operators names from alias_map
-  std::string original;
-  for (const std::pair<std::string, std::string> &alias : aliases)
+  for (const auto &alias : aliases)
     {
-      original = alias.second;
+      const auto &original = alias.second;
       if (modules[modules_map[original]].mode == 1 && modules[modules_map[original]].streamOutCnt == 0)
         {
           names.push_back(alias.first);
@@ -558,17 +608,18 @@ get_spacing_for(int p_space, const std::string &str)
 }
 
 static std::string
-get_operator_description(const std::string &p_current_op_name, const std::vector<std::string> &help)
+get_operator_description(const std::string &p_current_op_name, const char **help)
 {
   std::string description = "";
-  unsigned long cur_help_idx;
-  bool help_contains_name;
+  unsigned long cur_help_idx = 0;
   std::string line;
   unsigned long operator_section = 0;
 
-  cur_help_idx = 0;
   // search for operator section
-  while (operator_section == 0 && cur_help_idx < help.size() - 1)
+  size_t help_size = 0;
+  while (help[help_size]) help_size++;
+  if (!help_size) return description;
+  while (operator_section == 0 && cur_help_idx < help_size - 1)
     {
       line = help[++cur_help_idx];
       if (line.find("OPERATORS") != std::string::npos)
@@ -582,7 +633,7 @@ get_operator_description(const std::string &p_current_op_name, const std::vector
       cur_help_idx = 0;
       line = help[0];
       std::string name_section = help[0];
-      help_contains_name = false;
+      bool help_contains_name = false;
       // search for the operator name in the description
       while (!line.empty())
         {
@@ -601,21 +652,21 @@ get_operator_description(const std::string &p_current_op_name, const std::vector
     }
   else
     {
-
-      line = help.at(++operator_section);
+      line = help[++operator_section];
       // search the operator section for current operator line
-      while (line.find(p_current_op_name + " ") == std::string::npos && !line.empty() && operator_section < help.size() - 1)
+      while (line.find(p_current_op_name + " ") == std::string::npos && !line.empty() && operator_section < help_size - 1)
         {
-          line = help.at(++operator_section);
+          line = help[++operator_section];
         }
       // if operator line found save description for later use
-      if (!line.empty() && line.find(p_current_op_name + " ") != std::string::npos)
+      if (!line.empty() && line.find("    " + p_current_op_name + " ") != std::string::npos)
         {
           auto op_name_start = line.find_first_not_of(" \t");
 
           description = line.substr(line.find_first_not_of(" \t", op_name_start + p_current_op_name.size()), line.size());
         }
     }
+
   return description;
 }
 
@@ -629,37 +680,28 @@ get_operator_description(const std::string &p_current_op_name, const std::vector
 void
 operatorPrintList(bool print_no_output)
 {
-  std::vector<std::string> output_list;
-  if (print_no_output)
-    {
-      output_list = get_no_output_operator_list();
-    }
-  else
-    {
-      output_list = get_sorted_operator_name_list();
-    }
+  std::vector<std::string> output_list = print_no_output ? get_no_output_operator_list() : get_sorted_operator_name_list();
 
-  unsigned long list_length = output_list.size();
-  module_t *current_module;
+  auto list_length = output_list.size();
 
   // help variables
 
-  for (unsigned long out_list_idx = 0; out_list_idx < list_length; out_list_idx++)
+  for (size_t out_list_idx = 0; out_list_idx < list_length; out_list_idx++)
     {
-      std::string current_op_name = output_list[out_list_idx];
-      current_module = &modules[get_module_name_to(current_op_name)];
+      auto &current_op_name = output_list[out_list_idx];
+      const auto *current_module = &modules[get_module_name_to(current_op_name)];
       if (aliases.find(current_op_name) != aliases.end())
         {
           output_list[out_list_idx] += std::string(get_spacing_for(16, current_op_name) + "--> " + aliases[current_op_name]);
         }
-      else if (!current_module->help.empty())
+      else if (current_module->help)
         {
           // add spaceing and saving output line to the output list
-          std::string description = get_operator_description(current_op_name, current_module->help);
+          auto description = get_operator_description(current_op_name, current_module->help);
           output_list[out_list_idx] += get_spacing_for(16, current_op_name) + description;
         }
       std::string in_out_info
-          = "(" + std::to_string(current_module->streamInCnt) + "|" + std::to_string(current_module->streamOutCnt) + ")";
+          = " (" + std::to_string(current_module->streamInCnt) + "|" + std::to_string(current_module->streamOutCnt) + ")";
       output_list[out_list_idx] += get_spacing_for(90, output_list[out_list_idx]) + in_out_info;
     }
   // print generated output list
@@ -681,7 +723,7 @@ get_original(const char *operatorName)
   const char *name = getOperatorName(operatorName);
   if (is_alias(name))
     {
-      std::string opName = aliases[std::string(name)];
+      const auto &opName = aliases[std::string(name)];
       char *original = (char *) malloc(opName.size() + 1);
       strcpy(original, opName.c_str());
       return original;
@@ -689,6 +731,26 @@ get_original(const char *operatorName)
   return name;
 }
 
+std::map<std::string, module_t>::iterator
+findModule(const std::string &operator_name)
+{
+  std::string operName = operator_name;
+
+  auto it = aliases.find(operName);
+  if (it != aliases.end())
+    {
+      operName = it->second;
+    }
+
+  auto modMapIter = modules_map.find(operName);
+  auto modIter = modules.end();
+  if (!(modMapIter == modules_map.end()))
+    {
+      modIter = modules.find(modMapIter->second);
+    }
+  return modIter;
+}
+
 module_t &
 getModule(const std::string &operator_name)
 {
