@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,8 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_LIBPROJ
-#include "proj_api.h"
+#ifdef HAVE_PROJ_H
+#include "proj.h"
 #endif
 
 #include <stdio.h>
@@ -54,7 +54,7 @@ check_xyvals(const size_t nvals, double *xvals, double *yvals)
 }
 
 #ifdef HAVE_LIBPROJ
-static char *
+static std::string
 gen_param(const char *fmt, ...)
 {
   va_list args;
@@ -62,80 +62,77 @@ gen_param(const char *fmt, ...)
 
   va_start(args, fmt);
 
-  int len = vsprintf(str, fmt, args);
+  vsprintf(str, fmt, args);
 
   va_end(args);
 
-  len++;
-  char *rstr = new char[len];
-  memcpy(rstr, str, len * sizeof(char));
-
-  return rstr;
+  std::string res(str);
+  return res;
 }
 
 static void
-proj_fwd_xyvals(projPJ &proj, size_t nvals, double *xvals, double *yvals)
+proj_fwd_xyvals(PJ *proj, size_t nvals, double *xvals, double *yvals)
 {
-  projUV p;
+  size_t num_nans = 0;
+  PJ_COORD p;
   for (size_t i = 0; i < nvals; i++)
     {
-      p.u = xvals[i] * DEG_TO_RAD;
-      p.v = yvals[i] * DEG_TO_RAD;
-      p = pj_fwd(p, proj);
-      xvals[i] = p.u;
-      yvals[i] = p.v;
+      p.uv.u = proj_torad(xvals[i]);
+      p.uv.v = proj_torad(yvals[i]);
+      p = proj_trans(proj, PJ_FWD, p);
+      if (std::isnan(p.uv.u) || std::isnan(p.uv.v)) num_nans++;
+      xvals[i] = p.uv.u;
+      yvals[i] = p.uv.v;
     }
+  if (num_nans) cdoWarning("%s: %zu of %zu projection coodinates are NaN!", __func__, num_nans, 2*nvals);
 }
 
 static void
-proj_inv_xyvals(projPJ &proj, size_t nvals, double *xvals, double *yvals)
+proj_inv_xyvals(PJ *proj, size_t nvals, double *xvals, double *yvals)
 {
-  projUV p;
+  size_t num_nans = 0;
+  PJ_COORD p;
   for (size_t i = 0; i < nvals; i++)
     {
-      p.u = xvals[i];
-      p.v = yvals[i];
-      p = pj_inv(p, proj);
-      xvals[i] = p.u * RAD_TO_DEG;
-      yvals[i] = p.v * RAD_TO_DEG;
+      p.uv.u = xvals[i];
+      p.uv.v = yvals[i];
+      p = proj_trans(proj, PJ_INV, p);
+      if (std::isnan(p.uv.u) || std::isnan(p.uv.v)) num_nans++;
+      xvals[i] = proj_todeg(p.uv.u);
+      yvals[i] = proj_todeg(p.uv.v);
     }
+  if (num_nans) cdoWarning("%s: %zu of %zu projection coodinates are NaN!", __func__, num_nans, 2*nvals);
+
 }
 
 static int
-do_proj_fwd(int nbpar, char *params[], size_t nvals, double *xvals, double *yvals)
+do_proj_fwd(const char *params, size_t nvals, double *xvals, double *yvals)
 {
-  if (Options::cdoVerbose)
-    for (int i = 0; i < nbpar; ++i) cdoPrint("Proj.param[%d] = %s", i + 1, params[i]);
+  if (Options::cdoVerbose) cdoPrint("Proj fwd: %s", params);
 
-  projPJ proj = pj_init(nbpar, &params[0]);
-  const int status = proj ? 0 : -1;
-
-  for (int i = 0; i < nbpar; ++i) delete[] params[i];
+  auto proj = proj_create(PJ_DEFAULT_CTX, params);
+  const int status = proj_errno(proj);
 
   if (status == 0)
     {
       proj_fwd_xyvals(proj, nvals, xvals, yvals);
-      pj_free(proj);
+      proj_destroy(proj);
     }
 
   return status;
 }
 
 static int
-do_proj_inv(int nbpar, char *params[], size_t nvals, double *xvals, double *yvals)
+do_proj_inv(const char *params, size_t nvals, double *xvals, double *yvals)
 {
-  if (Options::cdoVerbose)
-    for (int i = 0; i < nbpar; ++i) cdoPrint("Proj.param[%d] = %s", i + 1, params[i]);
-
-  projPJ proj = pj_init(nbpar, &params[0]);
-  const int status = proj ? 0 : -1;
-
-  for (int i = 0; i < nbpar; ++i) delete[] params[i];
+  if (Options::cdoVerbose) cdoPrint("Proj inv: %s", params);
 
+  auto proj = proj_create(PJ_DEFAULT_CTX, params);
+  const auto status = proj_errno(proj);
   if (status == 0)
     {
       proj_inv_xyvals(proj, nvals, xvals, yvals);
-      pj_free(proj);
+      proj_destroy(proj);
     }
 
   return status;
@@ -188,11 +185,11 @@ checkUpperRange(const char *projection, const char *name, const double val, cons
 static void
 verify_lcc_parameter(double lon_0, double lat_0, double lat_1, double lat_2, double a, double rf, double x_0, double y_0)
 {
-  const double grid_missval = cdiInqGridMissval();
+  const auto grid_missval = cdiInqGridMissval();
   const char *projection = "lambert_conformal_conic";
 
   checkUpperRange(projection, "earth_radius", a, grid_missval, 1.e10);
-  checkUpperRange(projection, "inverse_flattening", rf, grid_missval, 0);
+  checkUpperRange(projection, "inverse_flattening", rf, grid_missval, 400);
 
   checkLonRange(projection, "longitude_of_central_meridian", lon_0);
 
@@ -209,25 +206,24 @@ proj_lonlat_to_lcc(double missval, double lon_0, double lat_0, double lat_1, dou
                    double *xvals, double *yvals)
 {
 #ifdef HAVE_LIBPROJ
-  char *params[20];
+  std::string params = "";
 
-  int nbpar = 0;
-  params[nbpar++] = gen_param("proj=lcc");
-  if (IS_NOT_EQUAL(a, missval) && a > 0) params[nbpar++] = gen_param("a=%g", a);
-  if (IS_NOT_EQUAL(rf, missval) && rf > 0) params[nbpar++] = gen_param("rf=%g", rf);
-  params[nbpar++] = gen_param("lon_0=%g", lon_0);
-  params[nbpar++] = gen_param("lat_0=%g", lat_0);
-  params[nbpar++] = gen_param("lat_1=%g", lat_1);
-  params[nbpar++] = gen_param("lat_2=%g", lat_2);
-  params[nbpar++] = gen_param("units=m");
-  //  params[nbpar++] = gen_param("no_defs");
+  params += gen_param("+proj=lcc ");
+  if (IS_NOT_EQUAL(a, missval) && a > 0) params += gen_param("+a=%g ", a);
+  if (IS_NOT_EQUAL(rf, missval) && rf > 0) params += gen_param("+rf=%g ", rf);
+  params += gen_param("+lon_0=%g ", lon_0);
+  params += gen_param("+lat_0=%g ", lat_0);
+  params += gen_param("+lat_1=%g ", lat_1);
+  params += gen_param("+lat_2=%g ", lat_2);
+  params += gen_param("+units=m ");
+  //  params += gen_param("+no_defs ");
 
-  const int status = do_proj_fwd(nbpar, params, nvals, xvals, yvals);
+  const int status = do_proj_fwd(params.c_str(), nvals, xvals, yvals);
 #else
-  const int status = -1;
+  const int status = 1;
 #endif
 
-  if (status == -1) set_xyvals(missval, nvals, xvals, yvals);
+  if (status == 1) set_xyvals(missval, nvals, xvals, yvals);
 
   return status;
 }
@@ -238,16 +234,16 @@ lonlat_to_lcc(double missval, double lon_0, double lat_0, double lat_1, double l
 {
   const int status = proj_lonlat_to_lcc(missval, lon_0, lat_0, lat_1, lat_2, a, rf, nvals, xvals, yvals);
 #ifdef HAVE_LIBPROJ
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 #else
-  if (status == -1) cdoAbort("proj4 support not compiled in!");
+  if (status) cdoAbort("proj library support not compiled in!");
 #endif
 }
 
 int
 cdo_lonlat_to_lcc(int gridID, size_t nvals, double *xvals, double *yvals)
 {
-  const double grid_missval = cdiInqGridMissval();
+  const auto grid_missval = cdiInqGridMissval();
   double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
   gridInqParamLCC(gridID, grid_missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
 
@@ -260,27 +256,26 @@ int
 proj_lcc_to_lonlat(double missval, double lon_0, double lat_0, double lat_1, double lat_2, double a, double rf, double x_0,
                    double y_0, size_t nvals, double *xvals, double *yvals)
 {
-  const double grid_missval = cdiInqGridMissval();
+  const auto grid_missval = cdiInqGridMissval();
 #ifdef HAVE_LIBPROJ
-  char *params[20];
+  std::string params = "";
 
-  int nbpar = 0;
-  params[nbpar++] = gen_param("proj=lcc");
-  if (IS_NOT_EQUAL(a, grid_missval) && a > 0) params[nbpar++] = gen_param("a=%g", a);
-  if (IS_NOT_EQUAL(rf, grid_missval) && rf > 0) params[nbpar++] = gen_param("rf=%g", rf);
-  params[nbpar++] = gen_param("lon_0=%g", lon_0);
-  params[nbpar++] = gen_param("lat_0=%g", lat_0);
-  params[nbpar++] = gen_param("lat_1=%g", lat_1);
-  params[nbpar++] = gen_param("lat_2=%g", lat_2);
-  if (IS_NOT_EQUAL(x_0, grid_missval)) params[nbpar++] = gen_param("x_0=%g", x_0);
-  if (IS_NOT_EQUAL(y_0, grid_missval)) params[nbpar++] = gen_param("y_0=%g", y_0);
+  params += gen_param("+proj=lcc ");
+  if (IS_NOT_EQUAL(a, grid_missval) && a > 0) params += gen_param("+a=%g ", a);
+  if (IS_NOT_EQUAL(rf, grid_missval) && rf > 0) params += gen_param("+rf=%g ", rf);
+  params += gen_param("+lon_0=%g ", lon_0);
+  params += gen_param("+lat_0=%g ", lat_0);
+  params += gen_param("+lat_1=%g ", lat_1);
+  params += gen_param("+lat_2=%g ", lat_2);
+  if (IS_NOT_EQUAL(x_0, grid_missval)) params += gen_param("+x_0=%g ", x_0);
+  if (IS_NOT_EQUAL(y_0, grid_missval)) params += gen_param("+y_0=%g ", y_0);
 
-  const int status = do_proj_inv(nbpar, params, nvals, xvals, yvals);
+  const int status = do_proj_inv(params.c_str(), nvals, xvals, yvals);
 #else
-  const int status = -1;
+  const int status = 1;
 #endif
 
-  if (status == -1) set_xyvals(missval, nvals, xvals, yvals);
+  if (status == 1) set_xyvals(missval, nvals, xvals, yvals);
 
   return status;
 }
@@ -291,27 +286,26 @@ lcc_to_lonlat(double missval, double lon_0, double lat_0, double lat_1, double l
 {
   const int status = proj_lcc_to_lonlat(missval, lon_0, lat_0, lat_1, lat_2, a, rf, x_0, y_0, nvals, xvals, yvals);
 #ifdef HAVE_LIBPROJ
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 #else
-  if (status == -1) cdoAbort("proj4 support not compiled in!");
+  if (status == 1) cdoAbort("proj library support not compiled in!");
 #endif
 }
 
 int
 cdo_lcc_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
 {
-  double grid_missval = cdiInqGridMissval();
+  const auto grid_missval = cdiInqGridMissval();
   const char *projection = "lambert_conformal_conic";
 
   double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
   gridInqParamLCC(gridID, grid_missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
 
-  int status = 0;
-  if (!status && checkValIsMiss(projection, "longitude_of_central_meridian", lon_0, grid_missval)) status = 1;
-  if (!status && checkValIsMiss(projection, "latitude_of_central_meridian", lat_0, grid_missval)) status = 1;
-  if (!status && checkValIsMiss(projection, "standard_parallel", lat_1, grid_missval)) status = 1;
+  bool paramIsMissing = checkValIsMiss(projection, "longitude_of_central_meridian", lon_0, grid_missval)
+    || checkValIsMiss(projection, "latitude_of_projection_origin", lat_0, grid_missval)
+    || checkValIsMiss(projection, "standard_parallel", lat_1, grid_missval);
 
-  if (!status && IS_EQUAL(x_0, grid_missval) && IS_EQUAL(y_0, grid_missval) && IS_NOT_EQUAL(xval_0, grid_missval)
+  if (!paramIsMissing && IS_EQUAL(x_0, grid_missval) && IS_EQUAL(y_0, grid_missval) && IS_NOT_EQUAL(xval_0, grid_missval)
       && IS_NOT_EQUAL(yval_0, grid_missval))
     {
 #ifdef HAVE_LIBPROJ
@@ -321,13 +315,13 @@ cdo_lcc_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
       x_0 = -x_0;
       y_0 = -y_0;
 #else
-      status = 1;
+      paramIsMissing = true;
       cdoWarning("%s mapping parameter %s missing!", projection, "false_easting and false_northing");
-      cdoAbort("proj4 support not compiled in!");
+      cdoAbort("proj library support not compiled in!");
 #endif
     }
 
-  if (status) cdoAbort("%s mapping parameter missing!", projection);
+  if (paramIsMissing) cdoAbort("%s mapping parameter missing!", projection);
 
   verify_lcc_parameter(lon_0, lat_0, lat_1, lat_2, a, rf, x_0, y_0);
 
@@ -339,8 +333,8 @@ cdo_lcc_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
 static void
 verify_stere_parameter(double lon_0, double lat_ts, double lat_0, double a, double x_0, double y_0)
 {
-  const double grid_missval = cdiInqGridMissval();
-  const char *projection = "polar_stereographic";
+  const auto grid_missval = cdiInqGridMissval();
+  const char *const projection = "polar_stereographic";
 
   checkUpperRange(projection, "earth_radius", a, grid_missval, 1.e10);
 
@@ -358,23 +352,22 @@ proj_lonlat_to_stere(double missval, double lon_0, double lat_ts, double lat_0,
                      double *yvals)
 {
 #ifdef HAVE_LIBPROJ
-  char *params[20];
+  std::string params;
 
-  int nbpar = 0;
-  params[nbpar++] = gen_param("proj=stere");
-  if (IS_NOT_EQUAL(a, missval) && a > 0) params[nbpar++] = gen_param("a=%g", a);
-  params[nbpar++] = gen_param("lon_0=%g", lon_0);
-  params[nbpar++] = gen_param("lat_ts=%g", lat_ts);
-  params[nbpar++] = gen_param("lat_0=%g", lat_0);
-  params[nbpar++] = gen_param("units=m");
-  //  params[nbpar++] = gen_param("no_defs");
+  params += gen_param("+proj=stere ");
+  if (IS_NOT_EQUAL(a, missval) && a > 0) params += gen_param("+a=%g ", a);
+  params += gen_param("+lon_0=%g ", lon_0);
+  params += gen_param("+lat_ts=%g ", lat_ts);
+  params += gen_param("+lat_0=%g ", lat_0);
+  params += gen_param("+units=m ");
+  //  params += gen_param("+no_defs ");
 
-  const int status = do_proj_fwd(nbpar, params, nvals, xvals, yvals);
+  const int status = do_proj_fwd(params.c_str(), nvals, xvals, yvals);
 #else
-  const int status = -1;
+  const int status = 1;
 #endif
 
-  if (status == -1) set_xyvals(missval, nvals, xvals, yvals);
+  if (status == 1) set_xyvals(missval, nvals, xvals, yvals);
 
   return status;
 }
@@ -384,16 +377,16 @@ lonlat_to_stere(double missval, double lon_0, double lat_ts, double lat_0, doubl
 {
   const int status = proj_lonlat_to_stere(missval, lon_0, lat_ts, lat_0, a, nvals, xvals, yvals);
 #ifdef HAVE_LIBPROJ
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 #else
-  if (status == -1) cdoAbort("proj4 support not compiled in!");
+  if (status == 1) cdoAbort("proj library support not compiled in!");
 #endif
 }
 
 int
 cdo_lonlat_to_stere(int gridID, size_t nvals, double *xvals, double *yvals)
 {
-  const double grid_missval = cdiInqGridMissval();
+  const auto grid_missval = cdiInqGridMissval();
   double lon_0, lat_ts, lat_0, a, xval_0, yval_0, x_0, y_0;
   gridInqParamSTERE(gridID, grid_missval, &lon_0, &lat_ts, &lat_0, &a, &xval_0, &yval_0, &x_0, &y_0);
 
@@ -407,24 +400,23 @@ proj_stere_to_lonlat(double missval, double lon_0, double lat_ts, double lat_0,
                      double *xvals, double *yvals)
 {
 #ifdef HAVE_LIBPROJ
-  const double grid_missval = cdiInqGridMissval();
-  char *params[20];
+  const auto grid_missval = cdiInqGridMissval();
+  std::string params;
 
-  int nbpar = 0;
-  params[nbpar++] = gen_param("proj=stere");
-  if (IS_NOT_EQUAL(a, grid_missval) && a > 0) params[nbpar++] = gen_param("a=%g", a);
-  params[nbpar++] = gen_param("lon_0=%g", lon_0);
-  params[nbpar++] = gen_param("lat_ts=%g", lat_ts);
-  params[nbpar++] = gen_param("lat_0=%g", lat_0);
-  if (IS_NOT_EQUAL(x_0, grid_missval)) params[nbpar++] = gen_param("x_0=%g", x_0);
-  if (IS_NOT_EQUAL(y_0, grid_missval)) params[nbpar++] = gen_param("y_0=%g", y_0);
+  params += gen_param("+proj=stere ");
+  if (IS_NOT_EQUAL(a, grid_missval) && a > 0) params += gen_param("+a=%g ", a);
+  params += gen_param("+lon_0=%g ", lon_0);
+  params += gen_param("+lat_ts=%g ", lat_ts);
+  params += gen_param("+lat_0=%g ", lat_0);
+  if (IS_NOT_EQUAL(x_0, grid_missval)) params += gen_param("+x_0=%g ", x_0);
+  if (IS_NOT_EQUAL(y_0, grid_missval)) params += gen_param("+y_0=%g ", y_0);
 
-  const int status = do_proj_inv(nbpar, params, nvals, xvals, yvals);
+  const int status = do_proj_inv(params.c_str(), nvals, xvals, yvals);
 #else
-  const int status = -1;
+  const int status = 1;
 #endif
 
-  if (status == -1) set_xyvals(missval, nvals, xvals, yvals);
+  if (status == 1) set_xyvals(missval, nvals, xvals, yvals);
 
   return status;
 }
@@ -435,27 +427,26 @@ stere_to_lonlat(double missval, double lon_0, double lat_ts, double lat_0, doubl
 {
   const int status = proj_stere_to_lonlat(missval, lon_0, lat_ts, lat_0, a, x_0, y_0, nvals, xvals, yvals);
 #ifdef HAVE_LIBPROJ
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 #else
-  if (status == -1) cdoAbort("proj4 support not compiled in!");
+  if (status == 1) cdoAbort("proj library support not compiled in!");
 #endif
 }
 
 int
 cdo_stere_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
 {
-  const double grid_missval = cdiInqGridMissval();
+  const auto grid_missval = cdiInqGridMissval();
   const char *projection = "polar_stereographic";
 
   double lon_0, lat_ts, lat_0, a, xval_0, yval_0, x_0, y_0;
   gridInqParamSTERE(gridID, grid_missval, &lon_0, &lat_ts, &lat_0, &a, &xval_0, &yval_0, &x_0, &y_0);
 
-  int status = 0;
-  if (!status && checkValIsMiss(projection, "straight_vertical_longitude_from_pole", lon_0, grid_missval)) status = 1;
-  if (!status && checkValIsMiss(projection, "latitude_of_projection_origin", lat_0, grid_missval)) status = 1;
-  if (!status && checkValIsMiss(projection, "standard_parallel", lat_ts, grid_missval)) status = 1;
+  bool paramIsMissing = checkValIsMiss(projection, "straight_vertical_longitude_from_pole", lon_0, grid_missval)
+    || checkValIsMiss(projection, "latitude_of_projection_origin", lat_0, grid_missval)
+    || checkValIsMiss(projection, "standard_parallel", lat_ts, grid_missval);
 
-  if (!status && IS_EQUAL(x_0, grid_missval) && IS_EQUAL(y_0, grid_missval) && IS_NOT_EQUAL(xval_0, grid_missval)
+  if (!paramIsMissing && IS_EQUAL(x_0, grid_missval) && IS_EQUAL(y_0, grid_missval) && IS_NOT_EQUAL(xval_0, grid_missval)
       && IS_NOT_EQUAL(yval_0, grid_missval))
     {
 #ifdef HAVE_LIBPROJ
@@ -465,13 +456,13 @@ cdo_stere_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
       x_0 = -x_0;
       y_0 = -y_0;
 #else
-      status = 1;
+      paramIsMissing = true;
       cdoWarning("%s mapping parameter %s missing!", projection, "false_easting and false_northing");
-      cdoAbort("proj4 support not compiled in!");
+      cdoAbort("proj library support not compiled in!");
 #endif
     }
 
-  if (status) cdoAbort("%s mapping parameter missing!", projection);
+  if (paramIsMissing) cdoAbort("%s mapping parameter missing!", projection);
 
   verify_stere_parameter(lon_0, lat_ts, lat_0, a, x_0, y_0);
 
@@ -483,10 +474,10 @@ cdo_stere_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
 void
 grid_def_param_sinu(int gridID)
 {
-  const char *projection = "sinusoidal";
-  cdiGridDefKeyStr(gridID, CDI_KEY_MAPNAME, (int) strlen(projection) + 1, projection);
-  const char *mapvarname = "Sinusoidal";
-  cdiGridDefKeyStr(gridID, CDI_KEY_MAPPING, (int) strlen(mapvarname) + 1, mapvarname);
+  const char *const projection = "sinusoidal";
+  cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, projection);
+  const char *const gmapvarname = "Sinusoidal";
+  cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, gmapvarname);
 
   cdiDefAttTxt(gridID, CDI_GLOBAL, "grid_mapping_name", (int) strlen(projection), projection);
 }
@@ -494,10 +485,10 @@ grid_def_param_sinu(int gridID)
 void
 grid_def_param_laea(int gridID, double a, double lon_0, double lat_0)
 {
-  const char *projection = "lambert_azimuthal_equal_area";
-  cdiGridDefKeyStr(gridID, CDI_KEY_MAPNAME, (int) strlen(projection) + 1, projection);
-  const char *mapvarname = "Lambert_AEA";
-  cdiGridDefKeyStr(gridID, CDI_KEY_MAPPING, (int) strlen(mapvarname) + 1, mapvarname);
+  const char *const projection = "lambert_azimuthal_equal_area";
+  cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, projection);
+  const char *const gmapvarname = "Lambert_AEA";
+  cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, gmapvarname);
 
   cdiDefAttTxt(gridID, CDI_GLOBAL, "grid_mapping_name", (int) strlen(projection), projection);
 
@@ -510,18 +501,17 @@ void
 cdo_sinu_to_lonlat(size_t nvals, double *xvals, double *yvals)
 {
 #ifdef HAVE_LIBPROJ
-  char *params[20];
+  std::string params;
 
-  int nbpar = 0;
-  params[nbpar++] = gen_param("proj=sinu");
-  params[nbpar++] = gen_param("ellps=WGS84");
+  params += gen_param("+proj=sinu ");
+  params += gen_param("+ellps=WGS84 ");
 
-  const int status = do_proj_inv(nbpar, params, nvals, xvals, yvals);
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  const int status = do_proj_inv(params.c_str(), nvals, xvals, yvals);
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 
   check_xyvals(nvals, xvals, yvals);
 #else
-  cdoAbort("proj4 support not compiled in!");
+  cdoAbort("proj library support not compiled in!");
 #endif
 }
 
@@ -552,18 +542,18 @@ cdiInqAttConvertedToFloat(int gridID, int atttype, const char *attname, int attl
 static void
 grid_inq_param_laea(int gridID, double *a, double *lon_0, double *lat_0, double *x_0, double *y_0)
 {
-  *a = 0;
-  *lon_0 = 0;
-  *lat_0 = 0, *x_0 = 0, *y_0 = 0;
+  *a = 0.0;
+  *lon_0 = 0.0;
+  *lat_0 = 0.0, *x_0 = 0.0, *y_0 = 0.0;
 
-  const int gridtype = gridInqType(gridID);
+  const auto gridtype = gridInqType(gridID);
   if (gridtype == GRID_PROJECTION)
     {
       const char *projection = "lambert_azimuthal_equal_area";
-      char mapping[CDI_MAX_NAME];
-      mapping[0] = 0;
-      cdiGridInqKeyStr(gridID, CDI_KEY_MAPNAME, CDI_MAX_NAME, mapping);
-      if (mapping[0] && cstrIsEqual(mapping, projection))
+      char gmapname[CDI_MAX_NAME];
+      int length = CDI_MAX_NAME;
+      cdiInqKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, gmapname, &length);
+      if (gmapname[0] && cdo_cmpstr(gmapname, projection))
         {
           char attname[CDI_MAX_NAME + 1];
 
@@ -580,11 +570,11 @@ grid_inq_param_laea(int gridID, double *a, double *lon_0, double *lat_0, double
               if (cdiInqAttConvertedToFloat(gridID, atttype, attname, attlen, &attflt))
                 {
                   // clang-format off
-                  if      (cstrIsEqual(attname, "earth_radius")) *a = attflt;
-                  else if (cstrIsEqual(attname, "longitude_of_projection_origin")) *lon_0 = attflt;
-                  else if (cstrIsEqual(attname, "latitude_of_projection_origin")) *lat_0 = attflt;
-                  else if (cstrIsEqual(attname, "false_easting")) *x_0 = attflt;
-                  else if (cstrIsEqual(attname, "false_northing")) *y_0 = attflt;
+                  if      (cdo_cmpstr(attname, "earth_radius")) *a = attflt;
+                  else if (cdo_cmpstr(attname, "longitude_of_projection_origin")) *lon_0 = attflt;
+                  else if (cdo_cmpstr(attname, "latitude_of_projection_origin")) *lat_0 = attflt;
+                  else if (cdo_cmpstr(attname, "false_easting")) *x_0 = attflt;
+                  else if (cdo_cmpstr(attname, "false_northing")) *y_0 = attflt;
                   // clang-format on
                 }
             }
@@ -599,76 +589,57 @@ void
 cdo_laea_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals)
 {
 #ifdef HAVE_LIBPROJ
-  char *params[20];
+  std::string params;
 
   double a, lon_0, lat_0, x_0, y_0;
   grid_inq_param_laea(gridID, &a, &lon_0, &lat_0, &x_0, &y_0);
 
-  int nbpar = 0;
-  params[nbpar++] = gen_param("proj=laea");
-  if (a > 0) params[nbpar++] = gen_param("a=%g", a);
-  params[nbpar++] = gen_param("lon_0=%g", lon_0);
-  params[nbpar++] = gen_param("lat_0=%g", lat_0);
-  if (IS_NOT_EQUAL(x_0, 0)) params[nbpar++] = gen_param("x_0=%g", x_0);
-  if (IS_NOT_EQUAL(y_0, 0)) params[nbpar++] = gen_param("y_0=%g", y_0);
+  params += gen_param("+proj=laea ");
+  if (a > 0) params += gen_param("+a=%g ", a);
+  params += gen_param("+lon_0=%g ", lon_0);
+  params += gen_param("+lat_0=%g ", lat_0);
+  if (IS_NOT_EQUAL(x_0, 0)) params += gen_param("+x_0=%g ", x_0);
+  if (IS_NOT_EQUAL(y_0, 0)) params += gen_param("+y_0=%g ", y_0);
 
-  const int status = do_proj_inv(nbpar, params, nvals, xvals, yvals);
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  const int status = do_proj_inv(params.c_str(), nvals, xvals, yvals);
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 
   check_xyvals(nvals, xvals, yvals);
 #else
-  cdoAbort("proj4 support not compiled in!");
+  cdoAbort("proj library support not compiled in!");
 #endif
 }
 
 void
-cdo_proj_to_lonlat(char *proj4params, size_t nvals, double *xvals, double *yvals)
+cdo_proj_to_lonlat(char *proj_params, size_t nvals, double *xvals, double *yvals)
 {
 #ifdef HAVE_LIBPROJ
-  char *params[99];
-
-  int nbpar;
-  for (nbpar = 0; nbpar < 99; ++nbpar)
-    {
-      while (*proj4params == ' ' || *proj4params == '+') proj4params++;
-      if (*proj4params == 0) break;
-      char *cstart = proj4params;
-      while (*proj4params != ' ' && *proj4params != 0) proj4params++;
-      char *cend = proj4params;
-      size_t len = cend - cstart;
-      if (len <= 0) break;
-      bool lend = *cend == 0;
-      if (!lend) *cend = 0;
-      params[nbpar] = strdup(cstart);
-      if (!lend) *cend = ' ';
-    }
-
-  const int status = do_proj_inv(nbpar, params, nvals, xvals, yvals);
-  if (status == -1) cdoAbort("proj error: %s", pj_strerrno(pj_errno));
+  const int status = do_proj_inv(proj_params, nvals, xvals, yvals);
+  if (status) cdoAbort("proj library error: %s", proj_errno_string(status));
 
   check_xyvals(nvals, xvals, yvals);
 #else
-  cdoAbort("proj4 support not compiled in!");
+  cdoAbort("proj library support not compiled in!");
 #endif
 }
 
 double
 gridGetPlanetRadius(int gridID)
 {
-  double planetRadius = 0;
+  double planetRadius = 0.0;
 
-  const int gridtype = gridInqType(gridID);
-  const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  const auto gridtype = gridInqType(gridID);
+  const auto projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
   if (projtype == CDI_PROJ_LCC)
     {
-      const double grid_missval = cdiInqGridMissval();
+      const auto grid_missval = cdiInqGridMissval();
       double lon_0, lat_0, lat_1, lat_2, a, rf, xval_0, yval_0, x_0, y_0;
       gridInqParamLCC(gridID, grid_missval, &lon_0, &lat_0, &lat_1, &lat_2, &a, &rf, &xval_0, &yval_0, &x_0, &y_0);
       planetRadius = a;
     }
   else if (projtype == CDI_PROJ_STERE)
     {
-      const double grid_missval = cdiInqGridMissval();
+      const auto grid_missval = cdiInqGridMissval();
       double lon_0, lat_ts, lat_0, a, xval_0, yval_0, x_0, y_0;
       gridInqParamSTERE(gridID, grid_missval, &lon_0, &lat_ts, &lat_0, &a, &xval_0, &yval_0, &x_0, &y_0);
       planetRadius = a;
