@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,6 +16,7 @@
 */
 #ifndef GRID_PROJ_H
 #define GRID_PROJ_H
+
 #include <cstddef>
 
 int cdo_lonlat_to_lcc(int gridID, size_t nvals, double *xvals, double *yvals);
@@ -25,7 +26,7 @@ int cdo_stere_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals);
 void cdo_sinu_to_lonlat(size_t nvals, double *xvals, double *yvals);
 void cdo_laea_to_lonlat(int gridID, size_t nvals, double *xvals, double *yvals);
 
-void cdo_proj_to_lonlat(char *proj4param, size_t nvals, double *xvals, double *yvals);
+void cdo_proj_to_lonlat(char *proj_param, size_t nvals, double *xvals, double *yvals);
 
 int proj_lonlat_to_lcc(double missval, double lon_0, double lat_0, double lat_1, double lat_2, double a, double rf, size_t nvals,
                        double *xvals, double *yvals);
