@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,21 +17,41 @@
 #ifndef MPIM_GRID_H
 #define MPIM_GRID_H
 
+#include <cstdio>
+#include <vector>
+
+#include <cdi.h>
+
 #include "grid_proj.h"
 #include "grid_rot.h"
 #include "grid_convert.h"
 
-#include <cstdio>
-#include <vector>
-
-extern "C" int qu2reg3_double(double *pfield, int *kpoint, int klat, int klon, double msval, int *kret, int omisng,
-                               int operio, int oveggy);
+extern "C" int qu2reg3_double(double *pfield, int *kpoint, int klat, int klon, double msval, int *kret, int omisng, int operio,
+                              int oveggy);
 
 extern bool gridVerbose;
-extern char *gridSearchDir;
+
+static inline bool
+gridProjIsSupported(int gridID)
+{
+  const auto gridtype = gridInqType(gridID);
+  const auto pt = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
+  return (pt == CDI_PROJ_RLL || pt == CDI_PROJ_LCC || pt == CDI_PROJ_LAEA || pt == CDI_PROJ_STERE || pt == CDI_PROJ_SINU);
+}
+
+static inline bool
+gridHasCoordinates(int gridID)
+{
+  return (gridInqXvals(gridID, nullptr) && gridInqYvals(gridID, nullptr));
+}
+
+static inline bool
+gridHasBounds(int gridID)
+{
+  return (gridInqXbounds(gridID, nullptr) && gridInqYbounds(gridID, nullptr));
+}
 
 void gridEnableVerbose(bool enable);
-void gridSetSearchDir(char *dir);
 
 int nfc_to_nlat(int nfc, int ntr);
 int nlat_to_ntr(int nlat);
@@ -43,13 +63,14 @@ int ntr_to_nlat_cubic(int ntr);
 int nlat_to_nlon(int nlat);
 int nlat_to_nlon_cubic(int nlat);
 
-void grid_copy_attributes(int gridID1, int gridID2);
+void grid_copy_keys(int gridID1, int gridID2);
 void grid_copy_mapping(int gridID1, int gridID2);
 
 bool grid_is_distance_generic(int gridID);
 
 void grid_to_radian(const char *units, size_t nvals, double *values, const char *description);
-void grid_to_degree(const char *units, size_t nvals, double *values, const char *description);
+void cdo_grid_to_radian(int gridID, int varID, size_t nvals, double *values, const char *description);
+void cdo_grid_to_degree(int gridID, int varID, size_t nvals, double *values, const char *description);
 
 void grid_gen_corners(size_t n, const double *vals, double *corners);
 void grid_gen_bounds(size_t n, const std::vector<double> &vals, std::vector<double> &bounds);
@@ -58,11 +79,6 @@ void grid_check_lat_borders(int n, double *ybounds);
 void grid_gen_xbounds2D(size_t nx, size_t ny, const std::vector<double> &xbounds, std::vector<double> &xbounds2D);
 void grid_gen_ybounds2D(size_t nx, size_t ny, const std::vector<double> &ybounds, std::vector<double> &ybounds2D);
 
-void grid_cell_center_to_bounds_X2D(const char *xunitstr, size_t xsize, size_t ysize, const double *grid_center_lon,
-                                    double *grid_corner_lon, double dlon);
-void grid_cell_center_to_bounds_Y2D(const char *yunitstr, size_t xsize, size_t ysize, const double *grid_center_lat,
-                                    double *grid_corner_lat);
-
 int gridWeights(int gridID, double *weights);
 int gridGenArea(int gridID, double *area);
 int gridGenAreaReg2Dweights(int gridID, double *area);
@@ -84,6 +100,6 @@ void gme_grid(int lbounds, size_t gridsize, double *rlon, double *rlat, double *
 
 void cdoPrintGrid(int gridID, int opt);
 
-bool grid_has_proj4param(int gridID);
+bool grid_has_proj_params(int gridID);
 
 #endif /* MPIM_GRID_H */
