@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,15 @@
 #include <string>
 #include <sstream>
 
+enum ColorEnabled
+{
+  No,
+  All,
+  Auto
+};
+
+void mpmoColorSet(int p_colorBehaviour);
+
 enum TextMode
 {
   MODELESS = 0,
@@ -50,10 +59,9 @@ extern bool stdin_is_tty;
 extern bool stdout_is_tty;
 extern bool stderr_is_tty;
 
-extern int CDO_Color;
+bool colorEnabled();
 
-#define COLOR_STDOUT (stdout_is_tty && CDO_Color)
-#define COLOR_STDERR (stderr_is_tty && CDO_Color)
+int mpmoGetColorMode();
 
 void set_text_color(FILE *fp, TextMode attr, TextColor fg);
 void set_text_color(FILE *fp, TextMode attr);
@@ -86,9 +94,7 @@ color(TextColor foreground = NO_COLOR, TextMode mode = MODELESS, TextColor backg
   bool tty = true;
   std::stringstream s;
 
-  if (!COLOR_STDOUT) return "";
-  if (!COLOR_STDERR) return "";
-
+  if (!colorEnabled()) return "";
   s << "\033[";
   if (!tty)
     {
