@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,8 +15,8 @@
   GNU General Public License for more details.
 */
 
-#include <stdio.h>
-#include <ctype.h>
+#include <cstdio>
+#include <cctype>
 
 #include "namelist.h"
 
@@ -24,32 +24,31 @@
 NamelistToken *
 NamelistParser::allocToken()
 {
-  const unsigned int TOK_MEM_INCR = 64;
-
   if (this->toknext >= this->num_tokens)
     {
+      constexpr unsigned int TOK_MEM_INCR = 64;
       this->num_tokens += TOK_MEM_INCR;
       this->tokens.resize(this->num_tokens);
     }
 
-  NamelistToken *tok = &this->tokens[this->toknext++];
+  auto tok = &this->tokens[this->toknext++];
   tok->start = tok->end = -1;
   return tok;
 }
 
 // Fills token type and boundaries.
 void
-NamelistToken::fill(NamelistType type, int start, int end)
+NamelistToken::fill(NamelistType _type, int _start, int _end)
 {
-  this->type = type;
-  this->start = start;
-  this->end = end;
+  this->type = _type;
+  this->start = _start;
+  this->end = _end;
 }
 
 void
 NamelistParser::newObject()
 {
-  NamelistToken *token = this->allocToken();
+  auto token = this->allocToken();
   token->type = NamelistType::OBJECT;
   token->start = this->pos;
 }
@@ -58,8 +57,7 @@ NamelistParser::newObject()
 NamelistError
 NamelistParser::parseWord(const char *buf, size_t len)
 {
-  NamelistToken *token;
-  int start = this->pos;
+  const auto start = this->pos;
 
   for (; this->pos < len && buf[this->pos] != '\0'; this->pos++)
     {
@@ -85,7 +83,7 @@ NamelistParser::parseWord(const char *buf, size_t len)
 
 found:
 
-  token = this->allocToken();
+  auto token = this->allocToken();
   token->fill(NamelistType::WORD, start, this->pos);
   this->pos--;
 
@@ -96,19 +94,19 @@ found:
 NamelistError
 NamelistParser::parseString(const char *buf, size_t len, char quote)
 {
-  int start = this->pos;
+  const auto start = this->pos;
 
   this->pos++;
 
   /* Skip starting quote */
   for (; this->pos < len && buf[this->pos] != '\0'; this->pos++)
     {
-      char c = buf[this->pos];
+      const auto c = buf[this->pos];
 
       /* Quote: end of string */
       if (c == quote)
         {
-          NamelistToken *token = this->allocToken();
+          auto token = this->allocToken();
           token->fill(NamelistType::STRING, start + 1, this->pos);
           return NamelistError::UNDEFINED;
         }
@@ -126,8 +124,7 @@ NamelistParser::parseString(const char *buf, size_t len, char quote)
             case 'f':
             case 'r':
             case 'n':
-            case 't':
-              break;
+            case 't': break;
             // Allows escaped symbol \uXXXX
             case 'u':
               this->pos++;
@@ -166,8 +163,11 @@ NamelistParser::checkKeyname(const char *buf, NamelistToken *t)
       if ((t->end - t->start) < 1) return NamelistError::EMKEY;
       for (int i = t->start; i < t->end; ++i)
         if (isspace((int) buf[i])) return NamelistError::INKEY;
-    case NamelistType::WORD: t->type = NamelistType::KEY; break;
-    default: return NamelistError::INTYP;
+      t->type = NamelistType::KEY; break;
+    case NamelistType::WORD:
+      t->type = NamelistType::KEY; break;
+    default:
+      return NamelistError::INTYP;
     }
 
   return NamelistError::UNDEFINED;
@@ -176,20 +176,20 @@ NamelistParser::checkKeyname(const char *buf, NamelistToken *t)
 NamelistError
 NamelistParser::parse(const char *buf, size_t len)
 {
-  NamelistError status = NamelistError::UNDEFINED;
+  auto status = NamelistError::UNDEFINED;
 
   this->lineno = 1;
 
   for (; this->pos < len && buf[this->pos] != '\0'; this->pos++)
     {
-      char c = buf[this->pos];
+      const auto c = buf[this->pos];
       switch (c)
         {
         case '&': this->newObject(); break;
         case '/':
           for (int i = this->toknext - 1; i >= 0; i--)
             {
-              NamelistToken *token = &this->tokens[i];
+              auto token = &this->tokens[i];
               if (token->start != -1 && token->end == -1)
                 {
                   if (token->type != NamelistType::OBJECT) return NamelistError::INOBJ;
@@ -202,7 +202,9 @@ NamelistParser::parse(const char *buf, size_t len)
         case ' ': break;
         case '\r':
           if (this->pos + 1 < len && buf[this->pos + 1] == '\n') this->pos++;
-        case '\n': this->lineno++; break;
+          this->lineno++; break;
+        case '\n':
+          this->lineno++; break;
         case ',': break;
         case '#':
         case '!':  // Skip to end of line
@@ -229,14 +231,14 @@ NamelistParser::parse(const char *buf, size_t len)
 void
 NamelistParser::dump(const char *buf)
 {
-  unsigned int ntok = this->toknext;
-  printf("Number of tokens %d\n", ntok);
+  const auto ntok = this->toknext;
+  printf("Number of tokens %u\n", ntok);
 
   for (unsigned int it = 0; it < ntok; ++it)
     {
-      NamelistToken *t = &this->tokens[it];
-      int length = t->end - t->start;
-      const char *start = buf + t->start;
+      auto t = &this->tokens[it];
+      auto length = t->end - t->start;
+      const auto start = buf + t->start;
       printf("Token %u", it + 1);
       if (t->type == NamelistType::OBJECT)
         {
@@ -266,11 +268,11 @@ NamelistParser::dump(const char *buf)
 int
 NamelistParser::verify()
 {
-  unsigned int ntok = this->toknext;
+  const auto ntok = this->toknext;
 
   if (ntok)
     {
-      NamelistToken *t = &this->tokens[0];
+      const auto t = &this->tokens[0];
       if (t->type != NamelistType::OBJECT && t->type != NamelistType::KEY) return -1;
     }
 
