@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,10 +17,11 @@
 #ifndef PERCENTILES_H
 #define PERCENTILES_H
 
-#include <cstddef>
+#include <string>
 
-double percentile(double *array, size_t len, double pn);
-void percentile_set_method(const char *methodstr);
-void percentile_check_number(double pn);
+void percentile_set_method(const std::string &methodstr);
+
+template <typename T>
+double percentile(T *array, size_t len, double pn);
 
 #endif /* PERCENTILES_H */
