@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,25 +18,13 @@
 #ifndef PIPE_H
 #define PIPE_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <sys/types.h>
-
-#ifdef HAVE_LIBPTHREAD
-
-#include <pthread.h>
-#include <iostream>
-
-#include <condition_variable>
 #include <mutex>
+#include <condition_variable>
 
-#include "pthread_debug.h"
+#include "field.h"
 
 struct pipe_t
 {
-
 public:
   pipe_t();
   int pipeInqVlist(int &vlistID);
@@ -49,8 +37,15 @@ public:
   int pipeInqRecord(int *varID, int *levelID);
 
   void pipeWriteRecord(double *p_data, size_t p_nmiss);
+  void pipeWriteRecord(float *p_data, size_t p_nmiss);
+  void pipeWriteRecord(Field *p_data, size_t p_nmiss);
+
   size_t pipeReadRecord(int p_vlistID, double *data, size_t *nmiss);
+  size_t pipeReadRecord(int p_vlistID, float *data, size_t *nmiss);
+  size_t pipeReadRecord(int p_vlistID, Field *data, size_t *nmiss);
+
   size_t pipeReadPipeRecord(double *data, int vlistID, size_t *p_nmiss);
+  size_t pipeReadPipeRecord(float *data, int vlistID, size_t *p_nmiss);
 
   void pipeSetName(int processID, int inputIDX);
   void close();
@@ -58,19 +53,23 @@ public:
   bool EOP;
   bool usedata;
   bool hasdata;
+  bool data_is_float;
   int nrecs;
   int varID, levelID;
   int recIDr, recIDw, tsIDr, tsIDw;
   size_t nmiss;
-  double *data;
+  double *data_d;
+  float *data_f;
 
   std::mutex m_mutex;
-  std::condition_variable tsDef, tsInq, vlistDef, isclosed;
-  std::condition_variable recDef, recInq;
-  std::condition_variable writeCond, readCond;
+  std::condition_variable tsDef_cond, tsInq_cond, vlistDef_cond, isClosed_cond;
+  std::condition_variable recDef_cond, recInq_cond;
+  std::condition_variable write_cond, read_cond;
 
   std::string name;
+
+private:
+  void wait_for_read();  
 };
-#endif
 
 #endif /* PIPE_H */
