@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,21 +15,25 @@
   GNU General Public License for more details.
 */
 
-#include <pthread.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <cdi.h>
 
-#include "pthread_debug.h"
 #include "pipeStream.h"
-#include "cdo_options.h"
 #include "cdo_output.h"
 
+#ifdef HAVE_LIBPTHREAD
+
+#include "pthread_debug.h"
+
 PipeStream::PipeStream(int p_processID)
 {
   ispipe = true;
   m_pipe = std::make_shared<pipe_t>();
   m_pipe->pipeSetName(p_processID, m_cdoStreamID);
   m_name = m_pipe->name;
-
 }
 
 int
@@ -39,10 +43,11 @@ PipeStream::openRead()
   isopen = true;
   return m_cdoStreamID;
 }
+
 int
 PipeStream::openWrite(int filetype)
 {
-  Debug(PSTREAM, "pipe %s", m_pipe->name);
+  Debug(PIPE_STREAM, "pipe %s", m_pipe->name);
 
   wthreadID = pthread_self();
   m_filetype = filetype;
@@ -54,7 +59,7 @@ PipeStream::openWrite(int filetype)
 int
 PipeStream::openAppend()
 {
-  cdoWarning("Operator does not suport pipes!");
+  cdoWarning("Operator does not support pipes!");
   return -1;
 }
 
@@ -62,19 +67,19 @@ int
 PipeStream::inqVlist()
 {
   // m_vlist is changed when the whlist was successfully defined by another pipe!!
-  int vlistID = m_pipe->pipeInqVlist(m_vlistID);
-  //
-
+  auto vlistID = m_pipe->pipeInqVlist(m_vlistID);
   if (vlistID == -1) cdoAbort("Couldn't read data from input stream %s!", m_name.c_str());
   return vlistID;
 }
+
 void
 PipeStream::defVlist(int p_vlistID)
 {
-  Debug(PSTREAM, "%s pstreamID %d", m_pipe->name, m_cdoStreamID);
-  int vlistIDcp = vlistDuplicate(p_vlistID);
+  Debug(PIPE_STREAM, "%s pstreamID %d", m_pipe->name, m_cdoStreamID);
+  auto vlistIDcp = vlistDuplicate(p_vlistID);
   m_pipe->pipeDefVlist(m_vlistID, vlistIDcp);
 }
+
 void
 PipeStream::inqRecord(int *varID, int *levelID)
 {
@@ -88,15 +93,27 @@ PipeStream::defRecord(int varID, int levelID)
 }
 
 void
-PipeStream::readRecord(float *data, size_t *nmiss)
+PipeStream::readRecord(float *p_data, size_t *p_nmiss)
 {
-  cdoAbort("pipeReadRecord not implemented for memtype float!");
+  m_nvals += m_pipe->pipeReadRecord(m_vlistID, p_data, p_nmiss);
 }
 
 void
-PipeStream::readRecord(double *data, size_t *nmiss)
+PipeStream::readRecord(double *p_data, size_t *p_nmiss)
 {
-  m_nvals += m_pipe->pipeReadRecord(m_vlistID, data, nmiss);
+  m_nvals += m_pipe->pipeReadRecord(m_vlistID, p_data, p_nmiss);
+}
+
+void
+PipeStream::readRecord(Field *p_field, size_t *p_nmiss)
+{
+  m_nvals += m_pipe->pipeReadRecord(m_vlistID, p_field, p_nmiss);
+}
+
+void
+PipeStream::writeRecord(float *p_data, size_t p_nmiss)
+{
+  m_pipe->pipeWriteRecord(p_data, p_nmiss);
 }
 
 void
@@ -106,9 +123,9 @@ PipeStream::writeRecord(double *p_data, size_t p_nmiss)
 }
 
 void
-PipeStream::writeRecord(float *p_data, size_t p_nmiss)
+PipeStream::writeRecord(Field *p_field, size_t p_nmiss)
 {
-  cdoAbort("pipeWriteRecord not implemented for memtype float!");
+  m_pipe->pipeWriteRecord(p_field, p_nmiss);
 }
 
 void
@@ -119,19 +136,21 @@ PipeStream::copyRecord(CdoStreamID p_destination)
   // Cdi handles this. And in cdo we would have to decompress and recompress for copy operations
   // which is very resource intensive (also lossy compression)
   cdoWarning("Copy Record not possible with piped streams");
-
 }
 
 int
 PipeStream::inqTimestep(int p_tsID)
 {
-  return m_pipe->pipeInqTimestep(p_tsID);
+  auto nrecs = m_pipe->pipeInqTimestep(p_tsID);
+  m_tsID = p_tsID;
+  Debug(PIPE_STREAM, "PipeStream: Current TsID: %d,  nrecs: %d", m_tsID, nrecs);
+  return nrecs;
 }
 
 void
 PipeStream::defTimestep(int p_tsID)
 {
-  Debug(PSTREAM, "%s pstreamID %d", m_pipe->name, m_cdoStreamID);
+  Debug(PIPE_STREAM, "%s pstreamID %d", m_pipe->name, m_cdoStreamID);
   m_pipe->pipeDefTimestep(m_vlistID, p_tsID);
 }
 
@@ -140,6 +159,7 @@ PipeStream::inqFileType()
 {
   return m_filetype;
 }
+
 int
 PipeStream::inqByteorder()
 {
@@ -153,17 +173,17 @@ PipeStream::waitForPipe()
   std::unique_lock<std::mutex> locked_mutex(m_pipe->m_mutex);
   while (isopen)
     {
-      Debug(PSTREAM, "wait of read close");
-      pthread_cond_wait(m_pipe->isclosed, locked_mutex);
+      Debug(PIPE_STREAM, "wait of read close");
+      m_pipe->isClosed_cond.wait(locked_mutex);
     }
-  locked_mutex.unlock();
 }
+
 void
 PipeStream::close()
 {
-  pthread_t threadID = pthread_self();
+  auto threadID = pthread_self();
 
-  Debug(PSTREAM, "thID: %d rthID: %d wthID: %d", threadID, rthreadID, wthreadID);
+  Debug(PIPE_STREAM, "thID: %d rthID: %d wthID: %d", threadID, rthreadID, wthreadID);
 
   if (pthread_equal(threadID, rthreadID))
     {
@@ -186,3 +206,5 @@ PipeStream::getNvals()
 {
   return m_nvals;
 }
+
+#endif
