@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,11 +18,11 @@
 #ifndef PIPESTREAM_H
 #define PIPESTREAM_H
 
-#include <memory>
-
 #include "cdoStream.h"
 #include "pipe.h"
 
+#ifdef HAVE_LIBPTHREAD
+
 class FileStream;  // Predeclaration only for copyRecord(...)
 
 class PipeStream : public CdoStream
@@ -43,10 +43,14 @@ public:
   void inqRecord(int *varID, int *levelID);
   void defRecord(int varID, int levelID);
 
-  void readRecord(double *data, size_t *nmiss);
-  void readRecord(float *data, size_t *nmiss);
-  void writeRecord(double *data, size_t nmiss);
-  void writeRecord(float *data, size_t nmiss);
+  void readRecord(float *p_data, size_t *nmiss);
+  void readRecord(double *p_data, size_t *nmiss);
+  void readRecord(Field *p_field, size_t *nmiss);
+
+  void writeRecord(float *p_data, size_t nmiss);
+  void writeRecord(double *p_data, size_t nmiss);
+  void writeRecord(Field *p_field, size_t nmiss);
+
   void copyRecord(CdoStreamID p_fileStream);
 
   int inqTimestep(int tsID);
@@ -66,9 +70,10 @@ public:
 private:
   PipeStream() = delete;
   std::shared_ptr<pipe_t> m_pipe;
-  pthread_t rthreadID; /* read  thread ID */
-  pthread_t wthreadID; /* write thread ID */
+  pthread_t rthreadID; // read  thread ID
+  pthread_t wthreadID; // write thread ID
   void waitForPipe();
 };
+#endif
 
 #endif
