@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,8 +15,8 @@
   GNU General Public License for more details.
 */
 
-#include <string.h>
-#include <stdlib.h>
+#include <cstring>
+#include <cstdlib>
 #include <vector>
 
 #include "pmlist.h"
@@ -38,25 +38,26 @@ KVList::print() const
 }
 
 int
-KVList::parseArguments(const int argc, const char **argv)
+KVList::parseArguments(const int argc, const std::vector<std::string> &argv)
 {
   // Assume key = value pairs. That is, if argv[i] contains no '=' then treat it as if it belongs to the values of argv[i-1].
   char key[256];
   int i = 0;
   while (i < argc)
     {
-      const char *end = strchr(argv[i], '=');
+      const char *currentArgv = argv[i].c_str();
+      const char *end = strchr(currentArgv, '=');
       if (end == nullptr)
         {
-          fprintf(stderr, "Missing '=' in key/value string: >%s<\n", argv[i]);
+          fprintf(stderr, "Missing '=' in key/value string: >%s<\n", currentArgv);
           return -1;
         }
 
-      snprintf(key, sizeof(key), "%.*s", (int) (end - argv[i]), argv[i]);
+      snprintf(key, sizeof(key), "%.*s", (int) (end - currentArgv), currentArgv);
       key[sizeof(key) - 1] = 0;
 
       int j = 1;
-      while (i + j < argc && strchr(argv[i + j], '=') == nullptr) j++;
+      while (i + j < argc && strchr(argv[i + j].c_str(), '=') == nullptr) j++;
 
       int nvalues = j;
 
@@ -89,78 +90,39 @@ KVList::search(const std::string &key) const
   return nullptr;
 }
 
-keyValues_t *
-kvlist_search(list_t *kvlist, const char *key)
+char *
+KVList::get_first_value(const char *key, const char *replacer)
 {
-  if (key)
-    {
-      listNode_t *node = kvlist->head;
-      while (node)
-        {
-          keyValues_t *kv = *(keyValues_t **) node->data;
-          if (kv->key && *(kv->key) == *key && strcmp(kv->key, key) == 0) return kv;
-          node = node->next;
-        }
-    }
-
-  return nullptr;
+  auto kv = this->search(key);
+  if (kv && kv->nvalues > 0) return (char *) kv->values[0].c_str();
+  if (replacer)
+    return (char *) replacer;
+  else
+    return nullptr;
 }
 
 void
-free_keyval(void *data)
-{
-  keyValues_t *keyval = *(keyValues_t **) data;
-  if (keyval->key) free(keyval->key);
-  int nvalues = keyval->nvalues;
-  for (int i = 0; i < nvalues; ++i)
-    if (keyval->values[i]) free(keyval->values[i]);
-  free(keyval->values);
-  free(keyval);
-}
-
-void
-free_kvlist(void *data)
-{
-  list_t *kvlist = *(list_t **) data;
-  // int n = list_size(kvlist);
-  list_destroy(kvlist);
-  // printf("Successfully freed %d keyvalues...\n", n);
-}
-
-list_t *
-kvlist_new(const char *name)
-{
-  return list_new(sizeof(keyValues_t *), free_keyval, name);
-}
-
-void
-kvlist_destroy(list_t *list)
-{
-  list_destroy(list);
-}
-
-void
-kvlist_append(list_t *kvlist, const char *key, const char **values, int nvalues)
-{
-  keyValues_t *keyval = (keyValues_t *) malloc(sizeof(keyValues_t));
-  keyval->key = strdup(key);
-  keyval->nvalues = nvalues;
-  keyval->values = (char **) malloc(nvalues * sizeof(char *));
-  for (int i = 0; i < nvalues; ++i) keyval->values[i] = strdup(values[i]);
-  list_append(kvlist, &keyval);
-}
-
-void
-KVList::append(const char *key, const char **values, int nvalues)
+KVList::append(const char *key, const char *const *values, int nvalues)
 {
   KeyValues kv;
   kv.key = strdup(key);
   kv.nvalues = nvalues;
   kv.values.resize(nvalues);
-  for (int i = 0; i < nvalues; ++i) kv.values[i] = values[i];
+  for (int i = 0; i < nvalues; ++i) kv.values[i] = strdup(values[i]);
   this->push_back(kv);
 }
 
+// Remove only one list item
+void
+KVList::remove(const std::string &inkey)
+{
+  std::list<KeyValues>::iterator i;
+  for (i = this->begin(); i != this->end(); ++i)
+    if (i->key == inkey) break;
+  if (i->key == inkey)
+    this->erase(i);
+}
+
 const KVList *
 PMList::searchKVListVentry(const std::string &key, const std::string &value, const std::vector<std::string> &entry)
 {
@@ -169,7 +131,7 @@ PMList::searchKVListVentry(const std::string &key, const std::string &value, con
       for (const auto &s : entry)
         if (kvlist.name == s)
           {
-            const KeyValues *kv = kvlist.search(key);
+            auto kv = kvlist.search(key);
             if (kv && kv->nvalues > 0 && kv->values[0] == value) return &kvlist;
           }
     }
@@ -190,7 +152,7 @@ PMList::getKVListVentry(const std::vector<std::string> &entry)
 }
 
 static void
-KVListAppendNamelist(KVList &kvlist, const char *key, const char *buffer, NamelistToken *t, int nvalues)
+KVListAppendNamelist(KVList &kvlist, const char *key, const char *buffer, NamelistToken *t, int nvalues, bool cdocmor)
 {
   char vbuf[4096];
   std::vector<char> value;
@@ -202,17 +164,37 @@ KVListAppendNamelist(KVList &kvlist, const char *key, const char *buffer, Nameli
   for (int i = 0; i < nvalues; ++i)
     {
       const size_t len = t[i].end - t[i].start;
+      /** CMOR_MAX_STRING cannot be used **/
+      if (cdocmor && len > 1024) cdoAbort("A string value is larger than the maximum size allowed by CMOR (1024 signs).");
+
       value.resize(len + 1);
       // printf(" value[%d] >%.*s<\n", i, (int)len, buffer+t[i].start);
-      const char *pval = buffer + t[i].start;
+      auto pval = buffer + t[i].start;
+
       if (len < sizeof(vbuf))  // snprintf seems to call strlen(pval)
         {
           memcpy(vbuf, buffer + t[i].start, len);
           vbuf[len] = 0;
           pval = vbuf;
         }
-      snprintf(value.data(), len + 1, "%.*s", (int) len, pval);
-      value[len] = 0;
+
+      if (cdocmor && strcmp(key, "code") == 0 && !strstr(pval, ",") )
+        {
+          value.resize(4);
+          const auto code = atol(pval);
+          if (code > 0 && code < 1000)
+            snprintf(value.data(), 4, "%03ld", code);
+          else
+            {
+              cdoWarning("In parsing a line of a file:\n          "
+                         "Codes could not be transformed into the code format (three digit integer). Codes wont be used.");
+            }
+        }
+      else
+        {
+          snprintf(value.data(), len + 1, "%.*s", (int) len, pval);
+          value[len] = 0;
+        }
       kv.values[i] = value.data();
     }
 
@@ -226,25 +208,42 @@ getNumberOfValues(const int ntok, const NamelistToken *tokens)
 
   for (it = 0; it < ntok; ++it)
     {
-      NamelistType type = tokens[it].type;
+      auto type = tokens[it].type;
       if (type != NamelistType::WORD && type != NamelistType::STRING) break;
     }
 
   return it;
 }
 
-static int
-parseNamelist(PMList &pmlist, NamelistParser &parser, char *buf)
+static void
+replaceName(char *name)
+{
+  for ( size_t pos = 0; pos < strlen(name); pos++ )
+    {
+      name[pos] = tolower(name[pos]);
+    }
+
+  if (strcmp(name, "conventions") == 0) strcpy(name, "Conventions");
+  if (strcmp(name, "cn") == 0) strcpy(name, "cmor_name");
+  if (strcmp(name, "c") == 0) strcpy(name, "code");
+  if (strcmp(name, "n") == 0) strcpy(name, "name");
+  if (strcmp(name, "pmt") == 0) strcpy(name, "project_mip_table");
+  if (strcmp(name, "cordex_domain") == 0) strcpy(name, "CORDEX_domain");
+  if (strcmp(name, "char_axis_landuse") == 0) strcpy(name, "char_axis_landUse");
+}
+
+int
+parseNamelist(PMList &pmlist, NamelistParser &parser, char *buf, bool cdocmor)
 {
   char name[4096];
   KVList kvlist;
-  std::vector<NamelistToken> &tokens = parser.tokens;
-  const int ntok = parser.toknext;
+  auto &tokens = parser.tokens;
+  const auto ntok = parser.toknext;
   // printf("Number of tokens %d\n", ntok);
 
-  for (int it = 0; it < ntok; ++it)
+  for (unsigned it = 0; it < ntok; ++it)
     {
-      const NamelistToken &t = tokens[it];
+      const auto &t = tokens[it];
       // printf("Token %u", it+1);
       if (t.type == NamelistType::OBJECT)
         {
@@ -252,8 +251,8 @@ parseNamelist(PMList &pmlist, NamelistParser &parser, char *buf)
           if (it + 1 < ntok && tokens[it + 1].type == NamelistType::WORD)
             {
               it++;
-              const NamelistToken &t = tokens[it];
-              snprintf(name, sizeof(name), "%.*s", t.end - t.start, buf + t.start);
+              const auto &t2 = tokens[it];
+              snprintf(name, sizeof(name), "%.*s", t2.end - t2.start, buf + t2.start);
               name[sizeof(name) - 1] = 0;
             }
 
@@ -270,10 +269,19 @@ parseNamelist(PMList &pmlist, NamelistParser &parser, char *buf)
           // printf(" key >%.*s<\n", t.end - t.start, buf+t.start);
           snprintf(name, sizeof(name), "%.*s", t.end - t.start, buf + t.start);
           name[sizeof(name) - 1] = 0;
+          const auto nvalues = getNumberOfValues(ntok - it - 1, &tokens[it + 1]);
 
-          const int nvalues = getNumberOfValues(ntok - it - 1, &tokens[it + 1]);
-          // printf("nvalues %d\n", nvalues);
-          KVListAppendNamelist(kvlist, name, buf, &tokens[it + 1], nvalues);
+          if (cdocmor)
+            {
+              if (nvalues == 0)
+                {
+                  cdoWarning("Could not find values for key '%s'.", name);
+                  continue;
+                }
+              replaceName(name);
+            }
+
+          KVListAppendNamelist(kvlist, name, buf, &tokens[it + 1], nvalues, cdocmor);
           it += nvalues;
         }
       else
@@ -289,26 +297,25 @@ parseNamelist(PMList &pmlist, NamelistParser &parser, char *buf)
 }
 
 int
-parseListBuffer(PMList &pmlist, ListBuffer &listBuffer)
+parseListBuffer(NamelistParser &p, ListBuffer &listBuffer)
 {
   const char *errMsg = "Namelist error";
-  const char *name = listBuffer.name.c_str();
+  const auto name = listBuffer.name.c_str();
 
-  NamelistParser p;
-  const NamelistError status = p.parse(listBuffer.buffer.data(), listBuffer.buffer.size());
+  const auto status = p.parse(listBuffer.buffer.data(), listBuffer.buffer.size());
   if (status != NamelistError::UNDEFINED)
     {
       switch (status)
         {
         case NamelistError::INVAL:
-          fprintf(stderr, "%s: Invalid character in %s (line=%d character='%c')!\n", errMsg, name, p.lineno,
-                  listBuffer.buffer[p.pos]);
+          fprintf(stderr, "%s: Invalid character in %s (line=%d character='%c' dec=%u)!\n", errMsg, name, p.lineno,
+                  listBuffer.buffer[p.pos], (unsigned char)listBuffer.buffer[p.pos]);
           break;
         case NamelistError::PART: fprintf(stderr, "%s: End of string not found in %s (line=%d)!\n", errMsg, name, p.lineno); break;
-        case NamelistError::INKEY: fprintf(stderr, "%s: Invalid key word in %s (line=%d)!\n", errMsg, name, p.lineno); break;
-        case NamelistError::INTYP: fprintf(stderr, "%s: Invalid key word type in %s (line=%d)!\n", errMsg, name, p.lineno); break;
+        case NamelistError::INKEY: fprintf(stderr, "%s: Invalid keyword in %s (line=%d)!\n", errMsg, name, p.lineno); break;
+        case NamelistError::INTYP: fprintf(stderr, "%s: Invalid keyword type in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         case NamelistError::INOBJ: fprintf(stderr, "%s: Invalid object in %s (line=%d)!\n", errMsg, name, p.lineno); break;
-        case NamelistError::EMKEY: fprintf(stderr, "%s: Emtry key name in %s (line=%d)!\n", errMsg, name, p.lineno); break;
+        case NamelistError::EMKEY: fprintf(stderr, "%s: Empty key name in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         default: fprintf(stderr, "%s in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         }
       cdoAbort("%s!", errMsg);
@@ -321,8 +328,6 @@ parseListBuffer(PMList &pmlist, ListBuffer &listBuffer)
       cdoAbort("Namelist error!");
     }
 
-  parseNamelist(pmlist, p, listBuffer.buffer.data());
-
   return 0;
 }
 
@@ -332,8 +337,11 @@ PMList::readNamelist(FILE *fp, const char *name)
   ListBuffer listBuffer;
   if (listBuffer.read(fp, name)) cdoAbort("Read error on namelist %s!", name);
 
-  const int status = parseListBuffer(*this, listBuffer);
+  NamelistParser p;
+  const auto status = parseListBuffer(p, listBuffer);
   if (status) cdoAbort("Namelist not found!");
+
+  parseNamelist(*this, p, listBuffer.buffer.data(), false);
 }
 
 void
@@ -341,7 +349,7 @@ PMList::print()
 {
   for (const auto &kvlist : *this)
     {
-      const char *listname = kvlist.name.c_str();
+      const auto listname = kvlist.name.c_str();
       printf("\nFound %s list with %zu key/values: \n", listname ? listname : "", kvlist.size());
       kvlist.print();
     }
