@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -20,27 +20,39 @@
 #include <list>
 #include <vector>
 #include <string>
+
 #include "listbuffer.h"
+#include "namelist.h"
 
 struct KeyValues
 {
-  int nvalues;
+  int nvalues = 0;
   std::string key;
   std::vector<std::string> values;
 };
 
-class KVList : public std::list<KeyValues>
+class // KVList
+#ifdef __GNUG__
+[[gnu::warn_unused]]
+#endif
+KVList : public std::list<KeyValues>
 {
 public:
   std::string name;
 
   void print() const;
-  int parseArguments(const int argc, const char **argv);
+  int parseArguments(const int argc, const std::vector<std::string> &argv);
   const KeyValues *search(const std::string &key) const;
-  void append(const char *, const char **, int);
+  void remove(const std::string &inkey);
+  void append(const char *, const char *const *, int);
+  char *get_first_value(const char *key, const char *replacer);
 };
 
-class PMList : public std::list<KVList>
+class // PMList
+#ifdef __GNUG__
+[[gnu::warn_unused]]
+#endif
+PMList : public std::list<KVList>
 {
 public:
   const KVList *searchKVListVentry(const std::string &key, const std::string &value, const std::vector<std::string> &entry);
@@ -50,24 +62,7 @@ public:
   void readCmortable(FILE *fp, const char *name);
 };
 
-#include "list.h"
-
-typedef struct
-{
-  int nvalues;
-  char *key;
-  char **values;
-} keyValues_t;
-
-keyValues_t *kvlist_search(list_t *kvlist, const char *key);
-
-void free_keyval(void *data);
-void free_kvlist(void *data);
-
-list_t *kvlist_new(const char *name);
-void kvlist_destroy(list_t *list);
-void kvlist_append(list_t *kvlist, const char *key, const char **values, int nvalues);
-
-int parseListBuffer(PMList &pmlist, ListBuffer &listBuffer);
+int parseNamelist(PMList &pmlist, NamelistParser &parser, char *buf, bool cdocmor);
+int parseListBuffer(NamelistParser &p, ListBuffer &listBuffer);
 
 #endif
