@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -19,9 +19,10 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H 
 #include <pthread.h>
 #endif
+
 #ifdef _OPENMP
 #include <omp.h>
 #endif
@@ -32,29 +33,33 @@
 #include <sstream>
 
 #include "process.h"
-#include "cdo_output.h"
-#include "readline.h"
-
 #include "cdo_wtime.h"
-
-#include "dmemory.h"
-#include "pthread.h"
 #include "util_string.h"
-#include "util_files.h"
-#include "mpmo_color.h"
 #include "cdo_options.h"
 #include "fileStream.h"
 #include "pipeStream.h"
 
-Process::Process(int p_ID, const char *p_operatorName, const char *operatorCommand) : m_ID(p_ID), operatorName(p_operatorName)
+static int processNum = 0;
+
+int
+getProcessNum()
 {
-  initProcess();
-  setOperatorArgv(operatorCommand);
-  m_operatorCommand = operatorCommand;
+  return processNum;
+}
 
-  defPrompt();  // has to be called after get operatorName
+void
+setProcessNum(int p_num)
+{
+  processNum = p_num;
+}
 
-  m_module = getModule(p_operatorName);
+Process::Process(int p_ID, const char *p_operatorName, const char *p_operatorCommand) : m_ID(p_ID), operatorName(p_operatorName)
+{
+  m_isActive = true;
+
+  startTime = cdo_get_wtime();
+
+  initProcess(p_operatorName, p_operatorCommand);
 }
 
 void
@@ -62,45 +67,52 @@ Process::setOperatorArgv(const char *operatorArguments)
 {
   if (operatorArguments)
     {
-      char *operatorArg = strdup(operatorArguments);
-      // fprintf(stderr, "processDefArgument: %d %s\n", oargc, operatorArg);
-      Debug(PROCESS && strchr(operatorArg, ',') != nullptr, "Setting operator arguments: %s", operatorArguments);
+      m_oargv.clear();
+      Debug(PROCESS && strchr(operatorArguments, ',') != nullptr, "Setting operator arguments: %s", operatorArguments);
 
-      while ((operatorArg = strchr(operatorArg, ',')) != nullptr)
+      std::istringstream f(operatorArguments);
+      std::string s;
+      std::string argument = "";
+      getline(f, s, ',');
+      while (getline(f, s, ','))
         {
-          *operatorArg = '\0';
-          operatorArg++;
-          if (strlen(operatorArg))
+          argument = s;
+          while (argument.back() == '\\')
             {
-              m_oargv.push_back(operatorArg);
+              argument.back() = ',';
+              getline(f, s, ',');
+              argument.append(s);
             }
+
+          Debug(PROCESS, "Adding argument %s", argument);
+          m_oargv.push_back(argument);
         }
     }
 
   m_oargc = m_oargv.size();
 }
 
+bool
+Process::inputIsVariable()
+{
+  return m_module.streamInCnt == -1;
+}
+
 void
-Process::initProcess()
+Process::initProcess(const char *p_operatorName, const char *operatorCommand)
 {
 #ifdef HAVE_LIBPTHREAD
   threadID = pthread_self();
 #endif
-  m_isActive = true;
-  nchild = 0;
 
-  startTime = cdo_get_wtime();
+  setOperatorArgv(operatorCommand);
+  m_operatorCommand = operatorCommand;
 
-  nvars = 0;
-  ntimesteps = 0;
+  defPrompt();  // has to be called after get operatorName
 
-  m_streamCnt = 0;
-
-  m_oargc = 0;
-  m_operatorCommand = "UNINITALIZED";
-  operatorArg = "UNINITALIZED";
-
-  m_noper = 0;
+  m_module = getModule(p_operatorName);
+  // if (m_module.func == nullptr) cdoAbort("Module for operator <%s> not implemented!\n", p_operatorName);
+  operatorName = p_operatorName;
 }
 
 int
@@ -119,9 +131,9 @@ void
 Process::defPrompt()
 {
   if (m_ID == 0)
-    snprintf(prompt, sizeof(prompt), "%s    %s", Cdo::progname, operatorName);
+    snprintf(prompt, sizeof(prompt), "%s    %s", cdo::progname, operatorName);
   else
-    snprintf(prompt, sizeof(prompt), "%s(%d) %s", Cdo::progname, m_ID, operatorName);
+    snprintf(prompt, sizeof(prompt), "%s(%d) %s", cdo::progname, m_ID, operatorName);
 }
 
 const char *
@@ -131,9 +143,9 @@ Process::inqPrompt() const
 }
 
 void
-Process::handleProcessErr(ProcessStatus p_proErr)
+Process::handleProcessErr()
 {
-  switch (p_proErr)
+  switch (m_status)
     {
     case ProcessStatus::UnlimitedIOCounts:
       {
@@ -153,19 +165,18 @@ Process::handleProcessErr(ProcessStatus p_proErr)
     case ProcessStatus::TooManyStreams:
     case ProcessStatus::TooFewStreams:
       {
-        const int inCnt = m_module.streamInCnt;
-        int outCnt = m_module.streamOutCnt;
+        const auto inCnt = m_module.streamInCnt;
+        auto outCnt = m_module.streamOutCnt;
         const bool lobase = outCnt == -1;
         if (lobase) outCnt = 1;
 
-        std::string caseCount = ((p_proErr == ProcessStatus::TooManyStreams) ? "many" : "few");
-        std::string pluralIn = ((inCnt > 1) ? "s" : "");
-        std::string pluralOut = ((outCnt > 1) ? "s" : "");
+        std::string caseCount = (m_status == ProcessStatus::TooManyStreams) ? "many" : "few";
+        std::string pluralIn = (inCnt > 1) ? "s" : "";
+        std::string pluralOut = (outCnt > 1) ? "s" : "";
 
         std::stringstream errMsg;
-        errMsg << "Too " << caseCount << " streams specified! Operator " << m_operatorCommand << " needs " << inCnt
-               << " input stream" << pluralIn << " and " << outCnt << " output " << (lobase ? "basename" : "stream") << pluralOut
-               << "!";
+        errMsg << "Too " << caseCount << " streams specified! Operator " << operatorName << " needs " << inCnt << " input stream"
+               << pluralIn << " and " << outCnt << " output " << (lobase ? "basename" : "stream") << pluralOut << "!";
         cdoAbort(errMsg.str());
         break;
       }
@@ -176,11 +187,11 @@ Process::handleProcessErr(ProcessStatus p_proErr)
 void
 Process::validate()
 {
-  const ProcessStatus processStatus = checkStreamCnt();
-  if (processStatus != ProcessStatus::Ok) handleProcessErr(processStatus);
+  checkStreamCnt();
+  if (m_status != ProcessStatus::Ok) handleProcessErr();
 }
 
-ProcessStatus
+void
 Process::checkStreamCnt()
 {
   int streamCnt = 0;
@@ -197,35 +208,42 @@ Process::checkStreamCnt()
       obase = true;
     }
 
-  if (wantedStreamInCnt == -1 && wantedStreamOutCnt == -1) return ProcessStatus::UnlimitedIOCounts;
+  if (wantedStreamInCnt == -1 && wantedStreamOutCnt == -1)
+    {
+      m_status = ProcessStatus::UnlimitedIOCounts;
+    }
 
   // printf(" wantedStreamInCnt,wantedStreamOutCnt %d %d\n",
   // wantedStreamInCnt,wantedStreamOutCnt);
-  if (wantedStreamInCnt == -1)
+  else if (wantedStreamInCnt == -1)
     {
       wantedStreamInCnt = m_streamCnt - wantedStreamOutCnt;
-      if (wantedStreamInCnt < 1) return ProcessStatus::MissInput;
+      if (wantedStreamInCnt < 1) m_status = ProcessStatus::MissInput;
     }
 
-  if (wantedStreamOutCnt == -1)
+  else if (wantedStreamOutCnt == -1)
     {
       wantedStreamOutCnt = m_streamCnt - wantedStreamInCnt;
-      if (wantedStreamOutCnt < 1) return ProcessStatus::MissOutput;
+      if (wantedStreamOutCnt < 1) m_status = ProcessStatus::MissOutput;
     }
-  // printf(" wantedStreamInCnt,wantedStreamOutCnt %d %d\n",
-  // wantedStreamInCnt,wantedStreamOutCnt);
+  else 
+    {
+      // printf(" wantedStreamInCnt,wantedStreamOutCnt %d %d\n",
+      // wantedStreamInCnt,wantedStreamOutCnt);
 
-  streamCnt = wantedStreamInCnt + wantedStreamOutCnt;
-  // printf(" streamCnt %d %d\n", m_streamCnt, streamCnt);
+      streamCnt = wantedStreamInCnt + wantedStreamOutCnt;
+      // printf(" streamCnt %d %d\n", m_streamCnt, streamCnt);
 
-  if (m_streamCnt > streamCnt) return ProcessStatus::TooManyStreams;
+      if (m_streamCnt > streamCnt)
+        m_status = ProcessStatus::TooManyStreams;
+      else if (m_streamCnt < streamCnt && !obase)
+        m_status = ProcessStatus::TooFewStreams;
+      else if (wantedStreamInCnt > (int) inputStreams.size())
+        m_status = ProcessStatus::TooFewStreams;
 
-  if (m_streamCnt < streamCnt && !obase) return ProcessStatus::TooFewStreams;
-  if (wantedStreamInCnt > (int) inputStreams.size()) return ProcessStatus::TooFewStreams;
-
-  if (wantedStreamInCnt == 1 && streamInCnt0 == -1) return ProcessStatus::Ok;
-
-  return ProcessStatus::Ok;
+      else if (wantedStreamInCnt == 1 && streamInCnt0 == -1)
+        m_status = ProcessStatus::Ok;
+    }
 }
 
 bool
@@ -242,66 +260,6 @@ Process::setInactive()
   m_isActive = false;
 }
 
-void
-Process::inqUserInputForOpArg(const char *enter)
-{
-  int oargc = m_oargc;
-
-  if (oargc == 0)
-    {
-      char line[1024];
-      char *pline = line;
-      size_t pos, len, linelen;
-      int lreadline = 1;
-
-      if (enter)
-        {
-          set_text_color(stderr, BRIGHT, MAGENTA);
-          fprintf(stderr, "%-16s : ", prompt);
-          reset_text_color(stderr);
-          // set_text_color(stderr, BLINK, BLACK);
-          fprintf(stderr, "Enter %s > ", enter);
-          // reset_text_color(stderr);
-        }
-
-      while (lreadline)
-        {
-          readline(stdin, pline, 1024);
-
-          lreadline = 0;
-          while (1)
-            {
-
-              pos = 0;
-              while (pline[pos] == ' ' || pline[pos] == ',') pos++;
-              pline += pos;
-              linelen = strlen(pline);
-              if (linelen > 0)
-                {
-                  if (pline[0] == '\\')
-                    {
-                      lreadline = 1;
-                      break;
-                    }
-                  len = 0;
-                  while (pline[len] != ' ' && pline[len] != ',' && pline[len] != '\\' && len < linelen) len++;
-
-                  m_oargv.push_back((char *) Malloc(len + 1));
-                  memcpy(m_oargv[oargc], pline, len);
-                  m_oargv[oargc][len] = '\0';
-                  oargc++;
-
-                  pline += len;
-                }
-              else
-                break;
-            }
-        }
-
-      m_oargc = oargc;
-    }
-}
-
 int
 Process::operatorAdd(const char *name, int f1, int f2, const char *enter)
 {
@@ -309,10 +267,7 @@ Process::operatorAdd(const char *name, int f1, int f2, const char *enter)
 
   if (operID >= MAX_OPERATOR) cdoAbort("Maximum number of %d operators reached!", MAX_OPERATOR);
 
-  oper[m_noper].f1 = f1;
-  oper[m_noper].f2 = f2;
-  oper[m_noper].name = name;
-  oper[m_noper].enter = enter;
+  oper[m_noper] = { f1, f2, name, enter };
 
   m_noper++;
 
@@ -335,7 +290,7 @@ Process::getOperatorID()
         }
       if (operID == m_noper)
         {
-          cdoAbort("%s Operator not callable by this name! Name is: %s", prompt, operatorName);
+          cdoAbort("Operator not callable by this name! Name is: %s", operatorName);
         }
     }
   else
@@ -390,6 +345,7 @@ Process::addParent(Process *parentProcess)
   m_posInParent = parentProcess->inputStreams.size() - 1;
   addPipeOutStream();
 }
+
 void
 Process::addPipeOutStream()
 {
@@ -402,28 +358,24 @@ Process::run()
 {
   Debug(PROCESS, "starting new thread for process %d", m_ID);
   pthread_attr_t attr;
-  int status = pthread_attr_init(&attr);
+  auto status = pthread_attr_init(&attr);
   if (status) cdoSysError("pthread_attr_init failed for '%s'", operatorName);
   status = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
   if (status) cdoSysError("pthread_attr_setdetachstate failed for '%s'", operatorName);
   /*
     param.sched_priority = 0;
     status = pthread_attr_setschedparam(&attr, &param);
-    if ( status ) cdoSysError("pthread_attr_setschedparam failed for '%s'",
-    newarg+1);
+    if ( status ) cdoSysError("pthread_attr_setschedparam failed for '%s'", newarg+1);
   */
   /* status = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED); */
-  /* if ( status ) cdoSysError("pthread_attr_setinheritsched failed for '%s'",
-   * newarg+1); */
+  /* if ( status ) cdoSysError("pthread_attr_setinheritsched failed for '%s'", newarg+1); */
 
   int pthreadScope;
   pthread_attr_getscope(&attr, &pthreadScope);
 
   /* status = pthread_attr_setscope(&attr, PTHREAD_SCOPE_PROCESS); */
-  /* if ( status ) cdoSysError("pthread_attr_setscope failed for '%s'", newarg+1);
-   */
-  /* If system scheduling scope is specified, then the thread is scheduled
-   * against all threads in the system */
+  /* if ( status ) cdoSysError("pthread_attr_setscope failed for '%s'", newarg+1); */
+  /* If system scheduling scope is specified, then the thread is scheduled against all threads in the system */
   /* pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM); */
 
   size_t stacksize = 0;
@@ -435,7 +387,7 @@ Process::run()
     }
 
   pthread_t thrID;
-  const int rval = pthread_create(&thrID, &attr, m_module.func, this);
+  const auto rval = pthread_create(&thrID, &attr, m_module.func, this);
   if (rval != 0)
     {
       errno = rval;
@@ -450,12 +402,14 @@ Process::run()
 void
 Process::printBenchmarks(double p_walltime, const char *p_memstring)
 {
-#ifdef HAVE_SYS_TIMES_H
   if (m_ID == 0)
     {
-      fprintf(stderr, " [%.2fs%s]", p_walltime, p_memstring);
+      const auto number_of_used_thread = getProcessNum();
+      if (ITSME)
+        fprintf(stderr, " [%.2fs%s %dthread%s]", p_walltime, p_memstring, number_of_used_thread, ADD_PLURAL(number_of_used_thread));
+      else
+        fprintf(stderr, " [%.2fs%s]", p_walltime, p_memstring);
     }
-#endif
 }
 
 // local helper function
@@ -463,7 +417,7 @@ extern "C" size_t getPeakRSS();
 static void
 getMaxMemString(char *p_memstring, size_t memstringLen)
 {
-  size_t memmax = getPeakRSS();
+  auto memmax = getPeakRSS();
   if (memmax)
     {
       size_t muindex = 0;
@@ -477,6 +431,7 @@ getMaxMemString(char *p_memstring, size_t memstringLen)
       snprintf(p_memstring, memstringLen, " %zu%s", memmax, mu[muindex]);
     }
 }
+
 void
 Process::printProcessedValues()
 {
@@ -484,26 +439,18 @@ Process::printProcessedValues()
   fprintf(stderr, "%s: ", prompt);
   reset_text_color(stderr);
 
-  const int64_t nvals = inqNvals();
+  const auto nvals = inqNvals();
 
   if (nvals > 0)
     {
-      if (sizeof(int64_t) > sizeof(size_t))
-#ifdef _WIN32
-        fprintf(stderr, "Processed %I64d value%s from %d variable%s",
-#else
-        fprintf(stderr, "Processed %jd value%s from %d variable%s",
-#endif
-                (intmax_t) nvals, ADD_PLURAL(nvals), nvars, ADD_PLURAL(nvars));
-      else
-        fprintf(stderr, "Processed %zu value%s from %d variable%s", (size_t) nvals, ADD_PLURAL(nvals), nvars, ADD_PLURAL(nvars));
+      fprintf(stderr, "Processed %zu value%s from %d variable%s", nvals, ADD_PLURAL(nvals), nvars, ADD_PLURAL(nvars));
     }
   else if (nvars > 0)
     {
       fprintf(stderr, "Processed %d variable%s", nvars, ADD_PLURAL(nvars));
     }
 
-  if (ntimesteps > 0) fprintf(stderr, " over %d timestep%s", ntimesteps, ADD_PLURAL(ntimesteps));
+  if ((nvals || nvars) && ntimesteps > 0) fprintf(stderr, " over %d timestep%s", ntimesteps, ADD_PLURAL(ntimesteps));
 
   char memstring[32] = { "" };
   if (m_ID == 0) getMaxMemString(memstring, sizeof(memstring));
@@ -516,7 +463,7 @@ Process::printProcessedValues()
 bool
 Process::hasOutStream(const CdoStreamID p_streamID)
 {
-  for (const CdoStreamID streamID : outputStreams)
+  for (const CdoStreamID &streamID : outputStreams)
     {
       if (streamID == p_streamID) return true;
     }
@@ -526,7 +473,7 @@ Process::hasOutStream(const CdoStreamID p_streamID)
 bool
 Process::hasInStream(const CdoStreamID p_streamID)
 {
-  for (const CdoStreamID streamID : inputStreams)
+  for (const CdoStreamID &streamID : inputStreams)
     {
       if (streamID == p_streamID) return true;
     }
@@ -538,13 +485,14 @@ Process::getCommand()
 {
   return m_operatorCommand + 1;
 }
+
 size_t
 Process::inqNvals()
 {
   size_t nvals = 0;
   for (size_t i = 0; i < inputStreams.size(); i++)
     {
-    Debug(PROCESS, "Inquiring nvlas from instream %s", inputStreams[i]->m_name);
+      Debug(PROCESS, "Inquiring nvals from instream %s", inputStreams[i]->m_name);
       nvals += inputStreams[i]->getNvals();
     }
   return nvals;
@@ -556,8 +504,28 @@ Process::hasNoPipes()
   if (childProcesses.size() == 0) return true;
   return false;
 }
+
 const char *
 Process::getOutStreamName()
 {
   return outputStreams[0]->m_name.c_str();
 }
+
+size_t
+Process::getOperArgc()
+{
+  return m_oargc;
+}
+
+std::string
+Process::getArgv(int p_idx)
+{
+  if (!(p_idx > m_oargc && p_idx > 0)) cdoAbort("Process Argv not found. Idx: %d, Process argc: %d", p_idx, m_oargv.size());
+  return m_oargv[p_idx];
+}
+
+std::string
+Process::getObase()
+{
+  return m_obase;
+}
