@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 #ifndef PROCESS_H
 #define PROCESS_H
 
-#include "pstream.h"
+#include "cdoStream.h"
 #include "modules.h"
 
 #include <vector>
@@ -26,7 +26,7 @@
 #include <string>
 #include <set>
 
-constexpr int MAX_PROCESS = 128;
+constexpr int MAX_PROCESS = 65536;
 constexpr int MAX_OPERATOR = 128;
 
 enum class ProcessStatus
@@ -39,12 +39,16 @@ enum class ProcessStatus
   TooFewStreams = -5,
 };
 
-struct oper_t
+class oper_t
 {
-  int f1;
-  int f2;
-  const char *name;
-  const char *enter;
+public:
+  int f1 = 0;
+  int f2 = 0;
+  const char *name = nullptr;
+  const char *enter = nullptr;
+
+  oper_t() {}
+  oper_t(int _f1, int _f2, const char *_name, const char *_enter) : f1(_f1), f2(_f2), name(_name), enter(_enter) {}
 };
 
 class Process
@@ -55,7 +59,7 @@ public:
 #ifdef HAVE_LIBPTHREAD
   pthread_t threadID;
 #endif
-  short nchild;
+  short nchild = 0;
   std::vector<Process *> childProcesses;
   std::vector<Process *> parentProcesses;
   std::vector<CdoStreamID> inputStreams;
@@ -64,28 +68,27 @@ public:
 
   double startTime;
 
-  short nvars;
+  short nvars = 0;
 
-  int ntimesteps;
-  int m_streamCnt;
-  char const *m_operatorCommand;
+  int ntimesteps = 0;
+  int m_streamCnt = 0;
+  char const *m_operatorCommand = "UNINITALIZED";
   char const *operatorName;
-  char const *operatorArg;
-  char prompt[64];
-  short m_noper;
-  bool m_isActive;
   const char *m_obase;
+  char prompt[64];
+
+  short m_noper = 0;
+  bool m_isActive;
 
   module_t m_module;
-  std::vector<char *> m_oargv;
-  /*TEMP*/  // remove m_oargc and replace usage with
-            // m_oargv.size()/operatorArgc()
-  int m_oargc;
+  std::vector<std::string> m_oargv;
+  int m_oargc = 0;
   oper_t oper[MAX_OPERATOR];
 
   Process(int p_ID, const char *p_operatorNamme, const char *operatorCommand);
 
   pthread_t run();
+  ProcessStatus m_status = ProcessStatus::Ok;
 
   /**
    * returns the number of in streams this process currently has.
@@ -100,12 +103,7 @@ public:
    * returns full command through which this process was created including operator name and operator arguments
    */
   const char *getCommand();
-  /**
-   * Splits the operator arguments stores them.
-   * Operator arguments are stored as char arrays and appended with
-   * a zero termination.
-   */
-  void setOperatorArgv(const char *operatorArguments);
+
   /**
    * Adds a Process as child and creates and adds a new pipe stream.
    */
@@ -136,7 +134,6 @@ public:
    * Adds and creates a new file pstream to the out streams
    */
   void addPipeOutStream();
-  void inqUserInputForOpArg(const char *enter);
   /**
    * Adds an operator to the process
    */
@@ -149,20 +146,33 @@ public:
   const char *inqPrompt() const;
   void printProcessedValues();
   void printBenchmarks(double p_walltime, const char *p_memstring);
-  ProcessStatus checkStreamCnt();
+  void checkStreamCnt();
   void validate();
-  void handleProcessErr(ProcessStatus p_proErr);
+  void handleProcessErr();
 
   bool hasOutStream(const CdoStreamID p_streamPtr);
   bool hasInStream(const CdoStreamID p_streamPtr);
   bool hasNoPipes();
   size_t inqNvals();
-  const char * getOutStreamName();
+  const char *getOutStreamName();
+  size_t getOperArgc();
+  std::string getArgv(int idx);
+  std::string getObase();
+  bool inputIsVariable();
+
+  void initProcess(const char *p_operatorName, const char *operatorCommand);
 
 private:
-  void initProcess();
   Process();
   void defPrompt();
+  /**
+   * Splits the operator arguments stores them.
+   * Operator arguments are stored as char arrays and appended with a zero termination.
+   */
+  void setOperatorArgv(const char *operatorArguments);
 };
 
+int getProcessNum();
+void setProcessNum(int p_num);
+
 #endif /* PROCESS_H */
