@@ -6,10 +6,10 @@
 #include "fileStream.h"
 
 #include <stack>
+#include <mutex>
+
+static std::mutex processMutex;
 
-#ifdef HAVE_LIBPTHREAD
-static pthread_mutex_t processMutex = PTHREAD_MUTEX_INITIALIZER;
-#endif
 static std::string parse_err_msg = "";
 
 void
@@ -45,12 +45,34 @@ ProcessManager::killProcesses()
     }
 }
 
+int
+ProcessManager::getNumVariableInputOperators()
+{
+  int num = 0;
+  for (auto &process : m_processes)
+    {
+      if (process.second.inputIsVariable()) num++;
+    }
+  return num;
+}
+
 void
 ProcessManager::validateProcesses()
 {
   for (auto &process : m_processes)
     {
-      process.second.validate();
+      process.second.checkStreamCnt();
+      auto stat = process.second.m_status;
+      if (stat != ProcessStatus::Ok)
+        {
+          if (stat == ProcessStatus::TooFewStreams || stat == ProcessStatus::TooManyStreams)
+            if (getNumVariableInputOperators() > 1)
+              {
+                cdoWarning("Did you forget to use '[' and/or ']' for multiple variable input operators?");
+                cdoWarning("use option --variableInput, for description");
+              }
+          process.second.handleProcessErr();
+        }
     }
 }
 
@@ -66,7 +88,7 @@ ProcessManager::clearProcesses()
 Process *
 ProcessManager::createProcess(const char *command)
 {
-  Debug(PROCESS, "Creating new process for command: %s with ID: %d ", command, m_numProcesses);
+  Debug(PROCESS_MANAGER, "Creating new process for command: %s with ID: %d ", command, m_numProcesses);
   int processID = m_numProcesses++;
 
   const char *operatorName = get_original(command);
@@ -77,7 +99,7 @@ ProcessManager::createProcess(const char *command)
     }
 
   m_numProcessesActive++;
-  Debug(PROCESS, "m_numProcessesActive: %d", m_numProcessesActive);
+  Debug(PROCESS_MANAGER, "m_numProcessesActive: %d", m_numProcessesActive);
 
   if (processID >= MAX_PROCESS) cdoAbort("Limit of %d processes reached!", MAX_PROCESS);
 
@@ -87,32 +109,16 @@ ProcessManager::createProcess(const char *command)
 int
 ProcessManager::processNums(void)
 {
-#ifdef HAVE_LIBPTHREAD
-  pthread_mutex_lock(&processMutex);
-#endif
-
+  std::unique_lock<std::mutex> locked_mutex(processMutex);
   int pnums = m_processes.size();
-
-#ifdef HAVE_LIBPTHREAD
-  pthread_mutex_unlock(&processMutex);
-#endif
-
   return pnums;
 }
 
 int
 ProcessManager::processNumsActive(void)
 {
-#ifdef HAVE_LIBPTHREAD
-  pthread_mutex_lock(&processMutex);
-#endif
-
+  std::unique_lock<std::mutex> locked_mutex(processMutex);
   int pnums = m_numProcessesActive;
-
-#ifdef HAVE_LIBPTHREAD
-  pthread_mutex_unlock(&processMutex);
-#endif
-
   return pnums;
 }
 
@@ -136,18 +142,19 @@ ProcessManager::addProcess(Process *p_parentProcess, const char *argvEntry)
       cdoAbort("operator -%s can not take -%s  with 0 outputs as input", p_parentProcess->operatorName, newProcess->operatorName);
     }
 
-  Debug(PROCESS, "Adding new child %s to %s", newProcess->getCommand(), p_parentProcess->getCommand());
+  Debug(PROCESS_MANAGER, "Adding new child %s to %s", newProcess->getCommand(), p_parentProcess->getCommand());
   p_parentProcess->addChild(newProcess);
   newProcess->addParent(p_parentProcess);
   return newProcess;
 }
 
-void
+std::set<std::string>
 ProcessManager::handleFirstOperator(int p_argcStart, int argc, const char **argv, Process *p_rootProcess)
 {
+  std::set<std::string> files;
   for (int i = p_argcStart; i < argc; i++)
     {
-      Debug(PROCESS, "Creating new pstream for output file: %s", argv[i]);
+      Debug(PROCESS_MANAGER, "Creating new pstream for output file: %s", argv[i]);
       if (strcmp(argv[i], "]") == 0)
         {
           cdoAbort("missing output file");
@@ -157,7 +164,9 @@ ProcessManager::handleFirstOperator(int p_argcStart, int argc, const char **argv
           cdoAbort("missing Output file, did you add a '-' to a file?");
         }
       p_rootProcess->addFileOutStream(argv[i]);
+      files.insert(argv[i]);
     }
+  return files;
 }
 
 void
@@ -165,7 +174,7 @@ ProcessManager::checkSingleBracketOnly(const char *p_argvEntry, char p_bracketTy
 {
   if (strlen(p_argvEntry) != 1)
     {
-      cdoAbort("Only single %s allowed", p_bracketType);
+      cdoAbort("Only single %c allowed", p_bracketType);
     }
 }
 
@@ -180,6 +189,7 @@ ProcessManager::createProcesses(int argc, const char **argv)
     }
   validateProcesses();
 }
+
 #include <algorithm>
 std::string
 getStackAsString(std::stack<Process *> p_stack)
@@ -242,12 +252,14 @@ getStackAsString(std::stack<Process *> p_stack)
 ParseStatus
 ProcessManager::createProcessesFromInput(int argc, const char **argv)
 {
-  Debug(PROCESS, "== Process Creation Start ==");
-  Debug(PROCESS, "operators: %s", argvToString(argc, argv));
+  Debug(PROCESS_MANAGER, "== Process Creation Start ==");
+  Debug(PROCESS_MANAGER, "operators: %s", argvToString(argc, argv));
 
   Process *root_process = createProcess(argv[0]);
   int cntOutFiles = (int) root_process->m_module.streamOutCnt;
 
+  std::set<std::string> files;
+
   unsigned int lastNonOutputIdx = argc - cntOutFiles;
   if (cntOutFiles == -1)
     {
@@ -261,12 +273,12 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
         {
           return ParseStatus::MissingOutFile;
         }
-      handleFirstOperator(lastNonOutputIdx, argc, argv, root_process);
+      files = handleFirstOperator(lastNonOutputIdx, argc, argv, root_process);
     }
 
   Process *currentProcess;
   std::stack<Process *> processStack;
-  std::set<Process *> bracketOperators;
+  std::vector<Process *> bracketOperators;
   const char *argvEntry;
   int unclosedBrackets = 0;
   unsigned int idx = 1;
@@ -274,16 +286,16 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
   processStack.push(root_process);
   while (!processStack.empty() && idx < lastNonOutputIdx)
     {
-      Debug(PROCESS, "%s", getStackAsString(processStack));
+      Debug(PROCESS_MANAGER, "%s", getStackAsString(processStack));
       currentProcess = processStack.top();
-      Debug(PROCESS, "iteration %d, current argv:%s ,currentProcess: ", idx, argv[idx], currentProcess->m_operatorCommand);
+      Debug(PROCESS_MANAGER, "iteration %d, current argv:%s, currentProcess: ", idx, argv[idx], currentProcess->m_operatorCommand);
 
       argvEntry = argv[idx];
       //------------------------------------------------------
       // case: operator
       if (argvEntry[0] == '-')
         {
-          Debug(PROCESS, "Found new Operator: %s", argvEntry);
+          Debug(PROCESS_MANAGER, "Found new Operator: %s", argvEntry);
           currentProcess = addProcess(currentProcess, argvEntry);
           if (currentProcess->m_module.restrictions == OnlyFirst)
             {
@@ -297,7 +309,7 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
       else if (argvEntry[0] == '[')
         {
           checkSingleBracketOnly(argvEntry, '[');
-          bracketOperators.insert(currentProcess);
+          bracketOperators.push_back(currentProcess);
           unclosedBrackets++;
         }
       // - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -306,22 +318,33 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
         {
           checkSingleBracketOnly(argvEntry, ']');
           unclosedBrackets--;
-          bracketOperators.erase(processStack.top());
-          processStack.pop();
+          bracketOperators.pop_back();
+          //this check is for double bracktets like -info [ [ file1 file2 file3 ] ]  which can happen when cdo wildcards are used.
+          auto it = std::find(bracketOperators.begin(), bracketOperators.end(), processStack.top());
+          if (it == bracketOperators.end()) processStack.pop();
         }
       // - - - - - - - - - - - - - - - - - - - - - - - - - - -
       // case: file
       else if (currentProcess->m_module.streamInCnt != 0)
         {
-          Debug(PROCESS, "adding in file to %s", currentProcess->operatorName);
-          currentProcess->addFileInStream(argvEntry);
+          Debug(PROCESS_MANAGER, "adding in file to %s", currentProcess->operatorName);
+          if (files.find(argvEntry) == files.end())
+            {
+              currentProcess->addFileInStream(argvEntry);
+            }
+          else
+            {
+              parse_err_msg
+                  += std::string(argvEntry) + " is used as an output, files can not be used as in- and output at the same time";
+              return ParseStatus::FileIsInAndOutput;
+            }
         }
       // -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
       // remove finished
       while (!processStack.empty() && processStack.top()->hasAllInputs()
-             && bracketOperators.find(processStack.top()) == bracketOperators.end())
+             && std::find(bracketOperators.begin(), bracketOperators.end(),processStack.top()) == bracketOperators.end())
         {
-          Debug(PROCESS, "Removing %s from stack", processStack.top()->operatorName);
+          Debug(PROCESS_MANAGER, "Removing %s from stack", processStack.top()->operatorName);
           processStack.pop();
         }
       //------------------------------------------------------
@@ -345,7 +368,7 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
       return ParseStatus::UnprocessedInput;
     }
 
-  Debug(PROCESS, "== Process Creation End ==");
+  Debug(PROCESS_MANAGER, "== Process Creation End ==");
 
   setProcessNum(m_processes.size());
   FileStream::enableTimers(m_processes.size() == 1 && Threading::ompNumThreads == 1);
@@ -382,6 +405,14 @@ ProcessManager::handleParseError(ParseStatus p_errCode)
         cdoAbort("%s", parse_err_msg);
         break;
       }
+    case ParseStatus::MissingObase:
+      {
+        break;
+      }
+    case ParseStatus::FileIsInAndOutput:
+      {
+        cdoAbort("%s", parse_err_msg);
+      }
     case ParseStatus::Ok:
       {
         return;
@@ -392,12 +423,13 @@ ProcessManager::handleParseError(ParseStatus p_errCode)
 Process &
 ProcessManager::getProcess(int p_processID)
 {
-  pthread_mutex_lock(&processMutex);
-  auto process = m_processes.find(p_processID);
+  std::unique_lock<std::mutex> locked_mutex(processMutex);
+
+  const auto process = m_processes.find(p_processID);
   if (process == m_processes.end())
     {
       cdoAbort("Process with ID: %d not found", p_processID);
     }
-  pthread_mutex_unlock(&processMutex);
+
   return process->second;
 }
