@@ -3,7 +3,10 @@
 
 // Stdlib includes
 #include <map>
+#include <set>
+#include <string>
 #include <vector>
+#include <string>
 
 #include <pthread.h>
 
@@ -20,7 +23,10 @@ enum class ParseStatus
   ClosingBracketMissing = -2,
   UnprocessedInput = -3,
   MissingOutFile = -4,
-  MissingObase = -5
+  MissingObase = -5,
+  OperatorNotFirst = -6,
+  FileIsInAndOutput = -7
+
 };
 class ProcessManager
 {
@@ -32,9 +38,10 @@ private:
   int m_numProcesses = 0;
   int m_numProcessesActive = 0;
 
-  void handleFirstOperator(int p_argcStart, int argc, const char **argv, Process *p_rootProcess);
+  std::set<std::string> handleFirstOperator(int p_argcStart, int argc, const char **argv, Process *p_rootProcess);
   void checkSingleBracketOnly(const char *p_argvEntry, char p_bracketType);
   void handleParseError(ParseStatus p_parseStatus);
+  int getNumVariableInputOperators();
 
   Process *createProcess(const char *p_command);
   Process *addProcess(Process *p_parentProcess, const char *p_command);
@@ -49,7 +56,6 @@ public:
   int processNums();
   int processNumsActive();
   Process &getProcess(int p_processID);
-  Process &getProcessByThreadID(pthread_t p_threadID);
 };
 
 #endif
