@@ -1,18 +1,19 @@
+#include <cstdio>
+#include <algorithm>
+
 #include "progress.h"
-#include "process_int.h"
 
-#include <stdio.h>
-
-
-namespace Progress
+namespace progress
 {
-bool stdout_is_tty = 0;
+
+bool stdout_is_tty = false;
 bool silentMode = false;
 
-bool ps_lhead = false;
-int ps_nch = 0;
-int ps_cval = -1;
-const char *context = "";
+static bool ps_lhead = false;
+static int ps_nch = 0;
+static int ps_cval = -1;
+
+static const char *context = "";
 const char *(*getContext)(void) = nullptr;
 
 /**
@@ -33,16 +34,16 @@ init()
   ps_nch = 0;
   ps_cval = -1;
 
-  if (getContext != nullptr){
-      context = getContext();
-  }
+  if (getContext != nullptr) context = getContext();
 }
+
 void
-init(const char* p_context)
+init(const char *p_context)
 {
   ps_lhead = false;
   ps_nch = 0;
   ps_cval = -1;
+
   context = p_context;
 }
 
@@ -52,14 +53,11 @@ update(double offset, double refval, double curval)
   if (silentMode) return;
   if (!stdout_is_tty) return;
 
-  offset = offset < 0 ? 0 : offset;
-  offset = offset > 1 ? 1 : offset;
-  refval = refval < 0 ? 0 : refval;
-  refval = refval > 1 ? 1 : refval;
-  curval = curval < 0 ? 0 : curval;
-  curval = curval > 1 ? 1 : curval;
+  offset = std::min(std::max(offset, 0.), 1.);
+  refval = std::min(std::max(refval, 0.), 1.);
+  curval = std::min(std::max(curval, 0.), 1.);
 
-  int ival = (offset + refval * curval) * 100;
+  const int ival = (offset + refval * curval) * 100;
 
   if (ps_cval == -1)
     {
@@ -82,4 +80,5 @@ update(double offset, double refval, double curval)
       fflush(stdout);
     }
 }
-}  // namespace Progress
+
+}  // namespace progress
