@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,13 @@
 #ifndef READLINE_H
 #define READLINE_H
 
+#include <cstdio>
+
+namespace cdo
+{
+
 int readline(FILE *fp, char *line, int len);
 
+}
+
 #endif
