@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,6 @@
   GNU General Public License for more details.
 */
 
-
 #include "process_int.h"
 #include "cdo_wtime.h"
 #include "remap.h"
@@ -26,35 +25,34 @@
 
 // bilinear interpolation
 
+static inline void
+limit_dphi_bounds(double &dphi)
+{
+  if (dphi > 3.0 * PIH) dphi -= PI2;
+  if (dphi < -3.0 * PIH) dphi += PI2;
+}
+
 bool
-remapFindWeights(double plon, double plat, double *restrict src_lons, double *restrict src_lats, double *ig, double *jg)
+remapFindWeights(const LonLatPoint &llpoint, const double (&src_lons)[4], const double (&src_lats)[4], double *ig, double *jg)
 {
-  long iter;                           // iteration counters
-  double deli, delj;                   // corrections to iw,jw
-  double dthp, dphp;                   // difference between point and sw corner
-  double mat1, mat2, mat3, mat4;       // matrix elements
-  double determinant;                  // matrix determinant
   constexpr double converge = 1.e-10;  // Convergence criterion
   extern long remap_max_iter;
 
-  /* Iterate to find iw,jw for bilinear approximation  */
+  // Iterate to find iw,jw for bilinear approximation
 
   // some latitude  differences
-  double dth1 = src_lats[1] - src_lats[0];
-  double dth2 = src_lats[3] - src_lats[0];
-  double dth3 = src_lats[2] - src_lats[1] - dth2;
+  const auto dth1 = src_lats[1] - src_lats[0];
+  const auto dth2 = src_lats[3] - src_lats[0];
+  const auto dth3 = src_lats[2] - src_lats[1] - dth2;
 
   // some longitude differences
-  double dph1 = src_lons[1] - src_lons[0];
-  double dph2 = src_lons[3] - src_lons[0];
-  double dph3 = src_lons[2] - src_lons[1];
+  auto dph1 = src_lons[1] - src_lons[0];
+  auto dph2 = src_lons[3] - src_lons[0];
+  auto dph3 = src_lons[2] - src_lons[1];
 
-  if (dph1 > 3.0 * PIH) dph1 -= PI2;
-  if (dph2 > 3.0 * PIH) dph2 -= PI2;
-  if (dph3 > 3.0 * PIH) dph3 -= PI2;
-  if (dph1 < -3.0 * PIH) dph1 += PI2;
-  if (dph2 < -3.0 * PIH) dph2 += PI2;
-  if (dph3 < -3.0 * PIH) dph3 += PI2;
+  limit_dphi_bounds(dph1);
+  limit_dphi_bounds(dph2);
+  limit_dphi_bounds(dph3);
 
   dph3 = dph3 - dph2;
 
@@ -62,25 +60,25 @@ remapFindWeights(double plon, double plat, double *restrict src_lons, double *re
   double iguess = 0.5;
   double jguess = 0.5;
 
+  long iter = 0;  // iteration counters
   for (iter = 0; iter < remap_max_iter; ++iter)
     {
-      dthp = plat - src_lats[0] - dth1 * iguess - dth2 * jguess - dth3 * iguess * jguess;
-      dphp = plon - src_lons[0];
+      const auto dthp = llpoint.lat - src_lats[0] - dth1 * iguess - dth2 * jguess - dth3 * iguess * jguess;
+      auto dphp = llpoint.lon - src_lons[0];
 
-      if (dphp > 3.0 * PIH) dphp -= PI2;
-      if (dphp < -3.0 * PIH) dphp += PI2;
+      limit_dphi_bounds(dphp);
 
       dphp = dphp - dph1 * iguess - dph2 * jguess - dph3 * iguess * jguess;
 
-      mat1 = dth1 + dth3 * jguess;
-      mat2 = dth2 + dth3 * iguess;
-      mat3 = dph1 + dph3 * jguess;
-      mat4 = dph2 + dph3 * iguess;
+      const auto mat1 = dth1 + dth3 * jguess;
+      const auto mat2 = dth2 + dth3 * iguess;
+      const auto mat3 = dph1 + dph3 * jguess;
+      const auto mat4 = dph2 + dph3 * iguess;
 
-      determinant = mat1 * mat4 - mat2 * mat3;
+      const auto determinant = mat1 * mat4 - mat2 * mat3;
 
-      deli = (dthp * mat4 - dphp * mat2) / determinant;
-      delj = (dphp * mat1 - dthp * mat3) / determinant;
+      const auto deli = (dthp * mat4 - dphp * mat2) / determinant;
+      const auto delj = (dphp * mat1 - dthp * mat3) / determinant;
 
       if (std::fabs(deli) < converge && std::fabs(delj) < converge) break;
 
@@ -95,24 +93,25 @@ remapFindWeights(double plon, double plat, double *restrict src_lons, double *re
 }
 
 static void
-bilinearSetWeights(double iw, double jw, double wgts[4])
+bilinearSetWeights(double iw, double jw, double (&weights)[4])
 {
   // clang-format off
-  wgts[0] = (1.-iw) * (1.-jw);
-  wgts[1] =     iw  * (1.-jw);
-  wgts[2] =     iw  *     jw;
-  wgts[3] = (1.-iw) *     jw;
+  weights[0] = (1.0-iw) * (1.0-jw);
+  weights[1] =      iw  * (1.0-jw);
+  weights[2] =      iw  *      jw;
+  weights[3] = (1.0-iw) *      jw;
   // clang-format on
 }
 
-unsigned
-num_src_points(const std::vector<bool> &mask, const size_t src_add[4], double src_lats[4])
+template <typename T>
+int
+num_src_points(const Varray<T> &mask, const size_t (&src_add)[4], double (&src_lats)[4])
 {
-  unsigned icount = 0;
+  int icount = 0;
 
-  for (unsigned n = 0; n < 4; ++n)
+  for (int n = 0; n < 4; ++n)
     {
-      if (mask[src_add[n]])
+      if (mask[src_add[n]] != 0)
         icount++;
       else
         src_lats[n] = 0.;
@@ -121,28 +120,16 @@ num_src_points(const std::vector<bool> &mask, const size_t src_add[4], double sr
   return icount;
 }
 
-unsigned
-num_src_points(const int *restrict mask, const size_t src_add[4], double src_lats[4])
-{
-  unsigned icount = 0;
-
-  for (unsigned n = 0; n < 4; ++n)
-    {
-      if (mask[src_add[n]])
-        icount++;
-      else
-        src_lats[n] = 0.;
-    }
-
-  return icount;
-}
+// Explicit instantiation
+template int num_src_points(const Varray<uint8_t> &mask, const size_t (&src_add)[4], double (&src_lats)[4]);
+template int num_src_points(const Varray<int> &mask, const size_t (&src_add)[4], double (&src_lats)[4]);
 
 static void
-renormalizeWeights(const double src_lats[4], double wgts[4])
+renormalizeWeights(const double (&src_lats)[4], double (&weights)[4])
 {
-  double sum_wgts = 0.0;  // sum of weights for normalization
-  for (unsigned n = 0; n < 4; ++n) sum_wgts += std::fabs(src_lats[n]);
-  for (unsigned n = 0; n < 4; ++n) wgts[n] = std::fabs(src_lats[n]) / sum_wgts;
+  double sum_weights = 0.0;  // sum of weights for normalization
+  for (unsigned n = 0; n < 4; ++n) sum_weights += std::fabs(src_lats[n]);
+  for (unsigned n = 0; n < 4; ++n) weights[n] = std::fabs(src_lats[n]) / sum_weights;
 }
 
 static void
@@ -157,16 +144,15 @@ bilinearWarning()
     }
 }
 
-static void
-bilinearRemap(double *restrict tgt_point, const double *restrict src_array, const double wgts[4], const size_t src_add[4])
+static inline double
+bilinearRemap(const double *restrict src_array, const double (&weights)[4], const size_t (&src_add)[4])
 {
   // *tgt_point = 0.;
-  // for (unsigned n = 0; n < 4; ++n) *tgt_point += src_array[src_add[n]]*wgts[n];
-  *tgt_point = src_array[src_add[0]] * wgts[0] + src_array[src_add[1]] * wgts[1] + src_array[src_add[2]] * wgts[2]
-               + src_array[src_add[3]] * wgts[3];
+  // for (unsigned n = 0; n < 4; ++n) *tgt_point += src_array[src_add[n]]*weights[n];
+  return src_array[src_add[0]] * weights[0] + src_array[src_add[1]] * weights[1] + src_array[src_add[2]] * weights[2]
+         + src_array[src_add[3]] * weights[3];
 }
 
-
 // This routine computes the weights for a bilinear interpolation.
 void
 remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
@@ -178,9 +164,9 @@ remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
 
   if (src_grid->rank != 2) cdoAbort("Can't do bilinear interpolation when source grid rank != 2");
 
-  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
-  Progress::init();
+  progress::init();
 
   // Compute mappings from source to target grid
 
@@ -189,7 +175,7 @@ remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
   std::vector<WeightLinks> weightLinks(tgt_grid_size);
   weightLinksAlloc(4, tgt_grid_size, weightLinks);
 
-  double findex = 0;
+  auto findex = 0.0;
 
   // Loop over destination grid
 
@@ -202,22 +188,21 @@ remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
 #pragma omp atomic
 #endif
       findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / tgt_grid_size);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / tgt_grid_size);
 
       weightLinks[tgt_cell_add].nlinks = 0;
 
       if (!tgt_grid->mask[tgt_cell_add]) continue;
 
-      double plon = 0, plat = 0;
-      remapgrid_get_lonlat(tgt_grid, tgt_cell_add, &plon, &plat);
+      const auto llpoint = remapgrid_get_lonlat(tgt_grid, tgt_cell_add);
 
-      size_t src_add[4];   //  address for the four source points
       double src_lats[4];  //  latitudes  of four bilinear corners
       double src_lons[4];  //  longitudes of four bilinear corners
-      double wgts[4];      //  bilinear weights for four corners
+      double weights[4];   //  bilinear weights for four corners
+      size_t src_add[4];   //  address for the four source points
 
       // Find nearest square of grid points on source grid
-      int search_result = remapSearchSquare(rsearch, plon, plat, src_add, src_lats, src_lons);
+      auto search_result = remapSearchSquare(rsearch, llpoint, src_add, src_lats, src_lons);
 
       // Check to see if points are mask points
       if (search_result > 0)
@@ -229,14 +214,14 @@ remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
       // If point found, find local iw,jw coordinates for weights
       if (search_result > 0)
         {
-          tgt_grid->cell_frac[tgt_cell_add] = 1.;
+          tgt_grid->cell_frac[tgt_cell_add] = 1.0;
 
-          double iw, jw;  // current guess for bilinear coordinate
-          if (remapFindWeights(plon, plat, src_lons, src_lats, &iw, &jw))
+          double iw = 0.0, jw = 0.0;  // current guess for bilinear coordinate
+          if (remapFindWeights(llpoint, src_lons, src_lats, &iw, &jw))
             {
               // Successfully found iw,jw - compute weights
-              bilinearSetWeights(iw, jw, wgts);
-              storeWeightLinks(0, 4, src_add, wgts, tgt_cell_add, weightLinks);
+              bilinearSetWeights(iw, jw, weights);
+              store_weightlinks(0, 4, src_add, weights, tgt_cell_add, weightLinks);
             }
           else
             {
@@ -251,23 +236,22 @@ remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
       */
       if (search_result < 0)
         {
-          if (num_src_points(&src_grid->mask[0], src_add, src_lats) > 0)
+          if (num_src_points(src_grid->mask, src_add, src_lats) > 0)
             {
-              tgt_grid->cell_frac[tgt_cell_add] = 1.;
-              renormalizeWeights(src_lats, wgts);
-              storeWeightLinks(0, 4, src_add, wgts, tgt_cell_add, weightLinks);
+              tgt_grid->cell_frac[tgt_cell_add] = 1.0;
+              renormalizeWeights(src_lats, weights);
+              store_weightlinks(0, 4, src_add, weights, tgt_cell_add, weightLinks);
             }
         }
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   weightLinksToRemapLinks(0, tgt_grid_size, weightLinks, rv);
 
   if (Options::cdoVerbose) cdoPrint("Square search nearest: %.2f seconds", cdo_get_wtime() - start);
 }  // remapBilinearWeights
 
-
 // This routine computes and apply the weights for a bilinear interpolation.
 void
 remapBilinear(RemapSearch &rsearch, const double *restrict src_array, double *restrict tgt_array, double missval)
@@ -279,29 +263,28 @@ remapBilinear(RemapSearch &rsearch, const double *restrict src_array, double *re
 
   if (src_grid->rank != 2) cdoAbort("Can't do bilinear interpolation when source grid rank != 2");
 
-  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
-  Progress::init();
+  progress::init();
 
   auto tgt_grid_size = tgt_grid->size;
   auto src_grid_size = src_grid->size;
 
-  std::vector<bool> src_grid_mask(src_grid_size);
+  Varray<uint8_t> src_grid_mask(src_grid_size);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) schedule(static) shared(src_grid_size, src_array, src_grid_mask, missval)
 #endif
-  for (size_t i = 0; i < src_grid_size; ++i)
-    src_grid_mask[i] = !DBL_IS_EQUAL(src_array[i], missval);
+  for (size_t i = 0; i < src_grid_size; ++i) src_grid_mask[i] = !DBL_IS_EQUAL(src_array[i], missval);
 
   // Compute mappings from source to target grid
 
-  double findex = 0;
+  auto findex = 0.0;
 
   // Loop over destination grid
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) schedule(static) \
-  shared(findex, rsearch, tgt_grid_size, src_grid, tgt_grid, src_array, tgt_array, missval, src_grid_mask)
+    shared(findex, rsearch, tgt_grid_size, src_grid, tgt_grid, src_array, tgt_array, missval, src_grid_mask)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
@@ -309,42 +292,41 @@ remapBilinear(RemapSearch &rsearch, const double *restrict src_array, double *re
 #pragma omp atomic
 #endif
       findex++;
-      if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / tgt_grid_size);
+      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, findex / tgt_grid_size);
 
       tgt_array[tgt_cell_add] = missval;
 
       if (!tgt_grid->mask[tgt_cell_add]) continue;
 
-      double plon = 0, plat = 0;
-      remapgrid_get_lonlat(tgt_grid, tgt_cell_add, &plon, &plat);
+      const auto llpoint = remapgrid_get_lonlat(tgt_grid, tgt_cell_add);
 
-      size_t src_add[4];   //  address for the four source points
       double src_lats[4];  //  latitudes  of four bilinear corners
       double src_lons[4];  //  longitudes of four bilinear corners
-      double wgts[4];      //  bilinear weights for four corners
+      double weights[4];   //  bilinear weights for four corners
+      size_t src_add[4];   //  address for the four source points
 
       // Find nearest square of grid points on source grid
-      int search_result = remapSearchSquare(rsearch, plon, plat, src_add, src_lats, src_lons);
+      auto search_result = remapSearchSquare(rsearch, llpoint, src_add, src_lats, src_lons);
 
       // Check to see if points are mask points
       if (search_result > 0)
         {
           for (unsigned n = 0; n < 4; ++n)
-            if (!src_grid_mask[src_add[n]]) search_result = 0;
+            if (src_grid_mask[src_add[n]] == 0) search_result = 0;
         }
 
       // If point found, find local iw,jw coordinates for weights
       if (search_result > 0)
         {
-          tgt_grid->cell_frac[tgt_cell_add] = 1.;
+          tgt_grid->cell_frac[tgt_cell_add] = 1.0;
 
-          double iw, jw;  // current guess for bilinear coordinate
-          if (remapFindWeights(plon, plat, src_lons, src_lats, &iw, &jw))
+          double iw = 0.0, jw = 0.0;  // current guess for bilinear coordinate
+          if (remapFindWeights(llpoint, src_lons, src_lats, &iw, &jw))
             {
               // Successfully found iw,jw - compute weights
-              bilinearSetWeights(iw, jw, wgts);
-              sort_add_and_wgts(4, src_add, wgts);
-              bilinearRemap(&tgt_array[tgt_cell_add], src_array, wgts, src_add);
+              bilinearSetWeights(iw, jw, weights);
+              sort_weights_n4(src_add, weights);
+              tgt_array[tgt_cell_add] = bilinearRemap(src_array, weights, src_add);
             }
           else
             {
@@ -361,15 +343,15 @@ remapBilinear(RemapSearch &rsearch, const double *restrict src_array, double *re
         {
           if (num_src_points(src_grid_mask, src_add, src_lats) > 0)
             {
-              tgt_grid->cell_frac[tgt_cell_add] = 1.;
-              renormalizeWeights(src_lats, wgts);
-              sort_add_and_wgts(4, src_add, wgts);
-              bilinearRemap(&tgt_array[tgt_cell_add], src_array, wgts, src_add);
+              tgt_grid->cell_frac[tgt_cell_add] = 1.0;
+              renormalizeWeights(src_lats, weights);
+              sort_weights_n4(src_add, weights);
+              tgt_array[tgt_cell_add] = bilinearRemap(src_array, weights, src_add);
             }
         }
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   if (Options::cdoVerbose) cdoPrint("Square search nearest: %.2f seconds", cdo_get_wtime() - start);
 }  // remapBilinear
