@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "cdo_wtime.h"
 #include "remap.h"
 #include "remap_store_link_cnsrv.h"
@@ -98,8 +98,6 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
   double s1, s2, determ;                     /*  variables used for linear solve to   */
   double mat1, mat2, mat3, mat4, rhs1, rhs2; /* find intersection */
 
-  double *srch_corner_x; /*  x of each corner of srch cells */
-  double *srch_corner_y; /*  y of each corner of srch cells */
 
   /*printf("pole_intersection: %g %g %g %g\n", beglat, beglon, endlat,
    * endlon);*/
@@ -116,8 +114,8 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
 
   /* Convert coordinates */
 
-  srch_corner_x = (double *) Malloc(srch_corners * num_srch_cells * sizeof(double));
-  srch_corner_y = (double *) Malloc(srch_corners * num_srch_cells * sizeof(double));
+  std::vector<double> srch_corner_x(srch_corners * num_srch_cells);
+  std::vector<double> srch_corner_y(srch_corners * num_srch_cells);
 
   if (beglat > ZERO)
     {
@@ -137,26 +135,26 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
     }
   else
     {
-      x1 = rns * TWO * sin(pi4 - HALF * beglat) * cos(beglon);
-      y1 = TWO * sin(pi4 - HALF * beglat) * sin(beglon);
+      x1 = rns * TWO * std::sin(pi4 - HALF * beglat) * std::cos(beglon);
+      y1 = TWO * std::sin(pi4 - HALF * beglat) * std::sin(beglon);
       *luse_last = true;
     }
 
-  x2 = rns * TWO * sin(pi4 - HALF * endlat) * cos(endlon);
-  y2 = TWO * sin(pi4 - HALF * endlat) * sin(endlon);
+  x2 = rns * TWO * std::sin(pi4 - HALF * endlat) * std::cos(endlon);
+  y2 = TWO * std::sin(pi4 - HALF * endlat) * std::sin(endlon);
 
   for (n = 0; n < srch_corners * num_srch_cells; ++n)
     {
-      srch_corner_x[n] = rns * TWO * sin(pi4 - HALF * srch_corner_lat[n]) * cos(srch_corner_lon[n]);
-      srch_corner_y[n] = TWO * sin(pi4 - HALF * srch_corner_lat[n]) * sin(srch_corner_lon[n]);
+      srch_corner_x[n] = rns * TWO * std::sin(pi4 - HALF * srch_corner_lat[n]) * std::cos(srch_corner_lon[n]);
+      srch_corner_y[n] = TWO * std::sin(pi4 - HALF * srch_corner_lat[n]) * std::sin(srch_corner_lon[n]);
     }
 
   begx = x1;
   begy = y1;
   endx = x2;
   endy = y2;
-  begsegx = rns * TWO * sin(pi4 - HALF * begseg[0]) * cos(begseg[1]);
-  begsegy = TWO * sin(pi4 - HALF * begseg[0]) * sin(begseg[1]);
+  begsegx = rns * TWO * std::sin(pi4 - HALF * begseg[0]) * std::cos(begseg[1]);
+  begsegy = TWO * std::sin(pi4 - HALF * begseg[0]) * std::sin(begseg[1]);
   *intrsct_x = endx;
   *intrsct_y = endy;
 
@@ -287,8 +285,6 @@ pole_intersection(long *location, double *intrsct_lat, double *intrsct_lon, bool
 
       if (s1 >= ONE)
         {
-          Free(srch_corner_y);
-          Free(srch_corner_x);
           *luse_last = false;
           return;
         }
@@ -392,9 +388,9 @@ after_srch_loop:
                   if (*intrsct_lon < ZERO) *intrsct_lon = *intrsct_lon + PI2;
 
                   if (std::fabs(*intrsct_x) > 1.e-10)
-                    *intrsct_lat = (pi4 - asin(rns * HALF * (*intrsct_x) / cos(*intrsct_lon))) * TWO;
+                    *intrsct_lat = (pi4 - std::asin(rns * HALF * (*intrsct_x) / std::cos(*intrsct_lon))) * TWO;
                   else if (std::fabs(*intrsct_y) > 1.e-10)
-                    *intrsct_lat = (pi4 - asin(HALF * (*intrsct_y) / sin(*intrsct_lon))) * TWO;
+                    *intrsct_lat = (pi4 - std::asin(HALF * (*intrsct_y) / std::sin(*intrsct_lon))) * TWO;
                   else
                     *intrsct_lat = TWO * pi4;
 
@@ -432,9 +428,6 @@ after_srch_loop:
 
     } /* intrsct_loop */
 
-  Free(srch_corner_y);
-  Free(srch_corner_x);
-
   /*
      If segment manages to cross over pole, shift the beginning
      endpoint in order to avoid hitting pole directly
@@ -979,10 +972,10 @@ line_integral(double *weights, double in_phi1, double in_phi2, double theta1, do
 
   /*  Weights for the general case based on a trapezoidal approx to the integrals. */
 
-  sinth1 = sin(theta1);
-  sinth2 = sin(theta2);
-  costh1 = cos(theta1);
-  costh2 = cos(theta2);
+  sinth1 = std::sin(theta1);
+  sinth2 = std::sin(theta2);
+  costh1 = std::cos(theta1);
+  costh2 = std::cos(theta2);
 
   dphi = in_phi1 - in_phi2;
   if (dphi > PI)
@@ -1014,8 +1007,8 @@ line_integral(double *weights, double in_phi1, double in_phi2, double theta1, do
 } /* line_integral */
 
 static void
-correct_pole(RemapGrid *src_grid, RemapGrid *tgt_grid, RemapVars &rv, double *src_centroid_lat, double *src_centroid_lon,
-             double *tgt_centroid_lat, double *tgt_centroid_lon, grid_store_t *grid_store)
+correct_pole(RemapGrid *src_grid, RemapGrid *tgt_grid, RemapVars &rv, std::vector<double> &src_centroid_lat, std::vector<double> &src_centroid_lon,
+             std::vector<double> &tgt_centroid_lat, std::vector<double> &tgt_centroid_lon, GridStore &grid_store)
 {
   /*
      Correct for situations where N/S pole not explicitly included in
@@ -1136,66 +1129,89 @@ correct_pole(RemapGrid *src_grid, RemapGrid *tgt_grid, RemapVars &rv, double *sr
 static inline void
 norm_weight(double norm_factor, double *weights, double src_centroid_lat, double src_centroid_lon)
 {
-  double weight0 = weights[0];
-
+  const auto weight0 = weights[0];
   weights[0] = weight0 * norm_factor;
   weights[1] = (weights[1] - weight0 * src_centroid_lat) * norm_factor;
   weights[2] = (weights[2] - weight0 * src_centroid_lon) * norm_factor;
 }
 
 static void
-normalize_weights(RemapGrid *tgt_grid, RemapVars &rv, double *src_centroid_lat, double *src_centroid_lon)
+normalize_weights(RemapGrid *tgt_grid, RemapVars &rv, std::vector<double> &src_centroid_lat, std::vector<double> &src_centroid_lon)
 {
   /* Include centroids in weights and normalize using destination area if requested */
   long num_links = rv.num_links;
-  long src_cell_add; /* current linear address for source grid cell   */
-  long tgt_cell_add; /* current linear address for target grid cell   */
+  // src_cell_add: current linear address for source grid cell
+  // tgt_cell_add: current linear address for target grid cell
   double *weights = &rv.wts[0];
-  double norm_factor = 0; /* factor for normalizing wts */
 
   if (rv.normOpt == NormOpt::DESTAREA)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon) private(src_cell_add, tgt_cell_add, norm_factor)
+#pragma omp parallel for default(none) shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon)
 #endif
       for (long n = 0; n < num_links; ++n)
         {
-          src_cell_add = rv.src_cell_add[n];
-          tgt_cell_add = rv.tgt_cell_add[n];
-          norm_factor = IS_NOT_EQUAL(tgt_grid->cell_area[tgt_cell_add], 0) ? ONE / tgt_grid->cell_area[tgt_cell_add] : ZERO;
-          norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_cell_add], src_centroid_lon[src_cell_add]);
+          const auto src_add = rv.src_cell_add[n];
+          const auto tgt_add = rv.tgt_cell_add[n];
+          const auto tgt_cell_area = tgt_grid->cell_area[tgt_add];
+          const auto norm_factor = IS_NOT_EQUAL(tgt_cell_area, 0) ? ONE / tgt_cell_area : ZERO;
+          norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_add], src_centroid_lon[src_add]);
         }
     }
   else if (rv.normOpt == NormOpt::FRACAREA)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon) private(src_cell_add, tgt_cell_add, norm_factor)
+#pragma omp parallel for default(none) shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon)
 #endif
       for (long n = 0; n < num_links; ++n)
         {
-          src_cell_add = rv.src_cell_add[n];
-          tgt_cell_add = rv.tgt_cell_add[n];
-          norm_factor = IS_NOT_EQUAL(tgt_grid->cell_frac[tgt_cell_add], 0) ? ONE / tgt_grid->cell_frac[tgt_cell_add] : ZERO;
-          norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_cell_add], src_centroid_lon[src_cell_add]);
+          const auto src_add = rv.src_cell_add[n];
+          const auto tgt_add = rv.tgt_cell_add[n];
+          const auto tgt_cell_frac = tgt_grid->cell_frac[tgt_add];
+          const auto norm_factor = IS_NOT_EQUAL(tgt_cell_frac, 0) ? ONE / tgt_cell_frac : ZERO;
+          // printf("i: %ld %g  %g %g  %g %g %g\n", n+1, norm_factor, src_centroid_lon[src_add], src_centroid_lat[src_add],
+          // weights[n * 3], weights[n * 3+1], weights[n * 3+2]);
+          norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_add], src_centroid_lon[src_add]);
+          // printf("o: %ld %g  %g %g  %g %g %g\n", n+1, norm_factor, src_centroid_lon[src_add], src_centroid_lat[src_add],
+          // weights[n * 3], weights[n * 3+1], weights[n * 3+2]);
         }
     }
   else if (rv.normOpt == NormOpt::NONE)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon) private(src_cell_add, norm_factor)
+#pragma omp parallel for default(none) shared(num_links, rv, weights, tgt_grid, src_centroid_lat, src_centroid_lon)
 #endif
       for (long n = 0; n < num_links; ++n)
         {
-          src_cell_add = rv.src_cell_add[n];
-          norm_factor = ONE;
-          norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_cell_add], src_centroid_lon[src_cell_add]);
+          const auto src_add = rv.src_cell_add[n];
+          const auto norm_factor = ONE;
+          norm_weight(norm_factor, &weights[n * 3], src_centroid_lat[src_add], src_centroid_lon[src_add]);
         }
     }
 }
 
+static void
+srvout(const char *fname, int nlon, int nlat, double *data)
+{
+  int ihead[8];
+
+  ihead[0] = 99;
+  ihead[1] = 0;
+  ihead[2] = 0;
+  ihead[3] = 0;
+  ihead[4] = nlon;
+  ihead[5] = nlat;
+  ihead[6] = 0;
+  ihead[7] = 0;
+
+  FILE *fp = fopen(fname, "w");
+  fprintf(fp, "%d %d %d %d %d %d %d %d\n", ihead[0], ihead[1], ihead[2], ihead[3], ihead[4], ihead[5], ihead[6], ihead[7]);
+
+  for (int i = 0; i < nlon * nlat; ++i) fprintf(fp, "%g\n", data[i]);
+
+  fclose(fp);
+}
+
 /*
   -----------------------------------------------------------------------
 
@@ -1223,11 +1239,11 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  Progress::init();
+  progress::init();
 
   long num_wts = rv.num_wts;
 
-  grid_store_t *grid_store = (grid_store_t *) Malloc(sizeof(grid_store_t));
+  GridStore grid_store;
   grid_store_init(grid_store, tgt_grid->size);
 
   if (Options::cdoVerbose)
@@ -1246,10 +1262,14 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 
   /* Initialize centroid arrays */
 
-  double *src_centroid_lat = (double *) Malloc(src_grid_size * sizeof(double));
-  double *src_centroid_lon = (double *) Malloc(src_grid_size * sizeof(double));
-  double *tgt_centroid_lat = (double *) Malloc(tgt_grid_size * sizeof(double));
-  double *tgt_centroid_lon = (double *) Malloc(tgt_grid_size * sizeof(double));
+  //double *src_centroid_lat = (double *) Malloc(src_grid_size * sizeof(double));
+  //double *src_centroid_lon = (double *) Malloc(src_grid_size * sizeof(double));
+  //double *tgt_centroid_lat = (double *) Malloc(tgt_grid_size * sizeof(double));
+  //double *tgt_centroid_lon = (double *) Malloc(tgt_grid_size * sizeof(double));
+  std::vector<double> src_centroid_lat (src_grid_size);
+  std::vector<double> src_centroid_lon (src_grid_size);
+  std::vector<double> tgt_centroid_lat (tgt_grid_size);
+  std::vector<double> tgt_centroid_lon (tgt_grid_size);
 
   for (long n = 0; n < src_grid_size; ++n)
     {
@@ -1289,13 +1309,13 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 #endif
   for (long src_cell_add = 0; src_cell_add < src_grid_size; ++src_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) Progress::update(0, 0.5, findex / src_grid_size);
+      if (ompthID == 0) progress::update(0, 0.5, findex / src_grid_size);
 
       bool lcoinc;  // flag for coincident segments
       bool lthresh = false;
@@ -1485,13 +1505,13 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
 #endif
   for (long tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) Progress::update(0.5, 0.5, findex / tgt_grid_size);
+      if (ompthID == 0) progress::update(0.5, 0.5, findex / tgt_grid_size);
 
       bool lcoinc;  // flag for coincident segments
       bool lthresh = false;
@@ -1649,7 +1669,7 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
         }
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   /* Finished with all cells: deallocate search arrays */
 
@@ -1670,18 +1690,23 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
   correct_pole(src_grid, tgt_grid, rv, src_centroid_lat, src_centroid_lon, tgt_centroid_lat, tgt_centroid_lon, grid_store);
 
   grid_store_delete(grid_store);
-  Free(grid_store);
 
   if (rv.num_links != rv.max_links) remapVarsResize(rv, rv.num_links);
 
   /* Finish centroid computation */
 
+  // int nlon = src_grid->dims[0];
+  // int nlat = src_grid->dims[1];
+  // srvout("clat1.srv", nlon, nlat, src_centroid_lat);
+  // srvout("clon1.srv", nlon, nlat, src_centroid_lon);
   for (long n = 0; n < src_grid_size; ++n)
     if (IS_NOT_EQUAL(src_grid->cell_area[n], 0))
       {
         src_centroid_lat[n] /= src_grid->cell_area[n];
         src_centroid_lon[n] /= src_grid->cell_area[n];
       }
+  // srvout("clat2.srv", nlon, nlat, src_centroid_lat);
+  // srvout("clon2.srv", nlon, nlat, src_centroid_lon);
 
   for (long n = 0; n < tgt_grid_size; ++n)
     if (IS_NOT_EQUAL(tgt_grid->cell_area[n], 0))
@@ -1793,10 +1818,5 @@ remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv)
         }
     }  // lcheck
 
-  Free(src_centroid_lat);
-  Free(src_centroid_lon);
-  Free(tgt_centroid_lat);
-  Free(tgt_centroid_lon);
-
   if (Options::cdoVerbose) cdoPrint("Cells search: %.2f seconds", cdo_get_wtime() - start);
 }  // remapConservWeightsScrip
