@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "cdo_wtime.h"
 #include "remap.h"
 #include "remap_store_link.h"
@@ -29,16 +29,16 @@
 void
 remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
 {
-  RemapGrid *src_grid = rsearch.srcGrid;
-  RemapGrid *tgt_grid = rsearch.tgtGrid;
+  auto src_grid = rsearch.srcGrid;
+  auto tgt_grid = rsearch.tgtGrid;
 
   if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  Progress::init();
+  progress::init();
 
   // Compute mappings from source to target grid
 
-  size_t tgt_grid_size = tgt_grid->size;
+  auto tgt_grid_size = tgt_grid->size;
 
   std::vector<WeightLinks> weightLinks(tgt_grid_size);
   weightLinksAlloc(numNeighbors, tgt_grid_size, weightLinks);
@@ -46,11 +46,11 @@ remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
-  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
   // Loop over destination grid
 
-  double findex = 0;
+  double findex = 0.0;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
@@ -58,38 +58,36 @@ remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) Progress::update(0, 1, findex / tgt_grid_size);
+      if (ompthID == 0) progress::update(0, 1, findex / tgt_grid_size);
 
       weightLinks[tgt_cell_add].nlinks = 0;
 
       if (!tgt_grid->mask[tgt_cell_add]) continue;
 
-      double plon = 0, plat = 0;
-      remapgrid_get_lonlat(tgt_grid, tgt_cell_add, &plon, &plat);
+      const auto llpoint = remapgrid_get_lonlat(tgt_grid, tgt_cell_add);
 
       // Find nearest grid points on source grid and distances to each point
-      remapSearchPoints(rsearch, plon, plat, knnWeights[ompthID]);
+      remapSearchPoints(rsearch, llpoint, knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(src_grid->mask.data());
+      const auto nadds = knnWeights[ompthID].computeWeights(src_grid->mask);
 
       for (size_t n = 0; n < nadds; ++n)
         if (knnWeights[ompthID].m_mask[n]) tgt_grid->cell_frac[tgt_cell_add] = 1.0;
 
       // Store the link
-      storeWeightLinks(0, nadds, knnWeights[ompthID].m_addr.data(), knnWeights[ompthID].m_dist.data(), tgt_cell_add, weightLinks);
+      store_weightlinks(0, nadds, knnWeights[ompthID].m_addr.data(), knnWeights[ompthID].m_dist.data(), tgt_cell_add, weightLinks);
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
-  if (rsearch.gps) gridPointSearchDelete(rsearch.gps);
-  rsearch.gps = nullptr;
+  gridPointSearchDelete(rsearch.gps);
 
   weightLinksToRemapLinks(0, tgt_grid_size, weightLinks, rv);
 
@@ -102,16 +100,23 @@ void
 remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *restrict src_array, double *restrict tgt_array,
              double missval)
 {
-  RemapGrid *src_grid = rsearch.srcGrid;
-  RemapGrid *tgt_grid = rsearch.tgtGrid;
+  auto src_grid = rsearch.srcGrid;
+  auto tgt_grid = rsearch.tgtGrid;
 
   if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  Progress::init();
+  progress::init();
 
   // Compute mappings from source to target grid
 
-  size_t tgt_grid_size = tgt_grid->size;
+  auto tgt_grid_size = tgt_grid->size;
+  auto src_grid_size = src_grid->size;
+
+  Varray<int> src_grid_mask(src_grid_size);
+#ifdef _OPENMP
+#pragma omp parallel for default(none) schedule(static) shared(src_grid_size, src_array, src_grid_mask, missval)
+#endif
+  for (size_t i = 0; i < src_grid_size; ++i) src_grid_mask[i] = !DBL_IS_EQUAL(src_array[i], missval);
 
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
@@ -120,113 +125,110 @@ remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *restrict s
 
   // Loop over destination grid
 
-  double findex = 0;
+  double findex = 0.0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(findex, rsearch, numNeighbors, src_grid, tgt_grid, tgt_grid_size, src_array, tgt_array, missval, knnWeights)
+#pragma omp parallel for default(none) shared(findex, rsearch, numNeighbors, src_grid, tgt_grid, tgt_grid_size, src_array, \
+                                              tgt_array, missval, src_grid_mask, knnWeights)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) Progress::update(0, 1, findex / tgt_grid_size);
+      if (ompthID == 0) progress::update(0, 1, findex / tgt_grid_size);
 
       tgt_array[tgt_cell_add] = missval;
 
       if (!tgt_grid->mask[tgt_cell_add]) continue;
 
-      double plon = 0, plat = 0;
-      remapgrid_get_lonlat(tgt_grid, tgt_cell_add, &plon, &plat);
+      const auto llpoint = remapgrid_get_lonlat(tgt_grid, tgt_cell_add);
 
       // Find nearest grid points on source grid and distances to each point
-      remapSearchPoints(rsearch, plon, plat, knnWeights[ompthID]);
+      remapSearchPoints(rsearch, llpoint, knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(src_grid->mask.data());
+      const auto nadds = knnWeights[ompthID].computeWeights(src_grid_mask);
       if (nadds) tgt_array[tgt_cell_add] = knnWeights[ompthID].arrayWeightsSum(src_array);
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
 }  // remapDistwgt
 
-void remapInit(remapType &remap);
+void remapInit(RemapType &remap);
 
 void
 intgriddis(Field &field1, Field &field2, size_t numNeighbors)
 {
-  RemapMethod mapType = RemapMethod::DISTWGT;
-  int gridID1 = field1.grid;
-  int gridID2 = field2.grid;
-  double src_missval = field1.missval;
-  double tgt_missval = field2.missval;
-  double *src_array = field1.vec.data();
-  double *tgt_array = field2.vec.data();
+  auto mapType = RemapMethod::DISTWGT;
+  auto gridID1 = field1.grid;
+  auto gridID2 = field2.grid;
+  auto src_missval = field1.missval;
+  auto tgt_missval = field2.missval;
+  auto src_array = field1.vec_d.data();
+  auto tgt_array = field2.vec_d.data();
 
   if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
-  Progress::init();
+  progress::init();
 
   // Interpolate from source to target grid
 
-  remapType remap;
+  RemapType remap;
   remapInit(remap);
 
   bool remap_extrapolate = false;
   remapInitGrids(mapType, remap_extrapolate, gridID1, remap.src_grid, gridID2, remap.tgt_grid);
 
-  size_t src_grid_size = remap.src_grid.size;
-  size_t tgt_grid_size = remap.tgt_grid.size;
+  auto src_grid_size = remap.src_grid.size;
+  auto tgt_grid_size = remap.tgt_grid.size;
 
-  std::vector<int> src_mask(src_grid_size);
+  Varray<int> src_mask(src_grid_size);
   for (size_t i = 0; i < src_grid_size; ++i) src_mask[i] = !DBL_IS_EQUAL(src_array[i], src_missval);
-  std::vector<int> tgt_mask(tgt_grid_size);
-  for (size_t i = 0; i < tgt_grid_size; ++i) tgt_mask[i] = 1;
+  Varray<int> tgt_mask(tgt_grid_size, 1);
 
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
   remapSearchInit(mapType, remap.search, remap.src_grid, remap.tgt_grid);
 
-  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
   // Loop over destination grid
 
   size_t nmiss = 0;
-  double findex = 0;
+  double findex = 0.0;
 
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const auto ompthID = cdo_omp_get_thread_num();
 
       findex++;
-      if (ompthID == 0) Progress::update(0, 1, findex / tgt_grid_size);
+      if (ompthID == 0) progress::update(0, 1, findex / tgt_grid_size);
 
       tgt_array[tgt_cell_add] = tgt_missval;
 
       if (!tgt_mask[tgt_cell_add]) continue;
 
-      double plon = 0, plat = 0;
-      remapgrid_get_lonlat(&remap.tgt_grid, tgt_cell_add, &plon, &plat);
+      const auto llpoint = remapgrid_get_lonlat(&remap.tgt_grid, tgt_cell_add);
 
       // Find nearest grid points on source grid and distances to each point
-      remapSearchPoints(remap.search, plon, plat, knnWeights[ompthID]);
+      remapSearchPoints(remap.search, llpoint, knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights(src_mask.data());
+      const auto nadds = knnWeights[ompthID].computeWeights(src_mask);
       if (nadds)
         tgt_array[tgt_cell_add] = knnWeights[ompthID].arrayWeightsSum(src_array);
       else
         nmiss++;
     }
 
-  Progress::update(0, 1, 1);
+  progress::update(0, 1, 1);
 
   field2.nmiss = nmiss;
 
