@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,6 @@
 */
 
 #include "remap_grid_cell_search.h"
-#include "dmemory.h"
 #include "cdo_output.h"
 #include "compare.h"
 #include "array.h"
@@ -33,14 +32,14 @@ void
 setCellSearchMethod(const char *methodstr)
 {
   // clang-format off
-  if      (cstrIsEqual(methodstr, "spherepart")) cellSearchMethod = CellSearchMethod::spherepart;
-  else if (cstrIsEqual(methodstr, "latbins"))    cellSearchMethod = CellSearchMethod::latbins;
+  if      (cdo_cmpstr(methodstr, "spherepart")) cellSearchMethod = CellSearchMethod::spherepart;
+  else if (cdo_cmpstr(methodstr, "latbins"))    cellSearchMethod = CellSearchMethod::latbins;
   else cdoAbort("Grid cell search method %s not available!", methodstr);
   // clang-format on
 }
 
 static void
-gridBoundboxReg2d(size_t nx, size_t ny, const double *reg2d_corner_lon, const double *reg2d_corner_lat, double *grid_bound_box)
+gridBoundboxReg2d(size_t nx, size_t ny, const Varray<double> &reg2d_corner_lon, const Varray<double> &reg2d_corner_lat, double *grid_bound_box)
 {
   grid_bound_box[0] = reg2d_corner_lat[0];
   grid_bound_box[1] = reg2d_corner_lat[ny];
@@ -53,56 +52,52 @@ gridBoundboxReg2d(size_t nx, size_t ny, const double *reg2d_corner_lon, const do
   grid_bound_box[3] = reg2d_corner_lon[nx];
 }
 
-GridCellSearch *
-gridCellSearchCreateReg2d(size_t dims[2], const double *reg2d_corner_lon, const double *reg2d_corner_lat)
+void
+gridCellSearchCreateReg2d(GridCellSearch &gcs, size_t dims[2], const Varray<double> &reg2d_corner_lon, const Varray<double> &reg2d_corner_lat)
 {
-  GridCellSearch *gcs = (GridCellSearch *) Calloc(1, sizeof(GridCellSearch));
+  gcs.is_reg2d = true;
+  gcs.dims[0] = dims[0];
+  gcs.dims[1] = dims[1];
+  const auto nx = dims[0];
+  const auto ny = dims[1];
+  const auto nxp1 = nx + 1;
+  const auto nyp1 = ny + 1;
 
-  gcs->is_reg2d = true;
-  gcs->dims[0] = dims[0];
-  gcs->dims[1] = dims[1];
-  size_t nx = dims[0];
-  size_t ny = dims[1];
-  size_t nxp1 = nx + 1;
-  size_t nyp1 = ny + 1;
+  gcs.reg2d_corner_lon.resize(nxp1);
+  gcs.reg2d_corner_lat.resize(nyp1);
 
-  gcs->reg2d_corner_lon = (double *) Malloc(nxp1 * sizeof(double));
-  gcs->reg2d_corner_lat = (double *) Malloc(nyp1 * sizeof(double));
+  varrayCopy(nxp1, reg2d_corner_lon, gcs.reg2d_corner_lon);
+  varrayCopy(nyp1, reg2d_corner_lat, gcs.reg2d_corner_lat);
 
-  arrayCopy(nxp1, reg2d_corner_lon, gcs->reg2d_corner_lon);
-  arrayCopy(nyp1, reg2d_corner_lat, gcs->reg2d_corner_lat);
+  gridBoundboxReg2d(nx, ny, gcs.reg2d_corner_lon, gcs.reg2d_corner_lat, gcs.gridBoundboxReg2d);
 
-  gridBoundboxReg2d(nx, ny, gcs->reg2d_corner_lon, gcs->reg2d_corner_lat, gcs->gridBoundboxReg2d);
-
-  return gcs;
-}
-
-GridCellSearch *
-gridCellSearchCreate(size_t numCells, size_t numCellCorners, double *cellCornerLon, double *cellCornerLat)
-{
-  GridCellSearch *gcs = (GridCellSearch *) Calloc(1, sizeof(GridCellSearch));
-
-  gcs->method = cellSearchMethod;
-
-  grid *yacSrcGrid = yac_scrip_grid_new(cellCornerLon, cellCornerLat, numCells, numCellCorners);
-  gcs->yacSearch = yac_sphere_part_search_new(yacSrcGrid);
-  gcs->yacSrcGrid = yacSrcGrid;
-
-  return gcs;
+  gcs.in_use = true;
 }
 
 void
-gridCellSearchDelete(GridCellSearch *gcs)
+gridCellSearchCreate(GridCellSearch &gcs, size_t numCells, size_t numCellCorners, Varray<double> &cellCornerLon, Varray<double> &cellCornerLat)
 {
-  if (gcs)
+  gcs.method = cellSearchMethod;
+
+  auto yacSrcGrid = yac_scrip_grid_new(cellCornerLon.data(), cellCornerLat.data(), numCells, numCellCorners);
+  gcs.yacSearch = yac_sphere_part_search_new(yacSrcGrid);
+  gcs.yacSrcGrid = yacSrcGrid;
+
+  gcs.in_use = true;
+}
+
+void
+gridCellSearchDelete(GridCellSearch &gcs)
+{
+  if (gcs.in_use)
     {
-      if (gcs->reg2d_corner_lon) Free(gcs->reg2d_corner_lon);
-      if (gcs->reg2d_corner_lat) Free(gcs->reg2d_corner_lat);
+      varrayFree(gcs.reg2d_corner_lon);
+      varrayFree(gcs.reg2d_corner_lat);
 
-      if (gcs->yacSearch) yac_delete_grid_search((grid_search *) gcs->yacSearch);
-      if (gcs->yacSrcGrid) yac_delete_grid((grid *) gcs->yacSrcGrid);
+      if (gcs.yacSearch) yac_delete_grid_search((grid_search *) gcs.yacSearch);
+      if (gcs.yacSrcGrid) yac_delete_grid((grid *) gcs.yacSrcGrid);
 
-      Free(gcs);
+      gcs.in_use = false;
     }
 }
 
@@ -112,6 +107,7 @@ doGridCellSearchYac(grid_search *yacSearch, bool isReg2dCell, grid_cell &gridCel
   size_t numCellCorners = gridCell.num_corners;
   bounding_circle bndCircle;
   double(*xyz)[3] = gridCell.coordinates_xyz;
+
   if (numCellCorners == 4 && isReg2dCell)
     yac_get_cell_bounding_circle_reg_quad(xyz[0], xyz[1], xyz[2], &bndCircle);
   else if (numCellCorners == 3)
@@ -121,7 +117,7 @@ doGridCellSearchYac(grid_search *yacSearch, bool isReg2dCell, grid_cell &gridCel
 
   dep_list *resultList = yac_do_bnd_circle_search(yacSearch, &bndCircle, 1);
 
-  size_t numSrchCells = yac_get_num_dependencies_of_element(resultList, 0);
+  const size_t numSrchCells = yac_get_num_dependencies_of_element(resultList, 0);
   unsigned const *currNeighs = yac_get_dependencies_of_element(resultList, 0);
   for (size_t i = 0; i < numSrchCells; ++i) srchAddr[i] = currNeighs[i];
 
@@ -131,19 +127,16 @@ doGridCellSearchYac(grid_search *yacSearch, bool isReg2dCell, grid_cell &gridCel
 }
 
 size_t
-doGridCellSearch(GridCellSearch *gcs, bool isReg2dCell, grid_cell &gridCell, size_t *srchAddr)
+doGridCellSearch(GridCellSearch &gcs, bool isReg2dCell, grid_cell &gridCell, size_t *srchAddr)
 {
-  if (gcs)
+  if (gcs.in_use)
     {
-      size_t nadds = 0;
       // clang-format off
-      if (gcs->method == CellSearchMethod::spherepart)
-        nadds = doGridCellSearchYac((grid_search*)gcs->yacSearch, isReg2dCell, gridCell, srchAddr);
+      if (gcs.method == CellSearchMethod::spherepart)
+        return doGridCellSearchYac((grid_search*)gcs.yacSearch, isReg2dCell, gridCell, srchAddr);
       else
         cdoAbort("%s::method undefined!", __func__);
       // clang-format on
-
-      return nadds;
     }
 
   return 0;
