@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,10 @@
 #include <time.h>
 
 #include <cdi.h>
-#include "cdo_options.h"
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
+#include "griddes.h"
 #include <mpim_grid.h>
 #include "remap.h"
 #include "commandline.h"
@@ -85,35 +86,6 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 
 #ifdef HAVE_LIBNETCDF
 
-  int nc_file_id = -1;          /* id for NetCDF file                       */
-  int nc_srcgrdsize_id = -1;    /* id for source grid size                  */
-  int nc_dstgrdsize_id = -1;    /* id for destination grid size             */
-  int nc_srcgrdcorn_id = 0;     /* id for number of source grid corners     */
-  int nc_dstgrdcorn_id = 0;     /* id for number of dest grid corners       */
-  int nc_srcgrdrank_id = -1;    /* id for source grid rank                  */
-  int nc_dstgrdrank_id = -1;    /* id for dest grid rank                    */
-  int nc_numlinks_id = -1;      /* id for number of links in mapping        */
-  int nc_numwgts_id = -1;       /* id for number of weights for mapping     */
-  int nc_srcgrddims_id = -1;    /* id for source grid dimensions            */
-  int nc_dstgrddims_id = -1;    /* id for dest grid dimensions              */
-  int nc_srcgrdcntrlat_id = -1; /* id for source grid center latitude       */
-  int nc_dstgrdcntrlat_id = -1; /* id for dest grid center latitude         */
-  int nc_srcgrdcntrlon_id = -1; /* id for source grid center longitude      */
-  int nc_dstgrdcntrlon_id = -1; /* id for dest grid center longitude        */
-  int nc_srcgrdimask_id = -1;   /* id for source grid mask                  */
-  int nc_dstgrdimask_id = -1;   /* id for dest grid mask                    */
-  int nc_srcgrdcrnrlat_id = -1; /* id for latitude of source grid corners   */
-  int nc_srcgrdcrnrlon_id = -1; /* id for longitude of source grid corners  */
-  int nc_dstgrdcrnrlat_id = -1; /* id for latitude of dest grid corners     */
-  int nc_dstgrdcrnrlon_id = -1; /* id for longitude of dest grid corners    */
-  int nc_srcgrdarea_id = -1;    /* id for area of source grid cells         */
-  int nc_dstgrdarea_id = -1;    /* id for area of dest grid cells           */
-  int nc_srcgrdfrac_id = -1;    /* id for area fraction on source grid      */
-  int nc_dstgrdfrac_id = -1;    /* id for area fraction on dest grid        */
-  int nc_srcadd_id = -1;        /* id for map source address                */
-  int nc_dstadd_id = -1;        /* id for map destination address           */
-  int nc_rmpmatrix_id = -1;     /* id for remapping matrix                  */
-
   int nc_dims2_id[2]; /* NetCDF ids for 2d array dims             */
 
   const char *map_name = "SCRIP remapping with CDO";
@@ -174,10 +146,8 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
     case RemapMethod::UNDEF: break;
     }
 
-  /*
-  if ( rv.num_links == 0 )
-    cdoAbort("Number of remap links is 0, no remap weights found!");
-  */
+  // if (rv.num_links == 0) cdoAbort("Number of remap links is 0, no remap weights found!");
+
   {
     size_t nlinks = rv.num_links;
     size_t nele1 = 4 * 8 + 4;
@@ -222,6 +192,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   }
 
   // Create NetCDF file for mapping and define some global attributes
+  int nc_file_id = -1;
   nce(nc_create(interp_file, writemode, &nc_file_id));
 
   // Map name
@@ -264,32 +235,39 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   // Prepare NetCDF dimension info
 
   // Define grid size dimensions
+  int nc_srcgrdsize_id = -1, nc_dstgrdsize_id = -1;
   nce(nc_def_dim(nc_file_id, "src_grid_size", src_grid.size, &nc_srcgrdsize_id));
   nce(nc_def_dim(nc_file_id, "dst_grid_size", tgt_grid.size, &nc_dstgrdsize_id));
 
   // Define grid corner dimension
+  int nc_srcgrdcorn_id = -1, nc_dstgrdcorn_id = -1;
   if (src_grid.lneed_cell_corners) nce(nc_def_dim(nc_file_id, "src_grid_corners", src_grid.num_cell_corners, &nc_srcgrdcorn_id));
   if (tgt_grid.lneed_cell_corners) nce(nc_def_dim(nc_file_id, "dst_grid_corners", tgt_grid.num_cell_corners, &nc_dstgrdcorn_id));
 
   // Define grid rank dimension
+  int nc_srcgrdrank_id = -1, nc_dstgrdrank_id = -1;
   nce(nc_def_dim(nc_file_id, "src_grid_rank", src_grid.rank, &nc_srcgrdrank_id));
   nce(nc_def_dim(nc_file_id, "dst_grid_rank", tgt_grid.rank, &nc_dstgrdrank_id));
 
   // Define map size dimensions
+  int nc_numlinks_id = -1, nc_numwgts_id = -1;
   nce(nc_def_dim(nc_file_id, "num_links", rv.num_links, &nc_numlinks_id));
   nce(nc_def_dim(nc_file_id, "num_wgts", rv.num_wts, &nc_numwgts_id));
 
   // Define grid dimensions
+  int nc_srcgrddims_id = -1, nc_dstgrddims_id = -1;
   nce(nc_def_var(nc_file_id, "src_grid_dims", sizetype, 1, &nc_srcgrdrank_id, &nc_srcgrddims_id));
   nce(nc_def_var(nc_file_id, "dst_grid_dims", sizetype, 1, &nc_dstgrdrank_id, &nc_dstgrddims_id));
 
   // Define all arrays for NetCDF descriptors
 
   // Define grid center latitude array
+  int nc_srcgrdcntrlat_id = -1, nc_dstgrdcntrlat_id = -1;
   nce(nc_def_var(nc_file_id, "src_grid_center_lat", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdcntrlat_id));
   nce(nc_def_var(nc_file_id, "dst_grid_center_lat", NC_DOUBLE, 1, &nc_dstgrdsize_id, &nc_dstgrdcntrlat_id));
 
   // Define grid center longitude array
+  int nc_srcgrdcntrlon_id = -1, nc_dstgrdcntrlon_id = -1;
   nce(nc_def_var(nc_file_id, "src_grid_center_lon", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdcntrlon_id));
   nce(nc_def_var(nc_file_id, "dst_grid_center_lon", NC_DOUBLE, 1, &nc_dstgrdsize_id, &nc_dstgrdcntrlon_id));
 
@@ -298,6 +276,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   nc_dims2_id[0] = nc_srcgrdsize_id;
   nc_dims2_id[1] = nc_srcgrdcorn_id;
 
+  int nc_srcgrdcrnrlat_id = -1, nc_srcgrdcrnrlon_id = -1;
   if (src_grid.lneed_cell_corners)
     {
       nce(nc_def_var(nc_file_id, "src_grid_corner_lat", NC_DOUBLE, 2, nc_dims2_id, &nc_srcgrdcrnrlat_id));
@@ -307,6 +286,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   nc_dims2_id[0] = nc_dstgrdsize_id;
   nc_dims2_id[1] = nc_dstgrdcorn_id;
 
+  int nc_dstgrdcrnrlat_id = -1, nc_dstgrdcrnrlon_id = -1;
   if (tgt_grid.lneed_cell_corners)
     {
       nce(nc_def_var(nc_file_id, "dst_grid_corner_lat", NC_DOUBLE, 2, nc_dims2_id, &nc_dstgrdcrnrlat_id));
@@ -332,14 +312,17 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 
   // Define grid mask
 
+  int nc_srcgrdimask_id = -1;
   nce(nc_def_var(nc_file_id, "src_grid_imask", NC_INT, 1, &nc_srcgrdsize_id, &nc_srcgrdimask_id));
   nce(nc_put_att_text(nc_file_id, nc_srcgrdimask_id, "units", 8, "unitless"));
 
+  int nc_dstgrdimask_id = -1;
   nce(nc_def_var(nc_file_id, "dst_grid_imask", NC_INT, 1, &nc_dstgrdsize_id, &nc_dstgrdimask_id));
   nce(nc_put_att_text(nc_file_id, nc_dstgrdimask_id, "units", 8, "unitless"));
 
   // Define grid area arrays
 
+  int nc_srcgrdarea_id = -1, nc_dstgrdarea_id = -1;
   if (lgridarea)
     {
       nce(nc_def_var(nc_file_id, "src_grid_area", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdarea_id));
@@ -351,20 +334,24 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 
   // Define grid fraction arrays
 
+  int nc_srcgrdfrac_id = -1;
   nce(nc_def_var(nc_file_id, "src_grid_frac", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdfrac_id));
   nce(nc_put_att_text(nc_file_id, nc_srcgrdfrac_id, "units", 8, "unitless"));
 
+  int nc_dstgrdfrac_id = -1;
   nce(nc_def_var(nc_file_id, "dst_grid_frac", NC_DOUBLE, 1, &nc_dstgrdsize_id, &nc_dstgrdfrac_id));
   nce(nc_put_att_text(nc_file_id, nc_dstgrdfrac_id, "units", 8, "unitless"));
 
   // Define mapping arrays
 
+  int nc_srcadd_id = -1, nc_dstadd_id = -1;
   nce(nc_def_var(nc_file_id, "src_address", sizetype, 1, &nc_numlinks_id, &nc_srcadd_id));
   nce(nc_def_var(nc_file_id, "dst_address", sizetype, 1, &nc_numlinks_id, &nc_dstadd_id));
 
   nc_dims2_id[0] = nc_numlinks_id;
   nc_dims2_id[1] = nc_numwgts_id;
 
+  int nc_rmpmatrix_id = -1;
   nce(nc_def_var(nc_file_id, "remap_matrix", NC_DOUBLE, 2, nc_dims2_id, &nc_rmpmatrix_id));
 
   // End definition stage
@@ -384,22 +371,22 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   nce(nc_put_var_int(nc_file_id, nc_srcgrdimask_id, &src_grid.mask[0]));
   nce(nc_put_var_int(nc_file_id, nc_dstgrdimask_id, &tgt_grid.mask[0]));
 
-  if (src_grid.cell_center_lat) nce(nc_put_var_double(nc_file_id, nc_srcgrdcntrlat_id, src_grid.cell_center_lat));
-  if (src_grid.cell_center_lon) nce(nc_put_var_double(nc_file_id, nc_srcgrdcntrlon_id, src_grid.cell_center_lon));
+  if (!src_grid.cell_center_lat.empty()) nce(nc_put_var_double(nc_file_id, nc_srcgrdcntrlat_id, src_grid.cell_center_lat.data()));
+  if (!src_grid.cell_center_lon.empty()) nce(nc_put_var_double(nc_file_id, nc_srcgrdcntrlon_id, src_grid.cell_center_lon.data()));
 
   if (src_grid.lneed_cell_corners)
     {
-      nce(nc_put_var_double(nc_file_id, nc_srcgrdcrnrlat_id, src_grid.cell_corner_lat));
-      nce(nc_put_var_double(nc_file_id, nc_srcgrdcrnrlon_id, src_grid.cell_corner_lon));
+      nce(nc_put_var_double(nc_file_id, nc_srcgrdcrnrlat_id, src_grid.cell_corner_lat.data()));
+      nce(nc_put_var_double(nc_file_id, nc_srcgrdcrnrlon_id, src_grid.cell_corner_lon.data()));
     }
 
-  if (tgt_grid.cell_center_lat) nce(nc_put_var_double(nc_file_id, nc_dstgrdcntrlat_id, tgt_grid.cell_center_lat));
-  if (tgt_grid.cell_center_lon) nce(nc_put_var_double(nc_file_id, nc_dstgrdcntrlon_id, tgt_grid.cell_center_lon));
+  if (!tgt_grid.cell_center_lat.empty()) nce(nc_put_var_double(nc_file_id, nc_dstgrdcntrlat_id, tgt_grid.cell_center_lat.data()));
+  if (!tgt_grid.cell_center_lon.empty()) nce(nc_put_var_double(nc_file_id, nc_dstgrdcntrlon_id, tgt_grid.cell_center_lon.data()));
 
   if (tgt_grid.lneed_cell_corners)
     {
-      nce(nc_put_var_double(nc_file_id, nc_dstgrdcrnrlat_id, tgt_grid.cell_corner_lat));
-      nce(nc_put_var_double(nc_file_id, nc_dstgrdcrnrlon_id, tgt_grid.cell_corner_lon));
+      nce(nc_put_var_double(nc_file_id, nc_dstgrdcrnrlat_id, tgt_grid.cell_corner_lat.data()));
+      nce(nc_put_var_double(nc_file_id, nc_dstgrdcrnrlon_id, tgt_grid.cell_corner_lon.data()));
     }
 
   if (lgridarea) nce(nc_put_var_double(nc_file_id, nc_srcgrdarea_id, &src_grid.cell_area[0]));
@@ -407,7 +394,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
   nce(nc_put_var_double(nc_file_id, nc_srcgrdfrac_id, &src_grid.cell_frac[0]));
 
   /*
-  if ( luse_cell_area )
+  if (luse_cell_area)
     nce(nc_put_var_double(nc_file_id, nc_dstgrdarea_id, tgt_grid.cell_area_in));
   else
   */
@@ -437,7 +424,7 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 
 #ifdef HAVE_LIBNETCDF
 RemapMethod
-getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int &remapOrder)
+getMapType(int nc_file_id, SubmapType &submapType, int &numNeighbors, int &remapOrder)
 {
   // Map method
   size_t attlen;
@@ -446,13 +433,13 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int &remap
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "map_method", &attlen));
   map_method[attlen] = 0;
 
-  *submapType = SubmapType::NONE;
+  submapType = SubmapType::NONE;
   remapOrder = 1;
 
   RemapMethod mapType = RemapMethod::UNDEF;
-  if (cmpstr(map_method, "Conservative") == 0)
+  if (cdo_cmpstrLenRhs(map_method, "Conservative"))
     {
-      if (cmpstr(map_method, "Conservative remapping using clipping on sphere") == 0)
+      if (cdo_cmpstrLenRhs(map_method, "Conservative remapping using clipping on sphere"))
         mapType = RemapMethod::CONSERV;
       else
         mapType = RemapMethod::CONSERV_SCRIP;
@@ -461,24 +448,24 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int &remap
       const int status = nc_get_att_int(nc_file_id, NC_GLOBAL, "remap_order", &iatt);
       if (status == NC_NOERR) remapOrder = iatt;
     }
-  else if (cmpstr(map_method, "Bilinear") == 0)
+  else if (cdo_cmpstrLenRhs(map_method, "Bilinear"))
     mapType = RemapMethod::BILINEAR;
-  else if (cmpstr(map_method, "Bicubic") == 0)
+  else if (cdo_cmpstrLenRhs(map_method, "Bicubic"))
     mapType = RemapMethod::BICUBIC;
-  else if (cmpstr(map_method, "Distance") == 0)
+  else if (cdo_cmpstrLenRhs(map_method, "Distance"))
     {
       mapType = RemapMethod::DISTWGT;
-      *numNeighbors = 4;
+      numNeighbors = 4;
     }
-  else if (cmpstr(map_method, "Nearest") == 0)
+  else if (cdo_cmpstrLenRhs(map_method, "Nearest"))
     {
       mapType = RemapMethod::DISTWGT;
-      *numNeighbors = 1;
+      numNeighbors = 1;
     }
-  else if (cmpstr(map_method, "Largest") == 0)
+  else if (cdo_cmpstrLenRhs(map_method, "Largest"))
     {
       mapType = RemapMethod::CONSERV_SCRIP;
-      *submapType = SubmapType::LAF;
+      submapType = SubmapType::LAF;
     }
   else
     {
@@ -493,8 +480,8 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int &remap
 #endif
 
 void
-remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMethod *mapType, SubmapType *submapType,
-                   int *numNeighbors, int &remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv)
+remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMethod &mapType, SubmapType &submapType,
+                   int &numNeighbors, int &remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv)
 {
   // The routine reads a NetCDF file to extract remapping info in SCRIP format
 
@@ -502,33 +489,6 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
 
   bool lgridarea = false;
   int status;
-  int nc_srcgrdsize_id;    /* id for source grid size                  */
-  int nc_dstgrdsize_id;    /* id for destination grid size             */
-  int nc_srcgrdcorn_id;    /* id for number of source grid corners     */
-  int nc_dstgrdcorn_id;    /* id for number of dest grid corners       */
-  int nc_srcgrdrank_id;    /* id for source grid rank                  */
-  int nc_dstgrdrank_id;    /* id for dest grid rank                    */
-  int nc_numlinks_id;      /* id for number of links in mapping        */
-  int nc_numwgts_id;       /* id for number of weights for mapping     */
-  int nc_srcgrddims_id;    /* id for source grid dimensions            */
-  int nc_dstgrddims_id;    /* id for dest grid dimensions              */
-  int nc_srcgrdcntrlat_id; /* id for source grid center latitude       */
-  int nc_dstgrdcntrlat_id; /* id for dest grid center latitude         */
-  int nc_srcgrdcntrlon_id; /* id for source grid center longitude      */
-  int nc_dstgrdcntrlon_id; /* id for dest grid center longitude        */
-  int nc_srcgrdimask_id;   /* id for source grid mask                  */
-  int nc_dstgrdimask_id;   /* id for dest grid mask                    */
-  int nc_srcgrdcrnrlat_id; /* id for latitude of source grid corners   */
-  int nc_srcgrdcrnrlon_id; /* id for longitude of source grid corners  */
-  int nc_dstgrdcrnrlat_id; /* id for latitude of dest grid corners     */
-  int nc_dstgrdcrnrlon_id; /* id for longitude of dest grid corners    */
-  int nc_srcgrdarea_id;    /* id for area of source grid cells         */
-  int nc_dstgrdarea_id;    /* id for area of dest grid cells           */
-  int nc_srcgrdfrac_id;    /* id for area fraction on source grid      */
-  int nc_dstgrdfrac_id;    /* id for area fraction on dest grid        */
-  int nc_srcadd_id;        /* id for map source address                */
-  int nc_dstadd_id;        /* id for map destination address           */
-  int nc_rmpmatrix_id;     /* id for remapping matrix                  */
 
   char map_name[1024];
   char normalize_opt[64]; /* character string for normalization option */
@@ -543,8 +503,7 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
 
   // Open file and read some global information
 
-  // nce(nc_open(interp_file, NC_NOWRITE, &nc_file_id));
-  int nc_file_id = cdf_openread(interp_file);
+  const auto nc_file_id = cdo_cdf_openread(interp_file);
 
   // Map name
 
@@ -559,13 +518,13 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
     }
 
   // Map Tyoe
-  *mapType = getMapType(nc_file_id, submapType, numNeighbors, remapOrder);
+  mapType = getMapType(nc_file_id, submapType, numNeighbors, remapOrder);
 
-  if (*mapType == RemapMethod::CONSERV_SCRIP) lgridarea = true;
+  if (mapType == RemapMethod::CONSERV_SCRIP) lgridarea = true;
 
-  remapVarsInit(*mapType, remapOrder, rv);
+  remapVarsInit(mapType, remapOrder, rv);
 
-  rv.mapType = *mapType;
+  rv.mapType = mapType;
   rv.links_per_value = -1;
   rv.sort_add = false;
 
@@ -575,9 +534,9 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   normalize_opt[attlen] = 0;
 
   // clang-format off
-  if      (cstrIsEqual(normalize_opt, "none"))     rv.normOpt = NormOpt::NONE;
-  else if (cstrIsEqual(normalize_opt, "fracarea")) rv.normOpt = NormOpt::FRACAREA;
-  else if (cstrIsEqual(normalize_opt, "destarea")) rv.normOpt = NormOpt::DESTAREA;
+  if      (cdo_cmpstr(normalize_opt, "none"))     rv.normOpt = NormOpt::NONE;
+  else if (cdo_cmpstr(normalize_opt, "fracarea")) rv.normOpt = NormOpt::FRACAREA;
+  else if (cdo_cmpstr(normalize_opt, "destarea")) rv.normOpt = NormOpt::DESTAREA;
   else
     {
       cdoPrint("normalize_opt = %s", normalize_opt);
@@ -592,10 +551,10 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "conventions", &attlen));
   convention[attlen] = 0;
 
-  if (!cstrIsEqual(convention, "SCRIP"))
+  if (!cdo_cmpstr(convention, "SCRIP"))
     {
       cdoPrint("convention = %s", convention);
-      if (cstrIsEqual(convention, "NCAR-CSM"))
+      if (cdo_cmpstr(convention, "NCAR-CSM"))
         cdoAbort("Unsupported file convention!");
       else
         cdoAbort("Unknown file convention!");
@@ -620,20 +579,19 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   remapGridInit(tgt_grid);
 
   // Read dimension information
+  int nc_srcgrdsize_id;
   nce(nc_inq_dimid(nc_file_id, "src_grid_size", &nc_srcgrdsize_id));
   nce(nc_inq_dimlen(nc_file_id, nc_srcgrdsize_id, &dimlen));
   src_grid.size = dimlen;
-  /*
-  if ( src_grid.size != gridInqSize(gridID1) )
-    cdoAbort("Source grids have different size!");
-  */
+  // if (src_grid.size != gridInqSize(gridID1)) cdoAbort("Source grids have different size!");
+
+  int nc_dstgrdsize_id;
   nce(nc_inq_dimid(nc_file_id, "dst_grid_size", &nc_dstgrdsize_id));
   nce(nc_inq_dimlen(nc_file_id, nc_dstgrdsize_id, &dimlen));
   tgt_grid.size = dimlen;
-  /*
-  if ( tgt_grid.size != gridInqSize(gridID2) )
-    cdoAbort("Target grids have different size!");
-  */
+  // if (tgt_grid.size != gridInqSize(gridID2)) cdoAbort("Target grids have different size!");
+
+  int nc_srcgrdcorn_id;
   status = nc_inq_dimid(nc_file_id, "src_grid_corners", &nc_srcgrdcorn_id);
   if (status == NC_NOERR)
     {
@@ -643,6 +601,7 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
       src_grid.lneed_cell_corners = true;
     }
 
+  int nc_dstgrdcorn_id;
   status = nc_inq_dimid(nc_file_id, "dst_grid_corners", &nc_dstgrdcorn_id);
   if (status == NC_NOERR)
     {
@@ -652,21 +611,23 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
       tgt_grid.lneed_cell_corners = true;
     }
 
+  int nc_srcgrdrank_id;
   nce(nc_inq_dimid(nc_file_id, "src_grid_rank", &nc_srcgrdrank_id));
   nce(nc_inq_dimlen(nc_file_id, nc_srcgrdrank_id, &dimlen));
   src_grid.rank = dimlen;
 
+  int nc_dstgrdrank_id;
   nce(nc_inq_dimid(nc_file_id, "dst_grid_rank", &nc_dstgrdrank_id));
   nce(nc_inq_dimlen(nc_file_id, nc_dstgrdrank_id, &dimlen));
   tgt_grid.rank = dimlen;
 
+  int nc_numlinks_id;
   nce(nc_inq_dimid(nc_file_id, "num_links", &nc_numlinks_id));
   nce(nc_inq_dimlen(nc_file_id, nc_numlinks_id, &dimlen));
   rv.num_links = dimlen;
-  /*
-  if ( rv.num_links == 0 )
-    cdoAbort("Number of remap links is 0, no remap weights found!");
-  */
+  // if (rv.num_links == 0) cdoAbort("Number of remap links is 0, no remap weights found!");
+
+  int nc_numwgts_id;
   nce(nc_inq_dimid(nc_file_id, "num_wgts", &nc_numwgts_id));
   nce(nc_inq_dimlen(nc_file_id, nc_numwgts_id, &dimlen));
   rv.num_wts = dimlen;
@@ -696,34 +657,44 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
 
   // Get variable ids
 
+  int nc_srcgrddims_id, nc_srcgrdimask_id;
+  int nc_srcgrdcntrlat_id, nc_srcgrdcntrlon_id;
   nce(nc_inq_varid(nc_file_id, "src_grid_dims", &nc_srcgrddims_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_imask", &nc_srcgrdimask_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_center_lat", &nc_srcgrdcntrlat_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_center_lon", &nc_srcgrdcntrlon_id));
 
+  int nc_srcgrdcrnrlat_id = -1, nc_srcgrdcrnrlon_id = -1;
   if (src_grid.num_cell_corners)
     {
       nce(nc_inq_varid(nc_file_id, "src_grid_corner_lat", &nc_srcgrdcrnrlat_id));
       nce(nc_inq_varid(nc_file_id, "src_grid_corner_lon", &nc_srcgrdcrnrlon_id));
     }
 
+  int nc_srcgrdarea_id = -1, nc_srcgrdfrac_id;
   if (lgridarea) nce(nc_inq_varid(nc_file_id, "src_grid_area", &nc_srcgrdarea_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_frac", &nc_srcgrdfrac_id));
 
+  int nc_dstgrddims_id, nc_dstgrdimask_id;
+  int nc_dstgrdcntrlat_id, nc_dstgrdcntrlon_id;
   nce(nc_inq_varid(nc_file_id, "dst_grid_dims", &nc_dstgrddims_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_imask", &nc_dstgrdimask_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_center_lat", &nc_dstgrdcntrlat_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_center_lon", &nc_dstgrdcntrlon_id));
 
+  int nc_dstgrdcrnrlat_id = -1, nc_dstgrdcrnrlon_id = -1;
   if (tgt_grid.num_cell_corners)
     {
       nce(nc_inq_varid(nc_file_id, "dst_grid_corner_lat", &nc_dstgrdcrnrlat_id));
       nce(nc_inq_varid(nc_file_id, "dst_grid_corner_lon", &nc_dstgrdcrnrlon_id));
     }
 
+  int nc_dstgrdarea_id = -1, nc_dstgrdfrac_id;
   if (lgridarea) nce(nc_inq_varid(nc_file_id, "dst_grid_area", &nc_dstgrdarea_id));
   nce(nc_inq_varid(nc_file_id, "dst_grid_frac", &nc_dstgrdfrac_id));
 
+  int nc_srcadd_id, nc_dstadd_id;
+  int nc_rmpmatrix_id;
   nce(nc_inq_varid(nc_file_id, "src_address", &nc_srcadd_id));
   nce(nc_inq_varid(nc_file_id, "dst_address", &nc_dstadd_id));
   nce(nc_inq_varid(nc_file_id, "remap_matrix", &nc_rmpmatrix_id));
@@ -737,27 +708,27 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
 
   nce(nc_get_var_int(nc_file_id, nc_srcgrdimask_id, &src_grid.mask[0]));
 
-  nce(nc_get_var_double(nc_file_id, nc_srcgrdcntrlat_id, src_grid.cell_center_lat));
-  nce(nc_get_var_double(nc_file_id, nc_srcgrdcntrlon_id, src_grid.cell_center_lon));
+  nce(nc_get_var_double(nc_file_id, nc_srcgrdcntrlat_id, src_grid.cell_center_lat.data()));
+  nce(nc_get_var_double(nc_file_id, nc_srcgrdcntrlon_id, src_grid.cell_center_lon.data()));
 
   nce(nc_get_att_text(nc_file_id, nc_srcgrdcntrlat_id, "units", src_grid_units));
   nce(nc_inq_attlen(nc_file_id, nc_srcgrdcntrlat_id, "units", &attlen));
   src_grid_units[attlen] = 0;
 
-  grid_to_radian(src_grid_units, src_grid.size, src_grid.cell_center_lon, "source grid center lon");
-  grid_to_radian(src_grid_units, src_grid.size, src_grid.cell_center_lat, "source grid center lat");
+  grid_to_radian(src_grid_units, src_grid.size, src_grid.cell_center_lon.data(), "source grid center lon");
+  grid_to_radian(src_grid_units, src_grid.size, src_grid.cell_center_lat.data(), "source grid center lat");
 
   if (src_grid.num_cell_corners)
     {
-      nce(nc_get_var_double(nc_file_id, nc_srcgrdcrnrlat_id, src_grid.cell_corner_lat));
-      nce(nc_get_var_double(nc_file_id, nc_srcgrdcrnrlon_id, src_grid.cell_corner_lon));
+      nce(nc_get_var_double(nc_file_id, nc_srcgrdcrnrlat_id, src_grid.cell_corner_lat.data()));
+      nce(nc_get_var_double(nc_file_id, nc_srcgrdcrnrlon_id, src_grid.cell_corner_lon.data()));
 
       nce(nc_get_att_text(nc_file_id, nc_srcgrdcrnrlat_id, "units", src_grid_units));
       nce(nc_inq_attlen(nc_file_id, nc_srcgrdcrnrlat_id, "units", &attlen));
       src_grid_units[attlen] = 0;
 
-      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lon, "source grid corner lon");
-      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lat, "source grid corner lat");
+      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lon.data(), "source grid corner lon");
+      grid_to_radian(src_grid_units, src_grid.num_cell_corners * src_grid.size, src_grid.cell_corner_lat.data(), "source grid corner lat");
     }
 
   if (lgridarea) nce(nc_get_var_double(nc_file_id, nc_srcgrdarea_id, &src_grid.cell_area[0]));
@@ -770,27 +741,27 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
 
   nce(nc_get_var_int(nc_file_id, nc_dstgrdimask_id, &tgt_grid.mask[0]));
 
-  nce(nc_get_var_double(nc_file_id, nc_dstgrdcntrlat_id, tgt_grid.cell_center_lat));
-  nce(nc_get_var_double(nc_file_id, nc_dstgrdcntrlon_id, tgt_grid.cell_center_lon));
+  nce(nc_get_var_double(nc_file_id, nc_dstgrdcntrlat_id, tgt_grid.cell_center_lat.data()));
+  nce(nc_get_var_double(nc_file_id, nc_dstgrdcntrlon_id, tgt_grid.cell_center_lon.data()));
 
   nce(nc_get_att_text(nc_file_id, nc_dstgrdcntrlat_id, "units", tgt_grid_units));
   nce(nc_inq_attlen(nc_file_id, nc_dstgrdcntrlat_id, "units", &attlen));
   tgt_grid_units[attlen] = 0;
 
-  grid_to_radian(tgt_grid_units, tgt_grid.size, tgt_grid.cell_center_lon, "target grid center lon");
-  grid_to_radian(tgt_grid_units, tgt_grid.size, tgt_grid.cell_center_lat, "target grid center lat");
+  grid_to_radian(tgt_grid_units, tgt_grid.size, tgt_grid.cell_center_lon.data(), "target grid center lon");
+  grid_to_radian(tgt_grid_units, tgt_grid.size, tgt_grid.cell_center_lat.data(), "target grid center lat");
 
   if (tgt_grid.num_cell_corners)
     {
-      nce(nc_get_var_double(nc_file_id, nc_dstgrdcrnrlat_id, tgt_grid.cell_corner_lat));
-      nce(nc_get_var_double(nc_file_id, nc_dstgrdcrnrlon_id, tgt_grid.cell_corner_lon));
+      nce(nc_get_var_double(nc_file_id, nc_dstgrdcrnrlat_id, tgt_grid.cell_corner_lat.data()));
+      nce(nc_get_var_double(nc_file_id, nc_dstgrdcrnrlon_id, tgt_grid.cell_corner_lon.data()));
 
       nce(nc_get_att_text(nc_file_id, nc_dstgrdcrnrlat_id, "units", tgt_grid_units));
       nce(nc_inq_attlen(nc_file_id, nc_dstgrdcrnrlat_id, "units", &attlen));
       tgt_grid_units[attlen] = 0;
 
-      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lon, "target grid corner lon");
-      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lat, "target grid corner lat");
+      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lon.data(), "target grid corner lon");
+      grid_to_radian(tgt_grid_units, tgt_grid.num_cell_corners * tgt_grid.size, tgt_grid.cell_corner_lat.data(), "target grid corner lat");
     }
 
   if (lgridarea) nce(nc_get_var_double(nc_file_id, nc_dstgrdarea_id, &tgt_grid.cell_area[0]));
@@ -812,7 +783,7 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
     }
 
   // Close input file
-  nce(nc_close(nc_file_id));
+  cdo_cdf_close(nc_file_id);
 
 #else
   cdoAbort("NetCDF support not compiled in!");
