@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,8 +14,14 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include "cdo_int.h"
-#include "cdo_math.h"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <algorithm>
+
+#include "cdo_output.h"
 #include "cdo_options.h"
 #include <mpim_grid.h>
 #include "remap.h"
@@ -45,8 +51,8 @@ calcBinAddr(GridSearchBins &searchBins)
           const size_t n2 = n << 1;
           if (cell_bound_box_lat1 <= bin_lats[n2 + 1] && cell_bound_box_lat2 >= bin_lats[n2])
             {
-              bin_addr[n2] = cdo::min(nele, bin_addr[n2]);
-              bin_addr[n2 + 1] = cdo::max(nele, bin_addr[n2 + 1]);
+              bin_addr[n2] = std::min(nele, bin_addr[n2]);
+              bin_addr[n2 + 1] = std::max(nele, bin_addr[n2 + 1]);
             }
         }
     }
@@ -165,10 +171,10 @@ gridSearchSquareCurv2dNNScrip(size_t min_add, size_t max_add, size_t *restrict n
 {
   int search_result = 0;
 
-  const double coslat_dst = cos(plat);
-  const double sinlat_dst = sin(plat);
-  const double coslon_dst = cos(plon);
-  const double sinlon_dst = sin(plon);
+  const double coslat_dst = std::cos(plat);
+  const double sinlat_dst = std::sin(plat);
+  const double coslon_dst = std::cos(plon);
+  const double sinlon_dst = std::sin(plon);
 
   double dist_min = DBL_MAX;
   for (unsigned n = 0; n < 4; ++n) nbr_dist[n] = DBL_MAX;
@@ -176,9 +182,9 @@ gridSearchSquareCurv2dNNScrip(size_t min_add, size_t max_add, size_t *restrict n
   double distance;
   for (size_t srch_add = min_add; srch_add <= max_add; ++srch_add)
     {
-      distance = acos(coslat_dst * cos(src_center_lat[srch_add])
-                          * (coslon_dst * cos(src_center_lon[srch_add]) + sinlon_dst * sin(src_center_lon[srch_add]))
-                      + sinlat_dst * sin(src_center_lat[srch_add]));
+      distance = std::acos(coslat_dst * std::cos(src_center_lat[srch_add])
+                               * (coslon_dst * std::cos(src_center_lon[srch_add]) + sinlon_dst * std::sin(src_center_lon[srch_add]))
+                           + sinlat_dst * std::sin(src_center_lat[srch_add]));
 
       if (distance < dist_min)
         {
@@ -297,15 +303,15 @@ pointInQuad(bool isCyclic, size_t nx, size_t ny, size_t i, size_t j, size_t adds
   idx[2] = jp1 * nx + ip1;  // north-east
   idx[3] = jp1 * nx + i;    // north
 
-  for (unsigned j = 0; j < 4; ++j) lons[j] = centerLon[idx[j]];
-  for (unsigned j = 0; j < 4; ++j) lats[j] = centerLat[idx[j]];
+  for (unsigned k = 0; k < 4; ++k) lons[k] = centerLon[idx[k]];
+  for (unsigned k = 0; k < 4; ++k) lats[k] = centerLat[idx[k]];
 
   unsigned n = quadCrossProducts(plon, plat, lons, lats);
 
   // If cross products all same sign, we found the location
   if (n >= 4)
     {
-      for (unsigned j = 0; j < 4; ++j) adds[j] = idx[j];
+      for (unsigned k = 0; k < 4; ++k) adds[k] = idx[k];
       search_result = true;
     }
 
@@ -313,7 +319,7 @@ pointInQuad(bool isCyclic, size_t nx, size_t ny, size_t i, size_t j, size_t adds
 }
 
 int
-gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t *restrict src_add, double *restrict src_lats, double *restrict src_lons,
+gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t (&src_add)[4], double (&src_lats)[4], double (&src_lons)[4],
                             double plat, double plon, GridSearchBins &srcBins)
 {
   /*
@@ -333,9 +339,9 @@ gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t *restrict src_add, doubl
     double src_center_lat[]        ! latitude  of each src grid center
     double src_center_lon[]        ! longitude of each src grid center
 
-    float src_grid_bound_box[][4] ! bound box for source grid
+    float src_grid_bound_box[][4]  ! bound box for source grid
 
-    int src_bin_addr[][2]           ! latitude bins for restricting
+    int src_bin_addr[][2]          ! latitude bins for restricting
   */
   int search_result = 0;
 
@@ -383,8 +389,8 @@ gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t *restrict src_add, doubl
           size_t j = srch_add / nx;
           size_t i = srch_add - j * nx;
 
-          if (pointInQuad(src_grid->is_cyclic, nx, ny, i, j, src_add, src_lons, src_lats, plon, plat, src_grid->cell_center_lon,
-                          src_grid->cell_center_lat))
+          if (pointInQuad(src_grid->is_cyclic, nx, ny, i, j, src_add, src_lons, src_lats, plon, plat, src_grid->cell_center_lon.data(),
+                          src_grid->cell_center_lat.data()))
             {
               search_result = 1;
               return search_result;
@@ -406,8 +412,8 @@ gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t *restrict src_add, doubl
     printf("Could not find location for %g %g\n", plat*RAD2DEG, plon*RAD2DEG);
     printf("Using nearest-neighbor average for this point\n");
   */
-  search_result = gridSearchSquareCurv2dNNScrip(min_add, max_add, src_add, src_lats, plat, plon, src_grid->cell_center_lat,
-                                                src_grid->cell_center_lon);
+  search_result = gridSearchSquareCurv2dNNScrip(min_add, max_add, src_add, src_lats, plat, plon, src_grid->cell_center_lat.data(),
+                                                src_grid->cell_center_lon.data());
 
   return search_result;
 }
