@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,19 +14,19 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include "cdo_int.h"
+
 #include "remap.h"
 
 int
-gridSearchSquareReg2dNN(size_t nx, size_t ny, size_t *restrict nbr_add, double *restrict nbr_dist, double plat, double plon,
-                        const double *restrict src_center_lat, const double *restrict src_center_lon)
+gridSearchSquareReg2dNN(size_t nx, size_t ny, size_t *nbr_add, double *nbr_dist, double plat, double plon,
+                        const Varray<double> &src_center_lat, const Varray<double> &src_center_lon)
 {
   int search_result = 0;
 
-  const double coslat_dst = cos(plat);
-  const double sinlat_dst = sin(plat);
-  const double coslon_dst = cos(plon);
-  const double sinlon_dst = sin(plon);
+  const auto coslat_dst = std::cos(plat);
+  const auto sinlat_dst = std::sin(plat);
+  const auto coslon_dst = std::cos(plon);
+  const auto sinlon_dst = std::sin(plon);
 
   double dist_min = DBL_MAX;
   for (unsigned n = 0; n < 4; ++n) nbr_dist[n] = DBL_MAX;
@@ -64,23 +64,24 @@ gridSearchSquareReg2dNN(size_t nx, size_t ny, size_t *restrict nbr_add, double *
 
   std::vector<double> sincoslon(nx);
 
-  for (size_t ii = 0; ii < nx; ++ii) sincoslon[ii] = coslon_dst * cos(src_center_lon[ii]) + sinlon_dst * sin(src_center_lon[ii]);
+  for (size_t ii = 0; ii < nx; ++ii)
+    sincoslon[ii] = coslon_dst * std::cos(src_center_lon[ii]) + sinlon_dst * std::sin(src_center_lon[ii]);
 
   for (size_t jj = jjf; jj <= jjl; ++jj)
     {
-      const double coslat = coslat_dst * cos(src_center_lat[jj]);
-      const double sinlat = sinlat_dst * sin(src_center_lat[jj]);
+      const auto coslat = coslat_dst * std::cos(src_center_lat[jj]);
+      const auto sinlat = sinlat_dst * std::sin(src_center_lat[jj]);
 
-      size_t jjskip = jj > 1 && jj < (ny - 2);
+      const auto jjskip = jj > 1 && jj < (ny - 2);
 
       for (size_t ii = 0; ii < nx; ++ii)
         {
           if (jjskip && ii > 1 && ii < (nx - 2)) continue;
 
-          double distance = acos(coslat * sincoslon[ii] + sinlat);
+          const auto distance = std::acos(coslat * sincoslon[ii] + sinlat);
           if (distance < dist_min)
             {
-              size_t srch_add = jj * nx + ii;
+              const auto srch_add = jj * nx + ii;
               for (unsigned n = 0; n < 4; ++n)
                 {
                   if (distance < nbr_dist[n])
@@ -110,40 +111,40 @@ gridSearchSquareReg2dNN(size_t nx, size_t ny, size_t *restrict nbr_add, double *
 }
 
 int
-gridSearchSquareReg2d(RemapGrid *src_grid, size_t *restrict src_add, double *restrict src_lats, double *restrict src_lons,
+gridSearchSquareReg2d(RemapGrid *src_grid, size_t (&src_add)[4], double (&src_lats)[4], double (&src_lons)[4],
                       double plat, double plon)
 {
   /*
     Input variables:
 
-      double plat                    ! latitude  of the search point
-      double plon                    ! longitude of the search point
+      plat : latitude  of the search point
+      plon : longitude of the search point
 
     Output variables:
 
-      int    src_add[4]              ! address of each corner point enclosing P
-      double src_lats[4]             ! latitudes  of the four corner points
-      double src_lons[4]             ! longitudes of the four corner points
+      src_add[4]  : address of each corner point enclosing P
+      src_lats[4] : latitudes  of the four corner points
+      src_lons[4] : longitudes of the four corner points
   */
   int search_result = 0;
-  const double *restrict src_center_lat = src_grid->reg2d_center_lat;
-  const double *restrict src_center_lon = src_grid->reg2d_center_lon;
+  const auto &src_center_lat = src_grid->reg2d_center_lat;
+  const auto &src_center_lon = src_grid->reg2d_center_lon;
 
   for (unsigned n = 0; n < 4; ++n) src_add[n] = 0;
 
-  const size_t nx = src_grid->dims[0];
-  const size_t ny = src_grid->dims[1];
+  const auto nx = src_grid->dims[0];
+  const auto ny = src_grid->dims[1];
 
-  const size_t nxm = src_grid->is_cyclic ? nx + 1 : nx;
+  const auto nxm = src_grid->is_cyclic ? nx + 1 : nx;
 
   if (plon < src_center_lon[0]) plon += PI2;
   if (plon > src_center_lon[nxm - 1]) plon -= PI2;
 
   size_t ii, jj;
-  int lfound = rect_grid_search(&ii, &jj, plon, plat, nxm, ny, src_center_lon, src_center_lat);
+  int lfound = rect_grid_search(ii, jj, plon, plat, nxm, ny, src_center_lon, src_center_lat);
   if (lfound)
     {
-      size_t iix = ii;
+      auto iix = ii;
       if (src_grid->is_cyclic && iix == (nxm - 1)) iix = 0;
       src_add[0] = (jj - 1) * nx + (ii - 1);
       src_add[1] = (jj - 1) * nx + (iix);
@@ -152,7 +153,7 @@ gridSearchSquareReg2d(RemapGrid *src_grid, size_t *restrict src_add, double *res
 
       src_lons[0] = src_center_lon[ii - 1];
       src_lons[1] = src_center_lon[iix];
-      /* For consistency, we must make sure all lons are in same 2pi interval */
+      // For consistency, we must make sure all lons are in same 2pi interval
       if (src_lons[0] > PI2) src_lons[0] -= PI2;
       if (src_lons[0] < 0) src_lons[0] += PI2;
       if (src_lons[1] > PI2) src_lons[1] -= PI2;
