@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,9 @@
 */
 
 #include <algorithm>
-#include "cdo_int.h"
+#include <array>
+
+#include "dmemory.h"
 #include "cdo_options.h"
 #include "remap.h"
 #include "remap_store_link.h"
@@ -57,7 +59,7 @@ sortAddweights(size_t numWeights, Addweight *addweights)
     if (addweights[n].add < addweights[n - 1].add) break;
   if (n == numWeights) return;
 
-  qsort(addweights, numWeights, sizeof(Addweight), qcompareAdds);
+  std::qsort(addweights, numWeights, sizeof(Addweight), qcompareAdds);
 }
 
 static void
@@ -68,11 +70,37 @@ sortAddweights4(Addweight4 *addweights)
     if (addweights[n].add < addweights[n - 1].add) break;
   if (n == 4) return;
 
-  qsort(addweights, 4, sizeof(Addweight4), qcompareAdds4);
+  std::qsort(addweights, 4, sizeof(Addweight4), qcompareAdds4);
 }
 
 void
-sort_add_and_wgts(size_t numWeights, size_t *src_add, double *wgts)
+sort_weights_n4(size_t *src_add, double *weights)
+{
+  constexpr size_t numWeights = 4;
+  size_t n;
+  for (n = 1; n < numWeights; ++n)
+    if (src_add[n] < src_add[n - 1]) break;
+  if (n == numWeights) return;
+
+  std::array<Addweight, numWeights> addweights;
+
+  for (n = 0; n < numWeights; ++n)
+    {
+      addweights[n].add = src_add[n];
+      addweights[n].weight = weights[n];
+    }
+
+  std::sort(addweights.begin(), addweights.end(), compareAdds);
+
+  for (n = 0; n < numWeights; ++n)
+    {
+      src_add[n] = addweights[n].add;
+      weights[n] = addweights[n].weight;
+    }
+}
+
+void
+sort_weights(size_t numWeights, size_t *src_add, double *weights)
 {
   size_t n;
   for (n = 1; n < numWeights; ++n)
@@ -86,7 +114,7 @@ sort_add_and_wgts(size_t numWeights, size_t *src_add, double *wgts)
       for (n = 0; n < numWeights; ++n)
         {
           addweights[n].add = src_add[n];
-          addweights[n].weight = wgts[n];
+          addweights[n].weight = weights[n];
         }
 
       std::sort(addweights.begin(), addweights.end(), compareAdds);
@@ -94,42 +122,40 @@ sort_add_and_wgts(size_t numWeights, size_t *src_add, double *wgts)
       for (n = 0; n < numWeights; ++n)
         {
           src_add[n] = addweights[n].add;
-          wgts[n] = addweights[n].weight;
+          weights[n] = addweights[n].weight;
         }
     }
 }
 
 void
-sort_add_and_wgts4(size_t numWeights, size_t *src_add, double wgts[4][4])
+sort_weights_bicubic(size_t *src_add, double (&weights)[4][4])
 {
+  constexpr size_t numWeights = 4;
   size_t n;
   for (n = 1; n < numWeights; ++n)
     if (src_add[n] < src_add[n - 1]) break;
   if (n == numWeights) return;
 
-  if (numWeights > 1)
+  std::array<Addweight4, numWeights> addweights;
+
+  for (n = 0; n < numWeights; ++n)
     {
-      std::vector<Addweight4> addweights(numWeights);
+      addweights[n].add = src_add[n];
+      for (unsigned k = 0; k < 4; ++k) addweights[n].weight[k] = weights[n][k];
+    }
 
-      for (n = 0; n < numWeights; ++n)
-        {
-          addweights[n].add = src_add[n];
-          for (unsigned k = 0; k < 4; ++k) addweights[n].weight[k] = wgts[n][k];
-        }
+  std::sort(addweights.begin(), addweights.end(), compareAdds4);
 
-      std::sort(addweights.begin(), addweights.end(), compareAdds4);
-
-      for (n = 0; n < numWeights; ++n)
-        {
-          src_add[n] = addweights[n].add;
-          for (unsigned k = 0; k < 4; ++k) wgts[n][k] = addweights[n].weight[k];
-        }
+  for (n = 0; n < numWeights; ++n)
+    {
+      src_add[n] = addweights[n].add;
+      for (unsigned k = 0; k < 4; ++k) weights[n][k] = addweights[n].weight[k];
     }
 }
 
 void
-storeWeightLinks(int lalloc, size_t numWeights, size_t *srch_add, double *weights, size_t cell_add,
-                 std::vector<WeightLinks> &weightLinks)
+store_weightlinks(int lalloc, size_t numWeights, size_t *srch_add, double *weights, size_t cell_add,
+                  std::vector<WeightLinks> &weightLinks)
 {
   weightLinks[cell_add].nlinks = 0;
   weightLinks[cell_add].offset = 0;
@@ -157,7 +183,7 @@ storeWeightLinks(int lalloc, size_t numWeights, size_t *srch_add, double *weight
 }
 
 void
-storeWeightLinks4(size_t *srch_add, double weights[4][4], size_t cell_add, std::vector<WeightLinks4> &weightLinks)
+store_weightlinks_bicubic(size_t *srch_add, double (&weights)[4][4], size_t cell_add, std::vector<WeightLinks4> &weightLinks)
 {
   weightLinks[cell_add].nlinks = 0;
   weightLinks[cell_add].offset = 0;
@@ -179,7 +205,6 @@ void
 weightLinksToRemapLinks(int lalloc, size_t gridSize, std::vector<WeightLinks> &weightLinks, RemapVars &rv)
 {
   size_t nlinks = 0;
-
   for (size_t i = 0; i < gridSize; ++i)
     {
       if (weightLinks[i].nlinks)
@@ -194,23 +219,23 @@ weightLinksToRemapLinks(int lalloc, size_t gridSize, std::vector<WeightLinks> &w
 
   if (nlinks)
     {
-      size_t num_wts = rv.num_wts;
+      auto num_wts = rv.num_wts;
       rv.src_cell_add.resize(nlinks);
       rv.tgt_cell_add.resize(nlinks);
       rv.wts.resize(nlinks * num_wts);
-      size_t *restrict src_cell_adds = rv.src_cell_add.data();
-      size_t *restrict tgt_cell_adds = rv.tgt_cell_add.data();
-      double *restrict wts = rv.wts.data();
+      auto &src_cell_adds = rv.src_cell_add;
+      auto &tgt_cell_adds = rv.tgt_cell_add;
+      auto &wts = rv.wts;
 
 #ifdef _OPENMP
 #pragma omp parallel for schedule(static) default(none) shared(src_cell_adds, tgt_cell_adds, wts, weightLinks, gridSize, num_wts)
 #endif
       for (size_t i = 0; i < gridSize; ++i)
         {
-          size_t num_links = weightLinks[i].nlinks;
+          const auto num_links = weightLinks[i].nlinks;
           if (num_links)
             {
-              size_t offset = weightLinks[i].offset;
+              const auto offset = weightLinks[i].offset;
               Addweight *addweights = weightLinks[i].addweights;
               for (size_t ilink = 0; ilink < num_links; ++ilink)
                 {
@@ -225,7 +250,7 @@ weightLinksToRemapLinks(int lalloc, size_t gridSize, std::vector<WeightLinks> &w
         {
           for (size_t i = 0; i < gridSize; ++i)
             {
-              size_t num_links = weightLinks[i].nlinks;
+              const auto num_links = weightLinks[i].nlinks;
               if (num_links) Free(weightLinks[i].addweights);
             }
         }
@@ -240,7 +265,6 @@ void
 weightLinks4ToRemapLinks(size_t gridSize, std::vector<WeightLinks4> &weightLinks, RemapVars &rv)
 {
   size_t nlinks = 0;
-
   for (size_t i = 0; i < gridSize; ++i)
     {
       if (weightLinks[i].nlinks)
@@ -257,20 +281,20 @@ weightLinks4ToRemapLinks(size_t gridSize, std::vector<WeightLinks4> &weightLinks
       rv.src_cell_add.resize(nlinks);
       rv.tgt_cell_add.resize(nlinks);
       rv.wts.resize(4 * nlinks);
-      size_t *restrict src_cell_adds = &rv.src_cell_add[0];
-      size_t *restrict tgt_cell_adds = &rv.tgt_cell_add[0];
-      double *restrict wts = &rv.wts[0];
+      auto &src_cell_adds = rv.src_cell_add;
+      auto &tgt_cell_adds = rv.tgt_cell_add;
+      auto &wts = rv.wts;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(src_cell_adds, tgt_cell_adds, wts, weightLinks, gridSize)
 #endif
       for (size_t i = 0; i < gridSize; ++i)
         {
-          size_t num_links = weightLinks[i].nlinks;
+          const auto num_links = weightLinks[i].nlinks;
           if (num_links)
             {
-              size_t offset = weightLinks[i].offset;
-              Addweight4 *addweights = weightLinks[i].addweights;
+              const auto offset = weightLinks[i].offset;
+              const auto addweights = weightLinks[i].addweights;
               for (size_t ilink = 0; ilink < num_links; ++ilink)
                 {
                   src_cell_adds[offset + ilink] = addweights[ilink].add;
