@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,11 @@
 #ifndef REMAP_STORE_LINK_H
 #define REMAP_STORE_LINK_H
 
+#include <vector>
+
+// Predeclarations
+struct RemapVars;
+
 struct Addweight
 {
   size_t add;
@@ -45,12 +50,13 @@ struct WeightLinks4
 
 void weightLinksAlloc(size_t numNeighbors, size_t gridSize, std::vector<WeightLinks> &weightLinks);
 void weightLinks4Alloc(size_t gridSize, std::vector<WeightLinks4> &weightLinks);
-void storeWeightLinks(int lalloc, size_t numWeights, size_t *srch_add, double *weights, size_t cell_add,
-                      std::vector<WeightLinks> &weightLinks);
-void storeWeightLinks4(size_t *srch_add, double weights[4][4], size_t cell_add, std::vector<WeightLinks4> &weightLinks);
+void store_weightlinks(int lalloc, size_t numWeights, size_t *srch_add, double *weights, size_t cell_add,
+                       std::vector<WeightLinks> &weightLinks);
+void store_weightlinks_bicubic(size_t *srch_add, double (&weights)[4][4], size_t cell_add, std::vector<WeightLinks4> &weightLinks);
 void weightLinksToRemapLinks(int lalloc, size_t gridSize, std::vector<WeightLinks> &weightLinks, RemapVars &rv);
 void weightLinks4ToRemapLinks(size_t gridSize, std::vector<WeightLinks4> &weightLinks, RemapVars &rv);
-void sort_add_and_wgts(size_t numWeights, size_t *src_add, double *wgts);
-void sort_add_and_wgts4(size_t numWeights, size_t *src_add, double wgts[4][4]);
+void sort_weights(size_t numWeights, size_t *src_add, double *weights);
+void sort_weights_n4(size_t *src_add, double *weights);
+void sort_weights_bicubic(size_t *src_add, double (&weights)[4][4]);
 
 #endif /* REMAP_STORE_LINK */
