@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,13 +15,14 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include "cdo_options.h"
 #include "remap.h"
 #include "remap_store_link_cnsrv.h"
 
 void
-grid_store_init(grid_store_t *grid_store, long gridsize)
+grid_store_init(GridStore &grid_store, long gridsize)
 {
   long iblk;
   long blksize[] = { 128, 256, 512, 1024, 2048, 4096, 8192 };
@@ -33,43 +34,43 @@ grid_store_init(grid_store_t *grid_store, long gridsize)
 
   if (iblk < 0) iblk = 0;
 
-  /* grid_store->blk_size = BLK_SIZE; */
-  grid_store->blk_size = blksize[iblk];
-  grid_store->max_size = gridsize;
+  /* grid_store.blk_size = BLK_SIZE; */
+  grid_store.blk_size = blksize[iblk];
+  grid_store.max_size = gridsize;
 
-  grid_store->nblocks = grid_store->max_size / grid_store->blk_size;
-  if (grid_store->max_size % grid_store->blk_size > 0) grid_store->nblocks++;
+  grid_store.nblocks = grid_store.max_size / grid_store.blk_size;
+  if (grid_store.max_size % grid_store.blk_size > 0) grid_store.nblocks++;
 
   if (Options::cdoVerbose)
-    fprintf(stdout, "blksize = %ld  lastblksize = %ld  max_size = %ld  nblocks = %ld\n", grid_store->blk_size,
-            grid_store->max_size % grid_store->blk_size, grid_store->max_size, grid_store->nblocks);
+    fprintf(stdout, "blksize = %ld  lastblksize = %ld  max_size = %ld  nblocks = %ld\n", grid_store.blk_size,
+            grid_store.max_size % grid_store.blk_size, grid_store.max_size, grid_store.nblocks);
 
-  grid_store->blksize = (long *) Malloc(grid_store->nblocks * sizeof(long));
-  grid_store->nlayers = (long *) Malloc(grid_store->nblocks * sizeof(long));
-  grid_store->layers = (grid_layer_t **) Malloc(grid_store->nblocks * sizeof(grid_layer_t *));
+  grid_store.blksize = (long *) Malloc(grid_store.nblocks * sizeof(long));
+  grid_store.nlayers = (long *) Malloc(grid_store.nblocks * sizeof(long));
+  grid_store.layers = (GridLayer **) Malloc(grid_store.nblocks * sizeof(GridLayer *));
 
-  nblocks = grid_store->nblocks;
+  nblocks = grid_store.nblocks;
   for (iblk = 0; iblk < nblocks; ++iblk)
     {
-      grid_store->blksize[iblk] = grid_store->blk_size;
-      grid_store->nlayers[iblk] = 0;
-      grid_store->layers[iblk] = nullptr;
+      grid_store.blksize[iblk] = grid_store.blk_size;
+      grid_store.nlayers[iblk] = 0;
+      grid_store.layers[iblk] = nullptr;
     }
-  if (grid_store->max_size % grid_store->blk_size > 0)
-    grid_store->blksize[grid_store->nblocks - 1] = grid_store->max_size % grid_store->blk_size;
+  if (grid_store.max_size % grid_store.blk_size > 0)
+    grid_store.blksize[grid_store.nblocks - 1] = grid_store.max_size % grid_store.blk_size;
 }
 
 void
-grid_store_delete(grid_store_t *grid_store)
+grid_store_delete(GridStore &grid_store)
 {
-  long nblocks = grid_store->nblocks;
+  long nblocks = grid_store.nblocks;
 
   for (long iblk = 0; iblk < nblocks; ++iblk)
     {
       long j = 0;
-      grid_layer_t *grid_layer = grid_store->layers[iblk];
-      long nlayers = grid_store->nlayers[iblk];
-      long blksize = grid_store->blksize[iblk];
+      GridLayer *grid_layer = grid_store.layers[iblk];
+      long nlayers = grid_store.nlayers[iblk];
+      long blksize = grid_store.blksize[iblk];
       for (long ilayer = 0; ilayer < nlayers; ++ilayer)
         {
           if (Options::cdoVerbose)
@@ -78,7 +79,7 @@ grid_store_delete(grid_store_t *grid_store)
                 if (grid_layer->grid2_link[i] != -1) j++;
             }
 
-          grid_layer_t *grid_layer_f = grid_layer;
+          GridLayer *grid_layer_f = grid_layer;
           Free(grid_layer->grid2_link);
           grid_layer = grid_layer->next;
           Free(grid_layer_f);
@@ -92,9 +93,9 @@ grid_store_delete(grid_store_t *grid_store)
       */
     }
 
-  Free(grid_store->blksize);
-  Free(grid_store->layers);
-  Free(grid_store->nlayers);
+  Free(grid_store.blksize);
+  Free(grid_store.layers);
+  Free(grid_store.nlayers);
 }
 
 /*
@@ -102,7 +103,7 @@ grid_store_delete(grid_store_t *grid_store)
     address and weight arrays and resizes those arrays if necessary.
 */
 void
-store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weights, grid_store_t *grid_store)
+store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weights, GridStore &grid_store)
 {
   /*
     Input variables:
@@ -112,7 +113,7 @@ store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weig
   */
   /* Local variables */
   long nlink = 0; /* link index */
-  grid_layer_t *grid_layer = nullptr;
+  GridLayer *grid_layer = nullptr;
 
   /*  If all weights are ZERO, do not bother storing the link */
   if (num_wts == 3)
@@ -130,8 +131,8 @@ store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weig
   long iadd2 = BLK_IDX(add2);
 
   bool lstore_link = false;
-  grid_layer_t **grid_layer2 = &grid_store->layers[iblk];
-  long nlayer = grid_store->nlayers[iblk];
+  GridLayer **grid_layer2 = &grid_store.layers[iblk];
+  long nlayer = grid_store.nlayers[iblk];
   long ilayer;
   for (ilayer = 0; ilayer < nlayer; ++ilayer)
     {
@@ -156,28 +157,27 @@ store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weig
     }
 
   /*
-     If the link does not yet exist, increment number of links and
-     check to see if remap arrays need to be increased to accomodate
-     the new link. Then store the link.
+     If the link does not yet exist, increment number of links and check to see if remap arrays 
+     need to be increased to accomodate the new link. Then store the link.
   */
   nlink = rv.num_links;
 
-  if (ilayer < grid_store->nlayers[iblk])
+  if (ilayer < grid_store.nlayers[iblk])
     {
       grid_layer->grid2_link[iadd2] = nlink;
     }
   else
     {
-      grid_layer = (grid_layer_t *) Malloc(sizeof(grid_layer_t));
+      grid_layer = (GridLayer *) Malloc(sizeof(GridLayer));
       grid_layer->next = nullptr;
-      grid_layer->grid2_link = (long *) Malloc(grid_store->blksize[iblk] * sizeof(long));
+      grid_layer->grid2_link = (long *) Malloc(grid_store.blksize[iblk] * sizeof(long));
 
-      long blksize = grid_store->blksize[iblk];
+      long blksize = grid_store.blksize[iblk];
       for (long i = 0; i < blksize; ++i) grid_layer->grid2_link[i] = -1;
 
       grid_layer->grid2_link[iadd2] = nlink;
       *grid_layer2 = grid_layer;
-      grid_store->nlayers[iblk]++;
+      grid_store.nlayers[iblk]++;
     }
 
   rv.num_links++;
