@@ -1,18 +1,12 @@
-#include "cdo_int.h"
+
+#include "array.h"
+#include "process_int.h"
 #include "cdo_wtime.h"
 #include "cdo_options.h"
 #include "remap_vars.h"
 #include "timer.h"
 #include "cimdOmp.h"
 
-template <typename T>
-inline void
-vectorFree(std::vector<T> &v)
-{
-  v.clear();
-  v.shrink_to_fit();
-}
-
 /*
   -----------------------------------------------------------------------
 
@@ -24,27 +18,27 @@ vectorFree(std::vector<T> &v)
 static void
 remapFirstOrder(double *restrict dst_array, const RemapVars &rv, const double *restrict src_array)
 {
-  size_t num_links = rv.num_links;
-  size_t num_wts = rv.num_wts;
-  const double *restrict map_wts = rv.wts.data();
-  const size_t *restrict dst_add = rv.tgt_cell_add.data();
-  const size_t *restrict src_add = rv.src_cell_add.data();
-  remaplink_t links = rv.links;
-  long links_per_value = rv.links_per_value;
+  auto num_links = rv.num_links;
+  auto num_wts = rv.num_wts;
+  const auto &map_wts = rv.wts;
+  const auto &dst_add = rv.tgt_cell_add;
+  const auto &src_add = rv.src_cell_add;
+  const auto &links = rv.links;
+  const auto links_per_value = rv.links_per_value;
 
   if (links.option)
     {
 #ifdef HAVE_OPENMP4
 #pragma omp parallel for simd default(none) shared(num_links, dst_array, dst_add)
 #endif
-      for (size_t n = 0; n < num_links; ++n) dst_array[dst_add[n]] = 0.;
+      for (size_t n = 0; n < num_links; ++n) dst_array[dst_add[n]] = 0.0;
 
       for (size_t j = 0; j < links.num_blks; ++j)
         {
-          const size_t *restrict dst_addx = links.dst_add[j];
-          const size_t *restrict src_addx = links.src_add[j];
-          const size_t *restrict windex = links.w_index[j];
-          size_t nlinks = links.num_links[j];
+          const auto &dst_addx = links.dst_add[j];
+          const auto &src_addx = links.src_add[j];
+          const auto &windex = links.w_index[j];
+          auto nlinks = links.num_links[j];
 
 #ifdef HAVE_OPENMP4
 #pragma omp parallel for simd default(none) shared(nlinks, dst_array, src_array, dst_addx, src_addx, map_wts, num_wts, windex)
@@ -57,7 +51,7 @@ remapFirstOrder(double *restrict dst_array, const RemapVars &rv, const double *r
     }
   else
     {
-      long lpv = links_per_value;
+      auto lpv = links_per_value;
       if (lpv > 0)
         {
           size_t nlinks = num_links / lpv;
@@ -79,7 +73,7 @@ remapFirstOrder(double *restrict dst_array, const RemapVars &rv, const double *r
 #endif
               for (size_t n = 0; n < nlinks; ++n)
                 {
-                  size_t noff = n * 4;
+                  const auto noff = n * 4;
                   dst_array[dst_add[noff]] = src_array[src_add[noff]] * map_wts[num_wts * (noff)]
                                              + src_array[src_add[noff + 1]] * map_wts[num_wts * (noff + 1)]
                                              + src_array[src_add[noff + 2]] * map_wts[num_wts * (noff + 2)]
@@ -93,7 +87,7 @@ remapFirstOrder(double *restrict dst_array, const RemapVars &rv, const double *r
 #endif
               for (size_t n = 0; n < nlinks; ++n)
                 {
-                  size_t noff = n * lpv;
+                  const auto noff = n * lpv;
                   dst_array[dst_add[noff]] = src_array[src_add[noff]] * map_wts[num_wts * noff];
                   for (size_t k = 1; k < (size_t) lpv; ++k)
                     dst_array[dst_add[noff]] += src_array[src_add[noff + k]] * map_wts[num_wts * (noff + k)];
@@ -105,7 +99,7 @@ remapFirstOrder(double *restrict dst_array, const RemapVars &rv, const double *r
 #ifdef SX
 #pragma cdir nodep
 #endif
-          for (size_t n = 0; n < num_links; ++n) dst_array[dst_add[n]] = 0.;
+          for (size_t n = 0; n < num_links; ++n) dst_array[dst_add[n]] = 0.0;
 
           for (size_t n = 0; n < num_links; ++n)
             {
@@ -119,35 +113,38 @@ remapFirstOrder(double *restrict dst_array, const RemapVars &rv, const double *r
 static void
 remapSecondOrder(double *restrict dst_array, const RemapVars &rv, const double *restrict src_array, RemapGradients &gradients)
 {
-  const double *restrict src_grad1 = gradients.grad_lat.size() ? gradients.grad_lat.data() : nullptr;
-  const double *restrict src_grad2 = gradients.grad_lon.size() ? gradients.grad_lon.data() : nullptr;
-  const double *restrict src_grad3 = gradients.grad_latlon.size() ? gradients.grad_latlon.data() : nullptr;
+  const auto &grad1 = gradients.grad_lat;
+  const auto &grad2 = gradients.grad_lon;
+  const auto &grad3 = gradients.grad_latlon;
 
-  size_t num_links = rv.num_links;
-  size_t num_wts = rv.num_wts;
-  const double *restrict map_wts = rv.wts.data();
-  const size_t *restrict dst_add = rv.tgt_cell_add.data();
-  const size_t *restrict src_add = rv.src_cell_add.data();
+  const auto num_links = rv.num_links;
+  const auto num_wts = rv.num_wts;
+  const auto &map_wts = rv.wts;
+  const auto &dst_add = rv.tgt_cell_add;
+  const auto &src_add = rv.src_cell_add;
 
 #ifdef SX
 #pragma cdir nodep
 #endif
-  for (size_t n = 0; n < num_links; ++n) dst_array[dst_add[n]] = 0.;
+  for (size_t n = 0; n < num_links; ++n) dst_array[dst_add[n]] = 0.0;
 
   if (num_wts == 3)
     {
       for (size_t n = 0; n < num_links; ++n)
         {
-          dst_array[dst_add[n]] += src_array[src_add[n]] * map_wts[3 * n] + src_grad1[src_add[n]] * map_wts[3 * n + 1]
-                                   + src_grad2[src_add[n]] * map_wts[3 * n + 2];
+          const auto i = src_add[n];
+          const auto w = &map_wts[3 * n];
+          dst_array[dst_add[n]] += src_array[i] * w[0] + grad1[i] * w[1] + grad2[i] * w[2];
+          // printf("%zu %zu %.5f %.5f %.5f %.5f %.5f\n", n, src_add[n], grad1[i], grad2[i], w[0], w[1], w[2]);
         }
     }
   else if (num_wts == 4)
     {
       for (size_t n = 0; n < num_links; ++n)
         {
-          dst_array[dst_add[n]] += src_array[src_add[n]] * map_wts[4 * n] + src_grad1[src_add[n]] * map_wts[4 * n + 1]
-                                   + src_grad2[src_add[n]] * map_wts[4 * n + 2] + src_grad3[src_add[n]] * map_wts[4 * n + 3];
+          const auto i = src_add[n];
+          const auto w = &map_wts[4 * n];
+          dst_array[dst_add[n]] += src_array[i] * w[0] + grad1[i] * w[1] + grad2[i] * w[2] + grad3[i] * w[3];
         }
     }
 }
@@ -159,30 +156,27 @@ remap(double *restrict dst_array, double missval, size_t dst_size, const RemapVa
   /*
     Input arrays:
 
-    int *dst_add         ! destination address for each link
-    int *src_add         ! source      address for each link
+      dst_add    destination address for each link
+      src_add    source      address for each link
+      num_wts    num of weights used in remapping
+      map_wts    remapping weights for each link
+      src_array  array with source field to be remapped
 
-    int num_wts          ! num of weights used in remapping
+    Optional:
 
-    double *map_wts      ! remapping weights for each link
+      gradients  gradient arrays on source grid necessary for higher-order remappings
 
-    double *src_array    ! array with source field to be remapped
+    Output variables:
 
-    optional:
-
-    RemapGradients &gradients  ! gradient arrays on source grid necessary for higher-order remappings
-
-    output variables:
-
-    double *dst_array    ! array for remapped field on destination grid
+      dst_array  array for remapped field on destination grid
   */
   extern int timer_remap;
 
-  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
+  const auto start = Options::cdoVerbose ? cdo_get_wtime() : 0.0;
 
   // Check the order of the interpolation
 
-  bool firstOrder = gradients.grad_lat.size() == 0;
+  const bool firstOrder = gradients.grad_lat.size() == 0;
 
 #ifdef HAVE_OPENMP4
 #pragma omp parallel for simd default(none) shared(dst_size, dst_array, missval)
@@ -206,28 +200,27 @@ remap(double *restrict dst_array, double missval, size_t dst_size, const RemapVa
 }
 
 static size_t
-get_max_add(size_t num_links, size_t size, const size_t *restrict add)
+get_max_add(size_t num_links, size_t size, const Varray<size_t> &add)
 {
-  size_t *isum = (size_t *) Calloc(size, sizeof(size_t));
+  std::vector<size_t> isum(size, 0);
 
   for (size_t n = 0; n < num_links; ++n) isum[add[n]]++;
 
   size_t max_add = 0;
   for (size_t i = 0; i < size; ++i)
     if (isum[i] > max_add) max_add = isum[i];
-  Free(isum);
 
   return max_add;
 }
 
 static size_t
-binary_search_int(const size_t *array, size_t len, size_t value)
+binary_search_int(const Varray<size_t> &array, size_t len, size_t value)
 {
-  int64_t low = 0, high = len - 1, midpoint = 0;
+  int64_t low = 0, high = len - 1;
 
   while (low <= high)
     {
-      midpoint = low + (high - low) / 2;
+      const auto midpoint = low + (high - low) / 2;
 
       // check to see if value is equal to item in array
       if (value == array[midpoint]) return midpoint;
@@ -253,39 +246,30 @@ void
 remap_laf(double *restrict dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *restrict src_array)
 {
   /*
-    Input arrays:
+    Input:
+      src_array : array with source field to be remapped
 
-    int *dst_add         ! destination address for each link
-    int *src_add         ! source      address for each link
-
-    int num_wts          ! num of weights used in remapping
-
-    double *map_wts      ! remapping weights for each link
-
-    double *src_array    ! array with source field to be remapped
-
-    output variables:
-
-    double *dst_array    ! array for remapped field on destination grid
+    Output:
+      dst_array : array for remapped field on destination grid
   */
-  size_t num_links = rv.num_links;
-  size_t num_wts = rv.num_wts;
-  const double *restrict map_wts = &rv.wts[0];
-  const size_t *restrict dst_add = &rv.tgt_cell_add[0];
-  const size_t *restrict src_add = &rv.src_cell_add[0];
+  auto num_links = rv.num_links;
+  auto num_wts = rv.num_wts;              // num of weights used in remapping
+  const auto &map_wts = rv.wts;           // remapping weights for each link
+  const auto &dst_add = rv.tgt_cell_add;  // destination address for each link
+  const auto &src_add = rv.src_cell_add;  // source      address for each link
 
-  arrayFill(dst_size, dst_array, missval);
+  varrayFill(dst_size, dst_array, missval);
 
   if (num_links == 0) return;
 
-  size_t max_cls = get_max_add(num_links, dst_size, dst_add);
+  auto max_cls = get_max_add(num_links, dst_size, dst_add);
 
 #ifdef _OPENMP
-  VECTOR_2D(double, src_cls2, Threading::ompNumThreads, max_cls);
-  VECTOR_2D(double, src_wts2, Threading::ompNumThreads, max_cls);
+  Varray2D<double> src_cls2(Threading::ompNumThreads, Varray<double>(max_cls));
+  Varray2D<double> src_wts2(Threading::ompNumThreads, Varray<double>(max_cls));
 #else
-  std::vector<double> src_cls(max_cls);
-  std::vector<double> src_wts(max_cls);
+  Varray<double> src_cls(max_cls);
+  Varray<double> src_wts(max_cls);
 #endif
 
   for (size_t n = 0; n < num_links; ++n)
@@ -300,12 +284,12 @@ remap_laf(double *restrict dst_array, double missval, size_t dst_size, const Rem
       size_t k;
       size_t ncls;
 #ifdef _OPENMP
-      int ompthID = cdo_omp_get_thread_num();
-      double *src_cls = &src_cls2[ompthID][0];
-      double *src_wts = &src_wts2[ompthID][0];
+      const auto ompthID = cdo_omp_get_thread_num();
+      auto &src_cls = src_cls2[ompthID];
+      auto &src_wts = src_wts2[ompthID];
 #endif
-      arrayFill(max_cls, &src_cls[0], 0.0);
-      arrayFill(max_cls, &src_wts[0], 0.0);
+      varrayFill(src_cls, 0.0);
+      varrayFill(src_wts, 0.0);
       /*
       ncls = 0;
       for ( n = 0; n < num_links; n++ )
@@ -325,11 +309,11 @@ remap_laf(double *restrict dst_array, double missval, size_t dst_size, const Rem
             }
         }
       */
-      /* only for sorted dst_add! */
+      // only for sorted dst_add!
       {
         size_t min_add = 1, max_add = 0;
 
-        size_t n = binary_search_int(dst_add, num_links, i);
+        auto n = binary_search_int(dst_add, num_links, i);
 
         if (n < num_links)
           {
@@ -349,23 +333,26 @@ remap_laf(double *restrict dst_array, double missval, size_t dst_size, const Rem
         ncls = 0;
         for (n = min_add; n < max_add; ++n)
           {
+            const auto value = src_array[src_add[n]];
+
             for (k = 0; k < ncls; ++k)
-              if (IS_EQUAL(src_array[src_add[n]], src_cls[k])) break;
+              if (IS_EQUAL(value, src_cls[k])) break;
 
             if (k == ncls)
               {
-                src_cls[k] = src_array[src_add[n]];
+                src_cls[k] = value;
                 ncls++;
               }
 
             src_wts[k] += map_wts[num_wts * n];
           }
+        // printf("i, min_add, max_add, ncls %zu %zu %zu %zu\n", i, min_add, max_add, ncls);
       }
 
       if (ncls)
         {
           size_t imax = 0;
-          double wts = src_wts[0];
+          auto wts = src_wts[0];
           for (k = 1; k < ncls; ++k)
             {
               if (src_wts[k] > wts)
@@ -375,44 +362,27 @@ remap_laf(double *restrict dst_array, double missval, size_t dst_size, const Rem
                 }
             }
 
+          // for (k = 0; k < ncls; ++k) printf("   k, src_wts[k],  src_cls[k] %zu %g %g\n", k, src_wts[k],  src_cls[k]);
+          // printf("imax, src_wts[imax],  src_cls[imax] %zu %zu %g %g\n", i , imax, src_wts[imax],  src_cls[imax]);
           dst_array[i] = src_cls[imax];
         }
     }
 }
 
-/*
-  -----------------------------------------------------------------------
-
-  Performs the remapping based on weights computed elsewhere
-
-  -----------------------------------------------------------------------
-*/
 void
-remap_sum(double *restrict dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *restrict src_array)
+remap_avg(double *restrict dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *restrict src_array)
 {
+  const auto num_links = rv.num_links;
+  const auto num_wts = rv.num_wts;        // num of weights used in remapping
+  const auto &map_wts = rv.wts;           // remapping weights for each link
+  const auto &dst_add = rv.tgt_cell_add;  // destination address for each link
+  const auto &src_add = rv.src_cell_add;  // source      address for each link
+
   /*
-    Input arrays:
-
-    int *dst_add         ! destination address for each link
-    int *src_add         ! source      address for each link
-
-    int num_wts          ! num of weights used in remapping
-
-    double *map_wts      ! remapping weights for each link
-    double *src_array    ! array with source field to be remapped
-
-    output variables:
-
-    double *dst_array    ! array for remapped field on destination grid
-  */
-  size_t num_links = rv.num_links;
-  size_t num_wts = rv.num_wts;
-  const double *restrict map_wts = &rv.wts[0];
-  const size_t *restrict dst_add = &rv.tgt_cell_add[0];
-  const size_t *restrict src_add = &rv.src_cell_add[0];
-
   for (size_t n = 0; n < dst_size; ++n) dst_array[n] = missval;
 
+  std::vector<int> count(dst_size, 0);
+
 #ifdef SX
 #pragma cdir nodep
 #endif
@@ -421,15 +391,73 @@ remap_sum(double *restrict dst_array, double missval, size_t dst_size, const Rem
 
   for (size_t n = 0; n < num_links; ++n)
     {
-      /*
-        printf("%5d %5d %5d %g # dst_add src_add n\n", dst_add[n], src_add[n],
-        n, map_wts[num_wts*n]);
-      */
+      // printf("%5d %5d %5d %g # dst_add src_add n\n", dst_add[n], src_add[n], n, map_wts[num_wts*n]);
       // dst_array[dst_add[n]] += src_array[src_add[n]]*map_wts[num_wts*n];
-      dst_array[dst_add[n]] += src_array[src_add[n]] * map_wts[num_wts * n];
-      printf("%zu %zu %zu %g %g %g\n", n, dst_add[n], src_add[n], src_array[src_add[n]], map_wts[num_wts * n],
-             dst_array[dst_add[n]]);
+      dst_array[dst_add[n]] += src_array[src_add[n]];
+      count[dst_add[n]] += 1;
+      if (src_cell_frac[src_add[n]] < 1.0)
+        printf("%zu %zu %zu %g %g %g %g\n", n, dst_add[n], src_add[n], src_array[src_add[n]], map_wts[num_wts * n],
+               dst_array[dst_add[n]], src_cell_frac[src_add[n]]);
     }
+
+  for (size_t i = 0; i < dst_size; ++i)
+    {
+      if (count[i] > 0) dst_array[i] /= count[i];
+    }
+  */
+  /*
+#ifdef _OPENMP
+#pragma omp parallel for default(none) shared(missval, dst_size, num_links, dst_add, src_add, src_array, map_wts, \
+                                              num_wts, dst_array) schedule(dynamic, 1)
+#endif
+  */
+  size_t max_values = 0;
+  for (size_t i = 0; i < dst_size; ++i)
+    {
+      size_t nvalues = 0;
+      double sum = 0.0;
+      double wts = 0.0;
+      // only for sorted dst_add!
+      {
+        size_t min_add = 1, max_add = 0;
+
+        auto n = binary_search_int(dst_add, num_links, i);
+
+        if (n < num_links)
+          {
+            min_add = n;
+
+            for (n = min_add + 1; n < num_links; ++n)
+              if (i != dst_add[n]) break;
+
+            max_add = n;
+
+            for (n = min_add; n > 0; --n)
+              if (i != dst_add[n - 1]) break;
+
+            min_add = n;
+          }
+
+        // const auto nadds = (max_add - min_add) + 1;
+        // const double lim = 0.1 / nadds;
+        const double lim = 0.0;
+        for (n = min_add; n < max_add; ++n)
+          {
+            const auto value = src_array[src_add[n]];
+            if (map_wts[num_wts * n] > lim && !DBL_IS_EQUAL(value, missval))
+              {
+                sum += value;
+                wts += map_wts[num_wts * n];
+                nvalues++;
+              }
+          }
+      }
+
+      dst_array[i] = (nvalues > 0) ? sum / nvalues : missval;
+      // printf("%zu %zu %g %g\n", i+1, nvalues, wts, sum);
+      max_values += nvalues;
+    }
+  // printf("max_values = %zu  num_links = %zu\n", max_values, num_links);
 }
 
 void
@@ -452,10 +480,6 @@ remapVarsInit(RemapMethod mapType, int remapOrder, RemapVars &rv)
   rv.links.option = false;
   rv.links.max_links = 0;
   rv.links.num_blks = 0;
-  rv.links.num_links = nullptr;
-  rv.links.src_add = nullptr;
-  rv.links.dst_add = nullptr;
-  rv.links.w_index = nullptr;
 }
 
 void
@@ -484,10 +508,9 @@ remapVarsResize(RemapVars &rv, size_t size)
 void
 remapVarsReorder(RemapVars &rv)
 {
-  size_t j, nval = 0, num_blks = 0;
-  size_t n;
+  size_t nval = 0, num_blks = 0;
 
-  size_t num_links = rv.num_links;
+  const auto num_links = rv.num_links;
 
   printf("remapVarsReorder\n");
   printf("  num_links %zu\n", num_links);
@@ -495,7 +518,7 @@ remapVarsReorder(RemapVars &rv)
 
   size_t lastval = -1;
   size_t max_links = 0;
-  for (n = 0; n < num_links; n++)
+  for (size_t n = 0; n < num_links; n++)
     {
       if (rv.tgt_cell_add[n] == lastval)
         nval++;
@@ -515,26 +538,26 @@ remapVarsReorder(RemapVars &rv)
 
       printf("num_links %zu  max_links %zu  num_blks %zu\n", rv.num_links, max_links, num_blks);
 
-      rv.links.num_links = (size_t *) Malloc(num_blks * sizeof(size_t));
-      rv.links.dst_add = (size_t **) Malloc(num_blks * sizeof(size_t *));
-      rv.links.src_add = (size_t **) Malloc(num_blks * sizeof(size_t *));
-      rv.links.w_index = (size_t **) Malloc(num_blks * sizeof(size_t *));
+      rv.links.num_links.resize(num_blks);
+      rv.links.dst_add.resize(num_blks);
+      rv.links.src_add.resize(num_blks);
+      rv.links.w_index.resize(num_blks);
     }
 
-  for (j = 0; j < num_blks; j++)
+  for (size_t j = 0; j < num_blks; j++)
     {
-      rv.links.dst_add[j] = (size_t *) Malloc(max_links * sizeof(size_t));
-      rv.links.src_add[j] = (size_t *) Malloc(max_links * sizeof(size_t));
-      rv.links.w_index[j] = (size_t *) Malloc(max_links * sizeof(size_t));
+      rv.links.dst_add[j].resize(max_links);
+      rv.links.src_add[j].resize(max_links);
+      rv.links.w_index[j].resize(max_links);
     }
 
-  for (j = 0; j < num_blks; j++)
+  for (size_t j = 0; j < num_blks; j++)
     {
       nval = 0;
       lastval = -1;
       size_t nlinks = 0;
 
-      for (n = 0; n < num_links; n++)
+      for (size_t n = 0; n < num_links; n++)
         {
           if (rv.tgt_cell_add[n] == lastval)
             nval++;
@@ -566,9 +589,9 @@ remapVarsFree(RemapVars &rv)
       rv.pinit = false;
       rv.sort_add = false;
 
-      vectorFree(rv.src_cell_add);
-      vectorFree(rv.tgt_cell_add);
-      vectorFree(rv.wts);
+      varrayFree(rv.src_cell_add);
+      varrayFree(rv.tgt_cell_add);
+      varrayFree(rv.wts);
 
       if (rv.links.option)
         {
@@ -576,43 +599,41 @@ remapVarsFree(RemapVars &rv)
 
           if (rv.links.num_blks)
             {
-              Free(rv.links.num_links);
-              size_t num_blks = rv.links.num_blks;
+              varrayFree(rv.links.num_links);
+              const auto num_blks = rv.links.num_blks;
               for (size_t i = 0; i < num_blks; ++i)
                 {
-                  Free(rv.links.src_add[i]);
-                  Free(rv.links.dst_add[i]);
-                  Free(rv.links.w_index[i]);
+                  varrayFree(rv.links.src_add[i]);
+                  varrayFree(rv.links.dst_add[i]);
+                  varrayFree(rv.links.w_index[i]);
                 }
-              Free(rv.links.src_add);
-              Free(rv.links.dst_add);
-              Free(rv.links.w_index);
+              varrayFree(rv.links.src_add);
+              varrayFree(rv.links.dst_add);
+              varrayFree(rv.links.w_index);
             }
         }
     }
   else
     fprintf(stderr, "%s Warning: vars not initialized!\n", __func__);
 
-} /* remapVarsFree */
+} // remapVarsFree
 
 void
 remapVarsCheckWeights(const RemapVars &rv)
 {
-  auto num_links = rv.num_links;
-  auto num_wts = rv.num_wts;
-  auto normOpt = rv.normOpt;
-  auto src_cell_add = &rv.src_cell_add[0];
-  auto tgt_cell_add = &rv.tgt_cell_add[0];
-  auto wts = &rv.wts[0];
+  const auto num_links = rv.num_links;
+  const auto num_wts = rv.num_wts;
+  const auto normOpt = rv.normOpt;
+  const auto &src_add = rv.src_cell_add;
+  const auto &tgt_add = rv.tgt_cell_add;
+  const auto &wts = rv.wts;
 
   for (size_t n = 0; n < num_links; ++n)
     {
       if (wts[n * num_wts] < -0.01)
-        cdoPrint("Map weight < 0! grid1idx=%zu grid2idx=%zu nlink=%zu wts=%g", src_cell_add[n], tgt_cell_add[n], n,
-                 wts[n * num_wts]);
+        cdoPrint("Map weight < 0! grid1idx=%zu grid2idx=%zu nlink=%zu wts=%g", src_add[n], tgt_add[n], n, wts[n * num_wts]);
 
       if (normOpt != NormOpt::NONE && wts[n * num_wts] > 1.01)
-        cdoPrint("Map weight > 1! grid1idx=%zu grid2idx=%zu nlink=%zu wts=%g", src_cell_add[n], tgt_cell_add[n], n,
-                 wts[n * num_wts]);
+        cdoPrint("Map weight > 1! grid1idx=%zu grid2idx=%zu nlink=%zu wts=%g", src_add[n], tgt_add[n], n, wts[n * num_wts]);
     }
 }
