@@ -1,14 +1,15 @@
 #ifndef REMAP_VARS_H
 #define REMAP_VARS_H
 
-#include <vector>
+#include <cstdio>  // size_t
+#include <array>
 
 class RemapGradients
 {
 public:
-  std::vector<double> grad_lat;
-  std::vector<double> grad_lon;
-  std::vector<double> grad_latlon;
+  Varray<double> grad_lat;
+  Varray<double> grad_lon;
+  Varray<double> grad_latlon;
 
   void
   init(size_t size)
@@ -22,7 +23,7 @@ public:
   RemapGradients() {}
 };
 
-enum struct RemapMethod
+enum class RemapMethod
 {
   UNDEF,
   BILINEAR,
@@ -32,22 +33,22 @@ enum struct RemapMethod
   CONSERV_SCRIP
 };
 
-enum struct NormOpt
+enum class NormOpt
 {
   NONE,
   DESTAREA,
   FRACAREA
 };
 
-struct remaplink_t
+struct RemapLink
 {
   bool option;
   size_t max_links;
   size_t num_blks;
-  size_t *num_links;
-  size_t **src_add;
-  size_t **dst_add;
-  size_t **w_index;
+  Varray<size_t> num_links;
+  Varray2D<size_t> src_add;
+  Varray2D<size_t> dst_add;
+  Varray2D<size_t> w_index;
 };
 
 struct RemapVars
@@ -62,17 +63,17 @@ struct RemapVars
   size_t num_wts;           // num of weights used in remapping
   size_t resize_increment;  // default amount to increase array size
 
-  std::vector<size_t> src_cell_add;  // source grid address for each link
-  std::vector<size_t> tgt_cell_add;  // target grid address for each link
-  std::vector<double> wts;           // map weights for each link [max_links*num_wts]
+  Varray<size_t> src_cell_add;  // source grid address for each link
+  Varray<size_t> tgt_cell_add;  // target grid address for each link
+  Varray<double> wts;           // map weights for each link [max_links*num_wts]
 
-  remaplink_t links;
+  RemapLink links;
 };
 
 void remap(double *dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *src_array,
            RemapGradients &gradients);
 void remap_laf(double *dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *src_array);
-void remap_sum(double *dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *src_array);
+void remap_avg(double *dst_array, double missval, size_t dst_size, const RemapVars &rv, const double *src_array);
 void remapVarsInit(RemapMethod mapType, int remapOrder, RemapVars &rv);
 void remapVarsEnsureSize(RemapVars &rv, size_t size);
 void remapVarsResize(RemapVars &rv, size_t size);
