@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -56,10 +56,10 @@
 */
 /*
   2013-11-08 Uwe Schulzweida: split remapgrid class to src_grid and tgt_grid
-  2012-01-16 Uwe Schulzweida: alloc grid2_bound_box only for conservative
-  remapping 2011-01-07 Uwe Schulzweida: Changed remap weights from 2D to 1D
-  array 2009-05-25 Uwe Schulzweida: Changed restrict data type from double to
-  int 2009-01-11 Uwe Schulzweida: OpenMP parallelization
+  2012-01-16 Uwe Schulzweida: alloc grid2_bound_box only for conservative remapping
+  2011-01-07 Uwe Schulzweida: Changed remap weights from 2D to 1D array
+  2009-05-25 Uwe Schulzweida: Changed restrict data type from double to int
+  2009-01-11 Uwe Schulzweida: OpenMP parallelization
  */
 
 #ifdef HAVE_CONFIG_H
@@ -70,7 +70,6 @@
 
 #include "cdo_options.h"
 #include "cimdOmp.h"
-#include "dmemory.h"
 #include "process_int.h"
 #include "cdo_wtime.h"
 #include <mpim_grid.h>
@@ -99,7 +98,7 @@ remap_set_int(int remapvar, int value)
 
 /*****************************************************************************/
 static void
-boundboxFromCorners(size_t size, size_t nc, const double *restrict corner_lon, const double *restrict corner_lat,
+boundboxFromCorners(size_t size, size_t nc, const Varray<double> &corner_lon, const Varray<double> &corner_lat,
                     float *restrict bound_box)
 {
 #ifdef _OPENMP
@@ -128,21 +127,20 @@ boundboxFromCorners(size_t size, size_t nc, const double *restrict corner_lon, c
 }
 
 static void
-boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const double *restrict center_lon,
-                   const double *restrict center_lat, float *restrict bound_box)
+boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const Varray<double> &center_lon,
+                   const Varray<double> &center_lat, float *restrict bound_box)
 {
-  size_t idx[4];
-  float tmp_lats[4], tmp_lons[4];
-
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(lonIsCyclic, size, nx, ny, center_lon, center_lat, bound_box) private(idx, tmp_lats, tmp_lons)
+#pragma omp parallel for default(none)  shared(lonIsCyclic, size, nx, ny, center_lon, center_lat, bound_box)
 #endif
   for (size_t n = 0; n < size; n++)
     {
+      size_t idx[4];
+      float tmp_lats[4], tmp_lons[4];
+
       size_t n4 = n << 2;
 
-      /* Find N,S and NE points to this grid point */
+      // Find N,S and NE points to this grid point
 
       size_t j = n / nx;
       size_t i = n - j * nx;
@@ -155,10 +153,10 @@ boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const do
       idx[2] = jp1 * nx + ip1;  // north-east
       idx[3] = jp1 * nx + i;    // north
 
-      /* Find N,S and NE lat/lon coords and check bounding box */
+      // Find N,S and NE lat/lon coords and check bounding box
 
-      for (unsigned j = 0; j < 4; ++j) tmp_lons[j] = center_lon[idx[j]];
-      for (unsigned j = 0; j < 4; ++j) tmp_lats[j] = center_lat[idx[j]];
+      for (unsigned k = 0; k < 4; ++k) tmp_lons[k] = center_lon[idx[k]];
+      for (unsigned k = 0; k < 4; ++k) tmp_lats[k] = center_lat[idx[k]];
 
       bound_box[n4 + 0] = tmp_lats[0];
       bound_box[n4 + 1] = tmp_lats[0];
@@ -175,56 +173,92 @@ boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const do
     }
 }
 
-void
-remapgrid_get_lonlat(RemapGrid *grid, size_t cell_add, double *plon, double *plat)
+LonLatPoint
+remapgrid_get_lonlat(RemapGrid *grid, size_t cell_add)
 {
+  double lon, lat;
+
   if (grid->type == RemapGridType::Reg2D)
     {
-      size_t nx = grid->dims[0];
-      size_t iy = cell_add / nx;
-      size_t ix = cell_add - iy * nx;
-      *plat = grid->reg2d_center_lat[iy];
-      *plon = grid->reg2d_center_lon[ix];
-      if (*plon < 0) *plon += PI2;
+      const auto nx = grid->dims[0];
+      const auto iy = cell_add / nx;
+      const auto ix = cell_add - iy * nx;
+      lat = grid->reg2d_center_lat[iy];
+      lon = grid->reg2d_center_lon[ix];
+      if (lon < 0) lon += PI2;
     }
   else
     {
-      *plat = grid->cell_center_lat[cell_add];
-      *plon = grid->cell_center_lon[cell_add];
+      lat = grid->cell_center_lat[cell_add];
+      lon = grid->cell_center_lon[cell_add];
     }
+
+  return LonLatPoint(lon, lat);
 }
 
 void
-check_lon_range(size_t nlons, double *lons)
+check_lon_range(const char *txt, size_t nlons, Varray<double> &lons)
 {
-  assert(lons != nullptr);
+  assert(!lons.empty());
+
+  if (txt)
+    {
+      double minval = 1.e36;
+      double maxval = -1.e36;
+#ifdef _OPENMP
+#pragma omp parallel for default(none) shared(nlons, lons)  reduction(min : minval) reduction(max : maxval)
+#endif
+      for (size_t i = 0; i < nlons; ++i)
+        {
+          minval = std::min(minval, lons[i]);
+          maxval = std::max(maxval, lons[i]);
+        }
+      if (minval < -PI2 || maxval > 2 * PI2)
+        cdoWarning("%s grid cell center longitudes out of range (min=%.3g/max=%.3g)!", txt, RAD2DEG*minval, RAD2DEG*maxval);
+    }
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(nlons, lons)
 #endif
-  for (size_t n = 0; n < nlons; ++n)
+  for (size_t i = 0; i < nlons; ++i)
     {
       // remove missing values
-      if (lons[n] < -PI2) lons[n] = 0;
-      if (lons[n] > 2 * PI2) lons[n] = PI2;
+      if (lons[i] < -PI2) lons[i] = 0;
+      if (lons[i] > 2 * PI2) lons[i] = PI2;
 
-      if (lons[n] > PI2) lons[n] -= PI2;
-      if (lons[n] < 0.0) lons[n] += PI2;
+      if (lons[i] > PI2) lons[i] -= PI2;
+      if (lons[i] < 0.0) lons[i] += PI2;
     }
 }
 
 void
-check_lat_range(size_t nlats, double *lats)
+check_lat_range(const char *txt, size_t nlats, Varray<double> &lats)
 {
-  assert(lats != nullptr);
+  assert(!lats.empty());
+
+  if (txt)
+    {
+      double minval = 1.e36;
+      double maxval = -1.e36;
+#ifdef _OPENMP
+#pragma omp parallel for default(none) shared(nlats, lats)  reduction(min : minval) reduction(max : maxval)
+#endif
+      for (size_t i = 0; i < nlats; ++i)
+        {
+          minval = std::min(minval, lats[i]);
+          maxval = std::max(maxval, lats[i]);
+        }
+      if (minval < -PIH || maxval > PIH)
+        cdoWarning("%s grid cell center latitudes out of range (min=%.3g/max=%.3g)!", txt, RAD2DEG*minval, RAD2DEG*maxval);
+    }
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(nlats, lats)
 #endif
-  for (size_t n = 0; n < nlats; ++n)
+  for (size_t i = 0; i < nlats; ++i)
     {
-      if (lats[n] > PIH) lats[n] = PIH;
-      if (lats[n] < -PIH) lats[n] = -PIH;
+      if (lats[i] > PIH) lats[i] = PIH;
+      if (lats[i] < -PIH) lats[i] = -PIH;
     }
 }
 
@@ -238,7 +272,7 @@ check_lon_boundbox_range(size_t nlons, float *bound_box)
 #endif
   for (size_t n = 0; n < nlons; ++n)
     {
-      size_t n4 = n << 2;
+      const auto n4 = n << 2;
       if (fabsf(bound_box[n4 + 3] - bound_box[n4 + 2]) > PI_f)
         {
           bound_box[n4 + 2] = 0.0f;
@@ -248,7 +282,7 @@ check_lon_boundbox_range(size_t nlons, float *bound_box)
 }
 
 static void
-check_lat_boundbox_range(size_t nlats, float *restrict bound_box, double *restrict lats)
+check_lat_boundbox_range(size_t nlats, float *restrict bound_box, Varray<double> &lats)
 {
   assert(bound_box != nullptr);
 
@@ -257,7 +291,7 @@ check_lat_boundbox_range(size_t nlats, float *restrict bound_box, double *restri
 #endif
   for (size_t n = 0; n < nlats; ++n)
     {
-      size_t n4 = n << 2;
+      const auto n4 = n << 2;
       if ((float) lats[n] < bound_box[n4]) bound_box[n4] = -PIH_f;
       if ((float) lats[n] > bound_box[n4 + 1]) bound_box[n4 + 1] = PIH_f;
     }
@@ -266,49 +300,80 @@ check_lat_boundbox_range(size_t nlats, float *restrict bound_box, double *restri
 /*****************************************************************************/
 
 static void
-grid_check_lat_borders_rad(size_t n, double *ybounds)
+grid_check_lat_borders_rad(size_t n, Varray<double> &ybounds)
 {
+  constexpr double YMAX = PIH;
   constexpr double YLIM = 88 * DEG2RAD;
-  if (ybounds[0] > ybounds[n - 1])
+  const auto lrev = (ybounds[0] > ybounds[n - 1]);
+  if (lrev)
     {
-      if (ybounds[0] > YLIM) ybounds[0] = PIH;
-      if (ybounds[n - 1] < -YLIM) ybounds[n - 1] = -PIH;
+      if (ybounds[0] > ybounds[1])
+        {
+          if (ybounds[0] > YLIM) ybounds[0] = YMAX;
+          if (ybounds[n - 1] < -YLIM) ybounds[n - 1] = -YMAX;
+        }
+      else
+        {
+          if (ybounds[1] > YLIM) ybounds[1] = YMAX;
+          if (ybounds[n - 2] < -YLIM) ybounds[n - 2] = -YMAX;
+         }
     }
   else
     {
-      if (ybounds[0] < -YLIM) ybounds[0] = -PIH;
-      if (ybounds[n - 1] > YLIM) ybounds[n - 1] = PIH;
+      if (ybounds[0] < ybounds[1])
+        {
+          if (ybounds[0] < -YLIM) ybounds[0] = -YMAX;
+          if (ybounds[n - 1] > YLIM) ybounds[n - 1] = YMAX;
+        }
+      else
+        {
+          if (ybounds[1] < -YLIM) ybounds[1] = -YMAX;
+          if (ybounds[n - 2] > YLIM) ybounds[n - 2] = YMAX;
+        }
+    }
+}
+
+static void
+convert_bounds_reg2d(size_t n, const Varray<double> &boundsIn, Varray<double> &boundsOut)
+{
+  auto lrev = (boundsIn[0] > boundsIn[2 * n - 1]);
+  if (boundsIn[0] > boundsIn[1]) lrev = !lrev;
+  if (lrev)
+    {
+      boundsOut[0] = boundsIn[1];
+      for (size_t i = 0; i < n; ++i) boundsOut[i + 1] = boundsIn[2 * i];
+    }
+  else
+    {
+      boundsOut[0] = boundsIn[0];
+      for (size_t i = 0; i < n; ++i) boundsOut[i + 1] = boundsIn[2 * i + 1];
     }
 }
 
 static void
 remap_define_reg2d(int gridID, RemapGrid &grid)
 {
-  size_t nx = grid.dims[0];
-  size_t ny = grid.dims[1];
-  size_t nxp1 = nx + 1;
-  size_t nyp1 = ny + 1;
+  const auto nx = grid.dims[0];
+  const auto ny = grid.dims[1];
+  const auto nxp1 = nx + 1;
+  const auto nyp1 = ny + 1;
 
-  size_t nxm = nx;
+  auto nxm = nx;
   if (grid.is_cyclic) nxm++;
 
   if (grid.size != nx * ny) cdoAbort("Internal error, wrong dimensions!");
 
-  grid.reg2d_center_lon = (double *) Malloc(nxm * sizeof(double));
-  grid.reg2d_center_lat = (double *) Malloc(ny * sizeof(double));
+  grid.reg2d_center_lon.resize(nxm);
+  grid.reg2d_center_lat.resize(ny);
 
   grid.reg2d_center_lon[0] = 0;
   grid.reg2d_center_lat[0] = 0;
-  gridInqXvals(gridID, grid.reg2d_center_lon);
-  gridInqYvals(gridID, grid.reg2d_center_lat);
+  gridInqXvals(gridID, grid.reg2d_center_lon.data());
+  gridInqYvals(gridID, grid.reg2d_center_lat.data());
 
   // Convert lat/lon units if required
-  char yunits[CDI_MAX_NAME];
-  yunits[0] = 0;
-  cdiGridInqKeyStr(gridID, CDI_KEY_YUNITS, CDI_MAX_NAME, yunits);
-
-  grid_to_radian(yunits, nx, grid.reg2d_center_lon, "grid reg2d center lon");
-  grid_to_radian(yunits, ny, grid.reg2d_center_lat, "grid reg2d center lat");
+  cdo_grid_to_radian(gridID, CDI_XAXIS, nx, grid.reg2d_center_lon.data(), "grid reg2d center lon");
+  cdo_grid_to_radian(gridID, CDI_YAXIS, ny, grid.reg2d_center_lat.data(), "grid reg2d center lat");
 
   if (grid.reg2d_center_lon[nx - 1] < grid.reg2d_center_lon[0])
     for (size_t i = 1; i < nx; ++i)
@@ -316,12 +381,33 @@ remap_define_reg2d(int gridID, RemapGrid &grid)
 
   if (grid.is_cyclic) grid.reg2d_center_lon[nx] = grid.reg2d_center_lon[0] + PI2;
 
-  grid.reg2d_corner_lon = (double *) Malloc(nxp1 * sizeof(double));
-  grid.reg2d_corner_lat = (double *) Malloc(nyp1 * sizeof(double));
+  grid.reg2d_corner_lon.resize(nxp1);
+  grid.reg2d_corner_lat.resize(nyp1);
 
-  grid_gen_corners(nx, grid.reg2d_center_lon, grid.reg2d_corner_lon);
-  grid_gen_corners(ny, grid.reg2d_center_lat, grid.reg2d_corner_lat);
-  grid_check_lat_borders_rad(ny + 1, grid.reg2d_corner_lat);
+  if (gridInqXbounds(gridID, nullptr))
+    {
+      Varray<double> xbounds(2 * nx);
+      gridInqXbounds(gridID, xbounds.data());
+      convert_bounds_reg2d(nx, xbounds, grid.reg2d_corner_lon);
+      cdo_grid_to_radian(gridID, CDI_XAXIS, nx + 1, grid.reg2d_corner_lon.data(), "grid reg2d corner lon");
+    }
+  else
+    {
+      grid_gen_corners(nx, grid.reg2d_center_lon.data(), grid.reg2d_corner_lon.data());
+    }
+
+  if (gridInqYbounds(gridID, nullptr))
+    {
+      Varray<double> ybounds(2 * ny);
+      gridInqYbounds(gridID, ybounds.data());
+      convert_bounds_reg2d(ny, ybounds, grid.reg2d_corner_lat);
+      cdo_grid_to_radian(gridID, CDI_YAXIS, ny + 1, grid.reg2d_corner_lat.data(), "grid reg2d corner lat");
+    }
+  else
+    {
+      grid_gen_corners(ny, grid.reg2d_center_lat.data(), grid.reg2d_corner_lat.data());
+      grid_check_lat_borders_rad(ny + 1, grid.reg2d_corner_lat);
+    }
 }
 
 static void
@@ -330,7 +416,7 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
   bool lgrid_destroy = false;
   int gridID_gme = -1;
 
-  auto gridtype = gridInqType(grid.gridID);
+  const auto gridtype = gridInqType(grid.gridID);
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
       if (gridtype == GRID_GME)
@@ -371,7 +457,7 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
 
   remapGridAlloc(mapType, grid);
 
-  /* Initialize logical mask */
+  // Initialize logical mask
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(gridsize, grid)
@@ -388,61 +474,42 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
 
   if (!remap_write_remap && grid.type == RemapGridType::Reg2D) return;
 
-  if (!(gridInqXvals(gridID, nullptr) && gridInqYvals(gridID, nullptr))) cdoAbort("%s grid cell center coordinates missing!", txt);
+  if (!gridHasCoordinates(gridID)) cdoAbort("%s grid cell center coordinates missing!", txt);
 
-  gridInqXvals(gridID, grid.cell_center_lon);
-  gridInqYvals(gridID, grid.cell_center_lat);
-
-  char xunits[CDI_MAX_NAME];
-  xunits[0] = 0;
-  char yunits[CDI_MAX_NAME];
-  yunits[0] = 0;
-  cdiGridInqKeyStr(gridID, CDI_KEY_XUNITS, CDI_MAX_NAME, xunits);
-  cdiGridInqKeyStr(gridID, CDI_KEY_YUNITS, CDI_MAX_NAME, yunits);
+  gridInqXvals(gridID, grid.cell_center_lon.data());
+  gridInqYvals(gridID, grid.cell_center_lat.data());
 
   if (grid.lneed_cell_corners)
     {
-      if (gridInqXbounds(gridID, nullptr) && gridInqYbounds(gridID, nullptr))
-        {
-          gridInqXbounds(gridID, grid.cell_corner_lon);
-          gridInqYbounds(gridID, grid.cell_corner_lat);
-        }
-      else
-        {
-          cdoAbort("%s grid cell corner coordinates missing!", txt);
-        }
+      if (!gridHasBounds(gridID)) cdoAbort("%s grid cell corner coordinates missing!", txt);
+
+      gridInqXbounds(gridID, grid.cell_corner_lon.data());
+      gridInqYbounds(gridID, grid.cell_corner_lat.data());
     }
 
   if (gridInqType(grid.gridID) == GRID_GME) gridInqMaskGME(gridID_gme, &grid.vgpm[0]);
 
-  /* Convert lat/lon units if required */
-
-  grid_to_radian(xunits, grid.size, grid.cell_center_lon, "grid center lon");
-  grid_to_radian(yunits, grid.size, grid.cell_center_lat, "grid center lat");
-  /* Note: using units from cell center instead from bounds */
+  // Convert lat/lon units if required
+  cdo_grid_to_radian(gridID, CDI_XAXIS, grid.size, grid.cell_center_lon.data(), "grid center lon");
+  cdo_grid_to_radian(gridID, CDI_YAXIS, grid.size, grid.cell_center_lat.data(), "grid center lat");
   if (grid.num_cell_corners && grid.lneed_cell_corners)
     {
-      grid_to_radian(xunits, grid.num_cell_corners * grid.size, grid.cell_corner_lon, "grid corner lon");
-      grid_to_radian(yunits, grid.num_cell_corners * grid.size, grid.cell_corner_lat, "grid corner lat");
+      cdo_grid_to_radian(gridID, CDI_XAXIS, grid.num_cell_corners * grid.size, grid.cell_corner_lon.data(), "grid corner lon");
+      cdo_grid_to_radian(gridID, CDI_YAXIS, grid.num_cell_corners * grid.size, grid.cell_corner_lat.data(), "grid corner lat");
     }
 
   if (lgrid_destroy) gridDestroy(gridID);
 
-  /* Convert longitudes to 0,2pi interval */
+  // Convert longitudes to 0,2pi interval
+  check_lon_range(txt, grid.size, grid.cell_center_lon);
+  if (grid.num_cell_corners && grid.lneed_cell_corners) check_lon_range(nullptr, grid.num_cell_corners * grid.size, grid.cell_corner_lon);
 
-  check_lon_range(grid.size, grid.cell_center_lon);
-
-  if (grid.num_cell_corners && grid.lneed_cell_corners) check_lon_range(grid.num_cell_corners * grid.size, grid.cell_corner_lon);
-
-  /*  Make sure input latitude range is within the machine values for +/- pi/2
-   */
-
-  check_lat_range(grid.size, grid.cell_center_lat);
-
-  if (grid.num_cell_corners && grid.lneed_cell_corners) check_lat_range(grid.num_cell_corners * grid.size, grid.cell_corner_lat);
+  // Make sure input latitude range is within the machine values for +/- pi/2
+  check_lat_range(txt, grid.size, grid.cell_center_lat);
+  if (grid.num_cell_corners && grid.lneed_cell_corners) check_lat_range(nullptr, grid.num_cell_corners * grid.size, grid.cell_corner_lat);
 }
 
-/*  Compute bounding boxes for restricting future grid searches */
+// Compute bounding boxes for restricting future grid searches
 static void
 cell_bounding_boxes(RemapGrid &grid, float *cell_bound_box, int remap_grid_basis)
 {
@@ -457,7 +524,7 @@ cell_bounding_boxes(RemapGrid &grid, float *cell_bound_box, int remap_grid_basis
         {
           if (Options::cdoVerbose) cdoPrint("Grid: bounds missing -> full grid search!");
 
-          size_t gridsize = grid.size;
+          const auto gridsize = grid.size;
           for (size_t i = 0; i < gridsize; ++i)
             {
               cell_bound_box[i * 4] = -PIH_f;
@@ -472,8 +539,8 @@ cell_bounding_boxes(RemapGrid &grid, float *cell_bound_box, int remap_grid_basis
       if (Options::cdoVerbose) cdoPrint("Grid: boundboxFromCenter");
       if (grid.rank != 2) cdoAbort("Internal problem, grid rank = %d!", grid.rank);
 
-      size_t nx = grid.dims[0];
-      size_t ny = grid.dims[1];
+      const auto nx = grid.dims[0];
+      const auto ny = grid.dims[1];
       boundboxFromCenter(grid.is_cyclic, grid.size, nx, ny, grid.cell_center_lon, grid.cell_center_lat, cell_bound_box);
     }
 
@@ -493,8 +560,8 @@ remapGridAlloc(RemapMethod mapType, RemapGrid &grid)
 
   if (remap_write_remap || grid.type != RemapGridType::Reg2D)
     {
-      grid.cell_center_lon = (double *) Malloc(grid.size * sizeof(double));
-      grid.cell_center_lat = (double *) Malloc(grid.size * sizeof(double));
+      grid.cell_center_lon.resize(grid.size);
+      grid.cell_center_lat.resize(grid.size);
     }
 
   if (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV)
@@ -508,9 +575,9 @@ remapGridAlloc(RemapMethod mapType, RemapGrid &grid)
     {
       if (grid.num_cell_corners > 0)
         {
-          size_t nalloc = grid.num_cell_corners * grid.size;
-          grid.cell_corner_lon = (double *) Calloc(nalloc, sizeof(double));
-          grid.cell_corner_lat = (double *) Calloc(nalloc, sizeof(double));
+          const auto nalloc = grid.num_cell_corners * grid.size;
+          grid.cell_corner_lon.resize(nalloc, 0);
+          grid.cell_corner_lat.resize(nalloc, 0);
         }
     }
 }
@@ -526,16 +593,6 @@ remapGridInit(RemapGrid &grid)
   grid.lneed_cell_corners = false;
 
   grid.nvgp = 0;
-
-  grid.reg2d_center_lon = nullptr;
-  grid.reg2d_center_lat = nullptr;
-  grid.reg2d_corner_lon = nullptr;
-  grid.reg2d_corner_lat = nullptr;
-
-  grid.cell_center_lon = nullptr;
-  grid.cell_center_lat = nullptr;
-  grid.cell_corner_lon = nullptr;
-  grid.cell_corner_lat = nullptr;
 }
 
 void
@@ -544,15 +601,15 @@ remapGridFree(RemapGrid &grid)
   varrayFree(grid.vgpm);
   varrayFree(grid.mask);
 
-  if (grid.reg2d_center_lat) Free(grid.reg2d_center_lat);
-  if (grid.reg2d_center_lon) Free(grid.reg2d_center_lon);
-  if (grid.reg2d_corner_lat) Free(grid.reg2d_corner_lat);
-  if (grid.reg2d_corner_lon) Free(grid.reg2d_corner_lon);
+  varrayFree(grid.reg2d_center_lat);
+  varrayFree(grid.reg2d_center_lon);
+  varrayFree(grid.reg2d_corner_lat);
+  varrayFree(grid.reg2d_corner_lon);
 
-  if (grid.cell_center_lat) Free(grid.cell_center_lat);
-  if (grid.cell_center_lon) Free(grid.cell_center_lon);
-  if (grid.cell_corner_lat) Free(grid.cell_corner_lat);
-  if (grid.cell_corner_lon) Free(grid.cell_corner_lon);
+  varrayFree(grid.cell_center_lat);
+  varrayFree(grid.cell_center_lon);
+  varrayFree(grid.cell_corner_lat);
+  varrayFree(grid.cell_corner_lon);
 
   varrayFree(grid.cell_area);
   varrayFree(grid.cell_frac);
@@ -575,9 +632,6 @@ remapSearchInit(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid, R
   search.srcBins.nbins = remap_num_srch_bins;
   search.tgtBins.nbins = remap_num_srch_bins;
 
-  search.gps = nullptr;
-  search.gcs = nullptr;
-
   bool usePointsearch = mapType == RemapMethod::DISTWGT;
   if (src_grid.type != RemapGridType::Reg2D && pointSearchMethod != PointSearchMethod::latbins)
     {
@@ -594,10 +648,9 @@ remapSearchInit(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid, R
     {
       bool xIsCyclic = src_grid.is_cyclic;
       if (src_grid.type == RemapGridType::Reg2D)
-        search.gps = gridPointSearchCreateReg2d(xIsCyclic, src_grid.dims, src_grid.reg2d_center_lon, src_grid.reg2d_center_lat);
+        gridPointSearchCreateReg2d(search.gps, xIsCyclic, src_grid.dims, src_grid.reg2d_center_lon, src_grid.reg2d_center_lat);
       else
-        search.gps
-            = gridPointSearchCreate(xIsCyclic, src_grid.dims, src_grid.size, src_grid.cell_center_lon, src_grid.cell_center_lat);
+        gridPointSearchCreate(search.gps, xIsCyclic, src_grid.dims, src_grid.size, src_grid.cell_center_lon, src_grid.cell_center_lat);
 
       if (src_grid.lextrapolate) gridPointSearchExtrapolate(search.gps);
 
@@ -606,10 +659,9 @@ remapSearchInit(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid, R
   else if (useCellsearch)
     {
       if (src_grid.type == RemapGridType::Reg2D)
-        search.gcs = gridCellSearchCreateReg2d(src_grid.dims, src_grid.reg2d_corner_lon, src_grid.reg2d_corner_lat);
+        gridCellSearchCreateReg2d(search.gcs, src_grid.dims, src_grid.reg2d_corner_lon, src_grid.reg2d_corner_lat);
       else
-        search.gcs
-            = gridCellSearchCreate(src_grid.size, src_grid.num_cell_corners, src_grid.cell_corner_lon, src_grid.cell_corner_lat);
+        gridCellSearchCreate(search.gcs, src_grid.size, src_grid.num_cell_corners, src_grid.cell_corner_lon, src_grid.cell_corner_lat);
 
       if (Options::cdoVerbose) cdoPrint("Cell search created: %.2f seconds", cdo_get_wtime() - start);
     }
@@ -645,18 +697,15 @@ remapSearchFree(RemapSearch &search)
   varrayFree(search.tgtBins.bin_lats);
   varrayFree(search.tgtBins.cell_bound_box);
 
-  if (search.gps) gridPointSearchDelete(search.gps);
-  search.gps = nullptr;
-
-  if (search.gcs) gridCellSearchDelete(search.gcs);
-  search.gcs = nullptr;
+  gridPointSearchDelete(search.gps);
+  gridCellSearchDelete(search.gcs);
 }
 
 void
 remapInitGrids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid &src_grid, int gridID2, RemapGrid &tgt_grid)
 {
-  int reg2d_src_gridID = gridID1;
-  int reg2d_tgt_gridID = gridID2;
+  auto reg2d_src_gridID = gridID1;
+  auto reg2d_tgt_gridID = gridID2;
 
   remapGridInit(src_grid);
   remapGridInit(tgt_grid);
@@ -704,44 +753,29 @@ remapInitGrids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid &s
   src_grid.gridID = gridID1;
   tgt_grid.gridID = gridID2;
 
-  if (gridInqType(gridID1) == GRID_UNSTRUCTURED)
+  if (gridInqType(gridID1) == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID1))
     {
-      if (gridInqYvals(gridID1, nullptr) == 0 || gridInqXvals(gridID1, nullptr) == 0)
-        {
-          if (gridInqNumber(gridID1) > 0)
-            {
-              src_grid.gridID = gridID1 = referenceToGrid(gridID1);
-              if (gridID1 == -1) cdoAbort("Reference to source grid not found!");
-            }
-        }
+      const auto reference = dereferenceGrid(gridID1);
+      if (reference.isValid) src_grid.gridID = gridID1 = reference.gridID;
+      if (reference.notFound) cdoAbort("Reference to source grid not found!");
     }
 
-  if (gridInqType(gridID2) == GRID_UNSTRUCTURED)
+  if (gridInqType(gridID2) == GRID_UNSTRUCTURED && !gridHasCoordinates(gridID2))
     {
-      if (gridInqYvals(gridID2, nullptr) == 0 || gridInqXvals(gridID2, nullptr) == 0)
-        {
-          if (gridInqNumber(gridID2) > 0)
-            {
-              tgt_grid.gridID = gridID2 = referenceToGrid(gridID2);
-              if (gridID2 == -1) cdoAbort("Reference to target grid not found!");
-            }
-        }
-    }
+      const auto reference = dereferenceGrid(gridID2);
+      if (reference.isValid) tgt_grid.gridID = gridID2 = reference.gridID;
+      if (reference.notFound) cdoAbort("Reference to target grid not found!");
+   }
 
-  int sgridID = src_grid.gridID;
-  if (gridInqSize(sgridID) > 1
-      && ((gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_LCC)
-          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_STERE)
-          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_RLL)
-          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_LAEA)
-          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_SINU)))
+  const auto sgridID = src_grid.gridID;
+  if (gridInqSize(sgridID) > 1 && gridProjIsSupported(sgridID))
     {
       bool lbounds = true;
       src_grid.gridID = gridID1 = gridToCurvilinear(src_grid.gridID, lbounds);
       src_grid.tmpgridID = src_grid.gridID;
     }
 
-  // if ( src_grid.type != RemapGridType::Reg2D )
+  // if (src_grid.type != RemapGridType::Reg2D)
   remapDefineGrid(mapType, gridID1, src_grid, "Source");
   remapDefineGrid(mapType, gridID2, tgt_grid, "Target");
 
@@ -764,12 +798,11 @@ remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &r
     cdoPrint("First order mapping from grid1 to grid2:");
   cdoPrint("----------------------------------------------");
 
-  double mean, minval, maxval;
-  varrayMinMaxMeanMV(src_grid.size, array1, missval, minval, maxval, mean);
-  cdoPrint("  Grid1 min,mean,max: %g %g %g", minval, mean, maxval);
+  auto mmm = varrayMinMaxMeanMV(src_grid.size, array1, missval);
+  cdoPrint("  Grid1 min,mean,max: %g %g %g", mmm.min, mmm.mean, mmm.max);
 
-  varrayMinMaxMeanMV(tgt_grid.size, array2, missval, minval, maxval, mean);
-  cdoPrint("  Grid2 min,mean,max: %g %g %g", minval, mean, maxval);
+  mmm = varrayMinMaxMeanMV(tgt_grid.size, array2, missval);
+  cdoPrint("  Grid2 min,mean,max: %g %g %g", mmm.min, mmm.mean, mmm.max);
 
   // Conservation Test
 
@@ -848,13 +881,13 @@ remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &r
 /*****************************************************************************/
 
 void
-remapGradients(RemapGrid &grid, const std::vector<bool> &mask, const double *restrict array, RemapGradients &gradients)
+remapGradients(RemapGrid &grid, const Varray<uint8_t> &mask, const double *restrict array, RemapGradients &gradients)
 {
   if (grid.rank != 2) cdoAbort("Internal problem (remapGradients), grid rank = %d!", grid.rank);
 
-  size_t grid_size = grid.size;
-  size_t nx = grid.dims[0];
-  size_t ny = grid.dims[1];
+  auto grid_size = grid.size;
+  auto nx = grid.dims[0];
+  auto ny = grid.dims[1];
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(grid_size, gradients, grid, nx, ny, array, mask)
@@ -868,31 +901,31 @@ remapGradients(RemapGrid &grid, const std::vector<bool> &mask, const double *res
       if (mask[n])
         {
           // clang-format off
-          double delew = 0.5;
-          double delns = 0.5;
+          auto delew = 0.5;
+          auto delns = 0.5;
 
-          const size_t j = n / nx + 1;
-          const size_t i = n - (j - 1) * nx + 1;
+          const auto j = n / nx + 1;
+          const auto i = n - (j - 1) * nx + 1;
 
-          size_t ip1 = i + 1;
-          size_t im1 = i - 1;
-          size_t jp1 = j + 1;
-          size_t jm1 = j - 1;
+          auto ip1 = i + 1;
+          auto im1 = i - 1;
+          auto jp1 = j + 1;
+          auto jm1 = j - 1;
 
           if (ip1 > nx) ip1 = ip1 - nx;
           if (im1 < 1)  im1 = nx;
           if (jp1 > ny) { jp1 = j; delns = 1.0; }
           if (jm1 < 1)  { jm1 = j; delns = 1.0; }
 
-          size_t in = (jp1 - 1) * nx + i - 1;
-          size_t is = (jm1 - 1) * nx + i - 1;
-          size_t ie = (j - 1) * nx + ip1 - 1;
-          size_t iw = (j - 1) * nx + im1 - 1;
+          auto in = (jp1 - 1) * nx + i - 1;
+          auto is = (jm1 - 1) * nx + i - 1;
+          auto ie = (j - 1) * nx + ip1 - 1;
+          auto iw = (j - 1) * nx + im1 - 1;
 
-          size_t ine = (jp1 - 1) * nx + ip1 - 1;
-          size_t inw = (jp1 - 1) * nx + im1 - 1;
-          size_t ise = (jm1 - 1) * nx + ip1 - 1;
-          size_t isw = (jm1 - 1) * nx + im1 - 1;
+          auto ine = (jp1 - 1) * nx + ip1 - 1;
+          auto inw = (jp1 - 1) * nx + im1 - 1;
+          auto ise = (jm1 - 1) * nx + ip1 - 1;
+          auto isw = (jm1 - 1) * nx + im1 - 1;
 
           // Compute i-gradient
           if (!mask[ie]) { ie = n; delew = 1.0; }
@@ -957,7 +990,7 @@ remapGradients(RemapGrid &grid, const std::vector<bool> &mask, const double *res
                 }
             }
 
-          const double grad_lat_zero = delew * (array[ine] - array[inw]);
+          const auto grad_lat_zero = delew * (array[ine] - array[inw]);
 
           if (!mask[ise])
             {
@@ -1005,8 +1038,7 @@ remapGradients(RemapGrid &grid, const std::vector<bool> &mask, const double *res
                 }
             }
 
-          const double grad_lon_zero = delew * (array[ise] - array[isw]);
-
+          const auto grad_lon_zero = delew * (array[ise] - array[isw]);
           gradients.grad_latlon[n] = delns * (grad_lat_zero - grad_lon_zero);
         }
     }
@@ -1015,13 +1047,12 @@ remapGradients(RemapGrid &grid, const std::vector<bool> &mask, const double *res
 void
 remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gradients)
 {
-  size_t grid_size = grid.size;
-  std::vector<bool> mask(grid_size);
+  auto grid_size = grid.size;
+  Varray<uint8_t> mask(grid_size);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) schedule(static) shared(grid_size, grid, mask)
 #endif
-  for (size_t i = 0; i < grid_size; ++i)
-    mask[i] = grid.mask[i] > 0;
+  for (size_t i = 0; i < grid_size; ++i) mask[i] = grid.mask[i] > 0;
 
   remapGradients(grid, mask, array, gradients);
 }
