@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,8 @@
 #include "config.h"
 #endif
 
-#include "cdo_int.h"
+#include "cdo_output.h"
+#include "compare.h"
 #include "template_parser.h"
 #include "magics_template_parser.h"
 #include "results_template_parser.h"
@@ -38,11 +39,10 @@ extern int GetMagicsParameterInfo(const char *user_name, char *param_value);
 #endif
 
 /* Recursive function that sets the results parameters from the XML structure */
-
+#ifdef HAVE_LIBXML2
 int
 results_template_parser(void *node, const char *varname)
 {
-#ifdef HAVE_LIBXML2
   xmlNode *a_node = (xmlNode *) node;
   xmlNode *cur_node = nullptr;
   xmlAttrPtr attr = nullptr;
@@ -50,7 +50,7 @@ results_template_parser(void *node, const char *varname)
 
   if (a_node == nullptr) return 1;
 
-  if (cstrIsEqual((const char *) a_node->name, "results"))
+  if (cdo_cmpstr((const char *) a_node->name, "results"))
     {
       const char *value = (const char *) xmlGetProp(a_node, (const xmlChar *) "version");
 
@@ -126,11 +126,17 @@ results_template_parser(void *node, const char *varname)
             }
         }
     }
-#else
-
-  cdoAbort("XML2 support not compiled in!");
-
-#endif
 
   return 0;
 }
+#else
+
+int
+results_template_parser(void *node, const char *varname)
+{
+  (void) node;
+  (void) varname;
+  cdoAbort("XML2 support not compiled in!");
+  return 0;
+}
+#endif  // HAVE_LIBXML2
