@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,9 +14,10 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include <stdio.h>
+
 #include <string.h>
 #include <assert.h>
+#include "compare.h"
 #include "stdnametable.h"
 
 struct stdnametable_t
@@ -28,27 +29,29 @@ struct stdnametable_t
   const char *units;    // Units
 };
 
-const stdnametable_t stdnametable[] = {
+static const stdnametable_t stdnametable[] = {
   // clang-format off
-  // varid                       code    name                standard name                 units
-  { air_pressure,                   1,  "apres",            "air_pressure",               "Pa" },
-  { pressure_thickness,             2,  "dpress",           "pressure_thickness",         "Pa" },
-  { surface_geopotential,         129,  "geosp",            "surface_geopotential",       "m2 s-2" },
-  { geopotential,                 129,  "z",                "geopotential",               "m2 s-2" },
-  { air_temperature,              130,  "ta",               "air_temperature",            "K" },
-  { specific_humidity,            133,  "hus",              "specific_humidity",          "1" },
-  { surface_air_pressure,         134,  "aps",              "surface_air_pressure",       "Pa" },
-  { air_pressure_at_sea_level,    151,  "psl",              "air_pressure_at_sea_level",  "Pa" },
-  { geopotential_height,          156,  "zh",               "geopotential_height",        "m" },
+  // varid                                  code    name           standard name                             units
+  { air_pressure,                              1,  "apres",       "air_pressure",                           "Pa" },
+  { pressure_thickness,                        2,  "dpress",      "pressure_thickness",                     "Pa" },
+  { surface_geopotential,                    129,  "geosp",       "surface_geopotential",                   "m2 s-2" },
+  { geopotential,                            129,  "z",           "geopotential",                           "m2 s-2" },
+  { air_temperature,                         130,  "ta",          "air_temperature",                        "K" },
+  { specific_humidity,                       133,  "hus",         "specific_humidity",                      "1" },
+  { surface_air_pressure,                    134,  "aps",         "surface_air_pressure",                   "Pa" },
+  { air_pressure_at_sea_level,               151,  "psl",         "air_pressure_at_sea_level",              "Pa" },
+  { geopotential_height,                     156,  "zh",          "geopotential_height",                    "m" },
+  { geometric_height_at_full_level_center,    21,  "zg",          "geometric_height_at_full_level_center",  "m" },
+  { geometric_height_at_half_level_center,    22,  "zghalf",      "geometric_height_at_half_level_center",  "m" },
   // clang-format on
 };
 
 static int
 stdnametable_idx(int varid)
 {
-  int idx;
   int num_entries = (int) (sizeof(stdnametable) / sizeof(stdnametable_t));
 
+  int idx;
   for (idx = 0; idx < num_entries; ++idx)
     if (stdnametable[idx].varid == varid) break;
 
@@ -87,13 +90,13 @@ echamcode_from_stdname(const char *stdname)
   int code = -1;
 
   // clang-format off
-  if      ( strcmp(stdname, var_stdname(surface_geopotential))      == 0 ) code = 129;
-  else if ( strcmp(stdname, var_stdname(geopotential))              == 0 ) code = 129;
-  else if ( strcmp(stdname, var_stdname(air_temperature))           == 0 ) code = 130;
-  else if ( strcmp(stdname, var_stdname(specific_humidity))         == 0 ) code = 133;
-  else if ( strcmp(stdname, var_stdname(surface_air_pressure))      == 0 ) code = 134;
-  else if ( strcmp(stdname, var_stdname(air_pressure_at_sea_level)) == 0 ) code = 151;
-  else if ( strcmp(stdname, var_stdname(geopotential_height))       == 0 ) code = 156;
+  if      (cdo_cmpstr(stdname, var_stdname(surface_geopotential))     ) code = 129;
+  else if (cdo_cmpstr(stdname, var_stdname(geopotential))             ) code = 129;
+  else if (cdo_cmpstr(stdname, var_stdname(air_temperature))          ) code = 130;
+  else if (cdo_cmpstr(stdname, var_stdname(specific_humidity))        ) code = 133;
+  else if (cdo_cmpstr(stdname, var_stdname(surface_air_pressure))     ) code = 134;
+  else if (cdo_cmpstr(stdname, var_stdname(air_pressure_at_sea_level))) code = 151;
+  else if (cdo_cmpstr(stdname, var_stdname(geopotential_height))      ) code = 156;
   // clang-format on
 
   return code;
