@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,9 @@ enum stdnameid
   specific_humidity,
   surface_air_pressure,
   air_pressure_at_sea_level,
-  geopotential_height
+  geopotential_height,
+  geometric_height_at_full_level_center,
+  geometric_height_at_half_level_center
 };
 
 int var_echamcode(int varid);
@@ -39,18 +41,18 @@ int echamcode_from_stdname(const char *stdname);
 
 struct gribcode_t
 {
-  int geopot;
-  int temp;
-  int hum;
-  int ps;
-  int lsp;
-  int gheight;
-  int wind;
-  int uwind;
-  int vwind;
+  int geopot = 0;
+  int temp = 0;
+  int hum = 0;
+  int ps = 0;
+  int lsp = 0;
+  int gheight = 0;
+  int wind = 0;
+  int uwind = 0;
+  int vwind = 0;
 };
 
-enum struct ModelMode
+enum class ModelMode
 {
   UNDEF,
   ECHAM,
