@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#if defined(HAVE_SYS_TIMES_H)
+#ifdef HAVE_SYS_TIMES_H
 #include <sys/times.h>
 #endif
 
@@ -31,9 +31,8 @@ int timer_read, timer_write;
 void
 cdoProcessTime(double *utime, double *stime)
 {
-#if defined(HAVE_SYS_TIMES_H)
+#ifdef HAVE_SYS_TIMES_H
   struct tms tbuf;
-  double clock_ticks = 0;
 
   if ((int) times(&tbuf) == -1)
     {
@@ -42,7 +41,8 @@ cdoProcessTime(double *utime, double *stime)
     }
   else
     {
-#if defined(_SC_CLK_TCK)
+      double clock_ticks = 0;
+#ifdef _SC_CLK_TCK
       clock_ticks = (double) sysconf(_SC_CLK_TCK);
 #endif
       if (clock_ticks > 0)
@@ -72,7 +72,7 @@ void util_init_real_time(void);
 void util_read_real_time(void *it);
 void util_diff_real_time(void *it1, void *it2, double *t);
 
-FILE *rt_unit = NULL;
+FILE *rt_unit = nullptr;
 
 enum
 {
@@ -295,30 +295,21 @@ timer_stop(int it)
 
 #include "counter.h"
 
-static void
-counter_init(counter_t *counter)
+namespace cdo
 {
-  counter->cputime = 0;
-}
-
 void
-counter_start(counter_t *counter)
+Counter::start()
 {
-  counter_init(counter);
+  m_cputime = 0;
 
   if (timer_need_init) timer_init();
 
-  set_time_mark(counter->mark);
+  set_time_mark(mark);
 }
 
 void
-counter_stop(counter_t *counter)
+Counter::stop()
 {
-  counter->cputime = get_time_val(counter->mark);
-}
-
-double
-counter_cputime(counter_t counter)
-{
-  return counter.cputime;
+  m_cputime = get_time_val(mark);
 }
+}  // namespace cdo
