@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -14,11 +14,11 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
+
 #ifndef TIMER_H
 #define TIMER_H
 
-extern int timer_read, timer_write;  // refactor: both pstream.cc and CDIread.cc
-                                     // CDIwrite.cc defined in cdo.cc
+extern int timer_read, timer_write;
 
 void cdoProcessTime(double *utime, double *stime);
 int timer_new(const char *text);
