@@ -16,20 +16,20 @@
 #include "readline.h"
 #include "mpmo_color.h"
 
-#include "cdo_defaultValues.h"
+#include "cdo_default_values.h"
 
 bool
 fileExists(const char *restrict filename)
 {
-  bool status = false;
   struct stat buf;
+  const auto status = stat(filename, &buf);
+  return (status == 0) && (S_ISREG(buf.st_mode) && buf.st_size > 0);
+}
 
-  if (stat(filename, &buf) == 0)
-    {
-      if (S_ISREG(buf.st_mode) && buf.st_size > 0) status = true;
-    }
-
-  return status;
+bool
+fileExists(const std::string &filename)
+{
+  return fileExists(filename.c_str());
 }
 
 bool
@@ -41,7 +41,7 @@ userFileOverwrite(const char *restrict filename)
     {
       fprintf(stderr, "File %s already exists, overwrite? (yes/no): ", filename);
       char line[1024];
-      readline(stdin, line, 1024);
+      cdo::readline(stdin, line, 1024);
       char *pline = line;
       while (isspace((int) *pline)) pline++;
       const int len = (int) strlen(pline);
@@ -62,7 +62,7 @@ userFileOverwrite(const char *restrict filename)
 }
 
 off_t
-fileSize(const char *restrict filename)
+fileSize(const char *filename)
 {
   off_t filesize = 0;
 
@@ -143,11 +143,11 @@ genFileSuffix(int filetype, const char *refname, size_t maxlen, char *filesuffix
 void
 cdoGenFileSuffix(char *filesuffix, size_t maxlen, int filetype, int vlistID, const char *refname)
 {
-  if (strncmp(Cdo::File_Suffix, "NULL", 4) != 0)
+  if (strncmp(cdo::File_Suffix, "NULL", 4) != 0)
     {
-      if (Cdo::File_Suffix[0] != 0)
+      if (cdo::File_Suffix[0] != 0)
         {
-          strncat(filesuffix, Cdo::File_Suffix, maxlen - 1);
+          strncat(filesuffix, cdo::File_Suffix, maxlen - 1);
         }
       else
         {
