@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,9 @@
 #define UTIL_FILES_H
 
 #include <sys/types.h>
+#include <string>
 
+bool fileExists(const std::string &filename);
 bool fileExists(const char *filename);
 bool userFileOverwrite(const char *filename);
 off_t fileSize(const char *filename);
