@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -17,17 +17,26 @@
 
 #include <cassert>
 #include <string>
+#include <cstring>
 #include <cctype>
 #include <algorithm>
 #include <cdi.h>
 
 std::string
-stringToLower(std::string s)
+stringToUpper(std::string p_str)
 {
-  std::transform(s.begin(), s.end(), s.begin(), [](unsigned char c) { return std::tolower(c); });
-  return s;
+  std::transform(p_str.begin(), p_str.end(), p_str.begin(), [](unsigned char c) { return std::toupper(c); });
+  return p_str;
 }
 
+std::string
+stringToLower(std::string p_str)
+{
+  std::transform(p_str.begin(), p_str.end(), p_str.begin(), [](unsigned char c) { return std::tolower(c); });
+  return p_str;
+}
+
+
 void
 cstrToLowerCase(char *str)
 {
