@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,14 @@
 
 #define ADD_PLURAL(n) ((n) != 1 ? "s" : "")
 
-std::string string2lower(std::string str);
-void strtolower(char *str);
-void strtoupper(char *str);
-char *double_to_attstr(int digits, char *str, size_t len, double value);
+std::string stringToUpper(std::string p_str);
+std::string stringToLower(std::string p_str);
 
-const char *tunit2str(int tunits);
-const char *calendar2str(int calendar);
+void cstrToLowerCase(char *str);
+void cstrToUpperCase(char *str);
+char *doubleToAttstr(int digits, char *str, size_t len, double value);
+
+const char *tunitToCstr(int tunits);
+const char *calendarToCstr(int calendar);
 
 #endif
