@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-/*TEMP*/
 
 #include <cstdlib>
 #include <cstddef>
@@ -35,6 +34,8 @@
 #include <fnmatch.h>
 #endif
 
+#include "util_wildcards.h"
+
 #ifdef HAVE_GLOB_H
 static int
 get_glob_flags(void)
@@ -103,9 +104,30 @@ expandWildCards(std::vector<std::string> argv)
   int flags = WRDE_UNDEF;
   wordexp_t glob_results;
 
+  bool applyActive = false;
+  int bracketsOpen = 0;
   for (size_t idx = 1; idx < argv.size(); idx++)
     {
       // if argv[idx] contains wildcard (* or [?]+), multiple ** are ignored
+      if (argv[idx].compare(0, 6, "-apply") == 0)
+        {
+          applyActive = true;
+          continue;
+        }
+      if (argv[idx].size() == 1 && argv[idx][0] == '[')
+        {
+          bracketsOpen++;
+          continue;
+        }
+      if (argv[idx].size() == 1 && argv[idx][0] == ']')
+        {
+          bracketsOpen--;
+          if (bracketsOpen == 0)
+            {
+              applyActive = false;
+            }
+          continue;
+        }
       if (argv[idx][0] != '-' && argv[idx].find_first_of("*?[ ") != std::string::npos)
         {
           const int status = wordexp(argv[idx].c_str(), &glob_results, flags);
@@ -130,9 +152,14 @@ expandWildCards(std::vector<std::string> argv)
           // range based insert (glob_results.we_wordv is inserted before wildcard
           if (std::string(glob_results.we_wordv[0]).find_first_of("*?[ ") == std::string::npos)
             {
-              argv.insert(argv.begin() + idx + 1, "]");
-              argv.insert(argv.begin() + idx + 1, "[");
-              argv.insert(argv.begin() + idx + 2, glob_results.we_wordv, glob_results.we_wordv + glob_results.we_wordc);
+                int insertAt = idx + 1;
+              if (applyActive == false)
+                {
+                  argv.insert(argv.begin() + insertAt, "]");
+                  argv.insert(argv.begin() + insertAt, "[");
+                  insertAt+=1;
+                }
+              argv.insert(argv.begin() + insertAt, glob_results.we_wordv, glob_results.we_wordv + glob_results.we_wordc);
               argv.erase(argv.begin() + idx);
             }
           // delete wildcard
@@ -151,6 +178,12 @@ expandWildCards(std::vector<std::string> argv)
 #endif
 
 #ifdef HAVE_FNMATCH_H
+int
+wildcardmatch(const std::string &pattern, const char *string)
+{
+  return wildcardmatch(pattern.c_str(), string);
+}
+
 int
 wildcardmatch(const char *pattern, const char *string)
 {
@@ -162,6 +195,12 @@ wildcardmatch(const char *pattern, const char *string)
 // * --> Matches with 0 or more instances of any character or set of characters.
 // ? --> Matches with any one character.
 // source code from http://www.geeksforgeeks.org/wildcard-character-matching/
+int
+wildcardmatch(const std::string &w, const char *s)
+{
+  return wildcardmatch(w.c_str(), s);
+}
+
 int
 wildcardmatch(const char *w, const char *s)
 {
