@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,20 +23,24 @@
 #include <cstdio>
 #include <cmath>
 
-class Vector3d
+class // Vector3d
+#ifdef __GNUG__
+[[gnu::warn_unused]]
+#endif
+Vector3d
 {
 private:
   double X, Y, Z;
 
 public:
   // Sets all members to zero
-  Vector3d() : X(0), Y(0), Z(0) {}
+  Vector3d() noexcept : X(0), Y(0), Z(0) {}
 
-  Vector3d(const double &x, const double &y, const double &z) : X(x), Y(y), Z(z) {}
+  Vector3d(const double &x, const double &y, const double &z) noexcept : X(x), Y(y), Z(z) {}
 
-  Vector3d(const double coords[3]) : X(coords[0]), Y(coords[1]), Z(coords[2]) {}
+  Vector3d(const double coords[3]) noexcept : X(coords[0]), Y(coords[1]), Z(coords[2]) {}
 
-  double operator[](size_t index) const
+  double operator[](size_t index) const noexcept
   {
     switch (index)
       {
@@ -49,39 +53,39 @@ public:
   }
 
   Vector3d
-  operator+(const Vector3d &other) const
+  operator+(const Vector3d &other) const noexcept
   {
     return Vector3d(X + other.X, Y + other.Y, Z + other.Z);
   }
 
   Vector3d
-  operator-(const Vector3d &other) const
+  operator-(const Vector3d &other) const noexcept
   {
     return Vector3d(X - other.X, Y - other.Y, Z - other.Z);
   }
 
   Vector3d
-  operator-(void) const
+  operator-(void) const noexcept
   {
     return Vector3d(-X, -Y, -Z);
   }
 
   // Calculate the cross/outer/vector product
   Vector3d
-  operator%(const Vector3d &other) const
+  operator%(const Vector3d &other) const noexcept
   {
     return Vector3d(Y * other.Z - Z * other.Y, Z * other.X - X * other.Z, X * other.Y - Y * other.X);
   }
 
   // Division by scalars
   Vector3d
-  operator/(const double scalar) const
+  operator/(const double scalar) const noexcept
   {
     return Vector3d(X / scalar, Y / scalar, Z / scalar);
   }
 
   Vector3d
-  operator/=(const double scalar)
+  operator/=(const double scalar) noexcept
   {
     return *this = *this / scalar;
   }
@@ -90,31 +94,31 @@ public:
   double operator*(const Vector3d &other) const { return (X * other.X) + (Y * other.Y) + (Z * other.Z); }
 
   double
-  magnitude() const
+  magnitude() const noexcept
   {
     return std::sqrt((X * X) + (Y * Y) + (Z * Z));
   }
 
   Vector3d
-  normalised() const
+  normalised() const noexcept
   {
     return Vector3d(*this) / magnitude();
   }
   /*
   Vector3d d_normalize()
   {
-    const double dnorm = sqrt(*this * *this);
+    const double dnorm = std::sqrt(*this * *this);
     return Vector3d(*this) / dnorm;
   }
   */
   double
-  longitude() const
+  longitude() const noexcept
   {
     return std::atan2(Y, X);
   }
 
   double
-  latitude() const
+  latitude() const noexcept
   {
     return M_PI_2 - std::acos(Z);
   }
