@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -15,8 +15,10 @@
   GNU General Public License for more details.
 */
 #include <cdi.h>
+
 #include "cdi_uuid.h"
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 
 void cdoPrintAttributes(FILE *fp, int cdiID, int varID, int nblanks);
 
@@ -41,9 +43,9 @@ static void
 zaxisPrintKernel(int zaxisID, FILE *fp)
 {
   char attstr[CDI_MAX_NAME];
-  const int type = zaxisInqType(zaxisID);
-  const int nlevels = zaxisInqSize(zaxisID);
-  const int prec = zaxisInqDatatype(zaxisID);
+  const auto type = zaxisInqType(zaxisID);
+  const auto nlevels = zaxisInqSize(zaxisID);
+  const auto prec = zaxisInqDatatype(zaxisID);
   const size_t nvals = (size_t) zaxisInqLevels(zaxisID, nullptr);
 
   const int dig = (prec == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
@@ -53,17 +55,16 @@ zaxisPrintKernel(int zaxisID, FILE *fp)
 
   if (nlevels == 1 && zaxisInqScalar(zaxisID)) fprintf(fp, "scalar    = true\n");
 
-  attstr[0] = 0;
-  cdiZaxisInqKeyStr(zaxisID, CDI_KEY_NAME, CDI_MAX_NAME, attstr);
+  int length = CDI_MAX_NAME;
+  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_NAME, attstr, &length);
   if (attstr[0]) fprintf(fp, "name      = %s\n", attstr);
-  attstr[0] = 0;
-  cdiZaxisInqKeyStr(zaxisID, CDI_KEY_LONGNAME, CDI_MAX_NAME, attstr);
+  length = CDI_MAX_NAME;
+  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_LONGNAME, attstr, &length);
   if (attstr[0]) fprintf(fp, "longname  = \"%s\"\n", attstr);
-  attstr[0] = 0;
-  cdiZaxisInqKeyStr(zaxisID, CDI_KEY_UNITS, CDI_MAX_NAME, attstr);
+  length = CDI_MAX_NAME;
+  cdiInqKeyString(zaxisID, CDI_GLOBAL, CDI_KEY_UNITS, attstr, &length);
   if (attstr[0]) fprintf(fp, "units     = \"%s\"\n", attstr);
 
-  char **cvals = nullptr;
   std::vector<double> vals;
   if (nvals) vals.resize(nvals);
 
@@ -76,13 +77,15 @@ zaxisPrintKernel(int zaxisID, FILE *fp)
   else if (type == ZAXIS_CHAR)
     {
       const int clen = zaxisInqCLen(zaxisID);
+      char **cvals = nullptr;
       zaxisInqCVals(zaxisID, &cvals);
       fprintf(fp, "levels    = \n");
       for (int i = 0; i < nlevels; i++)
         {
           fprintf(fp, "     [%2d] = %.*s\n", i, clen, cvals[i]);
-          Free(cvals[i]);
+          free(cvals[i]);
         }
+      if (cvals) free(cvals);
     }
 
   if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
@@ -100,8 +103,6 @@ zaxisPrintKernel(int zaxisID, FILE *fp)
       }
     }
 
-  if (cvals) Free(cvals);
-
   if (type == ZAXIS_HYBRID || type == ZAXIS_HYBRID_HALF)
     {
       const int vctsize = zaxisInqVctSize(zaxisID);
@@ -117,8 +118,9 @@ zaxisPrintKernel(int zaxisID, FILE *fp)
 
   if (type == ZAXIS_REFERENCE)
     {
-      unsigned char uuid[CDI_UUID_SIZE];
-      zaxisInqUUID(zaxisID, uuid);
+      unsigned char uuid[CDI_UUID_SIZE] = { 0 };
+      length = CDI_UUID_SIZE;
+      cdiInqKeyBytes(zaxisID, CDI_GLOBAL, CDI_KEY_UUID, uuid, &length);
       if (!cdiUUIDIsNull(uuid))
         {
           char uuidStr[37];
