@@ -2,15 +2,33 @@
 // BANDIT NEEDS TO BE INCLUDED FIRST!!!
 
 #include "../../src/modules.h"
-#include "../../src/operator_help.h"
 #include "../../src/process_int.h"
+#include "../../src/cdo_options.h"
 #include "test_module_list.h"
 #include <iostream>
 
+ProcessManager g_processManager;
+
+void
+cdoExit()
+{
+  g_processManager.killProcesses();
+  exit(EXIT_FAILURE);
+}
+
 go_bandit([]() {
-  add_module("in1_out1", module_in1_out1);
-  add_module("in2_out1", module_in2_out1);
-  add_module("inVariable_out1", module_inVariable_out1);
+
+
+  cdo::progname = "cdo_bandit_test";
+  cdo::setExitFunction(cdoExit);
+  cdo::setContextFunction(processInqPrompt);
+
+  modules={
+  {"in1_out1", module_in1_out1},
+  {"in2_out1", module_in2_out1},
+  {"inVariable_out1", module_inVariable_out1}};
+
+  registerOperatorsFromModules();
 
   // this test checks if operators with non variable input numbers can be
   // chained
